=== modified file 'Makefile'
--- Makefile	2011-05-18 15:45:07 +0000
+++ Makefile	2011-12-08 19:00:14 +0000
@@ -42,7 +42,10 @@
 	set -e
 	# Generate a stream for PQM to watch.
 	-$(RM) -f selftest.log
-	$(PYTHON) -Werror -Wignore::ImportWarning -O ./bzr selftest --subunit $(tests) | tee selftest.log
+	echo `date` ": selftest starts" 1>&2
+	$(PYTHON) -Werror -Wignore::ImportWarning -O ./bzr selftest -Oselftest.timeout=120 \
+	  --subunit $(tests) | tee selftest.log
+	echo `date` ": selftest ends" 1>&2
 	# An empty log file should catch errors in the $(PYTHON)
 	# command above (the '|' swallow any errors since 'make'
 	# sees the 'tee' exit code for the whole line
@@ -429,7 +432,7 @@
 		)
 
 po/bzr.pot: $(PYFILES) $(DOCFILES)
-	$(PYTHON) ./bzr export-pot > po/bzr.pot
+	$(PYTHON) ./bzr export-pot --include-duplicates > po/bzr.pot
 	echo $(TRANSLATABLE_PYFILES) | xargs \
 	  xgettext --package-name "bzr" \
 	  --msgid-bugs-address "<bazaar@canonical.com>" \

=== modified file 'bzr'
--- bzr	2011-08-12 09:49:24 +0000
+++ bzr	2012-01-03 11:38:01 +0000
@@ -16,6 +16,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 """Bazaar -- a free distributed version-control tool"""
 
 import os
@@ -25,29 +27,12 @@
 # update this on each release
 _script_version = (2, 5, 0)
 
-try:
-    version_info = sys.version_info
-except AttributeError:
-    version_info = 1, 5  # 1.5 or older
-
-REINVOKE = "__BZR_REINVOKE"
 NEED_VERS = (2, 6)
-KNOWN_PYTHONS = ('python2.6', 'python2.7')
 
-if version_info < NEED_VERS:
-    if not os.environ.has_key(REINVOKE):
-        # mutating os.environ doesn't work in old Pythons
-        os.putenv(REINVOKE, "1")
-        for python in KNOWN_PYTHONS:
-            try:
-                os.execvp(python, [python] + sys.argv)
-            except OSError:
-                pass
+if sys.version_info < NEED_VERS:
     sys.stderr.write("bzr: error: cannot find a suitable python interpreter\n")
     sys.stderr.write("  (need %d.%d or later)\n" % NEED_VERS)
     sys.exit(1)
-if hasattr(os, "unsetenv"):
-    os.unsetenv(REINVOKE)
 
 
 profiling = False
@@ -84,6 +69,10 @@
             '  Although this should be no problem for bzr itself, it might\n'
             '  cause problems with some plugins. To investigate the issue,\n'
             '  look at the output of the locale(1p) tool.\n' % e)
+    # Use better default than ascii with posix filesystems that deal in bytes
+    # natively even when the C locale or no locale at all is given. Note that
+    # we need an immortal string for the hack, hence the lack of a hyphen.
+    sys._bzr_default_fs_enc = "utf8"
 
 
 # The python2.6 release includes some libraries that have deprecation warnings

=== modified file 'bzrlib/__init__.py'
--- bzrlib/__init__.py	2011-08-12 09:49:24 +0000
+++ bzrlib/__init__.py	2011-12-19 13:23:58 +0000
@@ -31,12 +31,15 @@
 We hope you enjoy this library.
 """
 
+from __future__ import absolute_import
+
 import time
 
 # Keep track of when bzrlib was first imported, so that we can give rough
 # timestamps relative to program start in the log file kept by bzrlib.trace.
 _start_time = time.time()
 
+import codecs
 import sys
 
 
@@ -52,7 +55,7 @@
 # Python version 2.0 is (2, 0, 0, 'final', 0)."  Additionally we use a
 # releaselevel of 'dev' for unreleased under-development code.
 
-version_info = (2, 5, 0, 'dev', 1)
+version_info = (2, 5, 0, 'dev', 5)
 
 # API compatibility version
 api_minimum_version = (2, 4, 0)
@@ -131,13 +134,54 @@
 __version__ = _format_version_tuple(version_info)
 version_string = __version__
 
+
+def _patch_filesystem_default_encoding(new_enc):
+    """Change the Python process global encoding for filesystem names
+    
+    The effect is to change how open() and other builtin functions handle
+    unicode filenames on posix systems. This should only be done near startup.
+
+    The new encoding string passed to this function must survive until process
+    termination, otherwise the interpreter may access uninitialized memory.
+    The use of intern() may defer breakage is but is not enough, the string
+    object should be secure against module reloading and during teardown.
+    """
+    try:
+        import ctypes
+    except ImportError:
+        return
+    pythonapi = getattr(ctypes, "pythonapi", None)
+    if pythonapi is None:
+        # Not CPython ctypes implementation
+        return
+    old_ptr = ctypes.c_void_p.in_dll(pythonapi, "Py_FileSystemDefaultEncoding")
+    new_ptr = ctypes.cast(ctypes.c_char_p(intern(new_enc)), ctypes.c_void_p)
+    old_ptr.value = new_ptr.value
+    if sys.getfilesystemencoding() != new_enc:
+        raise RuntimeError("Failed to change the filesystem default encoding")
+    return new_enc
+
+
+# When running under the bzr script, override bad filesystem default encoding.
+# This is not safe to do for all users of bzrlib, other scripts should instead
+# just ensure a usable locale is set via the $LANG variable on posix systems.
+_fs_enc = sys.getfilesystemencoding()
+if getattr(sys, "_bzr_default_fs_enc", None) is not None:
+    if (_fs_enc is None or codecs.lookup(_fs_enc).name == "ascii"):
+        _fs_enc = _patch_filesystem_default_encoding(sys._bzr_default_fs_enc)
+if _fs_enc is None:
+    _fs_enc = "ascii"
+else:
+    _fs_enc = codecs.lookup(_fs_enc).name
+
+
 # bzr has various bits of global state that are slowly being eliminated.
 # This variable is intended to permit any new state-like things to be attached
 # to a library_state.BzrLibraryState object rather than getting new global
 # variables that need to be hunted down. Accessing the current BzrLibraryState
 # through this variable is not encouraged: it is better to pass it around as
 # part of the context of an operation than to look it up directly, but when
-# that is too hard, it is better to use this variable than to make a branch new
+# that is too hard, it is better to use this variable than to make a brand new
 # global variable.
 # If using this variable by looking it up (because it can't be easily obtained)
 # it is important to store the reference you get, rather than looking it up

=== modified file 'bzrlib/_annotator_py.py'
--- bzrlib/_annotator_py.py	2010-06-17 09:23:19 +0000
+++ bzrlib/_annotator_py.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,8 @@
 
 """Functionality for doing annotations in the 'optimal' way"""
 
+from __future__ import absolute_import
+
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 from bzrlib import (

=== modified file 'bzrlib/_btree_serializer_py.py'
--- bzrlib/_btree_serializer_py.py	2010-02-17 17:11:16 +0000
+++ bzrlib/_btree_serializer_py.py	2011-12-19 13:23:58 +0000
@@ -17,6 +17,8 @@
 
 """B+Tree index parsing."""
 
+from __future__ import absolute_import
+
 from bzrlib import static_tuple
 
 

=== modified file 'bzrlib/_chk_map_py.py'
--- bzrlib/_chk_map_py.py	2010-05-11 10:45:26 +0000
+++ bzrlib/_chk_map_py.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,8 @@
 
 """Python implementation of _search_key functions, etc."""
 
+from __future__ import absolute_import
+
 import zlib
 import struct
 

=== modified file 'bzrlib/_chunks_to_lines_py.py'
--- bzrlib/_chunks_to_lines_py.py	2009-03-23 14:59:43 +0000
+++ bzrlib/_chunks_to_lines_py.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,8 @@
 
 """The python implementation of chunks_to_lines"""
 
+from __future__ import absolute_import
+
 
 def chunks_to_lines(chunks):
     """Re-split chunks into simple lines.

=== modified file 'bzrlib/_dirstate_helpers_py.py'
--- bzrlib/_dirstate_helpers_py.py	2009-06-22 14:32:48 +0000
+++ bzrlib/_dirstate_helpers_py.py	2011-12-19 13:23:58 +0000
@@ -16,7 +16,11 @@
 
 """Python implementations of Dirstate Helper functions."""
 
+from __future__ import absolute_import
+
+import binascii
 import os
+import struct
 
 # We cannot import the dirstate module, because it loads this module
 # All we really need is the IN_MEMORY_MODIFIED constant
@@ -24,6 +28,30 @@
 from bzrlib.dirstate import DirState
 
 
+def pack_stat(st, _b64=binascii.b2a_base64, _pack=struct.Struct('>6L').pack):
+    """Convert stat values into a packed representation
+
+    Not all of the fields from the stat included are strictly needed, and by
+    just encoding the mtime and mode a slight speed increase could be gained.
+    However, using the pyrex version instead is a bigger win.
+    """
+    # base64 encoding always adds a final newline, so strip it off
+    return _b64(_pack(st.st_size & 0xFFFFFFFF, int(st.st_mtime) & 0xFFFFFFFF,
+        int(st.st_ctime) & 0xFFFFFFFF, st.st_dev & 0xFFFFFFFF,
+        st.st_ino & 0xFFFFFFFF, st.st_mode))[:-1]
+
+
+def _unpack_stat(packed_stat):
+    """Turn a packed_stat back into the stat fields.
+
+    This is meant as a debugging tool, should not be used in real code.
+    """
+    (st_size, st_mtime, st_ctime, st_dev, st_ino,
+     st_mode) = struct.unpack('>6L', binascii.a2b_base64(packed_stat))
+    return dict(st_size=st_size, st_mtime=st_mtime, st_ctime=st_ctime,
+                st_dev=st_dev, st_ino=st_ino, st_mode=st_mode)
+
+
 def _bisect_path_left(paths, path):
     """Return the index where to insert path into paths.
 

=== modified file 'bzrlib/_dirstate_helpers_pyx.pyx'
--- bzrlib/_dirstate_helpers_pyx.pyx	2011-04-22 14:12:22 +0000
+++ bzrlib/_dirstate_helpers_pyx.pyx	2011-10-06 16:41:45 +0000
@@ -97,6 +97,7 @@
     object PyTuple_GetItem_void_object "PyTuple_GET_ITEM" (void* tpl, int index)
     object PyTuple_GET_ITEM(object tpl, Py_ssize_t index)
 
+    unsigned long PyInt_AsUnsignedLongMask(object number) except? -1
 
     char *PyString_AsString(object p)
     char *PyString_AsString_obj "PyString_AsString" (PyObject *string)
@@ -811,7 +812,6 @@
 _encode = binascii.b2a_base64
 
 
-from struct import pack
 cdef _pack_stat(stat_value):
     """return a string representing the stat value's key fields.
 
@@ -821,16 +821,22 @@
     cdef char result[6*4] # 6 long ints
     cdef int *aliased
     aliased = <int *>result
-    aliased[0] = htonl(stat_value.st_size)
-    aliased[1] = htonl(int(stat_value.st_mtime))
-    aliased[2] = htonl(int(stat_value.st_ctime))
-    aliased[3] = htonl(stat_value.st_dev)
-    aliased[4] = htonl(stat_value.st_ino & 0xFFFFFFFF)
-    aliased[5] = htonl(stat_value.st_mode)
+    aliased[0] = htonl(PyInt_AsUnsignedLongMask(stat_value.st_size))
+    # mtime and ctime will often be floats but get converted to PyInt within
+    aliased[1] = htonl(PyInt_AsUnsignedLongMask(stat_value.st_mtime))
+    aliased[2] = htonl(PyInt_AsUnsignedLongMask(stat_value.st_ctime))
+    aliased[3] = htonl(PyInt_AsUnsignedLongMask(stat_value.st_dev))
+    aliased[4] = htonl(PyInt_AsUnsignedLongMask(stat_value.st_ino))
+    aliased[5] = htonl(PyInt_AsUnsignedLongMask(stat_value.st_mode))
     packed = PyString_FromStringAndSize(result, 6*4)
     return _encode(packed)[:-1]
 
 
+def pack_stat(stat_value):
+    """Convert stat value into a packed representation quickly with pyrex"""
+    return _pack_stat(stat_value)
+
+
 def update_entry(self, entry, abspath, stat_value):
     """Update the entry based on what is actually on disk.
 
@@ -1793,6 +1799,7 @@
                 advance_entry = -1
                 advance_path = -1
                 result = None
+                changed = None
                 path_handled = 0
                 if current_entry is None:
                     # unversioned -  the check for path_handled when the path

=== modified file 'bzrlib/_groupcompress_py.py'
--- bzrlib/_groupcompress_py.py	2009-04-29 05:53:21 +0000
+++ bzrlib/_groupcompress_py.py	2011-12-19 13:23:58 +0000
@@ -20,6 +20,8 @@
 useless stuff.
 """
 
+from __future__ import absolute_import
+
 from bzrlib import osutils
 
 

=== modified file 'bzrlib/_knit_load_data_py.py'
--- bzrlib/_knit_load_data_py.py	2009-03-23 14:59:43 +0000
+++ bzrlib/_knit_load_data_py.py	2011-12-18 15:28:38 +0000
@@ -14,6 +14,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
 
 from bzrlib import errors
 

=== modified file 'bzrlib/_known_graph_py.py'
--- bzrlib/_known_graph_py.py	2010-02-17 17:11:16 +0000
+++ bzrlib/_known_graph_py.py	2011-12-19 13:23:58 +0000
@@ -17,6 +17,8 @@
 """Implementation of Graph algorithms when we have already loaded everything.
 """
 
+from __future__ import absolute_import
+
 from collections import deque
 from bzrlib import (
     errors,

=== modified file 'bzrlib/_patiencediff_py.py'
--- bzrlib/_patiencediff_py.py	2009-03-23 14:59:43 +0000
+++ bzrlib/_patiencediff_py.py	2011-12-18 15:28:38 +0000
@@ -15,6 +15,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
 
 from bisect import bisect
 import difflib

=== modified file 'bzrlib/_readdir_py.py'
--- bzrlib/_readdir_py.py	2010-04-30 11:03:59 +0000
+++ bzrlib/_readdir_py.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,7 @@
 
 """Python implementation of readdir interface."""
 
+from __future__ import absolute_import
 
 import stat
 

=== modified file 'bzrlib/_rio_py.py'
--- bzrlib/_rio_py.py	2009-05-14 23:53:07 +0000
+++ bzrlib/_rio_py.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,8 @@
 
 """Python implementation of _read_stanza_*."""
 
+from __future__ import absolute_import
+
 import re
 
 from bzrlib.rio import (

=== modified file 'bzrlib/_static_tuple_py.py'
--- bzrlib/_static_tuple_py.py	2010-02-23 07:43:11 +0000
+++ bzrlib/_static_tuple_py.py	2011-12-19 13:23:58 +0000
@@ -20,6 +20,8 @@
 strings.
 """
 
+from __future__ import absolute_import
+
 
 class StaticTuple(tuple):
     """A static type, similar to a tuple of strings."""

=== modified file 'bzrlib/add.py'
--- bzrlib/add.py	2011-08-19 21:01:49 +0000
+++ bzrlib/add.py	2011-12-19 16:41:49 +0000
@@ -16,6 +16,8 @@
 
 """Helper functions for adding files to working trees."""
 
+from __future__ import absolute_import
+
 import sys
 import os
 
@@ -23,7 +25,7 @@
     osutils,
     ui, 
     )
-
+from bzrlib.i18n import gettext
 
 class AddAction(object):
     """A class which defines what action to take when adding a file."""
@@ -73,27 +75,25 @@
 class AddWithSkipLargeAction(AddAction):
     """A class that can decide to skip a file if it's considered too large"""
 
-    # default 20 MB
-    _DEFAULT_MAX_FILE_SIZE = 20000000
-    _optionName = 'add.maximum_file_size'
     _maxSize = None
 
     def skip_file(self, tree, path, kind, stat_value = None):
         if kind != 'file':
-            return False            
+            return False
+        opt_name = 'add.maximum_file_size'
         if self._maxSize is None:
-            config = tree.branch.get_config()
-            self._maxSize = config.get_user_option_as_int_from_SI(
-                self._optionName,  
-                self._DEFAULT_MAX_FILE_SIZE)
+            # FIXME: We use the branch config as there is no tree config
+            # -- vila 2011-12-16
+            config = tree.branch.get_config_stack()
+            self._maxSize = config.get(opt_name)
         if stat_value is None:
             file_size = os.path.getsize(path);
         else:
             file_size = stat_value.st_size;
         if self._maxSize > 0 and file_size > self._maxSize:
-            ui.ui_factory.show_warning(
-                "skipping %s (larger than %s of %d bytes)" % 
-                (path, self._optionName,  self._maxSize))
+            ui.ui_factory.show_warning(gettext(
+                "skipping {0} (larger than {1} of {2} bytes)").format(
+                path, opt_name,  self._maxSize))
             return True
         return False
 

=== modified file 'bzrlib/annotate.py'
--- bzrlib/annotate.py	2011-06-27 15:52:06 +0000
+++ bzrlib/annotate.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,8 @@
 
 """File annotate based on weave storage"""
 
+from __future__ import absolute_import
+
 # TODO: Choice of more or less verbose formats:
 #
 # interposed: show more details between blocks of modified lines
@@ -105,7 +107,7 @@
         current_rev = Revision(CURRENT_REVISION)
         current_rev.parent_ids = tree.get_parent_ids()
         try:
-            current_rev.committer = branch.get_config().username()
+            current_rev.committer = branch.get_config_stack().get('email')
         except errors.NoWhoami:
             current_rev.committer = 'local user'
         current_rev.message = "?"

=== modified file 'bzrlib/api.py'
--- bzrlib/api.py	2011-04-07 10:36:24 +0000
+++ bzrlib/api.py	2011-12-19 13:23:58 +0000
@@ -21,6 +21,8 @@
 examples.
 """
 
+from __future__ import absolute_import
+
 import bzrlib
 from bzrlib.errors import IncompatibleAPI
 

=== modified file 'bzrlib/atomicfile.py'
--- bzrlib/atomicfile.py	2011-07-15 09:04:54 +0000
+++ bzrlib/atomicfile.py	2011-12-18 12:46:49 +0000
@@ -14,6 +14,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
 
 import os
 
@@ -78,7 +79,7 @@
             # the common case is that we won't, though.
             st = os.fstat(self._fd)
             if stat.S_IMODE(st.st_mode) != new_mode:
-                os.chmod(self.tmpfilename, new_mode)
+                osutils.chmod_if_possible(self.tmpfilename, new_mode)
 
     def __repr__(self):
         return '%s(%r)' % (self.__class__.__name__,

=== modified file 'bzrlib/bencode.py'
--- bzrlib/bencode.py	2009-09-11 06:36:50 +0000
+++ bzrlib/bencode.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,8 @@
 
 """Wrapper around the bencode pyrex and python implementation"""
 
+from __future__ import absolute_import
+
 from bzrlib import osutils
 
 try:

=== modified file 'bzrlib/bisect_multi.py'
--- bzrlib/bisect_multi.py	2009-03-23 14:59:43 +0000
+++ bzrlib/bisect_multi.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,8 @@
 
 """Bisection lookup multiple keys."""
 
+from __future__ import absolute_import
+
 __all__ = [
     'bisect_multi_bytes',
     ]

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2011-08-27 16:59:43 +0000
+++ bzrlib/branch.py	2012-01-03 13:47:01 +0000
@@ -14,6 +14,9 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
+import bzrlib.bzrdir
 
 from cStringIO import StringIO
 
@@ -21,28 +24,36 @@
 lazy_import(globals(), """
 import itertools
 from bzrlib import (
-        bzrdir,
-        cache_utf8,
-        cleanup,
-        config as _mod_config,
-        debug,
-        errors,
-        fetch,
-        graph as _mod_graph,
-        lockdir,
-        lockable_files,
-        remote,
-        repository,
-        revision as _mod_revision,
-        rio,
-        tag as _mod_tag,
-        transport,
-        ui,
-        urlutils,
-        )
+    bzrdir,
+    controldir,
+    cache_utf8,
+    cleanup,
+    config as _mod_config,
+    debug,
+    errors,
+    fetch,
+    graph as _mod_graph,
+    lockdir,
+    lockable_files,
+    remote,
+    repository,
+    revision as _mod_revision,
+    rio,
+    tag as _mod_tag,
+    transport,
+    ui,
+    urlutils,
+    vf_search,
+    )
+from bzrlib.i18n import gettext, ngettext
 """)
 
+# Explicitly import bzrlib.bzrdir so that the BzrProber
+# is guaranteed to be registered.
+import bzrlib.bzrdir
+
 from bzrlib import (
+    bzrdir,
     controldir,
     )
 from bzrlib.decorators import (
@@ -83,7 +94,7 @@
     def user_transport(self):
         return self.bzrdir.user_transport
 
-    def __init__(self, *ignored, **ignored_too):
+    def __init__(self, possible_transports=None):
         self.tags = self._format.make_tags(self)
         self._revision_history_cache = None
         self._revision_id_to_revno_cache = None
@@ -93,24 +104,24 @@
         self._last_revision_info_cache = None
         self._master_branch_cache = None
         self._merge_sorted_revisions_cache = None
-        self._open_hook()
+        self._open_hook(possible_transports)
         hooks = Branch.hooks['open']
         for hook in hooks:
             hook(self)
 
-    def _open_hook(self):
+    def _open_hook(self, possible_transports):
         """Called by init to allow simpler extension of the base class."""
 
-    def _activate_fallback_location(self, url):
+    def _activate_fallback_location(self, url, possible_transports):
         """Activate the branch/repository from url as a fallback repository."""
         for existing_fallback_repo in self.repository._fallback_repositories:
             if existing_fallback_repo.user_url == url:
                 # This fallback is already configured.  This probably only
-                # happens because BzrDir.sprout is a horrible mess.  To avoid
+                # happens because ControlDir.sprout is a horrible mess.  To avoid
                 # confusing _unstack we don't add this a second time.
                 mutter('duplicate activation of fallback %r on %r', url, self)
                 return
-        repo = self._get_fallback_repository(url)
+        repo = self._get_fallback_repository(url, possible_transports)
         if repo.has_same_location(self.repository):
             raise errors.UnstackableLocationError(self.user_url, url)
         self.repository.add_fallback_repository(repo)
@@ -170,15 +181,18 @@
         For instance, if the branch is at URL/.bzr/branch,
         Branch.open(URL) -> a Branch instance.
         """
-        control = bzrdir.BzrDir.open(base, _unsupported,
-                                     possible_transports=possible_transports)
-        return control.open_branch(unsupported=_unsupported)
+        control = controldir.ControlDir.open(base,
+            possible_transports=possible_transports, _unsupported=_unsupported)
+        return control.open_branch(unsupported=_unsupported,
+            possible_transports=possible_transports)
 
     @staticmethod
-    def open_from_transport(transport, name=None, _unsupported=False):
+    def open_from_transport(transport, name=None, _unsupported=False,
+            possible_transports=None):
         """Open the branch rooted at transport"""
-        control = bzrdir.BzrDir.open_from_transport(transport, _unsupported)
-        return control.open_branch(name=name, unsupported=_unsupported)
+        control = controldir.ControlDir.open_from_transport(transport, _unsupported)
+        return control.open_branch(name=name, unsupported=_unsupported,
+            possible_transports=possible_transports)
 
     @staticmethod
     def open_containing(url, possible_transports=None):
@@ -192,9 +206,10 @@
         format, UnknownFormatError or UnsupportedFormatError are raised.
         If there is one, it is returned, along with the unused portion of url.
         """
-        control, relpath = bzrdir.BzrDir.open_containing(url,
+        control, relpath = controldir.ControlDir.open_containing(url,
                                                          possible_transports)
-        return control.open_branch(), relpath
+        branch = control.open_branch(possible_transports=possible_transports)
+        return (branch, relpath)
 
     def _push_should_merge_tags(self):
         """Should _basic_push merge this branch's tags into the target?
@@ -214,6 +229,16 @@
         """
         return _mod_config.BranchConfig(self)
 
+    def get_config_stack(self):
+        """Get a bzrlib.config.BranchStack for this Branch.
+
+        This can then be used to get and set configuration options for the
+        branch.
+
+        :return: A bzrlib.config.BranchStack.
+        """
+        return _mod_config.BranchStack(self)
+
     def _get_config(self):
         """Get the concrete config for just the config in this branch.
 
@@ -226,11 +251,10 @@
         """
         raise NotImplementedError(self._get_config)
 
-    def _get_fallback_repository(self, url):
+    def _get_fallback_repository(self, url, possible_transports):
         """Get the repository we fallback to at url."""
         url = urlutils.join(self.base, url)
-        a_branch = Branch.open(url,
-            possible_transports=[self.bzrdir.root_transport])
+        a_branch = Branch.open(url, possible_transports=possible_transports)
         return a_branch.repository
 
     @needs_read_lock
@@ -641,15 +665,17 @@
         """
         raise errors.UpgradeRequired(self.user_url)
 
+    def get_append_revisions_only(self):
+        """Whether it is only possible to append revisions to the history.
+        """
+        if not self._format.supports_set_append_revisions_only():
+            return False
+        return self.get_config_stack().get('append_revisions_only')
+
     def set_append_revisions_only(self, enabled):
         if not self._format.supports_set_append_revisions_only():
             raise errors.UpgradeRequired(self.user_url)
-        if enabled:
-            value = 'True'
-        else:
-            value = 'False'
-        self.get_config().set_user_option('append_revisions_only', value,
-            warn_masked=True)
+        self.get_config_stack().set('append_revisions_only', enabled)
 
     def set_reference_info(self, file_id, tree_path, branch_location):
         """Set the branch location to use for a tree reference."""
@@ -684,7 +710,7 @@
         """
         raise errors.UpgradeRequired(self.user_url)
 
-    def get_commit_builder(self, parents, config=None, timestamp=None,
+    def get_commit_builder(self, parents, config_stack=None, timestamp=None,
                            timezone=None, committer=None, revprops=None,
                            revision_id=None, lossy=False):
         """Obtain a CommitBuilder for this branch.
@@ -700,10 +726,10 @@
             represented, when pushing to a foreign VCS 
         """
 
-        if config is None:
-            config = self.get_config()
+        if config_stack is None:
+            config_stack = self.get_config_stack()
 
-        return self.repository.get_commit_builder(self, parents, config,
+        return self.repository.get_commit_builder(self, parents, config_stack,
             timestamp, timezone, committer, revprops, revision_id,
             lossy)
 
@@ -714,16 +740,18 @@
         """
         return None
 
+    @deprecated_method(deprecated_in((2, 5, 0)))
     def get_revision_delta(self, revno):
         """Return the delta for one revision.
 
         The delta is relative to its mainline predecessor, or the
         empty tree for revision 1.
         """
-        rh = self.revision_history()
-        if not (1 <= revno <= len(rh)):
+        try:
+            revid = self.get_rev_id(revno)
+        except errors.NoSuchRevision:
             raise errors.InvalidRevisionNumber(revno)
-        return self.repository.get_revision_delta(rh[revno-1])
+        return self.repository.get_revision_delta(revid)
 
     def get_stacked_on_url(self):
         """Get the URL this branch is stacked against.
@@ -828,7 +856,8 @@
                 return
             self._unstack()
         else:
-            self._activate_fallback_location(url)
+            self._activate_fallback_location(url,
+                possible_transports=[self.bzrdir.root_transport])
         # write this out after the repository is stacked to avoid setting a
         # stacked config that doesn't work.
         self._set_config_location('stacked_on_location', url)
@@ -840,7 +869,7 @@
         """
         pb = ui.ui_factory.nested_progress_bar()
         try:
-            pb.update("Unstacking")
+            pb.update(gettext("Unstacking"))
             # The basic approach here is to fetch the tip of the branch,
             # including all available ghosts, from the existing stacked
             # repository into a new repository object without the fallbacks. 
@@ -860,7 +889,8 @@
             # stream from one of them to the other.  This does mean doing
             # separate SSH connection setup, but unstacking is not a
             # common operation so it's tolerable.
-            new_bzrdir = bzrdir.BzrDir.open(self.bzrdir.root_transport.base)
+            new_bzrdir = controldir.ControlDir.open(
+                self.bzrdir.root_transport.base)
             new_repository = new_bzrdir.find_repository()
             if new_repository._fallback_repositories:
                 raise AssertionError("didn't expect %r to have "
@@ -909,7 +939,7 @@
                     tags_to_fetch = set(self.tags.get_reverse_tag_dict())
                 except errors.TagsNotSupported:
                     tags_to_fetch = set()
-                fetch_spec = _mod_graph.NotInOtherForRevs(self.repository,
+                fetch_spec = vf_search.NotInOtherForRevs(self.repository,
                     old_repository, required_ids=[self.last_revision()],
                     if_present_ids=tags_to_fetch, find_ghosts=True).execute()
                 self.repository.fetch(old_repository, fetch_spec=fetch_spec)
@@ -983,6 +1013,7 @@
         """
         raise NotImplementedError(self._gen_revision_history)
 
+    @deprecated_method(deprecated_in((2, 5, 0)))
     @needs_read_lock
     def revision_history(self):
         """Return sequence of revision ids on this branch.
@@ -990,6 +1021,9 @@
         This method will cache the revision history for as long as it is safe to
         do so.
         """
+        return self._revision_history()
+
+    def _revision_history(self):
         if 'evil' in debug.debug_flags:
             mutter_callsite(3, "revision_history scales with history.")
         if self._revision_history_cache is not None:
@@ -1065,7 +1099,7 @@
         """Given a revision id, return its revno"""
         if _mod_revision.is_null(revision_id):
             return 0
-        history = self.revision_history()
+        history = self._revision_history()
         try:
             return history.index(revision_id) + 1
         except ValueError:
@@ -1136,17 +1170,17 @@
     def _set_config_location(self, name, url, config=None,
                              make_relative=False):
         if config is None:
-            config = self.get_config()
+            config = self.get_config_stack()
         if url is None:
             url = ''
         elif make_relative:
             url = urlutils.relative_url(self.base, url)
-        config.set_user_option(name, url, warn_masked=True)
+        config.set(name, url)
 
     def _get_config_location(self, name, config=None):
         if config is None:
-            config = self.get_config()
-        location = config.get_user_option(name)
+            config = self.get_config_stack()
+        location = config.get(name)
         if location == '':
             location = None
         return location
@@ -1368,11 +1402,11 @@
         # TODO: We should probably also check that self.revision_history
         # matches the repository for older branch formats.
         # If looking for the code that cross-checks repository parents against
-        # the iter_reverse_revision_history output, that is now a repository
+        # the Graph.iter_lefthand_ancestry output, that is now a repository
         # specific check.
         return result
 
-    def _get_checkout_format(self):
+    def _get_checkout_format(self, lightweight=False):
         """Return the most suitable metadir for a checkout of this branch.
         Weaves are used if this branch's repository uses weaves.
         """
@@ -1424,14 +1458,13 @@
         """
         t = transport.get_transport(to_location)
         t.ensure_base()
+        format = self._get_checkout_format(lightweight=lightweight)
         if lightweight:
-            format = self._get_checkout_format()
             checkout = format.initialize_on_transport(t)
             from_branch = BranchReferenceFormat().initialize(checkout, 
                 target_branch=self)
         else:
-            format = self._get_checkout_format()
-            checkout_branch = bzrdir.BzrDir.create_branch_convenience(
+            checkout_branch = controldir.ControlDir.create_branch_convenience(
                 to_location, force_new_tree=False, format=format)
             checkout = checkout_branch.bzrdir
             checkout_branch.bind(self)
@@ -1533,14 +1566,11 @@
             heads that must be fetched if present, but no error is necessary if
             they are not present.
         """
-        # For bzr native formats must_fetch is just the tip, and if_present_fetch
-        # are the tags.
+        # For bzr native formats must_fetch is just the tip, and
+        # if_present_fetch are the tags.
         must_fetch = set([self.last_revision()])
         if_present_fetch = set()
-        c = self.get_config()
-        include_tags = c.get_user_option_as_bool('branch.fetch_tags',
-                                                 default=False)
-        if include_tags:
+        if self.get_config_stack().get('branch.fetch_tags'):
             try:
                 if_present_fetch = set(self.tags.get_reverse_tag_dict())
             except errors.TagsNotSupported:
@@ -1555,7 +1585,7 @@
 
     Formats provide three things:
      * An initialization routine,
-     * a format string,
+     * a format description
      * an open routine.
 
     Formats are placed in an dict by their format string for reference
@@ -1568,8 +1598,6 @@
     object will be created every time regardless.
     """
 
-    can_set_append_revisions_only = True
-
     def __eq__(self, other):
         return self.__class__ is other.__class__
 
@@ -1577,18 +1605,6 @@
         return not (self == other)
 
     @classmethod
-    def find_format(klass, a_bzrdir, name=None):
-        """Return the format for the branch object in a_bzrdir."""
-        try:
-            transport = a_bzrdir.get_branch_transport(None, name=name)
-            format_string = transport.get_bytes("format")
-            return format_registry.get(format_string)
-        except errors.NoSuchFile:
-            raise errors.NotBranchError(path=transport.base, bzrdir=a_bzrdir)
-        except KeyError:
-            raise errors.UnknownFormatError(format=format_string, kind='branch')
-
-    @classmethod
     @deprecated_method(deprecated_in((2, 4, 0)))
     def get_default_format(klass):
         """Return the current default format."""
@@ -1604,52 +1620,49 @@
         """
         return format_registry._get_all()
 
-    def get_reference(self, a_bzrdir, name=None):
-        """Get the target reference of the branch in a_bzrdir.
+    def get_reference(self, controldir, name=None):
+        """Get the target reference of the branch in controldir.
 
         format probing must have been completed before calling
         this method - it is assumed that the format of the branch
-        in a_bzrdir is correct.
+        in controldir is correct.
 
-        :param a_bzrdir: The bzrdir to get the branch data from.
+        :param controldir: The controldir to get the branch data from.
         :param name: Name of the colocated branch to fetch
         :return: None if the branch is not a reference branch.
         """
         return None
 
     @classmethod
-    def set_reference(self, a_bzrdir, name, to_branch):
-        """Set the target reference of the branch in a_bzrdir.
+    def set_reference(self, controldir, name, to_branch):
+        """Set the target reference of the branch in controldir.
 
         format probing must have been completed before calling
         this method - it is assumed that the format of the branch
-        in a_bzrdir is correct.
+        in controldir is correct.
 
-        :param a_bzrdir: The bzrdir to set the branch reference for.
+        :param controldir: The controldir to set the branch reference for.
         :param name: Name of colocated branch to set, None for default
         :param to_branch: branch that the checkout is to reference
         """
         raise NotImplementedError(self.set_reference)
 
-    def get_format_string(self):
-        """Return the ASCII format string that identifies this format."""
-        raise NotImplementedError(self.get_format_string)
-
     def get_format_description(self):
         """Return the short format description for this format."""
         raise NotImplementedError(self.get_format_description)
 
-    def _run_post_branch_init_hooks(self, a_bzrdir, name, branch):
+    def _run_post_branch_init_hooks(self, controldir, name, branch):
         hooks = Branch.hooks['post_branch_init']
         if not hooks:
             return
-        params = BranchInitHookParams(self, a_bzrdir, name, branch)
+        params = BranchInitHookParams(self, controldir, name, branch)
         for hook in hooks:
             hook(params)
 
-    def initialize(self, a_bzrdir, name=None, repository=None):
-        """Create a branch of this format in a_bzrdir.
-        
+    def initialize(self, controldir, name=None, repository=None,
+                   append_revisions_only=None):
+        """Create a branch of this format in controldir.
+
         :param name: Name of the colocated branch to create.
         """
         raise NotImplementedError(self.initialize)
@@ -1687,11 +1700,11 @@
         """
         raise NotImplementedError(self.network_name)
 
-    def open(self, a_bzrdir, name=None, _found=False, ignore_fallbacks=False,
-            found_repository=None):
-        """Return the branch object for a_bzrdir
+    def open(self, controldir, name=None, _found=False, ignore_fallbacks=False,
+            found_repository=None, possible_transports=None):
+        """Return the branch object for controldir.
 
-        :param a_bzrdir: A BzrDir that contains a branch.
+        :param controldir: A ControlDir that contains a branch.
         :param name: Name of colocated branch to open
         :param _found: a private parameter, do not use it. It is used to
             indicate if format probing has already be done.
@@ -1739,6 +1752,14 @@
         """True if this format supports tags stored in the branch"""
         return False  # by default
 
+    def tags_are_versioned(self):
+        """Whether the tag container for this branch versions tags."""
+        return False
+
+    def supports_tags_referencing_ghosts(self):
+        """True if tags can reference ghost revisions."""
+        return True
+
 
 class MetaDirBranchFormatFactory(registry._LazyObjectGetter):
     """A factory for a BranchFormat object, permitting simple lazy registration.
@@ -1758,7 +1779,7 @@
         """
         registry._LazyObjectGetter.__init__(self, module_name, member_name)
         self._format_string = format_string
-        
+
     def get_format_string(self):
         """See BranchFormat.get_format_string."""
         return self._format_string
@@ -1909,7 +1930,7 @@
     There are 4 fields that hooks may wish to access:
 
     :ivar format: the branch format
-    :ivar bzrdir: the BzrDir where the branch will be/has been initialized
+    :ivar bzrdir: the ControlDir where the branch will be/has been initialized
     :ivar name: name of colocated branch, if any (or None)
     :ivar branch: the branch created
 
@@ -1918,11 +1939,11 @@
     branch, which refer to the original branch.
     """
 
-    def __init__(self, format, a_bzrdir, name, branch):
+    def __init__(self, format, controldir, name, branch):
         """Create a group of BranchInitHook parameters.
 
         :param format: the branch format
-        :param a_bzrdir: the BzrDir where the branch will be/has been
+        :param controldir: the ControlDir where the branch will be/has been
             initialized
         :param name: name of colocated branch, if any (or None)
         :param branch: the branch created
@@ -1932,7 +1953,7 @@
         in branch, which refer to the original branch.
         """
         self.format = format
-        self.bzrdir = a_bzrdir
+        self.bzrdir = controldir
         self.name = name
         self.branch = branch
 
@@ -1948,7 +1969,7 @@
 
     There are 4 fields that hooks may wish to access:
 
-    :ivar control_dir: BzrDir of the checkout to change
+    :ivar control_dir: ControlDir of the checkout to change
     :ivar to_branch: branch that the checkout is to reference
     :ivar force: skip the check for local commits in a heavy checkout
     :ivar revision_id: revision ID to switch to (or None)
@@ -1957,7 +1978,7 @@
     def __init__(self, control_dir, to_branch, force, revision_id):
         """Create a group of SwitchHook parameters.
 
-        :param control_dir: BzrDir of the checkout to change
+        :param control_dir: ControlDir of the checkout to change
         :param to_branch: branch that the checkout is to reference
         :param force: skip the check for local commits in a heavy checkout
         :param revision_id: revision ID to switch to (or None)
@@ -1976,13 +1997,39 @@
             self.revision_id)
 
 
-class BranchFormatMetadir(BranchFormat):
-    """Common logic for meta-dir based branch formats."""
+class BranchFormatMetadir(bzrdir.BzrFormat, BranchFormat):
+    """Base class for branch formats that live in meta directories.
+    """
+
+    def __init__(self):
+        BranchFormat.__init__(self)
+        bzrdir.BzrFormat.__init__(self)
+
+    @classmethod
+    def find_format(klass, controldir, name=None):
+        """Return the format for the branch object in controldir."""
+        try:
+            transport = controldir.get_branch_transport(None, name=name)
+        except errors.NoSuchFile:
+            raise errors.NotBranchError(path=name, bzrdir=controldir)
+        try:
+            format_string = transport.get_bytes("format")
+        except errors.NoSuchFile:
+            raise errors.NotBranchError(path=transport.base, bzrdir=controldir)
+        return klass._find_format(format_registry, 'branch', format_string)
 
     def _branch_class(self):
         """What class to instantiate on open calls."""
         raise NotImplementedError(self._branch_class)
 
+    def _get_initial_config(self, append_revisions_only=None):
+        if append_revisions_only:
+            return "append_revisions_only = True\n"
+        else:
+            # Avoid writing anything if append_revisions_only is disabled,
+            # as that is the default.
+            return ""
+
     def _initialize_helper(self, a_bzrdir, utf8_files, name=None,
                            repository=None):
         """Initialize a branch in a bzrdir, with specified files
@@ -2000,7 +2047,7 @@
         control_files.create_lock()
         control_files.lock_write()
         try:
-            utf8_files += [('format', self.get_format_string())]
+            utf8_files += [('format', self.as_string())]
             for (filename, content) in utf8_files:
                 branch_transport.put_bytes(
                     filename, content,
@@ -2012,18 +2059,11 @@
         self._run_post_branch_init_hooks(a_bzrdir, name, branch)
         return branch
 
-    def network_name(self):
-        """A simple byte string uniquely identifying this format for RPC calls.
-
-        Metadir branch formats use their format string.
-        """
-        return self.get_format_string()
-
     def open(self, a_bzrdir, name=None, _found=False, ignore_fallbacks=False,
-            found_repository=None):
+            found_repository=None, possible_transports=None):
         """See BranchFormat.open()."""
         if not _found:
-            format = BranchFormat.find_format(a_bzrdir, name=name)
+            format = BranchFormatMetadir.find_format(a_bzrdir, name=name)
             if format.__class__ != self.__class__:
                 raise AssertionError("wrong format %r found for %r" %
                     (format, self))
@@ -2038,14 +2078,16 @@
                               name=name,
                               a_bzrdir=a_bzrdir,
                               _repository=found_repository,
-                              ignore_fallbacks=ignore_fallbacks)
+                              ignore_fallbacks=ignore_fallbacks,
+                              possible_transports=possible_transports)
         except errors.NoSuchFile:
             raise errors.NotBranchError(path=transport.base, bzrdir=a_bzrdir)
 
-    def __init__(self):
-        super(BranchFormatMetadir, self).__init__()
-        self._matchingbzrdir = bzrdir.BzrDirMetaFormat1()
-        self._matchingbzrdir.set_branch_format(self)
+    @property
+    def _matchingbzrdir(self):
+        ret = bzrdir.BzrDirMetaFormat1()
+        ret.set_branch_format(self)
+        return ret
 
     def supports_tags(self):
         return True
@@ -2053,6 +2095,14 @@
     def supports_leaving_lock(self):
         return True
 
+    def check_support_status(self, allow_unsupported, recommend_upgrade=True,
+            basedir=None):
+        BranchFormat.check_support_status(self,
+            allow_unsupported=allow_unsupported, recommend_upgrade=recommend_upgrade,
+            basedir=basedir)
+        bzrdir.BzrFormat.check_support_status(self, allow_unsupported=allow_unsupported,
+            recommend_upgrade=recommend_upgrade, basedir=basedir)
+
 
 class BzrBranchFormat5(BranchFormatMetadir):
     """Bzr branch format 5.
@@ -2070,7 +2120,8 @@
     def _branch_class(self):
         return BzrBranch5
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See BranchFormat.get_format_string()."""
         return "Bazaar-NG branch format 5\n"
 
@@ -2078,8 +2129,11 @@
         """See BranchFormat.get_format_description()."""
         return "Branch format 5"
 
-    def initialize(self, a_bzrdir, name=None, repository=None):
+    def initialize(self, a_bzrdir, name=None, repository=None,
+                   append_revisions_only=None):
         """Create a branch of this format in a_bzrdir."""
+        if append_revisions_only:
+            raise errors.UpgradeRequired(a_bzrdir.user_url)
         utf8_files = [('revision-history', ''),
                       ('branch-name', ''),
                       ]
@@ -2103,7 +2157,8 @@
     def _branch_class(self):
         return BzrBranch6
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See BranchFormat.get_format_string()."""
         return "Bazaar Branch Format 6 (bzr 0.15)\n"
 
@@ -2111,10 +2166,12 @@
         """See BranchFormat.get_format_description()."""
         return "Branch format 6"
 
-    def initialize(self, a_bzrdir, name=None, repository=None):
+    def initialize(self, a_bzrdir, name=None, repository=None,
+                   append_revisions_only=None):
         """Create a branch of this format in a_bzrdir."""
         utf8_files = [('last-revision', '0 null:\n'),
-                      ('branch.conf', ''),
+                      ('branch.conf',
+                          self._get_initial_config(append_revisions_only)),
                       ('tags', ''),
                       ]
         return self._initialize_helper(a_bzrdir, utf8_files, name, repository)
@@ -2133,7 +2190,8 @@
     def _branch_class(self):
         return BzrBranch8
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See BranchFormat.get_format_string()."""
         return "Bazaar Branch Format 8 (needs bzr 1.15)\n"
 
@@ -2141,10 +2199,12 @@
         """See BranchFormat.get_format_description()."""
         return "Branch format 8"
 
-    def initialize(self, a_bzrdir, name=None, repository=None):
+    def initialize(self, a_bzrdir, name=None, repository=None,
+                   append_revisions_only=None):
         """Create a branch of this format in a_bzrdir."""
         utf8_files = [('last-revision', '0 null:\n'),
-                      ('branch.conf', ''),
+                      ('branch.conf',
+                          self._get_initial_config(append_revisions_only)),
                       ('tags', ''),
                       ('references', '')
                       ]
@@ -2172,10 +2232,12 @@
     This format was introduced in bzr 1.6.
     """
 
-    def initialize(self, a_bzrdir, name=None, repository=None):
+    def initialize(self, a_bzrdir, name=None, repository=None,
+                   append_revisions_only=None):
         """Create a branch of this format in a_bzrdir."""
         utf8_files = [('last-revision', '0 null:\n'),
-                      ('branch.conf', ''),
+                      ('branch.conf',
+                          self._get_initial_config(append_revisions_only)),
                       ('tags', ''),
                       ]
         return self._initialize_helper(a_bzrdir, utf8_files, name, repository)
@@ -2183,7 +2245,8 @@
     def _branch_class(self):
         return BzrBranch7
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See BranchFormat.get_format_string()."""
         return "Bazaar Branch Format 7 (needs bzr 1.6)\n"
 
@@ -2204,7 +2267,7 @@
     supports_reference_locations = False
 
 
-class BranchReferenceFormat(BranchFormat):
+class BranchReferenceFormat(BranchFormatMetadir):
     """Bzr branch reference format.
 
     Branch references are used in implementing checkouts, they
@@ -2215,7 +2278,8 @@
      - a format string
     """
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See BranchFormat.get_format_string()."""
         return "Bazaar-NG Branch Reference Format 1\n"
 
@@ -2234,7 +2298,7 @@
         location = transport.put_bytes('location', to_branch.base)
 
     def initialize(self, a_bzrdir, name=None, target_branch=None,
-            repository=None):
+            repository=None, append_revisions_only=None):
         """Create a branch of this format in a_bzrdir."""
         if target_branch is None:
             # this format does not implement branch itself, thus the implicit
@@ -2245,19 +2309,14 @@
             raise errors.IncompatibleFormat(self, a_bzrdir._format)
         branch_transport = a_bzrdir.get_branch_transport(self, name=name)
         branch_transport.put_bytes('location',
-            target_branch.bzrdir.user_url)
-        branch_transport.put_bytes('format', self.get_format_string())
+            target_branch.user_url)
+        branch_transport.put_bytes('format', self.as_string())
         branch = self.open(
             a_bzrdir, name, _found=True,
             possible_transports=[target_branch.bzrdir.root_transport])
         self._run_post_branch_init_hooks(a_bzrdir, name, branch)
         return branch
 
-    def __init__(self):
-        super(BranchReferenceFormat, self).__init__()
-        self._matchingbzrdir = bzrdir.BzrDirMetaFormat1()
-        self._matchingbzrdir.set_branch_format(self)
-
     def _make_reference_clone_function(format, a_branch):
         """Create a clone() routine for a branch dynamically."""
         def clone(to_bzrdir, revision_id=None,
@@ -2286,16 +2345,17 @@
         :param possible_transports: An optional reusable transports list.
         """
         if not _found:
-            format = BranchFormat.find_format(a_bzrdir, name=name)
+            format = BranchFormatMetadir.find_format(a_bzrdir, name=name)
             if format.__class__ != self.__class__:
                 raise AssertionError("wrong format %r found for %r" %
                     (format, self))
         if location is None:
             location = self.get_reference(a_bzrdir, name)
-        real_bzrdir = bzrdir.BzrDir.open(
+        real_bzrdir = controldir.ControlDir.open(
             location, possible_transports=possible_transports)
         result = real_bzrdir.open_branch(name=name, 
-            ignore_fallbacks=ignore_fallbacks)
+            ignore_fallbacks=ignore_fallbacks,
+            possible_transports=possible_transports)
         # this changes the behaviour of result.clone to create a new reference
         # rather than a copy of the content of the branch.
         # I did not use a proxy object because that needs much more extensive
@@ -2382,31 +2442,29 @@
 
     def __init__(self, _format=None,
                  _control_files=None, a_bzrdir=None, name=None,
-                 _repository=None, ignore_fallbacks=False):
+                 _repository=None, ignore_fallbacks=False,
+                 possible_transports=None):
         """Create new branch object at a particular location."""
         if a_bzrdir is None:
             raise ValueError('a_bzrdir must be supplied')
         else:
             self.bzrdir = a_bzrdir
-        self._base = self.bzrdir.transport.clone('..').base
+        self._user_transport = self.bzrdir.transport.clone('..')
+        if name is not None:
+            self._user_transport.set_segment_parameter(
+                "branch", urlutils.escape(name))
+        self._base = self._user_transport.base
         self.name = name
-        # XXX: We should be able to just do
-        #   self.base = self.bzrdir.root_transport.base
-        # but this does not quite work yet -- mbp 20080522
         self._format = _format
         if _control_files is None:
             raise ValueError('BzrBranch _control_files is None')
         self.control_files = _control_files
         self._transport = _control_files._transport
         self.repository = _repository
-        Branch.__init__(self)
+        Branch.__init__(self, possible_transports)
 
     def __str__(self):
-        if self.name is None:
-            return '%s(%s)' % (self.__class__.__name__, self.user_url)
-        else:
-            return '%s(%s,%s)' % (self.__class__.__name__, self.user_url,
-                self.name)
+        return '%s(%s)' % (self.__class__.__name__, self.user_url)
 
     __repr__ = __str__
 
@@ -2416,9 +2474,16 @@
 
     base = property(_get_base, doc="The URL for the root of this branch.")
 
+    @property
+    def user_transport(self):
+        return self._user_transport
+
     def _get_config(self):
         return _mod_config.TransportConfig(self._transport, 'branch.conf')
 
+    def _get_config_store(self):
+        return _mod_config.BranchStore(self)
+
     def is_locked(self):
         return self.control_files.is_locked()
 
@@ -2504,7 +2569,7 @@
             raise errors.InvalidRevisionId(revision_id=revision_id, branch=self)
         revision_id = _mod_revision.ensure_null(revision_id)
         old_revno, old_revid = self.last_revision_info()
-        if self._get_append_revisions_only():
+        if self.get_append_revisions_only():
             self._check_history_violation(revision_id)
         self._run_pre_change_branch_tip_hooks(revno, revision_id)
         self._write_last_revision_info(revno, revision_id)
@@ -2653,6 +2718,16 @@
         self._transport.put_bytes('last-revision', out_string,
             mode=self.bzrdir._get_file_mode())
 
+    @needs_write_lock
+    def update_feature_flags(self, updated_flags):
+        """Update the feature flags for this branch.
+
+        :param updated_flags: Dictionary mapping feature names to necessities
+            A necessity can be None to indicate the feature should be removed
+        """
+        self._format._update_feature_flags(updated_flags)
+        self.control_transport.put_bytes('format', self._format.as_string())
+
 
 class FullHistoryBzrBranch(BzrBranch):
     """Bzr branch which contains the full revision history."""
@@ -2671,7 +2746,7 @@
         self._set_revision_history(history)
 
     def _read_last_revision_info(self):
-        rh = self.revision_history()
+        rh = self._revision_history()
         revno = len(rh)
         if revno:
             return (revno, rh[-1])
@@ -2731,7 +2806,7 @@
         if revision_id == _mod_revision.NULL_REVISION:
             new_history = []
         else:
-            new_history = self.revision_history()
+            new_history = self._revision_history()
         if revision_id is not None and new_history != []:
             try:
                 new_history = new_history[:new_history.index(revision_id) + 1]
@@ -2765,9 +2840,11 @@
 class BzrBranch8(BzrBranch):
     """A branch that stores tree-reference locations."""
 
-    def _open_hook(self):
+    def _open_hook(self, possible_transports=None):
         if self._ignore_fallbacks:
             return
+        if possible_transports is None:
+            possible_transports = [self.bzrdir.root_transport]
         try:
             url = self.get_stacked_on_url()
         except (errors.UnstackableRepositoryFormat, errors.NotStacked,
@@ -2781,7 +2858,8 @@
                     raise AssertionError(
                         "'transform_fallback_location' hook %s returned "
                         "None, not a URL." % hook_name)
-            self._activate_fallback_location(url)
+            self._activate_fallback_location(url,
+                possible_transports=possible_transports)
 
     def __init__(self, *args, **kwargs):
         self._ignore_fallbacks = kwargs.get('ignore_fallbacks', False)
@@ -2905,28 +2983,27 @@
         """See Branch.set_push_location."""
         self._master_branch_cache = None
         result = None
-        config = self.get_config()
+        conf = self.get_config_stack()
         if location is None:
-            if config.get_user_option('bound') != 'True':
+            if not conf.get('bound'):
                 return False
             else:
-                config.set_user_option('bound', 'False', warn_masked=True)
+                conf.set('bound', 'False')
                 return True
         else:
             self._set_config_location('bound_location', location,
-                                      config=config)
-            config.set_user_option('bound', 'True', warn_masked=True)
+                                      config=conf)
+            conf.set('bound', 'True')
         return True
 
     def _get_bound_location(self, bound):
         """Return the bound location in the config file.
 
         Return None if the bound parameter does not match"""
-        config = self.get_config()
-        config_bound = (config.get_user_option('bound') == 'True')
-        if config_bound != bound:
+        conf = self.get_config_stack()
+        if conf.get('bound') != bound:
             return None
-        return self._get_config_location('bound_location', config=config)
+        return self._get_config_location('bound_location', config=conf)
 
     def get_bound_location(self):
         """See Branch.set_push_location."""
@@ -2942,16 +3019,12 @@
         ## self._check_stackable_repo()
         # stacked_on_location is only ever defined in branch.conf, so don't
         # waste effort reading the whole stack of config files.
-        config = self.get_config()._get_branch_data_config()
+        conf = _mod_config.BranchOnlyStack(self)
         stacked_url = self._get_config_location('stacked_on_location',
-            config=config)
+                                                config=conf)
         if stacked_url is None:
             raise errors.NotStacked(self)
-        return stacked_url
-
-    def _get_append_revisions_only(self):
-        return self.get_config(
-            ).get_user_option_as_bool('append_revisions_only')
+        return stacked_url.encode('utf-8')
 
     @needs_read_lock
     def get_rev_id(self, revno, history=None):
@@ -2987,6 +3060,8 @@
             except errors.RevisionNotPresent, e:
                 raise errors.GhostRevisionsHaveNoRevno(revision_id, e.revision_id)
             index = len(self._partial_revision_history_cache) - 1
+            if index < 0:
+                raise errors.NoSuchRevision(self, revision_id)
             if self._partial_revision_history_cache[index] != revision_id:
                 raise errors.NoSuchRevision(self, revision_id)
         return self.revno() - index
@@ -3044,6 +3119,7 @@
     :ivar local_branch: target branch if there is a Master, else None
     :ivar target_branch: Target/destination branch object. (write locked)
     :ivar tag_conflicts: A list of tag conflicts, see BasicTags.merge_to
+    :ivar tag_updates: A dict with new tags, see BasicTags.merge_to
     """
 
     @deprecated_method(deprecated_in((2, 3, 0)))
@@ -3055,11 +3131,18 @@
         return self.new_revno - self.old_revno
 
     def report(self, to_file):
+        tag_conflicts = getattr(self, "tag_conflicts", None)
+        tag_updates = getattr(self, "tag_updates", None)
         if not is_quiet():
-            if self.old_revid == self.new_revid:
-                to_file.write('No revisions to pull.\n')
-            else:
+            if self.old_revid != self.new_revid:
                 to_file.write('Now on revision %d.\n' % self.new_revno)
+            if tag_updates:
+                to_file.write('%d tag(s) updated.\n' % len(tag_updates))
+            if self.old_revid == self.new_revid and not tag_updates:
+                if not tag_conflicts:
+                    to_file.write('No revisions or tags to pull.\n')
+                else:
+                    to_file.write('No revisions to pull.\n')
         self._show_tag_conficts(to_file)
 
 
@@ -3091,11 +3174,22 @@
         return self.new_revno - self.old_revno
 
     def report(self, to_file):
-        """Write a human-readable description of the result."""
-        if self.old_revid == self.new_revid:
-            note('No new revisions to push.')
-        else:
-            note('Pushed up to revision %d.' % self.new_revno)
+        # TODO: This function gets passed a to_file, but then
+        # ignores it and calls note() instead. This is also
+        # inconsistent with PullResult(), which writes to stdout.
+        # -- JRV20110901, bug #838853
+        tag_conflicts = getattr(self, "tag_conflicts", None)
+        tag_updates = getattr(self, "tag_updates", None)
+        if not is_quiet():
+            if self.old_revid != self.new_revid:
+                note(gettext('Pushed up to revision %d.') % self.new_revno)
+            if tag_updates:
+                note(ngettext('%d tag updated.', '%d tags updated.', len(tag_updates)) % len(tag_updates))
+            if self.old_revid == self.new_revid and not tag_updates:
+                if not tag_conflicts:
+                    note(gettext('No new revisions or tags to push.'))
+                else:
+                    note(gettext('No new revisions to push.'))
         self._show_tag_conficts(to_file)
 
 
@@ -3115,10 +3209,10 @@
         :param verbose: Requests more detailed display of what was checked,
             if any.
         """
-        note('checked branch %s format %s', self.branch.user_url,
-            self.branch._format)
+        note(gettext('checked branch {0} format {1}').format(
+                                self.branch.user_url, self.branch._format))
         for error in self.errors:
-            note('found error:%s', error)
+            note(gettext('found error:%s'), error)
 
 
 class Converter5to6(object):
@@ -3140,7 +3234,7 @@
 
         # Copying done; now update target format
         new_branch._transport.put_bytes('format',
-            format.get_format_string(),
+            format.as_string(),
             mode=new_branch.bzrdir._get_file_mode())
 
         # Clean up old files
@@ -3159,7 +3253,7 @@
         format = BzrBranchFormat7()
         branch._set_config_location('stacked_on_location', '')
         # update target format
-        branch._transport.put_bytes('format', format.get_format_string())
+        branch._transport.put_bytes('format', format.as_string())
 
 
 class Converter7to8(object):
@@ -3169,7 +3263,7 @@
         format = BzrBranchFormat8()
         branch._transport.put_bytes('references', '')
         # update target format
-        branch._transport.put_bytes('format', format.get_format_string())
+        branch._transport.put_bytes('format', format.as_string())
 
 
 class InterBranch(InterObject):
@@ -3409,8 +3503,8 @@
             self._update_revisions(stop_revision, overwrite=overwrite,
                     graph=graph)
         if self.source._push_should_merge_tags():
-            result.tag_conflicts = self.source.tags.merge_to(self.target.tags,
-                overwrite)
+            result.tag_updates, result.tag_conflicts = (
+                self.source.tags.merge_to(self.target.tags, overwrite))
         result.new_revno, result.new_revid = self.target.last_revision_info()
         return result
 
@@ -3499,8 +3593,9 @@
             # TODO: The old revid should be specified when merging tags, 
             # so a tags implementation that versions tags can only 
             # pull in the most recent changes. -- JRV20090506
-            result.tag_conflicts = self.source.tags.merge_to(self.target.tags,
-                overwrite, ignore_master=not merge_tags_to_master)
+            result.tag_updates, result.tag_conflicts = (
+                self.source.tags.merge_to(self.target.tags, overwrite,
+                    ignore_master=not merge_tags_to_master))
             result.new_revno, result.new_revid = self.target.last_revision_info()
             if _hook_master:
                 result.master_branch = _hook_master

=== modified file 'bzrlib/branchbuilder.py'
--- bzrlib/branchbuilder.py	2011-07-15 09:22:16 +0000
+++ bzrlib/branchbuilder.py	2011-12-19 13:23:58 +0000
@@ -16,8 +16,10 @@
 
 """Utility for create branches with particular contents."""
 
+from __future__ import absolute_import
+
 from bzrlib import (
-    bzrdir,
+    controldir,
     commit,
     errors,
     memorytree,
@@ -64,7 +66,7 @@
             If the path of the transport does not exist but its parent does
             it will be created.
         :param format: Either a BzrDirFormat, or the name of a format in the
-            bzrdir format registry for the branch to be built.
+            controldir format registry for the branch to be built.
         :param branch: An already constructed branch to use.  This param is
             mutually exclusive with the transport and format params.
         """
@@ -82,12 +84,13 @@
             if format is None:
                 format = 'default'
             if isinstance(format, str):
-                format = bzrdir.format_registry.make_bzrdir(format)
-            self._branch = bzrdir.BzrDir.create_branch_convenience(
+                format = controldir.format_registry.make_bzrdir(format)
+            self._branch = controldir.ControlDir.create_branch_convenience(
                 transport.base, format=format, force_new_tree=False)
         self._tree = None
 
-    def build_commit(self, **commit_kwargs):
+    def build_commit(self, parent_ids=None, allow_leftmost_as_ghost=False,
+                     **commit_kwargs):
         """Build a commit on the branch.
 
         This makes a commit with no real file content for when you only want
@@ -96,9 +99,20 @@
         :param commit_kwargs: Arguments to pass through to commit, such as
              timestamp.
         """
+        if parent_ids is not None:
+            if len(parent_ids) == 0:
+                base_id = revision.NULL_REVISION
+            else:
+                base_id = parent_ids[0]
+            if base_id != self._branch.last_revision():
+                self._move_branch_pointer(base_id,
+                    allow_leftmost_as_ghost=allow_leftmost_as_ghost)
         tree = memorytree.MemoryTree.create_on_branch(self._branch)
         tree.lock_write()
         try:
+            if parent_ids is not None:
+                tree.set_parent_ids(parent_ids,
+                    allow_leftmost_as_ghost=allow_leftmost_as_ghost)
             tree.add('')
             return self._do_commit(tree, **commit_kwargs)
         finally:

=== modified file 'bzrlib/breakin.py'
--- bzrlib/breakin.py	2011-02-19 17:37:45 +0000
+++ bzrlib/breakin.py	2011-12-18 12:46:49 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 import os
 import signal
 

=== modified file 'bzrlib/btree_index.py'
--- bzrlib/btree_index.py	2011-08-19 22:34:02 +0000
+++ bzrlib/btree_index.py	2011-12-19 13:23:58 +0000
@@ -17,6 +17,8 @@
 
 """B+Tree indices"""
 
+from __future__ import absolute_import
+
 import cStringIO
 
 from bzrlib.lazy_import import lazy_import
@@ -294,8 +296,10 @@
             flag when writing out. This is used by the _spill_mem_keys_to_disk
             functionality.
         """
+        new_leaf = False
         if rows[-1].writer is None:
             # opening a new leaf chunk;
+            new_leaf = True
             for pos, internal_row in enumerate(rows[:-1]):
                 # flesh out any internal nodes that are needed to
                 # preserve the height of the tree
@@ -320,6 +324,11 @@
                 optimize_for_size=self._optimize_for_size)
             rows[-1].writer.write(_LEAF_FLAG)
         if rows[-1].writer.write(line):
+            # if we failed to write, despite having an empty page to write to,
+            # then line is too big. raising the error avoids infinite recursion
+            # searching for a suitably large page that will not be found.
+            if new_leaf:
+                raise errors.BadIndexKey(string_key)
             # this key did not fit in the node:
             rows[-1].finish_node()
             key_line = string_key + "\n"

=== modified file 'bzrlib/bugtracker.py'
--- bzrlib/bugtracker.py	2011-02-25 12:12:39 +0000
+++ bzrlib/bugtracker.py	2011-12-18 12:46:49 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 from bzrlib import registry
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
@@ -48,8 +50,14 @@
 
     bzr commit --fixes <tracker>:<id>
 
+or::
+
+    bzr commit --fixes <id>
+
 where "<tracker>" is an identifier for the bug tracker, and "<id>" is the
 identifier for that bug within the bugtracker, usually the bug number.
+If "<tracker>" is not specified the ``bugtracker`` set in the branch
+or global configuration is used.
 
 Bazaar knows about a few bug trackers that have many users. If
 you use one of these bug trackers then there is no setup required to

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2011-09-01 20:46:47 +0000
+++ bzrlib/builtins.py	2012-01-05 09:50:04 +0000
@@ -16,11 +16,16 @@
 
 """builtin bzr commands"""
 
+from __future__ import absolute_import
+
 import os
 
+import bzrlib.bzrdir
+
 from bzrlib import lazy_import
 lazy_import.lazy_import(globals(), """
 import cStringIO
+import errno
 import sys
 import time
 
@@ -29,7 +34,7 @@
     bugtracker,
     bundle,
     btree_index,
-    bzrdir,
+    controldir,
     directory_service,
     delta,
     config as _mod_config,
@@ -57,6 +62,7 @@
 from bzrlib.revisionspec import RevisionSpec, RevisionInfo
 from bzrlib.smtp_connection import SMTPConnection
 from bzrlib.workingtree import WorkingTree
+from bzrlib.i18n import gettext, ngettext
 """)
 
 from bzrlib.commands import (
@@ -112,7 +118,7 @@
             if view_files:
                 file_list = view_files
                 view_str = views.view_display_str(view_files)
-                note("Ignoring files outside view. View is %s" % view_str)
+                note(gettext("Ignoring files outside view. View is %s") % view_str)
     return tree, file_list
 
 
@@ -120,8 +126,8 @@
     if revisions is None:
         return None
     if len(revisions) != 1:
-        raise errors.BzrCommandError(
-            'bzr %s --revision takes exactly one revision identifier' % (
+        raise errors.BzrCommandError(gettext(
+            'bzr %s --revision takes exactly one revision identifier') % (
                 command_name,))
     return revisions[0]
 
@@ -196,7 +202,7 @@
     the --directory option is used to specify a different branch."""
     if directory is not None:
         return (None, Branch.open(directory), filename)
-    return bzrdir.BzrDir.open_containing_tree_or_branch(filename)
+    return controldir.ControlDir.open_containing_tree_or_branch(filename)
 
 
 # TODO: Make sure no commands unconditionally use the working directory as a
@@ -288,8 +294,8 @@
         from bzrlib.status import show_tree_status
 
         if revision and len(revision) > 2:
-            raise errors.BzrCommandError('bzr status --revision takes exactly'
-                                         ' one or two revision specifiers')
+            raise errors.BzrCommandError(gettext('bzr status --revision takes exactly'
+                                         ' one or two revision specifiers'))
 
         tree, relfile_list = WorkingTree.open_containing_paths(file_list)
         # Avoid asking for specific files when that is not needed.
@@ -332,18 +338,18 @@
     @display_command
     def run(self, revision_id=None, revision=None, directory=u'.'):
         if revision_id is not None and revision is not None:
-            raise errors.BzrCommandError('You can only supply one of'
-                                         ' revision_id or --revision')
+            raise errors.BzrCommandError(gettext('You can only supply one of'
+                                         ' revision_id or --revision'))
         if revision_id is None and revision is None:
-            raise errors.BzrCommandError('You must supply either'
-                                         ' --revision or a revision_id')
+            raise errors.BzrCommandError(gettext('You must supply either'
+                                         ' --revision or a revision_id'))
 
-        b = bzrdir.BzrDir.open_containing_tree_or_branch(directory)[1]
+        b = controldir.ControlDir.open_containing_tree_or_branch(directory)[1]
 
         revisions = b.repository.revisions
         if revisions is None:
-            raise errors.BzrCommandError('Repository %r does not support '
-                'access to raw revision texts')
+            raise errors.BzrCommandError(gettext('Repository %r does not support '
+                'access to raw revision texts'))
 
         b.repository.lock_read()
         try:
@@ -353,14 +359,14 @@
                 try:
                     self.print_revision(revisions, revision_id)
                 except errors.NoSuchRevision:
-                    msg = "The repository %s contains no revision %s." % (
+                    msg = gettext("The repository {0} contains no revision {1}.").format(
                         b.repository.base, revision_id)
                     raise errors.BzrCommandError(msg)
             elif revision is not None:
                 for rev in revision:
                     if rev is None:
                         raise errors.BzrCommandError(
-                            'You cannot specify a NULL revision.')
+                            gettext('You cannot specify a NULL revision.'))
                     rev_id = rev.as_revision_id(b)
                     self.print_revision(revisions, rev_id)
         finally:
@@ -472,15 +478,15 @@
             location_list=['.']
 
         for location in location_list:
-            d = bzrdir.BzrDir.open(location)
-            
+            d = controldir.ControlDir.open(location)
+
             try:
                 working = d.open_workingtree()
             except errors.NoWorkingTree:
-                raise errors.BzrCommandError("No working tree to remove")
+                raise errors.BzrCommandError(gettext("No working tree to remove"))
             except errors.NotLocalUrl:
-                raise errors.BzrCommandError("You cannot remove the working tree"
-                                             " of a remote path")
+                raise errors.BzrCommandError(gettext("You cannot remove the working tree"
+                                             " of a remote path"))
             if not force:
                 if (working.has_changes()):
                     raise errors.UncommittedChanges(working)
@@ -488,8 +494,8 @@
                     raise errors.ShelvedChanges(working)
 
             if working.user_url != working.branch.user_url:
-                raise errors.BzrCommandError("You cannot remove the working tree"
-                                             " from a lightweight checkout")
+                raise errors.BzrCommandError(gettext("You cannot remove the working tree"
+                                             " from a lightweight checkout"))
 
             d.destroy_workingtree()
 
@@ -527,10 +533,10 @@
                 pass # There seems to be a real error here, so we'll reset
             else:
                 # Refuse
-                raise errors.BzrCommandError(
+                raise errors.BzrCommandError(gettext(
                     'The tree does not appear to be corrupt. You probably'
                     ' want "bzr revert" instead. Use "--force" if you are'
-                    ' sure you want to reset the working tree.')
+                    ' sure you want to reset the working tree.'))
         if revision is None:
             revision_ids = None
         else:
@@ -539,12 +545,11 @@
             tree.reset_state(revision_ids)
         except errors.BzrError, e:
             if revision_ids is None:
-                extra = (', the header appears corrupt, try passing -r -1'
-                         ' to set the state to the last commit')
+                extra = (gettext(', the header appears corrupt, try passing -r -1'
+                         ' to set the state to the last commit'))
             else:
                 extra = ''
-            raise errors.BzrCommandError('failed to reset the tree state'
-                                         + extra)
+            raise errors.BzrCommandError(gettext('failed to reset the tree state{0}').format(extra))
 
 
 class cmd_revno(Command):
@@ -556,29 +561,42 @@
     _see_also = ['info']
     takes_args = ['location?']
     takes_options = [
-        Option('tree', help='Show revno of working tree'),
+        Option('tree', help='Show revno of working tree.'),
+        'revision',
         ]
 
     @display_command
-    def run(self, tree=False, location=u'.'):
+    def run(self, tree=False, location=u'.', revision=None):
+        if revision is not None and tree:
+            raise errors.BzrCommandError(gettext("--tree and --revision can "
+                "not be used together"))
+
         if tree:
             try:
                 wt = WorkingTree.open_containing(location)[0]
                 self.add_cleanup(wt.lock_read().unlock)
             except (errors.NoWorkingTree, errors.NotLocalUrl):
                 raise errors.NoWorkingTree(location)
+            b = wt.branch
             revid = wt.last_revision()
-            try:
-                revno_t = wt.branch.revision_id_to_dotted_revno(revid)
-            except errors.NoSuchRevision:
-                revno_t = ('???',)
-            revno = ".".join(str(n) for n in revno_t)
         else:
             b = Branch.open_containing(location)[0]
             self.add_cleanup(b.lock_read().unlock)
-            revno = b.revno()
+            if revision:
+                if len(revision) != 1:
+                    raise errors.BzrCommandError(gettext(
+                        "Tags can only be placed on a single revision, "
+                        "not on a range"))
+                revid = revision[0].as_revision_id(b)
+            else:
+                revid = b.last_revision()
+        try:
+            revno_t = b.revision_id_to_dotted_revno(revid)
+        except errors.NoSuchRevision:
+            revno_t = ('???',)
+        revno = ".".join(str(n) for n in revno_t)
         self.cleanup_now()
-        self.outf.write(str(revno) + '\n')
+        self.outf.write(revno + '\n')
 
 
 class cmd_revision_info(Command):
@@ -591,7 +609,7 @@
         custom_help('directory',
             help='Branch to examine, '
                  'rather than the one containing the working directory.'),
-        Option('tree', help='Show revno of working tree'),
+        Option('tree', help='Show revno of working tree.'),
         ]
 
     @display_command
@@ -653,6 +671,9 @@
     are added.  This search proceeds recursively into versioned
     directories.  If no names are given '.' is assumed.
 
+    A warning will be printed when nested trees are encountered,
+    unless they are explicitly ignored.
+
     Therefore simply saying 'bzr add' will version all files that
     are currently unknown.
 
@@ -723,8 +744,9 @@
             if verbose:
                 for glob in sorted(ignored.keys()):
                     for path in ignored[glob]:
-                        self.outf.write("ignored %s matching \"%s\"\n"
-                                        % (path, glob))
+                        self.outf.write(
+                         gettext("ignored {0} matching \"{1}\"\n").format(
+                         path, glob))
 
 
 class cmd_mkdir(Command):
@@ -734,19 +756,44 @@
     """
 
     takes_args = ['dir+']
+    takes_options = [
+        Option(
+            'parents',
+            help='No error if existing, make parent directories as needed.',
+            short_name='p'
+            )
+        ]
     encoding_type = 'replace'
 
-    def run(self, dir_list):
-        for d in dir_list:
-            wt, dd = WorkingTree.open_containing(d)
-            base = os.path.dirname(dd)
-            id = wt.path2id(base)
-            if id != None:
-                os.mkdir(d)
-                wt.add([dd])
-                self.outf.write('added %s\n' % d)
+    @classmethod
+    def add_file_with_parents(cls, wt, relpath):
+        if wt.path2id(relpath) is not None:
+            return
+        cls.add_file_with_parents(wt, osutils.dirname(relpath))
+        wt.add([relpath])
+
+    @classmethod
+    def add_file_single(cls, wt, relpath):
+        wt.add([relpath])
+
+    def run(self, dir_list, parents=False):
+        if parents:
+            add_file = self.add_file_with_parents
+        else:
+            add_file = self.add_file_single
+        for dir in dir_list:
+            wt, relpath = WorkingTree.open_containing(dir)
+            if parents:
+                try:
+                    os.makedirs(dir)
+                except OSError, e:
+                    if e.errno != errno.EEXIST:
+                        raise
             else:
-                raise errors.NotVersionedError(path=base)
+                os.mkdir(dir)
+            add_file(wt, relpath)
+            if not is_quiet():
+                self.outf.write(gettext('added %s\n') % dir)
 
 
 class cmd_relpath(Command):
@@ -788,7 +835,7 @@
     @display_command
     def run(self, revision=None, show_ids=False, kind=None, file_list=None):
         if kind and kind not in ['file', 'directory', 'symlink']:
-            raise errors.BzrCommandError('invalid kind %r specified' % (kind,))
+            raise errors.BzrCommandError(gettext('invalid kind %r specified') % (kind,))
 
         revision = _get_one_revision('inventory', revision)
         work_tree, file_list = WorkingTree.open_containing_paths(file_list)
@@ -858,22 +905,25 @@
         if auto:
             return self.run_auto(names_list, after, dry_run)
         elif dry_run:
-            raise errors.BzrCommandError('--dry-run requires --auto.')
+            raise errors.BzrCommandError(gettext('--dry-run requires --auto.'))
         if names_list is None:
             names_list = []
         if len(names_list) < 2:
-            raise errors.BzrCommandError("missing file argument")
+            raise errors.BzrCommandError(gettext("missing file argument"))
         tree, rel_names = WorkingTree.open_containing_paths(names_list, canonicalize=False)
+        for file_name in rel_names[0:-1]:
+            if file_name == '':
+                raise errors.BzrCommandError(gettext("can not move root of branch"))
         self.add_cleanup(tree.lock_tree_write().unlock)
         self._run(tree, names_list, rel_names, after)
 
     def run_auto(self, names_list, after, dry_run):
         if names_list is not None and len(names_list) > 1:
-            raise errors.BzrCommandError('Only one path may be specified to'
-                                         ' --auto.')
+            raise errors.BzrCommandError(gettext('Only one path may be specified to'
+                                         ' --auto.'))
         if after:
-            raise errors.BzrCommandError('--after cannot be specified with'
-                                         ' --auto.')
+            raise errors.BzrCommandError(gettext('--after cannot be specified with'
+                                         ' --auto.'))
         work_tree, file_list = WorkingTree.open_containing_paths(
             names_list, default_directory='.')
         self.add_cleanup(work_tree.lock_tree_write().unlock)
@@ -909,9 +959,9 @@
                     self.outf.write("%s => %s\n" % (src, dest))
         else:
             if len(names_list) != 2:
-                raise errors.BzrCommandError('to mv multiple files the'
+                raise errors.BzrCommandError(gettext('to mv multiple files the'
                                              ' destination must be a versioned'
-                                             ' directory')
+                                             ' directory'))
 
             # for cicp file-systems: the src references an existing inventory
             # item:
@@ -981,6 +1031,10 @@
     location to use the default.  To change the default, use --remember. The
     value will only be saved if the remote location can be accessed.
 
+    The --verbose option will display the revisions pulled using the log_format
+    configuration option. You can use a different format by overriding it with
+    -Olog_format=<other_format>.
+
     Note: The location can be specified either in the form of a branch,
     or in the form of a path to a file containing a merge directive generated
     with bzr send.
@@ -1023,7 +1077,7 @@
             self.add_cleanup(branch_to.lock_write().unlock)
 
         if tree_to is None and show_base:
-            raise errors.BzrCommandError("Need working tree for --show-base.")
+            raise errors.BzrCommandError(gettext("Need working tree for --show-base."))
 
         if local and not branch_to.get_bound_location():
             raise errors.LocalRequiresBoundBranch()
@@ -1039,20 +1093,20 @@
         stored_loc = branch_to.get_parent()
         if location is None:
             if stored_loc is None:
-                raise errors.BzrCommandError("No pull location known or"
-                                             " specified.")
+                raise errors.BzrCommandError(gettext("No pull location known or"
+                                             " specified."))
             else:
                 display_url = urlutils.unescape_for_display(stored_loc,
                         self.outf.encoding)
                 if not is_quiet():
-                    self.outf.write("Using saved parent location: %s\n" % display_url)
+                    self.outf.write(gettext("Using saved parent location: %s\n") % display_url)
                 location = stored_loc
 
         revision = _get_one_revision('pull', revision)
         if mergeable is not None:
             if revision is not None:
-                raise errors.BzrCommandError(
-                    'Cannot use -r with merge directives or bundles')
+                raise errors.BzrCommandError(gettext(
+                    'Cannot use -r with merge directives or bundles'))
             mergeable.install_revisions(branch_to.repository)
             base_revision_id, revision_id, verified = \
                 mergeable.get_merge_request(branch_to.repository)
@@ -1116,6 +1170,10 @@
     --no-remember to avoid setting it).  After that, you can omit the
     location to use the default.  To change the default, use --remember. The
     value will only be saved if the remote location can be accessed.
+
+    The --verbose option will display the revisions pushed using the log_format
+    configuration option. You can use a different format by overriding it with
+    -Olog_format=<other_format>.
     """
 
     _see_also = ['pull', 'update', 'working-trees']
@@ -1159,7 +1217,7 @@
             directory = '.'
         # Get the source branch
         (tree, br_from,
-         _unused) = bzrdir.BzrDir.open_containing_tree_or_branch(directory)
+         _unused) = controldir.ControlDir.open_containing_tree_or_branch(directory)
         # Get the tip's revision_id
         revision = _get_one_revision('push', revision)
         if revision is not None:
@@ -1186,19 +1244,27 @@
                     # error by the feedback given to them. RBC 20080227.
                     stacked_on = parent_url
             if not stacked_on:
-                raise errors.BzrCommandError(
-                    "Could not determine branch to refer to.")
+                raise errors.BzrCommandError(gettext(
+                    "Could not determine branch to refer to."))
 
         # Get the destination location
         if location is None:
             stored_loc = br_from.get_push_location()
             if stored_loc is None:
-                raise errors.BzrCommandError(
-                    "No push location known or specified.")
+                parent_loc = br_from.get_parent()
+                if parent_loc:
+                    raise errors.BzrCommandError(gettext(
+                        "No push location known or specified. To push to the "
+                        "parent branch (at %s), use 'bzr push :parent'." %
+                        urlutils.unescape_for_display(parent_loc,
+                            self.outf.encoding)))
+                else:
+                    raise errors.BzrCommandError(gettext(
+                        "No push location known or specified."))
             else:
                 display_url = urlutils.unescape_for_display(stored_loc,
                         self.outf.encoding)
-                note("Using saved push location: %s" % display_url)
+                note(gettext("Using saved push location: %s") % display_url)
                 location = stored_loc
 
         _show_push_branch(br_from, revision_id, location, self.outf,
@@ -1262,7 +1328,7 @@
                 deprecated_name=self.invoked_as,
                 recommended_name='branch',
                 deprecated_in_version='2.4')
-        accelerator_tree, br_from = bzrdir.BzrDir.open_tree_or_branch(
+        accelerator_tree, br_from = controldir.ControlDir.open_tree_or_branch(
             from_location)
         if not (hardlink or files_from):
             # accelerator_tree is usually slower because you have to read N
@@ -1281,77 +1347,126 @@
             # RBC 20060209
             revision_id = br_from.last_revision()
         if to_location is None:
-            to_location = urlutils.derive_to_location(from_location)
+            to_location = getattr(br_from, "name", None)
+            if to_location is None:
+                to_location = urlutils.derive_to_location(from_location)
         to_transport = transport.get_transport(to_location)
         try:
             to_transport.mkdir('.')
         except errors.FileExists:
-            if not use_existing_dir:
-                raise errors.BzrCommandError('Target directory "%s" '
-                    'already exists.' % to_location)
+            try:
+                to_dir = controldir.ControlDir.open_from_transport(
+                    to_transport)
+            except errors.NotBranchError:
+                if not use_existing_dir:
+                    raise errors.BzrCommandError(gettext('Target directory "%s" '
+                        'already exists.') % to_location)
+                else:
+                    to_dir = None
             else:
                 try:
-                    bzrdir.BzrDir.open_from_transport(to_transport)
+                    to_dir.open_branch()
                 except errors.NotBranchError:
                     pass
                 else:
                     raise errors.AlreadyBranchError(to_location)
         except errors.NoSuchFile:
-            raise errors.BzrCommandError('Parent of "%s" does not exist.'
+            raise errors.BzrCommandError(gettext('Parent of "%s" does not exist.')
                                          % to_location)
-        try:
-            # preserve whatever source format we have.
-            dir = br_from.bzrdir.sprout(to_transport.base, revision_id,
-                                        possible_transports=[to_transport],
-                                        accelerator_tree=accelerator_tree,
-                                        hardlink=hardlink, stacked=stacked,
-                                        force_new_repo=standalone,
-                                        create_tree_if_local=not no_tree,
-                                        source_branch=br_from)
-            branch = dir.open_branch()
-        except errors.NoSuchRevision:
-            to_transport.delete_tree('.')
-            msg = "The branch %s has no revision %s." % (from_location,
-                revision)
-            raise errors.BzrCommandError(msg)
+        else:
+            to_dir = None
+        if to_dir is None:
+            try:
+                # preserve whatever source format we have.
+                to_dir = br_from.bzrdir.sprout(to_transport.base, revision_id,
+                                            possible_transports=[to_transport],
+                                            accelerator_tree=accelerator_tree,
+                                            hardlink=hardlink, stacked=stacked,
+                                            force_new_repo=standalone,
+                                            create_tree_if_local=not no_tree,
+                                            source_branch=br_from)
+                branch = to_dir.open_branch(
+                    possible_transports=[
+                        br_from.bzrdir.root_transport, to_transport])
+            except errors.NoSuchRevision:
+                to_transport.delete_tree('.')
+                msg = gettext("The branch {0} has no revision {1}.").format(
+                    from_location, revision)
+                raise errors.BzrCommandError(msg)
+        else:
+            branch = br_from.sprout(to_dir, revision_id=revision_id)
         _merge_tags_if_possible(br_from, branch)
         # If the source branch is stacked, the new branch may
         # be stacked whether we asked for that explicitly or not.
         # We therefore need a try/except here and not just 'if stacked:'
         try:
-            note('Created new stacked branch referring to %s.' %
+            note(gettext('Created new stacked branch referring to %s.') %
                 branch.get_stacked_on_url())
         except (errors.NotStacked, errors.UnstackableBranchFormat,
             errors.UnstackableRepositoryFormat), e:
-            note('Branched %d revision(s).' % branch.revno())
+            note(ngettext('Branched %d revision.', 'Branched %d revisions.', branch.revno()) % branch.revno())
         if bind:
             # Bind to the parent
             parent_branch = Branch.open(from_location)
             branch.bind(parent_branch)
-            note('New branch bound to %s' % from_location)
+            note(gettext('New branch bound to %s') % from_location)
         if switch:
             # Switch to the new branch
             wt, _ = WorkingTree.open_containing('.')
             _mod_switch.switch(wt.bzrdir, branch)
-            note('Switched to branch: %s',
+            note(gettext('Switched to branch: %s'),
                 urlutils.unescape_for_display(branch.base, 'utf-8'))
 
 
 class cmd_branches(Command):
     __doc__ = """List the branches available at the current location.
 
-    This command will print the names of all the branches at the current location.
+    This command will print the names of all the branches at the current
+    location.
     """
 
     takes_args = ['location?']
+    takes_options = [
+                  Option('recursive', short_name='R',
+                         help='Recursively scan for branches rather than '
+                              'just looking in the specified location.')]
 
-    def run(self, location="."):
-        dir = bzrdir.BzrDir.open_containing(location)[0]
-        for branch in dir.list_branches():
-            if branch.name is None:
-                self.outf.write(" (default)\n")
-            else:
-                self.outf.write(" %s\n" % branch.name.encode(self.outf.encoding))
+    def run(self, location=".", recursive=False):
+        if recursive:
+            t = transport.get_transport(location)
+            if not t.listable():
+                raise errors.BzrCommandError(
+                    "Can't scan this type of location.")
+            for b in controldir.ControlDir.find_branches(t):
+                self.outf.write("%s\n" % urlutils.unescape_for_display(
+                    urlutils.relative_url(t.base, b.base),
+                    self.outf.encoding).rstrip("/"))
+        else:
+            dir = controldir.ControlDir.open_containing(location)[0]
+            try:
+                active_branch = dir.open_branch(name=None)
+            except errors.NotBranchError:
+                active_branch = None
+            branches = dir.get_branches()
+            names = {}
+            for name, branch in branches.iteritems():
+                if name is None:
+                    continue
+                active = (active_branch is not None and
+                          active_branch.base == branch.base)
+                names[name] = active
+            # Only mention the current branch explicitly if it's not
+            # one of the colocated branches
+            if not any(names.values()) and active_branch is not None:
+                self.outf.write("* %s\n" % gettext("(default)"))
+            for name in sorted(names.keys()):
+                active = names[name]
+                if active:
+                    prefix = "*"
+                else:
+                    prefix = " "
+                self.outf.write("%s %s\n" % (
+                    prefix, name.encode(self.outf.encoding)))
 
 
 class cmd_checkout(Command):
@@ -1398,7 +1513,7 @@
         if branch_location is None:
             branch_location = osutils.getcwd()
             to_location = branch_location
-        accelerator_tree, source = bzrdir.BzrDir.open_tree_or_branch(
+        accelerator_tree, source = controldir.ControlDir.open_tree_or_branch(
             branch_location)
         if not (hardlink or files_from):
             # accelerator_tree is usually slower because you have to read N
@@ -1459,20 +1574,33 @@
 
 
 class cmd_update(Command):
-    __doc__ = """Update a tree to have the latest code committed to its branch.
-
-    This will perform a merge into the working tree, and may generate
-    conflicts. If you have any local changes, you will still
-    need to commit them after the update for the update to be complete.
-
-    If you want to discard your local changes, you can just do a
-    'bzr revert' instead of 'bzr commit' after the update.
-
-    If you want to restore a file that has been removed locally, use
-    'bzr revert' instead of 'bzr update'.
-
-    If the tree's branch is bound to a master branch, it will also update
+    __doc__ = """Update a working tree to a new revision.
+
+    This will perform a merge of the destination revision (the tip of the
+    branch, or the specified revision) into the working tree, and then make
+    that revision the basis revision for the working tree.  
+
+    You can use this to visit an older revision, or to update a working tree
+    that is out of date from its branch.
+    
+    If there are any uncommitted changes in the tree, they will be carried
+    across and remain as uncommitted changes after the update.  To discard
+    these changes, use 'bzr revert'.  The uncommitted changes may conflict
+    with the changes brought in by the change in basis revision.
+
+    If the tree's branch is bound to a master branch, bzr will also update
     the branch from the master.
+
+    You cannot update just a single file or directory, because each Bazaar
+    working tree has just a single basis revision.  If you want to restore a
+    file that has been removed locally, use 'bzr revert' instead of 'bzr
+    update'.  If you want to restore a file to its state in a previous
+    revision, use 'bzr revert' with a '-r' option, or use 'bzr cat' to write
+    out the old content of that file to a new location.
+
+    The 'dir' argument, if given, must be the location of the root of a
+    working tree to update.  By default, the working tree that contains the 
+    current working directory is used.
     """
 
     _see_also = ['pull', 'working-trees', 'status-flags']
@@ -1483,11 +1611,19 @@
                      ]
     aliases = ['up']
 
-    def run(self, dir='.', revision=None, show_base=None):
+    def run(self, dir=None, revision=None, show_base=None):
         if revision is not None and len(revision) != 1:
-            raise errors.BzrCommandError(
-                        "bzr update --revision takes exactly one revision")
-        tree = WorkingTree.open_containing(dir)[0]
+            raise errors.BzrCommandError(gettext(
+                "bzr update --revision takes exactly one revision"))
+        if dir is None:
+            tree = WorkingTree.open_containing('.')[0]
+        else:
+            tree, relpath = WorkingTree.open_containing(dir)
+            if relpath:
+                # See bug 557886.
+                raise errors.BzrCommandError(gettext(
+                    "bzr update can only update a whole tree, "
+                    "not a file or subdirectory"))
         branch = tree.branch
         possible_transports = []
         master = branch.get_master_branch(
@@ -1517,8 +1653,8 @@
             revision_id = branch.last_revision()
         if revision_id == _mod_revision.ensure_null(tree.last_revision()):
             revno = branch.revision_id_to_dotted_revno(revision_id)
-            note("Tree is up to date at revision %s of branch %s" %
-                ('.'.join(map(str, revno)), branch_location))
+            note(gettext("Tree is up to date at revision {0} of branch {1}"
+                        ).format('.'.join(map(str, revno)), branch_location))
             return 0
         view_info = _get_view_info_for_change_reporter(tree)
         change_reporter = delta._ChangeReporter(
@@ -1532,19 +1668,19 @@
                 old_tip=old_tip,
                 show_base=show_base)
         except errors.NoSuchRevision, e:
-            raise errors.BzrCommandError(
+            raise errors.BzrCommandError(gettext(
                                   "branch has no revision %s\n"
                                   "bzr update --revision only works"
-                                  " for a revision in the branch history"
+                                  " for a revision in the branch history")
                                   % (e.revision))
         revno = tree.branch.revision_id_to_dotted_revno(
             _mod_revision.ensure_null(tree.last_revision()))
-        note('Updated to revision %s of branch %s' %
-             ('.'.join(map(str, revno)), branch_location))
+        note(gettext('Updated to revision {0} of branch {1}').format(
+             '.'.join(map(str, revno)), branch_location))
         parent_ids = tree.get_parent_ids()
         if parent_ids[1:] and parent_ids[1:] != existing_pending_merges:
-            note('Your local commits will now show as pending merges with '
-                 "'bzr status', and can be committed with 'bzr commit'.")
+            note(gettext('Your local commits will now show as pending merges with '
+                 "'bzr status', and can be committed with 'bzr commit'."))
         if conflicts != 0:
             return 1
         else:
@@ -1591,7 +1727,7 @@
         else:
             noise_level = 0
         from bzrlib.info import show_bzrdir_info
-        show_bzrdir_info(bzrdir.BzrDir.open_containing(location)[0],
+        show_bzrdir_info(controldir.ControlDir.open_containing(location)[0],
                          verbose=noise_level, outfile=self.outf)
 
 
@@ -1622,8 +1758,8 @@
     def run(self, file_list, verbose=False, new=False,
         file_deletion_strategy='safe'):
         if file_deletion_strategy == 'force':
-            note("(The --force option is deprecated, rather use --no-backup "
-                "in future.)")
+            note(gettext("(The --force option is deprecated, rather use --no-backup "
+                "in future.)"))
             file_deletion_strategy = 'no-backup'
 
         tree, file_list = WorkingTree.open_containing_paths(file_list)
@@ -1639,7 +1775,7 @@
                 specific_files=file_list).added
             file_list = sorted([f[0] for f in added], reverse=True)
             if len(file_list) == 0:
-                raise errors.BzrCommandError('No matching files.')
+                raise errors.BzrCommandError(gettext('No matching files.'))
         elif file_list is None:
             # missing files show up in iter_changes(basis) as
             # versioned-with-no-kind.
@@ -1729,7 +1865,7 @@
 
     def run(self, branch=".", canonicalize_chks=False):
         from bzrlib.reconcile import reconcile
-        dir = bzrdir.BzrDir.open(branch)
+        dir = controldir.ControlDir.open(branch)
         reconcile(dir, canonicalize_chks=canonicalize_chks)
 
 
@@ -1744,7 +1880,11 @@
     @display_command
     def run(self, location="."):
         branch = Branch.open_containing(location)[0]
-        for revid in branch.revision_history():
+        self.add_cleanup(branch.lock_read().unlock)
+        graph = branch.repository.get_graph()
+        history = list(graph.iter_lefthand_ancestry(branch.last_revision(),
+            [_mod_revision.NULL_REVISION]))
+        for revid in reversed(history):
             self.outf.write(revid)
             self.outf.write('\n')
 
@@ -1811,7 +1951,7 @@
                 help='Specify a format for this branch. '
                 'See "help formats".',
                 lazy_registry=('bzrlib.bzrdir', 'format_registry'),
-                converter=lambda name: bzrdir.format_registry.make_bzrdir(name),
+                converter=lambda name: controldir.format_registry.make_bzrdir(name),
                 value_switches=True,
                 title="Branch format",
                 ),
@@ -1824,7 +1964,7 @@
     def run(self, location=None, format=None, append_revisions_only=False,
             create_prefix=False, no_tree=False):
         if format is None:
-            format = bzrdir.format_registry.make_bzrdir('default')
+            format = controldir.format_registry.make_bzrdir('default')
         if location is None:
             location = u'.'
 
@@ -1839,18 +1979,18 @@
             to_transport.ensure_base()
         except errors.NoSuchFile:
             if not create_prefix:
-                raise errors.BzrCommandError("Parent directory of %s"
+                raise errors.BzrCommandError(gettext("Parent directory of %s"
                     " does not exist."
                     "\nYou may supply --create-prefix to create all"
-                    " leading parent directories."
+                    " leading parent directories.")
                     % location)
             to_transport.create_prefix()
 
         try:
-            a_bzrdir = bzrdir.BzrDir.open_from_transport(to_transport)
+            a_bzrdir = controldir.ControlDir.open_from_transport(to_transport)
         except errors.NotBranchError:
             # really a NotBzrDir error...
-            create_branch = bzrdir.BzrDir.create_branch_convenience
+            create_branch = controldir.ControlDir.create_branch_convenience
             if no_tree:
                 force_new_tree = False
             else:
@@ -1867,14 +2007,14 @@
                         raise errors.BranchExistsWithoutWorkingTree(location)
                 raise errors.AlreadyBranchError(location)
             branch = a_bzrdir.create_branch()
-            if not no_tree:
+            if not no_tree and not a_bzrdir.has_workingtree():
                 a_bzrdir.create_workingtree()
         if append_revisions_only:
             try:
                 branch.set_append_revisions_only(True)
             except errors.UpgradeRequired:
-                raise errors.BzrCommandError('This branch format cannot be set'
-                    ' to append-revisions-only.  Try --default.')
+                raise errors.BzrCommandError(gettext('This branch format cannot be set'
+                    ' to append-revisions-only.  Try --default.'))
         if not is_quiet():
             from bzrlib.info import describe_layout, describe_format
             try:
@@ -1884,7 +2024,8 @@
             repository = branch.repository
             layout = describe_layout(repository, branch, tree).lower()
             format = describe_format(a_bzrdir, repository, branch, tree)
-            self.outf.write("Created a %s (format: %s)\n" % (layout, format))
+            self.outf.write(gettext("Created a {0} (format: {1})\n").format(
+                  layout, format))
             if repository.is_shared():
                 #XXX: maybe this can be refactored into transport.path_or_url()
                 url = repository.bzrdir.root_transport.external_url()
@@ -1892,7 +2033,7 @@
                     url = urlutils.local_path_from_url(url)
                 except errors.InvalidURL:
                     pass
-                self.outf.write("Using shared repository: %s\n" % url)
+                self.outf.write(gettext("Using shared repository: %s\n") % url)
 
 
 class cmd_init_repository(Command):
@@ -1928,8 +2069,8 @@
     takes_options = [RegistryOption('format',
                             help='Specify a format for this repository. See'
                                  ' "bzr help formats" for details.',
-                            lazy_registry=('bzrlib.bzrdir', 'format_registry'),
-                            converter=lambda name: bzrdir.format_registry.make_bzrdir(name),
+                            lazy_registry=('bzrlib.controldir', 'format_registry'),
+                            converter=lambda name: controldir.format_registry.make_bzrdir(name),
                             value_switches=True, title='Repository format'),
                      Option('no-trees',
                              help='Branches in the repository will default to'
@@ -1939,20 +2080,22 @@
 
     def run(self, location, format=None, no_trees=False):
         if format is None:
-            format = bzrdir.format_registry.make_bzrdir('default')
+            format = controldir.format_registry.make_bzrdir('default')
 
         if location is None:
             location = '.'
 
         to_transport = transport.get_transport(location)
-        to_transport.ensure_base()
 
-        newdir = format.initialize_on_transport(to_transport)
-        repo = newdir.create_repository(shared=True)
-        repo.set_make_working_trees(not no_trees)
+        (repo, newdir, require_stacking, repository_policy) = (
+            format.initialize_on_transport_ex(to_transport,
+            create_prefix=True, make_working_trees=not no_trees,
+            shared_repo=True, force_new_repo=True,
+            use_existing_dir=True,
+            repo_format_name=format.repository_format.get_format_string()))
         if not is_quiet():
             from bzrlib.info import show_bzrdir_info
-            show_bzrdir_info(repo.bzrdir, verbose=0, outfile=self.outf)
+            show_bzrdir_info(newdir, verbose=0, outfile=self.outf)
 
 
 class cmd_diff(Command):
@@ -2089,17 +2232,18 @@
         elif ':' in prefix:
             old_label, new_label = prefix.split(":")
         else:
-            raise errors.BzrCommandError(
+            raise errors.BzrCommandError(gettext(
                 '--prefix expects two values separated by a colon'
-                ' (eg "old/:new/")')
+                ' (eg "old/:new/")'))
 
         if revision and len(revision) > 2:
-            raise errors.BzrCommandError('bzr diff --revision takes exactly'
-                                         ' one or two revision specifiers')
+            raise errors.BzrCommandError(gettext('bzr diff --revision takes exactly'
+                                         ' one or two revision specifiers'))
 
         if using is not None and format is not None:
-            raise errors.BzrCommandError('--using and --format are mutually '
-                'exclusive.')
+            raise errors.BzrCommandError(gettext(
+                '{0} and {1} are mutually exclusive').format(
+                '--using', '--format'))
 
         (old_tree, new_tree,
          old_branch, new_branch,
@@ -2213,7 +2357,7 @@
     try:
         return int(limitstring)
     except ValueError:
-        msg = "The limit argument must be an integer."
+        msg = gettext("The limit argument must be an integer.")
         raise errors.BzrCommandError(msg)
 
 
@@ -2221,7 +2365,7 @@
     try:
         return int(s)
     except ValueError:
-        msg = "The levels argument must be an integer."
+        msg = gettext("The levels argument must be an integer.")
         raise errors.BzrCommandError(msg)
 
 
@@ -2419,14 +2563,18 @@
             Option('show-diff',
                    short_name='p',
                    help='Show changes made in each revision as a patch.'),
-            Option('include-merges',
+            Option('include-merged',
                    help='Show merged revisions like --levels 0 does.'),
+            Option('include-merges', hidden=True,
+                   help='Historical alias for --include-merged.'),
+            Option('omit-merges',
+                   help='Do not report commits with more than one parent.'),
             Option('exclude-common-ancestry',
                    help='Display only the revisions that are not part'
-                   ' of both ancestries (require -rX..Y)'
+                   ' of both ancestries (require -rX..Y).'
                    ),
             Option('signatures',
-                   help='Show digital signature validity'),
+                   help='Show digital signature validity.'),
             ListOption('match',
                 short_name='m',
                 help='Show revisions whose properties match this '
@@ -2463,7 +2611,7 @@
             message=None,
             limit=None,
             show_diff=False,
-            include_merges=False,
+            include_merged=None,
             authors=None,
             exclude_common_ancestry=False,
             signatures=False,
@@ -2472,6 +2620,8 @@
             match_committer=None,
             match_author=None,
             match_bugs=None,
+            omit_merges=False,
+            include_merges=symbol_versioning.DEPRECATED_PARAMETER,
             ):
         from bzrlib.log import (
             Logger,
@@ -2479,23 +2629,40 @@
             _get_info_for_log_files,
             )
         direction = (forward and 'forward') or 'reverse'
+        if symbol_versioning.deprecated_passed(include_merges):
+            ui.ui_factory.show_user_warning(
+                'deprecated_command_option',
+                deprecated_name='--include-merges',
+                recommended_name='--include-merged',
+                deprecated_in_version='2.5',
+                command=self.invoked_as)
+            if include_merged is None:
+                include_merged = include_merges
+            else:
+                raise errors.BzrCommandError(gettext(
+                    '{0} and {1} are mutually exclusive').format(
+                    '--include-merges', '--include-merged'))
+        if include_merged is None:
+            include_merged = False
         if (exclude_common_ancestry
             and (revision is None or len(revision) != 2)):
-            raise errors.BzrCommandError(
-                '--exclude-common-ancestry requires -r with two revisions')
-        if include_merges:
+            raise errors.BzrCommandError(gettext(
+                '--exclude-common-ancestry requires -r with two revisions'))
+        if include_merged:
             if levels is None:
                 levels = 0
             else:
-                raise errors.BzrCommandError(
-                    '--levels and --include-merges are mutually exclusive')
+                raise errors.BzrCommandError(gettext(
+                    '{0} and {1} are mutually exclusive').format(
+                    '--levels', '--include-merged'))
 
         if change is not None:
             if len(change) > 1:
                 raise errors.RangeInChangeOption()
             if revision is not None:
-                raise errors.BzrCommandError(
-                    '--revision and --change are mutually exclusive')
+                raise errors.BzrCommandError(gettext(
+                    '{0} and {1} are mutually exclusive').format(
+                    '--revision', '--change'))
             else:
                 revision = change
 
@@ -2507,8 +2674,8 @@
                 revision, file_list, self.add_cleanup)
             for relpath, file_id, kind in file_info_list:
                 if file_id is None:
-                    raise errors.BzrCommandError(
-                        "Path unknown at end or start of revision range: %s" %
+                    raise errors.BzrCommandError(gettext(
+                        "Path unknown at end or start of revision range: %s") %
                         relpath)
                 # If the relpath is the top of the tree, we log everything
                 if relpath == '':
@@ -2526,7 +2693,7 @@
                 location = revision[0].get_branch()
             else:
                 location = '.'
-            dir, relpath = bzrdir.BzrDir.open_containing(location)
+            dir, relpath = controldir.ControlDir.open_containing(location)
             b = dir.open_branch()
             self.add_cleanup(b.lock_read().unlock)
             rev1, rev2 = _get_revision_range(revision, b, self.name())
@@ -2591,7 +2758,7 @@
             match_dict['author'] = match_author
         if match_bugs:
             match_dict['bugs'] = match_bugs
-            
+
         # Build the LogRequest and execute it
         if len(file_ids) == 0:
             file_ids = None
@@ -2601,7 +2768,7 @@
             message_search=message, delta_type=delta_type,
             diff_type=diff_type, _match_using_deltas=match_using_deltas,
             exclude_common_ancestry=exclude_common_ancestry, match=match_dict,
-            signature=signatures
+            signature=signatures, omit_merges=omit_merges,
             )
         Logger(b, rqst).show(lf)
 
@@ -2624,9 +2791,9 @@
             # b is taken from revision[0].get_branch(), and
             # show_log will use its revision_history. Having
             # different branches will lead to weird behaviors.
-            raise errors.BzrCommandError(
+            raise errors.BzrCommandError(gettext(
                 "bzr %s doesn't accept two revisions in different"
-                " branches." % command_name)
+                " branches.") % command_name)
         if start_spec.spec is None:
             # Avoid loading all the history.
             rev1 = RevisionInfo(branch, None, None)
@@ -2640,8 +2807,8 @@
         else:
             rev2 = end_spec.in_history(branch)
     else:
-        raise errors.BzrCommandError(
-            'bzr %s --revision takes one or two values.' % command_name)
+        raise errors.BzrCommandError(gettext(
+            'bzr %s --revision takes one or two values.') % command_name)
     return rev1, rev2
 
 
@@ -2718,10 +2885,10 @@
             null=False, kind=None, show_ids=False, path=None, directory=None):
 
         if kind and kind not in ('file', 'directory', 'symlink'):
-            raise errors.BzrCommandError('invalid kind specified')
+            raise errors.BzrCommandError(gettext('invalid kind specified'))
 
         if verbose and null:
-            raise errors.BzrCommandError('Cannot set both --verbose and --null')
+            raise errors.BzrCommandError(gettext('Cannot set both --verbose and --null'))
         all = not (unknown or versioned or ignored)
 
         selection = {'I':ignored, '?':unknown, 'V':versioned}
@@ -2730,8 +2897,8 @@
             fs_path = '.'
         else:
             if from_root:
-                raise errors.BzrCommandError('cannot specify both --from-root'
-                                             ' and PATH')
+                raise errors.BzrCommandError(gettext('cannot specify both --from-root'
+                                             ' and PATH'))
             fs_path = path
         tree, branch, relpath = \
             _open_directory_or_containing_tree_or_branch(fs_path, directory)
@@ -2753,7 +2920,7 @@
             if view_files:
                 apply_view = True
                 view_str = views.view_display_str(view_files)
-                note("Ignoring files outside view. View is %s" % view_str)
+                note(gettext("Ignoring files outside view. View is %s") % view_str)
 
         self.add_cleanup(tree.lock_read().unlock)
         for fp, fc, fkind, fid, entry in tree.list_files(include_root=False,
@@ -2906,23 +3073,27 @@
                 self.outf.write("%s\n" % pattern)
             return
         if not name_pattern_list:
-            raise errors.BzrCommandError("ignore requires at least one "
-                "NAME_PATTERN or --default-rules.")
+            raise errors.BzrCommandError(gettext("ignore requires at least one "
+                "NAME_PATTERN or --default-rules."))
         name_pattern_list = [globbing.normalize_pattern(p)
                              for p in name_pattern_list]
         bad_patterns = ''
+        bad_patterns_count = 0
         for p in name_pattern_list:
             if not globbing.Globster.is_pattern_valid(p):
+                bad_patterns_count += 1
                 bad_patterns += ('\n  %s' % p)
         if bad_patterns:
-            msg = ('Invalid ignore pattern(s) found. %s' % bad_patterns)
+            msg = (ngettext('Invalid ignore pattern found. %s', 
+                            'Invalid ignore patterns found. %s',
+                            bad_patterns_count) % bad_patterns)
             ui.ui_factory.show_error(msg)
             raise errors.InvalidPattern('')
         for name_pattern in name_pattern_list:
             if (name_pattern[0] == '/' or
                 (len(name_pattern) > 1 and name_pattern[1] == ':')):
-                raise errors.BzrCommandError(
-                    "NAME_PATTERN should not be an absolute path")
+                raise errors.BzrCommandError(gettext(
+                    "NAME_PATTERN should not be an absolute path"))
         tree, relpath = WorkingTree.open_containing(directory)
         ignores.tree_ignores_add_patterns(tree, name_pattern_list)
         ignored = globbing.Globster(name_pattern_list)
@@ -2935,10 +3106,10 @@
                 if ignored.match(filename):
                     matches.append(filename)
         if len(matches) > 0:
-            self.outf.write("Warning: the following files are version controlled and"
-                  " match your ignore pattern:\n%s"
+            self.outf.write(gettext("Warning: the following files are version "
+                  "controlled and match your ignore pattern:\n%s"
                   "\nThese files will continue to be version controlled"
-                  " unless you 'bzr remove' them.\n" % ("\n".join(matches),))
+                  " unless you 'bzr remove' them.\n") % ("\n".join(matches),))
 
 
 class cmd_ignored(Command):
@@ -2983,7 +3154,7 @@
         try:
             revno = int(revno)
         except ValueError:
-            raise errors.BzrCommandError("not a valid revision-number: %r"
+            raise errors.BzrCommandError(gettext("not a valid revision-number: %r")
                                          % revno)
         revid = WorkingTree.open_containing(directory)[0].branch.get_rev_id(revno)
         self.outf.write("%s\n" % revid)
@@ -3032,25 +3203,36 @@
         Option('per-file-timestamps',
                help='Set modification time of files to that of the last '
                     'revision in which it was changed.'),
+        Option('uncommitted',
+               help='Export the working tree contents rather than that of the '
+                    'last revision.'),
         ]
     def run(self, dest, branch_or_subdir=None, revision=None, format=None,
-        root=None, filters=False, per_file_timestamps=False, directory=u'.'):
+        root=None, filters=False, per_file_timestamps=False, uncommitted=False,
+        directory=u'.'):
         from bzrlib.export import export
 
         if branch_or_subdir is None:
-            tree = WorkingTree.open_containing(directory)[0]
-            b = tree.branch
-            subdir = None
+            branch_or_subdir = directory
+
+        (tree, b, subdir) = controldir.ControlDir.open_containing_tree_or_branch(
+            branch_or_subdir)
+        if tree is not None:
+            self.add_cleanup(tree.lock_read().unlock)
+
+        if uncommitted:
+            if tree is None:
+                raise errors.BzrCommandError(
+                    gettext("--uncommitted requires a working tree"))
+            export_tree = tree
         else:
-            b, subdir = Branch.open_containing(branch_or_subdir)
-            tree = None
-
-        rev_tree = _get_one_revision_tree('export', revision, branch=b, tree=tree)
+            export_tree = _get_one_revision_tree('export', revision, branch=b, tree=tree)
         try:
-            export(rev_tree, dest, format, root, subdir, filtered=filters,
+            export(export_tree, dest, format, root, subdir, filtered=filters,
                    per_file_timestamps=per_file_timestamps)
         except errors.NoSuchExportFormat, e:
-            raise errors.BzrCommandError('Unsupported export format: %s' % e.format)
+            raise errors.BzrCommandError(
+                gettext('Unsupported export format: %s') % e.format)
 
 
 class cmd_cat(Command):
@@ -3076,8 +3258,8 @@
     def run(self, filename, revision=None, name_from_revision=False,
             filters=False, directory=None):
         if revision is not None and len(revision) != 1:
-            raise errors.BzrCommandError("bzr cat --revision takes exactly"
-                                         " one revision specifier")
+            raise errors.BzrCommandError(gettext("bzr cat --revision takes exactly"
+                                         " one revision specifier"))
         tree, branch, relpath = \
             _open_directory_or_containing_tree_or_branch(filename, directory)
         self.add_cleanup(branch.lock_read().unlock)
@@ -3100,8 +3282,8 @@
         if name_from_revision:
             # Try in revision if requested
             if old_file_id is None:
-                raise errors.BzrCommandError(
-                    "%r is not present in revision %s" % (
+                raise errors.BzrCommandError(gettext(
+                    "{0!r} is not present in revision {1}").format(
                         filename, rev_tree.get_revision_id()))
             else:
                 actual_file_id = old_file_id
@@ -3112,8 +3294,8 @@
             elif old_file_id is not None:
                 actual_file_id = old_file_id
             else:
-                raise errors.BzrCommandError(
-                    "%r is not present in revision %s" % (
+                raise errors.BzrCommandError(gettext(
+                    "{0!r} is not present in revision {1}").format(
                         filename, rev_tree.get_revision_id()))
         if filtered:
             from bzrlib.filter_tree import ContentFilterTree
@@ -3236,23 +3418,39 @@
     aliases = ['ci', 'checkin']
 
     def _iter_bug_fix_urls(self, fixes, branch):
+        default_bugtracker  = None
         # Configure the properties for bug fixing attributes.
         for fixed_bug in fixes:
             tokens = fixed_bug.split(':')
-            if len(tokens) != 2:
-                raise errors.BzrCommandError(
+            if len(tokens) == 1:
+                if default_bugtracker is None:
+                    branch_config = branch.get_config()
+                    default_bugtracker = branch_config.get_user_option(
+                        "bugtracker")
+                if default_bugtracker is None:
+                    raise errors.BzrCommandError(gettext(
+                        "No tracker specified for bug %s. Use the form "
+                        "'tracker:id' or specify a default bug tracker "
+                        "using the `bugtracker` option.\nSee "
+                        "\"bzr help bugs\" for more information on this "
+                        "feature. Commit refused.") % fixed_bug)
+                tag = default_bugtracker
+                bug_id = tokens[0]
+            elif len(tokens) != 2:
+                raise errors.BzrCommandError(gettext(
                     "Invalid bug %s. Must be in the form of 'tracker:id'. "
                     "See \"bzr help bugs\" for more information on this "
-                    "feature.\nCommit refused." % fixed_bug)
-            tag, bug_id = tokens
+                    "feature.\nCommit refused.") % fixed_bug)
+            else:
+                tag, bug_id = tokens
             try:
                 yield bugtracker.get_bug_url(tag, branch, bug_id)
             except errors.UnknownBugTrackerAbbreviation:
-                raise errors.BzrCommandError(
-                    'Unrecognized bug %s. Commit refused.' % fixed_bug)
+                raise errors.BzrCommandError(gettext(
+                    'Unrecognized bug %s. Commit refused.') % fixed_bug)
             except errors.MalformedBugIdentifier, e:
-                raise errors.BzrCommandError(
-                    "%s\nCommit refused." % (str(e),))
+                raise errors.BzrCommandError(gettext(
+                    "%s\nCommit refused.") % (str(e),))
 
     def run(self, message=None, file=None, verbose=False, selected_list=None,
             unchanged=False, strict=False, local=False, fixes=None,
@@ -3275,8 +3473,8 @@
             try:
                 commit_stamp, offset = timestamp.parse_patch_date(commit_time)
             except ValueError, e:
-                raise errors.BzrCommandError(
-                    "Could not parse --commit-time: " + str(e))
+                raise errors.BzrCommandError(gettext(
+                    "Could not parse --commit-time: " + str(e)))
 
         properties = {}
 
@@ -3315,8 +3513,8 @@
                 message = message.replace('\r\n', '\n')
                 message = message.replace('\r', '\n')
             if file:
-                raise errors.BzrCommandError(
-                    "please specify either --message or --file")
+                raise errors.BzrCommandError(gettext(
+                    "please specify either --message or --file"))
 
         def get_message(commit_obj):
             """Callback to get commit message"""
@@ -3345,13 +3543,13 @@
                     my_message = edit_commit_message_encoded(text,
                         start_message=start_message)
                 if my_message is None:
-                    raise errors.BzrCommandError("please specify a commit"
-                        " message with either --message or --file")
+                    raise errors.BzrCommandError(gettext("please specify a commit"
+                        " message with either --message or --file"))
                 if my_message == "":
-                    raise errors.BzrCommandError("Empty commit message specified."
+                    raise errors.BzrCommandError(gettext("Empty commit message specified."
                             " Please specify a commit message with either"
                             " --message or --file or leave a blank message"
-                            " with --message \"\".")
+                            " with --message \"\"."))
             return my_message
 
         # The API permits a commit with a filter of [] to mean 'select nothing'
@@ -3368,21 +3566,21 @@
                         exclude=tree.safe_relpath_files(exclude),
                         lossy=lossy)
         except PointlessCommit:
-            raise errors.BzrCommandError("No changes to commit."
+            raise errors.BzrCommandError(gettext("No changes to commit."
                 " Please 'bzr add' the files you want to commit, or use"
-                " --unchanged to force an empty commit.")
+                " --unchanged to force an empty commit."))
         except ConflictsInTree:
-            raise errors.BzrCommandError('Conflicts detected in working '
+            raise errors.BzrCommandError(gettext('Conflicts detected in working '
                 'tree.  Use "bzr conflicts" to list, "bzr resolve FILE" to'
-                ' resolve.')
+                ' resolve.'))
         except StrictCommitFailed:
-            raise errors.BzrCommandError("Commit refused because there are"
-                              " unknown files in the working tree.")
+            raise errors.BzrCommandError(gettext("Commit refused because there are"
+                              " unknown files in the working tree."))
         except errors.BoundBranchOutOfDate, e:
-            e.extra_help = ("\n"
+            e.extra_help = (gettext("\n"
                 'To commit to master branch, run update and then commit.\n'
                 'You can also pass --local to commit to continue working '
-                'disconnected.')
+                'disconnected.'))
             raise
 
 
@@ -3495,8 +3693,8 @@
         RegistryOption('format',
             help='Upgrade to a specific format.  See "bzr help'
                  ' formats" for details.',
-            lazy_registry=('bzrlib.bzrdir', 'format_registry'),
-            converter=lambda name: bzrdir.format_registry.make_bzrdir(name),
+            lazy_registry=('bzrlib.controldir', 'format_registry'),
+            converter=lambda name: controldir.format_registry.make_bzrdir(name),
             value_switches=True, title='Branch format'),
         Option('clean',
             help='Remove the backup.bzr directory if successful.'),
@@ -3543,20 +3741,22 @@
             if directory is None:
                 # use branch if we're inside one; otherwise global config
                 try:
-                    c = Branch.open_containing(u'.')[0].get_config()
+                    c = Branch.open_containing(u'.')[0].get_config_stack()
                 except errors.NotBranchError:
-                    c = _mod_config.GlobalConfig()
+                    c = _mod_config.GlobalStack()
             else:
-                c = Branch.open(directory).get_config()
+                c = Branch.open(directory).get_config_stack()
+            identity = c.get('email')
             if email:
-                self.outf.write(c.user_email() + '\n')
+                self.outf.write(_mod_config.extract_email_address(identity)
+                                + '\n')
             else:
-                self.outf.write(c.username() + '\n')
+                self.outf.write(identity + '\n')
             return
 
         if email:
-            raise errors.BzrCommandError("--email can only be used to display existing "
-                                         "identity")
+            raise errors.BzrCommandError(gettext("--email can only be used to display existing "
+                                         "identity"))
 
         # display a warning if an email address isn't included in the given name.
         try:
@@ -3568,12 +3768,12 @@
         # use global config unless --branch given
         if branch:
             if directory is None:
-                c = Branch.open_containing(u'.')[0].get_config()
+                c = Branch.open_containing(u'.')[0].get_config_stack()
             else:
-                c = Branch.open(directory).get_config()
+                c = Branch.open(directory).get_config_stack()
         else:
-            c = _mod_config.GlobalConfig()
-        c.set_user_option('email', name)
+            c = _mod_config.GlobalStack()
+        c.set('email', name)
 
 
 class cmd_nick(Command):
@@ -3641,8 +3841,8 @@
 
     def remove_alias(self, alias_name):
         if alias_name is None:
-            raise errors.BzrCommandError(
-                'bzr alias --remove expects an alias to remove.')
+            raise errors.BzrCommandError(gettext(
+                'bzr alias --remove expects an alias to remove.'))
         # If alias is not found, print something like:
         # unalias: foo: not found
         c = _mod_config.GlobalConfig()
@@ -3786,6 +3986,9 @@
                                 param_name='starting_with', short_name='s',
                                 help=
                                 'Load only the tests starting with TESTID.'),
+                     Option('sync',
+                            help="By default we disable fsync and fdatasync"
+                                 " while running the test suite.")
                      ]
     encoding_type = 'replace'
 
@@ -3799,7 +4002,8 @@
             first=False, list_only=False,
             randomize=None, exclude=None, strict=False,
             load_list=None, debugflag=None, starting_with=None, subunit=False,
-            parallel=None, lsprof_tests=False):
+            parallel=None, lsprof_tests=False,
+            sync=False):
 
         # During selftest, disallow proxying, as it can cause severe
         # performance penalties and is only needed for thread
@@ -3819,8 +4023,8 @@
             try:
                 from bzrlib.tests import SubUnitBzrRunner
             except ImportError:
-                raise errors.BzrCommandError("subunit not available. subunit "
-                    "needs to be installed to use --subunit.")
+                raise errors.BzrCommandError(gettext("subunit not available. subunit "
+                    "needs to be installed to use --subunit."))
             self.additional_selftest_args['runner_class'] = SubUnitBzrRunner
             # On Windows, disable automatic conversion of '\n' to '\r\n' in
             # stdout, which would corrupt the subunit stream. 
@@ -3835,14 +4039,16 @@
             self.additional_selftest_args.setdefault(
                 'suite_decorators', []).append(parallel)
         if benchmark:
-            raise errors.BzrCommandError(
+            raise errors.BzrCommandError(gettext(
                 "--benchmark is no longer supported from bzr 2.2; "
-                "use bzr-usertest instead")
+                "use bzr-usertest instead"))
         test_suite_factory = None
         if not exclude:
             exclude_pattern = None
         else:
             exclude_pattern = '(' + '|'.join(exclude) + ')'
+        if not sync:
+            self._disable_fsync()
         selftest_kwargs = {"verbose": verbose,
                           "pattern": pattern,
                           "stop_on_failure": one,
@@ -3870,6 +4076,15 @@
             cleanup()
         return int(not result)
 
+    def _disable_fsync(self):
+        """Change the 'os' functionality to not synchronize."""
+        self._orig_fsync = getattr(os, 'fsync', None)
+        if self._orig_fsync is not None:
+            os.fsync = lambda filedes: None
+        self._orig_fdatasync = getattr(os, 'fdatasync', None)
+        if self._orig_fdatasync is not None:
+            os.fdatasync = lambda filedes: None
+
 
 class cmd_version(Command):
     __doc__ = """Show version of bzr."""
@@ -3895,7 +4110,7 @@
 
     @display_command
     def run(self):
-        self.outf.write("It sure does!\n")
+        self.outf.write(gettext("It sure does!\n"))
 
 
 class cmd_find_merge_base(Command):
@@ -3919,7 +4134,7 @@
         graph = branch1.repository.get_graph(branch2.repository)
         base_rev_id = graph.find_unique_lca(last1, last2)
 
-        self.outf.write('merge base is revision %s\n' % base_rev_id)
+        self.outf.write(gettext('merge base is revision %s\n') % base_rev_id)
 
 
 class cmd_merge(Command):
@@ -4050,8 +4265,8 @@
 
         tree = WorkingTree.open_containing(directory)[0]
         if tree.branch.revno() == 0:
-            raise errors.BzrCommandError('Merging into empty branches not currently supported, '
-                                         'https://bugs.launchpad.net/bzr/+bug/308562')
+            raise errors.BzrCommandError(gettext('Merging into empty branches not currently supported, '
+                                         'https://bugs.launchpad.net/bzr/+bug/308562'))
 
         try:
             basis_tree = tree.revision_tree(tree.last_revision())
@@ -4077,19 +4292,19 @@
                 mergeable = None
             else:
                 if uncommitted:
-                    raise errors.BzrCommandError('Cannot use --uncommitted'
-                        ' with bundles or merge directives.')
+                    raise errors.BzrCommandError(gettext('Cannot use --uncommitted'
+                        ' with bundles or merge directives.'))
 
                 if revision is not None:
-                    raise errors.BzrCommandError(
-                        'Cannot use -r with merge directives or bundles')
+                    raise errors.BzrCommandError(gettext(
+                        'Cannot use -r with merge directives or bundles'))
                 merger, verified = _mod_merge.Merger.from_mergeable(tree,
                    mergeable, None)
 
         if merger is None and uncommitted:
             if revision is not None and len(revision) > 0:
-                raise errors.BzrCommandError('Cannot use --uncommitted and'
-                    ' --revision at the same time.')
+                raise errors.BzrCommandError(gettext('Cannot use --uncommitted and'
+                    ' --revision at the same time.'))
             merger = self.get_merger_from_uncommitted(tree, location, None)
             allow_pending = False
 
@@ -4108,22 +4323,22 @@
             if merger.interesting_files:
                 if not merger.other_tree.has_filename(
                     merger.interesting_files[0]):
-                    note("merger: " + str(merger))
+                    note(gettext("merger: ") + str(merger))
                     raise errors.PathsDoNotExist([location])
-            note('Nothing to do.')
+            note(gettext('Nothing to do.'))
             return 0
         if pull and not preview:
             if merger.interesting_files is not None:
-                raise errors.BzrCommandError('Cannot pull individual files')
+                raise errors.BzrCommandError(gettext('Cannot pull individual files'))
             if (merger.base_rev_id == tree.last_revision()):
                 result = tree.pull(merger.other_branch, False,
                                    merger.other_rev_id)
                 result.report(self.outf)
                 return 0
         if merger.this_basis is None:
-            raise errors.BzrCommandError(
+            raise errors.BzrCommandError(gettext(
                 "This branch has no commits."
-                " (perhaps you would prefer 'bzr pull')")
+                " (perhaps you would prefer 'bzr pull')"))
         if preview:
             return self._do_preview(merger)
         elif interactive:
@@ -4180,8 +4395,8 @@
     def sanity_check_merger(self, merger):
         if (merger.show_base and
             not merger.merge_type is _mod_merge.Merge3Merger):
-            raise errors.BzrCommandError("Show-base is not supported for this"
-                                         " merge type. %s" % merger.merge_type)
+            raise errors.BzrCommandError(gettext("Show-base is not supported for this"
+                                         " merge type. %s") % merger.merge_type)
         if merger.reprocess is None:
             if merger.show_base:
                 merger.reprocess = False
@@ -4189,12 +4404,12 @@
                 # Use reprocess if the merger supports it
                 merger.reprocess = merger.merge_type.supports_reprocess
         if merger.reprocess and not merger.merge_type.supports_reprocess:
-            raise errors.BzrCommandError("Conflict reduction is not supported"
-                                         " for merge type %s." %
+            raise errors.BzrCommandError(gettext("Conflict reduction is not supported"
+                                         " for merge type %s.") %
                                          merger.merge_type)
         if merger.reprocess and merger.show_base:
-            raise errors.BzrCommandError("Cannot do conflict reduction and"
-                                         " show base.")
+            raise errors.BzrCommandError(gettext("Cannot do conflict reduction and"
+                                         " show base."))
 
     def _get_merger_from_branch(self, tree, location, revision, remember,
                                 possible_transports, pb):
@@ -4304,10 +4519,10 @@
             stored_location_type = "parent"
         mutter("%s", stored_location)
         if stored_location is None:
-            raise errors.BzrCommandError("No location specified or remembered")
+            raise errors.BzrCommandError(gettext("No location specified or remembered"))
         display_url = urlutils.unescape_for_display(stored_location, 'utf-8')
-        note(u"%s remembered %s location %s", verb_string,
-                stored_location_type, display_url)
+        note(gettext("{0} remembered {1} location {2}").format(verb_string,
+                stored_location_type, display_url))
         return stored_location
 
 
@@ -4350,9 +4565,9 @@
         self.add_cleanup(tree.lock_write().unlock)
         parents = tree.get_parent_ids()
         if len(parents) != 2:
-            raise errors.BzrCommandError("Sorry, remerge only works after normal"
+            raise errors.BzrCommandError(gettext("Sorry, remerge only works after normal"
                                          " merges.  Not cherrypicking or"
-                                         " multi-merges.")
+                                         " multi-merges."))
         repository = tree.branch.repository
         interesting_ids = None
         new_conflicts = []
@@ -4517,7 +4732,7 @@
 
     @display_command
     def run(self, context=None):
-        import shellcomplete
+        from bzrlib import shellcomplete
         shellcomplete.shellcomplete(context)
 
 
@@ -4577,8 +4792,10 @@
             type=_parse_revision_str,
             help='Filter on local branch revisions (inclusive). '
                 'See "help revisionspec" for details.'),
-        Option('include-merges',
+        Option('include-merged',
                'Show all revisions in addition to the mainline ones.'),
+        Option('include-merges', hidden=True,
+               help='Historical alias for --include-merged.'),
         ]
     encoding_type = 'replace'
 
@@ -4587,13 +4804,29 @@
             theirs_only=False,
             log_format=None, long=False, short=False, line=False,
             show_ids=False, verbose=False, this=False, other=False,
-            include_merges=False, revision=None, my_revision=None,
-            directory=u'.'):
+            include_merged=None, revision=None, my_revision=None,
+            directory=u'.',
+            include_merges=symbol_versioning.DEPRECATED_PARAMETER):
         from bzrlib.missing import find_unmerged, iter_log_revisions
         def message(s):
             if not is_quiet():
                 self.outf.write(s)
 
+        if symbol_versioning.deprecated_passed(include_merges):
+            ui.ui_factory.show_user_warning(
+                'deprecated_command_option',
+                deprecated_name='--include-merges',
+                recommended_name='--include-merged',
+                deprecated_in_version='2.5',
+                command=self.invoked_as)
+            if include_merged is None:
+                include_merged = include_merges
+            else:
+                raise errors.BzrCommandError(gettext(
+                    '{0} and {1} are mutually exclusive').format(
+                    '--include-merges', '--include-merged'))
+        if include_merged is None:
+            include_merged = False
         if this:
             mine_only = this
         if other:
@@ -4614,12 +4847,12 @@
         if other_branch is None:
             other_branch = parent
             if other_branch is None:
-                raise errors.BzrCommandError("No peer location known"
-                                             " or specified.")
+                raise errors.BzrCommandError(gettext("No peer location known"
+                                             " or specified."))
             display_url = urlutils.unescape_for_display(parent,
                                                         self.outf.encoding)
-            message("Using saved parent location: "
-                    + display_url + "\n")
+            message(gettext("Using saved parent location: {0}\n").format(
+                    display_url))
 
         remote_branch = Branch.open(other_branch)
         if remote_branch.base == local_branch.base:
@@ -4638,7 +4871,7 @@
         local_extra, remote_extra = find_unmerged(
             local_branch, remote_branch, restrict,
             backward=not reverse,
-            include_merges=include_merges,
+            include_merged=include_merged,
             local_revid_range=local_revid_range,
             remote_revid_range=remote_revid_range)
 
@@ -4651,7 +4884,9 @@
 
         status_code = 0
         if local_extra and not theirs_only:
-            message("You have %d extra revision(s):\n" %
+            message(ngettext("You have %d extra revision:\n",
+                             "You have %d extra revisions:\n", 
+                             len(local_extra)) %
                 len(local_extra))
             for revision in iter_log_revisions(local_extra,
                                 local_branch.repository,
@@ -4665,7 +4900,9 @@
         if remote_extra and not mine_only:
             if printed_local is True:
                 message("\n\n\n")
-            message("You are missing %d revision(s):\n" %
+            message(ngettext("You are missing %d revision:\n",
+                             "You are missing %d revisions:\n",
+                             len(remote_extra)) %
                 len(remote_extra))
             for revision in iter_log_revisions(remote_extra,
                                 remote_branch.repository,
@@ -4675,15 +4912,15 @@
 
         if mine_only and not local_extra:
             # We checked local, and found nothing extra
-            message('This branch has no new revisions.\n')
+            message(gettext('This branch has no new revisions.\n'))
         elif theirs_only and not remote_extra:
             # We checked remote, and found nothing extra
-            message('Other branch has no new revisions.\n')
+            message(gettext('Other branch has no new revisions.\n'))
         elif not (mine_only or theirs_only or local_extra or
                   remote_extra):
             # We checked both branches, and neither one had extra
             # revisions
-            message("Branches are up to date.\n")
+            message(gettext("Branches are up to date.\n"))
         self.cleanup_now()
         if not status_code and parent is None and other_branch is not None:
             self.add_cleanup(local_branch.lock_write().unlock)
@@ -4719,7 +4956,7 @@
         ]
 
     def run(self, branch_or_repo='.', clean_obsolete_packs=False):
-        dir = bzrdir.BzrDir.open_containing(branch_or_repo)[0]
+        dir = controldir.ControlDir.open_containing(branch_or_repo)[0]
         try:
             branch = dir.open_branch()
             repository = branch.repository
@@ -4850,16 +5087,16 @@
 
     def run(self, revision_id_list=None, revision=None, directory=u'.'):
         if revision_id_list is not None and revision is not None:
-            raise errors.BzrCommandError('You can only supply one of revision_id or --revision')
+            raise errors.BzrCommandError(gettext('You can only supply one of revision_id or --revision'))
         if revision_id_list is None and revision is None:
-            raise errors.BzrCommandError('You must supply either --revision or a revision_id')
+            raise errors.BzrCommandError(gettext('You must supply either --revision or a revision_id'))
         b = WorkingTree.open_containing(directory)[0].branch
         self.add_cleanup(b.lock_write().unlock)
         return self._run(b, revision_id_list, revision)
 
     def _run(self, b, revision_id_list, revision):
         import bzrlib.gpg as gpg
-        gpg_strategy = gpg.GPGStrategy(b.get_config())
+        gpg_strategy = gpg.GPGStrategy(b.get_config_stack())
         if revision_id_list is not None:
             b.repository.start_write_group()
             try:
@@ -4890,7 +5127,7 @@
                 if to_revid is None:
                     to_revno = b.revno()
                 if from_revno is None or to_revno is None:
-                    raise errors.BzrCommandError('Cannot sign a range of non-revision-history revisions')
+                    raise errors.BzrCommandError(gettext('Cannot sign a range of non-revision-history revisions'))
                 b.repository.start_write_group()
                 try:
                     for revno in range(from_revno, to_revno + 1):
@@ -4902,7 +5139,7 @@
                 else:
                     b.repository.commit_write_group()
             else:
-                raise errors.BzrCommandError('Please supply either one revision, or a range.')
+                raise errors.BzrCommandError(gettext('Please supply either one revision, or a range.'))
 
 
 class cmd_bind(Command):
@@ -4927,21 +5164,21 @@
             try:
                 location = b.get_old_bound_location()
             except errors.UpgradeRequired:
-                raise errors.BzrCommandError('No location supplied.  '
-                    'This format does not remember old locations.')
+                raise errors.BzrCommandError(gettext('No location supplied.  '
+                    'This format does not remember old locations.'))
             else:
                 if location is None:
                     if b.get_bound_location() is not None:
-                        raise errors.BzrCommandError('Branch is already bound')
+                        raise errors.BzrCommandError(gettext('Branch is already bound'))
                     else:
-                        raise errors.BzrCommandError('No location supplied '
-                            'and no previous location known')
+                        raise errors.BzrCommandError(gettext('No location supplied '
+                            'and no previous location known'))
         b_other = Branch.open(location)
         try:
             b.bind(b_other)
         except errors.DivergedBranches:
-            raise errors.BzrCommandError('These branches have diverged.'
-                                         ' Try merging, and then bind again.')
+            raise errors.BzrCommandError(gettext('These branches have diverged.'
+                                         ' Try merging, and then bind again.'))
         if b.get_config().has_explicit_nickname():
             b.nick = b_other.nick
 
@@ -4960,7 +5197,7 @@
     def run(self, directory=u'.'):
         b, relpath = Branch.open_containing(directory)
         if not b.unbind():
-            raise errors.BzrCommandError('Local branch is not bound')
+            raise errors.BzrCommandError(gettext('Local branch is not bound'))
 
 
 class cmd_uncommit(Command):
@@ -5002,7 +5239,7 @@
             revision=None, force=False, local=False, keep_tags=False):
         if location is None:
             location = u'.'
-        control, relpath = bzrdir.BzrDir.open_containing(location)
+        control, relpath = controldir.ControlDir.open_containing(location)
         try:
             tree = control.open_workingtree()
             b = tree.branch
@@ -5039,7 +5276,7 @@
                 rev_id = b.get_rev_id(revno)
 
         if rev_id is None or _mod_revision.is_null(rev_id):
-            self.outf.write('No revisions to uncommit.\n')
+            self.outf.write(gettext('No revisions to uncommit.\n'))
             return 1
 
         lf = log_formatter('short',
@@ -5054,25 +5291,25 @@
                  end_revision=last_revno)
 
         if dry_run:
-            self.outf.write('Dry-run, pretending to remove'
-                            ' the above revisions.\n')
+            self.outf.write(gettext('Dry-run, pretending to remove'
+                            ' the above revisions.\n'))
         else:
-            self.outf.write('The above revision(s) will be removed.\n')
+            self.outf.write(gettext('The above revision(s) will be removed.\n'))
 
         if not force:
             if not ui.ui_factory.confirm_action(
-                    u'Uncommit these revisions',
+                    gettext(u'Uncommit these revisions'),
                     'bzrlib.builtins.uncommit',
                     {}):
-                self.outf.write('Canceled\n')
+                self.outf.write(gettext('Canceled\n'))
                 return 0
 
         mutter('Uncommitting from {%s} to {%s}',
                last_rev_id, rev_id)
         uncommit(b, tree=tree, dry_run=dry_run, verbose=verbose,
                  revno=revno, local=local, keep_tags=keep_tags)
-        self.outf.write('You can restore the old tip by running:\n'
-             '  bzr pull . -r revid:%s\n' % last_rev_id)
+        self.outf.write(gettext('You can restore the old tip by running:\n'
+             '  bzr pull . -r revid:%s\n') % last_rev_id)
 
 
 class cmd_break_lock(Command):
@@ -5112,7 +5349,7 @@
             conf = _mod_config.LockableConfig(file_name=location)
             conf.break_lock()
         else:
-            control, relpath = bzrdir.BzrDir.open_containing(location)
+            control, relpath = controldir.ControlDir.open_containing(location)
             try:
                 control.break_lock()
             except NotImplementedError:
@@ -5162,6 +5399,8 @@
                     'option leads to global uncontrolled write access to your '
                     'file system.'
                 ),
+        Option('client-timeout', type=float,
+               help='Override the default idle client timeout (5min).'),
         ]
 
     def get_host_and_port(self, port):
@@ -5184,18 +5423,30 @@
         return host, port
 
     def run(self, port=None, inet=False, directory=None, allow_writes=False,
-            protocol=None):
+            protocol=None, client_timeout=None):
         from bzrlib import transport
         if directory is None:
             directory = os.getcwd()
         if protocol is None:
             protocol = transport.transport_server_registry.get()
         host, port = self.get_host_and_port(port)
-        url = urlutils.local_path_to_url(directory)
+        url = transport.location_to_url(directory)
         if not allow_writes:
             url = 'readonly+' + url
-        t = transport.get_transport(url)
-        protocol(t, host, port, inet)
+        t = transport.get_transport_from_url(url)
+        try:
+            protocol(t, host, port, inet, client_timeout)
+        except TypeError, e:
+            # We use symbol_versioning.deprecated_in just so that people
+            # grepping can find it here.
+            # symbol_versioning.deprecated_in((2, 5, 0))
+            symbol_versioning.warn(
+                'Got TypeError(%s)\ntrying to call protocol: %s.%s\n'
+                'Most likely it needs to be updated to support a'
+                ' "timeout" parameter (added in bzr 2.5.0)'
+                % (e, protocol.__module__, protocol),
+                DeprecationWarning)
+            protocol(t, host, port, inet)
 
 
 class cmd_join(Command):
@@ -5224,9 +5475,9 @@
         containing_tree = WorkingTree.open_containing(parent_dir)[0]
         repo = containing_tree.branch.repository
         if not repo.supports_rich_root():
-            raise errors.BzrCommandError(
+            raise errors.BzrCommandError(gettext(
                 "Can't join trees because %s doesn't support rich root data.\n"
-                "You can use bzr upgrade on the repository."
+                "You can use bzr upgrade on the repository.")
                 % (repo,))
         if reference:
             try:
@@ -5234,14 +5485,14 @@
             except errors.BadReferenceTarget, e:
                 # XXX: Would be better to just raise a nicely printable
                 # exception from the real origin.  Also below.  mbp 20070306
-                raise errors.BzrCommandError("Cannot join %s.  %s" %
-                                             (tree, e.reason))
+                raise errors.BzrCommandError(
+                       gettext("Cannot join {0}.  {1}").format(tree, e.reason))
         else:
             try:
                 containing_tree.subsume(sub_tree)
             except errors.BadSubsumeSource, e:
-                raise errors.BzrCommandError("Cannot join %s.  %s" %
-                                             (tree, e.reason))
+                raise errors.BzrCommandError(
+                       gettext("Cannot join {0}.  {1}").format(tree, e.reason))
 
 
 class cmd_split(Command):
@@ -5331,7 +5582,7 @@
         if submit_branch is None:
             submit_branch = branch.get_parent()
         if submit_branch is None:
-            raise errors.BzrCommandError('No submit branch specified or known')
+            raise errors.BzrCommandError(gettext('No submit branch specified or known'))
 
         stored_public_branch = branch.get_public_branch()
         if public_branch is None:
@@ -5339,13 +5590,13 @@
         elif stored_public_branch is None:
             branch.set_public_branch(public_branch)
         if not include_bundle and public_branch is None:
-            raise errors.BzrCommandError('No public branch specified or'
-                                         ' known')
+            raise errors.BzrCommandError(gettext('No public branch specified or'
+                                         ' known'))
         base_revision_id = None
         if revision is not None:
             if len(revision) > 2:
-                raise errors.BzrCommandError('bzr merge-directive takes '
-                    'at most two one revision identifiers')
+                raise errors.BzrCommandError(gettext('bzr merge-directive takes '
+                    'at most two one revision identifiers'))
             revision_id = revision[-1].as_revision_id(branch)
             if len(revision) == 2:
                 base_revision_id = revision[0].as_revision_id(branch)
@@ -5353,7 +5604,7 @@
             revision_id = branch.last_revision()
         revision_id = ensure_null(revision_id)
         if revision_id == NULL_REVISION:
-            raise errors.BzrCommandError('No revisions to bundle.')
+            raise errors.BzrCommandError(gettext('No revisions to bundle.'))
         directive = merge_directive.MergeDirective2.from_objects(
             branch.repository, revision_id, time.time(),
             osutils.local_time_offset(), submit_branch,
@@ -5367,7 +5618,7 @@
                 self.outf.writelines(directive.to_lines())
         else:
             message = directive.to_email(mail_to, branch, sign)
-            s = SMTPConnection(branch.get_config())
+            s = SMTPConnection(branch.get_config_stack())
             s.send_email(message)
 
 
@@ -5613,27 +5864,37 @@
         self.add_cleanup(branch.lock_write().unlock)
         if delete:
             if tag_name is None:
-                raise errors.BzrCommandError("No tag specified to delete.")
+                raise errors.BzrCommandError(gettext("No tag specified to delete."))
             branch.tags.delete_tag(tag_name)
-            note('Deleted tag %s.' % tag_name)
+            note(gettext('Deleted tag %s.') % tag_name)
         else:
             if revision:
                 if len(revision) != 1:
-                    raise errors.BzrCommandError(
+                    raise errors.BzrCommandError(gettext(
                         "Tags can only be placed on a single revision, "
-                        "not on a range")
+                        "not on a range"))
                 revision_id = revision[0].as_revision_id(branch)
             else:
                 revision_id = branch.last_revision()
             if tag_name is None:
                 tag_name = branch.automatic_tag_name(revision_id)
                 if tag_name is None:
-                    raise errors.BzrCommandError(
-                        "Please specify a tag name.")
-            if (not force) and branch.tags.has_tag(tag_name):
+                    raise errors.BzrCommandError(gettext(
+                        "Please specify a tag name."))
+            try:
+                existing_target = branch.tags.lookup_tag(tag_name)
+            except errors.NoSuchTag:
+                existing_target = None
+            if not force and existing_target not in (None, revision_id):
                 raise errors.TagAlreadyExists(tag_name)
-            branch.tags.set_tag(tag_name, revision_id)
-            note('Created tag %s.' % tag_name)
+            if existing_target == revision_id:
+                note(gettext('Tag %s already exists for that revision.') % tag_name)
+            else:
+                branch.tags.set_tag(tag_name, revision_id)
+                if existing_target is None:
+                    note(gettext('Created tag %s.') % tag_name)
+                else:
+                    note(gettext('Updated tag %s.') % tag_name)
 
 
 class cmd_tags(Command):
@@ -5665,12 +5926,8 @@
 
         self.add_cleanup(branch.lock_read().unlock)
         if revision:
-            graph = branch.repository.get_graph()
-            rev1, rev2 = _get_revision_range(revision, branch, self.name())
-            revid1, revid2 = rev1.rev_id, rev2.rev_id
-            # only show revisions between revid1 and revid2 (inclusive)
-            tags = [(tag, revid) for tag, revid in tags if
-                graph.is_between(revid, revid1, revid2)]
+            # Restrict to the specified range
+            tags = self._tags_for_range(branch, revision)
         if sort is None:
             sort = tag_sort_methods.get()
         sort(branch, tags)
@@ -5681,7 +5938,9 @@
                     revno = branch.revision_id_to_dotted_revno(revid)
                     if isinstance(revno, tuple):
                         revno = '.'.join(map(str, revno))
-                except (errors.NoSuchRevision, errors.GhostRevisionsHaveNoRevno):
+                except (errors.NoSuchRevision,
+                        errors.GhostRevisionsHaveNoRevno,
+                        errors.UnsupportedOperation):
                     # Bad tag data/merges can lead to tagged revisions
                     # which are not in this branch. Fail gracefully ...
                     revno = '?'
@@ -5690,6 +5949,32 @@
         for tag, revspec in tags:
             self.outf.write('%-20s %s\n' % (tag, revspec))
 
+    def _tags_for_range(self, branch, revision):
+        range_valid = True
+        rev1, rev2 = _get_revision_range(revision, branch, self.name())
+        revid1, revid2 = rev1.rev_id, rev2.rev_id
+        # _get_revision_range will always set revid2 if it's not specified.
+        # If revid1 is None, it means we want to start from the branch
+        # origin which is always a valid ancestor. If revid1 == revid2, the
+        # ancestry check is useless.
+        if revid1 and revid1 != revid2:
+            # FIXME: We really want to use the same graph than
+            # branch.iter_merge_sorted_revisions below, but this is not
+            # easily available -- vila 2011-09-23
+            if branch.repository.get_graph().is_ancestor(revid2, revid1):
+                # We don't want to output anything in this case...
+                return []
+        # only show revisions between revid1 and revid2 (inclusive)
+        tagged_revids = branch.tags.get_reverse_tag_dict()
+        found = []
+        for r in branch.iter_merge_sorted_revisions(
+            start_revision_id=revid2, stop_revision_id=revid1,
+            stop_rule='include'):
+            revid_tags = tagged_revids.get(r[0], None)
+            if revid_tags:
+                found.extend([(tag, r[0]) for tag in revid_tags])
+        return found
+
 
 class cmd_reconfigure(Command):
     __doc__ = """Reconfigure the type of a bzr directory.
@@ -5709,18 +5994,30 @@
     takes_args = ['location?']
     takes_options = [
         RegistryOption.from_kwargs(
-            'target_type',
-            title='Target type',
-            help='The type to reconfigure the directory to.',
+            'tree_type',
+            title='Tree type',
+            help='The relation between branch and tree.',
             value_switches=True, enum_switch=False,
             branch='Reconfigure to be an unbound branch with no working tree.',
             tree='Reconfigure to be an unbound branch with a working tree.',
             checkout='Reconfigure to be a bound branch with a working tree.',
             lightweight_checkout='Reconfigure to be a lightweight'
                 ' checkout (with no local history).',
+            ),
+        RegistryOption.from_kwargs(
+            'repository_type',
+            title='Repository type',
+            help='Location fo the repository.',
+            value_switches=True, enum_switch=False,
             standalone='Reconfigure to be a standalone branch '
                 '(i.e. stop using shared repository).',
             use_shared='Reconfigure to use a shared repository.',
+            ),
+        RegistryOption.from_kwargs(
+            'repository_trees',
+            title='Trees in Repository',
+            help='Whether new branches in the repository have trees.',
+            value_switches=True, enum_switch=False,
             with_trees='Reconfigure repository to create '
                 'working trees on branches by default.',
             with_no_trees='Reconfigure repository to not create '
@@ -5740,12 +6037,12 @@
             ),
         ]
 
-    def run(self, location=None, target_type=None, bind_to=None, force=False,
-            stacked_on=None,
-            unstacked=None):
-        directory = bzrdir.BzrDir.open(location)
+    def run(self, location=None, bind_to=None, force=False,
+            tree_type=None, repository_type=None, repository_trees=None,
+            stacked_on=None, unstacked=None):
+        directory = controldir.ControlDir.open(location)
         if stacked_on and unstacked:
-            raise errors.BzrCommandError("Can't use both --stacked-on and --unstacked")
+            raise errors.BzrCommandError(gettext("Can't use both --stacked-on and --unstacked"))
         elif stacked_on is not None:
             reconfigure.ReconfigureStackedOn().apply(directory, stacked_on)
         elif unstacked:
@@ -5753,33 +6050,44 @@
         # At the moment you can use --stacked-on and a different
         # reconfiguration shape at the same time; there seems no good reason
         # to ban it.
-        if target_type is None:
+        if (tree_type is None and
+            repository_type is None and
+            repository_trees is None):
             if stacked_on or unstacked:
                 return
             else:
-                raise errors.BzrCommandError('No target configuration '
-                    'specified')
-        elif target_type == 'branch':
+                raise errors.BzrCommandError(gettext('No target configuration '
+                    'specified'))
+        reconfiguration = None
+        if tree_type == 'branch':
             reconfiguration = reconfigure.Reconfigure.to_branch(directory)
-        elif target_type == 'tree':
+        elif tree_type == 'tree':
             reconfiguration = reconfigure.Reconfigure.to_tree(directory)
-        elif target_type == 'checkout':
+        elif tree_type == 'checkout':
             reconfiguration = reconfigure.Reconfigure.to_checkout(
                 directory, bind_to)
-        elif target_type == 'lightweight-checkout':
+        elif tree_type == 'lightweight-checkout':
             reconfiguration = reconfigure.Reconfigure.to_lightweight_checkout(
                 directory, bind_to)
-        elif target_type == 'use-shared':
+        if reconfiguration:
+            reconfiguration.apply(force)
+            reconfiguration = None
+        if repository_type == 'use-shared':
             reconfiguration = reconfigure.Reconfigure.to_use_shared(directory)
-        elif target_type == 'standalone':
+        elif repository_type == 'standalone':
             reconfiguration = reconfigure.Reconfigure.to_standalone(directory)
-        elif target_type == 'with-trees':
+        if reconfiguration:
+            reconfiguration.apply(force)
+            reconfiguration = None
+        if repository_trees == 'with-trees':
             reconfiguration = reconfigure.Reconfigure.set_repository_trees(
                 directory, True)
-        elif target_type == 'with-no-trees':
+        elif repository_trees == 'with-no-trees':
             reconfiguration = reconfigure.Reconfigure.set_repository_trees(
                 directory, False)
-        reconfiguration.apply(force)
+        if reconfiguration:
+            reconfiguration.apply(force)
+            reconfiguration = None
 
 
 class cmd_switch(Command):
@@ -5820,11 +6128,11 @@
         from bzrlib import switch
         tree_location = directory
         revision = _get_one_revision('switch', revision)
-        control_dir = bzrdir.BzrDir.open_containing(tree_location)[0]
+        control_dir = controldir.ControlDir.open_containing(tree_location)[0]
         if to_location is None:
             if revision is None:
-                raise errors.BzrCommandError('You must supply either a'
-                                             ' revision or a location')
+                raise errors.BzrCommandError(gettext('You must supply either a'
+                                             ' revision or a location'))
             to_location = tree_location
         try:
             branch = control_dir.open_branch()
@@ -5834,31 +6142,49 @@
             had_explicit_nick = False
         if create_branch:
             if branch is None:
-                raise errors.BzrCommandError('cannot create branch without'
-                                             ' source branch')
+                raise errors.BzrCommandError(gettext('cannot create branch without'
+                                             ' source branch'))
             to_location = directory_service.directories.dereference(
                               to_location)
             if '/' not in to_location and '\\' not in to_location:
                 # This path is meant to be relative to the existing branch
                 this_url = self._get_branch_location(control_dir)
-                to_location = urlutils.join(this_url, '..', to_location)
+                # Perhaps the target control dir supports colocated branches?
+                try:
+                    root = controldir.ControlDir.open(this_url,
+                        possible_transports=[control_dir.user_transport])
+                except errors.NotBranchError:
+                    colocated = False
+                else:
+                    colocated = root._format.colocated_branches
+                if colocated:
+                    to_location = urlutils.join_segment_parameters(this_url,
+                        {"branch": urlutils.escape(to_location)})
+                else:
+                    to_location = urlutils.join(
+                        this_url, '..', urlutils.escape(to_location))
             to_branch = branch.bzrdir.sprout(to_location,
                                  possible_transports=[branch.bzrdir.root_transport],
                                  source_branch=branch).open_branch()
         else:
+            # Perhaps it's a colocated branch?
             try:
-                to_branch = Branch.open(to_location)
-            except errors.NotBranchError:
-                this_url = self._get_branch_location(control_dir)
-                to_branch = Branch.open(
-                    urlutils.join(this_url, '..', to_location))
+                to_branch = control_dir.open_branch(to_location)
+            except (errors.NotBranchError, errors.NoColocatedBranchSupport):
+                try:
+                    to_branch = Branch.open(to_location)
+                except errors.NotBranchError:
+                    this_url = self._get_branch_location(control_dir)
+                    to_branch = Branch.open(
+                        urlutils.join(
+                            this_url, '..', urlutils.escape(to_location)))
         if revision is not None:
             revision = revision.as_revision_id(to_branch)
         switch.switch(control_dir, to_branch, force, revision_id=revision)
         if had_explicit_nick:
             branch = control_dir.open_branch() #get the new branch!
             branch.nick = to_branch.nick
-        note('Switched to branch: %s',
+        note(gettext('Switched to branch: %s'),
             urlutils.unescape_for_display(to_branch.base, 'utf-8'))
 
     def _get_branch_location(self, control_dir):
@@ -5973,38 +6299,38 @@
             name = current_view
         if delete:
             if file_list:
-                raise errors.BzrCommandError(
-                    "Both --delete and a file list specified")
+                raise errors.BzrCommandError(gettext(
+                    "Both --delete and a file list specified"))
             elif switch:
-                raise errors.BzrCommandError(
-                    "Both --delete and --switch specified")
+                raise errors.BzrCommandError(gettext(
+                    "Both --delete and --switch specified"))
             elif all:
                 tree.views.set_view_info(None, {})
-                self.outf.write("Deleted all views.\n")
+                self.outf.write(gettext("Deleted all views.\n"))
             elif name is None:
-                raise errors.BzrCommandError("No current view to delete")
+                raise errors.BzrCommandError(gettext("No current view to delete"))
             else:
                 tree.views.delete_view(name)
-                self.outf.write("Deleted '%s' view.\n" % name)
+                self.outf.write(gettext("Deleted '%s' view.\n") % name)
         elif switch:
             if file_list:
-                raise errors.BzrCommandError(
-                    "Both --switch and a file list specified")
+                raise errors.BzrCommandError(gettext(
+                    "Both --switch and a file list specified"))
             elif all:
-                raise errors.BzrCommandError(
-                    "Both --switch and --all specified")
+                raise errors.BzrCommandError(gettext(
+                    "Both --switch and --all specified"))
             elif switch == 'off':
                 if current_view is None:
-                    raise errors.BzrCommandError("No current view to disable")
+                    raise errors.BzrCommandError(gettext("No current view to disable"))
                 tree.views.set_view_info(None, view_dict)
-                self.outf.write("Disabled '%s' view.\n" % (current_view))
+                self.outf.write(gettext("Disabled '%s' view.\n") % (current_view))
             else:
                 tree.views.set_view_info(switch, view_dict)
                 view_str = views.view_display_str(tree.views.lookup_view())
-                self.outf.write("Using '%s' view: %s\n" % (switch, view_str))
+                self.outf.write(gettext("Using '{0}' view: {1}\n").format(switch, view_str))
         elif all:
             if view_dict:
-                self.outf.write('Views defined:\n')
+                self.outf.write(gettext('Views defined:\n'))
                 for view in sorted(view_dict):
                     if view == current_view:
                         active = "=>"
@@ -6013,25 +6339,25 @@
                     view_str = views.view_display_str(view_dict[view])
                     self.outf.write('%s %-20s %s\n' % (active, view, view_str))
             else:
-                self.outf.write('No views defined.\n')
+                self.outf.write(gettext('No views defined.\n'))
         elif file_list:
             if name is None:
                 # No name given and no current view set
                 name = 'my'
             elif name == 'off':
-                raise errors.BzrCommandError(
-                    "Cannot change the 'off' pseudo view")
+                raise errors.BzrCommandError(gettext(
+                    "Cannot change the 'off' pseudo view"))
             tree.views.set_view(name, sorted(file_list))
             view_str = views.view_display_str(tree.views.lookup_view())
-            self.outf.write("Using '%s' view: %s\n" % (name, view_str))
+            self.outf.write(gettext("Using '{0}' view: {1}\n").format(name, view_str))
         else:
             # list the files
             if name is None:
                 # No name given and no current view set
-                self.outf.write('No current view.\n')
+                self.outf.write(gettext('No current view.\n'))
             else:
                 view_str = views.view_display_str(tree.views.lookup_view(name))
-                self.outf.write("'%s' view is: %s\n" % (name, view_str))
+                self.outf.write(gettext("'{0}' view is: {1}\n").format(name, view_str))
 
 
 class cmd_hooks(Command):
@@ -6051,7 +6377,7 @@
                         self.outf.write("    %s\n" %
                                         (some_hooks.get_hook_name(hook),))
                 else:
-                    self.outf.write("    <no hooks installed>\n")
+                    self.outf.write(gettext("    <no hooks installed>\n"))
 
 
 class cmd_remove_branch(Command):
@@ -6158,7 +6484,7 @@
         manager = tree.get_shelf_manager()
         shelves = manager.active_shelves()
         if len(shelves) == 0:
-            note('No shelved changes.')
+            note(gettext('No shelved changes.'))
             return 0
         for shelf_id in reversed(shelves):
             message = manager.get_metadata(shelf_id).get('message')
@@ -6253,7 +6579,7 @@
         if path is not None:
             branchdir = path
         tree, branch, relpath =(
-            bzrdir.BzrDir.open_containing_tree_or_branch(branchdir))
+            controldir.ControlDir.open_containing_tree_or_branch(branchdir))
         if path is not None:
             path = relpath
         if tree is None:
@@ -6287,10 +6613,18 @@
     __doc__ = """Export command helps and error messages in po format."""
 
     hidden = True
+    takes_options = [Option('plugin', 
+                            help='Export help text from named command '\
+                                 '(defaults to all built in commands).',
+                            type=str),
+                     Option('include-duplicates',
+                            help='Output multiple copies of the same msgid '
+                                 'string if it appears more than once.'),
+                            ]
 
-    def run(self):
+    def run(self, plugin=None, include_duplicates=False):
         from bzrlib.export_pot import export_pot
-        export_pot(self.outf)
+        export_pot(self.outf, plugin, include_duplicates)
 
 
 def _register_lazy_builtins():

=== modified file 'bzrlib/bundle/__init__.py'
--- bzrlib/bundle/__init__.py	2011-07-23 16:33:38 +0000
+++ bzrlib/bundle/__init__.py	2011-12-18 15:28:38 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 from cStringIO import StringIO
 
 from bzrlib.lazy_import import lazy_import
@@ -25,6 +27,7 @@
     )
 from bzrlib.bundle import serializer as _serializer
 from bzrlib.merge_directive import MergeDirective
+from bzrlib.i18n import gettext
 """)
 from bzrlib.trace import note
 
@@ -53,7 +56,7 @@
         url, filename = urlutils.split(exception.target,
                                        exclude_trailing_slash=False)
         if not filename:
-            raise errors.NotABundle('A directory cannot be a bundle')
+            raise errors.NotABundle(gettext('A directory cannot be a bundle'))
         return _mod_transport.get_transport_from_url(url)
 
     try:

=== modified file 'bzrlib/bundle/apply_bundle.py'
--- bzrlib/bundle/apply_bundle.py	2011-05-02 15:29:58 +0000
+++ bzrlib/bundle/apply_bundle.py	2011-12-19 13:23:58 +0000
@@ -13,15 +13,19 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
 """\
 This contains functionality for installing bundles into repositories
 """
 
+from __future__ import absolute_import
+
 from bzrlib import ui
+from bzrlib.i18n import gettext
+from bzrlib.merge import Merger
 from bzrlib.progress import ProgressPhase
-from bzrlib.merge import Merger
+from bzrlib.trace import note
 from bzrlib.vf_repository import install_revision
-from bzrlib.trace import note
 
 
 def install_bundle(repository, bundle_reader):
@@ -33,7 +37,7 @@
     try:
         real_revisions = bundle_reader.real_revisions
         for i, revision in enumerate(reversed(real_revisions)):
-            pb.update("Install revisions",i, len(real_revisions))
+            pb.update(gettext("Install revisions"),i, len(real_revisions))
             if repository.has_revision(revision.revision_id):
                 continue
             cset_tree = bundle_reader.revision_tree(repository,
@@ -64,7 +68,7 @@
         merger.pp.next_phase()
         merger.find_base()
         if merger.base_rev_id == merger.other_rev_id:
-            note("Nothing to do.")
+            note(gettext("Nothing to do."))
             return 0
         merger.merge_type = merge_type
         merger.show_base = show_base

=== modified file 'bzrlib/bundle/bundle_data.py'
--- bzrlib/bundle/bundle_data.py	2011-05-13 14:02:14 +0000
+++ bzrlib/bundle/bundle_data.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,8 @@
 
 """Read in a bundle stream, and process it into a BundleReader object."""
 
+from __future__ import absolute_import
+
 import base64
 from cStringIO import StringIO
 import os

=== modified file 'bzrlib/bundle/commands.py'
--- bzrlib/bundle/commands.py	2011-04-07 10:36:24 +0000
+++ bzrlib/bundle/commands.py	2011-12-19 13:23:58 +0000
@@ -13,6 +13,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
 """\
 This is an attempt to take the internal delta object, and represent
 it as a single-file text-only changeset.
@@ -20,6 +21,8 @@
 and for applying a changeset.
 """
 
+from __future__ import absolute_import
+
 from cStringIO import StringIO
 
 from bzrlib.lazy_import import lazy_import
@@ -32,6 +35,7 @@
     urlutils,
     transport,
     )
+from bzrlib.i18n import gettext
 """)
 
 from bzrlib.commands import Command
@@ -56,11 +60,11 @@
             bundle_info = read_bundle(bundle_file)
         else:
             if verbose:
-                raise errors.BzrCommandError('--verbose requires a merge'
-                    ' directive')
+                raise errors.BzrCommandError(gettext(
+                            '--verbose requires a merge directive'))
         reader_method = getattr(bundle_info, 'get_bundle_reader', None)
         if reader_method is None:
-            raise errors.BzrCommandError('Bundle format not supported')
+            raise errors.BzrCommandError(gettext('Bundle format not supported'))
 
         by_kind = {}
         file_ids = set()
@@ -70,13 +74,13 @@
                 (bytes, parents, repo_kind, revision_id, file_id))
             if file_id is not None:
                 file_ids.add(file_id)
-        self.outf.write('Records\n')
+        self.outf.write(gettext('Records\n'))
         for kind, records in sorted(by_kind.iteritems()):
             multiparent = sum(1 for b, m, k, r, f in records if
                               len(m.get('parents', [])) > 1)
-            self.outf.write('%s: %d (%d multiparent)\n' % \
-                (kind, len(records), multiparent))
-        self.outf.write('unique files: %d\n' % len(file_ids))
+            self.outf.write(gettext('{0}: {1} ({2} multiparent)\n').format(
+                kind, len(records), multiparent))
+        self.outf.write(gettext('unique files: %d\n') % len(file_ids))
         self.outf.write('\n')
         nicks = set()
         committers = set()
@@ -85,10 +89,10 @@
                 nicks.add(revision.properties['branch-nick'])
             committers.add(revision.committer)
 
-        self.outf.write('Revisions\n')
-        self.outf.write(('nicks: %s\n'
+        self.outf.write(gettext('Revisions\n'))
+        self.outf.write((gettext('nicks: %s\n')
             % ', '.join(sorted(nicks))).encode(term_encoding, 'replace'))
-        self.outf.write(('committers: \n%s\n' %
+        self.outf.write((gettext('committers: \n%s\n') %
         '\n'.join(sorted(committers)).encode(term_encoding, 'replace')))
         if verbose:
             self.outf.write('\n')
@@ -96,6 +100,6 @@
             line = bundle_file.readline()
             line = bundle_file.readline()
             content = bundle_file.read().decode('bz2')
-            self.outf.write("Decoded contents\n")
+            self.outf.write(gettext("Decoded contents\n"))
             self.outf.write(content)
             self.outf.write('\n')

=== modified file 'bzrlib/bundle/serializer/__init__.py'
--- bzrlib/bundle/serializer/__init__.py	2011-06-16 11:58:22 +0000
+++ bzrlib/bundle/serializer/__init__.py	2011-12-19 13:23:58 +0000
@@ -17,6 +17,8 @@
 """Serializer factory for reading and writing bundles.
 """
 
+from __future__ import absolute_import
+
 import base64
 from StringIO import StringIO
 import re

=== modified file 'bzrlib/bundle/serializer/v08.py'
--- bzrlib/bundle/serializer/v08.py	2011-06-14 02:21:41 +0000
+++ bzrlib/bundle/serializer/v08.py	2011-12-19 13:23:58 +0000
@@ -17,7 +17,7 @@
 """Serializer factory for reading and writing bundles.
 """
 
-import os
+from __future__ import absolute_import
 
 from bzrlib import (
     errors,
@@ -27,15 +27,13 @@
                                       _get_bundle_header,
                                      )
 from bzrlib.bundle.serializer import binary_diff
-from bzrlib.bundle.bundle_data import (RevisionInfo, BundleInfo, BundleTree)
+from bzrlib.bundle.bundle_data import (RevisionInfo, BundleInfo)
 from bzrlib.diff import internal_diff
-from bzrlib.osutils import pathjoin
 from bzrlib.revision import NULL_REVISION
 from bzrlib.testament import StrictTestament
 from bzrlib.timestamp import (
     format_highres_date,
-    unpack_highres_date,
-)
+    )
 from bzrlib.textfile import text_file
 from bzrlib.trace import mutter
 

=== modified file 'bzrlib/bundle/serializer/v09.py'
--- bzrlib/bundle/serializer/v09.py	2011-04-18 15:53:40 +0000
+++ bzrlib/bundle/serializer/v09.py	2011-12-18 15:28:38 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 from bzrlib.bundle.serializer import _get_bundle_header
 from bzrlib.bundle.serializer.v08 import BundleSerializerV08, BundleReader
 from bzrlib.testament import StrictTestament3

=== modified file 'bzrlib/bundle/serializer/v4.py'
--- bzrlib/bundle/serializer/v4.py	2010-08-10 20:03:44 +0000
+++ bzrlib/bundle/serializer/v4.py	2011-12-18 15:28:38 +0000
@@ -14,12 +14,13 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 from cStringIO import StringIO
 import bz2
 import re
 
 from bzrlib import (
-    diff,
     errors,
     iterablefile,
     lru_cache,
@@ -33,6 +34,7 @@
     versionedfile as _mod_versionedfile,
     )
 from bzrlib.bundle import bundle_data, serializer as bundle_serializer
+from bzrlib.i18n import ngettext
 from bzrlib import bencode
 
 
@@ -322,7 +324,8 @@
 
     def do_write(self):
         """Write all data to the bundle"""
-        trace.note('Bundling %d revision(s).', len(self.revision_ids))
+        trace.note(ngettext('Bundling %d revision.', 'Bundling %d revisions.',
+                            len(self.revision_ids)), len(self.revision_ids))
         self.repository.lock_read()
         try:
             self.bundle.begin()

=== renamed file 'tools/build_mo.py' => 'bzrlib/bzr_distutils.py'
--- tools/build_mo.py	2011-05-25 03:37:36 +0000
+++ bzrlib/bzr_distutils.py	2011-12-19 13:23:58 +0000
@@ -1,13 +1,11 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2007 Lukáš Lalinský <lalinsky@gmail.com>
-# Copyright (C) 2007,2009 Alexander Belchenko <bialix@ukr.net>
-# Copyright 2011 Canonical Ltd.
+# Copyright (C) 2007,2009,2011 Canonical Ltd.
 #
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License
-# as published by the Free Software Foundation; either version 2
-# of the License, or (at your option) any later version.
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -16,12 +14,14 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-# This code is bring from bzr-explorer and modified for bzr.
+# This code is from bzr-explorer and modified for bzr.
 
 """build_mo command for setup.py"""
 
+from __future__ import absolute_import
+
 from distutils import log
 from distutils.core import Command
 from distutils.dep_util import newer

=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2011-08-17 09:27:29 +0000
+++ bzrlib/bzrdir.py	2011-12-22 19:54:56 +0000
@@ -25,6 +25,8 @@
 objects returned.
 """
 
+from __future__ import absolute_import
+
 import sys
 
 from bzrlib.lazy_import import lazy_import
@@ -46,6 +48,7 @@
     transport as _mod_transport,
     ui,
     urlutils,
+    vf_search,
     win32utils,
     workingtree_3,
     workingtree_4,
@@ -55,6 +58,7 @@
     do_catching_redirections,
     local,
     )
+from bzrlib.i18n import gettext
 """)
 
 from bzrlib.trace import (
@@ -65,7 +69,6 @@
 from bzrlib import (
     config,
     controldir,
-    hooks,
     registry,
     )
 from bzrlib.symbol_versioning import (
@@ -200,7 +203,7 @@
                 if (result_repo.user_url == result.user_url
                     and not require_stacking and
                     revision_id is not None):
-                    fetch_spec = graph.PendingAncestryResult(
+                    fetch_spec = vf_search.PendingAncestryResult(
                         [revision_id], local_repo)
                     result_repo.fetch(local_repo, fetch_spec=fetch_spec)
                 else:
@@ -221,7 +224,7 @@
             # the tree and fail.
             result.root_transport.local_abspath('.')
             if result_repo is None or result_repo.make_working_trees():
-                self.open_workingtree().clone(result)
+                self.open_workingtree().clone(result, revision_id=revision_id)
         except (errors.NoWorkingTree, errors.NotLocalUrl):
             pass
         return result
@@ -232,97 +235,6 @@
         t = _mod_transport.get_transport(url)
         t.ensure_base()
 
-    @staticmethod
-    def find_bzrdirs(transport, evaluate=None, list_current=None):
-        """Find bzrdirs recursively from current location.
-
-        This is intended primarily as a building block for more sophisticated
-        functionality, like finding trees under a directory, or finding
-        branches that use a given repository.
-
-        :param evaluate: An optional callable that yields recurse, value,
-            where recurse controls whether this bzrdir is recursed into
-            and value is the value to yield.  By default, all bzrdirs
-            are recursed into, and the return value is the bzrdir.
-        :param list_current: if supplied, use this function to list the current
-            directory, instead of Transport.list_dir
-        :return: a generator of found bzrdirs, or whatever evaluate returns.
-        """
-        if list_current is None:
-            def list_current(transport):
-                return transport.list_dir('')
-        if evaluate is None:
-            def evaluate(bzrdir):
-                return True, bzrdir
-
-        pending = [transport]
-        while len(pending) > 0:
-            current_transport = pending.pop()
-            recurse = True
-            try:
-                bzrdir = BzrDir.open_from_transport(current_transport)
-            except (errors.NotBranchError, errors.PermissionDenied):
-                pass
-            else:
-                recurse, value = evaluate(bzrdir)
-                yield value
-            try:
-                subdirs = list_current(current_transport)
-            except (errors.NoSuchFile, errors.PermissionDenied):
-                continue
-            if recurse:
-                for subdir in sorted(subdirs, reverse=True):
-                    pending.append(current_transport.clone(subdir))
-
-    @staticmethod
-    def find_branches(transport):
-        """Find all branches under a transport.
-
-        This will find all branches below the transport, including branches
-        inside other branches.  Where possible, it will use
-        Repository.find_branches.
-
-        To list all the branches that use a particular Repository, see
-        Repository.find_branches
-        """
-        def evaluate(bzrdir):
-            try:
-                repository = bzrdir.open_repository()
-            except errors.NoRepositoryPresent:
-                pass
-            else:
-                return False, ([], repository)
-            return True, (bzrdir.list_branches(), None)
-        ret = []
-        for branches, repo in BzrDir.find_bzrdirs(transport,
-                                                  evaluate=evaluate):
-            if repo is not None:
-                ret.extend(repo.find_branches())
-            if branches is not None:
-                ret.extend(branches)
-        return ret
-
-    @staticmethod
-    def create_branch_and_repo(base, force_new_repo=False, format=None):
-        """Create a new BzrDir, Branch and Repository at the url 'base'.
-
-        This will use the current default BzrDirFormat unless one is
-        specified, and use whatever
-        repository format that that uses via bzrdir.create_branch and
-        create_repository. If a shared repository is available that is used
-        preferentially.
-
-        The created Branch object is returned.
-
-        :param base: The URL to create the branch at.
-        :param force_new_repo: If True a new repository is always created.
-        :param format: If supplied, the format of branch to create.  If not
-            supplied, the default is used.
-        """
-        bzrdir = BzrDir.create(base, format)
-        bzrdir._find_or_create_repository(force_new_repo)
-        return bzrdir.create_branch()
-
     def determine_repository_policy(self, force_new_repo=False, stack_on=None,
                                     stack_on_pwd=None, require_stacking=False):
         """Return an object representing a policy to use.
@@ -461,12 +373,20 @@
         if revision_id is not None:
             fetch_spec_factory.add_revision_ids([revision_id])
             fetch_spec_factory.source_branch_stop_revision_id = revision_id
+        if possible_transports is None:
+            possible_transports = []
+        else:
+            possible_transports = list(possible_transports) + [
+                self.root_transport]
         target_transport = _mod_transport.get_transport(url,
             possible_transports)
         target_transport.ensure_base()
         cloning_format = self.cloning_metadir(stacked)
         # Create/update the result branch
-        result = cloning_format.initialize_on_transport(target_transport)
+        try:
+            result = controldir.ControlDir.open_from_transport(target_transport)
+        except errors.NotBranchError:
+            result = cloning_format.initialize_on_transport(target_transport)
         source_branch, source_repository = self._find_source_repo(
             add_cleanup, source_branch)
         fetch_spec_factory.source_branch = source_branch
@@ -478,7 +398,8 @@
             stacked_branch_url = None
         repository_policy = result.determine_repository_policy(
             force_new_repo, stacked_branch_url, require_stacking=stacked)
-        result_repo, is_new_repo = repository_policy.acquire_repository()
+        result_repo, is_new_repo = repository_policy.acquire_repository(
+            possible_transports=possible_transports)
         add_cleanup(result_repo.lock_write().unlock)
         fetch_spec_factory.source_repo = source_repository
         fetch_spec_factory.target_repo = result_repo
@@ -505,7 +426,7 @@
         mutter("created new branch %r" % (result_branch,))
 
         # Create/update the result working tree
-        if (create_tree_if_local and
+        if (create_tree_if_local and not result.has_workingtree() and
             isinstance(target_transport, local.LocalTransport) and
             (result_repo is None or result_repo.make_working_trees())):
             wt = result.create_workingtree(accelerator_tree=accelerator_tree,
@@ -543,75 +464,6 @@
                     stacked=stacked)
         return result
 
-
-
-    @staticmethod
-    def create_branch_convenience(base, force_new_repo=False,
-                                  force_new_tree=None, format=None,
-                                  possible_transports=None):
-        """Create a new BzrDir, Branch and Repository at the url 'base'.
-
-        This is a convenience function - it will use an existing repository
-        if possible, can be told explicitly whether to create a working tree or
-        not.
-
-        This will use the current default BzrDirFormat unless one is
-        specified, and use whatever
-        repository format that that uses via bzrdir.create_branch and
-        create_repository. If a shared repository is available that is used
-        preferentially. Whatever repository is used, its tree creation policy
-        is followed.
-
-        The created Branch object is returned.
-        If a working tree cannot be made due to base not being a file:// url,
-        no error is raised unless force_new_tree is True, in which case no
-        data is created on disk and NotLocalUrl is raised.
-
-        :param base: The URL to create the branch at.
-        :param force_new_repo: If True a new repository is always created.
-        :param force_new_tree: If True or False force creation of a tree or
-                               prevent such creation respectively.
-        :param format: Override for the bzrdir format to create.
-        :param possible_transports: An optional reusable transports list.
-        """
-        if force_new_tree:
-            # check for non local urls
-            t = _mod_transport.get_transport(base, possible_transports)
-            if not isinstance(t, local.LocalTransport):
-                raise errors.NotLocalUrl(base)
-        bzrdir = BzrDir.create(base, format, possible_transports)
-        repo = bzrdir._find_or_create_repository(force_new_repo)
-        result = bzrdir.create_branch()
-        if force_new_tree or (repo.make_working_trees() and
-                              force_new_tree is None):
-            try:
-                bzrdir.create_workingtree()
-            except errors.NotLocalUrl:
-                pass
-        return result
-
-    @staticmethod
-    def create_standalone_workingtree(base, format=None):
-        """Create a new BzrDir, WorkingTree, Branch and Repository at 'base'.
-
-        'base' must be a local path or a file:// url.
-
-        This will use the current default BzrDirFormat unless one is
-        specified, and use whatever
-        repository format that that uses for bzrdirformat.create_workingtree,
-        create_branch and create_repository.
-
-        :param format: Override for the bzrdir format to create.
-        :return: The WorkingTree object.
-        """
-        t = _mod_transport.get_transport(base)
-        if not isinstance(t, local.LocalTransport):
-            raise errors.NotLocalUrl(base)
-        bzrdir = BzrDir.create_branch_and_repo(base,
-                                               force_new_repo=True,
-                                               format=format).bzrdir
-        return bzrdir.create_workingtree()
-
     @deprecated_method(deprecated_in((2, 3, 0)))
     def generate_backup_name(self, base):
         return self._available_backup_name(base)
@@ -634,7 +486,7 @@
             old_path = self.root_transport.abspath('.bzr')
             backup_dir = self._available_backup_name('backup.bzr')
             new_path = self.root_transport.abspath(backup_dir)
-            ui.ui_factory.note('making backup of %s\n  to %s' % (
+            ui.ui_factory.note(gettext('making backup of {0}\n  to {1}').format(
                 urlutils.unescape_for_display(old_path, 'utf-8'),
                 urlutils.unescape_for_display(new_path, 'utf-8')))
             self.root_transport.copy_tree('.bzr', backup_dir)
@@ -657,8 +509,8 @@
             try:
                 to_path = '.bzr.retired.%d' % i
                 self.root_transport.rename('.bzr', to_path)
-                note("renamed %s to %s"
-                    % (self.root_transport.abspath('.bzr'), to_path))
+                note(gettext("renamed {0} to {1}").format(
+                    self.root_transport.abspath('.bzr'), to_path))
                 return
             except (errors.TransportError, IOError, errors.PathError):
                 i += 1
@@ -691,7 +543,7 @@
                 return None
             # find the next containing bzrdir
             try:
-                found_bzrdir = BzrDir.open_containing_from_transport(
+                found_bzrdir = self.open_containing_from_transport(
                     next_transport)[0]
             except errors.NotBranchError:
                 return None
@@ -814,150 +666,6 @@
         # add new tests for it to the appropriate place.
         return filename == '.bzr' or filename.startswith('.bzr/')
 
-    @staticmethod
-    def open_unsupported(base):
-        """Open a branch which is not supported."""
-        return BzrDir.open(base, _unsupported=True)
-
-    @staticmethod
-    def open(base, _unsupported=False, possible_transports=None):
-        """Open an existing bzrdir, rooted at 'base' (url).
-
-        :param _unsupported: a private parameter to the BzrDir class.
-        """
-        t = _mod_transport.get_transport(base, possible_transports)
-        return BzrDir.open_from_transport(t, _unsupported=_unsupported)
-
-    @staticmethod
-    def open_from_transport(transport, _unsupported=False,
-                            _server_formats=True):
-        """Open a bzrdir within a particular directory.
-
-        :param transport: Transport containing the bzrdir.
-        :param _unsupported: private.
-        """
-        for hook in BzrDir.hooks['pre_open']:
-            hook(transport)
-        # Keep initial base since 'transport' may be modified while following
-        # the redirections.
-        base = transport.base
-        def find_format(transport):
-            return transport, controldir.ControlDirFormat.find_format(
-                transport, _server_formats=_server_formats)
-
-        def redirected(transport, e, redirection_notice):
-            redirected_transport = transport._redirected_to(e.source, e.target)
-            if redirected_transport is None:
-                raise errors.NotBranchError(base)
-            note('%s is%s redirected to %s',
-                 transport.base, e.permanently, redirected_transport.base)
-            return redirected_transport
-
-        try:
-            transport, format = do_catching_redirections(find_format,
-                                                         transport,
-                                                         redirected)
-        except errors.TooManyRedirections:
-            raise errors.NotBranchError(base)
-
-        format.check_support_status(_unsupported)
-        return format.open(transport, _found=True)
-
-    @staticmethod
-    def open_containing(url, possible_transports=None):
-        """Open an existing branch which contains url.
-
-        :param url: url to search from.
-
-        See open_containing_from_transport for more detail.
-        """
-        transport = _mod_transport.get_transport(url, possible_transports)
-        return BzrDir.open_containing_from_transport(transport)
-
-    @staticmethod
-    def open_containing_from_transport(a_transport):
-        """Open an existing branch which contains a_transport.base.
-
-        This probes for a branch at a_transport, and searches upwards from there.
-
-        Basically we keep looking up until we find the control directory or
-        run into the root.  If there isn't one, raises NotBranchError.
-        If there is one and it is either an unrecognised format or an unsupported
-        format, UnknownFormatError or UnsupportedFormatError are raised.
-        If there is one, it is returned, along with the unused portion of url.
-
-        :return: The BzrDir that contains the path, and a Unicode path
-                for the rest of the URL.
-        """
-        # this gets the normalised url back. I.e. '.' -> the full path.
-        url = a_transport.base
-        while True:
-            try:
-                result = BzrDir.open_from_transport(a_transport)
-                return result, urlutils.unescape(a_transport.relpath(url))
-            except errors.NotBranchError, e:
-                pass
-            try:
-                new_t = a_transport.clone('..')
-            except errors.InvalidURLJoin:
-                # reached the root, whatever that may be
-                raise errors.NotBranchError(path=url)
-            if new_t.base == a_transport.base:
-                # reached the root, whatever that may be
-                raise errors.NotBranchError(path=url)
-            a_transport = new_t
-
-    @classmethod
-    def open_tree_or_branch(klass, location):
-        """Return the branch and working tree at a location.
-
-        If there is no tree at the location, tree will be None.
-        If there is no branch at the location, an exception will be
-        raised
-        :return: (tree, branch)
-        """
-        bzrdir = klass.open(location)
-        return bzrdir._get_tree_branch()
-
-    @classmethod
-    def open_containing_tree_or_branch(klass, location):
-        """Return the branch and working tree contained by a location.
-
-        Returns (tree, branch, relpath).
-        If there is no tree at containing the location, tree will be None.
-        If there is no branch containing the location, an exception will be
-        raised
-        relpath is the portion of the path that is contained by the branch.
-        """
-        bzrdir, relpath = klass.open_containing(location)
-        tree, branch = bzrdir._get_tree_branch()
-        return tree, branch, relpath
-
-    @classmethod
-    def open_containing_tree_branch_or_repository(klass, location):
-        """Return the working tree, branch and repo contained by a location.
-
-        Returns (tree, branch, repository, relpath).
-        If there is no tree containing the location, tree will be None.
-        If there is no branch containing the location, branch will be None.
-        If there is no repository containing the location, repository will be
-        None.
-        relpath is the portion of the path that is contained by the innermost
-        BzrDir.
-
-        If no tree, branch or repository is found, a NotBranchError is raised.
-        """
-        bzrdir, relpath = klass.open_containing(location)
-        try:
-            tree, branch = bzrdir._get_tree_branch()
-        except errors.NotBranchError:
-            try:
-                repo = bzrdir.find_repository()
-                return None, None, repo, relpath
-            except (errors.NoRepositoryPresent):
-                raise errors.NotBranchError(location)
-        return tree, branch, branch.repository, relpath
-
     def _cloning_metadir(self):
         """Produce a metadir suitable for cloning with.
 
@@ -1003,7 +711,7 @@
 
         :require_stacking: If True, non-stackable formats will be upgraded
             to similar stackable formats.
-        :returns: a BzrDirFormat with all component formats either set
+        :returns: a ControlDirFormat with all component formats either set
             appropriately or set to None if that component should not be
             created.
         """
@@ -1021,24 +729,6 @@
             format.require_stacking()
         return format
 
-    @classmethod
-    def create(cls, base, format=None, possible_transports=None):
-        """Create a new BzrDir at the url 'base'.
-
-        :param format: If supplied, the format of branch to create.  If not
-            supplied, the default is used.
-        :param possible_transports: If supplied, a list of transports that
-            can be reused to share a remote connection.
-        """
-        if cls is not BzrDir:
-            raise AssertionError("BzrDir.create always creates the"
-                "default format, not one of %r" % cls)
-        t = _mod_transport.get_transport(base, possible_transports)
-        t.ensure_base()
-        if format is None:
-            format = controldir.ControlDirFormat.get_default_format()
-        return format.initialize_on_transport(t)
-
     def get_branch_transport(self, branch_format, name=None):
         """Get the transport for use by branch format in this BzrDir.
 
@@ -1078,60 +768,36 @@
         """
         raise NotImplementedError(self.get_workingtree_transport)
 
-
-class BzrDirHooks(hooks.Hooks):
-    """Hooks for BzrDir operations."""
-
-    def __init__(self):
-        """Create the default hooks."""
-        hooks.Hooks.__init__(self, "bzrlib.bzrdir", "BzrDir.hooks")
-        self.add_hook('pre_open',
-            "Invoked before attempting to open a BzrDir with the transport "
-            "that the open will use.", (1, 14))
-        self.add_hook('post_repo_init',
-            "Invoked after a repository has been initialized. "
-            "post_repo_init is called with a "
-            "bzrlib.bzrdir.RepoInitHookParams.",
-            (2, 2))
-
-# install the default hooks
-BzrDir.hooks = BzrDirHooks()
-
-
-class RepoInitHookParams(object):
-    """Object holding parameters passed to `*_repo_init` hooks.
-
-    There are 4 fields that hooks may wish to access:
-
-    :ivar repository: Repository created
-    :ivar format: Repository format
-    :ivar bzrdir: The bzrdir for the repository
-    :ivar shared: The repository is shared
-    """
-
-    def __init__(self, repository, format, a_bzrdir, shared):
-        """Create a group of RepoInitHook parameters.
-
-        :param repository: Repository created
-        :param format: Repository format
-        :param bzrdir: The bzrdir for the repository
-        :param shared: The repository is shared
+    @classmethod
+    def create(cls, base, format=None, possible_transports=None):
+        """Create a new BzrDir at the url 'base'.
+
+        :param format: If supplied, the format of branch to create.  If not
+            supplied, the default is used.
+        :param possible_transports: If supplied, a list of transports that
+            can be reused to share a remote connection.
         """
-        self.repository = repository
-        self.format = format
-        self.bzrdir = a_bzrdir
-        self.shared = shared
-
-    def __eq__(self, other):
-        return self.__dict__ == other.__dict__
+        if cls is not BzrDir:
+            raise AssertionError("BzrDir.create always creates the "
+                "default format, not one of %r" % cls)
+        return controldir.ControlDir.create(base, format=format,
+                possible_transports=possible_transports)
 
     def __repr__(self):
-        if self.repository:
-            return "<%s for %s>" % (self.__class__.__name__,
-                self.repository)
-        else:
-            return "<%s for %s>" % (self.__class__.__name__,
-                self.bzrdir)
+        return "<%s at %r>" % (self.__class__.__name__, self.user_url)
+
+    def update_feature_flags(self, updated_flags):
+        """Update the features required by this bzrdir.
+
+        :param updated_flags: Dictionary mapping feature names to necessities
+            A necessity can be None to indicate the feature should be removed
+        """
+        self.control_files.lock_write()
+        try:
+            self._format._update_feature_flags(updated_flags)
+            self.transport.put_bytes('branch-format', self._format.as_string())
+        finally:
+            self.control_files.unlock()
 
 
 class BzrDirMeta1(BzrDir):
@@ -1143,14 +809,23 @@
     present within a BzrDir.
     """
 
+    def __init__(self, _transport, _format):
+        super(BzrDirMeta1, self).__init__(_transport, _format)
+        self.control_files = lockable_files.LockableFiles(self.control_transport,
+            self._format._lock_file_name, self._format._lock_class)
+
     def can_convert_format(self):
         """See BzrDir.can_convert_format()."""
         return True
 
-    def create_branch(self, name=None, repository=None):
+    def create_branch(self, name=None, repository=None,
+            append_revisions_only=None):
         """See BzrDir.create_branch."""
+        if name is None:
+            name = self._get_selected_branch()
         return self._format.get_branch_format().initialize(self, name=name,
-                repository=repository)
+                repository=repository,
+                append_revisions_only=append_revisions_only)
 
     def destroy_branch(self, name=None):
         """See BzrDir.create_branch."""
@@ -1164,7 +839,10 @@
 
     def destroy_repository(self):
         """See BzrDir.destroy_repository."""
-        self.transport.delete_tree('repository')
+        try:
+            self.transport.delete_tree('repository')
+        except errors.NoSuchFile:
+            raise errors.NoRepositoryPresent(self)
 
     def create_workingtree(self, revision_id=None, from_branch=None,
                            accelerator_tree=None, hardlink=False):
@@ -1192,8 +870,8 @@
 
         This might be a synthetic object for e.g. RemoteBranch and SVN.
         """
-        from bzrlib.branch import BranchFormat
-        return BranchFormat.find_format(self, name=name)
+        from bzrlib.branch import BranchFormatMetadir
+        return BranchFormatMetadir.find_format(self, name=name)
 
     def _get_mkdir_mode(self):
         """Figure out the mode to use when creating a bzrdir subdir."""
@@ -1203,8 +881,8 @@
 
     def get_branch_reference(self, name=None):
         """See BzrDir.get_branch_reference()."""
-        from bzrlib.branch import BranchFormat
-        format = BranchFormat.find_format(self, name=name)
+        from bzrlib.branch import BranchFormatMetadir
+        format = BranchFormatMetadir.find_format(self, name=name)
         return format.get_reference(self, name=name)
 
     def get_branch_transport(self, branch_format, name=None):
@@ -1259,16 +937,17 @@
         Note: if you're going to open the working tree, you should just go
         ahead and try, and not ask permission first.
         """
-        from bzrlib.workingtree import WorkingTreeFormat
+        from bzrlib.workingtree import WorkingTreeFormatMetaDir
         try:
-            WorkingTreeFormat.find_format_string(self)
+            WorkingTreeFormatMetaDir.find_format_string(self)
         except errors.NoWorkingTree:
             return False
         return True
 
     def needs_format_conversion(self, format):
         """See BzrDir.needs_format_conversion()."""
-        if not isinstance(self._format, format.__class__):
+        if (not isinstance(self._format, format.__class__) or
+            self._format.get_format_string() != format.get_format_string()):
             # it is not a meta dir format, conversion is needed.
             return True
         # we might want to push this down to the repository?
@@ -1295,25 +974,28 @@
         return False
 
     def open_branch(self, name=None, unsupported=False,
-                    ignore_fallbacks=False):
-        """See BzrDir.open_branch."""
+                    ignore_fallbacks=False, possible_transports=None):
+        """See ControlDir.open_branch."""
+        if name is None:
+            name = self._get_selected_branch()
         format = self.find_branch_format(name=name)
         format.check_support_status(unsupported)
         return format.open(self, name=name,
-            _found=True, ignore_fallbacks=ignore_fallbacks)
+            _found=True, ignore_fallbacks=ignore_fallbacks,
+            possible_transports=possible_transports)
 
     def open_repository(self, unsupported=False):
         """See BzrDir.open_repository."""
-        from bzrlib.repository import RepositoryFormat
-        format = RepositoryFormat.find_format(self)
+        from bzrlib.repository import RepositoryFormatMetaDir
+        format = RepositoryFormatMetaDir.find_format(self)
         format.check_support_status(unsupported)
         return format.open(self, _found=True)
 
     def open_workingtree(self, unsupported=False,
             recommend_upgrade=True):
         """See BzrDir.open_workingtree."""
-        from bzrlib.workingtree import WorkingTreeFormat
-        format = WorkingTreeFormat.find_format(self)
+        from bzrlib.workingtree import WorkingTreeFormatMetaDir
+        format = WorkingTreeFormatMetaDir.find_format(self)
         format.check_support_status(unsupported, recommend_upgrade,
             basedir=self.root_transport.base)
         return format.open(self, _found=True)
@@ -1322,6 +1004,243 @@
         return config.TransportConfig(self.transport, 'control.conf')
 
 
+class BzrDirMeta1Colo(BzrDirMeta1):
+    """BzrDirMeta1 with support for colocated branches.
+
+    This format is experimental, and will eventually be merged back into
+    BzrDirMeta1.
+    """
+
+    def _get_branch_path(self, name):
+        """Obtain the branch path to use.
+
+        This uses the API specified branch name first, and then falls back to
+        the branch name specified in the URL. If neither of those is specified,
+        it uses the default branch.
+
+        :param name: Optional branch name to use
+        :return: Relative path to branch
+        """
+        if name is None:
+            return 'branch'
+        return urlutils.join('branches', name.encode("utf-8"))
+
+    def _read_branch_list(self):
+        """Read the branch list.
+
+        :return: List of utf-8 encoded branch names.
+        """
+        try:
+            f = self.control_transport.get('branch-list')
+        except errors.NoSuchFile:
+            return []
+
+        ret = []
+        try:
+            for name in f:
+                ret.append(name.rstrip("\n"))
+        finally:
+            f.close()
+        return ret
+
+    def _write_branch_list(self, branches):
+        """Write out the branch list.
+
+        :param branches: List of utf-8 branch names to write
+        """
+        self.transport.put_bytes('branch-list',
+            "".join([name+"\n" for name in branches]))
+
+    def destroy_branch(self, name=None):
+        """See BzrDir.create_branch."""
+        if name is None:
+            name = self._get_selected_branch()
+        path = self._get_branch_path(name)
+        if name is not None:
+            self.control_files.lock_write()
+            try:
+                branches = self._read_branch_list()
+                try:
+                    branches.remove(name.encode("utf-8"))
+                except ValueError:
+                    raise errors.NotBranchError(name)
+                self._write_branch_list(branches)
+            finally:
+                self.control_files.unlock()
+        self.transport.delete_tree(path)
+
+    def get_branches(self):
+        """See ControlDir.get_branches."""
+        ret = {}
+        try:
+            ret[None] = self.open_branch()
+        except (errors.NotBranchError, errors.NoRepositoryPresent):
+            pass
+
+        for name in self._read_branch_list():
+            ret[name] = self.open_branch(name.decode('utf-8'))
+
+        return ret
+
+    def get_branch_transport(self, branch_format, name=None):
+        """See BzrDir.get_branch_transport()."""
+        path = self._get_branch_path(name)
+        # XXX: this shouldn't implicitly create the directory if it's just
+        # promising to get a transport -- mbp 20090727
+        if branch_format is None:
+            return self.transport.clone(path)
+        try:
+            branch_format.get_format_string()
+        except NotImplementedError:
+            raise errors.IncompatibleFormat(branch_format, self._format)
+        if name is not None:
+            try:
+                self.transport.mkdir('branches', mode=self._get_mkdir_mode())
+            except errors.FileExists:
+                pass
+            branches = self._read_branch_list()
+            utf8_name = name.encode("utf-8")
+            if not utf8_name in branches:
+                self.control_files.lock_write()
+                try:
+                    branches = self._read_branch_list()
+                    branches.append(utf8_name)
+                    self._write_branch_list(branches)
+                finally:
+                    self.control_files.unlock()
+        try:
+            self.transport.mkdir(path, mode=self._get_mkdir_mode())
+        except errors.FileExists:
+            pass
+        return self.transport.clone(path)
+
+
+class BzrFormat(object):
+    """Base class for all formats of things living in metadirs.
+
+    This class manages the format string that is stored in the 'format'
+    or 'branch-format' file.
+
+    All classes for (branch-, repository-, workingtree-) formats that
+    live in meta directories and have their own 'format' file
+    (i.e. different from .bzr/branch-format) derive from this class,
+    as well as the relevant base class for their kind
+    (BranchFormat, WorkingTreeFormat, RepositoryFormat).
+
+    Each format is identified by a "format" or "branch-format" file with a
+    single line containing the base format name and then an optional list of
+    feature flags.
+
+    Feature flags are supported as of bzr 2.5. Setting feature flags on formats
+    will render them inaccessible to older versions of bzr.
+
+    :ivar features: Dictionary mapping feature names to their necessity
+    """
+
+    _present_features = set()
+
+    def __init__(self):
+        self.features = {}
+
+    @classmethod
+    def register_feature(cls, name):
+        """Register a feature as being present.
+
+        :param name: Name of the feature
+        """
+        if " " in name:
+            raise ValueError("spaces are not allowed in feature names")
+        if name in cls._present_features:
+            raise errors.FeatureAlreadyRegistered(name)
+        cls._present_features.add(name)
+
+    @classmethod
+    def unregister_feature(cls, name):
+        """Unregister a feature."""
+        cls._present_features.remove(name)
+
+    def check_support_status(self, allow_unsupported, recommend_upgrade=True,
+            basedir=None):
+        for name, necessity in self.features.iteritems():
+            if name in self._present_features:
+                continue
+            if necessity == "optional":
+                mutter("ignoring optional missing feature %s", name)
+                continue
+            elif necessity == "required":
+                raise errors.MissingFeature(name)
+            else:
+                mutter("treating unknown necessity as require for %s",
+                       name)
+                raise errors.MissingFeature(name)
+
+    @classmethod
+    def get_format_string(cls):
+        """Return the ASCII format string that identifies this format."""
+        raise NotImplementedError(cls.get_format_string)
+
+    @classmethod
+    def from_string(cls, text):
+        format_string = cls.get_format_string()
+        if not text.startswith(format_string):
+            raise AssertionError("Invalid format header %r for %r" % (text, cls))
+        lines = text[len(format_string):].splitlines()
+        ret = cls()
+        for lineno, line in enumerate(lines):
+            try:
+                (necessity, feature) = line.split(" ", 1)
+            except ValueError:
+                raise errors.ParseFormatError(format=cls, lineno=lineno+2,
+                    line=line, text=text)
+            ret.features[feature] = necessity
+        return ret
+
+    def as_string(self):
+        """Return the string representation of this format.
+        """
+        lines = [self.get_format_string()]
+        lines.extend([("%s %s\n" % (item[1], item[0])) for item in
+            self.features.iteritems()])
+        return "".join(lines)
+
+    @classmethod
+    def _find_format(klass, registry, kind, format_string):
+        try:
+            first_line = format_string[:format_string.index("\n")+1]
+        except ValueError:
+            first_line = format_string
+        try:
+            cls = registry.get(first_line)
+        except KeyError:
+            raise errors.UnknownFormatError(format=first_line, kind=kind)
+        return cls.from_string(format_string)
+
+    def network_name(self):
+        """A simple byte string uniquely identifying this format for RPC calls.
+
+        Metadir branch formats use their format string.
+        """
+        return self.as_string()
+
+    def __eq__(self, other):
+        return (self.__class__ is other.__class__ and
+                self.features == other.features)
+
+    def _update_feature_flags(self, updated_flags):
+        """Update the feature flags in this format.
+
+        :param updated_flags: Updated feature flags
+        """
+        for name, necessity in updated_flags.iteritems():
+            if necessity is None:
+                try:
+                    del self.features[name]
+                except KeyError:
+                    pass
+            else:
+                self.features[name] = necessity
+
+
 class BzrProber(controldir.Prober):
     """Prober for formats that use a .bzr/ control directory."""
 
@@ -1346,9 +1265,14 @@
         except errors.NoSuchFile:
             raise errors.NotBranchError(path=transport.base)
         try:
-            return klass.formats.get(format_string)
+            first_line = format_string[:format_string.index("\n")+1]
+        except ValueError:
+            first_line = format_string
+        try:
+            cls = klass.formats.get(first_line)
         except KeyError:
-            raise errors.UnknownFormatError(format=format_string, kind='bzrdir')
+            raise errors.UnknownFormatError(format=first_line, kind='bzrdir')
+        return cls.from_string(format_string)
 
     @classmethod
     def known_formats(cls):
@@ -1397,7 +1321,7 @@
         return set([RemoteBzrDirFormat()])
 
 
-class BzrDirFormat(controldir.ControlDirFormat):
+class BzrDirFormat(BzrFormat, controldir.ControlDirFormat):
     """ControlDirFormat base class for .bzr/ directories.
 
     Formats are placed in a dict by their format string for reference
@@ -1414,11 +1338,6 @@
     # _lock_class must be set in subclasses to the lock type, typ.
     # TransportLock or LockDir
 
-    @classmethod
-    def get_format_string(cls):
-        """Return the ASCII format string that identifies this format."""
-        raise NotImplementedError(cls.get_format_string)
-
     def initialize_on_transport(self, transport):
         """Initialize a new bzrdir in the base directory of a Transport."""
         try:
@@ -1559,7 +1478,7 @@
                        "This is a Bazaar control directory.\n"
                        "Do not change any files in this directory.\n"
                        "See http://bazaar.canonical.com/ for more information about Bazaar.\n"),
-                      ('branch-format', self.get_format_string()),
+                      ('branch-format', self.as_string()),
                       ]
         # NB: no need to escape relative paths that are url safe.
         control_files = lockable_files.LockableFiles(bzrdir_transport,
@@ -1609,6 +1528,19 @@
             compatible with whatever sub formats are supported by self.
         :return: None.
         """
+        other_format.features = dict(self.features)
+
+    def supports_transport(self, transport):
+        # bzr formats can be opened over all known transports
+        return True
+
+    def check_support_status(self, allow_unsupported, recommend_upgrade=True,
+            basedir=None):
+        controldir.ControlDirFormat.check_support_status(self,
+            allow_unsupported=allow_unsupported, recommend_upgrade=recommend_upgrade,
+            basedir=basedir)
+        BzrFormat.check_support_status(self, allow_unsupported=allow_unsupported,
+            recommend_upgrade=recommend_upgrade, basedir=basedir)
 
 
 class BzrDirMetaFormat1(BzrDirFormat):
@@ -1628,7 +1560,10 @@
 
     fixed_components = False
 
+    colocated_branches = False
+
     def __init__(self):
+        BzrDirFormat.__init__(self)
         self._workingtree_format = None
         self._branch_format = None
         self._repository_format = None
@@ -1640,6 +1575,8 @@
             return False
         if other.workingtree_format != self.workingtree_format:
             return False
+        if other.features != self.features:
+            return False
         return True
 
     def __ne__(self, other):
@@ -1723,8 +1660,8 @@
                     new_repo_format = None
             if new_repo_format is not None:
                 self.repository_format = new_repo_format
-                note('Source repository format does not support stacking,'
-                     ' using format:\n  %s',
+                note(gettext('Source repository format does not support stacking,'
+                     ' using format:\n  %s'),
                      new_repo_format.get_format_description())
 
         if not self.get_branch_format().supports_stacking():
@@ -1743,14 +1680,20 @@
             if new_branch_format is not None:
                 # Does support stacking, use its format.
                 self.set_branch_format(new_branch_format)
-                note('Source branch format does not support stacking,'
-                     ' using format:\n  %s',
+                note(gettext('Source branch format does not support stacking,'
+                     ' using format:\n  %s'),
                      new_branch_format.get_format_description())
 
     def get_converter(self, format=None):
         """See BzrDirFormat.get_converter()."""
         if format is None:
             format = BzrDirFormat.get_default_format()
+        if (type(self) is BzrDirMetaFormat1 and
+            type(format) is BzrDirMetaFormat1Colo):
+            return ConvertMetaToColo(format)
+        if (type(self) is BzrDirMetaFormat1Colo and
+            type(format) is BzrDirMetaFormat1):
+            return ConvertMetaRemoveColo(format)
         if not isinstance(self, format.__class__):
             # converting away from metadir is not implemented
             raise NotImplementedError(self.get_converter)
@@ -1765,9 +1708,6 @@
         """See BzrDirFormat.get_format_description()."""
         return "Meta directory format 1"
 
-    def network_name(self):
-        return self.get_format_string()
-
     def _open(self, transport):
         """See BzrDirFormat._open."""
         # Create a new format instance because otherwise initialisation of new
@@ -1802,6 +1742,7 @@
             compatible with whatever sub formats are supported by self.
         :return: None.
         """
+        super(BzrDirMetaFormat1, self)._supply_sub_formats_to(other_format)
         if getattr(self, '_repository_format', None) is not None:
             other_format.repository_format = self.repository_format
         if self._branch_format is not None:
@@ -1820,6 +1761,9 @@
     def __set_workingtree_format(self, wt_format):
         self._workingtree_format = wt_format
 
+    def __repr__(self):
+        return "<%r>" % (self.__class__.__name__,)
+
     workingtree_format = property(__get_workingtree_format,
                                   __set_workingtree_format)
 
@@ -1830,6 +1774,34 @@
 controldir.ControlDirFormat._default_format = BzrDirMetaFormat1()
 
 
+class BzrDirMetaFormat1Colo(BzrDirMetaFormat1):
+    """BzrDirMeta1 format with support for colocated branches."""
+
+    colocated_branches = True
+
+    @classmethod
+    def get_format_string(cls):
+        """See BzrDirFormat.get_format_string()."""
+        return "Bazaar meta directory, format 1 (with colocated branches)\n"
+
+    def get_format_description(self):
+        """See BzrDirFormat.get_format_description()."""
+        return "Meta directory format 1 with support for colocated branches"
+
+    def _open(self, transport):
+        """See BzrDirFormat._open."""
+        # Create a new format instance because otherwise initialisation of new
+        # metadirs share the global default format object leading to alias
+        # problems.
+        format = BzrDirMetaFormat1Colo()
+        self._supply_sub_formats_to(format)
+        return BzrDirMeta1Colo(transport, format)
+
+
+BzrProber.formats.register(BzrDirMetaFormat1Colo.get_format_string(),
+    BzrDirMetaFormat1Colo)
+
+
 class ConvertMetaToMeta(controldir.Converter):
     """Converts the components of metadirs."""
 
@@ -1854,7 +1826,7 @@
         else:
             if not isinstance(repo._format, self.target_format.repository_format.__class__):
                 from bzrlib.repository import CopyConverter
-                ui.ui_factory.note('starting repository conversion')
+                ui.ui_factory.note(gettext('starting repository conversion'))
                 converter = CopyConverter(self.target_format.repository_format)
                 converter.convert(repo, pb)
         for branch in self.bzrdir.list_branches():
@@ -1908,6 +1880,49 @@
         return to_convert
 
 
+class ConvertMetaToColo(controldir.Converter):
+    """Add colocated branch support."""
+
+    def __init__(self, target_format):
+        """Create a converter.that upgrades a metadir to the colo format.
+
+        :param target_format: The final metadir format that is desired.
+        """
+        self.target_format = target_format
+
+    def convert(self, to_convert, pb):
+        """See Converter.convert()."""
+        to_convert.transport.put_bytes('branch-format',
+            self.target_format.as_string())
+        return BzrDir.open_from_transport(to_convert.root_transport)
+
+
+class ConvertMetaRemoveColo(controldir.Converter):
+    """Remove colocated branch support from a bzrdir."""
+
+    def __init__(self, target_format):
+        """Create a converter.that downgrades a colocated branch metadir
+        to a regular metadir.
+
+        :param target_format: The final metadir format that is desired.
+        """
+        self.target_format = target_format
+
+    def convert(self, to_convert, pb):
+        """See Converter.convert()."""
+        to_convert.control_files.lock_write()
+        try:
+            branches = to_convert.list_branches()
+            if len(branches) > 1:
+                raise errors.BzrError("remove all but a single "
+                    "colocated branch when downgrading")
+        finally:
+            to_convert.control_files.unlock()
+        to_convert.transport.put_bytes('branch-format',
+            self.target_format.as_string())
+        return BzrDir.open_from_transport(to_convert.root_transport)
+
+
 controldir.ControlDirFormat.register_server_prober(RemoteBzrProber)
 
 
@@ -1990,7 +2005,8 @@
         else:
             self._require_stacking = True
 
-    def acquire_repository(self, make_working_trees=None, shared=False):
+    def acquire_repository(self, make_working_trees=None, shared=False,
+            possible_transports=None):
         """Acquire a repository for this bzrdir.
 
         Implementations may create a new repository or use a pre-exising
@@ -2002,7 +2018,7 @@
         :return: A repository, is_new_flag (True if the repository was
             created).
         """
-        raise NotImplemented(RepositoryAcquisitionPolicy.acquire_repository)
+        raise NotImplementedError(RepositoryAcquisitionPolicy.acquire_repository)
 
 
 class CreateRepository(RepositoryAcquisitionPolicy):
@@ -2021,23 +2037,29 @@
                                              require_stacking)
         self._bzrdir = bzrdir
 
-    def acquire_repository(self, make_working_trees=None, shared=False):
+    def acquire_repository(self, make_working_trees=None, shared=False,
+            possible_transports=None):
         """Implementation of RepositoryAcquisitionPolicy.acquire_repository
 
         Creates the desired repository in the bzrdir we already have.
         """
+        if possible_transports is None:
+            possible_transports = []
+        else:
+            possible_transports = list(possible_transports)
+        possible_transports.append(self._bzrdir.root_transport)
         stack_on = self._get_full_stack_on()
         if stack_on:
             format = self._bzrdir._format
             format.require_stacking(stack_on=stack_on,
-                                    possible_transports=[self._bzrdir.root_transport])
+                                    possible_transports=possible_transports)
             if not self._require_stacking:
                 # We have picked up automatic stacking somewhere.
-                note('Using default stacking branch %s at %s', self._stack_on,
-                    self._stack_on_pwd)
+                note(gettext('Using default stacking branch {0} at {1}').format(
+                    self._stack_on, self._stack_on_pwd))
         repository = self._bzrdir.create_repository(shared=shared)
         self._add_fallback(repository,
-                           possible_transports=[self._bzrdir.transport])
+                           possible_transports=possible_transports)
         if make_working_trees is not None:
             repository.set_make_working_trees(make_working_trees)
         return repository, True
@@ -2059,13 +2081,19 @@
                                              require_stacking)
         self._repository = repository
 
-    def acquire_repository(self, make_working_trees=None, shared=False):
+    def acquire_repository(self, make_working_trees=None, shared=False,
+            possible_transports=None):
         """Implementation of RepositoryAcquisitionPolicy.acquire_repository
 
         Returns an existing repository to use.
         """
+        if possible_transports is None:
+            possible_transports = []
+        else:
+            possible_transports = list(possible_transports)
+        possible_transports.append(self._repository.bzrdir.transport)
         self._add_fallback(self._repository,
-                       possible_transports=[self._repository.bzrdir.transport])
+                       possible_transports=possible_transports)
         return self._repository, False
 
 
@@ -2075,11 +2103,11 @@
          tree_format=None,
          hidden=False,
          experimental=False,
-         alias=False):
+         alias=False, bzrdir_format=None):
     """Register a metadir subformat.
 
-    These all use a BzrDirMetaFormat1 bzrdir, but can be parameterized
-    by the Repository/Branch/WorkingTreeformats.
+    These all use a meta bzrdir, but can be parameterized by the
+    Repository/Branch/WorkingTreeformats.
 
     :param repository_format: The fully-qualified repository format class
         name as a string.
@@ -2088,8 +2116,10 @@
     :param tree_format: Fully-qualified tree format class name as
         a string.
     """
+    if bzrdir_format is None:
+        bzrdir_format = BzrDirMetaFormat1
     # This should be expanded to support setting WorkingTree and Branch
-    # formats, once BzrDirMetaFormat1 supports that.
+    # formats, once the API supports that.
     def _load(full_name):
         mod_name, factory_name = full_name.rsplit('.', 1)
         try:
@@ -2102,7 +2132,7 @@
         return factory()
 
     def helper():
-        bd = BzrDirMetaFormat1()
+        bd = bzrdir_format()
         if branch_format is not None:
             bd.set_branch_format(_load(branch_format))
         if tree_format is not None:
@@ -2122,34 +2152,33 @@
     deprecated=True)
 register_metadir(controldir.format_registry, 'dirstate',
     'bzrlib.repofmt.knitrepo.RepositoryFormatKnit1',
-    help='New in 0.15: Fast local operations. Compatible with bzr 0.8 and '
-        'above when accessed over the network.',
+    help='Format using dirstate for working trees. '
+        'Compatible with bzr 0.8 and '
+        'above when accessed over the network. Introduced in bzr 0.15.',
     branch_format='bzrlib.branch.BzrBranchFormat5',
     tree_format='bzrlib.workingtree_4.WorkingTreeFormat4',
     hidden=True,
     deprecated=True)
 register_metadir(controldir.format_registry, 'dirstate-tags',
     'bzrlib.repofmt.knitrepo.RepositoryFormatKnit1',
-    help='New in 0.15: Fast local operations and improved scaling for '
-        'network operations. Additionally adds support for tags.'
-        ' Incompatible with bzr < 0.15.',
+    help='Variant of dirstate with support for tags. '
+        'Introduced in bzr 0.15.',
     branch_format='bzrlib.branch.BzrBranchFormat6',
     tree_format='bzrlib.workingtree_4.WorkingTreeFormat4',
     hidden=True,
     deprecated=True)
 register_metadir(controldir.format_registry, 'rich-root',
     'bzrlib.repofmt.knitrepo.RepositoryFormatKnit4',
-    help='New in 1.0.  Better handling of tree roots.  Incompatible with'
-        ' bzr < 1.0.',
+    help='Variant of dirstate with better handling of tree roots. '
+        'Introduced in bzr 1.0',
     branch_format='bzrlib.branch.BzrBranchFormat6',
     tree_format='bzrlib.workingtree_4.WorkingTreeFormat4',
     hidden=True,
     deprecated=True)
 register_metadir(controldir.format_registry, 'dirstate-with-subtree',
     'bzrlib.repofmt.knitrepo.RepositoryFormatKnit3',
-    help='New in 0.15: Fast local operations and improved scaling for '
-        'network operations. Additionally adds support for versioning nested '
-        'bzr branches. Incompatible with bzr < 0.15.',
+    help='Variant of dirstate with support for nested trees. '
+         'Introduced in 0.15.',
     branch_format='bzrlib.branch.BzrBranchFormat6',
     tree_format='bzrlib.workingtree_4.WorkingTreeFormat4',
     experimental=True,
@@ -2157,31 +2186,34 @@
     )
 register_metadir(controldir.format_registry, 'pack-0.92',
     'bzrlib.repofmt.knitpack_repo.RepositoryFormatKnitPack1',
-    help='New in 0.92: Pack-based format with data compatible with '
-        'dirstate-tags format repositories. Interoperates with '
-        'bzr repositories before 0.92 but cannot be read by bzr < 0.92. '
+    help='Pack-based format used in 1.x series. Introduced in 0.92. '
+        'Interoperates with bzr repositories before 0.92 but cannot be '
+        'read by bzr < 0.92. '
         ,
     branch_format='bzrlib.branch.BzrBranchFormat6',
     tree_format='bzrlib.workingtree_4.WorkingTreeFormat4',
+    deprecated=True,
     )
 register_metadir(controldir.format_registry, 'pack-0.92-subtree',
     'bzrlib.repofmt.knitpack_repo.RepositoryFormatKnitPack3',
-    help='New in 0.92: Pack-based format with data compatible with '
-        'dirstate-with-subtree format repositories. Interoperates with '
+    help='Pack-based format used in 1.x series, with subtree support. '
+        'Introduced in 0.92. Interoperates with '
         'bzr repositories before 0.92 but cannot be read by bzr < 0.92. '
         ,
     branch_format='bzrlib.branch.BzrBranchFormat6',
     tree_format='bzrlib.workingtree_4.WorkingTreeFormat4',
     hidden=True,
+    deprecated=True,
     experimental=True,
     )
 register_metadir(controldir.format_registry, 'rich-root-pack',
     'bzrlib.repofmt.knitpack_repo.RepositoryFormatKnitPack4',
-    help='New in 1.0: A variant of pack-0.92 that supports rich-root data '
-         '(needed for bzr-svn and bzr-git).',
+    help='A variant of pack-0.92 that supports rich-root data '
+         '(needed for bzr-svn and bzr-git). Introduced in 1.0.',
     branch_format='bzrlib.branch.BzrBranchFormat6',
     tree_format='bzrlib.workingtree_4.WorkingTreeFormat4',
     hidden=True,
+    deprecated=True,
     )
 register_metadir(controldir.format_registry, '1.6',
     'bzrlib.repofmt.knitpack_repo.RepositoryFormatKnitPack5',
@@ -2191,6 +2223,7 @@
     branch_format='bzrlib.branch.BzrBranchFormat7',
     tree_format='bzrlib.workingtree_4.WorkingTreeFormat4',
     hidden=True,
+    deprecated=True,
     )
 register_metadir(controldir.format_registry, '1.6.1-rich-root',
     'bzrlib.repofmt.knitpack_repo.RepositoryFormatKnitPack5RichRoot',
@@ -2199,6 +2232,7 @@
     branch_format='bzrlib.branch.BzrBranchFormat7',
     tree_format='bzrlib.workingtree_4.WorkingTreeFormat4',
     hidden=True,
+    deprecated=True,
     )
 register_metadir(controldir.format_registry, '1.9',
     'bzrlib.repofmt.knitpack_repo.RepositoryFormatKnitPack6',
@@ -2208,6 +2242,7 @@
     branch_format='bzrlib.branch.BzrBranchFormat7',
     tree_format='bzrlib.workingtree_4.WorkingTreeFormat4',
     hidden=True,
+    deprecated=True,
     )
 register_metadir(controldir.format_registry, '1.9-rich-root',
     'bzrlib.repofmt.knitpack_repo.RepositoryFormatKnitPack6RichRoot',
@@ -2216,12 +2251,15 @@
     branch_format='bzrlib.branch.BzrBranchFormat7',
     tree_format='bzrlib.workingtree_4.WorkingTreeFormat4',
     hidden=True,
+    deprecated=True,
     )
 register_metadir(controldir.format_registry, '1.14',
     'bzrlib.repofmt.knitpack_repo.RepositoryFormatKnitPack6',
     help='A working-tree format that supports content filtering.',
     branch_format='bzrlib.branch.BzrBranchFormat7',
     tree_format='bzrlib.workingtree_4.WorkingTreeFormat5',
+    hidden=True,
+    deprecated=True,
     )
 register_metadir(controldir.format_registry, '1.14-rich-root',
     'bzrlib.repofmt.knitpack_repo.RepositoryFormatKnitPack6RichRoot',
@@ -2229,6 +2267,8 @@
          '(needed for bzr-svn and bzr-git).',
     branch_format='bzrlib.branch.BzrBranchFormat7',
     tree_format='bzrlib.workingtree_4.WorkingTreeFormat5',
+    hidden=True,
+    deprecated=True,
     )
 # The following un-numbered 'development' formats should always just be aliases.
 register_metadir(controldir.format_registry, 'development-subtree',
@@ -2262,12 +2302,22 @@
     alias=False,
     )
 
+register_metadir(controldir.format_registry, 'development-colo',
+    'bzrlib.repofmt.groupcompress_repo.RepositoryFormat2a',
+    help='The 2a format with experimental support for colocated branches.\n',
+    branch_format='bzrlib.branch.BzrBranchFormat7',
+    tree_format='bzrlib.workingtree_4.WorkingTreeFormat6',
+    experimental=True,
+    bzrdir_format=BzrDirMetaFormat1Colo,
+    )
+
+
 # And the development formats above will have aliased one of the following:
 
 # Finally, the current format.
 register_metadir(controldir.format_registry, '2a',
     'bzrlib.repofmt.groupcompress_repo.RepositoryFormat2a',
-    help='First format for bzr 2.0 series.\n'
+    help='Format for the bzr 2.0 series.\n'
         'Uses group-compress storage.\n'
         'Provides rich roots which are a one-way transition.\n',
         # 'storage in packs, 255-way hashed CHK inventory, bencode revision, group compress, '

=== modified file 'bzrlib/cache_utf8.py'
--- bzrlib/cache_utf8.py	2009-06-10 03:56:49 +0000
+++ bzrlib/cache_utf8.py	2011-12-19 13:23:58 +0000
@@ -19,9 +19,10 @@
 
 """Some functions to enable caching the conversion between unicode to utf8"""
 
+from __future__ import absolute_import
+
 import codecs
 
-
 _utf8_encode = codecs.utf_8_encode
 _utf8_decode = codecs.utf_8_decode
 def _utf8_decode_with_None(bytestring, _utf8_decode=_utf8_decode):

=== modified file 'bzrlib/cethread.py'
--- bzrlib/cethread.py	2011-05-19 09:32:38 +0000
+++ bzrlib/cethread.py	2011-12-18 12:46:49 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 import sys
 import threading
 

=== modified file 'bzrlib/check.py'
--- bzrlib/check.py	2011-05-18 16:42:48 +0000
+++ bzrlib/check.py	2011-12-19 13:23:58 +0000
@@ -47,16 +47,18 @@
   indicating that the revision was found/not found.
 """
 
+from __future__ import absolute_import
+
 from bzrlib import (
     errors,
     ui,
     )
 from bzrlib.branch import Branch
-from bzrlib.bzrdir import BzrDir
+from bzrlib.controldir import ControlDir
 from bzrlib.revision import NULL_REVISION
 from bzrlib.trace import note
 from bzrlib.workingtree import WorkingTree
-
+from bzrlib.i18n import gettext
 
 class Check(object):
     """Check a repository"""
@@ -103,17 +105,17 @@
         self.repository.lock_read()
         self.progress = ui.ui_factory.nested_progress_bar()
         try:
-            self.progress.update('check', 0, 4)
+            self.progress.update(gettext('check'), 0, 4)
             if self.check_repo:
-                self.progress.update('checking revisions', 0)
+                self.progress.update(gettext('checking revisions'), 0)
                 self.check_revisions()
-                self.progress.update('checking commit contents', 1)
+                self.progress.update(gettext('checking commit contents'), 1)
                 self.repository._check_inventories(self)
-                self.progress.update('checking file graphs', 2)
+                self.progress.update(gettext('checking file graphs'), 2)
                 # check_weaves is done after the revision scan so that
                 # revision index is known to be valid.
                 self.check_weaves()
-            self.progress.update('checking branches and trees', 3)
+            self.progress.update(gettext('checking branches and trees'), 3)
             if callback_refs:
                 repo = self.repository
                 # calculate all refs, and callback the objects requesting them.
@@ -200,56 +202,57 @@
             result.report_results(verbose)
 
     def _report_repo_results(self, verbose):
-        note('checked repository %s format %s',
+        note(gettext('checked repository {0} format {1}').format(
             self.repository.user_url,
-            self.repository._format)
-        note('%6d revisions', self.checked_rev_cnt)
-        note('%6d file-ids', len(self.checked_weaves))
+            self.repository._format))
+        note(gettext('%6d revisions'), self.checked_rev_cnt)
+        note(gettext('%6d file-ids'), len(self.checked_weaves))
         if verbose:
-            note('%6d unreferenced text versions',
+            note(gettext('%6d unreferenced text versions'),
                 len(self.unreferenced_versions))
         if verbose and len(self.unreferenced_versions):
                 for file_id, revision_id in self.unreferenced_versions:
-                    note('unreferenced version: {%s} in %s', revision_id,
-                        file_id)
+                    note(gettext('unreferenced version: {{{0}}} in {1}').format(revision_id,
+                        file_id))
         if self.missing_inventory_sha_cnt:
-            note('%6d revisions are missing inventory_sha1',
+            note(gettext('%6d revisions are missing inventory_sha1'),
                  self.missing_inventory_sha_cnt)
         if self.missing_revision_cnt:
-            note('%6d revisions are mentioned but not present',
+            note(gettext('%6d revisions are mentioned but not present'),
                  self.missing_revision_cnt)
         if len(self.ghosts):
-            note('%6d ghost revisions', len(self.ghosts))
+            note(gettext('%6d ghost revisions'), len(self.ghosts))
             if verbose:
                 for ghost in self.ghosts:
                     note('      %s', ghost)
         if len(self.missing_parent_links):
-            note('%6d revisions missing parents in ancestry',
+            note(gettext('%6d revisions missing parents in ancestry'),
                  len(self.missing_parent_links))
             if verbose:
                 for link, linkers in self.missing_parent_links.items():
-                    note('      %s should be in the ancestry for:', link)
+                    note(gettext('      %s should be in the ancestry for:'), link)
                     for linker in linkers:
                         note('       * %s', linker)
         if len(self.inconsistent_parents):
-            note('%6d inconsistent parents', len(self.inconsistent_parents))
+            note(gettext('%6d inconsistent parents'), len(self.inconsistent_parents))
             if verbose:
                 for info in self.inconsistent_parents:
                     revision_id, file_id, found_parents, correct_parents = info
-                    note('      * %s version %s has parents %r '
-                         'but should have %r'
-                         % (file_id, revision_id, found_parents,
+                    note(gettext('      * {0} version {1} has parents {2!r} '
+                         'but should have {3!r}').format(
+                         file_id, revision_id, found_parents,
                              correct_parents))
         if self.revs_with_bad_parents_in_index:
-            note('%6d revisions have incorrect parents in the revision index',
+            note(gettext(
+                 '%6d revisions have incorrect parents in the revision index'),
                  len(self.revs_with_bad_parents_in_index))
             if verbose:
                 for item in self.revs_with_bad_parents_in_index:
                     revision_id, index_parents, actual_parents = item
-                    note(
-                        '       %s has wrong parents in index: '
-                        '%r should be %r',
-                        revision_id, index_parents, actual_parents)
+                    note(gettext(
+                        '       {0} has wrong parents in index: '
+                        '{1!r} should be {2!r}').format(
+                        revision_id, index_parents, actual_parents))
         for item in self._report_items:
             note(item)
 
@@ -260,8 +263,8 @@
         :param rev: A revision or None to indicate a missing revision.
         """
         if rev.revision_id != rev_id:
-            self._report_items.append(
-                'Mismatched internal revid {%s} and index revid {%s}' % (
+            self._report_items.append(gettext(
+                'Mismatched internal revid {{{0}}} and index revid {{{1}}}').format(
                 rev.revision_id, rev_id))
             rev_id = rev.revision_id
         # Check this revision tree etc, and count as seen when we encounter a
@@ -290,8 +293,8 @@
         existing = self.pending_keys.get(key)
         if existing:
             if sha1 != existing[1]:
-                self._report_items.append('Multiple expected sha1s for %s. {%s}'
-                    ' expects {%s}, {%s} expects {%s}', (
+                self._report_items.append(gettext('Multiple expected sha1s for {0}. {{{1}}}'
+                    ' expects {{{2}}}, {{{3}}} expects {{{4}}}').format(
                     key, referer, sha1, existing[1], existing[0]))
         else:
             self.pending_keys[key] = (kind, sha1, referer)
@@ -348,7 +351,7 @@
     :param needed_refs: Refs we are accumulating.
     :param to_unlock: The unlock list accumulating.
     """
-    note("Checking branch at '%s'." % (branch.base,))
+    note(gettext("Checking branch at '%s'.") % (branch.base,))
     branch.lock_read()
     to_unlock.append(branch)
     branch_refs = branch._get_check_refs()
@@ -368,7 +371,7 @@
     """
     if base_tree is not None and tree.basedir == base_tree.basedir:
         return
-    note("Checking working tree at '%s'." % (tree.basedir,))
+    note(gettext("Checking working tree at '%s'.") % (tree.basedir,))
     tree.lock_read()
     to_unlock.append(tree)
     tree_refs = tree._get_check_refs()
@@ -385,7 +388,7 @@
     """
     try:
         base_tree, branch, repo, relpath = \
-                        BzrDir.open_containing_tree_branch_or_repository(path)
+                        ControlDir.open_containing_tree_branch_or_repository(path)
     except errors.NotBranchError:
         base_tree = branch = repo = None
 
@@ -421,23 +424,23 @@
                     if do_branch:
                         scan_branch(branch, needed_refs, to_unlock)
             if do_branch and not branches:
-                note("No branch found at specified location.")
+                note(gettext("No branch found at specified location."))
             if do_tree and base_tree is None and not saw_tree:
-                note("No working tree found at specified location.")
+                note(gettext("No working tree found at specified location."))
             if do_repo or do_branch or do_tree:
                 if do_repo:
-                    note("Checking repository at '%s'."
+                    note(gettext("Checking repository at '%s'.")
                          % (repo.user_url,))
                 result = repo.check(None, callback_refs=needed_refs,
                     check_repo=do_repo)
                 result.report_results(verbose)
         else:
             if do_tree:
-                note("No working tree found at specified location.")
+                note(gettext("No working tree found at specified location."))
             if do_branch:
-                note("No branch found at specified location.")
+                note(gettext("No branch found at specified location."))
             if do_repo:
-                note("No repository found at specified location.")
+                note(gettext("No repository found at specified location."))
     finally:
         for thing in to_unlock:
             thing.unlock()

=== modified file 'bzrlib/chk_map.py'
--- bzrlib/chk_map.py	2011-05-20 14:46:02 +0000
+++ bzrlib/chk_map.py	2011-12-19 13:23:58 +0000
@@ -37,6 +37,8 @@
 
 """
 
+from __future__ import absolute_import
+
 import heapq
 import threading
 
@@ -920,7 +922,7 @@
         bytes = ''.join(lines)
         if len(bytes) != self._current_size():
             raise AssertionError('Invalid _current_size')
-        _get_cache().add(self._key, bytes)
+        _get_cache()[self._key] = bytes
         return [self._key]
 
     def refs(self):
@@ -1193,7 +1195,7 @@
                     prefix, node_key_filter = keys[record.key]
                     node_and_filters.append((node, node_key_filter))
                     self._items[prefix] = node
-                    _get_cache().add(record.key, bytes)
+                    _get_cache()[record.key] = bytes
                 for info in node_and_filters:
                     yield info
 
@@ -1319,7 +1321,7 @@
             lines.append(serialised[prefix_len:])
         sha1, _, _ = store.add_lines((None,), (), lines)
         self._key = StaticTuple("sha1:" + sha1,).intern()
-        _get_cache().add(self._key, ''.join(lines))
+        _get_cache()[self._key] = ''.join(lines)
         yield self._key
 
     def _search_key(self, key):

=== modified file 'bzrlib/chk_serializer.py'
--- bzrlib/chk_serializer.py	2011-02-19 22:39:03 +0000
+++ bzrlib/chk_serializer.py	2011-12-19 13:23:58 +0000
@@ -16,11 +16,23 @@
 
 """Serializer object for CHK based inventory storage."""
 
+from __future__ import absolute_import
+
+from cStringIO import StringIO
+
+from bzrlib import lazy_import
+lazy_import.lazy_import(globals(),
+"""
+from bzrlib import (
+    xml_serializer,
+    )
+""")
 from bzrlib import (
     bencode,
     cache_utf8,
+    errors,
     revision as _mod_revision,
-    xml8,
+    serializer,
     )
 
 
@@ -129,17 +141,106 @@
         return self.read_revision_from_string(f.read())
 
 
-class CHKSerializer(xml8.Serializer_v8):
+class CHKSerializer(serializer.Serializer):
     """A CHKInventory based serializer with 'plain' behaviour."""
 
     format_num = '9'
     revision_format_num = None
     support_altered_by_hack = False
+    supported_kinds = set(['file', 'directory', 'symlink'])
 
     def __init__(self, node_size, search_key_name):
         self.maximum_size = node_size
         self.search_key_name = search_key_name
 
+    def _unpack_inventory(self, elt, revision_id=None, entry_cache=None,
+                          return_from_cache=False):
+        """Construct from XML Element"""
+        inv = xml_serializer.unpack_inventory_flat(elt, self.format_num,
+            xml_serializer.unpack_inventory_entry, entry_cache,
+            return_from_cache)
+        return inv
+
+    def read_inventory_from_string(self, xml_string, revision_id=None,
+                                   entry_cache=None, return_from_cache=False):
+        """Read xml_string into an inventory object.
+
+        :param xml_string: The xml to read.
+        :param revision_id: If not-None, the expected revision id of the
+            inventory.
+        :param entry_cache: An optional cache of InventoryEntry objects. If
+            supplied we will look up entries via (file_id, revision_id) which
+            should map to a valid InventoryEntry (File/Directory/etc) object.
+        :param return_from_cache: Return entries directly from the cache,
+            rather than copying them first. This is only safe if the caller
+            promises not to mutate the returned inventory entries, but it can
+            make some operations significantly faster.
+        """
+        try:
+            return self._unpack_inventory(
+                xml_serializer.fromstring(xml_string), revision_id,
+                entry_cache=entry_cache,
+                return_from_cache=return_from_cache)
+        except xml_serializer.ParseError, e:
+            raise errors.UnexpectedInventoryFormat(e)
+
+    def read_inventory(self, f, revision_id=None):
+        """Read an inventory from a file-like object."""
+        try:
+            try:
+                return self._unpack_inventory(self._read_element(f),
+                    revision_id=None)
+            finally:
+                f.close()
+        except xml_serializer.ParseError, e:
+            raise errors.UnexpectedInventoryFormat(e)
+
+    def write_inventory_to_lines(self, inv):
+        """Return a list of lines with the encoded inventory."""
+        return self.write_inventory(inv, None)
+
+    def write_inventory_to_string(self, inv, working=False):
+        """Just call write_inventory with a StringIO and return the value.
+
+        :param working: If True skip history data - text_sha1, text_size,
+            reference_revision, symlink_target.
+        """
+        sio = StringIO()
+        self.write_inventory(inv, sio, working)
+        return sio.getvalue()
+
+    def write_inventory(self, inv, f, working=False):
+        """Write inventory to a file.
+
+        :param inv: the inventory to write.
+        :param f: the file to write. (May be None if the lines are the desired
+            output).
+        :param working: If True skip history data - text_sha1, text_size,
+            reference_revision, symlink_target.
+        :return: The inventory as a list of lines.
+        """
+        output = []
+        append = output.append
+        if inv.revision_id is not None:
+            revid1 = ' revision_id="'
+            revid2 = xml_serializer.encode_and_escape(inv.revision_id)
+        else:
+            revid1 = ""
+            revid2 = ""
+        append('<inventory format="%s"%s%s>\n' % (
+            self.format_num, revid1, revid2))
+        append('<directory file_id="%s name="%s revision="%s />\n' % (
+            xml_serializer.encode_and_escape(inv.root.file_id),
+            xml_serializer.encode_and_escape(inv.root.name),
+            xml_serializer.encode_and_escape(inv.root.revision)))
+        xml_serializer.serialize_inventory_flat(inv,
+            append,
+            root_id=None, supported_kinds=self.supported_kinds, 
+            working=working)
+        if f is not None:
+            f.writelines(output)
+        return output
+
 
 chk_serializer_255_bigpage = CHKSerializer(65536, 'hash-255-way')
 

=== modified file 'bzrlib/chunk_writer.py'
--- bzrlib/chunk_writer.py	2011-05-19 09:32:38 +0000
+++ bzrlib/chunk_writer.py	2011-12-19 13:23:58 +0000
@@ -17,6 +17,8 @@
 
 """ChunkWriter: write compressed data out with a fixed upper bound."""
 
+from __future__ import absolute_import
+
 import zlib
 from zlib import Z_FINISH, Z_SYNC_FLUSH
 

=== modified file 'bzrlib/clean_tree.py'
--- bzrlib/clean_tree.py	2010-08-23 17:54:07 +0000
+++ bzrlib/clean_tree.py	2011-12-18 12:46:49 +0000
@@ -14,20 +14,21 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
 
 import errno
 import os
 import shutil
 
 from bzrlib import (
-    bzrdir,
+    controldir,
     errors,
     ui,
     )
 from bzrlib.osutils import isdir
 from bzrlib.trace import note
 from bzrlib.workingtree import WorkingTree
-
+from bzrlib.i18n import gettext
 
 def is_detritus(subp):
     """Return True if the supplied path is detritus, False otherwise"""
@@ -59,16 +60,14 @@
             ignored=ignored, detritus=detritus))
         deletables = _filter_out_nested_bzrdirs(deletables)
         if len(deletables) == 0:
-            note('Nothing to delete.')
+            note(gettext('Nothing to delete.'))
             return 0
         if not no_prompt:
             for path, subp in deletables:
-                # FIXME using print is very bad idea
-                # clean_tree should accept to_file argument to write the output
-                print subp
-            val = raw_input('Are you sure you wish to delete these [y/N]?')
-            if val.lower() not in ('y', 'yes'):
-                print 'Canceled'
+                ui.ui_factory.note(subp)
+            prompt = gettext('Are you sure you wish to delete these')
+            if not ui.ui_factory.get_boolean(prompt):
+                ui.ui_factory.note(gettext('Canceled'))
                 return 0
         delete_items(deletables, dry_run=dry_run)
     finally:
@@ -85,7 +84,7 @@
         # directory and therefore delete it. (worth to FIXME?)
         if isdir(path):
             try:
-                bzrdir.BzrDir.open(path)
+                controldir.ControlDir.open(path)
             except errors.NotBranchError:
                 result.append((path,subp))
             else:
@@ -105,11 +104,11 @@
         # Other errors are re-raised.
         if function is not os.remove or excinfo[1].errno != errno.EACCES:
             raise
-        ui.ui_factory.show_warning('unable to remove %s' % path)
+        ui.ui_factory.show_warning(gettext('unable to remove %s') % path)
     has_deleted = False
     for path, subp in deletables:
         if not has_deleted:
-            note("deleting paths:")
+            note(gettext("deleting paths:"))
             has_deleted = True
         if not dry_run:
             if isdir(path):
@@ -122,9 +121,10 @@
                     # We handle only permission error here
                     if e.errno != errno.EACCES:
                         raise e
-                    ui.ui_factory.show_warning(
-                        'unable to remove "%s": %s.' % (path, e.strerror))
+                    ui.ui_factory.show_warning(gettext(
+                        'unable to remove "{0}": {1}.').format(
+                                                    path, e.strerror))
         else:
             note('  ' + subp)
     if not has_deleted:
-        note("No files deleted.")
+        note(gettext("No files deleted."))

=== modified file 'bzrlib/cleanup.py'
--- bzrlib/cleanup.py	2011-05-26 08:05:45 +0000
+++ bzrlib/cleanup.py	2011-12-19 13:23:58 +0000
@@ -41,6 +41,7 @@
 details.
 """
 
+from __future__ import absolute_import
 
 from collections import deque
 import sys

=== modified file 'bzrlib/cmd_test_script.py'
--- bzrlib/cmd_test_script.py	2010-11-08 10:53:53 +0000
+++ bzrlib/cmd_test_script.py	2011-12-19 13:23:58 +0000
@@ -20,6 +20,8 @@
 This module should be importable even if testtools aren't available.
 """
 
+from __future__ import absolute_import
+
 import os
 
 from bzrlib import (

=== modified file 'bzrlib/cmd_version_info.py'
--- bzrlib/cmd_version_info.py	2011-04-07 10:36:24 +0000
+++ bzrlib/cmd_version_info.py	2011-12-30 12:52:54 +0000
@@ -16,6 +16,8 @@
 
 """Commands for generating snapshot information about a bzr tree."""
 
+from __future__ import absolute_import
+
 from bzrlib.lazy_import import lazy_import
 
 lazy_import(globals(), """
@@ -25,6 +27,7 @@
     version_info_formats,
     workingtree,
     )
+from bzrlib.i18n import gettext
 """)
 
 from bzrlib.commands import Command
@@ -41,9 +44,9 @@
         return version_info_formats.get_builder(format)
     except KeyError:
         formats = version_info_formats.get_builder_formats()
-        raise errors.BzrCommandError('No known version info format %s.'
-                                     ' Supported types are: %s'
-                                     % (format, formats))
+        raise errors.BzrCommandError(gettext('No known version info format {0}.'
+                                     ' Supported types are: {1}').format(
+                                     format, formats))
 
 
 class cmd_version_info(Command):
@@ -82,6 +85,7 @@
                      Option('include-file-revisions',
                             help='Include the last revision for each file.'),
                      Option('template', type=str, help='Template for the output.'),
+                     'revision',
                      ]
     takes_args = ['location?']
 
@@ -89,7 +93,13 @@
 
     def run(self, location=None, format=None,
             all=False, check_clean=False, include_history=False,
-            include_file_revisions=False, template=None):
+            include_file_revisions=False, template=None,
+            revision=None):
+
+        if revision and len(revision) > 1:
+            raise errors.BzrCommandError(
+                gettext('bzr version-info --revision takes exactly'
+                        ' one revision specifier'))
 
         if location is None:
             location = '.'
@@ -97,22 +107,32 @@
         if format is None:
             format = version_info_formats.format_registry.get()
 
-        wt = None
         try:
             wt = workingtree.WorkingTree.open_containing(location)[0]
         except errors.NoWorkingTree:
             b = branch.Branch.open(location)
+            wt = None
         else:
             b = wt.branch
 
-        if all or template:
+        if all:
             include_history = True
             check_clean = True
-            include_file_revisions=True
+            include_file_revisions = True
+        if template:
+            include_history = True
+            include_file_revisions = True
+            if '{clean}' in template:
+                check_clean = True
+
+        if revision is not None:
+            revision_id = revision[0].as_revision_id(b)
+        else:
+            revision_id = None
 
         builder = format(b, working_tree=wt,
                 check_for_clean=check_clean,
                 include_revision_history=include_history,
                 include_file_revisions=include_file_revisions,
-                template=template)
+                template=template, revision_id=revision_id)
         builder.generate(self.outf)

=== modified file 'bzrlib/cmdline.py'
--- bzrlib/cmdline.py	2011-02-07 01:39:42 +0000
+++ bzrlib/cmdline.py	2011-12-19 13:23:58 +0000
@@ -20,6 +20,8 @@
 configuring_bazaar.txt.
 """
 
+from __future__ import absolute_import
+
 import re
 
 

=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2011-07-15 14:27:20 +0000
+++ bzrlib/commands.py	2011-12-18 12:46:49 +0000
@@ -14,6 +14,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
 
 # TODO: Define arguments by objects, rather than just using names.
 # Those objects can specify the expected type of the argument, which
@@ -50,11 +51,6 @@
 from bzrlib.option import Option
 from bzrlib.plugin import disable_plugins, load_plugins
 from bzrlib import registry
-from bzrlib.symbol_versioning import (
-    deprecated_function,
-    deprecated_in,
-    deprecated_method,
-    )
 
 
 class CommandInfo(object):
@@ -229,7 +225,7 @@
     try:
         return _get_cmd_object(cmd_name, plugins_override)
     except KeyError:
-        raise errors.BzrCommandError('unknown command "%s"' % cmd_name)
+        raise errors.BzrCommandError(gettext('unknown command "%s"') % cmd_name)
 
 
 def _get_cmd_object(cmd_name, plugins_override=True, check_missing=True):
@@ -465,7 +461,7 @@
             usage help (e.g. Purpose, Usage, Options) with a
             message explaining how to obtain full help.
         """
-        if self.l10n and not i18n.installed():
+        if self.l10n:
             i18n.install()  # Install i18n only for get_help_text for now.
         doc = self.help()
         if doc:
@@ -554,7 +550,8 @@
                         see_also_links.append(item)
                     else:
                         # Use a Sphinx link for this entry
-                        link_text = gettext(":doc:`%s <%s-help>`") % (item, item)
+                        link_text = gettext(":doc:`{0} <{1}-help>`").format(
+                                                                    item, item)
                         see_also_links.append(link_text)
                 see_also = see_also_links
             result += gettext(':See also: %s') % ', '.join(see_also) + '\n'
@@ -662,7 +659,6 @@
             opts['quiet'] = trace.is_quiet()
         elif opts.has_key('quiet'):
             del opts['quiet']
-
         # mix arguments and options into one dictionary
         cmdargs = _match_argform(self.name(), self.takes_args, args)
         cmdopts = {}
@@ -815,7 +811,8 @@
     try:
         options, args = parser.parse_args(args)
     except UnicodeEncodeError,e:
-        raise errors.BzrCommandError('Only ASCII permitted in option names')
+        raise errors.BzrCommandError(
+            gettext('Only ASCII permitted in option names'))
 
     opts = dict([(k, v) for k, v in options.__dict__.iteritems() if
                  v is not option.OptionParser.DEFAULT_VALUE])
@@ -839,29 +836,33 @@
                 argdict[argname + '_list'] = None
         elif ap[-1] == '+':
             if not args:
-                raise errors.BzrCommandError("command %r needs one or more %s"
-                                             % (cmd, argname.upper()))
+                raise errors.BzrCommandError(gettext(
+                      "command {0!r} needs one or more {1}").format(
+                      cmd, argname.upper()))
             else:
                 argdict[argname + '_list'] = args[:]
                 args = []
         elif ap[-1] == '$': # all but one
             if len(args) < 2:
-                raise errors.BzrCommandError("command %r needs one or more %s"
-                                             % (cmd, argname.upper()))
+                raise errors.BzrCommandError(
+                      gettext("command {0!r} needs one or more {1}").format(
+                                             cmd, argname.upper()))
             argdict[argname + '_list'] = args[:-1]
             args[:-1] = []
         else:
             # just a plain arg
             argname = ap
             if not args:
-                raise errors.BzrCommandError("command %r requires argument %s"
-                               % (cmd, argname.upper()))
+                raise errors.BzrCommandError(
+                     gettext("command {0!r} requires argument {1}").format(
+                               cmd, argname.upper()))
             else:
                 argdict[argname] = args.pop(0)
 
     if args:
-        raise errors.BzrCommandError("extra argument to command %s: %s"
-                                     % (cmd, args[0]))
+        raise errors.BzrCommandError( gettext(
+                              "extra argument to command {0}: {1}").format(
+                                       cmd, args[0]) )
 
     return argdict
 
@@ -957,7 +958,7 @@
         stats.pprint()
     else:
         stats.save(filename)
-        trace.note('Profile data written to "%s".', filename)
+        trace.note(gettext('Profile data written to "%s".'), filename)
     return ret
 
 
@@ -1035,6 +1036,7 @@
 
     argv_copy = []
     i = 0
+    override_config = []
     while i < len(argv):
         a = argv[i]
         if a == '--profile':
@@ -1063,10 +1065,14 @@
             pass # already handled in startup script Bug #588277
         elif a.startswith('-D'):
             debug.debug_flags.add(a[2:])
+        elif a.startswith('-O'):
+            override_config.append(a[2:])
         else:
             argv_copy.append(a)
         i += 1
 
+    bzrlib.global_state.cmdline_overrides._from_cmdline(override_config)
+
     debug.set_debug_flags_from_config()
 
     if not opt_no_plugins:
@@ -1124,6 +1130,8 @@
         if 'memory' in debug.debug_flags:
             trace.debug_memory('Process status after command:', short=False)
         option._verbosity_level = saved_verbosity_level
+        # Reset the overrides 
+        bzrlib.global_state.cmdline_overrides._reset()
 
 
 def display_command(func):
@@ -1158,8 +1166,9 @@
         "bzr plugin commands")
     Command.hooks.install_named_hook("get_command", _get_external_command,
         "bzr external command lookup")
-    Command.hooks.install_named_hook("get_missing_command", _try_plugin_provider,
-        "bzr plugin-provider-db check")
+    Command.hooks.install_named_hook("get_missing_command",
+                                     _try_plugin_provider,
+                                     "bzr plugin-provider-db check")
 
 
 

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2011-06-28 17:25:26 +0000
+++ bzrlib/commit.py	2011-12-18 12:46:49 +0000
@@ -14,6 +14,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
 
 # The newly committed revision is going to have a shape corresponding
 # to that of the working tree.  Files that are not in the
@@ -72,7 +73,7 @@
 from bzrlib.inventory import Inventory, InventoryEntry, make_entry
 from bzrlib import symbol_versioning
 from bzrlib.urlutils import unescape_for_display
-
+from bzrlib.i18n import gettext
 
 class NullCommitReporter(object):
     """I report on progress of a commit."""
@@ -113,7 +114,7 @@
         note(format, *args)
 
     def snapshot_change(self, change, path):
-        if path == '' and change in ('added', 'modified'):
+        if path == '' and change in (gettext('added'), gettext('modified')):
             return
         self._note("%s %s", change, path)
 
@@ -127,10 +128,10 @@
                                    "to started.", DeprecationWarning,
                                    stacklevel=2)
             location = ''
-        self._note('Committing%s', location)
+        self._note(gettext('Committing%s'), location)
 
     def completed(self, revno, rev_id):
-        self._note('Committed revision %d.', revno)
+        self._note(gettext('Committed revision %d.'), revno)
         # self._note goes to the console too; so while we want to log the
         # rev_id, we can't trivially only log it. (See bug 526425). Long
         # term we should rearrange the reporting structure, but for now
@@ -139,10 +140,10 @@
         mutter('Committed revid %s as revno %d.', rev_id, revno)
 
     def deleted(self, path):
-        self._note('deleted %s', path)
+        self._note(gettext('deleted %s'), path)
 
     def missing(self, path):
-        self._note('missing %s', path)
+        self._note(gettext('missing %s'), path)
 
     def renamed(self, change, old_path, new_path):
         self._note('%s %s => %s', change, old_path, new_path)
@@ -165,13 +166,13 @@
     """
     def __init__(self,
                  reporter=None,
-                 config=None):
+                 config_stack=None):
         """Create a Commit object.
 
         :param reporter: the default reporter to use or None to decide later
         """
         self.reporter = reporter
-        self.config = config
+        self.config_stack = config_stack
 
     @staticmethod
     def update_revprops(revprops, branch, authors=None, author=None,
@@ -268,7 +269,7 @@
         operation = OperationWithCleanups(self._commit)
         self.revprops = revprops or {}
         # XXX: Can be set on __init__ or passed in - this is a bit ugly.
-        self.config = config or self.config
+        self.config_stack = config or self.config_stack
         return operation.run(
                message=message,
                timestamp=timestamp,
@@ -363,15 +364,15 @@
         self._check_bound_branch(operation, possible_master_transports)
 
         # Check that the working tree is up to date
-        old_revno, new_revno = self._check_out_of_date_tree()
+        old_revno, old_revid, new_revno = self._check_out_of_date_tree()
 
         # Complete configuration setup
         if reporter is not None:
             self.reporter = reporter
         elif self.reporter is None:
             self.reporter = self._select_reporter()
-        if self.config is None:
-            self.config = self.branch.get_config()
+        if self.config_stack is None:
+            self.config_stack = self.branch.get_config_stack()
 
         self._set_specific_file_ids()
 
@@ -405,12 +406,18 @@
         self._set_progress_stage("Collecting changes", counter=True)
         self._lossy = lossy
         self.builder = self.branch.get_commit_builder(self.parents,
-            self.config, timestamp, timezone, committer, self.revprops,
+            self.config_stack, timestamp, timezone, committer, self.revprops,
             rev_id, lossy=lossy)
         if not self.builder.supports_record_entry_contents and self.exclude:
             self.builder.abort()
             raise errors.ExcludesUnsupported(self.branch.repository)
 
+        if self.builder.updates_branch and self.bound_branch:
+            self.builder.abort()
+            raise AssertionError(
+                "bound branches not supported for commit builders "
+                "that update the branch")
+
         try:
             self.builder.will_record_deletes()
             # find the location being committed to
@@ -445,30 +452,7 @@
             self.builder.abort()
             raise
 
-        self._process_pre_hooks(old_revno, new_revno)
-
-        # Upload revision data to the master.
-        # this will propagate merged revisions too if needed.
-        if self.bound_branch:
-            self._set_progress_stage("Uploading data to master branch")
-            # 'commit' to the master first so a timeout here causes the
-            # local branch to be out of date
-            (new_revno, self.rev_id) = self.master_branch.import_last_revision_info_and_tags(
-                self.branch, new_revno, self.rev_id, lossy=lossy)
-            if lossy:
-                self.branch.fetch(self.master_branch, self.rev_id)
-
-        # and now do the commit locally.
-        self.branch.set_last_revision_info(new_revno, self.rev_id)
-
-        # Merge local tags to remote
-        if self.bound_branch:
-            self._set_progress_stage("Merging tags to master branch")
-            tag_conflicts = self.branch.tags.merge_to(self.master_branch.tags)
-            if tag_conflicts:
-                warning_lines = ['    ' + name for name, _, _ in tag_conflicts]
-                note("Conflicting tags in bound branch:\n" +
-                    "\n".join(warning_lines))
+        self._update_branches(old_revno, old_revid, new_revno)
 
         # Make the working tree be up to date with the branch. This
         # includes automatic changes scheduled to be made to the tree, such
@@ -481,6 +465,52 @@
         self._process_post_hooks(old_revno, new_revno)
         return self.rev_id
 
+    def _update_branches(self, old_revno, old_revid, new_revno):
+        """Update the master and local branch to the new revision.
+
+        This will try to make sure that the master branch is updated
+        before the local branch.
+
+        :param old_revno: Revision number of master branch before the
+            commit
+        :param old_revid: Tip of master branch before the commit
+        :param new_revno: Revision number of the new commit
+        """
+        if not self.builder.updates_branch:
+            self._process_pre_hooks(old_revno, new_revno)
+
+            # Upload revision data to the master.
+            # this will propagate merged revisions too if needed.
+            if self.bound_branch:
+                self._set_progress_stage("Uploading data to master branch")
+                # 'commit' to the master first so a timeout here causes the
+                # local branch to be out of date
+                (new_revno, self.rev_id) = self.master_branch.import_last_revision_info_and_tags(
+                    self.branch, new_revno, self.rev_id, lossy=self._lossy)
+                if self._lossy:
+                    self.branch.fetch(self.master_branch, self.rev_id)
+
+            # and now do the commit locally.
+            self.branch.set_last_revision_info(new_revno, self.rev_id)
+        else:
+            try:
+                self._process_pre_hooks(old_revno, new_revno)
+            except:
+                # The commit builder will already have updated the branch,
+                # revert it.
+                self.branch.set_last_revision_info(old_revno, old_revid)
+                raise
+
+        # Merge local tags to remote
+        if self.bound_branch:
+            self._set_progress_stage("Merging tags to master branch")
+            tag_updates, tag_conflicts = self.branch.tags.merge_to(
+                self.master_branch.tags)
+            if tag_conflicts:
+                warning_lines = ['    ' + name for name, _, _ in tag_conflicts]
+                note( gettext("Conflicting tags in bound branch:\n{0}".format(
+                    "\n".join(warning_lines))) )
+
     def _select_reporter(self):
         """Select the CommitReporter to use."""
         if is_quiet():
@@ -543,7 +573,8 @@
     def _check_out_of_date_tree(self):
         """Check that the working tree is up to date.
 
-        :return: old_revision_number,new_revision_number tuple
+        :return: old_revision_number, old_revision_id, new_revision_number
+            tuple
         """
         try:
             first_tree_parent = self.work_tree.get_parent_ids()[0]
@@ -562,7 +593,7 @@
         else:
             # ghost parents never appear in revision history.
             new_revno = 1
-        return old_revno,new_revno
+        return old_revno, master_last, new_revno
 
     def _process_pre_hooks(self, old_revno, new_revno):
         """Process any registered pre commit hooks."""
@@ -574,9 +605,10 @@
         # Process the post commit hooks, if any
         self._set_progress_stage("Running post_commit hooks")
         # old style commit hooks - should be deprecated ? (obsoleted in
-        # 0.15)
-        if self.config.post_commit() is not None:
-            hooks = self.config.post_commit().split(' ')
+        # 0.15^H^H^H^H 2.5.0)
+        post_commit = self.config_stack.get('post_commit')
+        if post_commit is not None:
+            hooks = post_commit.split(' ')
             # this would be nicer with twisted.python.reflect.namedAny
             for hook in hooks:
                 result = eval(hook + '(branch, rev_id)',
@@ -694,6 +726,8 @@
                 # Reset the new path (None) and new versioned flag (False)
                 change = (change[0], (change[1][0], None), change[2],
                     (change[3][0], False)) + change[4:]
+                new_path = change[1][1]
+                versioned = False
             elif kind == 'tree-reference':
                 if self.recursive == 'down':
                     self._commit_nested_tree(change[0], change[1][1])
@@ -703,15 +737,15 @@
                     if new_path is None:
                         reporter.deleted(old_path)
                     elif old_path is None:
-                        reporter.snapshot_change('added', new_path)
+                        reporter.snapshot_change(gettext('added'), new_path)
                     elif old_path != new_path:
-                        reporter.renamed('renamed', old_path, new_path)
+                        reporter.renamed(gettext('renamed'), old_path, new_path)
                     else:
                         if (new_path or 
                             self.work_tree.branch.repository._format.rich_root_data):
                             # Don't report on changes to '' in non rich root
                             # repositories.
-                            reporter.snapshot_change('modified', new_path)
+                            reporter.snapshot_change(gettext('modified'), new_path)
             self._next_progress_entry()
         # Unversion IDs that were found to be deleted
         self.deleted_ids = deleted_ids
@@ -940,7 +974,7 @@
             self.reporter.renamed(change, old_path, path)
             self._next_progress_entry()
         else:
-            if change == 'unchanged':
+            if change == gettext('unchanged'):
                 return
             self.reporter.snapshot_change(change, path)
             self._next_progress_entry()
@@ -962,10 +996,10 @@
 
     def _emit_progress(self):
         if self.pb_entries_count is not None:
-            text = "%s [%d] - Stage" % (self.pb_stage_name,
+            text = gettext("{0} [{1}] - Stage").format(self.pb_stage_name,
                 self.pb_entries_count)
         else:
-            text = "%s - Stage" % (self.pb_stage_name, )
+            text = gettext("%s - Stage") % (self.pb_stage_name, )
         self.pb.update(text, self.pb_stage_count, self.pb_stage_total)
 
     def _set_specific_file_ids(self):

=== modified file 'bzrlib/commit_signature_commands.py'
--- bzrlib/commit_signature_commands.py	2011-08-04 10:28:02 +0000
+++ bzrlib/commit_signature_commands.py	2011-12-19 13:23:58 +0000
@@ -17,10 +17,12 @@
 """Command which looks for unsigned commits by the current user, and signs them.
 """
 
+from __future__ import absolute_import
+
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 from bzrlib import (
-    bzrdir as _mod_bzrdir,
+    controldir,
     errors,
     gpg,
     revision as _mod_revision,
@@ -51,17 +53,17 @@
 
     def run(self, location=None, committer=None, dry_run=False):
         if location is None:
-            bzrdir = _mod_bzrdir.BzrDir.open_containing('.')[0]
+            bzrdir = controldir.ControlDir.open_containing('.')[0]
         else:
             # Passed in locations should be exact
-            bzrdir = _mod_bzrdir.BzrDir.open(location)
+            bzrdir = controldir.ControlDir.open(location)
         branch = bzrdir.open_branch()
         repo = branch.repository
         branch_config = branch.get_config()
 
         if committer is None:
             committer = branch_config.username()
-        gpg_strategy = gpg.GPGStrategy(branch_config)
+        gpg_strategy = gpg.GPGStrategy(branch.get_config_stack())
 
         count = 0
         repo.lock_write()
@@ -116,10 +118,10 @@
 
     def run(self, acceptable_keys=None, revision=None, verbose=None,
                                                             location=u'.'):
-        bzrdir = _mod_bzrdir.BzrDir.open_containing(location)[0]
+        bzrdir = controldir.ControlDir.open_containing(location)[0]
         branch = bzrdir.open_branch()
         repo = branch.repository
-        branch_config = branch.get_config()
+        branch_config = branch.get_config_stack()
         gpg_strategy = gpg.GPGStrategy(branch_config)
 
         gpg_strategy.set_acceptable_keys(acceptable_keys)
@@ -141,8 +143,8 @@
                 if to_revid is None:
                     to_revno = branch.revno()
                 if from_revno is None or to_revno is None:
-                    raise errors.BzrCommandError('Cannot verify a range of '\
-                                               'non-revision-history revisions')
+                    raise errors.BzrCommandError(gettext(
+                    'Cannot verify a range of non-revision-history revisions'))
                 for revno in range(from_revno, to_revno + 1):
                     revisions.append(branch.get_rev_id(revno))
         else:

=== modified file 'bzrlib/config.py'
--- bzrlib/config.py	2011-08-29 12:01:49 +0000
+++ bzrlib/config.py	2012-01-03 12:56:06 +0000
@@ -72,12 +72,12 @@
 up=pull
 """
 
+from __future__ import absolute_import
+
 import os
-import string
 import sys
-import re
-
-
+
+import bzrlib
 from bzrlib.decorators import needs_write_lock
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
@@ -87,10 +87,11 @@
 
 from bzrlib import (
     atomicfile,
-    bzrdir,
+    controldir,
     debug,
     errors,
     lazy_regex,
+    library_state,
     lockdir,
     mail_client,
     mergetools,
@@ -102,11 +103,13 @@
     urlutils,
     win32utils,
     )
+from bzrlib.i18n import gettext
 from bzrlib.util.configobj import configobj
 """)
 from bzrlib import (
     commands,
     hooks,
+    lazy_regex,
     registry,
     )
 from bzrlib.symbol_versioning import (
@@ -149,6 +152,30 @@
 STORE_GLOBAL = 4
 
 
+def signature_policy_from_unicode(signature_string):
+    """Convert a string to a signing policy."""
+    if signature_string.lower() == 'check-available':
+        return CHECK_IF_POSSIBLE
+    if signature_string.lower() == 'ignore':
+        return CHECK_NEVER
+    if signature_string.lower() == 'require':
+        return CHECK_ALWAYS
+    raise ValueError("Invalid signatures policy '%s'"
+                     % signature_string)
+
+
+def signing_policy_from_unicode(signature_string):
+    """Convert a string to a signing policy."""
+    if signature_string.lower() == 'when-required':
+        return SIGN_WHEN_REQUIRED
+    if signature_string.lower() == 'never':
+        return SIGN_NEVER
+    if signature_string.lower() == 'always':
+        return SIGN_ALWAYS
+    raise ValueError("Invalid signing policy '%s'"
+                     % signature_string)
+
+
 class ConfigObj(configobj.ConfigObj):
 
     def __init__(self, infile=None, **kwargs):
@@ -414,14 +441,15 @@
             # add) the final ','
             l = [l]
         return l
-        
-    def get_user_option_as_int_from_SI(self,  option_name,  default=None):
+
+    @deprecated_method(deprecated_in((2, 5, 0)))
+    def get_user_option_as_int_from_SI(self, option_name, default=None):
         """Get a generic option from a human readable size in SI units, e.g 10MB
-        
+
         Accepted suffixes are K,M,G. It is case-insensitive and may be followed
         by a trailing b (i.e. Kb, MB). This is intended to be practical and not
         pedantic.
-        
+
         :return Integer, expanded to its base-10 value if a proper SI unit is 
             found. If the option doesn't exist, or isn't a value in 
             SI units, return default (which defaults to None)
@@ -445,15 +473,15 @@
                         elif m.group(2).lower() == 'g':
                             val *= 10**9
                 else:
-                    ui.ui_factory.show_warning('Invalid config value for "%s" '
-                                               ' value %r is not an SI unit.'
-                                                % (option_name, val))
+                    ui.ui_factory.show_warning(gettext('Invalid config value for "{0}" '
+                                               ' value {1!r} is not an SI unit.').format(
+                                                option_name, val))
                     val = default
             except TypeError:
                 val = default
         return val
-        
 
+    @deprecated_method(deprecated_in((2, 5, 0)))
     def gpg_signing_command(self):
         """What program should be used to sign signatures?"""
         result = self._gpg_signing_command()
@@ -465,6 +493,7 @@
         """See gpg_signing_command()."""
         return None
 
+    @deprecated_method(deprecated_in((2, 5, 0)))
     def log_format(self):
         """What log format should be used"""
         result = self._log_format()
@@ -489,6 +518,7 @@
         """See validate_signatures_in_log()."""
         return None
 
+    @deprecated_method(deprecated_in((2, 5, 0)))
     def acceptable_keys(self):
         """Comma separated list of key patterns acceptable to 
         verify-signatures command"""
@@ -499,6 +529,7 @@
         """See acceptable_keys()."""
         return None
 
+    @deprecated_method(deprecated_in((2, 5, 0)))
     def post_commit(self):
         """An ordered list of python functions to call.
 
@@ -530,15 +561,7 @@
         v = self._get_user_id()
         if v:
             return v
-        v = os.environ.get('EMAIL')
-        if v:
-            return v.decode(osutils.get_user_encoding())
-        name, email = _auto_user_id()
-        if name and email:
-            return '%s <%s>' % (name, email)
-        elif email:
-            return email
-        raise errors.NoWhoami()
+        return default_email()
 
     def ensure_username(self):
         """Raise errors.NoWhoami if username is not set.
@@ -547,6 +570,7 @@
         """
         self.username()
 
+    @deprecated_method(deprecated_in((2, 5, 0)))
     def signature_checking(self):
         """What is the current policy for signature checking?."""
         policy = self._get_signature_checking()
@@ -554,6 +578,7 @@
             return policy
         return CHECK_IF_POSSIBLE
 
+    @deprecated_method(deprecated_in((2, 5, 0)))
     def signing_policy(self):
         """What is the current policy for signature checking?."""
         policy = self._get_signing_policy()
@@ -561,6 +586,7 @@
             return policy
         return SIGN_WHEN_REQUIRED
 
+    @deprecated_method(deprecated_in((2, 5, 0)))
     def signature_needed(self):
         """Is a signature needed when committing ?."""
         policy = self._get_signing_policy()
@@ -575,6 +601,7 @@
             return True
         return False
 
+    @deprecated_method(deprecated_in((2, 5, 0)))
     def gpg_signing_key(self):
         """GPG user-id to sign commits"""
         key = self.get_user_option('gpg_signing_key')
@@ -622,7 +649,7 @@
         for (oname, value, section, conf_id, parser) in self._get_options():
             if oname.startswith('bzr.mergetool.'):
                 tool_name = oname[len('bzr.mergetool.'):]
-                tools[tool_name] = value
+                tools[tool_name] = self.get_user_option(oname)
         trace.mutter('loaded merge tools: %r' % tools)
         return tools
 
@@ -865,13 +892,13 @@
         """See Config._get_signature_checking."""
         policy = self._get_user_option('check_signatures')
         if policy:
-            return self._string_to_signature_policy(policy)
+            return signature_policy_from_unicode(policy)
 
     def _get_signing_policy(self):
         """See Config._get_signing_policy"""
         policy = self._get_user_option('create_signatures')
         if policy:
-            return self._string_to_signing_policy(policy)
+            return signing_policy_from_unicode(policy)
 
     def _get_user_id(self):
         """Get the user id from the 'email' key in the current section."""
@@ -922,28 +949,6 @@
         """See Config.post_commit."""
         return self._get_user_option('post_commit')
 
-    def _string_to_signature_policy(self, signature_string):
-        """Convert a string to a signing policy."""
-        if signature_string.lower() == 'check-available':
-            return CHECK_IF_POSSIBLE
-        if signature_string.lower() == 'ignore':
-            return CHECK_NEVER
-        if signature_string.lower() == 'require':
-            return CHECK_ALWAYS
-        raise errors.BzrError("Invalid signatures policy '%s'"
-                              % signature_string)
-
-    def _string_to_signing_policy(self, signature_string):
-        """Convert a string to a signing policy."""
-        if signature_string.lower() == 'when-required':
-            return SIGN_WHEN_REQUIRED
-        if signature_string.lower() == 'never':
-            return SIGN_NEVER
-        if signature_string.lower() == 'always':
-            return SIGN_ALWAYS
-        raise errors.BzrError("Invalid signing policy '%s'"
-                              % signature_string)
-
     def _get_alias(self, value):
         try:
             return self._get_parser().get_value("ALIASES",
@@ -1393,13 +1398,6 @@
         e.g. "John Hacker <jhacker@example.com>"
         This is looked up in the email controlfile for the branch.
         """
-        try:
-            return (self.branch._transport.get_bytes("email")
-                    .decode(osutils.get_user_encoding())
-                    .rstrip("\r\n"))
-        except errors.NoSuchFile, e:
-            pass
-
         return self._get_best_value('_get_user_id')
 
     def _get_change_editor(self):
@@ -1639,6 +1637,21 @@
         f.close()
 
 
+def default_email():
+    v = os.environ.get('BZR_EMAIL')
+    if v:
+        return v.decode(osutils.get_user_encoding())
+    v = os.environ.get('EMAIL')
+    if v:
+        return v.decode(osutils.get_user_encoding())
+    name, email = _auto_user_id()
+    if name and email:
+        return u'%s <%s>' % (name, email)
+    elif email:
+        return email
+    raise errors.NoWhoami()
+
+
 def _auto_user_id():
     """Calculate automatic user identification.
 
@@ -1833,7 +1846,7 @@
         :param user: login (optional)
 
         :param path: the absolute path on the server (optional)
-        
+
         :param realm: the http authentication realm (optional)
 
         :return: A dict containing the matching credentials or None.
@@ -2278,6 +2291,11 @@
             return f
         except errors.NoSuchFile:
             return StringIO()
+        except errors.PermissionDenied, e:
+            trace.warning("Permission denied while trying to open "
+                "configuration file %s.", urlutils.unescape_for_display(
+                urlutils.join(self._transport.base, self._filename), "utf-8"))
+            return StringIO()
 
     def _external_url(self):
         return urlutils.join(self._transport.external_url(), self._filename)
@@ -2314,17 +2332,22 @@
     encoutered, in which config files it can be stored.
     """
 
-    def __init__(self, name, default=None, default_from_env=None,
-                 help=None,
-                 from_unicode=None, invalid=None):
+    def __init__(self, name, override_from_env=None,
+                 default=None, default_from_env=None,
+                 help=None, from_unicode=None, invalid=None, unquote=True):
         """Build an option definition.
 
         :param name: the name used to refer to the option.
 
+        :param override_from_env: A list of environment variables which can
+           provide override any configuration setting.
+
         :param default: the default value to use when none exist in the config
             stores. This is either a string that ``from_unicode`` will convert
-            into the proper type or a python object that can be stringified (so
-            only the empty list is supported for example).
+            into the proper type, a callable returning a unicode string so that
+            ``from_unicode`` can be used on the return value, or a python
+            object that can be stringified (so only the empty list is supported
+            for example).
 
         :param default_from_env: A list of environment variables which can
            provide a default value. 'default' will be used only if none of the
@@ -2342,10 +2365,18 @@
             TypeError. Accepted values are: None (ignore invalid values),
             'warning' (emit a warning), 'error' (emit an error message and
             terminates).
+
+        :param unquote: should the unicode value be unquoted before conversion.
+           This should be used only when the store providing the values cannot
+           safely unquote them (see http://pad.lv/906897). It is provided so
+           daughter classes can handle the quoting themselves.
         """
+        if override_from_env is None:
+            override_from_env = []
         if default_from_env is None:
             default_from_env = []
         self.name = name
+        self.override_from_env = override_from_env
         # Convert the default value to a unicode string so all values are
         # strings internally before conversion (via from_unicode) is attempted.
         if default is None:
@@ -2356,9 +2387,11 @@
                 raise AssertionError(
                     'Only empty lists are supported as default values')
             self.default = u','
-        elif isinstance(default, (str, unicode, bool, int)):
+        elif isinstance(default, (str, unicode, bool, int, float)):
             # Rely on python to convert strings, booleans and integers
             self.default = u'%s' % (default,)
+        elif callable(default):
+            self.default = default
         else:
             # other python objects are not expected
             raise AssertionError('%r is not supported as a default value'
@@ -2366,11 +2399,14 @@
         self.default_from_env = default_from_env
         self.help = help
         self.from_unicode = from_unicode
+        self.unquote = unquote
         if invalid and invalid not in ('warning', 'error'):
             raise AssertionError("%s not supported for 'invalid'" % (invalid,))
         self.invalid = invalid
 
-    def convert_from_unicode(self, unicode_value):
+    def convert_from_unicode(self, store, unicode_value):
+        if self.unquote and store is not None and unicode_value is not None:
+            unicode_value = store.unquote(unicode_value)
         if self.from_unicode is None or unicode_value is None:
             # Don't convert or nothing to convert
             return unicode_value
@@ -2388,18 +2424,35 @@
                 raise errors.ConfigOptionValueError(self.name, unicode_value)
         return converted
 
+    def get_override(self):
+        value = None
+        for var in self.override_from_env:
+            try:
+                # If the env variable is defined, its value takes precedence
+                value = os.environ[var].decode(osutils.get_user_encoding())
+                break
+            except KeyError:
+                continue
+        return value
+
     def get_default(self):
         value = None
         for var in self.default_from_env:
             try:
                 # If the env variable is defined, its value is the default one
-                value = os.environ[var]
+                value = os.environ[var].decode(osutils.get_user_encoding())
                 break
             except KeyError:
                 continue
         if value is None:
             # Otherwise, fallback to the value defined at registration
-            value = self.default
+            if callable(self.default):
+                value = self.default()
+                if not isinstance(value, unicode):
+                    raise AssertionError(
+                    'Callable default values should be unicode')
+            else:
+                value = self.default
         return value
 
     def get_help_text(self, additional_see_also=None, plain=True):
@@ -2421,20 +2474,79 @@
     return int(unicode_str)
 
 
-def list_from_store(unicode_str):
-    # ConfigObj return '' instead of u''. Use 'str' below to catch all cases.
-    if isinstance(unicode_str, (str, unicode)):
-        if unicode_str:
-            # A single value, most probably the user forgot (or didn't care to
-            # add) the final ','
-            l = [unicode_str]
+_unit_suffixes = dict(K=10**3, M=10**6, G=10**9)
+
+def int_SI_from_store(unicode_str):
+    """Convert a human readable size in SI units, e.g 10MB into an integer.
+
+    Accepted suffixes are K,M,G. It is case-insensitive and may be followed
+    by a trailing b (i.e. Kb, MB). This is intended to be practical and not
+    pedantic.
+
+    :return Integer, expanded to its base-10 value if a proper SI unit is 
+        found, None otherwise.
+    """
+    regexp = "^(\d+)(([" + ''.join(_unit_suffixes) + "])b?)?$"
+    p = re.compile(regexp, re.IGNORECASE)
+    m = p.match(unicode_str)
+    val = None
+    if m is not None:
+        val, _, unit = m.groups()
+        val = int(val)
+        if unit:
+            try:
+                coeff = _unit_suffixes[unit.upper()]
+            except KeyError:
+                raise ValueError(gettext('{0} is not an SI unit.').format(unit))
+            val *= coeff
+    return val
+
+
+def float_from_store(unicode_str):
+    return float(unicode_str)
+
+
+# Use a an empty dict to initialize an empty configobj avoiding all
+# parsing and encoding checks
+_list_converter_config = configobj.ConfigObj(
+    {}, encoding='utf-8', list_values=True, interpolation=False)
+
+
+class ListOption(Option):
+
+    def __init__(self, name, default=None, default_from_env=None,
+                 help=None, invalid=None):
+        """A list Option definition.
+
+        This overrides the base class so the conversion from a unicode string
+        can take quoting into account.
+        """
+        super(ListOption, self).__init__(
+            name, default=default, default_from_env=default_from_env,
+            from_unicode=self.from_unicode, help=help,
+            invalid=invalid, unquote=False)
+
+    def from_unicode(self, unicode_str):
+        if not isinstance(unicode_str, basestring):
+            raise TypeError
+        # Now inject our string directly as unicode. All callers got their
+        # value from configobj, so values that need to be quoted are already
+        # properly quoted.
+        _list_converter_config.reset()
+        _list_converter_config._parse([u"list=%s" % (unicode_str,)])
+        maybe_list = _list_converter_config['list']
+        if isinstance(maybe_list, basestring):
+            if maybe_list:
+                # A single value, most probably the user forgot (or didn't care
+                # to add) the final ','
+                l = [maybe_list]
+            else:
+                # The empty string, convert to empty list
+                l = []
         else:
-            # The empty string, convert to empty list
-            l = []
-    else:
-        # We rely on ConfigObj providing us with a list already
-        l = unicode_str
-    return l
+            # We rely on ConfigObj providing us with a list already
+            l = maybe_list
+        return l
 
 
 class OptionRegistry(registry.Registry):
@@ -2481,6 +2593,58 @@
 # Registered options in lexicographical order
 
 option_registry.register(
+    Option('append_revisions_only',
+           default=None, from_unicode=bool_from_store, invalid='warning',
+           help='''\
+Whether to only append revisions to the mainline.
+
+If this is set to true, then it is not possible to change the
+existing mainline of the branch.
+'''))
+option_registry.register(
+    ListOption('acceptable_keys',
+           default=None,
+           help="""\
+List of GPG key patterns which are acceptable for verification.
+"""))
+option_registry.register(
+    Option('add.maximum_file_size',
+           default=u'20MB', from_unicode=int_SI_from_store,
+           help="""\
+Size above which files should be added manually.
+
+Files below this size are added automatically when using ``bzr add`` without
+arguments.
+
+A negative value means disable the size check.
+"""))
+option_registry.register(
+    Option('bound',
+           default=None, from_unicode=bool_from_store,
+           help="""\
+Is the branch bound to ``bound_location``.
+
+If set to "True", the branch should act as a checkout, and push each commit to
+the bound_location.  This option is normally set by ``bind``/``unbind``.
+
+See also: bound_location.
+"""))
+option_registry.register(
+    Option('bound_location',
+           default=None,
+           help="""\
+The location that commits should go to when acting as a checkout.
+
+This option is normally set by ``bind``.
+
+See also: bound.
+"""))
+option_registry.register(
+    Option('branch.fetch_tags', default=False,  from_unicode=bool_from_store,
+           help="""\
+Whether revisions associated with tags should be fetched.
+"""))
+option_registry.register(
     Option('bzr.workingtree.worth_saving_limit', default=10,
            from_unicode=int_from_store,  invalid='warning',
            help='''\
@@ -2493,6 +2657,29 @@
 a file has been touched.
 '''))
 option_registry.register(
+    Option('check_signatures', default=CHECK_IF_POSSIBLE,
+           from_unicode=signature_policy_from_unicode,
+           help='''\
+GPG checking policy.
+
+Possible values: require, ignore, check-available (default)
+
+this option will control whether bzr will require good gpg
+signatures, ignore them, or check them if they are
+present.
+'''))
+option_registry.register(
+    Option('create_signatures', default=SIGN_WHEN_REQUIRED,
+           from_unicode=signing_policy_from_unicode,
+           help='''\
+GPG Signing policy.
+
+Possible values: always, never, when-required (default)
+
+This option controls whether bzr will always create
+gpg signatures or not on commits.
+'''))
+option_registry.register(
     Option('dirstate.fdatasync', default=True,
            from_unicode=bool_from_store,
            help='''\
@@ -2503,15 +2690,44 @@
 should not be lost if the machine crashes.  See also repository.fdatasync.
 '''))
 option_registry.register(
-    Option('debug_flags', default=[], from_unicode=list_from_store,
+    ListOption('debug_flags', default=[],
            help='Debug flags to activate.'))
 option_registry.register(
     Option('default_format', default='2a',
            help='Format used when creating branches.'))
 option_registry.register(
+    Option('dpush_strict', default=None,
+           from_unicode=bool_from_store,
+           help='''\
+The default value for ``dpush --strict``.
+
+If present, defines the ``--strict`` option default value for checking
+uncommitted changes before pushing into a different VCS without any
+custom bzr metadata.
+'''))
+option_registry.register(
     Option('editor',
            help='The command called to launch an editor to enter a message.'))
 option_registry.register(
+    Option('email', override_from_env=['BZR_EMAIL'], default=default_email,
+           help='The users identity'))
+option_registry.register(
+    Option('gpg_signing_command',
+           default='gpg',
+           help="""\
+Program to use use for creating signatures.
+
+This should support at least the -u and --clearsign options.
+"""))
+option_registry.register(
+    Option('gpg_signing_key',
+           default=None,
+           help="""\
+GPG key to use for signing.
+
+This defaults to the first key associated with the users email.
+"""))
+option_registry.register(
     Option('ignore_missing_extensions', default=False,
            from_unicode=bool_from_store,
            help='''\
@@ -2535,10 +2751,62 @@
 Otherwise, bzr will prompt as normal to break the lock.
 '''))
 option_registry.register(
+    Option('log_format', default='long',
+           help= '''\
+Log format to use when displaying revisions.
+
+Standard log formats are ``long``, ``short`` and ``line``. Additional formats
+may be provided by plugins.
+'''))
+option_registry.register(
     Option('output_encoding',
            help= 'Unicode encoding for output'
            ' (terminal encoding if not specified).'))
 option_registry.register(
+    Option('parent_location',
+           default=None,
+           help="""\
+The location of the default branch for pull or merge.
+
+This option is normally set when creating a branch, the first ``pull`` or by
+``pull --remember``.
+"""))
+option_registry.register(
+    Option('post_commit', default=None,
+           help='''\
+Post commit functions.
+
+An ordered list of python functions to call, separated by spaces.
+
+Each function takes branch, rev_id as parameters.
+'''))
+option_registry.register(
+    Option('public_branch',
+           default=None,
+           help="""\
+A publically-accessible version of this branch.
+
+This implies that the branch setting this option is not publically-accessible.
+Used and set by ``bzr send``.
+"""))
+option_registry.register(
+    Option('push_location',
+           default=None,
+           help="""\
+The location of the default branch for push.
+
+This option is normally set by the first ``push`` or ``push --remember``.
+"""))
+option_registry.register(
+    Option('push_strict', default=None,
+           from_unicode=bool_from_store,
+           help='''\
+The default value for ``push --strict``.
+
+If present, defines the ``--strict`` option default value for checking
+uncommitted changes before sending a merge directive.
+'''))
+option_registry.register(
     Option('repository.fdatasync', default=True,
            from_unicode=bool_from_store,
            help='''\
@@ -2548,6 +2816,47 @@
 to physical disk.  This is somewhat slower, but means data should not be
 lost if the machine crashes.  See also dirstate.fdatasync.
 '''))
+option_registry.register_lazy('smtp_server',
+    'bzrlib.smtp_connection', 'smtp_server')
+option_registry.register_lazy('smtp_password',
+    'bzrlib.smtp_connection', 'smtp_password')
+option_registry.register_lazy('smtp_username',
+    'bzrlib.smtp_connection', 'smtp_username')
+option_registry.register(
+    Option('selftest.timeout',
+        default='600',
+        from_unicode=int_from_store,
+        help='Abort selftest if one test takes longer than this many seconds',
+        ))
+
+option_registry.register(
+    Option('send_strict', default=None,
+           from_unicode=bool_from_store,
+           help='''\
+The default value for ``send --strict``.
+
+If present, defines the ``--strict`` option default value for checking
+uncommitted changes before sending a bundle.
+'''))
+
+option_registry.register(
+    Option('serve.client_timeout',
+           default=300.0, from_unicode=float_from_store,
+           help="If we wait for a new request from a client for more than"
+                " X seconds, consider the client idle, and hangup."))
+option_registry.register(
+    Option('stacked_on_location',
+           default=None,
+           help="""The location where this branch is stacked on."""))
+option_registry.register(
+    Option('submit_branch',
+           default=None,
+           help="""\
+The branch you intend to submit your current work to.
+
+This is automatically set by ``bzr send`` and ``bzr merge``, and is also used
+by the ``submit:`` revision spec.
+"""))
 
 
 class Section(object):
@@ -2563,9 +2872,13 @@
         # We re-use the dict-like object received
         self.options = options
 
-    def get(self, name, default=None):
+    def get(self, name, default=None, expand=True):
         return self.options.get(name, default)
 
+    def iter_option_names(self):
+        for k in self.options.iterkeys():
+            yield k
+
     def __repr__(self):
         # Mostly for debugging use
         return "<config.%s id=%s>" % (self.__class__.__name__, self.id)
@@ -2630,6 +2943,20 @@
         """
         raise NotImplementedError(self.unload)
 
+    def quote(self, value):
+        """Quote a configuration option value for storing purposes.
+
+        This allows Stacks to present values as they will be stored.
+        """
+        return value
+
+    def unquote(self, value):
+        """Unquote a configuration option value into unicode.
+
+        The received value is quoted as stored.
+        """
+        return value
+
     def save(self):
         """Saves the Store to persistent storage."""
         raise NotImplementedError(self.save)
@@ -2640,14 +2967,14 @@
     def get_sections(self):
         """Returns an ordered iterable of existing sections.
 
-        :returns: An iterable of (name, dict).
+        :returns: An iterable of (store, section).
         """
         raise NotImplementedError(self.get_sections)
 
-    def get_mutable_section(self, section_name=None):
+    def get_mutable_section(self, section_id=None):
         """Returns the specified mutable section.
 
-        :param section_name: The section identifier
+        :param section_id: The section identifier
         """
         raise NotImplementedError(self.get_mutable_section)
 
@@ -2657,6 +2984,41 @@
                                     self.external_url())
 
 
+class CommandLineStore(Store):
+    "A store to carry command line overrides for the config options."""
+
+    def __init__(self, opts=None):
+        super(CommandLineStore, self).__init__()
+        if opts is None:
+            opts = {}
+        self.options = {}
+        self.id = 'cmdline'
+
+    def _reset(self):
+        # The dict should be cleared but not replaced so it can be shared.
+        self.options.clear()
+
+    def _from_cmdline(self, overrides):
+        # Reset before accepting new definitions
+        self._reset()
+        for over in overrides:
+            try:
+                name, value = over.split('=', 1)
+            except ValueError:
+                raise errors.BzrCommandError(
+                    gettext("Invalid '%s', should be of the form 'name=value'")
+                    % (over,))
+            self.options[name] = value
+
+    def external_url(self):
+        # Not an url but it makes debugging easier and is never needed
+        # otherwise
+        return 'cmdline'
+
+    def get_sections(self):
+        yield self,  self.readonly_section_class(None, self.options)
+
+
 class IniFileStore(Store):
     """A config Store using ConfigObj for storage.
 
@@ -2668,16 +3030,10 @@
         serialize/deserialize the config file.
     """
 
-    def __init__(self, transport, file_name):
+    def __init__(self):
         """A config Store using ConfigObj for storage.
-
-        :param transport: The transport object where the config file is located.
-
-        :param file_name: The config file basename in the transport directory.
         """
         super(IniFileStore, self).__init__()
-        self.transport = transport
-        self.file_name = file_name
         self._config_obj = None
 
     def is_loaded(self):
@@ -2686,11 +3042,29 @@
     def unload(self):
         self._config_obj = None
 
+    def _load_content(self):
+        """Load the config file bytes.
+
+        This should be provided by subclasses
+
+        :return: Byte string
+        """
+        raise NotImplementedError(self._load_content)
+
+    def _save_content(self, content):
+        """Save the config file bytes.
+
+        This should be provided by subclasses
+
+        :param content: Config file bytes to write
+        """
+        raise NotImplementedError(self._save_content)
+
     def load(self):
         """Load the store from the associated file."""
         if self.is_loaded():
             return
-        content = self.transport.get_bytes(self.file_name)
+        content = self._load_content()
         self._load_from_string(content)
         for hook in ConfigHooks['load']:
             hook(self)
@@ -2705,7 +3079,8 @@
         co_input = StringIO(bytes)
         try:
             # The config files are always stored utf8-encoded
-            self._config_obj = ConfigObj(co_input, encoding='utf-8')
+            self._config_obj = ConfigObj(co_input, encoding='utf-8',
+                                         list_values=False)
         except configobj.ConfigObjError, e:
             self._config_obj = None
             raise errors.ParseConfigError(e.errors, self.external_url())
@@ -2718,47 +3093,90 @@
             return
         out = StringIO()
         self._config_obj.write(out)
-        self.transport.put_bytes(self.file_name, out.getvalue())
+        self._save_content(out.getvalue())
         for hook in ConfigHooks['save']:
             hook(self)
 
-    def external_url(self):
-        # FIXME: external_url should really accepts an optional relpath
-        # parameter (bug #750169) :-/ -- vila 2011-04-04
-        # The following will do in the interim but maybe we don't want to
-        # expose a path here but rather a config ID and its associated
-        # object </hand wawe>.
-        return urlutils.join(self.transport.external_url(), self.file_name)
-
     def get_sections(self):
         """Get the configobj section in the file order.
 
-        :returns: An iterable of (name, dict).
+        :returns: An iterable of (store, section).
         """
         # We need a loaded store
         try:
             self.load()
-        except errors.NoSuchFile:
-            # If the file doesn't exist, there is no sections
+        except (errors.NoSuchFile, errors.PermissionDenied):
+            # If the file can't be read, there is no sections
             return
         cobj = self._config_obj
         if cobj.scalars:
-            yield self.readonly_section_class(None, cobj)
+            yield self, self.readonly_section_class(None, cobj)
         for section_name in cobj.sections:
-            yield self.readonly_section_class(section_name, cobj[section_name])
+            yield (self,
+                   self.readonly_section_class(section_name,
+                                               cobj[section_name]))
 
-    def get_mutable_section(self, section_name=None):
+    def get_mutable_section(self, section_id=None):
         # We need a loaded store
         try:
             self.load()
         except errors.NoSuchFile:
             # The file doesn't exist, let's pretend it was empty
             self._load_from_string('')
-        if section_name is None:
+        if section_id is None:
             section = self._config_obj
         else:
-            section = self._config_obj.setdefault(section_name, {})
-        return self.mutable_section_class(section_name, section)
+            section = self._config_obj.setdefault(section_id, {})
+        return self.mutable_section_class(section_id, section)
+
+    def quote(self, value):
+        try:
+            # configobj conflates automagical list values and quoting
+            self._config_obj.list_values = True
+            return self._config_obj._quote(value)
+        finally:
+            self._config_obj.list_values = False
+
+    def unquote(self, value):
+        if value and isinstance(value, basestring):
+            # _unquote doesn't handle None nor empty strings nor anything that
+            # is not a string, really.
+            value = self._config_obj._unquote(value)
+        return value
+
+
+class TransportIniFileStore(IniFileStore):
+    """IniFileStore that loads files from a transport.
+    """
+
+    def __init__(self, transport, file_name):
+        """A Store using a ini file on a Transport
+
+        :param transport: The transport object where the config file is located.
+        :param file_name: The config file basename in the transport directory.
+        """
+        super(TransportIniFileStore, self).__init__()
+        self.transport = transport
+        self.file_name = file_name
+
+    def _load_content(self):
+        try:
+            return self.transport.get_bytes(self.file_name)
+        except errors.PermissionDenied:
+            trace.warning("Permission denied while trying to load "
+                          "configuration store %s.", self.external_url())
+            raise
+
+    def _save_content(self, content):
+        self.transport.put_bytes(self.file_name, content)
+
+    def external_url(self):
+        # FIXME: external_url should really accepts an optional relpath
+        # parameter (bug #750169) :-/ -- vila 2011-04-04
+        # The following will do in the interim but maybe we don't want to
+        # expose a path here but rather a config ID and its associated
+        # object </hand wawe>.
+        return urlutils.join(self.transport.external_url(), self.file_name)
 
 
 # Note that LockableConfigObjStore inherits from ConfigObjStore because we need
@@ -2767,7 +3185,7 @@
 # they may face the same issue.
 
 
-class LockableIniFileStore(IniFileStore):
+class LockableIniFileStore(TransportIniFileStore):
     """A ConfigObjStore using locks on save to ensure store integrity."""
 
     def __init__(self, transport, file_name, lock_dir_name=None):
@@ -2823,6 +3241,7 @@
         t = transport.get_transport_from_path(
             config_dir(), possible_transports=possible_transports)
         super(GlobalStore, self).__init__(t, 'bazaar.conf')
+        self.id = 'bazaar'
 
 
 class LocationStore(LockableIniFileStore):
@@ -2831,14 +3250,16 @@
         t = transport.get_transport_from_path(
             config_dir(), possible_transports=possible_transports)
         super(LocationStore, self).__init__(t, 'locations.conf')
-
-
-class BranchStore(IniFileStore):
+        self.id = 'locations'
+
+
+class BranchStore(TransportIniFileStore):
 
     def __init__(self, branch):
         super(BranchStore, self).__init__(branch.control_transport,
                                           'branch.conf')
         self.branch = branch
+        self.id = 'branch'
 
     def lock_write(self, token=None):
         return self.branch.lock_write(token)
@@ -2861,13 +3282,14 @@
         super(ControlStore, self).__init__(bzrdir.transport,
                                           'control.conf',
                                            lock_dir_name='branch_lock')
+        self.id = 'control'
 
 
 class SectionMatcher(object):
     """Select sections into a given Store.
 
-    This intended to be used to postpone getting an iterable of sections from a
-    store.
+    This is intended to be used to postpone getting an iterable of sections
+    from a store.
     """
 
     def __init__(self, store):
@@ -2878,28 +3300,59 @@
         # sections.
         sections = self.store.get_sections()
         # Walk the revisions in the order provided
-        for s in sections:
+        for store, s in sections:
             if self.match(s):
-                yield s
-
-    def match(self, secion):
+                yield store, s
+
+    def match(self, section):
+        """Does the proposed section match.
+
+        :param section: A Section object.
+
+        :returns: True if the section matches, False otherwise.
+        """
         raise NotImplementedError(self.match)
 
 
+class NameMatcher(SectionMatcher):
+
+    def __init__(self, store, section_id):
+        super(NameMatcher, self).__init__(store)
+        self.section_id = section_id
+
+    def match(self, section):
+        return section.id == self.section_id
+
+
 class LocationSection(Section):
 
     def __init__(self, section, length, extra_path):
         super(LocationSection, self).__init__(section.id, section.options)
         self.length = length
         self.extra_path = extra_path
+        self.locals = {'relpath': extra_path,
+                       'basename': urlutils.basename(extra_path)}
 
-    def get(self, name, default=None):
+    def get(self, name, default=None, expand=True):
         value = super(LocationSection, self).get(name, default)
-        if value is not None:
+        if value is not None and expand:
             policy_name = self.get(name + ':policy', None)
             policy = _policy_value.get(policy_name, POLICY_NONE)
             if policy == POLICY_APPENDPATH:
                 value = urlutils.join(value, self.extra_path)
+            # expand section local options right now (since POLICY_APPENDPATH
+            # will never add options references, it's ok to expand after it).
+            chunks = []
+            for is_ref, chunk in iter_option_refs(value):
+                if not is_ref:
+                    chunks.append(chunk)
+                else:
+                    ref = chunk[1:-1]
+                    if ref in self.locals:
+                        chunks.append(self.locals[ref])
+                    else:
+                        chunks.append(chunk)
+            value = ''.join(chunks)
         return value
 
 
@@ -2919,7 +3372,7 @@
         all_sections = []
         # Filter out the no_name_section so _iter_for_location_by_parts can be
         # used (it assumes all sections have a name).
-        for section in self.store.get_sections():
+        for _, section in self.store.get_sections():
             if section.id is None:
                 no_name_section = section
             else:
@@ -2962,13 +3415,30 @@
             if ignore:
                 break
             # Finally, we have a valid section
-            yield section
+            yield self.store, section
+
+
+_option_ref_re = lazy_regex.lazy_compile('({[^{}\n]+})')
+"""Describes an expandable option reference.
+
+We want to match the most embedded reference first.
+
+I.e. for '{{foo}}' we will get '{foo}',
+for '{bar{baz}}' we will get '{baz}'
+"""
+
+def iter_option_refs(string):
+    # Split isolate refs so every other chunk is a ref
+    is_ref = False
+    for chunk  in _option_ref_re.split(string):
+        yield is_ref, chunk
+        is_ref = not is_ref
 
 
 class Stack(object):
     """A stack of configurations where an option can be defined"""
 
-    def __init__(self, sections_def, store=None, mutable_section_name=None):
+    def __init__(self, sections_def, store=None, mutable_section_id=None):
         """Creates a stack of sections with an optional store for changes.
 
         :param sections_def: A list of Section or callables that returns an
@@ -2978,15 +3448,15 @@
         :param store: The optional Store where modifications will be
             recorded. If none is specified, no modifications can be done.
 
-        :param mutable_section_name: The name of the MutableSection where
-            changes are recorded. This requires the ``store`` parameter to be
+        :param mutable_section_id: The id of the MutableSection where changes
+            are recorded. This requires the ``store`` parameter to be
             specified.
         """
         self.sections_def = sections_def
         self.store = store
-        self.mutable_section_name = mutable_section_name
+        self.mutable_section_id = mutable_section_id
 
-    def get(self, name):
+    def get(self, name, expand=None):
         """Return the *first* option value found in the sections.
 
         This is where we guarantee that sections coming from Store are loaded
@@ -2994,25 +3464,18 @@
         option exists or get its value, which in turn may require to discover
         in which sections it can be defined. Both of these (section and option
         existence) require loading the store (even partially).
+
+        :param name: The queried option.
+
+        :param expand: Whether options references should be expanded.
+
+        :returns: The value of the option.
         """
         # FIXME: No caching of options nor sections yet -- vila 20110503
+        if expand is None:
+            expand = _get_expand_default_value()
         value = None
-        # Ensuring lazy loading is achieved by delaying section matching (which
-        # implies querying the persistent storage) until it can't be avoided
-        # anymore by using callables to describe (possibly empty) section
-        # lists.
-        for section_or_callable in self.sections_def:
-            # Each section can expand to multiple ones when a callable is used
-            if callable(section_or_callable):
-                sections = section_or_callable()
-            else:
-                sections = [section_or_callable]
-            for section in sections:
-                value = section.get(name)
-                if value is not None:
-                    break
-            if value is not None:
-                break
+        found_store = None # Where the option value has been found
         # If the option is registered, it may provide additional info about
         # value handling
         try:
@@ -3020,37 +3483,138 @@
         except KeyError:
             # Not registered
             opt = None
-        if opt is not None:
-            value = opt.convert_from_unicode(value)
-            if value is None:
-                # The conversion failed or there was no value to convert,
-                # fallback to the default value
-                value = opt.convert_from_unicode(opt.get_default())
+
+        def expand_and_convert(val):
+            # This may need to be called in different contexts if the value is
+            # None or ends up being None during expansion or conversion.
+            if val is not None:
+                if expand:
+                    if isinstance(val, basestring):
+                        val = self._expand_options_in_string(val)
+                    else:
+                        trace.warning('Cannot expand "%s":'
+                                      ' %s does not support option expansion'
+                                      % (name, type(val)))
+                if opt is None:
+                    val = found_store.unquote(val)
+                else:
+                    val = opt.convert_from_unicode(found_store, val)
+            return val
+
+        # First of all, check if the environment can override the configuration
+        # value
+        if opt is not None and opt.override_from_env:
+            value = opt.get_override()
+            value = expand_and_convert(value)
+        if value is None:
+            # Ensuring lazy loading is achieved by delaying section matching
+            # (which implies querying the persistent storage) until it can't be
+            # avoided anymore by using callables to describe (possibly empty)
+            # section lists.
+            for sections in self.sections_def:
+                for store, section in sections():
+                    value = section.get(name)
+                    if value is not None:
+                        found_store = store
+                        break
+                if value is not None:
+                    break
+            value = expand_and_convert(value)
+            if opt is not None and value is None:
+                # If the option is registered, it may provide a default value
+                value = opt.get_default()
+                value = expand_and_convert(value)
         for hook in ConfigHooks['get']:
             hook(self, name, value)
         return value
 
+    def expand_options(self, string, env=None):
+        """Expand option references in the string in the configuration context.
+
+        :param string: The string containing option(s) to expand.
+
+        :param env: An option dict defining additional configuration options or
+            overriding existing ones.
+
+        :returns: The expanded string.
+        """
+        return self._expand_options_in_string(string, env)
+
+    def _expand_options_in_string(self, string, env=None, _refs=None):
+        """Expand options in the string in the configuration context.
+
+        :param string: The string to be expanded.
+
+        :param env: An option dict defining additional configuration options or
+            overriding existing ones.
+
+        :param _refs: Private list (FIFO) containing the options being expanded
+            to detect loops.
+
+        :returns: The expanded string.
+        """
+        if string is None:
+            # Not much to expand there
+            return None
+        if _refs is None:
+            # What references are currently resolved (to detect loops)
+            _refs = []
+        result = string
+        # We need to iterate until no more refs appear ({{foo}} will need two
+        # iterations for example).
+        expanded = True
+        while expanded:
+            expanded = False
+            chunks = []
+            for is_ref, chunk in iter_option_refs(result):
+                if not is_ref:
+                    chunks.append(chunk)
+                else:
+                    expanded = True
+                    name = chunk[1:-1]
+                    if name in _refs:
+                        raise errors.OptionExpansionLoop(string, _refs)
+                    _refs.append(name)
+                    value = self._expand_option(name, env, _refs)
+                    if value is None:
+                        raise errors.ExpandingUnknownOption(name, string)
+                    chunks.append(value)
+                    _refs.pop()
+            result = ''.join(chunks)
+        return result
+
+    def _expand_option(self, name, env, _refs):
+        if env is not None and name in env:
+            # Special case, values provided in env takes precedence over
+            # anything else
+            value = env[name]
+        else:
+            value = self.get(name, expand=False)
+            value = self._expand_options_in_string(value, env, _refs)
+        return value
+
     def _get_mutable_section(self):
         """Get the MutableSection for the Stack.
 
         This is where we guarantee that the mutable section is lazily loaded:
         this means we won't load the corresponding store before setting a value
         or deleting an option. In practice the store will often be loaded but
-        this allows helps catching some programming errors.
+        this helps catching some programming errors.
         """
-        section = self.store.get_mutable_section(self.mutable_section_name)
-        return section
+        store = self.store
+        section = store.get_mutable_section(self.mutable_section_id)
+        return store, section
 
     def set(self, name, value):
         """Set a new value for the option."""
-        section = self._get_mutable_section()
-        section.set(name, value)
+        store, section = self._get_mutable_section()
+        section.set(name, store.quote(value))
         for hook in ConfigHooks['set']:
             hook(self, name, value)
 
     def remove(self, name):
         """Remove an existing option."""
-        section = self._get_mutable_section()
+        _, section = self._get_mutable_section()
         section.remove(name)
         for hook in ConfigHooks['remove']:
             hook(self, name)
@@ -3059,6 +3623,35 @@
         # Mostly for debugging use
         return "<config.%s(%s)>" % (self.__class__.__name__, id(self))
 
+    def _get_overrides(self):
+        # Hack around library_state.initialize never called
+        if bzrlib.global_state is not None:
+            return bzrlib.global_state.cmdline_overrides.get_sections()
+        return []
+
+
+class MemoryStack(Stack):
+    """A configuration stack defined from a string.
+
+    This is mainly intended for tests and requires no disk resources.
+    """
+
+    def __init__(self, content=None):
+        """Create an in-memory stack from a given content.
+
+        It uses a single store based on configobj and support reading and
+        writing options.
+
+        :param content: The initial content of the store. If None, the store is
+            not loaded and ``_load_from_string`` can and should be used if
+            needed.
+        """
+        store = IniFileStore()
+        if content is not None:
+            store._load_from_string(content)
+        super(MemoryStack, self).__init__(
+            [store.get_sections], store)
+
 
 class _CompatibleStack(Stack):
     """Place holder for compatibility with previous design.
@@ -3069,8 +3662,8 @@
     One assumption made here is that the daughter classes will all use Stores
     derived from LockableIniFileStore).
 
-    It implements set() by re-loading the store before applying the
-    modification and saving it.
+    It implements set() and remove () by re-loading the store before applying
+    the modification and saving it.
 
     The long term plan being to implement a single write by store to save
     all modifications, this class should not be used in the interim.
@@ -3083,40 +3676,96 @@
         # Force a write to persistent storage
         self.store.save()
 
+    def remove(self, name):
+        # Force a reload
+        self.store.unload()
+        super(_CompatibleStack, self).remove(name)
+        # Force a write to persistent storage
+        self.store.save()
+
 
 class GlobalStack(_CompatibleStack):
-    """Global options only stack."""
+    """Global options only stack.
+
+    The following sections are queried:
+
+    * command-line overrides,
+
+    * the 'DEFAULT' section in bazaar.conf
+
+    This stack will use the ``DEFAULT`` section in bazaar.conf as its
+    MutableSection.
+    """
 
     def __init__(self):
-        # Get a GlobalStore
         gstore = GlobalStore()
-        super(GlobalStack, self).__init__([gstore.get_sections], gstore)
+        super(GlobalStack, self).__init__(
+            [self._get_overrides,
+             NameMatcher(gstore, 'DEFAULT').get_sections],
+            gstore, mutable_section_id='DEFAULT')
 
 
 class LocationStack(_CompatibleStack):
-    """Per-location options falling back to global options stack."""
+    """Per-location options falling back to global options stack.
+
+
+    The following sections are queried:
+
+    * command-line overrides,
+
+    * the sections matching ``location`` in ``locations.conf``, the order being
+      defined by the number of path components in the section glob, higher
+      numbers first (from most specific section to most generic).
+
+    * the 'DEFAULT' section in bazaar.conf
+
+    This stack will use the ``location`` section in locations.conf as its
+    MutableSection.
+    """
 
     def __init__(self, location):
         """Make a new stack for a location and global configuration.
-        
+
         :param location: A URL prefix to """
         lstore = LocationStore()
-        matcher = LocationMatcher(lstore, location)
+        if location.startswith('file://'):
+            location = urlutils.local_path_from_url(location)
         gstore = GlobalStore()
         super(LocationStack, self).__init__(
-            [matcher.get_sections, gstore.get_sections], lstore)
+            [self._get_overrides,
+             LocationMatcher(lstore, location).get_sections,
+             NameMatcher(gstore, 'DEFAULT').get_sections],
+            lstore, mutable_section_id=location)
 
 
 class BranchStack(_CompatibleStack):
-    """Per-location options falling back to branch then global options stack."""
+    """Per-location options falling back to branch then global options stack.
+
+    The following sections are queried:
+
+    * command-line overrides,
+
+    * the sections matching ``location`` in ``locations.conf``, the order being
+      defined by the number of path components in the section glob, higher
+      numbers first (from most specific section to most generic),
+
+    * the no-name section in branch.conf,
+
+    * the ``DEFAULT`` section in ``bazaar.conf``.
+
+    This stack will use the no-name section in ``branch.conf`` as its
+    MutableSection.
+    """
 
     def __init__(self, branch):
-        bstore = BranchStore(branch)
         lstore = LocationStore()
-        matcher = LocationMatcher(lstore, branch.base)
+        bstore = branch._get_config_store()
         gstore = GlobalStore()
         super(BranchStack, self).__init__(
-            [matcher.get_sections, bstore.get_sections, gstore.get_sections],
+            [self._get_overrides,
+             LocationMatcher(lstore, branch.base).get_sections,
+             NameMatcher(bstore, None).get_sections,
+             NameMatcher(gstore, 'DEFAULT').get_sections],
             bstore)
         self.branch = branch
 
@@ -3124,25 +3773,38 @@
 class RemoteControlStack(_CompatibleStack):
     """Remote control-only options stack."""
 
+    # FIXME 2011-11-22 JRV This should probably be renamed to avoid confusion
+    # with the stack used for remote bzr dirs. RemoteControlStack only uses
+    # control.conf and is used only for stack options.
+
     def __init__(self, bzrdir):
-        cstore = ControlStore(bzrdir)
+        cstore = bzrdir._get_config_store()
         super(RemoteControlStack, self).__init__(
-            [cstore.get_sections],
+            [NameMatcher(cstore, None).get_sections],
             cstore)
         self.bzrdir = bzrdir
 
 
-class RemoteBranchStack(_CompatibleStack):
-    """Remote branch-only options stack."""
+class BranchOnlyStack(_CompatibleStack):
+    """Branch-only options stack."""
+
+    # FIXME: _BranchOnlyStack only uses branch.conf and is used only for the
+    # stacked_on_location options waiting for http://pad.lv/832042 to be fixed.
+    # -- vila 2011-12-16
 
     def __init__(self, branch):
-        bstore = BranchStore(branch)
-        super(RemoteBranchStack, self).__init__(
-            [bstore.get_sections],
+        bstore = branch._get_config_store()
+        super(BranchOnlyStack, self).__init__(
+            [NameMatcher(bstore, None).get_sections],
             bstore)
         self.branch = branch
 
 
+# Use a an empty dict to initialize an empty configobj avoiding all
+# parsing and encoding checks
+_quoting_config = configobj.ConfigObj(
+    {}, encoding='utf-8', interpolation=False, list_values=True)
+
 class cmd_config(commands.Command):
     __doc__ = """Display, set or remove a configuration option.
 
@@ -3164,15 +3826,16 @@
     takes_options = [
         'directory',
         # FIXME: This should be a registry option so that plugins can register
-        # their own config files (or not) -- vila 20101002
+        # their own config files (or not) and will also address
+        # http://pad.lv/788991 -- vila 20101115
         commands.Option('scope', help='Reduce the scope to the specified'
-                        ' configuration file',
+                        ' configuration file.',
                         type=unicode),
         commands.Option('all',
             help='Display all the defined values for the matching options.',
             ),
         commands.Option('remove', help='Remove the option from'
-                        ' the configuration file'),
+                        ' the configuration file.'),
         ]
 
     _see_also = ['configuration']
@@ -3208,51 +3871,44 @@
                 # Set the option value
                 self._set_config_option(name, value, directory, scope)
 
-    def _get_configs(self, directory, scope=None):
-        """Iterate the configurations specified by ``directory`` and ``scope``.
+    def _get_stack(self, directory, scope=None):
+        """Get the configuration stack specified by ``directory`` and ``scope``.
 
         :param directory: Where the configurations are derived from.
 
         :param scope: A specific config to start from.
         """
+        # FIXME: scope should allow access to plugin-specific stacks (even
+        # reduced to the plugin-specific store), related to
+        # http://pad.lv/788991 -- vila 2011-11-15
         if scope is not None:
             if scope == 'bazaar':
-                yield GlobalConfig()
+                return GlobalStack()
             elif scope == 'locations':
-                yield LocationConfig(directory)
+                return LocationStack(directory)
             elif scope == 'branch':
-                (_, br, _) = bzrdir.BzrDir.open_containing_tree_or_branch(
-                    directory)
-                yield br.get_config()
+                (_, br, _) = (
+                    controldir.ControlDir.open_containing_tree_or_branch(
+                        directory))
+                return br.get_config_stack()
+            raise errors.NoSuchConfig(scope)
         else:
             try:
-                (_, br, _) = bzrdir.BzrDir.open_containing_tree_or_branch(
-                    directory)
-                yield br.get_config()
+                (_, br, _) = (
+                    controldir.ControlDir.open_containing_tree_or_branch(
+                        directory))
+                return br.get_config_stack()
             except errors.NotBranchError:
-                yield LocationConfig(directory)
-                yield GlobalConfig()
+                return LocationStack(directory)
 
     def _show_value(self, name, directory, scope):
-        displayed = False
-        for c in self._get_configs(directory, scope):
-            if displayed:
-                break
-            for (oname, value, section, conf_id, parser) in c._get_options():
-                if name == oname:
-                    # Display only the first value and exit
-
-                    # FIXME: We need to use get_user_option to take policies
-                    # into account and we need to make sure the option exists
-                    # too (hence the two for loops), this needs a better API
-                    # -- vila 20101117
-                    value = c.get_user_option(name)
-                    # Quote the value appropriately
-                    value = parser._quote(value)
-                    self.outf.write('%s\n' % (value,))
-                    displayed = True
-                    break
-        if not displayed:
+        conf = self._get_stack(directory, scope)
+        value = conf.get(name, expand=True)
+        if value is not None:
+            # Quote the value appropriately
+            value = _quoting_config._quote(value)
+            self.outf.write('%s\n' % (value,))
+        else:
             raise errors.NoSuchConfigOption(name)
 
     def _show_matching_options(self, name, directory, scope):
@@ -3261,55 +3917,50 @@
         # avoid the delay introduced by the lazy regexp.  But, we still do
         # want the nicer errors raised by lazy_regex.
         name._compile_and_collapse()
-        cur_conf_id = None
+        cur_store_id = None
         cur_section = None
-        for c in self._get_configs(directory, scope):
-            for (oname, value, section, conf_id, parser) in c._get_options():
-                if name.search(oname):
-                    if cur_conf_id != conf_id:
-                        # Explain where the options are defined
-                        self.outf.write('%s:\n' % (conf_id,))
-                        cur_conf_id = conf_id
-                        cur_section = None
-                    if (section not in (None, 'DEFAULT')
-                        and cur_section != section):
-                        # Display the section if it's not the default (or only)
-                        # one.
-                        self.outf.write('  [%s]\n' % (section,))
-                        cur_section = section
-                    self.outf.write('  %s = %s\n' % (oname, value))
+        conf = self._get_stack(directory, scope)
+        for sections in conf.sections_def:
+            for store, section in sections():
+                for oname in section.iter_option_names():
+                    if name.search(oname):
+                        if cur_store_id != store.id:
+                            # Explain where the options are defined
+                            self.outf.write('%s:\n' % (store.id,))
+                            cur_store_id = store.id
+                            cur_section = None
+                        if (section.id is not None
+                            and cur_section != section.id):
+                            # Display the section id as it appears in the store
+                            # (None doesn't appear by definition)
+                            self.outf.write('  [%s]\n' % (section.id,))
+                            cur_section = section.id
+                        value = section.get(oname, expand=False)
+                        # Since we don't use the stack, we need to restore a
+                        # proper quoting.
+                        try:
+                            opt = option_registry.get(oname)
+                            value = opt.convert_from_unicode(store, value)
+                        except KeyError:
+                            value = store.unquote(value)
+                        value = _quoting_config._quote(value)
+                        self.outf.write('  %s = %s\n' % (oname, value))
 
     def _set_config_option(self, name, value, directory, scope):
-        for conf in self._get_configs(directory, scope):
-            conf.set_user_option(name, value)
-            break
-        else:
-            raise errors.NoSuchConfig(scope)
+        conf = self._get_stack(directory, scope)
+        conf.set(name, value)
 
     def _remove_config_option(self, name, directory, scope):
         if name is None:
             raise errors.BzrCommandError(
                 '--remove expects an option to remove.')
-        removed = False
-        for conf in self._get_configs(directory, scope):
-            for (section_name, section, conf_id) in conf._get_sections():
-                if scope is not None and conf_id != scope:
-                    # Not the right configuration file
-                    continue
-                if name in section:
-                    if conf_id != conf.config_id():
-                        conf = self._get_configs(directory, conf_id).next()
-                    # We use the first section in the first config where the
-                    # option is defined to remove it
-                    conf.remove_user_option(name, section_name)
-                    removed = True
-                    break
-            break
-        else:
-            raise errors.NoSuchConfig(scope)
-        if not removed:
+        conf = self._get_stack(directory, scope)
+        try:
+            conf.remove(name)
+        except KeyError:
             raise errors.NoSuchConfigOption(name)
 
+
 # Test registries
 #
 # We need adapters that can build a Store or a Stack in a test context. Test
@@ -3318,7 +3969,7 @@
 # ready-to-use store or stack.  Plugins that define new store/stacks can also
 # register themselves here to be tested against the tests defined in
 # bzrlib.tests.test_config. Note that the builder can be called multiple times
-# for the same tests.
+# for the same test.
 
 # The registered object should be a callable receiving a test instance
 # parameter (inheriting from tests.TestCaseWithTransport) and returning a Store

=== modified file 'bzrlib/conflicts.py'
--- bzrlib/conflicts.py	2011-06-21 15:19:18 +0000
+++ bzrlib/conflicts.py	2011-12-18 12:46:49 +0000
@@ -17,6 +17,8 @@
 # TODO: 'bzr resolve' should accept a directory name and work from that
 # point down
 
+from __future__ import absolute_import
+
 import os
 
 from bzrlib.lazy_import import lazy_import
@@ -32,6 +34,7 @@
     transform,
     workingtree,
     )
+from bzrlib.i18n import gettext, ngettext
 """)
 from bzrlib import (
     commands,
@@ -79,13 +82,13 @@
 
 
 resolve_action_registry.register(
-    'done', 'done', 'Marks the conflict as resolved' )
+    'done', 'done', 'Marks the conflict as resolved.')
 resolve_action_registry.register(
     'take-this', 'take_this',
-    'Resolve the conflict preserving the version in the working tree' )
+    'Resolve the conflict preserving the version in the working tree.')
 resolve_action_registry.register(
     'take-other', 'take_other',
-    'Resolve the conflict taking the merged version into account' )
+    'Resolve the conflict taking the merged version into account.')
 resolve_action_registry.default_key = 'done'
 
 class ResolveActionOption(option.RegistryOption):
@@ -120,8 +123,8 @@
     def run(self, file_list=None, all=False, action=None, directory=None):
         if all:
             if file_list:
-                raise errors.BzrCommandError("If --all is specified,"
-                                             " no FILE may be provided")
+                raise errors.BzrCommandError(gettext("If --all is specified,"
+                                             " no FILE may be provided"))
             if directory is None:
                 directory = u'.'
             tree = workingtree.WorkingTree.open_containing(directory)[0]
@@ -145,13 +148,15 @@
             if file_list is None:
                 un_resolved, resolved = tree.auto_resolve()
                 if len(un_resolved) > 0:
-                    trace.note('%d conflict(s) auto-resolved.', len(resolved))
-                    trace.note('Remaining conflicts:')
+                    trace.note(ngettext('%d conflict auto-resolved.',
+                        '%d conflicts auto-resolved.', len(resolved)),
+                        len(resolved))
+                    trace.note(gettext('Remaining conflicts:'))
                     for conflict in un_resolved:
                         trace.note(unicode(conflict))
                     return 1
                 else:
-                    trace.note('All conflicts resolved.')
+                    trace.note(gettext('All conflicts resolved.'))
                     return 0
             else:
                 # FIXME: This can never occur but the block above needs some
@@ -160,8 +165,9 @@
                 pass
         else:
             before, after = resolve(tree, file_list, action=action)
-            trace.note('%d conflict(s) resolved, %d remaining'
-                       % (before - after, after))
+            trace.note(ngettext('{0} conflict resolved, {1} remaining',
+                                '{0} conflicts resolved, {1} remaining',
+                                before-after).format(before - after, after))
 
 
 def resolve(tree, paths=None, ignore_misses=False, recursive=False,

=== modified file 'bzrlib/controldir.py'
--- bzrlib/controldir.py	2011-08-17 18:13:57 +0000
+++ bzrlib/controldir.py	2012-01-03 13:47:01 +0000
@@ -22,21 +22,27 @@
 
 """
 
+from __future__ import absolute_import
+
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 import textwrap
 
 from bzrlib import (
     errors,
+    hooks,
     revision as _mod_revision,
     transport as _mod_transport,
+    trace,
     ui,
     urlutils,
     )
+from bzrlib.transport import local
 from bzrlib.push import (
     PushResult,
     )
 
+from bzrlib.i18n import gettext
 """)
 
 from bzrlib import registry
@@ -75,7 +81,6 @@
         return self.user_transport.base
 
 
-
 class ControlDir(ControlComponent):
     """A control directory.
 
@@ -103,10 +108,17 @@
         """Return a sequence of all branches local to this control directory.
 
         """
+        return self.get_branches().values()
+
+    def get_branches(self):
+        """Get all branches in this control directory, as a dictionary.
+        
+        :return: Dictionary mapping branch names to instances.
+        """
         try:
-            return [self.open_branch()]
+           return { None: self.open_branch() }
         except (errors.NotBranchError, errors.NoRepositoryPresent):
-            return []
+           return {}
 
     def is_control_filename(self, filename):
         """True if filename is the name of a path which is reserved for
@@ -146,11 +158,14 @@
         """Destroy the repository in this ControlDir."""
         raise NotImplementedError(self.destroy_repository)
 
-    def create_branch(self, name=None, repository=None):
+    def create_branch(self, name=None, repository=None,
+                      append_revisions_only=None):
         """Create a branch in this ControlDir.
 
         :param name: Name of the colocated branch to create, None for
             the default branch.
+        :param append_revisions_only: Whether this branch should only allow
+            appending new revisions to its history.
 
         The controldirs format will control what branch format is created.
         For more control see BranchFormatXX.create(a_controldir).
@@ -195,7 +210,7 @@
         raise NotImplementedError(self.destroy_workingtree_metadata)
 
     def find_branch_format(self, name=None):
-        """Find the branch 'format' for this bzrdir.
+        """Find the branch 'format' for this controldir.
 
         This might be a synthetic object for e.g. RemoteBranch and SVN.
         """
@@ -216,13 +231,14 @@
         return None
 
     def open_branch(self, name=None, unsupported=False,
-                    ignore_fallbacks=False):
+                    ignore_fallbacks=False, possible_transports=None):
         """Open the branch object at this ControlDir if one is present.
 
-        If unsupported is True, then no longer supported branch formats can
-        still be opened.
-
-        TODO: static convenience version of this?
+        :param unsupported: if True, then no longer supported branch formats can
+            still be opened.
+        :param ignore_fallbacks: Whether to open fallback repositories
+        :param possible_transports: Transports to use for opening e.g.
+            fallback repositories.
         """
         raise NotImplementedError(self.open_branch)
 
@@ -234,8 +250,6 @@
         get at a repository.
 
         :param _unsupported: a private parameter, not part of the api.
-
-        TODO: static convenience version of this?
         """
         raise NotImplementedError(self.open_repository)
 
@@ -248,7 +262,7 @@
         """
         raise NotImplementedError(self.find_repository)
 
-    def open_workingtree(self, _unsupported=False,
+    def open_workingtree(self, unsupported=False,
                          recommend_upgrade=True, from_branch=None):
         """Open the workingtree object at this ControlDir if one is present.
 
@@ -269,7 +283,7 @@
         branch and discards it, and that's somewhat expensive.)
         """
         try:
-            self.open_branch(name)
+            self.open_branch(name, ignore_fallbacks=True)
             return True
         except errors.NotBranchError:
             return False
@@ -316,7 +330,12 @@
         raise NotImplementedError(self.cloning_metadir)
 
     def checkout_metadir(self):
-        """Produce a metadir suitable for checkouts of this controldir."""
+        """Produce a metadir suitable for checkouts of this controldir.
+
+        :returns: A ControlDirFormat with all component formats
+            either set appropriately or set to None if that component
+            should not be created.
+        """
         return self.cloning_metadir()
 
     def sprout(self, url, revision_id=None, force_new_repo=False,
@@ -413,7 +432,7 @@
         return push_result
 
     def _get_tree_branch(self, name=None):
-        """Return the branch and tree, if any, for this bzrdir.
+        """Return the branch and tree, if any, for this controldir.
 
         :param name: Name of colocated branch to open.
 
@@ -438,12 +457,12 @@
         raise NotImplementedError(self.get_config)
 
     def check_conversion_target(self, target_format):
-        """Check that a bzrdir as a whole can be converted to a new format."""
+        """Check that a controldir as a whole can be converted to a new format."""
         raise NotImplementedError(self.check_conversion_target)
 
     def clone(self, url, revision_id=None, force_new_repo=False,
               preserve_stacking=False):
-        """Clone this bzrdir and its contents to url verbatim.
+        """Clone this controldir and its contents to url verbatim.
 
         :param url: The url create the clone at.  If url's last component does
             not exist, it will be created.
@@ -463,7 +482,7 @@
     def clone_on_transport(self, transport, revision_id=None,
         force_new_repo=False, preserve_stacking=False, stacked_on=None,
         create_prefix=False, use_existing_dir=True, no_tree=False):
-        """Clone this bzrdir and its contents to transport verbatim.
+        """Clone this controldir and its contents to transport verbatim.
 
         :param transport: The transport for the location to produce the clone
             at.  If the target directory does not exist, it will be created.
@@ -481,16 +500,354 @@
         """
         raise NotImplementedError(self.clone_on_transport)
 
+    @classmethod
+    def find_bzrdirs(klass, transport, evaluate=None, list_current=None):
+        """Find control dirs recursively from current location.
+
+        This is intended primarily as a building block for more sophisticated
+        functionality, like finding trees under a directory, or finding
+        branches that use a given repository.
+
+        :param evaluate: An optional callable that yields recurse, value,
+            where recurse controls whether this controldir is recursed into
+            and value is the value to yield.  By default, all bzrdirs
+            are recursed into, and the return value is the controldir.
+        :param list_current: if supplied, use this function to list the current
+            directory, instead of Transport.list_dir
+        :return: a generator of found bzrdirs, or whatever evaluate returns.
+        """
+        if list_current is None:
+            def list_current(transport):
+                return transport.list_dir('')
+        if evaluate is None:
+            def evaluate(controldir):
+                return True, controldir
+
+        pending = [transport]
+        while len(pending) > 0:
+            current_transport = pending.pop()
+            recurse = True
+            try:
+                controldir = klass.open_from_transport(current_transport)
+            except (errors.NotBranchError, errors.PermissionDenied):
+                pass
+            else:
+                recurse, value = evaluate(controldir)
+                yield value
+            try:
+                subdirs = list_current(current_transport)
+            except (errors.NoSuchFile, errors.PermissionDenied):
+                continue
+            if recurse:
+                for subdir in sorted(subdirs, reverse=True):
+                    pending.append(current_transport.clone(subdir))
+
+    @classmethod
+    def find_branches(klass, transport):
+        """Find all branches under a transport.
+
+        This will find all branches below the transport, including branches
+        inside other branches.  Where possible, it will use
+        Repository.find_branches.
+
+        To list all the branches that use a particular Repository, see
+        Repository.find_branches
+        """
+        def evaluate(controldir):
+            try:
+                repository = controldir.open_repository()
+            except errors.NoRepositoryPresent:
+                pass
+            else:
+                return False, ([], repository)
+            return True, (controldir.list_branches(), None)
+        ret = []
+        for branches, repo in klass.find_bzrdirs(
+                transport, evaluate=evaluate):
+            if repo is not None:
+                ret.extend(repo.find_branches())
+            if branches is not None:
+                ret.extend(branches)
+        return ret
+
+    @classmethod
+    def create_branch_and_repo(klass, base, force_new_repo=False, format=None):
+        """Create a new ControlDir, Branch and Repository at the url 'base'.
+
+        This will use the current default ControlDirFormat unless one is
+        specified, and use whatever
+        repository format that that uses via controldir.create_branch and
+        create_repository. If a shared repository is available that is used
+        preferentially.
+
+        The created Branch object is returned.
+
+        :param base: The URL to create the branch at.
+        :param force_new_repo: If True a new repository is always created.
+        :param format: If supplied, the format of branch to create.  If not
+            supplied, the default is used.
+        """
+        controldir = klass.create(base, format)
+        controldir._find_or_create_repository(force_new_repo)
+        return controldir.create_branch()
+
+    @classmethod
+    def create_branch_convenience(klass, base, force_new_repo=False,
+                                  force_new_tree=None, format=None,
+                                  possible_transports=None):
+        """Create a new ControlDir, Branch and Repository at the url 'base'.
+
+        This is a convenience function - it will use an existing repository
+        if possible, can be told explicitly whether to create a working tree or
+        not.
+
+        This will use the current default ControlDirFormat unless one is
+        specified, and use whatever
+        repository format that that uses via ControlDir.create_branch and
+        create_repository. If a shared repository is available that is used
+        preferentially. Whatever repository is used, its tree creation policy
+        is followed.
+
+        The created Branch object is returned.
+        If a working tree cannot be made due to base not being a file:// url,
+        no error is raised unless force_new_tree is True, in which case no
+        data is created on disk and NotLocalUrl is raised.
+
+        :param base: The URL to create the branch at.
+        :param force_new_repo: If True a new repository is always created.
+        :param force_new_tree: If True or False force creation of a tree or
+                               prevent such creation respectively.
+        :param format: Override for the controldir format to create.
+        :param possible_transports: An optional reusable transports list.
+        """
+        if force_new_tree:
+            # check for non local urls
+            t = _mod_transport.get_transport(base, possible_transports)
+            if not isinstance(t, local.LocalTransport):
+                raise errors.NotLocalUrl(base)
+        controldir = klass.create(base, format, possible_transports)
+        repo = controldir._find_or_create_repository(force_new_repo)
+        result = controldir.create_branch()
+        if force_new_tree or (repo.make_working_trees() and
+                              force_new_tree is None):
+            try:
+                controldir.create_workingtree()
+            except errors.NotLocalUrl:
+                pass
+        return result
+
+    @classmethod
+    def create_standalone_workingtree(klass, base, format=None):
+        """Create a new ControlDir, WorkingTree, Branch and Repository at 'base'.
+
+        'base' must be a local path or a file:// url.
+
+        This will use the current default ControlDirFormat unless one is
+        specified, and use whatever
+        repository format that that uses for bzrdirformat.create_workingtree,
+        create_branch and create_repository.
+
+        :param format: Override for the controldir format to create.
+        :return: The WorkingTree object.
+        """
+        t = _mod_transport.get_transport(base)
+        if not isinstance(t, local.LocalTransport):
+            raise errors.NotLocalUrl(base)
+        controldir = klass.create_branch_and_repo(base,
+                                               force_new_repo=True,
+                                               format=format).bzrdir
+        return controldir.create_workingtree()
+
+    @classmethod
+    def open_unsupported(klass, base):
+        """Open a branch which is not supported."""
+        return klass.open(base, _unsupported=True)
+
+    @classmethod
+    def open(klass, base, possible_transports=None, probers=None,
+             _unsupported=False):
+        """Open an existing controldir, rooted at 'base' (url).
+
+        :param _unsupported: a private parameter to the ControlDir class.
+        """
+        t = _mod_transport.get_transport(base, possible_transports)
+        return klass.open_from_transport(t, probers=probers,
+                _unsupported=_unsupported)
+
+    @classmethod
+    def open_from_transport(klass, transport, _unsupported=False,
+                            probers=None):
+        """Open a controldir within a particular directory.
+
+        :param transport: Transport containing the controldir.
+        :param _unsupported: private.
+        """
+        for hook in klass.hooks['pre_open']:
+            hook(transport)
+        # Keep initial base since 'transport' may be modified while following
+        # the redirections.
+        base = transport.base
+        def find_format(transport):
+            return transport, ControlDirFormat.find_format(transport,
+                probers=probers)
+
+        def redirected(transport, e, redirection_notice):
+            redirected_transport = transport._redirected_to(e.source, e.target)
+            if redirected_transport is None:
+                raise errors.NotBranchError(base)
+            trace.note(gettext('{0} is{1} redirected to {2}').format(
+                 transport.base, e.permanently, redirected_transport.base))
+            return redirected_transport
+
+        try:
+            transport, format = _mod_transport.do_catching_redirections(
+                find_format, transport, redirected)
+        except errors.TooManyRedirections:
+            raise errors.NotBranchError(base)
+
+        format.check_support_status(_unsupported)
+        return format.open(transport, _found=True)
+
+    @classmethod
+    def open_containing(klass, url, possible_transports=None):
+        """Open an existing branch which contains url.
+
+        :param url: url to search from.
+
+        See open_containing_from_transport for more detail.
+        """
+        transport = _mod_transport.get_transport(url, possible_transports)
+        return klass.open_containing_from_transport(transport)
+
+    @classmethod
+    def open_containing_from_transport(klass, a_transport):
+        """Open an existing branch which contains a_transport.base.
+
+        This probes for a branch at a_transport, and searches upwards from there.
+
+        Basically we keep looking up until we find the control directory or
+        run into the root.  If there isn't one, raises NotBranchError.
+        If there is one and it is either an unrecognised format or an unsupported
+        format, UnknownFormatError or UnsupportedFormatError are raised.
+        If there is one, it is returned, along with the unused portion of url.
+
+        :return: The ControlDir that contains the path, and a Unicode path
+                for the rest of the URL.
+        """
+        # this gets the normalised url back. I.e. '.' -> the full path.
+        url = a_transport.base
+        while True:
+            try:
+                result = klass.open_from_transport(a_transport)
+                return result, urlutils.unescape(a_transport.relpath(url))
+            except errors.NotBranchError, e:
+                pass
+            except errors.PermissionDenied:
+                pass
+            try:
+                new_t = a_transport.clone('..')
+            except errors.InvalidURLJoin:
+                # reached the root, whatever that may be
+                raise errors.NotBranchError(path=url)
+            if new_t.base == a_transport.base:
+                # reached the root, whatever that may be
+                raise errors.NotBranchError(path=url)
+            a_transport = new_t
+
+    @classmethod
+    def open_tree_or_branch(klass, location):
+        """Return the branch and working tree at a location.
+
+        If there is no tree at the location, tree will be None.
+        If there is no branch at the location, an exception will be
+        raised
+        :return: (tree, branch)
+        """
+        controldir = klass.open(location)
+        return controldir._get_tree_branch()
+
+    @classmethod
+    def open_containing_tree_or_branch(klass, location):
+        """Return the branch and working tree contained by a location.
+
+        Returns (tree, branch, relpath).
+        If there is no tree at containing the location, tree will be None.
+        If there is no branch containing the location, an exception will be
+        raised
+        relpath is the portion of the path that is contained by the branch.
+        """
+        controldir, relpath = klass.open_containing(location)
+        tree, branch = controldir._get_tree_branch()
+        return tree, branch, relpath
+
+    @classmethod
+    def open_containing_tree_branch_or_repository(klass, location):
+        """Return the working tree, branch and repo contained by a location.
+
+        Returns (tree, branch, repository, relpath).
+        If there is no tree containing the location, tree will be None.
+        If there is no branch containing the location, branch will be None.
+        If there is no repository containing the location, repository will be
+        None.
+        relpath is the portion of the path that is contained by the innermost
+        ControlDir.
+
+        If no tree, branch or repository is found, a NotBranchError is raised.
+        """
+        controldir, relpath = klass.open_containing(location)
+        try:
+            tree, branch = controldir._get_tree_branch()
+        except errors.NotBranchError:
+            try:
+                repo = controldir.find_repository()
+                return None, None, repo, relpath
+            except (errors.NoRepositoryPresent):
+                raise errors.NotBranchError(location)
+        return tree, branch, branch.repository, relpath
+
+    @classmethod
+    def create(klass, base, format=None, possible_transports=None):
+        """Create a new ControlDir at the url 'base'.
+
+        :param format: If supplied, the format of branch to create.  If not
+            supplied, the default is used.
+        :param possible_transports: If supplied, a list of transports that
+            can be reused to share a remote connection.
+        """
+        if klass is not ControlDir:
+            raise AssertionError("ControlDir.create always creates the"
+                "default format, not one of %r" % klass)
+        t = _mod_transport.get_transport(base, possible_transports)
+        t.ensure_base()
+        if format is None:
+            format = ControlDirFormat.get_default_format()
+        return format.initialize_on_transport(t)
+
+
+class ControlDirHooks(hooks.Hooks):
+    """Hooks for ControlDir operations."""
+
+    def __init__(self):
+        """Create the default hooks."""
+        hooks.Hooks.__init__(self, "bzrlib.controldir", "ControlDir.hooks")
+        self.add_hook('pre_open',
+            "Invoked before attempting to open a ControlDir with the transport "
+            "that the open will use.", (1, 14))
+        self.add_hook('post_repo_init',
+            "Invoked after a repository has been initialized. "
+            "post_repo_init is called with a "
+            "bzrlib.controldir.RepoInitHookParams.",
+            (2, 2))
+
+# install the default hooks
+ControlDir.hooks = ControlDirHooks()
+
 
 class ControlComponentFormat(object):
-    """A component that can live inside of a .bzr meta directory."""
+    """A component that can live inside of a control directory."""
 
     upgrade_recommended = False
 
-    def get_format_string(self):
-        """Return the format of this format, if usable in meta directories."""
-        raise NotImplementedError(self.get_format_string)
-
     def get_format_description(self):
         """Return the short description for this format."""
         raise NotImplementedError(self.get_format_description)
@@ -523,6 +880,10 @@
             ui.ui_factory.recommend_upgrade(
                 self.get_format_description(), basedir)
 
+    @classmethod
+    def get_format_string(cls):
+        raise NotImplementedError(cls.get_format_string)
+
 
 class ControlComponentFormatRegistry(registry.FormatRegistry):
     """A registry for control components (branch, workingtree, repository)."""
@@ -679,12 +1040,17 @@
     def is_supported(self):
         """Is this format supported?
 
-        Supported formats must be initializable and openable.
+        Supported formats must be openable.
         Unsupported formats may not support initialization or committing or
         some other features depending on the reason for not being supported.
         """
         return True
 
+    def is_initializable(self):
+        """Whether new control directories of this format can be initialized.
+        """
+        return self.is_supported()
+
     def check_support_status(self, allow_unsupported, recommend_upgrade=True,
         basedir=None):
         """Give an error or warning on old formats.
@@ -746,22 +1112,24 @@
         return self.get_format_description().rstrip()
 
     @classmethod
+    def all_probers(klass):
+        return klass._server_probers + klass._probers
+
+    @classmethod
     def known_formats(klass):
         """Return all the known formats.
         """
         result = set()
-        for prober_kls in klass._probers + klass._server_probers:
+        for prober_kls in klass.all_probers():
             result.update(prober_kls.known_formats())
         return result
 
     @classmethod
-    def find_format(klass, transport, _server_formats=True):
+    def find_format(klass, transport, probers=None):
         """Return the format present at transport."""
-        if _server_formats:
-            _probers = klass._server_probers + klass._probers
-        else:
-            _probers = klass._probers
-        for prober_kls in _probers:
+        if probers is None:
+            probers = klass.all_probers()
+        for prober_kls in probers:
             prober = prober_kls()
             try:
                 return prober.probe_transport(transport)
@@ -846,6 +1214,11 @@
         """Return the current default format."""
         return klass._default_format
 
+    def supports_transport(self, transport):
+        """Check if this format can be opened over a particular transport.
+        """
+        raise NotImplementedError(self.supports_transport)
+
 
 class Prober(object):
     """Abstract class that can be used to detect a particular kind of
@@ -874,7 +1247,7 @@
         raise NotImplementedError(self.probe_transport)
 
     @classmethod
-    def known_formats(cls):
+    def known_formats(klass):
         """Return the control dir formats known by this prober.
 
         Multiple probers can return the same formats, so this should
@@ -882,7 +1255,7 @@
 
         :return: A set of known formats.
         """
-        raise NotImplementedError(cls.known_formats)
+        raise NotImplementedError(klass.known_formats)
 
 
 class ControlDirFormatInfo(object):
@@ -1021,6 +1394,42 @@
             return output
 
 
+class RepoInitHookParams(object):
+    """Object holding parameters passed to `*_repo_init` hooks.
+
+    There are 4 fields that hooks may wish to access:
+
+    :ivar repository: Repository created
+    :ivar format: Repository format
+    :ivar bzrdir: The controldir for the repository
+    :ivar shared: The repository is shared
+    """
+
+    def __init__(self, repository, format, controldir, shared):
+        """Create a group of RepoInitHook parameters.
+
+        :param repository: Repository created
+        :param format: Repository format
+        :param controldir: The controldir for the repository
+        :param shared: The repository is shared
+        """
+        self.repository = repository
+        self.format = format
+        self.bzrdir = controldir
+        self.shared = shared
+
+    def __eq__(self, other):
+        return self.__dict__ == other.__dict__
+
+    def __repr__(self):
+        if self.repository:
+            return "<%s for %s>" % (self.__class__.__name__,
+                self.repository)
+        else:
+            return "<%s for %s>" % (self.__class__.__name__,
+                self.bzrdir)
+
+
 # Please register new formats after old formats so that formats
 # appear in chronological order and format descriptions can build
 # on previous ones.

=== modified file 'bzrlib/counted_lock.py'
--- bzrlib/counted_lock.py	2009-07-27 06:28:35 +0000
+++ bzrlib/counted_lock.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,7 @@
 
 """Counted lock class"""
 
+from __future__ import absolute_import
 
 from bzrlib import (
     errors,

=== modified file 'bzrlib/crash.py'
--- bzrlib/crash.py	2011-05-20 14:46:02 +0000
+++ bzrlib/crash.py	2011-12-19 13:23:58 +0000
@@ -14,7 +14,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-
 """Handling and reporting crashes.
 
 A crash is an exception propagated up almost to the top level of Bazaar.
@@ -37,6 +36,8 @@
 -Dno_apport.
 """
 
+from __future__ import absolute_import
+
 # for interactive testing, try the 'bzr assert-fail' command 
 # or see http://code.launchpad.net/~mbp/bzr/bzr-fail
 #
@@ -70,13 +71,11 @@
             return
     except ImportError, e:
         trace.mutter("couldn't find apport bug-reporting library: %s" % e)
-        pass
     except Exception, e:
         # this should only happen if apport is installed but it didn't
         # work, eg because of an io error writing the crash file
-        stderr.write("bzr: failed to report crash using apport:\n "
-            "    %r\n" % e)
-        pass
+        trace.mutter("bzr: failed to report crash using apport: %r" % e)
+        trace.log_exception_quietly()
     return report_bug_legacy(exc_info, stderr)
 
 

=== modified file 'bzrlib/debug.py'
--- bzrlib/debug.py	2011-08-20 07:49:15 +0000
+++ bzrlib/debug.py	2011-12-19 13:23:58 +0000
@@ -14,7 +14,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-
 """Set of flags that enable different debug behaviour.
 
 These are set with eg ``-Dlock`` on the bzr command line or in
@@ -24,6 +23,8 @@
 for a list of the available options.
 """
 
+from __future__ import absolute_import
+
 debug_flags = set()
 
 

=== modified file 'bzrlib/decorators.py'
--- bzrlib/decorators.py	2011-05-26 08:05:45 +0000
+++ bzrlib/decorators.py	2011-12-18 12:46:49 +0000
@@ -14,6 +14,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
 
 __all__ = ['needs_read_lock',
            'needs_write_lock',

=== modified file 'bzrlib/delta.py'
--- bzrlib/delta.py	2011-06-01 12:53:56 +0000
+++ bzrlib/delta.py	2011-12-18 12:46:49 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 from bzrlib import (
     osutils,
     )

=== modified file 'bzrlib/diff-delta.c'
--- bzrlib/diff-delta.c	2011-05-16 14:26:54 +0000
+++ bzrlib/diff-delta.c	2011-09-30 16:29:17 +0000
@@ -720,6 +720,11 @@
 
     max_num_entries = (src->size - 1)  / RABIN_WINDOW;
 
+    if (!max_num_entries) {
+        *fresh = old_index;
+        return DELTA_OK;
+    }
+
     /* allocate an array to hold whatever entries we find */
     entries = malloc(sizeof(*entry) * max_num_entries);
     if (!entries) /* malloc failure */

=== modified file 'bzrlib/diff.py'
--- bzrlib/diff.py	2011-07-15 15:31:01 +0000
+++ bzrlib/diff.py	2011-12-18 12:46:49 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 import difflib
 import os
 import re
@@ -27,9 +29,9 @@
 import tempfile
 
 from bzrlib import (
-    bzrdir,
+    cleanup,
     cmdline,
-    cleanup,
+    controldir,
     errors,
     osutils,
     patiencediff,
@@ -39,6 +41,7 @@
     )
 
 from bzrlib.workingtree import WorkingTree
+from bzrlib.i18n import gettext
 """)
 
 from bzrlib.registry import (
@@ -355,7 +358,7 @@
     if old_url is None:
         old_url = default_location
     working_tree, branch, relpath = \
-        bzrdir.BzrDir.open_containing_tree_or_branch(old_url)
+        controldir.ControlDir.open_containing_tree_or_branch(old_url)
     lock_tree_or_branch(working_tree, branch)
     if consider_relpath and relpath != '':
         if working_tree is not None and apply_view:
@@ -369,7 +372,7 @@
         new_url = default_location
     if new_url != old_url:
         working_tree, branch, relpath = \
-            bzrdir.BzrDir.open_containing_tree_or_branch(new_url)
+            controldir.ControlDir.open_containing_tree_or_branch(new_url)
         lock_tree_or_branch(working_tree, branch)
         if consider_relpath and relpath != '':
             if working_tree is not None and apply_view:
@@ -393,7 +396,7 @@
             if view_files:
                 specific_files = view_files
                 view_str = views.view_display_str(view_files)
-                note("*** Ignoring files outside view. View is %s" % view_str)
+                note(gettext("*** Ignoring files outside view. View is %s") % view_str)
 
     # Get extra trees that ought to be searched for file-ids
     extra_trees = None

=== modified file 'bzrlib/directory_service.py'
--- bzrlib/directory_service.py	2011-04-07 10:36:24 +0000
+++ bzrlib/directory_service.py	2011-12-19 13:23:58 +0000
@@ -20,6 +20,8 @@
 to true URLs.  Examples include lp:urls and per-user location aliases.
 """
 
+from __future__ import absolute_import
+
 from bzrlib import (
     errors,
     registry,
@@ -64,7 +66,6 @@
 
 directories = DirectoryServiceRegistry()
 
-
 class AliasDirectory(object):
     """Directory lookup for locations associated with a branch.
 
@@ -72,16 +73,22 @@
     supported.  On error, a subclass of DirectoryLookupFailure will be raised.
     """
 
+    branch_aliases = registry.Registry()
+    branch_aliases.register('parent', lambda b: b.get_parent(),
+        help="The parent of this branch.")
+    branch_aliases.register('submit', lambda b: b.get_submit_branch(),
+        help="The submit branch for this branch.")
+    branch_aliases.register('public', lambda b: b.get_public_branch(),
+        help="The public location of this branch.")
+    branch_aliases.register('bound', lambda b: b.get_bound_location(),
+        help="The branch this branch is bound to, for bound branches.")
+    branch_aliases.register('push', lambda b: b.get_push_location(),
+        help="The saved location used for `bzr push` with no arguments.")
+    branch_aliases.register('this', lambda b: b.base,
+        help="This branch.")
+
     def look_up(self, name, url):
         branch = _mod_branch.Branch.open_containing('.')[0]
-        lookups = {
-            'parent': branch.get_parent,
-            'submit': branch.get_submit_branch,
-            'public': branch.get_public_branch,
-            'bound': branch.get_bound_location,
-            'push': branch.get_push_location,
-            'this': lambda: branch.base
-        }
         parts = url.split('/', 1)
         if len(parts) == 2:
             name, extra = parts
@@ -89,16 +96,38 @@
             (name,) = parts
             extra = None
         try:
-            method = lookups[name[1:]]
+            method = self.branch_aliases.get(name[1:])
         except KeyError:
             raise errors.InvalidLocationAlias(url)
         else:
-            result = method()
+            result = method(branch)
         if result is None:
             raise errors.UnsetLocationAlias(url)
         if extra is not None:
             result = urlutils.join(result, extra)
         return result
 
+    @classmethod
+    def help_text(cls, topic):
+        alias_lines = []
+        for key in cls.branch_aliases.keys():
+            help = cls.branch_aliases.get_help(key)
+            alias_lines.append("  :%-10s%s\n" % (key, help))
+        return """\
+Location aliases
+================
+
+Bazaar defines several aliases for locations associated with a branch.  These
+can be used with most commands that expect a location, such as `bzr push`.
+
+The aliases are::
+
+%s
+For example, to push to the parent location::
+
+    bzr push :parent
+""" % "".join(alias_lines)
+
+
 directories.register(':', AliasDirectory,
                      'Easy access to remembered branch locations')

=== modified file 'bzrlib/dirstate.py'
--- bzrlib/dirstate.py	2011-07-25 07:11:56 +0000
+++ bzrlib/dirstate.py	2011-12-21 00:07:49 +0000
@@ -218,14 +218,14 @@
 
 """
 
+from __future__ import absolute_import
+
 import bisect
-import binascii
 import errno
 import operator
 import os
 from stat import S_IEXEC
 import stat
-import struct
 import sys
 import time
 import zlib
@@ -251,50 +251,6 @@
 ERROR_DIRECTORY = 267
 
 
-if not getattr(struct, '_compile', None):
-    # Cannot pre-compile the dirstate pack_stat
-    def pack_stat(st, _encode=binascii.b2a_base64, _pack=struct.pack):
-        """Convert stat values into a packed representation."""
-        return _encode(_pack('>LLLLLL', st.st_size, int(st.st_mtime),
-            int(st.st_ctime), st.st_dev, st.st_ino & 0xFFFFFFFF,
-            st.st_mode))[:-1]
-else:
-    # compile the struct compiler we need, so as to only do it once
-    from _struct import Struct
-    _compiled_pack = Struct('>LLLLLL').pack
-    def pack_stat(st, _encode=binascii.b2a_base64, _pack=_compiled_pack):
-        """Convert stat values into a packed representation."""
-        # jam 20060614 it isn't really worth removing more entries if we
-        # are going to leave it in packed form.
-        # With only st_mtime and st_mode filesize is 5.5M and read time is 275ms
-        # With all entries, filesize is 5.9M and read time is maybe 280ms
-        # well within the noise margin
-
-        # base64 encoding always adds a final newline, so strip it off
-        # The current version
-        return _encode(_pack(st.st_size, int(st.st_mtime), int(st.st_ctime),
-            st.st_dev, st.st_ino & 0xFFFFFFFF, st.st_mode))[:-1]
-        # This is 0.060s / 1.520s faster by not encoding as much information
-        # return _encode(_pack('>LL', int(st.st_mtime), st.st_mode))[:-1]
-        # This is not strictly faster than _encode(_pack())[:-1]
-        # return '%X.%X.%X.%X.%X.%X' % (
-        #      st.st_size, int(st.st_mtime), int(st.st_ctime),
-        #      st.st_dev, st.st_ino, st.st_mode)
-        # Similar to the _encode(_pack('>LL'))
-        # return '%X.%X' % (int(st.st_mtime), st.st_mode)
-
-
-def _unpack_stat(packed_stat):
-    """Turn a packed_stat back into the stat fields.
-
-    This is meant as a debugging tool, should not be used in real code.
-    """
-    (st_size, st_mtime, st_ctime, st_dev, st_ino,
-     st_mode) = struct.unpack('>LLLLLL', binascii.a2b_base64(packed_stat))
-    return dict(st_size=st_size, st_mtime=st_mtime, st_ctime=st_ctime,
-                st_dev=st_dev, st_ino=st_ino, st_mode=st_mode)
-
-
 class SHA1Provider(object):
     """An interface for getting sha1s of a file."""
 
@@ -1605,16 +1561,19 @@
                     else:
                         source_path = child_basename
                     if new_path_utf8:
-                        target_path = new_path_utf8 + source_path[len(old_path):]
+                        target_path = \
+                            new_path_utf8 + source_path[len(old_path_utf8):]
                     else:
-                        if old_path == '':
+                        if old_path_utf8 == '':
                             raise AssertionError("cannot rename directory to"
                                                  " itself")
-                        target_path = source_path[len(old_path) + 1:]
+                        target_path = source_path[len(old_path_utf8) + 1:]
                     adds.append((None, target_path, entry[0][2], entry[1][1], False))
                     deletes.append(
                         (source_path, target_path, entry[0][2], None, False))
-                deletes.append((old_path_utf8, new_path, file_id, None, False))
+                deletes.append(
+                    (old_path_utf8, new_path_utf8, file_id, None, False))
+
         self._check_delta_ids_absent(new_ids, delta, 1)
         try:
             # Finish expunging deletes/first half of renames.
@@ -1896,7 +1855,7 @@
                     file_id, "This parent is not a directory.")
 
     def _observed_sha1(self, entry, sha1, stat_value,
-        _stat_to_minikind=_stat_to_minikind, _pack_stat=pack_stat):
+        _stat_to_minikind=_stat_to_minikind):
         """Note the sha1 of a file.
 
         :param entry: The entry the sha1 is for.
@@ -1908,14 +1867,13 @@
         except KeyError:
             # Unhandled kind
             return None
-        packed_stat = _pack_stat(stat_value)
         if minikind == 'f':
             if self._cutoff_time is None:
                 self._sha_cutoff_time()
             if (stat_value.st_mtime < self._cutoff_time
                 and stat_value.st_ctime < self._cutoff_time):
                 entry[1][0] = ('f', sha1, stat_value.st_size, entry[1][0][3],
-                               packed_stat)
+                               pack_stat(stat_value))
                 self._mark_modified([entry])
 
     def _sha_cutoff_time(self):
@@ -2474,9 +2432,9 @@
             raise errors.BzrError('missing num_entries line')
         self._num_entries = int(num_entries_line[len('num_entries: '):-1])
 
-    def sha1_from_stat(self, path, stat_result, _pack_stat=pack_stat):
+    def sha1_from_stat(self, path, stat_result):
         """Find a sha1 given a stat lookup."""
-        return self._get_packed_stat_index().get(_pack_stat(stat_result), None)
+        return self._get_packed_stat_index().get(pack_stat(stat_result), None)
 
     def _get_packed_stat_index(self):
         """Get a packed_stat index of self._dirblocks."""
@@ -3397,8 +3355,7 @@
 
 
 def py_update_entry(state, entry, abspath, stat_value,
-                 _stat_to_minikind=DirState._stat_to_minikind,
-                 _pack_stat=pack_stat):
+                 _stat_to_minikind=DirState._stat_to_minikind):
     """Update the entry based on what is actually on disk.
 
     This function only calculates the sha if it needs to - if the entry is
@@ -3417,7 +3374,7 @@
     except KeyError:
         # Unhandled kind
         return None
-    packed_stat = _pack_stat(stat_value)
+    packed_stat = pack_stat(stat_value)
     (saved_minikind, saved_link_or_sha1, saved_file_size,
      saved_executable, saved_packed_stat) = entry[1][0]
 
@@ -4296,6 +4253,7 @@
         _bisect_path_left,
         _bisect_path_right,
         cmp_by_dirs,
+        pack_stat,
         ProcessEntryC as _process_entry,
         update_entry as update_entry,
         )
@@ -4307,6 +4265,7 @@
         _bisect_path_left,
         _bisect_path_right,
         cmp_by_dirs,
+        pack_stat,
         )
     # FIXME: It would be nice to be able to track moved lines so that the
     # corresponding python code can be moved to the _dirstate_helpers_py

=== modified file 'bzrlib/doc/__init__.py'
--- bzrlib/doc/__init__.py	2009-03-23 14:59:43 +0000
+++ bzrlib/doc/__init__.py	2011-12-19 13:23:58 +0000
@@ -20,6 +20,8 @@
 for man page generation.
 """
 
+from __future__ import absolute_import
+
 
 def load_tests(basic_tests, module, loader):
     suite = loader.suiteClass()

=== modified file 'bzrlib/doc/api/__init__.py'
--- bzrlib/doc/api/__init__.py	2011-05-18 16:42:48 +0000
+++ bzrlib/doc/api/__init__.py	2011-12-19 13:23:58 +0000
@@ -24,6 +24,8 @@
 For details on specific apis, see pydoc on the api, or read the source.
 """
 
+from __future__ import absolute_import
+
 import doctest
 import os
 

=== modified file 'bzrlib/doc/api/branch.txt'
--- bzrlib/doc/api/branch.txt	2006-10-05 22:42:09 +0000
+++ bzrlib/doc/api/branch.txt	2011-10-05 18:11:19 +0000
@@ -25,8 +25,8 @@
 
 A branch has a history of revisions on it:
 
-   >>> new_branch.revision_history()
-   []
+   >>> new_branch.last_revision()
+   'null:'
 
 We need to write some more documentation, showing
 push and pull operations. Cloning might also be nice.

=== modified file 'bzrlib/doc_generate/__init__.py'
--- bzrlib/doc_generate/__init__.py	2009-06-19 10:00:56 +0000
+++ bzrlib/doc_generate/__init__.py	2011-12-18 15:28:38 +0000
@@ -14,8 +14,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-
-import sys
+from __future__ import absolute_import
 
 
 def get_module(target):

=== modified file 'bzrlib/doc_generate/autodoc_bash_completion.py'
--- bzrlib/doc_generate/autodoc_bash_completion.py	2009-06-19 10:00:56 +0000
+++ bzrlib/doc_generate/autodoc_bash_completion.py	2011-12-19 13:23:58 +0000
@@ -16,8 +16,8 @@
 
 "bash_completion.py - create bash completion script from built-in bzr help"
 
-import os
-import sys
+from __future__ import absolute_import
+
 import time
 
 import bzrlib

=== modified file 'bzrlib/doc_generate/autodoc_man.py'
--- bzrlib/doc_generate/autodoc_man.py	2011-07-22 00:03:11 +0000
+++ bzrlib/doc_generate/autodoc_man.py	2011-12-19 13:23:58 +0000
@@ -21,8 +21,8 @@
   * add command aliases
 """
 
-import os
-import sys
+from __future__ import absolute_import
+
 import textwrap
 import time
 

=== modified file 'bzrlib/doc_generate/autodoc_rstx.py'
--- bzrlib/doc_generate/autodoc_rstx.py	2010-02-17 17:11:16 +0000
+++ bzrlib/doc_generate/autodoc_rstx.py	2011-12-19 13:23:58 +0000
@@ -20,8 +20,8 @@
 Written by the Bazaar community.
 """
 
-import os
-import sys
+from __future__ import absolute_import
+
 import time
 
 import bzrlib

=== modified file 'bzrlib/doc_generate/builders/__init__.py'
--- bzrlib/doc_generate/builders/__init__.py	2010-07-05 16:01:33 +0000
+++ bzrlib/doc_generate/builders/__init__.py	2011-12-19 13:23:58 +0000
@@ -16,3 +16,5 @@
 
 """Placeholder to allow sphinx builders imports."""
 
+from __future__ import absolute_import
+

=== modified file 'bzrlib/doc_generate/builders/texinfo.py'
--- bzrlib/doc_generate/builders/texinfo.py	2010-05-06 20:05:01 +0000
+++ bzrlib/doc_generate/builders/texinfo.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,8 @@
 
 """A sphinx builder producing texinfo output."""
 
+from __future__ import absolute_import
+
 from sphinx import builders
 from sphinx.builders import text as _text_builder
 

=== modified file 'bzrlib/doc_generate/conf.py'
--- bzrlib/doc_generate/conf.py	2011-05-16 10:19:33 +0000
+++ bzrlib/doc_generate/conf.py	2011-12-18 15:28:38 +0000
@@ -6,7 +6,7 @@
 # All configuration values have a default; values that are commented out
 # serve to show the default.
 
-import sys, os
+from __future__ import absolute_import
 
 # FIXME: better move the content of doc/en/conf.py here and cleanup the result
 # -- vila 20100428

=== modified file 'bzrlib/doc_generate/writers/__init__.py'
--- bzrlib/doc_generate/writers/__init__.py	2010-07-05 16:01:33 +0000
+++ bzrlib/doc_generate/writers/__init__.py	2011-12-19 13:23:58 +0000
@@ -16,3 +16,5 @@
 
 """Placeholder to allow sphinx writer imports."""
 
+from __future__ import absolute_import
+

=== modified file 'bzrlib/doc_generate/writers/texinfo.py'
--- bzrlib/doc_generate/writers/texinfo.py	2011-03-11 12:24:10 +0000
+++ bzrlib/doc_generate/writers/texinfo.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,8 @@
 
 """A sphinx/docutil writer producing texinfo output."""
 
+from __future__ import absolute_import
+
 from docutils import (
     nodes,
     writers,

=== modified file 'bzrlib/email_message.py'
--- bzrlib/email_message.py	2011-05-19 09:32:38 +0000
+++ bzrlib/email_message.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,8 @@
 
 """A convenience class around email.Message and email.MIMEMultipart."""
 
+from __future__ import absolute_import
+
 from email import (
     Header,
     Message,

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2011-08-21 13:49:30 +0000
+++ bzrlib/errors.py	2011-12-22 15:33:16 +0000
@@ -17,18 +17,7 @@
 """Exceptions for bzr, and reporting of them.
 """
 
-from bzrlib import (
-    osutils,
-    symbol_versioning,
-    )
-from bzrlib.patches import (
-    MalformedHunkHeader,
-    MalformedLine,
-    MalformedPatchHeader,
-    PatchConflict,
-    PatchSyntax,
-    )
-
+from __future__ import absolute_import
 
 # TODO: is there any value in providing the .args field used by standard
 # python exceptions?   A list of values with no names seems less useful
@@ -106,12 +95,15 @@
                 # __str__() should always return a 'str' object
                 # never a 'unicode' object.
                 return s
-        except (AttributeError, TypeError, NameError, ValueError, KeyError), e:
-            return 'Unprintable exception %s: dict=%r, fmt=%r, error=%r' \
-                % (self.__class__.__name__,
-                   self.__dict__,
-                   getattr(self, '_fmt', None),
-                   e)
+        except Exception, e:
+            pass # just bind to 'e' for formatting below
+        else:
+            e = None
+        return 'Unprintable exception %s: dict=%r, fmt=%r, error=%r' \
+            % (self.__class__.__name__,
+               self.__dict__,
+               getattr(self, '_fmt', None),
+               e)
 
     def __unicode__(self):
         u = self._format()
@@ -140,18 +132,8 @@
         """Return format string for this exception or None"""
         fmt = getattr(self, '_fmt', None)
         if fmt is not None:
-            return fmt
-        fmt = getattr(self, '__doc__', None)
-        if fmt is not None:
-            symbol_versioning.warn("%s uses its docstring as a format, "
-                    "it should use _fmt instead" % self.__class__.__name__,
-                    DeprecationWarning)
-            return fmt
-        return 'Unprintable exception %s: dict=%r, fmt=%r' \
-            % (self.__class__.__name__,
-               self.__dict__,
-               getattr(self, '_fmt', None),
-               )
+            from bzrlib.i18n import gettext
+            return gettext(unicode(fmt)) # _fmt strings should be ascii
 
     def __eq__(self, other):
         if self.__class__ is not other.__class__:
@@ -170,39 +152,6 @@
     internal_error = True
 
 
-class BzrNewError(BzrError):
-    """Deprecated error base class."""
-    # base classes should override the docstring with their human-
-    # readable explanation
-
-    def __init__(self, *args, **kwds):
-        # XXX: Use the underlying BzrError to always generate the args
-        # attribute if it doesn't exist.  We can't use super here, because
-        # exceptions are old-style classes in python2.4 (but new in 2.5).
-        # --bmc, 20060426
-        symbol_versioning.warn('BzrNewError was deprecated in bzr 0.13; '
-             'please convert %s to use BzrError instead'
-             % self.__class__.__name__,
-             DeprecationWarning,
-             stacklevel=2)
-        BzrError.__init__(self, *args)
-        for key, value in kwds.items():
-            setattr(self, key, value)
-
-    def __str__(self):
-        try:
-            # __str__() should always return a 'str' object
-            # never a 'unicode' object.
-            s = self.__doc__ % self.__dict__
-            if isinstance(s, unicode):
-                return s.encode('utf8')
-            return s
-        except (TypeError, NameError, ValueError, KeyError), e:
-            return 'Unprintable exception %s(%r): %r' \
-                % (self.__class__.__name__,
-                   self.__dict__, e)
-
-
 class AlreadyBuilding(BzrError):
 
     _fmt = "The tree builder is already building a tree."
@@ -790,19 +739,6 @@
         self.path = bzrdir.transport.clone('..').base
 
 
-class FileInWrongBranch(BzrError):
-
-    _fmt = 'File "%(path)s" is not in branch %(branch_base)s.'
-
-    # use PathNotChild instead
-    @symbol_versioning.deprecated_method(symbol_versioning.deprecated_in((2, 3, 0)))
-    def __init__(self, branch, path):
-        BzrError.__init__(self)
-        self.branch = branch
-        self.branch_base = branch.base
-        self.path = path
-
-
 class UnsupportedFormatError(BzrError):
 
     _fmt = "Unsupported branch format: %(format)s\nPlease run 'bzr upgrade'"
@@ -827,6 +763,18 @@
         self.bzrdir = bzrdir_format
 
 
+class ParseFormatError(BzrError):
+
+    _fmt = "Parse error on line %(lineno)d of %(format)s format: %(line)s"
+
+    def __init__(self, format, lineno, line, text):
+        BzrError.__init__(self)
+        self.format = format
+        self.lineno = lineno
+        self.line = line
+        self.text = text
+
+
 class IncompatibleRepositories(BzrError):
     """Report an error that two repositories are not compatible.
 
@@ -1221,11 +1169,6 @@
             self.extra = ''
 
 
-class HistoryMissing(BzrError):
-
-    _fmt = "%(branch)s is missing %(object_type)s {%(object_id)s}"
-
-
 class AppendRevisionsOnlyViolation(BzrError):
 
     _fmt = ('Operation denied because it would change the main history,'
@@ -1296,17 +1239,6 @@
             not_ancestor_id=not_ancestor_id)
 
 
-class AmbiguousBase(BzrError):
-
-    def __init__(self, bases):
-        symbol_versioning.warn("BzrError AmbiguousBase has been deprecated "
-            "as of bzrlib 0.8.", DeprecationWarning, stacklevel=2)
-        msg = ("The correct base is unclear, because %s are all equally close"
-                % ", ".join(bases))
-        BzrError.__init__(self, msg)
-        self.bases = bases
-
-
 class NoCommits(BranchError):
 
     _fmt = "Branch %(branch)s has no commits."
@@ -1572,6 +1504,7 @@
             problem we can raise the original error (value from sys.exc_info())
         """
         BzrError.__init__(self)
+        self.context = context
         self.reload_occurred = reload_occurred
         self.exc_info = exc_info
         self.orig_error = exc_info[1]
@@ -1659,6 +1592,7 @@
 
     def __init__(self, exc_info):
         import traceback
+        # GZ 2010-08-10: Cycle with exc_tb/exc_info affects at least one test
         self.exc_type, self.exc_value, self.exc_tb = exc_info
         self.exc_info = exc_info
         traceback_strings = traceback.format_exception(
@@ -1703,6 +1637,11 @@
     _fmt = "Connection closed: %(msg)s %(orig_error)s"
 
 
+class ConnectionTimeout(ConnectionError):
+
+    _fmt = "Connection Timeout: %(msg)s%(orig_error)s"
+
+
 class InvalidRange(TransportError):
 
     _fmt = "Invalid range access in %(path)s at %(offset)s: %(msg)s"
@@ -1972,7 +1911,7 @@
         self.prefix = prefix
 
 
-class MalformedTransform(BzrError):
+class MalformedTransform(InternalBzrError):
 
     _fmt = "Tree transform is malformed %(conflicts)r"
 
@@ -2072,22 +2011,6 @@
         BzrMoveFailedError.__init__(self, from_path, to_path, extra)
 
 
-class BzrRemoveChangedFilesError(BzrError):
-    """Used when user is trying to remove changed files."""
-
-    _fmt = ("Can't safely remove modified or unknown files:\n"
-        "%(changes_as_text)s"
-        "Use --keep to not delete them, or --force to delete them regardless.")
-
-    def __init__(self, tree_delta):
-        symbol_versioning.warn(symbol_versioning.deprecated_in((2, 3, 0)) %
-            "BzrRemoveChangedFilesError", DeprecationWarning, stacklevel=2)
-        BzrError.__init__(self)
-        self.changes_as_text = tree_delta.get_changes_as_text()
-        #self.paths_as_string = '\n'.join(changed_files)
-        #self.paths_as_string = '\n'.join([quotefn(p) for p in changed_files])
-
-
 class BzrBadParameterNotString(BzrBadParameter):
 
     _fmt = "Parameter %(param)s is not a string or unicode string."
@@ -2349,6 +2272,14 @@
     """
 
 
+class GhostTagsNotSupported(BzrError):
+
+    _fmt = "Ghost tags not supported by format %(format)r."
+
+    def __init__(self, format):
+        self.format = format
+
+
 class BinaryFile(BzrError):
 
     _fmt = "File is binary but should be text."
@@ -2784,7 +2715,7 @@
     _fmt = "Container has multiple records with the same name: %(name)s"
 
     def __init__(self, name):
-        self.name = name
+        self.name = name.decode("utf-8")
 
 
 class NoDestinationAddress(InternalBzrError):
@@ -3019,7 +2950,7 @@
         from bzrlib.osutils import get_user_encoding
         self.path = path
         self.kind = kind
-        self.user_encoding = osutils.get_user_encoding()
+        self.user_encoding = get_user_encoding()
 
 
 class NoSuchConfig(BzrError):
@@ -3086,31 +3017,6 @@
         BzrError.__init__(self, unknowns_str=", ".join(unknowns))
 
 
-class HookFailed(BzrError):
-    """Raised when a pre_change_branch_tip hook function fails anything other
-    than TipChangeRejected.
-
-    Note that this exception is no longer raised, and the import is only left
-    to be nice to code which might catch it in a plugin.
-    """
-
-    _fmt = ("Hook '%(hook_name)s' during %(hook_stage)s failed:\n"
-            "%(traceback_text)s%(exc_value)s")
-
-    def __init__(self, hook_stage, hook_name, exc_info, warn=True):
-        if warn:
-            symbol_versioning.warn("BzrError HookFailed has been deprecated "
-                "as of bzrlib 2.1.", DeprecationWarning, stacklevel=2)
-        import traceback
-        self.hook_stage = hook_stage
-        self.hook_name = hook_name
-        self.exc_info = exc_info
-        self.exc_type = exc_info[0]
-        self.exc_value = exc_info[1]
-        self.exc_tb = exc_info[2]
-        self.traceback_text = ''.join(traceback.format_tb(self.exc_tb))
-
-
 class TipChangeRejected(BzrError):
     """A pre_change_branch_tip hook function may raise this to cleanly and
     explicitly abort a change to a branch tip.
@@ -3334,3 +3240,83 @@
     def __init__(self, method, arguments):
         self.method = method
         self.arguments = arguments
+
+
+class UnsupportedKindChange(BzrError):
+
+    _fmt = ("Kind change from %(from_kind)s to %(to_kind)s for "
+            "%(path)s not supported by format %(format)r")
+
+    def __init__(self, path, from_kind, to_kind, format):
+        self.path = path
+        self.from_kind = from_kind
+        self.to_kind = to_kind
+        self.format = format
+
+
+class MissingFeature(BzrError):
+
+    _fmt = ("Missing feature %(feature)s not provided by this "
+            "version of Bazaar or any plugin.")
+
+    def __init__(self, feature):
+        self.feature = feature
+
+
+class PatchSyntax(BzrError):
+    """Base class for patch syntax errors."""
+
+
+class BinaryFiles(BzrError):
+
+    _fmt = 'Binary files section encountered.'
+
+    def __init__(self, orig_name, mod_name):
+        self.orig_name = orig_name
+        self.mod_name = mod_name
+
+
+class MalformedPatchHeader(PatchSyntax):
+
+    _fmt = "Malformed patch header.  %(desc)s\n%(line)r"
+
+    def __init__(self, desc, line):
+        self.desc = desc
+        self.line = line
+
+
+class MalformedHunkHeader(PatchSyntax):
+
+    _fmt = "Malformed hunk header.  %(desc)s\n%(line)r"
+
+    def __init__(self, desc, line):
+        self.desc = desc
+        self.line = line
+
+
+class MalformedLine(PatchSyntax):
+
+    _fmt = "Malformed line.  %(desc)s\n%(line)r"
+
+    def __init__(self, desc, line):
+        self.desc = desc
+        self.line = line
+
+
+class PatchConflict(BzrError):
+
+    _fmt = ('Text contents mismatch at line %(line_no)d.  Original has '
+            '"%(orig_line)s", but patch says it should be "%(patch_line)s"')
+
+    def __init__(self, line_no, orig_line, patch_line):
+        self.line_no = line_no
+        self.orig_line = orig_line.rstrip('\n')
+        self.patch_line = patch_line.rstrip('\n')
+
+
+class FeatureAlreadyRegistered(BzrError):
+
+    _fmt = 'The feature %(feature)s has already been registered.'
+
+    def __init__(self, feature):
+        self.feature = feature

=== added file 'bzrlib/estimate_compressed_size.py'
--- bzrlib/estimate_compressed_size.py	1970-01-01 00:00:00 +0000
+++ bzrlib/estimate_compressed_size.py	2011-12-19 13:23:58 +0000
@@ -0,0 +1,70 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Code to estimate the entropy of content"""
+
+from __future__ import absolute_import
+
+import zlib
+
+
+class ZLibEstimator(object):
+    """Uses zlib.compressobj to estimate compressed size."""
+
+    def __init__(self, target_size, min_compression=2.0):
+        """Create a new estimator.
+
+        :param target_size: The desired size of the compressed content.
+        :param min_compression: Estimated minimum compression. By default we
+            assume that the content is 'text', which means a min compression of
+            about 2:1.
+        """
+        self._target_size = target_size
+        self._compressor = zlib.compressobj()
+        self._uncompressed_size_added = 0
+        self._compressed_size_added = 0
+        self._unflushed_size_added = 0
+        self._estimated_compression = 2.0
+
+    def add_content(self, content):
+        self._uncompressed_size_added += len(content)
+        self._unflushed_size_added += len(content)
+        z_size = len(self._compressor.compress(content))
+        if z_size > 0:
+            self._record_z_len(z_size)
+
+    def _record_z_len(self, count):
+        # We got some compressed bytes, update the counters
+        self._compressed_size_added += count
+        self._unflushed_size_added = 0
+        # So far we've read X uncompressed bytes, and written Y compressed
+        # bytes. We should have a decent estimate of the final compression.
+        self._estimated_compression = (float(self._uncompressed_size_added)
+            / self._compressed_size_added)
+
+    def full(self):
+        """Have we reached the target size?"""
+        if self._unflushed_size_added:
+            remaining_size = self._target_size - self._compressed_size_added
+            # Estimate how much compressed content the unflushed data will
+            # consume
+            est_z_size = (self._unflushed_size_added /
+                          self._estimated_compression)
+            if est_z_size >= remaining_size:
+                # We estimate we are close to remaining
+                z_size = len(self._compressor.flush(zlib.Z_SYNC_FLUSH))
+                self._record_z_len(z_size)
+        return self._compressed_size_added >= self._target_size

=== modified file 'bzrlib/export/__init__.py'
--- bzrlib/export/__init__.py	2011-07-08 23:38:46 +0000
+++ bzrlib/export/__init__.py	2011-12-19 13:23:58 +0000
@@ -17,6 +17,8 @@
 """Export trees to tarballs, non-controlled directories, zipfiles, etc.
 """
 
+from __future__ import absolute_import
+
 import os
 import time
 import warnings
@@ -195,6 +197,8 @@
     :param tree: A tree object.
     :param subdir: None or the path of an entry to start exporting from.
     :param skip_special: Whether to skip .bzr files.
+    :return: iterator over tuples with final path, tree path and inventory
+        entry for each entry to export
     """
     if subdir == '':
         subdir = None
@@ -221,7 +225,7 @@
         if not tree.has_filename(path):
             continue
 
-        yield final_path, entry
+        yield final_path, path, entry
 
 
 register_lazy_exporter(None, [], 'bzrlib.export.dir_exporter',

=== modified file 'bzrlib/export/dir_exporter.py'
--- bzrlib/export/dir_exporter.py	2011-07-04 21:55:35 +0000
+++ bzrlib/export/dir_exporter.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,8 @@
 
 """Export a bzrlib.tree.Tree to a new or empty directory."""
 
+from __future__ import absolute_import
+
 import errno
 import os
 
@@ -52,15 +54,15 @@
     # Note in the case of revision trees, this does trigger a double inventory
     # lookup, hopefully it isn't too expensive.
     to_fetch = []
-    for dp, ie in _export_iter_entries(tree, subdir):
+    for dp, tp, ie in _export_iter_entries(tree, subdir):
         fullpath = osutils.pathjoin(dest, dp)
         if ie.kind == "file":
-            to_fetch.append((ie.file_id, (dp, tree.is_executable(ie.file_id))))
+            to_fetch.append((ie.file_id, (dp, tp, ie.file_id)))
         elif ie.kind == "directory":
             os.mkdir(fullpath)
         elif ie.kind == "symlink":
             try:
-                symlink_target = tree.get_symlink_target(ie.file_id)
+                symlink_target = tree.get_symlink_target(ie.file_id, tp)
                 os.symlink(symlink_target, fullpath)
             except OSError, e:
                 raise errors.BzrError(
@@ -74,11 +76,11 @@
     # The data returned here can be in any order, but we've already created all
     # the directories
     flags = os.O_CREAT | os.O_TRUNC | os.O_WRONLY | getattr(os, 'O_BINARY', 0)
-    for (relpath, executable), chunks in tree.iter_files_bytes(to_fetch):
+    for (relpath, treepath, file_id), chunks in tree.iter_files_bytes(to_fetch):
         fullpath = osutils.pathjoin(dest, relpath)
         # We set the mode and let the umask sort out the file info
         mode = 0666
-        if executable:
+        if tree.is_executable(file_id, treepath):
             mode = 0777
         out = os.fdopen(os.open(fullpath, flags, mode), 'wb')
         try:
@@ -88,11 +90,7 @@
         if force_mtime is not None:
             mtime = force_mtime
         else:
-            if subdir is None:
-                file_id = tree.path2id(relpath)
-            else:
-                file_id = tree.path2id(osutils.pathjoin(subdir, relpath))
-            mtime = tree.get_file_mtime(file_id, relpath)
+            mtime = tree.get_file_mtime(file_id, treepath)
         os.utime(fullpath, (mtime, mtime))
 
         yield

=== modified file 'bzrlib/export/tar_exporter.py'
--- bzrlib/export/tar_exporter.py	2011-07-21 07:08:05 +0000
+++ bzrlib/export/tar_exporter.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,8 @@
 
 """Export a tree to a tarball."""
 
+from __future__ import absolute_import
+
 import os
 import StringIO
 import sys
@@ -28,15 +30,13 @@
 from bzrlib.export import _export_iter_entries
 
 
-def prepare_tarball_item(tree, root, final_path, entry, force_mtime=None):
+def prepare_tarball_item(tree, root, final_path, tree_path, entry, force_mtime=None):
     """Prepare a tarball item for exporting
 
     :param tree: Tree to export
-
     :param final_path: Final path to place item
-
+    :param tree_path: Path for the entry in the tree
     :param entry: Entry to export
-
     :param force_mtime: Option mtime to force, instead of using tree
         timestamps.
 
@@ -47,10 +47,10 @@
     if force_mtime is not None:
         item.mtime = force_mtime
     else:
-        item.mtime = tree.get_file_mtime(entry.file_id, final_path)
+        item.mtime = tree.get_file_mtime(entry.file_id, tree_path)
     if entry.kind == "file":
         item.type = tarfile.REGTYPE
-        if tree.is_executable(entry.file_id):
+        if tree.is_executable(entry.file_id, tree_path):
             item.mode = 0755
         else:
             item.mode = 0644
@@ -58,7 +58,7 @@
         # the tarfile contract, which wants the size of the file up front.  We
         # want to make sure it doesn't change, and we need to read it in one
         # go for content filtering.
-        content = tree.get_file_text(entry.file_id)
+        content = tree.get_file_text(entry.file_id, tree_path)
         item.size = len(content)
         fileobj = StringIO.StringIO(content)
     elif entry.kind == "directory":
@@ -71,7 +71,7 @@
         item.type = tarfile.SYMTYPE
         item.size = 0
         item.mode = 0755
-        item.linkname = tree.get_symlink_target(entry.file_id)
+        item.linkname = tree.get_symlink_target(entry.file_id, tree_path)
         fileobj = None
     else:
         raise errors.BzrError("don't know how to export {%s} of kind %r"
@@ -97,9 +97,9 @@
         timestamps.
     """
     try:
-        for final_path, entry in _export_iter_entries(tree, subdir):
+        for final_path, tree_path, entry in _export_iter_entries(tree, subdir):
             (item, fileobj) = prepare_tarball_item(
-                tree, root, final_path, entry, force_mtime)
+                tree, root, final_path, tree_path, entry, force_mtime)
             ball.addfile(item, fileobj)
             yield
     finally:

=== modified file 'bzrlib/export/zip_exporter.py'
--- bzrlib/export/zip_exporter.py	2011-07-04 21:55:35 +0000
+++ bzrlib/export/zip_exporter.py	2011-12-19 13:23:58 +0000
@@ -17,6 +17,8 @@
 """Export a Tree to a zip file.
 """
 
+from __future__ import absolute_import
+
 import os
 import stat
 import sys
@@ -55,7 +57,7 @@
         dest = sys.stdout
     zipf = zipfile.ZipFile(dest, "w", compression)
     try:
-        for dp, ie in _export_iter_entries(tree, subdir):
+        for dp, tp, ie in _export_iter_entries(tree, subdir):
             file_id = ie.file_id
             mutter("  export {%s} kind %s to %s", file_id, ie.kind, dest)
 
@@ -64,7 +66,7 @@
             if force_mtime is not None:
                 mtime = force_mtime
             else:
-                mtime = tree.get_file_mtime(ie.file_id, dp)
+                mtime = tree.get_file_mtime(ie.file_id, tp)
             date_time = time.localtime(mtime)[:6]
             filename = osutils.pathjoin(root, dp).encode('utf8')
             if ie.kind == "file":
@@ -73,7 +75,7 @@
                             date_time=date_time)
                 zinfo.compress_type = compression
                 zinfo.external_attr = _FILE_ATTR
-                content = tree.get_file_text(file_id)
+                content = tree.get_file_text(file_id, tp)
                 zipf.writestr(zinfo, content)
             elif ie.kind == "directory":
                 # Directories must contain a trailing slash, to indicate
@@ -91,7 +93,7 @@
                             date_time=date_time)
                 zinfo.compress_type = compression
                 zinfo.external_attr = _FILE_ATTR
-                zipf.writestr(zinfo, tree.get_symlink_target(file_id))
+                zipf.writestr(zinfo, tree.get_symlink_target(file_id, tp))
             yield
 
         zipf.close()

=== modified file 'bzrlib/export_pot.py'
--- bzrlib/export_pot.py	2011-05-28 12:17:22 +0000
+++ bzrlib/export_pot.py	2011-12-19 13:23:58 +0000
@@ -18,8 +18,16 @@
 # with Python under the Python License, which is GPL compatible.
 
 """Extract docstrings from Bazaar commands.
+
+This module only handles bzrlib objects that use strings not directly wrapped
+by a gettext() call. To generate a complete translation template file, this
+output needs to be combined with that of xgettext or a similar command for
+extracting those strings, as is done in the bzr Makefile. Sorting the output
+is also left to that stage of the process.
 """
 
+from __future__ import absolute_import
+
 import inspect
 import os
 
@@ -27,12 +35,15 @@
     commands as _mod_commands,
     errors,
     help_topics,
+    option,
     plugin,
+    help,
     )
 from bzrlib.trace import (
     mutter,
     note,
     )
+from bzrlib.i18n import gettext
 
 
 def _escape(s):
@@ -60,114 +71,169 @@
     return s
 
 
-_FOUND_MSGID = None # set by entry function.
-
-def _poentry(outf, path, lineno, s, comment=None):
-    if s in _FOUND_MSGID:
-        return
-    _FOUND_MSGID.add(s)
-    if comment is None:
-        comment = ''
-    else:
-        comment = "# %s\n" % comment
-    mutter("Exporting msg %r at line %d in %r", s[:20], lineno, path)
-    print >>outf, ('#: %s:%d\n' % (path, lineno) +
-           comment+
-           'msgid %s\n' % _normalize(s) +
-           'msgstr ""\n')
-
-def _poentry_per_paragraph(outf, path, lineno, msgid, filter=lambda x: False):
-    # TODO: How to split long help?
-    paragraphs = msgid.split('\n\n')
-    for p in paragraphs:
-        if filter(p):
-            continue
-        _poentry(outf, path, lineno, p)
-        lineno += p.count('\n') + 2
-
-_LAST_CACHE = _LAST_CACHED_SRC = None
-
-def _offsets_of_literal(src):
-    global _LAST_CACHE, _LAST_CACHED_SRC
-    if src == _LAST_CACHED_SRC:
-        return _LAST_CACHE.copy()
-
+def _parse_source(source_text):
+    """Get object to lineno mappings from given source_text"""
     import ast
-    root = ast.parse(src)
-    offsets = {}
-    for node in ast.walk(root):
-        if not isinstance(node, ast.Str):
-            continue
-        offsets[node.s] = node.lineno - node.s.count('\n')
-
-    _LAST_CACHED_SRC = src
-    _LAST_CACHE = offsets.copy()
-    return offsets
-
-def _standard_options(outf):
-    from bzrlib.option import Option
-    src = inspect.findsource(Option)[0]
-    src = ''.join(src)
-    path = 'bzrlib/option.py'
-    offsets = _offsets_of_literal(src)
-
-    for name in sorted(Option.OPTIONS.keys()):
-        opt = Option.OPTIONS[name]
-        if getattr(opt, 'hidden', False):
-            continue
-        if getattr(opt, 'title', None):
-            lineno = offsets.get(opt.title, 9999)
-            if lineno == 9999:
-                note("%r is not found in bzrlib/option.py" % opt.title)
-            _poentry(outf, path, lineno, opt.title,
-                     'title of %r option' % name)
-        if getattr(opt, 'help', None):
-            lineno = offsets.get(opt.help, 9999)
-            if lineno == 9999:
-                note("%r is not found in bzrlib/option.py" % opt.help)
-            _poentry(outf, path, lineno, opt.help,
-                     'help of %r option' % name)
-
-def _command_options(outf, path, cmd):
-    src, default_lineno = inspect.findsource(cmd.__class__)
-    offsets = _offsets_of_literal(''.join(src))
+    cls_to_lineno = {}
+    str_to_lineno = {}
+    for node in ast.walk(ast.parse(source_text)):
+        # TODO: worry about duplicates?
+        if isinstance(node, ast.ClassDef):
+            # TODO: worry about nesting?
+            cls_to_lineno[node.name] = node.lineno
+        elif isinstance(node, ast.Str):
+            # Python AST gives location of string literal as the line the
+            # string terminates on. It's more useful to have the line the
+            # string begins on. Unfortunately, counting back newlines is
+            # only an approximation as the AST is ignorant of escaping.
+            str_to_lineno[node.s] = node.lineno - node.s.count('\n')
+    return cls_to_lineno, str_to_lineno
+
+
+class _ModuleContext(object):
+    """Record of the location within a source tree"""
+
+    def __init__(self, path, lineno=1, _source_info=None):
+        self.path = path
+        self.lineno = lineno
+        if _source_info is not None:
+            self._cls_to_lineno, self._str_to_lineno = _source_info
+
+    @classmethod
+    def from_module(cls, module):
+        """Get new context from module object and parse source for linenos"""
+        sourcepath = inspect.getsourcefile(module)
+        # TODO: fix this to do the right thing rather than rely on cwd
+        relpath = os.path.relpath(sourcepath)
+        return cls(relpath,
+            _source_info=_parse_source("".join(inspect.findsource(module)[0])))
+
+    def from_class(self, cls):
+        """Get new context with same details but lineno of class in source"""
+        try:
+            lineno = self._cls_to_lineno[cls.__name__]
+        except (AttributeError, KeyError):
+            mutter("Definition of %r not found in %r", cls, self.path)
+            return self
+        return self.__class__(self.path, lineno,
+            (self._cls_to_lineno, self._str_to_lineno))
+
+    def from_string(self, string):
+        """Get new context with same details but lineno of string in source"""
+        try:
+            lineno = self._str_to_lineno[string]
+        except (AttributeError, KeyError):
+            mutter("String %r not found in %r", string[:20], self.path)
+            return self
+        return self.__class__(self.path, lineno,
+            (self._cls_to_lineno, self._str_to_lineno))
+
+
+class _PotExporter(object):
+    """Write message details to output stream in .pot file format"""
+
+    def __init__(self, outf, include_duplicates=False):
+        self.outf = outf
+        if include_duplicates:
+            self._msgids = None
+        else:
+            self._msgids = set()
+        self._module_contexts = {}
+
+    def poentry(self, path, lineno, s, comment=None):
+        if self._msgids is not None:
+            if s in self._msgids:
+                return
+            self._msgids.add(s)
+        if comment is None:
+            comment = ''
+        else:
+            comment = "# %s\n" % comment
+        mutter("Exporting msg %r at line %d in %r", s[:20], lineno, path)
+        self.outf.write(
+            "#: {path}:{lineno}\n"
+            "{comment}"
+            "msgid {msg}\n"
+            "msgstr \"\"\n"
+            "\n".format(
+                path=path, lineno=lineno, comment=comment, msg=_normalize(s)))
+
+    def poentry_in_context(self, context, string, comment=None):
+        context = context.from_string(string)
+        self.poentry(context.path, context.lineno, string, comment)
+
+    def poentry_per_paragraph(self, path, lineno, msgid, include=None):
+        # TODO: How to split long help?
+        paragraphs = msgid.split('\n\n')
+        if include is not None:
+            paragraphs = filter(include, paragraphs)
+        for p in paragraphs:
+            self.poentry(path, lineno, p)
+            lineno += p.count('\n') + 2
+
+    def get_context(self, obj):
+        module = inspect.getmodule(obj)
+        try:
+            context = self._module_contexts[module.__name__]
+        except KeyError:
+            context = _ModuleContext.from_module(module)
+            self._module_contexts[module.__name__] = context
+        if inspect.isclass(obj):
+            context = context.from_class(obj)
+        return context
+
+
+def _write_option(exporter, context, opt, note):
+    if getattr(opt, 'hidden', False):
+        return   
+    optname = opt.name
+    if getattr(opt, 'title', None):
+        exporter.poentry_in_context(context, opt.title,
+            "title of {name!r} {what}".format(name=optname, what=note))
+    for name, _, _, helptxt in opt.iter_switches():
+        if name != optname:
+            if opt.is_hidden(name):
+                continue
+            name = "=".join([optname, name])
+        if helptxt:
+            exporter.poentry_in_context(context, helptxt,
+                "help of {name!r} {what}".format(name=name, what=note))
+
+
+def _standard_options(exporter):
+    OPTIONS = option.Option.OPTIONS
+    context = exporter.get_context(option)
+    for name in sorted(OPTIONS.keys()):
+        opt = OPTIONS[name]
+        _write_option(exporter, context.from_string(name), opt, "option")
+
+
+def _command_options(exporter, context, cmd):
+    note = "option of {0!r} command".format(cmd.name())
     for opt in cmd.takes_options:
-        if isinstance(opt, str):
-            continue
-        if getattr(opt, 'hidden', False):
-            continue
-        name = opt.name
-        if getattr(opt, 'title', None):
-            lineno = offsets.get(opt.title, default_lineno)
-            _poentry(outf, path, lineno, opt.title,
-                     'title of %r option of %r command' % (name, cmd.name()))
-        if getattr(opt, 'help', None):
-            lineno = offsets.get(opt.help, default_lineno)
-            _poentry(outf, path, lineno, opt.help,
-                     'help of %r option of %r command' % (name, cmd.name()))
-
-
-def _write_command_help(outf, cmd):
-    path = inspect.getfile(cmd.__class__)
-    if path.endswith('.pyc'):
-        path = path[:-1]
-    path = os.path.relpath(path)
-    src, lineno = inspect.findsource(cmd.__class__)
-    offsets = _offsets_of_literal(''.join(src))
-    lineno = offsets[cmd.__doc__]
-    doc = inspect.getdoc(cmd)
-
-    def filter(p):
+        # String values in Command option lists are for global options
+        if not isinstance(opt, str):
+            _write_option(exporter, context, opt, note)
+
+
+def _write_command_help(exporter, cmd):
+    context = exporter.get_context(cmd.__class__)
+    rawdoc = cmd.__doc__
+    dcontext = context.from_string(rawdoc)
+    doc = inspect.cleandoc(rawdoc)
+
+    def exclude_usage(p):
         # ':Usage:' has special meaning in help topics.
         # This is usage example of command and should not be translated.
-        if p.splitlines()[0] == ':Usage:':
+        if p.splitlines()[0] != ':Usage:':
             return True
 
-    _poentry_per_paragraph(outf, path, lineno, doc, filter)
-    _command_options(outf, path, cmd)
-
-
-def _command_helps(outf):
+    exporter.poentry_per_paragraph(dcontext.path, dcontext.lineno, doc,
+        exclude_usage)
+    _command_options(exporter, context, cmd)
+
+
+def _command_helps(exporter, plugin_name=None):
     """Extract docstrings from path.
 
     This respects the Bazaar cmdtable/table convention and will
@@ -180,34 +246,36 @@
         command = _mod_commands.get_cmd_object(cmd_name, False)
         if command.hidden:
             continue
-        note("Exporting messages from builtin command: %s", cmd_name)
-        _write_command_help(outf, command)
+        if plugin_name is not None:
+            # only export builtins if we are not exporting plugin commands
+            continue
+        note(gettext("Exporting messages from builtin command: %s"), cmd_name)
+        _write_command_help(exporter, command)
 
     plugin_path = plugin.get_core_plugin_path()
     core_plugins = glob(plugin_path + '/*/__init__.py')
     core_plugins = [os.path.basename(os.path.dirname(p))
                         for p in core_plugins]
-    # core plugins
+    # plugins
     for cmd_name in _mod_commands.plugin_command_names():
         command = _mod_commands.get_cmd_object(cmd_name, False)
         if command.hidden:
             continue
-        if command.plugin_name() not in core_plugins:
+        if plugin_name is not None and command.plugin_name() != plugin_name:
+            # if we are exporting plugin commands, skip plugins we have not specified.
+            continue
+        if plugin_name is None and command.plugin_name() not in core_plugins:
             # skip non-core plugins
             # TODO: Support extracting from third party plugins.
             continue
-        note("Exporting messages from plugin command: %s in %s",
-             cmd_name, command.plugin_name())
-        _write_command_help(outf, command)
-
-
-def _error_messages(outf):
+        note(gettext("Exporting messages from plugin command: {0} in {1}").format(
+             cmd_name, command.plugin_name() ))
+        _write_command_help(exporter, command)
+
+
+def _error_messages(exporter):
     """Extract fmt string from bzrlib.errors."""
-    path = errors.__file__
-    if path.endswith('.pyc'):
-        path = path[:-1]
-    offsets = _offsets_of_literal(open(path).read())
-
+    context = exporter.get_context(errors)
     base_klass = errors.BzrError
     for name in dir(errors):
         klass = getattr(errors, name)
@@ -221,30 +289,34 @@
             continue
         fmt = getattr(klass, "_fmt", None)
         if fmt:
-            note("Exporting message from error: %s", name)
-            _poentry(outf, 'bzrlib/errors.py',
-                     offsets.get(fmt, 9999), fmt)
-
-def _help_topics(outf):
+            note(gettext("Exporting message from error: %s"), name)
+            exporter.poentry_in_context(context, fmt)
+
+
+def _help_topics(exporter):
     topic_registry = help_topics.topic_registry
     for key in topic_registry.keys():
         doc = topic_registry.get(key)
         if isinstance(doc, str):
-            _poentry_per_paragraph(
-                    outf,
+            exporter.poentry_per_paragraph(
                     'dummy/help_topics/'+key+'/detail.txt',
                     1, doc)
-
+        elif callable(doc): # help topics from files
+            exporter.poentry_per_paragraph(
+                    'en/help_topics/'+key+'.txt',
+                    1, doc(key))
         summary = topic_registry.get_summary(key)
         if summary is not None:
-            _poentry(outf, 'dummy/help_topics/'+key+'/summary.txt',
+            exporter.poentry('dummy/help_topics/'+key+'/summary.txt',
                      1, summary)
 
-def export_pot(outf):
-    global _FOUND_MSGID
-    _FOUND_MSGID = set()
-    _standard_options(outf)
-    _command_helps(outf)
-    _error_messages(outf)
-    # disable exporting help topics until we decide  how to translate it.
-    #_help_topics(outf)
+
+def export_pot(outf, plugin=None, include_duplicates=False):
+    exporter = _PotExporter(outf, include_duplicates)
+    if plugin is None:
+        _standard_options(exporter)
+        _command_helps(exporter)
+        _error_messages(exporter)
+        _help_topics(exporter)
+    else:
+        _command_helps(exporter, plugin)

=== modified file 'bzrlib/externalcommand.py'
--- bzrlib/externalcommand.py	2010-04-30 11:03:59 +0000
+++ bzrlib/externalcommand.py	2011-12-18 15:28:38 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 # TODO: Perhaps rather than mapping options and arguments back and
 # forth, we should just pass in the whole argv, and allow
 # ExternalCommands to handle it differently to internal commands?

=== modified file 'bzrlib/fetch.py'
--- bzrlib/fetch.py	2011-05-20 08:25:15 +0000
+++ bzrlib/fetch.py	2011-12-19 13:23:58 +0000
@@ -14,7 +14,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-
 """Copying of history from one branch to another.
 
 The basic plan is that every branch knows the history of everything
@@ -23,20 +22,23 @@
 branch.
 """
 
+from __future__ import absolute_import
+
 import operator
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 from bzrlib import (
-    graph as _mod_graph,
     tsort,
     versionedfile,
+    vf_search,
     )
 """)
 from bzrlib import (
     errors,
     ui,
     )
+from bzrlib.i18n import gettext
 from bzrlib.revision import NULL_REVISION
 from bzrlib.trace import mutter
 
@@ -93,12 +95,12 @@
         pb = ui.ui_factory.nested_progress_bar()
         pb.show_pct = pb.show_count = False
         try:
-            pb.update("Finding revisions", 0, 2)
+            pb.update(gettext("Finding revisions"), 0, 2)
             search_result = self._revids_to_fetch()
             mutter('fetching: %s', search_result)
             if search_result.is_empty():
                 return
-            pb.update("Fetching revisions", 1, 2)
+            pb.update(gettext("Fetching revisions"), 1, 2)
             self._fetch_everything_for_search(search_result)
         finally:
             pb.finished()
@@ -160,13 +162,13 @@
         elif self._last_revision == NULL_REVISION:
             # fetch_spec is None + last_revision is null => empty fetch.
             # explicit limit of no revisions needed
-            return _mod_graph.EmptySearchResult()
+            return vf_search.EmptySearchResult()
         elif self._last_revision is not None:
-            return _mod_graph.NotInOtherForRevs(self.to_repository,
+            return vf_search.NotInOtherForRevs(self.to_repository,
                 self.from_repository, [self._last_revision],
                 find_ghosts=self.find_ghosts).execute()
         else: # self._last_revision is None:
-            return _mod_graph.EverythingNotInOther(self.to_repository,
+            return vf_search.EverythingNotInOther(self.to_repository,
                 self.from_repository,
                 find_ghosts=self.find_ghosts).execute()
 
@@ -388,11 +390,11 @@
                     "limit is only supported with a source branch set")
             # Caller hasn't specified any revisions or source branch
             if self.target_repo_kind == TargetRepoKinds.EMPTY:
-                return _mod_graph.EverythingResult(self.source_repo)
+                return vf_search.EverythingResult(self.source_repo)
             else:
                 # We want everything not already in the target (or target's
                 # fallbacks).
-                return _mod_graph.EverythingNotInOther(
+                return vf_search.EverythingNotInOther(
                     self.target_repo, self.source_repo).execute()
         heads_to_fetch = set(self._explicit_rev_ids)
         if self.source_branch is not None:
@@ -415,7 +417,7 @@
             # heads_to_fetch will almost certainly be present so this doesn't
             # matter much.
             all_heads = heads_to_fetch.union(if_present_fetch)
-            ret = _mod_graph.PendingAncestryResult(all_heads, self.source_repo)
+            ret = vf_search.PendingAncestryResult(all_heads, self.source_repo)
             if self.limit is not None:
                 graph = self.source_repo.get_graph()
                 topo_order = list(graph.iter_topo_order(ret.get_keys()))
@@ -423,6 +425,6 @@
                 ret = self.source_repo.revision_ids_to_search_result(result_set)
             return ret
         else:
-            return _mod_graph.NotInOtherForRevs(self.target_repo, self.source_repo,
+            return vf_search.NotInOtherForRevs(self.target_repo, self.source_repo,
                 required_ids=heads_to_fetch, if_present_ids=if_present_fetch,
                 limit=self.limit).execute()

=== modified file 'bzrlib/fifo_cache.py'
--- bzrlib/fifo_cache.py	2009-03-23 14:59:43 +0000
+++ bzrlib/fifo_cache.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,8 @@
 
 """A simple first-in-first-out (FIFO) cache."""
 
+from __future__ import absolute_import
+
 from collections import deque
 
 

=== modified file 'bzrlib/filter_tree.py'
--- bzrlib/filter_tree.py	2011-07-04 21:10:37 +0000
+++ bzrlib/filter_tree.py	2011-12-19 13:23:58 +0000
@@ -17,19 +17,14 @@
 """Content-filtered view of any tree.
 """
 
+from __future__ import absolute_import
 
 from bzrlib import (
     tree,
     )
 from bzrlib.filters import (
-    ContentFilter,
     ContentFilterContext,
-    filtered_input_file,
     filtered_output_bytes,
-    _get_filter_stack_for,
-    _get_registered_names,
-    internal_size_sha_file_byname,
-    register_filter_stack_map,
     )
 
 

=== modified file 'bzrlib/filters/__init__.py'
--- bzrlib/filters/__init__.py	2011-04-07 10:36:24 +0000
+++ bzrlib/filters/__init__.py	2011-12-19 13:23:58 +0000
@@ -14,7 +14,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-
 """Working tree content filtering support.
 
 A filter consists of a read converter, write converter pair.
@@ -39,6 +38,8 @@
 Note that context is currently only supported for write converters.
 """
 
+from __future__ import absolute_import
+
 from cStringIO import StringIO
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
@@ -282,6 +283,4 @@
     return original
 
 
-# Register the standard filters
-from bzrlib.filters import eol
-eol.register_eol_content_filter()
+lazy_register_filter_stack_map('eol', 'bzrlib.filters.eol', 'eol_lookup')

=== modified file 'bzrlib/filters/eol.py'
--- bzrlib/filters/eol.py	2010-06-17 09:23:19 +0000
+++ bzrlib/filters/eol.py	2011-12-19 10:58:39 +0000
@@ -19,10 +19,13 @@
 See bzr help eol for details.
 """
 
+from __future__ import absolute_import
+
 
 import re, sys
 
 from bzrlib.errors import BzrError
+from bzrlib.filters import ContentFilter
 
 
 # Real Unix newline - \n without \r before it
@@ -47,29 +50,24 @@
         return [_UNIX_NL_RE.sub('\r\n', content)]
 
 
-# Register the eol content filter.
-def register_eol_content_filter():
-    from bzrlib.filters import ContentFilter, register_filter_stack_map
- 
-    if sys.platform == 'win32':
-        _native_output = _to_crlf_converter
-    else:
-        _native_output = _to_lf_converter
-    _eol_filter_stack_map = {
-        'exact': [],
-        'native': [ContentFilter(_to_lf_converter, _native_output)],
-        'lf':     [ContentFilter(_to_lf_converter, _to_lf_converter)],
-        'crlf':   [ContentFilter(_to_lf_converter, _to_crlf_converter)],
-        'native-with-crlf-in-repo':
-            [ContentFilter(_to_crlf_converter, _native_output)],
-        'lf-with-crlf-in-repo':
-            [ContentFilter(_to_crlf_converter, _to_lf_converter)],
-        'crlf-with-crlf-in-repo':
-            [ContentFilter(_to_crlf_converter, _to_crlf_converter)],
-        }
-    def eol_lookup(key):
-        filter = _eol_filter_stack_map.get(key)
-        if filter is None:
-            raise BzrError("Unknown eol value '%s'" % key)
-        return filter
-    register_filter_stack_map('eol', eol_lookup)
+if sys.platform == 'win32':
+    _native_output = _to_crlf_converter
+else:
+    _native_output = _to_lf_converter
+_eol_filter_stack_map = {
+    'exact': [],
+    'native': [ContentFilter(_to_lf_converter, _native_output)],
+    'lf':     [ContentFilter(_to_lf_converter, _to_lf_converter)],
+    'crlf':   [ContentFilter(_to_lf_converter, _to_crlf_converter)],
+    'native-with-crlf-in-repo':
+        [ContentFilter(_to_crlf_converter, _native_output)],
+    'lf-with-crlf-in-repo':
+        [ContentFilter(_to_crlf_converter, _to_lf_converter)],
+    'crlf-with-crlf-in-repo':
+        [ContentFilter(_to_crlf_converter, _to_crlf_converter)],
+    }
+def eol_lookup(key):
+    filter = _eol_filter_stack_map.get(key)
+    if filter is None:
+        raise BzrError("Unknown eol value '%s'" % key)
+    return filter

=== modified file 'bzrlib/foreign.py'
--- bzrlib/foreign.py	2011-05-14 07:56:12 +0000
+++ bzrlib/foreign.py	2011-12-19 13:23:58 +0000
@@ -14,9 +14,10 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-
 """Foreign branch utilities."""
 
+from __future__ import absolute_import
+
 
 from bzrlib.branch import (
     Branch,
@@ -31,6 +32,7 @@
     registry,
     transform,
     )
+from bzrlib.i18n import gettext
 """)
 
 class VcsMapping(object):
@@ -263,7 +265,6 @@
     branch unless the --no-rebase option is used, in which case 
     the two branches will be out of sync after the push. 
     """
-    hidden = True
     takes_args = ['location?']
     takes_options = [
         'remember',
@@ -282,7 +283,7 @@
     def run(self, location=None, remember=False, directory=None,
             no_rebase=False, strict=None):
         from bzrlib import urlutils
-        from bzrlib.bzrdir import BzrDir
+        from bzrlib.controldir import ControlDir
         from bzrlib.errors import BzrCommandError, NoWorkingTree
         from bzrlib.workingtree import WorkingTree
 
@@ -302,23 +303,24 @@
         stored_loc = source_branch.get_push_location()
         if location is None:
             if stored_loc is None:
-                raise BzrCommandError("No push location known or specified.")
+                raise BzrCommandError(gettext("No push location known or specified."))
             else:
                 display_url = urlutils.unescape_for_display(stored_loc,
                         self.outf.encoding)
-                self.outf.write("Using saved location: %s\n" % display_url)
+                self.outf.write(
+                       gettext("Using saved location: %s\n") % display_url)
                 location = stored_loc
 
-        bzrdir = BzrDir.open(location)
-        target_branch = bzrdir.open_branch()
+        controldir = ControlDir.open(location)
+        target_branch = controldir.open_branch()
         target_branch.lock_write()
         try:
             try:
                 push_result = source_branch.push(target_branch, lossy=True)
             except errors.LossyPushToSameVCS:
-                raise BzrCommandError("%r and %r are in the same VCS, lossy "
-                    "push not necessary. Please use regular push." %
-                    (source_branch, target_branch))
+                raise BzrCommandError(gettext("{0!r} and {1!r} are in the same VCS, lossy "
+                    "push not necessary. Please use regular push.").format(
+                    source_branch, target_branch))
             # We successfully created the target, remember it
             if source_branch.get_push_location() is None or remember:
                 source_branch.set_push_location(target_branch.base)

=== modified file 'bzrlib/generate_ids.py'
--- bzrlib/generate_ids.py	2011-04-07 10:36:24 +0000
+++ bzrlib/generate_ids.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,8 @@
 
 """Common code for generating file or revision ids."""
 
+from __future__ import absolute_import
+
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 import time

=== modified file 'bzrlib/globbing.py'
--- bzrlib/globbing.py	2011-06-28 22:25:28 +0000
+++ bzrlib/globbing.py	2011-12-19 13:23:58 +0000
@@ -20,6 +20,8 @@
 expressions.
 """
 
+from __future__ import absolute_import
+
 import re
 
 from bzrlib import (

=== modified file 'bzrlib/gpg.py'
--- bzrlib/gpg.py	2011-08-23 10:34:53 +0000
+++ bzrlib/gpg.py	2011-12-19 13:23:58 +0000
@@ -17,6 +17,8 @@
 
 """GPG signing and checking logic."""
 
+from __future__ import absolute_import
+
 import os
 import sys
 from StringIO import StringIO
@@ -27,26 +29,17 @@
 import subprocess
 
 from bzrlib import (
+    config,
     errors,
     trace,
     ui,
     )
+from bzrlib.i18n import (
+    gettext, 
+    ngettext,
+    )
 """)
 
-class i18n:
-    """this class is ready to use bzrlib.i18n but bzrlib.i18n is not ready to
-    use so here is a stub until it is"""
-    @staticmethod
-    def gettext(string):
-        return string
-        
-    @staticmethod
-    def ngettext(single, plural, number):
-        if number == 1:
-            return single
-        else:
-            return plural
-
 #verification results
 SIGNATURE_VALID = 0
 SIGNATURE_KEY_MISSING = 1
@@ -115,7 +108,7 @@
         all_verifiable = True
         for rev_id in revisions:
             verification_result, uid =\
-                                repository.verify_revision(rev_id,self)
+                repository.verify_revision_signature(rev_id,self)
             result.append([rev_id, verification_result, uid])
             count[verification_result] += 1
             if verification_result != SIGNATURE_VALID:
@@ -123,31 +116,31 @@
         return (count, result, all_verifiable)
 
     def valid_commits_message(self, count):
-        return i18n.gettext(u"{0} commits with valid signatures").format(
-                                        count[SIGNATURE_VALID])            
+        return gettext(u"{0} commits with valid signatures").format(
+                                        count[SIGNATURE_VALID])
 
     def unknown_key_message(self, count):
-        return i18n.ngettext(u"{0} commit with unknown key",
+        return ngettext(u"{0} commit with unknown key",
                              u"{0} commits with unknown keys",
                              count[SIGNATURE_KEY_MISSING]).format(
                                         count[SIGNATURE_KEY_MISSING])
 
     def commit_not_valid_message(self, count):
-        return i18n.ngettext(u"{0} commit not valid",
+        return ngettext(u"{0} commit not valid",
                              u"{0} commits not valid",
                              count[SIGNATURE_NOT_VALID]).format(
                                             count[SIGNATURE_NOT_VALID])
 
     def commit_not_signed_message(self, count):
-        return i18n.ngettext(u"{0} commit not signed",
+        return ngettext(u"{0} commit not signed",
                              u"{0} commits not signed",
                              count[SIGNATURE_NOT_SIGNED]).format(
                                         count[SIGNATURE_NOT_SIGNED])
 
     def expired_commit_message(self, count):
-        return i18n.ngettext(u"{0} commit with key now expired",
-                             u"{0} commits with key now expired",
-                             count[SIGNATURE_EXPIRED]).format(
+        return ngettext(u"{0} commit with key now expired",
+                        u"{0} commits with key now expired",
+                        count[SIGNATURE_EXPIRED]).format(
                                         count[SIGNATURE_EXPIRED])
 
 
@@ -169,6 +162,14 @@
 
     acceptable_keys = None
 
+    def __init__(self, config_stack):
+        self._config_stack = config_stack
+        try:
+            import gpgme
+            self.context = gpgme.Context()
+        except ImportError, error:
+            pass # can't use verify()
+
     @staticmethod
     def verify_signatures_available():
         """
@@ -183,17 +184,13 @@
             return False
 
     def _command_line(self):
-        
-        return [self._config.gpg_signing_command(), '--clearsign', '-u',
-                                                self._config.gpg_signing_key()]
-
-    def __init__(self, config):
-        self._config = config
-        try:
-            import gpgme
-            self.context = gpgme.Context()
-        except ImportError, error:
-            pass # can't use verify()
+        key = self._config_stack.get('gpg_signing_key')
+        if key is None or key == 'default':
+            # 'default' or not setting gpg_signing_key at all means we should
+            # use the user email address
+            key = config.extract_email_address(self._config_stack.get('email'))
+        return [self._config_stack.get('gpg_signing_command'), '--clearsign',
+                '-u', key]
 
     def sign(self, content):
         if isinstance(content, unicode):
@@ -246,7 +243,6 @@
 
         signature = StringIO(content)
         plain_output = StringIO()
-        
         try:
             result = self.context.verify(signature, None, plain_output)
         except gpgme.GpgmeError,error:
@@ -256,16 +252,16 @@
         # test_verify_invalid()
         if len(result) == 0:
             return SIGNATURE_NOT_VALID, None
-        # User has specified a list of acceptable keys, check our result is in it.
-        # test_verify_unacceptable_key()
+        # User has specified a list of acceptable keys, check our result is in
+        # it.  test_verify_unacceptable_key()
         fingerprint = result[0].fpr
         if self.acceptable_keys is not None:
-            if not fingerprint in self.acceptable_keys:                
+            if not fingerprint in self.acceptable_keys:
                 return SIGNATURE_KEY_MISSING, fingerprint[-8:]
         # Check the signature actually matches the testament.
         # test_verify_bad_testament()
         if testament != plain_output.getvalue():
-            return SIGNATURE_NOT_VALID, None 
+            return SIGNATURE_NOT_VALID, None
         # Yay gpgme set the valid bit.
         # Can't write a test for this one as you can't set a key to be
         # trusted using gpgme.
@@ -282,12 +278,12 @@
         # test_verify_unknown_key()
         if result[0].summary & gpgme.SIGSUM_KEY_MISSING:
             return SIGNATURE_KEY_MISSING, fingerprint[-8:]
-        # Summary isn't set if sig is valid but key is untrusted
-        # but if user has explicity set the key as acceptable we can validate it.
+        # Summary isn't set if sig is valid but key is untrusted but if user
+        # has explicity set the key as acceptable we can validate it.
         if result[0].summary == 0 and self.acceptable_keys is not None:
             if fingerprint in self.acceptable_keys:
                 # test_verify_untrusted_but_accepted()
-                return SIGNATURE_VALID, None 
+                return SIGNATURE_VALID, None
         # test_verify_valid_but_untrusted()
         if result[0].summary == 0 and self.acceptable_keys is None:
             return SIGNATURE_NOT_VALID, None
@@ -311,24 +307,25 @@
                                                  "verification result")
 
     def set_acceptable_keys(self, command_line_input):
-        """sets the acceptable keys for verifying with this GPGStrategy
+        """Set the acceptable keys for verifying with this GPGStrategy.
         
         :param command_line_input: comma separated list of patterns from
                                 command line
         :return: nothing
         """
         key_patterns = None
-        acceptable_keys_config = self._config.acceptable_keys()
+        acceptable_keys_config = self._config_stack.get('acceptable_keys')
         try:
             if isinstance(acceptable_keys_config, unicode):
                 acceptable_keys_config = str(acceptable_keys_config)
         except UnicodeEncodeError:
-            #gpg Context.keylist(pattern) does not like unicode
-            raise errors.BzrCommandError('Only ASCII permitted in option names')
+            # gpg Context.keylist(pattern) does not like unicode
+            raise errors.BzrCommandError(
+                gettext('Only ASCII permitted in option names'))
 
         if acceptable_keys_config is not None:
             key_patterns = acceptable_keys_config
-        if command_line_input is not None: #command line overrides config
+        if command_line_input is not None: # command line overrides config
             key_patterns = command_line_input
         if key_patterns is not None:
             patterns = key_patterns.split(",")
@@ -342,8 +339,8 @@
                     self.acceptable_keys.append(key.subkeys[0].fpr)
                     trace.mutter("Added acceptable key: " + key.subkeys[0].fpr)
                 if not found_key:
-                    trace.note(i18n.gettext(
-                            "No GnuPG key results for pattern: {}"
+                    trace.note(gettext(
+                            "No GnuPG key results for pattern: {0}"
                                 ).format(pattern))
 
     def do_verifications(self, revisions, repository,
@@ -368,7 +365,7 @@
         all_verifiable = True
         for rev_id in revisions:
             verification_result, uid =\
-                                repository.verify_revision(rev_id,self)
+                repository.verify_revision_signature(rev_id, self)
             result.append([rev_id, verification_result, uid])
             count[verification_result] += 1
             if verification_result != SIGNATURE_VALID:
@@ -386,7 +383,7 @@
                 signers[uid] += 1
         result = []
         for uid, number in signers.items():
-             result.append( i18n.ngettext(u"{0} signed {1} commit", 
+             result.append( ngettext(u"{0} signed {1} commit",
                              u"{0} signed {1} commits",
                              number).format(uid, number) )
         return result
@@ -403,7 +400,7 @@
                 signers[authors] += 1
         result = []
         for authors, number in signers.items():
-            result.append( i18n.ngettext(u"{0} commit by author {1}", 
+            result.append( ngettext(u"{0} commit by author {1}",
                                  u"{0} commits by author {1}",
                                  number).format(number, authors) )
         return result
@@ -419,7 +416,7 @@
                 signers[authors] += 1
         result = []
         for authors, number in signers.items():
-            result.append( i18n.ngettext(u"{0} commit by author {1}", 
+            result.append( ngettext(u"{0} commit by author {1}",
                                  u"{0} commits by author {1}",
                                  number).format(number, authors) )
         return result
@@ -433,7 +430,7 @@
                 signers[fingerprint] += 1
         result = []
         for fingerprint, number in signers.items():
-            result.append( i18n.ngettext(u"Unknown key {0} signed {1} commit", 
+            result.append( ngettext(u"Unknown key {0} signed {1} commit",
                                  u"Unknown key {0} signed {1} commits",
                                  number).format(fingerprint, number) )
         return result
@@ -451,43 +448,42 @@
                 fingerprint_to_authors[fingerprint] = authors
         result = []
         for fingerprint, number in signers.items():
-            result.append( i18n.ngettext(u"{0} commit by author {1} with "\
-                                          "key {2} now expired", 
-                                 u"{0} commits by author {1} with key {2} now "\
-                                  "expired",
-                                 number).format(number,
-                            fingerprint_to_authors[fingerprint], fingerprint) )
+            result.append(
+                ngettext(u"{0} commit by author {1} with key {2} now expired",
+                         u"{0} commits by author {1} with key {2} now expired",
+                         number).format(
+                    number, fingerprint_to_authors[fingerprint], fingerprint) )
         return result
 
     def valid_commits_message(self, count):
         """returns message for number of commits"""
-        return i18n.gettext(u"{0} commits with valid signatures").format(
+        return gettext(u"{0} commits with valid signatures").format(
                                         count[SIGNATURE_VALID])
 
     def unknown_key_message(self, count):
         """returns message for number of commits"""
-        return i18n.ngettext(u"{0} commit with unknown key",
-                             u"{0} commits with unknown keys",
-                             count[SIGNATURE_KEY_MISSING]).format(
+        return ngettext(u"{0} commit with unknown key",
+                        u"{0} commits with unknown keys",
+                        count[SIGNATURE_KEY_MISSING]).format(
                                         count[SIGNATURE_KEY_MISSING])
 
     def commit_not_valid_message(self, count):
         """returns message for number of commits"""
-        return i18n.ngettext(u"{0} commit not valid",
-                             u"{0} commits not valid",
-                             count[SIGNATURE_NOT_VALID]).format(
+        return ngettext(u"{0} commit not valid",
+                        u"{0} commits not valid",
+                        count[SIGNATURE_NOT_VALID]).format(
                                             count[SIGNATURE_NOT_VALID])
 
     def commit_not_signed_message(self, count):
         """returns message for number of commits"""
-        return i18n.ngettext(u"{0} commit not signed",
-                             u"{0} commits not signed",
-                             count[SIGNATURE_NOT_SIGNED]).format(
+        return ngettext(u"{0} commit not signed",
+                        u"{0} commits not signed",
+                        count[SIGNATURE_NOT_SIGNED]).format(
                                         count[SIGNATURE_NOT_SIGNED])
 
     def expired_commit_message(self, count):
         """returns message for number of commits"""
-        return i18n.ngettext(u"{0} commit with key now expired",
-                             u"{0} commits with key now expired",
-                             count[SIGNATURE_EXPIRED]).format(
-                                        count[SIGNATURE_EXPIRED])
+        return ngettext(u"{0} commit with key now expired",
+                        u"{0} commits with key now expired",
+                        count[SIGNATURE_EXPIRED]).format(
+                                    count[SIGNATURE_EXPIRED])

=== modified file 'bzrlib/graph.py'
--- bzrlib/graph.py	2011-08-26 11:35:50 +0000
+++ bzrlib/graph.py	2011-12-18 15:28:38 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 import time
 
 from bzrlib import (
@@ -1346,12 +1348,10 @@
         return ('_BreadthFirstSearcher(iterations=%d, %s,'
                 ' seen=%r)' % (self._iterations, search, list(self.seen)))
 
-    def get_result(self):
-        """Get a SearchResult for the current state of this searcher.
+    def get_state(self):
+        """Get the current state of this searcher.
 
-        :return: A SearchResult for this search so far. The SearchResult is
-            static - the search can be advanced and the search result will not
-            be invalidated or altered.
+        :return: Tuple with started keys, excludes and included keys
         """
         if self._returning == 'next':
             # We have to know the current nodes children to be able to list the
@@ -1368,7 +1368,18 @@
             next_query = self._next_query
         excludes = self._stopped_keys.union(next_query)
         included_keys = self.seen.difference(excludes)
-        return SearchResult(self._started_keys, excludes, len(included_keys),
+        return self._started_keys, excludes, included_keys
+
+    def _get_result(self):
+        """Get a SearchResult for the current state of this searcher.
+
+        :return: A SearchResult for this search so far. The SearchResult is
+            static - the search can be advanced and the search result will not
+            be invalidated or altered.
+        """
+        from bzrlib.vf_search import SearchResult
+        (started_keys, excludes, included_keys) = self.get_state()
+        return SearchResult(started_keys, excludes, len(included_keys),
             included_keys)
 
     def step(self):
@@ -1583,347 +1594,6 @@
             return revs, ghosts
 
 
-class AbstractSearchResult(object):
-    """The result of a search, describing a set of keys.
-    
-    Search results are typically used as the 'fetch_spec' parameter when
-    fetching revisions.
-
-    :seealso: AbstractSearch
-    """
-
-    def get_recipe(self):
-        """Return a recipe that can be used to replay this search.
-
-        The recipe allows reconstruction of the same results at a later date.
-
-        :return: A tuple of `(search_kind_str, *details)`.  The details vary by
-            kind of search result.
-        """
-        raise NotImplementedError(self.get_recipe)
-
-    def get_network_struct(self):
-        """Return a tuple that can be transmitted via the HPSS protocol."""
-        raise NotImplementedError(self.get_network_struct)
-
-    def get_keys(self):
-        """Return the keys found in this search.
-
-        :return: A set of keys.
-        """
-        raise NotImplementedError(self.get_keys)
-
-    def is_empty(self):
-        """Return false if the search lists 1 or more revisions."""
-        raise NotImplementedError(self.is_empty)
-
-    def refine(self, seen, referenced):
-        """Create a new search by refining this search.
-
-        :param seen: Revisions that have been satisfied.
-        :param referenced: Revision references observed while satisfying some
-            of this search.
-        :return: A search result.
-        """
-        raise NotImplementedError(self.refine)
-
-
-class AbstractSearch(object):
-    """A search that can be executed, producing a search result.
-
-    :seealso: AbstractSearchResult
-    """
-
-    def execute(self):
-        """Construct a network-ready search result from this search description.
-
-        This may take some time to search repositories, etc.
-
-        :return: A search result (an object that implements
-            AbstractSearchResult's API).
-        """
-        raise NotImplementedError(self.execute)
-
-
-class SearchResult(AbstractSearchResult):
-    """The result of a breadth first search.
-
-    A SearchResult provides the ability to reconstruct the search or access a
-    set of the keys the search found.
-    """
-
-    def __init__(self, start_keys, exclude_keys, key_count, keys):
-        """Create a SearchResult.
-
-        :param start_keys: The keys the search started at.
-        :param exclude_keys: The keys the search excludes.
-        :param key_count: The total number of keys (from start to but not
-            including exclude).
-        :param keys: The keys the search found. Note that in future we may get
-            a SearchResult from a smart server, in which case the keys list is
-            not necessarily immediately available.
-        """
-        self._recipe = ('search', start_keys, exclude_keys, key_count)
-        self._keys = frozenset(keys)
-
-    def __repr__(self):
-        kind, start_keys, exclude_keys, key_count = self._recipe
-        if len(start_keys) > 5:
-            start_keys_repr = repr(list(start_keys)[:5])[:-1] + ', ...]'
-        else:
-            start_keys_repr = repr(start_keys)
-        if len(exclude_keys) > 5:
-            exclude_keys_repr = repr(list(exclude_keys)[:5])[:-1] + ', ...]'
-        else:
-            exclude_keys_repr = repr(exclude_keys)
-        return '<%s %s:(%s, %s, %d)>' % (self.__class__.__name__,
-            kind, start_keys_repr, exclude_keys_repr, key_count)
-
-    def get_recipe(self):
-        """Return a recipe that can be used to replay this search.
-
-        The recipe allows reconstruction of the same results at a later date
-        without knowing all the found keys. The essential elements are a list
-        of keys to start and to stop at. In order to give reproducible
-        results when ghosts are encountered by a search they are automatically
-        added to the exclude list (or else ghost filling may alter the
-        results).
-
-        :return: A tuple ('search', start_keys_set, exclude_keys_set,
-            revision_count). To recreate the results of this search, create a
-            breadth first searcher on the same graph starting at start_keys.
-            Then call next() (or next_with_ghosts()) repeatedly, and on every
-            result, call stop_searching_any on any keys from the exclude_keys
-            set. The revision_count value acts as a trivial cross-check - the
-            found revisions of the new search should have as many elements as
-            revision_count. If it does not, then additional revisions have been
-            ghosted since the search was executed the first time and the second
-            time.
-        """
-        return self._recipe
-
-    def get_network_struct(self):
-        start_keys = ' '.join(self._recipe[1])
-        stop_keys = ' '.join(self._recipe[2])
-        count = str(self._recipe[3])
-        return (self._recipe[0], '\n'.join((start_keys, stop_keys, count)))
-
-    def get_keys(self):
-        """Return the keys found in this search.
-
-        :return: A set of keys.
-        """
-        return self._keys
-
-    def is_empty(self):
-        """Return false if the search lists 1 or more revisions."""
-        return self._recipe[3] == 0
-
-    def refine(self, seen, referenced):
-        """Create a new search by refining this search.
-
-        :param seen: Revisions that have been satisfied.
-        :param referenced: Revision references observed while satisfying some
-            of this search.
-        """
-        start = self._recipe[1]
-        exclude = self._recipe[2]
-        count = self._recipe[3]
-        keys = self.get_keys()
-        # New heads = referenced + old heads - seen things - exclude
-        pending_refs = set(referenced)
-        pending_refs.update(start)
-        pending_refs.difference_update(seen)
-        pending_refs.difference_update(exclude)
-        # New exclude = old exclude + satisfied heads
-        seen_heads = start.intersection(seen)
-        exclude.update(seen_heads)
-        # keys gets seen removed
-        keys = keys - seen
-        # length is reduced by len(seen)
-        count -= len(seen)
-        return SearchResult(pending_refs, exclude, count, keys)
-
-
-class PendingAncestryResult(AbstractSearchResult):
-    """A search result that will reconstruct the ancestry for some graph heads.
-
-    Unlike SearchResult, this doesn't hold the complete search result in
-    memory, it just holds a description of how to generate it.
-    """
-
-    def __init__(self, heads, repo):
-        """Constructor.
-
-        :param heads: an iterable of graph heads.
-        :param repo: a repository to use to generate the ancestry for the given
-            heads.
-        """
-        self.heads = frozenset(heads)
-        self.repo = repo
-
-    def __repr__(self):
-        if len(self.heads) > 5:
-            heads_repr = repr(list(self.heads)[:5])[:-1]
-            heads_repr += ', <%d more>...]' % (len(self.heads) - 5,)
-        else:
-            heads_repr = repr(self.heads)
-        return '<%s heads:%s repo:%r>' % (
-            self.__class__.__name__, heads_repr, self.repo)
-
-    def get_recipe(self):
-        """Return a recipe that can be used to replay this search.
-
-        The recipe allows reconstruction of the same results at a later date.
-
-        :seealso SearchResult.get_recipe:
-
-        :return: A tuple ('proxy-search', start_keys_set, set(), -1)
-            To recreate this result, create a PendingAncestryResult with the
-            start_keys_set.
-        """
-        return ('proxy-search', self.heads, set(), -1)
-
-    def get_network_struct(self):
-        parts = ['ancestry-of']
-        parts.extend(self.heads)
-        return parts
-
-    def get_keys(self):
-        """See SearchResult.get_keys.
-
-        Returns all the keys for the ancestry of the heads, excluding
-        NULL_REVISION.
-        """
-        return self._get_keys(self.repo.get_graph())
-
-    def _get_keys(self, graph):
-        NULL_REVISION = revision.NULL_REVISION
-        keys = [key for (key, parents) in graph.iter_ancestry(self.heads)
-                if key != NULL_REVISION and parents is not None]
-        return keys
-
-    def is_empty(self):
-        """Return false if the search lists 1 or more revisions."""
-        if revision.NULL_REVISION in self.heads:
-            return len(self.heads) == 1
-        else:
-            return len(self.heads) == 0
-
-    def refine(self, seen, referenced):
-        """Create a new search by refining this search.
-
-        :param seen: Revisions that have been satisfied.
-        :param referenced: Revision references observed while satisfying some
-            of this search.
-        """
-        referenced = self.heads.union(referenced)
-        return PendingAncestryResult(referenced - seen, self.repo)
-
-
-class EmptySearchResult(AbstractSearchResult):
-    """An empty search result."""
-
-    def is_empty(self):
-        return True
-    
-
-class EverythingResult(AbstractSearchResult):
-    """A search result that simply requests everything in the repository."""
-
-    def __init__(self, repo):
-        self._repo = repo
-
-    def __repr__(self):
-        return '%s(%r)' % (self.__class__.__name__, self._repo)
-
-    def get_recipe(self):
-        raise NotImplementedError(self.get_recipe)
-
-    def get_network_struct(self):
-        return ('everything',)
-
-    def get_keys(self):
-        if 'evil' in debug.debug_flags:
-            from bzrlib import remote
-            if isinstance(self._repo, remote.RemoteRepository):
-                # warn developers (not users) not to do this
-                trace.mutter_callsite(
-                    2, "EverythingResult(RemoteRepository).get_keys() is slow.")
-        return self._repo.all_revision_ids()
-
-    def is_empty(self):
-        # It's ok for this to wrongly return False: the worst that can happen
-        # is that RemoteStreamSource will initiate a get_stream on an empty
-        # repository.  And almost all repositories are non-empty.
-        return False
-
-    def refine(self, seen, referenced):
-        heads = set(self._repo.all_revision_ids())
-        heads.difference_update(seen)
-        heads.update(referenced)
-        return PendingAncestryResult(heads, self._repo)
-
-
-class EverythingNotInOther(AbstractSearch):
-    """Find all revisions in that are in one repo but not the other."""
-
-    def __init__(self, to_repo, from_repo, find_ghosts=False):
-        self.to_repo = to_repo
-        self.from_repo = from_repo
-        self.find_ghosts = find_ghosts
-
-    def execute(self):
-        return self.to_repo.search_missing_revision_ids(
-            self.from_repo, find_ghosts=self.find_ghosts)
-
-
-class NotInOtherForRevs(AbstractSearch):
-    """Find all revisions missing in one repo for a some specific heads."""
-
-    def __init__(self, to_repo, from_repo, required_ids, if_present_ids=None,
-            find_ghosts=False, limit=None):
-        """Constructor.
-
-        :param required_ids: revision IDs of heads that must be found, or else
-            the search will fail with NoSuchRevision.  All revisions in their
-            ancestry not already in the other repository will be included in
-            the search result.
-        :param if_present_ids: revision IDs of heads that may be absent in the
-            source repository.  If present, then their ancestry not already
-            found in other will be included in the search result.
-        :param limit: maximum number of revisions to fetch
-        """
-        self.to_repo = to_repo
-        self.from_repo = from_repo
-        self.find_ghosts = find_ghosts
-        self.required_ids = required_ids
-        self.if_present_ids = if_present_ids
-        self.limit = limit
-
-    def __repr__(self):
-        if len(self.required_ids) > 5:
-            reqd_revs_repr = repr(list(self.required_ids)[:5])[:-1] + ', ...]'
-        else:
-            reqd_revs_repr = repr(self.required_ids)
-        if self.if_present_ids and len(self.if_present_ids) > 5:
-            ifp_revs_repr = repr(list(self.if_present_ids)[:5])[:-1] + ', ...]'
-        else:
-            ifp_revs_repr = repr(self.if_present_ids)
-
-        return ("<%s from:%r to:%r find_ghosts:%r req'd:%r if-present:%r"
-                "limit:%r>") % (
-                self.__class__.__name__, self.from_repo, self.to_repo,
-                self.find_ghosts, reqd_revs_repr, ifp_revs_repr,
-                self.limit)
-
-    def execute(self):
-        return self.to_repo.search_missing_revision_ids(
-            self.from_repo, revision_ids=self.required_ids,
-            if_present_ids=self.if_present_ids, find_ghosts=self.find_ghosts,
-            limit=self.limit)
-
-
 def invert_parent_map(parent_map):
     """Given a map from child => parents, create a map of parent=>children"""
     child_map = {}
@@ -1939,145 +1609,6 @@
     return child_map
 
 
-def _find_possible_heads(parent_map, tip_keys, depth):
-    """Walk backwards (towards children) through the parent_map.
-
-    This finds 'heads' that will hopefully succinctly describe our search
-    graph.
-    """
-    child_map = invert_parent_map(parent_map)
-    heads = set()
-    current_roots = tip_keys
-    walked = set(current_roots)
-    while current_roots and depth > 0:
-        depth -= 1
-        children = set()
-        children_update = children.update
-        for p in current_roots:
-            # Is it better to pre- or post- filter the children?
-            try:
-                children_update(child_map[p])
-            except KeyError:
-                heads.add(p)
-        # If we've seen a key before, we don't want to walk it again. Note that
-        # 'children' stays relatively small while 'walked' grows large. So
-        # don't use 'difference_update' here which has to walk all of 'walked'.
-        # '.difference' is smart enough to walk only children and compare it to
-        # walked.
-        children = children.difference(walked)
-        walked.update(children)
-        current_roots = children
-    if current_roots:
-        # We walked to the end of depth, so these are the new tips.
-        heads.update(current_roots)
-    return heads
-
-
-def _run_search(parent_map, heads, exclude_keys):
-    """Given a parent map, run a _BreadthFirstSearcher on it.
-
-    Start at heads, walk until you hit exclude_keys. As a further improvement,
-    watch for any heads that you encounter while walking, which means they were
-    not heads of the search.
-
-    This is mostly used to generate a succinct recipe for how to walk through
-    most of parent_map.
-
-    :return: (_BreadthFirstSearcher, set(heads_encountered_by_walking))
-    """
-    g = Graph(DictParentsProvider(parent_map))
-    s = g._make_breadth_first_searcher(heads)
-    found_heads = set()
-    while True:
-        try:
-            next_revs = s.next()
-        except StopIteration:
-            break
-        for parents in s._current_parents.itervalues():
-            f_heads = heads.intersection(parents)
-            if f_heads:
-                found_heads.update(f_heads)
-        stop_keys = exclude_keys.intersection(next_revs)
-        if stop_keys:
-            s.stop_searching_any(stop_keys)
-    for parents in s._current_parents.itervalues():
-        f_heads = heads.intersection(parents)
-        if f_heads:
-            found_heads.update(f_heads)
-    return s, found_heads
-
-
-def limited_search_result_from_parent_map(parent_map, missing_keys, tip_keys,
-                                          depth):
-    """Transform a parent_map that is searching 'tip_keys' into an
-    approximate SearchResult.
-
-    We should be able to generate a SearchResult from a given set of starting
-    keys, that covers a subset of parent_map that has the last step pointing at
-    tip_keys. This is to handle the case that really-long-searches shouldn't be
-    started from scratch on each get_parent_map request, but we *do* want to
-    filter out some of the keys that we've already seen, so we don't get
-    information that we already know about on every request.
-
-    The server will validate the search (that starting at start_keys and
-    stopping at stop_keys yields the exact key_count), so we have to be careful
-    to give an exact recipe.
-
-    Basic algorithm is:
-        1) Invert parent_map to get child_map (todo: have it cached and pass it
-           in)
-        2) Starting at tip_keys, walk towards children for 'depth' steps.
-        3) At that point, we have the 'start' keys.
-        4) Start walking parent_map from 'start' keys, counting how many keys
-           are seen, and generating stop_keys for anything that would walk
-           outside of the parent_map.
-
-    :param parent_map: A map from {child_id: (parent_ids,)}
-    :param missing_keys: parent_ids that we know are unavailable
-    :param tip_keys: the revision_ids that we are searching
-    :param depth: How far back to walk.
-    """
-    if not parent_map:
-        # No search to send, because we haven't done any searching yet.
-        return [], [], 0
-    heads = _find_possible_heads(parent_map, tip_keys, depth)
-    s, found_heads = _run_search(parent_map, heads, set(tip_keys))
-    _, start_keys, exclude_keys, key_count = s.get_result().get_recipe()
-    if found_heads:
-        # Anything in found_heads are redundant start_keys, we hit them while
-        # walking, so we can exclude them from the start list.
-        start_keys = set(start_keys).difference(found_heads)
-    return start_keys, exclude_keys, key_count
-
-
-def search_result_from_parent_map(parent_map, missing_keys):
-    """Transform a parent_map into SearchResult information."""
-    if not parent_map:
-        # parent_map is empty or None, simple search result
-        return [], [], 0
-    # start_set is all the keys in the cache
-    start_set = set(parent_map)
-    # result set is all the references to keys in the cache
-    result_parents = set()
-    for parents in parent_map.itervalues():
-        result_parents.update(parents)
-    stop_keys = result_parents.difference(start_set)
-    # We don't need to send ghosts back to the server as a position to
-    # stop either.
-    stop_keys.difference_update(missing_keys)
-    key_count = len(parent_map)
-    if (revision.NULL_REVISION in result_parents
-        and revision.NULL_REVISION in missing_keys):
-        # If we pruned NULL_REVISION from the stop_keys because it's also
-        # in our cache of "missing" keys we need to increment our key count
-        # by 1, because the reconsitituted SearchResult on the server will
-        # still consider NULL_REVISION to be an included key.
-        key_count += 1
-    included_keys = start_set.intersection(result_parents)
-    start_set.difference_update(included_keys)
-    return start_set, stop_keys, key_count
-
-
 def collapse_linear_regions(parent_map):
     """Collapse regions of the graph that are 'linear'.
 

=== modified file 'bzrlib/groupcompress.py'
--- bzrlib/groupcompress.py	2011-05-31 13:52:18 +0000
+++ bzrlib/groupcompress.py	2011-12-19 13:23:58 +0000
@@ -16,12 +16,10 @@
 
 """Core compression logic for compressing streams of related files."""
 
+from __future__ import absolute_import
+
 import time
 import zlib
-try:
-    import pylzma
-except ImportError:
-    pylzma = None
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
@@ -39,6 +37,7 @@
     )
 
 from bzrlib.repofmt import pack_repo
+from bzrlib.i18n import gettext
 """)
 
 from bzrlib.btree_index import BTreeBuilder
@@ -56,8 +55,6 @@
 # groupcompress blocks.
 BATCH_SIZE = 2**16
 
-_USE_LZMA = False and (pylzma is not None)
-
 # osutils.sha_string('')
 _null_sha1 = 'da39a3ee5e6b4b0d3255bfef95601890afd80709'
 
@@ -152,6 +149,7 @@
                 self._content = ''
             elif self._compressor_name == 'lzma':
                 # We don't do partial lzma decomp yet
+                import pylzma
                 self._content = pylzma.decompress(z_content)
             elif self._compressor_name == 'zlib':
                 # Start a zlib decompressor
@@ -298,16 +296,6 @@
         self._content = content
         self._z_content_chunks = None
 
-    def _create_z_content_using_lzma(self):
-        if self._content_chunks is not None:
-            self._content = ''.join(self._content_chunks)
-            self._content_chunks = None
-        if self._content is None:
-            raise AssertionError('Nothing to compress')
-        z_content = pylzma.compress(self._content)
-        self._z_content_chunks = (z_content,)
-        self._z_content_length = len(z_content)
-
     def _create_z_content_from_chunks(self, chunks):
         compressor = zlib.compressobj(zlib.Z_DEFAULT_COMPRESSION)
         # Peak in this point is 1 fulltext, 1 compressed text, + zlib overhead
@@ -321,9 +309,6 @@
     def _create_z_content(self):
         if self._z_content_chunks is not None:
             return
-        if _USE_LZMA:
-            self._create_z_content_using_lzma()
-            return
         if self._content_chunks is not None:
             chunks = self._content_chunks
         else:
@@ -333,10 +318,7 @@
     def to_chunks(self):
         """Create the byte stream as a series of 'chunks'"""
         self._create_z_content()
-        if _USE_LZMA:
-            header = self.GCB_LZ_HEADER
-        else:
-            header = self.GCB_HEADER
+        header = self.GCB_HEADER
         chunks = ['%s%d\n%d\n'
                   % (header, self._z_content_length, self._content_length),
                  ]
@@ -1754,8 +1736,8 @@
                 raise errors.RevisionNotPresent(record.key, self)
             if random_id:
                 if record.key in inserted_keys:
-                    trace.note('Insert claimed random_id=True,'
-                               ' but then inserted %r two times', record.key)
+                    trace.note(gettext('Insert claimed random_id=True,'
+                               ' but then inserted %r two times'), record.key)
                     continue
                 inserted_keys.add(record.key)
             if reuse_blocks:

=== modified file 'bzrlib/hashcache.py'
--- bzrlib/hashcache.py	2010-05-31 21:31:18 +0000
+++ bzrlib/hashcache.py	2011-12-18 15:28:38 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 # TODO: Up-front, stat all files in order and remove those which are deleted or
 # out-of-date.  Don't actually re-read them until they're needed.  That ought
 # to bring all the inodes into core so that future stats to them are fast, and

=== modified file 'bzrlib/help.py'
--- bzrlib/help.py	2011-08-09 16:51:13 +0000
+++ bzrlib/help.py	2011-12-18 15:28:38 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 # TODO: Some way to get a list of external commands (defined by shell
 # scripts) so that they can be included in the help listing as well.
 # It should be enough to just list the plugin directory and look for

=== modified file 'bzrlib/help_topics/__init__.py'
--- bzrlib/help_topics/__init__.py	2011-08-21 09:24:01 +0000
+++ bzrlib/help_topics/__init__.py	2011-12-18 12:46:49 +0000
@@ -33,13 +33,14 @@
 rendering on the screen naturally.
 """
 
-import sys
+from __future__ import absolute_import
 
 import bzrlib
 from bzrlib import (
     config,
     osutils,
     registry,
+    i18n,
     )
 
 
@@ -330,6 +331,10 @@
                will be a pickle.
 --coverage     Generate line coverage report in the specified directory.
 
+-Oname=value   Override the ``name`` config option setting it to ``value`` for
+               the duration of the command.  This can be used multiple times if
+               several options need to be overridden.
+
 See http://doc.bazaar.canonical.com/developers/profiling.html for more
 information on profiling.
 
@@ -617,6 +622,8 @@
 BZR_PDB             Control whether to launch a debugger on error.
 BZR_SIGQUIT_PDB     Control whether SIGQUIT behaves normally or invokes a
                     breakin debugger.
+BZR_TEXTUI_INPUT    Force console input mode for prompts to line-based (instead
+                    of char-based).
 =================== ===========================================================
 """
 
@@ -762,6 +769,9 @@
                         'Information on configuration and log files')
 topic_registry.register_lazy('hooks', 'bzrlib.hooks', 'hooks_help_text',
                         'Points at which custom processing can be added')
+topic_registry.register_lazy('location-alias', 'bzrlib.directory_service',
+                        'AliasDirectory.help_text',
+                        'Aliases for remembered locations')
 
 # Load some of the help topics from files. Note that topics which reproduce API
 # details will tend to skew (quickly usually!) so please seek other solutions
@@ -774,8 +784,6 @@
                         'Types of conflicts and what to do about them')
 topic_registry.register('debug-flags', _load_from_file,
                         'Options to show or record debug information')
-topic_registry.register('location-alias', _load_from_file,
-                        'Aliases for remembered locations')
 topic_registry.register('log-formats', _load_from_file,
                         'Details on the logging formats available')
 topic_registry.register('url-special-chars', _load_from_file,
@@ -879,6 +887,8 @@
         result += _format_see_also(additional_see_also)
         if plain:
             result = help_as_plain_text(result)
+        i18n.install()
+        result = i18n.gettext_per_paragraph(result)
         return result
 
     def get_help_topic(self):

=== modified file 'bzrlib/help_topics/en/configuration.txt'
--- bzrlib/help_topics/en/configuration.txt	2011-08-19 22:26:03 +0000
+++ bzrlib/help_topics/en/configuration.txt	2011-11-14 16:36:31 +0000
@@ -221,7 +221,7 @@
 ~~~~~~~~~~~~~~
 
 An ini file has three types of contructs: section headers, section
-variables and comments.
+options and comments.
 
 Comments
 ^^^^^^^^
@@ -240,9 +240,9 @@
 
 The only valid section headers for bazaar.conf currently are [DEFAULT] and
 [ALIASES].  Section headers are case sensitive. The default section provides for
-setting variables which can be overridden with the branch config file.
+setting options which can be overridden with the branch config file.
 
-For ``locations.conf``, the variables from the section with the
+For ``locations.conf``, the options from the section with the
 longest matching section header are used to the exclusion of other
 potentially valid section headers. A section header uses the path for
 the branch as the section header. Some examples include::
@@ -251,29 +251,27 @@
     [/home/jdoe/branches/]
 
 
-Section variables
-^^^^^^^^^^^^^^^^^
+Section options
+^^^^^^^^^^^^^^^
 
-A section variable resides within a section. A section variable contains a
-variable name, an equals sign and a value.  For example::
+A section option resides within a section. A section option contains an
+option name, an equals sign and a value.  For example::
 
     email            = John Doe <jdoe@isp.com>
     gpg_signing_key  = Amy Pond <amy@example.com>
 
-A variable can reference other variables **in the same configuration file** by
-enclosing them in curly brackets::
+A option can reference other options by enclosing them in curly brackets::
 
     my_branch_name = feature_x
     my_server      = bzr+ssh://example.com
     push_location   = {my_server}/project/{my_branch_name}
 
-
-Variable policies
-^^^^^^^^^^^^^^^^^
-
-Variables defined in a section affect the named directory or URL plus
-any locations they contain.  Policies can be used to change how a
-variable value is interpreted for contained locations.  Currently
+Option policies
+^^^^^^^^^^^^^^^
+
+Options defined in a section affect the named directory or URL plus
+any locations they contain.  Policies can be used to change how an
+option value is interpreted for contained locations.  Currently
 there are three policies available:
 
  none:
@@ -286,7 +284,7 @@
    for contained locations, any additional path components are
    appended to the value.
 
-Policies are specified by keys with names of the form "$var:policy".
+Policies are specified by keys with names of the form "<option_name>:policy".
 For example, to define the push location for a tree of branches, the
 following could be used::
 
@@ -297,6 +295,48 @@
 With this configuration, the push location for ``/top/location/branch1``
 would be ``sftp://example.com/location/branch1``.
 
+Section local options
+^^^^^^^^^^^^^^^^^^^^^
+
+Some options are defined automatically inside a given section and can be
+refered to in this section only. 
+
+For example, the ``appendpath`` policy can be used like this::
+
+  [/home/vila/src/bzr/bugs]
+  mypush = lp:~vila/bzr
+  mypush:policy=appendpath
+
+Using ``relpath`` to achieve the same result is done like this::
+
+  [/home/vila/src/bzr/bugs]
+  mypush = lp:~vila/bzr/{relpath}
+
+In both cases, when used in a directory like
+``/home/vila/src/bzr/bugs/832013-expand-in-stack`` we'll get::
+
+   $ bzr config mypush
+   lp:~vila/bzr/832013-expand-in-stack
+
+Another such option is ``basename`` which can be used like this::
+
+  [/home/vila/src/bzr]
+  mypush = lp:~vila/bzr/{basename}
+
+When used in a directory like
+``/home/vila/src/bzr/bugs/832013-expand-in-stack`` we'll get::
+
+   $ bzr config mypush
+   lp:~vila/bzr/832013-expand-in-stack
+
+Note that ``basename`` here refers to the base name of ``relpath`` which
+itself is defined as the relative path between the section name and the
+location it matches.
+
+When an option is local to a Section, it cannot be referred to from option
+values in any other section from the same ``Store`` nor from any other
+``Store``.
+
 
 The main configuration file, bazaar.conf
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -337,14 +377,14 @@
 of bzr that requires authentication (smtp for example).
 
 The syntax of the file obeys the same rules as the others except for the
-variable policies which don't apply.
+option policies which don't apply.
 
 For more information on the possible uses of the authentication configuration
 file see :doc:`authentication-help`.
 
 
-Common variable options
------------------------
+Common options
+--------------
 
 debug_flags
 ~~~~~~~~~~~

=== modified file 'bzrlib/help_topics/en/debug-flags.txt'
--- bzrlib/help_topics/en/debug-flags.txt	2011-05-27 17:10:05 +0000
+++ bzrlib/help_topics/en/debug-flags.txt	2011-10-10 13:51:29 +0000
@@ -24,6 +24,8 @@
 -Dindex           Trace major index operations.
 -Dknit            Trace knit operations.
 -Dlock            Trace when lockdir locks are taken or released.
+-Dnoretry         If a connection is reset, fail immediately rather than
+                  retrying the request.
 -Dprogress        Trace progress bar operations.
 -Dmem_dump        Dump memory to a file upon an out of memory error.
 -Dmerge           Emit information for debugging merges.

=== removed file 'bzrlib/help_topics/en/location-alias.txt'
--- bzrlib/help_topics/en/location-alias.txt	2010-02-11 07:18:20 +0000
+++ bzrlib/help_topics/en/location-alias.txt	1970-01-01 00:00:00 +0000
@@ -1,19 +0,0 @@
-Location aliases
-================
-
-Bazaar defines several aliases for locations associated with a branch.  These
-can be used with most commands that expect a location, such as `bzr push`.
-
-The aliases are::
-
-  :parent    the parent of this branch
-  :submit    the submit branch for this branch
-  :public    the public location of this branch
-  :bound     the branch this branch is bound to, for bound branches
-  :push      the saved location used for `bzr push` with no arguments
-  :this      this branch
-
-For example, to push to the parent location::
-
-    bzr push :parent
-

=== modified file 'bzrlib/help_topics/en/log-formats.txt'
--- bzrlib/help_topics/en/log-formats.txt	2010-01-03 01:41:00 +0000
+++ bzrlib/help_topics/en/log-formats.txt	2011-08-31 11:39:56 +0000
@@ -19,6 +19,7 @@
   branch nick             yes            -             -
   foreign vcs properties  yes            yes           -
   preferred levels        all            1             1
+  digital signature       optional       -             -
 
 The default format is ``long``. To change this, define the ``log_format``
 setting in the ``[DEFAULT]`` section of ``bazaar.conf`` like this (say)::

=== modified file 'bzrlib/hooks.py'
--- bzrlib/hooks.py	2011-06-14 10:07:16 +0000
+++ bzrlib/hooks.py	2011-12-24 10:10:59 +0000
@@ -14,9 +14,10 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-
 """Support for plugin hooking logic."""
 
+from __future__ import absolute_import
+
 from bzrlib import (
     registry,
     symbol_versioning,
@@ -30,6 +31,7 @@
     errors,
     pyutils,
     )
+from bzrlib.i18n import gettext
 """)
 
 
@@ -70,7 +72,7 @@
 
 _builtin_known_hooks = (
     ('bzrlib.branch', 'Branch.hooks', 'BranchHooks'),
-    ('bzrlib.bzrdir', 'BzrDir.hooks', 'BzrDirHooks'),
+    ('bzrlib.controldir', 'ControlDir.hooks', 'ControlDirHooks'),
     ('bzrlib.commands', 'Command.hooks', 'CommandHooks'),
     ('bzrlib.config', 'ConfigHooks', '_ConfigHooks'),
     ('bzrlib.info', 'hooks', 'InfoHooks'),
@@ -81,6 +83,7 @@
     ('bzrlib.smart.client', '_SmartClient.hooks', 'SmartClientHooks'),
     ('bzrlib.smart.server', 'SmartTCPServer.hooks', 'SmartServerHooks'),
     ('bzrlib.status', 'hooks', 'StatusHooks'),
+    ('bzrlib.transport', 'Transport.hooks', 'TransportHooks'),
     ('bzrlib.version_info_formats.format_rio', 'RioVersionInfoBuilder.hooks',
         'RioVersionInfoBuilderHooks'),
     ('bzrlib.merge_directive', 'BaseMergeDirective.hooks',
@@ -126,6 +129,7 @@
         """
         dict.__init__(self)
         self._callable_names = {}
+        self._lazy_callable_names = {}
         self._module = module
         self._member_name = member_name
 
@@ -195,7 +199,15 @@
         the code names are rarely meaningful for end users and this is not
         intended for debugging.
         """
-        return self._callable_names.get(a_callable, "No hook name")
+        name = self._callable_names.get(a_callable, None)
+        if name is None and a_callable is not None:
+            name = self._lazy_callable_names.get((a_callable.__module__,
+                                                  a_callable.__name__),
+                                                 None)
+        if name is None:
+            return 'No hook name'
+        return name
+
 
     def install_named_hook_lazy(self, hook_name, callable_module,
         callable_member, name):
@@ -220,6 +232,8 @@
                 self)
         else:
             hook_lazy(callable_module, callable_member, name)
+        if name is not None:
+            self.name_hook_lazy(callable_module, callable_member, name)
 
     def install_named_hook(self, hook_name, a_callable, name):
         """Install a_callable in to the hook hook_name, and label it name.
@@ -265,6 +279,10 @@
         """Associate name with a_callable to show users what is running."""
         self._callable_names[a_callable] = name
 
+    def name_hook_lazy(self, callable_module, callable_member, callable_name):
+        self._lazy_callable_names[(callable_module, callable_member)]= \
+            callable_name
+
 
 class HookPoint(object):
     """A single hook that clients can register to be called back when it fires.
@@ -310,10 +328,10 @@
             introduced_string = _format_version_tuple(self.introduced)
         else:
             introduced_string = 'unknown'
-        strings.append('Introduced in: %s' % introduced_string)
+        strings.append(gettext('Introduced in: %s') % introduced_string)
         if self.deprecated:
             deprecated_string = _format_version_tuple(self.deprecated)
-            strings.append('Deprecated in: %s' % deprecated_string)
+            strings.append(gettext('Deprecated in: %s') % deprecated_string)
         strings.append('')
         strings.extend(textwrap.wrap(self.__doc__,
             break_long_words=False))

=== modified file 'bzrlib/i18n.py'
--- bzrlib/i18n.py	2011-08-25 10:33:14 +0000
+++ bzrlib/i18n.py	2011-12-19 13:23:58 +0000
@@ -22,10 +22,13 @@
 
 """i18n and l10n support for Bazaar."""
 
+from __future__ import absolute_import
+
 import gettext as _gettext
 import os
 import sys
 
+
 _translations = None
 
 
@@ -34,6 +37,7 @@
     
     :returns: translated message as unicode.
     """
+    install()
     return _translations.ugettext(message)
 
 
@@ -46,6 +50,7 @@
 
     :returns: translated message as unicode.
     """
+    install()
     return _translations.ungettext(singular, plural, number)
 
 
@@ -59,6 +64,7 @@
 
     :returns: concatenated translated message as unicode.
     """
+    install()
     paragraphs = message.split(u'\n\n')
     ugettext = _translations.ugettext
     # Be careful not to translate the empty string -- it holds the
@@ -66,37 +72,78 @@
     return u'\n\n'.join(ugettext(p) if p else u'' for p in paragraphs)
 
 
+def disable_i18n():
+    """Do not allow i18n to be enabled.  Useful for third party users
+    of bzrlib."""
+    global _translations
+    _translations = _gettext.NullTranslations()
+
+
 def installed():
+    """Returns whether translations are in use or not."""
     return _translations is not None
 
 
 def install(lang=None):
+    """Enables gettext translations in bzr."""
     global _translations
+    if installed():
+        return
+    _translations = install_translations(lang)
+
+
+def install_translations(lang=None, domain='bzr', locale_base=None):
+    """Create a gettext translation object.
+    
+    :param lang: language to install.
+    :param domain: translation domain to install.
+    :param locale_base: plugins can specify their own directory.
+
+    :returns: a gettext translations object to use
+    """
     if lang is None:
         lang = _get_current_locale()
     if lang is not None:
         languages = lang.split(':')
     else:
         languages = None
-    _translations = _gettext.translation(
-            'bzr',
-            localedir=_get_locale_dir(),
+    translation = _gettext.translation(
+            domain,
+            localedir=_get_locale_dir(locale_base),
             languages=languages,
             fallback=True)
+    return translation
+
+
+def add_fallback(fallback):
+    """
+    Add a fallback translations object.  Typically used by plugins.
+
+    :param fallback: gettext.GNUTranslations object
+    """
+    install()
+    _translations.add_fallback(fallback)
 
 
 def uninstall():
+    """Disables gettext translations."""
     global _translations
     _translations = None
 
 
-def _get_locale_dir():
+def _get_locale_dir(base):
+    """Returns directory to find .mo translations file in, either local or system
+
+    :param base: plugins can specify their own local directory
+    """
     if hasattr(sys, 'frozen'):
-        base = os.path.dirname(
+        if base is None:
+            base = os.path.dirname(
                 unicode(sys.executable, sys.getfilesystemencoding()))
         return os.path.join(base, u'locale')
     else:
-        base = os.path.dirname(unicode(__file__, sys.getfilesystemencoding()))
+        if base is None:
+            base = os.path.dirname(unicode(__file__, sys.getfilesystemencoding()))
         dirpath = os.path.realpath(os.path.join(base, u'locale'))
         if os.path.exists(dirpath):
             return dirpath
@@ -145,3 +192,16 @@
         if lang:
             return lang
     return None
+
+
+def load_plugin_translations(domain):
+    """Load the translations for a specific plugin.
+
+    :param domain: Gettext domain name (usually 'bzr-PLUGINNAME')
+    """
+    locale_base = os.path.dirname(
+        unicode(__file__, sys.getfilesystemencoding()))
+    translation = install_translations(domain=domain,
+        locale_base=locale_base)
+    add_fallback(translation)
+    return translation

=== modified file 'bzrlib/identitymap.py'
--- bzrlib/identitymap.py	2010-04-30 11:03:59 +0000
+++ bzrlib/identitymap.py	2011-12-19 13:23:58 +0000
@@ -17,6 +17,7 @@
 
 """This module provides an IdentityMap."""
 
+from __future__ import absolute_import
 
 from bzrlib import (
     errors,

=== modified file 'bzrlib/ignores.py'
--- bzrlib/ignores.py	2011-04-07 10:36:24 +0000
+++ bzrlib/ignores.py	2011-12-19 14:40:24 +0000
@@ -16,6 +16,8 @@
 
 """Lists of ignore files, etc."""
 
+from __future__ import absolute_import
+
 import errno
 import os
 from cStringIO import StringIO
@@ -27,9 +29,8 @@
     atomicfile,
     config,
     globbing,
+    trace,
     )
-
-from trace import warning
 """)
 
 # ~/.bazaar/ignore will be filled out using
@@ -72,7 +73,8 @@
                 unicode_lines.append(line.decode('utf-8'))
             except UnicodeDecodeError:
                 # report error about line (idx+1)
-                warning('.bzrignore: On Line #%d, malformed utf8 character. '
+                trace.warning(
+                        '.bzrignore: On Line #%d, malformed utf8 character. '
                         'Ignoring line.' % (line_number+1))
 
     # Append each line to ignore list if it's not a comment line

=== modified file 'bzrlib/index.py'
--- bzrlib/index.py	2011-07-05 06:41:44 +0000
+++ bzrlib/index.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,8 @@
 
 """Indexing facilities."""
 
+from __future__ import absolute_import
+
 __all__ = [
     'CombinedGraphIndex',
     'GraphIndex',

=== modified file 'bzrlib/info.py'
--- bzrlib/info.py	2011-04-17 09:57:36 +0000
+++ bzrlib/info.py	2011-12-18 15:28:38 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 __all__ = ['show_bzrdir_info']
 
 from cStringIO import StringIO
@@ -22,6 +24,7 @@
 
 from bzrlib import (
     bzrdir,
+    controldir,
     errors,
     hooks as _mod_hooks,
     osutils,
@@ -76,9 +79,9 @@
         return ["  %*s: %s\n" % (max_len, l, u) for l, u in self.locs ]
 
 
-def gather_location_info(repository, branch=None, working=None):
+def gather_location_info(repository=None, branch=None, working=None,
+        control=None):
     locs = {}
-    repository_path = repository.user_url
     if branch is not None:
         branch_path = branch.user_url
         master_path = branch.get_bound_location()
@@ -87,6 +90,11 @@
     else:
         branch_path = None
         master_path = None
+        try:
+            if control is not None and control.get_branch_reference():
+                locs['checkout of branch'] = control.get_branch_reference()
+        except NotBranchError:
+            pass
     if working:
         working_path = working.user_url
         if working_path != branch_path:
@@ -105,22 +113,29 @@
             locs['branch root'] = branch_path
     else:
         working_path = None
-        if repository.is_shared():
+        if repository is not None and repository.is_shared():
             # lightweight checkout of branch in shared repository
             if branch_path is not None:
                 locs['repository branch'] = branch_path
         elif branch_path is not None:
             # standalone
             locs['branch root'] = branch_path
-            if master_path != branch_path:
-                locs['bound to branch'] = master_path
+        elif repository is not None:
+            locs['repository'] = repository.user_url
+        elif control is not None:
+            locs['control directory'] = control.user_url
         else:
-            locs['repository'] = repository_path
-    if repository.is_shared():
+            # Really, at least a control directory should be
+            # passed in for this method to be useful.
+            pass
+        if master_path != branch_path:
+            locs['bound to branch'] = master_path
+    if repository is not None and repository.is_shared():
         # lightweight checkout of branch in shared repository
-        locs['shared repository'] = repository_path
-    order = ['light checkout root', 'repository checkout root',
-             'checkout root', 'checkout of branch', 'shared repository',
+        locs['shared repository'] = repository.user_url
+    order = ['control directory', 'light checkout root',
+             'repository checkout root', 'checkout root',
+             'checkout of branch', 'shared repository',
              'repository', 'repository branch', 'branch root',
              'bound to branch']
     return [(n, locs[n]) for n in order if n in locs]
@@ -158,6 +173,14 @@
         outfile.writelines(locs.get_lines())
 
 
+def _show_control_dir_info(control, outfile):
+    """Show control dir information."""
+    if control._format.colocated_branches:
+        outfile.write('\n')
+        outfile.write('Control directory:\n')
+        outfile.write('         %d branches\n' % len(control.list_branches()))
+
+
 def _show_format_info(control=None, repository=None, branch=None,
                       working=None, outfile=None):
     """Show known formats for control, working, branch and repository."""
@@ -221,7 +244,10 @@
     """Show missing revisions in working tree."""
     branch = working.branch
     basis = working.basis_tree()
-    branch_revno, branch_last_revision = branch.last_revision_info()
+    try:
+        branch_revno, branch_last_revision = branch.last_revision_info()
+    except errors.UnsupportedOperation:
+        return
     try:
         tree_last_id = working.get_parent_ids()[0]
     except IndexError:
@@ -268,7 +294,10 @@
 
 def _show_branch_stats(branch, verbose, outfile):
     """Show statistics about a branch."""
-    revno, head = branch.last_revision_info()
+    try:
+        revno, head = branch.last_revision_info()
+    except errors.UnsupportedOperation:
+        return {}
     outfile.write('\n')
     outfile.write('Branch history:\n')
     outfile.write('  %8d revision%s\n' % (revno, plural(revno)))
@@ -320,7 +349,7 @@
     try:
         tree = a_bzrdir.open_workingtree(
             recommend_upgrade=False)
-    except (NoWorkingTree, NotLocalUrl):
+    except (NoWorkingTree, NotLocalUrl, NotBranchError):
         tree = None
         try:
             branch = a_bzrdir.open_branch()
@@ -329,9 +358,8 @@
             try:
                 repository = a_bzrdir.open_repository()
             except NoRepositoryPresent:
-                # Return silently; cmd_info already returned NotBranchError
-                # if no bzrdir could be opened.
-                return
+                lockable = None
+                repository = None
             else:
                 lockable = repository
         else:
@@ -342,12 +370,14 @@
         repository = branch.repository
         lockable = tree
 
-    lockable.lock_read()
+    if lockable is not None:
+        lockable.lock_read()
     try:
         show_component_info(a_bzrdir, repository, branch, tree, verbose,
                             outfile)
     finally:
-        lockable.unlock()
+        if lockable is not None:
+            lockable.unlock()
 
 
 def show_component_info(control, repository, branch=None, working=None,
@@ -359,17 +389,20 @@
         verbose = 1
     if verbose is True:
         verbose = 2
-    layout = describe_layout(repository, branch, working)
+    layout = describe_layout(repository, branch, working, control)
     format = describe_format(control, repository, branch, working)
     outfile.write("%s (format: %s)\n" % (layout, format))
-    _show_location_info(gather_location_info(repository, branch, working),
-                        outfile)
+    _show_location_info(
+        gather_location_info(control=control, repository=repository,
+            branch=branch, working=working),
+        outfile)
     if branch is not None:
         _show_related_info(branch, outfile)
     if verbose == 0:
         return
     _show_format_info(control, repository, branch, working, outfile)
     _show_locking_info(repository, branch, working, outfile)
+    _show_control_dir_info(control, outfile)
     if branch is not None:
         _show_missing_revisions_branch(branch, outfile)
     if working is not None:
@@ -387,13 +420,21 @@
     _show_repository_stats(repository, stats, outfile)
 
 
-def describe_layout(repository=None, branch=None, tree=None):
+def describe_layout(repository=None, branch=None, tree=None, control=None):
     """Convert a control directory layout into a user-understandable term
 
     Common outputs include "Standalone tree", "Repository branch" and
     "Checkout".  Uncommon outputs include "Unshared repository with trees"
     and "Empty control directory"
     """
+    if branch is None and control is not None:
+        try:
+            branch_reference = control.get_branch_reference()
+        except NotBranchError:
+            pass
+        else:
+            if branch_reference is not None:
+                return "Dangling branch reference"
     if repository is None:
         return 'Empty control directory'
     if branch is None and tree is None:
@@ -445,10 +486,10 @@
         branch.user_url != tree.user_url):
         branch = None
         repository = None
-    non_aliases = set(bzrdir.format_registry.keys())
-    non_aliases.difference_update(bzrdir.format_registry.aliases())
+    non_aliases = set(controldir.format_registry.keys())
+    non_aliases.difference_update(controldir.format_registry.aliases())
     for key in non_aliases:
-        format = bzrdir.format_registry.make_bzrdir(key)
+        format = controldir.format_registry.make_bzrdir(key)
         if isinstance(format, bzrdir.BzrDirMetaFormat1):
             if (tree and format.workingtree_format !=
                 tree._format):
@@ -466,7 +507,7 @@
         return 'unnamed'
     candidates.sort()
     new_candidates = [c for c in candidates if not
-        bzrdir.format_registry.get_info(c).hidden]
+        controldir.format_registry.get_info(c).hidden]
     if len(new_candidates) > 0:
         # If there are any non-hidden formats that match, only return those to
         # avoid listing hidden formats except when only a hidden format will

=== modified file 'bzrlib/inspect_for_copy.py'
--- bzrlib/inspect_for_copy.py	2009-03-23 14:59:43 +0000
+++ bzrlib/inspect_for_copy.py	2011-12-19 13:23:58 +0000
@@ -25,6 +25,8 @@
 overhead.
 """
 
+from __future__ import absolute_import
+
 
 ####
 # These are the only 2 functions that 'copy' needs from 'inspect'

=== modified file 'bzrlib/inter.py'
--- bzrlib/inter.py	2011-05-19 09:32:38 +0000
+++ bzrlib/inter.py	2011-12-19 13:23:58 +0000
@@ -14,9 +14,9 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-
 """Inter-object utility class."""
 
+from __future__ import absolute_import
 
 from bzrlib.errors import NoCompatibleInter
 

=== modified file 'bzrlib/intset.py'
--- bzrlib/intset.py	2009-03-23 14:59:43 +0000
+++ bzrlib/intset.py	2011-12-18 15:28:38 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 # Author: Martin Pool <mbp@canonical.com>
 
 

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2011-08-12 09:49:24 +0000
+++ bzrlib/inventory.py	2011-12-18 12:46:49 +0000
@@ -23,6 +23,8 @@
 # But those depend on its position within a particular inventory, and
 # it would be nice not to need to hold the backpointer here.
 
+from __future__ import absolute_import
+
 # This should really be an id randomly assigned when the tree is
 # created, but it's not for now.
 ROOT_ID = "TREE_ROOT"

=== modified file 'bzrlib/inventory_delta.py'
--- bzrlib/inventory_delta.py	2009-08-13 00:20:29 +0000
+++ bzrlib/inventory_delta.py	2011-12-19 13:23:58 +0000
@@ -22,6 +22,8 @@
  - InventoryDeltaSerializer - object to read/write inventory deltas.
 """
 
+from __future__ import absolute_import
+
 __all__ = ['InventoryDeltaSerializer']
 
 from bzrlib import errors

=== modified file 'bzrlib/iterablefile.py'
--- bzrlib/iterablefile.py	2009-03-23 14:59:43 +0000
+++ bzrlib/iterablefile.py	2011-12-18 15:28:38 +0000
@@ -15,6 +15,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
 
 class IterableFileBase(object):
     """Create a file-like object from any iterable"""

=== modified file 'bzrlib/knit.py'
--- bzrlib/knit.py	2011-05-18 16:42:48 +0000
+++ bzrlib/knit.py	2011-12-19 13:23:58 +0000
@@ -51,6 +51,8 @@
 
 """
 
+from __future__ import absolute_import
+
 
 from cStringIO import StringIO
 from itertools import izip
@@ -76,6 +78,7 @@
     )
 
 from bzrlib.repofmt import pack_repo
+from bzrlib.i18n import gettext
 """)
 from bzrlib import (
     annotate,
@@ -1760,7 +1763,7 @@
                         key_records.append((key, details[0]))
                 records_iter = enumerate(self._read_records_iter(key_records))
                 for (key_idx, (key, data, sha_value)) in records_iter:
-                    pb.update('Walking content', key_idx, total)
+                    pb.update(gettext('Walking content'), key_idx, total)
                     compression_parent = build_details[key][1]
                     if compression_parent is None:
                         # fulltext
@@ -1796,7 +1799,7 @@
                 source_keys.add(key)
                 yield line, key
             keys.difference_update(source_keys)
-        pb.update('Walking content', total, total)
+        pb.update(gettext('Walking content'), total, total)
 
     def _make_line_delta(self, delta_seq, new_content):
         """Generate a line delta from delta_seq and new_content."""
@@ -3209,145 +3212,6 @@
                 yield data
 
 
-class _DirectPackAccess(object):
-    """Access to data in one or more packs with less translation."""
-
-    def __init__(self, index_to_packs, reload_func=None, flush_func=None):
-        """Create a _DirectPackAccess object.
-
-        :param index_to_packs: A dict mapping index objects to the transport
-            and file names for obtaining data.
-        :param reload_func: A function to call if we determine that the pack
-            files have moved and we need to reload our caches. See
-            bzrlib.repo_fmt.pack_repo.AggregateIndex for more details.
-        """
-        self._container_writer = None
-        self._write_index = None
-        self._indices = index_to_packs
-        self._reload_func = reload_func
-        self._flush_func = flush_func
-
-    def add_raw_records(self, key_sizes, raw_data):
-        """Add raw knit bytes to a storage area.
-
-        The data is spooled to the container writer in one bytes-record per
-        raw data item.
-
-        :param sizes: An iterable of tuples containing the key and size of each
-            raw data segment.
-        :param raw_data: A bytestring containing the data.
-        :return: A list of memos to retrieve the record later. Each memo is an
-            opaque index memo. For _DirectPackAccess the memo is (index, pos,
-            length), where the index field is the write_index object supplied
-            to the PackAccess object.
-        """
-        if type(raw_data) is not str:
-            raise AssertionError(
-                'data must be plain bytes was %s' % type(raw_data))
-        result = []
-        offset = 0
-        for key, size in key_sizes:
-            p_offset, p_length = self._container_writer.add_bytes_record(
-                raw_data[offset:offset+size], [])
-            offset += size
-            result.append((self._write_index, p_offset, p_length))
-        return result
-
-    def flush(self):
-        """Flush pending writes on this access object.
-
-        This will flush any buffered writes to a NewPack.
-        """
-        if self._flush_func is not None:
-            self._flush_func()
-            
-    def get_raw_records(self, memos_for_retrieval):
-        """Get the raw bytes for a records.
-
-        :param memos_for_retrieval: An iterable containing the (index, pos,
-            length) memo for retrieving the bytes. The Pack access method
-            looks up the pack to use for a given record in its index_to_pack
-            map.
-        :return: An iterator over the bytes of the records.
-        """
-        # first pass, group into same-index requests
-        request_lists = []
-        current_index = None
-        for (index, offset, length) in memos_for_retrieval:
-            if current_index == index:
-                current_list.append((offset, length))
-            else:
-                if current_index is not None:
-                    request_lists.append((current_index, current_list))
-                current_index = index
-                current_list = [(offset, length)]
-        # handle the last entry
-        if current_index is not None:
-            request_lists.append((current_index, current_list))
-        for index, offsets in request_lists:
-            try:
-                transport, path = self._indices[index]
-            except KeyError:
-                # A KeyError here indicates that someone has triggered an index
-                # reload, and this index has gone missing, we need to start
-                # over.
-                if self._reload_func is None:
-                    # If we don't have a _reload_func there is nothing that can
-                    # be done
-                    raise
-                raise errors.RetryWithNewPacks(index,
-                                               reload_occurred=True,
-                                               exc_info=sys.exc_info())
-            try:
-                reader = pack.make_readv_reader(transport, path, offsets)
-                for names, read_func in reader.iter_records():
-                    yield read_func(None)
-            except errors.NoSuchFile:
-                # A NoSuchFile error indicates that a pack file has gone
-                # missing on disk, we need to trigger a reload, and start over.
-                if self._reload_func is None:
-                    raise
-                raise errors.RetryWithNewPacks(transport.abspath(path),
-                                               reload_occurred=False,
-                                               exc_info=sys.exc_info())
-
-    def set_writer(self, writer, index, transport_packname):
-        """Set a writer to use for adding data."""
-        if index is not None:
-            self._indices[index] = transport_packname
-        self._container_writer = writer
-        self._write_index = index
-
-    def reload_or_raise(self, retry_exc):
-        """Try calling the reload function, or re-raise the original exception.
-
-        This should be called after _DirectPackAccess raises a
-        RetryWithNewPacks exception. This function will handle the common logic
-        of determining when the error is fatal versus being temporary.
-        It will also make sure that the original exception is raised, rather
-        than the RetryWithNewPacks exception.
-
-        If this function returns, then the calling function should retry
-        whatever operation was being performed. Otherwise an exception will
-        be raised.
-
-        :param retry_exc: A RetryWithNewPacks exception.
-        """
-        is_error = False
-        if self._reload_func is None:
-            is_error = True
-        elif not self._reload_func():
-            # The reload claimed that nothing changed
-            if not retry_exc.reload_occurred:
-                # If there wasn't an earlier reload, then we really were
-                # expecting to find changes. We didn't find them, so this is a
-                # hard error
-                is_error = True
-        if is_error:
-            exc_class, exc_value, exc_traceback = retry_exc.exc_info
-            raise exc_class, exc_value, exc_traceback
-
-
 def annotate_knit(knit, revision_id):
     """Annotate a knit with no cached annotations.
 
@@ -3463,7 +3327,7 @@
                 for idx, (sub_key, text, num_lines) in enumerate(
                                                 self._extract_texts(records)):
                     if pb is not None:
-                        pb.update('annotating', idx, len(records))
+                        pb.update(gettext('annotating'), idx, len(records))
                     yield sub_key, text, num_lines
                 for sub_key in ann_keys:
                     text = self._text_cache[sub_key]

=== modified file 'bzrlib/lazy_import.py'
--- bzrlib/lazy_import.py	2012-01-04 19:10:20 +0000
+++ bzrlib/lazy_import.py	2012-01-05 09:50:04 +0000
@@ -41,6 +41,8 @@
 to inherit from them).
 """
 
+from __future__ import absolute_import
+
 
 class ScopeReplacer(object):
     """A lazy object that will replace itself in the appropriate scope.
@@ -195,10 +197,10 @@
         module_path = object.__getattribute__(self, '_module_path')
         module_python_path = '.'.join(module_path)
         if member is not None:
-            module = __import__(module_python_path, scope, scope, [member])
+            module = __import__(module_python_path, scope, scope, [member], level=0)
             return getattr(module, member)
         else:
-            module = __import__(module_python_path, scope, scope, [])
+            module = __import__(module_python_path, scope, scope, [], level=0)
             for path in module_path[1:]:
                 module = getattr(module, path)
 

=== modified file 'bzrlib/lazy_regex.py'
--- bzrlib/lazy_regex.py	2011-06-13 22:39:49 +0000
+++ bzrlib/lazy_regex.py	2011-12-19 13:23:58 +0000
@@ -24,6 +24,8 @@
 be used by existing Python modules that create regexs.
 """
 
+from __future__ import absolute_import
+
 import re
 
 from bzrlib import errors
@@ -71,6 +73,19 @@
             # cleaner message to the user.
             raise errors.InvalidPattern('"' + args[0] + '" ' +str(e))
 
+    def __getstate__(self):
+        """Return the state to use when pickling."""
+        return {
+            "args": self._regex_args,
+            "kwargs": self._regex_kwargs,
+            }
+
+    def __setstate__(self, dict):
+        """Restore from a pickled state."""
+        self._real_regex = None
+        setattr(self, "_regex_args", dict["args"])
+        setattr(self, "_regex_kwargs", dict["kwargs"])
+
     def __getattr__(self, attr):
         """Return a member from the proxied regex object.
 

=== modified file 'bzrlib/library_state.py'
--- bzrlib/library_state.py	2011-03-29 09:50:04 +0000
+++ bzrlib/library_state.py	2011-12-19 13:23:58 +0000
@@ -16,13 +16,25 @@
 
 """The core state needed to make use of bzr is managed here."""
 
+from __future__ import absolute_import
+
 __all__ = [
     'BzrLibraryState',
     ]
 
-import sys
 
 import bzrlib
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+from bzrlib import (
+    cleanup,
+    config,
+    osutils,
+    symbol_versioning,
+    trace,
+    ui,
+    )
+""")
 
 
 class BzrLibraryState(object):
@@ -61,6 +73,9 @@
         """
         self._ui = ui
         self._trace = trace
+        # There is no overrides by default, they are set later when the command
+        # arguments are parsed.
+        self.cmdline_overrides = config.CommandLineStore()
         self.started = False
 
     def __enter__(self):
@@ -69,23 +84,17 @@
         return self # This is bound to the 'as' clause in a with statement.
 
     def _start(self):
-        """Do all initialization.
-        """        
+        """Do all initialization."""
         # NB: This function tweaks so much global state it's hard to test it in
         # isolation within the same interpreter.  It's not reached on normal
         # in-process run_bzr calls.  If it's broken, we expect that
         # TestRunBzrSubprocess may fail.
-        import bzrlib
+        self.cleanups = cleanup.ObjectWithCleanups()
+
         if bzrlib.version_info[3] == 'final':
-            from bzrlib.symbol_versioning import suppress_deprecation_warnings
-            warning_cleanup = suppress_deprecation_warnings(override=True)
-        else:
-            warning_cleanup = None
+            self.cleanups.add_cleanup(
+                symbol_versioning.suppress_deprecation_warnings(override=True))
 
-        import bzrlib.cleanup
-        self.cleanups = bzrlib.cleanup.ObjectWithCleanups()
-        if warning_cleanup:
-            self.cleanups.add_cleanup(warning_cleanup)
         self._trace.__enter__()
 
         self._orig_ui = bzrlib.ui.ui_factory
@@ -98,14 +107,11 @@
 
     def __exit__(self, exc_type, exc_val, exc_tb):
         self.cleanups.cleanup_now()
-        import bzrlib.ui
-        bzrlib.trace._flush_stdout_stderr()
-        bzrlib.trace._flush_trace()
-        import bzrlib.osutils
-        bzrlib.osutils.report_extension_load_failures()
+        trace._flush_stdout_stderr()
+        trace._flush_trace()
+        osutils.report_extension_load_failures()
         self._ui.__exit__(None, None, None)
         self._trace.__exit__(None, None, None)
-        bzrlib.ui.ui_factory = self._orig_ui
-        global global_state
-        global_state = self.saved_state
+        ui.ui_factory = self._orig_ui
+        bzrlib.global_state = self.saved_state
         return False # propogate exceptions.

=== modified file 'bzrlib/lock.py'
--- bzrlib/lock.py	2011-08-17 16:11:26 +0000
+++ bzrlib/lock.py	2011-12-19 13:23:58 +0000
@@ -14,7 +14,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-
 """Locking using OS file locks or file existence.
 
 Note: This method of locking is generally deprecated in favour of LockDir, but
@@ -34,6 +33,8 @@
 unlock() method.
 """
 
+from __future__ import absolute_import
+
 import errno
 import os
 import sys
@@ -46,7 +47,7 @@
     trace,
     )
 from bzrlib.hooks import Hooks
-
+from bzrlib.i18n import gettext
 
 class LockHooks(Hooks):
 
@@ -544,6 +545,6 @@
                 type_name = 'read'
             else:
                 type_name = 'write'
-            trace.note('%r was %s locked again', self, type_name)
+            trace.note(gettext('{0!r} was {1} locked again'), self, type_name)
         self._prev_lock = lock_type
 

=== modified file 'bzrlib/lockable_files.py'
--- bzrlib/lockable_files.py	2011-06-28 21:47:24 +0000
+++ bzrlib/lockable_files.py	2011-12-18 12:46:49 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 import warnings

=== modified file 'bzrlib/lockdir.py'
--- bzrlib/lockdir.py	2011-08-12 12:11:44 +0000
+++ bzrlib/lockdir.py	2011-12-19 13:23:58 +0000
@@ -98,6 +98,8 @@
 but helps protect against colliding host names.
 """
 
+from __future__ import absolute_import
+
 
 # TODO: We sometimes have the problem that our attempt to rename '1234' to
 # 'held' fails because the transport server moves into an existing directory,
@@ -107,7 +109,6 @@
 # the existing locking code and needs a new format of the containing object.
 # -- robertc, mbp 20070628
 
-import errno
 import os
 import time
 
@@ -137,6 +138,7 @@
         )
 from bzrlib.trace import mutter, note
 from bzrlib.osutils import format_delta, rand_chars, get_host_name
+from bzrlib.i18n import gettext
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
@@ -316,7 +318,7 @@
             self.transport.delete(tmpname + self.__INFO_NAME)
             self.transport.rmdir(tmpname)
         except PathError, e:
-            note("error removing pending lock: %s", e)
+            note(gettext("error removing pending lock: %s"), e)
 
     def _create_pending_dir(self):
         tmpname = '%s/%s.tmp' % (self.path, rand_chars(10))
@@ -611,19 +613,20 @@
             new_info = self.peek()
             if new_info is not None and new_info != last_info:
                 if last_info is None:
-                    start = 'Unable to obtain'
+                    start = gettext('Unable to obtain')
                 else:
-                    start = 'Lock owner changed for'
+                    start = gettext('Lock owner changed for')
                 last_info = new_info
-                msg = u'%s lock %s %s.' % (start, lock_url, new_info)
+                msg = gettext('{0} lock {1} {2}.').format(start, lock_url,
+                                                                    new_info)
                 if deadline_str is None:
                     deadline_str = time.strftime('%H:%M:%S',
                                                     time.localtime(deadline))
                 if timeout > 0:
-                    msg += ('\nWill continue to try until %s, unless '
-                        'you press Ctrl-C.'
-                        % deadline_str)
-                msg += '\nSee "bzr help break-lock" for more.'
+                    msg += '\n' + gettext(
+                             'Will continue to try until %s, unless '
+                             'you press Ctrl-C.') % deadline_str
+                msg += '\n' + gettext('See "bzr help break-lock" for more.')
                 self._report_function(msg)
             if (max_attempts is not None) and (attempt_count >= max_attempts):
                 self._trace("exceeded %d attempts")
@@ -732,9 +735,9 @@
     def __unicode__(self):
         """Return a user-oriented description of this object."""
         d = self.to_readable_dict()
-        return (
+        return ( gettext(
             u'held by %(user)s on %(hostname)s (process #%(pid)s), '
-            u'acquired %(time_ago)s' % d)
+            u'acquired %(time_ago)s') % d)
 
     def to_readable_dict(self):
         """Turn the holder info into a dict of human-readable attributes.

=== modified file 'bzrlib/log.py'
--- bzrlib/log.py	2011-07-25 07:41:52 +0000
+++ bzrlib/log.py	2011-12-19 13:23:58 +0000
@@ -14,8 +14,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-
-
 """Code to show logs of changes.
 
 Various flavors of log can be produced:
@@ -49,6 +47,8 @@
 all the changes since the previous revision that touched hello.c.
 """
 
+from __future__ import absolute_import
+
 import codecs
 from cStringIO import StringIO
 from itertools import (
@@ -65,8 +65,8 @@
 lazy_import(globals(), """
 
 from bzrlib import (
-    bzrdir,
     config,
+    controldir,
     diff,
     errors,
     foreign,
@@ -74,8 +74,8 @@
     revision as _mod_revision,
     revisionspec,
     tsort,
-    i18n,
     )
+from bzrlib.i18n import gettext, ngettext
 """)
 
 from bzrlib import (
@@ -105,7 +105,10 @@
     last_ie = None
     last_path = None
     revno = 1
-    for revision_id in branch.revision_history():
+    graph = branch.repository.get_graph()
+    history = list(graph.iter_lefthand_ancestry(branch.last_revision(),
+        [_mod_revision.NULL_REVISION]))
+    for revision_id in reversed(history):
         this_inv = branch.repository.get_inventory(revision_id)
         if this_inv.has_id(file_id):
             this_ie = this_inv[file_id]
@@ -215,7 +218,7 @@
     Logger(branch, rqst).show(lf)
 
 
-# Note: This needs to be kept this in sync with the defaults in
+# Note: This needs to be kept in sync with the defaults in
 # make_log_request_dict() below
 _DEFAULT_REQUEST_PARAMS = {
     'direction': 'reverse',
@@ -232,7 +235,7 @@
                           delta_type=None,
                           diff_type=None, _match_using_deltas=True,
                           exclude_common_ancestry=False, match=None,
-                          signature=False,
+                          signature=False, omit_merges=False,
                           ):
     """Convenience function for making a logging request dictionary.
 
@@ -288,6 +291,9 @@
       revisions. Keys can be 'message', 'author', 'committer', 'bugs' or
       the empty string to match any of the preceding properties.
 
+    :param omit_merges: If True, commits with more than one parent are
+      omitted.
+
     """
     # Take care of old style message_search parameter
     if message_search:
@@ -311,6 +317,7 @@
         'exclude_common_ancestry': exclude_common_ancestry,
         'signature': signature,
         'match': match,
+        'omit_merges': omit_merges,
         # Add 'private' attributes for features that may be deprecated
         '_match_using_deltas': _match_using_deltas,
     }
@@ -334,7 +341,7 @@
     from bzrlib import gpg
 
     gpg_strategy = gpg.GPGStrategy(None)
-    result = repo.verify_revision(rev_id, gpg_strategy)
+    result = repo.verify_revision_signature(rev_id, gpg_strategy)
     if result[0] == gpg.SIGNATURE_VALID:
         return "valid signature from {0}".format(result[1])
     if result[0] == gpg.SIGNATURE_KEY_MISSING:
@@ -447,6 +454,7 @@
         limit = rqst.get('limit')
         diff_type = rqst.get('diff_type')
         show_signature = rqst.get('signature')
+        omit_merges = rqst.get('omit_merges')
         log_count = 0
         revision_iterator = self._create_log_revision_iterator()
         for revs in revision_iterator:
@@ -454,6 +462,8 @@
                 # 0 levels means show everything; merge_depth counts from 0
                 if levels != 0 and merge_depth >= levels:
                     continue
+                if omit_merges and len(rev.parent_ids) > 1:
+                    continue
                 if diff_type is None:
                     diff = None
                 else:
@@ -558,10 +568,10 @@
              a list of the same tuples.
     """
     if (exclude_common_ancestry and start_rev_id == end_rev_id):
-        raise errors.BzrCommandError(
-            '--exclude-common-ancestry requires two different revisions')
+        raise errors.BzrCommandError(gettext(
+            '--exclude-common-ancestry requires two different revisions'))
     if direction not in ('reverse', 'forward'):
-        raise ValueError('invalid direction %r' % direction)
+        raise ValueError(gettext('invalid direction %r') % direction)
     br_revno, br_rev_id = branch.last_revision_info()
     if br_revno == 0:
         return []
@@ -570,20 +580,32 @@
         and (not generate_merge_revisions
              or not _has_merges(branch, end_rev_id))):
         # If a single revision is requested, check we can handle it
-        iter_revs = _generate_one_revision(branch, end_rev_id, br_rev_id,
-                                           br_revno)
-    elif not generate_merge_revisions:
-        # If we only want to see linear revisions, we can iterate ...
-        iter_revs = _generate_flat_revisions(branch, start_rev_id, end_rev_id,
-                                             direction, exclude_common_ancestry)
-        if direction == 'forward':
-            iter_revs = reversed(iter_revs)
-    else:
-        iter_revs = _generate_all_revisions(branch, start_rev_id, end_rev_id,
-                                            direction, delayed_graph_generation,
-                                            exclude_common_ancestry)
-        if direction == 'forward':
-            iter_revs = _rebase_merge_depth(reverse_by_depth(list(iter_revs)))
+        return  _generate_one_revision(branch, end_rev_id, br_rev_id,
+                                       br_revno)
+    if not generate_merge_revisions:
+        try:
+            # If we only want to see linear revisions, we can iterate ...
+            iter_revs = _linear_view_revisions(
+                branch, start_rev_id, end_rev_id,
+                exclude_common_ancestry=exclude_common_ancestry)
+            # If a start limit was given and it's not obviously an
+            # ancestor of the end limit, check it before outputting anything
+            if (direction == 'forward'
+                or (start_rev_id and not _is_obvious_ancestor(
+                        branch, start_rev_id, end_rev_id))):
+                    iter_revs = list(iter_revs)
+            if direction == 'forward':
+                iter_revs = reversed(iter_revs)
+            return iter_revs
+        except _StartNotLinearAncestor:
+            # Switch to the slower implementation that may be able to find a
+            # non-obvious ancestor out of the left-hand history.
+            pass
+    iter_revs = _generate_all_revisions(branch, start_rev_id, end_rev_id,
+                                        direction, delayed_graph_generation,
+                                        exclude_common_ancestry)
+    if direction == 'forward':
+        iter_revs = _rebase_merge_depth(reverse_by_depth(list(iter_revs)))
     return iter_revs
 
 
@@ -596,23 +618,6 @@
         return [(rev_id, revno_str, 0)]
 
 
-def _generate_flat_revisions(branch, start_rev_id, end_rev_id, direction,
-                             exclude_common_ancestry=False):
-    result = _linear_view_revisions(
-        branch, start_rev_id, end_rev_id,
-        exclude_common_ancestry=exclude_common_ancestry)
-    # If a start limit was given and it's not obviously an
-    # ancestor of the end limit, check it before outputting anything
-    if direction == 'forward' or (start_rev_id
-        and not _is_obvious_ancestor(branch, start_rev_id, end_rev_id)):
-        try:
-            result = list(result)
-        except _StartNotLinearAncestor:
-            raise errors.BzrCommandError('Start revision not found in'
-                ' left-hand history of end revision.')
-    return result
-
-
 def _generate_all_revisions(branch, start_rev_id, end_rev_id, direction,
                             delayed_graph_generation,
                             exclude_common_ancestry=False):
@@ -654,8 +659,8 @@
         except _StartNotLinearAncestor:
             # A merge was never detected so the lower revision limit can't
             # be nested down somewhere
-            raise errors.BzrCommandError('Start revision not found in'
-                ' history of end revision.')
+            raise errors.BzrCommandError(gettext('Start revision not found in'
+                ' history of end revision.'))
 
     # We exit the loop above because we encounter a revision with merges, from
     # this revision, we need to switch to _graph_view_revisions.
@@ -1072,10 +1077,10 @@
     if branch_revno != 0:
         if (start_rev_id == _mod_revision.NULL_REVISION
             or end_rev_id == _mod_revision.NULL_REVISION):
-            raise errors.BzrCommandError('Logging revision 0 is invalid.')
+            raise errors.BzrCommandError(gettext('Logging revision 0 is invalid.'))
         if start_revno > end_revno:
-            raise errors.BzrCommandError("Start revision must be older than "
-                                         "the end revision.")
+            raise errors.BzrCommandError(gettext("Start revision must be "
+                                         "older than the end revision."))
     return (start_rev_id, end_rev_id)
 
 
@@ -1130,10 +1135,10 @@
 
     if ((start_rev_id == _mod_revision.NULL_REVISION)
         or (end_rev_id == _mod_revision.NULL_REVISION)):
-        raise errors.BzrCommandError('Logging revision 0 is invalid.')
+        raise errors.BzrCommandError(gettext('Logging revision 0 is invalid.'))
     if start_revno > end_revno:
-        raise errors.BzrCommandError("Start revision must be older than "
-                                     "the end revision.")
+        raise errors.BzrCommandError(gettext("Start revision must be older "
+                                     "than the end revision."))
 
     if end_revno < start_revno:
         return None, None, None, None
@@ -1405,7 +1410,7 @@
             if advice_sep:
                 self.to_file.write(advice_sep)
             self.to_file.write(
-                "Use --include-merges or -n0 to see merged revisions.\n")
+                "Use --include-merged or -n0 to see merged revisions.\n")
 
     def get_advice_separator(self):
         """Get the text separating the log from the closing advice."""
@@ -1789,20 +1794,21 @@
         return self.get(name)(*args, **kwargs)
 
     def get_default(self, branch):
-        return self.get(branch.get_config().log_format())
+        c = branch.get_config_stack()
+        return self.get(c.get('log_format'))
 
 
 log_formatter_registry = LogFormatterRegistry()
 
 
 log_formatter_registry.register('short', ShortLogFormatter,
-                                'Moderately short log format')
+                                'Moderately short log format.')
 log_formatter_registry.register('long', LongLogFormatter,
-                                'Detailed log format')
+                                'Detailed log format.')
 log_formatter_registry.register('line', LineLogFormatter,
-                                'Log format with one line per revision')
+                                'Log format with one line per revision.')
 log_formatter_registry.register('gnu-changelog', GnuChangelogLogFormatter,
-                                'Format used by GNU ChangeLog files')
+                                'Format used by GNU ChangeLog files.')
 
 
 def register_formatter(name, formatter):
@@ -1818,7 +1824,7 @@
     try:
         return log_formatter_registry.make_formatter(name, *args, **kwargs)
     except KeyError:
-        raise errors.BzrCommandError("unknown log formatter: %r" % name)
+        raise errors.BzrCommandError(gettext("unknown log formatter: %r") % name)
 
 
 def author_list_all(rev):
@@ -2017,8 +2023,9 @@
       kind is one of values 'directory', 'file', 'symlink', 'tree-reference'.
       branch will be read-locked.
     """
-    from builtins import _get_revision_range
-    tree, b, path = bzrdir.BzrDir.open_containing_tree_or_branch(file_list[0])
+    from bzrlib.builtins import _get_revision_range
+    tree, b, path = controldir.ControlDir.open_containing_tree_or_branch(
+        file_list[0])
     add_cleanup(b.lock_read().unlock)
     # XXX: It's damn messy converting a list of paths to relative paths when
     # those paths might be deleted ones, they might be on a case-insensitive
@@ -2113,7 +2120,8 @@
                           len(row) > 1 and row[1] == 'fixed']
 
         if fixed_bug_urls:
-            return {'fixes bug(s)': ' '.join(fixed_bug_urls)}
+            return {ngettext('fixes bug', 'fixes bugs', len(fixed_bug_urls)):\
+                    ' '.join(fixed_bug_urls)}
     return {}
 
 properties_handler_registry.register('bugs_properties_handler',

=== modified file 'bzrlib/lru_cache.py'
--- bzrlib/lru_cache.py	2010-07-15 13:05:40 +0000
+++ bzrlib/lru_cache.py	2011-12-19 13:23:58 +0000
@@ -16,7 +16,10 @@
 
 """A simple least-recently-used (LRU) cache."""
 
+from __future__ import absolute_import
+
 from bzrlib import (
+    symbol_versioning,
     trace,
     )
 
@@ -25,18 +28,13 @@
 class _LRUNode(object):
     """This maintains the linked-list which is the lru internals."""
 
-    __slots__ = ('prev', 'next_key', 'key', 'value', 'cleanup', 'size')
+    __slots__ = ('prev', 'next_key', 'key', 'value')
 
-    def __init__(self, key, value, cleanup=None):
+    def __init__(self, key, value):
         self.prev = None
         self.next_key = _null_key
         self.key = key
         self.value = value
-        self.cleanup = cleanup
-        # TODO: We could compute this 'on-the-fly' like we used to, and remove
-        #       one pointer from this object, we just need to decide if it
-        #       actually costs us much of anything in normal usage
-        self.size = None
 
     def __repr__(self):
         if self.prev is None:
@@ -46,16 +44,6 @@
         return '%s(%r n:%r p:%r)' % (self.__class__.__name__, self.key,
                                      self.next_key, prev_key)
 
-    def run_cleanup(self):
-        try:
-            if self.cleanup is not None:
-                self.cleanup(self.key, self.value)
-        finally:
-            # cleanup might raise an exception, but we want to make sure
-            # to break refcycles, etc
-            self.cleanup = None
-            self.value = None
-
 
 class LRUCache(object):
     """A class which manages a cache of entries, removing unused ones."""
@@ -105,62 +93,23 @@
     def __len__(self):
         return len(self._cache)
 
-    def _walk_lru(self):
-        """Walk the LRU list, only meant to be used in tests."""
-        node = self._most_recently_used
-        if node is not None:
-            if node.prev is not None:
-                raise AssertionError('the _most_recently_used entry is not'
-                                     ' supposed to have a previous entry'
-                                     ' %s' % (node,))
-        while node is not None:
-            if node.next_key is _null_key:
-                if node is not self._least_recently_used:
-                    raise AssertionError('only the last node should have'
-                                         ' no next value: %s' % (node,))
-                node_next = None
-            else:
-                node_next = self._cache[node.next_key]
-                if node_next.prev is not node:
-                    raise AssertionError('inconsistency found, node.next.prev'
-                                         ' != node: %s' % (node,))
-            if node.prev is None:
-                if node is not self._most_recently_used:
-                    raise AssertionError('only the _most_recently_used should'
-                                         ' not have a previous node: %s'
-                                         % (node,))
-            else:
-                if node.prev.next_key != node.key:
-                    raise AssertionError('inconsistency found, node.prev.next'
-                                         ' != node: %s' % (node,))
-            yield node
-            node = node_next
-
+    @symbol_versioning.deprecated_method(
+        symbol_versioning.deprecated_in((2, 5, 0)))
     def add(self, key, value, cleanup=None):
-        """Add a new value to the cache.
-
-        Also, if the entry is ever removed from the cache, call
-        cleanup(key, value).
-
-        :param key: The key to store it under
-        :param value: The object to store
-        :param cleanup: None or a function taking (key, value) to indicate
-                        'value' should be cleaned up.
-        """
+        if cleanup is not None:
+            raise ValueError("Per-node cleanup functions no longer supported")
+        return self.__setitem__(key, value)
+
+    def __setitem__(self, key, value):
+        """Add a new value to the cache"""
         if key is _null_key:
             raise ValueError('cannot use _null_key as a key')
         if key in self._cache:
             node = self._cache[key]
-            try:
-                node.run_cleanup()
-            finally:
-                # Maintain the LRU properties, even if cleanup raises an
-                # exception
-                node.value = value
-                node.cleanup = cleanup
-                self._record_access(node)
+            node.value = value
+            self._record_access(node)
         else:
-            node = _LRUNode(key, value, cleanup=cleanup)
+            node = _LRUNode(key, value)
             self._cache[key] = node
             self._record_access(node)
 
@@ -190,10 +139,13 @@
         """
         return self._cache.keys()
 
-    def items(self):
-        """Get the key:value pairs as a dict."""
+    def as_dict(self):
+        """Get a new dict with the same key:value pairs as the cache"""
         return dict((k, n.value) for k, n in self._cache.iteritems())
 
+    items = symbol_versioning.deprecated_method(
+        symbol_versioning.deprecated_in((2, 5, 0)))(as_dict)
+
     def cleanup(self):
         """Clear the cache until it shrinks to the requested size.
 
@@ -204,10 +156,6 @@
         while len(self._cache) > self._after_cleanup_count:
             self._remove_lru()
 
-    def __setitem__(self, key, value):
-        """Add a value to the cache, there will be no cleanup function."""
-        self.add(key, value, cleanup=None)
-
     def _record_access(self, node):
         """Record that key was accessed."""
         # Move 'node' to the front of the queue
@@ -241,19 +189,14 @@
         # If we have removed all entries, remove the head pointer as well
         if self._least_recently_used is None:
             self._most_recently_used = None
-        try:
-            node.run_cleanup()
-        finally:
-            # cleanup might raise an exception, but we want to make sure to
-            # maintain the linked list
-            if node.prev is not None:
-                node.prev.next_key = node.next_key
-            if node.next_key is not _null_key:
-                node_next = self._cache[node.next_key]
-                node_next.prev = node.prev
-            # And remove this node's pointers
-            node.prev = None
-            node.next_key = _null_key
+        if node.prev is not None:
+            node.prev.next_key = node.next_key
+        if node.next_key is not _null_key:
+            node_next = self._cache[node.next_key]
+            node_next.prev = node.prev
+        # And remove this node's pointers
+        node.prev = None
+        node.next_key = _null_key
 
     def _remove_lru(self):
         """Remove one entry from the lru, and handle consequences.
@@ -316,17 +259,8 @@
         self._update_max_size(max_size, after_cleanup_size=after_cleanup_size)
         LRUCache.__init__(self, max_cache=max(int(max_size/512), 1))
 
-    def add(self, key, value, cleanup=None):
-        """Add a new value to the cache.
-
-        Also, if the entry is ever removed from the cache, call
-        cleanup(key, value).
-
-        :param key: The key to store it under
-        :param value: The object to store
-        :param cleanup: None or a function taking (key, value) to indicate
-                        'value' should be cleaned up.
-        """
+    def __setitem__(self, key, value):
+        """Add a new value to the cache"""
         if key is _null_key:
             raise ValueError('cannot use _null_key as a key')
         node = self._cache.get(key, None)
@@ -341,15 +275,12 @@
             if node is not None:
                 # We won't be replacing the old node, so just remove it
                 self._remove_node(node)
-            if cleanup is not None:
-                cleanup(key, value)
             return
         if node is None:
-            node = _LRUNode(key, value, cleanup=cleanup)
+            node = _LRUNode(key, value)
             self._cache[key] = node
         else:
-            self._value_size -= node.size
-        node.size = value_len
+            self._value_size -= self._compute_size(node.value)
         self._value_size += value_len
         self._record_access(node)
 
@@ -368,7 +299,7 @@
             self._remove_lru()
 
     def _remove_node(self, node):
-        self._value_size -= node.size
+        self._value_size -= self._compute_size(node.value)
         LRUCache._remove_node(self, node)
 
     def resize(self, max_size, after_cleanup_size=None):

=== modified file 'bzrlib/lsprof.py'
--- bzrlib/lsprof.py	2011-04-12 14:55:57 +0000
+++ bzrlib/lsprof.py	2011-12-18 15:28:38 +0000
@@ -3,6 +3,8 @@
 # I made one modification to profile so that it returns a pair
 # instead of just the Stats object
 
+from __future__ import absolute_import
+
 import cPickle
 import os
 import sys

=== modified file 'bzrlib/mail_client.py'
--- bzrlib/mail_client.py	2010-04-23 08:51:52 +0000
+++ bzrlib/mail_client.py	2011-12-19 10:58:39 +0000
@@ -14,12 +14,13 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 import errno
 import os
 import subprocess
 import sys
 import tempfile
-import urllib
 
 import bzrlib
 from bzrlib import (
@@ -310,7 +311,7 @@
             message_options['attachment'] = urlutils.local_path_to_url(
                 attach_path)
         if body is not None:
-            options_list = ['body=%s' % urllib.quote(self._encode_safe(body))]
+            options_list = ['body=%s' % urlutils.quote(self._encode_safe(body))]
         else:
             options_list = []
         options_list.extend(["%s='%s'" % (k, v) for k, v in
@@ -352,15 +353,15 @@
         """See ExternalMailClient._get_compose_commandline"""
         compose_url = []
         if from_ is not None:
-            compose_url.append('from=' + urllib.quote(from_))
+            compose_url.append('from=' + urlutils.quote(from_))
         if subject is not None:
-            # Don't use urllib.quote_plus because Claws doesn't seem
+            # Don't use urlutils.quote_plus because Claws doesn't seem
             # to recognise spaces encoded as "+".
             compose_url.append(
-                'subject=' + urllib.quote(self._encode_safe(subject)))
+                'subject=' + urlutils.quote(self._encode_safe(subject)))
         if body is not None:
             compose_url.append(
-                'body=' + urllib.quote(self._encode_safe(body)))
+                'body=' + urlutils.quote(self._encode_safe(body)))
         # to must be supplied for the claws-mail --compose syntax to work.
         if to is None:
             raise errors.NoMailAddressSpecified()

=== modified file 'bzrlib/memorytree.py'
--- bzrlib/memorytree.py	2011-04-19 10:42:59 +0000
+++ bzrlib/memorytree.py	2011-12-19 13:23:58 +0000
@@ -19,6 +19,7 @@
 See MemoryTree for more details.
 """
 
+from __future__ import absolute_import
 
 import os
 

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2011-07-06 09:22:00 +0000
+++ bzrlib/merge.py	2012-01-02 21:30:59 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 import warnings
 
 from bzrlib.lazy_import import lazy_import
@@ -38,11 +40,13 @@
     versionedfile,
     workingtree,
     )
+from bzrlib.i18n import gettext
 """)
 from bzrlib import (
     decorators,
     errors,
     hooks,
+    registry,
     )
 from bzrlib.symbol_versioning import (
     deprecated_in,
@@ -74,6 +78,15 @@
             "See the AbstractPerFileMerger API docs for details on how it is "
             "used by merge.",
             (2, 1))
+        self.add_hook('pre_merge',
+            'Called before a merge. '
+            'Receives a Merger object as the single argument.',
+            (2, 5))
+        self.add_hook('post_merge',
+            'Called after a merge. '
+            'Receives a Merger object as the single argument. '
+            'The return value is ignored.',
+            (2, 5))
 
 
 class AbstractPerFileMerger(object):
@@ -91,7 +104,7 @@
     def merge_contents(self, merge_params):
         """Attempt to merge the contents of a single file.
         
-        :param merge_params: A bzrlib.merge.MergeHookParams
+        :param merge_params: A bzrlib.merge.MergeFileHookParams
         :return: A tuple of (status, chunks), where status is one of
             'not_applicable', 'success', 'conflicted', or 'delete'.  If status
             is 'success' or 'conflicted', then chunks should be an iterable of
@@ -118,14 +131,14 @@
 
     def get_filename(self, params, tree):
         """Lookup the filename (i.e. basename, not path), given a Tree (e.g.
-        self.merger.this_tree) and a MergeHookParams.
+        self.merger.this_tree) and a MergeFileHookParams.
         """
         return osutils.basename(tree.id2path(params.file_id))
 
     def get_filepath(self, params, tree):
         """Calculate the path to the file in a tree.
 
-        :param params: A MergeHookParams describing the file to merge
+        :param params: A MergeFileHookParams describing the file to merge
         :param tree: a Tree, e.g. self.merger.this_tree.
         """
         return tree.id2path(params.file_id)
@@ -138,7 +151,7 @@
             params.winner == 'other' or
             # THIS and OTHER aren't both files.
             not params.is_file_merge() or
-            # The filename doesn't match *.xml
+            # The filename doesn't match
             not self.file_matches(params)):
             return 'not_applicable', None
         return self.merge_matching(params)
@@ -220,7 +233,7 @@
         raise NotImplementedError(self.merge_text)
 
 
-class MergeHookParams(object):
+class MergeFileHookParams(object):
     """Object holding parameters passed to merge_file_content hooks.
 
     There are some fields hooks can access:
@@ -441,51 +454,6 @@
         revision_id = _mod_revision.ensure_null(revision_id)
         return branch, self.revision_tree(revision_id, branch)
 
-    @deprecated_method(deprecated_in((2, 1, 0)))
-    def ensure_revision_trees(self):
-        if self.this_revision_tree is None:
-            self.this_basis_tree = self.revision_tree(self.this_basis)
-            if self.this_basis == self.this_rev_id:
-                self.this_revision_tree = self.this_basis_tree
-
-        if self.other_rev_id is None:
-            other_basis_tree = self.revision_tree(self.other_basis)
-            if other_basis_tree.has_changes(self.other_tree):
-                raise errors.WorkingTreeNotRevision(self.this_tree)
-            other_rev_id = self.other_basis
-            self.other_tree = other_basis_tree
-
-    @deprecated_method(deprecated_in((2, 1, 0)))
-    def file_revisions(self, file_id):
-        self.ensure_revision_trees()
-        if self.this_rev_id is None:
-            if self.this_basis_tree.get_file_sha1(file_id) != \
-                self.this_tree.get_file_sha1(file_id):
-                raise errors.WorkingTreeNotRevision(self.this_tree)
-
-        trees = (self.this_basis_tree, self.other_tree)
-        return [tree.get_file_revision(file_id) for tree in trees]
-
-    @deprecated_method(deprecated_in((2, 1, 0)))
-    def check_basis(self, check_clean, require_commits=True):
-        if self.this_basis is None and require_commits is True:
-            raise errors.BzrCommandError(
-                "This branch has no commits."
-                " (perhaps you would prefer 'bzr pull')")
-        if check_clean:
-            self.compare_basis()
-            if self.this_basis != self.this_rev_id:
-                raise errors.UncommittedChanges(self.this_tree)
-
-    @deprecated_method(deprecated_in((2, 1, 0)))
-    def compare_basis(self):
-        try:
-            basis_tree = self.revision_tree(self.this_tree.last_revision())
-        except errors.NoSuchRevision:
-            basis_tree = self.this_tree.basis_tree()
-        if not self.this_tree.has_changes(basis_tree):
-            self.this_rev_id = self.this_basis
-
     def set_interesting_files(self, file_list):
         self.interesting_files = file_list
 
@@ -536,7 +504,7 @@
                 raise errors.NoCommits(self.other_branch)
         if self.other_rev_id is not None:
             self._cached_trees[self.other_rev_id] = self.other_tree
-        self._maybe_fetch(self.other_branch,self.this_branch, self.other_basis)
+        self._maybe_fetch(self.other_branch, self.this_branch, self.other_basis)
 
     def set_other_revision(self, revision_id, other_branch):
         """Set 'other' based on a branch and revision id
@@ -591,11 +559,11 @@
                 else:
                     self.base_rev_id = self.revision_graph.find_unique_lca(
                                             *lcas)
-                sorted_lca_keys = self.revision_graph.find_merge_order(                
+                sorted_lca_keys = self.revision_graph.find_merge_order(
                     revisions[0], lcas)
                 if self.base_rev_id == _mod_revision.NULL_REVISION:
                     self.base_rev_id = sorted_lca_keys[0]
-                
+
             if self.base_rev_id == _mod_revision.NULL_REVISION:
                 raise errors.UnrelatedBranches()
             if self._is_criss_cross:
@@ -604,7 +572,8 @@
                 trace.mutter('Criss-cross lcas: %r' % lcas)
                 if self.base_rev_id in lcas:
                     trace.mutter('Unable to find unique lca. '
-                                 'Fallback %r as best option.' % self.base_rev_id)
+                                 'Fallback %r as best option.'
+                                 % self.base_rev_id)
                 interesting_revision_ids = set(lcas)
                 interesting_revision_ids.add(self.base_rev_id)
                 interesting_trees = dict((t.get_revision_id(), t)
@@ -643,11 +612,12 @@
             self._maybe_fetch(base_branch, self.this_branch, self.base_rev_id)
 
     def make_merger(self):
-        kwargs = {'working_tree':self.this_tree, 'this_tree': self.this_tree,
+        kwargs = {'working_tree': self.this_tree, 'this_tree': self.this_tree,
                   'other_tree': self.other_tree,
                   'interesting_ids': self.interesting_ids,
                   'interesting_files': self.interesting_files,
                   'this_branch': self.this_branch,
+                  'other_branch': self.other_branch,
                   'do_merge': False}
         if self.merge_type.requires_base:
             kwargs['base_tree'] = self.base_tree
@@ -679,7 +649,11 @@
         merge = self.make_merger()
         if self.other_branch is not None:
             self.other_branch.update_references(self.this_branch)
+        for hook in Merger.hooks['pre_merge']:
+            hook(merge)
         merge.do_merge()
+        for hook in Merger.hooks['post_merge']:
+            hook(merge)
         if self.recurse == 'down':
             for relpath, file_id in self.this_tree.iter_references():
                 sub_tree = self.this_tree.get_nested_tree(file_id, relpath)
@@ -689,7 +663,8 @@
                     continue
                 sub_merge = Merger(sub_tree.branch, this_tree=sub_tree)
                 sub_merge.merge_type = self.merge_type
-                other_branch = self.other_branch.reference_parent(file_id, relpath)
+                other_branch = self.other_branch.reference_parent(file_id,
+                                                                  relpath)
                 sub_merge.set_other_revision(other_revision, other_branch)
                 base_revision = self.base_tree.get_reference_revision(file_id)
                 sub_merge.base_tree = \
@@ -711,9 +686,9 @@
         merge = operation.run_simple()
         if len(merge.cooked_conflicts) == 0:
             if not self.ignore_zero and not trace.is_quiet():
-                trace.note("All changes applied successfully.")
+                trace.note(gettext("All changes applied successfully."))
         else:
-            trace.note("%d conflicts encountered."
+            trace.note(gettext("%d conflicts encountered.")
                        % len(merge.cooked_conflicts))
 
         return len(merge.cooked_conflicts)
@@ -751,7 +726,8 @@
                  interesting_ids=None, reprocess=False, show_base=False,
                  pb=None, pp=None, change_reporter=None,
                  interesting_files=None, do_merge=True,
-                 cherrypick=False, lca_trees=None, this_branch=None):
+                 cherrypick=False, lca_trees=None, this_branch=None,
+                 other_branch=None):
         """Initialize the merger object and perform the merge.
 
         :param working_tree: The working tree to apply the merge to
@@ -760,6 +736,7 @@
         :param other_tree: The other tree to merge changes from
         :param this_branch: The branch associated with this_tree.  Defaults to
             this_tree.branch if not supplied.
+        :param other_branch: The branch associated with other_tree, if any.
         :param interesting_ids: The file_ids of files that should be
             participate in the merge.  May not be combined with
             interesting_files.
@@ -787,10 +764,12 @@
             this_branch = this_tree.branch
         self.interesting_ids = interesting_ids
         self.interesting_files = interesting_files
-        self.this_tree = working_tree
+        self.working_tree = working_tree
+        self.this_tree = this_tree
         self.base_tree = base_tree
         self.other_tree = other_tree
         self.this_branch = this_branch
+        self.other_branch = other_branch
         self._raw_conflicts = []
         self.cooked_conflicts = []
         self.reprocess = reprocess
@@ -812,7 +791,9 @@
 
     def do_merge(self):
         operation = cleanup.OperationWithCleanups(self._do_merge)
-        self.this_tree.lock_tree_write()
+        self.working_tree.lock_tree_write()
+        operation.add_cleanup(self.working_tree.unlock)
+        self.this_tree.lock_read()
         operation.add_cleanup(self.this_tree.unlock)
         self.base_tree.lock_read()
         operation.add_cleanup(self.base_tree.unlock)
@@ -821,13 +802,13 @@
         operation.run()
 
     def _do_merge(self, operation):
-        self.tt = transform.TreeTransform(self.this_tree, None)
+        self.tt = transform.TreeTransform(self.working_tree, None)
         operation.add_cleanup(self.tt.finalize)
         self._compute_transform()
         results = self.tt.apply(no_conflicts=True)
         self.write_modified(results)
         try:
-            self.this_tree.add_conflicts(self.cooked_conflicts)
+            self.working_tree.add_conflicts(self.cooked_conflicts)
         except errors.UnsupportedOperation:
             pass
 
@@ -840,7 +821,7 @@
         return operation.run_simple()
 
     def _make_preview_transform(self):
-        self.tt = transform.TransformPreview(self.this_tree)
+        self.tt = transform.TransformPreview(self.working_tree)
         self._compute_transform()
         return self.tt
 
@@ -851,14 +832,18 @@
         else:
             entries = self._entries_lca()
             resolver = self._lca_multi_way
+        # Prepare merge hooks
+        factories = Merger.hooks['merge_file_content']
+        # One hook for each registered one plus our default merger
+        hooks = [factory(self) for factory in factories] + [self]
+        self.active_hooks = [hook for hook in hooks if hook is not None]
         child_pb = ui.ui_factory.nested_progress_bar()
         try:
-            factories = Merger.hooks['merge_file_content']
-            hooks = [factory(self) for factory in factories] + [self]
-            self.active_hooks = [hook for hook in hooks if hook is not None]
             for num, (file_id, changed, parents3, names3,
                       executable3) in enumerate(entries):
-                child_pb.update('Preparing file merge', num, len(entries))
+                # Try merging each entry
+                child_pb.update(gettext('Preparing file merge'),
+                                num, len(entries))
                 self._merge_names(file_id, parents3, names3, resolver=resolver)
                 if changed:
                     file_status = self._do_merge_contents(file_id)
@@ -1141,14 +1126,14 @@
     def write_modified(self, results):
         modified_hashes = {}
         for path in results.modified_paths:
-            file_id = self.this_tree.path2id(self.this_tree.relpath(path))
+            file_id = self.working_tree.path2id(self.working_tree.relpath(path))
             if file_id is None:
                 continue
-            hash = self.this_tree.get_file_sha1(file_id)
+            hash = self.working_tree.get_file_sha1(file_id)
             if hash is None:
                 continue
             modified_hashes[file_id] = hash
-        self.this_tree.set_merge_modified(modified_hashes)
+        self.working_tree.set_merge_modified(modified_hashes)
 
     @staticmethod
     def parent(entry, file_id):
@@ -1378,7 +1363,7 @@
         # We have a hypothetical conflict, but if we have files, then we
         # can try to merge the content
         trans_id = self.tt.trans_id_file_id(file_id)
-        params = MergeHookParams(self, file_id, trans_id, this_pair[0],
+        params = MergeFileHookParams(self, file_id, trans_id, this_pair[0],
             other_pair[0], winner)
         hooks = self.active_hooks
         hook_status = 'not_applicable'
@@ -1387,18 +1372,54 @@
             if hook_status != 'not_applicable':
                 # Don't try any more hooks, this one applies.
                 break
+        # If the merge ends up replacing the content of the file, we get rid of
+        # it at the end of this method (this variable is used to track the
+        # exceptions to this rule).
+        keep_this = False
         result = "modified"
         if hook_status == 'not_applicable':
-            # This is a contents conflict, because none of the available
-            # functions could merge it.
+            # No merge hook was able to resolve the situation. Two cases exist:
+            # a content conflict or a duplicate one.
             result = None
             name = self.tt.final_name(trans_id)
             parent_id = self.tt.final_parent(trans_id)
-            if self.this_tree.has_id(file_id):
-                self.tt.unversion_file(trans_id)
-            file_group = self._dump_conflicts(name, parent_id, file_id,
-                                              set_version=True)
-            self._raw_conflicts.append(('contents conflict', file_group))
+            duplicate = False
+            inhibit_content_conflict = False
+            if params.this_kind is None: # file_id is not in THIS
+                # Is the name used for a different file_id ?
+                dupe_path = self.other_tree.id2path(file_id)
+                this_id = self.this_tree.path2id(dupe_path)
+                if this_id is not None:
+                    # Two entries for the same path
+                    keep_this = True
+                    # versioning the merged file will trigger a duplicate
+                    # conflict
+                    self.tt.version_file(file_id, trans_id)
+                    transform.create_from_tree(
+                        self.tt, trans_id, self.other_tree, file_id,
+                        filter_tree_path=self._get_filter_tree_path(file_id))
+                    inhibit_content_conflict = True
+            elif params.other_kind is None: # file_id is not in OTHER
+                # Is the name used for a different file_id ?
+                dupe_path = self.this_tree.id2path(file_id)
+                other_id = self.other_tree.path2id(dupe_path)
+                if other_id is not None:
+                    # Two entries for the same path again, but here, the other
+                    # entry will also be merged.  We simply inhibit the
+                    # 'content' conflict creation because we know OTHER will
+                    # create (or has already created depending on ordering) an
+                    # entry at the same path. This will trigger a 'duplicate'
+                    # conflict later.
+                    keep_this = True
+                    inhibit_content_conflict = True
+            if not inhibit_content_conflict:
+                if params.this_kind is not None:
+                    self.tt.unversion_file(trans_id)
+                # This is a contents conflict, because none of the available
+                # functions could merge it.
+                file_group = self._dump_conflicts(name, parent_id, file_id,
+                                                  set_version=True)
+                self._raw_conflicts.append(('contents conflict', file_group))
         elif hook_status == 'success':
             self.tt.create_file(lines, trans_id)
         elif hook_status == 'conflicted':
@@ -1420,36 +1441,23 @@
             raise AssertionError('unknown hook_status: %r' % (hook_status,))
         if not self.this_tree.has_id(file_id) and result == "modified":
             self.tt.version_file(file_id, trans_id)
-        # The merge has been performed, so the old contents should not be
-        # retained.
-        self.tt.delete_contents(trans_id)
+        if not keep_this:
+            # The merge has been performed and produced a new content, so the
+            # old contents should not be retained.
+            self.tt.delete_contents(trans_id)
         return result
 
     def _default_other_winner_merge(self, merge_hook_params):
         """Replace this contents with other."""
         file_id = merge_hook_params.file_id
         trans_id = merge_hook_params.trans_id
-        file_in_this = self.this_tree.has_id(file_id)
         if self.other_tree.has_id(file_id):
             # OTHER changed the file
-            wt = self.this_tree
-            if wt.supports_content_filtering():
-                # We get the path from the working tree if it exists.
-                # That fails though when OTHER is adding a file, so
-                # we fall back to the other tree to find the path if
-                # it doesn't exist locally.
-                try:
-                    filter_tree_path = wt.id2path(file_id)
-                except errors.NoSuchId:
-                    filter_tree_path = self.other_tree.id2path(file_id)
-            else:
-                # Skip the id2path lookup for older formats
-                filter_tree_path = None
-            transform.create_from_tree(self.tt, trans_id,
-                             self.other_tree, file_id,
-                             filter_tree_path=filter_tree_path)
+            transform.create_from_tree(
+                self.tt, trans_id, self.other_tree, file_id,
+                filter_tree_path=self._get_filter_tree_path(file_id))
             return 'done', None
-        elif file_in_this:
+        elif self.this_tree.has_id(file_id):
             # OTHER deleted the file
             return 'delete', None
         else:
@@ -1529,6 +1537,20 @@
                                               other_lines)
             file_group.append(trans_id)
 
+
+    def _get_filter_tree_path(self, file_id):
+        if self.this_tree.supports_content_filtering():
+            # We get the path from the working tree if it exists.
+            # That fails though when OTHER is adding a file, so
+            # we fall back to the other tree to find the path if
+            # it doesn't exist locally.
+            try:
+                return self.this_tree.id2path(file_id)
+            except errors.NoSuchId:
+                return self.other_tree.id2path(file_id)
+        # Skip the id2path lookup for older formats
+        return None
+
     def _dump_conflicts(self, name, parent_id, file_id, this_lines=None,
                         base_lines=None, other_lines=None, set_version=False,
                         no_base=False):
@@ -1652,10 +1674,12 @@
                 for trans_id in conflict[1]:
                     file_id = self.tt.final_file_id(trans_id)
                     if file_id is not None:
+                        # Ok we found the relevant file-id
                         break
                 path = fp.get_path(trans_id)
                 for suffix in ('.BASE', '.THIS', '.OTHER'):
                     if path.endswith(suffix):
+                        # Here is the raw path
                         path = path[:-len(suffix)]
                         break
                 c = _mod_conflicts.Conflict.factory(conflict_type,
@@ -1906,7 +1930,7 @@
             entries = self._entries_to_incorporate()
             entries = list(entries)
             for num, (entry, parent_id) in enumerate(entries):
-                child_pb.update('Preparing file merge', num, len(entries))
+                child_pb.update(gettext('Preparing file merge'), num, len(entries))
                 parent_trans_id = self.tt.trans_id_file_id(parent_id)
                 trans_id = transform.new_by_entry(self.tt, entry,
                     parent_trans_id, self.other_tree)
@@ -1996,13 +2020,24 @@
     merger.set_base_revision(get_revision_id(), this_branch)
     return merger.do_merge()
 
+
+merge_type_registry = registry.Registry()
+merge_type_registry.register('diff3', Diff3Merger,
+                             "Merge using external diff3.")
+merge_type_registry.register('lca', LCAMerger,
+                             "LCA-newness merge.")
+merge_type_registry.register('merge3', Merge3Merger,
+                             "Native diff3-style merge.")
+merge_type_registry.register('weave', WeaveMerger,
+                             "Weave-based merge.")
+
+
 def get_merge_type_registry():
-    """Merge type registry is in bzrlib.option to avoid circular imports.
+    """Merge type registry was previously in bzrlib.option
 
-    This method provides a sanctioned way to retrieve it.
+    This method provides a backwards compatible way to retrieve it.
     """
-    from bzrlib import option
-    return option._merge_type_registry
+    return merge_type_registry
 
 
 def _plan_annotate_merge(annotated_a, annotated_b, ancestors_a, ancestors_b):

=== modified file 'bzrlib/merge3.py'
--- bzrlib/merge3.py	2010-04-26 16:59:56 +0000
+++ bzrlib/merge3.py	2011-12-18 15:28:38 +0000
@@ -14,6 +14,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
 
 # mbp: "you know that thing where cvs gives you conflict markers?"
 # s: "i hate that."

=== modified file 'bzrlib/merge_directive.py'
--- bzrlib/merge_directive.py	2011-04-07 10:36:24 +0000
+++ bzrlib/merge_directive.py	2011-12-18 15:28:38 +0000
@@ -14,6 +14,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
 
 from StringIO import StringIO
 import re
@@ -82,8 +83,8 @@
     multiple_output_files = False
 
     def __init__(self, revision_id, testament_sha1, time, timezone,
-                 target_branch, patch=None, source_branch=None, message=None,
-                 bundle=None):
+                 target_branch, patch=None, source_branch=None,
+                 message=None, bundle=None):
         """Constructor.
 
         :param revision_id: The revision to merge
@@ -91,7 +92,7 @@
             merge.
         :param time: The current POSIX timestamp time
         :param timezone: The timezone offset
-        :param target_branch: The branch to apply the merge to
+        :param target_branch: Location of branch to apply the merge to
         :param patch: The text of a diff or bundle
         :param source_branch: A public location to merge the revision from
         :param message: The message to use when committing this merge
@@ -165,9 +166,9 @@
         :param target_branch: The url of the branch to merge into
         :param patch_type: 'bundle', 'diff' or None, depending on the type of
             patch desired.
-        :param local_target_branch: a local copy of the target branch
-        :param public_branch: location of a public branch containing the target
-            revision.
+        :param local_target_branch: the submit branch, either itself or a local copy
+        :param public_branch: location of a public branch containing
+            the target revision.
         :param message: Message to use when committing the merge
         :return: The merge directive
 
@@ -181,7 +182,10 @@
         if revision_id == _mod_revision.NULL_REVISION:
             t_revision_id = None
         t = testament.StrictTestament3.from_revision(repository, t_revision_id)
-        submit_branch = _mod_branch.Branch.open(target_branch)
+        if local_target_branch is None:
+            submit_branch = _mod_branch.Branch.open(target_branch)
+        else:
+            submit_branch = local_target_branch
         if submit_branch.get_public_branch() is not None:
             target_branch = submit_branch.get_public_branch()
         if patch_type is None:
@@ -244,7 +248,7 @@
         :param branch: The source branch, to get the signing strategy
         :return: a string
         """
-        my_gpg = gpg.GPGStrategy(branch.get_config())
+        my_gpg = gpg.GPGStrategy(branch.get_config_stack())
         return my_gpg.sign(''.join(self.to_lines()))
 
     def to_email(self, mail_to, branch, sign=False):
@@ -373,7 +377,7 @@
             merge.
         :param time: The current POSIX timestamp time
         :param timezone: The timezone offset
-        :param target_branch: The branch to apply the merge to
+        :param target_branch: Location of the branch to apply the merge to
         :param patch: The text of a diff or bundle
         :param patch_type: None, "diff" or "bundle", depending on the contents
             of patch
@@ -567,9 +571,9 @@
         :param target_branch: The url of the branch to merge into
         :param include_patch: If true, include a preview patch
         :param include_bundle: If true, include a bundle
-        :param local_target_branch: a local copy of the target branch
-        :param public_branch: location of a public branch containing the target
-            revision.
+        :param local_target_branch: the target branch, either itself or a local copy
+        :param public_branch: location of a public branch containing
+            the target revision.
         :param message: Message to use when committing the merge
         :return: The merge directive
 
@@ -588,7 +592,10 @@
                 t_revision_id = None
             t = testament.StrictTestament3.from_revision(repository,
                 t_revision_id)
-            submit_branch = _mod_branch.Branch.open(target_branch)
+            if local_target_branch is None:
+                submit_branch = _mod_branch.Branch.open(target_branch)
+            else:
+                submit_branch = local_target_branch
             submit_branch.lock_read()
             locked.append(submit_branch)
             if submit_branch.get_public_branch() is not None:

=== modified file 'bzrlib/mergetools.py'
--- bzrlib/mergetools.py	2011-01-21 23:51:15 +0000
+++ bzrlib/mergetools.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,8 @@
 
 """Utility functions for managing external merge tools such as kdiff3."""
 
+from __future__ import absolute_import
+
 import os
 import shutil
 import subprocess

=== modified file 'bzrlib/missing.py'
--- bzrlib/missing.py	2010-04-30 11:03:59 +0000
+++ bzrlib/missing.py	2011-12-19 13:23:58 +0000
@@ -16,8 +16,11 @@
 
 """Display what revisions are missing in 'other' from 'this' and vice versa."""
 
+from __future__ import absolute_import
+
 from bzrlib import (
     log,
+    symbol_versioning,
     )
 import bzrlib.revision as _mod_revision
 
@@ -42,8 +45,9 @@
 
 
 def find_unmerged(local_branch, remote_branch, restrict='all',
-                  include_merges=False, backward=False,
-                  local_revid_range=None, remote_revid_range=None):
+                  include_merged=None, backward=False,
+                  local_revid_range=None, remote_revid_range=None,
+                  include_merges=symbol_versioning.DEPRECATED_PARAMETER):
     """Find revisions from each side that have not been merged.
 
     :param local_branch: Compare the history of local_branch
@@ -53,7 +57,7 @@
         unique revisions from both sides. If 'local', we will return None
         for the remote revisions, similarly if 'remote' we will return None for
         the local revisions.
-    :param include_merges: Show mainline revisions only if False,
+    :param include_merged: Show mainline revisions only if False,
         all revisions otherwise.
     :param backward: Show oldest versions first when True, newest versions
         first when False.
@@ -61,17 +65,27 @@
         revisions (lower bound, upper bound)
     :param remote_revid_range: Revision-id range for filtering remote_branch
         revisions (lower bound, upper bound)
+    :param include_merges: Deprecated historical alias for include_merged
 
     :return: A list of [(revno, revision_id)] for the mainline revisions on
         each side.
     """
+    if symbol_versioning.deprecated_passed(include_merges):
+        symbol_versioning.warn(
+            'include_merges was deprecated in 2.5.'
+            ' Use include_merged instead.',
+            DeprecationWarning, stacklevel=2)
+        if include_merged is None:
+            include_merged = include_merges
+    if include_merged is None:
+        include_merged = False
     local_branch.lock_read()
     try:
         remote_branch.lock_read()
         try:
             return _find_unmerged(
                 local_branch, remote_branch, restrict=restrict,
-                include_merges=include_merges, backward=backward,
+                include_merged=include_merged, backward=backward,
                 local_revid_range=local_revid_range,
                 remote_revid_range=remote_revid_range)
         finally:
@@ -160,7 +174,7 @@
 
 
 def _find_unmerged(local_branch, remote_branch, restrict,
-                   include_merges, backward,
+                   include_merged, backward,
                    local_revid_range=None, remote_revid_range=None):
     """See find_unmerged.
 
@@ -186,7 +200,7 @@
                              ' "remote": %r' % (restrict,))
         local_extra, remote_extra = graph.find_difference(local_revision_id,
                                                           remote_revision_id)
-    if include_merges:
+    if include_merged:
         locals = _enumerate_with_merges(local_branch, local_extra,
                                         graph, local_revno,
                                         local_revision_id, backward)

=== modified file 'bzrlib/msgeditor.py'
--- bzrlib/msgeditor.py	2011-07-23 16:33:38 +0000
+++ bzrlib/msgeditor.py	2011-12-19 13:23:58 +0000
@@ -14,9 +14,10 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-
 """Commit message editor support."""
 
+from __future__ import absolute_import
+
 import codecs
 import os
 from subprocess import call

=== modified file 'bzrlib/multiparent.py'
--- bzrlib/multiparent.py	2011-04-05 01:12:15 +0000
+++ bzrlib/multiparent.py	2011-12-18 15:28:38 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 from bzrlib.lazy_import import lazy_import
 
 lazy_import(globals(), """
@@ -421,7 +423,7 @@
                             if not (lines == self.get_line_list([revision])[0]):
                                 raise AssertionError()
                             self.clear_cache()
-                    pb.update('Importing revisions',
+                    pb.update(gettext('Importing revisions'),
                               (total - len(revisions)) + len(added), total)
                 revisions = [r for r in revisions if r not in added]
         finally:

=== modified file 'bzrlib/mutabletree.py'
--- bzrlib/mutabletree.py	2011-08-15 20:10:03 +0000
+++ bzrlib/mutabletree.py	2011-12-19 13:23:58 +0000
@@ -19,6 +19,7 @@
 See MutableTree for more details.
 """
 
+from __future__ import absolute_import
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
@@ -71,7 +72,7 @@
     conformance tests for - rather we are testing MemoryTree specifically, and
     interface testing implementations of WorkingTree.
 
-    A mutable tree always has an associated Branch and BzrDir object - the
+    A mutable tree always has an associated Branch and ControlDir object - the
     branch and bzrdir attributes.
     """
     def __init__(self, *args, **kw):
@@ -260,7 +261,7 @@
         :param more_warning: Details about what is happening.
         """
         if strict is None:
-            strict = self.branch.get_config().get_user_option_as_bool(opt_name)
+            strict = self.branch.get_config_stack().get(opt_name)
         if strict is not False:
             err_class = None
             if (self.has_changes()):
@@ -746,9 +747,10 @@
                 # which is perhaps reasonable: adding a new reference is a
                 # special operation and can have a special behaviour.  mbp
                 # 20070306
-                trace.mutter("%r is a nested bzr tree", abspath)
+                trace.warning("skipping nested tree %r", abspath)
             else:
-                this_ie = self._add_one_and_parent(parent_ie, directory, kind, inv_path)
+                this_ie = self._add_one_and_parent(parent_ie, directory, kind,
+                    inv_path)
 
             if kind == 'directory' and not sub_tree:
                 if this_ie.kind != 'directory':

=== modified file 'bzrlib/option.py'
--- bzrlib/option.py	2011-06-27 15:42:09 +0000
+++ bzrlib/option.py	2011-12-18 12:46:49 +0000
@@ -17,6 +17,8 @@
 # TODO: For things like --diff-prefix, we want a way to customize the display
 # of the option argument.
 
+from __future__ import absolute_import
+
 import optparse
 import re
 
@@ -25,6 +27,7 @@
 from bzrlib import (
     errors,
     revisionspec,
+    i18n,
     )
 """)
 
@@ -331,7 +334,8 @@
         :param short_name: The short name for the enum switch, if any
         :param short_value_switches: A dict mapping values to short names
         """
-        Option.__init__(self, name, help, type=self.convert, short_name=short_name)
+        Option.__init__(self, name, help, type=self.convert,
+                        short_name=short_name)
         self._registry = registry
         if registry is None:
             if lazy_registry is None:
@@ -429,10 +433,26 @@
 
     DEFAULT_VALUE = object()
 
+    def __init__(self):
+        optparse.OptionParser.__init__(self)
+        self.formatter = GettextIndentedHelpFormatter()
+
     def error(self, message):
         raise errors.BzrCommandError(message)
 
 
+class GettextIndentedHelpFormatter(optparse.IndentedHelpFormatter):
+    """Adds gettext() call to format_option()"""
+    def __init__(self):
+        optparse.IndentedHelpFormatter.__init__(self)
+
+    def format_option(self, option):
+        """code taken from Python's optparse.py"""
+        if option.help:
+            option.help = i18n.gettext(option.help)
+        return optparse.IndentedHelpFormatter.format_option(self, option)
+
+
 def get_optparser(options):
     """Generate an optparse parser for bzrlib-style options"""
 
@@ -457,6 +477,12 @@
     Option.STD_OPTIONS[name] = Option(name, **kwargs)
     Option.OPTIONS[name] = Option.STD_OPTIONS[name]
 
+def _standard_list_option(name, **kwargs):
+    """Register a standard option."""
+    # All standard options are implicitly 'global' ones
+    Option.STD_OPTIONS[name] = ListOption(name, **kwargs)
+    Option.OPTIONS[name] = Option.STD_OPTIONS[name]
+
 
 def _global_option(name, **kwargs):
     """Register a global option."""
@@ -495,94 +521,54 @@
             _verbosity_level = -1
 
 
-class MergeTypeRegistry(_mod_registry.Registry):
-
-    pass
-
-
-_merge_type_registry = MergeTypeRegistry()
-_merge_type_registry.register_lazy('merge3', 'bzrlib.merge', 'Merge3Merger',
-                                   "Native diff3-style merge")
-_merge_type_registry.register_lazy('diff3', 'bzrlib.merge', 'Diff3Merger',
-                                   "Merge using external diff3")
-_merge_type_registry.register_lazy('weave', 'bzrlib.merge', 'WeaveMerger',
-                                   "Weave-based merge")
-_merge_type_registry.register_lazy('lca', 'bzrlib.merge', 'LCAMerger',
-                                   "LCA-newness merge")
-
 # Declare the standard options
 _standard_option('help', short_name='h',
                  help='Show help message.')
+_standard_option('quiet', short_name='q',
+                 help="Only display errors and warnings.",
+                 custom_callback=_verbosity_level_callback)
 _standard_option('usage',
                  help='Show usage message and options.')
 _standard_option('verbose', short_name='v',
                  help='Display more information.',
                  custom_callback=_verbosity_level_callback)
-_standard_option('quiet', short_name='q',
-                 help="Only display errors and warnings.",
-                 custom_callback=_verbosity_level_callback)
 
 # Declare commonly used options
-_global_option('all')
-_global_option('overwrite', help='Ignore differences between branches and '
-               'overwrite unconditionally.')
-_global_option('basis', type=str)
-_global_option('bound')
-_global_option('diff-options', type=str)
+_global_option('change',
+               type=_parse_change_str,
+               short_name='c',
+               param_name='revision',
+               help='Select changes introduced by the specified revision. See also "help revisionspec".')
+_global_option('directory', short_name='d', type=unicode,
+               help='Branch to operate on, instead of working directory.')
 _global_option('file', type=unicode, short_name='F')
-_global_option('force')
-_global_option('format', type=unicode)
-_global_option('forward')
+_global_registry_option('log-format', "Use specified log format.",
+                        lazy_registry=('bzrlib.log', 'log_formatter_registry'),
+                        value_switches=True, title='Log format',
+                        short_value_switches={'short': 'S'})
+_global_registry_option('merge-type', 'Select a particular merge algorithm.',
+                        lazy_registry=('bzrlib.merge', 'merge_type_registry'),
+                        value_switches=True, title='Merge algorithm')
 _global_option('message', type=unicode,
                short_name='m',
                help='Message string.')
-_global_option('no-recurse')
 _global_option('null', short_name='0',
                  help='Use an ASCII NUL (\\0) separator rather than '
                       'a newline.')
-_global_option('profile',
-               help='Show performance profiling information.')
+_global_option('overwrite', help='Ignore differences between branches and '
+               'overwrite unconditionally.')
+_global_option('remember', help='Remember the specified location as a'
+               ' default.')
+_global_option('reprocess', help='Reprocess to reduce spurious conflicts.')
 _global_option('revision',
                type=_parse_revision_str,
                short_name='r',
                help='See "help revisionspec" for details.')
-_global_option('change',
-               type=_parse_change_str,
-               short_name='c',
-               param_name='revision',
-               help='Select changes introduced by the specified revision. See also "help revisionspec".')
 _global_option('show-ids',
                help='Show internal object ids.')
 _global_option('timezone',
                type=str,
                help='Display timezone as local, original, or utc.')
-_global_option('unbound')
-_global_option('version')
-_global_option('email')
-_global_option('update')
-_global_registry_option('log-format', "Use specified log format.",
-                        lazy_registry=('bzrlib.log', 'log_formatter_registry'),
-                        value_switches=True, title='Log format',
-                        short_value_switches={'short': 'S'})
-_global_option('long', help='Use detailed log format. Same as --log-format long',
-               short_name='l')
-_global_option('short', help='Use moderately short log format. Same as --log-format short')
-_global_option('line', help='Use log format with one line per revision. Same as --log-format line')
-_global_option('root', type=str)
-_global_option('no-backup')
-_global_registry_option('merge-type', 'Select a particular merge algorithm.',
-                        _merge_type_registry, value_switches=True,
-                        title='Merge algorithm')
-_global_option('pattern', type=str)
-_global_option('remember', help='Remember the specified location as a'
-               ' default.')
-_global_option('reprocess', help='Reprocess to reduce spurious conflicts.')
-_global_option('kind', type=str)
-_global_option('dry-run',
-               help="Show what would be done, but don't actually do anything.")
-_global_option('name-from-revision', help='The path name in the old tree.')
-_global_option('directory', short_name='d', type=unicode,
-               help='Branch to operate on, instead of working directory')
 
 diff_writer_registry = _mod_registry.Registry()
 diff_writer_registry.register('plain', lambda x: x, 'Plaintext diff output.')

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2011-08-24 23:55:03 +0000
+++ bzrlib/osutils.py	2012-01-03 17:11:56 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 import errno
 import os
 import re
@@ -28,6 +30,7 @@
 import getpass
 import ntpath
 import posixpath
+import select
 # We need to import both shutil and rmtree as we export the later on posix
 # and need the former on windows
 import shutil
@@ -47,6 +50,7 @@
     trace,
     win32utils,
     )
+from bzrlib.i18n import gettext
 """)
 
 from bzrlib.symbol_versioning import (
@@ -61,7 +65,7 @@
 
 
 import bzrlib
-from bzrlib import symbol_versioning
+from bzrlib import symbol_versioning, _fs_enc
 
 
 # Cross platform wall-clock time functionality with decent resolution.
@@ -89,8 +93,8 @@
         user_encoding = get_user_encoding()
         return [a.decode(user_encoding) for a in sys.argv[1:]]
     except UnicodeDecodeError:
-        raise errors.BzrError("Parameter %r encoding is unsupported by %s "
-            "application locale." % (a, user_encoding))
+        raise errors.BzrError(gettext("Parameter {0!r} encoding is unsupported by {1} "
+            "application locale.").format(a, user_encoding))
 
 
 def make_readonly(filename):
@@ -98,14 +102,33 @@
     mod = os.lstat(filename).st_mode
     if not stat.S_ISLNK(mod):
         mod = mod & 0777555
-        os.chmod(filename, mod)
+        chmod_if_possible(filename, mod)
 
 
 def make_writable(filename):
     mod = os.lstat(filename).st_mode
     if not stat.S_ISLNK(mod):
         mod = mod | 0200
-        os.chmod(filename, mod)
+        chmod_if_possible(filename, mod)
+
+
+def chmod_if_possible(filename, mode):
+    # Set file mode if that can be safely done.
+    # Sometimes even on unix the filesystem won't allow it - see
+    # https://bugs.launchpad.net/bzr/+bug/606537
+    try:
+        # It is probably faster to just do the chmod, rather than
+        # doing a stat, and then trying to compare
+        os.chmod(filename, mode)
+    except (IOError, OSError),e:
+        # Permission/access denied seems to commonly happen on smbfs; there's
+        # probably no point warning about it.
+        # <https://bugs.launchpad.net/bzr/+bug/606537>
+        if getattr(e, 'errno') in (errno.EPERM, errno.EACCES):
+            trace.mutter("ignore error on chmod of %r: %r" % (
+                filename, e))
+            return
+        raise
 
 
 def minimum_path_selection(paths):
@@ -190,7 +213,7 @@
             if e.errno == errno.ENOENT:
                 return False;
             else:
-                raise errors.BzrError("lstat/stat of (%r): %r" % (f, e))
+                raise errors.BzrError(gettext("lstat/stat of ({0!r}): {1!r}").format(f, e))
 
 
 def fancy_rename(old, new, rename_func, unlink_func):
@@ -272,19 +295,60 @@
 # choke on a Unicode string containing a relative path if
 # os.getcwd() returns a non-sys.getdefaultencoding()-encoded
 # string.
-_fs_enc = sys.getfilesystemencoding() or 'utf-8'
 def _posix_abspath(path):
     # jam 20060426 rather than encoding to fsencoding
     # copy posixpath.abspath, but use os.getcwdu instead
     if not posixpath.isabs(path):
         path = posixpath.join(getcwd(), path)
-    return posixpath.normpath(path)
+    return _posix_normpath(path)
 
 
 def _posix_realpath(path):
     return posixpath.realpath(path.encode(_fs_enc)).decode(_fs_enc)
 
 
+def _posix_normpath(path):
+    path = posixpath.normpath(path)
+    # Bug 861008: posixpath.normpath() returns a path normalized according to
+    # the POSIX standard, which stipulates (for compatibility reasons) that two
+    # leading slashes must not be simplified to one, and only if there are 3 or
+    # more should they be simplified as one. So we treat the leading 2 slashes
+    # as a special case here by simply removing the first slash, as we consider
+    # that breaking POSIX compatibility for this obscure feature is acceptable.
+    # This is not a paranoid precaution, as we notably get paths like this when
+    # the repo is hosted at the root of the filesystem, i.e. in "/".    
+    if path.startswith('//'):
+        path = path[1:]
+    return path
+
+
+def _posix_path_from_environ(key):
+    """Get unicode path from `key` in environment or None if not present
+
+    Note that posix systems use arbitrary byte strings for filesystem objects,
+    so a path that raises BadFilenameEncoding here may still be accessible.
+    """
+    val = os.environ.get(key, None)
+    if val is None:
+        return val
+    try:
+        return val.decode(_fs_enc)
+    except UnicodeDecodeError:
+        # GZ 2011-12-12:Ideally want to include `key` in the exception message
+        raise errors.BadFilenameEncoding(val, _fs_enc)
+
+
+def _posix_getuser_unicode():
+    """Get username from environment or password database as unicode"""
+    name = getpass.getuser()
+    user_encoding = get_user_encoding()
+    try:
+        return name.decode(user_encoding)
+    except UnicodeDecodeError:
+        raise errors.BzrError("Encoding of username %r is unsupported by %s "
+            "application locale." % (name, user_encoding))
+
+
 def _win32_fixdrive(path):
     """Force drive letters to be consistent.
 
@@ -378,7 +442,9 @@
 abspath = _posix_abspath
 realpath = _posix_realpath
 pathjoin = os.path.join
-normpath = os.path.normpath
+normpath = _posix_normpath
+path_from_environ = _posix_path_from_environ
+getuser_unicode = _posix_getuser_unicode
 getcwd = os.getcwdu
 rename = os.rename
 dirname = os.path.dirname
@@ -440,6 +506,8 @@
     f = win32utils.get_unicode_argv     # special function or None
     if f is not None:
         get_unicode_argv = f
+    path_from_environ = win32utils.get_environ_unicode
+    getuser_unicode = win32utils.get_user_name
 
 elif sys.platform == 'darwin':
     getcwd = _mac_getcwd
@@ -877,19 +945,16 @@
     return os.fstat(f.fileno())[stat.ST_SIZE]
 
 
-# Define rand_bytes based on platform.
-try:
-    # Python 2.4 and later have os.urandom,
-    # but it doesn't work on some arches
-    os.urandom(1)
-    rand_bytes = os.urandom
-except (NotImplementedError, AttributeError):
-    # If python doesn't have os.urandom, or it doesn't work,
-    # then try to first pull random data from /dev/urandom
+# Alias os.urandom to support platforms (which?) without /dev/urandom and 
+# override if it doesn't work. Avoid checking on windows where there is
+# significant initialisation cost that can be avoided for some bzr calls.
+
+rand_bytes = os.urandom
+
+if rand_bytes.__module__ != "nt":
     try:
-        rand_bytes = file('/dev/urandom', 'rb').read
-    # Otherwise, use this hack as a last resort
-    except (IOError, OSError):
+        rand_bytes(1)
+    except NotImplementedError:
         # not well seeded, but better than nothing
         def rand_bytes(n):
             import random
@@ -925,7 +990,7 @@
     rps = []
     for f in ps:
         if f == '..':
-            raise errors.BzrError("sorry, %r not allowed in path" % f)
+            raise errors.BzrError(gettext("sorry, %r not allowed in path") % f)
         elif (f == '.') or (f == ''):
             pass
         else:
@@ -936,7 +1001,7 @@
 def joinpath(p):
     for f in p:
         if (f == '..') or (f is None) or (f == ''):
-            raise errors.BzrError("sorry, %r not allowed in path" % f)
+            raise errors.BzrError(gettext("sorry, %r not allowed in path") % f)
     return pathjoin(*p)
 
 
@@ -1154,7 +1219,7 @@
 
     if len(base) < MIN_ABS_PATHLENGTH:
         # must have space for e.g. a drive letter
-        raise ValueError('%r is too short to calculate a relative path'
+        raise ValueError(gettext('%r is too short to calculate a relative path')
             % (base,))
 
     rp = abspath(path)
@@ -1735,7 +1800,6 @@
     """
     global _selected_dir_reader
     if _selected_dir_reader is None:
-        fs_encoding = _fs_enc.upper()
         if sys.platform == "win32" and win32utils.winver == 'Windows NT':
             # Win98 doesn't have unicode apis like FindFirstFileW
             # TODO: We possibly could support Win98 by falling back to the
@@ -1747,8 +1811,7 @@
                 _selected_dir_reader = Win32ReadDir()
             except ImportError:
                 pass
-        elif fs_encoding in ('UTF-8', 'US-ASCII', 'ANSI_X3.4-1968'):
-            # ANSI_X3.4-1968 is a form of ASCII
+        elif _fs_enc in ('utf-8', 'ascii'):
             try:
                 from bzrlib._readdir_pyx import UTF8DirReader
                 _selected_dir_reader = UTF8DirReader()
@@ -1990,6 +2053,28 @@
     return get_terminal_encoding()
 
 
+_message_encoding = None
+
+
+def get_message_encoding():
+    """Return the encoding used for messages
+
+    While the message encoding is a general setting it should usually only be
+    needed for decoding system error strings such as from OSError instances.
+    """
+    global _message_encoding
+    if _message_encoding is None:
+        if os.name == "posix":
+            import locale
+            # This is a process-global setting that can change, but should in
+            # general just get set once at process startup then be constant.
+            _message_encoding = locale.getlocale(locale.LC_MESSAGES)[1]
+        else:
+            # On windows want the result of GetACP() which this boils down to.
+            _message_encoding = get_user_encoding()
+    return _message_encoding or "ascii"
+
+
 def get_host_name():
     """Return the current unicode host name.
 
@@ -1997,7 +2082,6 @@
     behaves inconsistently on different platforms.
     """
     if sys.platform == "win32":
-        import win32utils
         return win32utils.get_host_name()
     else:
         import socket
@@ -2239,13 +2323,13 @@
 
 
 if sys.platform == "win32":
-    import msvcrt
     def getchar():
+        import msvcrt
         return msvcrt.getch()
 else:
-    import tty
-    import termios
     def getchar():
+        import tty
+        import termios
         fd = sys.stdin.fileno()
         settings = termios.tcgetattr(fd)
         try:
@@ -2300,14 +2384,15 @@
     if concurrency is None:
         try:
             import multiprocessing
-        except ImportError:
+            concurrency = multiprocessing.cpu_count()
+        except (ImportError, NotImplementedError):
             # multiprocessing is only available on Python >= 2.6
+            # and multiprocessing.cpu_count() isn't implemented on all
+            # platforms
             try:
                 concurrency = _local_concurrency()
             except (OSError, IOError):
                 pass
-        else:
-            concurrency = multiprocessing.cpu_count()
     try:
         concurrency = int(concurrency)
     except (TypeError, ValueError):
@@ -2375,30 +2460,6 @@
     open_file = open
 
 
-def getuser_unicode():
-    """Return the username as unicode.
-    """
-    try:
-        user_encoding = get_user_encoding()
-        username = getpass.getuser().decode(user_encoding)
-    except UnicodeDecodeError:
-        raise errors.BzrError("Can't decode username as %s." % \
-                user_encoding)
-    except ImportError, e:
-        if sys.platform != 'win32':
-            raise
-        if str(e) != 'No module named pwd':
-            raise
-        # https://bugs.launchpad.net/bzr/+bug/660174
-        # getpass.getuser() is unable to return username on Windows
-        # if there is no USERNAME environment variable set.
-        # That could be true if bzr is running as a service,
-        # e.g. running `bzr serve` as a service on Windows.
-        # We should not fail with traceback in this case.
-        username = u'UNKNOWN'
-    return username
-
-
 def available_backup_name(base, exists):
     """Find a non-existing backup file name.
 
@@ -2501,3 +2562,31 @@
     fn = getattr(os, 'fdatasync', getattr(os, 'fsync', None))
     if fn is not None:
         fn(fileno)
+
+
+def ensure_empty_directory_exists(path, exception_class):
+    """Make sure a local directory exists and is empty.
+    
+    If it does not exist, it is created.  If it exists and is not empty, an
+    instance of exception_class is raised.
+    """
+    try:
+        os.mkdir(path)
+    except OSError, e:
+        if e.errno != errno.EEXIST:
+            raise
+        if os.listdir(path) != []:
+            raise exception_class(path)
+
+
+def is_environment_error(evalue):
+    """True if exception instance is due to a process environment issue
+
+    This includes OSError and IOError, but also other errors that come from
+    the operating system or core libraries but are not subclasses of those.
+    """
+    if isinstance(evalue, (EnvironmentError, select.error)):
+        return True
+    if sys.platform == "win32" and win32utils._is_pywintypes_error(evalue):
+        return True
+    return False

=== modified file 'bzrlib/pack.py'
--- bzrlib/pack.py	2011-04-08 14:45:25 +0000
+++ bzrlib/pack.py	2011-12-19 13:23:58 +0000
@@ -20,6 +20,8 @@
 doc/developers/container-format.txt.
 """
 
+from __future__ import absolute_import
+
 from cStringIO import StringIO
 import re
 
@@ -75,14 +77,12 @@
         """Return the bytes to finish a container."""
         return "E"
 
-    def bytes_record(self, bytes, names):
-        """Return the bytes for a Bytes record with the given name and
-        contents.
-        """
+    def bytes_header(self, length, names):
+        """Return the header for a Bytes record."""
         # Kind marker
         byte_sections = ["B"]
         # Length
-        byte_sections.append(str(len(bytes)) + "\n")
+        byte_sections.append(str(length) + "\n")
         # Names
         for name_tuple in names:
             # Make sure we're writing valid names.  Note that we will leave a
@@ -92,18 +92,17 @@
             byte_sections.append('\x00'.join(name_tuple) + "\n")
         # End of headers
         byte_sections.append("\n")
-        # Finally, the contents.
-        byte_sections.append(bytes)
-        # XXX: This causes a memory copy of bytes in size, but is usually
-        # faster than two write calls (12 vs 13 seconds to output a gig of
-        # 1k records.) - results may differ on significantly larger records
-        # like .iso's but as they should be rare in any case and thus not
-        # likely to be the common case. The biggest issue is causing extreme
-        # memory pressure in that case. One possibly improvement here is to
-        # check the size of the content before deciding to join here vs call
-        # write twice.
         return ''.join(byte_sections)
 
+    def bytes_record(self, bytes, names):
+        """Return the bytes for a Bytes record with the given name and
+        contents.
+
+        If the content may be large, construct the header separately and then
+        stream out the contents.
+        """
+        return self.bytes_header(len(bytes), names) + bytes
+
 
 class ContainerWriter(object):
     """A class for writing containers to a file.
@@ -113,6 +112,10 @@
         introduced by the begin() and end() methods.
     """
 
+    # Join up headers with the body if writing fewer than this many bytes:
+    # trades off memory usage and copying to do less IO ops.
+    _JOIN_WRITES_THRESHOLD = 100000
+
     def __init__(self, write_func):
         """Constructor.
 
@@ -151,8 +154,13 @@
             and thus are only suitable for use by a ContainerReader.
         """
         current_offset = self.current_offset
-        serialised_record = self._serialiser.bytes_record(bytes, names)
-        self.write_func(serialised_record)
+        length = len(bytes)
+        if length < self._JOIN_WRITES_THRESHOLD:
+            self.write_func(self._serialiser.bytes_header(length, names)
+                + bytes)
+        else:
+            self.write_func(self._serialiser.bytes_header(length, names))
+            self.write_func(bytes)
         self.records_written += 1
         # return a memo of where we wrote data to allow random access.
         return current_offset, self.current_offset - current_offset
@@ -333,7 +341,7 @@
                 # risk that the same unicode string has been encoded two
                 # different ways.
                 if name_tuple in all_names:
-                    raise errors.DuplicateRecordNameError(name_tuple)
+                    raise errors.DuplicateRecordNameError(name_tuple[0])
                 all_names.add(name_tuple)
         excess_bytes = self.reader_func(1)
         if excess_bytes != '':

=== modified file 'bzrlib/patch.py'
--- bzrlib/patch.py	2009-03-23 14:59:43 +0000
+++ bzrlib/patch.py	2011-12-18 15:28:38 +0000
@@ -14,6 +14,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
 
 import errno
 import os

=== modified file 'bzrlib/patches.py'
--- bzrlib/patches.py	2010-02-25 06:17:27 +0000
+++ bzrlib/patches.py	2011-12-18 15:28:38 +0000
@@ -14,58 +14,23 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+from __future__ import absolute_import
+
+from bzrlib.errors import (
+    BinaryFiles,
+    MalformedHunkHeader,
+    MalformedLine,
+    MalformedPatchHeader,
+    PatchConflict,
+    PatchSyntax,
+    )
+
 import re
 
 
 binary_files_re = 'Binary files (.*) and (.*) differ\n'
 
-
-class BinaryFiles(Exception):
-
-    def __init__(self, orig_name, mod_name):
-        self.orig_name = orig_name
-        self.mod_name = mod_name
-        Exception.__init__(self, 'Binary files section encountered.')
-
-
-class PatchSyntax(Exception):
-    def __init__(self, msg):
-        Exception.__init__(self, msg)
-
-
-class MalformedPatchHeader(PatchSyntax):
-    def __init__(self, desc, line):
-        self.desc = desc
-        self.line = line
-        msg = "Malformed patch header.  %s\n%r" % (self.desc, self.line)
-        PatchSyntax.__init__(self, msg)
-
-
-class MalformedHunkHeader(PatchSyntax):
-    def __init__(self, desc, line):
-        self.desc = desc
-        self.line = line
-        msg = "Malformed hunk header.  %s\n%r" % (self.desc, self.line)
-        PatchSyntax.__init__(self, msg)
-
-
-class MalformedLine(PatchSyntax):
-    def __init__(self, desc, line):
-        self.desc = desc
-        self.line = line
-        msg = "Malformed line.  %s\n%s" % (self.desc, self.line)
-        PatchSyntax.__init__(self, msg)
-
-
-class PatchConflict(Exception):
-    def __init__(self, line_no, orig_line, patch_line):
-        orig = orig_line.rstrip('\n')
-        patch = str(patch_line).rstrip('\n')
-        msg = 'Text contents mismatch at line %d.  Original has "%s",'\
-            ' but patch says it should be "%s"' % (line_no, orig, patch)
-        Exception.__init__(self, msg)
-
-
 def get_patch_names(iter_lines):
     try:
         line = iter_lines.next()

=== modified file 'bzrlib/patiencediff.py'
--- bzrlib/patiencediff.py	2011-04-05 01:12:15 +0000
+++ bzrlib/patiencediff.py	2011-12-18 15:28:38 +0000
@@ -15,6 +15,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """

=== modified file 'bzrlib/plugin.py'
--- bzrlib/plugin.py	2011-08-12 07:32:22 +0000
+++ bzrlib/plugin.py	2011-12-19 13:23:58 +0000
@@ -14,7 +14,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-
 """bzr python plugin support.
 
 When load_plugins() is invoked, any python module in any directory in
@@ -30,6 +29,8 @@
 called.
 """
 
+from __future__ import absolute_import
+
 import os
 import sys
 
@@ -48,6 +49,7 @@
     errors,
     trace,
     )
+from bzrlib.i18n import gettext
 from bzrlib import plugins as _mod_plugins
 """)
 
@@ -137,8 +139,8 @@
         try:
             name, path = spec.split('@')
         except ValueError:
-            raise errors.BzrCommandError(
-                '"%s" is not a valid <plugin_name>@<plugin_path> description '
+            raise errors.BzrCommandError(gettext(
+                '"%s" is not a valid <plugin_name>@<plugin_path> description ')
                 % spec)
         specs.append((name, path))
     return specs
@@ -630,7 +632,7 @@
         return None
 
     def load_module(self, fullname):
-        """Load a plugin from a specific directory."""
+        """Load a plugin from a specific directory (or file)."""
         # We are called only for specific paths
         plugin_path = self.specific_paths[fullname]
         loading_path = None

=== modified file 'bzrlib/plugins/__init__.py'
--- bzrlib/plugins/__init__.py	2009-03-23 14:59:43 +0000
+++ bzrlib/plugins/__init__.py	2011-12-19 13:23:58 +0000
@@ -15,3 +15,5 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 """Null placeholder plugin"""
+from __future__ import absolute_import
+

=== modified file 'bzrlib/plugins/bash_completion/__init__.py'
--- bzrlib/plugins/bash_completion/__init__.py	2010-05-25 21:43:32 +0000
+++ bzrlib/plugins/bash_completion/__init__.py	2011-12-18 12:46:49 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 __doc__ = """Generate a shell function for bash command line completion.
 
 This plugin provides a command called bash-completion that generates a

=== modified file 'bzrlib/plugins/bash_completion/bashcomp.py'
--- bzrlib/plugins/bash_completion/bashcomp.py	2010-05-29 08:47:10 +0000
+++ bzrlib/plugins/bash_completion/bashcomp.py	2011-12-20 17:56:21 +0000
@@ -16,6 +16,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 from bzrlib import (
     cmdline,
     commands,
@@ -26,6 +28,7 @@
 )
 import bzrlib
 import re
+import sys
 
 
 class BashCodeGen(object):
@@ -430,8 +433,6 @@
         ]
 
     def run(self, **kwargs):
-        import sys
-        from bashcomp import bash_completion_function
         if 'plugin' in kwargs:
             # work around bug #387117 which prevents us from using param_name
             if len(kwargs['plugin']) > 0:
@@ -442,7 +443,6 @@
 
 if __name__ == '__main__':
 
-    import sys
     import locale
     import optparse
 

=== modified file 'bzrlib/plugins/bash_completion/tests/test_bashcomp.py'
--- bzrlib/plugins/bash_completion/tests/test_bashcomp.py	2011-01-28 10:28:35 +0000
+++ bzrlib/plugins/bash_completion/tests/test_bashcomp.py	2011-12-19 16:08:00 +0000
@@ -21,7 +21,6 @@
 from bzrlib.tests import features
 from bzrlib.plugins.bash_completion.bashcomp import *
 
-import os
 import subprocess
 
 
@@ -325,3 +324,9 @@
                 self.assertSubset(['2a'], opt.registry_keys)
                 return
         raise AssertionError('Option --format not found')
+
+
+class BlackboxTests(tests.TestCase):
+
+    def test_bash_completion(self):
+        self.run_bzr("bash-completion")

=== modified file 'bzrlib/plugins/changelog_merge/__init__.py'
--- bzrlib/plugins/changelog_merge/__init__.py	2011-03-31 15:10:05 +0000
+++ bzrlib/plugins/changelog_merge/__init__.py	2011-12-18 12:46:49 +0000
@@ -14,9 +14,11 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 __doc__ = """Merge hook for GNU-format ChangeLog files
 
-To enable this plugin, add a section to your location.conf
+To enable this plugin, add a section to your locations.conf
 like::
 
     [/home/user/proj]

=== modified file 'bzrlib/plugins/changelog_merge/changelog_merge.py'
--- bzrlib/plugins/changelog_merge/changelog_merge.py	2011-05-13 06:50:20 +0000
+++ bzrlib/plugins/changelog_merge/changelog_merge.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,8 @@
 
 """Merge logic for changelog_merge plugin."""
 
+from __future__ import absolute_import
+
 import difflib
 
 from bzrlib import merge

=== modified file 'bzrlib/plugins/changelog_merge/tests/test_changelog_merge.py'
--- bzrlib/plugins/changelog_merge/tests/test_changelog_merge.py	2011-05-13 06:34:55 +0000
+++ bzrlib/plugins/changelog_merge/tests/test_changelog_merge.py	2011-12-21 14:43:39 +0000
@@ -188,7 +188,7 @@
         merger = builder.make_merger(merge.Merge3Merger, ['clog-id'])
         merger.this_branch.get_config().set_user_option(
             'changelog_merge_files', 'ChangeLog')
-        merge_hook_params = merge.MergeHookParams(merger, 'clog-id', None,
+        merge_hook_params = merge.MergeFileHookParams(merger, 'clog-id', None,
             'file', 'file', 'conflict')
         changelog_merger = changelog_merge.ChangeLogMerger(merger)
         return changelog_merger, merge_hook_params

=== modified file 'bzrlib/plugins/launchpad/__init__.py'
--- bzrlib/plugins/launchpad/__init__.py	2011-08-21 09:24:01 +0000
+++ bzrlib/plugins/launchpad/__init__.py	2011-12-18 12:46:49 +0000
@@ -35,6 +35,8 @@
 
 """
 
+from __future__ import absolute_import
+
 # The XMLRPC server address can be overridden by setting the environment
 # variable $BZR_LP_XMLRPC_URL
 
@@ -46,6 +48,7 @@
     ui,
     trace,
     )
+from bzrlib.i18n import gettext
 """)
 
 from bzrlib import (
@@ -144,14 +147,16 @@
             try:
                 b = _mod_branch.Branch.open_containing('.')[0]
             except NotBranchError:
-                raise BzrCommandError('register-branch requires a public '
-                    'branch url - see bzr help register-branch.')
+                raise BzrCommandError(gettext(
+                            'register-branch requires a public '
+                            'branch url - see bzr help register-branch.'))
             public_url = b.get_public_branch()
             if public_url is None:
                 raise NoPublicBranch(b)
         if product is not None:
             project = product
-            trace.note('--product is deprecated; please use --project.')
+            trace.note(gettext(
+                '--product is deprecated; please use --project.'))
 
 
         rego = BranchRegistrationRequest(branch_url=public_url,
@@ -223,7 +228,7 @@
         if location is None:
             location = u'.'
         web_url = self._get_web_url(LaunchpadService(), location)
-        trace.note('Opening %s in web browser' % web_url)
+        trace.note(gettext('Opening %s in web browser') % web_url)
         if not dry_run:
             import webbrowser   # this import should not be lazy
                                 # otherwise bzr.exe lacks this module
@@ -267,22 +272,23 @@
                 if check_account:
                     account.check_lp_login(username)
                     if verbose:
-                        self.outf.write(
-                            "Launchpad user ID exists and has SSH keys.\n")
+                        self.outf.write(gettext(
+                            "Launchpad user ID exists and has SSH keys.\n"))
                 self.outf.write(username + '\n')
             else:
-                self.outf.write('No Launchpad user ID configured.\n')
+                self.outf.write(gettext('No Launchpad user ID configured.\n'))
                 return 1
         else:
             name = name.lower()
             if check_account:
                 account.check_lp_login(name)
                 if verbose:
-                    self.outf.write(
-                        "Launchpad user ID exists and has SSH keys.\n")
+                    self.outf.write(gettext(
+                        "Launchpad user ID exists and has SSH keys.\n"))
             account.set_lp_login(name)
             if verbose:
-                self.outf.write("Launchpad user ID set to '%s'.\n" % (name,))
+                self.outf.write(gettext("Launchpad user ID set to '%s'.\n") %
+                                                                        (name,))
 
 register_command(cmd_launchpad_login)
 
@@ -400,8 +406,8 @@
             revno = self._find_merged_revno(revision, b, pb)
             merged = self._find_proposals(revno, b, pb)
             if len(merged) == 0:
-                raise BzrCommandError('No review found.')
-            trace.note('%d proposals(s) found.' % len(merged))
+                raise BzrCommandError(gettext('No review found.'))
+            trace.note(gettext('%d proposals(s) found.') % len(merged))
             for mp in merged:
                 webbrowser.open(lp_api.canonical_url(mp))
         finally:
@@ -411,27 +417,27 @@
     def _find_merged_revno(self, revision, b, pb):
         if revision is None:
             return b.revno()
-        pb.update('Finding revision-id')
+        pb.update(gettext('Finding revision-id'))
         revision_id = revision[0].as_revision_id(b)
         # a revno spec is necessarily on the mainline.
         if self._is_revno_spec(revision[0]):
             merging_revision = revision_id
         else:
             graph = b.repository.get_graph()
-            pb.update('Finding merge')
+            pb.update(gettext('Finding merge'))
             merging_revision = graph.find_lefthand_merger(
                 revision_id, b.last_revision())
             if merging_revision is None:
                 raise InvalidRevisionSpec(revision[0].user_spec, b)
-        pb.update('Finding revno')
+        pb.update(gettext('Finding revno'))
         return b.revision_id_to_revno(merging_revision)
 
     def _find_proposals(self, revno, b, pb):
         launchpad = lp_api.login(lp_registration.LaunchpadService())
-        pb.update('Finding Launchpad branch')
+        pb.update(gettext('Finding Launchpad branch'))
         lpb = lp_api.LaunchpadBranch.from_bzr(launchpad, b,
                                               create_missing=False)
-        pb.update('Finding proposals')
+        pb.update(gettext('Finding proposals'))
         return list(lpb.lp.getMergeProposals(status=['Merged'],
                                              merged_revnos=[revno]))
 

=== modified file 'bzrlib/plugins/launchpad/account.py'
--- bzrlib/plugins/launchpad/account.py	2011-07-23 16:33:38 +0000
+++ bzrlib/plugins/launchpad/account.py	2011-12-19 13:23:58 +0000
@@ -20,13 +20,15 @@
 than once for each place that needs to take it into account.
 """
 
+from __future__ import absolute_import
+
 from bzrlib import (
     errors,
     trace,
     transport,
     )
 from bzrlib.config import AuthenticationConfig, GlobalConfig
-
+from bzrlib.i18n import gettext
 
 LAUNCHPAD_BASE = 'https://launchpad.net/'
 
@@ -61,7 +63,7 @@
         auth_username = _get_auth_user(auth)
         # Auto-upgrading
         if auth_username is None:
-            trace.note('Setting ssh/sftp usernames for launchpad.net.')
+            trace.note(gettext('Setting ssh/sftp usernames for launchpad.net.'))
             _set_auth_user(username, auth)
         elif auth_username != username:
             raise MismatchedUsernames()

=== modified file 'bzrlib/plugins/launchpad/lp_api.py'
--- bzrlib/plugins/launchpad/lp_api.py	2011-02-11 05:27:33 +0000
+++ bzrlib/plugins/launchpad/lp_api.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,8 @@
 
 """Tools for dealing with the Launchpad API."""
 
+from __future__ import absolute_import
+
 # Importing this module will be expensive, since it imports launchpadlib and
 # its dependencies. However, our plan is to only load this module when it is
 # needed by a command that uses it.
@@ -33,6 +35,7 @@
     trace,
     transport,
     )
+from bzrlib.i18n import gettext
 from bzrlib.plugins.launchpad.lp_registration import (
     InvalidLaunchpadInstance,
     )
@@ -134,7 +137,7 @@
         proxy_info=proxy_info)
     # XXX: Work-around a minor security bug in launchpadlib 1.5.1, which would
     # create this directory with default umask.
-    os.chmod(cache_directory, 0700)
+    osutils.chmod_if_possible(cache_directory, 0700)
     return launchpad
 
 
@@ -235,12 +238,13 @@
         """Create a Bazaar branch on Launchpad for the supplied branch."""
         url = cls.tweak_url(bzr_branch.get_push_location(), launchpad)
         if not cls.plausible_launchpad_url(url):
-            raise errors.BzrError('%s is not registered on Launchpad' %
+            raise errors.BzrError(gettext('%s is not registered on Launchpad') %
                                   bzr_branch.base)
         bzr_branch.create_clone_on_transport(transport.get_transport(url))
         lp_branch = launchpad.branches.getByUrl(url=url)
         if lp_branch is None:
-            raise errors.BzrError('%s is not registered on Launchpad' % url)
+            raise errors.BzrError(gettext('%s is not registered on Launchpad') %
+                                                                            url)
         return lp_branch
 
     def get_target(self):
@@ -249,18 +253,21 @@
         if lp_branch.project is not None:
             dev_focus = lp_branch.project.development_focus
             if dev_focus is None:
-                raise errors.BzrError('%s has no development focus.' %
+                raise errors.BzrError(gettext('%s has no development focus.') %
                                   lp_branch.bzr_identity)
             target = dev_focus.branch
             if target is None:
-                raise errors.BzrError('development focus %s has no branch.' % dev_focus)
+                raise errors.BzrError(gettext(
+                        'development focus %s has no branch.') % dev_focus)
         elif lp_branch.sourcepackage is not None:
             target = lp_branch.sourcepackage.getBranch(pocket="Release")
             if target is None:
-                raise errors.BzrError('source package %s has no branch.' %
+                raise errors.BzrError(gettext(
+                                      'source package %s has no branch.') %
                                       lp_branch.sourcepackage)
         else:
-            raise errors.BzrError('%s has no associated product or source package.' %
+            raise errors.BzrError(gettext(
+                        '%s has no associated product or source package.') %
                                   lp_branch.bzr_identity)
         return LaunchpadBranch(target, target.bzr_identity)
 
@@ -272,14 +279,14 @@
         try:
             if self.lp.last_scanned_id is not None:
                 if self.bzr.last_revision() == self.lp.last_scanned_id:
-                    trace.note('%s is already up-to-date.' %
+                    trace.note(gettext('%s is already up-to-date.') %
                                self.lp.bzr_identity)
                     return
                 graph = self.bzr.repository.get_graph()
                 if not graph.is_ancestor(self.lp.last_scanned_id,
                                          self.bzr.last_revision()):
                     raise errors.DivergedBranches(self.bzr, self.push_bzr)
-                trace.note('Pushing to %s' % self.lp.bzr_identity)
+                trace.note(gettext('Pushing to %s') % self.lp.bzr_identity)
             self.bzr.push(self.push_bzr)
         finally:
             self.bzr.unlock()

=== modified file 'bzrlib/plugins/launchpad/lp_api_lite.py'
--- bzrlib/plugins/launchpad/lp_api_lite.py	2011-08-21 09:24:01 +0000
+++ bzrlib/plugins/launchpad/lp_api_lite.py	2011-12-19 13:23:58 +0000
@@ -22,6 +22,8 @@
 get the WADL.
 """
 
+from __future__ import absolute_import
+
 try:
     # Use simplejson if available, much faster, and can be easily installed in
     # older versions of python

=== modified file 'bzrlib/plugins/launchpad/lp_directory.py'
--- bzrlib/plugins/launchpad/lp_directory.py	2011-03-24 11:38:25 +0000
+++ bzrlib/plugins/launchpad/lp_directory.py	2011-12-19 13:23:58 +0000
@@ -14,9 +14,10 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-
 """Directory lookup that uses Launchpad."""
 
+from __future__ import absolute_import
+
 from urlparse import urlsplit
 import xmlrpclib
 
@@ -26,6 +27,7 @@
     trace,
     transport,
     )
+from bzrlib.i18n import gettext
 
 from bzrlib.plugins.launchpad.lp_registration import (
     LaunchpadService, ResolveLaunchpadPathRequest)
@@ -130,7 +132,7 @@
             else:
                 # There are either 0 or > 2 path parts, neither of which is
                 # supported for these schemes.
-                raise errors.InvalidURL('Bad path: %s' % result.path)
+                raise errors.InvalidURL('Bad path: %s' % url)
             # Expand any series shortcuts, but keep unknown series.
             series = distro_series.get(series, series)
             # Hack the url and let the following do the final resolution.
@@ -164,8 +166,9 @@
             if 'launchpad' in debug.debug_flags:
                 local_res = result
                 result = self._resolve_via_xmlrpc(path, url, _request_factory)
-                trace.note('resolution for %s\n  local: %s\n remote: %s'
-                           % (url, local_res['urls'], result['urls']))
+                trace.note(gettext(
+                    'resolution for {0}\n  local: {1}\n remote: {2}').format(
+                           url, local_res['urls'], result['urls']))
         else:
             result = self._resolve_via_xmlrpc(path, url, _request_factory)
 

=== modified file 'bzrlib/plugins/launchpad/lp_propose.py'
--- bzrlib/plugins/launchpad/lp_propose.py	2011-04-07 10:36:24 +0000
+++ bzrlib/plugins/launchpad/lp_propose.py	2011-12-18 15:28:38 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 from bzrlib import (
     errors,
     hooks,
@@ -25,6 +27,7 @@
 from bzrlib import (
     msgeditor,
     )
+from bzrlib.i18n import gettext
 from bzrlib.plugins.launchpad import (
     lp_api,
     lp_registration,
@@ -142,8 +145,8 @@
             if mp.queue_status in ('Merged', 'Rejected'):
                 continue
             if mp.target_branch.self_link == self.target_branch.lp.self_link:
-                raise errors.BzrCommandError(
-                    'There is already a branch merge proposal: %s' %
+                raise errors.BzrCommandError(gettext(
+                    'There is already a branch merge proposal: %s') %
                     lp_api.canonical_url(mp))
 
     def _get_prerequisite_branch(self):

=== modified file 'bzrlib/plugins/launchpad/lp_registration.py'
--- bzrlib/plugins/launchpad/lp_registration.py	2011-03-24 11:41:42 +0000
+++ bzrlib/plugins/launchpad/lp_registration.py	2011-12-19 10:58:39 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 
 import os
 import socket
@@ -166,8 +168,8 @@
             # the url?  perhaps a bit more secure against accidentally
             # revealing it.  std66 s3.2.1 discourages putting the
             # password in the url.
-            hostinfo = '%s:%s@%s' % (urllib.quote(self.registrant_email),
-                                     urllib.quote(self.registrant_password),
+            hostinfo = '%s:%s@%s' % (urlutils.quote(self.registrant_email),
+                                     urlutils.quote(self.registrant_password),
                                      hostinfo)
             url = urlunsplit((scheme, hostinfo, path, '', ''))
         else:

=== modified file 'bzrlib/plugins/launchpad/test_lp_directory.py'
--- bzrlib/plugins/launchpad/test_lp_directory.py	2011-07-11 07:02:20 +0000
+++ bzrlib/plugins/launchpad/test_lp_directory.py	2011-10-06 06:45:46 +0000
@@ -121,6 +121,17 @@
         self.assertResolve('bzr+ssh://bazaar.launchpad.net/+branch/ubuntu',
                            'lp:ubuntu')
 
+    def test_ubuntu_invalid(self):
+        """Invalid ubuntu urls don't crash.
+
+        :seealso: http://pad.lv/843900
+        """
+        # This ought to be natty-updates.
+        self.assertRaises(errors.InvalidURL,
+            self.assertResolve,
+            '',
+            'ubuntu:natty/updates/smartpm')
+
     def test_ubuntu_apt(self):
         self.assertResolve('bzr+ssh://bazaar.launchpad.net/+branch/ubuntu/apt',
                            'lp:ubuntu/apt')

=== modified file 'bzrlib/plugins/netrc_credential_store/__init__.py'
--- bzrlib/plugins/netrc_credential_store/__init__.py	2011-04-07 10:36:24 +0000
+++ bzrlib/plugins/netrc_credential_store/__init__.py	2011-12-18 12:46:49 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 __doc__ = """Use ~/.netrc as a credential store for authentication.conf."""
 
 # Since we are a built-in plugin we share the bzrlib version

=== modified file 'bzrlib/plugins/news_merge/__init__.py'
--- bzrlib/plugins/news_merge/__init__.py	2011-03-30 14:23:59 +0000
+++ bzrlib/plugins/news_merge/__init__.py	2011-12-18 12:46:49 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 __doc__ = """Merge hook for bzr's NEWS file.
 
 To enable this plugin, add a section to your branch.conf or location.conf

=== modified file 'bzrlib/plugins/news_merge/news_merge.py'
--- bzrlib/plugins/news_merge/news_merge.py	2010-04-20 01:53:31 +0000
+++ bzrlib/plugins/news_merge/news_merge.py	2011-12-20 22:13:51 +0000
@@ -16,6 +16,8 @@
 
 """Merge logic for news_merge plugin."""
 
+from __future__ import absolute_import
+
 
 from bzrlib.plugins.news_merge.parser import simple_parse_lines
 from bzrlib import merge, merge3
@@ -28,7 +30,7 @@
 
     def merge_text(self, params):
         """Perform a simple 3-way merge of a bzr NEWS file.
-        
+
         Each section of a bzr NEWS file is essentially an ordered set of bullet
         points, so we can simply take a set of bullet points, determine which
         bullets to add and which to remove, sort, and reserialize.

=== modified file 'bzrlib/plugins/news_merge/parser.py'
--- bzrlib/plugins/news_merge/parser.py	2010-04-19 05:41:54 +0000
+++ bzrlib/plugins/news_merge/parser.py	2011-12-19 13:23:58 +0000
@@ -24,6 +24,8 @@
 simple_parse's docstring).
 """
 
+from __future__ import absolute_import
+
 
 def simple_parse_lines(lines):
     """Same as simple_parse, but takes an iterable of strs rather than a single

=== added directory 'bzrlib/plugins/po_merge'
=== added file 'bzrlib/plugins/po_merge/README'
--- bzrlib/plugins/po_merge/README	1970-01-01 00:00:00 +0000
+++ bzrlib/plugins/po_merge/README	2011-11-29 10:13:56 +0000
@@ -0,0 +1,7 @@
+A plugin for merging .po files.
+
+This plugin is controlled via configuration variables, see 'bzr help po_merge'.
+
+This hook can avoid conflicts in ``.po` files by invoking msgmerge with the
+appropriate options. If it can't apply, it falls back to the default bzr
+merge algorithm.

=== added file 'bzrlib/plugins/po_merge/__init__.py'
--- bzrlib/plugins/po_merge/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/plugins/po_merge/__init__.py	2011-12-20 22:09:13 +0000
@@ -0,0 +1,92 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+from __future__ import absolute_import
+
+__doc__ = """Merge hook for ``.po`` files.
+
+To enable this plugin, add a section to your branch.conf or location.conf
+like::
+
+    [/home/user/code/bzr]
+    po_merge.pot_dirs = po,doc/po4a/po
+
+The ``po_merge.pot_dirs`` config option takes a list of directories that can
+contain ``.po`` files, separated by commas (if several directories are
+needed). Each directory should contain a single ``.pot`` file.
+
+The ``po_merge.command`` is the command whose output is used as the result of
+the merge. It defaults to::
+
+   msgmerge -N "{other}" "{pot_file}" -C "{this}" -o "{result}"
+
+where:
+
+* ``this`` is the ``.po`` file content before the merge in the current branch,
+* ``other`` is the ``.po`` file content in the branch merged from,
+* ``pot_file`` is the path to the ``.pot`` file corresponding to the ``.po``
+  file being merged.
+
+If conflicts occur in a ``.pot`` file during a given merge, the ``.po`` files
+will use the ``.pot`` file present in tree before the merge. If this doesn't
+suit your needs, you should can disable the plugin during the merge with::
+
+  bzr merge <usual merge args> -Opo_merge.po_dirs=
+
+This will allow you to resolve the conflicts in the ``.pot`` file and then
+merge the ``.po`` files again with::
+
+  bzr remerge po/*.po doc/po4a/po/*.po
+
+"""
+
+from bzrlib import (
+    config,
+    # Since we are a built-in plugin we share the bzrlib version
+    version_info,
+    )
+from bzrlib.hooks import install_lazy_named_hook
+
+
+def register_lazy_option(key, member):
+    config.option_registry.register_lazy(
+        key, 'bzrlib.plugins.po_merge.po_merge', member)
+
+
+register_lazy_option('po_merge.command', 'command_option')
+register_lazy_option('po_merge.po_dirs', 'po_dirs_option')
+register_lazy_option('po_merge.po_glob', 'po_glob_option')
+register_lazy_option('po_merge.pot_glob', 'pot_glob_option')
+
+
+def po_merge_hook(merger):
+    """Merger.merge_file_content hook for po files."""
+    from bzrlib.plugins.po_merge.po_merge import PoMerger
+    return PoMerger(merger)
+
+
+install_lazy_named_hook("bzrlib.merge", "Merger.hooks", "merge_file_content",
+    po_merge_hook, ".po file merge")
+
+
+def load_tests(basic_tests, module, loader):
+    testmod_names = [
+        'tests',
+        ]
+    basic_tests.addTest(loader.loadTestsFromModuleNames(
+            ["%s.%s" % (__name__, tmn) for tmn in testmod_names]))
+    return basic_tests
+

=== added file 'bzrlib/plugins/po_merge/po_merge.py'
--- bzrlib/plugins/po_merge/po_merge.py	1970-01-01 00:00:00 +0000
+++ bzrlib/plugins/po_merge/po_merge.py	2011-12-21 10:42:34 +0000
@@ -0,0 +1,174 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Merge logic for po_merge plugin."""
+
+from __future__ import absolute_import
+
+from bzrlib import (
+    config,
+    merge,
+    )
+
+
+from bzrlib.lazy_import import lazy_import
+lazy_import(globals(), """
+import fnmatch
+import subprocess
+import tempfile
+import sys
+
+from bzrlib import (
+    cmdline,
+    osutils,
+    trace,
+    )
+""")
+
+
+command_option = config.Option(
+        'po_merge.command',
+        default='msgmerge -N "{other}" "{pot_file}" -C "{this}" -o "{result}"',
+        help='''\
+Command used to create a conflict-free .po file during merge.
+
+The following parameters are provided by the hook:
+``this`` is the ``.po`` file content before the merge in the current branch,
+``other`` is the ``.po`` file content in the branch merged from,
+``pot_file`` is the path to the ``.pot`` file corresponding to the ``.po``
+file being merged.
+``result`` is the path where ``msgmerge`` will output its result. The hook will
+use the content of this file to produce the resulting ``.po`` file.
+
+All paths are absolute.
+''')
+
+
+po_dirs_option = config.ListOption(
+        'po_merge.po_dirs', default='po,debian/po',
+        help='List of dirs containing .po files that the hook applies to.')
+
+
+po_glob_option = config.Option(
+        'po_merge.po_glob', default='*.po',
+        help='Glob matching all ``.po`` files in one of ``po_merge.po_dirs``.')
+
+pot_glob_option = config.Option(
+        'po_merge.pot_glob', default='*.pot',
+        help='Glob matching the ``.pot`` file in one of ``po_merge.po_dirs``.')
+
+
+class PoMerger(merge.PerFileMerger):
+    """Merge .po files."""
+
+    def __init__(self, merger):
+        super(merge.PerFileMerger, self).__init__(merger)
+        # config options are cached locally until config files are (see
+        # http://pad.lv/832042)
+
+        # FIXME: We use the branch config as there is no tree config
+        # -- vila 2011-11-23
+        self.conf = merger.this_branch.get_config_stack()
+        # Which dirs are targeted by the hook 
+        self.po_dirs = self.conf.get('po_merge.po_dirs')
+        # Which files are targeted by the hook 
+        self.po_glob = self.conf.get('po_merge.po_glob')
+        # Which .pot file should be used
+        self.pot_glob = self.conf.get('po_merge.pot_glob')
+        self.command = self.conf.get('po_merge.command', expand=False)
+        # file_matches() will set the following for merge_text()
+        self.pot_file_abspath = None
+        trace.mutter('PoMerger created')
+
+    def file_matches(self, params):
+        """Return True if merge_matching should be called on this file."""
+        if not self.po_dirs or not self.command:
+            # Return early if there is no options defined
+            return False
+        po_dir = None
+        po_path = self.get_filepath(params, self.merger.this_tree)
+        for po_dir in self.po_dirs:
+            glob = osutils.pathjoin(po_dir, self.po_glob)
+            if fnmatch.fnmatch(po_path, glob):
+                trace.mutter('po %s matches: %s' % (po_path, glob))
+                break
+        else:
+            trace.mutter('PoMerger did not match for %s and %s'
+                         % (self.po_dirs, self.po_glob))
+            return False
+        # Do we have the corresponding .pot file
+        for inv_entry in self.merger.this_tree.list_files(from_dir=po_dir,
+                                                          recursive=False):
+            trace.mutter('inv_entry: %r' % (inv_entry,))
+            pot_name, pot_file_id = inv_entry[0], inv_entry[3]
+            if fnmatch.fnmatch(pot_name, self.pot_glob):
+                relpath = osutils.pathjoin(po_dir, pot_name)
+                self.pot_file_abspath = self.merger.this_tree.abspath(relpath)
+                # FIXME: I can't find an easy way to know if the .pot file has
+                # conflicts *during* the merge itself. So either the actual
+                # content on disk is fine and msgmerge will work OR it's not
+                # and it will fail. Conversely, either the result is ok for the
+                # user and he's happy OR the user needs to resolve the
+                # conflicts in the .pot file and use remerge.
+                # -- vila 2011-11-24
+                trace.mutter('will msgmerge %s using %s'
+                             % (po_path, self.pot_file_abspath))
+                return True
+        else:
+            return False
+
+    def _invoke(self, command):
+        trace.mutter('Will msgmerge: %s' % (command,))
+        # We use only absolute paths so we don't care about the cwd
+        proc = subprocess.Popen(cmdline.split(command),
+                                stdout=subprocess.PIPE,
+                                stderr=subprocess.PIPE,
+                                stdin=subprocess.PIPE)
+        out, err = proc.communicate()
+        return proc.returncode, out, err
+
+    def merge_matching(self, params):
+        return self.merge_text(params)
+
+    def merge_text(self, params):
+        """Calls msgmerge when .po files conflict.
+
+        This requires a valid .pot file to reconcile both sides.
+        """
+        # Create tmp files with the 'this' and 'other' content
+        tmpdir = tempfile.mkdtemp(prefix='po_merge')
+        env = {}
+        env['this'] = osutils.pathjoin(tmpdir, 'this')
+        env['other'] = osutils.pathjoin(tmpdir, 'other')
+        env['result'] = osutils.pathjoin(tmpdir, 'result')
+        env['pot_file'] = self.pot_file_abspath
+        try:
+            with osutils.open_file(env['this'], 'wb') as f:
+                f.writelines(params.this_lines)
+            with osutils.open_file(env['other'], 'wb') as f:
+                f.writelines(params.other_lines)
+            command = self.conf.expand_options(self.command, env)
+            retcode, out, err = self._invoke(command)
+            with osutils.open_file(env['result']) as f:
+                # FIXME: To avoid the list() construct below which means the
+                # whole 'result' file is kept in memory, there may be a way to
+                # use an iterator that will close the file when it's done, but
+                # there is still the issue of removing the tmp dir...
+                # -- vila 2011-11-24
+                return 'success', list(f.readlines())
+        finally:
+            osutils.rmtree(tmpdir)
+        return 'not applicable', []

=== added directory 'bzrlib/plugins/po_merge/tests'
=== added file 'bzrlib/plugins/po_merge/tests/__init__.py'
--- bzrlib/plugins/po_merge/tests/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/plugins/po_merge/tests/__init__.py	2011-11-28 16:31:24 +0000
@@ -0,0 +1,23 @@
+# Copyright (C) 2011 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+def load_tests(basic_tests, module, loader):
+    testmod_names = [
+        'test_po_merge',
+        ]
+    basic_tests.addTest(loader.loadTestsFromModuleNames(
+            ["%s.%s" % (__name__, tmn) for tmn in testmod_names]))
+    return basic_tests

=== added file 'bzrlib/plugins/po_merge/tests/test_po_merge.py'
--- bzrlib/plugins/po_merge/tests/test_po_merge.py	1970-01-01 00:00:00 +0000
+++ bzrlib/plugins/po_merge/tests/test_po_merge.py	2011-12-08 08:32:36 +0000
@@ -0,0 +1,451 @@
+# -*- coding: utf-8
+# Copyright (C) 2011 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+import os
+
+from bzrlib import (
+    merge,
+    tests,
+    )
+from bzrlib.tests import (
+    features,
+    script,
+    )
+
+from bzrlib.plugins import po_merge
+
+class BlackboxTestPoMerger(script.TestCaseWithTransportAndScript):
+
+    _test_needs_features = [features.msgmerge_feature]
+
+    def setUp(self):
+        super(BlackboxTestPoMerger, self).setUp()
+        self.builder = make_adduser_branch(self, 'adduser')
+        # We need to install our hook as the test framework cleared it as part
+        # of the initialization
+        merge.Merger.hooks.install_named_hook(
+            "merge_file_content", po_merge.po_merge_hook, ".po file merge")
+
+    def test_merge_with_hook_gives_unexpected_results(self):
+        # Since the conflicts in .pot are not seen *during* the merge, the .po
+        # merge triggers the hook and creates no conflicts for fr.po. But the
+        # .pot used is the one present in the tree *before* the merge.
+        self.run_script("""\
+$ bzr branch adduser -rrevid:this work
+2>Branched 2 revisions.
+$ cd work
+$ bzr merge ../adduser -rrevid:other
+2> M  po/adduser.pot
+2> M  po/fr.po
+2>Text conflict in po/adduser.pot
+2>1 conflicts encountered.
+""")
+
+    def test_called_on_remerge(self):
+        # Merge with no config for the hook to create the conflicts
+        self.run_script("""\
+$ bzr branch adduser -rrevid:this work
+2>Branched 2 revisions.
+$ cd work
+# set po_dirs to an empty list
+$ bzr merge ../adduser -rrevid:other -Opo_merge.po_dirs=
+2> M  po/adduser.pot
+2> M  po/fr.po
+2>Text conflict in po/adduser.pot
+2>Text conflict in po/fr.po
+2>2 conflicts encountered.
+""")
+        # Fix the conflicts in the .pot file
+        with open('po/adduser.pot', 'w') as f:
+            f.write(_Adduser['resolved_pot'])
+        # Tell bzr the conflict is resolved
+        self.run_script("""\
+$ bzr resolve po/adduser.pot
+2>1 conflict resolved, 1 remaining
+# Use remerge to trigger the hook, we use the default config options here
+$ bzr remerge po/*.po
+2>All changes applied successfully.
+# There should be no conflicts anymore
+$ bzr conflicts
+""")
+
+
+def make_adduser_branch(test, relpath):
+    """Helper for po_merge blackbox tests.
+
+    This creates a branch containing the needed base revisions so tests can
+    attempt merges and conflict resolutions.
+    """
+    builder = test.make_branch_builder(relpath)
+    builder.start_series()
+    builder.build_snapshot('base', None,
+                           [('add', ('', 'root-id', 'directory', '')),
+                            # Create empty files
+                            ('add', ('po', 'dir-id', 'directory', None),),
+                            ('add', ('po/adduser.pot', 'pot-id', 'file',
+                                     _Adduser['base_pot'])),
+                            ('add', ('po/fr.po', 'po-id', 'file',
+                                     _Adduser['base_po'])),
+            ])
+    # The 'other' branch
+    builder.build_snapshot('other', ['base'],
+                           [('modify', ('pot-id',
+                                        _Adduser['other_pot'])),
+                            ('modify', ('po-id',
+                                        _Adduser['other_po'])),
+                            ])
+    # The 'this' branch
+    builder.build_snapshot('this', ['base'],
+                           [('modify', ('pot-id', _Adduser['this_pot'])),
+                            ('modify', ('po-id', _Adduser['this_po'])),
+                            ])
+    # builder.get_branch() tip is now 'this'
+    builder.finish_series()
+    return builder
+
+
+class TestAdduserBranch(script.TestCaseWithTransportAndScript):
+    """Sanity checks on the adduser branch content."""
+
+    def setUp(self):
+        super(TestAdduserBranch, self).setUp()
+        self.builder = make_adduser_branch(self, 'adduser')
+
+    def assertAdduserBranchContent(self, revid):
+        env = dict(revid=revid, branch_name=revid)
+        self.run_script("""\
+$ bzr branch adduser -rrevid:%(revid)s %(branch_name)s
+""" % env, null_output_matches_anything=True)
+        self.assertFileEqual(_Adduser['%(revid)s_pot' % env],
+                             '%(branch_name)s/po/adduser.pot' % env)
+        self.assertFileEqual(_Adduser['%(revid)s_po' % env],
+                             '%(branch_name)s/po/fr.po' % env )
+
+    def test_base(self):
+        self.assertAdduserBranchContent('base')
+
+    def test_this(self):
+        self.assertAdduserBranchContent('this')
+
+    def test_other(self):
+        self.assertAdduserBranchContent('other')
+
+
+# Real content from the adduser package so we don't have to guess about format
+# details. This is declared at the end of the file to avoid cluttering the
+# beginning of the file.
+
+_Adduser = dict(
+    base_pot = r"""# SOME DESCRIPTIVE TITLE.
+# Copyright (C) YEAR THE PACKAGE'S COPYRIGHT HOLDER
+# This file is distributed under the same license as the PACKAGE package.
+# FIRST AUTHOR <EMAIL@ADDRESS>, YEAR.
+#
+#, fuzzy
+msgid ""
+msgstr ""
+"Project-Id-Version: PACKAGE VERSION\n"
+"Report-Msgid-Bugs-To: adduser-devel@example.com\n"
+"POT-Creation-Date: 2007-01-17 21:50+0100\n"
+"PO-Revision-Date: YEAR-MO-DA HO:MI+ZONE\n"
+"Last-Translator: FULL NAME <EMAIL@ADDRESS>\n"
+"Language-Team: LANGUAGE <LL@example.com>\n"
+"MIME-Version: 1.0\n"
+"Content-Type: text/plain; charset=CHARSET\n"
+"Content-Transfer-Encoding: 8bit\n"
+
+#. everyone can issue "--help" and "--version", but only root can go on
+#: ../adduser:135
+msgid "Only root may add a user or group to the system.\n"
+msgstr ""
+
+#: ../adduser:188
+msgid "Warning: The home dir you specified already exists.\n"
+msgstr ""
+
+""",
+    this_pot = r"""# SOME DESCRIPTIVE TITLE.
+# Copyright (C) YEAR THE PACKAGE'S COPYRIGHT HOLDER
+# This file is distributed under the same license as the PACKAGE package.
+# FIRST AUTHOR <EMAIL@ADDRESS>, YEAR.
+#
+#, fuzzy
+msgid ""
+msgstr ""
+"Project-Id-Version: PACKAGE VERSION\n"
+"Report-Msgid-Bugs-To: adduser-devel@example.com\n"
+"POT-Creation-Date: 2011-01-06 21:06+0000\n"
+"PO-Revision-Date: YEAR-MO-DA HO:MI+ZONE\n"
+"Last-Translator: FULL NAME <EMAIL@ADDRESS>\n"
+"Language-Team: LANGUAGE <LL@example.com>\n"
+"Language: \n"
+"MIME-Version: 1.0\n"
+"Content-Type: text/plain; charset=CHARSET\n"
+"Content-Transfer-Encoding: 8bit\n"
+
+#. everyone can issue "--help" and "--version", but only root can go on
+#: ../adduser:152
+msgid "Only root may add a user or group to the system.\n"
+msgstr ""
+
+#: ../adduser:208
+#, perl-format
+msgid "Warning: The home dir %s you specified already exists.\n"
+msgstr ""
+
+#: ../adduser:210
+#, perl-format
+msgid "Warning: The home dir %s you specified can't be accessed: %s\n"
+msgstr ""
+
+""",
+    other_pot = r"""# SOME DESCRIPTIVE TITLE.
+# Copyright (C) YEAR THE PACKAGE'S COPYRIGHT HOLDER
+# This file is distributed under the same license as the PACKAGE package.
+# FIRST AUTHOR <EMAIL@ADDRESS>, YEAR.
+#
+#, fuzzy
+msgid ""
+msgstr ""
+"Project-Id-Version: PACKAGE VERSION\n"
+"Report-Msgid-Bugs-To: adduser-devel@example.com\n"
+"POT-Creation-Date: 2010-11-21 17:13-0400\n"
+"PO-Revision-Date: YEAR-MO-DA HO:MI+ZONE\n"
+"Last-Translator: FULL NAME <EMAIL@ADDRESS>\n"
+"Language-Team: LANGUAGE <LL@example.com>\n"
+"Language: \n"
+"MIME-Version: 1.0\n"
+"Content-Type: text/plain; charset=CHARSET\n"
+"Content-Transfer-Encoding: 8bit\n"
+
+#. everyone can issue "--help" and "--version", but only root can go on
+#: ../adduser:150
+msgid "Only root may add a user or group to the system.\n"
+msgstr ""
+
+#: ../adduser:206
+#, perl-format
+msgid "Warning: The home dir %s you specified already exists.\n"
+msgstr ""
+
+#: ../adduser:208
+#, perl-format
+msgid "Warning: The home dir %s you specified can't be accessed: %s\n"
+msgstr ""
+
+""",
+    resolved_pot = r"""# SOME DESCRIPTIVE TITLE.
+# Copyright (C) YEAR THE PACKAGE'S COPYRIGHT HOLDER
+# This file is distributed under the same license as the PACKAGE package.
+# FIRST AUTHOR <EMAIL@ADDRESS>, YEAR.
+#
+#, fuzzy
+msgid ""
+msgstr ""
+"Project-Id-Version: PACKAGE VERSION\n"
+"Report-Msgid-Bugs-To: adduser-devel@example.com\n"
+"POT-Creation-Date: 2011-10-19 12:50-0700\n"
+"PO-Revision-Date: YEAR-MO-DA HO:MI+ZONE\n"
+"Last-Translator: FULL NAME <EMAIL@ADDRESS>\n"
+"Language-Team: LANGUAGE <LL@example.com>\n"
+"Language: \n"
+"MIME-Version: 1.0\n"
+"Content-Type: text/plain; charset=CHARSET\n"
+"Content-Transfer-Encoding: 8bit\n"
+
+#. everyone can issue "--help" and "--version", but only root can go on
+#: ../adduser:152
+msgid "Only root may add a user or group to the system.\n"
+msgstr ""
+
+#: ../adduser:208
+#, perl-format
+msgid "Warning: The home dir %s you specified already exists.\n"
+msgstr ""
+
+#: ../adduser:210
+#, perl-format
+msgid "Warning: The home dir %s you specified can't be accessed: %s\n"
+msgstr ""
+
+""",
+    base_po = r"""# adduser's manpages translation to French
+# Copyright (C) 2004 Software in the Public Interest
+# This file is distributed under the same license as the adduser package
+#
+# Translators:
+# Jean-Baka Domelevo Entfellner <domelevo@example.com>, 2009.
+#
+msgid ""
+msgstr ""
+"Project-Id-Version: adduser 3.111\n"
+"Report-Msgid-Bugs-To: adduser-devel@example.com\n"
+"POT-Creation-Date: 2007-01-17 21:50+0100\n"
+"PO-Revision-Date: 2010-01-21 10:36+0100\n"
+"Last-Translator: Jean-Baka Domelevo Entfellner <domelevo@example.com>\n"
+"Language-Team: Debian French Team <debian-l10n-french@example.com>\n"
+"Language: \n"
+"MIME-Version: 1.0\n"
+"Content-Type: text/plain; charset=UTF-8\n"
+"Content-Transfer-Encoding: 8bit\n"
+"X-Poedit-Language: French\n"
+"X-Poedit-Country: FRANCE\n"
+
+# type: Plain text
+#. everyone can issue "--help" and "--version", but only root can go on
+#: ../adduser:135
+msgid "Only root may add a user or group to the system.\n"
+msgstr ""
+"Seul le superutilisateur est autorisé à ajouter un utilisateur ou un groupe "
+"au système.\n"
+
+#: ../adduser:188
+msgid "Warning: The home dir you specified already exists.\n"
+msgstr ""
+"Attention ! Le répertoire personnel que vous avez indiqué existe déjà.\n"
+
+""",
+    this_po = r"""# adduser's manpages translation to French
+# Copyright (C) 2004 Software in the Public Interest
+# This file is distributed under the same license as the adduser package
+#
+# Translators:
+# Jean-Baka Domelevo Entfellner <domelevo@example.com>, 2009.
+#
+msgid ""
+msgstr ""
+"Project-Id-Version: adduser 3.111\n"
+"Report-Msgid-Bugs-To: adduser-devel@example.com\n"
+"POT-Creation-Date: 2010-10-12 15:48+0200\n"
+"PO-Revision-Date: 2010-01-21 10:36+0100\n"
+"Last-Translator: Jean-Baka Domelevo Entfellner <domelevo@example.com>\n"
+"Language-Team: Debian French Team <debian-l10n-french@example.com>\n"
+"Language: \n"
+"MIME-Version: 1.0\n"
+"Content-Type: text/plain; charset=UTF-8\n"
+"Content-Transfer-Encoding: 8bit\n"
+"X-Poedit-Language: French\n"
+"X-Poedit-Country: FRANCE\n"
+
+# type: Plain text
+#. everyone can issue "--help" and "--version", but only root can go on
+#: ../adduser:152
+msgid "Only root may add a user or group to the system.\n"
+msgstr ""
+"Seul le superutilisateur est autorisé à ajouter un utilisateur ou un groupe "
+"au système.\n"
+
+#: ../adduser:208
+#, fuzzy, perl-format
+msgid "Warning: The home dir %s you specified already exists.\n"
+msgstr ""
+"Attention ! Le répertoire personnel que vous avez indiqué existe déjà.\n"
+
+#: ../adduser:210
+#, fuzzy, perl-format
+msgid "Warning: The home dir %s you specified can't be accessed: %s\n"
+msgstr ""
+"Attention ! Le répertoire personnel que vous avez indiqué existe déjà.\n"
+
+""",
+    other_po = r"""# adduser's manpages translation to French
+# Copyright (C) 2004 Software in the Public Interest
+# This file is distributed under the same license as the adduser package
+#
+# Translators:
+# Jean-Baka Domelevo Entfellner <domelevo@example.com>, 2009, 2010.
+#
+msgid ""
+msgstr ""
+"Project-Id-Version: adduser 3.112+nmu2\n"
+"Report-Msgid-Bugs-To: adduser-devel@example.com\n"
+"POT-Creation-Date: 2010-11-21 17:13-0400\n"
+"PO-Revision-Date: 2010-11-10 11:08+0100\n"
+"Last-Translator: Jean-Baka Domelevo-Entfellner <domelevo@example.com>\n"
+"Language-Team: Debian French Team <debian-l10n-french@example.com>\n"
+"Language: \n"
+"MIME-Version: 1.0\n"
+"Content-Type: text/plain; charset=UTF-8\n"
+"Content-Transfer-Encoding: 8bit\n"
+"X-Poedit-Country: FRANCE\n"
+
+# type: Plain text
+#. everyone can issue "--help" and "--version", but only root can go on
+#: ../adduser:150
+msgid "Only root may add a user or group to the system.\n"
+msgstr ""
+"Seul le superutilisateur est autorisé à ajouter un utilisateur ou un groupe "
+"au système.\n"
+
+#: ../adduser:206
+#, perl-format
+msgid "Warning: The home dir %s you specified already exists.\n"
+msgstr ""
+"Attention ! Le répertoire personnel que vous avez indiqué (%s) existe déjà.\n"
+
+#: ../adduser:208
+#, perl-format
+msgid "Warning: The home dir %s you specified can't be accessed: %s\n"
+msgstr ""
+"Attention ! Impossible d'accéder au répertoire personnel que vous avez "
+"indiqué (%s) : %s.\n"
+
+""",
+    resolved_po = r"""# adduser's manpages translation to French
+# Copyright (C) 2004 Software in the Public Interest
+# This file is distributed under the same license as the adduser package
+#
+# Translators:
+# Jean-Baka Domelevo Entfellner <domelevo@example.com>, 2009, 2010.
+#
+msgid ""
+msgstr ""
+"Project-Id-Version: adduser 3.112+nmu2\n"
+"Report-Msgid-Bugs-To: adduser-devel@example.com\n"
+"POT-Creation-Date: 2011-10-19 12:50-0700\n"
+"PO-Revision-Date: 2010-11-10 11:08+0100\n"
+"Last-Translator: Jean-Baka Domelevo-Entfellner <domelevo@example.com>\n"
+"Language-Team: Debian French Team <debian-l10n-french@example.com>\n"
+"Language: \n"
+"MIME-Version: 1.0\n"
+"Content-Type: text/plain; charset=UTF-8\n"
+"Content-Transfer-Encoding: 8bit\n"
+"X-Poedit-Country: FRANCE\n"
+
+# type: Plain text
+#. everyone can issue "--help" and "--version", but only root can go on
+#: ../adduser:152
+msgid "Only root may add a user or group to the system.\n"
+msgstr ""
+"Seul le superutilisateur est autorisé à ajouter un utilisateur ou un groupe "
+"au système.\n"
+
+#: ../adduser:208
+#, perl-format
+msgid "Warning: The home dir %s you specified already exists.\n"
+msgstr ""
+"Attention ! Le répertoire personnel que vous avez indiqué (%s) existe déjà.\n"
+
+#: ../adduser:210
+#, perl-format
+msgid "Warning: The home dir %s you specified can't be accessed: %s\n"
+msgstr ""
+"Attention ! Impossible d'accéder au répertoire personnel que vous avez "
+"indiqué (%s) : %s.\n"
+
+""",
+)

=== modified file 'bzrlib/plugins/weave_fmt/__init__.py'
--- bzrlib/plugins/weave_fmt/__init__.py	2011-05-09 09:01:38 +0000
+++ bzrlib/plugins/weave_fmt/__init__.py	2011-12-18 12:46:49 +0000
@@ -19,6 +19,8 @@
 These were formats present in pre-1.0 version of Bazaar.
 """
 
+from __future__ import absolute_import
+
 # Since we are a built-in plugin we share the bzrlib version
 from bzrlib import version_info
 

=== modified file 'bzrlib/plugins/weave_fmt/branch.py'
--- bzrlib/plugins/weave_fmt/branch.py	2011-04-19 17:14:47 +0000
+++ bzrlib/plugins/weave_fmt/branch.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,8 @@
 
 """Weave-era branch implementations."""
 
+from __future__ import absolute_import
+
 from bzrlib import (
     errors,
     lockable_files,
@@ -32,13 +34,17 @@
 class BzrBranch4(FullHistoryBzrBranch):
     """Branch format 4."""
 
-    def _get_checkout_format(self):
+    def _get_checkout_format(self, lightweight=False):
         """Return the most suitable metadir for a checkout of this branch.
         """
         from bzrlib.plugins.weave_fmt.repository import RepositoryFormat7
         from bzrlib.bzrdir import BzrDirMetaFormat1
         format = BzrDirMetaFormat1()
-        format.repository_format = RepositoryFormat7()
+        if lightweight:
+            format.set_branch_format(self._format)
+            format.repository_format = self.bzrdir._format.repository_format
+        else:
+            format.repository_format = RepositoryFormat7()
         return format
 
     def unbind(self):
@@ -70,13 +76,16 @@
     It does not support binding.
     """
 
-    def initialize(self, a_bzrdir, name=None, repository=None):
+    def initialize(self, a_bzrdir, name=None, repository=None,
+                   append_revisions_only=None):
         """Create a branch of this format in a_bzrdir.
 
         :param a_bzrdir: The bzrdir to initialize the branch in
         :param name: Name of colocated branch to create, if any
         :param repository: Repository for this branch (unused)
         """
+        if append_revisions_only:
+            raise errors.UpgradeRequired(a_bzrdir.user_url)
         if repository is not None:
             raise NotImplementedError(
                 "initialize(repository=<not None>) on %r" % (self,))
@@ -127,7 +136,7 @@
         return "Branch format 4"
 
     def open(self, a_bzrdir, name=None, _found=False, ignore_fallbacks=False,
-            found_repository=None):
+            found_repository=None, possible_transports=None):
         """See BranchFormat.open()."""
         if name is not None:
             raise errors.NoColocatedBranchSupport(self)
@@ -140,7 +149,8 @@
                          _control_files=a_bzrdir._control_files,
                          a_bzrdir=a_bzrdir,
                          name=name,
-                         _repository=found_repository)
+                         _repository=found_repository,
+                         possible_transports=possible_transports)
 
     def __str__(self):
         return "Bazaar-NG branch format 4"

=== modified file 'bzrlib/plugins/weave_fmt/bzrdir.py'
--- bzrlib/plugins/weave_fmt/bzrdir.py	2011-05-04 22:23:20 +0000
+++ bzrlib/plugins/weave_fmt/bzrdir.py	2012-01-03 13:47:01 +0000
@@ -16,6 +16,8 @@
 
 """Weave-era BzrDir formats."""
 
+from __future__ import absolute_import
+
 from bzrlib.bzrdir import (
     BzrDir,
     BzrDirFormat,
@@ -44,6 +46,7 @@
     weave,
     xml5,
     )
+from bzrlib.i18n import gettext
 from bzrlib.store.versioned import VersionedFileStore
 from bzrlib.transactions import WriteTransaction
 from bzrlib.transport import (
@@ -80,6 +83,12 @@
             stack_on_pwd=stack_on_pwd, repo_format_name=repo_format_name,
             make_working_trees=make_working_trees, shared_repo=shared_repo)
 
+    @classmethod
+    def from_string(cls, format_string):
+        if format_string != cls.get_format_string():
+            raise AssertionError("unexpected format string %r" % format_string)
+        return cls()
+
 
 class BzrDirFormat5(BzrDirFormatAllInOne):
     """Bzr control format 5.
@@ -97,7 +106,8 @@
     def __eq__(self, other):
         return type(self) == type(other)
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See BzrDirFormat.get_format_string()."""
         return "Bazaar-NG branch, format 5\n"
 
@@ -160,7 +170,8 @@
     def __eq__(self, other):
         return type(self) == type(other)
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See BzrDirFormat.get_format_string()."""
         return "Bazaar-NG branch, format 6\n"
 
@@ -222,10 +233,10 @@
         """See Converter.convert()."""
         self.bzrdir = to_convert
         if pb is not None:
-            warnings.warn("pb parameter to convert() is deprecated")
+            warnings.warn(gettext("pb parameter to convert() is deprecated"))
         self.pb = ui.ui_factory.nested_progress_bar()
         try:
-            ui.ui_factory.note('starting upgrade from format 4 to 5')
+            ui.ui_factory.note(gettext('starting upgrade from format 4 to 5'))
             if isinstance(self.bzrdir.transport, local.LocalTransport):
                 self.bzrdir.get_workingtree_transport(None).delete('stat-cache')
             self._convert_to_weaves()
@@ -234,7 +245,8 @@
             self.pb.finished()
 
     def _convert_to_weaves(self):
-        ui.ui_factory.note('note: upgrade may be faster if all store files are ungzipped first')
+        ui.ui_factory.note(gettext(
+          'note: upgrade may be faster if all store files are ungzipped first'))
         try:
             # TODO permissions
             stat = self.bzrdir.transport.stat('weaves')
@@ -250,7 +262,7 @@
         self.bzrdir.transport.delete('branch-format')
         self.branch = self.bzrdir.open_branch()
         self._convert_working_inv()
-        rev_history = self.branch.revision_history()
+        rev_history = self.branch._revision_history()
         # to_read is a stack holding the revisions we still need to process;
         # appending to it adds new highest-priority revisions
         self.known_revisions = set(rev_history)
@@ -263,15 +275,17 @@
         self.pb.clear()
         to_import = self._make_order()
         for i, rev_id in enumerate(to_import):
-            self.pb.update('converting revision', i, len(to_import))
+            self.pb.update(gettext('converting revision'), i, len(to_import))
             self._convert_one_rev(rev_id)
         self.pb.clear()
         self._write_all_weaves()
         self._write_all_revs()
-        ui.ui_factory.note('upgraded to weaves:')
-        ui.ui_factory.note('  %6d revisions and inventories' % len(self.revisions))
-        ui.ui_factory.note('  %6d revisions not present' % len(self.absent_revisions))
-        ui.ui_factory.note('  %6d texts' % self.text_count)
+        ui.ui_factory.note(gettext('upgraded to weaves:'))
+        ui.ui_factory.note('  ' + gettext('%6d revisions and inventories') %
+                                                        len(self.revisions))
+        ui.ui_factory.note('  ' + gettext('%6d revisions not present') %
+                                                    len(self.absent_revisions))
+        ui.ui_factory.note('  ' + gettext('%6d texts') % self.text_count)
         self._cleanup_spare_files_after_format4()
         self.branch._transport.put_bytes(
             'branch-format',
@@ -307,12 +321,13 @@
         try:
             i = 0
             for file_id, file_weave in self.text_weaves.items():
-                self.pb.update('writing weave', i, len(self.text_weaves))
+                self.pb.update(gettext('writing weave'), i,
+                                                        len(self.text_weaves))
                 weaves._put_weave(file_id, file_weave, transaction)
                 i += 1
-            self.pb.update('inventory', 0, 1)
+            self.pb.update(gettext('inventory'), 0, 1)
             controlweaves._put_weave('inventory', self.inv_weave, transaction)
-            self.pb.update('inventory', 1, 1)
+            self.pb.update(gettext('inventory'), 1, 1)
         finally:
             self.pb.clear()
 
@@ -329,7 +344,8 @@
             lambda:True, lambda:True)
         try:
             for i, rev_id in enumerate(self.converted_revs):
-                self.pb.update('write revision', i, len(self.converted_revs))
+                self.pb.update(gettext('write revision'), i,
+                                                len(self.converted_revs))
                 text = serializer_v5.write_revision_to_string(
                     self.revisions[rev_id])
                 key = (rev_id,)
@@ -342,13 +358,13 @@
 
         Any parents not either loaded or abandoned get queued to be
         loaded."""
-        self.pb.update('loading revision',
+        self.pb.update(gettext('loading revision'),
                        len(self.revisions),
                        len(self.known_revisions))
         if not self.branch.repository.has_revision(rev_id):
             self.pb.clear()
-            ui.ui_factory.note('revision {%s} not present in branch; '
-                         'will be converted as a ghost' %
+            ui.ui_factory.note(gettext('revision {%s} not present in branch; '
+                         'will be converted as a ghost') %
                          rev_id)
             self.absent_revisions.add(rev_id)
         else:
@@ -487,7 +503,7 @@
         self.bzrdir = to_convert
         pb = ui.ui_factory.nested_progress_bar()
         try:
-            ui.ui_factory.note('starting upgrade from format 5 to 6')
+            ui.ui_factory.note(gettext('starting upgrade from format 5 to 6'))
             self._convert_to_prefixed()
             return BzrDir.open(self.bzrdir.user_url)
         finally:
@@ -497,7 +513,7 @@
         from bzrlib.store import TransportStore
         self.bzrdir.transport.delete('branch-format')
         for store_name in ["weaves", "revision-store"]:
-            ui.ui_factory.note("adding prefixes to %s" % store_name)
+            ui.ui_factory.note(gettext("adding prefixes to %s") % store_name)
             store_transport = self.bzrdir.transport.clone(store_name)
             store = TransportStore(store_transport, prefixed=True)
             for urlfilename in store_transport.list_dir('.'):
@@ -537,7 +553,7 @@
         self.dir_mode = self.bzrdir._get_dir_mode()
         self.file_mode = self.bzrdir._get_file_mode()
 
-        ui.ui_factory.note('starting upgrade from format 6 to metadir')
+        ui.ui_factory.note(gettext('starting upgrade from format 6 to metadir'))
         self.bzrdir.transport.put_bytes(
                 'branch-format',
                 "Converting to format 6",
@@ -545,12 +561,12 @@
         # its faster to move specific files around than to open and use the apis...
         # first off, nuke ancestry.weave, it was never used.
         try:
-            self.step('Removing ancestry.weave')
+            self.step(gettext('Removing ancestry.weave'))
             self.bzrdir.transport.delete('ancestry.weave')
         except errors.NoSuchFile:
             pass
         # find out whats there
-        self.step('Finding branch files')
+        self.step(gettext('Finding branch files'))
         last_revision = self.bzrdir.open_branch().last_revision()
         bzrcontents = self.bzrdir.transport.list_dir('.')
         for name in bzrcontents:
@@ -560,7 +576,7 @@
         repository_names = [('inventory.weave', True),
                             ('revision-store', True),
                             ('weaves', True)]
-        self.step('Upgrading repository  ')
+        self.step(gettext('Upgrading repository') + '  ')
         self.bzrdir.transport.mkdir('repository', mode=self.dir_mode)
         self.make_lock('repository')
         # we hard code the formats here because we are converting into
@@ -570,7 +586,7 @@
         for entry in repository_names:
             self.move_entry('repository', entry)
 
-        self.step('Upgrading branch      ')
+        self.step(gettext('Upgrading branch') + '      ')
         self.bzrdir.transport.mkdir('branch', mode=self.dir_mode)
         self.make_lock('branch')
         self.put_format('branch', BzrBranchFormat5())
@@ -593,14 +609,14 @@
         else:
             has_checkout = True
         if not has_checkout:
-            ui.ui_factory.note('No working tree.')
+            ui.ui_factory.note(gettext('No working tree.'))
             # If some checkout files are there, we may as well get rid of them.
             for name, mandatory in checkout_files:
                 if name in bzrcontents:
                     self.bzrdir.transport.delete(name)
         else:
             from bzrlib.workingtree_3 import WorkingTreeFormat3
-            self.step('Upgrading working tree')
+            self.step(gettext('Upgrading working tree'))
             self.bzrdir.transport.mkdir('checkout', mode=self.dir_mode)
             self.make_lock('checkout')
             self.put_format(
@@ -621,7 +637,7 @@
 
     def make_lock(self, name):
         """Make a lock for the new control dir name."""
-        self.step('Make %s lock' % name)
+        self.step(gettext('Make %s lock') % name)
         ld = lockdir.LockDir(self.bzrdir.transport,
                              '%s/lock' % name,
                              file_modebits=self.file_mode,
@@ -632,7 +648,7 @@
         """Move then entry name into new_dir."""
         name = entry[0]
         mandatory = entry[1]
-        self.step('Moving %s' % name)
+        self.step(gettext('Moving %s') % name)
         try:
             self.bzrdir.transport.move(name, '%s/%s' % (new_dir, name))
         except errors.NoSuchFile:
@@ -663,7 +679,8 @@
     def __eq__(self, other):
         return type(self) == type(other)
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See BzrDirFormat.get_format_string()."""
         return "Bazaar-NG branch, format 0.0.4\n"
 
@@ -702,6 +719,12 @@
         return RepositoryFormat4()
     repository_format = property(__return_repository_format)
 
+    @classmethod
+    def from_string(cls, format_string):
+        if format_string != cls.get_format_string():
+            raise AssertionError("unexpected format string %r" % format_string)
+        return cls()
+
 
 class BzrDirPreSplitOut(BzrDir):
     """A common class for the all-in-one formats."""
@@ -747,12 +770,14 @@
             tree.clone(result)
         return result
 
-    def create_branch(self, name=None, repository=None):
+    def create_branch(self, name=None, repository=None,
+                      append_revisions_only=None):
         """See BzrDir.create_branch."""
         if repository is not None:
             raise NotImplementedError(
                 "create_branch(repository=<not None>) on %r" % (self,))
-        return self._format.get_branch_format().initialize(self, name=name)
+        return self._format.get_branch_format().initialize(self, name=name,
+            append_revisions_only=append_revisions_only)
 
     def destroy_branch(self, name=None):
         """See BzrDir.destroy_branch."""
@@ -859,12 +884,13 @@
         return not isinstance(self._format, format.__class__)
 
     def open_branch(self, name=None, unsupported=False,
-                    ignore_fallbacks=False):
+                    ignore_fallbacks=False, possible_transports=None):
         """See BzrDir.open_branch."""
         from bzrlib.plugins.weave_fmt.branch import BzrBranchFormat4
         format = BzrBranchFormat4()
         format.check_support_status(unsupported)
-        return format.open(self, name, _found=True)
+        return format.open(self, name, _found=True,
+            possible_transports=possible_transports)
 
     def sprout(self, url, revision_id=None, force_new_repo=False,
                possible_transports=None, accelerator_tree=None,
@@ -940,7 +966,7 @@
         from bzrlib.plugins.weave_fmt.repository import RepositoryFormat5
         return RepositoryFormat5().open(self, _found=True)
 
-    def open_workingtree(self, _unsupported=False,
+    def open_workingtree(self, unsupported=False,
             recommend_upgrade=True):
         """See BzrDir.create_workingtree."""
         from bzrlib.plugins.weave_fmt.workingtree import WorkingTreeFormat2
@@ -965,8 +991,7 @@
         from bzrlib.plugins.weave_fmt.repository import RepositoryFormat6
         return RepositoryFormat6().open(self, _found=True)
 
-    def open_workingtree(self, _unsupported=False,
-        recommend_upgrade=True):
+    def open_workingtree(self, unsupported=False, recommend_upgrade=True):
         """See BzrDir.create_workingtree."""
         # we don't warn here about upgrades; that ought to be handled for the
         # bzrdir as a whole

=== modified file 'bzrlib/plugins/weave_fmt/repository.py'
--- bzrlib/plugins/weave_fmt/repository.py	2011-05-18 10:41:51 +0000
+++ bzrlib/plugins/weave_fmt/repository.py	2011-12-19 19:15:58 +0000
@@ -20,10 +20,11 @@
 ghosts.
 """
 
+from __future__ import absolute_import
+
 import gzip
 import os
 from cStringIO import StringIO
-import urllib
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
@@ -52,7 +53,7 @@
 from bzrlib.decorators import needs_read_lock, needs_write_lock
 from bzrlib.repository import (
     InterRepository,
-    RepositoryFormat,
+    RepositoryFormatMetaDir,
     )
 from bzrlib.store.text import TextStore
 from bzrlib.versionedfile import (
@@ -272,6 +273,7 @@
     supports_ghosts = False
     supports_external_lookups = False
     supports_chks = False
+    supports_nesting_repositories = True
     _fetch_order = 'topological'
     _fetch_reconcile = True
     fast_deltas = False
@@ -321,7 +323,6 @@
             raise NotImplementedError
 
         repo_transport = a_bzrdir.get_repository_transport(None)
-        control_files = a_bzrdir._control_files
         result = AllInOneRepository(_format=self, a_bzrdir=a_bzrdir)
         result.revisions = self._get_revisions(repo_transport, result)
         result.signatures = self._get_signatures(repo_transport, result)
@@ -504,7 +505,8 @@
     def _serializer(self):
         return xml5.serializer_v5
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See RepositoryFormat.get_format_string()."""
         return "Bazaar-NG Repository format 7"
 
@@ -561,7 +563,7 @@
                                     than normal. I.e. during 'upgrade'.
         """
         if not _found:
-            format = RepositoryFormat.find_format(a_bzrdir)
+            format = RepositoryFormatMetaDir.find_format(a_bzrdir)
         if _override_transport is not None:
             repo_transport = _override_transport
         else:
@@ -707,7 +709,7 @@
             raise errors.ObjectNotLocked(self)
         relpaths = set()
         for quoted_relpath in self._transport.iter_files_recursive():
-            relpath = urllib.unquote(quoted_relpath)
+            relpath = urlutils.unquote(quoted_relpath)
             path, ext = os.path.splitext(relpath)
             if ext == '.gz':
                 relpath = path
@@ -747,7 +749,7 @@
             raise errors.ObjectNotLocked(self)
         relpaths = set()
         for quoted_relpath in self._transport.iter_files_recursive():
-            relpath = urllib.unquote(quoted_relpath)
+            relpath = urlutils.unquote(quoted_relpath)
             path, ext = os.path.splitext(relpath)
             if ext == '.gz':
                 relpath = path

=== modified file 'bzrlib/plugins/weave_fmt/test_bzrdir.py'
--- bzrlib/plugins/weave_fmt/test_bzrdir.py	2011-04-15 22:57:31 +0000
+++ bzrlib/plugins/weave_fmt/test_bzrdir.py	2011-12-19 13:23:58 +0000
@@ -19,6 +19,8 @@
 For interface contract tests, see tests/per_bzr_dir.
 """
 
+from __future__ import absolute_import
+
 import os
 import sys
 
@@ -33,7 +35,6 @@
     )
 from bzrlib.osutils import (
     getcwd,
-    lexists,
     )
 from bzrlib.tests.test_bundle import V4BundleTester
 from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
@@ -308,7 +309,8 @@
         # other tests.
         self.assertIsInstance(control._format, bzrdir.BzrDirMetaFormat1)
         b = control.open_branch()
-        self.assertEquals(b.revision_history(),
+        self.addCleanup(b.lock_read().unlock)
+        self.assertEquals(b._revision_history(),
            ['mbp@sourcefrog.net-20051004035611-176b16534b086b3c',
             'mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd'])
 
@@ -323,7 +325,8 @@
         self.assertIsInstance(
             control._format,
             bzrdir.BzrDirFormat.get_default_format().__class__)
-        rh = b.revision_history()
+        self.addCleanup(b.lock_read().unlock)
+        rh = b._revision_history()
         eq(rh,
            ['mbp@sourcefrog.net-20051004035611-176b16534b086b3c',
             'mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd'])
@@ -378,7 +381,8 @@
         self.build_tree_contents(_ghost_template)
         upgrade.upgrade(u'.')
         b = branch.Branch.open(u'.')
-        revision_id = b.revision_history()[1]
+        self.addCleanup(b.lock_read().unlock)
+        revision_id = b._revision_history()[1]
         rev = b.repository.get_revision(revision_id)
         eq(len(rev.parent_ids), 2)
         eq(rev.parent_ids[1], 'wibble@wobble-2')
@@ -407,7 +411,8 @@
         self.build_tree_contents(_upgrade_dir_template)
         upgrade.upgrade('.', bzrdir.BzrDirMetaFormat1())
         tree = workingtree.WorkingTree.open('.')
-        self.assertEqual([tree.branch.revision_history()[-1]],
+        self.addCleanup(tree.lock_read().unlock)
+        self.assertEqual([tree.branch._revision_history()[-1]],
             tree.get_parent_ids())
 
 

=== modified file 'bzrlib/plugins/weave_fmt/test_repository.py'
--- bzrlib/plugins/weave_fmt/test_repository.py	2011-06-28 15:39:43 +0000
+++ bzrlib/plugins/weave_fmt/test_repository.py	2011-12-19 13:23:58 +0000
@@ -20,6 +20,8 @@
 
 """
 
+from __future__ import absolute_import
+
 from cStringIO import StringIO
 from stat import S_ISDIR
 import sys

=== modified file 'bzrlib/plugins/weave_fmt/test_workingtree.py'
--- bzrlib/plugins/weave_fmt/test_workingtree.py	2011-03-10 13:29:54 +0000
+++ bzrlib/plugins/weave_fmt/test_workingtree.py	2011-12-19 13:23:58 +0000
@@ -17,6 +17,8 @@
 
 """Tests for weave-era working tree formats."""
 
+from __future__ import absolute_import
+
 import os
 
 from bzrlib import (

=== modified file 'bzrlib/plugins/weave_fmt/workingtree.py'
--- bzrlib/plugins/weave_fmt/workingtree.py	2011-06-28 11:29:03 +0000
+++ bzrlib/plugins/weave_fmt/workingtree.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,8 @@
 
 """Weave-era working tree objects."""
 
+from __future__ import absolute_import
+
 from cStringIO import StringIO
 
 from bzrlib import (
@@ -30,9 +32,11 @@
 from bzrlib.decorators import needs_read_lock
 from bzrlib.transport.local import LocalTransport
 from bzrlib.workingtree import (
-    InventoryWorkingTree,
     WorkingTreeFormat,
     )
+from bzrlib.workingtree_3 import (
+    PreDirStateWorkingTree,
+    )
 
 
 def get_conflicted_stem(path):
@@ -135,7 +139,7 @@
         return wt
 
 
-class WorkingTree2(InventoryWorkingTree):
+class WorkingTree2(PreDirStateWorkingTree):
     """This is the Format 2 working tree.
 
     This was the first weave based working tree.
@@ -143,8 +147,8 @@
      - uses the branch last-revision.
     """
 
-    def __init__(self, *args, **kwargs):
-        super(WorkingTree2, self).__init__(*args, **kwargs)
+    def __init__(self, basedir, *args, **kwargs):
+        super(WorkingTree2, self).__init__(basedir, *args, **kwargs)
         # WorkingTree2 has more of a constraint that self._inventory must
         # exist. Because this is an older format, we don't mind the overhead
         # caused by the extra computation here.
@@ -158,6 +162,7 @@
         """Return the references needed to perform a check of this tree."""
         return [('trees', self.last_revision())]
 
+
     def lock_tree_write(self):
         """See WorkingTree.lock_tree_write().
 

=== modified file 'bzrlib/plugins/weave_fmt/xml4.py'
--- bzrlib/plugins/weave_fmt/xml4.py	2011-02-19 15:23:08 +0000
+++ bzrlib/plugins/weave_fmt/xml4.py	2011-12-18 15:28:38 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 from bzrlib.xml_serializer import (
     Element,
     SubElement,

=== modified file 'bzrlib/progress.py'
--- bzrlib/progress.py	2010-09-14 04:13:48 +0000
+++ bzrlib/progress.py	2011-12-19 13:23:58 +0000
@@ -14,25 +14,18 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-
 """Progress indicators.
 
 The usual way to use this is via bzrlib.ui.ui_factory.nested_progress_bar which
 will manage a conceptual stack of nested activities.
 """
 
+from __future__ import absolute_import
 
-import sys
 import time
 import os
 
 
-from bzrlib.symbol_versioning import (
-    deprecated_in,
-    deprecated_method,
-    )
-
-
 def _supports_progress(f):
     """Detect if we can use pretty progress bars on file F.
 

=== modified file 'bzrlib/push.py'
--- bzrlib/push.py	2011-05-17 15:01:58 +0000
+++ bzrlib/push.py	2011-12-19 13:23:58 +0000
@@ -16,8 +16,10 @@
 
 """UI helper for the push command."""
 
+from __future__ import absolute_import
+
 from bzrlib import (
-    bzrdir,
+    controldir,
     errors,
     revision as _mod_revision,
     transport,
@@ -26,6 +28,7 @@
     note,
     warning,
     )
+from bzrlib.i18n import gettext
 
 
 class PushResult(object):
@@ -47,10 +50,10 @@
         """Write a human-readable description of the result."""
         if self.branch_push_result is None:
             if self.stacked_on is not None:
-                note('Created new stacked branch referring to %s.' %
+                note(gettext('Created new stacked branch referring to %s.') %
                     self.stacked_on)
             else:
-                note('Created new branch.')
+                note(gettext('Created new branch.'))
         else:
             self.branch_push_result.report(to_file)
 
@@ -78,7 +81,7 @@
     """
     to_transport = transport.get_transport(location)
     try:
-        dir_to = bzrdir.BzrDir.open_from_transport(to_transport)
+        dir_to = controldir.ControlDir.open_from_transport(to_transport)
     except errors.NotBranchError:
         # Didn't find anything
         dir_to = None
@@ -91,23 +94,23 @@
                 no_tree=no_tree)
         except errors.FileExists, err:
             if err.path.endswith('/.bzr'):
-                raise errors.BzrCommandError(
+                raise errors.BzrCommandError(gettext(
                     "Target directory %s already contains a .bzr directory, "
-                    "but it is not valid." % (location,))
+                    "but it is not valid.") % (location,))
             if not use_existing_dir:
-                raise errors.BzrCommandError("Target directory %s"
+                raise errors.BzrCommandError(gettext("Target directory %s"
                      " already exists, but does not have a .bzr"
                      " directory. Supply --use-existing-dir to push"
-                     " there anyway." % location)
+                     " there anyway.") % location)
             # This shouldn't occur, but if it does the FileExists error will be
             # more informative than an UnboundLocalError for br_to.
             raise
         except errors.NoSuchFile:
             if not create_prefix:
-                raise errors.BzrCommandError("Parent directory of %s"
+                raise errors.BzrCommandError(gettext("Parent directory of %s"
                     " does not exist."
                     "\nYou may supply --create-prefix to create all"
-                    " leading parent directories."
+                    " leading parent directories.")
                     % location)
             # This shouldn't occur (because create_prefix is true, so
             # create_clone_on_transport should be catching NoSuchFile and
@@ -116,8 +119,8 @@
             # UnboundLocalError for br_to.
             raise
         except errors.TooManyRedirections:
-            raise errors.BzrCommandError("Too many redirections trying "
-                                         "to make %s." % location)
+            raise errors.BzrCommandError(gettext("Too many redirections trying "
+                                         "to make %s.") % location)
         push_result = PushResult()
         # TODO: Some more useful message about what was copied
         try:
@@ -141,20 +144,20 @@
             push_result = dir_to.push_branch(br_from, revision_id, overwrite, 
                 remember, create_prefix)
         except errors.DivergedBranches:
-            raise errors.BzrCommandError('These branches have diverged.'
+            raise errors.BzrCommandError(gettext('These branches have diverged.'
                                     '  See "bzr help diverged-branches"'
-                                    ' for more information.')
+                                    ' for more information.'))
         except errors.NoRoundtrippingSupport, e:
-            raise errors.BzrCommandError("It is not possible to losslessly "
-                "push to %s. You may want to use dpush instead." % 
+            raise errors.BzrCommandError(gettext("It is not possible to losslessly "
+                "push to %s. You may want to use dpush instead.") % 
                     e.target_branch.mapping.vcs.abbreviation)
         except errors.NoRepositoryPresent:
-            # we have a bzrdir but no branch or repository
+            # we have a controldir but no branch or repository
             # XXX: Figure out what to do other than complain.
-            raise errors.BzrCommandError("At %s you have a valid .bzr"
+            raise errors.BzrCommandError(gettext("At %s you have a valid .bzr"
                 " control directory, but not a branch or repository. This"
                 " is an unsupported configuration. Please move the target"
-                " directory out of the way and try again." % location)
+                " directory out of the way and try again.") % location)
         if push_result.workingtree_updated == False:
             warning("This transport does not update the working " 
                     "tree of: %s. See 'bzr help working-trees' for "

=== modified file 'bzrlib/pyutils.py'
--- bzrlib/pyutils.py	2010-10-15 12:35:00 +0000
+++ bzrlib/pyutils.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,7 @@
 
 """General Python convenience functions."""
 
+from __future__ import absolute_import
 
 import sys
 

=== modified file 'bzrlib/reconcile.py'
--- bzrlib/reconcile.py	2011-06-16 16:06:33 +0000
+++ bzrlib/reconcile.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,7 @@
 
 """Reconcilers are able to fix some potential data errors in a branch."""
 
+from __future__ import absolute_import
 
 __all__ = [
     'KnitReconciler',
@@ -35,6 +36,7 @@
 from bzrlib.trace import mutter
 from bzrlib.tsort import topo_sort
 from bzrlib.versionedfile import AdapterFactory, FulltextContentFactory
+from bzrlib.i18n import gettext
 
 
 def reconcile(dir, canonicalize_chks=False):
@@ -92,33 +94,33 @@
             # Nothing to check here
             self.fixed_branch_history = None
             return
-        ui.ui_factory.note('Reconciling branch %s' % self.branch.base)
+        ui.ui_factory.note(gettext('Reconciling branch %s') % self.branch.base)
         branch_reconciler = self.branch.reconcile(thorough=True)
         self.fixed_branch_history = branch_reconciler.fixed_history
 
     def _reconcile_repository(self):
         self.repo = self.bzrdir.find_repository()
-        ui.ui_factory.note('Reconciling repository %s' %
+        ui.ui_factory.note(gettext('Reconciling repository %s') %
             self.repo.user_url)
-        self.pb.update("Reconciling repository", 0, 1)
+        self.pb.update(gettext("Reconciling repository"), 0, 1)
         if self.canonicalize_chks:
             try:
                 self.repo.reconcile_canonicalize_chks
             except AttributeError:
                 raise errors.BzrError(
-                    "%s cannot canonicalize CHKs." % (self.repo,))
+                    gettext("%s cannot canonicalize CHKs.") % (self.repo,))
             repo_reconciler = self.repo.reconcile_canonicalize_chks()
         else:
             repo_reconciler = self.repo.reconcile(thorough=True)
         self.inconsistent_parents = repo_reconciler.inconsistent_parents
         self.garbage_inventories = repo_reconciler.garbage_inventories
         if repo_reconciler.aborted:
-            ui.ui_factory.note(
-                'Reconcile aborted: revision index has inconsistent parents.')
-            ui.ui_factory.note(
-                'Run "bzr check" for more details.')
+            ui.ui_factory.note(gettext(
+                'Reconcile aborted: revision index has inconsistent parents.'))
+            ui.ui_factory.note(gettext(
+                'Run "bzr check" for more details.'))
         else:
-            ui.ui_factory.note('Reconciliation complete.')
+            ui.ui_factory.note(gettext('Reconciliation complete.'))
 
 
 class BranchReconciler(object):
@@ -161,13 +163,14 @@
             # set_revision_history, as this will regenerate it again.
             # Not really worth a whole BranchReconciler class just for this,
             # though.
-            ui.ui_factory.note('Fixing last revision info %s => %s' % (
-                 last_revno, len(real_history)))
+            ui.ui_factory.note(gettext('Fixing last revision info {0} '\
+                                       ' => {1}').format(
+                                       last_revno, len(real_history)))
             self.branch.set_last_revision_info(len(real_history),
                                                last_revision_id)
         else:
             self.fixed_history = False
-            ui.ui_factory.note('revision_history ok.')
+            ui.ui_factory.note(gettext('revision_history ok.'))
 
 
 class RepoReconciler(object):
@@ -228,7 +231,7 @@
         (self.thorough) are treated as requiring the reweave.
         """
         transaction = self.repo.get_transaction()
-        self.pb.update('Reading inventory data')
+        self.pb.update(gettext('Reading inventory data'))
         self.inventory = self.repo.inventories
         self.revisions = self.repo.revisions
         # the total set of revisions to process
@@ -248,11 +251,11 @@
         # (no garbage inventories or we are not doing a thorough check)
         if (not self.inconsistent_parents and
             (not self.garbage_inventories or not self.thorough)):
-            ui.ui_factory.note('Inventory ok.')
+            ui.ui_factory.note(gettext('Inventory ok.'))
             return
-        self.pb.update('Backing up inventory', 0, 0)
+        self.pb.update(gettext('Backing up inventory'), 0, 0)
         self.repo._backup_inventory()
-        ui.ui_factory.note('Backup inventory created.')
+        ui.ui_factory.note(gettext('Backup inventory created.'))
         new_inventories = self.repo._temp_inventories()
 
         # we have topological order of revisions and non ghost parents ready.
@@ -268,10 +271,10 @@
         if not (set(new_inventories.keys()) ==
             set([(revid,) for revid in self.pending])):
             raise AssertionError()
-        self.pb.update('Writing weave')
+        self.pb.update(gettext('Writing weave'))
         self.repo._activate_new_inventory()
         self.inventory = None
-        ui.ui_factory.note('Inventory regenerated.')
+        ui.ui_factory.note(gettext('Inventory regenerated.'))
 
     def _new_inv_parents(self, revision_key):
         """Lookup ghost-filtered parents for revision_key."""
@@ -365,24 +368,24 @@
     def _load_indexes(self):
         """Load indexes for the reconciliation."""
         self.transaction = self.repo.get_transaction()
-        self.pb.update('Reading indexes', 0, 2)
+        self.pb.update(gettext('Reading indexes'), 0, 2)
         self.inventory = self.repo.inventories
-        self.pb.update('Reading indexes', 1, 2)
+        self.pb.update(gettext('Reading indexes'), 1, 2)
         self.repo._check_for_inconsistent_revision_parents()
         self.revisions = self.repo.revisions
-        self.pb.update('Reading indexes', 2, 2)
+        self.pb.update(gettext('Reading indexes'), 2, 2)
 
     def _gc_inventory(self):
         """Remove inventories that are not referenced from the revision store."""
-        self.pb.update('Checking unused inventories', 0, 1)
+        self.pb.update(gettext('Checking unused inventories'), 0, 1)
         self._check_garbage_inventories()
-        self.pb.update('Checking unused inventories', 1, 3)
+        self.pb.update(gettext('Checking unused inventories'), 1, 3)
         if not self.garbage_inventories:
-            ui.ui_factory.note('Inventory ok.')
+            ui.ui_factory.note(gettext('Inventory ok.'))
             return
-        self.pb.update('Backing up inventory', 0, 0)
+        self.pb.update(gettext('Backing up inventory'), 0, 0)
         self.repo._backup_inventory()
-        ui.ui_factory.note('Backup Inventory created')
+        ui.ui_factory.note(gettext('Backup Inventory created'))
         # asking for '' should never return a non-empty weave
         new_inventories = self.repo._temp_inventories()
         # we have topological order of revisions and non ghost parents ready.
@@ -399,10 +402,10 @@
         # the revisionds list
         if not(set(new_inventories.keys()) == set(revision_keys)):
             raise AssertionError()
-        self.pb.update('Writing weave')
+        self.pb.update(gettext('Writing weave'))
         self.repo._activate_new_inventory()
         self.inventory = None
-        ui.ui_factory.note('Inventory regenerated.')
+        ui.ui_factory.note(gettext('Inventory regenerated.'))
 
     def _fix_text_parents(self):
         """Fix bad versionedfile parent entries.
@@ -440,7 +443,7 @@
             versions_list.append(text_key[1])
         # Do the reconcile of individual weaves.
         for num, file_id in enumerate(per_id_bad_parents):
-            self.pb.update('Fixing text parents', num,
+            self.pb.update(gettext('Fixing text parents'), num,
                            len(per_id_bad_parents))
             versions_with_bad_parents = per_id_bad_parents[file_id]
             id_unused_versions = set(key[-1] for key in unused_versions

=== modified file 'bzrlib/reconfigure.py'
--- bzrlib/reconfigure.py	2010-04-23 00:44:15 +0000
+++ bzrlib/reconfigure.py	2011-12-19 13:23:58 +0000
@@ -14,22 +14,24 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-"""Reconfigure a bzrdir into a new tree/branch/repository layout.
+"""Reconfigure a controldir into a new tree/branch/repository layout.
 
 Various types of reconfiguration operation are available either by
 constructing a class or using a factory method on Reconfigure.
 """
 
+from __future__ import absolute_import
+
 
 from bzrlib import (
     branch,
-    bzrdir,
+    controldir,
     errors,
     trace,
     ui,
     urlutils,
     )
-
+from bzrlib.i18n import gettext
 
 # TODO: common base class for all reconfigure operations, making no
 # assumptions about what kind of change will be done.
@@ -48,9 +50,9 @@
         try:
             branch.set_stacked_on_url(on_url)
             if not trace.is_quiet():
-                ui.ui_factory.note(
-                    "%s is now stacked on %s\n"
-                    % (branch.base, branch.get_stacked_on_url()))
+                ui.ui_factory.note(gettext(
+                    "{0} is now stacked on {1}\n").format(
+                      branch.base, branch.get_stacked_on_url()))
         finally:
             branch.unlock()
 
@@ -63,8 +65,8 @@
         try:
             branch.set_stacked_on_url(None)
             if not trace.is_quiet():
-                ui.ui_factory.note(
-                    "%s is now not stacked\n"
+                ui.ui_factory.note(gettext(
+                    "%s is now not stacked\n")
                     % (branch.base,))
         finally:
             branch.unlock()
@@ -344,7 +346,8 @@
                 reference_branch.repository.fetch(self.repository)
             elif self.local_branch is not None and not self._destroy_branch:
                 up = self.local_branch.user_transport.clone('..')
-                up_bzrdir = bzrdir.BzrDir.open_containing_from_transport(up)[0]
+                up_bzrdir = controldir.ControlDir.open_containing_from_transport(
+                    up)[0]
                 new_repo = up_bzrdir.find_repository()
                 new_repo.fetch(self.repository)
         last_revision_info = None

=== modified file 'bzrlib/recordcounter.py'
--- bzrlib/recordcounter.py	2010-06-12 02:58:42 +0000
+++ bzrlib/recordcounter.py	2011-12-19 13:23:58 +0000
@@ -13,8 +13,11 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
 """Record counting support for showing progress of revision fetch."""
 
+from __future__ import absolute_import
+
 
 class RecordCounter(object):
     """Container for maintains estimates of work requires for fetch.

=== modified file 'bzrlib/registry.py'
--- bzrlib/registry.py	2011-05-12 16:32:48 +0000
+++ bzrlib/registry.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,7 @@
 
 """Classes to provide name-to-object registry-like support."""
 
+from __future__ import absolute_import
 
 from bzrlib.pyutils import get_named_object
 

=== modified file 'bzrlib/remote.py'
--- bzrlib/remote.py	2011-08-26 11:35:50 +0000
+++ bzrlib/remote.py	2012-01-04 16:02:14 +0000
@@ -14,25 +14,34 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 import bz2
+import zlib
 
 from bzrlib import (
     bencode,
     branch,
     bzrdir as _mod_bzrdir,
-    config,
+    config as _mod_config,
     controldir,
     debug,
     errors,
+    gpg,
     graph,
+    inventory_delta,
     lock,
     lockdir,
+    osutils,
+    registry,
     repository as _mod_repository,
     revision as _mod_revision,
     static_tuple,
     symbol_versioning,
+    testament as _mod_testament,
     urlutils,
     vf_repository,
+    vf_search,
     )
 from bzrlib.branch import BranchReferenceFormat, BranchWriteLockResult
 from bzrlib.decorators import needs_read_lock, needs_write_lock, only_raises
@@ -40,12 +49,17 @@
     NoSuchRevision,
     SmartProtocolError,
     )
+from bzrlib.i18n import gettext
+from bzrlib.inventory import Inventory
 from bzrlib.lockable_files import LockableFiles
 from bzrlib.smart import client, vfs, repository as smart_repo
 from bzrlib.smart.client import _SmartClient
 from bzrlib.revision import NULL_REVISION
+from bzrlib.revisiontree import InventoryRevisionTree
 from bzrlib.repository import RepositoryWriteLockResult, _LazyListJoin
-from bzrlib.trace import mutter, note, warning
+from bzrlib.serializer import format_registry as serializer_format_registry
+from bzrlib.trace import mutter, note, warning, log_exception_quietly
+from bzrlib.versionedfile import ChunkedContentFactory, FulltextContentFactory
 
 
 _DEFAULT_SEARCH_DEPTH = 100
@@ -113,8 +127,13 @@
 
     def get_format_description(self):
         if self._network_name:
-            real_format = controldir.network_format_registry.get(self._network_name)
-            return 'Remote: ' + real_format.get_format_description()
+            try:
+                real_format = controldir.network_format_registry.get(
+                        self._network_name)
+            except KeyError:
+                pass
+            else:
+                return 'Remote: ' + real_format.get_format_description()
         return 'bzr remote bzrdir'
 
     def get_format_string(self):
@@ -330,6 +349,62 @@
         _mod_bzrdir.BzrDirMetaFormat1._set_repository_format) #.im_func)
 
 
+class RemoteControlStore(_mod_config.IniFileStore):
+    """Control store which attempts to use HPSS calls to retrieve control store.
+
+    Note that this is specific to bzr-based formats.
+    """
+
+    def __init__(self, bzrdir):
+        super(RemoteControlStore, self).__init__()
+        self.bzrdir = bzrdir
+        self._real_store = None
+
+    def lock_write(self, token=None):
+        self._ensure_real()
+        return self._real_store.lock_write(token)
+
+    def unlock(self):
+        self._ensure_real()
+        return self._real_store.unlock()
+
+    @needs_write_lock
+    def save(self):
+        # We need to be able to override the undecorated implementation
+        self.save_without_locking()
+
+    def save_without_locking(self):
+        super(RemoteControlStore, self).save()
+
+    def _ensure_real(self):
+        self.bzrdir._ensure_real()
+        if self._real_store is None:
+            self._real_store = _mod_config.ControlStore(self.bzrdir)
+
+    def external_url(self):
+        return self.bzrdir.user_url
+
+    def _load_content(self):
+        medium = self.bzrdir._client._medium
+        path = self.bzrdir._path_for_remote_call(self.bzrdir._client)
+        try:
+            response, handler = self.bzrdir._call_expecting_body(
+                'BzrDir.get_config_file', path)
+        except errors.UnknownSmartMethod:
+            self._ensure_real()
+            return self._real_store._load_content()
+        if len(response) and response[0] != 'ok':
+            raise errors.UnexpectedSmartServerResponse(response)
+        return handler.read_body_bytes()
+
+    def _save_content(self, content):
+        # FIXME JRV 2011-11-22: Ideally this should use a
+        # HPSS call too, but at the moment it is not possible
+        # to write lock control directories.
+        self._ensure_real()
+        return self._real_store._save_content(content)
+
+
 class RemoteBzrDir(_mod_bzrdir.BzrDir, _RpcHelper):
     """Control directory on a remote server, accessed via bzr:// or similar."""
 
@@ -406,7 +481,7 @@
                 warning('VFS BzrDir access triggered\n%s',
                     ''.join(traceback.format_stack()))
             self._real_bzrdir = _mod_bzrdir.BzrDir.open_from_transport(
-                self.root_transport, _server_formats=False)
+                self.root_transport, probers=[_mod_bzrdir.BzrProber])
             self._format._network_name = \
                 self._real_bzrdir._format.network_name()
 
@@ -419,6 +494,48 @@
         self._next_open_branch_result = None
         return _mod_bzrdir.BzrDir.break_lock(self)
 
+    def _vfs_checkout_metadir(self):
+        self._ensure_real()
+        return self._real_bzrdir.checkout_metadir()
+
+    def checkout_metadir(self):
+        """Retrieve the controldir format to use for checkouts of this one.
+        """
+        medium = self._client._medium
+        if medium._is_remote_before((2, 5)):
+            return self._vfs_checkout_metadir()
+        path = self._path_for_remote_call(self._client)
+        try:
+            response = self._client.call('BzrDir.checkout_metadir',
+                path)
+        except errors.UnknownSmartMethod:
+            medium._remember_remote_is_before((2, 5))
+            return self._vfs_checkout_metadir()
+        if len(response) != 3:
+            raise errors.UnexpectedSmartServerResponse(response)
+        control_name, repo_name, branch_name = response
+        try:
+            format = controldir.network_format_registry.get(control_name)
+        except KeyError:
+            raise errors.UnknownFormatError(kind='control',
+                format=control_name)
+        if repo_name:
+            try:
+                repo_format = _mod_repository.network_format_registry.get(
+                    repo_name)
+            except KeyError:
+                raise errors.UnknownFormatError(kind='repository',
+                    format=repo_name)
+            format.repository_format = repo_format
+        if branch_name:
+            try:
+                format.set_branch_format(
+                    branch.network_format_registry.get(branch_name))
+            except KeyError:
+                raise errors.UnknownFormatError(kind='branch',
+                    format=branch_name)
+        return format
+
     def _vfs_cloning_metadir(self, require_stacking=False):
         self._ensure_real()
         return self._real_bzrdir.cloning_metadir(
@@ -454,10 +571,18 @@
         if len(branch_info) != 2:
             raise errors.UnexpectedSmartServerResponse(response)
         branch_ref, branch_name = branch_info
-        format = controldir.network_format_registry.get(control_name)
+        try:
+            format = controldir.network_format_registry.get(control_name)
+        except KeyError:
+            raise errors.UnknownFormatError(kind='control', format=control_name)
+
         if repo_name:
-            format.repository_format = _mod_repository.network_format_registry.get(
-                repo_name)
+            try:
+                format.repository_format = _mod_repository.network_format_registry.get(
+                    repo_name)
+            except KeyError:
+                raise errors.UnknownFormatError(kind='repository',
+                    format=repo_name)
         if branch_ref == 'ref':
             # XXX: we need possible_transports here to avoid reopening the
             # connection to the referenced location
@@ -466,8 +591,13 @@
             format.set_branch_format(branch_format)
         elif branch_ref == 'branch':
             if branch_name:
-                format.set_branch_format(
-                    branch.network_format_registry.get(branch_name))
+                try:
+                    branch_format = branch.network_format_registry.get(
+                        branch_name)
+                except KeyError:
+                    raise errors.UnknownFormatError(kind='branch',
+                        format=branch_name)
+                format.set_branch_format(branch_format)
         else:
             raise errors.UnexpectedSmartServerResponse(response)
         return format
@@ -483,14 +613,23 @@
 
     def destroy_repository(self):
         """See BzrDir.destroy_repository"""
-        self._ensure_real()
-        self._real_bzrdir.destroy_repository()
+        path = self._path_for_remote_call(self._client)
+        try:
+            response = self._call('BzrDir.destroy_repository', path)
+        except errors.UnknownSmartMethod:
+            self._ensure_real()
+            self._real_bzrdir.destroy_repository()
+            return
+        if response[0] != 'ok':
+            raise SmartProtocolError('unexpected response code %s' % (response,))
 
-    def create_branch(self, name=None, repository=None):
+    def create_branch(self, name=None, repository=None,
+                      append_revisions_only=None):
         # as per meta1 formats - just delegate to the format object which may
         # be parameterised.
         real_branch = self._format.get_branch_format().initialize(self,
-            name=name, repository=repository)
+            name=name, repository=repository,
+            append_revisions_only=append_revisions_only)
         if not isinstance(real_branch, RemoteBranch):
             if not isinstance(repository, RemoteRepository):
                 raise AssertionError(
@@ -510,9 +649,21 @@
 
     def destroy_branch(self, name=None):
         """See BzrDir.destroy_branch"""
-        self._ensure_real()
-        self._real_bzrdir.destroy_branch(name=name)
+        path = self._path_for_remote_call(self._client)
+        try:
+            if name is not None:
+                args = (name, )
+            else:
+                args = ()
+            response = self._call('BzrDir.destroy_branch', path, *args)
+        except errors.UnknownSmartMethod:
+            self._ensure_real()
+            self._real_bzrdir.destroy_branch(name=name)
+            self._next_open_branch_result = None
+            return
         self._next_open_branch_result = None
+        if response[0] != 'ok':
+            raise SmartProtocolError('unexpected response code %s' % (response,))
 
     def create_workingtree(self, revision_id=None, from_branch=None,
         accelerator_tree=None, hardlink=False):
@@ -572,7 +723,7 @@
         return None, self.open_branch(name=name)
 
     def open_branch(self, name=None, unsupported=False,
-                    ignore_fallbacks=False):
+                    ignore_fallbacks=False, possible_transports=None):
         if unsupported:
             raise NotImplementedError('unsupported flag support not implemented yet.')
         if self._next_open_branch_result is not None:
@@ -585,13 +736,15 @@
             # a branch reference, use the existing BranchReference logic.
             format = BranchReferenceFormat()
             return format.open(self, name=name, _found=True,
-                location=response[1], ignore_fallbacks=ignore_fallbacks)
+                location=response[1], ignore_fallbacks=ignore_fallbacks,
+                possible_transports=possible_transports)
         branch_format_name = response[1]
         if not branch_format_name:
             branch_format_name = None
         format = RemoteBranchFormat(network_name=branch_format_name)
         return RemoteBranch(self, self.find_repository(), format=format,
-            setup_stacking=not ignore_fallbacks, name=name)
+            setup_stacking=not ignore_fallbacks, name=name,
+            possible_transports=possible_transports)
 
     def _open_repo_v1(self, path):
         verb = 'BzrDir.find_repository'
@@ -660,8 +813,16 @@
 
     def has_workingtree(self):
         if self._has_working_tree is None:
-            self._ensure_real()
-            self._has_working_tree = self._real_bzrdir.has_workingtree()
+            path = self._path_for_remote_call(self._client)
+            try:
+                response = self._call('BzrDir.has_workingtree', path)
+            except errors.UnknownSmartMethod:
+                self._ensure_real()
+                self._has_working_tree = self._real_bzrdir.has_workingtree()
+            else:
+                if response[0] not in ('yes', 'no'):
+                    raise SmartProtocolError('unexpected response code %s' % (response,))
+                self._has_working_tree = (response[0] == 'yes')
         return self._has_working_tree
 
     def open_workingtree(self, recommend_upgrade=True):
@@ -695,15 +856,12 @@
         """Upgrading of remote bzrdirs is not supported yet."""
         return False
 
-    def clone(self, url, revision_id=None, force_new_repo=False,
-              preserve_stacking=False):
-        self._ensure_real()
-        return self._real_bzrdir.clone(url, revision_id=revision_id,
-            force_new_repo=force_new_repo, preserve_stacking=preserve_stacking)
-
     def _get_config(self):
         return RemoteBzrDirConfig(self)
 
+    def _get_config_store(self):
+        return RemoteControlStore(self)
+
 
 class RemoteRepositoryFormat(vf_repository.VersionedFileRepositoryFormat):
     """Format for repositories accessed over a _SmartClient.
@@ -739,6 +897,7 @@
         self._supports_external_lookups = None
         self._supports_tree_reference = None
         self._supports_funky_characters = None
+        self._supports_nesting_repositories = None
         self._rich_root_data = None
 
     def __repr__(self):
@@ -781,6 +940,14 @@
         return self._supports_funky_characters
 
     @property
+    def supports_nesting_repositories(self):
+        if self._supports_nesting_repositories is None:
+            self._ensure_real()
+            self._supports_nesting_repositories = \
+                self._custom_format.supports_nesting_repositories
+        return self._supports_nesting_repositories
+
+    @property
     def supports_tree_reference(self):
         if self._supports_tree_reference is None:
             self._ensure_real()
@@ -868,8 +1035,12 @@
 
     def _ensure_real(self):
         if self._custom_format is None:
-            self._custom_format = _mod_repository.network_format_registry.get(
-                self._network_name)
+            try:
+                self._custom_format = _mod_repository.network_format_registry.get(
+                    self._network_name)
+            except KeyError:
+                raise errors.UnknownFormatError(kind='repository',
+                    format=self._network_name)
 
     @property
     def _fetch_order(self):
@@ -910,8 +1081,8 @@
         return self._custom_format._serializer
 
 
-class RemoteRepository(_RpcHelper, lock._RelockDebugMixin,
-    controldir.ControlComponent):
+class RemoteRepository(_mod_repository.Repository, _RpcHelper,
+        lock._RelockDebugMixin):
     """Repository accessed over rpc.
 
     For the moment most operations are performed using local transport-backed
@@ -941,6 +1112,7 @@
         self._format = format
         self._lock_mode = None
         self._lock_token = None
+        self._write_group_tokens = None
         self._lock_count = 0
         self._leave_lock = False
         # Cache of revision parents; misses are cached during read locks, and
@@ -986,9 +1158,30 @@
 
         :param suppress_errors: see Repository.abort_write_group.
         """
-        self._ensure_real()
-        return self._real_repository.abort_write_group(
-            suppress_errors=suppress_errors)
+        if self._real_repository:
+            self._ensure_real()
+            return self._real_repository.abort_write_group(
+                suppress_errors=suppress_errors)
+        if not self.is_in_write_group():
+            if suppress_errors:
+                mutter('(suppressed) not in write group')
+                return
+            raise errors.BzrError("not in write group")
+        path = self.bzrdir._path_for_remote_call(self._client)
+        try:
+            response = self._call('Repository.abort_write_group', path,
+                self._lock_token, self._write_group_tokens)
+        except Exception, exc:
+            self._write_group = None
+            if not suppress_errors:
+                raise
+            mutter('abort_write_group failed')
+            log_exception_quietly()
+            note(gettext('bzr: ERROR (ignored): %s'), exc)
+        else:
+            if response != ('ok', ):
+                raise errors.UnexpectedSmartServerResponse(response)
+            self._write_group_tokens = None
 
     @property
     def chk_bytes(self):
@@ -1008,16 +1201,40 @@
         for older plugins that don't use e.g. the CommitBuilder
         facility.
         """
-        self._ensure_real()
-        return self._real_repository.commit_write_group()
+        if self._real_repository:
+            self._ensure_real()
+            return self._real_repository.commit_write_group()
+        if not self.is_in_write_group():
+            raise errors.BzrError("not in write group")
+        path = self.bzrdir._path_for_remote_call(self._client)
+        response = self._call('Repository.commit_write_group', path,
+            self._lock_token, self._write_group_tokens)
+        if response != ('ok', ):
+            raise errors.UnexpectedSmartServerResponse(response)
+        self._write_group_tokens = None
+        # Refresh data after writing to the repository.
+        self.refresh_data()
 
     def resume_write_group(self, tokens):
-        self._ensure_real()
-        return self._real_repository.resume_write_group(tokens)
+        if self._real_repository:
+            return self._real_repository.resume_write_group(tokens)
+        path = self.bzrdir._path_for_remote_call(self._client)
+        try:
+            response = self._call('Repository.check_write_group', path,
+               self._lock_token, tokens)
+        except errors.UnknownSmartMethod:
+            self._ensure_real()
+            return self._real_repository.resume_write_group(tokens)
+        if response != ('ok', ):
+            raise errors.UnexpectedSmartServerResponse(response)
+        self._write_group_tokens = tokens
 
     def suspend_write_group(self):
-        self._ensure_real()
-        return self._real_repository.suspend_write_group()
+        if self._real_repository:
+            return self._real_repository.suspend_write_group()
+        ret = self._write_group_tokens or []
+        self._write_group_tokens = None
+        return ret
 
     def get_missing_parent_inventories(self, check_for_missing_texts=True):
         self._ensure_real()
@@ -1233,15 +1450,23 @@
 
     def get_physical_lock_status(self):
         """See Repository.get_physical_lock_status()."""
-        # should be an API call to the server.
-        self._ensure_real()
-        return self._real_repository.get_physical_lock_status()
+        path = self.bzrdir._path_for_remote_call(self._client)
+        try:
+            response = self._call('Repository.get_physical_lock_status', path)
+        except errors.UnknownSmartMethod:
+            self._ensure_real()
+            return self._real_repository.get_physical_lock_status()
+        if response[0] not in ('yes', 'no'):
+            raise errors.UnexpectedSmartServerResponse(response)
+        return (response[0] == 'yes')
 
     def is_in_write_group(self):
         """Return True if there is an open write group.
 
         write groups are only applicable locally for the smart server..
         """
+        if self._write_group_tokens is not None:
+            return True
         if self._real_repository:
             return self._real_repository.is_in_write_group()
 
@@ -1382,6 +1607,10 @@
             self._real_repository.lock_write(self._lock_token)
         elif self._lock_mode == 'r':
             self._real_repository.lock_read()
+        if self._write_group_tokens is not None:
+            # if we are already in a write group, resume it
+            self._real_repository.resume_write_group(self._write_group_tokens)
+            self._write_group_tokens = None
 
     def start_write_group(self):
         """Start a write group on the decorated repository.
@@ -1391,8 +1620,23 @@
         for older plugins that don't use e.g. the CommitBuilder
         facility.
         """
-        self._ensure_real()
-        return self._real_repository.start_write_group()
+        if self._real_repository:
+            self._ensure_real()
+            return self._real_repository.start_write_group()
+        if not self.is_write_locked():
+            raise errors.NotWriteLocked(self)
+        if self._write_group_tokens is not None:
+            raise errors.BzrError('already in a write group')
+        path = self.bzrdir._path_for_remote_call(self._client)
+        try:
+            response = self._call('Repository.start_write_group', path,
+                self._lock_token)
+        except (errors.UnknownSmartMethod, errors.UnsuspendableWriteGroup):
+            self._ensure_real()
+            return self._real_repository.start_write_group()
+        if response[0] != 'ok':
+            raise errors.UnexpectedSmartServerResponse(response)
+        self._write_group_tokens = response[1]
 
     def _unlock(self, token):
         path = self.bzrdir._path_for_remote_call(self._client)
@@ -1425,6 +1669,8 @@
             # This is just to let the _real_repository stay up to date.
             if self._real_repository is not None:
                 self._real_repository.unlock()
+            elif self._write_group_tokens is not None:
+                self.abort_write_group()
         finally:
             # The rpc-level lock should be released even if there was a
             # problem releasing the vfs-based lock.
@@ -1442,8 +1688,14 @@
 
     def break_lock(self):
         # should hand off to the network
-        self._ensure_real()
-        return self._real_repository.break_lock()
+        path = self.bzrdir._path_for_remote_call(self._client)
+        try:
+            response = self._call("Repository.break_lock", path)
+        except errors.UnknownSmartMethod:
+            self._ensure_real()
+            return self._real_repository.break_lock()
+        if response != ('ok',):
+            raise errors.UnexpectedSmartServerResponse(response)
 
     def _get_tarball(self, compression):
         """Return a TemporaryFile containing a repository tarball.
@@ -1467,36 +1719,81 @@
             return t
         raise errors.UnexpectedSmartServerResponse(response)
 
+    @needs_read_lock
     def sprout(self, to_bzrdir, revision_id=None):
-        # TODO: Option to control what format is created?
-        self._ensure_real()
-        dest_repo = self._real_repository._format.initialize(to_bzrdir,
-                                                             shared=False)
+        """Create a descendent repository for new development.
+
+        Unlike clone, this does not copy the settings of the repository.
+        """
+        dest_repo = self._create_sprouting_repo(to_bzrdir, shared=False)
         dest_repo.fetch(self, revision_id=revision_id)
         return dest_repo
 
+    def _create_sprouting_repo(self, a_bzrdir, shared):
+        if not isinstance(a_bzrdir._format, self.bzrdir._format.__class__):
+            # use target default format.
+            dest_repo = a_bzrdir.create_repository()
+        else:
+            # Most control formats need the repository to be specifically
+            # created, but on some old all-in-one formats it's not needed
+            try:
+                dest_repo = self._format.initialize(a_bzrdir, shared=shared)
+            except errors.UninitializableFormat:
+                dest_repo = a_bzrdir.open_repository()
+        return dest_repo
+
     ### These methods are just thin shims to the VFS object for now.
 
+    @needs_read_lock
     def revision_tree(self, revision_id):
-        self._ensure_real()
-        return self._real_repository.revision_tree(revision_id)
+        revision_id = _mod_revision.ensure_null(revision_id)
+        if revision_id == _mod_revision.NULL_REVISION:
+            return InventoryRevisionTree(self,
+                Inventory(root_id=None), _mod_revision.NULL_REVISION)
+        else:
+            return list(self.revision_trees([revision_id]))[0]
 
     def get_serializer_format(self):
-        self._ensure_real()
-        return self._real_repository.get_serializer_format()
+        path = self.bzrdir._path_for_remote_call(self._client)
+        try:
+            response = self._call('VersionedFileRepository.get_serializer_format',
+                path)
+        except errors.UnknownSmartMethod:
+            self._ensure_real()
+            return self._real_repository.get_serializer_format()
+        if response[0] != 'ok':
+            raise errors.UnexpectedSmartServerResponse(response)
+        return response[1]
 
     def get_commit_builder(self, branch, parents, config, timestamp=None,
                            timezone=None, committer=None, revprops=None,
                            revision_id=None, lossy=False):
-        # FIXME: It ought to be possible to call this without immediately
-        # triggering _ensure_real.  For now it's the easiest thing to do.
-        self._ensure_real()
-        real_repo = self._real_repository
-        builder = real_repo.get_commit_builder(branch, parents,
-                config, timestamp=timestamp, timezone=timezone,
-                committer=committer, revprops=revprops,
-                revision_id=revision_id, lossy=lossy)
-        return builder
+        """Obtain a CommitBuilder for this repository.
+
+        :param branch: Branch to commit to.
+        :param parents: Revision ids of the parents of the new revision.
+        :param config: Configuration to use.
+        :param timestamp: Optional timestamp recorded for commit.
+        :param timezone: Optional timezone for timestamp.
+        :param committer: Optional committer to set for commit.
+        :param revprops: Optional dictionary of revision properties.
+        :param revision_id: Optional revision id.
+        :param lossy: Whether to discard data that can not be natively
+            represented, when pushing to a foreign VCS
+        """
+        if self._fallback_repositories and not self._format.supports_chks:
+            raise errors.BzrError("Cannot commit directly to a stacked branch"
+                " in pre-2a formats. See "
+                "https://bugs.launchpad.net/bzr/+bug/375013 for details.")
+        if self._format.rich_root_data:
+            commit_builder_kls = vf_repository.VersionedFileRootCommitBuilder
+        else:
+            commit_builder_kls = vf_repository.VersionedFileCommitBuilder
+        result = commit_builder_kls(self, parents, config,
+            timestamp, timezone, committer, revprops, revision_id,
+            lossy)
+        self.start_write_group()
+        return result
 
     def add_fallback_repository(self, repository):
         """Add a repository to use for looking up data not held locally.
@@ -1509,12 +1806,13 @@
         # We need to accumulate additional repositories here, to pass them in
         # on various RPC's.
         #
+        # Make the check before we lock: this raises an exception.
+        self._check_fallback_repository(repository)
         if self.is_locked():
             # We will call fallback.unlock() when we transition to the unlocked
             # state, so always add a lock here. If a caller passes us a locked
             # repository, they are responsible for unlocking it later.
             repository.lock_read()
-        self._check_fallback_repository(repository)
         self._fallback_repositories.append(repository)
         # If self._real_repository was parameterised already (e.g. because a
         # _real_branch had its get_stacked_on_url method called), then the
@@ -1546,24 +1844,159 @@
             delta, new_revision_id, parents, basis_inv=basis_inv,
             propagate_caches=propagate_caches)
 
-    def add_revision(self, rev_id, rev, inv=None, config=None):
-        self._ensure_real()
-        return self._real_repository.add_revision(
-            rev_id, rev, inv=inv, config=config)
+    def add_revision(self, revision_id, rev, inv=None):
+        _mod_revision.check_not_reserved_id(revision_id)
+        key = (revision_id,)
+        # check inventory present
+        if not self.inventories.get_parent_map([key]):
+            if inv is None:
+                raise errors.WeaveRevisionNotPresent(revision_id,
+                                                     self.inventories)
+            else:
+                # yes, this is not suitable for adding with ghosts.
+                rev.inventory_sha1 = self.add_inventory(revision_id, inv,
+                                                        rev.parent_ids)
+        else:
+            rev.inventory_sha1 = self.inventories.get_sha1s([key])[key]
+        self._add_revision(rev)
+
+    def _add_revision(self, rev):
+        if self._real_repository is not None:
+            return self._real_repository._add_revision(rev)
+        text = self._serializer.write_revision_to_string(rev)
+        key = (rev.revision_id,)
+        parents = tuple((parent,) for parent in rev.parent_ids)
+        self._write_group_tokens, missing_keys = self._get_sink().insert_stream(
+            [('revisions', [FulltextContentFactory(key, parents, None, text)])],
+            self._format, self._write_group_tokens)
 
     @needs_read_lock
     def get_inventory(self, revision_id):
+        return list(self.iter_inventories([revision_id]))[0]
+
+    def _iter_inventories_rpc(self, revision_ids, ordering):
+        if ordering is None:
+            ordering = 'unordered'
+        path = self.bzrdir._path_for_remote_call(self._client)
+        body = "\n".join(revision_ids)
+        response_tuple, response_handler = (
+            self._call_with_body_bytes_expecting_body(
+                "VersionedFileRepository.get_inventories",
+                (path, ordering), body))
+        if response_tuple[0] != "ok":
+            raise errors.UnexpectedSmartServerResponse(response_tuple)
+        deserializer = inventory_delta.InventoryDeltaDeserializer()
+        byte_stream = response_handler.read_streamed_body()
+        decoded = smart_repo._byte_stream_to_stream(byte_stream)
+        if decoded is None:
+            # no results whatsoever
+            return
+        src_format, stream = decoded
+        if src_format.network_name() != self._format.network_name():
+            raise AssertionError(
+                "Mismatched RemoteRepository and stream src %r, %r" % (
+                src_format.network_name(), self._format.network_name()))
+        # ignore the src format, it's not really relevant
+        prev_inv = Inventory(root_id=None,
+            revision_id=_mod_revision.NULL_REVISION)
+        # there should be just one substream, with inventory deltas
+        substream_kind, substream = stream.next()
+        if substream_kind != "inventory-deltas":
+            raise AssertionError(
+                 "Unexpected stream %r received" % substream_kind)
+        for record in substream:
+            (parent_id, new_id, versioned_root, tree_references, invdelta) = (
+                deserializer.parse_text_bytes(record.get_bytes_as("fulltext")))
+            if parent_id != prev_inv.revision_id:
+                raise AssertionError("invalid base %r != %r" % (parent_id,
+                    prev_inv.revision_id))
+            inv = prev_inv.create_by_apply_delta(invdelta, new_id)
+            yield inv, inv.revision_id
+            prev_inv = inv
+
+    def _iter_inventories_vfs(self, revision_ids, ordering=None):
         self._ensure_real()
-        return self._real_repository.get_inventory(revision_id)
+        return self._real_repository._iter_inventories(revision_ids, ordering)
 
     def iter_inventories(self, revision_ids, ordering=None):
-        self._ensure_real()
-        return self._real_repository.iter_inventories(revision_ids, ordering)
+        """Get many inventories by revision_ids.
+
+        This will buffer some or all of the texts used in constructing the
+        inventories in memory, but will only parse a single inventory at a
+        time.
+
+        :param revision_ids: The expected revision ids of the inventories.
+        :param ordering: optional ordering, e.g. 'topological'.  If not
+            specified, the order of revision_ids will be preserved (by
+            buffering if necessary).
+        :return: An iterator of inventories.
+        """
+        if ((None in revision_ids)
+            or (_mod_revision.NULL_REVISION in revision_ids)):
+            raise ValueError('cannot get null revision inventory')
+        for inv, revid in self._iter_inventories(revision_ids, ordering):
+            if inv is None:
+                raise errors.NoSuchRevision(self, revid)
+            yield inv
+
+    def _iter_inventories(self, revision_ids, ordering=None):
+        if len(revision_ids) == 0:
+            return
+        missing = set(revision_ids)
+        if ordering is None:
+            order_as_requested = True
+            invs = {}
+            order = list(revision_ids)
+            order.reverse()
+            next_revid = order.pop()
+        else:
+            order_as_requested = False
+            if ordering != 'unordered' and self._fallback_repositories:
+                raise ValueError('unsupported ordering %r' % ordering)
+        iter_inv_fns = [self._iter_inventories_rpc] + [
+            fallback._iter_inventories for fallback in
+            self._fallback_repositories]
+        try:
+            for iter_inv in iter_inv_fns:
+                request = [revid for revid in revision_ids if revid in missing]
+                for inv, revid in iter_inv(request, ordering):
+                    if inv is None:
+                        continue
+                    missing.remove(inv.revision_id)
+                    if ordering != 'unordered':
+                        invs[revid] = inv
+                    else:
+                        yield inv, revid
+                if order_as_requested:
+                    # Yield as many results as we can while preserving order.
+                    while next_revid in invs:
+                        inv = invs.pop(next_revid)
+                        yield inv, inv.revision_id
+                        try:
+                            next_revid = order.pop()
+                        except IndexError:
+                            # We still want to fully consume the stream, just
+                            # in case it is not actually finished at this point
+                            next_revid = None
+                            break
+        except errors.UnknownSmartMethod:
+            for inv, revid in self._iter_inventories_vfs(revision_ids, ordering):
+                yield inv, revid
+            return
+        # Report missing
+        if order_as_requested:
+            if next_revid is not None:
+                yield None, next_revid
+            while order:
+                revid = order.pop()
+                yield invs.get(revid), revid
+        else:
+            while missing:
+                yield None, missing.pop()
 
     @needs_read_lock
     def get_revision(self, revision_id):
-        self._ensure_real()
-        return self._real_repository.get_revision(revision_id)
+        return self.get_revisions([revision_id])[0]
 
     def get_transaction(self):
         self._ensure_real()
@@ -1571,13 +2004,22 @@
 
     @needs_read_lock
     def clone(self, a_bzrdir, revision_id=None):
-        self._ensure_real()
-        return self._real_repository.clone(a_bzrdir, revision_id=revision_id)
+        dest_repo = self._create_sprouting_repo(
+            a_bzrdir, shared=self.is_shared())
+        self.copy_content_into(dest_repo, revision_id)
+        return dest_repo
 
     def make_working_trees(self):
         """See Repository.make_working_trees"""
-        self._ensure_real()
-        return self._real_repository.make_working_trees()
+        path = self.bzrdir._path_for_remote_call(self._client)
+        try:
+            response = self._call('Repository.make_working_trees', path)
+        except errors.UnknownSmartMethod:
+            self._ensure_real()
+            return self._real_repository.make_working_trees()
+        if response[0] not in ('yes', 'no'):
+            raise SmartProtocolError('unexpected response code %s' % (response,))
+        return response[0] == 'yes'
 
     def refresh_data(self):
         """Re-read any data needed to synchronise with disk.
@@ -1592,6 +2034,9 @@
         """
         if self._real_repository is not None:
             self._real_repository.refresh_data()
+        # Refresh the parents cache for this object
+        self._unstacked_provider.disable_cache()
+        self._unstacked_provider.enable_cache()
 
     def revision_ids_to_search_result(self, result_set):
         """Convert a set of revision ids to a graph SearchResult."""
@@ -1602,7 +2047,7 @@
         included_keys = result_set.intersection(result_parents)
         start_keys = result_set.difference(included_keys)
         exclude_keys = result_parents.difference(result_set)
-        result = graph.SearchResult(start_keys, exclude_keys,
+        result = vf_search.SearchResult(start_keys, exclude_keys,
             len(result_set), result_set)
         return result
 
@@ -1656,6 +2101,10 @@
         # the InterRepository base class, which raises an
         # IncompatibleRepositories when asked to fetch.
         inter = _mod_repository.InterRepository.get(source, self)
+        if (fetch_spec is not None and
+            not getattr(inter, "supports_fetch_spec", False)):
+            raise errors.UnsupportedOperation(
+                "fetch_spec not supported for %r" % inter)
         return inter.fetch(revision_id=revision_id,
             find_ghosts=find_ghosts, fetch_spec=fetch_spec)
 
@@ -1679,11 +2128,80 @@
         return self._real_repository._get_versioned_file_checker(
             revisions, revision_versions_cache)
 
+    def _iter_files_bytes_rpc(self, desired_files, absent):
+        path = self.bzrdir._path_for_remote_call(self._client)
+        lines = []
+        identifiers = []
+        for (file_id, revid, identifier) in desired_files:
+            lines.append("%s\0%s" % (
+                osutils.safe_file_id(file_id),
+                osutils.safe_revision_id(revid)))
+            identifiers.append(identifier)
+        (response_tuple, response_handler) = (
+            self._call_with_body_bytes_expecting_body(
+            "Repository.iter_files_bytes", (path, ), "\n".join(lines)))
+        if response_tuple != ('ok', ):
+            response_handler.cancel_read_body()
+            raise errors.UnexpectedSmartServerResponse(response_tuple)
+        byte_stream = response_handler.read_streamed_body()
+        def decompress_stream(start, byte_stream, unused):
+            decompressor = zlib.decompressobj()
+            yield decompressor.decompress(start)
+            while decompressor.unused_data == "":
+                try:
+                    data = byte_stream.next()
+                except StopIteration:
+                    break
+                yield decompressor.decompress(data)
+            yield decompressor.flush()
+            unused.append(decompressor.unused_data)
+        unused = ""
+        while True:
+            while not "\n" in unused:
+                unused += byte_stream.next()
+            header, rest = unused.split("\n", 1)
+            args = header.split("\0")
+            if args[0] == "absent":
+                absent[identifiers[int(args[3])]] = (args[1], args[2])
+                unused = rest
+                continue
+            elif args[0] == "ok":
+                idx = int(args[1])
+            else:
+                raise errors.UnexpectedSmartServerResponse(args)
+            unused_chunks = []
+            yield (identifiers[idx],
+                decompress_stream(rest, byte_stream, unused_chunks))
+            unused = "".join(unused_chunks)
+
     def iter_files_bytes(self, desired_files):
         """See Repository.iter_file_bytes.
         """
-        self._ensure_real()
-        return self._real_repository.iter_files_bytes(desired_files)
+        try:
+            absent = {}
+            for (identifier, bytes_iterator) in self._iter_files_bytes_rpc(
+                    desired_files, absent):
+                yield identifier, bytes_iterator
+            for fallback in self._fallback_repositories:
+                if not absent:
+                    break
+                desired_files = [(key[0], key[1], identifier) for
+                    (identifier, key) in absent.iteritems()]
+                for (identifier, bytes_iterator) in fallback.iter_files_bytes(desired_files):
+                    del absent[identifier]
+                    yield identifier, bytes_iterator
+            if absent:
+                # There may be more missing items, but raise an exception
+                # for just one.
+                missing_identifier = absent.keys()[0]
+                missing_key = absent[missing_identifier]
+                raise errors.RevisionNotPresent(revision_id=missing_key[1],
+                    file_id=missing_key[0])
+        except errors.UnknownSmartMethod:
+            self._ensure_real()
+            for (identifier, bytes_iterator) in (
+                self._real_repository.iter_files_bytes(desired_files)):
+                yield identifier, bytes_iterator
 
     def get_cached_parent_map(self, revision_ids):
         """See bzrlib.CachingParentsProvider.get_cached_parent_map"""
@@ -1754,11 +2272,11 @@
             parents_map = {}
         if _DEFAULT_SEARCH_DEPTH <= 0:
             (start_set, stop_keys,
-             key_count) = graph.search_result_from_parent_map(
+             key_count) = vf_search.search_result_from_parent_map(
                 parents_map, self._unstacked_provider.missing_keys)
         else:
             (start_set, stop_keys,
-             key_count) = graph.limited_search_result_from_parent_map(
+             key_count) = vf_search.limited_search_result_from_parent_map(
                 parents_map, self._unstacked_provider.missing_keys,
                 keys, depth=_DEFAULT_SEARCH_DEPTH)
         recipe = ('manual', start_set, stop_keys, key_count)
@@ -1815,38 +2333,137 @@
 
     @needs_read_lock
     def get_signature_text(self, revision_id):
-        self._ensure_real()
-        return self._real_repository.get_signature_text(revision_id)
+        path = self.bzrdir._path_for_remote_call(self._client)
+        try:
+            response_tuple, response_handler = self._call_expecting_body(
+                'Repository.get_revision_signature_text', path, revision_id)
+        except errors.UnknownSmartMethod:
+            self._ensure_real()
+            return self._real_repository.get_signature_text(revision_id)
+        except errors.NoSuchRevision, err:
+            for fallback in self._fallback_repositories:
+                try:
+                    return fallback.get_signature_text(revision_id)
+                except errors.NoSuchRevision:
+                    pass
+            raise err
+        else:
+            if response_tuple[0] != 'ok':
+                raise errors.UnexpectedSmartServerResponse(response_tuple)
+            return response_handler.read_body_bytes()
 
     @needs_read_lock
     def _get_inventory_xml(self, revision_id):
+        # This call is used by older working tree formats,
+        # which stored a serialized basis inventory.
         self._ensure_real()
         return self._real_repository._get_inventory_xml(revision_id)
 
+    @needs_write_lock
     def reconcile(self, other=None, thorough=False):
-        self._ensure_real()
-        return self._real_repository.reconcile(other=other, thorough=thorough)
+        from bzrlib.reconcile import RepoReconciler
+        path = self.bzrdir._path_for_remote_call(self._client)
+        try:
+            response, handler = self._call_expecting_body(
+                'Repository.reconcile', path, self._lock_token)
+        except (errors.UnknownSmartMethod, errors.TokenLockingNotSupported):
+            self._ensure_real()
+            return self._real_repository.reconcile(other=other, thorough=thorough)
+        if response != ('ok', ):
+            raise errors.UnexpectedSmartServerResponse(response)
+        body = handler.read_body_bytes()
+        result = RepoReconciler(self)
+        for line in body.split('\n'):
+            if not line:
+                continue
+            key, val_text = line.split(':')
+            if key == "garbage_inventories":
+                result.garbage_inventories = int(val_text)
+            elif key == "inconsistent_parents":
+                result.inconsistent_parents = int(val_text)
+            else:
+                mutter("unknown reconcile key %r" % key)
+        return result
 
     def all_revision_ids(self):
-        self._ensure_real()
-        return self._real_repository.all_revision_ids()
+        path = self.bzrdir._path_for_remote_call(self._client)
+        try:
+            response_tuple, response_handler = self._call_expecting_body(
+                "Repository.all_revision_ids", path)
+        except errors.UnknownSmartMethod:
+            self._ensure_real()
+            return self._real_repository.all_revision_ids()
+        if response_tuple != ("ok", ):
+            raise errors.UnexpectedSmartServerResponse(response_tuple)
+        revids = set(response_handler.read_body_bytes().splitlines())
+        for fallback in self._fallback_repositories:
+            revids.update(set(fallback.all_revision_ids()))
+        return list(revids)
+
+    def _filtered_revision_trees(self, revision_ids, file_ids):
+        """Return Tree for a revision on this branch with only some files.
+
+        :param revision_ids: a sequence of revision-ids;
+          a revision-id may not be None or 'null:'
+        :param file_ids: if not None, the result is filtered
+          so that only those file-ids, their parents and their
+          children are included.
+        """
+        inventories = self.iter_inventories(revision_ids)
+        for inv in inventories:
+            # Should we introduce a FilteredRevisionTree class rather
+            # than pre-filter the inventory here?
+            filtered_inv = inv.filter(file_ids)
+            yield InventoryRevisionTree(self, filtered_inv, filtered_inv.revision_id)
 
     @needs_read_lock
     def get_deltas_for_revisions(self, revisions, specific_fileids=None):
-        self._ensure_real()
-        return self._real_repository.get_deltas_for_revisions(revisions,
-            specific_fileids=specific_fileids)
+        medium = self._client._medium
+        if medium._is_remote_before((1, 2)):
+            self._ensure_real()
+            for delta in self._real_repository.get_deltas_for_revisions(
+                    revisions, specific_fileids):
+                yield delta
+            return
+        # Get the revision-ids of interest
+        required_trees = set()
+        for revision in revisions:
+            required_trees.add(revision.revision_id)
+            required_trees.update(revision.parent_ids[:1])
+
+        # Get the matching filtered trees. Note that it's more
+        # efficient to pass filtered trees to changes_from() rather
+        # than doing the filtering afterwards. changes_from() could
+        # arguably do the filtering itself but it's path-based, not
+        # file-id based, so filtering before or afterwards is
+        # currently easier.
+        if specific_fileids is None:
+            trees = dict((t.get_revision_id(), t) for
+                t in self.revision_trees(required_trees))
+        else:
+            trees = dict((t.get_revision_id(), t) for
+                t in self._filtered_revision_trees(required_trees,
+                specific_fileids))
+
+        # Calculate the deltas
+        for revision in revisions:
+            if not revision.parent_ids:
+                old_tree = self.revision_tree(_mod_revision.NULL_REVISION)
+            else:
+                old_tree = trees[revision.parent_ids[0]]
+            yield trees[revision.revision_id].changes_from(old_tree)
 
     @needs_read_lock
     def get_revision_delta(self, revision_id, specific_fileids=None):
-        self._ensure_real()
-        return self._real_repository.get_revision_delta(revision_id,
-            specific_fileids=specific_fileids)
+        r = self.get_revision(revision_id)
+        return list(self.get_deltas_for_revisions([r],
+            specific_fileids=specific_fileids))[0]
 
     @needs_read_lock
     def revision_trees(self, revision_ids):
-        self._ensure_real()
-        return self._real_repository.revision_trees(revision_ids)
+        inventories = self.iter_inventories(revision_ids)
+        for inv in inventories:
+            yield InventoryRevisionTree(self, inv, inv.revision_id)
 
     @needs_read_lock
     def get_revision_reconcile(self, revision_id):
@@ -1860,17 +2477,20 @@
             callback_refs=callback_refs, check_repo=check_repo)
 
     def copy_content_into(self, destination, revision_id=None):
-        self._ensure_real()
-        return self._real_repository.copy_content_into(
-            destination, revision_id=revision_id)
+        """Make a complete copy of the content in self into destination.
+
+        This is a destructive operation! Do not use it on existing
+        repositories.
+        """
+        interrepo = _mod_repository.InterRepository.get(self, destination)
+        return interrepo.copy_content(revision_id)
 
     def _copy_repository_tarball(self, to_bzrdir, revision_id=None):
         # get a tarball of the remote repository, and copy from that into the
         # destination
-        from bzrlib import osutils
         import tarfile
         # TODO: Maybe a progress bar while streaming the tarball?
-        note("Copying repository content as tarball...")
+        note(gettext("Copying repository content as tarball..."))
         tar_file = self._get_tarball('bz2')
         if tar_file is None:
             return None
@@ -1905,19 +2525,28 @@
     @needs_write_lock
     def pack(self, hint=None, clean_obsolete_packs=False):
         """Compress the data within the repository.
-
-        This is not currently implemented within the smart server.
         """
-        self._ensure_real()
-        return self._real_repository.pack(hint=hint, clean_obsolete_packs=clean_obsolete_packs)
+        if hint is None:
+            body = ""
+        else:
+            body = "".join([l+"\n" for l in hint])
+        path = self.bzrdir._path_for_remote_call(self._client)
+        try:
+            response, handler = self._call_with_body_bytes_expecting_body(
+                'Repository.pack', (path, self._lock_token,
+                    str(clean_obsolete_packs)), body)
+        except errors.UnknownSmartMethod:
+            self._ensure_real()
+            return self._real_repository.pack(hint=hint,
+                clean_obsolete_packs=clean_obsolete_packs)
+        handler.cancel_read_body()
+        if response != ('ok', ):
+            raise errors.UnexpectedSmartServerResponse(response)
 
     @property
     def revisions(self):
         """Decorate the real repository for now.
 
-        In the short term this should become a real object to intercept graph
-        lookups.
-
         In the long term a full blown network facility is needed.
         """
         self._ensure_real()
@@ -1951,8 +2580,9 @@
 
     @needs_write_lock
     def sign_revision(self, revision_id, gpg_strategy):
-        self._ensure_real()
-        return self._real_repository.sign_revision(revision_id, gpg_strategy)
+        testament = _mod_testament.Testament.from_revision(self, revision_id)
+        plaintext = testament.as_short_text()
+        self.store_revision_signature(gpg_strategy, plaintext, revision_id)
 
     @property
     def texts(self):
@@ -1964,10 +2594,69 @@
         self._ensure_real()
         return self._real_repository.texts
 
+    def _iter_revisions_rpc(self, revision_ids):
+        body = "\n".join(revision_ids)
+        path = self.bzrdir._path_for_remote_call(self._client)
+        response_tuple, response_handler = (
+            self._call_with_body_bytes_expecting_body(
+            "Repository.iter_revisions", (path, ), body))
+        if response_tuple[0] != "ok":
+            raise errors.UnexpectedSmartServerResponse(response_tuple)
+        serializer_format = response_tuple[1]
+        serializer = serializer_format_registry.get(serializer_format)
+        byte_stream = response_handler.read_streamed_body()
+        decompressor = zlib.decompressobj()
+        chunks = []
+        for bytes in byte_stream:
+            chunks.append(decompressor.decompress(bytes))
+            if decompressor.unused_data != "":
+                chunks.append(decompressor.flush())
+                yield serializer.read_revision_from_string("".join(chunks))
+                unused = decompressor.unused_data
+                decompressor = zlib.decompressobj()
+                chunks = [decompressor.decompress(unused)]
+        chunks.append(decompressor.flush())
+        text = "".join(chunks)
+        if text != "":
+            yield serializer.read_revision_from_string("".join(chunks))
+
     @needs_read_lock
     def get_revisions(self, revision_ids):
-        self._ensure_real()
-        return self._real_repository.get_revisions(revision_ids)
+        if revision_ids is None:
+            revision_ids = self.all_revision_ids()
+        else:
+            for rev_id in revision_ids:
+                if not rev_id or not isinstance(rev_id, basestring):
+                    raise errors.InvalidRevisionId(
+                        revision_id=rev_id, branch=self)
+        try:
+            missing = set(revision_ids)
+            revs = {}
+            for rev in self._iter_revisions_rpc(revision_ids):
+                missing.remove(rev.revision_id)
+                revs[rev.revision_id] = rev
+        except errors.UnknownSmartMethod:
+            self._ensure_real()
+            return self._real_repository.get_revisions(revision_ids)
+        for fallback in self._fallback_repositories:
+            if not missing:
+                break
+            for revid in list(missing):
+                # XXX JRV 2011-11-20: It would be nice if there was a
+                # public method on Repository that could be used to query
+                # for revision objects *without* failing completely if one
+                # was missing. There is VersionedFileRepository._iter_revisions,
+                # but unfortunately that's private and not provided by
+                # all repository implementations.
+                try:
+                    revs[revid] = fallback.get_revision(revid)
+                except errors.NoSuchRevision:
+                    pass
+                else:
+                    missing.remove(revid)
+        if missing:
+            raise errors.NoSuchRevision(self, list(missing)[0])
+        return [revs[revid] for revid in revision_ids]
 
     def supports_rich_root(self):
         return self._format.rich_root_data
@@ -1981,18 +2670,56 @@
     def _serializer(self):
         return self._format._serializer
 
+    @needs_write_lock
     def store_revision_signature(self, gpg_strategy, plaintext, revision_id):
-        self._ensure_real()
-        return self._real_repository.store_revision_signature(
-            gpg_strategy, plaintext, revision_id)
+        signature = gpg_strategy.sign(plaintext)
+        self.add_signature_text(revision_id, signature)
 
     def add_signature_text(self, revision_id, signature):
-        self._ensure_real()
-        return self._real_repository.add_signature_text(revision_id, signature)
+        if self._real_repository:
+            # If there is a real repository the write group will
+            # be in the real repository as well, so use that:
+            self._ensure_real()
+            return self._real_repository.add_signature_text(
+                revision_id, signature)
+        path = self.bzrdir._path_for_remote_call(self._client)
+        response, handler = self._call_with_body_bytes_expecting_body(
+            'Repository.add_signature_text', (path, self._lock_token,
+                revision_id) + tuple(self._write_group_tokens), signature)
+        handler.cancel_read_body()
+        self.refresh_data()
+        if response[0] != 'ok':
+            raise errors.UnexpectedSmartServerResponse(response)
+        self._write_group_tokens = response[1:]
 
     def has_signature_for_revision_id(self, revision_id):
-        self._ensure_real()
-        return self._real_repository.has_signature_for_revision_id(revision_id)
+        path = self.bzrdir._path_for_remote_call(self._client)
+        try:
+            response = self._call('Repository.has_signature_for_revision_id',
+                path, revision_id)
+        except errors.UnknownSmartMethod:
+            self._ensure_real()
+            return self._real_repository.has_signature_for_revision_id(
+                revision_id)
+        if response[0] not in ('yes', 'no'):
+            raise SmartProtocolError('unexpected response code %s' % (response,))
+        if response[0] == 'yes':
+            return True
+        for fallback in self._fallback_repositories:
+            if fallback.has_signature_for_revision_id(revision_id):
+                return True
+        return False
+
+    @needs_read_lock
+    def verify_revision_signature(self, revision_id, gpg_strategy):
+        if not self.has_signature_for_revision_id(revision_id):
+            return gpg.SIGNATURE_NOT_SIGNED, None
+        signature = self.get_signature_text(revision_id)
+
+        testament = _mod_testament.Testament.from_revision(self, revision_id)
+        plaintext = testament.as_short_text()
+
+        return gpg_strategy.verify(signature, plaintext)
 
     def item_keys_introduced_by(self, revision_ids, _files_pb=None):
         self._ensure_real()
@@ -2188,9 +2915,9 @@
 
     def _real_stream(self, repo, search):
         """Get a stream for search from repo.
-        
-        This never called RemoteStreamSource.get_stream, and is a heler
-        for RemoteStreamSource._get_stream to allow getting a stream 
+
+        This never called RemoteStreamSource.get_stream, and is a helper
+        for RemoteStreamSource._get_stream to allow getting a stream
         reliably whether fallback back because of old servers or trying
         to stream from a non-RemoteRepository (which the stacked support
         code will do).
@@ -2238,7 +2965,7 @@
             except errors.UnknownSmartMethod:
                 medium._remember_remote_is_before(version)
             except errors.UnknownErrorFromSmartServer, e:
-                if isinstance(search, graph.EverythingResult):
+                if isinstance(search, vf_search.EverythingResult):
                     error_verb = e.error_from_smart_server.error_verb
                     if error_verb == 'BadSearch':
                         # Pre-2.4 servers don't support this sort of search.
@@ -2336,8 +3063,12 @@
 
     def _ensure_real(self):
         if self._custom_format is None:
-            self._custom_format = branch.network_format_registry.get(
-                self._network_name)
+            try:
+                self._custom_format = branch.network_format_registry.get(
+                    self._network_name)
+            except KeyError:
+                raise errors.UnknownFormatError(kind='branch',
+                    format=self._network_name)
 
     def get_format_description(self):
         self._ensure_real()
@@ -2350,7 +3081,7 @@
         return a_bzrdir.open_branch(name=name, 
             ignore_fallbacks=ignore_fallbacks)
 
-    def _vfs_initialize(self, a_bzrdir, name):
+    def _vfs_initialize(self, a_bzrdir, name, append_revisions_only):
         # Initialisation when using a local bzrdir object, or a non-vfs init
         # method is not available on the server.
         # self._custom_format is always set - the start of initialize ensures
@@ -2358,17 +3089,19 @@
         if isinstance(a_bzrdir, RemoteBzrDir):
             a_bzrdir._ensure_real()
             result = self._custom_format.initialize(a_bzrdir._real_bzrdir,
-                name)
+                name, append_revisions_only=append_revisions_only)
         else:
             # We assume the bzrdir is parameterised; it may not be.
-            result = self._custom_format.initialize(a_bzrdir, name)
+            result = self._custom_format.initialize(a_bzrdir, name,
+                append_revisions_only=append_revisions_only)
         if (isinstance(a_bzrdir, RemoteBzrDir) and
             not isinstance(result, RemoteBranch)):
             result = RemoteBranch(a_bzrdir, a_bzrdir.find_repository(), result,
                                   name=name)
         return result
 
-    def initialize(self, a_bzrdir, name=None, repository=None):
+    def initialize(self, a_bzrdir, name=None, repository=None,
+                   append_revisions_only=None):
         # 1) get the network name to use.
         if self._custom_format:
             network_name = self._custom_format.network_name()
@@ -2380,10 +3113,12 @@
             network_name = reference_format.network_name()
         # Being asked to create on a non RemoteBzrDir:
         if not isinstance(a_bzrdir, RemoteBzrDir):
-            return self._vfs_initialize(a_bzrdir, name=name)
+            return self._vfs_initialize(a_bzrdir, name=name,
+                append_revisions_only=append_revisions_only)
         medium = a_bzrdir._client._medium
         if medium._is_remote_before((1, 13)):
-            return self._vfs_initialize(a_bzrdir, name=name)
+            return self._vfs_initialize(a_bzrdir, name=name,
+                append_revisions_only=append_revisions_only)
         # Creating on a remote bzr dir.
         # 2) try direct creation via RPC
         path = a_bzrdir._path_for_remote_call(a_bzrdir._client)
@@ -2396,7 +3131,8 @@
         except errors.UnknownSmartMethod:
             # Fallback - use vfs methods
             medium._remember_remote_is_before((1, 13))
-            return self._vfs_initialize(a_bzrdir, name=name)
+            return self._vfs_initialize(a_bzrdir, name=name,
+                    append_revisions_only=append_revisions_only)
         if response[0] != 'ok':
             raise errors.UnexpectedSmartServerResponse(response)
         # Turn the response into a RemoteRepository object.
@@ -2423,6 +3159,8 @@
             remote_repo = RemoteRepository(repo_bzrdir, repo_format)
         remote_branch = RemoteBranch(a_bzrdir, remote_repo,
             format=format, setup_stacking=False, name=name)
+        if append_revisions_only:
+            remote_branch.set_append_revisions_only(append_revisions_only)
         # XXX: We know this is a new branch, so it must have revno 0, revid
         # NULL_REVISION. Creating the branch locked would make this be unable
         # to be wrong; here its simply very unlikely to be wrong. RBC 20090225
@@ -2461,6 +3199,68 @@
                 return True
         return False
 
+
+class RemoteBranchStore(_mod_config.IniFileStore):
+    """Branch store which attempts to use HPSS calls to retrieve branch store.
+
+    Note that this is specific to bzr-based formats.
+    """
+
+    def __init__(self, branch):
+        super(RemoteBranchStore, self).__init__()
+        self.branch = branch
+        self.id = "branch"
+        self._real_store = None
+
+    def lock_write(self, token=None):
+        return self.branch.lock_write(token)
+
+    def unlock(self):
+        return self.branch.unlock()
+
+    @needs_write_lock
+    def save(self):
+        # We need to be able to override the undecorated implementation
+        self.save_without_locking()
+
+    def save_without_locking(self):
+        super(RemoteBranchStore, self).save()
+
+    def external_url(self):
+        return self.branch.user_url
+
+    def _load_content(self):
+        path = self.branch._remote_path()
+        try:
+            response, handler = self.branch._call_expecting_body(
+                'Branch.get_config_file', path)
+        except errors.UnknownSmartMethod:
+            self._ensure_real()
+            return self._real_store._load_content()
+        if len(response) and response[0] != 'ok':
+            raise errors.UnexpectedSmartServerResponse(response)
+        return handler.read_body_bytes()
+
+    def _save_content(self, content):
+        path = self.branch._remote_path()
+        try:
+            response, handler = self.branch._call_with_body_bytes_expecting_body(
+                'Branch.put_config_file', (path,
+                    self.branch._lock_token, self.branch._repo_lock_token),
+                content)
+        except errors.UnknownSmartMethod:
+            self._ensure_real()
+            return self._real_store._save_content(content)
+        handler.cancel_read_body()
+        if response != ('ok', ):
+            raise errors.UnexpectedSmartServerResponse(response)
+
+    def _ensure_real(self):
+        self.branch._ensure_real()
+        if self._real_store is None:
+            self._real_store = _mod_config.BranchStore(self.branch)
+
+
 class RemoteBranch(branch.Branch, _RpcHelper, lock._RelockDebugMixin):
     """Branch stored on a server accessed by HPSS RPC.
 
@@ -2468,7 +3268,8 @@
     """
 
     def __init__(self, remote_bzrdir, remote_repository, real_branch=None,
-        _client=None, format=None, setup_stacking=True, name=None):
+        _client=None, format=None, setup_stacking=True, name=None,
+        possible_transports=None):
         """Create a RemoteBranch instance.
 
         :param real_branch: An optional local implementation of the branch
@@ -2539,9 +3340,9 @@
             hook(self)
         self._is_stacked = False
         if setup_stacking:
-            self._setup_stacking()
+            self._setup_stacking(possible_transports)
 
-    def _setup_stacking(self):
+    def _setup_stacking(self, possible_transports):
         # configure stacking into the remote repository, by reading it from
         # the vfs branch.
         try:
@@ -2550,11 +3351,20 @@
             errors.UnstackableRepositoryFormat), e:
             return
         self._is_stacked = True
-        self._activate_fallback_location(fallback_url)
+        if possible_transports is None:
+            possible_transports = []
+        else:
+            possible_transports = list(possible_transports)
+        possible_transports.append(self.bzrdir.root_transport)
+        self._activate_fallback_location(fallback_url,
+            possible_transports=possible_transports)
 
     def _get_config(self):
         return RemoteBranchConfig(self)
 
+    def _get_config_store(self):
+        return RemoteBranchStore(self)
+
     def _get_real_transport(self):
         # if we try vfs access, return the real branch's vfs transport
         self._ensure_real()
@@ -2623,15 +3433,17 @@
                 self.bzrdir, self._client)
         return self._control_files
 
-    def _get_checkout_format(self):
-        self._ensure_real()
-        return self._real_branch._get_checkout_format()
-
     def get_physical_lock_status(self):
         """See Branch.get_physical_lock_status()."""
-        # should be an API call to the server, as branches must be lockable.
-        self._ensure_real()
-        return self._real_branch.get_physical_lock_status()
+        try:
+            response = self._client.call('Branch.get_physical_lock_status',
+                self._remote_path())
+        except errors.UnknownSmartMethod:
+            self._ensure_real()
+            return self._real_branch.get_physical_lock_status()
+        if response[0] not in ('yes', 'no'):
+            raise errors.UnexpectedSmartServerResponse(response)
+        return (response[0] == 'yes')
 
     def get_stacked_on_url(self):
         """Get the URL this branch is stacked against.
@@ -2821,8 +3633,14 @@
             self.repository.unlock()
 
     def break_lock(self):
-        self._ensure_real()
-        return self._real_branch.break_lock()
+        try:
+            response = self._call(
+                'Branch.break_lock', self._remote_path())
+        except errors.UnknownSmartMethod:
+            self._ensure_real()
+            return self._real_branch.break_lock()
+        if response != ('ok',):
+            raise errors.UnexpectedSmartServerResponse(response)
 
     def leave_lock_in_place(self):
         if not self._lock_token:
@@ -3002,9 +3820,40 @@
         return self._lock_count >= 1
 
     @needs_read_lock
+    def revision_id_to_dotted_revno(self, revision_id):
+        """Given a revision id, return its dotted revno.
+
+        :return: a tuple like (1,) or (400,1,3).
+        """
+        try:
+            response = self._call('Branch.revision_id_to_revno',
+                self._remote_path(), revision_id)
+        except errors.UnknownSmartMethod:
+            self._ensure_real()
+            return self._real_branch.revision_id_to_dotted_revno(revision_id)
+        if response[0] == 'ok':
+            return tuple([int(x) for x in response[1:]])
+        else:
+            raise errors.UnexpectedSmartServerResponse(response)
+
+    @needs_read_lock
     def revision_id_to_revno(self, revision_id):
-        self._ensure_real()
-        return self._real_branch.revision_id_to_revno(revision_id)
+        """Given a revision id on the branch mainline, return its revno.
+
+        :return: an integer
+        """
+        try:
+            response = self._call('Branch.revision_id_to_revno',
+                self._remote_path(), revision_id)
+        except errors.UnknownSmartMethod:
+            self._ensure_real()
+            return self._real_branch.revision_id_to_revno(revision_id)
+        if response[0] == 'ok':
+            if len(response) == 2:
+                return int(response[1])
+            raise NoSuchRevision(self, revision_id)
+        else:
+            raise errors.UnexpectedSmartServerResponse(response)
 
     @needs_write_lock
     def set_last_revision_info(self, revno, revision_id):
@@ -3114,7 +3963,7 @@
                 value = section_obj.get(name, default)
         except errors.UnknownSmartMethod:
             value = self._vfs_get_option(name, section, default)
-        for hook in config.OldConfigHooks['get']:
+        for hook in _mod_config.OldConfigHooks['get']:
             hook(self, name, value)
         return value
 
@@ -3122,8 +3971,8 @@
         if len(response[0]) and response[0][0] != 'ok':
             raise errors.UnexpectedSmartServerResponse(response)
         lines = response[1].read_body_bytes().splitlines()
-        conf = config.ConfigObj(lines, encoding='utf-8')
-        for hook in config.OldConfigHooks['load']:
+        conf = _mod_config.ConfigObj(lines, encoding='utf-8')
+        for hook in _mod_config.OldConfigHooks['load']:
             hook(self)
         return conf
 
@@ -3239,7 +4088,6 @@
         return self._bzrdir._real_bzrdir
 
 
-
 def _extract_tar(tar, to_dir):
     """Extract all the contents of a tarfile object.
 
@@ -3249,6 +4097,10 @@
         tar.extract(tarinfo, to_dir)
 
 
+error_translators = registry.Registry()
+no_context_error_translators = registry.Registry()
+
+
 def _translate_error(err, **context):
     """Translate an ErrorFromSmartServer into a more useful error.
 
@@ -3283,67 +4135,101 @@
                     'Missing key %r in context %r', key_err.args[0], context)
                 raise err
 
-    if err.error_verb == 'NoSuchRevision':
-        raise NoSuchRevision(find('branch'), err.error_args[0])
-    elif err.error_verb == 'nosuchrevision':
-        raise NoSuchRevision(find('repository'), err.error_args[0])
-    elif err.error_verb == 'nobranch':
-        if len(err.error_args) >= 1:
-            extra = err.error_args[0]
-        else:
-            extra = None
-        raise errors.NotBranchError(path=find('bzrdir').root_transport.base,
-            detail=extra)
-    elif err.error_verb == 'norepository':
-        raise errors.NoRepositoryPresent(find('bzrdir'))
-    elif err.error_verb == 'UnlockableTransport':
-        raise errors.UnlockableTransport(find('bzrdir').root_transport)
-    elif err.error_verb == 'TokenMismatch':
-        raise errors.TokenMismatch(find('token'), '(remote token)')
-    elif err.error_verb == 'Diverged':
-        raise errors.DivergedBranches(find('branch'), find('other_branch'))
-    elif err.error_verb == 'NotStacked':
-        raise errors.NotStacked(branch=find('branch'))
-    elif err.error_verb == 'PermissionDenied':
-        path = get_path()
-        if len(err.error_args) >= 2:
-            extra = err.error_args[1]
-        else:
-            extra = None
-        raise errors.PermissionDenied(path, extra=extra)
-    elif err.error_verb == 'ReadError':
-        path = get_path()
-        raise errors.ReadError(path)
-    elif err.error_verb == 'NoSuchFile':
-        path = get_path()
-        raise errors.NoSuchFile(path)
-    _translate_error_without_context(err)
-
-
-def _translate_error_without_context(err):
-    """Translate any ErrorFromSmartServer values that don't require context"""
-    if err.error_verb == 'IncompatibleRepositories':
-        raise errors.IncompatibleRepositories(err.error_args[0],
-            err.error_args[1], err.error_args[2])
-    elif err.error_verb == 'LockContention':
-        raise errors.LockContention('(remote lock)')
-    elif err.error_verb == 'LockFailed':
-        raise errors.LockFailed(err.error_args[0], err.error_args[1])
-    elif err.error_verb == 'TipChangeRejected':
-        raise errors.TipChangeRejected(err.error_args[0].decode('utf8'))
-    elif err.error_verb == 'UnstackableBranchFormat':
-        raise errors.UnstackableBranchFormat(*err.error_args)
-    elif err.error_verb == 'UnstackableRepositoryFormat':
-        raise errors.UnstackableRepositoryFormat(*err.error_args)
-    elif err.error_verb == 'FileExists':
-        raise errors.FileExists(err.error_args[0])
-    elif err.error_verb == 'DirectoryNotEmpty':
-        raise errors.DirectoryNotEmpty(err.error_args[0])
-    elif err.error_verb == 'ShortReadvError':
-        args = err.error_args
-        raise errors.ShortReadvError(
-            args[0], int(args[1]), int(args[2]), int(args[3]))
-    elif err.error_verb in ('UnicodeEncodeError', 'UnicodeDecodeError'):
+    try:
+        translator = error_translators.get(err.error_verb)
+    except KeyError:
+        pass
+    else:
+        raise translator(err, find, get_path)
+    try:
+        translator = no_context_error_translators.get(err.error_verb)
+    except KeyError:
+        raise errors.UnknownErrorFromSmartServer(err)
+    else:
+        raise translator(err)
+
+
+error_translators.register('NoSuchRevision',
+    lambda err, find, get_path: NoSuchRevision(
+        find('branch'), err.error_args[0]))
+error_translators.register('nosuchrevision',
+    lambda err, find, get_path: NoSuchRevision(
+        find('repository'), err.error_args[0]))
+
+def _translate_nobranch_error(err, find, get_path):
+    if len(err.error_args) >= 1:
+        extra = err.error_args[0]
+    else:
+        extra = None
+    return errors.NotBranchError(path=find('bzrdir').root_transport.base,
+        detail=extra)
+
+error_translators.register('nobranch', _translate_nobranch_error)
+error_translators.register('norepository',
+    lambda err, find, get_path: errors.NoRepositoryPresent(
+        find('bzrdir')))
+error_translators.register('UnlockableTransport',
+    lambda err, find, get_path: errors.UnlockableTransport(
+        find('bzrdir').root_transport))
+error_translators.register('TokenMismatch',
+    lambda err, find, get_path: errors.TokenMismatch(
+        find('token'), '(remote token)'))
+error_translators.register('Diverged',
+    lambda err, find, get_path: errors.DivergedBranches(
+        find('branch'), find('other_branch')))
+error_translators.register('NotStacked',
+    lambda err, find, get_path: errors.NotStacked(branch=find('branch')))
+
+def _translate_PermissionDenied(err, find, get_path):
+    path = get_path()
+    if len(err.error_args) >= 2:
+        extra = err.error_args[1]
+    else:
+        extra = None
+    return errors.PermissionDenied(path, extra=extra)
+
+error_translators.register('PermissionDenied', _translate_PermissionDenied)
+error_translators.register('ReadError',
+    lambda err, find, get_path: errors.ReadError(get_path()))
+error_translators.register('NoSuchFile',
+    lambda err, find, get_path: errors.NoSuchFile(get_path()))
+error_translators.register('TokenLockingNotSupported',
+    lambda err, find, get_path: errors.TokenLockingNotSupported(
+        find('repository')))
+error_translators.register('UnsuspendableWriteGroup',
+    lambda err, find, get_path: errors.UnsuspendableWriteGroup(
+        repository=find('repository')))
+error_translators.register('UnresumableWriteGroup',
+    lambda err, find, get_path: errors.UnresumableWriteGroup(
+        repository=find('repository'), write_groups=err.error_args[0],
+        reason=err.error_args[1]))
+no_context_error_translators.register('IncompatibleRepositories',
+    lambda err: errors.IncompatibleRepositories(
+        err.error_args[0], err.error_args[1], err.error_args[2]))
+no_context_error_translators.register('LockContention',
+    lambda err: errors.LockContention('(remote lock)'))
+no_context_error_translators.register('LockFailed',
+    lambda err: errors.LockFailed(err.error_args[0], err.error_args[1]))
+no_context_error_translators.register('TipChangeRejected',
+    lambda err: errors.TipChangeRejected(err.error_args[0].decode('utf8')))
+no_context_error_translators.register('UnstackableBranchFormat',
+    lambda err: errors.UnstackableBranchFormat(*err.error_args))
+no_context_error_translators.register('UnstackableRepositoryFormat',
+    lambda err: errors.UnstackableRepositoryFormat(*err.error_args))
+no_context_error_translators.register('FileExists',
+    lambda err: errors.FileExists(err.error_args[0]))
+no_context_error_translators.register('DirectoryNotEmpty',
+    lambda err: errors.DirectoryNotEmpty(err.error_args[0]))
+
+def _translate_short_readv_error(err):
+    args = err.error_args
+    return errors.ShortReadvError(args[0], int(args[1]), int(args[2]),
+        int(args[3]))
+
+no_context_error_translators.register('ShortReadvError',
+    _translate_short_readv_error)
+
+def _translate_unicode_error(err):
         encoding = str(err.error_args[0]) # encoding must always be a string
         val = err.error_args[1]
         start = int(err.error_args[2])
@@ -3357,9 +4243,19 @@
             raise UnicodeDecodeError(encoding, val, start, end, reason)
         elif err.error_verb == 'UnicodeEncodeError':
             raise UnicodeEncodeError(encoding, val, start, end, reason)
-    elif err.error_verb == 'ReadOnlyError':
-        raise errors.TransportNotPossible('readonly transport')
-    elif err.error_verb == 'MemoryError':
-        raise errors.BzrError("remote server out of memory\n"
-            "Retry non-remotely, or contact the server admin for details.")
-    raise errors.UnknownErrorFromSmartServer(err)
+
+no_context_error_translators.register('UnicodeEncodeError',
+    _translate_unicode_error)
+no_context_error_translators.register('UnicodeDecodeError',
+    _translate_unicode_error)
+no_context_error_translators.register('ReadOnlyError',
+    lambda err: errors.TransportNotPossible('readonly transport'))
+no_context_error_translators.register('MemoryError',
+    lambda err: errors.BzrError("remote server out of memory\n"
+        "Retry non-remotely, or contact the server admin for details."))
+no_context_error_translators.register('RevisionNotPresent',
+    lambda err: errors.RevisionNotPresent(err.error_args[0], err.error_args[1]))
+
+no_context_error_translators.register('BzrCheckError',
+    lambda err: errors.BzrCheckError(msg=err.error_args[0]))
+

=== modified file 'bzrlib/rename_map.py'
--- bzrlib/rename_map.py	2009-03-24 15:47:32 +0000
+++ bzrlib/rename_map.py	2011-12-18 15:28:38 +0000
@@ -14,6 +14,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
 
 from cStringIO import StringIO
 
@@ -23,7 +24,7 @@
     trace,
 )
 from bzrlib.ui import ui_factory
-
+from bzrlib.i18n import gettext
 
 class RenameMap(object):
     """Determine a mapping of renames."""
@@ -65,7 +66,7 @@
         try:
             for num, (file_id, contents) in enumerate(
                 tree.iter_files_bytes(desired_files)):
-                task.update('Calculating hashes', num, len(file_ids))
+                task.update(gettext('Calculating hashes'), num, len(file_ids))
                 s = StringIO()
                 s.writelines(contents)
                 s.seek(0)
@@ -103,7 +104,7 @@
         task = ui_factory.nested_progress_bar()
         try:
             for num, path in enumerate(paths):
-                task.update('Determining hash hits', num, len(paths))
+                task.update(gettext('Determining hash hits'), num, len(paths))
                 hits = self.hitcounts(self.tree.get_file_lines(None,
                                                                path=path))
                 all_hits.extend((v, path, k) for k, v in hits.items())
@@ -238,7 +239,7 @@
             pp.next_phase()
             delta = rn._make_inventory_delta(matches)
             for old, new, file_id, entry in delta:
-                trace.note("%s => %s", old, new)
+                trace.note( gettext("{0} => {1}").format(old, new) )
             if not dry_run:
                 tree.add(required_parents)
                 tree.apply_inventory_delta(delta)

=== modified file 'bzrlib/repofmt/__init__.py'
--- bzrlib/repofmt/__init__.py	2009-03-23 14:59:43 +0000
+++ bzrlib/repofmt/__init__.py	2011-12-19 13:23:58 +0000
@@ -15,3 +15,6 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 """Repository formats"""
+
+from __future__ import absolute_import
+

=== modified file 'bzrlib/repofmt/groupcompress_repo.py'
--- bzrlib/repofmt/groupcompress_repo.py	2011-05-09 12:01:12 +0000
+++ bzrlib/repofmt/groupcompress_repo.py	2011-12-19 19:15:58 +0000
@@ -16,6 +16,8 @@
 
 """Repository formats using CHK inventories and groupcompress compression."""
 
+from __future__ import absolute_import
+
 import time
 
 from bzrlib import (
@@ -978,20 +980,24 @@
             if record.storage_kind != 'absent':
                 texts[record.key] = record.get_bytes_as('fulltext')
             else:
-                raise errors.NoSuchRevision(self, record.key)
+                texts[record.key] = None
         for key in keys:
-            yield inventory.CHKInventory.deserialise(self.chk_bytes, texts[key], key)
+            bytes = texts[key]
+            if bytes is None:
+                yield (None, key[-1])
+            else:
+                yield (inventory.CHKInventory.deserialise(
+                    self.chk_bytes, bytes, key), key[-1])
 
-    def _iter_inventory_xmls(self, revision_ids, ordering):
+    def _get_inventory_xml(self, revision_id):
+        """Get serialized inventory as a string."""
         # Without a native 'xml' inventory, this method doesn't make sense.
         # However older working trees, and older bundles want it - so we supply
         # it allowing _get_inventory_xml to work. Bundles currently use the
         # serializer directly; this also isn't ideal, but there isn't an xml
-        # iteration interface offered at all for repositories. We could make
-        # _iter_inventory_xmls be part of the contract, even if kept private.
-        inv_to_str = self._serializer.write_inventory_to_string
-        for inv in self.iter_inventories(revision_ids, ordering=ordering):
-            yield inv_to_str(inv), inv.revision_id
+        # iteration interface offered at all for repositories.
+        return self._serializer.write_inventory_to_string(
+            self.get_inventory(revision_id))
 
     def _find_present_inventory_keys(self, revision_keys):
         parent_map = self.inventories.get_parent_map(revision_keys)
@@ -1384,7 +1390,8 @@
 
     _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         return ('Bazaar repository format 2a (needs bzr 1.16 or later)\n')
 
     def get_format_description(self):
@@ -1406,7 +1413,8 @@
 
     _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         return ('Bazaar development format 8\n')
 
     def get_format_description(self):

=== modified file 'bzrlib/repofmt/knitpack_repo.py'
--- bzrlib/repofmt/knitpack_repo.py	2011-05-02 15:29:58 +0000
+++ bzrlib/repofmt/knitpack_repo.py	2011-12-19 19:15:58 +0000
@@ -16,6 +16,8 @@
 
 """Knit-based pack repository formats."""
 
+from __future__ import absolute_import
+
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 from itertools import izip
@@ -160,7 +162,8 @@
 
     _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See RepositoryFormat.get_format_string()."""
         return "Bazaar pack repository format 1 (needs bzr 0.92)\n"
 
@@ -200,7 +203,8 @@
 
     _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See RepositoryFormat.get_format_string()."""
         return "Bazaar pack repository format 1 with subtree support (needs bzr 0.92)\n"
 
@@ -238,7 +242,8 @@
 
     _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See RepositoryFormat.get_format_string()."""
         return ("Bazaar pack repository format 1 with rich root"
                 " (needs bzr 1.0)\n")
@@ -276,7 +281,8 @@
 
     _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See RepositoryFormat.get_format_string()."""
         return "Bazaar RepositoryFormatKnitPack5 (bzr 1.6)\n"
 
@@ -316,7 +322,8 @@
 
     _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See RepositoryFormat.get_format_string()."""
         return "Bazaar RepositoryFormatKnitPack5RichRoot (bzr 1.6.1)\n"
 
@@ -362,7 +369,8 @@
 
     _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See RepositoryFormat.get_format_string()."""
         return "Bazaar RepositoryFormatKnitPack5RichRoot (bzr 1.6)\n"
 
@@ -400,7 +408,8 @@
 
     _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See RepositoryFormat.get_format_string()."""
         return "Bazaar RepositoryFormatKnitPack6 (bzr 1.9)\n"
 
@@ -437,7 +446,8 @@
 
     _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See RepositoryFormat.get_format_string()."""
         return "Bazaar RepositoryFormatKnitPack6RichRoot (bzr 1.9)\n"
 
@@ -477,7 +487,8 @@
 
     _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See RepositoryFormat.get_format_string()."""
         return ("Bazaar development format 2 with subtree support "
             "(needs bzr.dev from before 1.8)\n")

=== modified file 'bzrlib/repofmt/knitrepo.py'
--- bzrlib/repofmt/knitrepo.py	2011-06-16 14:54:47 +0000
+++ bzrlib/repofmt/knitrepo.py	2011-12-19 19:15:58 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 import itertools
@@ -38,7 +40,7 @@
 from bzrlib.repository import (
     InterRepository,
     IsInWriteGroupError,
-    RepositoryFormat,
+    RepositoryFormatMetaDir,
     )
 from bzrlib.vf_repository import (
     InterSameDataRepository,
@@ -321,7 +323,7 @@
                                     than normal. I.e. during 'upgrade'.
         """
         if not _found:
-            format = RepositoryFormat.find_format(a_bzrdir)
+            format = RepositoryFormatMetaDir.find_format(a_bzrdir)
         if _override_transport is not None:
             repo_transport = _override_transport
         else:
@@ -367,7 +369,8 @@
     def __ne__(self, other):
         return self.__class__ is not other.__class__
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See RepositoryFormat.get_format_string()."""
         return "Bazaar-NG Knit Repository Format 1"
 
@@ -409,7 +412,8 @@
 
     _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See RepositoryFormat.get_format_string()."""
         return "Bazaar Knit Repository Format 3 (bzr 0.15)\n"
 
@@ -450,7 +454,8 @@
 
     _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See RepositoryFormat.get_format_string()."""
         return 'Bazaar Knit Repository Format 4 (bzr 1.0)\n'
 

=== modified file 'bzrlib/repofmt/pack_repo.py'
--- bzrlib/repofmt/pack_repo.py	2011-07-25 07:51:35 +0000
+++ bzrlib/repofmt/pack_repo.py	2011-12-19 19:15:58 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 import re
 import sys
 
@@ -55,7 +57,7 @@
 from bzrlib.repository import (
     _LazyListJoin,
     MetaDirRepository,
-    RepositoryFormat,
+    RepositoryFormatMetaDir,
     RepositoryWriteLockResult,
     )
 from bzrlib.vf_repository import (
@@ -1221,8 +1223,18 @@
         """
         for pack in packs:
             try:
-                pack.pack_transport.move(pack.file_name(),
-                    '../obsolete_packs/' + pack.file_name())
+                try:
+                    pack.pack_transport.move(pack.file_name(),
+                        '../obsolete_packs/' + pack.file_name())
+                except errors.NoSuchFile:
+                    # perhaps obsolete_packs was removed? Let's create it and
+                    # try again
+                    try:
+                        pack.pack_transport.mkdir('../obsolete_packs/')
+                    except errors.FileExists:
+                        pass
+                    pack.pack_transport.move(pack.file_name(),
+                        '../obsolete_packs/' + pack.file_name())
             except (errors.PathError, errors.TransportError), e:
                 # TODO: Should these be warnings or mutters?
                 mutter("couldn't rename obsolete pack, skipping it:\n%s"
@@ -1494,7 +1506,11 @@
         obsolete_pack_transport = self.transport.clone('obsolete_packs')
         if preserve is None:
             preserve = set()
-        for filename in obsolete_pack_transport.list_dir('.'):
+        try:
+            obsolete_pack_files = obsolete_pack_transport.list_dir('.')
+        except errors.NoSuchFile:
+            return found
+        for filename in obsolete_pack_files:
             name, ext = osutils.splitext(filename)
             if ext == '.pack':
                 found.append(name)
@@ -1906,7 +1922,7 @@
                                     than normal. I.e. during 'upgrade'.
         """
         if not _found:
-            format = RepositoryFormat.find_format(a_bzrdir)
+            format = RepositoryFormatMetaDir.find_format(a_bzrdir)
         if _override_transport is not None:
             repo_transport = _override_transport
         else:

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2011-08-21 09:24:01 +0000
+++ bzrlib/repository.py	2011-12-30 00:15:51 +0000
@@ -14,13 +14,14 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 import itertools
 import time
 
 from bzrlib import (
-    bzrdir,
     config,
     controldir,
     debug,
@@ -35,9 +36,11 @@
     gpg,
     )
 from bzrlib.bundle import serializer
+from bzrlib.i18n import gettext
 """)
 
 from bzrlib import (
+    bzrdir,
     errors,
     registry,
     symbol_versioning,
@@ -73,8 +76,11 @@
     record_root_entry = True
     # whether this commit builder supports the record_entry_contents interface
     supports_record_entry_contents = False
+    # whether this commit builder will automatically update the branch that is
+    # being committed to
+    updates_branch = False
 
-    def __init__(self, repository, parents, config, timestamp=None,
+    def __init__(self, repository, parents, config_stack, timestamp=None,
                  timezone=None, committer=None, revprops=None,
                  revision_id=None, lossy=False):
         """Initiate a CommitBuilder.
@@ -89,11 +95,11 @@
         :param lossy: Whether to discard data that can not be natively
             represented, when pushing to a foreign VCS 
         """
-        self._config = config
+        self._config_stack = config_stack
         self._lossy = lossy
 
         if committer is None:
-            self._committer = self._config.username()
+            self._committer = self._config_stack.get('email')
         elif not isinstance(committer, unicode):
             self._committer = committer.decode() # throw if non-ascii
         else:
@@ -280,7 +286,7 @@
                 raise
             mutter('abort_write_group failed')
             log_exception_quietly()
-            note('bzr: ERROR (ignored): %s', exc)
+            note(gettext('bzr: ERROR (ignored): %s'), exc)
         self._write_group = None
 
     def _abort_write_group(self):
@@ -341,15 +347,15 @@
         self.control_files.break_lock()
 
     @staticmethod
-    def create(a_bzrdir):
-        """Construct the current default format repository in a_bzrdir."""
-        return RepositoryFormat.get_default_format().initialize(a_bzrdir)
+    def create(controldir):
+        """Construct the current default format repository in controldir."""
+        return RepositoryFormat.get_default_format().initialize(controldir)
 
-    def __init__(self, _format, a_bzrdir, control_files):
+    def __init__(self, _format, controldir, control_files):
         """instantiate a Repository.
 
         :param _format: The format of the repository on disk.
-        :param a_bzrdir: The BzrDir of the repository.
+        :param controldir: The ControlDir of the repository.
         :param control_files: Control files to use for locking, etc.
         """
         # In the future we will have a single api for all stores for
@@ -358,10 +364,8 @@
         super(Repository, self).__init__()
         self._format = _format
         # the following are part of the public API for Repository:
-        self.bzrdir = a_bzrdir
+        self.bzrdir = controldir
         self.control_files = control_files
-        self._transport = control_files._transport
-        self.base = self._transport.base
         # for tests
         self._write_group = None
         # Additional places to query for data.
@@ -405,7 +409,7 @@
         """
         if self.__class__ is not other.__class__:
             return False
-        return (self._transport.base == other._transport.base)
+        return (self.control_url == other.control_url)
 
     def is_in_write_group(self):
         """Return True if there is an open write group.
@@ -548,22 +552,22 @@
             def __init__(self):
                 self.first_call = True
 
-            def __call__(self, bzrdir):
-                # On the first call, the parameter is always the bzrdir
+            def __call__(self, controldir):
+                # On the first call, the parameter is always the controldir
                 # containing the current repo.
                 if not self.first_call:
                     try:
-                        repository = bzrdir.open_repository()
+                        repository = controldir.open_repository()
                     except errors.NoRepositoryPresent:
                         pass
                     else:
                         return False, ([], repository)
                 self.first_call = False
-                value = (bzrdir.list_branches(), None)
+                value = (controldir.list_branches(), None)
                 return True, value
 
         ret = []
-        for branches, repository in bzrdir.BzrDir.find_bzrdirs(
+        for branches, repository in controldir.ControlDir.find_bzrdirs(
                 self.user_transport, evaluate=Evaluator()):
             if branches is not None:
                 ret.extend(branches)
@@ -603,7 +607,7 @@
         For instance, if the repository is at URL/.bzr/repository,
         Repository.open(URL) -> a Repository instance.
         """
-        control = bzrdir.BzrDir.open(base)
+        control = controldir.ControlDir.open(base)
         return control.open_repository()
 
     def copy_content_into(self, destination, revision_id=None):
@@ -640,6 +644,12 @@
         """
 
     def suspend_write_group(self):
+        """Suspend a write group.
+
+        :raise UnsuspendableWriteGroup: If the write group can not be
+            suspended.
+        :return: List of tokens
+        """
         raise errors.UnsuspendableWriteGroup(self)
 
     def refresh_data(self):
@@ -667,12 +677,10 @@
     def _resume_write_group(self, tokens):
         raise errors.UnsuspendableWriteGroup(self)
 
-    def fetch(self, source, revision_id=None, find_ghosts=False,
-            fetch_spec=None):
+    def fetch(self, source, revision_id=None, find_ghosts=False):
         """Fetch the content required to construct revision_id from source.
 
-        If revision_id is None and fetch_spec is None, then all content is
-        copied.
+        If revision_id is None, then all content is copied.
 
         fetch() may not be used when the repository is in a write group -
         either finish the current write group before using fetch, or use
@@ -684,14 +692,7 @@
         :param revision_id: If specified, all the content needed for this
             revision ID will be copied to the target.  Fetch will determine for
             itself which content needs to be copied.
-        :param fetch_spec: If specified, a SearchResult or
-            PendingAncestryResult that describes which revisions to copy.  This
-            allows copying multiple heads at once.  Mutually exclusive with
-            revision_id.
         """
-        if fetch_spec is not None and revision_id is not None:
-            raise AssertionError(
-                "fetch_spec and revision_id are mutually exclusive.")
         if self.is_in_write_group():
             raise errors.InternalBzrError(
                 "May not fetch while in a write group.")
@@ -699,7 +700,6 @@
         # TODO: lift out to somewhere common with RemoteRepository
         # <https://bugs.launchpad.net/bzr/+bug/401646>
         if (self.has_same_location(source)
-            and fetch_spec is None
             and self._has_same_fallbacks(source)):
             # check that last_revision is in 'from' and then return a
             # no-operation.
@@ -708,20 +708,19 @@
                 self.get_revision(revision_id)
             return 0, []
         inter = InterRepository.get(source, self)
-        return inter.fetch(revision_id=revision_id,
-            find_ghosts=find_ghosts, fetch_spec=fetch_spec)
+        return inter.fetch(revision_id=revision_id, find_ghosts=find_ghosts)
 
     def create_bundle(self, target, base, fileobj, format=None):
         return serializer.write_bundle(self, target, base, fileobj, format)
 
-    def get_commit_builder(self, branch, parents, config, timestamp=None,
+    def get_commit_builder(self, branch, parents, config_stack, timestamp=None,
                            timezone=None, committer=None, revprops=None,
                            revision_id=None, lossy=False):
         """Obtain a CommitBuilder for this repository.
 
         :param branch: Branch to commit to.
         :param parents: Revision ids of the parents of the new revision.
-        :param config: Configuration to use.
+        :param config_stack: Configuration stack to use.
         :param timestamp: Optional timestamp recorded for commit.
         :param timezone: Optional timezone for timestamp.
         :param committer: Optional committer to set for commit.
@@ -747,8 +746,8 @@
                 repo.unlock()
 
     @needs_read_lock
-    def clone(self, a_bzrdir, revision_id=None):
-        """Clone this repository into a_bzrdir using the current format.
+    def clone(self, controldir, revision_id=None):
+        """Clone this repository into controldir using the current format.
 
         Currently no check is made that the format of this repository and
         the bzrdir format are compatible. FIXME RBC 20060201.
@@ -757,7 +756,8 @@
         """
         # TODO: deprecate after 0.16; cloning this with all its settings is
         # probably not very useful -- mbp 20070423
-        dest_repo = self._create_sprouting_repo(a_bzrdir, shared=self.is_shared())
+        dest_repo = self._create_sprouting_repo(
+            controldir, shared=self.is_shared())
         self.copy_content_into(dest_repo, revision_id)
         return dest_repo
 
@@ -930,16 +930,6 @@
         parent_ids.discard(_mod_revision.NULL_REVISION)
         return parent_ids
 
-    def fileids_altered_by_revision_ids(self, revision_ids):
-        """Find the file ids and versions affected by revisions.
-
-        :param revisions: an iterable containing revision ids.
-        :return: a dictionary mapping altered file-ids to an iterable of
-            revision_ids. Each altered file-ids has the exact revision_ids
-            that altered it listed explicitly.
-        """
-        raise NotImplementedError(self.fileids_altered_by_revision_ids)
-
     def iter_files_bytes(self, desired_files):
         """Iterate through file versions.
 
@@ -1159,19 +1149,6 @@
                 [parents_provider, other_repository._make_parents_provider()])
         return graph.Graph(parents_provider)
 
-    def revision_ids_to_search_result(self, result_set):
-        """Convert a set of revision ids to a graph SearchResult."""
-        result_parents = set()
-        for parents in self.get_graph().get_parent_map(
-            result_set).itervalues():
-            result_parents.update(parents)
-        included_keys = result_set.intersection(result_parents)
-        start_keys = result_set.difference(included_keys)
-        exclude_keys = result_parents.difference(result_set)
-        result = graph.SearchResult(start_keys, exclude_keys,
-            len(result_set), result_set)
-        return result
-
     @needs_write_lock
     def set_make_working_trees(self, new_value):
         """Set the policy flag for making working trees when creating branches.
@@ -1195,7 +1172,7 @@
         self.store_revision_signature(gpg_strategy, plaintext, revision_id)
 
     @needs_read_lock
-    def verify_revision(self, revision_id, gpg_strategy):
+    def verify_revision_signature(self, revision_id, gpg_strategy):
         """Verify the signature on a revision.
         
         :param revision_id: the revision to verify
@@ -1316,14 +1293,23 @@
         """Returns the policy for making working trees on new branches."""
         return not self._transport.has('no-working-trees')
 
+    @needs_write_lock
+    def update_feature_flags(self, updated_flags):
+        """Update the feature flags for this branch.
+
+        :param updated_flags: Dictionary mapping feature names to necessities
+            A necessity can be None to indicate the feature should be removed
+        """
+        self._format._update_feature_flags(updated_flags)
+        self.control_transport.put_bytes('format', self._format.as_string())
+
 
 class RepositoryFormatRegistry(controldir.ControlComponentFormatRegistry):
     """Repository format registry."""
 
     def get_default(self):
         """Return the current default format."""
-        from bzrlib import bzrdir
-        return bzrdir.format_registry.make_bzrdir('default').repository_format
+        return controldir.format_registry.make_bzrdir('default').repository_format
 
 
 network_format_registry = registry.FormatRegistry()
@@ -1372,7 +1358,7 @@
     created.
 
     Common instance attributes:
-    _matchingbzrdir - the bzrdir format that the repository format was
+    _matchingbzrdir - the controldir format that the repository format was
     originally written to work with. This can be used if manually
     constructing a bzrdir and repository, or more commonly for test suite
     parameterization.
@@ -1418,6 +1404,11 @@
     rich_root_data = None
     # Does this format support explicitly versioned directories?
     supports_versioned_directories = None
+    # Can other repositories be nested into one of this format?
+    supports_nesting_repositories = None
+    # Is it possible for revisions to be present without being referenced
+    # somewhere ?
+    supports_unreferenced_revisions = None
 
     def __repr__(self):
         return "%s()" % self.__class__.__name__
@@ -1430,24 +1421,6 @@
         return not self == other
 
     @classmethod
-    def find_format(klass, a_bzrdir):
-        """Return the format for the repository object in a_bzrdir.
-
-        This is used by bzr native formats that have a "format" file in
-        the repository.  Other methods may be used by different types of
-        control directory.
-        """
-        try:
-            transport = a_bzrdir.get_repository_transport(None)
-            format_string = transport.get_bytes("format")
-            return format_registry.get(format_string)
-        except errors.NoSuchFile:
-            raise errors.NoRepositoryPresent(a_bzrdir)
-        except KeyError:
-            raise errors.UnknownFormatError(format=format_string,
-                                            kind='repository')
-
-    @classmethod
     @symbol_versioning.deprecated_method(symbol_versioning.deprecated_in((2, 4, 0)))
     def register_format(klass, format):
         format_registry.register(format)
@@ -1463,27 +1436,19 @@
         """Return the current default format."""
         return format_registry.get_default()
 
-    def get_format_string(self):
-        """Return the ASCII format string that identifies this format.
-
-        Note that in pre format ?? repositories the format string is
-        not permitted nor written to disk.
-        """
-        raise NotImplementedError(self.get_format_string)
-
     def get_format_description(self):
         """Return the short description for this format."""
         raise NotImplementedError(self.get_format_description)
 
-    def initialize(self, a_bzrdir, shared=False):
-        """Initialize a repository of this format in a_bzrdir.
+    def initialize(self, controldir, shared=False):
+        """Initialize a repository of this format in controldir.
 
-        :param a_bzrdir: The bzrdir to put the new repository in it.
+        :param controldir: The controldir to put the new repository in it.
         :param shared: The repository should be initialized as a sharable one.
         :returns: The new repository object.
 
         This may raise UninitializableFormat if shared repository are not
-        compatible the a_bzrdir.
+        compatible the controldir.
         """
         raise NotImplementedError(self.initialize)
 
@@ -1525,30 +1490,31 @@
                 'Does not support nested trees', target_format,
                 from_format=self)
 
-    def open(self, a_bzrdir, _found=False):
-        """Return an instance of this format for the bzrdir a_bzrdir.
+    def open(self, controldir, _found=False):
+        """Return an instance of this format for a controldir.
 
         _found is a private parameter, do not use it.
         """
         raise NotImplementedError(self.open)
 
-    def _run_post_repo_init_hooks(self, repository, a_bzrdir, shared):
-        from bzrlib.bzrdir import BzrDir, RepoInitHookParams
-        hooks = BzrDir.hooks['post_repo_init']
+    def _run_post_repo_init_hooks(self, repository, controldir, shared):
+        from bzrlib.controldir import ControlDir, RepoInitHookParams
+        hooks = ControlDir.hooks['post_repo_init']
         if not hooks:
             return
-        params = RepoInitHookParams(repository, self, a_bzrdir, shared)
+        params = RepoInitHookParams(repository, self, controldir, shared)
         for hook in hooks:
             hook(params)
 
 
-class MetaDirRepositoryFormat(RepositoryFormat):
+class RepositoryFormatMetaDir(bzrdir.BzrFormat, RepositoryFormat):
     """Common base class for the new repositories using the metadir layout."""
 
     rich_root_data = False
     supports_tree_reference = False
     supports_external_lookups = False
     supports_leaving_lock = True
+    supports_nesting_repositories = True
 
     @property
     def _matchingbzrdir(self):
@@ -1557,7 +1523,8 @@
         return matching
 
     def __init__(self):
-        super(MetaDirRepositoryFormat, self).__init__()
+        RepositoryFormat.__init__(self)
+        bzrdir.BzrFormat.__init__(self)
 
     def _create_control_files(self, a_bzrdir):
         """Create the required files and the initial control_files object."""
@@ -1587,15 +1554,34 @@
         finally:
             control_files.unlock()
 
-    def network_name(self):
-        """Metadir formats have matching disk and network format strings."""
-        return self.get_format_string()
+    @classmethod
+    def find_format(klass, a_bzrdir):
+        """Return the format for the repository object in a_bzrdir.
+
+        This is used by bzr native formats that have a "format" file in
+        the repository.  Other methods may be used by different types of
+        control directory.
+        """
+        try:
+            transport = a_bzrdir.get_repository_transport(None)
+            format_string = transport.get_bytes("format")
+        except errors.NoSuchFile:
+            raise errors.NoRepositoryPresent(a_bzrdir)
+        return klass._find_format(format_registry, 'repository', format_string)
+
+    def check_support_status(self, allow_unsupported, recommend_upgrade=True,
+            basedir=None):
+        RepositoryFormat.check_support_status(self,
+            allow_unsupported=allow_unsupported, recommend_upgrade=recommend_upgrade,
+            basedir=basedir)
+        bzrdir.BzrFormat.check_support_status(self, allow_unsupported=allow_unsupported,
+            recommend_upgrade=recommend_upgrade, basedir=basedir)
 
 
 # formats which have no format string are not discoverable or independently
 # creatable on disk, so are not registered in format_registry.  They're
 # all in bzrlib.repofmt.knitreponow.  When an instance of one of these is
-# needed, it's constructed directly by the BzrDir.  Non-native formats where
+# needed, it's constructed directly by the ControlDir.  Non-native formats where
 # the repository is not separately opened are similar.
 
 format_registry.register_lazy(
@@ -1712,8 +1698,7 @@
         self.target.fetch(self.source, revision_id=revision_id)
 
     @needs_write_lock
-    def fetch(self, revision_id=None, find_ghosts=False,
-            fetch_spec=None):
+    def fetch(self, revision_id=None, find_ghosts=False):
         """Fetch the content required to construct revision_id.
 
         The content is copied from self.source to self.target.
@@ -1799,25 +1784,25 @@
         # trigger an assertion if not such
         repo._format.get_format_string()
         self.repo_dir = repo.bzrdir
-        pb.update('Moving repository to repository.backup')
+        pb.update(gettext('Moving repository to repository.backup'))
         self.repo_dir.transport.move('repository', 'repository.backup')
         backup_transport =  self.repo_dir.transport.clone('repository.backup')
         repo._format.check_conversion_target(self.target_format)
         self.source_repo = repo._format.open(self.repo_dir,
             _found=True,
             _override_transport=backup_transport)
-        pb.update('Creating new repository')
+        pb.update(gettext('Creating new repository'))
         converted = self.target_format.initialize(self.repo_dir,
                                                   self.source_repo.is_shared())
         converted.lock_write()
         try:
-            pb.update('Copying content')
+            pb.update(gettext('Copying content'))
             self.source_repo.copy_content_into(converted)
         finally:
             converted.unlock()
-        pb.update('Deleting old repository content')
+        pb.update(gettext('Deleting old repository content'))
         self.repo_dir.transport.delete_tree('repository.backup')
-        ui.ui_factory.note('repository converted')
+        ui.ui_factory.note(gettext('repository converted'))
         pb.finished()
 
 

=== modified file 'bzrlib/revision.py'
--- bzrlib/revision.py	2011-05-10 07:46:15 +0000
+++ bzrlib/revision.py	2011-12-18 15:28:38 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 # TODO: Some kind of command-line display of revision properties:
 # perhaps show them in log -v and allow them as options to the commit command.
 

=== modified file 'bzrlib/revisionspec.py'
--- bzrlib/revisionspec.py	2011-03-03 12:50:21 +0000
+++ bzrlib/revisionspec.py	2011-12-19 10:58:39 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
@@ -27,6 +29,7 @@
     symbol_versioning,
     workingtree,
     )
+from bzrlib.i18n import gettext
 """)
 
 from bzrlib import (
@@ -37,9 +40,6 @@
     )
 
 
-_marker = []
-
-
 class RevisionInfo(object):
     """The results of applying a revision specification to a branch."""
 
@@ -57,24 +57,28 @@
     or treat the result as a tuple.
     """
 
-    def __init__(self, branch, revno, rev_id=_marker):
+    def __init__(self, branch, revno=None, rev_id=None):
         self.branch = branch
-        self.revno = revno
-        if rev_id is _marker:
+        self._has_revno = (revno is not None)
+        self._revno = revno
+        self.rev_id = rev_id
+        if self.rev_id is None and self._revno is not None:
             # allow caller to be lazy
-            if self.revno is None:
-                self.rev_id = None
-            else:
-                self.rev_id = branch.get_rev_id(self.revno)
-        else:
-            self.rev_id = rev_id
+            self.rev_id = branch.get_rev_id(self._revno)
+
+    @property
+    def revno(self):
+        if not self._has_revno and self.rev_id is not None:
+            try:
+                self._revno = self.branch.revision_id_to_revno(self.rev_id)
+            except errors.NoSuchRevision:
+                self._revno = None
+            self._has_revno = True
+        return self._revno
 
     def __nonzero__(self):
-        # first the easy ones...
         if self.rev_id is None:
             return False
-        if self.revno is not None:
-            return True
         # TODO: otherwise, it should depend on how I was built -
         # if it's in_history(branch), then check revision_history(),
         # if it's in_store(branch), do the check below
@@ -103,18 +107,17 @@
             self.revno, self.rev_id, self.branch)
 
     @staticmethod
-    def from_revision_id(branch, revision_id, revs):
+    def from_revision_id(branch, revision_id, revs=symbol_versioning.DEPRECATED_PARAMETER):
         """Construct a RevisionInfo given just the id.
 
         Use this if you don't know or care what the revno is.
         """
-        if revision_id == revision.NULL_REVISION:
-            return RevisionInfo(branch, 0, revision_id)
-        try:
-            revno = revs.index(revision_id) + 1
-        except ValueError:
-            revno = None
-        return RevisionInfo(branch, revno, revision_id)
+        if symbol_versioning.deprecated_passed(revs):
+            symbol_versioning.warn(
+                'RevisionInfo.from_revision_id(revs) was deprecated in 2.5.',
+                DeprecationWarning,
+                stacklevel=2)
+        return RevisionInfo(branch, revno=None, rev_id=revision_id)
 
 
 class RevisionSpec(object):
@@ -137,7 +140,8 @@
     """
 
     prefix = None
-    wants_revision_history = True
+    # wants_revision_history has been deprecated in 2.5.
+    wants_revision_history = False
     dwim_catchable_exceptions = (errors.InvalidRevisionSpec,)
     """Exceptions that RevisionSpec_dwim._match_on will catch.
 
@@ -208,7 +212,18 @@
     def in_history(self, branch):
         if branch:
             if self.wants_revision_history:
-                revs = branch.revision_history()
+                symbol_versioning.warn(
+                    "RevisionSpec.wants_revision_history was "
+                    "deprecated in 2.5 (%s)." % self.__class__.__name__,
+                    DeprecationWarning)
+                branch.lock_read()
+                try:
+                    graph = branch.repository.get_graph()
+                    revs = list(graph.iter_lefthand_ancestry(
+                        branch.last_revision(), [revision.NULL_REVISION]))
+                finally:
+                    branch.unlock()
+                revs.reverse()
             else:
                 revs = None
         else:
@@ -294,9 +309,6 @@
     """
 
     help_txt = None
-    # We don't need to build the revision history ourself, that's delegated to
-    # each revspec we try.
-    wants_revision_history = False
 
     _revno_regex = lazy_regex.lazy_compile(r'^(?:(\d+(\.\d+)*)|-\d+)(:.*)?$')
 
@@ -380,14 +392,13 @@
                                    your history is very long.
     """
     prefix = 'revno:'
-    wants_revision_history = False
 
     def _match_on(self, branch, revs):
         """Lookup a revision by revision number"""
-        branch, revno, revision_id = self._lookup(branch, revs)
+        branch, revno, revision_id = self._lookup(branch)
         return RevisionInfo(branch, revno, revision_id)
 
-    def _lookup(self, branch, revs_or_none):
+    def _lookup(self, branch):
         loc = self.spec.find(':')
         if loc == -1:
             revno_spec = self.spec
@@ -417,12 +428,8 @@
                 dotted = True
 
         if branch_spec:
-            # the user has override the branch to look in.
-            # we need to refresh the revision_history map and
-            # the branch object.
-            from bzrlib.branch import Branch
-            branch = Branch.open(branch_spec)
-            revs_or_none = None
+            # the user has overriden the branch to look in.
+            branch = _mod_branch.Branch.open(branch_spec)
 
         if dotted:
             try:
@@ -432,7 +439,7 @@
                 raise errors.InvalidRevisionSpec(self.user_spec, branch)
             else:
                 # there is no traditional 'revno' for dotted-decimal revnos.
-                # so for  API compatability we return None.
+                # so for API compatibility we return None.
                 return branch, None, revision_id
         else:
             last_revno, last_revision_id = branch.last_revision_info()
@@ -444,14 +451,14 @@
                 else:
                     revno = last_revno + revno + 1
             try:
-                revision_id = branch.get_rev_id(revno, revs_or_none)
+                revision_id = branch.get_rev_id(revno)
             except errors.NoSuchRevision:
                 raise errors.InvalidRevisionSpec(self.user_spec, branch)
         return branch, revno, revision_id
 
     def _as_revision_id(self, context_branch):
         # We would have the revno here, but we don't really care
-        branch, revno, revision_id = self._lookup(context_branch, None)
+        branch, revno, revision_id = self._lookup(context_branch)
         return revision_id
 
     def needs_branch(self):
@@ -467,12 +474,11 @@
 RevisionSpec_int = RevisionSpec_revno
 
 
-
 class RevisionIDSpec(RevisionSpec):
 
     def _match_on(self, branch, revs):
         revision_id = self.as_revision_id(branch)
-        return RevisionInfo.from_revision_id(branch, revision_id, revs)
+        return RevisionInfo.from_revision_id(branch, revision_id)
 
 
 class RevisionSpec_revid(RevisionIDSpec):
@@ -514,10 +520,10 @@
     prefix = 'last:'
 
     def _match_on(self, branch, revs):
-        revno, revision_id = self._revno_and_revision_id(branch, revs)
+        revno, revision_id = self._revno_and_revision_id(branch)
         return RevisionInfo(branch, revno, revision_id)
 
-    def _revno_and_revision_id(self, context_branch, revs_or_none):
+    def _revno_and_revision_id(self, context_branch):
         last_revno, last_revision_id = context_branch.last_revision_info()
 
         if self.spec == '':
@@ -536,7 +542,7 @@
 
         revno = last_revno - offset + 1
         try:
-            revision_id = context_branch.get_rev_id(revno, revs_or_none)
+            revision_id = context_branch.get_rev_id(revno)
         except errors.NoSuchRevision:
             raise errors.InvalidRevisionSpec(self.user_spec, context_branch)
         return revno, revision_id
@@ -544,7 +550,7 @@
     def _as_revision_id(self, context_branch):
         # We compute the revno as part of the process, but we don't really care
         # about it.
-        revno, revision_id = self._revno_and_revision_id(context_branch, None)
+        revno, revision_id = self._revno_and_revision_id(context_branch)
         return revision_id
 
 
@@ -582,14 +588,10 @@
             # We need to use the repository history here
             rev = branch.repository.get_revision(r.rev_id)
             if not rev.parent_ids:
-                revno = 0
                 revision_id = revision.NULL_REVISION
             else:
                 revision_id = rev.parent_ids[0]
-                try:
-                    revno = revs.index(revision_id) + 1
-                except ValueError:
-                    revno = None
+            revno = None
         else:
             revno = r.revno - 1
             try:
@@ -600,8 +602,7 @@
         return RevisionInfo(branch, revno, revision_id)
 
     def _as_revision_id(self, context_branch):
-        base_revspec = RevisionSpec.from_string(self.spec)
-        base_revision_id = base_revspec.as_revision_id(context_branch)
+        base_revision_id = RevisionSpec.from_string(self.spec)._as_revision_id(context_branch)
         if base_revision_id == revision.NULL_REVISION:
             raise errors.InvalidRevisionSpec(self.user_spec, context_branch,
                                          'cannot go before the null: revision')
@@ -637,8 +638,7 @@
     def _match_on(self, branch, revs):
         # Can raise tags not supported, NoSuchTag, etc
         return RevisionInfo.from_revision_id(branch,
-            branch.tags.lookup_tag(self.spec),
-            revs)
+            branch.tags.lookup_tag(self.spec))
 
     def _as_revision_id(self, context_branch):
         return context_branch.tags.lookup_tag(self.spec)
@@ -648,20 +648,19 @@
 class _RevListToTimestamps(object):
     """This takes a list of revisions, and allows you to bisect by date"""
 
-    __slots__ = ['revs', 'branch']
+    __slots__ = ['branch']
 
-    def __init__(self, revs, branch):
-        self.revs = revs
+    def __init__(self, branch):
         self.branch = branch
 
     def __getitem__(self, index):
         """Get the date of the index'd item"""
-        r = self.branch.repository.get_revision(self.revs[index])
+        r = self.branch.repository.get_revision(self.branch.get_rev_id(index))
         # TODO: Handle timezone.
         return datetime.datetime.fromtimestamp(r.timestamp)
 
     def __len__(self):
-        return len(self.revs)
+        return self.branch.revno()
 
 
 class RevisionSpec_date(RevisionSpec):
@@ -741,13 +740,12 @@
                     hour=hour, minute=minute, second=second)
         branch.lock_read()
         try:
-            rev = bisect.bisect(_RevListToTimestamps(revs, branch), dt)
+            rev = bisect.bisect(_RevListToTimestamps(branch), dt, 1)
         finally:
             branch.unlock()
-        if rev == len(revs):
+        if rev == branch.revno():
             raise errors.InvalidRevisionSpec(self.user_spec, branch)
-        else:
-            return RevisionInfo(branch, rev + 1)
+        return RevisionInfo(branch, rev)
 
 
 
@@ -784,11 +782,7 @@
     def _find_revision_info(branch, other_location):
         revision_id = RevisionSpec_ancestor._find_revision_id(branch,
                                                               other_location)
-        try:
-            revno = branch.revision_id_to_revno(revision_id)
-        except errors.NoSuchRevision:
-            revno = None
-        return RevisionInfo(branch, revno, revision_id)
+        return RevisionInfo(branch, None, revision_id)
 
     @staticmethod
     def _find_revision_id(branch, other_location):
@@ -848,11 +842,7 @@
                 branch.fetch(other_branch, revision_b)
             except errors.ReadOnlyError:
                 branch = other_branch
-        try:
-            revno = branch.revision_id_to_revno(revision_b)
-        except errors.NoSuchRevision:
-            revno = None
-        return RevisionInfo(branch, revno, revision_b)
+        return RevisionInfo(branch, None, revision_b)
 
     def _as_revision_id(self, context_branch):
         from bzrlib.branch import Branch
@@ -910,7 +900,8 @@
             location_type = 'parent branch'
         if submit_location is None:
             raise errors.NoSubmitBranch(branch)
-        trace.note('Using %s %s', location_type, submit_location)
+        trace.note(gettext('Using {0} {1}').format(location_type,
+                                                        submit_location))
         return submit_location
 
     def _match_on(self, branch, revs):

=== modified file 'bzrlib/revisiontree.py'
--- bzrlib/revisiontree.py	2011-06-16 11:58:04 +0000
+++ bzrlib/revisiontree.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,8 @@
 
 """RevisionTree - a Tree implementation backed by repository data for a revision."""
 
+from __future__ import absolute_import
+
 from cStringIO import StringIO
 
 from bzrlib import (
@@ -36,6 +38,10 @@
         self._revision_id = revision_id
         self._rules_searcher = None
 
+    def has_versioned_directories(self):
+        """See `Tree.has_versioned_directories`."""
+        return self._repository._format.supports_versioned_directories
+
     def supports_tree_reference(self):
         return getattr(self._repository._format, "supports_tree_reference",
             False)
@@ -61,8 +67,9 @@
         raise NotImplementedError(self.get_file_revision)
 
     def get_file_text(self, file_id, path=None):
-        _, content = list(self.iter_files_bytes([(file_id, None)]))[0]
-        return ''.join(content)
+        for (identifier, content) in self.iter_files_bytes([(file_id, None)]):
+            ret = "".join(content)
+        return ret
 
     def get_file(self, file_id, path=None):
         return StringIO(self.get_file_text(file_id))
@@ -211,14 +218,14 @@
     def iter_files_bytes(self, desired_files):
         """See Tree.iter_files_bytes.
 
-        This version is implemented on top of Repository.extract_files_bytes"""
+        This version is implemented on top of Repository.iter_files_bytes"""
         repo_desired_files = [(f, self.get_file_revision(f), i)
                               for f, i in desired_files]
         try:
             for result in self._repository.iter_files_bytes(repo_desired_files):
                 yield result
         except errors.RevisionNotPresent, e:
-            raise errors.NoSuchFile(e.revision_id)
+            raise errors.NoSuchFile(e.file_id)
 
     def annotate_iter(self, file_id,
                       default_revision=revision.CURRENT_REVISION):

=== modified file 'bzrlib/rio.py'
--- bzrlib/rio.py	2009-09-11 06:36:50 +0000
+++ bzrlib/rio.py	2011-12-18 15:28:38 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 # \subsection{\emph{rio} - simple text metaformat}
 #
 # \emph{r} stands for `restricted', `reproducible', or `rfc822-like'.

=== modified file 'bzrlib/rules.py'
--- bzrlib/rules.py	2011-04-19 07:52:11 +0000
+++ bzrlib/rules.py	2011-12-19 13:23:58 +0000
@@ -19,6 +19,8 @@
 See ``bzr help rules`` for details.
 """
 
+from __future__ import absolute_import
+
 from bzrlib import (
     config,
     cmdline,

=== modified file 'bzrlib/send.py'
--- bzrlib/send.py	2011-05-17 12:12:09 +0000
+++ bzrlib/send.py	2011-12-18 15:28:38 +0000
@@ -14,17 +14,19 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
 
 import os
 import time
 
 from bzrlib import (
-    bzrdir,
+    controldir,
     errors,
     osutils,
     registry,
     trace,
     )
+from bzrlib.i18n import gettext
 from bzrlib.branch import (
     Branch,
     )
@@ -36,10 +38,12 @@
 format_registry = registry.Registry()
 
 
-def send(submit_branch, revision, public_branch, remember, format,
-         no_bundle, no_patch, output, from_, mail_to, message, body,
+def send(target_branch, revision, public_branch, remember,
+         format, no_bundle, no_patch, output, from_, mail_to, message, body,
          to_file, strict=None):
-    tree, branch = bzrdir.BzrDir.open_containing_tree_or_branch(from_)[:2]
+    tree, branch = controldir.ControlDir.open_containing_tree_or_branch(
+        from_)[:2]
+    possible_transports = [tree.bzrdir.transport, branch.bzrdir.transport]
     # we may need to write data into branch's repository to calculate
     # the data to send.
     branch.lock_write()
@@ -51,46 +55,47 @@
             mail_client = config.get_mail_client()
             if (not getattr(mail_client, 'supports_body', False)
                 and body is not None):
-                raise errors.BzrCommandError(
-                    'Mail client "%s" does not support specifying body' %
+                raise errors.BzrCommandError(gettext(
+                    'Mail client "%s" does not support specifying body') %
                     mail_client.__class__.__name__)
-        if remember and submit_branch is None:
-            raise errors.BzrCommandError(
-                '--remember requires a branch to be specified.')
-        stored_submit_branch = branch.get_submit_branch()
-        remembered_submit_branch = None
-        if submit_branch is None:
-            submit_branch = stored_submit_branch
-            remembered_submit_branch = "submit"
+        if remember and target_branch is None:
+            raise errors.BzrCommandError(gettext(
+                '--remember requires a branch to be specified.'))
+        stored_target_branch = branch.get_submit_branch()
+        remembered_target_branch = None
+        if target_branch is None:
+            target_branch = stored_target_branch
+            remembered_target_branch = "submit"
         else:
             # Remembers if asked explicitly or no previous location is set
-            if remember or (remember is None and stored_submit_branch is None):
-                branch.set_submit_branch(submit_branch)
-        if submit_branch is None:
-            submit_branch = branch.get_parent()
-            remembered_submit_branch = "parent"
-        if submit_branch is None:
-            raise errors.BzrCommandError('No submit branch known or'
-                                         ' specified')
-        if remembered_submit_branch is not None:
-            trace.note('Using saved %s location "%s" to determine what '
-                       'changes to submit.', remembered_submit_branch,
-                       submit_branch)
+            if remember or (
+                    remember is None and stored_target_branch is None):
+                branch.set_submit_branch(target_branch)
+        if target_branch is None:
+            target_branch = branch.get_parent()
+            remembered_target_branch = "parent"
+        if target_branch is None:
+            raise errors.BzrCommandError(gettext('No submit branch known or'
+                                         ' specified'))
+        if remembered_target_branch is not None:
+            trace.note(gettext('Using saved {0} location "{1}" to determine '
+                       'what changes to submit.').format(
+                                    remembered_target_branch,
+                                    target_branch))
 
+        submit_branch = Branch.open(target_branch,
+            possible_transports=possible_transports)
+        possible_transports.append(submit_branch.bzrdir.root_transport)
         if mail_to is None or format is None:
-            # TODO: jam 20090716 we open the submit_branch here, but we *don't*
-            #       pass it down into the format creation, so it will have to
-            #       open it again
-            submit_br = Branch.open(submit_branch)
-            submit_config = submit_br.get_config()
+            submit_config = submit_branch.get_config()
             if mail_to is None:
                 mail_to = submit_config.get_user_option("child_submit_to")
             if format is None:
-                formatname = submit_br.get_child_submit_format()
+                formatname = submit_branch.get_child_submit_format()
                 try:
                     format = format_registry.get(formatname)
                 except KeyError:
-                    raise errors.BzrCommandError("No such send format '%s'." % 
+                    raise errors.BzrCommandError(gettext("No such send format '%s'.") % 
                                                  formatname)
 
         stored_public_branch = branch.get_public_branch()
@@ -101,14 +106,14 @@
               or (remember is None and stored_public_branch is None)):
             branch.set_public_branch(public_branch)
         if no_bundle and public_branch is None:
-            raise errors.BzrCommandError('No public branch specified or'
-                                         ' known')
+            raise errors.BzrCommandError(gettext('No public branch specified or'
+                                         ' known'))
         base_revision_id = None
         revision_id = None
         if revision is not None:
             if len(revision) > 2:
-                raise errors.BzrCommandError('bzr send takes '
-                    'at most two one revision identifiers')
+                raise errors.BzrCommandError(gettext('bzr send takes '
+                    'at most two one revision identifiers'))
             revision_id = revision[-1].as_revision_id(branch)
             if len(revision) == 2:
                 base_revision_id = revision[0].as_revision_id(branch)
@@ -120,19 +125,20 @@
                     more_warning='Uncommitted changes will not be sent.')
             revision_id = branch.last_revision()
         if revision_id == NULL_REVISION:
-            raise errors.BzrCommandError('No revisions to submit.')
+            raise errors.BzrCommandError(gettext('No revisions to submit.'))
         if format is None:
             format = format_registry.get()
-        directive = format(branch, revision_id, submit_branch,
-            public_branch, no_patch, no_bundle, message, base_revision_id)
+        directive = format(branch, revision_id, target_branch,
+            public_branch, no_patch, no_bundle, message, base_revision_id,
+            submit_branch)
         if output is None:
             directive.compose_merge_request(mail_client, mail_to, body,
                                             branch, tree)
         else:
             if directive.multiple_output_files:
                 if output == '-':
-                    raise errors.BzrCommandError('- not supported for '
-                        'merge directives that use more than one output file.')
+                    raise errors.BzrCommandError(gettext('- not supported for '
+                        'merge directives that use more than one output file.'))
                 if not os.path.exists(output):
                     os.mkdir(output, 0755)
                 for (filename, lines) in directive.to_files():
@@ -156,25 +162,29 @@
         branch.unlock()
 
 
-def _send_4(branch, revision_id, submit_branch, public_branch,
-            no_patch, no_bundle, message, base_revision_id):
+def _send_4(branch, revision_id, target_branch, public_branch,
+            no_patch, no_bundle, message,
+            base_revision_id, local_target_branch=None):
     from bzrlib import merge_directive
     return merge_directive.MergeDirective2.from_objects(
         branch.repository, revision_id, time.time(),
-        osutils.local_time_offset(), submit_branch,
-        public_branch=public_branch, include_patch=not no_patch,
+        osutils.local_time_offset(), target_branch,
+        public_branch=public_branch,
+        include_patch=not no_patch,
         include_bundle=not no_bundle, message=message,
-        base_revision_id=base_revision_id)
+        base_revision_id=base_revision_id,
+        local_target_branch=local_target_branch)
 
 
 def _send_0_9(branch, revision_id, submit_branch, public_branch,
-              no_patch, no_bundle, message, base_revision_id):
+              no_patch, no_bundle, message,
+              base_revision_id, local_target_branch=None):
     if not no_bundle:
         if not no_patch:
             patch_type = 'bundle'
         else:
-            raise errors.BzrCommandError('Format 0.9 does not'
-                ' permit bundle with no patch')
+            raise errors.BzrCommandError(gettext('Format 0.9 does not'
+                ' permit bundle with no patch'))
     else:
         if not no_patch:
             patch_type = 'diff'
@@ -185,10 +195,10 @@
         branch.repository, revision_id, time.time(),
         osutils.local_time_offset(), submit_branch,
         public_branch=public_branch, patch_type=patch_type,
-        message=message)
-
-
-format_registry.register('4', 
+        message=message, local_target_branch=local_target_branch)
+
+
+format_registry.register('4',
     _send_4, 'Bundle format 4, Merge Directive 2 (default)')
 format_registry.register('0.9',
     _send_0_9, 'Bundle format 0.9, Merge Directive 1')

=== modified file 'bzrlib/serializer.py'
--- bzrlib/serializer.py	2011-02-19 15:23:08 +0000
+++ bzrlib/serializer.py	2011-12-19 13:23:58 +0000
@@ -14,9 +14,9 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-
 """Inventory/revision serialization."""
 
+from __future__ import absolute_import
 
 from bzrlib import registry
 

=== modified file 'bzrlib/shelf.py'
--- bzrlib/shelf.py	2011-07-06 21:37:16 +0000
+++ bzrlib/shelf.py	2011-12-18 15:28:38 +0000
@@ -14,6 +14,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
 
 import errno
 import re

=== modified file 'bzrlib/shelf_ui.py'
--- bzrlib/shelf_ui.py	2010-11-04 17:48:47 +0000
+++ bzrlib/shelf_ui.py	2011-12-18 15:28:38 +0000
@@ -14,6 +14,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
 
 from cStringIO import StringIO
 import shutil
@@ -34,7 +35,7 @@
     ui,
     workingtree,
 )
-
+from bzrlib.i18n import gettext
 
 class UseEditor(Exception):
     """Use an editor instead of selecting hunks."""
@@ -42,17 +43,17 @@
 
 class ShelfReporter(object):
 
-    vocab = {'add file': 'Shelve adding file "%(path)s"?',
-             'binary': 'Shelve binary changes?',
-             'change kind': 'Shelve changing "%s" from %(other)s'
-             ' to %(this)s?',
-             'delete file': 'Shelve removing file "%(path)s"?',
-             'final': 'Shelve %d change(s)?',
-             'hunk': 'Shelve?',
-             'modify target': 'Shelve changing target of'
-             ' "%(path)s" from "%(other)s" to "%(this)s"?',
-             'rename': 'Shelve renaming "%(other)s" =>'
-                        ' "%(this)s"?'
+    vocab = {'add file': gettext('Shelve adding file "%(path)s"?'),
+             'binary': gettext('Shelve binary changes?'),
+             'change kind': gettext('Shelve changing "%s" from %(other)s'
+             ' to %(this)s?'),
+             'delete file': gettext('Shelve removing file "%(path)s"?'),
+             'final': gettext('Shelve %d change(s)?'),
+             'hunk': gettext('Shelve?'),
+             'modify target': gettext('Shelve changing target of'
+             ' "%(path)s" from "%(other)s" to "%(this)s"?'),
+             'rename': gettext('Shelve renaming "%(other)s" =>'
+                        ' "%(this)s"?')
              }
 
     invert_diff = False
@@ -66,15 +67,15 @@
 
     def shelved_id(self, shelf_id):
         """Report the id changes were shelved to."""
-        trace.note('Changes shelved with id "%d".' % shelf_id)
+        trace.note(gettext('Changes shelved with id "%d".') % shelf_id)
 
     def changes_destroyed(self):
         """Report that changes were made without shelving."""
-        trace.note('Selected changes destroyed.')
+        trace.note(gettext('Selected changes destroyed.'))
 
     def selected_changes(self, transform):
         """Report the changes that were selected."""
-        trace.note("Selected changes:")
+        trace.note(gettext("Selected changes:"))
         changes = transform.iter_changes()
         delta.report_changes(changes, self.delta_reporter)
 
@@ -94,16 +95,16 @@
 
 class ApplyReporter(ShelfReporter):
 
-    vocab = {'add file': 'Delete file "%(path)s"?',
-             'binary': 'Apply binary changes?',
-             'change kind': 'Change "%(path)s" from %(this)s'
-             ' to %(other)s?',
-             'delete file': 'Add file "%(path)s"?',
-             'final': 'Apply %d change(s)?',
-             'hunk': 'Apply change?',
-             'modify target': 'Change target of'
-             ' "%(path)s" from "%(this)s" to "%(other)s"?',
-             'rename': 'Rename "%(this)s" => "%(other)s"?',
+    vocab = {'add file': gettext('Delete file "%(path)s"?'),
+             'binary': gettext('Apply binary changes?'),
+             'change kind': gettext('Change "%(path)s" from %(this)s'
+             ' to %(other)s?'),
+             'delete file': gettext('Add file "%(path)s"?'),
+             'final': gettext('Apply %d change(s)?'),
+             'hunk': gettext('Apply change?'),
+             'modify target': gettext('Change target of'
+             ' "%(path)s" from "%(this)s" to "%(other)s"?'),
+             'rename': gettext('Rename "%(this)s" => "%(other)s"?'),
              }
 
     invert_diff = True
@@ -251,25 +252,10 @@
         diff_file.seek(0)
         return patches.parse_patch(diff_file)
 
-    def prompt(self, message):
-        """Prompt the user for a character.
-
-        :param message: The message to prompt a user with.
-        :return: A character.
-        """
-        if not sys.stdin.isatty():
-            # Since there is no controlling terminal we will hang when trying
-            # to prompt the user, better abort now.  See
-            # https://code.launchpad.net/~bialix/bzr/shelve-no-tty/+merge/14905
-            # for more context.
-            raise errors.BzrError("You need a controlling terminal.")
-        sys.stdout.write(message)
-        char = osutils.getchar()
-        sys.stdout.write("\r" + ' ' * len(message) + '\r')
-        sys.stdout.flush()
-        return char
-
-    def prompt_bool(self, question, long=False, allow_editor=False):
+    def prompt(self, message, choices, default):
+        return ui.ui_factory.choose(message, choices, default=default)
+
+    def prompt_bool(self, question, allow_editor=False):
         """Prompt the user with a yes/no question.
 
         This may be overridden by self.auto.  It may also *set* self.auto.  It
@@ -279,16 +265,19 @@
         """
         if self.auto:
             return True
-        editor_string = ''
-        if long:
-            if allow_editor:
-                editor_string = '(E)dit manually, '
-            prompt = ' [(y)es, (N)o, %s(f)inish, or (q)uit]' % editor_string
+        alternatives_chars = 'yn'
+        alternatives = '&yes\n&No'
+        if allow_editor:
+            alternatives_chars += 'e'
+            alternatives += '\n&edit manually'
+        alternatives_chars += 'fq'
+        alternatives += '\n&finish\n&quit'
+        choice = self.prompt(question, alternatives, 1)
+        if choice is None:
+            # EOF.
+            char = 'n'
         else:
-            if allow_editor:
-                editor_string = 'e'
-            prompt = ' [yN%sfq?]' % editor_string
-        char = self.prompt(question + prompt)
+            char = alternatives_chars[choice]
         if char == 'y':
             return True
         elif char == 'e' and allow_editor:
@@ -296,8 +285,6 @@
         elif char == 'f':
             self.auto = True
             return True
-        elif char == '?':
-            return self.prompt_bool(question, long=True)
         if char == 'q':
             raise errors.UserAbort()
         else:
@@ -412,7 +399,7 @@
             else:
                 shelf_id = manager.last_shelf()
                 if shelf_id is None:
-                    raise errors.BzrCommandError('No changes are shelved.')
+                    raise errors.BzrCommandError(gettext('No changes are shelved.'))
             apply_changes = True
             delete_shelf = True
             read_shelf = True
@@ -470,11 +457,11 @@
         cleanups = [self.tree.unlock]
         try:
             if self.read_shelf:
-                trace.note('Using changes with id "%d".' % self.shelf_id)
+                trace.note(gettext('Using changes with id "%d".') % self.shelf_id)
                 unshelver = self.manager.get_unshelver(self.shelf_id)
                 cleanups.append(unshelver.finalize)
                 if unshelver.message is not None:
-                    trace.note('Message: %s' % unshelver.message)
+                    trace.note(gettext('Message: %s') % unshelver.message)
                 change_reporter = delta._ChangeReporter()
                 merger = unshelver.make_merger(None)
                 merger.change_reporter = change_reporter
@@ -486,7 +473,7 @@
                     self.show_changes(merger)
             if self.delete_shelf:
                 self.manager.delete_shelf(self.shelf_id)
-                trace.note('Deleted changes with id "%d".' % self.shelf_id)
+                trace.note(gettext('Deleted changes with id "%d".') % self.shelf_id)
         finally:
             for cleanup in reversed(cleanups):
                 cleanup()

=== modified file 'bzrlib/shellcomplete.py'
--- bzrlib/shellcomplete.py	2010-02-17 17:11:16 +0000
+++ bzrlib/shellcomplete.py	2011-12-29 21:05:11 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 import sys
 
 
@@ -57,8 +59,7 @@
 
 def shellcomplete_commands(outfile = None):
     """List all commands"""
-    import inspect
-    import commands
+    from bzrlib import commands
     from inspect import getdoc
 
     commands.install_bzr_command_hooks()

=== modified file 'bzrlib/smart/__init__.py'
--- bzrlib/smart/__init__.py	2009-03-23 14:59:43 +0000
+++ bzrlib/smart/__init__.py	2011-12-18 12:46:49 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006,2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -33,25 +33,13 @@
 
 """
 
-# TODO: _translate_error should be on the client, not the transport because
-#     error coding is wire protocol specific.
+from __future__ import absolute_import
 
 # TODO: A plain integer from query_version is too simple; should give some
 # capabilities too?
 
-# TODO: Server should probably catch exceptions within itself and send them
-# back across the network.  (But shouldn't catch KeyboardInterrupt etc)
-# Also needs to somehow report protocol errors like bad requests.  Need to
-# consider how we'll handle error reporting, e.g. if we get halfway through a
-# bulk transfer and then something goes wrong.
-
 # TODO: Make each request and response self-validatable, e.g. with checksums.
 #
-# TODO: get/put objects could be changed to gradually read back the data as it
-# comes across the network
-#
-# TODO: What should the server do if it hits an error and has to terminate?
-#
 # TODO: is it useful to allow multiple chunks in the bulk data?
 #
 # TODO: If we get an exception during transmission of bulk data we can't just
@@ -60,44 +48,6 @@
 #   chunk, that indicates it is another chunk. Then you can send an 'error'
 #   chunk as long as you finish the previous chunk.
 #
-# TODO: Clone method on Transport; should work up towards parent directory;
-# unclear how this should be stored or communicated to the server... maybe
-# just pass it on all relevant requests?
-#
-# TODO: Better name than clone() for changing between directories.  How about
-# open_dir or change_dir or chdir?
-#
-# TODO: Is it really good to have the notion of current directory within the
-# connection?  Perhaps all Transports should factor out a common connection
-# from the thing that has the directory context?
-#
-# TODO: The server that manages a connection should be quite small and retain
-# minimum state because each of the requests are supposed to be stateless.
-# Then we can write another implementation that maps to http.
-#
-# TODO: What to do when a client connection is garbage collected?  Maybe just
-# abruptly drop the connection?
-#
-# TODO: Server in some cases will need to restrict access to files outside of
-# a particular root directory.  LocalTransport doesn't do anything to stop you
-# ascending above the base directory, so we need to prevent paths
-# containing '..' in either the server or transport layers.  (Also need to
-# consider what happens if someone creates a symlink pointing outside the
-# directory tree...)
-#
-# TODO: Server should rebase absolute paths coming across the network to put
-# them under the virtual root, if one is in use.  LocalTransport currently
-# doesn't do that; if you give it an absolute path it just uses it.
-#
-# XXX: Arguments can't contain newlines or ascii; possibly we should e.g.
-# urlescape them instead.  Indeed possibly this should just literally be
-# http-over-ssh.
-#
-# TODO: Probably want some way for server commands to gradually produce body
-# data rather than passing it as a string; they could perhaps pass an
-# iterator-like callback that will gradually yield data; it probably needs a
-# close() method that will always be closed to do any necessary cleanup.
-
 
 # Promote some attributes from submodules into this namespace
 from bzrlib.smart.request import SmartServerRequestHandler

=== modified file 'bzrlib/smart/branch.py'
--- bzrlib/smart/branch.py	2011-05-08 13:46:49 +0000
+++ bzrlib/smart/branch.py	2011-12-19 13:23:58 +0000
@@ -16,12 +16,14 @@
 
 """Server-side branch related request implmentations."""
 
+from __future__ import absolute_import
 
 from bzrlib import (
     bencode,
     errors,
+    revision as _mod_revision,
     )
-from bzrlib.bzrdir import BzrDir
+from bzrlib.controldir import ControlDir
 from bzrlib.smart.request import (
     FailedSmartServerResponse,
     SmartServerRequest,
@@ -45,10 +47,10 @@
         :return: A SmartServerResponse from self.do_with_branch().
         """
         transport = self.transport_from_client_path(path)
-        bzrdir = BzrDir.open_from_transport(transport)
-        if bzrdir.get_branch_reference() is not None:
+        controldir = ControlDir.open_from_transport(transport)
+        if controldir.get_branch_reference() is not None:
             raise errors.NotBranchError(transport.base)
-        branch = bzrdir.open_branch(ignore_fallbacks=True)
+        branch = controldir.open_branch(ignore_fallbacks=True)
         return self.do_with_branch(branch, *args)
 
 
@@ -76,6 +78,15 @@
             branch.repository.unlock()
 
 
+class SmartServerBranchBreakLock(SmartServerBranchRequest):
+
+    def do_with_branch(self, branch):
+        """Break a branch lock.
+        """
+        branch.break_lock()
+        return SuccessfulSmartServerResponse(('ok', ), )
+
+
 class SmartServerBranchGetConfigFile(SmartServerBranchRequest):
 
     def do_with_branch(self, branch):
@@ -84,12 +95,43 @@
         The body is not utf8 decoded - its the literal bytestream from disk.
         """
         try:
-            content = branch._transport.get_bytes('branch.conf')
+            content = branch.control_transport.get_bytes('branch.conf')
         except errors.NoSuchFile:
             content = ''
         return SuccessfulSmartServerResponse( ('ok', ), content)
 
 
+class SmartServerBranchPutConfigFile(SmartServerBranchRequest):
+    """Set the configuration data for a branch.
+
+    New in 2.5.
+    """
+
+    def do_with_branch(self, branch, branch_token, repo_token):
+        """Set the content of branch.conf.
+
+        The body is not utf8 decoded - its the literal bytestream for disk.
+        """
+        self._branch = branch
+        self._branch_token = branch_token
+        self._repo_token = repo_token
+        # Signal we want a body
+        return None
+
+    def do_body(self, body_bytes):
+        self._branch.repository.lock_write(token=self._repo_token)
+        try:
+            self._branch.lock_write(token=self._branch_token)
+            try:
+                self._branch.control_transport.put_bytes(
+                    'branch.conf', body_bytes)
+            finally:
+                self._branch.unlock()
+        finally:
+            self._branch.repository.unlock()
+        return SuccessfulSmartServerResponse(('ok', ))
+
+
 class SmartServerBranchGetParent(SmartServerBranchRequest):
 
     def do_with_branch(self, branch):
@@ -171,8 +213,16 @@
         The revision list is returned as the body content,
         with each revision utf8 encoded and \x00 joined.
         """
+        branch.lock_read()
+        try:
+            graph = branch.repository.get_graph()
+            stop_revisions = (None, _mod_revision.NULL_REVISION)
+            history = list(graph.iter_lefthand_ancestry(
+                branch.last_revision(), stop_revisions))
+        finally:
+            branch.unlock()
         return SuccessfulSmartServerResponse(
-            ('ok', ), ('\x00'.join(branch.revision_history())))
+            ('ok', ), ('\x00'.join(reversed(history))))
 
 
 class SmartServerBranchRequestLastRevisionInfo(SmartServerBranchRequest):
@@ -186,6 +236,23 @@
         return SuccessfulSmartServerResponse(('ok', str(revno), last_revision))
 
 
+class SmartServerBranchRequestRevisionIdToRevno(SmartServerBranchRequest):
+
+    def do_with_branch(self, branch, revid):
+        """Return branch.revision_id_to_revno().
+
+        New in 2.5.
+
+        The revno is encoded in decimal, the revision_id is encoded as utf8.
+        """
+        try:
+            dotted_revno = branch.revision_id_to_dotted_revno(revid)
+        except errors.NoSuchRevision:
+            return FailedSmartServerResponse(('NoSuchRevision', revid))
+        return SuccessfulSmartServerResponse(
+            ('ok', ) + tuple(map(str, dotted_revno)))
+
+
 class SmartServerSetTipRequest(SmartServerLockedBranchRequest):
     """Base class for handling common branch request logic for requests that
     update the branch tip.
@@ -368,3 +435,15 @@
         branch.unlock()
         return SuccessfulSmartServerResponse(('ok',))
 
+
+class SmartServerBranchRequestGetPhysicalLockStatus(SmartServerBranchRequest):
+    """Get the physical lock status for a branch.
+
+    New in 2.5.
+    """
+
+    def do_with_branch(self, branch):
+        if branch.get_physical_lock_status():
+            return SuccessfulSmartServerResponse(('yes',))
+        else:
+            return SuccessfulSmartServerResponse(('no',))

=== modified file 'bzrlib/smart/bzrdir.py'
--- bzrlib/smart/bzrdir.py	2011-05-19 09:32:38 +0000
+++ bzrlib/smart/bzrdir.py	2011-12-19 13:23:58 +0000
@@ -16,12 +16,12 @@
 
 """Server-side bzrdir related request implmentations."""
 
+from __future__ import absolute_import
 
 from bzrlib import branch, errors, repository, urlutils
 from bzrlib.bzrdir import (
     BzrDir,
     BzrDirFormat,
-    BzrDirMetaFormat1,
     BzrProber,
     )
 from bzrlib.controldir import (
@@ -121,6 +121,53 @@
         return '/'.join(segments)
 
 
+class SmartServerBzrDirRequestDestroyBranch(SmartServerRequestBzrDir):
+
+    def do_bzrdir_request(self, name=None):
+        """Destroy the branch with the specified name.
+
+        New in 2.5.0.
+        :return: On success, 'ok'.
+        """
+        try:
+            self._bzrdir.destroy_branch(name)
+        except errors.NotBranchError, e:
+            return FailedSmartServerResponse(('nobranch',))
+        return SuccessfulSmartServerResponse(('ok',))
+
+
+class SmartServerBzrDirRequestHasWorkingTree(SmartServerRequestBzrDir):
+
+    def do_bzrdir_request(self, name=None):
+        """Check whether there is a working tree present.
+
+        New in 2.5.0.
+
+        :return: If there is a working tree present, 'yes'.
+            Otherwise 'no'.
+        """
+        if self._bzrdir.has_workingtree():
+            return SuccessfulSmartServerResponse(('yes', ))
+        else:
+            return SuccessfulSmartServerResponse(('no', ))
+
+
+class SmartServerBzrDirRequestDestroyRepository(SmartServerRequestBzrDir):
+
+    def do_bzrdir_request(self, name=None):
+        """Destroy the repository.
+
+        New in 2.5.0.
+
+        :return: On success, 'ok'.
+        """
+        try:
+            self._bzrdir.destroy_repository()
+        except errors.NoRepositoryPresent, e:
+            return FailedSmartServerResponse(('norepository',))
+        return SuccessfulSmartServerResponse(('ok',))
+
+
 class SmartServerBzrDirRequestCloningMetaDir(SmartServerRequestBzrDir):
 
     def do_bzrdir_request(self, require_stacking):
@@ -150,9 +197,7 @@
         control_format = self._bzrdir.cloning_metadir(
             require_stacking=require_stacking)
         control_name = control_format.network_name()
-        # XXX: There should be a method that tells us that the format does/does
-        # not have subformats.
-        if isinstance(control_format, BzrDirMetaFormat1):
+        if not control_format.fixed_components:
             branch_name = ('branch',
                 control_format.get_branch_format().network_name())
             repository_name = control_format.repository_format.network_name()
@@ -164,6 +209,41 @@
             branch_name))
 
 
+class SmartServerBzrDirRequestCheckoutMetaDir(SmartServerRequestBzrDir):
+    """Get the format to use for checkouts.
+
+    New in 2.5.
+
+    :return: on success, a 3-tuple of network names for (control,
+        repository, branch) directories, where '' signifies "not present".
+        If this BzrDir contains a branch reference then this will fail with
+        BranchReference; clients should resolve branch references before
+        calling this RPC (they should not try to create a checkout of a
+        checkout).
+    """
+
+    def do_bzrdir_request(self):
+        try:
+            branch_ref = self._bzrdir.get_branch_reference()
+        except errors.NotBranchError:
+            branch_ref = None
+        if branch_ref is not None:
+            # The server shouldn't try to resolve references, and it quite
+            # possibly can't reach them anyway.  The client needs to resolve
+            # the branch reference to determine the cloning_metadir.
+            return FailedSmartServerResponse(('BranchReference',))
+        control_format = self._bzrdir.checkout_metadir()
+        control_name = control_format.network_name()
+        if not control_format.fixed_components:
+            branch_name = control_format.get_branch_format().network_name()
+            repo_name = control_format.repository_format.network_name()
+        else:
+            branch_name = ''
+            repo_name = ''
+        return SuccessfulSmartServerResponse(
+            (control_name, repo_name, branch_name))
+
+
 class SmartServerRequestCreateBranch(SmartServerRequestBzrDir):
 
     def do(self, path, network_name):

=== modified file 'bzrlib/smart/client.py'
--- bzrlib/smart/client.py	2011-03-30 11:45:54 +0000
+++ bzrlib/smart/client.py	2011-12-18 15:28:38 +0000
@@ -14,12 +14,20 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
+from bzrlib import lazy_import
+lazy_import.lazy_import(globals(), """
+from bzrlib.smart import request as _mod_request
+""")
+
 import bzrlib
 from bzrlib.smart import message, protocol
-from bzrlib.trace import warning
 from bzrlib import (
+    debug,
     errors,
     hooks,
+    trace,
     )
 
 
@@ -39,93 +47,12 @@
     def __repr__(self):
         return '%s(%r)' % (self.__class__.__name__, self._medium)
 
-    def _send_request(self, protocol_version, method, args, body=None,
-                      readv_body=None, body_stream=None):
-        encoder, response_handler = self._construct_protocol(
-            protocol_version)
-        encoder.set_headers(self._headers)
-        if body is not None:
-            if readv_body is not None:
-                raise AssertionError(
-                    "body and readv_body are mutually exclusive.")
-            if body_stream is not None:
-                raise AssertionError(
-                    "body and body_stream are mutually exclusive.")
-            encoder.call_with_body_bytes((method, ) + args, body)
-        elif readv_body is not None:
-            if body_stream is not None:
-                raise AssertionError(
-                    "readv_body and body_stream are mutually exclusive.")
-            encoder.call_with_body_readv_array((method, ) + args, readv_body)
-        elif body_stream is not None:
-            encoder.call_with_body_stream((method, ) + args, body_stream)
-        else:
-            encoder.call(method, *args)
-        return response_handler
-
-    def _run_call_hooks(self, method, args, body, readv_body):
-        if not _SmartClient.hooks['call']:
-            return
-        params = CallHookParams(method, args, body, readv_body, self._medium)
-        for hook in _SmartClient.hooks['call']:
-            hook(params)
-
     def _call_and_read_response(self, method, args, body=None, readv_body=None,
             body_stream=None, expect_response_body=True):
-        self._run_call_hooks(method, args, body, readv_body)
-        if self._medium._protocol_version is not None:
-            response_handler = self._send_request(
-                self._medium._protocol_version, method, args, body=body,
-                readv_body=readv_body, body_stream=body_stream)
-            return (response_handler.read_response_tuple(
-                        expect_body=expect_response_body),
-                    response_handler)
-        else:
-            for protocol_version in [3, 2]:
-                if protocol_version == 2:
-                    # If v3 doesn't work, the remote side is older than 1.6.
-                    self._medium._remember_remote_is_before((1, 6))
-                response_handler = self._send_request(
-                    protocol_version, method, args, body=body,
-                    readv_body=readv_body, body_stream=body_stream)
-                try:
-                    response_tuple = response_handler.read_response_tuple(
-                        expect_body=expect_response_body)
-                except errors.UnexpectedProtocolVersionMarker, err:
-                    # TODO: We could recover from this without disconnecting if
-                    # we recognise the protocol version.
-                    warning(
-                        'Server does not understand Bazaar network protocol %d,'
-                        ' reconnecting.  (Upgrade the server to avoid this.)'
-                        % (protocol_version,))
-                    self._medium.disconnect()
-                    continue
-                except errors.ErrorFromSmartServer:
-                    # If we received an error reply from the server, then it
-                    # must be ok with this protocol version.
-                    self._medium._protocol_version = protocol_version
-                    raise
-                else:
-                    self._medium._protocol_version = protocol_version
-                    return response_tuple, response_handler
-            raise errors.SmartProtocolError(
-                'Server is not a Bazaar server: ' + str(err))
-
-    def _construct_protocol(self, version):
-        request = self._medium.get_request()
-        if version == 3:
-            request_encoder = protocol.ProtocolThreeRequester(request)
-            response_handler = message.ConventionalResponseHandler()
-            response_proto = protocol.ProtocolThreeDecoder(
-                response_handler, expect_version_marker=True)
-            response_handler.setProtoAndMediumRequest(response_proto, request)
-        elif version == 2:
-            request_encoder = protocol.SmartClientRequestProtocolTwo(request)
-            response_handler = request_encoder
-        else:
-            request_encoder = protocol.SmartClientRequestProtocolOne(request)
-            response_handler = request_encoder
-        return request_encoder, response_handler
+        request = _SmartClientRequest(self, method, args, body=body,
+            readv_body=readv_body, body_stream=body_stream,
+            expect_response_body=expect_response_body)
+        return request.call_and_read_response()
 
     def call(self, method, *args):
         """Call a method on the remote server."""
@@ -191,6 +118,203 @@
         return self._medium.remote_path_from_transport(transport)
 
 
+class _SmartClientRequest(object):
+    """Encapsulate the logic for a single request.
+
+    This class handles things like reconnecting and sending the request a
+    second time when the connection is reset in the middle. It also handles the
+    multiple requests that get made if we don't know what protocol the server
+    supports yet.
+
+    Generally, you build up one of these objects, passing in the arguments that
+    you want to send to the server, and then use 'call_and_read_response' to
+    get the response from the server.
+    """
+
+    def __init__(self, client, method, args, body=None, readv_body=None,
+                 body_stream=None, expect_response_body=True):
+        self.client = client
+        self.method = method
+        self.args = args
+        self.body = body
+        self.readv_body = readv_body
+        self.body_stream = body_stream
+        self.expect_response_body = expect_response_body
+
+    def call_and_read_response(self):
+        """Send the request to the server, and read the initial response.
+
+        This doesn't read all of the body content of the response, instead it
+        returns (response_tuple, response_handler). response_tuple is the 'ok',
+        or 'error' information, and 'response_handler' can be used to get the
+        content stream out.
+        """
+        self._run_call_hooks()
+        protocol_version = self.client._medium._protocol_version
+        if protocol_version is None:
+            return self._call_determining_protocol_version()
+        else:
+            return self._call(protocol_version)
+
+    def _is_safe_to_send_twice(self):
+        """Check if the current method is re-entrant safe."""
+        if self.body_stream is not None or 'noretry' in debug.debug_flags:
+            # We can't restart a body stream that has already been consumed.
+            return False
+        request_type = _mod_request.request_handlers.get_info(self.method)
+        if request_type in ('read', 'idem', 'semi'):
+            return True
+        # If we have gotten this far, 'stream' cannot be retried, because we
+        # already consumed the local stream.
+        if request_type in ('semivfs', 'mutate', 'stream'):
+            return False
+        trace.mutter('Unknown request type: %s for method %s'
+                     % (request_type, self.method))
+        return False
+
+    def _run_call_hooks(self):
+        if not _SmartClient.hooks['call']:
+            return
+        params = CallHookParams(self.method, self.args, self.body,
+                                self.readv_body, self.client._medium)
+        for hook in _SmartClient.hooks['call']:
+            hook(params)
+
+    def _call(self, protocol_version):
+        """We know the protocol version.
+
+        So this just sends the request, and then reads the response. This is
+        where the code will be to retry requests if the connection is closed.
+        """
+        response_handler = self._send(protocol_version)
+        try:
+            response_tuple = response_handler.read_response_tuple(
+                expect_body=self.expect_response_body)
+        except errors.ConnectionReset, e:
+            self.client._medium.reset()
+            if not self._is_safe_to_send_twice():
+                raise
+            trace.warning('ConnectionReset reading response for %r, retrying'
+                          % (self.method,))
+            trace.log_exception_quietly()
+            encoder, response_handler = self._construct_protocol(
+                protocol_version)
+            self._send_no_retry(encoder)
+            response_tuple = response_handler.read_response_tuple(
+                expect_body=self.expect_response_body)
+        return (response_tuple, response_handler)
+
+    def _call_determining_protocol_version(self):
+        """Determine what protocol the remote server supports.
+
+        We do this by placing a request in the most recent protocol, and
+        handling the UnexpectedProtocolVersionMarker from the server.
+        """
+        for protocol_version in [3, 2]:
+            if protocol_version == 2:
+                # If v3 doesn't work, the remote side is older than 1.6.
+                self.client._medium._remember_remote_is_before((1, 6))
+            try:
+                response_tuple, response_handler = self._call(protocol_version)
+            except errors.UnexpectedProtocolVersionMarker, err:
+                # TODO: We could recover from this without disconnecting if
+                # we recognise the protocol version.
+                trace.warning(
+                    'Server does not understand Bazaar network protocol %d,'
+                    ' reconnecting.  (Upgrade the server to avoid this.)'
+                    % (protocol_version,))
+                self.client._medium.disconnect()
+                continue
+            except errors.ErrorFromSmartServer:
+                # If we received an error reply from the server, then it
+                # must be ok with this protocol version.
+                self.client._medium._protocol_version = protocol_version
+                raise
+            else:
+                self.client._medium._protocol_version = protocol_version
+                return response_tuple, response_handler
+        raise errors.SmartProtocolError(
+            'Server is not a Bazaar server: ' + str(err))
+
+    def _construct_protocol(self, version):
+        """Build the encoding stack for a given protocol version."""
+        request = self.client._medium.get_request()
+        if version == 3:
+            request_encoder = protocol.ProtocolThreeRequester(request)
+            response_handler = message.ConventionalResponseHandler()
+            response_proto = protocol.ProtocolThreeDecoder(
+                response_handler, expect_version_marker=True)
+            response_handler.setProtoAndMediumRequest(response_proto, request)
+        elif version == 2:
+            request_encoder = protocol.SmartClientRequestProtocolTwo(request)
+            response_handler = request_encoder
+        else:
+            request_encoder = protocol.SmartClientRequestProtocolOne(request)
+            response_handler = request_encoder
+        return request_encoder, response_handler
+
+    def _send(self, protocol_version):
+        """Encode the request, and send it to the server.
+
+        This will retry a request if we get a ConnectionReset while sending the
+        request to the server. (Unless we have a body_stream that we have
+        already started consuming, since we can't restart body_streams)
+
+        :return: response_handler as defined by _construct_protocol
+        """
+        encoder, response_handler = self._construct_protocol(protocol_version)
+        try:
+            self._send_no_retry(encoder)
+        except errors.ConnectionReset, e:
+            # If we fail during the _send_no_retry phase, then we can
+            # be confident that the server did not get our request, because we
+            # haven't started waiting for the reply yet. So try the request
+            # again. We only issue a single retry, because if the connection
+            # really is down, there is no reason to loop endlessly.
+
+            # Connection is dead, so close our end of it.
+            self.client._medium.reset()
+            if (('noretry' in debug.debug_flags)
+                or (self.body_stream is not None
+                    and encoder.body_stream_started)):
+                # We can't restart a body_stream that has been partially
+                # consumed, so we don't retry.
+                # Note: We don't have to worry about
+                #   SmartClientRequestProtocolOne or Two, because they don't
+                #   support client-side body streams.
+                raise
+            trace.warning('ConnectionReset calling %r, retrying'
+                          % (self.method,))
+            trace.log_exception_quietly()
+            encoder, response_handler = self._construct_protocol(
+                protocol_version)
+            self._send_no_retry(encoder)
+        return response_handler
+
+    def _send_no_retry(self, encoder):
+        """Just encode the request and try to send it."""
+        encoder.set_headers(self.client._headers)
+        if self.body is not None:
+            if self.readv_body is not None:
+                raise AssertionError(
+                    "body and readv_body are mutually exclusive.")
+            if self.body_stream is not None:
+                raise AssertionError(
+                    "body and body_stream are mutually exclusive.")
+            encoder.call_with_body_bytes((self.method, ) + self.args, self.body)
+        elif self.readv_body is not None:
+            if self.body_stream is not None:
+                raise AssertionError(
+                    "readv_body and body_stream are mutually exclusive.")
+            encoder.call_with_body_readv_array((self.method, ) + self.args,
+                                               self.readv_body)
+        elif self.body_stream is not None:
+            encoder.call_with_body_stream((self.method, ) + self.args,
+                                          self.body_stream)
+        else:
+            encoder.call(self.method, *self.args)
+
+
 class SmartClientHooks(hooks.Hooks):
 
     def __init__(self):

=== modified file 'bzrlib/smart/medium.py'
--- bzrlib/smart/medium.py	2011-08-21 13:45:27 +0000
+++ bzrlib/smart/medium.py	2011-12-24 09:59:02 +0000
@@ -24,13 +24,17 @@
 bzrlib/transport/smart/__init__.py.
 """
 
+from __future__ import absolute_import
+
+import errno
 import os
 import sys
-import urllib
+import time
 
 import bzrlib
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
+import select
 import socket
 import thread
 import weakref
@@ -39,10 +43,12 @@
     debug,
     errors,
     trace,
+    transport,
     ui,
     urlutils,
     )
-from bzrlib.smart import client, protocol, request, vfs
+from bzrlib.i18n import gettext
+from bzrlib.smart import client, protocol, request, signals, vfs
 from bzrlib.transport import ssh
 """)
 from bzrlib import osutils
@@ -175,6 +181,14 @@
         ui.ui_factory.report_transport_activity(self, bytes, direction)
 
 
+_bad_file_descriptor = (errno.EBADF,)
+if sys.platform == 'win32':
+    # Given on Windows if you pass a closed socket to select.select. Probably
+    # also given if you pass a file handle to select.
+    WSAENOTSOCK = 10038
+    _bad_file_descriptor += (WSAENOTSOCK,)
+
+
 class SmartServerStreamMedium(SmartMedium):
     """Handles smart commands coming over a stream.
 
@@ -193,7 +207,9 @@
         the stream.  See also the _push_back method.
     """
 
-    def __init__(self, backing_transport, root_client_path='/'):
+    _timer = time.time
+
+    def __init__(self, backing_transport, root_client_path='/', timeout=None):
         """Construct new server.
 
         :param backing_transport: Transport for the directory served.
@@ -202,6 +218,10 @@
         self.backing_transport = backing_transport
         self.root_client_path = root_client_path
         self.finished = False
+        if timeout is None:
+            raise AssertionError('You must supply a timeout.')
+        self._client_timeout = timeout
+        self._client_poll_timeout = min(timeout / 10.0, 1.0)
         SmartMedium.__init__(self)
 
     def serve(self):
@@ -213,9 +233,39 @@
             while not self.finished:
                 server_protocol = self._build_protocol()
                 self._serve_one_request(server_protocol)
+        except errors.ConnectionTimeout, e:
+            trace.note('%s' % (e,))
+            trace.log_exception_quietly()
+            self._disconnect_client()
+            # We reported it, no reason to make a big fuss.
+            return
         except Exception, e:
             stderr.write("%s terminating on exception %s\n" % (self, e))
             raise
+        self._disconnect_client()
+
+    def _stop_gracefully(self):
+        """When we finish this message, stop looking for more."""
+        trace.mutter('Stopping %s' % (self,))
+        self.finished = True
+
+    def _disconnect_client(self):
+        """Close the current connection. We stopped due to a timeout/etc."""
+        # The default implementation is a no-op, because that is all we used to
+        # do when disconnecting from a client. I suppose we never had the
+        # *server* initiate a disconnect, before
+
+    def _wait_for_bytes_with_timeout(self, timeout_seconds):
+        """Wait for more bytes to be read, but timeout if none available.
+
+        This allows us to detect idle connections, and stop trying to read from
+        them, without setting the socket itself to non-blocking. This also
+        allows us to specify when we watch for idle timeouts.
+
+        :return: Did we timeout? (True if we timed out, False if there is data
+            to be read)
+        """
+        raise NotImplementedError(self._wait_for_bytes_with_timeout)
 
     def _build_protocol(self):
         """Identifies the version of the incoming request, and returns an
@@ -226,6 +276,10 @@
 
         :returns: a SmartServerRequestProtocol.
         """
+        self._wait_for_bytes_with_timeout(self._client_timeout)
+        if self.finished:
+            # We're stopping, so don't try to do any more work
+            return None
         bytes = self._get_line()
         protocol_factory, unused_bytes = _get_protocol_factory_for_bytes(bytes)
         protocol = protocol_factory(
@@ -233,11 +287,44 @@
         protocol.accept_bytes(unused_bytes)
         return protocol
 
+    def _wait_on_descriptor(self, fd, timeout_seconds):
+        """select() on a file descriptor, waiting for nonblocking read()
+
+        This will raise a ConnectionTimeout exception if we do not get a
+        readable handle before timeout_seconds.
+        :return: None
+        """
+        t_end = self._timer() + timeout_seconds
+        poll_timeout = min(timeout_seconds, self._client_poll_timeout)
+        rs = xs = None
+        while not rs and not xs and self._timer() < t_end:
+            if self.finished:
+                return
+            try:
+                rs, _, xs = select.select([fd], [], [fd], poll_timeout)
+            except (select.error, socket.error) as e:
+                err = getattr(e, 'errno', None)
+                if err is None and getattr(e, 'args', None) is not None:
+                    # select.error doesn't have 'errno', it just has args[0]
+                    err = e.args[0]
+                if err in _bad_file_descriptor:
+                    return # Not a socket indicates read() will fail
+                elif err == errno.EINTR:
+                    # Interrupted, keep looping.
+                    continue
+                raise
+        if rs or xs:
+            return
+        raise errors.ConnectionTimeout('disconnecting client after %.1f seconds'
+                                       % (timeout_seconds,))
+
     def _serve_one_request(self, protocol):
         """Read one request from input, process, send back a response.
 
         :param protocol: a SmartServerRequestProtocol.
         """
+        if protocol is None:
+            return
         try:
             self._serve_one_request_unguarded(protocol)
         except KeyboardInterrupt:
@@ -259,16 +346,30 @@
 
 class SmartServerSocketStreamMedium(SmartServerStreamMedium):
 
-    def __init__(self, sock, backing_transport, root_client_path='/'):
+    def __init__(self, sock, backing_transport, root_client_path='/',
+                 timeout=None):
         """Constructor.
 
         :param sock: the socket the server will read from.  It will be put
             into blocking mode.
         """
         SmartServerStreamMedium.__init__(
-            self, backing_transport, root_client_path=root_client_path)
+            self, backing_transport, root_client_path=root_client_path,
+            timeout=timeout)
         sock.setblocking(True)
         self.socket = sock
+        # Get the getpeername now, as we might be closed later when we care.
+        try:
+            self._client_info = sock.getpeername()
+        except socket.error:
+            self._client_info = '<unknown>'
+
+    def __str__(self):
+        return '%s(client=%s)' % (self.__class__.__name__, self._client_info)
+
+    def __repr__(self):
+        return '%s.%s(client=%s)' % (self.__module__, self.__class__.__name__,
+            self._client_info)
 
     def _serve_one_request_unguarded(self, protocol):
         while protocol.next_read_size():
@@ -283,6 +384,22 @@
 
         self._push_back(protocol.unused_data)
 
+    def _disconnect_client(self):
+        """Close the current connection. We stopped due to a timeout/etc."""
+        self.socket.close()
+
+    def _wait_for_bytes_with_timeout(self, timeout_seconds):
+        """Wait for more bytes to be read, but timeout if none available.
+
+        This allows us to detect idle connections, and stop trying to read from
+        them, without setting the socket itself to non-blocking. This also
+        allows us to specify when we watch for idle timeouts.
+
+        :return: None, this will raise ConnectionTimeout if we time out before
+            data is available.
+        """
+        return self._wait_on_descriptor(self.socket, timeout_seconds)
+
     def _read_bytes(self, desired_count):
         return osutils.read_bytes_from_socket(
             self.socket, self._report_activity)
@@ -305,14 +422,15 @@
 
 class SmartServerPipeStreamMedium(SmartServerStreamMedium):
 
-    def __init__(self, in_file, out_file, backing_transport):
+    def __init__(self, in_file, out_file, backing_transport, timeout=None):
         """Construct new server.
 
         :param in_file: Python file from which requests can be read.
         :param out_file: Python file to write responses.
         :param backing_transport: Transport for the directory served.
         """
-        SmartServerStreamMedium.__init__(self, backing_transport)
+        SmartServerStreamMedium.__init__(self, backing_transport,
+            timeout=timeout)
         if sys.platform == 'win32':
             # force binary mode for files
             import msvcrt
@@ -323,6 +441,17 @@
         self._in = in_file
         self._out = out_file
 
+    def serve(self):
+        """See SmartServerStreamMedium.serve"""
+        # This is the regular serve, except it adds signal trapping for soft
+        # shutdown.
+        stop_gracefully = self._stop_gracefully
+        signals.register_on_hangup(id(self), stop_gracefully)
+        try:
+            return super(SmartServerPipeStreamMedium, self).serve()
+        finally:
+            signals.unregister_on_hangup(id(self))
+
     def _serve_one_request_unguarded(self, protocol):
         while True:
             # We need to be careful not to read past the end of the current
@@ -341,6 +470,27 @@
                 return
             protocol.accept_bytes(bytes)
 
+    def _disconnect_client(self):
+        self._in.close()
+        self._out.flush()
+        self._out.close()
+
+    def _wait_for_bytes_with_timeout(self, timeout_seconds):
+        """Wait for more bytes to be read, but timeout if none available.
+
+        This allows us to detect idle connections, and stop trying to read from
+        them, without setting the socket itself to non-blocking. This also
+        allows us to specify when we watch for idle timeouts.
+
+        :return: None, this will raise ConnectionTimeout if we time out before
+            data is available.
+        """
+        if (getattr(self._in, 'fileno', None) is None
+            or sys.platform == 'win32'):
+            # You can't select() file descriptors on Windows.
+            return
+        return self._wait_on_descriptor(self._in, timeout_seconds)
+
     def _read_bytes(self, desired_count):
         return self._in.read(desired_count)
 
@@ -561,8 +711,8 @@
         value['count'] = 0
         value['vfs_count'] = 0
         if count != 0:
-            trace.note('HPSS calls: %d (%d vfs) %s',
-                       count, vfs_count, medium_repr)
+            trace.note(gettext('HPSS calls: {0} ({1} vfs) {2}').format(
+                       count, vfs_count, medium_repr))
 
     def flush_all(self):
         for ref in list(self.counts.keys()):
@@ -692,7 +842,7 @@
         """
         medium_base = urlutils.join(self.base, '/')
         rel_url = urlutils.relative_url(medium_base, transport.base)
-        return urllib.unquote(rel_url)
+        return urlutils.unquote(rel_url)
 
 
 class SmartClientStreamMedium(SmartClientMedium):
@@ -733,6 +883,14 @@
         """
         return SmartClientStreamMediumRequest(self)
 
+    def reset(self):
+        """We have been disconnected, reset current state.
+
+        This resets things like _current_request and connected state.
+        """
+        self.disconnect()
+        self._current_request = None
+
 
 class SmartSimplePipesClientMedium(SmartClientStreamMedium):
     """A client medium using simple pipes.
@@ -747,11 +905,20 @@
 
     def _accept_bytes(self, bytes):
         """See SmartClientStreamMedium.accept_bytes."""
-        self._writeable_pipe.write(bytes)
+        try:
+            self._writeable_pipe.write(bytes)
+        except IOError, e:
+            if e.errno in (errno.EINVAL, errno.EPIPE):
+                raise errors.ConnectionReset(
+                    "Error trying to write to subprocess:\n%s" % (e,))
+            raise
         self._report_activity(len(bytes), 'write')
 
     def _flush(self):
         """See SmartClientStreamMedium._flush()."""
+        # Note: If flush were to fail, we'd like to raise ConnectionReset, etc.
+        #       However, testing shows that even when the child process is
+        #       gone, this doesn't error.
         self._writeable_pipe.flush()
 
     def _read_bytes(self, count):
@@ -776,8 +943,8 @@
 
 class SmartSSHClientMedium(SmartClientStreamMedium):
     """A client medium using SSH.
-    
-    It delegates IO to a SmartClientSocketMedium or
+
+    It delegates IO to a SmartSimplePipesClientMedium or
     SmartClientAlreadyConnectedSocketMedium (depending on platform).
     """
 
@@ -805,10 +972,14 @@
             maybe_port = ''
         else:
             maybe_port = ':%s' % self._ssh_params.port
-        return "%s(%s://%s@%s%s/)" % (
+        if self._ssh_params.username is None:
+            maybe_user = ''
+        else:
+            maybe_user = '%s@' % self._ssh_params.username
+        return "%s(%s://%s%s%s/)" % (
             self.__class__.__name__,
             self._scheme,
-            self._ssh_params.username,
+            maybe_user,
             self._ssh_params.host,
             maybe_port)
 
@@ -851,6 +1022,8 @@
             raise AssertionError(
                 "Unexpected io_kind %r from %r"
                 % (io_kind, self._ssh_connection))
+        for hook in transport.Transport.hooks["post_connect"]:
+            hook(self)
 
     def _flush(self):
         """See SmartClientStreamMedium._flush()."""
@@ -959,6 +1132,8 @@
             raise errors.ConnectionError("failed to connect to %s:%d: %s" %
                     (self._host, port, err_msg))
         self._connected = True
+        for hook in transport.Transport.hooks["post_connect"]:
+            hook(self)
 
 
 class SmartClientAlreadyConnectedSocketMedium(SmartClientSocketMedium):
@@ -1016,5 +1191,3 @@
         This invokes self._medium._flush to ensure all bytes are transmitted.
         """
         self._medium._flush()
-
-

=== modified file 'bzrlib/smart/message.py'
--- bzrlib/smart/message.py	2011-03-02 20:16:03 +0000
+++ bzrlib/smart/message.py	2011-12-18 15:28:38 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 import collections
 from cStringIO import StringIO
 

=== modified file 'bzrlib/smart/packrepository.py'
--- bzrlib/smart/packrepository.py	2009-03-23 14:59:43 +0000
+++ bzrlib/smart/packrepository.py	2011-12-18 15:28:38 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 """Server-side pack repository related request implmentations."""
 
 from bzrlib.smart.request import (

=== modified file 'bzrlib/smart/protocol.py'
--- bzrlib/smart/protocol.py	2011-05-19 09:32:38 +0000
+++ bzrlib/smart/protocol.py	2011-12-19 13:23:58 +0000
@@ -18,12 +18,13 @@
 client and server.
 """
 
+from __future__ import absolute_import
+
 import collections
 from cStringIO import StringIO
 import struct
 import sys
 import thread
-import threading
 import time
 
 import bzrlib
@@ -1081,9 +1082,6 @@
         self._real_write_func = write_func
 
     def _write_func(self, bytes):
-        # TODO: It is probably more appropriate to use sum(map(len, _buf))
-        #       for total number of bytes to write, rather than buffer based on
-        #       the number of write() calls
         # TODO: Another possibility would be to turn this into an async model.
         #       Where we let another thread know that we have some bytes if
         #       they want it, but we don't actually block for it
@@ -1292,6 +1290,7 @@
         _ProtocolThreeEncoder.__init__(self, medium_request.accept_bytes)
         self._medium_request = medium_request
         self._headers = {}
+        self.body_stream_started = None
 
     def set_headers(self, headers):
         self._headers = headers.copy()
@@ -1357,6 +1356,7 @@
             if path is not None:
                 mutter('                  (to %s)', path)
             self._request_start_time = osutils.timer_func()
+        self.body_stream_started = False
         self._write_protocol_version()
         self._write_headers(self._headers)
         self._write_structure(args)
@@ -1364,6 +1364,9 @@
         #       have finished sending the stream.  We would notice at the end
         #       anyway, but if the medium can deliver it early then it's good
         #       to short-circuit the whole request...
+        # Provoke any ConnectionReset failures before we start the body stream.
+        self.flush()
+        self.body_stream_started = True
         for exc_info, part in _iter_with_errors(stream):
             if exc_info is not None:
                 # Iterating the stream failed.  Cleanly abort the request.

=== modified file 'bzrlib/smart/repository.py'
--- bzrlib/smart/repository.py	2010-11-26 06:31:54 +0000
+++ bzrlib/smart/repository.py	2011-12-19 13:23:58 +0000
@@ -14,7 +14,9 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-"""Server-side repository related request implmentations."""
+"""Server-side repository related request implementations."""
+
+from __future__ import absolute_import
 
 import bz2
 import os
@@ -22,14 +24,19 @@
 import sys
 import tempfile
 import threading
+import zlib
 
 from bzrlib import (
     bencode,
     errors,
-    graph,
+    estimate_compressed_size,
+    inventory as _mod_inventory,
+    inventory_delta,
     osutils,
     pack,
+    trace,
     ui,
+    vf_search,
     )
 from bzrlib.bzrdir import BzrDir
 from bzrlib.smart.request import (
@@ -40,6 +47,7 @@
 from bzrlib.repository import _strip_NULL_ghosts, network_format_registry
 from bzrlib import revision as _mod_revision
 from bzrlib.versionedfile import (
+    ChunkedContentFactory,
     NetworkRecordStream,
     record_to_fulltext_bytes,
     )
@@ -82,11 +90,11 @@
             they expected and get it from elsewhere.
         """
         if search_bytes == 'everything':
-            return graph.EverythingResult(repository), None
+            return vf_search.EverythingResult(repository), None
         lines = search_bytes.split('\n')
         if lines[0] == 'ancestry-of':
             heads = lines[1:]
-            search_result = graph.PendingAncestryResult(heads, repository)
+            search_result = vf_search.PendingAncestryResult(heads, repository)
             return search_result, None
         elif lines[0] == 'search':
             return self.recreate_search_from_recipe(repository, lines[1:],
@@ -116,15 +124,16 @@
                 except StopIteration:
                     break
                 search.stop_searching_any(exclude_keys.intersection(next_revs))
-            search_result = search.get_result()
-            if (not discard_excess and
-                search_result.get_recipe()[3] != revision_count):
+            (started_keys, excludes, included_keys) = search.get_state()
+            if (not discard_excess and len(included_keys) != revision_count):
                 # we got back a different amount of data than expected, this
                 # gets reported as NoSuchRevision, because less revisions
                 # indicates missing revisions, and more should never happen as
                 # the excludes list considers ghosts and ensures that ghost
                 # filling races are not a problem.
                 return (None, FailedSmartServerResponse(('NoSuchRevision',)))
+            search_result = vf_search.SearchResult(started_keys, excludes,
+                len(included_keys), included_keys)
             return (search_result, None)
         finally:
             repository.unlock()
@@ -142,6 +151,16 @@
             repository.unlock()
 
 
+class SmartServerRepositoryBreakLock(SmartServerRepositoryRequest):
+    """Break a repository lock."""
+
+    def do_repository_request(self, repository):
+        repository.break_lock()
+        return SuccessfulSmartServerResponse(('ok', ))
+
+
+_lsprof_count = 0
+
 class SmartServerRepositoryGetParentMap(SmartServerRepositoryRequest):
     """Bzr 1.2+ - get parent data for revisions during a graph search."""
 
@@ -180,27 +199,11 @@
         finally:
             repository.unlock()
 
-    def _do_repository_request(self, body_bytes):
-        repository = self._repository
-        revision_ids = set(self._revision_ids)
-        include_missing = 'include-missing:' in revision_ids
-        if include_missing:
-            revision_ids.remove('include-missing:')
-        body_lines = body_bytes.split('\n')
-        search_result, error = self.recreate_search_from_recipe(
-            repository, body_lines)
-        if error is not None:
-            return error
-        # TODO might be nice to start up the search again; but thats not
-        # written or tested yet.
-        client_seen_revs = set(search_result.get_keys())
-        # Always include the requested ids.
-        client_seen_revs.difference_update(revision_ids)
-        lines = []
-        repo_graph = repository.get_graph()
+    def _expand_requested_revs(self, repo_graph, revision_ids, client_seen_revs,
+                               include_missing, max_size=65536):
         result = {}
         queried_revs = set()
-        size_so_far = 0
+        estimator = estimate_compressed_size.ZLibEstimator(max_size)
         next_revs = revision_ids
         first_loop_done = False
         while next_revs:
@@ -228,21 +231,47 @@
                     # add parents to the result
                     result[encoded_id] = parents
                     # Approximate the serialized cost of this revision_id.
-                    size_so_far += 2 + len(encoded_id) + sum(map(len, parents))
+                    line = '%s %s\n' % (encoded_id, ' '.join(parents))
+                    estimator.add_content(line)
             # get all the directly asked for parents, and then flesh out to
             # 64K (compressed) or so. We do one level of depth at a time to
             # stay in sync with the client. The 250000 magic number is
             # estimated compression ratio taken from bzr.dev itself.
-            if self.no_extra_results or (
-                first_loop_done and size_so_far > 250000):
+            if self.no_extra_results or (first_loop_done and estimator.full()):
+                trace.mutter('size: %d, z_size: %d'
+                             % (estimator._uncompressed_size_added,
+                                estimator._compressed_size_added))
                 next_revs = set()
                 break
             # don't query things we've already queried
-            next_revs.difference_update(queried_revs)
+            next_revs = next_revs.difference(queried_revs)
             first_loop_done = True
+        return result
+
+    def _do_repository_request(self, body_bytes):
+        repository = self._repository
+        revision_ids = set(self._revision_ids)
+        include_missing = 'include-missing:' in revision_ids
+        if include_missing:
+            revision_ids.remove('include-missing:')
+        body_lines = body_bytes.split('\n')
+        search_result, error = self.recreate_search_from_recipe(
+            repository, body_lines)
+        if error is not None:
+            return error
+        # TODO might be nice to start up the search again; but thats not
+        # written or tested yet.
+        client_seen_revs = set(search_result.get_keys())
+        # Always include the requested ids.
+        client_seen_revs.difference_update(revision_ids)
+
+        repo_graph = repository.get_graph()
+        result = self._expand_requested_revs(repo_graph, revision_ids,
+                                             client_seen_revs, include_missing)
 
         # sorting trivially puts lexographically similar revision ids together.
         # Compression FTW.
+        lines = []
         for revision, parents in sorted(result.items()):
             lines.append(' '.join((revision, ) + tuple(parents)))
 
@@ -313,6 +342,20 @@
                 ('history-incomplete', earliest_revno, earliest_revid))
 
 
+class SmartServerRepositoryGetSerializerFormat(SmartServerRepositoryRequest):
+
+    def do_repository_request(self, repository):
+        """Return the serializer format for this repository.
+
+        New in 2.5.0.
+
+        :param repository: The repository to query
+        :return: A smart server response ('ok', FORMAT)
+        """
+        serializer = repository.get_serializer_format()
+        return SuccessfulSmartServerResponse(('ok', serializer))
+
+
 class SmartServerRequestHasRevision(SmartServerRepositoryRequest):
 
     def do_repository_request(self, repository, revision_id):
@@ -320,8 +363,8 @@
 
         :param repository: The repository to query in.
         :param revision_id: The utf8 encoded revision_id to lookup.
-        :return: A smart server response of ('ok', ) if the revision is
-            present.
+        :return: A smart server response of ('yes', ) if the revision is
+            present. ('no', ) if it is missing.
         """
         if repository.has_revision(revision_id):
             return SuccessfulSmartServerResponse(('yes', ))
@@ -329,6 +372,30 @@
             return SuccessfulSmartServerResponse(('no', ))
 
 
+class SmartServerRequestHasSignatureForRevisionId(
+        SmartServerRepositoryRequest):
+
+    def do_repository_request(self, repository, revision_id):
+        """Return ok if a signature is present for a revision.
+
+        Introduced in bzr 2.5.0.
+
+        :param repository: The repository to query in.
+        :param revision_id: The utf8 encoded revision_id to lookup.
+        :return: A smart server response of ('yes', ) if a
+            signature for the revision is present,
+            ('no', ) if it is missing.
+        """
+        try:
+            if repository.has_signature_for_revision_id(revision_id):
+                return SuccessfulSmartServerResponse(('yes', ))
+            else:
+                return SuccessfulSmartServerResponse(('no', ))
+        except errors.NoSuchRevision:
+            return FailedSmartServerResponse(
+                ('nosuchrevision', revision_id))
+
+
 class SmartServerRepositoryGatherStats(SmartServerRepositoryRequest):
 
     def do_repository_request(self, repository, revid, committers):
@@ -354,7 +421,11 @@
             decoded_committers = True
         else:
             decoded_committers = None
-        stats = repository.gather_stats(decoded_revision_id, decoded_committers)
+        try:
+            stats = repository.gather_stats(decoded_revision_id,
+                decoded_committers)
+        except errors.NoSuchRevision:
+            return FailedSmartServerResponse(('nosuchrevision', revid))
 
         body = ''
         if stats.has_key('committers'):
@@ -371,6 +442,28 @@
         return SuccessfulSmartServerResponse(('ok', ), body)
 
 
+class SmartServerRepositoryGetRevisionSignatureText(
+        SmartServerRepositoryRequest):
+    """Return the signature text of a revision.
+
+    New in 2.5.
+    """
+
+    def do_repository_request(self, repository, revision_id):
+        """Return the result of repository.get_signature_text().
+
+        :param repository: The repository to query in.
+        :return: A smart server response of with the signature text as
+            body.
+        """
+        try:
+            text = repository.get_signature_text(revision_id)
+        except errors.NoSuchRevision, err:
+            return FailedSmartServerResponse(
+                ('nosuchrevision', err.revision))
+        return SuccessfulSmartServerResponse(('ok', ), text)
+
+
 class SmartServerRepositoryIsShared(SmartServerRepositoryRequest):
 
     def do_repository_request(self, repository):
@@ -386,6 +479,23 @@
             return SuccessfulSmartServerResponse(('no', ))
 
 
+class SmartServerRepositoryMakeWorkingTrees(SmartServerRepositoryRequest):
+
+    def do_repository_request(self, repository):
+        """Return the result of repository.make_working_trees().
+
+        Introduced in bzr 2.5.0.
+
+        :param repository: The repository to query in.
+        :return: A smart server response of ('yes', ) if the repository uses
+            working trees, and ('no', ) if it is not.
+        """
+        if repository.make_working_trees():
+            return SuccessfulSmartServerResponse(('yes', ))
+        else:
+            return SuccessfulSmartServerResponse(('no', ))
+
+
 class SmartServerRepositoryLockWrite(SmartServerRepositoryRequest):
 
     def do_repository_request(self, repository, token=''):
@@ -672,6 +782,19 @@
         return SuccessfulSmartServerResponse(('ok',))
 
 
+class SmartServerRepositoryGetPhysicalLockStatus(SmartServerRepositoryRequest):
+    """Get the physical lock status for a repository.
+
+    New in 2.5.
+    """
+
+    def do_repository_request(self, repository):
+        if repository.get_physical_lock_status():
+            return SuccessfulSmartServerResponse(('yes', ))
+        else:
+            return SuccessfulSmartServerResponse(('no', ))
+
+
 class SmartServerRepositorySetMakeWorkingTrees(SmartServerRepositoryRequest):
 
     def do_repository_request(self, repository, str_bool_new_value):
@@ -840,3 +963,342 @@
         self.do_insert_stream_request(repository, resume_tokens)
 
 
+class SmartServerRepositoryAddSignatureText(SmartServerRepositoryRequest):
+    """Add a revision signature text.
+
+    New in 2.5.
+    """
+
+    def do_repository_request(self, repository, lock_token, revision_id,
+            *write_group_tokens):
+        """Add a revision signature text.
+
+        :param repository: Repository to operate on
+        :param lock_token: Lock token
+        :param revision_id: Revision for which to add signature
+        :param write_group_tokens: Write group tokens
+        """
+        self._lock_token = lock_token
+        self._revision_id = revision_id
+        self._write_group_tokens = write_group_tokens
+        return None
+
+    def do_body(self, body_bytes):
+        """Add a signature text.
+
+        :param body_bytes: GPG signature text
+        :return: SuccessfulSmartServerResponse with arguments 'ok' and
+            the list of new write group tokens.
+        """
+        self._repository.lock_write(token=self._lock_token)
+        try:
+            self._repository.resume_write_group(self._write_group_tokens)
+            try:
+                self._repository.add_signature_text(self._revision_id,
+                    body_bytes)
+            finally:
+                new_write_group_tokens = self._repository.suspend_write_group()
+        finally:
+            self._repository.unlock()
+        return SuccessfulSmartServerResponse(
+            ('ok', ) + tuple(new_write_group_tokens))
+
+
+class SmartServerRepositoryStartWriteGroup(SmartServerRepositoryRequest):
+    """Start a write group.
+
+    New in 2.5.
+    """
+
+    def do_repository_request(self, repository, lock_token):
+        """Start a write group."""
+        repository.lock_write(token=lock_token)
+        try:
+            repository.start_write_group()
+            try:
+                tokens = repository.suspend_write_group()
+            except errors.UnsuspendableWriteGroup:
+                return FailedSmartServerResponse(('UnsuspendableWriteGroup',))
+        finally:
+            repository.unlock()
+        return SuccessfulSmartServerResponse(('ok', tokens))
+
+
+class SmartServerRepositoryCommitWriteGroup(SmartServerRepositoryRequest):
+    """Commit a write group.
+
+    New in 2.5.
+    """
+
+    def do_repository_request(self, repository, lock_token,
+            write_group_tokens):
+        """Commit a write group."""
+        repository.lock_write(token=lock_token)
+        try:
+            try:
+                repository.resume_write_group(write_group_tokens)
+            except errors.UnresumableWriteGroup, e:
+                return FailedSmartServerResponse(
+                    ('UnresumableWriteGroup', e.write_groups, e.reason))
+            try:
+                repository.commit_write_group()
+            except:
+                write_group_tokens = repository.suspend_write_group()
+                # FIXME JRV 2011-11-19: What if the write_group_tokens
+                # have changed?
+                raise
+        finally:
+            repository.unlock()
+        return SuccessfulSmartServerResponse(('ok', ))
+
+
+class SmartServerRepositoryAbortWriteGroup(SmartServerRepositoryRequest):
+    """Abort a write group.
+
+    New in 2.5.
+    """
+
+    def do_repository_request(self, repository, lock_token, write_group_tokens):
+        """Abort a write group."""
+        repository.lock_write(token=lock_token)
+        try:
+            try:
+                repository.resume_write_group(write_group_tokens)
+            except errors.UnresumableWriteGroup, e:
+                return FailedSmartServerResponse(
+                    ('UnresumableWriteGroup', e.write_groups, e.reason))
+                repository.abort_write_group()
+        finally:
+            repository.unlock()
+        return SuccessfulSmartServerResponse(('ok', ))
+
+
+class SmartServerRepositoryCheckWriteGroup(SmartServerRepositoryRequest):
+    """Check that a write group is still valid.
+
+    New in 2.5.
+    """
+
+    def do_repository_request(self, repository, lock_token, write_group_tokens):
+        """Abort a write group."""
+        repository.lock_write(token=lock_token)
+        try:
+            try:
+                repository.resume_write_group(write_group_tokens)
+            except errors.UnresumableWriteGroup, e:
+                return FailedSmartServerResponse(
+                    ('UnresumableWriteGroup', e.write_groups, e.reason))
+            else:
+                repository.suspend_write_group()
+        finally:
+            repository.unlock()
+        return SuccessfulSmartServerResponse(('ok', ))
+
+
+class SmartServerRepositoryAllRevisionIds(SmartServerRepositoryRequest):
+    """Retrieve all of the revision ids in a repository.
+
+    New in 2.5.
+    """
+
+    def do_repository_request(self, repository):
+        revids = repository.all_revision_ids()
+        return SuccessfulSmartServerResponse(("ok", ), "\n".join(revids))
+
+
+class SmartServerRepositoryReconcile(SmartServerRepositoryRequest):
+    """Reconcile a repository.
+
+    New in 2.5.
+    """
+
+    def do_repository_request(self, repository, lock_token):
+        try:
+            repository.lock_write(token=lock_token)
+        except errors.TokenLockingNotSupported, e:
+            return FailedSmartServerResponse(
+                ('TokenLockingNotSupported', ))
+        try:
+            reconciler = repository.reconcile()
+        finally:
+            repository.unlock()
+        body = [
+            "garbage_inventories: %d\n" % reconciler.garbage_inventories,
+            "inconsistent_parents: %d\n" % reconciler.inconsistent_parents,
+            ]
+        return SuccessfulSmartServerResponse(('ok', ), "".join(body))
+
+
+class SmartServerRepositoryPack(SmartServerRepositoryRequest):
+    """Pack a repository.
+
+    New in 2.5.
+    """
+
+    def do_repository_request(self, repository, lock_token, clean_obsolete_packs):
+        self._repository = repository
+        self._lock_token = lock_token
+        if clean_obsolete_packs == 'True':
+            self._clean_obsolete_packs = True
+        else:
+            self._clean_obsolete_packs = False
+        return None
+
+    def do_body(self, body_bytes):
+        if body_bytes == "":
+            hint = None
+        else:
+            hint = body_bytes.splitlines()
+        self._repository.lock_write(token=self._lock_token)
+        try:
+            self._repository.pack(hint, self._clean_obsolete_packs)
+        finally:
+            self._repository.unlock()
+        return SuccessfulSmartServerResponse(("ok", ), )
+
+
+class SmartServerRepositoryIterFilesBytes(SmartServerRepositoryRequest):
+    """Iterate over the contents of files.
+
+    The client sends a list of desired files to stream, one
+    per line, and as tuples of file id and revision, separated by
+    \0.
+
+    The server replies with a stream. Each entry is preceded by a header,
+    which can either be:
+
+    * "ok\x00IDX\n" where IDX is the index of the entry in the desired files
+        list sent by the client. This header is followed by the contents of
+        the file, bzip2-compressed.
+    * "absent\x00FILEID\x00REVISION\x00IDX" to indicate a text is missing.
+        The client can then raise an appropriate RevisionNotPresent error
+        or check its fallback repositories.
+
+    New in 2.5.
+    """
+
+    def body_stream(self, repository, desired_files):
+        self._repository.lock_read()
+        try:
+            text_keys = {}
+            for i, key in enumerate(desired_files):
+                text_keys[key] = i
+            for record in repository.texts.get_record_stream(text_keys,
+                    'unordered', True):
+                identifier = text_keys[record.key]
+                if record.storage_kind == 'absent':
+                    yield "absent\0%s\0%s\0%d\n" % (record.key[0],
+                        record.key[1], identifier)
+                    # FIXME: Way to abort early?
+                    continue
+                yield "ok\0%d\n" % identifier
+                compressor = zlib.compressobj()
+                for bytes in record.get_bytes_as('chunked'):
+                    data = compressor.compress(bytes)
+                    if data:
+                        yield data
+                data = compressor.flush()
+                if data:
+                    yield data
+        finally:
+            self._repository.unlock()
+
+    def do_body(self, body_bytes):
+        desired_files = [
+            tuple(l.split("\0")) for l in body_bytes.splitlines()]
+        return SuccessfulSmartServerResponse(('ok', ),
+            body_stream=self.body_stream(self._repository, desired_files))
+
+    def do_repository_request(self, repository):
+        # Signal that we want a body
+        return None
+
+
+class SmartServerRepositoryIterRevisions(SmartServerRepositoryRequest):
+    """Stream a list of revisions.
+
+    The client sends a list of newline-separated revision ids in the
+    body of the request and the server replies with the serializer format,
+    and a stream of bzip2-compressed revision texts (using the specified
+    serializer format).
+
+    Any revisions the server does not have are omitted from the stream.
+
+    New in 2.5.
+    """
+
+    def do_repository_request(self, repository):
+        self._repository = repository
+        # Signal there is a body
+        return None
+
+    def do_body(self, body_bytes):
+        revision_ids = body_bytes.split("\n")
+        return SuccessfulSmartServerResponse(
+            ('ok', self._repository.get_serializer_format()),
+            body_stream=self.body_stream(self._repository, revision_ids))
+
+    def body_stream(self, repository, revision_ids):
+        self._repository.lock_read()
+        try:
+            for record in repository.revisions.get_record_stream(
+                [(revid,) for revid in revision_ids], 'unordered', True):
+                if record.storage_kind == 'absent':
+                    continue
+                yield zlib.compress(record.get_bytes_as('fulltext'))
+        finally:
+            self._repository.unlock()
+
+
+class SmartServerRepositoryGetInventories(SmartServerRepositoryRequest):
+    """Get the inventory deltas for a set of revision ids.
+
+    This accepts a list of revision ids, and then sends a chain
+    of deltas for the inventories of those revisions. The first
+    revision will be empty.
+
+    The server writes back zlibbed serialized inventory deltas,
+    in the ordering specified. The base for each delta is the
+    inventory generated by the previous delta.
+
+    New in 2.5.
+    """
+
+    def _inventory_delta_stream(self, repository, ordering, revids):
+        prev_inv = _mod_inventory.Inventory(root_id=None,
+            revision_id=_mod_revision.NULL_REVISION)
+        serializer = inventory_delta.InventoryDeltaSerializer(
+            repository.supports_rich_root(),
+            repository._format.supports_tree_reference)
+        repository.lock_read()
+        try:
+            for inv, revid in repository._iter_inventories(revids, ordering):
+                if inv is None:
+                    continue
+                inv_delta = inv._make_delta(prev_inv)
+                lines = serializer.delta_to_lines(
+                    prev_inv.revision_id, inv.revision_id, inv_delta)
+                yield ChunkedContentFactory(inv.revision_id, None, None, lines)
+                prev_inv = inv
+        finally:
+            repository.unlock()
+
+    def body_stream(self, repository, ordering, revids):
+        substream = self._inventory_delta_stream(repository,
+            ordering, revids)
+        return _stream_to_byte_stream([('inventory-deltas', substream)],
+            repository._format)
+
+    def do_body(self, body_bytes):
+        return SuccessfulSmartServerResponse(('ok', ),
+            body_stream=self.body_stream(self._repository, self._ordering,
+                body_bytes.splitlines()))
+
+    def do_repository_request(self, repository, ordering):
+        if ordering == 'unordered':
+            # inventory deltas for a topologically sorted stream
+            # are likely to be smaller
+            ordering = 'topological'
+        self._ordering = ordering
+        # Signal that we want a body
+        return None

=== modified file 'bzrlib/smart/request.py'
--- bzrlib/smart/request.py	2011-05-19 09:32:38 +0000
+++ bzrlib/smart/request.py	2011-12-18 12:46:49 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -26,6 +26,8 @@
       of times during a request).
 """
 
+from __future__ import absolute_import
+
 # XXX: The class names are a little confusing: the protocol will instantiate a
 # SmartServerRequestHandler, whose dispatch_command method creates an instance
 # of a SmartServerRequest subclass.
@@ -415,12 +417,16 @@
     elif isinstance(err, errors.ShortReadvError):
         return ('ShortReadvError', err.path, str(err.offset), str(err.length),
                 str(err.actual))
+    elif isinstance(err, errors.RevisionNotPresent):
+        return ('RevisionNotPresent', err.revision_id, err.file_id)
     elif isinstance(err, errors.UnstackableRepositoryFormat):
         return (('UnstackableRepositoryFormat', str(err.format), err.url))
     elif isinstance(err, errors.UnstackableBranchFormat):
         return ('UnstackableBranchFormat', str(err.format), err.url)
     elif isinstance(err, errors.NotStacked):
         return ('NotStacked',)
+    elif isinstance(err, errors.BzrCheckError):
+        return ('BzrCheckError', err.msg)
     elif isinstance(err, UnicodeError):
         # If it is a DecodeError, than most likely we are starting
         # with a plain string
@@ -491,157 +497,277 @@
         return SuccessfulSmartServerResponse((answer,))
 
 
+# In the 'info' attribute, we store whether this request is 'safe' to retry if
+# we get a disconnect while reading the response. It can have the values:
+#   read    This is purely a read request, so retrying it is perfectly ok.
+#   idem    An idempotent write request. Something like 'put' where if you put
+#           the same bytes twice you end up with the same final bytes.
+#   semi    This is a request that isn't strictly idempotent, but doesn't
+#           result in corruption if it is retried. This is for things like
+#           'lock' and 'unlock'. If you call lock, it updates the disk
+#           structure. If you fail to read the response, you won't be able to
+#           use the lock, because you don't have the lock token. Calling lock
+#           again will fail, because the lock is already taken. However, we
+#           can't tell if the server received our request or not. If it didn't,
+#           then retrying the request is fine, as it will actually do what we
+#           want. If it did, we will interrupt the current operation, but we
+#           are no worse off than interrupting the current operation because of
+#           a ConnectionReset.
+#   semivfs Similar to semi, but specific to a Virtual FileSystem request.
+#   stream  This is a request that takes a stream that cannot be restarted if
+#           consumed. This request is 'safe' in that if we determine the
+#           connection is closed before we consume the stream, we can try
+#           again.
+#   mutate  State is updated in a way that replaying that request results in a
+#           different state. For example 'append' writes more bytes to a given
+#           file. If append succeeds, it moves the file pointer.
 request_handlers = registry.Registry()
 request_handlers.register_lazy(
-    'append', 'bzrlib.smart.vfs', 'AppendRequest')
+    'append', 'bzrlib.smart.vfs', 'AppendRequest', info='mutate')
+request_handlers.register_lazy(
+    'Branch.break_lock', 'bzrlib.smart.branch',
+    'SmartServerBranchBreakLock', info='idem')
 request_handlers.register_lazy(
     'Branch.get_config_file', 'bzrlib.smart.branch',
-    'SmartServerBranchGetConfigFile')
-request_handlers.register_lazy(
-    'Branch.get_parent', 'bzrlib.smart.branch', 'SmartServerBranchGetParent')
+    'SmartServerBranchGetConfigFile', info='read')
+request_handlers.register_lazy(
+    'Branch.get_parent', 'bzrlib.smart.branch', 'SmartServerBranchGetParent',
+    info='read')
+request_handlers.register_lazy(
+    'Branch.put_config_file', 'bzrlib.smart.branch',
+    'SmartServerBranchPutConfigFile', info='idem')
 request_handlers.register_lazy(
     'Branch.get_tags_bytes', 'bzrlib.smart.branch',
-    'SmartServerBranchGetTagsBytes')
+    'SmartServerBranchGetTagsBytes', info='read')
 request_handlers.register_lazy(
     'Branch.set_tags_bytes', 'bzrlib.smart.branch',
-    'SmartServerBranchSetTagsBytes')
+    'SmartServerBranchSetTagsBytes', info='idem')
 request_handlers.register_lazy(
     'Branch.heads_to_fetch', 'bzrlib.smart.branch',
-    'SmartServerBranchHeadsToFetch')
-request_handlers.register_lazy(
-    'Branch.get_stacked_on_url', 'bzrlib.smart.branch', 'SmartServerBranchRequestGetStackedOnURL')
-request_handlers.register_lazy(
-    'Branch.last_revision_info', 'bzrlib.smart.branch', 'SmartServerBranchRequestLastRevisionInfo')
-request_handlers.register_lazy(
-    'Branch.lock_write', 'bzrlib.smart.branch', 'SmartServerBranchRequestLockWrite')
-request_handlers.register_lazy( 'Branch.revision_history',
-    'bzrlib.smart.branch', 'SmartServerRequestRevisionHistory')
-request_handlers.register_lazy( 'Branch.set_config_option',
-    'bzrlib.smart.branch', 'SmartServerBranchRequestSetConfigOption')
-request_handlers.register_lazy( 'Branch.set_config_option_dict',
-    'bzrlib.smart.branch', 'SmartServerBranchRequestSetConfigOptionDict')
-request_handlers.register_lazy( 'Branch.set_last_revision',
-    'bzrlib.smart.branch', 'SmartServerBranchRequestSetLastRevision')
+    'SmartServerBranchHeadsToFetch', info='read')
+request_handlers.register_lazy(
+    'Branch.get_stacked_on_url', 'bzrlib.smart.branch',
+    'SmartServerBranchRequestGetStackedOnURL', info='read')
+request_handlers.register_lazy(
+    'Branch.get_physical_lock_status', 'bzrlib.smart.branch',
+    'SmartServerBranchRequestGetPhysicalLockStatus', info='read')
+request_handlers.register_lazy(
+    'Branch.last_revision_info', 'bzrlib.smart.branch',
+    'SmartServerBranchRequestLastRevisionInfo', info='read')
+request_handlers.register_lazy(
+    'Branch.lock_write', 'bzrlib.smart.branch',
+    'SmartServerBranchRequestLockWrite', info='semi')
+request_handlers.register_lazy(
+    'Branch.revision_history', 'bzrlib.smart.branch',
+    'SmartServerRequestRevisionHistory', info='read')
+request_handlers.register_lazy(
+    'Branch.set_config_option', 'bzrlib.smart.branch',
+    'SmartServerBranchRequestSetConfigOption', info='idem')
+request_handlers.register_lazy(
+    'Branch.set_config_option_dict', 'bzrlib.smart.branch',
+    'SmartServerBranchRequestSetConfigOptionDict', info='idem')
+request_handlers.register_lazy(
+    'Branch.set_last_revision', 'bzrlib.smart.branch',
+    'SmartServerBranchRequestSetLastRevision', info='idem')
 request_handlers.register_lazy(
     'Branch.set_last_revision_info', 'bzrlib.smart.branch',
-    'SmartServerBranchRequestSetLastRevisionInfo')
+    'SmartServerBranchRequestSetLastRevisionInfo', info='idem')
 request_handlers.register_lazy(
     'Branch.set_last_revision_ex', 'bzrlib.smart.branch',
-    'SmartServerBranchRequestSetLastRevisionEx')
+    'SmartServerBranchRequestSetLastRevisionEx', info='idem')
 request_handlers.register_lazy(
     'Branch.set_parent_location', 'bzrlib.smart.branch',
-    'SmartServerBranchRequestSetParentLocation')
-request_handlers.register_lazy(
-    'Branch.unlock', 'bzrlib.smart.branch', 'SmartServerBranchRequestUnlock')
+    'SmartServerBranchRequestSetParentLocation', info='idem')
+request_handlers.register_lazy(
+    'Branch.unlock', 'bzrlib.smart.branch',
+    'SmartServerBranchRequestUnlock', info='semi')
+request_handlers.register_lazy(
+    'Branch.revision_id_to_revno', 'bzrlib.smart.branch',
+    'SmartServerBranchRequestRevisionIdToRevno', info='read')
+request_handlers.register_lazy(
+    'BzrDir.checkout_metadir', 'bzrlib.smart.bzrdir',
+    'SmartServerBzrDirRequestCheckoutMetaDir', info='read')
 request_handlers.register_lazy(
     'BzrDir.cloning_metadir', 'bzrlib.smart.bzrdir',
-    'SmartServerBzrDirRequestCloningMetaDir')
+    'SmartServerBzrDirRequestCloningMetaDir', info='read')
 request_handlers.register_lazy(
     'BzrDir.create_branch', 'bzrlib.smart.bzrdir',
-    'SmartServerRequestCreateBranch')
+    'SmartServerRequestCreateBranch', info='semi')
 request_handlers.register_lazy(
     'BzrDir.create_repository', 'bzrlib.smart.bzrdir',
-    'SmartServerRequestCreateRepository')
+    'SmartServerRequestCreateRepository', info='semi')
 request_handlers.register_lazy(
     'BzrDir.find_repository', 'bzrlib.smart.bzrdir',
-    'SmartServerRequestFindRepositoryV1')
+    'SmartServerRequestFindRepositoryV1', info='read')
 request_handlers.register_lazy(
     'BzrDir.find_repositoryV2', 'bzrlib.smart.bzrdir',
-    'SmartServerRequestFindRepositoryV2')
+    'SmartServerRequestFindRepositoryV2', info='read')
 request_handlers.register_lazy(
     'BzrDir.find_repositoryV3', 'bzrlib.smart.bzrdir',
-    'SmartServerRequestFindRepositoryV3')
+    'SmartServerRequestFindRepositoryV3', info='read')
 request_handlers.register_lazy(
     'BzrDir.get_config_file', 'bzrlib.smart.bzrdir',
-    'SmartServerBzrDirRequestConfigFile')
+    'SmartServerBzrDirRequestConfigFile', info='read')
+request_handlers.register_lazy(
+    'BzrDir.destroy_branch', 'bzrlib.smart.bzrdir',
+    'SmartServerBzrDirRequestDestroyBranch', info='semi')
+request_handlers.register_lazy(
+    'BzrDir.destroy_repository', 'bzrlib.smart.bzrdir',
+    'SmartServerBzrDirRequestDestroyRepository', info='semi')
+request_handlers.register_lazy(
+    'BzrDir.has_workingtree', 'bzrlib.smart.bzrdir',
+    'SmartServerBzrDirRequestHasWorkingTree', info='read')
 request_handlers.register_lazy(
     'BzrDirFormat.initialize', 'bzrlib.smart.bzrdir',
-    'SmartServerRequestInitializeBzrDir')
+    'SmartServerRequestInitializeBzrDir', info='semi')
 request_handlers.register_lazy(
     'BzrDirFormat.initialize_ex_1.16', 'bzrlib.smart.bzrdir',
-    'SmartServerRequestBzrDirInitializeEx')
-request_handlers.register_lazy(
-    'BzrDir.open', 'bzrlib.smart.bzrdir', 'SmartServerRequestOpenBzrDir')
-request_handlers.register_lazy(
-    'BzrDir.open_2.1', 'bzrlib.smart.bzrdir', 'SmartServerRequestOpenBzrDir_2_1')
+    'SmartServerRequestBzrDirInitializeEx', info='semi')
+request_handlers.register_lazy(
+    'BzrDir.open', 'bzrlib.smart.bzrdir', 'SmartServerRequestOpenBzrDir',
+    info='read')
+request_handlers.register_lazy(
+    'BzrDir.open_2.1', 'bzrlib.smart.bzrdir',
+    'SmartServerRequestOpenBzrDir_2_1', info='read')
 request_handlers.register_lazy(
     'BzrDir.open_branch', 'bzrlib.smart.bzrdir',
-    'SmartServerRequestOpenBranch')
+    'SmartServerRequestOpenBranch', info='read')
 request_handlers.register_lazy(
     'BzrDir.open_branchV2', 'bzrlib.smart.bzrdir',
-    'SmartServerRequestOpenBranchV2')
+    'SmartServerRequestOpenBranchV2', info='read')
 request_handlers.register_lazy(
     'BzrDir.open_branchV3', 'bzrlib.smart.bzrdir',
-    'SmartServerRequestOpenBranchV3')
-request_handlers.register_lazy(
-    'delete', 'bzrlib.smart.vfs', 'DeleteRequest')
-request_handlers.register_lazy(
-    'get', 'bzrlib.smart.vfs', 'GetRequest')
-request_handlers.register_lazy(
-    'get_bundle', 'bzrlib.smart.request', 'GetBundleRequest')
-request_handlers.register_lazy(
-    'has', 'bzrlib.smart.vfs', 'HasRequest')
-request_handlers.register_lazy(
-    'hello', 'bzrlib.smart.request', 'HelloRequest')
-request_handlers.register_lazy(
-    'iter_files_recursive', 'bzrlib.smart.vfs', 'IterFilesRecursiveRequest')
-request_handlers.register_lazy(
-    'list_dir', 'bzrlib.smart.vfs', 'ListDirRequest')
-request_handlers.register_lazy(
-    'mkdir', 'bzrlib.smart.vfs', 'MkdirRequest')
-request_handlers.register_lazy(
-    'move', 'bzrlib.smart.vfs', 'MoveRequest')
-request_handlers.register_lazy(
-    'put', 'bzrlib.smart.vfs', 'PutRequest')
-request_handlers.register_lazy(
-    'put_non_atomic', 'bzrlib.smart.vfs', 'PutNonAtomicRequest')
-request_handlers.register_lazy(
-    'readv', 'bzrlib.smart.vfs', 'ReadvRequest')
-request_handlers.register_lazy(
-    'rename', 'bzrlib.smart.vfs', 'RenameRequest')
+    'SmartServerRequestOpenBranchV3', info='read')
+request_handlers.register_lazy(
+    'delete', 'bzrlib.smart.vfs', 'DeleteRequest', info='semivfs')
+request_handlers.register_lazy(
+    'get', 'bzrlib.smart.vfs', 'GetRequest', info='read')
+request_handlers.register_lazy(
+    'get_bundle', 'bzrlib.smart.request', 'GetBundleRequest', info='read')
+request_handlers.register_lazy(
+    'has', 'bzrlib.smart.vfs', 'HasRequest', info='read')
+request_handlers.register_lazy(
+    'hello', 'bzrlib.smart.request', 'HelloRequest', info='read')
+request_handlers.register_lazy(
+    'iter_files_recursive', 'bzrlib.smart.vfs', 'IterFilesRecursiveRequest',
+    info='read')
+request_handlers.register_lazy(
+    'list_dir', 'bzrlib.smart.vfs', 'ListDirRequest', info='read')
+request_handlers.register_lazy(
+    'mkdir', 'bzrlib.smart.vfs', 'MkdirRequest', info='semivfs')
+request_handlers.register_lazy(
+    'move', 'bzrlib.smart.vfs', 'MoveRequest', info='semivfs')
+request_handlers.register_lazy(
+    'put', 'bzrlib.smart.vfs', 'PutRequest', info='idem')
+request_handlers.register_lazy(
+    'put_non_atomic', 'bzrlib.smart.vfs', 'PutNonAtomicRequest', info='idem')
+request_handlers.register_lazy(
+    'readv', 'bzrlib.smart.vfs', 'ReadvRequest', info='read')
+request_handlers.register_lazy(
+    'rename', 'bzrlib.smart.vfs', 'RenameRequest', info='semivfs')
+request_handlers.register_lazy(
+    'Repository.add_signature_text', 'bzrlib.smart.repository',
+    'SmartServerRepositoryAddSignatureText', info='idem')
+request_handlers.register_lazy(
+    'Repository.all_revision_ids', 'bzrlib.smart.repository',
+    'SmartServerRepositoryAllRevisionIds', info='read')
 request_handlers.register_lazy(
     'PackRepository.autopack', 'bzrlib.smart.packrepository',
-    'SmartServerPackRepositoryAutopack')
-request_handlers.register_lazy('Repository.gather_stats',
-                               'bzrlib.smart.repository',
-                               'SmartServerRepositoryGatherStats')
-request_handlers.register_lazy('Repository.get_parent_map',
-                               'bzrlib.smart.repository',
-                               'SmartServerRepositoryGetParentMap')
-request_handlers.register_lazy(
-    'Repository.get_revision_graph', 'bzrlib.smart.repository', 'SmartServerRepositoryGetRevisionGraph')
-request_handlers.register_lazy(
-    'Repository.has_revision', 'bzrlib.smart.repository', 'SmartServerRequestHasRevision')
-request_handlers.register_lazy(
-    'Repository.insert_stream', 'bzrlib.smart.repository', 'SmartServerRepositoryInsertStream')
-request_handlers.register_lazy(
-    'Repository.insert_stream_1.19', 'bzrlib.smart.repository', 'SmartServerRepositoryInsertStream_1_19')
-request_handlers.register_lazy(
-    'Repository.insert_stream_locked', 'bzrlib.smart.repository', 'SmartServerRepositoryInsertStreamLocked')
-request_handlers.register_lazy(
-    'Repository.is_shared', 'bzrlib.smart.repository', 'SmartServerRepositoryIsShared')
-request_handlers.register_lazy(
-    'Repository.lock_write', 'bzrlib.smart.repository', 'SmartServerRepositoryLockWrite')
+    'SmartServerPackRepositoryAutopack', info='idem')
+request_handlers.register_lazy(
+    'Repository.break_lock', 'bzrlib.smart.repository',
+    'SmartServerRepositoryBreakLock', info='idem')
+request_handlers.register_lazy(
+    'Repository.gather_stats', 'bzrlib.smart.repository',
+    'SmartServerRepositoryGatherStats', info='read')
+request_handlers.register_lazy(
+    'Repository.get_parent_map', 'bzrlib.smart.repository',
+    'SmartServerRepositoryGetParentMap', info='read')
+request_handlers.register_lazy(
+    'Repository.get_revision_graph', 'bzrlib.smart.repository',
+    'SmartServerRepositoryGetRevisionGraph', info='read')
+request_handlers.register_lazy(
+    'Repository.get_revision_signature_text', 'bzrlib.smart.repository',
+    'SmartServerRepositoryGetRevisionSignatureText', info='read')
+request_handlers.register_lazy(
+    'Repository.has_revision', 'bzrlib.smart.repository',
+    'SmartServerRequestHasRevision', info='read')
+request_handlers.register_lazy(
+    'Repository.has_signature_for_revision_id', 'bzrlib.smart.repository',
+    'SmartServerRequestHasSignatureForRevisionId', info='read')
+request_handlers.register_lazy(
+    'Repository.insert_stream', 'bzrlib.smart.repository',
+    'SmartServerRepositoryInsertStream', info='stream')
+request_handlers.register_lazy(
+    'Repository.insert_stream_1.19', 'bzrlib.smart.repository',
+    'SmartServerRepositoryInsertStream_1_19', info='stream')
+request_handlers.register_lazy(
+    'Repository.insert_stream_locked', 'bzrlib.smart.repository',
+    'SmartServerRepositoryInsertStreamLocked', info='stream')
+request_handlers.register_lazy(
+    'Repository.is_shared', 'bzrlib.smart.repository',
+    'SmartServerRepositoryIsShared', info='read')
+request_handlers.register_lazy(
+    'Repository.iter_files_bytes', 'bzrlib.smart.repository',
+    'SmartServerRepositoryIterFilesBytes', info='read')
+request_handlers.register_lazy(
+    'Repository.lock_write', 'bzrlib.smart.repository',
+    'SmartServerRepositoryLockWrite', info='semi')
+request_handlers.register_lazy(
+    'Repository.make_working_trees', 'bzrlib.smart.repository',
+    'SmartServerRepositoryMakeWorkingTrees', info='read')
 request_handlers.register_lazy(
     'Repository.set_make_working_trees', 'bzrlib.smart.repository',
-    'SmartServerRepositorySetMakeWorkingTrees')
-request_handlers.register_lazy(
-    'Repository.unlock', 'bzrlib.smart.repository', 'SmartServerRepositoryUnlock')
+    'SmartServerRepositorySetMakeWorkingTrees', info='idem')
+request_handlers.register_lazy(
+    'Repository.unlock', 'bzrlib.smart.repository',
+    'SmartServerRepositoryUnlock', info='semi')
+request_handlers.register_lazy(
+    'Repository.get_physical_lock_status', 'bzrlib.smart.repository',
+    'SmartServerRepositoryGetPhysicalLockStatus', info='read')
 request_handlers.register_lazy(
     'Repository.get_rev_id_for_revno', 'bzrlib.smart.repository',
-    'SmartServerRepositoryGetRevIdForRevno')
+    'SmartServerRepositoryGetRevIdForRevno', info='read')
 request_handlers.register_lazy(
     'Repository.get_stream', 'bzrlib.smart.repository',
-    'SmartServerRepositoryGetStream')
+    'SmartServerRepositoryGetStream', info='read')
 request_handlers.register_lazy(
     'Repository.get_stream_1.19', 'bzrlib.smart.repository',
-    'SmartServerRepositoryGetStream_1_19')
+    'SmartServerRepositoryGetStream_1_19', info='read')
+request_handlers.register_lazy(
+    'Repository.iter_revisions', 'bzrlib.smart.repository',
+    'SmartServerRepositoryIterRevisions', info='read')
+request_handlers.register_lazy(
+    'Repository.pack', 'bzrlib.smart.repository',
+    'SmartServerRepositoryPack', info='idem')
+request_handlers.register_lazy(
+    'Repository.start_write_group', 'bzrlib.smart.repository',
+    'SmartServerRepositoryStartWriteGroup', info='semi')
+request_handlers.register_lazy(
+    'Repository.commit_write_group', 'bzrlib.smart.repository',
+    'SmartServerRepositoryCommitWriteGroup', info='semi')
+request_handlers.register_lazy(
+    'Repository.abort_write_group', 'bzrlib.smart.repository',
+    'SmartServerRepositoryAbortWriteGroup', info='semi')
+request_handlers.register_lazy(
+    'Repository.check_write_group', 'bzrlib.smart.repository',
+    'SmartServerRepositoryCheckWriteGroup', info='read')
+request_handlers.register_lazy(
+    'Repository.reconcile', 'bzrlib.smart.repository',
+    'SmartServerRepositoryReconcile', info='idem')
 request_handlers.register_lazy(
     'Repository.tarball', 'bzrlib.smart.repository',
-    'SmartServerRepositoryTarball')
-request_handlers.register_lazy(
-    'rmdir', 'bzrlib.smart.vfs', 'RmdirRequest')
-request_handlers.register_lazy(
-    'stat', 'bzrlib.smart.vfs', 'StatRequest')
-request_handlers.register_lazy(
-    'Transport.is_readonly', 'bzrlib.smart.request', 'SmartServerIsReadonly')
+    'SmartServerRepositoryTarball', info='read')
+request_handlers.register_lazy(
+    'VersionedFileRepository.get_serializer_format', 'bzrlib.smart.repository',
+    'SmartServerRepositoryGetSerializerFormat', info='read')
+request_handlers.register_lazy(
+    'VersionedFileRepository.get_inventories', 'bzrlib.smart.repository',
+    'SmartServerRepositoryGetInventories', info='read')
+request_handlers.register_lazy(
+    'rmdir', 'bzrlib.smart.vfs', 'RmdirRequest', info='semivfs')
+request_handlers.register_lazy(
+    'stat', 'bzrlib.smart.vfs', 'StatRequest', info='read')
+request_handlers.register_lazy(
+    'Transport.is_readonly', 'bzrlib.smart.request',
+    'SmartServerIsReadonly', info='read')

=== modified file 'bzrlib/smart/server.py'
--- bzrlib/smart/server.py	2011-07-23 16:33:38 +0000
+++ bzrlib/smart/server.py	2011-12-19 13:23:58 +0000
@@ -16,10 +16,13 @@
 
 """Server for smart-server protocol."""
 
+from __future__ import absolute_import
+
 import errno
 import os.path
 import socket
 import sys
+import time
 import threading
 
 from bzrlib.hooks import Hooks
@@ -28,14 +31,19 @@
     trace,
     transport as _mod_transport,
 )
+from bzrlib.i18n import gettext
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
-from bzrlib.smart import medium
+from bzrlib.smart import (
+    medium,
+    signals,
+    )
 from bzrlib.transport import (
     chroot,
     pathfilter,
     )
 from bzrlib import (
+    config,
     urlutils,
     )
 """)
@@ -50,7 +58,17 @@
     hooks: An instance of SmartServerHooks.
     """
 
-    def __init__(self, backing_transport, root_client_path='/'):
+    # This is the timeout on the socket we use .accept() on. It is exposed here
+    # so the test suite can set it faster. (It thread.interrupt_main() will not
+    # fire a KeyboardInterrupt during socket.accept)
+    _ACCEPT_TIMEOUT = 1.0
+    _SHUTDOWN_POLL_TIMEOUT = 1.0
+    _LOG_WAITING_TIMEOUT = 10.0
+
+    _timer = time.time
+
+    def __init__(self, backing_transport, root_client_path='/',
+                 client_timeout=None):
         """Construct a new server.
 
         To actually start it running, call either start_background_thread or
@@ -59,9 +77,16 @@
         :param backing_transport: The transport to serve.
         :param root_client_path: The client path that will correspond to root
             of backing_transport.
+        :param client_timeout: See SmartServerSocketStreamMedium's timeout
+            parameter.
         """
         self.backing_transport = backing_transport
         self.root_client_path = root_client_path
+        self._client_timeout = client_timeout
+        self._active_connections = []
+        # This is set to indicate we want to wait for clients to finish before
+        # we disconnect.
+        self._gracefully_stopping = False
 
     def start_server(self, host, port):
         """Create the server listening socket.
@@ -93,9 +118,15 @@
         self._sockname = self._server_socket.getsockname()
         self.port = self._sockname[1]
         self._server_socket.listen(1)
-        self._server_socket.settimeout(1)
+        self._server_socket.settimeout(self._ACCEPT_TIMEOUT)
+        # Once we start accept()ing connections, we set started.
         self._started = threading.Event()
+        # Once we stop accept()ing connections (and are closing the socket) we
+        # set _stopped
         self._stopped = threading.Event()
+        # Once we have finished waiting for all clients, etc. We set
+        # _fully_stopped
+        self._fully_stopped = threading.Event()
 
     def _backing_urls(self):
         # There are three interesting urls:
@@ -134,7 +165,38 @@
         for hook in SmartTCPServer.hooks['server_stopped']:
             hook(backing_urls, self.get_url())
 
+    def _stop_gracefully(self):
+        trace.note(gettext('Requested to stop gracefully'))
+        self._should_terminate = True
+        self._gracefully_stopping = True
+        for handler, _ in self._active_connections:
+            handler._stop_gracefully()
+
+    def _wait_for_clients_to_disconnect(self):
+        self._poll_active_connections()
+        if not self._active_connections:
+            return
+        trace.note(gettext('Waiting for %d client(s) to finish')
+                   % (len(self._active_connections),))
+        t_next_log = self._timer() + self._LOG_WAITING_TIMEOUT
+        while self._active_connections:
+            now = self._timer()
+            if now >= t_next_log:
+                trace.note(gettext('Still waiting for %d client(s) to finish')
+                           % (len(self._active_connections),))
+                t_next_log = now + self._LOG_WAITING_TIMEOUT
+            self._poll_active_connections(self._SHUTDOWN_POLL_TIMEOUT)
+
     def serve(self, thread_name_suffix=''):
+        # Note: There is a temptation to do
+        #       signals.register_on_hangup(id(self), self._stop_gracefully)
+        #       However, that creates a temporary object which is a bound
+        #       method. signals._on_sighup is a WeakKeyDictionary so it
+        #       immediately gets garbage collected, because nothing else
+        #       references it. Instead, we need to keep a real reference to the
+        #       bound method for the lifetime of the serve() function.
+        stop_gracefully = self._stop_gracefully
+        signals.register_on_hangup(id(self), stop_gracefully)
         self._should_terminate = False
         # for hooks we are letting code know that a server has started (and
         # later stopped).
@@ -150,14 +212,19 @@
                         pass
                     except self._socket_error, e:
                         # if the socket is closed by stop_background_thread
-                        # we might get a EBADF here, any other socket errors
-                        # should get logged.
-                        if e.args[0] != errno.EBADF:
-                            trace.warning("listening socket error: %s", e)
+                        # we might get a EBADF here, or if we get a signal we
+                        # can get EINTR, any other socket errors should get
+                        # logged.
+                        if e.args[0] not in (errno.EBADF, errno.EINTR):
+                            trace.warning(gettext("listening socket error: %s")
+                                          % (e,))
                     else:
                         if self._should_terminate:
+                            conn.close()
                             break
                         self.serve_conn(conn, thread_name_suffix)
+                    # Cleanout any threads that have finished processing.
+                    self._poll_active_connections()
             except KeyboardInterrupt:
                 # dont log when CTRL-C'd.
                 raise
@@ -165,32 +232,55 @@
                 trace.report_exception(sys.exc_info(), sys.stderr)
                 raise
         finally:
-            self._stopped.set()
             try:
                 # ensure the server socket is closed.
                 self._server_socket.close()
             except self._socket_error:
                 # ignore errors on close
                 pass
+            self._stopped.set()
+            signals.unregister_on_hangup(id(self))
             self.run_server_stopped_hooks()
+        if self._gracefully_stopping:
+            self._wait_for_clients_to_disconnect()
+        self._fully_stopped.set()
 
     def get_url(self):
         """Return the url of the server"""
         return "bzr://%s:%s/" % (self._sockname[0], self._sockname[1])
 
+    def _make_handler(self, conn):
+        return medium.SmartServerSocketStreamMedium(
+            conn, self.backing_transport, self.root_client_path,
+            timeout=self._client_timeout)
+
+    def _poll_active_connections(self, timeout=0.0):
+        """Check to see if any active connections have finished.
+
+        This will iterate through self._active_connections, and update any
+        connections that are finished.
+
+        :param timeout: The timeout to pass to thread.join(). By default, we
+            set it to 0, so that we don't hang if threads are not done yet.
+        :return: None
+        """
+        still_active = []
+        for handler, thread in self._active_connections:
+            thread.join(timeout)
+            if thread.isAlive():
+                still_active.append((handler, thread))
+        self._active_connections = still_active
+
     def serve_conn(self, conn, thread_name_suffix):
         # For WIN32, where the timeout value from the listening socket
         # propagates to the newly accepted socket.
         conn.setblocking(True)
         conn.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
-        handler = medium.SmartServerSocketStreamMedium(
-            conn, self.backing_transport, self.root_client_path)
         thread_name = 'smart-server-child' + thread_name_suffix
+        handler = self._make_handler(conn)
         connection_thread = threading.Thread(
             None, handler.serve, name=thread_name)
-        # FIXME: This thread is never joined, it should at least be collected
-        # somewhere so that tests that want to check for leaked threads can get
-        # rid of them -- vila 20100531
+        self._active_connections.append((handler, connection_thread))
         connection_thread.setDaemon(True)
         connection_thread.start()
         return connection_thread
@@ -340,18 +430,26 @@
             transport = _mod_transport.get_transport_from_url(expand_userdirs.get_url())
         self.transport = transport
 
-    def _make_smart_server(self, host, port, inet):
+    def _get_stdin_stdout(self):
+        return sys.stdin, sys.stdout
+
+    def _make_smart_server(self, host, port, inet, timeout):
+        if timeout is None:
+            c = config.GlobalStack()
+            timeout = c.get('serve.client_timeout')
         if inet:
+            stdin, stdout = self._get_stdin_stdout()
             smart_server = medium.SmartServerPipeStreamMedium(
-                sys.stdin, sys.stdout, self.transport)
+                stdin, stdout, self.transport, timeout=timeout)
         else:
             if host is None:
                 host = medium.BZR_DEFAULT_INTERFACE
             if port is None:
                 port = medium.BZR_DEFAULT_PORT
-            smart_server = SmartTCPServer(self.transport)
+            smart_server = SmartTCPServer(self.transport,
+                                          client_timeout=timeout)
             smart_server.start_server(host, port)
-            trace.note('listening on port: %s' % smart_server.port)
+            trace.note(gettext('listening on port: %s') % smart_server.port)
         self.smart_server = smart_server
 
     def _change_globals(self):
@@ -368,26 +466,31 @@
         self.cleanups.append(restore_default_ui_factory_and_lockdir_timeout)
         ui.ui_factory = ui.SilentUIFactory()
         lockdir._DEFAULT_TIMEOUT_SECONDS = 0
+        orig = signals.install_sighup_handler()
+        def restore_signals():
+            signals.restore_sighup_handler(orig)
+        self.cleanups.append(restore_signals)
 
-    def set_up(self, transport, host, port, inet):
+    def set_up(self, transport, host, port, inet, timeout):
         self._make_backing_transport(transport)
-        self._make_smart_server(host, port, inet)
+        self._make_smart_server(host, port, inet, timeout)
         self._change_globals()
 
     def tear_down(self):
         for cleanup in reversed(self.cleanups):
             cleanup()
 
-def serve_bzr(transport, host=None, port=None, inet=False):
+
+def serve_bzr(transport, host=None, port=None, inet=False, timeout=None):
     """This is the default implementation of 'bzr serve'.
-    
+
     It creates a TCP or pipe smart server on 'transport, and runs it.  The
     transport will be decorated with a chroot and pathfilter (using
     os.path.expanduser).
     """
     bzr_server = BzrServerFactory()
     try:
-        bzr_server.set_up(transport, host, port, inet)
+        bzr_server.set_up(transport, host, port, inet, timeout)
         bzr_server.smart_server.serve()
     except:
         hook_caught_exception = False

=== added file 'bzrlib/smart/signals.py'
--- bzrlib/smart/signals.py	1970-01-01 00:00:00 +0000
+++ bzrlib/smart/signals.py	2011-12-19 13:23:58 +0000
@@ -0,0 +1,116 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Signal handling for the smart server code."""
+
+from __future__ import absolute_import
+
+import signal
+import weakref
+
+from bzrlib import trace
+
+
+# I'm pretty sure this has to be global, since signal handling is per-process.
+_on_sighup = None
+# TODO: Using a dict means that the order of calls is unordered. We could use a
+#       list and then do something like LIFO ordering. A dict was chosen so
+#       that you could have a key to easily remove your entry. However, you
+#       could just use the callable itself as the indexed part, and even in
+#       large cases, we shouldn't have more than 100 or so callbacks
+#       registered.
+def _sighup_handler(signal_number, interrupted_frame):
+    """This is the actual function that is registered for handling SIGHUP.
+
+    It will call out to all the registered functions, letting them know that a
+    graceful termination has been requested.
+    """
+    if _on_sighup is None:
+        return
+    trace.mutter('Caught SIGHUP, sending graceful shutdown requests.')
+    for ref in _on_sighup.valuerefs():
+        try:
+            cb = ref()
+            if cb is not None:
+                cb()
+        except KeyboardInterrupt:
+            raise
+        except Exception:
+            trace.mutter('Error occurred while running SIGHUP handlers:')
+            trace.log_exception_quietly()
+
+
+def install_sighup_handler():
+    """Setup a handler for the SIGHUP signal."""
+    if getattr(signal, "SIGHUP", None) is None:
+        # If we can't install SIGHUP, there is no reason (yet) to do graceful
+        # shutdown.
+        old_signal = None
+    else:
+        old_signal = signal.signal(signal.SIGHUP, _sighup_handler)
+    old_dict = _setup_on_hangup_dict()
+    return old_signal, old_dict
+
+
+def _setup_on_hangup_dict():
+    """Create something for _on_sighup.
+
+    This is done when we install the sighup handler, and for tests that want to
+    test the functionality. If this hasn'nt been called, then
+    register_on_hangup is a no-op. As is unregister_on_hangup.
+    """
+    global _on_sighup
+    old = _on_sighup
+    _on_sighup = weakref.WeakValueDictionary()
+    return old
+
+
+def restore_sighup_handler(orig):
+    """Pass in the returned value from install_sighup_handler to reset."""
+    global _on_sighup
+    old_signal, old_dict = orig
+    if old_signal is not None:
+        signal.signal(signal.SIGHUP, old_signal)
+    _on_sighup = old_dict
+
+
+# TODO: Should these be single-use callables? Meaning that once we've triggered
+#       SIGHUP and called them, they should auto-remove themselves? I don't
+#       think so. Callers need to clean up during shutdown anyway, so that we
+#       don't end up with lots of garbage in the _on_sighup dict. On the other
+#       hand, we made _on_sighup a WeakValueDictionary in case cleanups didn't
+#       get fired properly. Maybe we just assume we don't have to do it?
+def register_on_hangup(identifier, a_callable):
+    """Register for us to call a_callable as part of a graceful shutdown."""
+    if _on_sighup is None:
+        return
+    _on_sighup[identifier] = a_callable
+
+
+def unregister_on_hangup(identifier):
+    """Remove a callback from being called during sighup."""
+    if _on_sighup is None:
+        return
+    try:
+        del _on_sighup[identifier]
+    except KeyboardInterrupt:
+        raise
+    except Exception:
+        # This usually runs as a tear-down step. So we don't want to propagate
+        # most exceptions.
+        trace.mutter('Error occurred during unregister_on_hangup:')
+        trace.log_exception_quietly()
+

=== modified file 'bzrlib/smart/vfs.py'
--- bzrlib/smart/vfs.py	2010-02-17 17:11:16 +0000
+++ bzrlib/smart/vfs.py	2011-12-19 13:23:58 +0000
@@ -24,6 +24,8 @@
 protocol, as implemented in bzr 0.11 and later.
 """
 
+from __future__ import absolute_import
+
 import os
 
 from bzrlib import errors

=== modified file 'bzrlib/smtp_connection.py'
--- bzrlib/smtp_connection.py	2009-04-27 16:10:10 +0000
+++ bzrlib/smtp_connection.py	2011-12-20 18:47:35 +0000
@@ -16,6 +16,8 @@
 
 """A convenience class around smtplib."""
 
+from __future__ import absolute_import
+
 from email import Utils
 import errno
 import smtplib
@@ -33,6 +35,20 @@
     )
 
 
+smtp_password = config.Option('smtp_password', default=None,
+        help='''\
+Password to use for authentication to SMTP server.
+''')
+smtp_server = config.Option('smtp_server', default=None,
+        help='''\
+Hostname of the SMTP server to use for sending email.
+''')
+smtp_username = config.Option('smtp_username', default=None,
+        help='''\
+Username to use for authentication to SMTP server.
+''')
+
+
 class SMTPConnection(object):
     """Connect to an SMTP server and send an email.
 
@@ -48,13 +64,13 @@
         if self._smtp_factory is None:
             self._smtp_factory = smtplib.SMTP
         self._config = config
-        self._config_smtp_server = config.get_user_option('smtp_server')
+        self._config_smtp_server = config.get('smtp_server')
         self._smtp_server = self._config_smtp_server
         if self._smtp_server is None:
             self._smtp_server = self._default_smtp_server
 
-        self._smtp_username = config.get_user_option('smtp_username')
-        self._smtp_password = config.get_user_option('smtp_password')
+        self._smtp_username = config.get('smtp_username')
+        self._smtp_password = config.get('smtp_password')
 
         self._connection = None
 

=== modified file 'bzrlib/static_tuple.py'
--- bzrlib/static_tuple.py	2010-02-17 17:11:16 +0000
+++ bzrlib/static_tuple.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,8 @@
 
 """Interface thunk for a StaticTuple implementation."""
 
+from __future__ import absolute_import
+
 from bzrlib import debug
 
 try:

=== modified file 'bzrlib/status.py'
--- bzrlib/status.py	2011-06-01 12:53:56 +0000
+++ bzrlib/status.py	2011-12-18 15:28:38 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 import sys
 
 from bzrlib import (
@@ -157,7 +159,7 @@
         try:
             for hook in hooks['pre_status']:
                 hook(StatusHookParams(old, new, to_file, versioned,
-                    show_ids, short, verbose))
+                    show_ids, short, verbose, specific_files=specific_files))
 
             specific_files, nonexistents \
                 = _filter_nonexistent(specific_files, old, new)
@@ -222,7 +224,7 @@
                 raise errors.PathsDoNotExist(nonexistents)
             for hook in hooks['post_status']:
                 hook(StatusHookParams(old, new, to_file, versioned,
-                    show_ids, short, verbose))
+                    show_ids, short, verbose, specific_files=specific_files))
         finally:
             old.unlock()
             new.unlock()
@@ -416,7 +418,7 @@
     """
 
     def __init__(self, old_tree, new_tree, to_file, versioned, show_ids,
-            short, verbose):
+            short, verbose, specific_files=None):
         """Create a group of post_status hook parameters.
 
         :param old_tree: Start tree (basis tree) for comparison.
@@ -426,6 +428,9 @@
         :param show_ids: Show internal object ids.
         :param short: Use short status indicators.
         :param verbose: Verbose flag.
+        :param specific_files: If set, a list of filenames whose status should be
+            shown.  It is an error to give a filename that is not in the working
+            tree, or in the working inventory or in the basis inventory.
         """
         self.old_tree = old_tree
         self.new_tree = new_tree
@@ -434,14 +439,15 @@
         self.show_ids = show_ids
         self.short = short
         self.verbose = verbose
+        self.specific_files = specific_files
 
     def __eq__(self, other):
         return self.__dict__ == other.__dict__
 
     def __repr__(self):
-        return "<%s(%s, %s, %s, %s, %s, %s, %s)>" % (self.__class__.__name__,
+        return "<%s(%s, %s, %s, %s, %s, %s, %s, %s)>" % (self.__class__.__name__,
             self.old_tree, self.new_tree, self.to_file, self.versioned,
-            self.show_ids, self.short, self.verbose)
+            self.show_ids, self.short, self.verbose, self.specific_files)
 
 
 def _show_shelve_summary(params):
@@ -449,6 +455,10 @@
 
     :param params: StatusHookParams.
     """
+    # Don't show shelves if status of specific files is being shown, only if
+    # no file arguments have been passed
+    if params.specific_files:
+        return
     get_shelf_manager = getattr(params.new_tree, 'get_shelf_manager', None)
     if get_shelf_manager is None:
         return

=== modified file 'bzrlib/store/__init__.py'
--- bzrlib/store/__init__.py	2009-06-23 06:58:46 +0000
+++ bzrlib/store/__init__.py	2011-12-19 13:23:58 +0000
@@ -24,6 +24,8 @@
 unique ID.
 """
 
+from __future__ import absolute_import
+
 import os
 
 from bzrlib import (

=== modified file 'bzrlib/store/text.py'
--- bzrlib/store/text.py	2010-05-31 21:31:18 +0000
+++ bzrlib/store/text.py	2011-12-19 13:23:58 +0000
@@ -20,9 +20,10 @@
 do any sort of delta compression.
 """
 
+from __future__ import absolute_import
+
 import gzip
 import os
-from cStringIO import StringIO
 
 from bzrlib import osutils
 from bzrlib.errors import BzrError, NoSuchFile, FileExists

=== modified file 'bzrlib/store/versioned/__init__.py'
--- bzrlib/store/versioned/__init__.py	2011-01-19 20:39:41 +0000
+++ bzrlib/store/versioned/__init__.py	2011-12-18 15:28:38 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 # XXX: Some consideration of the problems that might occur if there are
 # files whose id differs only in case.  That should probably be forbidden.
 

=== modified file 'bzrlib/strace.py'
--- bzrlib/strace.py	2011-06-14 01:26:41 +0000
+++ bzrlib/strace.py	2011-12-19 13:23:58 +0000
@@ -17,7 +17,8 @@
 
 """Support for running strace against the current process."""
 
-import errno
+from __future__ import absolute_import
+
 import os
 import signal
 import subprocess
@@ -26,11 +27,6 @@
 from bzrlib import errors
 
 
-# this is currently test-focused, so importing bzrlib.tests is ok. We might
-# want to move feature to its own module though.
-from bzrlib.tests.features import Feature
-
-
 def strace(function, *args, **kwargs):
     """Invoke strace on function.
 
@@ -54,7 +50,7 @@
     # start strace
     strace_cmd = ['strace', '-r', '-tt', '-p', str(pid), '-o', log_file.name]
     if follow_children:
-        strace_args.append('-f')
+        strace_cmd.append('-f')
     # need to catch both stdout and stderr to work around
     # bug 627208
     proc = subprocess.Popen(strace_cmd,

=== modified file 'bzrlib/switch.py'
--- bzrlib/switch.py	2011-05-04 13:26:14 +0000
+++ bzrlib/switch.py	2011-12-18 15:28:38 +0000
@@ -14,13 +14,15 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 # Original author: David Allouche
 
 from bzrlib import errors, merge, revision
 from bzrlib.branch import Branch
+from bzrlib.i18n import gettext
 from bzrlib.trace import note
 
-
 def _run_post_switch_hooks(control_dir, to_branch, force, revision_id):
     from bzrlib.branch import SwitchHookParams
     hooks = Branch.hooks['post_switch']
@@ -33,7 +35,7 @@
 def switch(control_dir, to_branch, force=False, quiet=False, revision_id=None):
     """Switch the branch associated with a checkout.
 
-    :param control_dir: BzrDir of the checkout to change
+    :param control_dir: ControlDir of the checkout to change
     :param to_branch: branch that the checkout is to reference
     :param force: skip the check for local commits in a heavy checkout
     :param revision_id: revision ID to switch to.
@@ -51,7 +53,7 @@
 def _check_pending_merges(control, force=False):
     """Check that there are no outstanding pending merges before switching.
 
-    :param control: BzrDir of the branch to check
+    :param control: ControlDir of the branch to check
     """
     try:
         tree = control.open_workingtree()
@@ -64,14 +66,14 @@
     # XXX: Should the tree be locked for get_parent_ids?
     existing_pending_merges = tree.get_parent_ids()[1:]
     if len(existing_pending_merges) > 0:
-        raise errors.BzrCommandError('Pending merges must be '
-            'committed or reverted before using switch.')
+        raise errors.BzrCommandError(gettext('Pending merges must be '
+            'committed or reverted before using switch.'))
 
 
 def _set_branch_location(control, to_branch, force=False):
     """Set location value of a branch reference.
 
-    :param control: BzrDir of the checkout to change
+    :param control: ControlDir of the checkout to change
     :param to_branch: branch that the checkout is to reference
     :param force: skip the check for local commits in a heavy checkout
     """
@@ -90,14 +92,14 @@
             possible_transports = []
             try:
                 if not force and _any_local_commits(b, possible_transports):
-                    raise errors.BzrCommandError(
+                    raise errors.BzrCommandError(gettext(
                         'Cannot switch as local commits found in the checkout. '
                         'Commit these to the bound branch or use --force to '
-                        'throw them away.')
+                        'throw them away.'))
             except errors.BoundBranchConnectionFailure, e:
-                raise errors.BzrCommandError(
+                raise errors.BzrCommandError(gettext(
                         'Unable to connect to current master branch %(target)s: '
-                        '%(error)s To switch anyway, use --force.' %
+                        '%(error)s To switch anyway, use --force.') %
                         e.__dict__)
             b.set_bound_location(None)
             b.pull(to_branch, overwrite=True,
@@ -105,8 +107,8 @@
             b.set_bound_location(to_branch.base)
             b.set_parent(b.get_master_branch().get_parent())
         else:
-            raise errors.BzrCommandError('Cannot switch a branch, '
-                'only a checkout.')
+            raise errors.BzrCommandError(gettext('Cannot switch a branch, '
+                'only a checkout.'))
 
 
 def _any_local_commits(this_branch, possible_transports):
@@ -141,12 +143,12 @@
             revision_id = to_branch.last_revision()
         if tree.last_revision() == revision_id:
             if not quiet:
-                note("Tree is up to date at revision %d.", to_branch.revno())
+                note(gettext("Tree is up to date at revision %d."), to_branch.revno())
             return
         base_tree = source_repository.revision_tree(tree.last_revision())
         merge.Merge3Merger(tree, tree, base_tree, to_branch.repository.revision_tree(revision_id))
         tree.set_last_revision(to_branch.last_revision())
         if not quiet:
-            note('Updated to revision %d.' % to_branch.revno())
+            note(gettext('Updated to revision %d.') % to_branch.revno())
     finally:
         tree.unlock()

=== modified file 'bzrlib/symbol_versioning.py'
--- bzrlib/symbol_versioning.py	2010-08-28 15:34:24 +0000
+++ bzrlib/symbol_versioning.py	2011-12-19 13:23:58 +0000
@@ -19,6 +19,8 @@
 The methods here allow for api symbol versioning.
 """
 
+from __future__ import absolute_import
+
 __all__ = ['deprecated_function',
            'deprecated_in',
            'deprecated_list',

=== modified file 'bzrlib/tag.py'
--- bzrlib/tag.py	2011-05-18 16:42:48 +0000
+++ bzrlib/tag.py	2011-12-19 13:23:58 +0000
@@ -22,6 +22,8 @@
   Branch.tags.add('name', 'value')
 """
 
+from __future__ import absolute_import
+
 # NOTE: I was going to call this tags.py, but vim seems to think all files
 # called tags* are ctags files... mbp 20070220.
 
@@ -47,6 +49,63 @@
     def __init__(self, branch):
         self.branch = branch
 
+    def get_tag_dict(self):
+        """Return a dictionary mapping tags to revision ids.
+        """
+        raise NotImplementedError(self.get_tag_dict)
+
+    def get_reverse_tag_dict(self):
+        """Return a dictionary mapping revision ids to list of tags.
+        """
+        raise NotImplementedError(self.get_reverse_tag_dict)
+
+    def merge_to(self, to_tags, overwrite=False, ignore_master=False):
+        """Merge new tags from this tags container into another.
+
+        :param to_tags: Tags container to merge into
+        :param overwrite: Whether to overwrite existing, divergent, tags.
+        :param ignore_master: Do not modify the tags in the target's master
+            branch (if any).  Default is false (so the master will be updated).
+            New in bzr 2.3.
+        :return: Tuple with tag updates as dictionary and tag conflicts
+        """
+        raise NotImplementedError(self.merge_to)
+
+    def set_tag(self, tag_name, revision):
+        """Set a tag.
+
+        :param tag_name: Tag name
+        :param revision: Revision id
+        :raise GhostTagsNotSupported: if revision is not present in
+            the branch repository
+        """
+        raise NotImplementedError(self.set_tag)
+
+    def lookup_tag(self, tag_name):
+        """Look up a tag.
+
+        :param tag_name: Tag to look up
+        :raise NoSuchTag: Raised when tag does not exist
+        :return: Matching revision id
+        """
+        raise NotImplementedError(self.lookup_tag)
+
+    def delete_tag(self, tag_name):
+        """Delete a tag.
+
+        :param tag_name: Tag to delete
+        :raise NoSuchTag: Raised when tag does not exist
+        """
+        raise NotImplementedError(self.delete_tag)
+
+    def rename_revisions(self, rename_map):
+        """Replace revision ids according to a rename map.
+
+        :param rename_map: Dictionary mapping old revision ids to
+            new revision ids.
+        """
+        raise NotImplementedError(self.rename_revisions)
+
     def has_tag(self, tag_name):
         return self.get_tag_dict().has_key(tag_name)
 
@@ -68,7 +127,7 @@
 
     def merge_to(self, to_tags, overwrite=False, ignore_master=False):
         # we never have anything to copy
-        pass
+        return {}, []
 
     def rename_revisions(self, rename_map):
         # No tags, so nothing to rename
@@ -201,7 +260,10 @@
             branch (if any).  Default is false (so the master will be updated).
             New in bzr 2.3.
 
-        :returns: A set of tags that conflicted, each of which is
+        :returns: Tuple with tag_updates and tag_conflicts.
+            tag_updates is a dictionary with new tags, None is used for
+            removed tags
+            tag_conflicts is a set of tags that conflicted, each of which is
             (tagname, source_target, dest_target), or None if no copying was
             done.
         """
@@ -211,15 +273,15 @@
     def _merge_to_operation(self, operation, to_tags, overwrite, ignore_master):
         add_cleanup = operation.add_cleanup
         if self.branch == to_tags.branch:
-            return
+            return {}, []
         if not self.branch.supports_tags():
             # obviously nothing to copy
-            return
+            return {}, []
         source_dict = self.get_tag_dict()
         if not source_dict:
             # no tags in the source, and we don't want to clobber anything
             # that's in the destination
-            return
+            return {}, []
         # We merge_to both master and child individually.
         #
         # It's possible for master and child to have differing sets of
@@ -239,25 +301,27 @@
             master = to_tags.branch.get_master_branch()
         if master is not None:
             add_cleanup(master.lock_write().unlock)
-        conflicts = self._merge_to(to_tags, source_dict, overwrite)
+        updates, conflicts = self._merge_to(to_tags, source_dict, overwrite)
         if master is not None:
-            conflicts += self._merge_to(master.tags, source_dict,
-                overwrite)
+            extra_updates, extra_conflicts = self._merge_to(master.tags,
+                source_dict, overwrite)
+            updates.update(extra_updates)
+            conflicts += extra_conflicts
         # We use set() to remove any duplicate conflicts from the master
         # branch.
-        return set(conflicts)
+        return updates, set(conflicts)
 
     def _merge_to(self, to_tags, source_dict, overwrite):
         dest_dict = to_tags.get_tag_dict()
-        result, conflicts = self._reconcile_tags(source_dict, dest_dict,
-                                                 overwrite)
+        result, updates, conflicts = self._reconcile_tags(source_dict,
+            dest_dict, overwrite)
         if result != dest_dict:
             to_tags._set_tag_dict(result)
-        return conflicts
+        return updates, conflicts
 
     def rename_revisions(self, rename_map):
         """Rename revisions in this tags dictionary.
-        
+
         :param rename_map: Dictionary mapping old revids to new revids
         """
         reverse_tags = self.get_reverse_tag_dict()
@@ -275,19 +339,21 @@
         * different definitions => if overwrite is False, keep destination
           value and give a warning, otherwise use the source value
 
-        :returns: (result_dict,
+        :returns: (result_dict, updates,
             [(conflicting_tag, source_target, dest_target)])
         """
         conflicts = []
+        updates = {}
         result = dict(dest_dict) # copy
         for name, target in source_dict.items():
-            if name not in result or overwrite:
+            if result.get(name) == target:
+                pass
+            elif name not in result or overwrite:
+                updates[name] = target
                 result[name] = target
-            elif result[name] == target:
-                pass
             else:
                 conflicts.append((name, target, result[name]))
-        return result, conflicts
+        return result, updates, conflicts
 
 
 def _merge_tags_if_possible(from_branch, to_branch, ignore_master=False):

=== modified file 'bzrlib/testament.py'
--- bzrlib/testament.py	2011-05-11 14:28:36 +0000
+++ bzrlib/testament.py	2011-12-19 13:23:58 +0000
@@ -59,6 +59,8 @@
 * the testament uses unix line-endings (\n)
 """
 
+from __future__ import absolute_import
+
 # XXX: At the moment, clients trust that the graph described in a weave
 # is accurate, but that's not covered by the testament.  Perhaps the best
 # fix is when verifying a revision to make sure that every file mentioned

=== modified file 'bzrlib/tests/TestUtil.py'
--- bzrlib/tests/TestUtil.py	2011-05-26 08:05:45 +0000
+++ bzrlib/tests/TestUtil.py	2011-10-04 16:39:38 +0000
@@ -19,6 +19,7 @@
 import sys
 import logging
 import unittest
+import weakref
 
 from bzrlib import pyutils
 
@@ -66,6 +67,24 @@
                     test, test.__class__)
 
 
+class FailedCollectionCase(unittest.TestCase):
+    """Pseudo-test to run and report failure if given case was uncollected"""
+
+    def __init__(self, case):
+        super(FailedCollectionCase, self).__init__("fail_uncollected")
+        # GZ 2011-09-16: Maybe catch errors from id() method as cases may be
+        #                in a bit of a funny state by now.
+        self._problem_case_id = case.id()
+
+    def id(self):
+        if self._problem_case_id[-1:] == ")":
+            return self._problem_case_id[:-1] + ",uncollected)"
+        return self._problem_case_id + "(uncollected)"
+
+    def fail_uncollected(self):
+        self.fail("Uncollected test case: " + self._problem_case_id)
+
+
 class TestSuite(unittest.TestSuite):
     """I am an extended TestSuite with a visitor interface.
     This is primarily to allow filtering of tests - and suites or
@@ -82,14 +101,34 @@
         tests = list(self)
         tests.reverse()
         self._tests = []
+        stored_count = 0
+        count_stored_tests = getattr(result, "_count_stored_tests", int)
+        from bzrlib.tests import selftest_debug_flags
+        notify = "uncollected_cases" in selftest_debug_flags
         while tests:
             if result.shouldStop:
                 self._tests = reversed(tests)
                 break
-            tests.pop().run(result)
+            case = _run_and_collect_case(tests.pop(), result)()
+            new_stored_count = count_stored_tests()
+            if case is not None and isinstance(case, unittest.TestCase):
+                if stored_count == new_stored_count and notify:
+                    # Testcase didn't fail, but somehow is still alive
+                    FailedCollectionCase(case).run(result)
+                    # Adding a new failure so need to reupdate the count
+                    new_stored_count = count_stored_tests()
+                # GZ 2011-09-16: Previously zombied the case at this point by
+                #                clearing the dict as fallback, skip for now.
+            stored_count = new_stored_count
         return result
 
 
+def _run_and_collect_case(case, res):
+    """Run test case against result and use weakref to drop the refcount"""
+    case.run(res)
+    return weakref.ref(case)
+
+
 class TestLoader(unittest.TestLoader):
     """Custom TestLoader to extend the stock python one."""
 

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2011-08-20 22:42:34 +0000
+++ bzrlib/tests/__init__.py	2011-12-24 10:10:59 +0000
@@ -16,6 +16,8 @@
 
 """Testing framework extensions"""
 
+from __future__ import absolute_import
+
 # NOTE: Some classes in here use camelCaseNaming() rather than
 # underscore_naming().  That's for consistency with unittest; it's not the
 # general style of bzrlib.  Please continue that consistency when adding e.g.
@@ -36,6 +38,7 @@
 import random
 import re
 import shlex
+import site
 import stat
 import subprocess
 import sys
@@ -61,6 +64,7 @@
     chk_map,
     commands as _mod_commands,
     config,
+    i18n,
     debug,
     errors,
     hooks,
@@ -93,12 +97,14 @@
     deprecated_in,
     )
 from bzrlib.tests import (
+    fixtures,
     test_server,
     TestUtil,
     treeshape,
     )
 from bzrlib.ui import NullProgressView
 from bzrlib.ui.text import TextUIFactory
+from bzrlib.tests.features import _CompatabilityThunkFeature
 
 # Mark this python module as being part of the implementation
 # of unittest: this gives us better tracebacks where the last
@@ -211,6 +217,27 @@
         osutils.set_or_unset_env(var, value)
 
 
+def _clear__type_equality_funcs(test):
+    """Cleanup bound methods stored on TestCase instances
+
+    Clear the dict breaking a few (mostly) harmless cycles in the affected
+    unittests released with Python 2.6 and initial Python 2.7 versions.
+
+    For a few revisions between Python 2.7.1 and Python 2.7.2 that annoyingly
+    shipped in Oneiric, an object with no clear method was used, hence the
+    extra complications, see bug 809048 for details.
+    """
+    type_equality_funcs = getattr(test, "_type_equality_funcs", None)
+    if type_equality_funcs is not None:
+        tef_clear = getattr(type_equality_funcs, "clear", None)
+        if tef_clear is None:
+            tef_instance_dict = getattr(type_equality_funcs, "__dict__", None)
+            if tef_instance_dict is not None:
+                tef_clear = tef_instance_dict.clear
+        if tef_clear is not None:
+            tef_clear()
+
+
 class ExtendedTestResult(testtools.TextTestResult):
     """Accepts, reports and accumulates the results of running tests.
 
@@ -384,18 +411,7 @@
         getDetails = getattr(test, "getDetails", None)
         if getDetails is not None:
             getDetails().clear()
-        # Clear _type_equality_funcs to try to stop TestCase instances
-        # from wasting memory. 'clear' is not available in all Python
-        # versions (bug 809048)
-        type_equality_funcs = getattr(test, "_type_equality_funcs", None)
-        if type_equality_funcs is not None:
-            tef_clear = getattr(type_equality_funcs, "clear", None)
-            if tef_clear is None:
-                tef_instance_dict = getattr(type_equality_funcs, "__dict__", None)
-                if tef_instance_dict is not None:
-                    tef_clear = tef_instance_dict.clear
-            if tef_clear is not None:
-                tef_clear()
+        _clear__type_equality_funcs(test)
         self._traceback_from_test = None
 
     def startTests(self):
@@ -502,6 +518,10 @@
         self.not_applicable_count += 1
         self.report_not_applicable(test, reason)
 
+    def _count_stored_tests(self):
+        """Count of tests instances kept alive due to not succeeding"""
+        return self.error_count + self.failure_count + self.known_failure_count
+
     def _post_mortem(self, tb=None):
         """Start a PDB post mortem session."""
         if os.environ.get('BZR_TEST_PDB', None):
@@ -979,9 +999,21 @@
 
     def setUp(self):
         super(TestCase, self).setUp()
+
+        timeout = config.GlobalStack().get('selftest.timeout')
+        if timeout:
+            timeout_fixture = fixtures.TimeoutFixture(timeout)
+            timeout_fixture.setUp()
+            self.addCleanup(timeout_fixture.cleanUp)
+
         for feature in getattr(self, '_test_needs_features', []):
             self.requireFeature(feature)
         self._cleanEnvironment()
+
+        if bzrlib.global_state is not None:
+            self.overrideAttr(bzrlib.global_state, 'cmdline_overrides',
+                              config.CommandLineStore())
+
         self._silenceUI()
         self._startLogFile()
         self._benchcalls = []
@@ -1005,6 +1037,8 @@
         self._counters = {}
         if 'config_stats' in selftest_debug_flags:
             self._install_config_stats_hooks()
+        # Do not use i18n for tests (unless the test reverses this)
+        i18n.disable_i18n()
 
     def debug(self):
         # debug a frame up.
@@ -1698,10 +1732,7 @@
         return result
 
     def _startLogFile(self):
-        """Send bzr and test log messages to a temporary file.
-
-        The file is removed as the test is torn down.
-        """
+        """Setup a in-memory target for bzr and testcase log messages"""
         pseudo_log_file = StringIO()
         def _get_log_contents_for_weird_testtools_api():
             return [pseudo_log_file.getvalue().decode(
@@ -1714,14 +1745,13 @@
         self.addCleanup(self._finishLogFile)
 
     def _finishLogFile(self):
-        """Finished with the log file.
-
-        Close the file and delete it.
-        """
+        """Flush and dereference the in-memory log for this testcase"""
         if trace._trace_file:
             # flush the log file, to get all content
             trace._trace_file.flush()
         trace.pop_log_file(self._log_memento)
+        # The logging module now tracks references for cleanup so discard ours
+        del self._log_memento
 
     def thisFailsStrictLockCheck(self):
         """It is known that this test would fail with -Dstrict_locks.
@@ -1962,8 +1992,8 @@
 
         self.log('run bzr: %r', args)
         # FIXME: don't call into logging here
-        handler = logging.StreamHandler(stderr)
-        handler.setLevel(logging.INFO)
+        handler = trace.EncodedStreamHandler(stderr, errors="replace",
+            level=logging.INFO)
         logger = logging.getLogger('')
         logger.addHandler(handler)
         old_ui_factory = ui.ui_factory
@@ -2157,6 +2187,11 @@
 
         if env_changes is None:
             env_changes = {}
+        # Because $HOME is set to a tempdir for the context of a test, modules
+        # installed in the user dir will not be found unless $PYTHONUSERBASE
+        # gets set to the computed directory of this parent process.
+        if site.USER_BASE is not None:
+            env_changes["PYTHONUSERBASE"] = site.USER_BASE
         old_env = {}
 
         def cleanup_environment():
@@ -2353,8 +2388,10 @@
         from bzrlib.smart import request
         request_handlers = request.request_handlers
         orig_method = request_handlers.get(verb)
+        orig_info = request_handlers.get_info(verb)
         request_handlers.remove(verb)
-        self.addCleanup(request_handlers.register, verb, orig_method)
+        self.addCleanup(request_handlers.register, verb, orig_method,
+            info=orig_info)
 
 
 class CapturedCall(object):
@@ -2561,7 +2598,13 @@
         real branch.
         """
         root = TestCaseWithMemoryTransport.TEST_ROOT
+        # Make sure we get a readable and accessible home for .bzr.log
+        # and/or config files, and not fallback to weird defaults (see
+        # http://pad.lv/825027).
+        self.assertIs(None, os.environ.get('BZR_HOME', None))
+        os.environ['BZR_HOME'] = root
         wt = bzrdir.BzrDir.create_standalone_workingtree(root)
+        del os.environ['BZR_HOME']
         # Hack for speed: remember the raw bytes of the dirstate file so that
         # we don't need to re-open the wt to check it hasn't changed.
         TestCaseWithMemoryTransport._SAFETY_NET_PRISTINE_DIRSTATE = (
@@ -2618,7 +2661,26 @@
     def make_branch(self, relpath, format=None):
         """Create a branch on the transport at relpath."""
         repo = self.make_repository(relpath, format=format)
-        return repo.bzrdir.create_branch()
+        return repo.bzrdir.create_branch(append_revisions_only=False)
+
+    def get_default_format(self):
+        return 'default'
+
+    def resolve_format(self, format):
+        """Resolve an object to a ControlDir format object.
+
+        The initial format object can either already be
+        a ControlDirFormat, None (for the default format),
+        or a string with the name of the control dir format.
+
+        :param format: Object to resolve
+        :return A ControlDirFormat instance
+        """
+        if format is None:
+            format = self.get_default_format()
+        if isinstance(format, basestring):
+            format = bzrdir.format_registry.make_bzrdir(format)
+        return format
 
     def make_bzrdir(self, relpath, format=None):
         try:
@@ -2628,15 +2690,12 @@
             t = _mod_transport.get_transport(maybe_a_url)
             if len(segments) > 1 and segments[-1] not in ('', '.'):
                 t.ensure_base()
-            if format is None:
-                format = 'default'
-            if isinstance(format, basestring):
-                format = bzrdir.format_registry.make_bzrdir(format)
+            format = self.resolve_format(format)
             return format.initialize_on_transport(t)
         except errors.UninitializableFormat:
             raise TestSkipped("Format %s is not initializable." % format)
 
-    def make_repository(self, relpath, shared=False, format=None):
+    def make_repository(self, relpath, shared=None, format=None):
         """Create a repository on our default transport at relpath.
 
         Note that relpath must be a relative path, not a full url.
@@ -2675,16 +2734,20 @@
 
     def setUp(self):
         super(TestCaseWithMemoryTransport, self).setUp()
-        # Ensure that ConnectedTransport doesn't leak sockets
-        def get_transport_from_url_with_cleanup(*args, **kwargs):
-            t = orig_get_transport_from_url(*args, **kwargs)
-            if isinstance(t, _mod_transport.ConnectedTransport):
-                self.addCleanup(t.disconnect)
-            return t
-
-        orig_get_transport_from_url = self.overrideAttr(
-            _mod_transport, 'get_transport_from_url',
-            get_transport_from_url_with_cleanup)
+
+        def _add_disconnect_cleanup(transport):
+            """Schedule disconnection of given transport at test cleanup
+
+            This needs to happen for all connected transports or leaks occur.
+
+            Note reconnections may mean we call disconnect multiple times per
+            transport which is suboptimal but seems harmless.
+            """
+            self.addCleanup(transport.disconnect)
+ 
+        _mod_transport.Transport.hooks.install_named_hook('post_connect',
+            _add_disconnect_cleanup, None)
+
         self._make_test_root()
         self.addCleanup(os.chdir, os.getcwdu())
         self.makeAndChdirToTestDir()
@@ -2696,6 +2759,7 @@
     def setup_smart_server_with_call_log(self):
         """Sets up a smart server as the transport server with a call log."""
         self.transport_server = test_server.SmartTCPServer_for_testing
+        self.hpss_connections = []
         self.hpss_calls = []
         import traceback
         # Skip the current stack down to the caller of
@@ -2704,11 +2768,16 @@
         def capture_hpss_call(params):
             self.hpss_calls.append(
                 CapturedCall(params, prefix_length))
+        def capture_connect(transport):
+            self.hpss_connections.append(transport)
         client._SmartClient.hooks.install_named_hook(
             'call', capture_hpss_call, None)
+        _mod_transport.Transport.hooks.install_named_hook(
+            'post_connect', capture_connect, None)
 
     def reset_smart_call_log(self):
         self.hpss_calls = []
+        self.hpss_connections = []
 
 
 class TestCaseInTempDir(TestCaseWithMemoryTransport):
@@ -2916,6 +2985,10 @@
         # this obviously requires a format that supports branch references
         # so check for that by checking bzrdir.BzrDirFormat.get_default_format()
         # RBC 20060208
+        format = self.resolve_format(format=format)
+        if not format.supports_workingtrees:
+            b = self.make_branch(relpath+'.branch', format=format)
+            return b.create_checkout(relpath, lightweight=True)
         b = self.make_branch(relpath, format=format)
         try:
             return b.bzrdir.create_workingtree()
@@ -3220,6 +3293,9 @@
                             result_decorators=result_decorators,
                             )
     runner.stop_on_failure=stop_on_failure
+    if isinstance(suite, unittest.TestSuite):
+        # Empty out _tests list of passed suite and populate new TestSuite
+        suite._tests[:], suite = [], TestSuite(suite)
     # built in decorator factories:
     decorators = [
         random_order(random_seed, runner),
@@ -3323,34 +3399,17 @@
 
 class TestDecorator(TestUtil.TestSuite):
     """A decorator for TestCase/TestSuite objects.
-    
-    Usually, subclasses should override __iter__(used when flattening test
-    suites), which we do to filter, reorder, parallelise and so on, run() and
-    debug().
+
+    Contains rather than flattening suite passed on construction
     """
 
-    def __init__(self, suite):
-        TestUtil.TestSuite.__init__(self)
-        self.addTest(suite)
-
-    def countTestCases(self):
-        cases = 0
-        for test in self:
-            cases += test.countTestCases()
-        return cases
-
-    def debug(self):
-        for test in self:
-            test.debug()
-
-    def run(self, result):
-        # Use iteration on self, not self._tests, to allow subclasses to hook
-        # into __iter__.
-        for test in self:
-            if result.shouldStop:
-                break
-            test.run(result)
-        return result
+    def __init__(self, suite=None):
+        super(TestDecorator, self).__init__()
+        if suite is not None:
+            self.addTest(suite)
+
+    # Don't need subclass run method with suite emptying
+    run = unittest.TestSuite.run
 
 
 class CountingDecorator(TestDecorator):
@@ -3367,90 +3426,50 @@
     """A decorator which excludes test matching an exclude pattern."""
 
     def __init__(self, suite, exclude_pattern):
-        TestDecorator.__init__(self, suite)
-        self.exclude_pattern = exclude_pattern
-        self.excluded = False
-
-    def __iter__(self):
-        if self.excluded:
-            return iter(self._tests)
-        self.excluded = True
-        suite = exclude_tests_by_re(self, self.exclude_pattern)
-        del self._tests[:]
-        self.addTests(suite)
-        return iter(self._tests)
+        super(ExcludeDecorator, self).__init__(
+            exclude_tests_by_re(suite, exclude_pattern))
 
 
 class FilterTestsDecorator(TestDecorator):
     """A decorator which filters tests to those matching a pattern."""
 
     def __init__(self, suite, pattern):
-        TestDecorator.__init__(self, suite)
-        self.pattern = pattern
-        self.filtered = False
-
-    def __iter__(self):
-        if self.filtered:
-            return iter(self._tests)
-        self.filtered = True
-        suite = filter_suite_by_re(self, self.pattern)
-        del self._tests[:]
-        self.addTests(suite)
-        return iter(self._tests)
+        super(FilterTestsDecorator, self).__init__(
+            filter_suite_by_re(suite, pattern))
 
 
 class RandomDecorator(TestDecorator):
     """A decorator which randomises the order of its tests."""
 
     def __init__(self, suite, random_seed, stream):
-        TestDecorator.__init__(self, suite)
-        self.random_seed = random_seed
-        self.randomised = False
-        self.stream = stream
-
-    def __iter__(self):
-        if self.randomised:
-            return iter(self._tests)
-        self.randomised = True
-        self.stream.write("Randomizing test order using seed %s\n\n" %
-            (self.actual_seed()))
+        random_seed = self.actual_seed(random_seed)
+        stream.write("Randomizing test order using seed %s\n\n" %
+            (random_seed,))
         # Initialise the random number generator.
-        random.seed(self.actual_seed())
-        suite = randomize_suite(self)
-        del self._tests[:]
-        self.addTests(suite)
-        return iter(self._tests)
+        random.seed(random_seed)
+        super(RandomDecorator, self).__init__(randomize_suite(suite))
 
-    def actual_seed(self):
-        if self.random_seed == "now":
+    @staticmethod
+    def actual_seed(seed):
+        if seed == "now":
             # We convert the seed to a long to make it reuseable across
             # invocations (because the user can reenter it).
-            self.random_seed = long(time.time())
+            return long(time.time())
         else:
             # Convert the seed to a long if we can
             try:
-                self.random_seed = long(self.random_seed)
-            except:
+                return long(seed)
+            except (TypeError, ValueError):
                 pass
-        return self.random_seed
+        return seed
 
 
 class TestFirstDecorator(TestDecorator):
     """A decorator which moves named tests to the front."""
 
     def __init__(self, suite, pattern):
-        TestDecorator.__init__(self, suite)
-        self.pattern = pattern
-        self.filtered = False
-
-    def __iter__(self):
-        if self.filtered:
-            return iter(self._tests)
-        self.filtered = True
-        suites = split_suite_by_re(self, self.pattern)
-        del self._tests[:]
-        self.addTests(suites)
-        return iter(self._tests)
+        super(TestFirstDecorator, self).__init__()
+        self.addTests(split_suite_by_re(suite, pattern))
 
 
 def partition_tests(suite, count):
@@ -3488,7 +3507,7 @@
     """
     concurrency = osutils.local_concurrency()
     result = []
-    from subunit import TestProtocolClient, ProtocolTestCase
+    from subunit import ProtocolTestCase
     from subunit.test_results import AutoTimingTestResultDecorator
     class TestInOtherProcess(ProtocolTestCase):
         # Should be in subunit, I think. RBC.
@@ -3500,30 +3519,42 @@
             try:
                 ProtocolTestCase.run(self, result)
             finally:
-                os.waitpid(self.pid, 0)
+                pid, status = os.waitpid(self.pid, 0)
+            # GZ 2011-10-18: If status is nonzero, should report to the result
+            #                that something went wrong.
 
     test_blocks = partition_tests(suite, concurrency)
+    # Clear the tests from the original suite so it doesn't keep them alive
+    suite._tests[:] = []
     for process_tests in test_blocks:
-        process_suite = TestUtil.TestSuite()
-        process_suite.addTests(process_tests)
+        process_suite = TestUtil.TestSuite(process_tests)
+        # Also clear each split list so new suite has only reference
+        process_tests[:] = []
         c2pread, c2pwrite = os.pipe()
         pid = os.fork()
         if pid == 0:
-            workaround_zealous_crypto_random()
             try:
+                stream = os.fdopen(c2pwrite, 'wb', 1)
+                workaround_zealous_crypto_random()
                 os.close(c2pread)
                 # Leave stderr and stdout open so we can see test noise
                 # Close stdin so that the child goes away if it decides to
                 # read from stdin (otherwise its a roulette to see what
                 # child actually gets keystrokes for pdb etc).
                 sys.stdin.close()
-                sys.stdin = None
-                stream = os.fdopen(c2pwrite, 'wb', 1)
                 subunit_result = AutoTimingTestResultDecorator(
-                    TestProtocolClient(stream))
+                    SubUnitBzrProtocolClient(stream))
                 process_suite.run(subunit_result)
-            finally:
-                os._exit(0)
+            except:
+                # Try and report traceback on stream, but exit with error even
+                # if stream couldn't be created or something else goes wrong.
+                # The traceback is formatted to a string and written in one go
+                # to avoid interleaving lines from multiple failing children.
+                try:
+                    stream.write(traceback.format_exc())
+                finally:
+                    os._exit(1)
+            os._exit(0)
         else:
             os.close(c2pwrite)
             stream = os.fdopen(c2pread, 'rb', 1)
@@ -3635,7 +3666,8 @@
 #                           with proper exclusion rules.
 #   -Ethreads               Will display thread ident at creation/join time to
 #                           help track thread leaks
-
+#   -Euncollected_cases     Display the identity of any test cases that weren't
+#                           deallocated after being completed.
 #   -Econfig_stats          Will collect statistics using addDetail
 selftest_debug_flags = set()
 
@@ -3946,6 +3978,7 @@
         'bzrlib.tests.test_email_message',
         'bzrlib.tests.test_eol_filters',
         'bzrlib.tests.test_errors',
+        'bzrlib.tests.test_estimate_compressed_size',
         'bzrlib.tests.test_export',
         'bzrlib.tests.test_export_pot',
         'bzrlib.tests.test_extract',
@@ -4035,6 +4068,7 @@
         'bzrlib.tests.test_smart',
         'bzrlib.tests.test_smart_add',
         'bzrlib.tests.test_smart_request',
+        'bzrlib.tests.test_smart_signals',
         'bzrlib.tests.test_smart_transport',
         'bzrlib.tests.test_smtp_connection',
         'bzrlib.tests.test_source',
@@ -4071,6 +4105,7 @@
         'bzrlib.tests.test_version',
         'bzrlib.tests.test_version_info',
         'bzrlib.tests.test_versionedfile',
+        'bzrlib.tests.test_vf_search',
         'bzrlib.tests.test_weave',
         'bzrlib.tests.test_whitebox',
         'bzrlib.tests.test_win32utils',
@@ -4437,6 +4472,10 @@
     from subunit.test_results import AutoTimingTestResultDecorator
     class SubUnitBzrProtocolClient(TestProtocolClient):
 
+        def stopTest(self, test):
+            super(SubUnitBzrProtocolClient, self).stopTest(test)
+            _clear__type_equality_funcs(test)
+
         def addSuccess(self, test, details=None):
             # The subunit client always includes the details in the subunit
             # stream, but we don't want to include it in ours.
@@ -4455,8 +4494,28 @@
     pass
 
 
-@deprecated_function(deprecated_in((2, 5, 0)))
-def ModuleAvailableFeature(name):
-    from bzrlib.tests import features
-    return features.ModuleAvailableFeature(name)
-    
+# API compatibility for old plugins; see bug 892622.
+for name in [
+    'Feature',
+    'HTTPServerFeature', 
+    'ModuleAvailableFeature',
+    'HTTPSServerFeature', 'SymlinkFeature', 'HardlinkFeature',
+    'OsFifoFeature', 'UnicodeFilenameFeature',
+    'ByteStringNamedFilesystem', 'UTF8Filesystem',
+    'BreakinFeature', 'CaseInsCasePresFilenameFeature',
+    'CaseInsensitiveFilesystemFeature', 'case_sensitive_filesystem_feature',
+    'posix_permissions_feature',
+    ]:
+    globals()[name] = _CompatabilityThunkFeature(
+        symbol_versioning.deprecated_in((2, 5, 0)),
+        'bzrlib.tests', name,
+        name, 'bzrlib.tests.features')
+
+
+for (old_name, new_name) in [
+    ('UnicodeFilename', 'UnicodeFilenameFeature'),
+    ]:
+    globals()[name] = _CompatabilityThunkFeature(
+        symbol_versioning.deprecated_in((2, 5, 0)),
+        'bzrlib.tests', old_name,
+        new_name, 'bzrlib.tests.features')

=== modified file 'bzrlib/tests/blackbox/__init__.py'
--- bzrlib/tests/blackbox/__init__.py	2011-08-15 15:48:12 +0000
+++ bzrlib/tests/blackbox/__init__.py	2011-12-29 21:03:53 +0000
@@ -64,6 +64,7 @@
                      'test_dpush',
                      'test_exceptions',
                      'test_export',
+                     'test_export_pot',
                      'test_filesystem_cicp',
                      'test_filtered_view_ops',
                      'test_find_merge_base',
@@ -84,6 +85,7 @@
                      'test_merge',
                      'test_merge_directive',
                      'test_missing',
+                     'test_mkdir',
                      'test_modified',
                      'test_mv',
                      'test_nick',
@@ -112,6 +114,7 @@
                      'test_send',
                      'test_serve',
                      'test_shared_repository',
+                     'test_shell_complete',
                      'test_shelve',
                      'test_sign_my_commits',
                      'test_split',

=== modified file 'bzrlib/tests/blackbox/test_annotate.py'
--- bzrlib/tests/blackbox/test_annotate.py	2011-06-13 15:18:27 +0000
+++ bzrlib/tests/blackbox/test_annotate.py	2011-12-30 15:26:18 +0000
@@ -29,6 +29,7 @@
     tests,
     )
 
+from bzrlib.tests.matchers import ContainsNoVfsCalls
 from bzrlib.urlutils import joinpath
 
 
@@ -307,3 +308,25 @@
         wt.commit('commit', committer='test@user')
         out, err = self.run_bzr(['annotate', '-d', 'a', 'hello.txt'])
         self.assertEqualDiff('1   test@us | my helicopter\n', out)
+
+
+class TestSmartServerAnnotate(tests.TestCaseWithTransport):
+
+    def test_simple_annotate(self):
+        self.setup_smart_server_with_call_log()
+        wt = self.make_branch_and_tree('branch')
+        self.build_tree_contents([('branch/hello.txt', 'my helicopter\n')])
+        wt.add(['hello.txt'])
+        wt.commit('commit', committer='test@user')
+        self.reset_smart_call_log()
+        out, err = self.run_bzr(['annotate', "-d", self.get_url('branch'),
+            "hello.txt"])
+        # This figure represent the amount of work to perform this use case. It
+        # is entirely ok to reduce this number if a test fails due to rpc_count
+        # being too low. If rpc_count increases, more network roundtrips have
+        # become necessary for this use case. Please do not adjust this number
+        # upwards without agreement from bzr's network support maintainers.
+        self.assertLength(16, self.hpss_calls)
+        self.assertLength(1, self.hpss_connections)
+        self.expectFailure("annotate accesses inventories, which require VFS access",
+            self.assertThat, self.hpss_calls, ContainsNoVfsCalls)

=== modified file 'bzrlib/tests/blackbox/test_bound_branches.py'
--- bzrlib/tests/blackbox/test_bound_branches.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/blackbox/test_bound_branches.py	2011-09-26 15:21:01 +0000
@@ -49,7 +49,7 @@
 
     def check_revno(self, val, loc='.'):
         self.assertEqual(
-            val, len(BzrDir.open(loc).open_branch().revision_history()))
+            val, BzrDir.open(loc).open_branch().last_revision_info()[0])
 
     def test_simple_binding(self):
         tree = self.make_branch_and_tree('base')
@@ -229,10 +229,9 @@
         child_tree.commit(message='merged')
         self.check_revno(3)
 
-        # After binding, the revision history should be unaltered
-        # take a copy before
-        base_history = base_branch.revision_history()
-        child_history = child_branch.revision_history()
+        self.assertEquals(
+            child_tree.branch.last_revision(),
+            base_tree.branch.last_revision())
 
     def test_bind_parent_ahead(self):
         base_tree = self.create_branches()[0]
@@ -333,7 +332,7 @@
         self.build_tree_contents([('other/c', 'file c\n')])
         other_tree.add('c')
         other_tree.commit(message='adding c')
-        new_rev_id = other_branch.revision_history()[-1]
+        new_rev_id = other_branch.last_revision()
 
         child_tree.merge_from_branch(other_branch)
 

=== modified file 'bzrlib/tests/blackbox/test_branch.py'
--- bzrlib/tests/blackbox/test_branch.py	2011-08-12 09:49:24 +0000
+++ bzrlib/tests/blackbox/test_branch.py	2012-01-02 10:24:02 +0000
@@ -22,6 +22,7 @@
 from bzrlib import (
     branch,
     bzrdir,
+    controldir,
     errors,
     revision as _mod_revision,
     )
@@ -29,13 +30,13 @@
 from bzrlib.tests import TestCaseWithTransport
 from bzrlib.tests import (
     fixtures,
-    script,
     test_server,
     )
 from bzrlib.tests.features import (
     HardlinkFeature,
     )
 from bzrlib.tests.blackbox import test_switch
+from bzrlib.tests.matchers import ContainsNoVfsCalls
 from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
 from bzrlib.tests.script import run_script
 from bzrlib.urlutils import local_path_to_url, strip_trailing_slash
@@ -44,14 +45,15 @@
 
 class TestBranch(TestCaseWithTransport):
 
-    def example_branch(self, path='.'):
-        tree = self.make_branch_and_tree(path)
+    def example_branch(self, path='.', format=None):
+        tree = self.make_branch_and_tree(path, format=format)
         self.build_tree_contents([(path + '/hello', 'foo')])
         tree.add('hello')
         tree.commit(message='setup')
         self.build_tree_contents([(path + '/goodbye', 'baz')])
         tree.add('goodbye')
         tree.commit(message='setup')
+        return tree
 
     def test_branch(self):
         """Branch from one branch to another."""
@@ -63,14 +65,54 @@
         self.assertFalse(b._transport.has('branch-name'))
         b.bzrdir.open_workingtree().commit(message='foo', allow_pointless=True)
 
+    def test_into_colocated(self):
+        """Branch from a branch into a colocated branch."""
+        self.example_branch('a')
+        out, err = self.run_bzr(
+            'init --format=development-colo file:b,branch=orig')
+        self.assertEqual(
+            """Created a lightweight checkout (format: development-colo)\n""",
+            out)
+        self.assertEqual('', err)
+        out, err = self.run_bzr(
+            'branch a file:b,branch=thiswasa')
+        self.assertEqual('', out)
+        self.assertEqual('Branched 2 revisions.\n', err)
+        out, err = self.run_bzr('branches b')
+        self.assertEqual("  orig\n  thiswasa\n", out)
+        self.assertEqual('', err)
+        out,err = self.run_bzr('branch a file:b,branch=orig', retcode=3)
+        self.assertEqual('', out)
+        self.assertEqual('bzr: ERROR: Already a branch: "file:b,branch=orig".\n', err)
+
+    def test_from_colocated(self):
+        """Branch from a colocated branch into a regular branch."""
+        tree = self.example_branch('a', format='development-colo')
+        tree.bzrdir.create_branch(name='somecolo')
+        out, err = self.run_bzr('branch %s,branch=somecolo' %
+            local_path_to_url('a'))
+        self.assertEqual('', out)
+        self.assertEqual('Branched 0 revisions.\n', err)
+        self.assertPathExists("somecolo")
+
     def test_branch_broken_pack(self):
         """branching with a corrupted pack file."""
         self.example_branch('a')
-        #now add some random corruption
-        fname = 'a/.bzr/repository/packs/' + os.listdir('a/.bzr/repository/packs')[0]
+        # add some corruption
+        packs_dir = 'a/.bzr/repository/packs/'
+        fname = packs_dir + os.listdir(packs_dir)[0]
         with open(fname, 'rb+') as f:
-            f.seek(750)
-            f.write("\xff")
+            # Start from the end of the file to avoid choosing a place bigger
+            # than the file itself.
+            f.seek(-5, os.SEEK_END)
+            c = f.read(1)
+            f.seek(-5, os.SEEK_END)
+            # Make sure we inject a value different than the one we just read
+            if c == '\xFF':
+                corrupt = '\x00'
+            else:
+                corrupt = '\xFF'
+            f.write(corrupt) # make sure we corrupt something
         self.run_bzr_error(['Corruption while decompressing repository file'], 
                             'branch a b', retcode=3)
 
@@ -149,7 +191,7 @@
 
         def make_shared_tree(path):
             shared_repo.bzrdir.root_transport.mkdir(path)
-            shared_repo.bzrdir.create_branch_convenience('repo/' + path)
+            controldir.ControlDir.create_branch_convenience('repo/' + path)
             return WorkingTree.open('repo/' + path)
         tree_a = make_shared_tree('a')
         self.build_tree(['repo/a/file'])
@@ -289,7 +331,7 @@
         builder = self.make_branch_builder('source')
         source = fixtures.build_branch_with_non_ancestral_rev(builder)
         source.tags.set_tag('tag-a', 'rev-2')
-        source.get_config().set_user_option('branch.fetch_tags', 'True')
+        source.get_config_stack().set('branch.fetch_tags', True)
         # Now source has a tag not in its ancestry.  Make a branch from it.
         self.run_bzr('branch source new-branch')
         new_branch = branch.Branch.open('new-branch')
@@ -334,7 +376,7 @@
         # mainline.
         out, err = self.run_bzr(['branch', 'branch', 'newbranch'])
         self.assertEqual('', out)
-        self.assertEqual('Branched 2 revision(s).\n',
+        self.assertEqual('Branched 2 revisions.\n',
             err)
         # it should have preserved the branch format, and so it should be
         # capable of supporting stacking, but not actually have a stacked_on
@@ -441,7 +483,10 @@
         # being too low. If rpc_count increases, more network roundtrips have
         # become necessary for this use case. Please do not adjust this number
         # upwards without agreement from bzr's network support maintainers.
-        self.assertLength(37, self.hpss_calls)
+        self.assertLength(2, self.hpss_connections)
+        self.assertLength(33, self.hpss_calls)
+        self.expectFailure("branching to the same branch requires VFS access",
+            self.assertThat, self.hpss_calls, ContainsNoVfsCalls)
 
     def test_branch_from_trivial_branch_streaming_acceptance(self):
         self.setup_smart_server_with_call_log()
@@ -456,7 +501,9 @@
         # being too low. If rpc_count increases, more network roundtrips have
         # become necessary for this use case. Please do not adjust this number
         # upwards without agreement from bzr's network support maintainers.
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)
         self.assertLength(10, self.hpss_calls)
+        self.assertLength(1, self.hpss_connections)
 
     def test_branch_from_trivial_stacked_branch_streaming_acceptance(self):
         self.setup_smart_server_with_call_log()
@@ -477,12 +524,14 @@
         # become necessary for this use case. Please do not adjust this number
         # upwards without agreement from bzr's network support maintainers.
         self.assertLength(15, self.hpss_calls)
+        self.assertLength(1, self.hpss_connections)
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)
 
     def test_branch_from_branch_with_tags(self):
         self.setup_smart_server_with_call_log()
         builder = self.make_branch_builder('source')
         source = fixtures.build_branch_with_non_ancestral_rev(builder)
-        source.get_config().set_user_option('branch.fetch_tags', 'True')
+        source.get_config_stack().set('branch.fetch_tags', True)
         source.tags.set_tag('tag-a', 'rev-2')
         source.tags.set_tag('tag-missing', 'missing-rev')
         # Now source has a tag not in its ancestry.  Make a branch from it.
@@ -494,6 +543,8 @@
         # become necessary for this use case. Please do not adjust this number
         # upwards without agreement from bzr's network support maintainers.
         self.assertLength(10, self.hpss_calls)
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)
+        self.assertLength(1, self.hpss_connections)
 
     def test_branch_to_stacked_from_trivial_branch_streaming_acceptance(self):
         self.setup_smart_server_with_call_log()
@@ -512,7 +563,10 @@
         readvs_of_rix_files = [
             c for c in self.hpss_calls
             if c.call.method == 'readv' and c.call.args[-1].endswith('.rix')]
+        self.assertLength(1, self.hpss_connections)
         self.assertLength(0, readvs_of_rix_files)
+        self.expectFailure("branching to stacked requires VFS access",
+            self.assertThat, self.hpss_calls, ContainsNoVfsCalls)
 
 
 class TestRemoteBranch(TestCaseWithSFTPServer):
@@ -561,7 +615,7 @@
                 $ bzr checkout %(option)s repo/trunk checkout
                 $ cd checkout
                 $ bzr branch --switch ../repo/trunk ../repo/branched
-                2>Branched 0 revision(s).
+                2>Branched 0 revisions.
                 2>Tree is up to date at revision 0.
                 2>Switched to branch:...branched...
                 $ cd ..

=== modified file 'bzrlib/tests/blackbox/test_branches.py'
--- bzrlib/tests/blackbox/test_branches.py	2011-08-15 15:48:12 +0000
+++ bzrlib/tests/blackbox/test_branches.py	2011-12-21 14:28:03 +0000
@@ -17,6 +17,7 @@
 
 """Black-box tests for bzr branches."""
 
+from bzrlib.branch import BranchReferenceFormat
 from bzrlib.tests import TestCaseWithTransport
 
 
@@ -27,7 +28,7 @@
         # support.
         self.run_bzr('init a')
         out, err = self.run_bzr('branches a')
-        self.assertEquals(out, " (default)\n")
+        self.assertEquals(out, "* (default)\n")
 
     def test_no_branch(self):
         # Listing the branches in a control directory without branches.
@@ -41,3 +42,38 @@
         self.run_bzr('init-repo a')
         out, err = self.run_bzr('branches', working_dir='a')
         self.assertEquals(out, "")
+
+    def test_recursive_current(self):
+        self.run_bzr('init .')
+        self.assertEquals(".\n", self.run_bzr('branches --recursive')[0])
+
+    def test_recursive(self):
+        self.run_bzr('init source')
+        self.run_bzr('init source/subsource')
+        self.run_bzr('checkout --lightweight source checkout')
+        self.run_bzr('init checkout/subcheckout')
+        self.run_bzr('init checkout/.bzr/subcheckout')
+        out = self.run_bzr('branches --recursive')[0]
+        lines = out.split('\n')
+        self.assertIs(True, 'source' in lines, lines)
+        self.assertIs(True, 'source/subsource' in lines, lines)
+        self.assertIs(True, 'checkout/subcheckout' in lines, lines)
+        self.assertIs(True, 'checkout' not in lines, lines)
+
+    def test_indicates_non_branch(self):
+        t = self.make_branch_and_tree('a', format='development-colo')
+        t.bzrdir.create_branch(name='another')
+        t.bzrdir.create_branch(name='colocated')
+        out, err = self.run_bzr('branches a')
+        self.assertEquals(out, "* (default)\n"
+                               "  another\n"
+                               "  colocated\n")
+
+    def test_indicates_branch(self):
+        t = self.make_repository('a', format='development-colo')
+        t.bzrdir.create_branch(name='another')
+        branch = t.bzrdir.create_branch(name='colocated')
+        BranchReferenceFormat().initialize(t.bzrdir, target_branch=branch)
+        out, err = self.run_bzr('branches a')
+        self.assertEquals(out, "  another\n"
+                               "* colocated\n")

=== modified file 'bzrlib/tests/blackbox/test_break_lock.py'
--- bzrlib/tests/blackbox/test_break_lock.py	2011-03-10 13:29:54 +0000
+++ bzrlib/tests/blackbox/test_break_lock.py	2011-12-14 20:21:52 +0000
@@ -24,6 +24,7 @@
     osutils,
     tests,
     )
+from bzrlib.tests.matchers import ContainsNoVfsCalls
 from bzrlib.tests.script import (
     run_script,
     )
@@ -124,3 +125,20 @@
                      stdin="y\n")
         self.assertRaises(errors.LockBroken, self.config.unlock)
 
+
+class TestSmartServerBreakLock(tests.TestCaseWithTransport):
+
+    def test_simple_branch_break_lock(self):
+        self.setup_smart_server_with_call_log()
+        t = self.make_branch_and_tree('branch')
+        t.branch.lock_write()
+        self.reset_smart_call_log()
+        out, err = self.run_bzr(['break-lock', '--force', self.get_url('branch')])
+        # This figure represent the amount of work to perform this use case. It
+        # is entirely ok to reduce this number if a test fails due to rpc_count
+        # being too low. If rpc_count increases, more network roundtrips have
+        # become necessary for this use case. Please do not adjust this number
+        # upwards without agreement from bzr's network support maintainers.
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)
+        self.assertLength(1, self.hpss_connections)
+        self.assertLength(5, self.hpss_calls)

=== modified file 'bzrlib/tests/blackbox/test_cat.py'
--- bzrlib/tests/blackbox/test_cat.py	2011-06-30 17:42:04 +0000
+++ bzrlib/tests/blackbox/test_cat.py	2011-12-14 21:57:37 +0000
@@ -21,6 +21,7 @@
 import os
 
 from bzrlib import tests
+from bzrlib.tests.matchers import ContainsNoVfsCalls
 from bzrlib.transport import memory
 
 
@@ -221,3 +222,23 @@
         url = self.get_readonly_url() + '/a'
         out, err = self.run_bzr(['cat', '-d', url, 'README'])
         self.assertEqual('contents of a/README\n', out)
+
+
+class TestSmartServerCat(tests.TestCaseWithTransport):
+
+    def test_simple_branch_cat(self):
+        self.setup_smart_server_with_call_log()
+        t = self.make_branch_and_tree('branch')
+        self.build_tree_contents([('branch/foo', 'thecontents')])
+        t.add("foo")
+        t.commit("message")
+        self.reset_smart_call_log()
+        out, err = self.run_bzr(['cat', "%s/foo" % self.get_url('branch')])
+        # This figure represent the amount of work to perform this use case. It
+        # is entirely ok to reduce this number if a test fails due to rpc_count
+        # being too low. If rpc_count increases, more network roundtrips have
+        # become necessary for this use case. Please do not adjust this number
+        # upwards without agreement from bzr's network support maintainers.
+        self.assertLength(9, self.hpss_calls)
+        self.assertLength(1, self.hpss_connections)
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)

=== modified file 'bzrlib/tests/blackbox/test_checkout.py'
--- bzrlib/tests/blackbox/test_checkout.py	2011-06-14 01:26:41 +0000
+++ bzrlib/tests/blackbox/test_checkout.py	2011-12-14 21:57:37 +0000
@@ -16,11 +16,7 @@
 
 """Tests for the 'checkout' CLI command."""
 
-from cStringIO import StringIO
 import os
-import re
-import shutil
-import sys
 
 from bzrlib import (
     branch as _mod_branch,
@@ -31,9 +27,7 @@
 from bzrlib.tests import (
     TestCaseWithTransport,
     )
-from bzrlib.tests import (
-    KnownFailure,
-    )
+from bzrlib.tests.matchers import ContainsNoVfsCalls
 from bzrlib.tests.features import (
     HardlinkFeature,
     )
@@ -175,3 +169,38 @@
         second_stat = os.stat('second/file1')
         target_stat = os.stat('target/file1')
         self.assertEqual(second_stat, target_stat)
+
+
+class TestSmartServerCheckout(TestCaseWithTransport):
+
+    def test_heavyweight_checkout(self):
+        self.setup_smart_server_with_call_log()
+        t = self.make_branch_and_tree('from')
+        for count in range(9):
+            t.commit(message='commit %d' % count)
+        self.reset_smart_call_log()
+        out, err = self.run_bzr(['checkout', self.get_url('from'), 'target'])
+        # This figure represent the amount of work to perform this use case. It
+        # is entirely ok to reduce this number if a test fails due to rpc_count
+        # being too low. If rpc_count increases, more network roundtrips have
+        # become necessary for this use case. Please do not adjust this number
+        # upwards without agreement from bzr's network support maintainers.
+        self.assertLength(10, self.hpss_calls)
+        self.assertLength(1, self.hpss_connections)
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)
+
+    def test_lightweight_checkout(self):
+        self.setup_smart_server_with_call_log()
+        t = self.make_branch_and_tree('from')
+        for count in range(9):
+            t.commit(message='commit %d' % count)
+        self.reset_smart_call_log()
+        out, err = self.run_bzr(['checkout', '--lightweight', self.get_url('from'),
+            'target'])
+        # This figure represent the amount of work to perform this use case. It
+        # is entirely ok to reduce this number if a test fails due to rpc_count
+        # being too low. If rpc_count increases, more network roundtrips have
+        # become necessary for this use case. Please do not adjust this number
+        # upwards without agreement from bzr's network support maintainers.
+        self.assertLength(15, self.hpss_calls)
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)

=== modified file 'bzrlib/tests/blackbox/test_clean_tree.py'
--- bzrlib/tests/blackbox/test_clean_tree.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/blackbox/test_clean_tree.py	2011-10-21 23:05:41 +0000
@@ -23,6 +23,7 @@
 
 from bzrlib import ignores
 from bzrlib.tests import TestCaseWithTransport
+from bzrlib.tests.script import run_script
 
 
 class TestBzrTools(TestCaseWithTransport):
@@ -88,3 +89,30 @@
         self.assertPathDoesNotExist('a/unknown')
         self.assertPathDoesNotExist('a/ignored')
         self.assertPathExists('a/added')
+
+    def test_clean_tree_interactive(self):
+        wt = self.make_branch_and_tree('.')
+        self.touch('bar')
+        self.touch('foo')
+        run_script(self, """
+        $ bzr clean-tree
+        bar
+        foo
+        2>Are you sure you wish to delete these? ([y]es, [n]o): no
+        <n
+        Canceled
+        """)
+        self.assertPathExists('bar')
+        self.assertPathExists('foo')
+        run_script(self, """
+        $ bzr clean-tree
+        bar
+        foo
+        2>Are you sure you wish to delete these? ([y]es, [n]o): yes
+        <y
+        2>deleting paths:
+        2>  bar
+        2>  foo
+        """)
+        self.assertPathDoesNotExist('bar')
+        self.assertPathDoesNotExist('foo')

=== modified file 'bzrlib/tests/blackbox/test_commit.py'
--- bzrlib/tests/blackbox/test_commit.py	2011-08-17 10:56:42 +0000
+++ bzrlib/tests/blackbox/test_commit.py	2011-12-30 15:26:18 +0000
@@ -36,6 +36,7 @@
     features,
     )
 from bzrlib.tests import TestCaseWithTransport
+from bzrlib.tests.matchers import ContainsNoVfsCalls
 
 
 class TestCommit(TestCaseWithTransport):
@@ -167,6 +168,26 @@
         finally:
             osutils.get_terminal_encoding = default_get_terminal_enc
 
+    def test_non_ascii_file_unversioned_utf8(self):
+        self.requireFeature(features.UnicodeFilenameFeature)
+        tree = self.make_branch_and_tree(".")
+        self.build_tree(["f"])
+        tree.add(["f"])
+        out, err = self.run_bzr(["commit", "-m", "Wrong filename", u"\xa7"],
+            encoding="utf-8", retcode=3)
+        self.assertContainsRe(err, "(?m)not versioned: \"\xc2\xa7\"$")
+
+    def test_non_ascii_file_unversioned_iso_8859_5(self):
+        self.requireFeature(features.UnicodeFilenameFeature)
+        tree = self.make_branch_and_tree(".")
+        self.build_tree(["f"])
+        tree.add(["f"])
+        out, err = self.run_bzr(["commit", "-m", "Wrong filename", u"\xa7"],
+            encoding="iso-8859-5", retcode=3)
+        self.expectFailure("Error messages are always written as UTF-8",
+            self.assertNotContainsString, err, "\xc2\xa7")
+        self.assertContainsRe(err, "(?m)not versioned: \"\xfd\"$")
+
     def test_warn_about_forgotten_commit_message(self):
         """Test that the lack of -m parameter is caught"""
         wt = self.make_branch_and_tree('.')
@@ -596,6 +617,25 @@
             'commit -m add-b --fixes=xxx:123',
             working_dir='tree')
 
+    def test_fixes_bug_with_default_tracker(self):
+        """commit --fixes=234 uses the default bug tracker."""
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/hello.txt'])
+        tree.add('hello.txt')
+        self.run_bzr_error(
+            ["bzr: ERROR: No tracker specified for bug 123. Use the form "
+            "'tracker:id' or specify a default bug tracker using the "
+            "`bugtracker` option.\n"
+            "See \"bzr help bugs\" for more information on this feature. "
+            "Commit refused."],
+            'commit -m add-b --fixes=123',
+            working_dir='tree')
+        tree.branch.get_config().set_user_option("bugtracker", "lp")
+        self.run_bzr('commit -m hello --fixes=234 tree/hello.txt')
+        last_rev = tree.branch.repository.get_revision(tree.last_revision())
+        self.assertEqual('https://launchpad.net/bugs/234 fixed',
+                         last_rev.properties['bugs'])
+
     def test_fixes_invalid_bug_number(self):
         tree = self.make_branch_and_tree('tree')
         self.build_tree(['tree/hello.txt'])
@@ -613,10 +653,10 @@
         self.build_tree(['tree/hello.txt'])
         tree.add('hello.txt')
         self.run_bzr_error(
-            [r"Invalid bug orange. Must be in the form of 'tracker:id'\. "
-             r"See \"bzr help bugs\" for more information on this feature.\n"
-             r"Commit refused\."],
-            'commit -m add-b --fixes=orange',
+            [r"Invalid bug orange:apples:bananas. Must be in the form of "
+             r"'tracker:id'\. See \"bzr help bugs\" for more information on "
+             r"this feature.\nCommit refused\."],
+            'commit -m add-b --fixes=orange:apples:bananas',
             working_dir='tree')
 
     def test_no_author(self):
@@ -685,6 +725,18 @@
         self.assertStartsWith(
             err, "bzr: ERROR: Could not parse --commit-time:")
 
+    def test_commit_time_missing_tz(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/hello.txt'])
+        tree.add('hello.txt')
+        out, err = self.run_bzr("commit -m hello "
+            "--commit-time='2009-10-10 08:00:00' tree/hello.txt", retcode=3)
+        self.assertStartsWith(
+            err, "bzr: ERROR: Could not parse --commit-time:")
+        # Test that it is actually checking and does not simply crash with
+        # some other exception
+        self.assertContainsString(err, "missing a timezone offset")
+
     def test_partial_commit_with_renames_in_tree(self):
         # this test illustrates bug #140419
         t = self.make_branch_and_tree('.')
@@ -802,3 +854,42 @@
         self.assertEqual(out, '')
         self.assertContainsRe(err,
             'Branch.*test_checkout.*appears to be bound to itself')
+
+    def test_mv_dirs_non_ascii(self):
+        """Move directory with non-ascii name and containing files.
+
+        Regression test for bug 185211.
+        """
+        tree = self.make_branch_and_tree('.')
+        self.build_tree([u'abc\xc3/', u'abc\xc3/foo'])
+
+        tree.add([u'abc\xc3/', u'abc\xc3/foo'])
+        tree.commit('checkin')
+
+        tree.rename_one(u'abc\xc3','abc')
+
+        self.run_bzr('ci -m "non-ascii mv"')
+
+
+class TestSmartServerCommit(TestCaseWithTransport):
+
+    def test_commit_to_lightweight(self):
+        self.setup_smart_server_with_call_log()
+        t = self.make_branch_and_tree('from')
+        for count in range(9):
+            t.commit(message='commit %d' % count)
+        out, err = self.run_bzr(['checkout', '--lightweight', self.get_url('from'),
+            'target'])
+        self.reset_smart_call_log()
+        self.build_tree(['target/afile'])
+        self.run_bzr(['add', 'target/afile'])
+        out, err = self.run_bzr(['commit', '-m', 'do something', 'target'])
+        # This figure represent the amount of work to perform this use case. It
+        # is entirely ok to reduce this number if a test fails due to rpc_count
+        # being too low. If rpc_count increases, more network roundtrips have
+        # become necessary for this use case. Please do not adjust this number
+        # upwards without agreement from bzr's network support maintainers.
+        self.assertLength(214, self.hpss_calls)
+        self.assertLength(2, self.hpss_connections)
+        self.expectFailure("commit still uses VFS calls",
+            self.assertThat, self.hpss_calls, ContainsNoVfsCalls)

=== modified file 'bzrlib/tests/blackbox/test_config.py'
--- bzrlib/tests/blackbox/test_config.py	2011-02-22 12:59:14 +0000
+++ bzrlib/tests/blackbox/test_config.py	2012-01-03 12:56:06 +0000
@@ -17,17 +17,16 @@
 
 """Black-box tests for bzr config."""
 
-import os
-
 from bzrlib import (
     config,
-    errors,
     tests,
     )
 from bzrlib.tests import (
     script,
     test_config as _t_config,
     )
+from bzrlib.tests.matchers import ContainsNoVfsCalls
+
 
 class TestWithoutConfig(tests.TestCaseWithTransport):
 
@@ -80,6 +79,7 @@
         script.run_script(self, '''\
             $ bzr config -d tree
             bazaar:
+              [DEFAULT]
               multiline = """1
             2
             """
@@ -96,14 +96,19 @@
             ''')
 
     def test_list_all_values(self):
+        config.option_registry.register(config.ListOption('list'))
+        self.addCleanup(config.option_registry.remove, 'list')
         self.bazaar_config.set_user_option('list', [1, 'a', 'with, a comma'])
         script.run_script(self, '''\
             $ bzr config -d tree
             bazaar:
+              [DEFAULT]
               list = 1, a, "with, a comma"
             ''')
 
     def test_list_value_only(self):
+        config.option_registry.register(config.ListOption('list'))
+        self.addCleanup(config.option_registry.remove, 'list')
         self.bazaar_config.set_user_option('list', [1, 'a', 'with, a comma'])
         script.run_script(self, '''\
             $ bzr config -d tree list
@@ -115,6 +120,7 @@
         script.run_script(self, '''\
             $ bzr config -d tree
             bazaar:
+              [DEFAULT]
               hello = world
             ''')
 
@@ -136,8 +142,21 @@
         script.run_script(self, '''\
             $ bzr config
             bazaar:
-              hello = world
-            ''')
+              [DEFAULT]
+              hello = world
+            ''')
+
+    def test_cmd_line(self):
+        self.bazaar_config.set_user_option('hello', 'world')
+        script.run_script(self, '''\
+            $ bzr config -Ohello=bzr
+            cmdline:
+              hello = bzr
+            bazaar:
+              [DEFAULT]
+              hello = world
+            ''')
+
 
 class TestConfigDisplayWithPolicy(tests.TestCaseWithTransport):
 
@@ -189,7 +208,7 @@
         # We need to delete the locations definition that overrides the branch
         # one
         script.run_script(self, '''\
-            $ bzr config -d tree --remove file
+            $ bzr config -d tree --scope locations --remove file
             $ bzr config -d tree file
             branch
             ''')
@@ -210,6 +229,7 @@
             $ bzr config --scope bazaar hello=world
             $ bzr config -d tree --all hello
             bazaar:
+              [DEFAULT]
               hello = world
             ''')
 
@@ -218,6 +238,7 @@
             $ bzr config -d tree --scope bazaar hello=world
             $ bzr config -d tree --all hello
             bazaar:
+              [DEFAULT]
               hello = world
             ''')
 
@@ -286,22 +307,25 @@
             branch:
               file = branch
             bazaar:
+              [DEFAULT]
               file = bazaar
             ''')
 
     def test_branch_config_default(self):
         script.run_script(self, '''\
-            $ bzr config -d tree --remove file
+            $ bzr config -d tree --scope locations --remove file
             $ bzr config -d tree --all file
             branch:
               file = branch
             bazaar:
+              [DEFAULT]
               file = bazaar
             ''')
         script.run_script(self, '''\
             $ bzr config -d tree --remove file
             $ bzr config -d tree --all file
             bazaar:
+              [DEFAULT]
               file = bazaar
             ''')
 
@@ -313,11 +337,30 @@
               [.../work/tree]
               file = locations
             bazaar:
+              [DEFAULT]
               file = bazaar
             ''')
         script.run_script(self, '''\
-            $ bzr config -d tree --remove file
+            $ bzr config -d tree --scope locations --remove file
             $ bzr config -d tree --all file
             bazaar:
+              [DEFAULT]
               file = bazaar
             ''')
+
+
+class TestSmartServerConfig(tests.TestCaseWithTransport):
+
+    def test_simple_branch_config(self):
+        self.setup_smart_server_with_call_log()
+        t = self.make_branch_and_tree('branch')
+        self.reset_smart_call_log()
+        out, err = self.run_bzr(['config', '-d', self.get_url('branch')])
+        # This figure represent the amount of work to perform this use case. It
+        # is entirely ok to reduce this number if a test fails due to rpc_count
+        # being too low. If rpc_count increases, more network roundtrips have
+        # become necessary for this use case. Please do not adjust this number
+        # upwards without agreement from bzr's network support maintainers.
+        self.assertLength(5, self.hpss_calls)
+        self.assertLength(1, self.hpss_connections)
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)

=== modified file 'bzrlib/tests/blackbox/test_dpush.py'
--- bzrlib/tests/blackbox/test_dpush.py	2011-04-16 08:41:33 +0000
+++ bzrlib/tests/blackbox/test_dpush.py	2011-09-22 12:13:12 +0000
@@ -146,8 +146,8 @@
     def set_config_push_strict(self, value):
         # set config var (any of bazaar.conf, locations.conf, branch.conf
         # should do)
-        conf = self.tree.branch.get_config()
-        conf.set_user_option('dpush_strict', value)
+        conf = self.tree.branch.get_config_stack()
+        conf.set('dpush_strict', value)
 
     _default_command = ['dpush', '../to']
 

=== modified file 'bzrlib/tests/blackbox/test_exceptions.py'
--- bzrlib/tests/blackbox/test_exceptions.py	2011-06-17 13:53:59 +0000
+++ bzrlib/tests/blackbox/test_exceptions.py	2011-12-09 17:09:06 +0000
@@ -60,6 +60,14 @@
             flags=re.MULTILINE)
         self.assertEquals(out, "")
 
+    def test_utf8_default_fs_enc(self):
+        """In the C locale bzr treats a posix filesystem as UTF-8 encoded"""
+        if os.name != "posix":
+            raise tests.TestNotApplicable("Needs system beholden to C locales")
+        out, err = self.run_bzr_subprocess(["init", "file:%C2%A7"],
+            env_changes={"LANG": "C", "LC_ALL": "C"})
+        self.assertContainsRe(out, "^Created a standalone tree .*$")
+
 
 class TestOptParseBugHandling(tests.TestCase):
     "Test that we handle http://bugs.python.org/issue2931"

=== modified file 'bzrlib/tests/blackbox/test_export.py'
--- bzrlib/tests/blackbox/test_export.py	2011-07-08 23:01:39 +0000
+++ bzrlib/tests/blackbox/test_export.py	2011-12-14 21:57:37 +0000
@@ -21,7 +21,6 @@
 from StringIO import StringIO
 import os
 import stat
-import sys
 import tarfile
 import time
 import zipfile
@@ -29,13 +28,12 @@
 
 from bzrlib import (
     export,
-    osutils,
-    tests,
     )
 from bzrlib.tests import (
     features,
     TestCaseWithTransport,
     )
+from bzrlib.tests.matchers import ContainsNoVfsCalls
 
 
 class TestExport(TestCaseWithTransport):
@@ -194,8 +192,6 @@
     def test_tbz2_export(self):
         self.run_tar_export_disk_and_stdout('tbz2', 'bz2')
 
-    # TODO: test_xz_export, I don't have pylzma working here to test it.
-
     def test_zip_export_unicode(self):
         self.requireFeature(features.UnicodeFilenameFeature)
         tree = self.make_branch_and_tree('zip')
@@ -407,6 +403,23 @@
         self.assertEqual(['goodbye', 'hello'], sorted(os.listdir('latest')))
         self.check_file_contents('latest/goodbye', 'baz')
 
+    def test_export_uncommitted(self):
+        """Test --uncommitted option"""
+        self.example_branch()
+        os.chdir('branch')
+        self.build_tree_contents([('goodbye', 'uncommitted data')])
+        self.run_bzr(['export', '--uncommitted', 'latest'])
+        self.check_file_contents('latest/goodbye', 'uncommitted data')
+
+    def test_export_uncommitted_no_tree(self):
+        """Test --uncommitted option only works with a working tree."""
+        tree = self.example_branch()
+        tree.bzrdir.destroy_workingtree()
+        os.chdir('branch')
+        self.run_bzr_error(
+            ['bzr: ERROR: --uncommitted requires a working tree'],
+            'export --uncommitted latest')
+
     def test_zip_export_per_file_timestamps(self):
         tree = self.example_branch()
         self.build_tree_contents([('branch/har', 'foo')])
@@ -418,3 +431,23 @@
         zfile = zipfile.ZipFile('test.zip')
         info = zfile.getinfo("test/har")
         self.assertEquals(time.localtime(timestamp)[:6], info.date_time)
+
+
+class TestSmartServerExport(TestCaseWithTransport):
+
+    def test_simple_export(self):
+        self.setup_smart_server_with_call_log()
+        t = self.make_branch_and_tree('branch')
+        self.build_tree_contents([('branch/foo', 'thecontents')])
+        t.add("foo")
+        t.commit("message")
+        self.reset_smart_call_log()
+        out, err = self.run_bzr(['export', "foo.tar.gz", self.get_url('branch')])
+        # This figure represent the amount of work to perform this use case. It
+        # is entirely ok to reduce this number if a test fails due to rpc_count
+        # being too low. If rpc_count increases, more network roundtrips have
+        # become necessary for this use case. Please do not adjust this number
+        # upwards without agreement from bzr's network support maintainers.
+        self.assertLength(7, self.hpss_calls)
+        self.assertLength(1, self.hpss_connections)
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)

=== added file 'bzrlib/tests/blackbox/test_export_pot.py'
--- bzrlib/tests/blackbox/test_export_pot.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_export_pot.py	2011-09-30 10:44:29 +0000
@@ -0,0 +1,37 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+
+"""External tests of 'bzr export-pot'"""
+import os
+
+from bzrlib import ignores, osutils
+from bzrlib.tests import TestCaseWithMemoryTransport
+from bzrlib.tests.features import ModuleAvailableFeature
+
+class TestExportPot(TestCaseWithMemoryTransport):
+
+    def test_export_pot(self):
+        out, err = self.run_bzr("export-pot")
+        self.assertContainsRe(err, 'Exporting messages from builtin command: add')
+        self.assertContainsRe(out, "help of 'change' option\n"\
+                                   "msgid \"Select changes introduced by the specified revision.")
+
+    def test_export_pot_plugin(self):
+        self.requireFeature(ModuleAvailableFeature('bzrlib.plugins.launchpad'))
+        out, err = self.run_bzr("export-pot --plugin=launchpad")
+        self.assertContainsRe(err, 'Exporting messages from plugin command: launchpad-login in launchpad')
+        self.assertContainsRe(out, 'msgid "Show or set the Launchpad user ID."')

=== modified file 'bzrlib/tests/blackbox/test_hooks.py'
--- bzrlib/tests/blackbox/test_hooks.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/blackbox/test_hooks.py	2011-11-28 18:57:59 +0000
@@ -19,6 +19,8 @@
 from bzrlib.branch import Branch
 from bzrlib.tests import TestCaseWithTransport
 
+def _foo_hook():
+    pass
 
 class TestHooks(TestCaseWithTransport):
 
@@ -51,3 +53,23 @@
 
     def test_hooks_no_branch(self):
         self.run_bzr('hooks')
+
+    def test_hooks_lazy_with_unnamed_hook(self):
+        self.make_branch('.')
+        def foo(): return
+        Branch.hooks.install_named_hook_lazy('set_rh',
+            'bzrlib.tests.blackbox.test_hooks',
+            '_foo_hook',
+            None)
+        out, err = self.run_bzr('hooks')
+        self._check_hooks_output(out, {'set_rh': ["No hook name"]})
+        
+    def test_hooks_lazy_with_named_hook(self):
+        self.make_branch('.')
+        def foo(): return
+        Branch.hooks.install_named_hook_lazy('set_rh',
+            'bzrlib.tests.blackbox.test_hooks',
+            '_foo_hook',
+            'hook has a name')
+        out, err = self.run_bzr('hooks')
+        self._check_hooks_output(out, {'set_rh': ["hook has a name"]})

=== modified file 'bzrlib/tests/blackbox/test_info.py'
--- bzrlib/tests/blackbox/test_info.py	2011-03-22 17:16:58 +0000
+++ bzrlib/tests/blackbox/test_info.py	2011-12-14 20:21:52 +0000
@@ -17,11 +17,13 @@
 
 """Tests for the info command of bzr."""
 
+import shutil
 import sys
 
 from bzrlib import (
     branch,
     bzrdir,
+    controldir,
     errors,
     info,
     osutils,
@@ -29,6 +31,7 @@
     upgrade,
     urlutils,
     )
+from bzrlib.tests.matchers import ContainsNoVfsCalls
 from bzrlib.transport import memory
 
 
@@ -45,6 +48,27 @@
         self.assertEqual(out, '')
         self.assertEqual(err, 'bzr: ERROR: Not a branch: "%s".\n' % location)
 
+    def test_info_empty_controldir(self):
+        self.make_bzrdir('ctrl')
+        out, err = self.run_bzr('info ctrl')
+        self.assertEquals(out,
+            'Empty control directory (format: 2a or pack-0.92)\n'
+            'Location:\n'
+            '  control directory: ctrl\n')
+        self.assertEquals(err, '')
+
+    def test_info_dangling_branch_reference(self):
+        br = self.make_branch('target')
+        br.create_checkout('from', lightweight=True)
+        shutil.rmtree('target')
+        out, err = self.run_bzr('info from')
+        self.assertEquals(out,
+            'Dangling branch reference (format: 2a or pack-0.92)\n'
+            'Location:\n'
+            '   control directory: from\n'
+            '  checkout of branch: target\n')
+        self.assertEquals(err, '')
+
     def test_info_standalone(self):
         transport = self.get_transport()
 
@@ -125,7 +149,7 @@
 """, out)
         self.assertEqual('', err)
         tree1.commit('commit one')
-        rev = branch1.repository.get_revision(branch1.revision_history()[0])
+        rev = branch1.repository.get_revision(branch1.last_revision())
         datestring_first = osutils.format_date(rev.timestamp, rev.timezone)
 
         # Branch standalone with push location
@@ -187,7 +211,7 @@
         branch1.bzrdir.sprout('bound')
         knit1_format = bzrdir.format_registry.make_bzrdir('knit')
         upgrade.upgrade('bound', knit1_format)
-        branch3 = bzrdir.BzrDir.open('bound').open_branch()
+        branch3 = controldir.ControlDir.open('bound').open_branch()
         branch3.bind(branch1)
         bound_tree = branch3.bzrdir.open_workingtree()
         out, err = self.run_bzr('info -v bound')
@@ -232,7 +256,7 @@
         self.assertEqual('', err)
 
         # Checkout standalone (same as above, but does not have parent set)
-        branch4 = bzrdir.BzrDir.create_branch_convenience('checkout',
+        branch4 = controldir.ControlDir.create_branch_convenience('checkout',
             format=knit1_format)
         branch4.bind(branch1)
         branch4.bzrdir.open_workingtree().update()
@@ -317,7 +341,7 @@
         self.build_tree(['standalone/b'])
         tree1.add('b')
         tree1.commit('commit two')
-        rev = branch1.repository.get_revision(branch1.revision_history()[-1])
+        rev = branch1.repository.get_revision(branch1.last_revision())
         datestring_last = osutils.format_date(rev.timestamp, rev.timezone)
 
         # Out of date branched standalone branch will not be detected
@@ -529,8 +553,8 @@
 
         # Create branch inside shared repository
         repo.bzrdir.root_transport.mkdir('branch')
-        branch1 = repo.bzrdir.create_branch_convenience('repo/branch',
-            format=format)
+        branch1 = controldir.ControlDir.create_branch_convenience(
+            'repo/branch', format=format)
         out, err = self.run_bzr('info -v repo/branch')
         self.assertEqualDiff(
 """Repository branch (format: dirstate or knit)
@@ -571,7 +595,7 @@
         self.build_tree(['tree/lightcheckout/a'])
         tree2.add('a')
         tree2.commit('commit one')
-        rev = repo.get_revision(branch2.revision_history()[0])
+        rev = repo.get_revision(branch2.last_revision())
         datestring_first = osutils.format_date(rev.timestamp, rev.timezone)
         out, err = self.run_bzr('info tree/lightcheckout --verbose')
         self.assertEqualDiff(
@@ -690,7 +714,7 @@
         tree3.commit('commit two')
 
         # Out of date lightweight checkout
-        rev = repo.get_revision(branch1.revision_history()[-1])
+        rev = repo.get_revision(branch1.last_revision())
         datestring_last = osutils.format_date(rev.timestamp, rev.timezone)
         out, err = self.run_bzr('info tree/lightcheckout --verbose')
         self.assertEqualDiff(
@@ -803,7 +827,7 @@
 
         # Create two branches
         repo.bzrdir.root_transport.mkdir('branch1')
-        branch1 = repo.bzrdir.create_branch_convenience('repo/branch1',
+        branch1 = controldir.ControlDir.create_branch_convenience('repo/branch1',
             format=format)
         branch2 = branch1.bzrdir.sprout('repo/branch2').open_branch()
 
@@ -846,7 +870,7 @@
         tree1 = branch1.bzrdir.open_workingtree()
         tree1.add('a')
         tree1.commit('commit one')
-        rev = repo.get_revision(branch1.revision_history()[0])
+        rev = repo.get_revision(branch1.last_revision())
         datestring_first = osutils.format_date(rev.timestamp, rev.timezone)
         out, err = self.run_bzr('info -v repo/branch1')
         self.assertEqualDiff(
@@ -1221,13 +1245,13 @@
                                     format=bzrdir.BzrDirMetaFormat1())
         repo.set_make_working_trees(False)
         repo.bzrdir.root_transport.mkdir('branch')
-        repo_branch = repo.bzrdir.create_branch_convenience('repo/branch',
-                                    format=bzrdir.BzrDirMetaFormat1())
+        repo_branch = controldir.ControlDir.create_branch_convenience(
+            'repo/branch', format=bzrdir.BzrDirMetaFormat1())
         # Do a heavy checkout
         transport.mkdir('tree')
         transport.mkdir('tree/checkout')
-        co_branch = bzrdir.BzrDir.create_branch_convenience('tree/checkout',
-            format=bzrdir.BzrDirMetaFormat1())
+        co_branch = controldir.ControlDir.create_branch_convenience(
+            'tree/checkout', format=bzrdir.BzrDirMetaFormat1())
         co_branch.bind(repo_branch)
         # Do a light checkout of the heavy one
         transport.mkdir('tree/lightcheckout')
@@ -1345,3 +1369,97 @@
      stacked on: mainline
 """, out)
         self.assertEqual("", err)
+
+    def test_info_revinfo_optional(self):
+        tree = self.make_branch_and_tree('.')
+        def last_revision_info(self):
+            raise errors.UnsupportedOperation(last_revision_info, self)
+        self.overrideAttr(
+            branch.Branch, "last_revision_info", last_revision_info)
+        out, err = self.run_bzr('info -v .')
+        self.assertEqual(
+"""Standalone tree (format: 2a)
+Location:
+  branch root: .
+
+Format:
+       control: Meta directory format 1
+  working tree: Working tree format 6
+        branch: Branch format 7
+    repository: Repository format 2a - rich roots, group compression and chk inventories
+
+In the working tree:
+         0 unchanged
+         0 modified
+         0 added
+         0 removed
+         0 renamed
+         0 unknown
+         0 ignored
+         0 versioned subdirectories
+""", out)
+        self.assertEqual("", err)
+
+    def test_info_shows_colocated_branches(self):
+        bzrdir = self.make_branch('.', format='development-colo').bzrdir
+        bzrdir.create_branch(name="colo1")
+        bzrdir.create_branch(name="colo2")
+        bzrdir.create_branch(name="colo3")
+        out, err = self.run_bzr('info -v .')
+        self.assertEqualDiff(
+"""Standalone branch (format: development-colo)
+Location:
+  branch root: .
+
+Format:
+       control: Meta directory format 1 with support for colocated branches
+        branch: Branch format 7
+    repository: Repository format 2a - rich roots, group compression and chk inventories
+
+Control directory:
+         4 branches
+
+Branch history:
+         0 revisions
+
+Repository:
+         0 revisions
+""", out)
+        self.assertEqual("", err)
+
+
+class TestSmartServerInfo(tests.TestCaseWithTransport):
+
+    def test_simple_branch_info(self):
+        self.setup_smart_server_with_call_log()
+        t = self.make_branch_and_tree('branch')
+        self.build_tree_contents([('branch/foo', 'thecontents')])
+        t.add("foo")
+        t.commit("message")
+        self.reset_smart_call_log()
+        out, err = self.run_bzr(['info', self.get_url('branch')])
+        # This figure represent the amount of work to perform this use case. It
+        # is entirely ok to reduce this number if a test fails due to rpc_count
+        # being too low. If rpc_count increases, more network roundtrips have
+        # become necessary for this use case. Please do not adjust this number
+        # upwards without agreement from bzr's network support maintainers.
+        self.assertLength(12, self.hpss_calls)
+        self.assertLength(1, self.hpss_connections)
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)
+
+    def test_verbose_branch_info(self):
+        self.setup_smart_server_with_call_log()
+        t = self.make_branch_and_tree('branch')
+        self.build_tree_contents([('branch/foo', 'thecontents')])
+        t.add("foo")
+        t.commit("message")
+        self.reset_smart_call_log()
+        out, err = self.run_bzr(['info', '-v', self.get_url('branch')])
+        # This figure represent the amount of work to perform this use case. It
+        # is entirely ok to reduce this number if a test fails due to rpc_count
+        # being too low. If rpc_count increases, more network roundtrips have
+        # become necessary for this use case. Please do not adjust this number
+        # upwards without agreement from bzr's network support maintainers.
+        self.assertLength(16, self.hpss_calls)
+        self.assertLength(1, self.hpss_connections)
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)

=== modified file 'bzrlib/tests/blackbox/test_init.py'
--- bzrlib/tests/blackbox/test_init.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/blackbox/test_init.py	2011-12-21 16:26:24 +0000
@@ -48,12 +48,22 @@
         self.assertIsDirectory('.bzr/checkout/lock', t)
 
     def test_init_format_2a(self):
-        """Smoke test for constructing a format 2a repoistory."""
+        """Smoke test for constructing a format 2a repository."""
         out, err = self.run_bzr('init --format=2a')
         self.assertEqual("""Created a standalone tree (format: 2a)\n""",
             out)
         self.assertEqual('', err)
 
+    def test_init_colocated(self):
+        """Smoke test for constructing a colocated branch."""
+        out, err = self.run_bzr('init --format=development-colo file:,branch=abranch')
+        self.assertEqual("""Created a lightweight checkout (format: development-colo)\n""",
+            out)
+        self.assertEqual('', err)
+        out, err = self.run_bzr('branches')
+        self.assertEqual("  abranch\n", out)
+        self.assertEqual('', err)
+
     def test_init_at_repository_root(self):
         # bzr init at the root of a repository should create a branch
         # and working tree even when creation of working trees is disabled.
@@ -203,10 +213,10 @@
     def test_init_append_revisions_only(self):
         self.run_bzr('init --dirstate-tags normal_branch6')
         branch = _mod_branch.Branch.open('normal_branch6')
-        self.assertEqual(None, branch._get_append_revisions_only())
+        self.assertEqual(None, branch.get_append_revisions_only())
         self.run_bzr('init --append-revisions-only --dirstate-tags branch6')
         branch = _mod_branch.Branch.open('branch6')
-        self.assertEqual(True, branch._get_append_revisions_only())
+        self.assertEqual(True, branch.get_append_revisions_only())
         self.run_bzr_error(['cannot be set to append-revisions-only'],
                            'init --append-revisions-only --knit knit')
 

=== modified file 'bzrlib/tests/blackbox/test_log.py'
--- bzrlib/tests/blackbox/test_log.py	2011-08-04 00:17:53 +0000
+++ bzrlib/tests/blackbox/test_log.py	2011-12-21 14:50:43 +0000
@@ -31,6 +31,7 @@
     test_log,
     features,
     )
+from bzrlib.tests.matchers import ContainsNoVfsCalls
 
 
 class TestLog(tests.TestCaseWithTransport, test_log.TestLogMixin):
@@ -89,8 +90,12 @@
     def get_captured_revisions(self):
         return self.log_catcher.revisions
 
-    def assertLogRevnos(self, args, expected_revnos, working_dir='.'):
-        self.run_bzr(['log'] + args, working_dir=working_dir)
+    def assertLogRevnos(self, args, expected_revnos, working_dir='.',
+                        out='', err=''):
+        actual_out, actual_err = self.run_bzr(['log'] + args,
+                                              working_dir=working_dir)
+        self.assertEqual(out, actual_out)
+        self.assertEqual(err, actual_err)
         self.assertEqual(expected_revnos,
                          [r.revno for r in self.get_captured_revisions()])
 
@@ -200,6 +205,10 @@
         # 4  1.1.4
         # | /
         # 5
+        # | \
+        # | 5.1.1
+        # | /
+        # 6
 
         # mainline
         builder.build_snapshot('1', None, [
@@ -218,6 +227,8 @@
         builder.build_snapshot('1.1.4', ['1.1.3', '4'], [])
         # merge branch into mainline
         builder.build_snapshot('5', ['4', '1.1.4'], [])
+        builder.build_snapshot('5.1.1', ['5'], [])
+        builder.build_snapshot('6', ['5', '5.1.1'], [])
         builder.finish_series()
 
     def test_n0(self):
@@ -236,6 +247,11 @@
         self.assertLogRevnos(['-n1', '-r1.1.1..1.1.4', '--forward'],
                              ['1.1.1', '1.1.2', '1.1.3', '1.1.4'])
 
+    def test_fallback_when_end_rev_is_not_on_mainline(self):
+        self.assertLogRevnos(['-n1', '-r1.1.1..5.1.1'],
+                             # We don't get 1.1.1 because we say -n1
+                             ['5.1.1', '5', '4', '3'])
+
 
 class Test_GenerateAllRevisions(TestLogWithLogCatcher):
 
@@ -253,6 +269,19 @@
         # The graph below may look a bit complicated (and it may be but I've
         # banged my head enough on it) but the bug requires at least dotted
         # revnos *and* merged revisions below that.
+        # 1
+        # | \
+        # 2  1.1.1
+        # | X
+        # 3  2.1.1
+        # |   |    \
+        # |  2.1.2  2.2.1
+        # |   |    X
+        # |  2.1.3  \
+        # | /       /
+        # 4        /
+        # |       /
+        # 5 -----/
         builder.build_snapshot('1', None, [
             ('add', ('', 'root-id', 'directory', ''))])
         builder.build_snapshot('2', ['1'], [])
@@ -377,7 +406,7 @@
     def test_log_reversed_dotted_revspecs(self):
         self.make_merged_branch()
         self.run_bzr_error(('bzr: ERROR: Start revision not found in '
-                            'left-hand history of end revision.\n',),
+                            'history of end revision.\n',),
                            "log -r 1.1.1..1")
 
     def test_log_bad_message_re(self):
@@ -536,19 +565,30 @@
 
     def test_include_merges(self):
         # Confirm --include-merges gives the same output as -n0
-        self.assertLogRevnos(['--include-merges'],
-                             ['2', '1.1.2', '1.2.1', '1.1.1', '1'],
-                             working_dir='level0')
-        self.assertLogRevnos(['--include-merges'],
-                             ['2', '1.1.2', '1.2.1', '1.1.1', '1'],
-                             working_dir='level0')
+        msg = ("The option '--include-merges' to 'bzr log' "
+               "has been deprecated in bzr 2.5. "
+               "Please use '--include-merged' instead.\n")
+        self.assertLogRevnos(['--include-merges'],
+                             ['2', '1.1.2', '1.2.1', '1.1.1', '1'],
+                             working_dir='level0', err=msg)
+        self.assertLogRevnos(['--include-merges'],
+                             ['2', '1.1.2', '1.2.1', '1.1.1', '1'],
+                             working_dir='level0', err=msg)
         out_im, err_im = self.run_bzr('log --include-merges',
                                       working_dir='level0')
         out_n0, err_n0 = self.run_bzr('log -n0', working_dir='level0')
-        self.assertEqual('', err_im)
+        self.assertEqual(msg, err_im)
         self.assertEqual('', err_n0)
         self.assertEqual(out_im, out_n0)
 
+    def test_include_merged(self):
+        # Confirm --include-merged gives the same output as -n0
+        expected = ['2', '1.1.2', '1.2.1', '1.1.1', '1']
+        self.assertLogRevnos(['--include-merged'],
+                             expected, working_dir='level0')
+        self.assertLogRevnos(['--include-merged'],
+                             expected, working_dir='level0')
+
     def test_force_merge_revisions_N(self):
         self.assertLogRevnos(['-n2'],
                              ['2', '1.1.2', '1.1.1', '1'],
@@ -572,6 +612,14 @@
                 [('2', 0), ('1.1.2', 1), ('1.2.1', 2)],
                 working_dir='level0')
 
+    def test_omit_merges_with_sidelines(self):
+        self.assertLogRevnos(['--omit-merges', '-n0'], ['1.2.1', '1.1.1', '1'],
+                             working_dir='level0')
+
+    def test_omit_merges_without_sidelines(self):
+        self.assertLogRevnos(['--omit-merges', '-n1'], ['1'],
+                             working_dir='level0')
+
 
 class TestLogDiff(TestLogWithLogCatcher):
 
@@ -1029,3 +1077,57 @@
         self.assertLogRevnos(["--match-author", "author"], ["2", "1"])
         self.assertLogRevnos(["--match-author", "author1", 
                               "--match-author", "author2"], ["2", "1"])
+
+
+class TestSmartServerLog(tests.TestCaseWithTransport):
+
+    def test_standard_log(self):
+        self.setup_smart_server_with_call_log()
+        t = self.make_branch_and_tree('branch')
+        self.build_tree_contents([('branch/foo', 'thecontents')])
+        t.add("foo")
+        t.commit("message")
+        self.reset_smart_call_log()
+        out, err = self.run_bzr(['log', self.get_url('branch')])
+        # This figure represent the amount of work to perform this use case. It
+        # is entirely ok to reduce this number if a test fails due to rpc_count
+        # being too low. If rpc_count increases, more network roundtrips have
+        # become necessary for this use case. Please do not adjust this number
+        # upwards without agreement from bzr's network support maintainers.
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)
+        self.assertLength(1, self.hpss_connections)
+        self.assertLength(10, self.hpss_calls)
+
+    def test_verbose_log(self):
+        self.setup_smart_server_with_call_log()
+        t = self.make_branch_and_tree('branch')
+        self.build_tree_contents([('branch/foo', 'thecontents')])
+        t.add("foo")
+        t.commit("message")
+        self.reset_smart_call_log()
+        out, err = self.run_bzr(['log', '-v', self.get_url('branch')])
+        # This figure represent the amount of work to perform this use case. It
+        # is entirely ok to reduce this number if a test fails due to rpc_count
+        # being too low. If rpc_count increases, more network roundtrips have
+        # become necessary for this use case. Please do not adjust this number
+        # upwards without agreement from bzr's network support maintainers.
+        self.assertLength(11, self.hpss_calls)
+        self.assertLength(1, self.hpss_connections)
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)
+
+    def test_per_file(self):
+        self.setup_smart_server_with_call_log()
+        t = self.make_branch_and_tree('branch')
+        self.build_tree_contents([('branch/foo', 'thecontents')])
+        t.add("foo")
+        t.commit("message")
+        self.reset_smart_call_log()
+        out, err = self.run_bzr(['log', '-v', self.get_url('branch') + "/foo"])
+        # This figure represent the amount of work to perform this use case. It
+        # is entirely ok to reduce this number if a test fails due to rpc_count
+        # being too low. If rpc_count increases, more network roundtrips have
+        # become necessary for this use case. Please do not adjust this number
+        # upwards without agreement from bzr's network support maintainers.
+        self.assertLength(15, self.hpss_calls)
+        self.assertLength(1, self.hpss_connections)
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)

=== modified file 'bzrlib/tests/blackbox/test_lookup_revision.py'
--- bzrlib/tests/blackbox/test_lookup_revision.py	2010-05-02 20:10:25 +0000
+++ bzrlib/tests/blackbox/test_lookup_revision.py	2011-11-22 23:29:09 +0000
@@ -18,8 +18,6 @@
 """Black-box tests for bzr lookup-revision.
 """
 
-import os
-
 from bzrlib import tests
 
 class TestLookupRevision(tests.TestCaseWithTransport):

=== modified file 'bzrlib/tests/blackbox/test_ls.py'
--- bzrlib/tests/blackbox/test_ls.py	2010-05-02 20:10:25 +0000
+++ bzrlib/tests/blackbox/test_ls.py	2011-12-14 21:57:37 +0000
@@ -18,8 +18,9 @@
 
 import os
 
-from bzrlib import ignores, osutils
+from bzrlib import ignores
 from bzrlib.tests import TestCaseWithTransport
+from bzrlib.tests.matchers import ContainsNoVfsCalls
 
 
 class TestLS(TestCaseWithTransport):
@@ -244,3 +245,23 @@
         self.wt.commit('commit')
         self.ls_equals('sub/\nsub/file\n', '--directory=dir')
         self.ls_equals('sub/file\n', '-d dir sub')
+
+
+class TestSmartServerLs(TestCaseWithTransport):
+
+    def test_simple_ls(self):
+        self.setup_smart_server_with_call_log()
+        t = self.make_branch_and_tree('branch')
+        self.build_tree_contents([('branch/foo', 'thecontents')])
+        t.add("foo")
+        t.commit("message")
+        self.reset_smart_call_log()
+        out, err = self.run_bzr(['ls', self.get_url('branch')])
+        # This figure represent the amount of work to perform this use case. It
+        # is entirely ok to reduce this number if a test fails due to rpc_count
+        # being too low. If rpc_count increases, more network roundtrips have
+        # become necessary for this use case. Please do not adjust this number
+        # upwards without agreement from bzr's network support maintainers.
+        self.assertLength(6, self.hpss_calls)
+        self.assertLength(1, self.hpss_connections)
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)

=== modified file 'bzrlib/tests/blackbox/test_merge.py'
--- bzrlib/tests/blackbox/test_merge.py	2011-08-09 13:30:21 +0000
+++ bzrlib/tests/blackbox/test_merge.py	2011-12-27 12:18:36 +0000
@@ -681,7 +681,7 @@
         builder.build_commit(message="Rev 2a", rev_id='rev-2a')
         source.tags.set_tag('tag-a', 'rev-2a')
         source.set_last_revision_info(1, 'rev-1')
-        source.get_config().set_user_option('branch.fetch_tags', 'True')
+        source.get_config_stack().set('branch.fetch_tags', True)
         builder.build_commit(message="Rev 2b", rev_id='rev-2b')
         # Merge from source
         self.run_bzr('merge -d target source')

=== modified file 'bzrlib/tests/blackbox/test_merge_directive.py'
--- bzrlib/tests/blackbox/test_merge_directive.py	2010-06-17 09:23:19 +0000
+++ bzrlib/tests/blackbox/test_merge_directive.py	2011-12-16 19:18:39 +0000
@@ -223,7 +223,7 @@
 
     def test_mail_uses_config(self):
         tree1, tree2 = self.prepare_merge_directive()
-        tree1.branch.get_config().set_user_option('smtp_server', 'bogushost')
+        tree1.branch.get_config_stack().set('smtp_server', 'bogushost')
         md_text, errr, connect_calls, sendmail_calls =\
             self.run_bzr_fakemail('merge-directive --mail-to'
                                   ' pqm@example.com --plain ../tree2 .')

=== modified file 'bzrlib/tests/blackbox/test_missing.py'
--- bzrlib/tests/blackbox/test_missing.py	2011-07-26 06:38:46 +0000
+++ bzrlib/tests/blackbox/test_missing.py	2011-09-16 15:37:58 +0000
@@ -49,7 +49,7 @@
         self.assertEqual('', err)
 
     def test_missing(self):
-        missing = "You are missing 1 revision(s):"
+        missing = "You are missing 1 revision:"
 
         # create a source branch
         a_tree = self.make_branch_and_tree('a')
@@ -92,7 +92,7 @@
 
         # compare again, but now we have the 'merge' commit extra
         lines = self.run_bzr('missing ../b', retcode=1)[0].splitlines()
-        self.assertEqual("You have 1 extra revision(s):", lines[0])
+        self.assertEqual("You have 1 extra revision:", lines[0])
         self.assertEqual(8, len(lines))
         lines2 = self.run_bzr('missing ../b --mine-only', retcode=1)[0]
         lines2 = lines2.splitlines()

=== added file 'bzrlib/tests/blackbox/test_mkdir.py'
--- bzrlib/tests/blackbox/test_mkdir.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_mkdir.py	2011-12-12 13:25:39 +0000
@@ -0,0 +1,59 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+
+"""Black-box tests for bzr mkdir.
+"""
+
+import os
+from bzrlib import tests
+
+
+class TestMkdir(tests.TestCaseWithTransport):
+
+    def test_mkdir(self):
+        tree = self.make_branch_and_tree('.')
+        self.run_bzr(['mkdir', 'somedir'])
+        self.assertEquals(tree.kind(tree.path2id('somedir')), "directory")
+
+    def test_mkdir_multi(self):
+        tree = self.make_branch_and_tree('.')
+        self.run_bzr(['mkdir', 'somedir', 'anotherdir'])
+        self.assertEquals(tree.kind(tree.path2id('somedir')), "directory")
+        self.assertEquals(tree.kind(tree.path2id('anotherdir')), "directory")
+
+    def test_mkdir_parents(self):
+        tree = self.make_branch_and_tree('.')
+        self.run_bzr(['mkdir', '-p', 'somedir/foo'])
+        self.assertEquals(tree.kind(tree.path2id('somedir/foo')), "directory")
+
+    def test_mkdir_parents_existing_versioned_dir(self):
+        tree = self.make_branch_and_tree('.')
+        tree.mkdir('somedir')
+        self.assertEquals(tree.kind(tree.path2id('somedir')), "directory")
+        self.run_bzr(['mkdir', '-p', 'somedir'])
+
+    def test_mkdir_parents_existing_unversioned_dir(self):
+        tree = self.make_branch_and_tree('.')
+        os.mkdir('somedir')
+        self.run_bzr(['mkdir', '-p', 'somedir'])
+        self.assertEquals(tree.kind(tree.path2id('somedir')), "directory")
+
+    def test_mkdir_parents_with_unversioned_parent(self):
+        tree = self.make_branch_and_tree('.')
+        os.mkdir('somedir')
+        self.run_bzr(['mkdir', '-p', 'somedir/foo'])
+        self.assertEquals(tree.kind(tree.path2id('somedir/foo')), "directory")

=== modified file 'bzrlib/tests/blackbox/test_mv.py'
--- bzrlib/tests/blackbox/test_mv.py	2011-07-11 06:47:32 +0000
+++ bzrlib/tests/blackbox/test_mv.py	2011-12-06 12:36:26 +0000
@@ -187,6 +187,12 @@
         self.run_bzr('move a b')
         self.run_bzr('rename b a')
 
+    def test_mv_no_root(self):
+        tree = self.make_branch_and_tree('.')
+        self.run_bzr_error(
+            ["bzr: ERROR: can not move root of branch"],
+            'mv . a')
+
     def test_mv_through_symlinks(self):
         self.requireFeature(SymlinkFeature)
         tree = self.make_branch_and_tree('.')
@@ -515,3 +521,14 @@
         self.build_tree([u"\xA7"])
         out, err = self.run_bzr_error(["Could not rename", "not versioned"],
             ["mv", u"\xA7", "b"])
+
+    def test_mv_removed_non_ascii(self):
+        """Clear error on mv of a removed non-ascii file, see lp:898541"""
+        self.requireFeature(UnicodeFilenameFeature)
+        tree = self.make_branch_and_tree(".")
+        self.build_tree([u"\xA7"])
+        tree.add([u"\xA7"])
+        tree.commit(u"Adding \xA7")
+        os.remove(u"\xA7")
+        out, err = self.run_bzr_error(["Could not rename", "not exist"],
+            ["mv", u"\xA7", "b"])

=== modified file 'bzrlib/tests/blackbox/test_non_ascii.py'
--- bzrlib/tests/blackbox/test_non_ascii.py	2011-08-04 00:17:53 +0000
+++ bzrlib/tests/blackbox/test_non_ascii.py	2011-09-02 00:53:06 +0000
@@ -267,7 +267,7 @@
 
         expected = osutils.pathjoin(osutils.getcwd(), dirname1)
         self.assertEqual(u'Using saved parent location: %s/\n'
-                'No revisions to pull.\n' % (expected,), txt)
+                'No revisions or tags to pull.\n' % (expected,), txt)
 
         self.build_tree_contents(
             [(osutils.pathjoin(dirname1, 'a'), 'and yet more\n')])

=== modified file 'bzrlib/tests/blackbox/test_pack.py'
--- bzrlib/tests/blackbox/test_pack.py	2011-05-13 20:44:45 +0000
+++ bzrlib/tests/blackbox/test_pack.py	2011-12-14 21:57:37 +0000
@@ -19,6 +19,7 @@
 import os
 
 from bzrlib import tests
+from bzrlib.tests.matchers import ContainsNoVfsCalls
 
 
 class TestPack(tests.TestCaseWithTransport):
@@ -83,3 +84,24 @@
 
         pack_names = transport.list_dir('repository/obsolete_packs')
         self.assertTrue(len(pack_names) == 0)
+
+
+class TestSmartServerPack(tests.TestCaseWithTransport):
+
+    def test_simple_pack(self):
+        self.setup_smart_server_with_call_log()
+        t = self.make_branch_and_tree('branch')
+        self.build_tree_contents([('branch/foo', 'thecontents')])
+        t.add("foo")
+        t.commit("message")
+        self.reset_smart_call_log()
+        out, err = self.run_bzr(['pack', self.get_url('branch')])
+        # This figure represent the amount of HPSS calls to perform this use
+        # case. It is entirely ok to reduce this number if a test fails due to
+        # rpc_count # being too low. If rpc_count increases, more network
+        # roundtrips have become necessary for this use case. Please do not
+        # adjust this number upwards without agreement from bzr's network
+        # support maintainers.
+        self.assertLength(6, self.hpss_calls)
+        self.assertLength(1, self.hpss_connections)
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)

=== modified file 'bzrlib/tests/blackbox/test_pull.py'
--- bzrlib/tests/blackbox/test_pull.py	2011-08-30 08:46:10 +0000
+++ bzrlib/tests/blackbox/test_pull.py	2012-01-02 10:24:02 +0000
@@ -23,6 +23,7 @@
 from bzrlib import (
     debug,
     remote,
+    tests,
     urlutils,
     )
 
@@ -31,13 +32,13 @@
 from bzrlib.osutils import pathjoin
 from bzrlib.tests import (
     fixtures,
-    TestCaseWithTransport,
+    script,
     )
 from bzrlib.uncommit import uncommit
 from bzrlib.workingtree import WorkingTree
 
 
-class TestPull(TestCaseWithTransport):
+class TestPull(tests.TestCaseWithTransport):
 
     def example_branch(self, path='.'):
         tree = self.make_branch_and_tree(path)
@@ -53,6 +54,7 @@
     def test_pull(self):
         """Pull changes from one branch to another."""
         a_tree = self.example_branch('a')
+        base_rev = a_tree.branch.last_revision()
         os.chdir('a')
         self.run_bzr('pull', retcode=3)
         self.run_bzr('missing', retcode=3)
@@ -71,16 +73,17 @@
         self.run_bzr('pull')
         os.mkdir('subdir')
         b_tree.add('subdir')
-        b_tree.commit(message='blah', allow_pointless=True)
+        new_rev = b_tree.commit(message='blah', allow_pointless=True)
 
         os.chdir('..')
         a = Branch.open('a')
         b = Branch.open('b')
-        self.assertEqual(a.revision_history(), b.revision_history()[:-1])
+        self.assertEqual(a.last_revision(), base_rev)
+        self.assertEqual(b.last_revision(), new_rev)
 
         os.chdir('a')
         self.run_bzr('pull ../b')
-        self.assertEqual(a.revision_history(), b.revision_history())
+        self.assertEqual(a.last_revision(), b.last_revision())
         a_tree.commit(message='blah2', allow_pointless=True)
         b_tree.commit(message='blah3', allow_pointless=True)
         # no overwrite
@@ -91,13 +94,13 @@
         os.chdir('overwriteme')
         self.run_bzr('pull --overwrite ../a')
         overwritten = Branch.open('.')
-        self.assertEqual(overwritten.revision_history(),
-                         a.revision_history())
+        self.assertEqual(overwritten.last_revision(),
+                         a.last_revision())
         a_tree.merge_from_branch(b_tree.branch)
         a_tree.commit(message="blah4", allow_pointless=True)
         os.chdir('../b/subdir')
         self.run_bzr('pull ../../a')
-        self.assertEqual(a.revision_history()[-1], b.revision_history()[-1])
+        self.assertEqual(a.last_revision(), b.last_revision())
         sub_tree = WorkingTree.open_containing('.')[0]
         sub_tree.commit(message="blah5", allow_pointless=True)
         sub_tree.commit(message="blah6", allow_pointless=True)
@@ -143,7 +146,7 @@
         self.run_bzr('pull -r 3')
         self.assertEqual(b.revno(),3)
         self.run_bzr('pull -r 4')
-        self.assertEqual(a.revision_history(), b.revision_history())
+        self.assertEqual(a.last_revision(), b.last_revision())
 
     def test_pull_tags(self):
         """Tags are updated by pull, and revisions named in those tags are
@@ -152,7 +155,7 @@
         # Make a source, sprout a target off it
         builder = self.make_branch_builder('source')
         source = fixtures.build_branch_with_non_ancestral_rev(builder)
-        source.get_config().set_user_option('branch.fetch_tags', 'True')
+        source.get_config_stack().set('branch.fetch_tags', True)
         target_bzrdir = source.bzrdir.sprout('target')
         source.tags.set_tag('tag-a', 'rev-2')
         # Pull from source
@@ -179,20 +182,18 @@
         self.build_tree_contents([('a/foo', 'a third change')])
         a_tree.commit(message='a third change')
 
-        rev_history_a = a_tree.branch.revision_history()
-        self.assertEqual(len(rev_history_a), 3)
+        self.assertEqual(a_tree.branch.last_revision_info()[0], 3)
 
         b_tree.merge_from_branch(a_tree.branch)
         b_tree.commit(message='merge')
 
-        self.assertEqual(len(b_tree.branch.revision_history()), 2)
+        self.assertEqual(b_tree.branch.last_revision_info()[0], 2)
 
         os.chdir('b')
         self.run_bzr('pull --overwrite ../a')
-        rev_history_b = b_tree.branch.revision_history()
-        self.assertEqual(len(rev_history_b), 3)
-
-        self.assertEqual(rev_history_b, rev_history_a)
+        (last_revinfo_b) = b_tree.branch.last_revision_info()
+        self.assertEqual(last_revinfo_b[0], 3)
+        self.assertEqual(last_revinfo_b[1], a_tree.branch.last_revision())
 
     def test_overwrite_children(self):
         # Make sure pull --overwrite sets the revision-history
@@ -210,27 +211,26 @@
         self.build_tree_contents([('a/foo', 'a third change')])
         a_tree.commit(message='a third change')
 
-        self.assertEqual(len(a_tree.branch.revision_history()), 3)
+        self.assertEqual(a_tree.branch.last_revision_info()[0], 3)
 
         b_tree.merge_from_branch(a_tree.branch)
         b_tree.commit(message='merge')
 
-        self.assertEqual(len(b_tree.branch.revision_history()), 2)
+        self.assertEqual(b_tree.branch.last_revision_info()[0], 2)
 
         self.build_tree_contents([('a/foo', 'a fourth change\n')])
         a_tree.commit(message='a fourth change')
 
-        rev_history_a = a_tree.branch.revision_history()
-        self.assertEqual(len(rev_history_a), 4)
+        rev_info_a = a_tree.branch.last_revision_info()
+        self.assertEqual(rev_info_a[0], 4)
 
         # With convergence, we could just pull over the
         # new change, but with --overwrite, we want to switch our history
         os.chdir('b')
         self.run_bzr('pull --overwrite ../a')
-        rev_history_b = b_tree.branch.revision_history()
-        self.assertEqual(len(rev_history_b), 4)
-
-        self.assertEqual(rev_history_b, rev_history_a)
+        rev_info_b = b_tree.branch.last_revision_info()
+        self.assertEqual(rev_info_b[0], 4)
+        self.assertEqual(rev_info_b, rev_info_a)
 
     def test_pull_remember(self):
         """Pull changes from one branch to another and test parent location."""
@@ -305,8 +305,8 @@
         self.assertEqual(err,
                 ' M  a\nAll changes applied successfully.\n')
 
-        self.assertEqualDiff(tree_a.branch.revision_history(),
-                             tree_b.branch.revision_history())
+        self.assertEqualDiff(tree_a.branch.last_revision(),
+                             tree_b.branch.last_revision())
 
         testament_a = Testament.from_revision(tree_a.branch.repository,
                                               tree_a.get_parent_ids()[0])
@@ -318,7 +318,7 @@
         # it is legal to attempt to pull an already-merged bundle
         out, err = self.run_bzr('pull ../bundle')
         self.assertEqual(err, '')
-        self.assertEqual(out, 'No revisions to pull.\n')
+        self.assertEqual(out, 'No revisions or tags to pull.\n')
 
     def test_pull_verbose_no_files(self):
         """Pull --verbose should not list modified files"""
@@ -421,9 +421,10 @@
         # become necessary for this use case. Please do not adjust this number
         # upwards without agreement from bzr's network support maintainers.
         self.assertLength(19, self.hpss_calls)
+        self.assertLength(1, self.hpss_connections)
         remote = Branch.open('stacked')
         self.assertEndsWith(remote.get_stacked_on_url(), '/parent')
-    
+
     def test_pull_cross_format_warning(self):
         """You get a warning for probably slow cross-format pulls.
         """
@@ -533,3 +534,51 @@
         out = self.run_bzr(['pull','-d','to','from'],retcode=1)
         self.assertEqual(out,
             ('No revisions to pull.\nConflicting tags:\n    mytag\n', ''))
+
+    def test_pull_tag_notification(self):
+        """pulling tags with conflicts will change the exit code"""
+        # create a branch, see that --show-base fails
+        from_tree = self.make_branch_and_tree('from')
+        from_tree.branch.tags.set_tag("mytag", "somerevid")
+        to_tree = self.make_branch_and_tree('to')
+        out = self.run_bzr(['pull', '-d', 'to', 'from'])
+        self.assertEqual(out,
+            ('1 tag(s) updated.\n', ''))
+
+    def test_pull_tag_overwrite(self):
+        """pulling tags with --overwrite only reports changed tags."""
+        # create a branch, see that --show-base fails
+        from_tree = self.make_branch_and_tree('from')
+        from_tree.branch.tags.set_tag("mytag", "somerevid")
+        to_tree = self.make_branch_and_tree('to')
+        to_tree.branch.tags.set_tag("mytag", "somerevid")
+        out = self.run_bzr(['pull', '--overwrite', '-d', 'to', 'from'])
+        self.assertEqual(out,
+            ('No revisions or tags to pull.\n', ''))
+
+
+class TestPullOutput(script.TestCaseWithTransportAndScript):
+
+    def test_pull_log_format(self):
+        self.run_script("""
+            $ bzr init trunk
+            Created a standalone tree (format: 2a)
+            $ cd trunk
+            $ echo foo > file
+            $ bzr add
+            adding file
+            $ bzr commit -m 'we need some foo'
+            2>Committing to:...trunk/
+            2>added file
+            2>Committed revision 1.
+            $ cd ..
+            $ bzr init feature
+            Created a standalone tree (format: 2a)
+            $ cd feature
+            $ bzr pull -v ../trunk -Olog_format=line
+            Now on revision 1.
+            Added Revisions:
+            1: jrandom@example.com ...we need some foo
+            2>+N  file
+            2>All changes applied successfully.
+            """)

=== modified file 'bzrlib/tests/blackbox/test_push.py'
--- bzrlib/tests/blackbox/test_push.py	2011-08-16 15:03:03 +0000
+++ bzrlib/tests/blackbox/test_push.py	2011-12-24 10:10:59 +0000
@@ -22,6 +22,7 @@
 from bzrlib import (
     branch,
     bzrdir,
+    controldir,
     errors,
     osutils,
     tests,
@@ -32,12 +33,12 @@
     )
 from bzrlib.repofmt import knitrepo
 from bzrlib.tests import (
-    blackbox,
     http_server,
     scenarios,
+    script,
     test_foreign,
-    test_server,
     )
+from bzrlib.tests.matchers import ContainsNoVfsCalls
 from bzrlib.transport import memory
 
 
@@ -56,6 +57,26 @@
                            ['push', public_url],
                            working_dir='source')
 
+    def test_push_suggests_parent_alias(self):
+        """Push suggests using :parent if there is a known parent branch."""
+        tree_a = self.make_branch_and_tree('a')
+        tree_a.commit('this is a commit')
+        tree_b = self.make_branch_and_tree('b')
+
+        # If there is no parent location set, :parent isn't mentioned.
+        out = self.run_bzr('push', working_dir='a', retcode=3)
+        self.assertEquals(out,
+                ('','bzr: ERROR: No push location known or specified.\n'))
+
+        # If there is a parent location set, the error suggests :parent.
+        tree_a.branch.set_parent(tree_b.branch.base)
+        out = self.run_bzr('push', working_dir='a', retcode=3)
+        self.assertEquals(out,
+            ('','bzr: ERROR: No push location known or specified. '
+                'To push to the parent branch '
+                '(at %s), use \'bzr push :parent\'.\n' %
+                urlutils.unescape_for_display(tree_b.branch.base, 'utf-8')))
+
     def test_push_remember(self):
         """Push changes from one branch to another and test push location."""
         transport = self.get_transport()
@@ -157,7 +178,9 @@
         self.run_bzr('push -d tree pushed-to')
         path = t.branch.get_push_location()
         out, err = self.run_bzr('push', working_dir="tree")
-        self.assertEqual('Using saved push location: %s\nNo new revisions to push.\n' % urlutils.local_path_from_url(path), err)
+        self.assertEqual('Using saved push location: %s\n'
+                         'No new revisions or tags to push.\n' %
+                         urlutils.local_path_from_url(path), err)
         out, err = self.run_bzr('push -q', working_dir="tree")
         self.assertEqual('', out)
         self.assertEqual('', err)
@@ -171,7 +194,7 @@
 
         def make_shared_tree(path):
             shared_repo.bzrdir.root_transport.mkdir(path)
-            shared_repo.bzrdir.create_branch_convenience('repo/' + path)
+            controldir.ControlDir.create_branch_convenience('repo/' + path)
             return workingtree.WorkingTree.open('repo/' + path)
         tree_a = make_shared_tree('a')
         self.build_tree(['repo/a/file'])
@@ -245,6 +268,8 @@
         # become necessary for this use case. Please do not adjust this number
         # upwards without agreement from bzr's network support maintainers.
         self.assertLength(9, self.hpss_calls)
+        self.assertLength(1, self.hpss_connections)
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)
 
     def test_push_smart_stacked_streaming_acceptance(self):
         self.setup_smart_server_with_call_log()
@@ -260,7 +285,9 @@
         # being too low. If rpc_count increases, more network roundtrips have
         # become necessary for this use case. Please do not adjust this number
         # upwards without agreement from bzr's network support maintainers.
-        self.assertLength(13, self.hpss_calls)
+        self.assertLength(14, self.hpss_calls)
+        self.assertLength(1, self.hpss_connections)
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)
         remote = branch.Branch.open('public')
         self.assertEndsWith(remote.get_stacked_on_url(), '/parent')
 
@@ -277,6 +304,8 @@
         # become necessary for this use case. Please do not adjust this number
         # upwards without agreement from bzr's network support maintainers.
         self.assertLength(11, self.hpss_calls)
+        self.assertLength(1, self.hpss_connections)
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)
 
     def test_push_smart_incremental_acceptance(self):
         self.setup_smart_server_with_call_log()
@@ -293,6 +322,8 @@
         # become necessary for this use case. Please do not adjust this number
         # upwards without agreement from bzr's network support maintainers.
         self.assertLength(11, self.hpss_calls)
+        self.assertLength(1, self.hpss_connections)
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)
 
     def test_push_smart_with_default_stacking_url_path_segment(self):
         # If the default stacked-on location is a path element then branches
@@ -674,8 +705,8 @@
     def set_config_push_strict(self, value):
         # set config var (any of bazaar.conf, locations.conf, branch.conf
         # should do)
-        conf = self.tree.branch.get_config()
-        conf.set_user_option('push_strict', value)
+        conf = self.tree.branch.get_config_stack()
+        conf.set('push_strict', value)
 
     _default_command = ['push', '../to']
     _default_wd = 'local'
@@ -840,3 +871,27 @@
         self.assertEquals("", output)
         self.assertEquals(error, "bzr: ERROR: It is not possible to losslessly"
             " push to dummy. You may want to use dpush instead.\n")
+
+
+class TestPushOutput(script.TestCaseWithTransportAndScript):
+
+    def test_push_log_format(self):
+        self.run_script("""
+            $ bzr init trunk
+            Created a standalone tree (format: 2a)
+            $ cd trunk
+            $ echo foo > file
+            $ bzr add
+            adding file
+            $ bzr commit -m 'we need some foo'
+            2>Committing to:...trunk/
+            2>added file
+            2>Committed revision 1.
+            $ bzr init ../feature
+            Created a standalone tree (format: 2a)
+            $ bzr push -v ../feature -Olog_format=line
+            Added Revisions:
+            1: jrandom@example.com ...we need some foo
+            2>All changes applied successfully.
+            2>Pushed up to revision 1.
+            """)

=== modified file 'bzrlib/tests/blackbox/test_re_sign.py'
--- bzrlib/tests/blackbox/test_re_sign.py	2010-05-02 20:10:25 +0000
+++ bzrlib/tests/blackbox/test_re_sign.py	2011-12-09 12:05:47 +0000
@@ -19,8 +19,6 @@
 """Black-box tests for bzr re-sign.
 """
 
-import os
-
 from bzrlib import (
     gpg,
     tests,

=== modified file 'bzrlib/tests/blackbox/test_reconcile.py'
--- bzrlib/tests/blackbox/test_reconcile.py	2010-01-25 17:48:22 +0000
+++ bzrlib/tests/blackbox/test_reconcile.py	2011-12-14 20:21:52 +0000
@@ -20,9 +20,9 @@
 from bzrlib import (
     bzrdir,
     inventory,
-    repository,
     tests,
     )
+from bzrlib.tests.matchers import ContainsNoVfsCalls
 
 
 class TrivialTest(tests.TestCaseWithTransport):
@@ -72,3 +72,20 @@
                      does_backup_text))
         self.assertEqualDiff(expected, out)
         self.assertEqualDiff(err, "")
+
+
+class TestSmartServerReconcile(tests.TestCaseWithTransport):
+
+    def test_simple_reconcile(self):
+        self.setup_smart_server_with_call_log()
+        self.make_branch('branch')
+        self.reset_smart_call_log()
+        out, err = self.run_bzr(['reconcile', self.get_url('branch')])
+        # This figure represent the amount of work to perform this use case. It
+        # is entirely ok to reduce this number if a test fails due to rpc_count
+        # being too low. If rpc_count increases, more network roundtrips have
+        # become necessary for this use case. Please do not adjust this number
+        # upwards without agreement from bzr's network support maintainers.
+        self.assertLength(10, self.hpss_calls)
+        self.assertLength(1, self.hpss_connections)
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)

=== modified file 'bzrlib/tests/blackbox/test_reconfigure.py'
--- bzrlib/tests/blackbox/test_reconfigure.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/blackbox/test_reconfigure.py	2011-09-06 18:37:06 +0000
@@ -21,9 +21,10 @@
     workingtree,
     )
 from bzrlib.branchbuilder import BranchBuilder
-
-
-class TestReconfigure(tests.TestCaseWithTransport):
+from bzrlib.tests.script import TestCaseWithTransportAndScript
+
+
+class TestReconfigure(TestCaseWithTransportAndScript):
 
     def test_no_type(self):
         branch = self.make_branch('branch')
@@ -175,15 +176,45 @@
         self.run_bzr('revert', working_dir='checkout')
         self.check_file_contents('checkout/file', 'foo\n')
 
-    def test_lightweight_knit_checkout_to_tree(self, format=None):
+    def test_lightweight_knit_checkout_to_tree(self):
         self.test_lightweight_format_checkout_to_tree('knit')
 
-    def test_lightweight_pack092_checkout_to_tree(self, format=None):
+    def test_lightweight_pack092_checkout_to_tree(self):
         self.test_lightweight_format_checkout_to_tree('pack-0.92')
 
-    def test_lightweight_rich_root_pack_checkout_to_tree(self, format=None):
+    def test_lightweight_rich_root_pack_checkout_to_tree(self):
         self.test_lightweight_format_checkout_to_tree('rich-root-pack')
 
+    def test_branch_and_use_shared(self):
+        self.run_script("""\
+$ bzr init -q branch
+$ echo foo > branch/foo
+$ bzr add -q branch/foo
+$ bzr commit -q -m msg branch
+$ bzr init-repo -q .
+$ bzr reconfigure --branch --use-shared branch
+$ bzr info branch
+Repository branch (format: ...)
+Location:
+  shared repository: .
+  repository branch: branch
+""")
+
+    def test_use_shared_and_branch(self):
+        self.run_script("""\
+$ bzr init -q branch
+$ echo foo > branch/foo
+$ bzr add -q branch/foo
+$ bzr commit -q -m msg branch
+$ bzr init-repo -q .
+$ bzr reconfigure --use-shared --branch branch
+$ bzr info branch
+Repository branch (format: ...)
+Location:
+  shared repository: .
+  repository branch: branch
+""")
+
 
 class TestReconfigureStacking(tests.TestCaseWithTransport):
 

=== modified file 'bzrlib/tests/blackbox/test_reference.py'
--- bzrlib/tests/blackbox/test_reference.py	2009-04-30 14:04:38 +0000
+++ bzrlib/tests/blackbox/test_reference.py	2011-09-21 20:29:48 +0000
@@ -24,11 +24,10 @@
 
 class TestReference(TestCaseWithTransport):
 
-    def make_branch(self, location, format=None):
-        if format is None:
-            format = bzrdir.format_registry.make_bzrdir('1.9')
-            format.set_branch_format(_mod_branch.BzrBranchFormat8())
-        return TestCaseWithTransport.make_branch(self, location, format=format)
+    def get_default_format(self):
+        format = bzrdir.format_registry.make_bzrdir('1.9')
+        format.set_branch_format(_mod_branch.BzrBranchFormat8())
+        return format
 
     def test_no_args_lists(self):
         branch = self.make_branch('branch')

=== modified file 'bzrlib/tests/blackbox/test_resolve.py'
--- bzrlib/tests/blackbox/test_resolve.py	2011-06-15 11:36:05 +0000
+++ bzrlib/tests/blackbox/test_resolve.py	2011-09-16 15:37:58 +0000
@@ -36,18 +36,18 @@
 Path conflict: mydir3 / mydir2
 Text conflict in myfile
 $ bzr resolve myfile
-2>1 conflict(s) resolved, 2 remaining
+2>1 conflict resolved, 2 remaining
 $ bzr resolve my_other_file
-2>1 conflict(s) resolved, 1 remaining
+2>1 conflict resolved, 1 remaining
 $ bzr resolve mydir2
-2>1 conflict(s) resolved, 0 remaining
+2>1 conflict resolved, 0 remaining
 """)
 
     def test_resolve_all(self):
         self.run_script("""\
 $ cd branch
 $ bzr resolve --all
-2>3 conflict(s) resolved, 0 remaining
+2>3 conflicts resolved, 0 remaining
 $ bzr conflicts
 """)
 
@@ -56,19 +56,19 @@
 $ mkdir branch/subdir
 $ cd branch/subdir
 $ bzr resolve ../myfile
-2>1 conflict(s) resolved, 2 remaining
+2>1 conflict resolved, 2 remaining
 """)
 
     def test_resolve_via_directory_option(self):
         self.run_script("""\
 $ bzr resolve -d branch myfile
-2>1 conflict(s) resolved, 2 remaining
+2>1 conflict resolved, 2 remaining
 """)
 
     def test_resolve_all_via_directory_option(self):
         self.run_script("""\
 $ bzr resolve -d branch --all
-2>3 conflict(s) resolved, 0 remaining
+2>3 conflicts resolved, 0 remaining
 $ bzr conflicts -d branch
 """)
 
@@ -107,7 +107,7 @@
 $ bzr resolve --take-other
 2>deleted dir/file.THIS
 2>deleted dir
-2>3 conflict(s) resolved, 0 remaining
+2>3 conflicts resolved, 0 remaining
 ''')
 
 
@@ -123,7 +123,7 @@
         file_conflict = conflicts.TextConflict('file', file_id='file_id')
         tree.set_conflicts(conflicts.ConflictList([file_conflict]))
         note = self.run_bzr('resolve', retcode=1, working_dir='tree')[1]
-        self.assertContainsRe(note, '0 conflict\\(s\\) auto-resolved.')
+        self.assertContainsRe(note, '0 conflicts auto-resolved.')
         self.assertContainsRe(note,
             'Remaining conflicts:\nText conflict in file')
         self.build_tree_contents([('tree/file', 'a\n')])

=== modified file 'bzrlib/tests/blackbox/test_revno.py'
--- bzrlib/tests/blackbox/test_revno.py	2009-06-30 16:59:21 +0000
+++ bzrlib/tests/blackbox/test_revno.py	2011-12-14 20:21:52 +0000
@@ -21,6 +21,8 @@
 import os
 
 from bzrlib import tests
+from bzrlib.tests.matchers import ContainsNoVfsCalls
+
 
 class TestRevno(tests.TestCaseWithTransport):
 
@@ -117,3 +119,61 @@
         out, err = self.run_bzr('revno --tree checkout_b')
         self.assertEqual('', err)
         self.assertEqual('???\n', out)
+
+    def test_revno_with_revision(self):
+        wt = self.make_branch_and_tree('.')
+        revid1 = wt.commit('rev1')
+        revid2 = wt.commit('rev2')
+
+        out, err = self.run_bzr('revno -r-2 .')
+        self.assertEqual('1\n', out)
+
+        out, err = self.run_bzr('revno -rrevid:%s .' % revid1)
+        self.assertEqual('1\n', out)
+
+    def test_revno_and_tree_mutually_exclusive(self):
+        wt = self.make_branch_and_tree('.')
+        out, err = self.run_bzr('revno -r-2 --tree .', retcode=3)
+        self.assertEqual('', out)
+        self.assertEqual(
+            'bzr: ERROR: --tree and --revision can not be used together\n',
+            err)
+
+
+class TestSmartServerRevno(tests.TestCaseWithTransport):
+
+    def test_simple_branch_revno(self):
+        self.setup_smart_server_with_call_log()
+        t = self.make_branch_and_tree('branch')
+        self.build_tree_contents([('branch/foo', 'thecontents')])
+        t.add("foo")
+        revid = t.commit("message")
+        self.reset_smart_call_log()
+        out, err = self.run_bzr(['revno', self.get_url('branch')])
+        # This figure represent the amount of work to perform this use case. It
+        # is entirely ok to reduce this number if a test fails due to rpc_count
+        # being too low. If rpc_count increases, more network roundtrips have
+        # become necessary for this use case. Please do not adjust this number
+        # upwards without agreement from bzr's network support maintainers.
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)
+        self.assertLength(1, self.hpss_connections)
+        self.assertLength(6, self.hpss_calls)
+
+    def test_simple_branch_revno_lookup(self):
+        self.setup_smart_server_with_call_log()
+        t = self.make_branch_and_tree('branch')
+        self.build_tree_contents([('branch/foo', 'thecontents')])
+        t.add("foo")
+        revid1 = t.commit("message")
+        revid2 = t.commit("message")
+        self.reset_smart_call_log()
+        out, err = self.run_bzr(['revno', '-rrevid:' + revid1,
+            self.get_url('branch')])
+        # This figure represent the amount of work to perform this use case. It
+        # is entirely ok to reduce this number if a test fails due to rpc_count
+        # being too low. If rpc_count increases, more network roundtrips have
+        # become necessary for this use case. Please do not adjust this number
+        # upwards without agreement from bzr's network support maintainers.
+        self.assertLength(5, self.hpss_calls)
+        self.assertLength(1, self.hpss_connections)
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)

=== modified file 'bzrlib/tests/blackbox/test_rmbranch.py'
--- bzrlib/tests/blackbox/test_rmbranch.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/blackbox/test_rmbranch.py	2011-12-14 20:21:52 +0000
@@ -23,18 +23,20 @@
 from bzrlib.tests import (
     TestCaseWithTransport,
     )
+from bzrlib.tests.matchers import ContainsNoVfsCalls
 
 
 class TestRemoveBranch(TestCaseWithTransport):
 
-    def example_branch(self, path='.'):
-        tree = self.make_branch_and_tree(path)
+    def example_branch(self, path='.', format=None):
+        tree = self.make_branch_and_tree(path, format=format)
         self.build_tree_contents([(path + '/hello', 'foo')])
         tree.add('hello')
         tree.commit(message='setup')
         self.build_tree_contents([(path + '/goodbye', 'baz')])
         tree.add('goodbye')
         tree.commit(message='setup')
+        return tree
 
     def test_remove_local(self):
         # Remove a local branch.
@@ -57,3 +59,30 @@
         self.run_bzr('rmbranch', working_dir='a')
         dir = bzrdir.BzrDir.open('a')
         self.assertFalse(dir.has_branch())
+
+    def test_remove_colo(self):
+        # Remove a colocated branch.
+        tree = self.example_branch('a', format='development-colo')
+        tree.bzrdir.create_branch(name="otherbranch")
+        self.assertTrue(tree.bzrdir.has_branch('otherbranch'))
+        self.run_bzr('rmbranch %s,branch=otherbranch' % tree.bzrdir.user_url)
+        dir = bzrdir.BzrDir.open('a')
+        self.assertFalse(dir.has_branch('otherbranch'))
+        self.assertTrue(dir.has_branch())
+
+
+class TestSmartServerRemoveBranch(TestCaseWithTransport):
+
+    def test_simple_remove_branch(self):
+        self.setup_smart_server_with_call_log()
+        self.make_branch('branch')
+        self.reset_smart_call_log()
+        out, err = self.run_bzr(['rmbranch', self.get_url('branch')])
+        # This figure represent the amount of work to perform this use case. It
+        # is entirely ok to reduce this number if a test fails due to rpc_count
+        # being too low. If rpc_count increases, more network roundtrips have
+        # become necessary for this use case. Please do not adjust this number
+        # upwards without agreement from bzr's network support maintainers.
+        self.assertLength(5, self.hpss_calls)
+        self.assertLength(1, self.hpss_connections)
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)

=== modified file 'bzrlib/tests/blackbox/test_send.py'
--- bzrlib/tests/blackbox/test_send.py	2011-05-17 15:01:58 +0000
+++ bzrlib/tests/blackbox/test_send.py	2011-12-14 20:21:52 +0000
@@ -16,7 +16,6 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
-import sys
 from cStringIO import StringIO
 
 from bzrlib import (
@@ -29,8 +28,8 @@
 from bzrlib.transport import memory
 from bzrlib.tests import (
     scenarios,
-    script,
     )
+from bzrlib.tests.matchers import ContainsNoVfsCalls
 
 
 load_tests = scenarios.load_tests_apply_scenarios
@@ -189,7 +188,7 @@
 
     def test_note_revisions(self):
         stderr = self.run_send([])[1]
-        self.assertEndsWith(stderr, '\nBundling 1 revision(s).\n')
+        self.assertEndsWith(stderr, '\nBundling 1 revision.\n')
 
     def test_mailto_option(self):
         b = branch.Branch.open('branch')
@@ -296,8 +295,8 @@
     def set_config_send_strict(self, value):
         # set config var (any of bazaar.conf, locations.conf, branch.conf
         # should do)
-        conf = self.local_tree.branch.get_config()
-        conf.set_user_option('send_strict', value)
+        conf = self.local_tree.branch.get_config_stack()
+        conf.set('send_strict', value)
 
     def assertSendFails(self, args):
         out, err = self.run_send(args, rc=3, err_re=self._default_errors)
@@ -311,7 +310,10 @@
         if revs is None:
             revs = self._default_sent_revs
         out, err = self.run_send(args, err_re=err_re)
-        bundling_revs = 'Bundling %d revision(s).\n' % len(revs)
+        if len(revs) == 1:
+            bundling_revs = 'Bundling %d revision.\n'% len(revs)
+        else:
+            bundling_revs = 'Bundling %d revisions.\n' % len(revs)
         if with_warning:
             self.assertContainsRe(err, self._default_additional_warning)
             self.assertEndsWith(err, bundling_revs)
@@ -436,3 +438,27 @@
 class TestBundleStrictWithoutChanges(TestSendStrictWithoutChanges):
 
     _default_command = ['bundle-revisions', '../parent']
+
+
+class TestSmartServerSend(tests.TestCaseWithTransport):
+
+    def test_send(self):
+        self.setup_smart_server_with_call_log()
+        t = self.make_branch_and_tree('branch')
+        self.build_tree_contents([('branch/foo', 'thecontents')])
+        t.add("foo")
+        t.commit("message")
+        local = t.bzrdir.sprout('local-branch').open_workingtree()
+        self.build_tree_contents([('branch/foo', 'thenewcontents')])
+        local.commit("anothermessage")
+        self.reset_smart_call_log()
+        out, err = self.run_bzr(
+            ['send', '-o', 'x.diff', self.get_url('branch')], working_dir='local-branch')
+        # This figure represent the amount of work to perform this use case. It
+        # is entirely ok to reduce this number if a test fails due to rpc_count
+        # being too low. If rpc_count increases, more network roundtrips have
+        # become necessary for this use case. Please do not adjust this number
+        # upwards without agreement from bzr's network support maintainers.
+        self.assertLength(9, self.hpss_calls)
+        self.assertLength(1, self.hpss_connections)
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)

=== modified file 'bzrlib/tests/blackbox/test_serve.py'
--- bzrlib/tests/blackbox/test_serve.py	2011-08-19 22:34:02 +0000
+++ bzrlib/tests/blackbox/test_serve.py	2011-11-01 18:40:39 +0000
@@ -25,6 +25,7 @@
 
 from bzrlib import (
     builtins,
+    config,
     errors,
     osutils,
     revision as _mod_revision,
@@ -66,7 +67,7 @@
             try:
                 # Run func if set
                 self.tcp_server = tcp_server
-                if not func is None:
+                if func is not None:
                     try:
                         func(*func_args, **func_kwargs)
                     except Exception, e:
@@ -87,6 +88,10 @@
         SmartTCPServer.hooks.install_named_hook(
             'server_started_ex', on_server_start,
             'run_bzr_serve_then_func hook')
+        # It seesm thread.interrupt_main() will not raise KeyboardInterrupt
+        # until after socket.accept returns. So we set the timeout low to make
+        # the test faster.
+        self.overrideAttr(SmartTCPServer, '_ACCEPT_TIMEOUT', 0.1)
         # start a TCP server
         try:
             out, err = self.run_bzr(['serve'] + list(serve_args),
@@ -266,6 +271,76 @@
         f.close()
         self.assertContainsRe(content, r'hpss request: \[[0-9-]+\]')
 
+    def test_bzr_serve_supports_configurable_timeout(self):
+        gs = config.GlobalStack()
+        gs.set('serve.client_timeout', 0.2)
+        process, url = self.start_server_port()
+        self.build_tree_contents([('a_file', 'contents\n')])
+        # We can connect and issue a request
+        t = transport.get_transport_from_url(url)
+        self.assertEqual('contents\n', t.get_bytes('a_file'))
+        # However, if we just wait for more content from the server, it will
+        # eventually disconnect us.
+        # TODO: Use something like signal.alarm() so that if the server doesn't
+        #       properly handle the timeout, we end up failing the test instead
+        #       of hanging forever.
+        m = t.get_smart_medium()
+        m.read_bytes(1)
+        # Now, we wait for timeout to trigger
+        err = process.stderr.readline()
+        self.assertEqual(
+            'Connection Timeout: disconnecting client after 0.2 seconds\n',
+            err)
+        self.assertServerFinishesCleanly(process)
+
+    def test_bzr_serve_supports_client_timeout(self):
+        process, url = self.start_server_port(['--client-timeout=0.1'])
+        self.build_tree_contents([('a_file', 'contents\n')])
+        # We can connect and issue a request
+        t = transport.get_transport_from_url(url)
+        self.assertEqual('contents\n', t.get_bytes('a_file'))
+        # However, if we just wait for more content from the server, it will
+        # eventually disconnect us.
+        # TODO: Use something like signal.alarm() so that if the server doesn't
+        #       properly handle the timeout, we end up failing the test instead
+        #       of hanging forever.
+        m = t.get_smart_medium()
+        m.read_bytes(1)
+        # Now, we wait for timeout to trigger
+        err = process.stderr.readline()
+        self.assertEqual(
+            'Connection Timeout: disconnecting client after 0.1 seconds\n',
+            err)
+        self.assertServerFinishesCleanly(process)
+
+    def test_bzr_serve_graceful_shutdown(self):
+        big_contents = 'a'*64*1024
+        self.build_tree_contents([('bigfile', big_contents)])
+        process, url = self.start_server_port(['--client-timeout=1.0'])
+        t = transport.get_transport_from_url(url)
+        m = t.get_smart_medium()
+        c = client._SmartClient(m)
+        # Start, but don't finish a response
+        resp, response_handler = c.call_expecting_body('get', 'bigfile')
+        self.assertEqual(('ok',), resp)
+        # Note: process.send_signal is a Python 2.6ism
+        process.send_signal(signal.SIGHUP)
+        # Wait for the server to notice the signal, and then read the actual
+        # body of the response. That way we know that it is waiting for the
+        # request to finish
+        self.assertEqual('Requested to stop gracefully\n',
+                         process.stderr.readline())
+        self.assertEqual('Waiting for 1 client(s) to finish\n',
+                         process.stderr.readline())
+        body = response_handler.read_body_bytes()
+        if body != big_contents:
+            self.fail('Failed to properly read the contents of "bigfile"')
+        # Now that our request is finished, the medium should notice it has
+        # been disconnected.
+        self.assertEqual('', m.read_bytes(1))
+        # And the server should be stopping
+        self.assertEqual(0, process.wait())
+
 
 class TestCmdServeChrooting(TestBzrServeBase):
 
@@ -327,7 +402,7 @@
             self.fake_expanduser, lambda t: base_path)
         mem_transport = self.get_transport()
         mem_transport.mkdir_multi(['home', 'home/user'])
-        bzr_server.set_up(mem_transport, None, None, inet=True)
+        bzr_server.set_up(mem_transport, None, None, inet=True, timeout=4.0)
         self.addCleanup(bzr_server.tear_down)
         return bzr_server
 
@@ -349,7 +424,7 @@
         base_url = urlutils.local_path_to_url(base_dir) + '/'
         # Define a fake 'protocol' to capture the transport that cmd_serve
         # passes to serve_bzr.
-        def capture_transport(transport, host, port, inet):
+        def capture_transport(transport, host, port, inet, timeout):
             self.bzr_serve_transport = transport
         cmd = builtins.cmd_serve()
         # Read-only
@@ -366,3 +441,10 @@
         self.assertEqual(base_url, self.bzr_serve_transport.base)
         self.assertEqual(base_dir,
             server_maker.get_base_path(self.bzr_serve_transport))
+        # Read-only, from a URL
+        cmd.run(directory=base_url, protocol=capture_transport)
+        server_maker = BzrServerFactory()
+        self.assertEqual(
+            'readonly+%s' % base_url, self.bzr_serve_transport.base)
+        self.assertEqual(
+            base_dir, server_maker.get_base_path(self.bzr_serve_transport))

=== modified file 'bzrlib/tests/blackbox/test_shared_repository.py'
--- bzrlib/tests/blackbox/test_shared_repository.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/blackbox/test_shared_repository.py	2011-12-30 14:36:46 +0000
@@ -18,10 +18,11 @@
 
 import os
 
-from bzrlib import osutils
 from bzrlib.bzrdir import BzrDir, BzrDirMetaFormat1
 import bzrlib.errors as errors
 from bzrlib.tests import TestCaseInTempDir
+from bzrlib.tests.matchers import ContainsNoVfsCalls
+
 
 class TestSharedRepo(TestCaseInTempDir):
 
@@ -120,7 +121,9 @@
         # being too low. If rpc_count increases, more network roundtrips have
         # become necessary for this use case. Please do not adjust this number
         # upwards without agreement from bzr's network support maintainers.
-        self.assertLength(15, self.hpss_calls)
+        self.assertLength(10, self.hpss_calls)
+        self.assertLength(1, self.hpss_connections)
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)
 
     def test_notification_on_branch_from_repository(self):
         out, err = self.run_bzr("init-repository -q a")

=== added file 'bzrlib/tests/blackbox/test_shell_complete.py'
--- bzrlib/tests/blackbox/test_shell_complete.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_shell_complete.py	2011-12-29 21:03:53 +0000
@@ -0,0 +1,27 @@
+# Copyright (C) 2011 Canonical Ltd
+# -*- coding: utf-8 -*-
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+
+"""Black-box tests for 'bzr shell-complete'."""
+
+from bzrlib.tests import TestCaseWithTransport
+
+
+class TestShellComplete(TestCaseWithTransport):
+
+    def test_shell_complete(self):
+        self.run_bzr("shell-complete")

=== modified file 'bzrlib/tests/blackbox/test_sign_my_commits.py'
--- bzrlib/tests/blackbox/test_sign_my_commits.py	2011-08-16 11:05:38 +0000
+++ bzrlib/tests/blackbox/test_sign_my_commits.py	2011-12-14 21:57:37 +0000
@@ -21,6 +21,7 @@
     gpg,
     tests,
     )
+from bzrlib.tests.matchers import ContainsNoVfsCalls
 
 
 class SignMyCommits(tests.TestCaseWithTransport):
@@ -137,3 +138,53 @@
                            '0 commits with unknown keys\n'
                            '0 commits not valid\n'
                            '1 commit not signed\n', ''), out)
+
+
+class TestSmartServerSignMyCommits(tests.TestCaseWithTransport):
+
+    def monkey_patch_gpg(self):
+        """Monkey patch the gpg signing strategy to be a loopback.
+
+        This also registers the cleanup, so that we will revert to
+        the original gpg strategy when done.
+        """
+        # monkey patch gpg signing mechanism
+        self.overrideAttr(gpg, 'GPGStrategy', gpg.LoopbackGPGStrategy)
+
+    def test_sign_single_commit(self):
+        self.setup_smart_server_with_call_log()
+        t = self.make_branch_and_tree('branch')
+        self.build_tree_contents([('branch/foo', 'thecontents')])
+        t.add("foo")
+        t.commit("message")
+        self.reset_smart_call_log()
+        self.monkey_patch_gpg()
+        out, err = self.run_bzr(['sign-my-commits', self.get_url('branch')])
+        # This figure represent the amount of work to perform this use case. It
+        # is entirely ok to reduce this number if a test fails due to rpc_count
+        # being too low. If rpc_count increases, more network roundtrips have
+        # become necessary for this use case. Please do not adjust this number
+        # upwards without agreement from bzr's network support maintainers.
+        self.assertLength(15, self.hpss_calls)
+        self.assertLength(1, self.hpss_connections)
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)
+
+    def test_verify_commits(self):
+        self.setup_smart_server_with_call_log()
+        t = self.make_branch_and_tree('branch')
+        self.build_tree_contents([('branch/foo', 'thecontents')])
+        t.add("foo")
+        t.commit("message")
+        self.monkey_patch_gpg()
+        out, err = self.run_bzr(['sign-my-commits', self.get_url('branch')])
+        self.reset_smart_call_log()
+        self.run_bzr('sign-my-commits')
+        out = self.run_bzr(['verify-signatures', self.get_url('branch')])
+        # This figure represent the amount of work to perform this use case. It
+        # is entirely ok to reduce this number if a test fails due to rpc_count
+        # being too low. If rpc_count increases, more network roundtrips have
+        # become necessary for this use case. Please do not adjust this number
+        # upwards without agreement from bzr's network support maintainers.
+        self.assertLength(10, self.hpss_calls)
+        self.assertLength(1, self.hpss_connections)
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)

=== modified file 'bzrlib/tests/blackbox/test_status.py'
--- bzrlib/tests/blackbox/test_status.py	2011-06-03 08:45:36 +0000
+++ bzrlib/tests/blackbox/test_status.py	2011-11-17 20:11:33 +0000
@@ -53,24 +53,25 @@
             status._show_shelve_summary,
             'bzr status')
 
-    def assertStatus(self, expected_lines, working_tree,
+    def assertStatus(self, expected_lines, working_tree, specific_files=None,
         revision=None, short=False, pending=True, verbose=False):
         """Run status in working_tree and look for output.
 
         :param expected_lines: The lines to look for.
         :param working_tree: The tree to run status in.
         """
-        output_string = self.status_string(working_tree, revision, short,
+        output_string = self.status_string(working_tree, specific_files, revision, short,
                 pending, verbose)
         self.assertEqual(expected_lines, output_string.splitlines(True))
 
-    def status_string(self, wt, revision=None, short=False, pending=True,
-        verbose=False):
+    def status_string(self, wt, specific_files=None, revision=None,
+        short=False, pending=True, verbose=False):
         # use a real file rather than StringIO because it doesn't handle
         # Unicode very well.
         tof = codecs.getwriter('utf-8')(TemporaryFile())
-        show_tree_status(wt, to_file=tof, revision=revision, short=short,
-                show_pending=pending, verbose=verbose)
+        show_tree_status(wt, specific_files=specific_files, to_file=tof,
+                revision=revision, short=short, show_pending=pending,
+                verbose=verbose)
         tof.seek(0)
         return tof.read().decode('utf-8')
 
@@ -569,14 +570,22 @@
             ],
             wt)
         self.run_bzr(['shelve', '--all', '-m', 'bar'])
-        self.build_tree(['spam.c'])
+        self.build_tree(['eggs.c', 'spam.c'])
+        wt.add('eggs.c')
         wt.add('spam.c')
         self.assertStatus([
                 'added:\n',
+                '  eggs.c\n',
                 '  spam.c\n',
                 '2 shelves exist. See "bzr shelve --list" for details.\n',
             ],
             wt)
+        self.assertStatus([
+                'added:\n',
+                '  spam.c\n',
+            ],
+            wt,
+            specific_files=['spam.c'])
 
 
 class CheckoutStatus(BranchStatus):

=== modified file 'bzrlib/tests/blackbox/test_switch.py'
--- bzrlib/tests/blackbox/test_switch.py	2011-07-18 13:58:52 +0000
+++ bzrlib/tests/blackbox/test_switch.py	2011-12-14 21:57:37 +0000
@@ -20,6 +20,7 @@
 
 import os
 
+from bzrlib.bzrdir import BzrDir
 from bzrlib import (
         osutils,
         urlutils,
@@ -30,8 +31,11 @@
         TestCaseWithTransport,
         script,
         )
+from bzrlib.tests.features import UnicodeFilenameFeature
 from bzrlib.directory_service import directories
 
+from bzrlib.tests.matchers import ContainsNoVfsCalls
+
 
 class TestSwitch(TestCaseWithTransport):
 
@@ -152,6 +156,21 @@
         self.assertEqual(branchb_id, checkout.last_revision())
         self.assertEqual(tree2.branch.base, checkout.branch.get_bound_location())
 
+    def test_switch_finds_relative_unicode_branch(self):
+        """Switch will find 'foo' relative to the branch the checkout is of."""
+        self.requireFeature(UnicodeFilenameFeature)
+        self.build_tree(['repo/'])
+        tree1 = self.make_branch_and_tree('repo/brancha')
+        tree1.commit('foo')
+        tree2 = self.make_branch_and_tree(u'repo/branch\xe9')
+        tree2.pull(tree1.branch)
+        branchb_id = tree2.commit('bar')
+        checkout =  tree1.branch.create_checkout('checkout', lightweight=True)
+        self.run_bzr(['switch', u'branch\xe9'], working_dir='checkout')
+        self.assertEqual(branchb_id, checkout.last_revision())
+        checkout = checkout.bzrdir.open_workingtree()
+        self.assertEqual(tree2.branch.base, checkout.branch.base)
+
     def test_switch_revision(self):
         tree = self._create_sample_tree()
         checkout = tree.branch.create_checkout('checkout', lightweight=True)
@@ -159,6 +178,65 @@
         self.assertPathExists('checkout/file-1')
         self.assertPathDoesNotExist('checkout/file-2')
 
+    def test_switch_existing_colocated(self):
+        # Create a branch branch-1 that initially is a checkout of 'foo'
+        # Use switch to change it to 'anotherbranch'
+        repo = self.make_repository('branch-1', format='development-colo')
+        target_branch = repo.bzrdir.create_branch(name='foo')
+        branch.BranchReferenceFormat().initialize(
+            repo.bzrdir, target_branch=target_branch)
+        tree = repo.bzrdir.create_workingtree()
+        self.build_tree(['branch-1/file-1', 'branch-1/file-2'])
+        tree.add('file-1')
+        revid1 = tree.commit('rev1')
+        tree.add('file-2')
+        revid2 = tree.commit('rev2')
+        otherbranch = tree.bzrdir.create_branch(name='anotherbranch')
+        otherbranch.generate_revision_history(revid1)
+        self.run_bzr(['switch', 'anotherbranch'], working_dir='branch-1')
+        tree = WorkingTree.open("branch-1")
+        self.assertEquals(tree.last_revision(), revid1)
+        self.assertEquals(tree.branch.control_url, otherbranch.control_url)
+
+    def test_switch_new_colocated(self):
+        # Create a branch branch-1 that initially is a checkout of 'foo'
+        # Use switch to create 'anotherbranch' which derives from that
+        repo = self.make_repository('branch-1', format='development-colo')
+        target_branch = repo.bzrdir.create_branch(name='foo')
+        branch.BranchReferenceFormat().initialize(
+            repo.bzrdir, target_branch=target_branch)
+        tree = repo.bzrdir.create_workingtree()
+        self.build_tree(['branch-1/file-1', 'branch-1/file-2'])
+        tree.add('file-1')
+        revid1 = tree.commit('rev1')
+        self.run_bzr(['switch', '-b', 'anotherbranch'], working_dir='branch-1')
+        bzrdir = BzrDir.open("branch-1")
+        self.assertEquals(
+            set([b.name for b in bzrdir.list_branches()]),
+            set(["foo", "anotherbranch"]))
+        self.assertEquals(bzrdir.open_branch().name, "anotherbranch")
+        self.assertEquals(bzrdir.open_branch().last_revision(), revid1)
+
+    def test_switch_new_colocated_unicode(self):
+        # Create a branch branch-1 that initially is a checkout of 'foo'
+        # Use switch to create 'branch\xe9' which derives from that
+        self.requireFeature(UnicodeFilenameFeature)
+        repo = self.make_repository('branch-1', format='development-colo')
+        target_branch = repo.bzrdir.create_branch(name='foo')
+        branch.BranchReferenceFormat().initialize(
+            repo.bzrdir, target_branch=target_branch)
+        tree = repo.bzrdir.create_workingtree()
+        self.build_tree(['branch-1/file-1', 'branch-1/file-2'])
+        tree.add('file-1')
+        revid1 = tree.commit('rev1')
+        self.run_bzr(['switch', '-b', u'branch\xe9'], working_dir='branch-1')
+        bzrdir = BzrDir.open("branch-1")
+        self.assertEquals(
+            set([b.name for b in bzrdir.list_branches()]),
+            set(["foo", u"branch\xe9"]))
+        self.assertEquals(bzrdir.open_branch().name, u"branch\xe9")
+        self.assertEquals(bzrdir.open_branch().last_revision(), revid1)
+
     def test_switch_only_revision(self):
         tree = self._create_sample_tree()
         checkout = tree.branch.create_checkout('checkout', lightweight=True)
@@ -352,3 +430,24 @@
         # This test should be cleaner to write, but see bug:
         #  https://bugs.launchpad.net/bzr/+bug/812295
         self.assertEqual(1, opened.count('master'))
+
+
+class TestSmartServerSwitch(TestCaseWithTransport):
+
+    def test_switch_lightweight(self):
+        self.setup_smart_server_with_call_log()
+        t = self.make_branch_and_tree('from')
+        for count in range(9):
+            t.commit(message='commit %d' % count)
+        out, err = self.run_bzr(['checkout', '--lightweight', self.get_url('from'),
+            'target'])
+        self.reset_smart_call_log()
+        self.run_bzr(['switch', self.get_url('from')], working_dir='target')
+        # This figure represent the amount of work to perform this use case. It
+        # is entirely ok to reduce this number if a test fails due to rpc_count
+        # being too low. If rpc_count increases, more network roundtrips have
+        # become necessary for this use case. Please do not adjust this number
+        # upwards without agreement from bzr's network support maintainers.
+        self.assertLength(24, self.hpss_calls)
+        self.assertLength(5, self.hpss_connections)
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)

=== modified file 'bzrlib/tests/blackbox/test_tags.py'
--- bzrlib/tests/blackbox/test_tags.py	2011-06-10 15:06:41 +0000
+++ bzrlib/tests/blackbox/test_tags.py	2011-12-14 20:21:52 +0000
@@ -17,7 +17,7 @@
 """Tests for commands related to tags"""
 
 from bzrlib import (
-    branchbuilder,
+    errors,
     tag,
     transform,
     )
@@ -28,6 +28,7 @@
     script,
     TestCaseWithTransport,
     )
+from bzrlib.tests.matchers import ContainsNoVfsCalls
 from bzrlib.workingtree import WorkingTree
 
 
@@ -75,10 +76,25 @@
         # can also delete an existing tag
         out, err = self.run_bzr('tag --delete -d branch tag2')
         # cannot replace an existing tag normally
-        out, err = self.run_bzr('tag -d branch NEWTAG', retcode=3)
+        out, err = self.run_bzr('tag -d branch NEWTAG -r0', retcode=3)
         self.assertContainsRe(err, 'Tag NEWTAG already exists\\.')
         # ... but can if you use --force
-        out, err = self.run_bzr('tag -d branch NEWTAG --force')
+        out, err = self.run_bzr('tag -d branch NEWTAG --force -r0')
+        self.assertEquals("Updated tag NEWTAG.\n", err)
+
+    def test_tag_same_revision(self):
+        t = self.make_branch_and_tree('branch')
+        t.commit(allow_pointless=True, message='initial commit',
+            rev_id='first-revid')
+        t.commit(allow_pointless=True, message='second commit',
+            rev_id='second-revid')
+        out, err = self.run_bzr('tag -rrevid:first-revid -d branch NEWTAG')
+        out, err = self.run_bzr('tag -rrevid:first-revid -d branch NEWTAG')
+        self.assertContainsRe(err,
+            'Tag NEWTAG already exists for that revision\\.')
+        out, err = self.run_bzr('tag -rrevid:second-revid -d branch NEWTAG',
+            retcode=3)
+        self.assertContainsRe(err, 'Tag NEWTAG already exists\\.')
 
     def test_tag_delete_requires_name(self):
         out, err = self.run_bzr('tag -d branch', retcode=3)
@@ -254,6 +270,17 @@
         self.assertEquals(err, '')
         self.assertContainsRe(out, r'tagD  *3\n')
 
+    def test_list_tags_dotted_revnos_unsupported(self):
+        tree = self.make_branch_and_tree('branch')
+        rev1 = tree.commit("rev1")
+        tree.branch.tags.set_tag("mytag", rev1)
+        def revision_id_to_dotted_revno(self, revid):
+            raise errors.UnsupportedOperation(revision_id_to_dotted_revno, self)
+        self.overrideAttr(Branch, "revision_id_to_dotted_revno",
+            revision_id_to_dotted_revno)
+        out, err = self.run_bzr('tags -d branch', encoding='utf-8')
+        self.assertEquals(out, 'mytag                ?\n')
+
     def test_list_tags_revision_filtering(self):
         tree1 = self.make_branch_and_tree('.')
         tree1.commit(allow_pointless=True, message='revision 1',
@@ -380,3 +407,42 @@
             'tag2                 2\n'
             'unknown              ?\n')
         self.assertEqual('', err)
+
+
+class TestSmartServerCat(TestCaseWithTransport):
+
+    def test_set_tag(self):
+        self.setup_smart_server_with_call_log()
+        t = self.make_branch_and_tree('branch')
+        self.build_tree_contents([('branch/foo', 'thecontents')])
+        t.add("foo")
+        t.commit("message")
+        self.reset_smart_call_log()
+        out, err = self.run_bzr(['tag', "-d", self.get_url('branch'), "tagname"])
+        # This figure represent the amount of work to perform this use case. It
+        # is entirely ok to reduce this number if a test fails due to rpc_count
+        # being too low. If rpc_count increases, more network roundtrips have
+        # become necessary for this use case. Please do not adjust this number
+        # upwards without agreement from bzr's network support maintainers.
+        self.assertLength(9, self.hpss_calls)
+        self.assertLength(1, self.hpss_connections)
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)
+
+    def test_show_tags(self):
+        self.setup_smart_server_with_call_log()
+        t = self.make_branch_and_tree('branch')
+        self.build_tree_contents([('branch/foo', 'thecontents')])
+        t.add("foo")
+        t.commit("message")
+        t.branch.tags.set_tag("sometag", "rev1")
+        t.branch.tags.set_tag("sometag", "rev2")
+        self.reset_smart_call_log()
+        out, err = self.run_bzr(['tags', "-d", self.get_url('branch')])
+        # This figure represent the amount of work to perform this use case. It
+        # is entirely ok to reduce this number if a test fails due to rpc_count
+        # being too low. If rpc_count increases, more network roundtrips have
+        # become necessary for this use case. Please do not adjust this number
+        # upwards without agreement from bzr's network support maintainers.
+        self.assertLength(6, self.hpss_calls)
+        self.assertLength(1, self.hpss_connections)
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)

=== modified file 'bzrlib/tests/blackbox/test_too_much.py'
--- bzrlib/tests/blackbox/test_too_much.py	2011-05-18 10:03:10 +0000
+++ bzrlib/tests/blackbox/test_too_much.py	2011-09-26 15:21:01 +0000
@@ -564,7 +564,7 @@
         url = self.get_readonly_url('from')
         self.run_bzr(['branch', url, 'to'])
         branch = Branch.open('to')
-        self.assertEqual(1, len(branch.revision_history()))
+        self.assertEqual(1, branch.last_revision_info()[0])
         # the branch should be set in to to from
         self.assertEqual(url + '/', branch.get_parent())
 

=== modified file 'bzrlib/tests/blackbox/test_uncommit.py'
--- bzrlib/tests/blackbox/test_uncommit.py	2011-08-22 10:44:29 +0000
+++ bzrlib/tests/blackbox/test_uncommit.py	2011-12-14 20:21:52 +0000
@@ -22,6 +22,7 @@
 from bzrlib.bzrdir import BzrDirMetaFormat1
 from bzrlib.errors import BoundBranchOutOfDate
 from bzrlib.tests import TestCaseWithTransport
+from bzrlib.tests.matchers import ContainsNoVfsCalls
 from bzrlib.tests.script import (
     run_script,
     ScriptRunner,
@@ -72,7 +73,7 @@
         $ bzr uncommit
         ...
         The above revision(s) will be removed.
-        2>Uncommit these revisions? [y/n]: 
+        2>Uncommit these revisions? ([y]es, [n]o): no
         <n
         Canceled
         """)
@@ -119,8 +120,8 @@
         t_a.commit('commit 3')
         b = t_a.branch.create_checkout('b').branch
         uncommit.uncommit(b)
-        self.assertEqual(len(b.revision_history()), 2)
-        self.assertEqual(len(t_a.branch.revision_history()), 2)
+        self.assertEqual(b.last_revision_info()[0], 2)
+        self.assertEqual(t_a.branch.last_revision_info()[0], 2)
         # update A's tree to not have the uncommitted revision referenced.
         t_a.update()
         t_a.commit('commit 3b')
@@ -294,3 +295,22 @@
         tree.branch.tags.set_tag("atag", revid)
         out, err = self.run_bzr('uncommit --keep-tags --force tree')
         self.assertEquals({"atag": revid}, tree.branch.tags.get_tag_dict())
+
+
+class TestSmartServerUncommit(TestCaseWithTransport):
+
+    def test_uncommit(self):
+        self.setup_smart_server_with_call_log()
+        t = self.make_branch_and_tree('from')
+        for count in range(2):
+            t.commit(message='commit %d' % count)
+        self.reset_smart_call_log()
+        out, err = self.run_bzr(['uncommit', '--force', self.get_url('from')])
+        # This figure represent the amount of work to perform this use case. It
+        # is entirely ok to reduce this number if a test fails due to rpc_count
+        # being too low. If rpc_count increases, more network roundtrips have
+        # become necessary for this use case. Please do not adjust this number
+        # upwards without agreement from bzr's network support maintainers.
+        self.assertLength(14, self.hpss_calls)
+        self.assertLength(1, self.hpss_connections)
+        self.assertThat(self.hpss_calls, ContainsNoVfsCalls)

=== modified file 'bzrlib/tests/blackbox/test_update.py'
--- bzrlib/tests/blackbox/test_update.py	2011-08-16 14:07:34 +0000
+++ bzrlib/tests/blackbox/test_update.py	2011-09-29 01:03:18 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2010 Canonical Ltd
+# Copyright (C) 2006-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -454,3 +454,31 @@
 >>>>>>> MERGE-SOURCE
 ''',
                              'lightweight/file')
+
+
+    def test_no_upgrade_single_file(self):
+        """There's one basis revision per tree.
+
+        Since you can't actually change the basis for a single file at the
+        moment, we don't let you think you can.
+
+        See bug 557886.
+        """
+        self.make_branch_and_tree('.')
+        self.build_tree_contents([('a/',),
+            ('a/file', 'content')])
+        sr = ScriptRunner()
+        sr.run_script(self, '''
+            $ bzr update ./a
+            2>bzr: ERROR: bzr update can only update a whole tree, not a file or subdirectory
+            $ bzr update ./a/file
+            2>bzr: ERROR: bzr update can only update a whole tree, not a file or subdirectory
+            $ bzr update .
+            2>Tree is up to date at revision 0 of branch ...
+            $ cd a
+            $ bzr update .
+            2>bzr: ERROR: bzr update can only update a whole tree, not a file or subdirectory
+            # however, you can update the whole tree from a subdirectory
+            $ bzr update
+            2>Tree is up to date at revision 0 of branch ...
+            ''')

=== modified file 'bzrlib/tests/blackbox/test_upgrade.py'
--- bzrlib/tests/blackbox/test_upgrade.py	2011-08-19 19:36:39 +0000
+++ bzrlib/tests/blackbox/test_upgrade.py	2011-12-07 14:16:12 +0000
@@ -59,7 +59,8 @@
     def get_converter(self, format=None):
         return ConvertOldTestToMeta()
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         return "Ancient Test Format"
 
     def _open(self, transport):
@@ -89,12 +90,13 @@
     def test_upgrade_up_to_date(self):
         self.make_current_format_branch_and_checkout()
         # when up to date we should get a message to that effect
-        (out, err) = self.run_bzr('upgrade current_format_branch', retcode=3)
-        err_msg = ('The branch format %s is already at the most recent format.'
-                   % ('Meta directory format 1'))
-        self.assertEqualDiff('conversion error: %s\nbzr: ERROR: %s\n'
-                             % (err_msg, err_msg),
-                             err)
+        burl = self.get_transport('current_format_branch').local_abspath(".")
+        (out, err) = self.run_bzr('upgrade current_format_branch', retcode=0)
+        self.assertEqual(
+            'Upgrading branch %s/ ...\n'
+            'The branch format %s is already at the most recent format.\n'
+            % (burl, 'Meta directory format 1'),
+            out)
 
     def test_upgrade_up_to_date_checkout_warns_branch_left_alone(self):
         self.make_current_format_branch_and_checkout()
@@ -103,17 +105,13 @@
         # date
         burl = self.get_transport('current_format_branch').local_abspath(".")
         curl = self.get_transport('current_format_checkout').local_abspath(".")
-        (out, err) = self.run_bzr('upgrade current_format_checkout', retcode=3)
+        (out, err) = self.run_bzr('upgrade current_format_checkout', retcode=0)
         self.assertEqual(
             'Upgrading branch %s/ ...\nThis is a checkout.'
             ' The branch (%s/) needs to be upgraded separately.\n'
-            % (curl, burl),
+            'The branch format %s is already at the most recent format.\n'
+            % (curl, burl, 'Meta directory format 1'),
             out)
-        msg = 'The branch format %s is already at the most recent format.' % (
-            'Meta directory format 1')
-        self.assertEqualDiff('conversion error: %s\nbzr: ERROR: %s\n'
-                             % (msg, msg),
-                             err)
 
     def test_upgrade_checkout(self):
         # upgrading a checkout should work

=== modified file 'bzrlib/tests/blackbox/test_version_info.py'
--- bzrlib/tests/blackbox/test_version_info.py	2010-11-05 20:54:32 +0000
+++ bzrlib/tests/blackbox/test_version_info.py	2011-12-30 13:02:27 +0000
@@ -19,6 +19,7 @@
 import os
 
 from bzrlib.tests import TestCaseWithTransport
+from bzrlib.version_info_formats import VersionInfoBuilder
 
 
 class TestVersionInfo(TestCaseWithTransport):
@@ -36,29 +37,27 @@
         self.build_tree(['branch/b'])
         wt.add('b')
         wt.commit('adding b', rev_id='r2')
-
-        self.revisions = wt.branch.revision_history()
         return wt
 
     def test_basic(self):
-        self.create_tree()
+        wt = self.create_tree()
 
         txt = self.run_bzr('version-info branch')[0]
         self.assertContainsRe(txt, 'date:')
         self.assertContainsRe(txt, 'build-date:')
         self.assertContainsRe(txt, 'revno: 2')
-        self.assertContainsRe(txt, 'revision-id: ' + self.revisions[-1])
+        self.assertContainsRe(txt, 'revision-id: ' + wt.branch.last_revision())
 
     def test_all(self):
         """'--all' includes clean, revision history, and file revisions"""
-        self.create_tree()
+        wt = self.create_tree()
         txt = self.run_bzr('version-info branch --all')[0]
         self.assertContainsRe(txt, 'date:')
         self.assertContainsRe(txt, 'revno: 2')
-        self.assertContainsRe(txt, 'revision-id: ' + self.revisions[-1])
+        self.assertContainsRe(txt, 'revision-id: ' + wt.branch.last_revision())
         self.assertContainsRe(txt, 'clean: True')
         self.assertContainsRe(txt, 'revisions:')
-        for rev_id in self.revisions:
+        for rev_id in wt.branch.repository.all_revision_ids():
             self.assertContainsRe(txt, 'id: ' + rev_id)
         self.assertContainsRe(txt, 'message: adding a')
         self.assertContainsRe(txt, 'message: adding b')
@@ -151,19 +150,35 @@
             '"{revno} {branch_nick} {clean}\n" branch')
         self.assertEqual("2 branch 0\n", out)
         self.assertEqual("", err)
-    
+
+    def test_custom_no_clean_in_template(self):
+        def should_not_be_called(self):
+            raise AssertionError("Method on %r should not have been used" % (self,))
+        self.overrideAttr(VersionInfoBuilder, "_extract_file_revisions",
+                          should_not_be_called)
+        self.create_tree()
+        out, err = self.run_bzr('version-info --custom --template=r{revno} branch')
+        self.assertEqual("r2", out)
+        self.assertEqual("", err)
+
     def test_non_ascii(self):
         """Test that we can output non-ascii data"""
-        
+
         commit_message = u'Non-ascii message with character not in latin-1: \u1234'
-        
+
         tree = self.make_branch_and_tree('.')
         self.build_tree(['a_file'])
         tree.add('a_file')
         tree.commit(commit_message)
         out, err = self.run_bzr(
             ['version-info', '--include-history'], encoding='latin-1')
-        
+
         self.assertContainsString(out, commit_message.encode('utf-8'))
 
+    def test_revision(self):
+        tree = self.create_tree()
+        branch = self.make_branch('just_branch')
+        branch.pull(tree.branch)
 
+        txt = self.run_bzr('version-info -r1 just_branch')[0]
+        self.assertStartsWith(txt, 'revision-id: r1\n')

=== modified file 'bzrlib/tests/blackbox/test_versioning.py'
--- bzrlib/tests/blackbox/test_versioning.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/blackbox/test_versioning.py	2011-11-24 17:34:23 +0000
@@ -41,13 +41,10 @@
         self.run_bzr(['mkdir', 'abc'], retcode=3)
         self.assertPathDoesNotExist('abc')
 
-
-class TestVersioning(TestCaseInTempDir):
-
     def test_mkdir(self):
         """Basic 'bzr mkdir' operation"""
 
-        self.run_bzr('init')
+        self.make_branch_and_tree('.')
         self.run_bzr(['mkdir', 'foo'])
         self.assert_(os.path.isdir('foo'))
 
@@ -66,7 +63,7 @@
     def test_mkdir_in_subdir(self):
         """'bzr mkdir' operation in subdirectory"""
 
-        self.run_bzr('init')
+        self.make_branch_and_tree('.')
         self.run_bzr(['mkdir', 'dir'])
         self.assert_(os.path.isdir('dir'))
 
@@ -90,14 +87,9 @@
     def test_mkdir_w_nested_trees(self):
         """'bzr mkdir' with nested trees"""
 
-        self.run_bzr('init')
-        os.mkdir('a')
-        os.chdir('a')
-        self.run_bzr('init')
-        os.mkdir('b')
-        os.chdir('b')
-        self.run_bzr('init')
-        os.chdir('../..')
+        self.make_branch_and_tree('.')
+        self.make_branch_and_tree('a')
+        self.make_branch_and_tree('a/b')
 
         self.run_bzr(['mkdir', 'dir', 'a/dir', 'a/b/dir'])
         self.assertTrue(os.path.isdir('dir'))
@@ -123,16 +115,13 @@
         self.assertEquals(delta.added[0][0], 'dir')
         self.assertFalse(delta.modified)
 
-    def check_branch(self):
-        """After all the above changes, run the check and upgrade commands.
-
-        The upgrade should be a no-op."""
-        b = Branch.open(u'.')
-        mutter('branch has %d revisions', b.revno())
-
-        mutter('check branch...')
-        from bzrlib.check import check
-        check(b, False)
+    def test_mkdir_quiet(self):
+        """'bzr mkdir --quiet' should not print a status message"""
+
+        self.make_branch_and_tree('.')
+        out, err = self.run_bzr(['mkdir', '--quiet', 'foo'])
+        self.assertEquals('', err)
+        self.assertEquals('', out)
 
 
 class SubdirCommit(TestCaseWithTransport):
@@ -159,9 +148,12 @@
         new = b.repository.revision_tree(b.get_rev_id(2))
         new.lock_read()
 
-        self.assertEqual(new.get_file_by_path('b/two').read(), 'old contents')
-        self.assertEqual(new.get_file_by_path('top').read(), 'old contents')
-        self.assertEqual(new.get_file_by_path('a/one').read(), 'new contents')
+        def get_text_by_path(tree, path):
+            return tree.get_file_text(tree.path2id(path), path)
+
+        self.assertEqual(get_text_by_path(new, 'b/two'), 'old contents')
+        self.assertEqual(get_text_by_path(new, 'top'), 'old contents')
+        self.assertEqual(get_text_by_path(new, 'a/one'), 'new contents')
         new.unlock()
 
         os.chdir('a')
@@ -169,17 +161,17 @@
         self.run_bzr(['commit', '.', '-m', 'commit subdir only', '--unchanged'])
         v3 = b.repository.revision_tree(b.get_rev_id(3))
         v3.lock_read()
-        self.assertEqual(v3.get_file_by_path('b/two').read(), 'old contents')
-        self.assertEqual(v3.get_file_by_path('top').read(), 'old contents')
-        self.assertEqual(v3.get_file_by_path('a/one').read(), 'new contents')
+        self.assertEqual(get_text_by_path(v3, 'b/two'), 'old contents')
+        self.assertEqual(get_text_by_path(v3, 'top'), 'old contents')
+        self.assertEqual(get_text_by_path(v3, 'a/one'), 'new contents')
         v3.unlock()
 
         # commit in subdirectory commits whole tree
         self.run_bzr(['commit', '-m', 'commit whole tree from subdir'])
         v4 = b.repository.revision_tree(b.get_rev_id(4))
         v4.lock_read()
-        self.assertEqual(v4.get_file_by_path('b/two').read(), 'new contents')
-        self.assertEqual(v4.get_file_by_path('top').read(), 'new contents')
+        self.assertEqual(get_text_by_path(v4, 'b/two'), 'new contents')
+        self.assertEqual(get_text_by_path(v4, 'top'), 'new contents')
         v4.unlock()
 
         # TODO: factor out some kind of assert_tree_state() method

=== modified file 'bzrlib/tests/features.py'
--- bzrlib/tests/features.py	2011-08-23 09:29:27 +0000
+++ bzrlib/tests/features.py	2011-12-05 14:21:55 +0000
@@ -26,7 +26,6 @@
 from bzrlib import (
     osutils,
     symbol_versioning,
-    tests,
     )
 
 
@@ -141,7 +140,8 @@
                                             % (self._module, self._name))
             use_msg = ' Use %s.%s instead.' % (self._replacement_module,
                                                self._replacement_name)
-            symbol_versioning.warn(depr_msg + use_msg, DeprecationWarning)
+            symbol_versioning.warn(depr_msg + use_msg, DeprecationWarning,
+                                   stacklevel=5)
             # Import the new feature and use it as a replacement for the
             # deprecated one.
             self._feature = pyutils.get_named_object(
@@ -290,6 +290,8 @@
         if CaseInsCasePresFilenameFeature.available():
             return False
 
+        from bzrlib import tests
+
         if tests.TestCaseWithMemoryTransport.TEST_ROOT is None:
             root = osutils.mkdtemp(prefix='testbzr-', suffix='.tmp')
             tests.TestCaseWithMemoryTransport.TEST_ROOT = root
@@ -407,8 +409,9 @@
 
 
 bash_feature = ExecutableFeature('bash')
+diff_feature = ExecutableFeature('diff')
 sed_feature = ExecutableFeature('sed')
-diff_feature = ExecutableFeature('diff')
+msgmerge_feature = ExecutableFeature('msgmerge')
 
 
 class _PosixPermissionsFeature(Feature):
@@ -421,7 +424,7 @@
             f = tempfile.mkstemp(prefix='bzr_perms_chk_')
             fd, name = f
             os.close(fd)
-            os.chmod(name, write_perms)
+            osutils.chmod_if_possible(name, write_perms)
 
             read_perms = os.stat(name).st_mode & 0777
             os.unlink(name)
@@ -490,26 +493,3 @@
 
 
 win32_feature = Win32Feature()
-
-
-for name in ['HTTPServerFeature', 
-    'HTTPSServerFeature', 'SymlinkFeature', 'HardlinkFeature',
-    'OsFifoFeature', 'UnicodeFilenameFeature',
-    'ByteStringNamedFilesystem', 'UTF8Filesystem',
-    'BreakinFeature', 'CaseInsCasePresFilenameFeature',
-    'CaseInsensitiveFilesystemFeature', 'case_sensitive_filesystem_feature',
-    'posix_permissions_feature',
-    ]:
-    setattr(tests, name, _CompatabilityThunkFeature(
-        symbol_versioning.deprecated_in((2, 5, 0)),
-        'bzrlib.tests', name,
-        name, 'bzrlib.tests.features'))
-
-
-for (old_name, new_name) in [
-    ('UnicodeFilename', 'UnicodeFilenameFeature'),
-    ]:
-    setattr(tests, name, _CompatabilityThunkFeature(
-        symbol_versioning.deprecated_in((2, 5, 0)),
-        'bzrlib.tests', old_name,
-        new_name, 'bzrlib.tests.features'))

=== modified file 'bzrlib/tests/fixtures.py'
--- bzrlib/tests/fixtures.py	2011-07-04 20:17:44 +0000
+++ bzrlib/tests/fixtures.py	2011-11-29 00:50:36 +0000
@@ -138,3 +138,23 @@
     testcase.build_tree_contents([('t/hello', 'hello world')])
     tree.add(['hello'], ['hello-id'])
     return tree
+
+
+class TimeoutFixture(object):
+    """Kill a test with sigalarm if it runs too long.
+    
+    Only works on Unix at present.
+    """
+
+    def __init__(self, timeout_secs):
+        import signal
+        self.timeout_secs = timeout_secs
+        self.alarm_fn = getattr(signal, 'alarm', None)
+
+    def setUp(self):
+        if self.alarm_fn is not None:
+            self.alarm_fn(self.timeout_secs)
+
+    def cleanUp(self):
+        if self.alarm_fn is not None:
+            self.alarm_fn(0)

=== modified file 'bzrlib/tests/http_server.py'
--- bzrlib/tests/http_server.py	2011-03-08 16:00:55 +0000
+++ bzrlib/tests/http_server.py	2011-12-18 15:43:47 +0000
@@ -22,9 +22,9 @@
 import re
 import SimpleHTTPServer
 import socket
-import urllib
 import urlparse
 
+from bzrlib import urlutils
 from bzrlib.tests import test_server
 
 
@@ -336,7 +336,7 @@
         """
         # abandon query parameters
         path = urlparse.urlparse(path)[2]
-        path = posixpath.normpath(urllib.unquote(path))
+        path = posixpath.normpath(urlutils.unquote(path))
         path = path.decode('utf-8')
         words = path.split('/')
         words = filter(None, words)

=== modified file 'bzrlib/tests/matchers.py'
--- bzrlib/tests/matchers.py	2011-08-16 13:26:27 +0000
+++ bzrlib/tests/matchers.py	2011-12-14 12:15:44 +0000
@@ -29,12 +29,20 @@
 __all__ = [
     'HasLayout',
     'MatchesAncestry',
+    'ContainsNoVfsCalls',
     'ReturnsUnlockable',
     ]
 
 from bzrlib import (
+    osutils,
     revision as _mod_revision,
     )
+from bzrlib import lazy_import
+lazy_import.lazy_import(globals(),
+"""
+from bzrlib.smart.request import request_handlers as smart_request_handlers
+from bzrlib.smart import vfs
+""")
 
 from testtools.matchers import Equals, Mismatch, Matcher
 
@@ -54,7 +62,7 @@
         self.lockable_thing = lockable_thing
 
     def __str__(self):
-        return ('ReturnsUnlockable(lockable_thing=%s)' % 
+        return ('ReturnsUnlockable(lockable_thing=%s)' %
             self.lockable_thing)
 
     def match(self, lock_method):
@@ -131,16 +139,80 @@
         """Get the (path, file_id) pairs for the current tree."""
         tree.lock_read()
         try:
-            return [(path, ie.file_id) for path, ie
-                    in tree.iter_entries_by_dir()]
+            for path, ie in tree.iter_entries_by_dir():
+                if ie.parent_id is None:
+                    yield (u"", ie.file_id)
+                else:
+                    yield (path+ie.kind_character(), ie.file_id)
         finally:
             tree.unlock()
 
+    @staticmethod
+    def _strip_unreferenced_directories(entries):
+        """Strip all directories that don't (in)directly contain any files.
+
+        :param entries: List of path strings or (path, ie) tuples to process
+        """
+        directories = []
+        for entry in entries:
+            if isinstance(entry, basestring):
+                path = entry
+            else:
+                path = entry[0]
+            if not path or path[-1] == "/":
+                # directory
+                directories.append((path, entry))
+            else:
+                # Yield the referenced parent directories
+                for dirpath, direntry in directories:
+                    if osutils.is_inside(dirpath, path):
+                        yield direntry
+                directories = []
+                yield entry
+
     def __str__(self):
-        return ('HasLayout(%r)' % self.entries)
+        return 'HasLayout(%r)' % self.entries
 
     def match(self, tree):
-        actual = self.get_tree_layout(tree)
+        actual = list(self.get_tree_layout(tree))
         if self.entries and isinstance(self.entries[0], basestring):
             actual = [path for (path, fileid) in actual]
-        return Equals(actual).match(self.entries)
+        if not tree.has_versioned_directories():
+            entries = list(self._strip_unreferenced_directories(self.entries))
+        else:
+            entries = self.entries
+        return Equals(entries).match(actual)
+
+
+class _NoVfsCallsMismatch(Mismatch):
+    """Mismatch describing a list of HPSS calls which includes VFS requests."""
+
+    def __init__(self, vfs_calls):
+        self.vfs_calls = vfs_calls
+
+    def describe(self):
+        return "no VFS calls expected, got: %s" % ",".join([
+            "%s(%s)" % (c.method,
+                ", ".join([repr(a) for a in c.args])) for c in self.vfs_calls])
+
+
+class ContainsNoVfsCalls(Matcher):
+    """Ensure that none of the specified calls are HPSS calls."""
+
+    def __str__(self):
+        return 'ContainsNoVfsCalls()'
+
+    @classmethod
+    def match(cls, hpss_calls):
+        vfs_calls = []
+        for call in hpss_calls:
+            try:
+                request_method = smart_request_handlers.get(call.call.method)
+            except KeyError:
+                # A method we don't know about doesn't count as a VFS method.
+                continue
+            if issubclass(request_method, vfs.VfsRequest):
+                vfs_calls.append(call.call)
+        if len(vfs_calls) == 0:
+            return None
+        return _NoVfsCallsMismatch(vfs_calls)

=== modified file 'bzrlib/tests/per_branch/__init__.py'
--- bzrlib/tests/per_branch/__init__.py	2011-05-18 16:42:48 +0000
+++ bzrlib/tests/per_branch/__init__.py	2011-10-05 12:51:41 +0000
@@ -67,29 +67,19 @@
 
     def get_branch(self):
         if self.branch is None:
-            self.branch = self.make_branch('')
+            self.branch = self.make_branch('abranch')
         return self.branch
 
+    def get_default_format(self):
+        format = self.bzrdir_format
+        self.assertEquals(format.get_branch_format(), self.branch_format)
+        return format
+
     def make_branch(self, relpath, format=None):
-        if format is not None:
-            return TestCaseWithControlDir.make_branch(self, relpath, format)
-        repo = self.make_repository(relpath)
-        # fixme RBC 20060210 this isnt necessarily a fixable thing,
-        # Skipped is the wrong exception to raise.
         try:
-            return self.branch_format.initialize(repo.bzrdir)
+            return super(TestCaseWithBranch, self).make_branch(relpath, format)
         except errors.UninitializableFormat:
-            raise tests.TestSkipped('Uninitializable branch format')
-
-    def make_branch_builder(self, relpath, format=None):
-        if format is None:
-            format = self.branch_format._matchingbzrdir
-        return super(TestCaseWithBranch, self).make_branch_builder(
-            relpath, format=format)
-
-    def make_repository(self, relpath, shared=False, format=None):
-        made_control = self.make_bzrdir(relpath, format=format)
-        return made_control.create_repository(shared=shared)
+            raise tests.TestNotApplicable('Uninitializable branch format')
 
     def create_tree_with_merge(self):
         """Create a branch with a simple ancestry.

=== modified file 'bzrlib/tests/per_branch/test_branch.py'
--- bzrlib/tests/per_branch/test_branch.py	2011-08-27 00:01:40 +0000
+++ bzrlib/tests/per_branch/test_branch.py	2011-10-15 00:10:14 +0000
@@ -24,6 +24,7 @@
     errors,
     gpg,
     merge,
+    osutils,
     urlutils,
     transport,
     remote,
@@ -76,9 +77,7 @@
         br = self.get_branch()
         br.fetch(wt.branch)
         br.generate_revision_history('rev3')
-        rh = br.revision_history()
-        self.assertEqual(['rev1', 'rev2', 'rev3'], rh)
-        for revision_id in rh:
+        for revision_id in ['rev3', 'rev2', 'rev1']:
             self.assertIsInstance(revision_id, str)
         last = br.last_revision()
         self.assertEqual('rev3', last)
@@ -114,10 +113,12 @@
         tree_a.add('vla', 'file2')
         tree_a.commit('rev2', rev_id='rev2')
 
-        delta = tree_a.branch.get_revision_delta(1)
+        delta = self.applyDeprecated(symbol_versioning.deprecated_in(
+            (2, 5, 0)), tree_a.branch.get_revision_delta, 1)
         self.assertIsInstance(delta, _mod_delta.TreeDelta)
         self.assertEqual([('foo', 'file1', 'file')], delta.added)
-        delta = tree_a.branch.get_revision_delta(2)
+        delta = self.applyDeprecated(symbol_versioning.deprecated_in(
+            (2, 5, 0)), tree_a.branch.get_revision_delta, 2)
         self.assertIsInstance(delta, _mod_delta.TreeDelta)
         self.assertEqual([('vla', 'file2', 'file')], delta.added)
 
@@ -215,6 +216,9 @@
     def test_record_initial_ghost(self):
         """Branches should support having ghosts."""
         wt = self.make_branch_and_tree('.')
+        if not wt.branch.repository._format.supports_ghosts:
+            raise tests.TestNotApplicable("repository format does not "
+                "support ghosts")
         wt.set_parent_ids(['non:existent@rev--ision--0--2'],
             allow_leftmost_as_ghost=True)
         self.assertEqual(['non:existent@rev--ision--0--2'],
@@ -228,6 +232,9 @@
     def test_record_two_ghosts(self):
         """Recording with all ghosts works."""
         wt = self.make_branch_and_tree('.')
+        if not wt.branch.repository._format.supports_ghosts:
+            raise tests.TestNotApplicable("repository format does not "
+                "support ghosts")
         wt.set_parent_ids([
                 'foo@azkhazan-123123-abcabc',
                 'wibble@fofof--20050401--1928390812',
@@ -245,78 +252,18 @@
                           self.get_branch().repository.get_revision,
                           None)
 
-# TODO 20051003 RBC:
-# compare the gpg-to-sign info for a commit with a ghost and
-#     an identical tree without a ghost
-# fetch missing should rewrite the TOC of weaves to list newly available parents.
-
-    def test_sign_existing_revision(self):
-        wt = self.make_branch_and_tree('.')
-        branch = wt.branch
-        wt.commit("base", allow_pointless=True, rev_id='A')
-        from bzrlib.testament import Testament
-        strategy = gpg.LoopbackGPGStrategy(None)
-        branch.repository.lock_write()
-        branch.repository.start_write_group()
-        branch.repository.sign_revision('A', strategy)
-        branch.repository.commit_write_group()
-        branch.repository.unlock()
-        self.assertEqual('-----BEGIN PSEUDO-SIGNED CONTENT-----\n' +
-                         Testament.from_revision(branch.repository,
-                         'A').as_short_text() +
-                         '-----END PSEUDO-SIGNED CONTENT-----\n',
-                         branch.repository.get_signature_text('A'))
-
-    def test_store_signature(self):
-        wt = self.make_branch_and_tree('.')
-        branch = wt.branch
-        branch.lock_write()
-        try:
-            branch.repository.start_write_group()
-            try:
-                branch.repository.store_revision_signature(
-                    gpg.LoopbackGPGStrategy(None), 'FOO', 'A')
-            except:
-                branch.repository.abort_write_group()
-                raise
-            else:
-                branch.repository.commit_write_group()
-        finally:
-            branch.unlock()
-        # A signature without a revision should not be accessible.
-        self.assertRaises(errors.NoSuchRevision,
-                          branch.repository.has_signature_for_revision_id,
-                          'A')
-        wt.commit("base", allow_pointless=True, rev_id='A')
-        self.assertEqual('-----BEGIN PSEUDO-SIGNED CONTENT-----\n'
-                         'FOO-----END PSEUDO-SIGNED CONTENT-----\n',
-                         branch.repository.get_signature_text('A'))
-
-    def test_branch_keeps_signatures(self):
-        wt = self.make_branch_and_tree('source')
-        wt.commit('A', allow_pointless=True, rev_id='A')
-        repo = wt.branch.repository
-        repo.lock_write()
-        repo.start_write_group()
-        repo.sign_revision('A', gpg.LoopbackGPGStrategy(None))
-        repo.commit_write_group()
-        repo.unlock()
-        #FIXME: clone should work to urls,
-        # wt.clone should work to disks.
-        self.build_tree(['target/'])
-        d2 = repo.bzrdir.clone(urlutils.local_path_to_url('target'))
-        self.assertEqual(repo.get_signature_text('A'),
-                         d2.open_repository().get_signature_text('A'))
-
-    def test_nicks(self):
-        """Test explicit and implicit branch nicknames.
+    def test_nicks_bzr(self):
+        """Test the behaviour of branch nicks specific to bzr branches.
 
         Nicknames are implicitly the name of the branch's directory, unless an
         explicit nickname is set.  That is, an explicit nickname always
         overrides the implicit one.
+
         """
         t = self.get_transport()
         branch = self.make_branch('bzr.dev')
+        if not isinstance(branch, _mod_branch.BzrBranch):
+            raise tests.TestNotApplicable("not a bzr branch format")
         # The nick will be 'bzr.dev', because there is no explicit nick set.
         self.assertEqual(branch.nick, 'bzr.dev')
         # Move the branch to a different directory, 'bzr.ab'.  Now that branch
@@ -338,6 +285,24 @@
         branch.nick = u"\u1234"
         self.assertEqual(branch.nick, u"\u1234")
 
+    def test_nicks(self):
+        """Test explicit and implicit branch nicknames.
+
+        A nickname is always available, whether set explicitly or not.
+        """
+        t = self.get_transport()
+        branch = self.make_branch('bzr.dev')
+        # An implicit nick name is set; what it is exactly depends on the
+        # format.
+        self.assertIsInstance(branch.nick, basestring)
+        # Set the branch nick explicitly.
+        branch.nick = "Aaron's branch"
+        # Because the nick has been set explicitly, the nick is now always
+        # "Aaron's branch".
+        self.assertEqual(branch.nick, "Aaron's branch")
+        branch.nick = u"\u1234"
+        self.assertEqual(branch.nick, u"\u1234")
+
     def test_commit_nicks(self):
         """Nicknames are committed to the revision"""
         wt = self.make_branch_and_tree('bzr.dev')
@@ -365,14 +330,45 @@
         self.branch_format.initialize(repo.bzrdir, name='branch2')
         self.assertEquals(2, len(repo.bzrdir.list_branches()))
 
+    def test_create_append_revisions_only(self):
+        try:
+            repo = self.make_repository('.', shared=True)
+        except errors.IncompatibleFormat:
+            return
+        for val in (True, False):
+            try:
+                branch = self.branch_format.initialize(repo.bzrdir,
+                    append_revisions_only=True)
+            except (errors.UninitializableFormat, errors.UpgradeRequired):
+                # branch references are not default init'able and
+                # not all branches support append_revisions_only
+                return
+            self.assertEquals(True, branch.get_append_revisions_only())
+            repo.bzrdir.destroy_branch()
+
+    def test_get_set_append_revisions_only(self):
+        branch = self.make_branch('.')
+        if branch._format.supports_set_append_revisions_only():
+            branch.set_append_revisions_only(True)
+            self.assertTrue(branch.get_append_revisions_only())
+            branch.set_append_revisions_only(False)
+            self.assertFalse(branch.get_append_revisions_only())
+        else:
+            self.assertRaises(errors.UpgradeRequired,
+                branch.set_append_revisions_only, True)
+            self.assertFalse(branch.get_append_revisions_only())
+
     def test_create_open_branch_uses_repository(self):
         try:
             repo = self.make_repository('.', shared=True)
         except errors.IncompatibleFormat:
-            return
+            raise tests.TestNotApplicable("requires shared repository support")
         child_transport = repo.bzrdir.root_transport.clone('child')
         child_transport.mkdir('.')
-        child_dir = self.bzrdir_format.initialize_on_transport(child_transport)
+        try:
+            child_dir = self.bzrdir_format.initialize_on_transport(child_transport)
+        except errors.UninitializableFormat:
+            raise tests.TestNotApplicable("control dir format not initializable")
         try:
             child_branch = self.branch_format.initialize(child_dir)
         except errors.UninitializableFormat:
@@ -401,16 +397,25 @@
         """Create a fake revision history easily."""
         tree = self.make_branch_and_tree('.')
         rev1 = tree.commit('foo')
-        orig_history = tree.branch.revision_history()
+        tree.lock_write()
+        self.addCleanup(tree.unlock)
+        graph = tree.branch.repository.get_graph()
+        orig_history = list(
+            graph.iter_lefthand_ancestry(
+                tree.branch.last_revision(), [revision.NULL_REVISION]))
         rev2 = tree.commit('bar', allow_pointless=True)
         tree.branch.generate_revision_history(rev1)
-        self.assertEqual(orig_history, tree.branch.revision_history())
+        self.assertEqual(orig_history, list(
+            graph.iter_lefthand_ancestry(
+                tree.branch.last_revision(), [revision.NULL_REVISION])))
 
     def test_generate_revision_history_NULL_REVISION(self):
         tree = self.make_branch_and_tree('.')
         rev1 = tree.commit('foo')
+        tree.lock_write()
+        self.addCleanup(tree.unlock)
         tree.branch.generate_revision_history(revision.NULL_REVISION)
-        self.assertEqual([], tree.branch.revision_history())
+        self.assertEqual(revision.NULL_REVISION, tree.branch.last_revision())
 
     def test_create_checkout(self):
         tree_a = self.make_branch_and_tree('a')
@@ -437,7 +442,11 @@
         tree_a = self.make_branch_and_tree('a')
         rev_id = tree_a.commit('put some content in the branch')
         # open the branch via a readonly transport
-        source_branch = _mod_branch.Branch.open(self.get_readonly_url('a'))
+        url = self.get_readonly_url(urlutils.basename(tree_a.branch.base))
+        t = transport.get_transport_from_url(url)
+        if not tree_a.branch.bzrdir._format.supports_transport(t):
+            raise tests.TestNotApplicable("format does not support transport")
+        source_branch = _mod_branch.Branch.open(url)
         # sanity check that the test will be valid
         self.assertRaises((errors.LockError, errors.TransportNotPossible),
             source_branch.lock_write)
@@ -449,7 +458,12 @@
         tree_a = self.make_branch_and_tree('a')
         rev_id = tree_a.commit('put some content in the branch')
         # open the branch via a readonly transport
-        source_branch = _mod_branch.Branch.open(self.get_readonly_url('a'))
+        url = self.get_readonly_url(
+            osutils.basename(tree_a.branch.base.rstrip('/')))
+        t = transport.get_transport_from_url(url)
+        if not tree_a.branch.bzrdir._format.supports_transport(t):
+            raise tests.TestNotApplicable("format does not support transport")
+        source_branch = _mod_branch.Branch.open(url)
         # sanity check that the test will be valid
         self.assertRaises((errors.LockError, errors.TransportNotPossible),
             source_branch.lock_write)
@@ -462,10 +476,10 @@
         br = tree.branch
         self.applyDeprecated(symbol_versioning.deprecated_in((2, 4, 0)),
             br.set_revision_history, ["rev1"])
-        self.assertEquals(br.revision_history(), ["rev1"])
+        self.assertEquals(br.last_revision(), "rev1")
         self.applyDeprecated(symbol_versioning.deprecated_in((2, 4, 0)),
             br.set_revision_history, [])
-        self.assertEquals(br.revision_history(), [])
+        self.assertEquals(br.last_revision(), 'null:')
 
     def test_heads_to_fetch(self):
         # heads_to_fetch is a method that returns a collection of revids that
@@ -509,6 +523,12 @@
             looked_up_format = registry.get(network_name)
             self.assertEqual(format.__class__, looked_up_format.__class__)
 
+    def get_get_config_calls(self):
+        # Smoke test that all branch succeed getting a config
+        br = self.make_branch('.')
+        br.get_config()
+        br.get_config_stack()
+
 
 class ChrootedTests(per_branch.TestCaseWithBranch):
     """A support class that provides readonly urls outside the local namespace.
@@ -531,6 +551,9 @@
                           _mod_branch.Branch.open_containing,
                           self.get_readonly_url('g/p/q'))
         branch = self.make_branch('.')
+        if not branch.bzrdir._format.supports_transport(
+            transport.get_transport_from_url(self.get_readonly_url('.'))):
+            raise tests.TestNotApplicable("format does not support transport")
         branch, relpath = _mod_branch.Branch.open_containing(
             self.get_readonly_url(''))
         self.assertEqual('', relpath)
@@ -837,12 +860,14 @@
     def test_fallbacks_not_opened(self):
         stacked = self.make_branch_with_fallback()
         self.get_transport('').rename('fallback', 'moved')
-        reopened = stacked.bzrdir.open_branch(ignore_fallbacks=True)
+        reopened_dir = bzrdir.BzrDir.open(stacked.base)
+        reopened = reopened_dir.open_branch(ignore_fallbacks=True)
         self.assertEqual([], reopened.repository._fallback_repositories)
 
     def test_fallbacks_are_opened(self):
         stacked = self.make_branch_with_fallback()
-        reopened = stacked.bzrdir.open_branch(ignore_fallbacks=False)
+        reopened_dir = bzrdir.BzrDir.open(stacked.base)
+        reopened = reopened_dir.open_branch(ignore_fallbacks=False)
         self.assertLength(1, reopened.repository._fallback_repositories)
 
 
@@ -856,8 +881,9 @@
             tree.add_reference(subtree)
         except errors.UnsupportedOperation:
             raise tests.TestNotApplicable('Tree cannot hold references.')
-        reference_parent = tree.branch.reference_parent('subtree-id',
-                                                        'subtree')
+        reference_parent = tree.branch.reference_parent(
+            'subtree-id',
+            urlutils.relative_url(tree.branch.user_url, subtree.branch.user_url))
         self.assertEqual(subtree.branch.base, reference_parent.base)
 
     def test_reference_parent_accepts_possible_transports(self):
@@ -869,7 +895,9 @@
         except errors.UnsupportedOperation:
             raise tests.TestNotApplicable('Tree cannot hold references.')
         reference_parent = tree.branch.reference_parent('subtree-id',
-            'subtree', possible_transports=[subtree.bzrdir.root_transport])
+            urlutils.relative_url(
+                tree.branch.user_url, subtree.branch.user_url),
+            possible_transports=[subtree.bzrdir.root_transport])
 
     def test_get_reference_info(self):
         branch = self.make_branch('branch')
@@ -1042,7 +1070,7 @@
 
 class TestBranchControlComponent(per_branch.TestCaseWithBranch):
     """Branch implementations adequately implement ControlComponent."""
-    
+
     def test_urls(self):
         br = self.make_branch('branch')
         self.assertIsInstance(br.user_url, str)

=== modified file 'bzrlib/tests/per_branch/test_break_lock.py'
--- bzrlib/tests/per_branch/test_break_lock.py	2010-05-06 23:41:35 +0000
+++ bzrlib/tests/per_branch/test_break_lock.py	2011-09-30 13:35:45 +0000
@@ -16,7 +16,8 @@
 
 """Tests for branch break-lock behaviour."""
 
-from  bzrlib import (
+from bzrlib import (
+    branch as _mod_branch,
     errors,
     ui,
     tests,
@@ -29,7 +30,7 @@
     def setUp(self):
         super(TestBreakLock, self).setUp()
         self.unused_branch = self.make_branch('branch')
-        self.branch = self.unused_branch.bzrdir.open_branch()
+        self.branch = _mod_branch.Branch.open(self.unused_branch.base)
 
     def test_unlocked(self):
         # break lock when nothing is locked should just return

=== modified file 'bzrlib/tests/per_branch/test_create_checkout.py'
--- bzrlib/tests/per_branch/test_create_checkout.py	2011-08-27 00:00:54 +0000
+++ bzrlib/tests/per_branch/test_create_checkout.py	2011-12-14 20:08:26 +0000
@@ -21,15 +21,25 @@
 
 class TestCreateCheckout(per_branch.TestCaseWithBranch):
 
-    def test_checkout_format(self):
-        """Make sure the new checkout uses the desired branch format."""
-        a_branch = self.make_branch('branch')
-        tree = a_branch.create_checkout('checkout')
-        # All branches can define the format they want checkouts made in.
-        # This checks it is honoured.
-        expected_format = a_branch._get_checkout_format().get_branch_format()
-        self.assertEqual(expected_format.__class__,
-                         tree.branch._format.__class__)
+    def test_checkout_format_lightweight(self):
+        """Make sure the new light checkout uses the desired branch format."""
+        a_branch = self.make_branch('branch')
+        tree = a_branch.create_checkout('checkout', lightweight=True)
+        # All branches can define the format they want checkouts made in.
+        # This checks it is honoured.
+        expected_format = a_branch._get_checkout_format(lightweight=True)
+        self.assertEqual(expected_format.get_branch_format().network_name(),
+                         tree.branch._format.network_name())
+
+    def test_checkout_format_heavyweight(self):
+        """Make sure the new heavy checkout uses the desired branch format."""
+        a_branch = self.make_branch('branch')
+        tree = a_branch.create_checkout('checkout', lightweight=False)
+        # All branches can define the format they want checkouts made in.
+        # This checks it is honoured.
+        expected_format = a_branch._get_checkout_format(lightweight=False)
+        self.assertEqual(expected_format.get_branch_format().network_name(),
+                         tree.branch._format.network_name())
 
     def test_create_revision_checkout(self):
         """Test that we can create a checkout from an earlier revision."""

=== modified file 'bzrlib/tests/per_branch/test_create_clone.py'
--- bzrlib/tests/per_branch/test_create_clone.py	2010-02-10 17:52:08 +0000
+++ bzrlib/tests/per_branch/test_create_clone.py	2011-10-04 13:29:25 +0000
@@ -96,8 +96,13 @@
         revid = tree.commit('a second commit')
         source = tree.branch
         target_transport = self.get_transport('target')
-        result = tree.branch.create_clone_on_transport(target_transport,
-            stacked_on=trunk.base)
+        try:
+            result = tree.branch.create_clone_on_transport(target_transport,
+                stacked_on=trunk.base)
+        except errors.UnstackableBranchFormat:
+            if not trunk.repository._format.supports_full_versioned_files:
+                raise tests.TestNotApplicable("can not stack on format")
+            raise
         self.assertEqual(revid, result.last_revision())
         self.assertEqual(trunk.base, result.get_stacked_on_url())
 
@@ -133,8 +138,13 @@
         self.hook_calls = []
         branch.Branch.hooks.install_named_hook(
             'pre_change_branch_tip', self.assertBranchHookBranchIsStacked, None)
-        result = tree.branch.create_clone_on_transport(target_transport,
-            stacked_on=trunk.base)
+        try:
+            result = tree.branch.create_clone_on_transport(target_transport,
+                stacked_on=trunk.base)
+        except errors.UnstackableBranchFormat:
+            if not trunk.repository._format.supports_full_versioned_files:
+                raise tests.TestNotApplicable("can not stack on format")
+            raise
         self.assertEqual(revid, result.last_revision())
         self.assertEqual(trunk.base, result.get_stacked_on_url())
         # Smart servers invoke hooks on both sides

=== modified file 'bzrlib/tests/per_branch/test_locking.py'
--- bzrlib/tests/per_branch/test_locking.py	2011-01-13 06:18:13 +0000
+++ bzrlib/tests/per_branch/test_locking.py	2011-09-19 18:25:53 +0000
@@ -42,13 +42,13 @@
         self.locks = []
         b = lock_helpers.LockWrapper(self.locks, self.get_branch(), 'b')
         b.repository = lock_helpers.LockWrapper(self.locks, b.repository, 'r')
-        bcf = b.control_files
+        bcf = getattr(b, "control_files", None)
         rcf = getattr(b.repository, 'control_files', None)
         if rcf is None:
             self.combined_branch = False
         else:
             # Look out for branch types that reuse their control files
-            self.combined_control = bcf is rcf
+            self.combined_control = bcf is rcf and bcf is not None
         try:
             b.control_files = lock_helpers.LockWrapper(
                 self.locks, b.control_files, 'bc')

=== modified file 'bzrlib/tests/per_branch/test_permissions.py'
--- bzrlib/tests/per_branch/test_permissions.py	2011-02-25 15:24:22 +0000
+++ bzrlib/tests/per_branch/test_permissions.py	2011-09-12 09:51:52 +0000
@@ -29,16 +29,12 @@
 import os
 import sys
 import stat
-from StringIO import StringIO
 
 from bzrlib import tests
-from bzrlib.branch import Branch
+from bzrlib.branch import BzrBranch
 from bzrlib.bzrdir import BzrDir
-from bzrlib.lockable_files import LockableFiles
 from bzrlib.remote import RemoteBranchFormat
-from bzrlib.tests.test_permissions import chmod_r, check_mode_r
-from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
-from bzrlib.workingtree import WorkingTree
+from bzrlib.tests.test_permissions import check_mode_r
 
 
 class _NullPermsStat(object):
@@ -63,10 +59,13 @@
                                           ' permission logic')
         if sys.platform == 'win32':
             raise tests.TestNotApplicable('chmod has no effect on win32')
-        # also, these are BzrBranch format specific things..
         os.mkdir('a')
         mode = stat.S_IMODE(os.stat('a').st_mode)
         t = self.make_branch_and_tree('.')
+        # also, these are BzrBranch format specific things..
+        if not isinstance(t.branch, BzrBranch):
+            raise tests.TestNotApplicable(
+                "Only applicable to bzr branches")
         b = t.branch
         self.assertEqualMode(mode, b.bzrdir._get_dir_mode())
         self.assertEqualMode(mode & ~07111, b.bzrdir._get_file_mode())
@@ -95,9 +94,12 @@
             # which users are not likely to be in, and this prevents us
             # from setting the sgid bit
             os.chown(self.test_dir, os.getuid(), os.getgid())
-        # also, these are BzrBranch format specific things..
         t = self.make_branch_and_tree('.')
         b = t.branch
+        # also, these are BzrBranch format specific things..
+        if not isinstance(b, BzrBranch):
+            raise tests.TestNotApplicable(
+                "Only applicable to bzr branches")
         os.mkdir('b')
         os.chmod('b', 02777)
         b = self.make_branch('b')

=== modified file 'bzrlib/tests/per_branch/test_pull.py'
--- bzrlib/tests/per_branch/test_pull.py	2011-08-09 14:18:05 +0000
+++ bzrlib/tests/per_branch/test_pull.py	2011-12-27 12:18:36 +0000
@@ -16,8 +16,6 @@
 
 """Tests for branch.pull behaviour."""
 
-import os
-
 from bzrlib import (
     branch,
     bzrdir,
@@ -44,7 +42,7 @@
         parent.merge_from_branch(mine.branch)
         parent.commit('merge my change', rev_id='P2')
         mine.pull(parent.branch)
-        self.assertEqual(['P1', 'P2'], mine.branch.revision_history())
+        self.assertEqual('P2', mine.branch.last_revision())
 
     def test_pull_merged_indirect(self):
         # it should be possible to do a pull from one branch into another
@@ -61,7 +59,7 @@
         parent.merge_from_branch(other.branch)
         parent.commit('merge other', rev_id='P2')
         mine.pull(parent.branch)
-        self.assertEqual(['P1', 'P2'], mine.branch.revision_history())
+        self.assertEqual('P2', mine.branch.last_revision())
 
     def test_pull_updates_checkout_and_master(self):
         """Pulling into a checkout updates the checkout and the master branch"""
@@ -73,8 +71,8 @@
         rev2 = other.commit('other commit')
         # now pull, which should update both checkout and master.
         checkout.branch.pull(other.branch)
-        self.assertEqual([rev1, rev2], checkout.branch.revision_history())
-        self.assertEqual([rev1, rev2], master_tree.branch.revision_history())
+        self.assertEqual(rev2, checkout.branch.last_revision())
+        self.assertEqual(rev2, master_tree.branch.last_revision())
 
     def test_pull_local_updates_checkout_only(self):
         """Pulling --local into a checkout updates the checkout and not the
@@ -87,8 +85,8 @@
         rev2 = other.commit('other commit')
         # now pull local, which should update checkout but not master.
         checkout.branch.pull(other.branch, local = True)
-        self.assertEqual([rev1, rev2], checkout.branch.revision_history())
-        self.assertEqual([rev1], master_tree.branch.revision_history())
+        self.assertEqual(rev2, checkout.branch.last_revision())
+        self.assertEqual(rev1, master_tree.branch.last_revision())
 
     def test_pull_local_raises_LocalRequiresBoundBranch_on_unbound(self):
         """Pulling --local into a branch that is not bound should fail."""
@@ -100,7 +98,7 @@
         # now pull --local, which should raise LocalRequiresBoundBranch error.
         self.assertRaises(errors.LocalRequiresBoundBranch,
                           master_tree.branch.pull, other.branch, local = True)
-        self.assertEqual([rev1], master_tree.branch.revision_history())
+        self.assertEqual(rev1, master_tree.branch.last_revision())
 
     def test_pull_returns_result(self):
         parent = self.make_branch_and_tree('parent')
@@ -117,7 +115,7 @@
         self.assertEqual('P1', result.old_revid)
         self.assertEqual(2, result.new_revno)
         self.assertEqual('M1', result.new_revid)
-        self.assertEqual(None, result.tag_conflicts)
+        self.assertEqual([], result.tag_conflicts)
 
     def test_pull_overwrite(self):
         tree_a = self.make_branch_and_tree('tree_a')
@@ -130,14 +128,15 @@
                           tree_a.branch.pull, tree_b.branch,
                           overwrite=False, stop_revision='rev2b')
         # It should not have updated the branch tip, but it should have fetched
-        # the revision
+        # the revision if the repository supports "invisible" revisions
         self.assertEqual('rev2a', tree_a.branch.last_revision())
-        self.assertTrue(tree_a.branch.repository.has_revision('rev2b'))
+        if tree_a.branch.repository._format.supports_unreferenced_revisions:
+            self.assertTrue(tree_a.branch.repository.has_revision('rev2b'))
         tree_a.branch.pull(tree_b.branch, overwrite=True,
                            stop_revision='rev2b')
         self.assertEqual('rev2b', tree_a.branch.last_revision())
-        self.assertEqual(tree_b.branch.revision_history(),
-                         tree_a.branch.revision_history())
+        self.assertEqual(tree_b.branch.last_revision(),
+                         tree_a.branch.last_revision())
 
     def test_pull_merges_and_fetches_tags(self):
         """Tags are updated by br.pull(source), and revisions named in those
@@ -156,7 +155,7 @@
         except errors.TagsNotSupported:
             raise TestNotApplicable('format does not support tags.')
         source.tags.set_tag('tag-a', 'rev-2')
-        source.get_config().set_user_option('branch.fetch_tags', 'True')
+        source.get_config_stack().set('branch.fetch_tags', True)
         target.pull(source)
         # The tag is present, and so is its revision.
         self.assertEqual('rev-2', target.tags.lookup_tag('tag-a'))
@@ -178,7 +177,7 @@
             source.tags.set_tag('tag-a', 'rev-2')
         except errors.TagsNotSupported:
             raise TestNotApplicable('format does not support tags.')
-        source.get_config().set_user_option('branch.fetch_tags', 'True')
+        source.get_config_stack().set('branch.fetch_tags', True)
         target.pull(source, 'rev-2-again')
         # The tag is present, and so is its revision.
         self.assertEqual('rev-2', target.tags.lookup_tag('tag-a'))

=== modified file 'bzrlib/tests/per_branch/test_push.py'
--- bzrlib/tests/per_branch/test_push.py	2011-07-23 16:33:38 +0000
+++ bzrlib/tests/per_branch/test_push.py	2011-09-26 15:21:01 +0000
@@ -53,7 +53,7 @@
         mine.merge_from_branch(other.branch)
         mine.commit('merge my change', rev_id='P2')
         result = mine.branch.push(other.branch)
-        self.assertEqual(['P1', 'P2'], other.branch.revision_history())
+        self.assertEqual('P2', other.branch.last_revision())
         # result object contains some structured data
         self.assertEqual(result.old_revid, 'M1')
         self.assertEqual(result.new_revid, 'P2')
@@ -78,7 +78,7 @@
         mine.merge_from_branch(other.branch)
         mine.commit('merge other', rev_id='P2')
         mine.branch.push(target.branch)
-        self.assertEqual(['P1', 'P2'], target.branch.revision_history())
+        self.assertEqual('P2', target.branch.last_revision())
 
     def test_push_to_checkout_updates_master(self):
         """Pushing into a checkout updates the checkout and the master branch"""
@@ -95,8 +95,8 @@
         rev2 = other.commit('other commit')
         # now push, which should update both checkout and master.
         other.branch.push(checkout.branch)
-        self.assertEqual([rev1, rev2], checkout.branch.revision_history())
-        self.assertEqual([rev1, rev2], master_tree.branch.revision_history())
+        self.assertEqual(rev2, checkout.branch.last_revision())
+        self.assertEqual(rev2, master_tree.branch.last_revision())
 
     def test_push_raises_specific_error_on_master_connection_error(self):
         master_tree = self.make_branch_and_tree('master')
@@ -157,6 +157,8 @@
         except (errors.IncompatibleFormat, errors.UninitializableFormat):
             # This Branch format cannot create shared repositories
             return
+        if not repo._format.supports_nesting_repositories:
+            return
         # This is a little bit trickier because make_branch_and_tree will not
         # re-use a shared repository.
         a_bzrdir = self.make_bzrdir('repo/tree')
@@ -251,7 +253,7 @@
         self.addCleanup(repo.lock_read().unlock)
         # We should have pushed 'C', but not 'B', since it isn't in the
         # ancestry
-        self.assertEqual([('A',), ('C',)], sorted(repo.revisions.keys()))
+        self.assertEqual(['A', 'C'], sorted(repo.all_revision_ids()))
 
     def test_push_with_default_stacking_does_not_create_broken_branch(self):
         """Pushing a new standalone branch works even when there's a default

=== modified file 'bzrlib/tests/per_branch/test_reconcile.py'
--- bzrlib/tests/per_branch/test_reconcile.py	2011-04-19 14:17:24 +0000
+++ bzrlib/tests/per_branch/test_reconcile.py	2011-10-16 14:20:03 +0000
@@ -17,13 +17,17 @@
 """Tests for branch implementations - test reconcile() functionality"""
 
 from bzrlib import errors, reconcile
+from bzrlib.branch import BzrBranch
 from bzrlib.symbol_versioning import deprecated_in
 from bzrlib.tests.per_branch import TestCaseWithBranch
+from bzrlib.tests import TestNotApplicable
 
 
 class TestBranchReconcile(TestCaseWithBranch):
 
     def test_reconcile_fixes_invalid_revhistory(self):
+        if not isinstance(self.branch_format, BzrBranch):
+            raise TestNotApplicable("test only applies to bzr formats")
         # Different formats have different ways of handling invalid revision
         # histories, so the setup portion is customized
         tree = self.make_branch_and_tree('test')
@@ -75,10 +79,12 @@
 
     def test_reconcile_handles_ghosts_in_revhistory(self):
         tree = self.make_branch_and_tree('test')
+        if not tree.branch.repository._format.supports_ghosts:
+            raise TestNotApplicable("repository format does not support ghosts")
         tree.set_parent_ids(["spooky"], allow_leftmost_as_ghost=True)
         r1 = tree.commit('one')
         r2 = tree.commit('two')
         tree.branch.set_last_revision_info(2, r2)
 
         reconciler = tree.branch.reconcile()
-        self.assertEquals([r1, r2], tree.branch.revision_history())
+        self.assertEquals(r2, tree.branch.last_revision())

=== modified file 'bzrlib/tests/per_branch/test_revision_history.py'
--- bzrlib/tests/per_branch/test_revision_history.py	2011-05-01 22:19:46 +0000
+++ bzrlib/tests/per_branch/test_revision_history.py	2011-10-05 12:45:41 +0000
@@ -22,7 +22,10 @@
     revision as _mod_revision,
     )
 from bzrlib.symbol_versioning import deprecated_in
-from bzrlib.tests import per_branch
+from bzrlib.tests import (
+    per_branch,
+    TestNotApplicable,
+    )
 
 
 class TestLastRevision(per_branch.TestCaseWithBranch):
@@ -39,9 +42,13 @@
         br = self.get_branch()
         br.fetch(wt.branch)
         br.set_last_revision_info(1, 'rev1')
-        self.assertEquals(br.revision_history(), ["rev1",])
+        self.assertEquals(
+            self.applyDeprecated(deprecated_in((2, 5, 0)), br.revision_history),
+            ["rev1",])
         br.set_last_revision_info(3, 'rev3')
-        self.assertEquals(br.revision_history(), ["rev1", "rev2", "rev3"])
+        self.assertEquals(
+            self.applyDeprecated(deprecated_in((2, 5, 0)), br.revision_history),
+            ["rev1", "rev2", "rev3"])
         # append_revision specifically prohibits some ids;
         # set_last_revision_info currently does not
         ## self.assertRaises(errors.ReservedId,
@@ -85,8 +92,8 @@
         """
         branch, calls = self.get_instrumented_branch()
         # Repeatedly call revision_history.
-        branch.revision_history()
-        branch.revision_history()
+        self.applyDeprecated(deprecated_in((2, 5, 0)), branch.revision_history)
+        self.applyDeprecated(deprecated_in((2, 5, 0)), branch.revision_history)
         self.assertEqual(
             ['_gen_revision_history', '_gen_revision_history'], calls)
 
@@ -98,8 +105,10 @@
         # Lock the branch, then repeatedly call revision_history.
         branch.lock_read()
         try:
-            branch.revision_history()
-            branch.revision_history()
+            self.applyDeprecated(deprecated_in((2, 5, 0)),
+                branch.revision_history)
+            self.applyDeprecated(deprecated_in((2, 5, 0)),
+                branch.revision_history)
             self.assertEqual(['_gen_revision_history'], calls)
         finally:
             branch.unlock()
@@ -116,7 +125,8 @@
         self.applyDeprecated(deprecated_in((2, 4, 0)),
             branch.set_revision_history, [])
         try:
-            branch.revision_history()
+            self.applyDeprecated(deprecated_in((2, 5, 0)),
+                branch.revision_history)
             self.assertEqual([], calls)
         finally:
             branch.unlock()
@@ -128,7 +138,8 @@
         branch, calls = self.get_instrumented_branch()
         self.applyDeprecated(deprecated_in((2, 4, 0)),
             branch.set_revision_history, [])
-        branch.revision_history()
+        self.applyDeprecated(deprecated_in((2, 5, 0)),
+            branch.revision_history)
         self.assertEqual(['_gen_revision_history'], calls)
 
     def test_set_last_revision_info_when_locked(self):
@@ -168,12 +179,14 @@
         a_branch, calls = self.get_instrumented_branch()
         # Lock the branch, cache the revision history.
         a_branch.lock_write()
-        a_branch.revision_history()
+        self.applyDeprecated(deprecated_in((2, 5, 0)),
+            a_branch.revision_history)
         # Set the last revision info, clearing the cache.
         a_branch.set_last_revision_info(0, _mod_revision.NULL_REVISION)
         del calls[:]
         try:
-            a_branch.revision_history()
+            self.applyDeprecated(deprecated_in((2, 5, 0)),
+                a_branch.revision_history)
             self.assertEqual(['_gen_revision_history'], calls)
         finally:
             a_branch.unlock()
@@ -189,14 +202,17 @@
         branch.lock_read()
         try:
             # The first time the data returned will not be in the cache.
-            history = branch.revision_history()
+            history = self.applyDeprecated(deprecated_in((2, 5, 0)),
+                branch.revision_history)
             history.append('one')
             # The second time the data comes from the cache.
-            history = branch.revision_history()
+            history = self.applyDeprecated(deprecated_in((2, 5, 0)),
+                branch.revision_history)
             history.append('two')
             # The revision_history() should still be unchanged, even though
             # we've mutated the return values from earlier calls.
-            self.assertEqual([], branch.revision_history())
+            self.assertEqual([], self.applyDeprecated(
+                deprecated_in((2, 5, 0)), branch.revision_history))
         finally:
             branch.unlock()
 
@@ -205,8 +221,11 @@
 
     def test_parent_ghost(self):
         tree = self.make_branch_and_tree('tree')
+        if not tree.branch.repository._format.supports_ghosts:
+            raise TestNotApplicable("repository format does not support ghosts")
         tree.add_parent_tree_id('ghost-revision',
                                 allow_leftmost_as_ghost=True)
         tree.commit('first non-ghost commit', rev_id='non-ghost-revision')
         self.assertEqual(['non-ghost-revision'],
-                         tree.branch.revision_history())
+                         self.applyDeprecated(deprecated_in((2, 5, 0)),
+                            tree.branch.revision_history))

=== modified file 'bzrlib/tests/per_branch/test_revision_id_to_revno.py'
--- bzrlib/tests/per_branch/test_revision_id_to_revno.py	2011-02-26 15:35:01 +0000
+++ bzrlib/tests/per_branch/test_revision_id_to_revno.py	2011-11-16 12:25:23 +0000
@@ -17,6 +17,7 @@
 """Tests for Branch.revision_id_to_revno()"""
 
 from bzrlib import errors
+from bzrlib.tests import TestNotApplicable
 
 from bzrlib.tests.per_branch import TestCaseWithBranch
 
@@ -41,6 +42,8 @@
 
     def test_mainline_ghost(self):
         tree = self.make_branch_and_tree('tree1')
+        if not tree.branch.repository._format.supports_ghosts:
+            raise TestNotApplicable("repository format does not support ghosts")
         tree.set_parent_ids(["spooky"], allow_leftmost_as_ghost=True)
         tree.add('')
         tree.commit('msg1', rev_id='rev1')
@@ -53,3 +56,9 @@
             tree.branch.revision_id_to_revno, "unknown")
         self.assertEquals(1, tree.branch.revision_id_to_revno("rev1"))
         self.assertEquals(2, tree.branch.revision_id_to_revno("rev2"))
+
+    def test_empty(self):
+        branch = self.make_branch('.')
+        self.assertRaises(errors.NoSuchRevision,
+            branch.revision_id_to_revno, "unknown")
+        self.assertEquals(0, branch.revision_id_to_revno('null:'))

=== modified file 'bzrlib/tests/per_branch/test_sprout.py'
--- bzrlib/tests/per_branch/test_sprout.py	2011-06-14 01:26:41 +0000
+++ bzrlib/tests/per_branch/test_sprout.py	2011-10-05 12:45:41 +0000
@@ -112,7 +112,7 @@
         repo.fetch(wt.branch.repository)
         branch2 = wt.branch.sprout(repo.bzrdir, revision_id='rev2-alt')
         self.assertEqual((2, 'rev2-alt'), branch2.last_revision_info())
-        self.assertEqual(['rev1', 'rev2-alt'], branch2.revision_history())
+        self.assertEqual('rev2-alt', branch2.last_revision())
 
     def test_sprout_preserves_tags(self):
         """Sprout preserves tags, even tags of absent revisions."""
@@ -124,9 +124,9 @@
         source = builder.get_branch()
         try:
             source.tags.set_tag('tag-a', 'missing-rev')
-        except errors.TagsNotSupported:
+        except (errors.TagsNotSupported, errors.GhostTagsNotSupported):
             raise tests.TestNotApplicable(
-                'Branch format does not support tags.')
+                'Branch format does not support tags or tags to ghosts.')
         # Now source has a tag pointing to an absent revision.  Sprout it.
         target_bzrdir = self.make_repository('target').bzrdir
         new_branch = source.sprout(target_bzrdir)
@@ -179,6 +179,9 @@
 
     def test_sprout_with_ghost_in_mainline(self):
         tree = self.make_branch_and_tree('tree1')
+        if not tree.branch.repository._format.supports_ghosts:
+            raise tests.TestNotApplicable(
+                "repository format does not support ghosts in mainline")
         tree.set_parent_ids(["spooky"], allow_leftmost_as_ghost=True)
         tree.add('')
         tree.commit('msg1', rev_id='rev1')

=== modified file 'bzrlib/tests/per_branch/test_stacking.py'
--- bzrlib/tests/per_branch/test_stacking.py	2011-02-24 16:45:27 +0000
+++ bzrlib/tests/per_branch/test_stacking.py	2011-11-24 17:24:21 +0000
@@ -150,7 +150,8 @@
         self.assertRevisionNotInRepository('newbranch', trunk_revid)
         tree = new_dir.open_branch().create_checkout('local')
         new_branch_revid = tree.commit('something local')
-        self.assertRevisionNotInRepository('mainline', new_branch_revid)
+        self.assertRevisionNotInRepository(
+            trunk_tree.branch.base, new_branch_revid)
         self.assertRevisionInRepository('newbranch', new_branch_revid)
 
     def test_sprout_stacked_from_smart_server(self):
@@ -171,7 +172,8 @@
         self.assertRevisionNotInRepository('newbranch', trunk_revid)
         tree = new_dir.open_branch().create_checkout('local')
         new_branch_revid = tree.commit('something local')
-        self.assertRevisionNotInRepository('mainline', new_branch_revid)
+        self.assertRevisionNotInRepository(trunk_tree.branch.user_url,
+            new_branch_revid)
         self.assertRevisionInRepository('newbranch', new_branch_revid)
 
     def test_unstack_fetches(self):
@@ -482,7 +484,9 @@
         rtree = target.repository.revision_tree('rev2')
         rtree.lock_read()
         self.addCleanup(rtree.unlock)
-        self.assertEqual('new content', rtree.get_file_by_path('a').read())
+        self.assertEqual(
+            'new content',
+            rtree.get_file_text(rtree.path2id('a'), 'a'))
         self.check_lines_added_or_present(target, 'rev2')
 
     def test_transform_fallback_location_hook(self):
@@ -511,6 +515,8 @@
             repo = self.make_repository('repo', shared=True)
         except errors.IncompatibleFormat:
             raise TestNotApplicable()
+        if not repo._format.supports_nesting_repositories:
+            raise TestNotApplicable()
         # Avoid make_branch, which produces standalone branches.
         bzrdir = self.make_bzrdir('repo/stack-on')
         try:
@@ -546,7 +552,7 @@
         self.assertEqual({}, repo.get_parent_map(['rev1']))
         # revision_history should work, even though the history is spread over
         # multiple repositories.
-        self.assertLength(2, stacked.branch.revision_history())
+        self.assertEquals((2, 'rev2'), stacked.branch.last_revision_info())
 
 
 class TestStackingConnections(
@@ -568,7 +574,7 @@
         stacked.set_last_revision_info(1, 'rev-base')
         stacked_relative = self.make_branch('stacked_relative',
                                             format=self.bzrdir_format)
-        stacked_relative.set_stacked_on_url('../base')
+        stacked_relative.set_stacked_on_url(base_tree.branch.user_url)
         stacked.set_last_revision_info(1, 'rev-base')
         self.start_logging_connections()
 

=== modified file 'bzrlib/tests/per_branch/test_tags.py'
--- bzrlib/tests/per_branch/test_tags.py	2011-08-26 23:59:59 +0000
+++ bzrlib/tests/per_branch/test_tags.py	2011-09-12 09:51:52 +0000
@@ -22,6 +22,7 @@
 
 from bzrlib import (
     branch,
+    bzrdir as _mod_bzrdir,
     errors,
     tests,
     )
@@ -39,13 +40,22 @@
             raise tests.TestSkipped(
                 "format %s doesn't support tags" % branch._format)
 
+    def make_branch_with_revisions(self, relpath, revisions):
+        builder = self.make_branch_builder(relpath)
+        builder.start_series()
+        for revid in revisions:
+            builder.build_commit(rev_id=revid)
+        builder.finish_series()
+        return builder.get_branch()
+
     def test_tags_initially_empty(self):
         b = self.make_branch('b')
         tags = b.tags.get_tag_dict()
         self.assertEqual(tags, {})
 
     def test_make_and_lookup_tag(self):
-        b = self.make_branch('b')
+        b = self.make_branch_with_revisions('b',
+            ['target-revid-1', 'target-revid-2'])
         b.tags.set_tag('tag-name', 'target-revid-1')
         b.tags.set_tag('other-name', 'target-revid-2')
         # then reopen the branch and see they're still there
@@ -62,16 +72,26 @@
         self.assertFalse(b.tags.has_tag('imaginary'))
 
     def test_reverse_tag_dict(self):
-        b = self.make_branch('b')
+        b = self.make_branch_with_revisions('b',
+            ['target-revid-1', 'target-revid-2'])
         b.tags.set_tag('tag-name', 'target-revid-1')
         b.tags.set_tag('other-name', 'target-revid-2')
         # then reopen the branch and check reverse map id->tags list
         b = branch.Branch.open('b')
-        self.assertEqual(b.tags.get_reverse_tag_dict(),
+        self.assertEqual(dict(b.tags.get_reverse_tag_dict()),
             {'target-revid-1': ['tag-name'],
              'target-revid-2': ['other-name'],
             })
 
+    def test_ghost_tag(self):
+        b = self.make_branch('b')
+        if not b._format.supports_tags_referencing_ghosts():
+            self.assertRaises(errors.GhostTagsNotSupported,
+                b.tags.set_tag, "ghost", "idontexist")
+        else:
+            b.tags.set_tag("ghost", "idontexist")
+            self.assertEquals("idontexist", b.tags.lookup_tag("ghost"))
+
     def test_no_such_tag(self):
         b = self.make_branch('b')
         try:
@@ -83,8 +103,8 @@
             self.fail("didn't get expected exception")
 
     def test_merge_tags(self):
-        b1 = self.make_branch('b1')
-        b2 = self.make_branch('b2')
+        b1 = self.make_branch_with_revisions('b1', ['revid', 'revid-1'])
+        b2 = self.make_branch_with_revisions('b2', ['revid', 'revid-2'])
         # if there are tags in the source and not the destination, then they
         # just go across
         b1.tags.set_tag('tagname', 'revid')
@@ -93,32 +113,34 @@
         # if a tag is in the destination and not in the source, it is not
         # removed when we merge them
         b2.tags.set_tag('in-destination', 'revid')
-        result = b1.tags.merge_to(b2.tags)
-        self.assertEquals(list(result), [])
+        updates, conflicts = b1.tags.merge_to(b2.tags)
+        self.assertEquals(list(conflicts), [])
+        self.assertEquals(updates, {})
         self.assertEquals(b2.tags.lookup_tag('in-destination'), 'revid')
         # if there's a conflicting tag, it's reported -- the command line
         # interface will say "these tags couldn't be copied"
         b1.tags.set_tag('conflicts', 'revid-1')
         b2.tags.set_tag('conflicts', 'revid-2')
-        result = b1.tags.merge_to(b2.tags)
-        self.assertEquals(list(result),
+        updates, conflicts = b1.tags.merge_to(b2.tags)
+        self.assertEquals(list(conflicts),
             [('conflicts', 'revid-1', 'revid-2')])
         # and it keeps the same value
+        self.assertEquals(updates, {})
         self.assertEquals(b2.tags.lookup_tag('conflicts'), 'revid-2')
 
     def test_unicode_tag(self):
-        b1 = self.make_branch('b')
         tag_name = u'\u3070'
         # in anticipation of the planned change to treating revision ids as
         # just 8bit strings
         revid = ('revid' + tag_name).encode('utf-8')
+        b1 = self.make_branch_with_revisions('b', [revid])
         b1.tags.set_tag(tag_name, revid)
         self.assertEquals(b1.tags.lookup_tag(tag_name), revid)
 
     def test_delete_tag(self):
-        b = self.make_branch('b')
         tag_name = u'\N{GREEK SMALL LETTER ALPHA}'
         revid = ('revid' + tag_name).encode('utf-8')
+        b = self.make_branch_with_revisions('b', [revid])
         b.tags.set_tag(tag_name, revid)
         # now try to delete it
         b.tags.delete_tag(tag_name)
@@ -145,9 +167,10 @@
         # Open the same branch twice.  Read-lock one, and then mutate the tags
         # in the second.  The read-locked branch never re-reads the tags, so it
         # never observes the changed/new tags.
-        b1 = self.make_branch('b')
+        b1 = self.make_branch_with_revisions('b',
+            ['rev-1', 'rev-1-changed', 'rev-2'])
         b1.tags.set_tag('one', 'rev-1')
-        b2 = b1.bzrdir.open_branch()
+        b2 = _mod_bzrdir.BzrDir.open('b').open_branch()
         b1.lock_read()
         self.assertEqual({'one': 'rev-1'}, b1.tags.get_tag_dict())
         # Add a tag and modify a tag in b2.  b1 is read-locked and has already
@@ -164,7 +187,8 @@
     def test_unlocked_does_not_cache_tags(self):
         """Unlocked branches do not cache tags."""
         # Open the same branch twice.
-        b1 = self.make_branch('b')
+        b1 = self.make_branch_with_revisions('b',
+            ['rev-1', 'rev-1-changed', 'rev-2'])
         b1.tags.set_tag('one', 'rev-1')
         b2 = b1.bzrdir.open_branch()
         self.assertEqual({'one': 'rev-1'}, b1.tags.get_tag_dict())
@@ -180,7 +204,8 @@
         returns a copy of the cached data so that callers cannot accidentally
         corrupt the cache.
         """
-        b = self.make_branch('b')
+        b = self.make_branch_with_revisions('b',
+            ['rev-1', 'rev-2', 'rev-3'])
         b.tags.set_tag('one', 'rev-1')
         self.addCleanup(b.lock_read().unlock)
         # The first time the data returned will not be in the cache
@@ -194,7 +219,8 @@
         self.assertEqual({'one': 'rev-1'}, b.tags.get_tag_dict())
 
     def make_write_locked_branch_with_one_tag(self):
-        b = self.make_branch('b')
+        b = self.make_branch_with_revisions('b',
+            ['rev-1', 'rev-1-changed', 'rev-2'])
         b.tags.set_tag('one', 'rev-1')
         self.addCleanup(b.lock_write().unlock)
         # Populate the cache
@@ -213,7 +239,7 @@
 
     def test_merge_to_invalides_cache(self):
         b1 = self.make_write_locked_branch_with_one_tag()
-        b2 = self.make_branch('b2')
+        b2 = self.make_branch_with_revisions('b2', ['rev-2', 'rev-1'])
         b2.tags.set_tag('two', 'rev-2')
         b2.tags.merge_to(b1.tags)
         self.assertEqual(
@@ -292,9 +318,10 @@
         child.bind(master)
         child.update()
         master.tags.set_tag('foo', 'rev-2')
-        tag_conflicts = other.tags.merge_to(child.tags, overwrite=True)
+        tag_updates, tag_conflicts = other.tags.merge_to(child.tags, overwrite=True)
         self.assertEquals('rev-1', child.tags.lookup_tag('foo'))
         self.assertEquals('rev-1', master.tags.lookup_tag('foo'))
+        self.assertEquals({"foo": "rev-1"}, tag_updates)
         self.assertLength(0, tag_conflicts)
 
     def test_merge_to_overwrite_conflict_in_child_and_master(self):
@@ -308,9 +335,11 @@
         child = self.make_branch('child')
         child.bind(master)
         child.update()
-        tag_conflicts = other.tags.merge_to(child.tags, overwrite=True)
+        tag_updates, tag_conflicts = other.tags.merge_to(
+            child.tags, overwrite=True)
         self.assertEquals('rev-1', child.tags.lookup_tag('foo'))
         self.assertEquals('rev-1', master.tags.lookup_tag('foo'))
+        self.assertEquals({u'foo': 'rev-1'}, tag_updates)
         self.assertLength(0, tag_conflicts)
 
     def test_merge_to_conflict_in_child_only(self):
@@ -325,13 +354,14 @@
         child.bind(master)
         child.update()
         master.tags.delete_tag('foo')
-        tag_conflicts = other.tags.merge_to(child.tags)
+        tag_updates, tag_conflicts = other.tags.merge_to(child.tags)
         # Conflict in child, so it is unchanged.
         self.assertEquals('rev-2', child.tags.lookup_tag('foo'))
         # No conflict in the master, so the 'foo' tag equals other's value here.
         self.assertEquals('rev-1', master.tags.lookup_tag('foo'))
         # The conflict is reported.
         self.assertEquals([(u'foo', 'rev-1', 'rev-2')], list(tag_conflicts))
+        self.assertEquals({u'foo': 'rev-1'}, tag_updates)
 
     def test_merge_to_conflict_in_master_only(self):
         """When new_tags.merge_to(child.tags) conflicts with the master but not
@@ -344,12 +374,13 @@
         child.bind(master)
         child.update()
         master.tags.set_tag('foo', 'rev-2')
-        tag_conflicts = other.tags.merge_to(child.tags)
+        tag_updates, tag_conflicts = other.tags.merge_to(child.tags)
         # No conflict in the child, so the 'foo' tag equals other's value here.
         self.assertEquals('rev-1', child.tags.lookup_tag('foo'))
         # Conflict in master, so it is unchanged.
         self.assertEquals('rev-2', master.tags.lookup_tag('foo'))
         # The conflict is reported.
+        self.assertEquals({u'foo': 'rev-1'}, tag_updates)
         self.assertEquals([(u'foo', 'rev-1', 'rev-2')], list(tag_conflicts))
 
     def test_merge_to_same_conflict_in_master_and_child(self):
@@ -363,12 +394,13 @@
         child = self.make_branch('child')
         child.bind(master)
         child.update()
-        tag_conflicts = other.tags.merge_to(child.tags)
+        tag_updates, tag_conflicts = other.tags.merge_to(child.tags)
         # Both master and child conflict, so both stay as rev-2
         self.assertEquals('rev-2', child.tags.lookup_tag('foo'))
         self.assertEquals('rev-2', master.tags.lookup_tag('foo'))
         # The conflict is reported exactly once, even though it occurs in both
         # master and child.
+        self.assertEquals({}, tag_updates)
         self.assertEquals([(u'foo', 'rev-1', 'rev-2')], list(tag_conflicts))
 
     def test_merge_to_different_conflict_in_master_and_child(self):
@@ -385,11 +417,12 @@
         # We use the private method _set_tag_dict because normally bzr tries to
         # avoid this scenario.
         child.tags._set_tag_dict({'foo': 'rev-3'})
-        tag_conflicts = other.tags.merge_to(child.tags)
+        tag_updates, tag_conflicts = other.tags.merge_to(child.tags)
         # Both master and child conflict, so both stay as they were.
         self.assertEquals('rev-3', child.tags.lookup_tag('foo'))
         self.assertEquals('rev-2', master.tags.lookup_tag('foo'))
         # Both conflicts are reported.
+        self.assertEquals({}, tag_updates)
         self.assertEquals(
             [(u'foo', 'rev-1', 'rev-2'), (u'foo', 'rev-1', 'rev-3')],
             sorted(tag_conflicts))
@@ -453,7 +486,7 @@
             get_tag_name, 'get tag name foo')
         self.assertEquals("foo", self.branch.automatic_tag_name(
             self.branch.last_revision()))
-    
+
     def test_uses_first_return(self):
         get_tag_name_1 = lambda br, revid: "foo1"
         get_tag_name_2 = lambda br, revid: "foo2"

=== modified file 'bzrlib/tests/per_branch/test_update.py'
--- bzrlib/tests/per_branch/test_update.py	2011-04-27 13:26:31 +0000
+++ bzrlib/tests/per_branch/test_update.py	2011-09-26 15:21:01 +0000
@@ -68,7 +68,7 @@
         # commit to the master making the child tree out of date and not a prefix.
         master_tree.commit('bar', rev_id='bar', allow_pointless=True)
         self.assertEqual('foo', child_tree.branch.update())
-        self.assertEqual(['bar'], child_tree.branch.revision_history())
+        self.assertEqual('bar', child_tree.branch.last_revision())
 
     def test_update_in_checkout_of_readonly(self):
         tree1 = self.make_branch_and_tree('tree1')

=== modified file 'bzrlib/tests/per_bzrdir/__init__.py'
--- bzrlib/tests/per_bzrdir/__init__.py	2010-09-25 20:08:01 +0000
+++ bzrlib/tests/per_bzrdir/__init__.py	2011-09-30 13:23:16 +0000
@@ -49,11 +49,8 @@
             self.bzrdir = self.make_bzrdir(None)
         return self.bzrdir
 
-    def make_bzrdir(self, relpath, format=None):
-        if format is None:
-            format = self.bzrdir_format
-        return super(TestCaseWithBzrDir, self).make_bzrdir(
-            relpath, format=format)
+    def get_default_format(self):
+        return self.bzrdir_format
 
 
 def load_tests(standard_tests, module, loader):

=== modified file 'bzrlib/tests/per_bzrdir/test_bzrdir.py'
--- bzrlib/tests/per_bzrdir/test_bzrdir.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/per_bzrdir/test_bzrdir.py	2011-12-12 12:09:50 +0000
@@ -21,13 +21,17 @@
 
 import bzrlib.branch
 from bzrlib import (
+    branch,
+    bzrdir,
     errors,
     repository,
     revision as _mod_revision,
     transport,
     workingtree,
     )
+from bzrlib.remote import RemoteBzrDirFormat
 from bzrlib.tests import (
+    TestNotApplicable,
     TestSkipped,
     )
 from bzrlib.tests.per_bzrdir import TestCaseWithBzrDir
@@ -159,8 +163,12 @@
                 for file_id, revision_id in text_index.iterkeys():
                     desired_files.append(
                         (file_id, revision_id, (file_id, revision_id)))
-                left_texts = list(left_repo.iter_files_bytes(desired_files))
-                right_texts = list(right_repo.iter_files_bytes(desired_files))
+                left_texts = [(identifier, "".join(bytes_iterator)) for
+                        (identifier, bytes_iterator) in
+                        left_repo.iter_files_bytes(desired_files)]
+                right_texts = [(identifier, "".join(bytes_iterator)) for
+                        (identifier, bytes_iterator) in
+                        right_repo.iter_files_bytes(desired_files)]
                 left_texts.sort()
                 right_texts.sort()
                 self.assertEqual(left_texts, right_texts)
@@ -328,7 +336,7 @@
                                      './.bzr/repository',
                                      ])
         self.assertRepositoryHasSameItems(tree.branch.repository,
-            target.open_repository())
+            target.open_branch().repository)
         target.open_workingtree().revert()
 
     def test_revert_inventory(self):
@@ -348,7 +356,7 @@
                                      './.bzr/repository',
                                      ])
         self.assertRepositoryHasSameItems(tree.branch.repository,
-            target.open_repository())
+            target.open_branch().repository)
 
         target.open_workingtree().revert()
         self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
@@ -360,7 +368,7 @@
                                      './.bzr/repository',
                                      ])
         self.assertRepositoryHasSameItems(tree.branch.repository,
-            target.open_repository())
+            target.open_branch().repository)
 
     def test_clone_bzrdir_tree_branch_reference(self):
         # a tree with a branch reference (aka a checkout)
@@ -500,8 +508,7 @@
         self.assertNotEqual(dir.transport.base, target.transport.base)
         self.assertDirectoriesEqual(dir.root_transport, target.root_transport,
                                     [
-                                     './.bzr/branch/branch.conf',
-                                     './.bzr/branch/parent',
+                                     './.bzr/branch',
                                      './.bzr/checkout/dirstate',
                                      './.bzr/checkout/stat-cache',
                                      './.bzr/checkout/inventory',
@@ -597,3 +604,94 @@
         self.assertTrue(isinstance(found_transport, transport.Transport))
         # and the dir which has been initialized for us must exist.
         found_transport.list_dir('.')
+
+    def assertInitializeEx(self, t, need_meta=False, **kwargs):
+        """Execute initialize_on_transport_ex and check it succeeded correctly.
+
+        This involves checking that the disk objects were created, open with
+        the same format returned, and had the expected disk format.
+
+        :param t: The transport to initialize on.
+        :param **kwargs: Additional arguments to pass to
+            initialize_on_transport_ex.
+        :return: the resulting repo, control dir tuple.
+        """
+        if not self.bzrdir_format.is_initializable():
+            raise TestNotApplicable("control dir format is not "
+                "initializable")
+        repo, control, require_stacking, repo_policy = \
+            self.bzrdir_format.initialize_on_transport_ex(t, **kwargs)
+        if repo is not None:
+            # Repositories are open write-locked
+            self.assertTrue(repo.is_write_locked())
+            self.addCleanup(repo.unlock)
+        self.assertIsInstance(control, bzrdir.BzrDir)
+        opened = bzrdir.BzrDir.open(t.base)
+        expected_format = self.bzrdir_format
+        if need_meta and expected_format.fixed_components:
+            # Pre-metadir formats change when we are making something that
+            # needs a metaformat, because clone is used for push.
+            expected_format = bzrdir.BzrDirMetaFormat1()
+        if not isinstance(expected_format, RemoteBzrDirFormat):
+            self.assertEqual(control._format.network_name(),
+                expected_format.network_name())
+            self.assertEqual(control._format.network_name(),
+                opened._format.network_name())
+        self.assertEqual(control.__class__, opened.__class__)
+        return repo, control
+
+    def test_format_initialize_on_transport_ex_default_stack_on(self):
+        # When initialize_on_transport_ex uses a stacked-on branch because of
+        # a stacking policy on the target, the location of the fallback
+        # repository is the same as the external location of the stacked-on
+        # branch.
+        balloon = self.make_bzrdir('balloon')
+        if isinstance(balloon._format, bzrdir.BzrDirMetaFormat1):
+            stack_on = self.make_branch('stack-on', format='1.9')
+        else:
+            stack_on = self.make_branch('stack-on')
+        if not stack_on.repository._format.supports_nesting_repositories:
+            raise TestNotApplicable("requires nesting repositories")
+        config = self.make_bzrdir('.').get_config()
+        try:
+            config.set_default_stack_on('stack-on')
+        except errors.BzrError:
+            raise TestNotApplicable('Only relevant for stackable formats.')
+        # Initialize a bzrdir subject to the policy.
+        t = self.get_transport('stacked')
+        repo_fmt = bzrdir.format_registry.make_bzrdir('1.9')
+        repo_name = repo_fmt.repository_format.network_name()
+        repo, control = self.assertInitializeEx(
+            t, need_meta=True, repo_format_name=repo_name, stacked_on=None)
+        # self.addCleanup(repo.unlock)
+        # There's one fallback repo, with a public location.
+        self.assertLength(1, repo._fallback_repositories)
+        fallback_repo = repo._fallback_repositories[0]
+        self.assertEqual(
+            stack_on.base, fallback_repo.bzrdir.root_transport.base)
+        # The bzrdir creates a branch in stacking-capable format.
+        new_branch = control.create_branch()
+        self.assertTrue(new_branch._format.supports_stacking())
+
+    def test_no_leftover_dirs(self):
+        # bug 886196: development-colo uses a branch-lock directory
+        # in the user directory rather than the control directory.
+        if not self.bzrdir_format.colocated_branches:
+            raise TestNotApplicable(
+                "format does not support colocated branches")
+        branch = self.make_branch('.', format='development-colo')
+        branch.bzrdir.create_branch(name="another-colocated-branch")
+        self.assertEquals(
+            branch.bzrdir.user_transport.list_dir("."),
+            [".bzr"])
+
+    def test_get_branches(self):
+        repo = self.make_repository('branch-1')
+        try:
+            target_branch = repo.bzrdir.create_branch(name='foo')
+        except errors.NoColocatedBranchSupport:
+            raise TestNotApplicable('Format does not support colocation')
+        reference = branch.BranchReferenceFormat().initialize(
+            repo.bzrdir, target_branch=target_branch)
+        self.assertEqual(set([None, 'foo']),
+                         set(repo.bzrdir.get_branches().keys()))

=== modified file 'bzrlib/tests/per_controldir/__init__.py'
--- bzrlib/tests/per_controldir/__init__.py	2011-03-12 14:39:47 +0000
+++ bzrlib/tests/per_controldir/__init__.py	2011-09-30 13:23:16 +0000
@@ -67,11 +67,8 @@
             self.bzrdir = self.make_bzrdir(None)
         return self.bzrdir
 
-    def make_bzrdir(self, relpath, format=None):
-        if format is None:
-            format = self.bzrdir_format
-        return super(TestCaseWithControlDir, self).make_bzrdir(
-            relpath, format=format)
+    def get_default_format(self):
+        return self.bzrdir_format
 
 
 def load_tests(standard_tests, module, loader):

=== modified file 'bzrlib/tests/per_controldir/test_controldir.py'
--- bzrlib/tests/per_controldir/test_controldir.py	2011-08-20 18:48:15 +0000
+++ bzrlib/tests/per_controldir/test_controldir.py	2011-12-27 12:18:36 +0000
@@ -101,6 +101,13 @@
                                     create_tree_if_local=create_tree_if_local)
         return target
 
+    def test_uninitializable(self):
+        if self.bzrdir_format.is_initializable():
+            raise TestNotApplicable("format is initializable")
+        t = self.get_transport()
+        self.assertRaises(errors.UninitializableFormat,
+            self.bzrdir_format.initialize, t.base)
+
     def test_create_null_workingtree(self):
         dir = self.make_bzrdir('dir1')
         dir.create_repository()
@@ -149,6 +156,8 @@
         except (errors.UnsupportedOperation, errors.TransportNotPossible):
             raise TestNotApplicable('Format does not support destroying'
                                     ' repository')
+        self.assertRaises(errors.NoRepositoryPresent,
+            bzrdir.destroy_repository)
         self.assertRaises(errors.NoRepositoryPresent, bzrdir.open_repository)
         bzrdir.create_repository()
         bzrdir.open_repository()
@@ -162,7 +171,8 @@
         if vfs_dir.has_workingtree():
             # This ControlDir format doesn't support ControlDirs without
             # working trees, so this test is irrelevant.
-            return
+            raise TestNotApplicable("format does not support "
+                "control directories without working tree")
         self.assertRaises(errors.NoWorkingTree, dir.open_workingtree)
 
     def test_clone_bzrdir_repository_under_shared(self):
@@ -172,12 +182,16 @@
         tree.commit('revision 1', rev_id='1')
         dir = self.make_bzrdir('source')
         repo = dir.create_repository()
+        if not repo._format.supports_nesting_repositories:
+            raise TestNotApplicable("repository format does not support "
+                "nesting")
         repo.fetch(tree.branch.repository)
         self.assertTrue(repo.has_revision('1'))
         try:
             self.make_repository('target', shared=True)
         except errors.IncompatibleFormat:
-            return
+            raise TestNotApplicable("repository format does not support "
+                "shared repositories")
         target = dir.clone(self.get_url('target/child'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
         self.assertRaises(errors.NoRepositoryPresent, target.open_repository)
@@ -187,7 +201,11 @@
         try:
             shared_repo = self.make_repository('shared', shared=True)
         except errors.IncompatibleFormat:
-            return
+            raise TestNotApplicable("repository format does not support "
+                "shared repositories")
+        if not shared_repo._format.supports_nesting_repositories:
+            raise TestNotApplicable("format does not support nesting "
+                "repositories")
         # Make a branch, 'commit_tree', and working tree outside of the shared
         # repository, and commit some revisions to it.
         tree = self.make_branch_and_tree('commit_tree')
@@ -218,7 +236,11 @@
         try:
             shared_repo = self.make_repository('shared', shared=True)
         except errors.IncompatibleFormat:
-            return
+            raise TestNotApplicable("repository format does not support "
+                "shared repositories")
+        if not shared_repo._format.supports_nesting_repositories:
+            raise TestNotApplicable("format does not support nesting "
+                "repositories")
         tree = self.make_branch_and_tree('commit_tree')
         self.build_tree(['commit_tree/foo'])
         tree.add('foo')
@@ -292,15 +314,19 @@
         tree.branch.repository.copy_content_into(source.repository)
         tree.branch.copy_content_into(source)
         try:
-            self.make_repository('target', shared=True)
+            shared_repo = self.make_repository('target', shared=True)
         except errors.IncompatibleFormat:
-            return
+            raise TestNotApplicable("repository format does not support "
+                "shared repositories")
+        if not shared_repo._format.supports_nesting_repositories:
+            raise TestNotApplicable("format does not support nesting "
+                "repositories")
         dir = source.bzrdir
         target = dir.clone(self.get_url('target/child'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
         self.assertRaises(errors.NoRepositoryPresent, target.open_repository)
-        self.assertEqual(source.revision_history(),
-                         target.open_branch().revision_history())
+        self.assertEqual(source.last_revision(),
+                         target.open_branch().last_revision())
 
     def test_clone_bzrdir_branch_revision(self):
         # test for revision limiting, [smoke test, not corner case checks].
@@ -369,16 +395,24 @@
             repo.set_make_working_trees(False)
             self.assertFalse(repo.make_working_trees())
 
-        dir = tree.bzrdir
-        a_dir = dir.clone(self.get_url('repo/a'))
-        a_dir.open_branch()
+        a_dir = tree.bzrdir.clone(self.get_url('repo/a'))
+        a_branch = a_dir.open_branch()
+        # If the new control dir actually uses the repository, it should
+        # not have a working tree.
+        if not a_branch.repository.has_same_location(repo):
+            raise TestNotApplicable('new control dir does not use repository')
         self.assertRaises(errors.NoWorkingTree, a_dir.open_workingtree)
 
     def test_clone_respects_stacked(self):
         branch = self.make_branch('parent')
         child_transport = self.get_transport('child')
-        child = branch.bzrdir.clone_on_transport(child_transport,
-                                                 stacked_on=branch.base)
+        try:
+            child = branch.bzrdir.clone_on_transport(child_transport,
+                                                     stacked_on=branch.base)
+        except (errors.UnstackableBranchFormat,
+                errors.UnstackableRepositoryFormat):
+            raise TestNotApplicable("branch or repository format does "
+                "not support stacking")
         self.assertEqual(child.open_branch().get_stacked_on_url(), branch.base)
 
     def test_get_branch_reference_on_reference(self):
@@ -390,7 +424,8 @@
                 target_branch=referenced_branch)
         except errors.IncompatibleFormat:
             # this is ok too, not all formats have to support references.
-            return
+            raise TestNotApplicable("control directory does not "
+                "support branch references")
         self.assertEqual(referenced_branch.bzrdir.root_transport.abspath('') + '/',
             dir.get_branch_reference())
 
@@ -404,13 +439,14 @@
         dir = self.make_bzrdir('source')
         if dir.has_branch():
             # this format does not support branchless bzrdirs.
-            return
+            raise TestNotApplicable("format does not support "
+                "branchless control directories")
         self.assertRaises(errors.NotBranchError, dir.get_branch_reference)
 
     def test_sprout_bzrdir_empty(self):
         dir = self.make_bzrdir('source')
         target = dir.sprout(self.get_url('target'))
-        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertNotEqual(dir.control_transport.base, target.control_transport.base)
         # creates a new repository branch and tree
         target.open_repository()
         target.open_branch()
@@ -422,7 +458,8 @@
         try:
             self.make_repository('target', shared=True)
         except errors.IncompatibleFormat:
-            return
+            raise TestNotApplicable("format does not support shared "
+                "repositories")
         target = dir.sprout(self.get_url('target/child'))
         self.assertRaises(errors.NoRepositoryPresent, target.open_repository)
         target.open_branch()
@@ -430,7 +467,8 @@
             target.open_workingtree()
         except errors.NoWorkingTree:
             # Some bzrdirs can never have working trees.
-            self.assertFalse(target._format.supports_workingtrees)
+            repo = target.find_repository()
+            self.assertFalse(repo.bzrdir._format.supports_workingtrees)
 
     def test_sprout_bzrdir_empty_under_shared_repo_force_new(self):
         # the force_new_repo parameter should force use of a new repo in an empty
@@ -439,7 +477,8 @@
         try:
             self.make_repository('target', shared=True)
         except errors.IncompatibleFormat:
-            return
+            raise TestNotApplicable("format does not support shared "
+                "repositories")
         target = dir.sprout(self.get_url('target/child'), force_new_repo=True)
         target.open_repository()
         target.open_branch()
@@ -460,16 +499,21 @@
         try:
             shared_repo = self.make_repository('target', shared=True)
         except errors.IncompatibleFormat:
-            return
+            raise TestNotApplicable("format does not support "
+                "shared repositories")
         target = dir.sprout(self.get_url('target/child'))
-        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertNotEqual(dir.user_transport.base, target.user_transport.base)
         self.assertTrue(shared_repo.has_revision('1'))
 
     def test_sprout_bzrdir_repository_branch_both_under_shared(self):
         try:
             shared_repo = self.make_repository('shared', shared=True)
         except errors.IncompatibleFormat:
-            return
+            raise TestNotApplicable("format does not support shared "
+                "repositories")
+        if not shared_repo._format.supports_nesting_repositories:
+            raise TestNotApplicable("format does not support nesting "
+                "repositories")
         tree = self.make_branch_and_tree('commit_tree')
         self.build_tree(['commit_tree/foo'])
         tree.add('foo')
@@ -490,7 +534,11 @@
         try:
             shared_repo = self.make_repository('shared', shared=True)
         except errors.IncompatibleFormat:
-            return
+            raise TestNotApplicable("format does not support shared "
+                "repositories")
+        if not shared_repo._format.supports_nesting_repositories:
+            raise TestNotApplicable("format does not support nesting "
+                "repositories")
         tree = self.make_branch_and_tree('commit_tree')
         self.build_tree(['commit_tree/foo'])
         tree.add('foo')
@@ -533,9 +581,12 @@
         try:
             shared_repo = self.make_repository('target', shared=True)
         except errors.IncompatibleFormat:
-            return
+            raise TestNotApplicable("format does not support shared "
+                "repositories")
         target = dir.sprout(self.get_url('target/child'), force_new_repo=True)
-        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertNotEqual(
+            dir.control_transport.base,
+            target.control_transport.base)
         self.assertFalse(shared_repo.has_revision('1'))
 
     def test_sprout_bzrdir_repository_revision(self):
@@ -571,7 +622,8 @@
         try:
             shared_repo = self.make_repository('target', shared=True)
         except errors.IncompatibleFormat:
-            return
+            raise TestNotApplicable("format does not support shared "
+                "repositories")
         target = dir.sprout(self.get_url('target/child'))
         self.assertTrue(shared_repo.has_revision('1'))
 
@@ -589,9 +641,10 @@
         try:
             shared_repo = self.make_repository('target', shared=True)
         except errors.IncompatibleFormat:
-            return
+            raise TestNotApplicable("format does not support shared "
+                "repositories")
         target = dir.sprout(self.get_url('target/child'), force_new_repo=True)
-        self.assertNotEqual(dir.transport.base, target.transport.base)
+        self.assertNotEqual(dir.control_transport.base, target.control_transport.base)
         self.assertFalse(shared_repo.has_revision('1'))
 
     def test_sprout_bzrdir_branch_reference(self):
@@ -602,8 +655,8 @@
             reference = bzrlib.branch.BranchReferenceFormat().initialize(dir,
                 target_branch=referenced_branch)
         except errors.IncompatibleFormat:
-            # this is ok too, not all formats have to support references.
-            return
+            raise TestNotApplicable("format does not support branch "
+                "references")
         self.assertRaises(errors.NoRepositoryPresent, dir.open_repository)
         target = dir.sprout(self.get_url('target'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
@@ -622,13 +675,14 @@
             reference = bzrlib.branch.BranchReferenceFormat().initialize(dir,
                 target_branch=referenced_tree.branch)
         except errors.IncompatibleFormat:
-            # this is ok too, not all formats have to support references.
-            return
+            raise TestNotApplicable("format does not support branch "
+                "references")
         self.assertRaises(errors.NoRepositoryPresent, dir.open_repository)
         try:
             shared_repo = self.make_repository('target', shared=True)
         except errors.IncompatibleFormat:
-            return
+            raise TestNotApplicable("format does not support "
+                "shared repositories")
         target = dir.sprout(self.get_url('target/child'))
         self.assertNotEqual(dir.transport.base, target.transport.base)
         # we want target to have a branch that is in-place.
@@ -649,12 +703,14 @@
                 target_branch=referenced_tree.branch)
         except errors.IncompatibleFormat:
             # this is ok too, not all formats have to support references.
-            return
+            raise TestNotApplicable("format does not support "
+                "branch references")
         self.assertRaises(errors.NoRepositoryPresent, dir.open_repository)
         try:
             shared_repo = self.make_repository('target', shared=True)
         except errors.IncompatibleFormat:
-            return
+            raise TestNotApplicable("format does not support shared "
+                "repositories")
         target = dir.sprout(self.get_url('target/child'), force_new_repo=True)
         self.assertNotEqual(dir.transport.base, target.transport.base)
         # we want target to have a branch that is in-place.
@@ -690,7 +746,7 @@
             source.tags.set_tag('tag-a', 'rev-2')
         except errors.TagsNotSupported:
             raise TestNotApplicable('Branch format does not support tags.')
-        source.get_config().set_user_option('branch.fetch_tags', 'True')
+        source.get_config_stack().set('branch.fetch_tags', True)
         # Now source has a tag not in its ancestry.  Sprout its controldir.
         dir = source.bzrdir
         target = dir.sprout(self.get_url('target'))
@@ -707,8 +763,9 @@
         source = builder.get_branch()
         try:
             source.tags.set_tag('tag-a', 'missing-rev')
-        except errors.TagsNotSupported:
-            raise TestNotApplicable('Branch format does not support tags.')
+        except (errors.TagsNotSupported, errors.GhostTagsNotSupported):
+            raise TestNotApplicable('Branch format does not support tags '
+                'or tags referencing ghost revisions.')
         # Now source has a tag pointing to an absent revision.  Sprout its
         # controldir.
         dir = source.bzrdir
@@ -725,8 +782,9 @@
         source = builder.get_branch()
         try:
             source.tags.set_tag('tag-a', 'missing-rev')
-        except errors.TagsNotSupported:
-            raise TestNotApplicable('Branch format does not support tags.')
+        except (errors.TagsNotSupported, errors.GhostTagsNotSupported):
+            raise TestNotApplicable('Branch format does not support tags '
+                'or tags referencing missing revisions.')
         # Now source has a tag pointing to an absent revision.  Sprout its
         # controldir.
         dir = source.bzrdir
@@ -759,18 +817,28 @@
         try:
             # Create a tag for B2, and for an absent rev
             source.tags.set_tag('tag-non-ancestry', 'rev-b2')
+        except errors.TagsNotSupported:
+            raise TestNotApplicable('Branch format does not support tags ')
+        try:
             source.tags.set_tag('tag-absent', 'absent-rev')
-        except errors.TagsNotSupported:
-            raise TestNotApplicable('Branch format does not support tags.')
-        source.get_config().set_user_option('branch.fetch_tags', 'True')
+        except errors.GhostTagsNotSupported:
+            has_ghost_tag = False
+        else:
+            has_ghost_tag = True
+        source.get_config_stack().set('branch.fetch_tags', True)
         # And ask sprout for C2
         dir = source.bzrdir
         target = dir.sprout(self.get_url('target'), revision_id='rev-c2')
         # The tags are present
         new_branch = target.open_branch()
-        self.assertEqual(
-            {'tag-absent': 'absent-rev', 'tag-non-ancestry': 'rev-b2'},
-            new_branch.tags.get_tag_dict())
+        if has_ghost_tag:
+            self.assertEqual(
+                {'tag-absent': 'absent-rev', 'tag-non-ancestry': 'rev-b2'},
+                new_branch.tags.get_tag_dict())
+        else:
+            self.assertEqual(
+                {'tag-non-ancestry': 'rev-b2'},
+                new_branch.tags.get_tag_dict())
         # And the revs for A2, B2 and C2's ancestries are present, but no
         # others.
         self.assertEqual(
@@ -787,7 +855,8 @@
                 target_branch=referenced_branch)
         except errors.IncompatibleFormat:
             # this is ok too, not all formats have to support references.
-            return
+            raise TestNotApplicable("format does not support "
+                "branch references")
         self.assertRaises(errors.NoRepositoryPresent, dir.open_repository)
         tree = self.createWorkingTreeOrSkip(dir)
         self.build_tree(['source/subdir/'])
@@ -813,7 +882,8 @@
                 target_branch=referenced_branch)
         except errors.IncompatibleFormat:
             # this is ok too, not all formats have to support references.
-            return
+            raise TestNotApplicable("format does not support "
+                "branch references")
         self.assertRaises(errors.NoRepositoryPresent, dir.open_repository)
         tree = self.createWorkingTreeOrSkip(dir)
         self.build_tree(['source/foo'])
@@ -899,11 +969,11 @@
 
     def test_format_initialize_find_open(self):
         # loopback test to check the current format initializes to itself.
-        if not self.bzrdir_format.is_supported():
+        if not self.bzrdir_format.is_initializable():
             # unsupported formats are not loopback testable
             # because the default open will not open them and
             # they may not be initializable.
-            return
+            raise TestNotApplicable("format is not initializable")
         # for remote formats, there must be no prior assumption about the
         # network name to use - it's possible that this may somehow have got
         # in through an unisolated test though - see
@@ -938,9 +1008,8 @@
         self.assertInitializeEx(t, use_existing_dir=True)
 
     def test_format_initialize_on_transport_ex_use_existing_dir_False(self):
-        if not self.bzrdir_format.is_supported():
-            # Not initializable - not a failure either.
-            return
+        if not self.bzrdir_format.is_initializable():
+            raise TestNotApplicable("format is not initializable")
         t = self.get_transport('dir')
         t.ensure_base()
         self.assertRaises(errors.FileExists,
@@ -952,9 +1021,8 @@
         self.assertInitializeEx(t, create_prefix=True)
 
     def test_format_initialize_on_transport_ex_create_prefix_False(self):
-        if not self.bzrdir_format.is_supported():
-            # Not initializable - not a failure either.
-            return
+        if not self.bzrdir_format.is_initializable():
+            raise TestNotApplicable("format is not initializable")
         t = self.get_transport('missing/dir')
         self.assertRaises(errors.NoSuchFile, self.assertInitializeEx, t,
             create_prefix=False)
@@ -967,9 +1035,6 @@
             repo_format_name=repo_name, shared_repo=True)[0]
         made_repo, control = self.assertInitializeEx(t.clone('branch'),
             force_new_repo=True, repo_format_name=repo_name)
-        if control is None:
-            # uninitialisable format
-            return
         self.assertNotEqual(repo.bzrdir.root_transport.base,
             made_repo.bzrdir.root_transport.base)
 
@@ -981,64 +1046,10 @@
             repo_format_name=repo_name, shared_repo=True)[0]
         made_repo, control = self.assertInitializeEx(t.clone('branch'),
             force_new_repo=False, repo_format_name=repo_name)
-        if control is None:
-            # uninitialisable format
-            return
         if not control._format.fixed_components:
             self.assertEqual(repo.bzrdir.root_transport.base,
                 made_repo.bzrdir.root_transport.base)
 
-    def test_format_initialize_on_transport_ex_stacked_on(self):
-        # trunk is a stackable format.  Note that its in the same server area
-        # which is what launchpad does, but not sufficient to exercise the
-        # general case.
-        trunk = self.make_branch('trunk', format='1.9')
-        t = self.get_transport('stacked')
-        old_fmt = bzrdir.format_registry.make_bzrdir('pack-0.92')
-        repo_name = old_fmt.repository_format.network_name()
-        # Should end up with a 1.9 format (stackable)
-        repo, control = self.assertInitializeEx(t, need_meta=True,
-            repo_format_name=repo_name, stacked_on='../trunk',
-            stack_on_pwd=t.base)
-        if control is None:
-            # uninitialisable format
-            return
-        self.assertLength(1, repo._fallback_repositories)
-
-    def test_format_initialize_on_transport_ex_default_stack_on(self):
-        # When initialize_on_transport_ex uses a stacked-on branch because of
-        # a stacking policy on the target, the location of the fallback
-        # repository is the same as the external location of the stacked-on
-        # branch.
-        balloon = self.make_bzrdir('balloon')
-        if isinstance(balloon._format, bzrdir.BzrDirMetaFormat1):
-            stack_on = self.make_branch('stack-on', format='1.9')
-        else:
-            stack_on = self.make_branch('stack-on')
-        config = self.make_bzrdir('.').get_config()
-        try:
-            config.set_default_stack_on('stack-on')
-        except errors.BzrError:
-            raise TestNotApplicable('Only relevant for stackable formats.')
-        # Initialize a bzrdir subject to the policy.
-        t = self.get_transport('stacked')
-        repo_fmt = bzrdir.format_registry.make_bzrdir('1.9')
-        repo_name = repo_fmt.repository_format.network_name()
-        repo, control = self.assertInitializeEx(
-            t, need_meta=True, repo_format_name=repo_name, stacked_on=None)
-        # self.addCleanup(repo.unlock)
-        if control is None:
-            # uninitialisable format
-            return
-        # There's one fallback repo, with a public location.
-        self.assertLength(1, repo._fallback_repositories)
-        fallback_repo = repo._fallback_repositories[0]
-        self.assertEqual(
-            stack_on.base, fallback_repo.bzrdir.root_transport.base)
-        # The bzrdir creates a branch in stacking-capable format.
-        new_branch = control.create_branch()
-        self.assertTrue(new_branch._format.supports_stacking())
-
     def test_format_initialize_on_transport_ex_repo_fmt_name_None(self):
         t = self.get_transport('dir')
         repo, control = self.assertInitializeEx(t)
@@ -1050,15 +1061,12 @@
         fmt = bzrdir.format_registry.make_bzrdir('1.6')
         repo_name = fmt.repository_format.network_name()
         repo, control = self.assertInitializeEx(t, repo_format_name=repo_name)
-        if control is None:
-            # uninitialisable format
-            return
         if self.bzrdir_format.fixed_components:
             # must stay with the all-in-one-format.
             repo_name = self.bzrdir_format.network_name()
         self.assertEqual(repo_name, repo._format.network_name())
 
-    def assertInitializeEx(self, t, need_meta=False, **kwargs):
+    def assertInitializeEx(self, t, **kwargs):
         """Execute initialize_on_transport_ex and check it succeeded correctly.
 
         This involves checking that the disk objects were created, open with
@@ -1069,9 +1077,9 @@
             initialize_on_transport_ex.
         :return: the resulting repo, control dir tuple.
         """
-        if not self.bzrdir_format.is_supported():
-            # Not initializable - not a failure either.
-            return None, None
+        if not self.bzrdir_format.is_initializable():
+            raise TestNotApplicable("control dir format is not "
+                "initializable")
         repo, control, require_stacking, repo_policy = \
             self.bzrdir_format.initialize_on_transport_ex(t, **kwargs)
         if repo is not None:
@@ -1081,10 +1089,6 @@
         self.assertIsInstance(control, controldir.ControlDir)
         opened = bzrdir.BzrDir.open(t.base)
         expected_format = self.bzrdir_format
-        if need_meta and expected_format.fixed_components:
-            # Pre-metadir formats change when we are making something that
-            # needs a metaformat, because clone is used for push.
-            expected_format = bzrdir.BzrDirMetaFormat1()
         if not isinstance(expected_format, RemoteBzrDirFormat):
             self.assertEqual(control._format.network_name(),
                 expected_format.network_name())
@@ -1124,11 +1128,11 @@
 
     def test_create_branch(self):
         # a bzrdir can construct a branch and repository for itself.
-        if not self.bzrdir_format.is_supported():
+        if not self.bzrdir_format.is_initializable():
             # unsupported formats are not loopback testable
             # because the default open will not open them and
             # they may not be initializable.
-            return
+            raise TestNotApplicable("format is not initializable")
         t = self.get_transport()
         made_control = self.bzrdir_format.initialize(t.base)
         made_repo = made_control.create_repository()
@@ -1136,12 +1140,32 @@
         self.assertIsInstance(made_branch, bzrlib.branch.Branch)
         self.assertEqual(made_control, made_branch.bzrdir)
 
+    def test_create_branch_append_revisions_only(self):
+        # a bzrdir can construct a branch and repository for itself.
+        if not self.bzrdir_format.is_initializable():
+            # unsupported formats are not loopback testable
+            # because the default open will not open them and
+            # they may not be initializable.
+            raise TestNotApplicable("format is not initializable")
+        t = self.get_transport()
+        made_control = self.bzrdir_format.initialize(t.base)
+        made_repo = made_control.create_repository()
+        try:
+            made_branch = made_control.create_branch(
+                append_revisions_only=True)
+        except errors.UpgradeRequired:
+            raise TestNotApplicable("format does not support "
+                "append_revisions_only setting")
+        self.assertIsInstance(made_branch, bzrlib.branch.Branch)
+        self.assertEquals(True, made_branch.get_append_revisions_only())
+        self.assertEqual(made_control, made_branch.bzrdir)
+
     def test_open_branch(self):
-        if not self.bzrdir_format.is_supported():
+        if not self.bzrdir_format.is_initializable():
             # unsupported formats are not loopback testable
             # because the default open will not open them and
             # they may not be initializable.
-            return
+            raise TestNotApplicable("format is not initializable")
         t = self.get_transport()
         made_control = self.bzrdir_format.initialize(t.base)
         made_repo = made_control.create_repository()
@@ -1152,11 +1176,8 @@
         self.assertIsInstance(opened_branch._format, made_branch._format.__class__)
 
     def test_list_branches(self):
-        if not self.bzrdir_format.is_supported():
-            # unsupported formats are not loopback testable
-            # because the default open will not open them and
-            # they may not be initializable.
-            return
+        if not self.bzrdir_format.is_initializable():
+            raise TestNotApplicable("format is not initializable")
         t = self.get_transport()
         made_control = self.bzrdir_format.initialize(t.base)
         made_repo = made_control.create_repository()
@@ -1171,13 +1192,18 @@
         else:
             self.assertEquals([], made_control.list_branches())
 
+    def test_get_branches(self):
+        repo = self.make_repository('branch-1')
+        target_branch = repo.bzrdir.create_branch()
+        self.assertEqual([None], repo.bzrdir.get_branches().keys())
+
     def test_create_repository(self):
         # a bzrdir can construct a repository for itself.
-        if not self.bzrdir_format.is_supported():
+        if not self.bzrdir_format.is_initializable():
             # unsupported formats are not loopback testable
             # because the default open will not open them and
             # they may not be initializable.
-            return
+            raise TestNotApplicable("format is not initializable")
         t = self.get_transport()
         made_control = self.bzrdir_format.initialize(t.base)
         made_repo = made_control.create_repository()
@@ -1188,11 +1214,11 @@
     def test_create_repository_shared(self):
         # a bzrdir can create a shared repository or
         # fail appropriately
-        if not self.bzrdir_format.is_supported():
+        if not self.bzrdir_format.is_initializable():
             # unsupported formats are not loopback testable
             # because the default open will not open them and
             # they may not be initializable.
-            return
+            raise TestNotApplicable("format is not initializable")
         t = self.get_transport()
         made_control = self.bzrdir_format.initialize(t.base)
         try:
@@ -1200,27 +1226,34 @@
         except errors.IncompatibleFormat:
             # Old bzrdir formats don't support shared repositories
             # and should raise IncompatibleFormat
-            return
+            raise TestNotApplicable("format does not support shared "
+                "repositories")
         self.assertTrue(made_repo.is_shared())
 
     def test_create_repository_nonshared(self):
         # a bzrdir can create a non-shared repository
-        if not self.bzrdir_format.is_supported():
+        if not self.bzrdir_format.is_initializable():
             # unsupported formats are not loopback testable
             # because the default open will not open them and
             # they may not be initializable.
-            return
+            raise TestNotApplicable("format is not initializable")
         t = self.get_transport()
         made_control = self.bzrdir_format.initialize(t.base)
-        made_repo = made_control.create_repository(shared=False)
+        try:
+            made_repo = made_control.create_repository(shared=False)
+        except errors.IncompatibleFormat:
+            # Some control dir formats don't support non-shared repositories
+            # and should raise IncompatibleFormat
+            raise TestNotApplicable("format does not support shared "
+                "repositories")
         self.assertFalse(made_repo.is_shared())
 
     def test_open_repository(self):
-        if not self.bzrdir_format.is_supported():
+        if not self.bzrdir_format.is_initializable():
             # unsupported formats are not loopback testable
             # because the default open will not open them and
             # they may not be initializable.
-            return
+            raise TestNotApplicable("format is not initializable")
         t = self.get_transport()
         made_control = self.bzrdir_format.initialize(t.base)
         made_repo = made_control.create_repository()
@@ -1231,11 +1264,11 @@
 
     def test_create_workingtree(self):
         # a bzrdir can construct a working tree for itself.
-        if not self.bzrdir_format.is_supported():
+        if not self.bzrdir_format.is_initializable():
             # unsupported formats are not loopback testable
             # because the default open will not open them and
             # they may not be initializable.
-            return
+            raise TestNotApplicable("format is not initializable")
         t = self.get_transport()
         made_control = self.bzrdir_format.initialize(t.base)
         made_repo = made_control.create_repository()
@@ -1246,6 +1279,8 @@
 
     def test_create_workingtree_revision(self):
         # a bzrdir can construct a working tree for itself @ a specific revision.
+        if not self.bzrdir_format.is_initializable():
+            raise TestNotApplicable("format is not initializable")
         t = self.get_transport()
         source = self.make_branch_and_tree('source')
         source.commit('a', rev_id='a', allow_pointless=True)
@@ -1262,11 +1297,8 @@
         self.assertEqual(['a'], made_tree.get_parent_ids())
 
     def test_open_workingtree(self):
-        if not self.bzrdir_format.is_supported():
-            # unsupported formats are not loopback testable
-            # because the default open will not open them and
-            # they may not be initializable.
-            return
+        if not self.bzrdir_format.is_initializable():
+            raise TestNotApplicable("format is not initializable")
         # this has to be tested with local access as we still support creating
         # format 6 bzrdirs
         t = self.get_transport()
@@ -1286,11 +1318,8 @@
     def test_get_selected_branch(self):
         # The segment parameters are accessible from the root transport
         # if a URL with segment parameters is opened.
-        if not self.bzrdir_format.is_supported():
-            # unsupported formats are not loopback testable
-            # because the default open will not open them and
-            # they may not be initializable.
-            return
+        if not self.bzrdir_format.is_initializable():
+            raise TestNotApplicable("format is not initializable")
         t = self.get_transport()
         try:
             made_control = self.bzrdir_format.initialize(t.base)
@@ -1304,11 +1333,8 @@
 
     def test_get_selected_branch_none_selected(self):
         # _get_selected_branch defaults to None
-        if not self.bzrdir_format.is_supported():
-            # unsupported formats are not loopback testable
-            # because the default open will not open them and
-            # they may not be initializable.
-            return
+        if not self.bzrdir_format.is_initializable():
+            raise TestNotApplicable("format is not initializable")
         t = self.get_transport()
         try:
             made_control = self.bzrdir_format.initialize(t.base)
@@ -1330,7 +1356,9 @@
             repo = self.make_repository('.', shared=True)
         except errors.IncompatibleFormat:
             # need a shared repository to test this.
-            return
+            raise TestNotApplicable("requires shared repository support")
+        if not repo._format.supports_nesting_repositories:
+            raise TestNotApplicable("requires nesting repositories")
         url = self.get_url('intermediate')
         t = self.get_transport()
         t.mkdir('intermediate')
@@ -1356,7 +1384,11 @@
             repo = self.make_repository('.', shared=True)
         except errors.IncompatibleFormat:
             # need a shared repository to test this.
-            return
+            raise TestNotApplicable("requires format with shared repository "
+                "support")
+        if not repo._format.supports_nesting_repositories:
+            raise TestNotApplicable("requires support for nesting "
+                "repositories")
         url = self.get_url('childbzrdir')
         self.get_transport().mkdir('childbzrdir')
         made_control = self.bzrdir_format.initialize(url)
@@ -1372,12 +1404,17 @@
                          found_repo.bzrdir.root_transport.base)
 
     def test_find_repository_standalone_with_containing_shared_repository(self):
-        # find repo inside a standalone repo inside a shared repo finds the standalone repo
+        # find repo inside a standalone repo inside a shared repo finds the
+        # standalone repo
         try:
             containing_repo = self.make_repository('.', shared=True)
         except errors.IncompatibleFormat:
             # need a shared repository to test this.
-            return
+            raise TestNotApplicable("requires support for shared "
+                "repositories")
+        if not containing_repo._format.supports_nesting_repositories:
+            raise TestNotApplicable("format does not support "
+                "nesting repositories")
         child_repo = self.make_repository('childrepo')
         opened_control = bzrdir.BzrDir.open(self.get_url('childrepo'))
         found_repo = opened_control.find_repository()
@@ -1390,7 +1427,11 @@
             containing_repo = self.make_repository('.', shared=True)
         except errors.IncompatibleFormat:
             # need a shared repository to test this.
-            return
+            raise TestNotApplicable("requires support for shared "
+                "repositories")
+        if not containing_repo._format.supports_nesting_repositories:
+            raise TestNotApplicable("requires support for nesting "
+                "repositories")
         url = self.get_url('childrepo')
         self.get_transport().mkdir('childrepo')
         child_control = self.bzrdir_format.initialize(url)
@@ -1409,7 +1450,11 @@
             repo = self.make_repository('.', shared=True)
         except errors.IncompatibleFormat:
             # need a shared repository to test this.
-            return
+            raise TestNotApplicable("requires support for shared "
+                "repositories")
+        if not repo._format.supports_nesting_repositories:
+            raise TestNotApplicable("requires support for nesting "
+                "repositories")
         url = self.get_url('intermediate')
         t = self.get_transport()
         t.mkdir('intermediate')
@@ -1513,7 +1558,7 @@
             # and thus this interaction cannot be tested at the interface
             # level.
             repo.unlock()
-            return
+            raise TestNotApplicable("format does not physically lock")
         # only one yes needed here: it should only be unlocking
         # the repo
         bzrlib.ui.ui_factory = CannedInputUIFactory([True])
@@ -1522,7 +1567,7 @@
         except NotImplementedError:
             # this bzrdir does not implement break_lock - so we cant test it.
             repo.unlock()
-            return
+            raise TestNotApplicable("format does not support breaking locks")
         lock_repo.lock_write()
         lock_repo.unlock()
         self.assertRaises(errors.LockBroken, repo.unlock)
@@ -1539,7 +1584,8 @@
             bzrlib.branch.BranchReferenceFormat().initialize(
                 thisdir, target_branch=master)
         except errors.IncompatibleFormat:
-            return
+            raise TestNotApplicable("format does not support "
+                "branch references")
         unused_repo = thisdir.create_repository()
         master.lock_write()
         unused_repo.lock_write()
@@ -1592,7 +1638,7 @@
             # raised a LockActive because we do still have a live locked
             # object.
             tree.unlock()
-            return
+            raise TestNotApplicable("format does not support breaking locks")
         self.assertEqual([True],
                 bzrlib.ui.ui_factory.responses)
         lock_tree = tree.bzrdir.open_workingtree()
@@ -1628,11 +1674,11 @@
     def test_find_repository_no_repository(self):
         # loopback test to check the current format fails to find a
         # share repository correctly.
-        if not self.bzrdir_format.is_supported():
+        if not self.bzrdir_format.is_initializable():
             # unsupported formats are not loopback testable
             # because the default open will not open them and
             # they may not be initializable.
-            return
+            raise TestNotApplicable("format is not initializable")
         # supported formats must be able to init and open
         # - do the vfs initialisation over the basic vfs transport
         # XXX: TODO this should become a 'bzrdirlocation' api call.

=== modified file 'bzrlib/tests/per_controldir/test_format.py'
--- bzrlib/tests/per_controldir/test_format.py	2011-03-22 12:10:34 +0000
+++ bzrlib/tests/per_controldir/test_format.py	2011-10-10 12:47:25 +0000
@@ -35,6 +35,10 @@
     def test_upgrade_recommended(self):
         self.assertIsInstance(self.bzrdir_format.upgrade_recommended, bool)
 
+    def test_supports_transport(self):
+        self.assertIsInstance(
+            self.bzrdir_format.supports_transport(self.get_transport()), bool)
+
     def test_check_support_status(self):
         if not self.bzrdir_format.is_supported():
             self.assertRaises(errors.UnsupportedFormatError,

=== modified file 'bzrlib/tests/per_controldir_colo/test_supported.py'
--- bzrlib/tests/per_controldir_colo/test_supported.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/per_controldir_colo/test_supported.py	2011-12-12 12:08:58 +0000
@@ -16,15 +16,18 @@
 
 """Tests for bzr directories that support colocated branches."""
 
-import bzrlib.branch
+from bzrlib.branch import Branch
 from bzrlib import (
     errors,
     tests,
-    transport,
+    urlutils,
     )
 from bzrlib.tests import (
     per_controldir,
     )
+from bzrlib.tests.features import (
+    UnicodeFilenameFeature,
+    )
 
 
 class TestColocatedBranchSupport(per_controldir.TestCaseWithControlDir):
@@ -53,7 +56,81 @@
         except errors.UninitializableFormat:
             raise tests.TestNotApplicable(
                 'Control dir does not support creating new branches.')
-        made_repo = made_control.create_repository()
+        made_control.create_repository()
         made_branch = made_control.create_branch("colo")
-        self.assertIsInstance(made_branch, bzrlib.branch.Branch)
-        self.assertEqual(made_control, made_branch.bzrdir)
+        self.assertIsInstance(made_branch, Branch)
+        self.assertEquals("colo", made_branch.name)
+        self.assertEqual(made_control, made_branch.bzrdir)
+
+    def test_open_by_url(self):
+        # a bzrdir can construct a branch and repository for itself.
+        if not self.bzrdir_format.is_supported():
+            # unsupported formats are not loopback testable
+            # because the default open will not open them and
+            # they may not be initializable.
+            raise tests.TestNotApplicable('Control dir format not supported')
+        t = self.get_transport()
+        try:
+            made_control = self.bzrdir_format.initialize(t.base)
+        except errors.UninitializableFormat:
+            raise tests.TestNotApplicable(
+                'Control dir does not support creating new branches.')
+        made_control.create_repository()
+        made_branch = made_control.create_branch(name="colo")
+        other_branch = made_control.create_branch(name="othercolo")
+        self.assertIsInstance(made_branch, Branch)
+        self.assertEqual(made_control, made_branch.bzrdir)
+        self.assertNotEqual(made_branch.user_url, other_branch.user_url)
+        self.assertNotEqual(made_branch.control_url, other_branch.control_url)
+        re_made_branch = Branch.open(made_branch.user_url)
+        self.assertEquals(re_made_branch.name, "colo")
+        self.assertEqual(made_branch.control_url, re_made_branch.control_url)
+        self.assertEqual(made_branch.user_url, re_made_branch.user_url)
+
+    def test_sprout_into_colocated(self):
+        # a bzrdir can construct a branch and repository for itself.
+        if not self.bzrdir_format.is_supported():
+            # unsupported formats are not loopback testable
+            # because the default open will not open them and
+            # they may not be initializable.
+            raise tests.TestNotApplicable('Control dir format not supported')
+        from_tree = self.make_branch_and_tree('from')
+        revid = from_tree.commit("rev1")
+        try:
+            other_branch = self.make_branch("to")
+        except errors.UninitializableFormat:
+            raise tests.TestNotApplicable(
+                'Control dir does not support creating new branches.')
+        to_dir = from_tree.bzrdir.sprout(
+            urlutils.join_segment_parameters(
+                other_branch.bzrdir.user_url, {"branch": "target"}))
+        to_branch = to_dir.open_branch(name="target")
+        self.assertEquals(revid, to_branch.last_revision())
+
+    def test_unicode(self):
+        self.requireFeature(UnicodeFilenameFeature)
+        if not self.bzrdir_format.is_supported():
+            # unsupported formats are not loopback testable
+            # because the default open will not open them and
+            # they may not be initializable.
+            raise tests.TestNotApplicable('Control dir format not supported')
+        t = self.get_transport()
+        try:
+            made_control = self.bzrdir_format.initialize(t.base)
+        except errors.UninitializableFormat:
+            raise tests.TestNotApplicable(
+                'Control dir does not support creating new branches.')
+        made_control.create_repository()
+        made_branch = made_control.create_branch(name=u"col\xe9")
+        self.assertTrue(
+            u"col\xe9" in [b.name for b in made_control.list_branches()])
+        made_branch = Branch.open(made_branch.user_url)
+        self.assertEquals(u"col\xe9", made_branch.name)
+        made_control.destroy_branch(u"col\xe9")
+
+    def test_get_branches(self):
+        repo = self.make_repository('branch-1')
+        target_branch = repo.bzrdir.create_branch(name='foo')
+        self.assertEqual(['foo'], repo.bzrdir.get_branches().keys())
+        self.assertEqual(target_branch.base,
+                         repo.bzrdir.get_branches()['foo'].base)

=== modified file 'bzrlib/tests/per_controldir_colo/test_unsupported.py'
--- bzrlib/tests/per_controldir_colo/test_unsupported.py	2011-03-05 03:53:34 +0000
+++ bzrlib/tests/per_controldir_colo/test_unsupported.py	2011-12-11 04:07:08 +0000
@@ -23,7 +23,6 @@
 from bzrlib import (
     errors,
     tests,
-    transport,
     )
 from bzrlib.tests import (
     per_controldir,
@@ -40,7 +39,10 @@
             # they may not be initializable.
             raise tests.TestNotApplicable('Control dir format not supported')
         t = self.get_transport()
-        made_control = self.bzrdir_format.initialize(t.base)
+        try:
+            made_control = self.bzrdir_format.initialize(t.base)
+        except errors.UninitializableFormat:
+            raise tests.TestNotApplicable('Control dir format not initializable')
         made_repo = made_control.create_repository()
         return made_control
 
@@ -66,3 +68,9 @@
         made_control = self.make_bzrdir_with_repo()
         self.assertRaises(errors.NoColocatedBranchSupport,
             made_control.get_branch_reference, "colo")
+
+    def test_get_branches(self):
+        made_control = self.make_bzrdir_with_repo()
+        made_control.create_branch()
+        self.assertEqual(made_control.get_branches().keys(),
+                         [None])

=== modified file 'bzrlib/tests/per_interbranch/__init__.py'
--- bzrlib/tests/per_interbranch/__init__.py	2011-04-27 13:26:31 +0000
+++ bzrlib/tests/per_interbranch/__init__.py	2011-10-03 23:12:01 +0000
@@ -27,16 +27,11 @@
 
 from bzrlib import (
     branchbuilder,
-    memorytree,
     )
 from bzrlib.branch import (
                            GenericInterBranch,
                            InterBranch,
                            )
-from bzrlib.bzrdir import (
-    BzrDirFormat,
-    BzrDirMetaFormat1,
-    )
 from bzrlib.tests import (
     TestCaseWithTransport,
     multiply_tests,
@@ -81,59 +76,79 @@
 class TestCaseWithInterBranch(TestCaseWithTransport):
 
     def make_from_branch(self, relpath):
-        repo = self.make_repository(relpath, format=self.branch_format_from._matchingbzrdir)
-        return self.branch_format_from.initialize(repo.bzrdir)
+        return self.make_branch(relpath, format=self.branch_format_from._matchingbzrdir)
 
     def make_from_branch_and_memory_tree(self, relpath):
         """Create a branch on the default transport and a MemoryTree for it."""
-        b = self.make_from_branch(relpath)
-        return memorytree.MemoryTree.create_on_branch(b)
+        self.assertEquals(
+            self.branch_format_from._matchingbzrdir.get_branch_format(),
+            self.branch_format_from)
+        return self.make_branch_and_memory_tree(
+            relpath, format=self.branch_format_from._matchingbzrdir)
 
     def make_from_branch_and_tree(self, relpath):
         """Create a branch on the default transport and a working tree for it."""
-        b = self.make_from_branch(relpath)
-        return b.bzrdir.create_workingtree()
+        self.assertEquals(
+            self.branch_format_from._matchingbzrdir.get_branch_format(),
+            self.branch_format_from)
+        return self.make_branch_and_tree(relpath,
+            format=self.branch_format_from._matchingbzrdir)
 
     def make_from_branch_builder(self, relpath):
-        default_format = BzrDirFormat.get_default_format()
-        format = BzrDirMetaFormat1()
-        format.set_branch_format(self.branch_format_from)
-        format.repository_format = default_format.repository_format
-        format.workingtree_format = default_format.workingtree_format
+        self.assertEquals(
+            self.branch_format_from._matchingbzrdir.get_branch_format(),
+            self.branch_format_from)
         return branchbuilder.BranchBuilder(self.get_transport(relpath),
-            format=format)
+            format=self.branch_format_from._matchingbzrdir)
 
     def make_to_branch(self, relpath):
-        repo = self.make_repository(relpath, format=self.branch_format_to._matchingbzrdir)
-        return self.branch_format_to.initialize(repo.bzrdir)
+        self.assertEquals(
+            self.branch_format_to._matchingbzrdir.get_branch_format(),
+            self.branch_format_to)
+        return self.make_branch(relpath, format=self.branch_format_to._matchingbzrdir)
 
     def make_to_branch_and_memory_tree(self, relpath):
         """Create a branch on the default transport and a MemoryTree for it."""
-        b = self.make_to_branch(relpath)
-        return memorytree.MemoryTree.create_on_branch(b)
+        self.assertEquals(
+            self.branch_format_to._matchingbzrdir.get_branch_format(),
+            self.branch_format_to)
+        return self.make_branch_and_memory_tree(
+            relpath, format=self.branch_format_to._matchingbzrdir)
 
     def make_to_branch_and_tree(self, relpath):
         """Create a branch on the default transport and a working tree for it."""
-        b = self.make_to_branch(relpath)
-        return b.bzrdir.create_workingtree()
+        self.assertEquals(
+            self.branch_format_to._matchingbzrdir.get_branch_format(),
+            self.branch_format_to)
+        return self.make_branch_and_tree(relpath,
+            format=self.branch_format_to._matchingbzrdir)
+
+    def _sprout(self, origdir, to_url, format):
+        if format.supports_workingtrees:
+            newbranch = self.make_branch(to_url, format=format)
+        else:
+            newbranch = self.make_branch(to_url+".branch", format=format)
+        origbranch = origdir.open_branch()
+        newbranch.repository.fetch(origbranch.repository)
+        origbranch.copy_content_into(newbranch)
+        if format.supports_workingtrees:
+            wt = newbranch.bzrdir.create_workingtree()
+        else:
+            wt = newbranch.create_checkout(to_url, lightweight=True)
+        return wt
 
     def sprout_to(self, origdir, to_url):
         """Sprout a bzrdir, using to_format for the new branch."""
-        newbranch = self.make_to_branch(to_url)
-        origbranch = origdir.open_branch()
-        newbranch.repository.fetch(origbranch.repository)
-        origbranch.copy_content_into(newbranch)
-        newbranch.bzrdir.create_workingtree()
-        return newbranch.bzrdir
+        wt = self._sprout(origdir, to_url, self.branch_format_to._matchingbzrdir)
+        self.assertEquals(wt.branch._format, self.branch_format_to)
+        return wt.bzrdir
 
     def sprout_from(self, origdir, to_url):
         """Sprout a bzrdir, using from_format for the new bzrdir."""
-        newbranch = self.make_from_branch(to_url)
-        origbranch = origdir.open_branch()
-        newbranch.repository.fetch(origbranch.repository)
-        origbranch.copy_content_into(newbranch)
-        newbranch.bzrdir.create_workingtree()
-        return newbranch.bzrdir
+        wt = self._sprout(origdir, to_url,
+            self.branch_format_from._matchingbzrdir)
+        self.assertEquals(wt.branch._format, self.branch_format_from)
+        return wt.bzrdir
 
 
 class StubWithFormat(object):

=== modified file 'bzrlib/tests/per_interbranch/test_pull.py'
--- bzrlib/tests/per_interbranch/test_pull.py	2010-06-17 09:23:19 +0000
+++ bzrlib/tests/per_interbranch/test_pull.py	2011-10-17 08:45:09 +0000
@@ -16,8 +16,6 @@
 
 """Tests for InterBranch.pull behaviour."""
 
-import os
-
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import BzrDir
 from bzrlib import errors
@@ -42,7 +40,7 @@
         parent.merge_from_branch(mine.branch)
         parent.commit('merge my change', rev_id='P2')
         mine.pull(parent.branch)
-        self.assertEqual(['P1', 'P2'], mine.branch.revision_history())
+        self.assertEqual('P2', mine.branch.last_revision())
 
     def test_pull_merged_indirect(self):
         # it should be possible to do a pull from one branch into another
@@ -59,7 +57,7 @@
         parent.merge_from_branch(other.branch)
         parent.commit('merge other', rev_id='P2')
         mine.pull(parent.branch)
-        self.assertEqual(['P1', 'P2'], mine.branch.revision_history())
+        self.assertEqual('P2', mine.branch.last_revision())
 
     def test_pull_updates_checkout_and_master(self):
         """Pulling into a checkout updates the checkout and the master branch"""
@@ -70,8 +68,8 @@
         rev2 = other.commit('other commit')
         # now pull, which should update both checkout and master.
         checkout.branch.pull(other.branch)
-        self.assertEqual([rev1, rev2], checkout.branch.revision_history())
-        self.assertEqual([rev1, rev2], master_tree.branch.revision_history())
+        self.assertEqual(rev2, checkout.branch.last_revision())
+        self.assertEqual(rev2, master_tree.branch.last_revision())
 
     def test_pull_raises_specific_error_on_master_connection_error(self):
         master_tree = self.make_from_branch_and_tree('master')
@@ -79,7 +77,7 @@
         other = self.sprout_to(master_tree.branch.bzrdir, 'other').open_branch()
         # move the branch out of the way on disk to cause a connection
         # error.
-        os.rename('master', 'master_gone')
+        master_tree.branch.bzrdir.destroy_branch()
         # try to pull, which should raise a BoundBranchConnectionFailure.
         self.assertRaises(errors.BoundBranchConnectionFailure,
                 checkout.branch.pull, other)
@@ -99,7 +97,7 @@
         self.assertEqual('P1', result.old_revid)
         self.assertEqual(2, result.new_revno)
         self.assertEqual('M1', result.new_revid)
-        self.assertEqual(None, result.tag_conflicts)
+        self.assertEqual([], result.tag_conflicts)
 
     def test_pull_overwrite(self):
         tree_a = self.make_from_branch_and_tree('tree_a')
@@ -112,14 +110,15 @@
                           tree_a.branch.pull, tree_b.branch,
                           overwrite=False, stop_revision='rev2b')
         # It should not have updated the branch tip, but it should have fetched
-        # the revision
+        # the revision if the repository supports "invisible" revisions.
         self.assertEqual('rev2a', tree_a.branch.last_revision())
-        self.assertTrue(tree_a.branch.repository.has_revision('rev2b'))
+        if tree_a.branch.repository._format.supports_unreferenced_revisions:
+            self.assertTrue(tree_a.branch.repository.has_revision('rev2b'))
         tree_a.branch.pull(tree_b.branch, overwrite=True,
                            stop_revision='rev2b')
         self.assertEqual('rev2b', tree_a.branch.last_revision())
-        self.assertEqual(tree_b.branch.revision_history(),
-                         tree_a.branch.revision_history())
+        self.assertEqual(tree_b.branch.last_revision(),
+                         tree_a.branch.last_revision())
 
 
 class TestPullHook(TestCaseWithInterBranch):

=== modified file 'bzrlib/tests/per_interbranch/test_push.py'
--- bzrlib/tests/per_interbranch/test_push.py	2011-05-17 21:58:49 +0000
+++ bzrlib/tests/per_interbranch/test_push.py	2011-12-11 15:42:02 +0000
@@ -17,7 +17,6 @@
 """Tests for branch.push behaviour."""
 
 from cStringIO import StringIO
-import os
 
 from testtools.matchers import (
     Equals,
@@ -61,7 +60,7 @@
         mine.merge_from_branch(other.branch)
         mine.commit('merge my change', rev_id='P2')
         result = mine.branch.push(other.branch)
-        self.assertEqual(['P1', 'P2'], other.branch.revision_history())
+        self.assertEqual('P2', other.branch.last_revision())
         # result object contains some structured data
         self.assertEqual(result.old_revid, 'M1')
         self.assertEqual(result.new_revid, 'P2')
@@ -86,7 +85,7 @@
         mine.merge_from_branch(other.branch)
         mine.commit('merge other', rev_id='P2')
         mine.branch.push(target.branch)
-        self.assertEqual(['P1', 'P2'], target.branch.revision_history())
+        self.assertEqual('P2', target.branch.last_revision())
 
     def test_push_to_checkout_updates_master(self):
         """Pushing into a checkout updates the checkout and the master branch"""
@@ -104,8 +103,8 @@
         rev2 = other.commit('other commit')
         # now push, which should update both checkout and master.
         other.branch.push(checkout.branch)
-        self.assertEqual([rev1, rev2], checkout.branch.revision_history())
-        self.assertEqual([rev1, rev2], master_tree.branch.revision_history())
+        self.assertEqual(rev2, checkout.branch.last_revision())
+        self.assertEqual(rev2, master_tree.branch.last_revision())
 
     def test_push_raises_specific_error_on_master_connection_error(self):
         master_tree = self.make_to_branch_and_tree('master')
@@ -119,7 +118,7 @@
         other = other_bzrdir.open_workingtree()
         # move the branch out of the way on disk to cause a connection
         # error.
-        os.rename('master', 'master_gone')
+        master_tree.bzrdir.destroy_branch()
         # try to push, which should raise a BoundBranchConnectionFailure.
         self.assertRaises(errors.BoundBranchConnectionFailure,
                 other.branch.push, checkout.branch)
@@ -159,6 +158,9 @@
             return
         try:
             tree = a_branch.bzrdir.create_workingtree()
+        except errors.UnsupportedOperation:
+            self.assertFalse(a_branch.bzrdir._format.supports_workingtrees)
+            tree = a_branch.create_checkout('repo/tree', lightweight=True)
         except errors.NotLocalUrl:
             if self.vfs_transport_factory is test_server.LocalURLServer:
                 # the branch is colocated on disk, we cannot create a checkout.
@@ -277,10 +279,10 @@
         # remote graph any further.
         bzr_core_trace = Equals(
             ['Repository.insert_stream_1.19', 'Repository.insert_stream_1.19',
-             'get', 'Branch.set_last_revision_info', 'Branch.unlock'])
+             'Branch.set_last_revision_info', 'Branch.unlock'])
         bzr_loom_trace = Equals(
             ['Repository.insert_stream_1.19', 'Repository.insert_stream_1.19',
-             'get', 'Branch.set_last_revision_info', 'get', 'Branch.unlock'])
+             'Branch.set_last_revision_info', 'get', 'Branch.unlock'])
         self.assertThat(calls_after_insert_stream,
             MatchesAny(bzr_core_trace, bzr_loom_trace))
 
@@ -367,7 +369,7 @@
         target.add('')
         rev1 = target.commit('rev 1')
         target.unlock()
-        sourcedir = target.bzrdir.clone(self.get_url('source'))
+        sourcedir = target.branch.bzrdir.clone(self.get_url('source'))
         source = MemoryTree.create_on_branch(sourcedir.open_branch())
         rev2 = source.commit('rev 2')
         Branch.hooks.install_named_hook('post_push',

=== modified file 'bzrlib/tests/per_interrepository/__init__.py'
--- bzrlib/tests/per_interrepository/__init__.py	2011-05-02 15:29:58 +0000
+++ bzrlib/tests/per_interrepository/__init__.py	2011-09-30 12:31:07 +0000
@@ -159,6 +159,12 @@
         if self.extra_setup:
             self.extra_setup(self)
 
+    def get_default_format(self):
+        self.assertEquals(
+            self.repository_format._matchingbzrdir.repository_format,
+            self.repository_format)
+        return self.repository_format._matchingbzrdir
+
     def make_branch(self, relpath, format=None):
         repo = self.make_repository(relpath, format=format)
         return repo.bzrdir.create_branch()

=== modified file 'bzrlib/tests/per_interrepository/test_fetch.py'
--- bzrlib/tests/per_interrepository/test_fetch.py	2011-05-08 13:07:11 +0000
+++ bzrlib/tests/per_interrepository/test_fetch.py	2011-12-05 14:12:23 +0000
@@ -27,7 +27,7 @@
 from bzrlib.errors import (
     NoSuchRevision,
     )
-from bzrlib.graph import (
+from bzrlib.vf_search import (
     SearchResult,
     )
 from bzrlib.revision import (

=== modified file 'bzrlib/tests/per_merger.py'
--- bzrlib/tests/per_merger.py	2011-01-12 01:01:53 +0000
+++ bzrlib/tests/per_merger.py	2011-12-05 14:21:55 +0000
@@ -22,7 +22,6 @@
 from bzrlib import (
     errors,
     merge as _mod_merge,
-    option,
     )
 from bzrlib.tests import (
     multiply_tests,
@@ -38,7 +37,7 @@
     result = loader.suiteClass()
     scenarios = [
         (name, {'merge_type': merger})
-        for name, merger in option._merge_type_registry.items()]
+        for name, merger in _mod_merge.merge_type_registry.items()]
     return multiply_tests(standard_tests, scenarios, result)
 
 
@@ -175,17 +174,33 @@
         transform.finalize()
         return (limbodir, deletiondir)
 
-    def test_merge_with_existing_limbo(self):
-        wt = self.make_branch_and_tree('this')
-        (limbodir, deletiondir) =  self.get_limbodir_deletiondir(wt)
-        os.mkdir(limbodir)
+    def test_merge_with_existing_limbo_empty(self):
+        """Empty limbo dir is just cleaned up - see bug 427773"""
+        wt = self.make_branch_and_tree('this')
+        (limbodir, deletiondir) =  self.get_limbodir_deletiondir(wt)
+        os.mkdir(limbodir)
+        self.do_merge(wt, wt)
+
+    def test_merge_with_existing_limbo_non_empty(self):
+        wt = self.make_branch_and_tree('this')
+        (limbodir, deletiondir) =  self.get_limbodir_deletiondir(wt)
+        os.mkdir(limbodir)
+        os.mkdir(os.path.join(limbodir, 'something'))
         self.assertRaises(errors.ExistingLimbo, self.do_merge, wt, wt)
         self.assertRaises(errors.LockError, wt.unlock)
 
-    def test_merge_with_pending_deletion(self):
-        wt = self.make_branch_and_tree('this')
-        (limbodir, deletiondir) =  self.get_limbodir_deletiondir(wt)
-        os.mkdir(deletiondir)
+    def test_merge_with_pending_deletion_empty(self):
+        wt = self.make_branch_and_tree('this')
+        (limbodir, deletiondir) =  self.get_limbodir_deletiondir(wt)
+        os.mkdir(deletiondir)
+        self.do_merge(wt, wt)
+
+    def test_merge_with_pending_deletion_non_empty(self):
+        """Also see bug 427773"""
+        wt = self.make_branch_and_tree('this')
+        (limbodir, deletiondir) =  self.get_limbodir_deletiondir(wt)
+        os.mkdir(deletiondir)
+        os.mkdir(os.path.join(deletiondir, 'something'))
         self.assertRaises(errors.ExistingPendingDeletion, self.do_merge, wt, wt)
         self.assertRaises(errors.LockError, wt.unlock)
 

=== modified file 'bzrlib/tests/per_repository/__init__.py'
--- bzrlib/tests/per_repository/__init__.py	2011-05-16 12:49:26 +0000
+++ bzrlib/tests/per_repository/__init__.py	2011-10-10 14:03:32 +0000
@@ -25,6 +25,7 @@
 """
 
 from bzrlib import (
+    bzrdir,
     repository,
     )
 from bzrlib.remote import RemoteRepositoryFormat
@@ -92,18 +93,20 @@
 
 class TestCaseWithRepository(TestCaseWithControlDir):
 
-    def make_repository(self, relpath, shared=False, format=None):
-        if format is None:
+    def get_default_format(self):
+        format = self.repository_format._matchingbzrdir
+        self.assertEquals(format.repository_format, self.repository_format)
+        return format
+
+    def make_repository(self, relpath, shared=None, format=None):
+        format = self.resolve_format(format)
+        repo = super(TestCaseWithRepository, self).make_repository(
+            relpath, shared=shared, format=format)
+        if format is None or format.repository_format is self.repository_format:
             # Create a repository of the type we are trying to test.
-            made_control = self.make_bzrdir(relpath)
-            repo = self.repository_format.initialize(made_control,
-                    shared=shared)
             if getattr(self, "repository_to_test_repository", None):
                 repo = self.repository_to_test_repository(repo)
-            return repo
-        else:
-            return super(TestCaseWithRepository, self).make_repository(
-                relpath, shared=shared, format=format)
+        return repo
 
 
 def load_tests(standard_tests, module, loader):
@@ -114,19 +117,18 @@
         'test_check',
         'test_commit_builder',
         'test_fetch',
-        'test_fileid_involved',
         'test_file_graph',
         'test_get_parent_map',
         'test_has_same_location',
         'test_has_revisions',
         'test_iter_reverse_revision_history',
         'test_locking',
-        'test_merge_directive',
         'test_pack',
         'test_reconcile',
         'test_refresh_data',
         'test_repository',
         'test_revision',
+        'test_signatures',
         'test_statistics',
         'test_write_group',
         ]

=== modified file 'bzrlib/tests/per_repository/test_check.py'
--- bzrlib/tests/per_repository/test_check.py	2011-05-18 18:49:43 +0000
+++ bzrlib/tests/per_repository/test_check.py	2011-12-09 13:00:30 +0000
@@ -18,43 +18,25 @@
 """Test operations that check the repository for corruption"""
 
 from bzrlib import (
-    config as _mod_config,
     revision as _mod_revision,
     )
 from bzrlib.tests.per_repository import TestCaseWithRepository
 
 
-class TestNoSpuriousInconsistentAncestors(TestCaseWithRepository):
-
-    def test_two_files_different_versions_no_inconsistencies_bug_165071(self):
-        """Two files, with different versions can be clean."""
-        tree = self.make_branch_and_tree('.')
-        self.build_tree(['foo'])
-        tree.smart_add(['.'])
-        revid1 = tree.commit('1')
-        self.build_tree(['bar'])
-        tree.smart_add(['.'])
-        revid2 = tree.commit('2')
-        check_object = tree.branch.repository.check([revid1, revid2])
-        check_object.report_results(verbose=True)
-        self.assertContainsRe(self.get_log(), "0 unreferenced text versions")
-
-
 class TestCleanRepository(TestCaseWithRepository):
 
     def test_new_repo(self):
-        repo = self.make_repository('foo')
-        repo.lock_write()
-        self.addCleanup(repo.unlock)
-        config = _mod_config.Config()
+        branch = self.make_branch('foo')
+        branch.lock_write()
+        self.addCleanup(branch.unlock)
         self.overrideEnv('BZR_EMAIL', 'foo@sample.com')
-        builder = repo.get_commit_builder(None, [], config)
+        builder = branch.get_commit_builder([], branch.get_config_stack())
         list(builder.record_iter_changes(None, _mod_revision.NULL_REVISION, [
             ('TREE_ROOT', (None, ''), True, (False, True), (None, None),
             (None, ''), (None, 'directory'), (None, False))]))
         builder.finish_inventory()
         rev_id = builder.commit('first post')
-        result = repo.check(None, check_repo=True)
+        result = branch.repository.check(None, check_repo=True)
         result.report_results(True)
         log = self.get_log()
         self.assertFalse('Missing' in log, "Something was missing in %r" % log)

=== modified file 'bzrlib/tests/per_repository/test_commit_builder.py'
--- bzrlib/tests/per_repository/test_commit_builder.py	2011-08-16 12:59:04 +0000
+++ bzrlib/tests/per_repository/test_commit_builder.py	2011-12-22 16:17:28 +0000
@@ -39,7 +39,7 @@
         branch = self.make_branch('.')
         branch.repository.lock_write()
         builder = branch.repository.get_commit_builder(
-            branch, [], branch.get_config())
+            branch, [], branch.get_config_stack())
         self.assertIsInstance(builder, repository.CommitBuilder)
         self.assertTrue(builder.random_revid)
         branch.repository.commit_write_group()
@@ -148,6 +148,21 @@
         rev = tree.branch.repository.get_revision(rev_id)
         self.assertEqual('foo bar blah', rev.message)
 
+    def test_updates_branch(self):
+        tree = self.make_branch_and_tree(".")
+        tree.lock_write()
+        try:
+            builder = tree.branch.get_commit_builder([])
+            list(builder.record_iter_changes(tree, tree.last_revision(),
+                tree.iter_changes(tree.basis_tree())))
+            builder.finish_inventory()
+            will_update_branch = builder.updates_branch
+            rev_id = builder.commit('might update the branch')
+        finally:
+            tree.unlock()
+        actually_updated_branch = (tree.branch.last_revision() == rev_id)
+        self.assertEquals(actually_updated_branch, will_update_branch)
+
     def test_commit_with_revision_id_record_entry_contents(self):
         tree = self.make_branch_and_tree(".")
         tree.lock_write()
@@ -218,7 +233,7 @@
         # on deserialisation, but thats all the current contract guarantees
         # anyway.
         self.assertEqual(revision_id,
-            tree.branch.repository.get_inventory(revision_id).revision_id)
+            tree.branch.repository.revision_tree(revision_id).get_revision_id())
 
     def test_commit_without_root_errors(self):
         tree = self.make_branch_and_tree(".")
@@ -233,6 +248,8 @@
                 except:
                     builder.abort()
                     raise
+                else:
+                    builder.commit("msg")
             self.assertRaises(errors.RootMissing, do_commit)
         finally:
             tree.unlock()
@@ -306,6 +323,7 @@
             # pointless commit.
             self.assertFalse(builder.any_changes())
             builder.finish_inventory()
+            builder.commit('')
             builder_tree = builder.revision_tree()
             new_root_id = builder_tree.get_root_id()
             new_root_revision = builder_tree.get_file_revision(new_root_id)
@@ -316,7 +334,6 @@
                 # We should see a new root revision
                 self.assertNotEqual(old_revision_id, new_root_revision)
         finally:
-            builder.abort()
             tree.unlock()
 
     def test_commit_record_entry_contents(self):
@@ -975,6 +992,7 @@
                                 [inv_key])[inv_key]
                 self.assertEqual(inv_sha1, builder.inv_sha1)
             self.assertIs(None, builder.new_inventory)
+            rev2 = builder.commit('')
             new_inventory = builder.revision_tree().inventory
             new_entry = new_inventory[file_id]
             if delta_against_basis:
@@ -983,7 +1001,6 @@
             else:
                 expected_delta = None
                 self.assertFalse(version_recorded)
-            rev2 = builder.commit('')
             tree.set_parent_ids([rev2])
         except:
             builder.abort()
@@ -1278,7 +1295,10 @@
         make_before(path)
 
         def change_kind():
-            osutils.delete_any(path)
+            if osutils.file_kind(path) == "directory":
+                osutils.rmtree(path)
+            else:
+                osutils.delete_any(path)
             make_after(path)
 
         self._add_commit_change_check_changed(tree, path, change_kind,
@@ -1289,16 +1309,26 @@
             expect_fs_hash=True)
 
     def test_last_modified_dir_file_ric(self):
-        self._check_kind_change(self.make_dir, self.make_file,
-            expect_fs_hash=True,
-            mini_commit=self.mini_commit_record_iter_changes)
+        try:
+            self._check_kind_change(self.make_dir, self.make_file,
+                expect_fs_hash=True,
+                mini_commit=self.mini_commit_record_iter_changes)
+        except errors.UnsupportedKindChange:
+            raise tests.TestSkipped(
+                "tree does not support changing entry kind from "
+                "directory to file")
 
     def test_last_modified_dir_link(self):
         self._check_kind_change(self.make_dir, self.make_link)
 
     def test_last_modified_dir_link_ric(self):
-        self._check_kind_change(self.make_dir, self.make_link,
-            mini_commit=self.mini_commit_record_iter_changes)
+        try:
+            self._check_kind_change(self.make_dir, self.make_link,
+                mini_commit=self.mini_commit_record_iter_changes)
+        except errors.UnsupportedKindChange:
+            raise tests.TestSkipped(
+                "tree does not support changing entry kind from "
+                "directory to link")
 
     def test_last_modified_link_file(self):
         self._check_kind_change(self.make_link, self.make_file,
@@ -1335,7 +1365,7 @@
         branch.repository.lock_write()
         self.addCleanup(branch.repository.unlock)
         self.assertRaises(ValueError, branch.repository.get_commit_builder,
-            branch, [], branch.get_config(),
+            branch, [], branch.get_config_stack(),
             revprops={'invalid': u'property\rwith\r\ninvalid chars'})
 
     def test_commit_builder_commit_with_invalid_message(self):
@@ -1343,7 +1373,7 @@
         branch.repository.lock_write()
         self.addCleanup(branch.repository.unlock)
         builder = branch.repository.get_commit_builder(branch, [],
-            branch.get_config())
+            branch.get_config_stack())
         self.addCleanup(branch.repository.abort_write_group)
         self.assertRaises(ValueError, builder.commit,
             u'Invalid\r\ncommit message\r\n')
@@ -1355,7 +1385,7 @@
         self.addCleanup(branch.repository.unlock)
         self.assertRaises(UnicodeDecodeError,
             branch.repository.get_commit_builder,
-            branch, [], branch.get_config(),
+            branch, [], branch.get_config_stack(),
             committer="Erik B\xe5gfors <erik@example.com>")
 
     def test_stacked_repositories_reject_commit_builder(self):
@@ -1372,10 +1402,10 @@
         self.addCleanup(repo_local.lock_write().unlock)
         if not repo_local._format.supports_chks:
             self.assertRaises(errors.BzrError, repo_local.get_commit_builder,
-                branch, [], branch.get_config())
+                branch, [], branch.get_config_stack())
         else:
             builder = repo_local.get_commit_builder(branch, [],
-                                                    branch.get_config())
+                                                    branch.get_config_stack())
             builder.abort()
 
     def test_committer_no_username(self):

=== modified file 'bzrlib/tests/per_repository/test_fetch.py'
--- bzrlib/tests/per_repository/test_fetch.py	2011-08-04 00:17:53 +0000
+++ bzrlib/tests/per_repository/test_fetch.py	2011-10-10 12:47:25 +0000
@@ -22,7 +22,6 @@
     gpg,
     remote,
     repository,
-    tests,
     )
 from bzrlib.inventory import ROOT_ID
 from bzrlib.tests import (
@@ -157,6 +156,9 @@
 
     def test_fetch_to_rich_root_set_parent_1_ghost_parent(self):
         # 1 ghost parent -> No parents
+        if not self.repository_format.supports_ghosts:
+            raise TestNotApplicable("repository format does not support "
+                 "ghosts")
         self.do_test_fetch_to_rich_root_sets_parents_correctly((),
             [('tip', ['ghost'], [('add', ('', ROOT_ID, 'directory', ''))]),
             ], allow_lefthand_ghost=True)
@@ -333,6 +335,8 @@
     def test_fetch_into_smart_with_ghost(self):
         trans = self.make_smart_server('target')
         source_b = self.make_simple_branch_with_ghost()
+        if not source_b.bzrdir._format.supports_transport(trans):
+            raise TestNotApplicable("format does not support transport")
         target = self.make_repository('target')
         # Re-open the repository over the smart protocol
         target = repository.Repository.open(trans.base)
@@ -350,6 +354,8 @@
     def test_fetch_from_smart_with_ghost(self):
         trans = self.make_smart_server('source')
         source_b = self.make_simple_branch_with_ghost()
+        if not source_b.bzrdir._format.supports_transport(trans):
+            raise TestNotApplicable("format does not support transport")
         target = self.make_repository('target')
         target.lock_write()
         self.addCleanup(target.unlock)

=== removed file 'bzrlib/tests/per_repository/test_fileid_involved.py'
--- bzrlib/tests/per_repository/test_fileid_involved.py	2011-06-16 13:29:36 +0000
+++ bzrlib/tests/per_repository/test_fileid_involved.py	1970-01-01 00:00:00 +0000
@@ -1,412 +0,0 @@
-# Copyright (C) 2005-2010 Canonical Ltd
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-
-import sys
-import time
-
-from bzrlib import (
-    errors,
-    inventory,
-    remote,
-    revision as _mod_revision,
-    tests,
-    transform,
-    )
-from bzrlib.tests import per_repository
-
-
-class FileIdInvolvedBase(per_repository.TestCaseWithRepository):
-
-    def touch(self, tree, filename):
-        # use the trees transport to not depend on the tree's location or type.
-        tree.bzrdir.root_transport.append_bytes(filename, "appended line\n")
-
-    def compare_tree_fileids(self, branch, old_rev, new_rev):
-        old_tree = self.branch.repository.revision_tree(old_rev)
-        new_tree = self.branch.repository.revision_tree(new_rev)
-        delta = new_tree.changes_from(old_tree)
-
-        l2 = [id for path, id, kind in delta.added] + \
-             [id for oldpath, newpath, id, kind, text_modified, \
-                meta_modified in delta.renamed] + \
-             [id for path, id, kind, text_modified, meta_modified in \
-                delta.modified]
-        return set(l2)
-
-
-class TestFileIdInvolved(FileIdInvolvedBase):
-
-    def setUp(self):
-        super(TestFileIdInvolved, self).setUp()
-        # create three branches, and merge it
-        #
-        #          ,-->J------>K                (branch2)
-        #         /             \
-        #  A --->B --->C---->D-->G              (main)
-        #  \          /     /
-        #   '--->E---+---->F                    (branch1)
-
-        # A changes:
-        # B changes: 'a-file-id-2006-01-01-abcd'
-        # C changes:  Nothing (perfect merge)
-        # D changes: 'b-file-id-2006-01-01-defg'
-        # E changes: 'file-d'
-        # F changes: 'file-d'
-        # G changes: 'b-file-id-2006-01-01-defg'
-        # J changes: 'b-file-id-2006-01-01-defg'
-        # K changes: 'c-funky<file-id>quiji%bo'
-
-        main_wt = self.make_branch_and_tree('main')
-        main_branch = main_wt.branch
-        self.build_tree(["main/a","main/b","main/c"])
-
-        main_wt.add(['a', 'b', 'c'], ['a-file-id-2006-01-01-abcd',
-                                 'b-file-id-2006-01-01-defg',
-                                 'c-funky<file-id>quiji%bo'])
-        try:
-            main_wt.commit("Commit one", rev_id="rev-A")
-        except errors.IllegalPath:
-            # TODO: jam 20060701 Consider raising a different exception
-            #       newer formats do support this, and nothin can done to
-            #       correct this test - its not a bug.
-            if sys.platform == 'win32':
-                raise tests.TestSkipped('Old repository formats do not'
-                                        ' support file ids with <> on win32')
-            # This is not a known error condition
-            raise
-
-        #-------- end A -----------
-
-        bt1 = self.make_branch_and_tree('branch1')
-        bt1.pull(main_branch)
-        b1 = bt1.branch
-        self.build_tree(["branch1/d"])
-        bt1.add(['d'], ['file-d'])
-        bt1.commit("branch1, Commit one", rev_id="rev-E")
-
-        #-------- end E -----------
-
-        self.touch(main_wt, "a")
-        main_wt.commit("Commit two", rev_id="rev-B")
-
-        #-------- end B -----------
-
-        bt2 = self.make_branch_and_tree('branch2')
-        bt2.pull(main_branch)
-        branch2_branch = bt2.branch
-        set_executability(bt2, 'b', True)
-        bt2.commit("branch2, Commit one", rev_id="rev-J")
-
-        #-------- end J -----------
-
-        main_wt.merge_from_branch(b1)
-        main_wt.commit("merge branch1, rev-11", rev_id="rev-C")
-
-        #-------- end C -----------
-
-        bt1.rename_one("d","e")
-        bt1.commit("branch1, commit two", rev_id="rev-F")
-
-        #-------- end F -----------
-
-        self.touch(bt2, "c")
-        bt2.commit("branch2, commit two", rev_id="rev-K")
-
-        #-------- end K -----------
-
-        main_wt.merge_from_branch(b1)
-        self.touch(main_wt, "b")
-        # D gets some funky characters to make sure the unescaping works
-        main_wt.commit("merge branch1, rev-12", rev_id="rev-<D>")
-
-        # end D
-
-        main_wt.merge_from_branch(branch2_branch)
-        main_wt.commit("merge branch1, rev-22",  rev_id="rev-G")
-
-        # end G
-        self.branch = main_branch
-
-    def test_fileids_altered_between_two_revs(self):
-        self.branch.lock_read()
-        self.addCleanup(self.branch.unlock)
-        self.branch.repository.fileids_altered_by_revision_ids(["rev-J","rev-K"])
-        self.assertEqual(
-            {'b-file-id-2006-01-01-defg':set(['rev-J']),
-             'c-funky<file-id>quiji%bo':set(['rev-K'])
-             },
-            self.branch.repository.fileids_altered_by_revision_ids(["rev-J","rev-K"]))
-
-        self.assertEqual(
-            {'b-file-id-2006-01-01-defg': set(['rev-<D>']),
-             'file-d': set(['rev-F']),
-             },
-            self.branch.repository.fileids_altered_by_revision_ids(['rev-<D>', 'rev-F']))
-
-        self.assertEqual(
-            {
-             'b-file-id-2006-01-01-defg': set(['rev-<D>', 'rev-G', 'rev-J']),
-             'c-funky<file-id>quiji%bo': set(['rev-K']),
-             'file-d': set(['rev-F']),
-             },
-            self.branch.repository.fileids_altered_by_revision_ids(
-                ['rev-<D>', 'rev-G', 'rev-F', 'rev-K', 'rev-J']))
-
-        self.assertEqual(
-            {'a-file-id-2006-01-01-abcd': set(['rev-B']),
-             'b-file-id-2006-01-01-defg': set(['rev-<D>', 'rev-G', 'rev-J']),
-             'c-funky<file-id>quiji%bo': set(['rev-K']),
-             'file-d': set(['rev-F']),
-             },
-            self.branch.repository.fileids_altered_by_revision_ids(
-                ['rev-G', 'rev-F', 'rev-C', 'rev-B', 'rev-<D>', 'rev-K', 'rev-J']))
-
-    def fileids_altered_by_revision_ids(self, revision_ids):
-        """This is a wrapper to strip TREE_ROOT if it occurs"""
-        repo = self.branch.repository
-        root_id = self.branch.basis_tree().get_root_id()
-        result = repo.fileids_altered_by_revision_ids(revision_ids)
-        if root_id in result:
-            del result[root_id]
-        return result
-
-    def test_fileids_altered_by_revision_ids(self):
-        self.branch.lock_read()
-        self.addCleanup(self.branch.unlock)
-        self.assertEqual(
-            {'a-file-id-2006-01-01-abcd':set(['rev-A']),
-             'b-file-id-2006-01-01-defg': set(['rev-A']),
-             'c-funky<file-id>quiji%bo': set(['rev-A']),
-             },
-            self.fileids_altered_by_revision_ids(["rev-A"]))
-        self.assertEqual(
-            {'a-file-id-2006-01-01-abcd':set(['rev-B'])
-             },
-            self.branch.repository.fileids_altered_by_revision_ids(["rev-B"]))
-        self.assertEqual(
-            {'b-file-id-2006-01-01-defg':set(['rev-<D>'])
-             },
-            self.branch.repository.fileids_altered_by_revision_ids(["rev-<D>"]))
-
-    def test_fileids_involved_full_compare(self):
-        # this tests that the result of each fileid_involved calculation
-        # along a revision history selects only the fileids selected by
-        # comparing the trees - no less, and no more. This is correct
-        # because in our sample data we do not revert any file ids along
-        # the revision history.
-        self.branch.lock_read()
-        self.addCleanup(self.branch.unlock)
-        pp=[]
-        history = self.branch.revision_history( )
-
-        if len(history) < 2: return
-
-        graph = self.branch.repository.get_graph()
-        for start in range(0,len(history)-1):
-            start_id = history[start]
-            for end in range(start+1,len(history)):
-                end_id = history[end]
-                unique_revs = graph.find_unique_ancestors(end_id, [start_id])
-                l1 = self.branch.repository.fileids_altered_by_revision_ids(
-                    unique_revs)
-                l1 = set(l1.keys())
-                l2 = self.compare_tree_fileids(self.branch, start_id, end_id)
-                self.assertEquals(l1, l2)
-
-
-class TestFileIdInvolvedNonAscii(FileIdInvolvedBase):
-
-    def test_utf8_file_ids_and_revision_ids(self):
-        main_wt = self.make_branch_and_tree('main')
-        main_branch = main_wt.branch
-        self.build_tree(["main/a"])
-
-        file_id = u'a-f\xedle-id'.encode('utf8')
-        main_wt.add(['a'], [file_id])
-        revision_id = u'r\xe9v-a'.encode('utf8')
-        try:
-            main_wt.commit('a', rev_id=revision_id)
-        except errors.NonAsciiRevisionId:
-            raise tests.TestSkipped('non-ascii revision ids not supported by %s'
-                                    % self.repository_format)
-
-        repo = main_wt.branch.repository
-        repo.lock_read()
-        self.addCleanup(repo.unlock)
-        file_ids = repo.fileids_altered_by_revision_ids([revision_id])
-        root_id = main_wt.basis_tree().get_root_id()
-        if root_id in file_ids:
-            self.assertEqual({file_id:set([revision_id]),
-                              root_id:set([revision_id])
-                             }, file_ids)
-        else:
-            self.assertEqual({file_id:set([revision_id])}, file_ids)
-
-
-class TestFileIdInvolvedSuperset(FileIdInvolvedBase):
-
-    def setUp(self):
-        super(TestFileIdInvolvedSuperset, self).setUp()
-
-        self.branch = None
-        main_wt = self.make_branch_and_tree('main')
-        main_branch = main_wt.branch
-        self.build_tree(["main/a","main/b","main/c"])
-
-        main_wt.add(['a', 'b', 'c'], ['a-file-id-2006-01-01-abcd',
-                                 'b-file-id-2006-01-01-defg',
-                                 'c-funky<file-id>quiji\'"%bo'])
-        try:
-            main_wt.commit("Commit one", rev_id="rev-A")
-        except errors.IllegalPath:
-            # TODO: jam 20060701 Consider raising a different exception
-            #       newer formats do support this, and nothin can done to
-            #       correct this test - its not a bug.
-            if sys.platform == 'win32':
-                raise tests.TestSkipped('Old repository formats do not'
-                                        ' support file ids with <> on win32')
-            # This is not a known error condition
-            raise
-
-        branch2_wt = self.make_branch_and_tree('branch2')
-        branch2_wt.pull(main_branch)
-        branch2_bzrdir = branch2_wt.bzrdir
-        branch2_branch = branch2_bzrdir.open_branch()
-        set_executability(branch2_wt, 'b', True)
-        branch2_wt.commit("branch2, Commit one", rev_id="rev-J")
-
-        main_wt.merge_from_branch(branch2_branch)
-        set_executability(main_wt, 'b', False)
-        main_wt.commit("merge branch1, rev-22",  rev_id="rev-G")
-
-        # end G
-        self.branch = main_branch
-
-    def test_fileid_involved_full_compare2(self):
-        # this tests that fileids_altered_by_revision_ids returns
-        # more information than compare_tree can, because it
-        # sees each change rather than the aggregate delta.
-        self.branch.lock_read()
-        self.addCleanup(self.branch.unlock)
-        history = self.branch.revision_history()
-        old_rev = history[0]
-        new_rev = history[1]
-        graph = self.branch.repository.get_graph()
-        unique_revs = graph.find_unique_ancestors(new_rev, [old_rev])
-
-        l1 = self.branch.repository.fileids_altered_by_revision_ids(
-            unique_revs)
-        l1 = set(l1.keys())
-
-        l2 = self.compare_tree_fileids(self.branch, old_rev, new_rev)
-        self.assertNotEqual(l2, l1)
-        self.assertSubset(l2, l1)
-
-
-class FileIdInvolvedWGhosts(per_repository.TestCaseWithRepository):
-
-    def create_branch_with_ghost_text(self):
-        builder = self.make_branch_builder('ghost')
-        builder.build_snapshot('A-id', None, [
-            ('add', ('', 'root-id', 'directory', None)),
-            ('add', ('a', 'a-file-id', 'file', 'some content\n'))])
-        b = builder.get_branch()
-        old_rt = b.repository.revision_tree('A-id')
-        new_inv = inventory.mutable_inventory_from_tree(old_rt)
-        new_inv.revision_id = 'B-id'
-        new_inv['a-file-id'].revision = 'ghost-id'
-        new_rev = _mod_revision.Revision('B-id',
-            timestamp=time.time(),
-            timezone=0,
-            message='Committing against a ghost',
-            committer='Joe Foo <joe@foo.com>',
-            properties={},
-            parent_ids=('A-id', 'ghost-id'),
-            )
-        b.lock_write()
-        self.addCleanup(b.unlock)
-        b.repository.start_write_group()
-        b.repository.add_revision('B-id', new_rev, new_inv)
-        self.disable_commit_write_group_paranoia(b.repository)
-        b.repository.commit_write_group()
-        return b
-
-    def disable_commit_write_group_paranoia(self, repo):
-        if isinstance(repo, remote.RemoteRepository):
-            # We can't easily disable the checks in a remote repo.
-            repo.abort_write_group()
-            raise tests.TestSkipped(
-                "repository format does not support storing revisions with "
-                "missing texts.")
-        pack_coll = getattr(repo, '_pack_collection', None)
-        if pack_coll is not None:
-            # Monkey-patch the pack collection instance to allow storing
-            # incomplete revisions.
-            pack_coll._check_new_inventories = lambda: []
-
-    def test_file_ids_include_ghosts(self):
-        b = self.create_branch_with_ghost_text()
-        repo = b.repository
-        self.assertEqual(
-            {'a-file-id':set(['ghost-id'])},
-            repo.fileids_altered_by_revision_ids(['B-id']))
-
-    def test_file_ids_uses_fallbacks(self):
-        builder = self.make_branch_builder('source',
-                                           format=self.bzrdir_format)
-        repo = builder.get_branch().repository
-        if not repo._format.supports_external_lookups:
-            raise tests.TestNotApplicable('format does not support stacking')
-        builder.start_series()
-        builder.build_snapshot('A-id', None, [
-            ('add', ('', 'root-id', 'directory', None)),
-            ('add', ('file', 'file-id', 'file', 'contents\n'))])
-        builder.build_snapshot('B-id', ['A-id'], [
-            ('modify', ('file-id', 'new-content\n'))])
-        builder.build_snapshot('C-id', ['B-id'], [
-            ('modify', ('file-id', 'yet more content\n'))])
-        builder.finish_series()
-        source_b = builder.get_branch()
-        source_b.lock_read()
-        self.addCleanup(source_b.unlock)
-        base = self.make_branch('base')
-        base.pull(source_b, stop_revision='B-id')
-        stacked = self.make_branch('stacked')
-        stacked.set_stacked_on_url('../base')
-        stacked.pull(source_b, stop_revision='C-id')
-
-        stacked.lock_read()
-        self.addCleanup(stacked.unlock)
-        repo = stacked.repository
-        keys = {'file-id': set(['A-id'])}
-        if stacked.repository.supports_rich_root():
-            keys['root-id'] = set(['A-id'])
-        self.assertEqual(keys, repo.fileids_altered_by_revision_ids(['A-id']))
-
-
-def set_executability(wt, path, executable=True):
-    """Set the executable bit for the file at path in the working tree
-
-    os.chmod() doesn't work on windows. But TreeTransform can mark or
-    unmark a file as executable.
-    """
-    file_id = wt.path2id(path)
-    tt = transform.TreeTransform(wt)
-    try:
-        tt.set_executability(executable, tt.trans_id_tree_file_id(file_id))
-        tt.apply()
-    finally:
-        tt.finalize()

=== modified file 'bzrlib/tests/per_repository/test_iter_reverse_revision_history.py'
--- bzrlib/tests/per_repository/test_iter_reverse_revision_history.py	2011-06-16 12:50:32 +0000
+++ bzrlib/tests/per_repository/test_iter_reverse_revision_history.py	2011-09-09 12:17:12 +0000
@@ -150,6 +150,9 @@
 
     def test_ghost(self):
         tree = self.make_branch_and_memory_tree('tree')
+        if not tree.branch.repository._format.supports_ghosts:
+            raise tests.TestNotApplicable(
+                "repository format does not support ghosts")
         tree.lock_write()
         try:
             tree.add('')

=== modified file 'bzrlib/tests/per_repository/test_repository.py'
--- bzrlib/tests/per_repository/test_repository.py	2011-08-27 00:00:23 +0000
+++ bzrlib/tests/per_repository/test_repository.py	2011-12-07 14:16:12 +0000
@@ -21,7 +21,7 @@
 
 from bzrlib import (
     branch as _mod_branch,
-    bzrdir,
+    controldir,
     delta as _mod_delta,
     errors,
     gpg,
@@ -68,6 +68,16 @@
         """Test the format.fast_deltas attribute."""
         self.assertFormatAttribute('fast_deltas', (True, False))
 
+    def test_attribute_supports_nesting_repositories(self):
+        """Test the format.supports_nesting_repositories."""
+        self.assertFormatAttribute('supports_nesting_repositories',
+            (True, False))
+
+    def test_attribute_supports_unreferenced_revisions(self):
+        """Test the format.supports_unreferenced_revisions."""
+        self.assertFormatAttribute('supports_unreferenced_revisions',
+            (True, False))
+
     def test_attribute__fetch_reconcile(self):
         """Test the _fetch_reconcile attribute."""
         self.assertFormatAttribute('_fetch_reconcile', (True, False))
@@ -150,8 +160,8 @@
         made_repo = self.repository_format.initialize(made_control)
         self.assertEqual(made_control, made_repo.bzrdir)
 
-        # find it via bzrdir opening:
-        opened_control = bzrdir.BzrDir.open(readonly_t.base)
+        # find it via controldir opening:
+        opened_control = controldir.ControlDir.open(readonly_t.base)
         direct_opened_repo = opened_control.open_repository()
         self.assertEqual(direct_opened_repo.__class__, made_repo.__class__)
         self.assertEqual(opened_control, direct_opened_repo.bzrdir)
@@ -169,7 +179,7 @@
         except NotImplementedError:
             return
         self.assertEqual(self.repository_format,
-                         repository.RepositoryFormat.find_format(opened_control))
+             repository.RepositoryFormatMetaDir.find_format(opened_control))
 
     def test_format_matchingbzrdir(self):
         self.assertEqual(self.repository_format,
@@ -277,54 +287,6 @@
         self.assertIsInstance(delta, _mod_delta.TreeDelta)
         self.assertEqual([('vla', 'file2', 'file')], delta.added)
 
-    def test_get_revision_delta_filtered(self):
-        tree_a = self.make_branch_and_tree('a')
-        self.build_tree(['a/foo', 'a/bar/', 'a/bar/b1', 'a/bar/b2', 'a/baz'])
-        tree_a.add(['foo', 'bar', 'bar/b1', 'bar/b2', 'baz'],
-                   ['foo-id', 'bar-id', 'b1-id', 'b2-id', 'baz-id'])
-        tree_a.commit('rev1', rev_id='rev1')
-        self.build_tree(['a/bar/b3'])
-        tree_a.add('bar/b3', 'b3-id')
-        tree_a.commit('rev2', rev_id='rev2')
-
-        # Test multiple files
-        delta = tree_a.branch.repository.get_revision_delta('rev1',
-            specific_fileids=['foo-id', 'baz-id'])
-        self.assertIsInstance(delta, _mod_delta.TreeDelta)
-        self.assertEqual([
-            ('baz', 'baz-id', 'file'),
-            ('foo', 'foo-id', 'file'),
-            ], delta.added)
-        # Test a directory
-        delta = tree_a.branch.repository.get_revision_delta('rev1',
-            specific_fileids=['bar-id'])
-        self.assertIsInstance(delta, _mod_delta.TreeDelta)
-        self.assertEqual([
-            ('bar', 'bar-id', 'directory'),
-            ('bar/b1', 'b1-id', 'file'),
-            ('bar/b2', 'b2-id', 'file'),
-            ], delta.added)
-        # Test a file in a directory
-        delta = tree_a.branch.repository.get_revision_delta('rev1',
-            specific_fileids=['b2-id'])
-        self.assertIsInstance(delta, _mod_delta.TreeDelta)
-        self.assertEqual([
-            ('bar', 'bar-id', 'directory'),
-            ('bar/b2', 'b2-id', 'file'),
-            ], delta.added)
-        # Try another revision
-        delta = tree_a.branch.repository.get_revision_delta('rev2',
-                specific_fileids=['b3-id'])
-        self.assertIsInstance(delta, _mod_delta.TreeDelta)
-        self.assertEqual([
-            ('bar', 'bar-id', 'directory'),
-            ('bar/b3', 'b3-id', 'file'),
-            ], delta.added)
-        delta = tree_a.branch.repository.get_revision_delta('rev2',
-                specific_fileids=['foo-id'])
-        self.assertIsInstance(delta, _mod_delta.TreeDelta)
-        self.assertEqual([], delta.added)
-
     def test_clone_bzrdir_repository_revision(self):
         # make a repository with some revisions,
         # and clone it, this should not have unreferenced revisions.
@@ -360,6 +322,9 @@
         self.assertFalse(result.open_repository().make_working_trees())
 
     def test_upgrade_preserves_signatures(self):
+        if not self.repository_format.supports_revision_signatures:
+            raise tests.TestNotApplicable(
+                "repository does not support signing revisions")
         wt = self.make_branch_and_tree('source')
         wt.commit('A', allow_pointless=True, rev_id='A')
         repo = wt.branch.repository
@@ -369,15 +334,16 @@
             repo.sign_revision('A', gpg.LoopbackGPGStrategy(None))
         except errors.UnsupportedOperation:
             self.assertFalse(repo._format.supports_revision_signatures)
-            raise TestNotApplicable("signatures not supported by repository format")
+            raise tests.TestNotApplicable("signatures not supported by repository format")
         repo.commit_write_group()
         repo.unlock()
         old_signature = repo.get_signature_text('A')
         try:
-            old_format = bzrdir.BzrDirFormat.get_default_format()
+            old_format = controldir.ControlDirFormat.get_default_format()
             # This gives metadir branches something they can convert to.
             # it would be nice to have a 'latest' vs 'default' concept.
-            format = bzrdir.format_registry.make_bzrdir('dirstate-with-subtree')
+            format = controldir.format_registry.make_bzrdir(
+                'dirstate-with-subtree')
             upgrade.upgrade(repo.bzrdir.root_transport.base, format=format)
         except errors.UpToDateFormat:
             # this is in the most current format already.
@@ -586,19 +552,21 @@
         b = builder.get_branch()
         b.lock_write()
         self.addCleanup(b.unlock)
-        b.repository.start_write_group()
-        self.addCleanup(b.repository.abort_write_group)
         if b.repository._format.supports_revision_signatures:
+            b.repository.start_write_group()
             b.repository.add_signature_text('A', 'This might be a signature')
+            b.repository.commit_write_group()
             self.assertEqual('This might be a signature',
                              b.repository.get_signature_text('A'))
         else:
+            b.repository.start_write_group()
+            self.addCleanup(b.repository.abort_write_group)
             self.assertRaises(errors.UnsupportedOperation,
                 b.repository.add_signature_text, 'A',
                 'This might be a signature')
 
     # XXX: this helper duplicated from tests.test_repository
-    def make_remote_repository(self, path, shared=False):
+    def make_remote_repository(self, path, shared=None):
         """Make a RemoteRepository object backed by a real repository that will
         be created at the given path."""
         repo = self.make_repository(path, shared=shared)
@@ -606,7 +574,11 @@
         self.start_server(smart_server, self.get_server())
         remote_transport = transport.get_transport_from_url(
             smart_server.get_url()).clone(path)
-        remote_bzrdir = bzrdir.BzrDir.open_from_transport(remote_transport)
+        if not repo.bzrdir._format.supports_transport(remote_transport):
+            raise tests.TestNotApplicable(
+                "format does not support transport")
+        remote_bzrdir = controldir.ControlDir.open_from_transport(
+            remote_transport)
         remote_repo = remote_bzrdir.open_repository()
         return remote_repo
 
@@ -619,9 +591,11 @@
         except errors.TransportNotPossible:
             raise tests.TestNotApplicable(
                 "Cannot lock_read old formats like AllInOne over HPSS.")
-        remote_backing_repo = bzrdir.BzrDir.open(
+        remote_backing_repo = controldir.ControlDir.open(
             self.get_vfs_only_url('remote')).open_repository()
-        self.assertEqual(remote_backing_repo._format, local_repo._format)
+        self.assertEqual(
+            remote_backing_repo._format.network_name(),
+            local_repo._format.network_name())
 
     def test_sprout_branch_from_hpss_preserves_repo_format(self):
         """branch.sprout from a smart server preserves the repository format.
@@ -637,7 +611,7 @@
             raise tests.TestNotApplicable(
                 "Cannot lock_read old formats like AllInOne over HPSS.")
         local_repo = local_bzrdir.open_repository()
-        remote_backing_repo = bzrdir.BzrDir.open(
+        remote_backing_repo = controldir.ControlDir.open(
             self.get_vfs_only_url('remote')).open_repository()
         self.assertEqual(remote_backing_repo._format, local_repo._format)
 
@@ -650,7 +624,7 @@
                 "Format can not be used over HPSS")
         # Make a shared repo
         remote_repo = self.make_remote_repository('remote', shared=True)
-        remote_backing_repo = bzrdir.BzrDir.open(
+        remote_backing_repo = controldir.ControlDir.open(
             self.get_vfs_only_url('remote')).open_repository()
         # Make a branch in that repo in an old format that isn't the default
         # branch format for the repo.
@@ -658,9 +632,9 @@
         format = remote_backing_repo.bzrdir.cloning_metadir()
         format._branch_format = BzrBranchFormat5()
         remote_transport = remote_repo.bzrdir.root_transport.clone('branch')
-        remote_backing_repo.bzrdir.create_branch_convenience(
+        controldir.ControlDir.create_branch_convenience(
             remote_transport.base, force_new_repo=False, format=format)
-        remote_branch = bzrdir.BzrDir.open_from_transport(
+        remote_branch = controldir.ControlDir.open_from_transport(
             remote_transport).open_branch()
         try:
             local_bzrdir = remote_branch.bzrdir.sprout('local')
@@ -747,20 +721,23 @@
         repo = self.make_repository('repo')
         repo._make_parents_provider().get_parent_map
 
-    def make_repository_and_foo_bar(self, shared):
+    def make_repository_and_foo_bar(self, shared=None):
         made_control = self.make_bzrdir('repository')
         repo = made_control.create_repository(shared=shared)
-        bzrdir.BzrDir.create_branch_convenience(self.get_url('repository/foo'),
-                                                force_new_repo=False)
-        bzrdir.BzrDir.create_branch_convenience(self.get_url('repository/bar'),
-                                                force_new_repo=True)
+        if not repo._format.supports_nesting_repositories:
+            raise tests.TestNotApplicable("repository does not support "
+                "nesting repositories")
+        controldir.ControlDir.create_branch_convenience(
+            self.get_url('repository/foo'), force_new_repo=False)
+        controldir.ControlDir.create_branch_convenience(
+            self.get_url('repository/bar'), force_new_repo=True)
         baz = self.make_bzrdir('repository/baz')
         qux = self.make_branch('repository/baz/qux')
         quxx = self.make_branch('repository/baz/qux/quxx')
         return repo
 
     def test_find_branches(self):
-        repo = self.make_repository_and_foo_bar(shared=False)
+        repo = self.make_repository_and_foo_bar()
         branches = repo.find_branches()
         self.assertContainsRe(branches[-1].base, 'repository/foo/$')
         self.assertContainsRe(branches[-3].base, 'repository/baz/qux/$')
@@ -789,6 +766,9 @@
 
     def test_find_branches_using_standalone(self):
         branch = self.make_branch('branch')
+        if not branch.repository._format.supports_nesting_repositories:
+            raise tests.TestNotApplicable("format does not support nesting "
+                "repositories")
         contained = self.make_branch('branch/contained')
         branches = branch.repository.find_branches(using=True)
         self.assertEqual([branch.base], [b.base for b in branches])
@@ -797,8 +777,11 @@
                          [b.base for b in branches])
 
     def test_find_branches_using_empty_standalone_repo(self):
-        repo = self.make_repository('repo')
-        self.assertFalse(repo.is_shared())
+        try:
+            repo = self.make_repository('repo', shared=False)
+        except errors.IncompatibleFormat:
+            raise tests.TestNotApplicable("format does not support standalone "
+                "repositories")
         try:
             repo.bzrdir.open_branch()
         except errors.NotBranchError:
@@ -923,7 +906,7 @@
         wt.add(['foo'], [FOO_ID])
         wt.commit('this is my new commit', rev_id=REV_ID)
         # now access over vfat; should be safe
-        branch = bzrdir.BzrDir.open(self.get_url('repo')).open_branch()
+        branch = controldir.ControlDir.open(self.get_url('repo')).open_branch()
         revtree = branch.repository.revision_tree(REV_ID)
         revtree.lock_read()
         self.addCleanup(revtree.unlock)
@@ -947,7 +930,73 @@
         repo = self.make_repository('repo')
         self.assertIsInstance(repo.user_url, str)
         self.assertEqual(repo.user_url, repo.user_transport.base)
-        # for all current bzrdir implementations the user dir must be 
+        # for all current bzrdir implementations the user dir must be
         # above the control dir but we might need to relax that?
         self.assertEqual(repo.control_url.find(repo.user_url), 0)
         self.assertEqual(repo.control_url, repo.control_transport.base)
+
+
+class TestDeltaRevisionFiltered(per_repository.TestCaseWithRepository):
+
+    def setUp(self):
+        super(TestDeltaRevisionFiltered, self).setUp()
+        tree_a = self.make_branch_and_tree('a')
+        self.build_tree(['a/foo', 'a/bar/', 'a/bar/b1', 'a/bar/b2', 'a/baz'])
+        tree_a.add(['foo', 'bar', 'bar/b1', 'bar/b2', 'baz'],
+                   ['foo-id', 'bar-id', 'b1-id', 'b2-id', 'baz-id'])
+        tree_a.commit('rev1', rev_id='rev1')
+        self.build_tree(['a/bar/b3'])
+        tree_a.add('bar/b3', 'b3-id')
+        tree_a.commit('rev2', rev_id='rev2')
+        self.repository = tree_a.branch.repository
+
+    def test_multiple_files(self):
+        # Test multiple files
+        delta = self.repository.get_revision_delta('rev1',
+            specific_fileids=['foo-id', 'baz-id'])
+        self.assertIsInstance(delta, _mod_delta.TreeDelta)
+        self.assertEqual([
+            ('baz', 'baz-id', 'file'),
+            ('foo', 'foo-id', 'file'),
+            ], delta.added)
+
+    def test_directory(self):
+        # Test a directory
+        delta = self.repository.get_revision_delta('rev1',
+            specific_fileids=['bar-id'])
+        self.assertIsInstance(delta, _mod_delta.TreeDelta)
+        self.assertEqual([
+            ('bar', 'bar-id', 'directory'),
+            ('bar/b1', 'b1-id', 'file'),
+            ('bar/b2', 'b2-id', 'file'),
+            ], delta.added)
+
+    def test_unrelated(self):
+        # Try another revision
+        delta = self.repository.get_revision_delta('rev2',
+                specific_fileids=['foo-id'])
+        self.assertIsInstance(delta, _mod_delta.TreeDelta)
+        self.assertEqual([], delta.added)
+
+    def test_file_in_directory(self):
+        # Test a file in a directory, both of which were added
+        delta = self.repository.get_revision_delta('rev1',
+            specific_fileids=['b2-id'])
+        self.assertIsInstance(delta, _mod_delta.TreeDelta)
+        self.assertEqual([
+            ('bar', 'bar-id', 'directory'),
+            ('bar/b2', 'b2-id', 'file'),
+            ], delta.added)
+
+    def test_file_in_unchanged_directory(self):
+        delta = self.repository.get_revision_delta('rev2',
+            specific_fileids=['b3-id'])
+        self.assertIsInstance(delta, _mod_delta.TreeDelta)
+        if delta.added == [
+            ('bar', 'bar-id', 'directory'),
+            ('bar/b3', 'b3-id', 'file')]:
+            self.knownFailure("bzr incorrectly reports 'bar' as added - "
+                              "bug 878217")
+        self.assertEqual([
+            ('bar/b3', 'b3-id', 'file'),
+            ], delta.added)

=== modified file 'bzrlib/tests/per_repository/test_revision.py'
--- bzrlib/tests/per_repository/test_revision.py	2009-06-10 03:56:49 +0000
+++ bzrlib/tests/per_repository/test_revision.py	2011-09-26 15:21:01 +0000
@@ -69,13 +69,14 @@
         tree1 = self.make_branch_and_tree("br1")
 
         # create a revision
-        tree1.commit(message="quux", allow_pointless=True, committer="jaq",
-                     revprops={'empty':'',
-                               'value':'one',
-                               'unicode':u'\xb5',
-                               'multiline':'foo\nbar\n\n'
-                              })
-        self.assertTrue(len(tree1.branch.revision_history()) > 0)
+        rev1 = tree1.commit(message="quux", allow_pointless=True,
+            committer="jaq",
+            revprops={'empty':'',
+                      'value':'one',
+                      'unicode':u'\xb5',
+                      'multiline':'foo\nbar\n\n'
+                      })
+        self.assertEquals(tree1.branch.last_revision(), rev1)
         rev_a = tree1.branch.repository.get_revision(
                             tree1.branch.last_revision())
 

=== added file 'bzrlib/tests/per_repository/test_signatures.py'
--- bzrlib/tests/per_repository/test_signatures.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_repository/test_signatures.py	2011-11-15 17:22:02 +0000
@@ -0,0 +1,143 @@
+# Copyright (C) 2005-2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for repository revision signatures."""
+
+from bzrlib import (
+    errors,
+    gpg,
+    tests,
+    urlutils,
+    )
+
+from bzrlib.testament import Testament
+from bzrlib.tests import per_repository
+
+class TestSignatures(per_repository.TestCaseWithRepository):
+
+    def setUp(self):
+        super(TestSignatures, self).setUp()
+        if not self.repository_format.supports_revision_signatures:
+            raise tests.TestNotApplicable(
+                "repository does not support signing revisions")
+
+# TODO 20051003 RBC:
+# compare the gpg-to-sign info for a commit with a ghost and
+#     an identical tree without a ghost
+# fetch missing should rewrite the TOC of weaves to list newly available parents.
+
+    def test_sign_existing_revision(self):
+        wt = self.make_branch_and_tree('.')
+        wt.commit("base", allow_pointless=True, rev_id='A')
+        strategy = gpg.LoopbackGPGStrategy(None)
+        repo = wt.branch.repository
+        self.addCleanup(repo.lock_write().unlock)
+        repo.start_write_group()
+        repo.sign_revision('A', strategy)
+        repo.commit_write_group()
+        self.assertEqual('-----BEGIN PSEUDO-SIGNED CONTENT-----\n' +
+                         Testament.from_revision(repo,
+                         'A').as_short_text() +
+                         '-----END PSEUDO-SIGNED CONTENT-----\n',
+                         repo.get_signature_text('A'))
+
+    def test_store_signature(self):
+        wt = self.make_branch_and_tree('.')
+        branch = wt.branch
+        branch.lock_write()
+        try:
+            branch.repository.start_write_group()
+            try:
+                branch.repository.store_revision_signature(
+                    gpg.LoopbackGPGStrategy(None), 'FOO', 'A')
+            except errors.NoSuchRevision:
+                branch.repository.abort_write_group()
+                raise tests.TestNotApplicable(
+                    "repository does not support signing non-present"
+                    "revisions")
+            except:
+                branch.repository.abort_write_group()
+                raise
+            else:
+                branch.repository.commit_write_group()
+        finally:
+            branch.unlock()
+        # A signature without a revision should not be accessible.
+        self.assertRaises(errors.NoSuchRevision,
+                          branch.repository.has_signature_for_revision_id,
+                          'A')
+        wt.commit("base", allow_pointless=True, rev_id='A')
+        self.assertEqual('-----BEGIN PSEUDO-SIGNED CONTENT-----\n'
+                         'FOO-----END PSEUDO-SIGNED CONTENT-----\n',
+                         branch.repository.get_signature_text('A'))
+
+    def test_clone_preserves_signatures(self):
+        wt = self.make_branch_and_tree('source')
+        wt.commit('A', allow_pointless=True, rev_id='A')
+        repo = wt.branch.repository
+        repo.lock_write()
+        repo.start_write_group()
+        repo.sign_revision('A', gpg.LoopbackGPGStrategy(None))
+        repo.commit_write_group()
+        repo.unlock()
+        #FIXME: clone should work to urls,
+        # wt.clone should work to disks.
+        self.build_tree(['target/'])
+        d2 = repo.bzrdir.clone(urlutils.local_path_to_url('target'))
+        self.assertEqual(repo.get_signature_text('A'),
+                         d2.open_repository().get_signature_text('A'))
+
+    def test_verify_revision_signature_not_signed(self):
+        wt = self.make_branch_and_tree('.')
+        wt.commit("base", allow_pointless=True, rev_id='A')
+        strategy = gpg.LoopbackGPGStrategy(None)
+        self.assertEquals(
+            (gpg.SIGNATURE_NOT_SIGNED, None),
+            wt.branch.repository.verify_revision_signature('A', strategy))
+
+    def test_verify_revision_signature(self):
+        wt = self.make_branch_and_tree('.')
+        wt.commit("base", allow_pointless=True, rev_id='A')
+        strategy = gpg.LoopbackGPGStrategy(None)
+        repo = wt.branch.repository
+        self.addCleanup(repo.lock_write().unlock)
+        repo.start_write_group()
+        repo.sign_revision('A', strategy)
+        repo.commit_write_group()
+        self.assertEqual('-----BEGIN PSEUDO-SIGNED CONTENT-----\n' +
+                         Testament.from_revision(repo,
+                         'A').as_short_text() +
+                         '-----END PSEUDO-SIGNED CONTENT-----\n',
+                         repo.get_signature_text('A'))
+        self.assertEquals(
+            (gpg.SIGNATURE_VALID, None, ),
+            repo.verify_revision_signature('A', strategy))
+
+
+class TestUnsupportedSignatures(per_repository.TestCaseWithRepository):
+
+    def test_sign_revision(self):
+        if self.repository_format.supports_revision_signatures:
+            raise tests.TestNotApplicable(
+                "repository supports signing revisions")
+        wt = self.make_branch_and_tree('source')
+        wt.commit('A', allow_pointless=True, rev_id='A')
+        repo = wt.branch.repository
+        repo.lock_write()
+        repo.start_write_group()
+        self.assertRaises(errors.UnsupportedOperation,
+            repo.sign_revision, 'A', gpg.LoopbackGPGStrategy(None))
+        repo.commit_write_group()

=== modified file 'bzrlib/tests/per_repository/test_statistics.py'
--- bzrlib/tests/per_repository/test_statistics.py	2011-04-02 21:43:42 +0000
+++ bzrlib/tests/per_repository/test_statistics.py	2011-09-09 12:19:35 +0000
@@ -38,12 +38,12 @@
         # now, in the same repository, asking for stats with/without the
         # committers flag generates the same date information.
         stats = tree.branch.repository.gather_stats(rev2, committers=False)
-        self.assertEqual({
-            'firstrev': (1170491381.0, 0),
-            'latestrev': (1171491381.0, 0),
-            'revisions': 3,
-            },
-            stats)
+        # this test explicitly only checks for certain keys
+        # in the dictionary, as implementations are allowed to
+        # provide arbitrary data in other keys.
+        self.assertEqual(stats['firstrev'], (1170491381.0, 0))
+        self.assertEqual(stats['latestrev'], (1171491381.0, 0))
+        self.assertEqual(stats['revisions'], 3)
         stats = tree.branch.repository.gather_stats(rev2, committers=True)
         self.assertEquals(2, stats["committers"])
         self.assertEquals((1170491381.0, 0), stats["firstrev"])

=== modified file 'bzrlib/tests/per_repository_chk/test_supported.py'
--- bzrlib/tests/per_repository_chk/test_supported.py	2009-09-09 18:52:56 +0000
+++ bzrlib/tests/per_repository_chk/test_supported.py	2011-12-13 22:41:31 +0000
@@ -22,7 +22,9 @@
     osutils,
     repository,
     )
+from bzrlib.remote import RemoteRepository
 from bzrlib.versionedfile import VersionedFiles
+from bzrlib.tests import TestNotApplicable
 from bzrlib.tests.per_repository_chk import TestCaseWithRepositoryCHK
 
 
@@ -221,7 +223,9 @@
         # check our setup: B-id and C-id should have identical chk root keys.
         inv_b = b.repository.get_inventory('B-id')
         inv_c = b.repository.get_inventory('C-id')
-        self.assertEqual(inv_b.id_to_entry.key(), inv_c.id_to_entry.key())
+        if not isinstance(repo, RemoteRepository):
+            # Remote repositories always return plain inventories
+            self.assertEqual(inv_b.id_to_entry.key(), inv_c.id_to_entry.key())
         # Now, manually insert objects for a stacked repo with only revision
         # C-id:
         # We need ('revisions', 'C-id'), ('inventories', 'C-id'),
@@ -250,6 +254,9 @@
         for a parent inventory of a new revision is missing.
         """
         repo = self.make_repository('damaged-repo')
+        if isinstance(repo, RemoteRepository):
+            raise TestNotApplicable(
+                "Unable to obtain CHKInventory from remote repo")
         b = self.make_branch_with_multiple_chk_nodes()
         src_repo = b.repository
         src_repo.lock_read()
@@ -293,6 +300,9 @@
         for a parent inventory of a new revision is missing.
         """
         repo = self.make_repository('damaged-repo')
+        if isinstance(repo, RemoteRepository):
+            raise TestNotApplicable(
+                "Unable to obtain CHKInventory from remote repo")
         b = self.make_branch_with_multiple_chk_nodes()
         b.lock_read()
         self.addCleanup(b.unlock)

=== modified file 'bzrlib/tests/per_repository_reference/__init__.py'
--- bzrlib/tests/per_repository_reference/__init__.py	2011-05-04 03:43:03 +0000
+++ bzrlib/tests/per_repository_reference/__init__.py	2011-10-01 11:54:46 +0000
@@ -25,6 +25,7 @@
 from bzrlib import (
     errors,
     remote,
+    urlutils,
     )
 from bzrlib.bzrdir import BzrDir
 from bzrlib.tests import multiply_tests
@@ -36,17 +37,18 @@
 
 class TestCaseWithExternalReferenceRepository(TestCaseWithRepository):
 
-    def make_referring(self, relpath, target_path):
+    def make_referring(self, relpath, a_repository):
         """Get a new repository that refers to a_repository.
 
         :param relpath: The path to create the repository at.
         :param a_repository: A repository to refer to.
         """
         repo = self.make_repository(relpath)
-        repo.add_fallback_repository(self.readonly_repository(target_path))
+        repo.add_fallback_repository(self.readonly_repository(a_repository))
         return repo
 
-    def readonly_repository(self, relpath):
+    def readonly_repository(self, repo):
+        relpath = urlutils.basename(repo.bzrdir.user_url.rstrip('/'))
         return BzrDir.open_from_transport(
             self.get_readonly_transport(relpath)).open_repository()
 
@@ -58,26 +60,45 @@
         # because developers use this api to setup the tree, branch and
         # repository for their tests: having it not give the right repository
         # type would invalidate the tests.
-        self.make_branch_and_tree('repo')
-        repo = self.make_referring('referring', 'repo')
+        tree = self.make_branch_and_tree('repo')
+        repo = self.make_referring('referring', tree.branch.repository)
         self.assertIsInstance(repo._format,
             self.repository_format.__class__)
 
 
 class TestIncompatibleStacking(TestCaseWithRepository):
 
-    def test_add_fallback_repository_rejects_incompatible(self):
-        # Repository.add_fallback_repository raises IncompatibleRepositories if
-        # you take two repositories in different serializations and try to
-        # stack them.
-        if self.make_repository('test')._format.supports_chks:
+    def make_repo_and_incompatible_fallback(self):
+        referring = self.make_repository('referring')
+        if referring._format.supports_chks:
             different_fmt = '1.9'
         else:
             different_fmt = '2a'
-        repo = self.make_repository('repo', format=different_fmt)
-        referring = self.make_repository('referring')
-        self.assertRaises(errors.IncompatibleRepositories,
-                referring.add_fallback_repository, repo)
+        fallback = self.make_repository('fallback', format=different_fmt)
+        return referring, fallback
+
+    def test_add_fallback_repository_rejects_incompatible(self):
+        # Repository.add_fallback_repository raises IncompatibleRepositories
+        # if you take two repositories in different serializations and try to
+        # stack them.
+        referring, fallback = self.make_repo_and_incompatible_fallback()
+        self.assertRaises(errors.IncompatibleRepositories,
+                referring.add_fallback_repository, fallback)
+
+    def test_add_fallback_doesnt_leave_fallback_locked(self):
+        # Bug #835035. If the referring repository is locked, it wants to lock
+        # the fallback repository. But if they are incompatible, the referring
+        # repository won't take ownership of the fallback, and thus should not
+        # leave the repository in a locked state.
+        referring, fallback = self.make_repo_and_incompatible_fallback()
+        self.addCleanup(referring.lock_read().unlock)
+        # Assert precondition.
+        self.assertFalse(fallback.is_locked())
+        # Assert action.
+        self.assertRaises(errors.IncompatibleRepositories,
+                referring.add_fallback_repository, fallback)
+        # Assert postcondition.
+        self.assertFalse(fallback.is_locked())
 
 
 def external_reference_test_scenarios():

=== modified file 'bzrlib/tests/per_repository_reference/test_add_inventory.py'
--- bzrlib/tests/per_repository_reference/test_add_inventory.py	2009-04-09 20:23:07 +0000
+++ bzrlib/tests/per_repository_reference/test_add_inventory.py	2011-10-01 11:54:46 +0000
@@ -33,7 +33,7 @@
         tree.lock_read()
         self.addCleanup(tree.unlock)
         base = self.make_repository('base')
-        repo = self.make_referring('referring', 'base')
+        repo = self.make_referring('referring', base)
         repo.lock_write()
         try:
             repo.start_write_group()

=== modified file 'bzrlib/tests/per_repository_reference/test_add_revision.py'
--- bzrlib/tests/per_repository_reference/test_add_revision.py	2009-09-08 14:51:23 +0000
+++ bzrlib/tests/per_repository_reference/test_add_revision.py	2011-10-01 11:54:46 +0000
@@ -34,7 +34,7 @@
         self.addCleanup(tree.unlock)
         rev = tree.branch.repository.get_revision(revid)
         base = self.make_repository('base')
-        repo = self.make_referring('referring', 'base')
+        repo = self.make_referring('referring', base)
         repo.lock_write()
         try:
             repo.start_write_group()

=== modified file 'bzrlib/tests/per_repository_reference/test_add_signature_text.py'
--- bzrlib/tests/per_repository_reference/test_add_signature_text.py	2009-09-08 14:51:23 +0000
+++ bzrlib/tests/per_repository_reference/test_add_signature_text.py	2011-10-01 11:54:46 +0000
@@ -33,7 +33,7 @@
         tree.lock_read()
         self.addCleanup(tree.unlock)
         base = self.make_repository('base')
-        repo = self.make_referring('referring', 'base')
+        repo = self.make_referring('referring', base)
         repo.lock_write()
         try:
             repo.start_write_group()

=== modified file 'bzrlib/tests/per_repository_reference/test_all_revision_ids.py'
--- bzrlib/tests/per_repository_reference/test_all_revision_ids.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/per_repository_reference/test_all_revision_ids.py	2011-10-01 11:54:46 +0000
@@ -16,7 +16,6 @@
 
 """Tests for all_revision_ids on a repository with external references."""
 
-from bzrlib import errors
 from bzrlib.tests.per_repository_reference import (
     TestCaseWithExternalReferenceRepository,
     )
@@ -26,20 +25,20 @@
 
     def test_all_revision_ids_empty(self):
         base = self.make_repository('base')
-        repo = self.make_referring('referring', 'base')
+        repo = self.make_referring('referring', base)
         self.assertEqual(set([]), set(repo.all_revision_ids()))
 
     def test_all_revision_ids_from_base(self):
         tree = self.make_branch_and_tree('base')
         revid = tree.commit('one')
-        repo = self.make_referring('referring', 'base')
+        repo = self.make_referring('referring', tree.branch.repository)
         self.assertEqual(set([revid]), set(repo.all_revision_ids()))
 
     def test_all_revision_ids_from_repo(self):
         tree = self.make_branch_and_tree('spare')
         revid = tree.commit('one')
         base = self.make_repository('base')
-        repo = self.make_referring('referring', 'base')
+        repo = self.make_referring('referring', base)
         repo.fetch(tree.branch.repository, revid)
         self.assertEqual(set([revid]), set(repo.all_revision_ids()))
 
@@ -48,7 +47,7 @@
         revid = tree.commit('one')
         base_tree = self.make_branch_and_tree('base')
         revid2 = base_tree.commit('two')
-        repo = self.make_referring('referring', 'base')
+        repo = self.make_referring('referring', base_tree.branch.repository)
         repo.fetch(tree.branch.repository, revid)
         self.assertEqual(set([revid, revid2]), set(repo.all_revision_ids()))
 
@@ -56,7 +55,7 @@
         tree = self.make_branch_and_tree('spare')
         revid = tree.commit('one')
         base = self.make_repository('base')
-        repo = self.make_referring('referring', 'base')
+        repo = self.make_referring('referring', base)
         repo.fetch(tree.branch.repository, revid)
         base.fetch(tree.branch.repository, revid)
         self.assertEqual(set([revid]), set(repo.all_revision_ids()))

=== modified file 'bzrlib/tests/per_repository_reference/test_break_lock.py'
--- bzrlib/tests/per_repository_reference/test_break_lock.py	2010-01-25 17:48:22 +0000
+++ bzrlib/tests/per_repository_reference/test_break_lock.py	2011-10-01 11:54:46 +0000
@@ -29,7 +29,7 @@
 
     def test_break_lock(self):
         base = self.make_repository('base')
-        repo = self.make_referring('referring', 'base')
+        repo = self.make_referring('referring', base)
         unused_repo = repo.bzrdir.open_repository()
         base.lock_write()
         self.addCleanup(base.unlock)

=== modified file 'bzrlib/tests/per_repository_reference/test_check.py'
--- bzrlib/tests/per_repository_reference/test_check.py	2010-02-17 17:11:16 +0000
+++ bzrlib/tests/per_repository_reference/test_check.py	2011-10-01 11:54:46 +0000
@@ -17,7 +17,6 @@
 """Tests for check on a repository with external references."""
 
 import bzrlib.ui
-from bzrlib import errors
 from bzrlib.tests.per_repository_reference import (
     TestCaseWithExternalReferenceRepository,
     )
@@ -31,7 +30,7 @@
         tree.add(['file'], ['file-id'])
         rev1_id = tree.commit('one')
         referring = self.make_branch_and_tree('referring')
-        readonly_base = self.readonly_repository('base')
+        readonly_base = self.readonly_repository(tree.branch.repository)
         referring.branch.repository.add_fallback_repository(readonly_base)
         local_tree = referring.branch.create_checkout('local')
         self.build_tree_contents([('local/file', 'change')])

=== modified file 'bzrlib/tests/per_repository_reference/test_fetch.py'
--- bzrlib/tests/per_repository_reference/test_fetch.py	2010-12-06 04:59:38 +0000
+++ bzrlib/tests/per_repository_reference/test_fetch.py	2011-12-05 14:12:23 +0000
@@ -17,11 +17,7 @@
 
 from bzrlib import (
     branch,
-    errors,
-    graph,
-    )
-from bzrlib.smart import (
-    server,
+    vf_search,
     )
 from bzrlib.tests.per_repository import TestCaseWithRepository
 
@@ -176,7 +172,7 @@
         self.addCleanup(target.lock_write().unlock)
         target.fetch(
             repo_missing_fallbacks,
-            fetch_spec=graph.EverythingResult(repo_missing_fallbacks))
+            fetch_spec=vf_search.EverythingResult(repo_missing_fallbacks))
         self.assertEqual(repo_missing_fallbacks.revisions.keys(),
             target.revisions.keys())
         self.assertEqual(repo_missing_fallbacks.inventories.keys(),

=== modified file 'bzrlib/tests/per_repository_reference/test_get_record_stream.py'
--- bzrlib/tests/per_repository_reference/test_get_record_stream.py	2009-07-15 18:22:38 +0000
+++ bzrlib/tests/per_repository_reference/test_get_record_stream.py	2011-10-01 11:54:46 +0000
@@ -82,7 +82,7 @@
         self.all_repo.lock_read()
         self.addCleanup(self.all_repo.unlock)
         self.base_repo = self.make_repository('base')
-        self.stacked_repo = self.make_referring('referring', 'base')
+        self.stacked_repo = self.make_referring('referring', self.base_repo)
 
     def make_simple_split(self):
         """Set up the repositories so that everything is in base except F"""

=== modified file 'bzrlib/tests/per_repository_reference/test_get_rev_id_for_revno.py'
--- bzrlib/tests/per_repository_reference/test_get_rev_id_for_revno.py	2009-08-20 04:09:58 +0000
+++ bzrlib/tests/per_repository_reference/test_get_rev_id_for_revno.py	2011-10-01 11:54:46 +0000
@@ -31,7 +31,8 @@
         spare_tree = tree.bzrdir.sprout('spare').open_workingtree()
         revid3 = spare_tree.commit('three')
         branch = spare_tree.branch.create_clone_on_transport(
-            self.get_transport('referring'), stacked_on=self.get_url('base'))
+            self.get_transport('referring'),
+            stacked_on=tree.branch.base)
         repo = branch.repository
         # Sanity check: now repo has 'revid3', and base has 'revid' + 'revid2'
         self.assertEqual(set([revid3]),

=== modified file 'bzrlib/tests/per_repository_reference/test_graph.py'
--- bzrlib/tests/per_repository_reference/test_graph.py	2011-08-26 11:26:29 +0000
+++ bzrlib/tests/per_repository_reference/test_graph.py	2011-10-01 11:39:29 +0000
@@ -20,8 +20,8 @@
 
 from bzrlib import (
     remote,
-    repository,
     tests,
+    urlutils,
     )
 from bzrlib.tests.per_repository import TestCaseWithRepository
 
@@ -43,9 +43,11 @@
         wt_a = self.make_branch_and_tree('a')
         branch_a = wt_a.branch
         branch_b = self.make_branch('b')
-        branch_b.set_stacked_on_url('../a')
+        branch_b.set_stacked_on_url(
+            urlutils.relative_url(branch_b.base, branch_a.base))
         branch_c = self.make_branch('c')
-        branch_c.set_stacked_on_url('../b')
+        branch_c.set_stacked_on_url(
+            urlutils.relative_url(branch_c.base, branch_b.base))
         revid_1 = wt_a.commit('first commit')
         return branch_a, branch_b, branch_c, revid_1
 

=== modified file 'bzrlib/tests/per_repository_vf/__init__.py'
--- bzrlib/tests/per_repository_vf/__init__.py	2011-08-15 16:13:40 +0000
+++ bzrlib/tests/per_repository_vf/__init__.py	2011-09-12 09:51:52 +0000
@@ -43,6 +43,8 @@
         'test_find_text_key_references',
         'test__generate_text_key_index',
         'test_fetch',
+        'test_fileid_involved',
+        'test_merge_directive',
         'test_reconcile',
         'test_refresh_data',
         'test_repository',

=== modified file 'bzrlib/tests/per_repository_vf/test_add_inventory_by_delta.py'
--- bzrlib/tests/per_repository_vf/test_add_inventory_by_delta.py	2011-05-02 22:39:15 +0000
+++ bzrlib/tests/per_repository_vf/test_add_inventory_by_delta.py	2011-12-13 22:41:31 +0000
@@ -109,4 +109,4 @@
         else:
             repo_delta.commit_write_group()
         self.assertEqual(add_validator, delta_validator)
-        self.assertEqual(new_inv, inv)
+        self.assertEqual(list(new_inv.iter_entries()), list(inv.iter_entries()))

=== modified file 'bzrlib/tests/per_repository_vf/test_check.py'
--- bzrlib/tests/per_repository_vf/test_check.py	2011-05-26 08:05:45 +0000
+++ bzrlib/tests/per_repository_vf/test_check.py	2011-10-15 00:12:58 +0000
@@ -117,3 +117,24 @@
     def branch_callback(self, refs):
         self.callbacks.append(('branch', refs))
         return self.branch_check(refs)
+
+
+class TestNoSpuriousInconsistentAncestors(TestCaseWithRepository):
+
+    scenarios = all_repository_vf_format_scenarios()
+
+    def test_two_files_different_versions_no_inconsistencies_bug_165071(self):
+        """Two files, with different versions can be clean."""
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['foo'])
+        tree.smart_add(['.'])
+        revid1 = tree.commit('1')
+        self.build_tree(['bar'])
+        tree.smart_add(['.'])
+        revid2 = tree.commit('2')
+        check_object = tree.branch.repository.check([revid1, revid2])
+        check_object.report_results(verbose=True)
+        self.assertContainsRe(self.get_log(), "0 unreferenced text versions")
+
+
+

=== modified file 'bzrlib/tests/per_repository_vf/test_check_reconcile.py'
--- bzrlib/tests/per_repository_vf/test_check_reconcile.py	2011-05-11 14:26:41 +0000
+++ bzrlib/tests/per_repository_vf/test_check_reconcile.py	2011-12-09 12:05:47 +0000
@@ -461,7 +461,8 @@
 
         # make rev1b: A well-formed revision, containing 'a-file'
         # rev1b of a-file has the exact same contents as rev1a.
-        file_contents = repo.revision_tree('rev1a').get_file_text('a-file-id')
+        file_contents = repo.texts.get_record_stream([('a-file-id', 'rev1a')],
+            "unordered", False).next().get_bytes_as('fulltext')
         inv = self.make_one_file_inventory(
             repo, 'rev1b', [], root_revision='rev1b',
             file_contents=file_contents)
@@ -822,7 +823,7 @@
         revision = Revision(revision_id, committer='jrandom@example.com',
             timestamp=0, inventory_sha1='', timezone=0, message='foo',
             parent_ids=parent_ids)
-        repo.add_revision(revision_id,revision, inv)
+        repo.add_revision(revision_id, revision, inv)
 
     def make_one_file_inventory(self, repo, revision, parents,
                                 inv_revision=None, root_revision=None,

=== modified file 'bzrlib/tests/per_repository_vf/test_fetch.py'
--- bzrlib/tests/per_repository_vf/test_fetch.py	2011-05-02 22:44:40 +0000
+++ bzrlib/tests/per_repository_vf/test_fetch.py	2011-12-05 14:12:23 +0000
@@ -17,7 +17,7 @@
 """Tests for fetch between repositories of the same type."""
 
 from bzrlib import (
-    graph,
+    vf_search,
     )
 from bzrlib.tests.per_repository_vf import (
     TestCaseWithRepository,
@@ -49,7 +49,7 @@
         self.addCleanup(b.unlock)
         repo = b.repository
         source = repo._get_source(repo._format)
-        search = graph.PendingAncestryResult(['tip'], repo)
+        search = vf_search.PendingAncestryResult(['tip'], repo)
         stream = source.get_stream(search)
         for substream_type, substream in stream:
             for record in substream:

=== added file 'bzrlib/tests/per_repository_vf/test_fileid_involved.py'
--- bzrlib/tests/per_repository_vf/test_fileid_involved.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/per_repository_vf/test_fileid_involved.py	2011-10-06 00:14:01 +0000
@@ -0,0 +1,432 @@
+# Copyright (C) 2005-2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+import sys
+import time
+
+from bzrlib import (
+    errors,
+    inventory,
+    remote,
+    revision as _mod_revision,
+    tests,
+    transform,
+    )
+from bzrlib.tests.scenarios import load_tests_apply_scenarios
+from bzrlib.tests.per_repository_vf import (
+    TestCaseWithRepository,
+    all_repository_vf_format_scenarios,
+    )
+
+
+load_tests = load_tests_apply_scenarios
+
+
+class FileIdInvolvedWGhosts(TestCaseWithRepository):
+
+    scenarios = all_repository_vf_format_scenarios()
+
+    def create_branch_with_ghost_text(self):
+        builder = self.make_branch_builder('ghost')
+        builder.build_snapshot('A-id', None, [
+            ('add', ('', 'root-id', 'directory', None)),
+            ('add', ('a', 'a-file-id', 'file', 'some content\n'))])
+        b = builder.get_branch()
+        old_rt = b.repository.revision_tree('A-id')
+        new_inv = inventory.mutable_inventory_from_tree(old_rt)
+        new_inv.revision_id = 'B-id'
+        new_inv['a-file-id'].revision = 'ghost-id'
+        new_rev = _mod_revision.Revision('B-id',
+            timestamp=time.time(),
+            timezone=0,
+            message='Committing against a ghost',
+            committer='Joe Foo <joe@foo.com>',
+            properties={},
+            parent_ids=('A-id', 'ghost-id'),
+            )
+        b.lock_write()
+        self.addCleanup(b.unlock)
+        b.repository.start_write_group()
+        b.repository.add_revision('B-id', new_rev, new_inv)
+        self.disable_commit_write_group_paranoia(b.repository)
+        b.repository.commit_write_group()
+        return b
+
+    def disable_commit_write_group_paranoia(self, repo):
+        if isinstance(repo, remote.RemoteRepository):
+            # We can't easily disable the checks in a remote repo.
+            repo.abort_write_group()
+            raise tests.TestSkipped(
+                "repository format does not support storing revisions with "
+                "missing texts.")
+        pack_coll = getattr(repo, '_pack_collection', None)
+        if pack_coll is not None:
+            # Monkey-patch the pack collection instance to allow storing
+            # incomplete revisions.
+            pack_coll._check_new_inventories = lambda: []
+
+    def test_file_ids_include_ghosts(self):
+        b = self.create_branch_with_ghost_text()
+        repo = b.repository
+        self.assertEqual(
+            {'a-file-id':set(['ghost-id'])},
+            repo.fileids_altered_by_revision_ids(['B-id']))
+
+    def test_file_ids_uses_fallbacks(self):
+        builder = self.make_branch_builder('source',
+                                           format=self.bzrdir_format)
+        repo = builder.get_branch().repository
+        if not repo._format.supports_external_lookups:
+            raise tests.TestNotApplicable('format does not support stacking')
+        builder.start_series()
+        builder.build_snapshot('A-id', None, [
+            ('add', ('', 'root-id', 'directory', None)),
+            ('add', ('file', 'file-id', 'file', 'contents\n'))])
+        builder.build_snapshot('B-id', ['A-id'], [
+            ('modify', ('file-id', 'new-content\n'))])
+        builder.build_snapshot('C-id', ['B-id'], [
+            ('modify', ('file-id', 'yet more content\n'))])
+        builder.finish_series()
+        source_b = builder.get_branch()
+        source_b.lock_read()
+        self.addCleanup(source_b.unlock)
+        base = self.make_branch('base')
+        base.pull(source_b, stop_revision='B-id')
+        stacked = self.make_branch('stacked')
+        stacked.set_stacked_on_url('../base')
+        stacked.pull(source_b, stop_revision='C-id')
+
+        stacked.lock_read()
+        self.addCleanup(stacked.unlock)
+        repo = stacked.repository
+        keys = {'file-id': set(['A-id'])}
+        if stacked.repository.supports_rich_root():
+            keys['root-id'] = set(['A-id'])
+        self.assertEqual(keys, repo.fileids_altered_by_revision_ids(['A-id']))
+
+
+class FileIdInvolvedBase(TestCaseWithRepository):
+
+    def touch(self, tree, filename):
+        # use the trees transport to not depend on the tree's location or type.
+        tree.bzrdir.root_transport.append_bytes(filename, "appended line\n")
+
+    def compare_tree_fileids(self, branch, old_rev, new_rev):
+        old_tree = self.branch.repository.revision_tree(old_rev)
+        new_tree = self.branch.repository.revision_tree(new_rev)
+        delta = new_tree.changes_from(old_tree)
+
+        l2 = [id for path, id, kind in delta.added] + \
+             [id for oldpath, newpath, id, kind, text_modified, \
+                meta_modified in delta.renamed] + \
+             [id for path, id, kind, text_modified, meta_modified in \
+                delta.modified]
+        return set(l2)
+
+
+class TestFileIdInvolved(FileIdInvolvedBase):
+
+    scenarios = all_repository_vf_format_scenarios()
+
+    def setUp(self):
+        super(TestFileIdInvolved, self).setUp()
+        # create three branches, and merge it
+        #
+        #          ,-->J------>K                (branch2)
+        #         /             \
+        #  A --->B --->C---->D-->G              (main)
+        #  \          /     /
+        #   '--->E---+---->F                    (branch1)
+
+        # A changes:
+        # B changes: 'a-file-id-2006-01-01-abcd'
+        # C changes:  Nothing (perfect merge)
+        # D changes: 'b-file-id-2006-01-01-defg'
+        # E changes: 'file-d'
+        # F changes: 'file-d'
+        # G changes: 'b-file-id-2006-01-01-defg'
+        # J changes: 'b-file-id-2006-01-01-defg'
+        # K changes: 'c-funky<file-id>quiji%bo'
+
+        main_wt = self.make_branch_and_tree('main')
+        main_branch = main_wt.branch
+        self.build_tree(["main/a","main/b","main/c"])
+
+        main_wt.add(['a', 'b', 'c'], ['a-file-id-2006-01-01-abcd',
+                                 'b-file-id-2006-01-01-defg',
+                                 'c-funky<file-id>quiji%bo'])
+        try:
+            main_wt.commit("Commit one", rev_id="rev-A")
+        except errors.IllegalPath:
+            # TODO: jam 20060701 Consider raising a different exception
+            #       newer formats do support this, and nothin can done to
+            #       correct this test - its not a bug.
+            if sys.platform == 'win32':
+                raise tests.TestSkipped('Old repository formats do not'
+                                        ' support file ids with <> on win32')
+            # This is not a known error condition
+            raise
+
+        #-------- end A -----------
+
+        bt1 = self.make_branch_and_tree('branch1')
+        bt1.pull(main_branch)
+        b1 = bt1.branch
+        self.build_tree(["branch1/d"])
+        bt1.add(['d'], ['file-d'])
+        bt1.commit("branch1, Commit one", rev_id="rev-E")
+
+        #-------- end E -----------
+
+        self.touch(main_wt, "a")
+        main_wt.commit("Commit two", rev_id="rev-B")
+
+        #-------- end B -----------
+
+        bt2 = self.make_branch_and_tree('branch2')
+        bt2.pull(main_branch)
+        branch2_branch = bt2.branch
+        set_executability(bt2, 'b', True)
+        bt2.commit("branch2, Commit one", rev_id="rev-J")
+
+        #-------- end J -----------
+
+        main_wt.merge_from_branch(b1)
+        main_wt.commit("merge branch1, rev-11", rev_id="rev-C")
+
+        #-------- end C -----------
+
+        bt1.rename_one("d","e")
+        bt1.commit("branch1, commit two", rev_id="rev-F")
+
+        #-------- end F -----------
+
+        self.touch(bt2, "c")
+        bt2.commit("branch2, commit two", rev_id="rev-K")
+
+        #-------- end K -----------
+
+        main_wt.merge_from_branch(b1)
+        self.touch(main_wt, "b")
+        # D gets some funky characters to make sure the unescaping works
+        main_wt.commit("merge branch1, rev-12", rev_id="rev-<D>")
+
+        # end D
+
+        main_wt.merge_from_branch(branch2_branch)
+        main_wt.commit("merge branch1, rev-22",  rev_id="rev-G")
+
+        # end G
+        self.branch = main_branch
+
+    def test_fileids_altered_between_two_revs(self):
+        self.branch.lock_read()
+        self.addCleanup(self.branch.unlock)
+        self.branch.repository.fileids_altered_by_revision_ids(["rev-J","rev-K"])
+        self.assertEqual(
+            {'b-file-id-2006-01-01-defg':set(['rev-J']),
+             'c-funky<file-id>quiji%bo':set(['rev-K'])
+             },
+            self.branch.repository.fileids_altered_by_revision_ids(["rev-J","rev-K"]))
+
+        self.assertEqual(
+            {'b-file-id-2006-01-01-defg': set(['rev-<D>']),
+             'file-d': set(['rev-F']),
+             },
+            self.branch.repository.fileids_altered_by_revision_ids(['rev-<D>', 'rev-F']))
+
+        self.assertEqual(
+            {
+             'b-file-id-2006-01-01-defg': set(['rev-<D>', 'rev-G', 'rev-J']),
+             'c-funky<file-id>quiji%bo': set(['rev-K']),
+             'file-d': set(['rev-F']),
+             },
+            self.branch.repository.fileids_altered_by_revision_ids(
+                ['rev-<D>', 'rev-G', 'rev-F', 'rev-K', 'rev-J']))
+
+        self.assertEqual(
+            {'a-file-id-2006-01-01-abcd': set(['rev-B']),
+             'b-file-id-2006-01-01-defg': set(['rev-<D>', 'rev-G', 'rev-J']),
+             'c-funky<file-id>quiji%bo': set(['rev-K']),
+             'file-d': set(['rev-F']),
+             },
+            self.branch.repository.fileids_altered_by_revision_ids(
+                ['rev-G', 'rev-F', 'rev-C', 'rev-B', 'rev-<D>', 'rev-K', 'rev-J']))
+
+    def fileids_altered_by_revision_ids(self, revision_ids):
+        """This is a wrapper to strip TREE_ROOT if it occurs"""
+        repo = self.branch.repository
+        root_id = self.branch.basis_tree().get_root_id()
+        result = repo.fileids_altered_by_revision_ids(revision_ids)
+        if root_id in result:
+            del result[root_id]
+        return result
+
+    def test_fileids_altered_by_revision_ids(self):
+        self.branch.lock_read()
+        self.addCleanup(self.branch.unlock)
+        self.assertEqual(
+            {'a-file-id-2006-01-01-abcd':set(['rev-A']),
+             'b-file-id-2006-01-01-defg': set(['rev-A']),
+             'c-funky<file-id>quiji%bo': set(['rev-A']),
+             },
+            self.fileids_altered_by_revision_ids(["rev-A"]))
+        self.assertEqual(
+            {'a-file-id-2006-01-01-abcd':set(['rev-B'])
+             },
+            self.branch.repository.fileids_altered_by_revision_ids(["rev-B"]))
+        self.assertEqual(
+            {'b-file-id-2006-01-01-defg':set(['rev-<D>'])
+             },
+            self.branch.repository.fileids_altered_by_revision_ids(["rev-<D>"]))
+
+    def test_fileids_involved_full_compare(self):
+        # this tests that the result of each fileid_involved calculation
+        # along a revision history selects only the fileids selected by
+        # comparing the trees - no less, and no more. This is correct
+        # because in our sample data we do not revert any file ids along
+        # the revision history.
+        self.branch.lock_read()
+        self.addCleanup(self.branch.unlock)
+        pp=[]
+        graph = self.branch.repository.get_graph()
+        history = list(graph.iter_lefthand_ancestry(self.branch.last_revision(),
+            [_mod_revision.NULL_REVISION]))
+        history.reverse()
+
+        if len(history) < 2:
+            return
+
+        for start in range(0,len(history)-1):
+            start_id = history[start]
+            for end in range(start+1,len(history)):
+                end_id = history[end]
+                unique_revs = graph.find_unique_ancestors(end_id, [start_id])
+                l1 = self.branch.repository.fileids_altered_by_revision_ids(
+                    unique_revs)
+                l1 = set(l1.keys())
+                l2 = self.compare_tree_fileids(self.branch, start_id, end_id)
+                self.assertEquals(l1, l2)
+
+
+class TestFileIdInvolvedNonAscii(FileIdInvolvedBase):
+
+    scenarios = all_repository_vf_format_scenarios()
+
+    def test_utf8_file_ids_and_revision_ids(self):
+        main_wt = self.make_branch_and_tree('main')
+        main_branch = main_wt.branch
+        self.build_tree(["main/a"])
+
+        file_id = u'a-f\xedle-id'.encode('utf8')
+        main_wt.add(['a'], [file_id])
+        revision_id = u'r\xe9v-a'.encode('utf8')
+        try:
+            main_wt.commit('a', rev_id=revision_id)
+        except errors.NonAsciiRevisionId:
+            raise tests.TestSkipped('non-ascii revision ids not supported by %s'
+                                    % self.repository_format)
+
+        repo = main_wt.branch.repository
+        repo.lock_read()
+        self.addCleanup(repo.unlock)
+        file_ids = repo.fileids_altered_by_revision_ids([revision_id])
+        root_id = main_wt.basis_tree().get_root_id()
+        if root_id in file_ids:
+            self.assertEqual({file_id:set([revision_id]),
+                              root_id:set([revision_id])
+                             }, file_ids)
+        else:
+            self.assertEqual({file_id:set([revision_id])}, file_ids)
+
+
+class TestFileIdInvolvedSuperset(FileIdInvolvedBase):
+
+    scenarios = all_repository_vf_format_scenarios()
+
+    def setUp(self):
+        super(TestFileIdInvolvedSuperset, self).setUp()
+
+        self.branch = None
+        main_wt = self.make_branch_and_tree('main')
+        main_branch = main_wt.branch
+        self.build_tree(["main/a","main/b","main/c"])
+
+        main_wt.add(['a', 'b', 'c'], ['a-file-id-2006-01-01-abcd',
+                                 'b-file-id-2006-01-01-defg',
+                                 'c-funky<file-id>quiji\'"%bo'])
+        try:
+            main_wt.commit("Commit one", rev_id="rev-A")
+        except errors.IllegalPath:
+            # TODO: jam 20060701 Consider raising a different exception
+            #       newer formats do support this, and nothin can done to
+            #       correct this test - its not a bug.
+            if sys.platform == 'win32':
+                raise tests.TestSkipped('Old repository formats do not'
+                                        ' support file ids with <> on win32')
+            # This is not a known error condition
+            raise
+
+        branch2_wt = self.make_branch_and_tree('branch2')
+        branch2_wt.pull(main_branch)
+        branch2_bzrdir = branch2_wt.bzrdir
+        branch2_branch = branch2_bzrdir.open_branch()
+        set_executability(branch2_wt, 'b', True)
+        branch2_wt.commit("branch2, Commit one", rev_id="rev-J")
+
+        main_wt.merge_from_branch(branch2_branch)
+        set_executability(main_wt, 'b', False)
+        main_wt.commit("merge branch1, rev-22",  rev_id="rev-G")
+
+        # end G
+        self.branch = main_branch
+
+    def test_fileid_involved_full_compare2(self):
+        # this tests that fileids_altered_by_revision_ids returns
+        # more information than compare_tree can, because it
+        # sees each change rather than the aggregate delta.
+        self.branch.lock_read()
+        self.addCleanup(self.branch.unlock)
+        graph = self.branch.repository.get_graph()
+        history = list(graph.iter_lefthand_ancestry(self.branch.last_revision(),
+            [_mod_revision.NULL_REVISION]))
+        history.reverse()
+        old_rev = history[0]
+        new_rev = history[1]
+        unique_revs = graph.find_unique_ancestors(new_rev, [old_rev])
+
+        l1 = self.branch.repository.fileids_altered_by_revision_ids(
+            unique_revs)
+        l1 = set(l1.keys())
+
+        l2 = self.compare_tree_fileids(self.branch, old_rev, new_rev)
+        self.assertNotEqual(l2, l1)
+        self.assertSubset(l2, l1)
+
+
+def set_executability(wt, path, executable=True):
+    """Set the executable bit for the file at path in the working tree
+
+    os.chmod() doesn't work on windows. But TreeTransform can mark or
+    unmark a file as executable.
+    """
+    file_id = wt.path2id(path)
+    tt = transform.TreeTransform(wt)
+    try:
+        tt.set_executability(executable, tt.trans_id_tree_file_id(file_id))
+        tt.apply()
+    finally:
+        tt.finalize()

=== renamed file 'bzrlib/tests/per_repository/test_merge_directive.py' => 'bzrlib/tests/per_repository_vf/test_merge_directive.py'
--- bzrlib/tests/per_repository/test_merge_directive.py	2009-07-22 17:22:06 +0000
+++ bzrlib/tests/per_repository_vf/test_merge_directive.py	2011-09-09 12:19:12 +0000
@@ -22,16 +22,22 @@
 
 from bzrlib import (
     chk_map,
-    errors,
     merge_directive,
-    tests,
-    )
-
-from bzrlib.tests.per_repository import TestCaseWithRepository
+    )
+
+from bzrlib.tests.scenarios import load_tests_apply_scenarios
+from bzrlib.tests.per_repository_vf import (
+    TestCaseWithRepository,
+    all_repository_vf_format_scenarios,
+    )
+
+load_tests = load_tests_apply_scenarios
 
 
 class TestMergeDirective(TestCaseWithRepository):
 
+    scenarios = all_repository_vf_format_scenarios()
+
     def make_two_branches(self):
         builder = self.make_branch_builder('source')
         builder.start_series()

=== modified file 'bzrlib/tests/per_repository_vf/test_reconcile.py'
--- bzrlib/tests/per_repository_vf/test_reconcile.py	2011-06-19 15:23:02 +0000
+++ bzrlib/tests/per_repository_vf/test_reconcile.py	2011-10-01 11:36:04 +0000
@@ -349,9 +349,10 @@
         # we should add a lower level api to allow constructing such cases.
 
         # first off the common logic:
-        tree = self.make_branch_and_tree('wrong-first-parent')
-        second_tree = self.make_branch_and_tree('reversed-secondary-parents')
-        for t in [tree, second_tree]:
+        self.first_tree = self.make_branch_and_tree('wrong-first-parent')
+        self.second_tree = self.make_branch_and_tree(
+            'reversed-secondary-parents')
+        for t in [self.first_tree, self.second_tree]:
             t.commit('1', rev_id='1')
             uncommit(t.branch, tree=t)
             t.commit('2', rev_id='2')
@@ -360,10 +361,10 @@
             uncommit(t.branch, tree=t)
         #second_tree = self.make_branch_and_tree('reversed-secondary-parents')
         #second_tree.pull(tree) # XXX won't copy the repo?
-        repo_secondary = second_tree.branch.repository
+        repo_secondary = self.second_tree.branch.repository
 
         # now setup the wrong-first parent case
-        repo = tree.branch.repository
+        repo = self.first_tree.branch.repository
         repo.lock_write()
         repo.start_write_group()
         inv = Inventory(revision_id='wrong-first-parent')
@@ -406,9 +407,7 @@
 
     def test_reconcile_wrong_order(self):
         # a wrong order in primary parents is optionally correctable
-        t = self.get_transport().clone('wrong-first-parent')
-        d = bzrlib.bzrdir.BzrDir.open_from_transport(t)
-        repo = d.open_repository()
+        repo = self.first_tree.branch.repository
         repo.lock_read()
         try:
             g = repo.get_graph()
@@ -417,7 +416,7 @@
                 raise TestSkipped('wrong-first-parent is not setup for testing')
         finally:
             repo.unlock()
-        self.checkUnreconciled(d, repo.reconcile())
+        self.checkUnreconciled(repo.bzrdir, repo.reconcile())
         # nothing should have been altered yet : inventories without
         # revisions are not data loss incurring for current format
         reconciler = repo.reconcile(thorough=True)
@@ -435,8 +434,6 @@
 
     def test_reconcile_wrong_order_secondary_inventory(self):
         # a wrong order in the parents for inventories is ignored.
-        t = self.get_transport().clone('reversed-secondary-parents')
-        d = bzrlib.bzrdir.BzrDir.open_from_transport(t)
-        repo = d.open_repository()
-        self.checkUnreconciled(d, repo.reconcile())
-        self.checkUnreconciled(d, repo.reconcile(thorough=True))
+        repo = self.second_tree.branch.repository
+        self.checkUnreconciled(repo.bzrdir, repo.reconcile())
+        self.checkUnreconciled(repo.bzrdir, repo.reconcile(thorough=True))

=== modified file 'bzrlib/tests/per_transport.py'
--- bzrlib/tests/per_transport.py	2011-08-19 22:34:02 +0000
+++ bzrlib/tests/per_transport.py	2011-12-23 19:38:22 +0000
@@ -53,9 +53,11 @@
 from bzrlib.tests.test_transport import TestTransportImplementation
 from bzrlib.transport import (
     ConnectedTransport,
+    Transport,
     _get_transport_modules,
     )
 from bzrlib.transport.memory import MemoryTransport
+from bzrlib.transport.remote import RemoteTransport
 
 
 def get_transport_test_permutations(module):
@@ -1791,6 +1793,18 @@
         transport = _mod_transport.get_transport_from_url(url)
         self.assertEquals(parameters, transport.get_segment_parameters())
 
+    def test_set_segment_parameters(self):
+        """Segment parameters can be set and show up in base."""
+        transport = self.get_transport("foo")
+        orig_base = transport.base
+        transport.set_segment_parameter("arm", "board")
+        self.assertEquals("%s,arm=board" % orig_base, transport.base)
+        self.assertEquals({"arm": "board"}, transport.get_segment_parameters())
+        transport.set_segment_parameter("arm", None)
+        transport.set_segment_parameter("nonexistant", None)
+        self.assertEquals({}, transport.get_segment_parameters())
+        self.assertEquals(orig_base, transport.base)
+
     def test_stat_symlink(self):
         # if a transport points directly to a symlink (and supports symlinks
         # at all) you can tell this.  helps with bug 32669.
@@ -1802,3 +1816,57 @@
         t2 = t.clone('link')
         st = t2.stat('')
         self.assertTrue(stat.S_ISLNK(st.st_mode))
+
+    def test_abspath_url_unquote_unreserved(self):
+        """URLs from abspath should have unreserved characters unquoted
+        
+        Need consistent quoting notably for tildes, see lp:842223 for more.
+        """
+        t = self.get_transport()
+        needlessly_escaped_dir = "%2D%2E%30%39%41%5A%5F%61%7A%7E/"
+        self.assertEqual(t.base + "-.09AZ_az~",
+            t.abspath(needlessly_escaped_dir))
+
+    def test_clone_url_unquote_unreserved(self):
+        """Base URL of a cloned branch needs unreserved characters unquoted
+        
+        Cloned transports should be prefix comparable for things like the
+        isolation checking of tests, see lp:842223 for more.
+        """
+        t1 = self.get_transport()
+        needlessly_escaped_dir = "%2D%2E%30%39%41%5A%5F%61%7A%7E/"
+        self.build_tree([needlessly_escaped_dir], transport=t1)
+        t2 = t1.clone(needlessly_escaped_dir)
+        self.assertEqual(t1.base + "-.09AZ_az~/", t2.base)
+
+    def test_hook_post_connection_one(self):
+        """Fire post_connect hook after a ConnectedTransport is first used"""
+        log = []
+        Transport.hooks.install_named_hook("post_connect", log.append, None)
+        t = self.get_transport()
+        self.assertEqual([], log)
+        t.has("non-existant")
+        if isinstance(t, RemoteTransport):
+            self.assertEqual([t.get_smart_medium()], log)
+        elif isinstance(t, ConnectedTransport):
+            self.assertEqual([t], log)
+        else:
+            self.assertEqual([], log)
+
+    def test_hook_post_connection_multi(self):
+        """Fire post_connect hook once per unshared underlying connection"""
+        log = []
+        Transport.hooks.install_named_hook("post_connect", log.append, None)
+        t1 = self.get_transport()
+        t2 = t1.clone(".")
+        t3 = self.get_transport()
+        self.assertEqual([], log)
+        t1.has("x")
+        t2.has("x")
+        t3.has("x")
+        if isinstance(t1, RemoteTransport):
+            self.assertEqual([t.get_smart_medium() for t in [t1, t3]], log)
+        elif isinstance(t1, ConnectedTransport):
+            self.assertEqual([t1, t3], log)
+        else:
+            self.assertEqual([], log)

=== modified file 'bzrlib/tests/per_tree/test_get_file_mtime.py'
--- bzrlib/tests/per_tree/test_get_file_mtime.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/per_tree/test_get_file_mtime.py	2011-09-21 00:30:44 +0000
@@ -18,6 +18,8 @@
 
 import time
 
+from bzrlib import errors
+
 from bzrlib.tests.per_tree import TestCaseWithTree
 
 
@@ -42,3 +44,10 @@
                         'now: %f, mtime_file_id: %f' % (now, mtime_file_id ))
         mtime_path = tree.get_file_mtime(file_id='one-id', path='one')
         self.assertEqual(mtime_file_id, mtime_path)
+
+    def test_nonexistant(self):
+        tree = self.get_basic_tree()
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
+        self.assertRaises(errors.NoSuchId,
+            tree.get_file_mtime, file_id='unexistant')

=== modified file 'bzrlib/tests/per_tree/test_path_content_summary.py'
--- bzrlib/tests/per_tree/test_path_content_summary.py	2011-06-14 01:26:41 +0000
+++ bzrlib/tests/per_tree/test_path_content_summary.py	2011-12-16 17:34:46 +0000
@@ -133,10 +133,7 @@
         self.assertEqual('file', summary[0])
         self.check_content_summary_size(tree, summary, 22)
         # not executable
-        if osutils.supports_executable:
-            self.assertEqual(False, summary[2])
-        else:
-            self.assertEqual(None, summary[2])
+        self.assertEqual(False, summary[2])
         # may have hash,
         self.assertSubset((summary[3],),
             (None, '0c352290ae1c26ca7f97d5b2906c4624784abd60'))

=== modified file 'bzrlib/tests/per_tree/test_test_trees.py'
--- bzrlib/tests/per_tree/test_test_trees.py	2011-06-14 01:26:41 +0000
+++ bzrlib/tests/per_tree/test_test_trees.py	2011-09-04 21:41:05 +0000
@@ -16,7 +16,6 @@
 
 """Tests for the test trees used by the per_tree tests."""
 
-from bzrlib import tests
 from bzrlib.tests import per_tree
 from bzrlib.tests import (
     features,

=== modified file 'bzrlib/tests/per_tree/test_tree.py'
--- bzrlib/tests/per_tree/test_tree.py	2011-08-05 13:34:40 +0000
+++ bzrlib/tests/per_tree/test_tree.py	2011-08-29 19:06:57 +0000
@@ -323,3 +323,11 @@
         if kind == "SHA1":
             expected = osutils.sha_strings('file content')
             self.assertEqual(expected, data)
+
+
+class TestHasVersionedDirectories(TestCaseWithTree):
+
+    def test_has_versioned_directories(self):
+        work_tree = self.make_branch_and_tree('tree')
+        tree = self._convert_tree(work_tree)
+        self.assertSubset([tree.has_versioned_directories()], (True, False))

=== modified file 'bzrlib/tests/per_workingtree/__init__.py'
--- bzrlib/tests/per_workingtree/__init__.py	2011-05-16 15:23:36 +0000
+++ bzrlib/tests/per_workingtree/__init__.py	2011-09-23 12:32:30 +0000
@@ -76,7 +76,7 @@
 
     def make_branch_builder(self, relpath, format=None):
         if format is None:
-            format = self.bzrdir_format
+            format = self.workingtree_format.get_controldir_for_branch()
         builder = branchbuilder.BranchBuilder(self.get_transport(relpath),
                                               format=format)
         return builder

=== modified file 'bzrlib/tests/per_workingtree/test_add.py'
--- bzrlib/tests/per_workingtree/test_add.py	2011-08-16 13:15:28 +0000
+++ bzrlib/tests/per_workingtree/test_add.py	2011-08-29 21:53:33 +0000
@@ -101,8 +101,8 @@
         tree.add(['dir/subdir/foo'], ['foo-id'])
         root_id = tree.get_root_id()
 
-        self.assertTreeLayout([('', root_id), ('dir', 'dir-id'),
-                               ('dir/subdir', 'subdir-id'),
+        self.assertTreeLayout([('', root_id), ('dir/', 'dir-id'),
+                               ('dir/subdir/', 'subdir-id'),
                                ('dir/subdir/foo', 'foo-id')], tree)
 
     def test_add_multiple(self):
@@ -113,7 +113,7 @@
         root_id = tree.get_root_id()
 
         self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id'),
-                               ('dir', 'dir-id'), ('dir/subdir', 'subdir-id'),
+                               ('dir/', 'dir-id'), ('dir/subdir/', 'subdir-id'),
                                ('dir/subdir/foo', 'foo-id')], tree)
 
     def test_add_invalid(self):

=== modified file 'bzrlib/tests/per_workingtree/test_check.py'
--- bzrlib/tests/per_workingtree/test_check.py	2011-05-04 21:10:36 +0000
+++ bzrlib/tests/per_workingtree/test_check.py	2011-09-04 21:47:01 +0000
@@ -18,11 +18,17 @@
 
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
 
+from bzrlib.workingtree import InventoryWorkingTree
+from bzrlib.tests import TestNotApplicable
+
 
 class TestCheck(TestCaseWithWorkingTree):
 
     def test__get_check_refs_new(self):
         tree = self.make_branch_and_tree('tree')
+        if not isinstance(tree, InventoryWorkingTree):
+            raise TestNotApplicable(
+                "_get_check_refs only relevant for inventory working trees")
         self.assertEqual(set([('trees', 'null:')]),
             set(tree._get_check_refs()))
 
@@ -30,6 +36,9 @@
         # with a basis, all current bzr trees cache it and so need the
         # inventory to cross-check.
         tree = self.make_branch_and_tree('tree')
+        if not isinstance(tree, InventoryWorkingTree):
+            raise TestNotApplicable(
+                "_get_check_refs only relevant for inventory working trees")
         revid = tree.commit('first post')
         self.assertEqual(set([('trees', revid)]),
             set(tree._get_check_refs()))
@@ -37,6 +46,9 @@
     def test__check_with_refs(self):
         # _check can be called with a dict of the things required.
         tree = self.make_branch_and_tree('tree')
+        if not isinstance(tree, InventoryWorkingTree):
+            raise TestNotApplicable(
+                "_get_check_refs only relevant for inventory working trees")
         tree.lock_write()
         self.addCleanup(tree.unlock)
         revid = tree.commit('first post')

=== modified file 'bzrlib/tests/per_workingtree/test_executable.py'
--- bzrlib/tests/per_workingtree/test_executable.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/per_workingtree/test_executable.py	2011-12-19 16:59:14 +0000
@@ -189,9 +189,7 @@
         self.assertFalse(b_executable)
 
     def test_use_exec_from_basis(self):
-        if osutils.supports_executable():
-            self.assertEqual(self.wt._is_executable_from_path_and_stat_from_stat,
-                             self.wt._is_executable_from_path_and_stat)
-        else:
-            self.assertEqual(self.wt._is_executable_from_path_and_stat_from_basis,
-                             self.wt._is_executable_from_path_and_stat)
+        self.wt._supports_executable = lambda: False
+        self.addCleanup(self.wt.lock_read().unlock)
+        self.assertTrue(self.wt.is_executable(self.a_id))
+        self.assertFalse(self.wt.is_executable(self.b_id))

=== modified file 'bzrlib/tests/per_workingtree/test_get_file_mtime.py'
--- bzrlib/tests/per_workingtree/test_get_file_mtime.py	2009-07-10 07:14:02 +0000
+++ bzrlib/tests/per_workingtree/test_get_file_mtime.py	2011-09-21 00:30:34 +0000
@@ -18,6 +18,7 @@
 
 import os
 
+from bzrlib import errors
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
 
 
@@ -98,3 +99,15 @@
             self.assertAlmostEqual(st.st_mtime, mtime)
         finally:
             tree.unlock()
+
+    def test_missing(self):
+        tree = self.make_basic_tree()
+
+        os.remove('tree/one')
+        tree.lock_read()
+        try:
+            self.assertRaises(errors.FileTimestampUnavailable,
+                tree.get_file_mtime, file_id='one-id')
+        finally:
+            tree.unlock()
+

=== modified file 'bzrlib/tests/per_workingtree/test_move.py'
--- bzrlib/tests/per_workingtree/test_move.py	2011-08-16 13:15:28 +0000
+++ bzrlib/tests/per_workingtree/test_move.py	2011-08-29 21:53:33 +0000
@@ -126,12 +126,12 @@
                           tree.move, ['d', 'c', 'b'], 'a')
         if osutils.lexists('a/c'):
             # If 'c' was actually moved, then 'd' should have also been moved
-            self.assertTreeLayout([('', root_id), ('a', 'a-id'),
+            self.assertTreeLayout([('', root_id), ('a/', 'a-id'),
                                    ('a/c', 'c-id'),  ('a/d', 'd-id')], tree)
         else:
-            self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('c', 'c-id'),
+            self.assertTreeLayout([('', root_id), ('a/', 'a-id'), ('c', 'c-id'),
                                    ('d', 'd-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('c', 'c-id'),
+        self.assertTreeLayout([('', root_id), ('a/', 'a-id'), ('c', 'c-id'),
                                ('d', 'd-id')], tree.basis_tree())
         tree._validate()
 
@@ -145,7 +145,7 @@
         tree.remove(['a/b'], keep_files=False)
         self.assertEqual([('b', 'a/b')], tree.move(['b'], 'a'))
         self.assertTreeLayout([('', root_id),
-                               ('a', 'a-id'),
+                               ('a/', 'a-id'),
                                ('a/b', 'b-id'),
                               ], tree)
         tree._validate()
@@ -156,16 +156,16 @@
         tree.add(['a', 'b', 'b/c'], ['a-id', 'b-id', 'c-id'])
         tree.commit('initial', rev_id='rev-1')
         root_id = tree.get_root_id()
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id'),
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id'),
                                ('b/c', 'c-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id'),
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id'),
                                ('b/c', 'c-id')], tree.basis_tree())
         a_contents = tree.get_file_text('a-id')
         self.assertEqual([('a', 'b/a')],
             tree.move(['a'], 'b'))
-        self.assertTreeLayout([('', root_id), ('b', 'b-id'), ('b/a', 'a-id'),
+        self.assertTreeLayout([('', root_id), ('b/', 'b-id'), ('b/a', 'a-id'),
                                ('b/c', 'c-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id'),
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id'),
                                ('b/c', 'c-id')], tree.basis_tree())
         self.assertPathDoesNotExist('a')
         self.assertFileEqual(a_contents, 'b/a')
@@ -180,9 +180,9 @@
         c_contents = tree.get_file_text('c-id')
         self.assertEqual([('b/c', 'c')],
             tree.move(['b/c'], ''))
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id'),
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id'),
                                ('c', 'c-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id'),
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id'),
                                ('b/c', 'c-id')], tree.basis_tree())
         self.assertPathDoesNotExist('b/c')
         self.assertFileEqual(c_contents, 'c')
@@ -200,13 +200,13 @@
         # 'c' may or may not have been moved, but either way the tree should
         # maintain a consistent state.
         if osutils.lexists('c'):
-            self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id'),
+            self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id'),
                                    ('c', 'c-id')], tree)
         else:
             self.assertPathExists('b/c')
-            self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id'),
+            self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id'),
                                    ('b/c', 'c-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id'),
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id'),
                                ('c', 'c-id')], tree.basis_tree())
         tree._validate()
 
@@ -238,15 +238,15 @@
         root_id = tree.get_root_id()
         os.rename('a', 'b/a')
 
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree)
         # We don't need after=True as long as source is missing and target
         # exists.
         self.assertEqual([('a', 'b/a')],
             tree.move(['a'], 'b'))
-        self.assertTreeLayout([('', root_id), ('b', 'b-id'), ('b/a', 'a-id')],
+        self.assertTreeLayout([('', root_id), ('b/', 'b-id'), ('b/a', 'a-id')],
                               tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree.basis_tree())
         tree._validate()
 
@@ -258,14 +258,14 @@
         root_id = tree.get_root_id()
         os.rename('a', 'b/a')
 
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree)
         # Passing after=True should work as well
         self.assertEqual([('a', 'b/a')],
             tree.move(['a'], 'b', after=True))
-        self.assertTreeLayout([('', root_id), ('b', 'b-id'), ('b/a', 'a-id')],
+        self.assertTreeLayout([('', root_id), ('b/', 'b-id'), ('b/a', 'a-id')],
                               tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree.basis_tree())
         tree._validate()
 
@@ -279,7 +279,7 @@
         # Passing after when the file hasn't been move raises an exception
         self.assertRaises(errors.BzrMoveFailedError,
                           tree.move, ['a'], 'b', after=True)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree.basis_tree())
         tree._validate()
 
@@ -303,20 +303,20 @@
         finally:
             ba_file.close()
 
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree)
         self.assertRaises(errors.RenameFailedFilesExist,
                           tree.move, ['a'], 'b', after=False)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree)
         self.assertFileEqual(a_text, 'a')
         self.assertFileEqual(ba_text, 'b/a')
         # But you can pass after=True
         self.assertEqual([('a', 'b/a')],
             tree.move(['a'], 'b', after=True))
-        self.assertTreeLayout([('', root_id), ('b', 'b-id'), ('b/a', 'a-id')],
+        self.assertTreeLayout([('', root_id), ('b/', 'b-id'), ('b/a', 'a-id')],
                               tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree.basis_tree())
         # But it shouldn't actually move anything
         self.assertFileEqual(a_text, 'a')
@@ -333,11 +333,11 @@
 
         self.assertEqual([('a', 'e/a')],
             tree.move(['a'], 'e'))
-        self.assertTreeLayout([('', root_id), ('e', 'e-id'), ('e/a', 'a-id'),
-                               ('e/a/b', 'b-id'), ('e/a/c', 'c-id'),
+        self.assertTreeLayout([('', root_id), ('e/', 'e-id'), ('e/a/', 'a-id'),
+                               ('e/a/b', 'b-id'), ('e/a/c/', 'c-id'),
                                ('e/a/c/d', 'd-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('e', 'e-id'),
-                               ('a/b', 'b-id'), ('a/c', 'c-id'),
+        self.assertTreeLayout([('', root_id), ('a/', 'a-id'), ('e/', 'e-id'),
+                               ('a/b', 'b-id'), ('a/c/', 'c-id'),
                                ('a/c/d', 'd-id')], tree.basis_tree())
         tree._validate()
 
@@ -355,9 +355,9 @@
         self.assertEqual([('c/b', 'b')],
                          tree.move(['c/b'], ''))
         self.assertTreeLayout([('', root_id),
-                               ('b', 'b-id'),
-                               ('c', 'c-id'),
-                               ('b/d', 'd-id'),
+                               ('b/', 'b-id'),
+                               ('c/', 'c-id'),
+                               ('b/d/', 'd-id'),
                               ], tree)
         tree._validate()
 
@@ -370,25 +370,18 @@
         root_id = tree.get_root_id()
 
         tree.rename_one('a/b', 'a/c/b')
-        if self.workingtree_format.supports_versioned_directories:
-            self.assertTreeLayout([('', root_id),
-                                   ('a', 'a-id'),
-                                   ('d', 'd-id'),
-                                   ('a/c', 'c-id'),
-                                   ('a/c/b', 'b-id'),
-                                  ], tree)
-        else:
-            self.assertTreeLayout([('', root_id),
-                                   ('a', 'a-id'),
-                                   ('a/c', 'c-id'),
-                                   ('a/c/b', 'b-id'),
-                                  ], tree)
+        self.assertTreeLayout([('', root_id),
+                               ('a/', 'a-id'),
+                               ('d/', 'd-id'),
+                               ('a/c/', 'c-id'),
+                               ('a/c/b', 'b-id'),
+                              ], tree)
         self.assertEqual([('a', 'd/a')],
                          tree.move(['a'], 'd'))
         self.assertTreeLayout([('', root_id),
-                               ('d', 'd-id'),
-                               ('d/a', 'a-id'),
-                               ('d/a/c', 'c-id'),
+                               ('d/', 'd-id'),
+                               ('d/a/', 'a-id'),
+                               ('d/a/c/', 'c-id'),
                                ('d/a/c/b', 'b-id'),
                               ], tree)
         tree._validate()
@@ -406,8 +399,8 @@
         self.assertEqual([('a', 'b/a')],
                          tree.move(['a'], 'b'))
         self.assertTreeLayout([('', root_id),
-                               ('b', 'b-id'),
-                               ('b/a', 'a-id'),
+                               ('b/', 'b-id'),
+                               ('b/a/', 'a-id'),
                                ('b/a/c', 'ac-id'),
                               ], tree)
         tree._validate()
@@ -425,8 +418,8 @@
         self.assertEqual([('a', 'b/a')],
                          tree.move(['a'], 'b'))
         self.assertTreeLayout([('', root_id),
-                               ('b', 'b-id'),
-                               ('b/a', 'a-id'),
+                               ('b/', 'b-id'),
+                               ('b/a/', 'a-id'),
                                ('b/a/b', 'ab-id'),
                                ('b/a/c', 'ac-id'),
                                ('b/a/d', 'ad-id'),
@@ -444,18 +437,18 @@
         self.assertEqual([('a/b', 'b')],
                          tree.move(['a/b'], ''))
         self.assertTreeLayout([('', root_id),
-                               ('a', 'a-id'),
+                               ('a/', 'a-id'),
                                ('b', 'b-id'),
                                ('d', 'd-id'),
-                               ('e', 'e-id'),
+                               ('e/', 'e-id'),
                                ('a/c', 'c-id'),
                               ], tree)
         self.assertEqual([('d', 'a/d')],
                          tree.move(['d'], 'a'))
         self.assertTreeLayout([('', root_id),
-                               ('a', 'a-id'),
+                               ('a/', 'a-id'),
                                ('b', 'b-id'),
-                               ('e', 'e-id'),
+                               ('e/', 'e-id'),
                                ('a/c', 'c-id'),
                                ('a/d', 'd-id'),
                               ], tree)
@@ -463,8 +456,8 @@
                          tree.move(['a'], 'e'))
         self.assertTreeLayout([('', root_id),
                                ('b', 'b-id'),
-                               ('e', 'e-id'),
-                               ('e/a', 'a-id'),
+                               ('e/', 'e-id'),
+                               ('e/a/', 'a-id'),
                                ('e/a/c', 'c-id'),
                                ('e/a/d', 'd-id'),
                               ], tree)
@@ -480,16 +473,16 @@
 
         tree.rename_one('a/b', 'a/d')
         self.assertTreeLayout([('', root_id),
-                               ('a', 'a-id'),
-                               ('d', 'd-id'),
+                               ('a/', 'a-id'),
+                               ('d/', 'd-id'),
                                ('a/c', 'c-id'),
                                ('a/d', 'b-id'),
                               ], tree)
         self.assertEqual([('a', 'd/a')],
                          tree.move(['a'], 'd'))
         self.assertTreeLayout([('', root_id),
-                               ('d', 'd-id'),
-                               ('d/a', 'a-id'),
+                               ('d/', 'd-id'),
+                               ('d/a/', 'a-id'),
                                ('d/a/c', 'c-id'),
                                ('d/a/d', 'b-id'),
                               ], tree)
@@ -507,8 +500,8 @@
         tree.rename_one('a/d', 'a/b')
         tree.rename_one('a/bb', 'a/d')
         self.assertTreeLayout([('', root_id),
-                               ('a', 'a-id'),
-                               ('e', 'e-id'),
+                               ('a/', 'a-id'),
+                               ('e/', 'e-id'),
                                ('a/b', 'd-id'),
                                ('a/c', 'c-id'),
                                ('a/d', 'b-id'),
@@ -516,8 +509,8 @@
         self.assertEqual([('a', 'e/a')],
                          tree.move(['a'], 'e'))
         self.assertTreeLayout([('', root_id),
-                               ('e', 'e-id'),
-                               ('e/a', 'a-id'),
+                               ('e/', 'e-id'),
+                               ('e/a/', 'a-id'),
                                ('e/a/b', 'd-id'),
                                ('e/a/c', 'c-id'),
                                ('e/a/d', 'b-id'),
@@ -534,16 +527,16 @@
 
         self.assertEqual([('a/b', 'c/b')],
             tree.move(['a/b'], 'c'))
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('c', 'c-id'),
+        self.assertTreeLayout([('', root_id), ('a/', 'a-id'), ('c/', 'c-id'),
                                ('c/b', 'b-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('c', 'c-id'),
+        self.assertTreeLayout([('', root_id), ('a/', 'a-id'), ('c/', 'c-id'),
                                ('a/b', 'b-id')], tree.basis_tree())
 
         self.assertEqual([('c/b', 'b')],
             tree.move(['c/b'], ''))
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id'),
-                               ('c', 'c-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('c', 'c-id'),
+        self.assertTreeLayout([('', root_id), ('a/', 'a-id'), ('b', 'b-id'),
+                               ('c/', 'c-id')], tree)
+        self.assertTreeLayout([('', root_id), ('a/', 'a-id'), ('c/', 'c-id'),
                                ('a/b', 'b-id')], tree.basis_tree())
         tree._validate()
 

=== modified file 'bzrlib/tests/per_workingtree/test_paths2ids.py'
--- bzrlib/tests/per_workingtree/test_paths2ids.py	2009-07-10 07:14:02 +0000
+++ bzrlib/tests/per_workingtree/test_paths2ids.py	2011-12-05 17:57:53 +0000
@@ -24,6 +24,7 @@
 from operator import attrgetter
 
 from bzrlib import errors
+from bzrlib.tests import features
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
 
 
@@ -188,3 +189,13 @@
             ['unversioned'], [tree])
         basis.unlock()
         tree.unlock()
+
+    def test_unversioned_non_ascii_one_tree(self):
+        self.requireFeature(features.UnicodeFilenameFeature)
+        tree = self.make_branch_and_tree('.')
+        self.build_tree([u"\xa7"])
+        self.assertExpectedIds([], tree, [u"\xa7"], require_versioned=False)
+        self.addCleanup(tree.lock_read().unlock)
+        e = self.assertRaises(errors.PathsNotVersionedError,
+            tree.paths2ids, [u"\xa7"])
+        self.assertEqual([u"\xa7"], e.paths)

=== modified file 'bzrlib/tests/per_workingtree/test_pull.py'
--- bzrlib/tests/per_workingtree/test_pull.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/per_workingtree/test_pull.py	2011-11-08 12:49:00 +0000
@@ -17,6 +17,7 @@
 
 
 from bzrlib import tests
+from bzrlib.revision import NULL_REVISION
 from bzrlib.tests import per_workingtree
 
 
@@ -30,6 +31,12 @@
         tree_b = self.make_branch_and_tree('to')
         return tree, tree_b
 
+    def test_pull_null(self):
+        tree_a, tree_b = self.get_pullable_trees()
+        root_id = tree_a.get_root_id()
+        tree_a.pull(tree_b.branch, stop_revision=NULL_REVISION, overwrite=True)
+        self.assertEquals(root_id, tree_a.get_root_id())
+
     def test_pull(self):
         tree_a, tree_b = self.get_pullable_trees()
         tree_b.pull(tree_a.branch)
@@ -39,7 +46,7 @@
     def test_pull_overwrites(self):
         tree_a, tree_b = self.get_pullable_trees()
         tree_b.commit('foo', rev_id='B')
-        self.assertEqual(['B'], tree_b.branch.revision_history())
+        self.assertEqual('B', tree_b.branch.last_revision())
         tree_b.pull(tree_a.branch, overwrite=True)
         self.assertTrue(tree_b.branch.repository.has_revision('A'))
         self.assertTrue(tree_b.branch.repository.has_revision('B'))

=== modified file 'bzrlib/tests/per_workingtree/test_readonly.py'
--- bzrlib/tests/per_workingtree/test_readonly.py	2011-05-08 13:07:11 +0000
+++ bzrlib/tests/per_workingtree/test_readonly.py	2011-08-31 22:52:43 +0000
@@ -26,6 +26,8 @@
     )
 from bzrlib.tests.per_workingtree import TestCaseWithWorkingTree
 
+from bzrlib.workingtree import InventoryWorkingTree
+
 
 class TestReadonly(TestCaseWithWorkingTree):
 
@@ -77,6 +79,8 @@
         """Even if the tree is unclean, we should still handle readonly dirs."""
         # First create a tree
         tree = self.create_basic_tree()
+        if not isinstance(tree, InventoryWorkingTree):
+            raise tests.TestNotApplicable("requires inventory working tree")
 
         # XXX: *Ugly* *ugly* hack, we need the hashcache to think it is out of
         # date, but we don't want to actually wait 3 seconds doing nothing.

=== modified file 'bzrlib/tests/per_workingtree/test_rename_one.py'
--- bzrlib/tests/per_workingtree/test_rename_one.py	2011-08-16 13:15:28 +0000
+++ bzrlib/tests/per_workingtree/test_rename_one.py	2011-08-29 21:53:33 +0000
@@ -81,9 +81,9 @@
 
         a_contents = tree.get_file_text('a-id')
         tree.rename_one('a', 'foo')
-        self.assertTreeLayout([('', root_id), ('b', 'b-id'), ('foo', 'a-id')],
+        self.assertTreeLayout([('', root_id), ('b/', 'b-id'), ('foo', 'a-id')],
                               tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree.basis_tree())
         self.assertPathDoesNotExist('a')
         self.assertFileEqual(a_contents, 'foo')
@@ -97,9 +97,9 @@
 
         a_contents = tree.get_file_text('a-id')
         tree.rename_one('a', 'b/foo')
-        self.assertTreeLayout([('', root_id), ('b', 'b-id'), ('b/foo', 'a-id')],
+        self.assertTreeLayout([('', root_id), ('b/', 'b-id'), ('b/foo', 'a-id')],
                               tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree.basis_tree())
         self.assertPathDoesNotExist('tree/a')
         self.assertFileEqual(a_contents, 'tree/b/foo')
@@ -110,15 +110,15 @@
         tree.add(['a', 'b', 'b/c'], ['a-id', 'b-id', 'c-id'])
         tree.commit('initial', rev_id='rev-1')
         root_id = tree.get_root_id()
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id'),
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id'),
                                ('b/c', 'c-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id'),
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id'),
                                ('b/c', 'c-id')], tree.basis_tree())
         a_contents = tree.get_file_text('a-id')
         tree.rename_one('a', 'b/d')
-        self.assertTreeLayout([('', root_id), ('b', 'b-id'), ('b/c', 'c-id'),
+        self.assertTreeLayout([('', root_id), ('b/', 'b-id'), ('b/c', 'c-id'),
                                ('b/d', 'a-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id'),
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id'),
                                ('b/c', 'c-id')], tree.basis_tree())
         self.assertPathDoesNotExist('a')
         self.assertFileEqual(a_contents, 'b/d')
@@ -131,9 +131,9 @@
         root_id = tree.get_root_id()
         c_contents = tree.get_file_text('c-id')
         tree.rename_one('b/c', 'd')
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id'),
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id'),
                                ('d', 'c-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id'),
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id'),
                                ('b/c', 'c-id')], tree.basis_tree())
         self.assertPathDoesNotExist('b/c')
         self.assertFileEqual(c_contents, 'd')
@@ -147,9 +147,9 @@
         # Target already exists
         self.assertRaises(errors.RenameFailedFilesExist,
                           tree.rename_one, 'a', 'b/a')
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id'),
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id'),
                                ('c', 'c-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id'),
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id'),
                                ('c', 'c-id')], tree.basis_tree())
 
     def test_rename_one_onto_existing(self):
@@ -187,14 +187,14 @@
         root_id = tree.get_root_id()
         os.rename('a', 'b/foo')
 
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree)
         # We don't need after=True as long as source is missing and target
         # exists.
         tree.rename_one('a', 'b/foo')
-        self.assertTreeLayout([('', root_id), ('b', 'b-id'),
+        self.assertTreeLayout([('', root_id), ('b/', 'b-id'),
                                ('b/foo', 'a-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree.basis_tree())
 
     def test_rename_one_after_with_after(self):
@@ -205,13 +205,13 @@
         root_id = tree.get_root_id()
         os.rename('a', 'b/foo')
 
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree)
         # Passing after=True should work as well
         tree.rename_one('a', 'b/foo', after=True)
-        self.assertTreeLayout([('', root_id), ('b', 'b-id'),
+        self.assertTreeLayout([('', root_id), ('b/', 'b-id'),
                                ('b/foo', 'a-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree.basis_tree())
 
     def test_rename_one_after_dest_versioned(self):
@@ -269,13 +269,13 @@
         os.rename('a', 'b/foo')
         tree.remove(['a'])
 
-        self.assertTreeLayout([('', root_id), ('b', 'b-id')], tree)
+        self.assertTreeLayout([('', root_id), ('b/', 'b-id')], tree)
         # We don't need after=True as long as source is missing and target
         # exists.
         tree.rename_one('a', 'b/foo')
-        self.assertTreeLayout([('', root_id), ('b', 'b-id'),
+        self.assertTreeLayout([('', root_id), ('b/', 'b-id'),
                                ('b/foo', 'a-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree.basis_tree())
 
     def test_rename_one_after_no_target(self):
@@ -288,9 +288,9 @@
         # Passing after when the file hasn't been rename_one raises an exception
         self.assertRaises(errors.BzrMoveFailedError,
                           tree.rename_one, 'a', 'b/foo', after=True)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree.basis_tree())
 
     def test_rename_one_after_source_and_dest(self):
@@ -313,19 +313,19 @@
         finally:
             foo_file.close()
 
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree)
         self.assertRaises(errors.RenameFailedFilesExist,
                           tree.rename_one, 'a', 'b/foo', after=False)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree)
         self.assertFileEqual(a_text, 'a')
         self.assertFileEqual(foo_text, 'b/foo')
         # But you can pass after=True
         tree.rename_one('a', 'b/foo', after=True)
-        self.assertTreeLayout([('', root_id), ('b', 'b-id'),
+        self.assertTreeLayout([('', root_id), ('b/', 'b-id'),
                                ('b/foo', 'a-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b', 'b-id')],
+        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('b/', 'b-id')],
                               tree.basis_tree())
         # But it shouldn't actually move anything
         self.assertFileEqual(a_text, 'a')
@@ -340,11 +340,11 @@
         root_id = tree.get_root_id()
 
         tree.rename_one('a', 'e/f')
-        self.assertTreeLayout([('', root_id), ('e', 'e-id'), ('e/f', 'a-id'),
-                               ('e/f/b', 'b-id'), ('e/f/c', 'c-id'),
+        self.assertTreeLayout([('', root_id), ('e/', 'e-id'), ('e/f/', 'a-id'),
+                               ('e/f/b', 'b-id'), ('e/f/c/', 'c-id'),
                                ('e/f/c/d', 'd-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('e', 'e-id'),
-                               ('a/b', 'b-id'), ('a/c', 'c-id'),
+        self.assertTreeLayout([('', root_id), ('a/', 'a-id'), ('e/', 'e-id'),
+                               ('a/b', 'b-id'), ('a/c/', 'c-id'),
                                ('a/c/d', 'd-id')], tree.basis_tree())
 
     def test_rename_one_moved(self):
@@ -356,15 +356,15 @@
         root_id = tree.get_root_id()
 
         tree.rename_one('a/b', 'c/foo')
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('c', 'c-id'),
+        self.assertTreeLayout([('', root_id), ('a/', 'a-id'), ('c/', 'c-id'),
                                ('c/foo', 'b-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('c', 'c-id'),
+        self.assertTreeLayout([('', root_id), ('a/', 'a-id'), ('c/', 'c-id'),
                                ('a/b', 'b-id')], tree.basis_tree())
 
         tree.rename_one('c/foo', 'bar')
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('bar', 'b-id'),
-                               ('c', 'c-id')], tree)
-        self.assertTreeLayout([('', root_id), ('a', 'a-id'), ('c', 'c-id'),
+        self.assertTreeLayout([('', root_id), ('a/', 'a-id'), ('bar', 'b-id'),
+                               ('c/', 'c-id')], tree)
+        self.assertTreeLayout([('', root_id), ('a/', 'a-id'), ('c/', 'c-id'),
                                ('a/b', 'b-id')], tree.basis_tree())
 
     def test_rename_to_denormalised_fails(self):

=== modified file 'bzrlib/tests/per_workingtree/test_smart_add.py'
--- bzrlib/tests/per_workingtree/test_smart_add.py	2011-06-14 01:26:41 +0000
+++ bzrlib/tests/per_workingtree/test_smart_add.py	2011-09-06 09:51:45 +0000
@@ -25,6 +25,7 @@
     ignores,
     osutils,
     tests,
+    trace,
     )
 from bzrlib.tests import (
     features,
@@ -91,6 +92,19 @@
         for path in paths:
             self.assertNotEqual(wt.path2id(path), None)
 
+    def test_skip_nested_trees(self):
+        """Test smart-adding a nested tree ignors it and warns."""
+        wt = self.make_branch_and_tree('.')
+        nested_wt = self.make_branch_and_tree('nested')
+        warnings = []
+        def warning(*args):
+            warnings.append(args[0] % args[1:])
+        self.overrideAttr(trace, 'warning', warning)
+        wt.smart_add((u".",))
+        self.assertIs(wt.path2id("nested"), None)
+        self.assertEquals(
+            ['skipping nested tree %r' % nested_wt.basedir], warnings)
+
     def test_add_dot_from_subdir(self):
         """Test adding . from a subdir of the tree."""
         paths = ("original/", "original/file1", "original/file2")
@@ -150,7 +164,8 @@
         self.build_tree(tree_shape)
         wt.smart_add(add_paths)
         for path in expected_paths:
-            self.assertNotEqual(wt.path2id(path), None, "No id added for %s" % path)
+            self.assertNotEqual(wt.path2id(path), None,
+                "No id added for %s" % path)
 
     def test_add_non_existant(self):
         """Test smart-adding a file that does not exist."""

=== modified file 'bzrlib/tests/per_workingtree/test_workingtree.py'
--- bzrlib/tests/per_workingtree/test_workingtree.py	2011-08-17 15:38:20 +0000
+++ bzrlib/tests/per_workingtree/test_workingtree.py	2011-12-16 17:34:46 +0000
@@ -22,11 +22,11 @@
 
 from bzrlib import (
     branch,
-    branchbuilder,
     bzrdir,
     config,
     errors,
     osutils,
+    revision as _mod_revision,
     symbol_versioning,
     tests,
     trace,
@@ -249,7 +249,7 @@
 
         wt.commit('create initial state')
 
-        revid = b.revision_history()[0]
+        revid = b.last_revision()
         self.log('first revision_id is {%s}' % revid)
 
         tree = b.repository.revision_tree(revid)
@@ -312,6 +312,12 @@
         cloned = cloned_dir.open_workingtree()
         self.assertEqual(cloned.get_parent_ids(), wt.get_parent_ids())
 
+    def test_clone_empty(self):
+        wt = self.make_branch_and_tree('source')
+        cloned_dir = wt.bzrdir.clone('target', revision_id=_mod_revision.NULL_REVISION)
+        cloned = cloned_dir.open_workingtree()
+        self.assertEqual(cloned.get_parent_ids(), wt.get_parent_ids())
+
     def test_last_revision(self):
         wt = self.make_branch_and_tree('source')
         self.assertEqual([], wt.get_parent_ids())
@@ -623,8 +629,8 @@
         # which should have pivoted the local tip into a merge
         self.assertEqual([master_tip, 'bar'], tree.get_parent_ids())
         # and the local branch history should match the masters now.
-        self.assertEqual(master_tree.branch.revision_history(),
-            tree.branch.revision_history())
+        self.assertEqual(master_tree.branch.last_revision(),
+            tree.branch.last_revision())
 
     def test_update_takes_revision_parameter(self):
         wt = self.make_branch_and_tree('wt')
@@ -940,6 +946,9 @@
             case_sensitive = True
         tree = self.make_branch_and_tree('test')
         self.assertEqual(case_sensitive, tree.case_sensitive)
+        if not isinstance(tree, InventoryWorkingTree):
+            raise TestNotApplicable("get_format_string is only available "
+                                    "on bzr working trees")
         # now we cheat, and make a file that matches the case-sensitive name
         t = tree.bzrdir.get_workingtree_transport(None)
         try:
@@ -951,6 +960,24 @@
         tree = tree.bzrdir.open_workingtree()
         self.assertFalse(tree.case_sensitive)
 
+    def test_supports_executable(self):
+        self.build_tree(['filename'])
+        tree = self.make_branch_and_tree('.')
+        tree.add('filename')
+        self.assertIsInstance(tree._supports_executable(), bool)
+        if tree._supports_executable():
+            tree.lock_read()
+            try:
+                self.assertFalse(tree.is_executable(tree.path2id('filename')))
+            finally:
+                tree.unlock()
+            os.chmod('filename', 0755)
+            self.addCleanup(tree.lock_read().unlock)
+            self.assertTrue(tree.is_executable(tree.path2id('filename')))
+        else:
+            self.addCleanup(tree.lock_read().unlock)
+            self.assertFalse(tree.is_executable(tree.path2id('filename')))
+
     def test_all_file_ids_with_missing(self):
         tree = self.make_branch_and_tree('tree')
         tree.lock_write()
@@ -1001,10 +1028,9 @@
             4 5-M
             |
             W
-         """
-        builder = branchbuilder.BranchBuilder(
-            self.get_transport(),
-            format=self.workingtree_format._matchingbzrdir)
+        """
+        format = self.workingtree_format.get_controldir_for_branch()
+        builder = self.make_branch_builder(".", format=format)
         builder.start_series()
         # mainline
         builder.build_snapshot(

=== modified file 'bzrlib/tests/script.py'
--- bzrlib/tests/script.py	2011-01-12 01:01:53 +0000
+++ bzrlib/tests/script.py	2011-09-28 15:06:42 +0000
@@ -481,6 +481,10 @@
     def setUp(self):
         super(TestCaseWithMemoryTransportAndScript, self).setUp()
         self.script_runner = ScriptRunner()
+        # FIXME: See shelf_ui.Shelver._char_based. This allow using shelve in
+        # scripts while providing a line-based input (better solution in
+        # progress). -- vila 2011-09-28
+        self.overrideEnv('INSIDE_EMACS', '1')
 
     def run_script(self, script, null_output_matches_anything=False):
         return self.script_runner.run_script(self, script, 
@@ -511,6 +515,10 @@
     def setUp(self):
         super(TestCaseWithTransportAndScript, self).setUp()
         self.script_runner = ScriptRunner()
+        # FIXME: See shelf_ui.Shelver._char_based. This allow using shelve in
+        # scripts while providing a line-based input (better solution in
+        # progress). -- vila 2011-09-28
+        self.overrideEnv('INSIDE_EMACS', '1')
 
     def run_script(self, script, null_output_matches_anything=False):
         return self.script_runner.run_script(self, script,

=== modified file 'bzrlib/tests/stub_sftp.py'
--- bzrlib/tests/stub_sftp.py	2011-08-15 12:37:25 +0000
+++ bzrlib/tests/stub_sftp.py	2011-10-06 07:43:13 +0000
@@ -118,9 +118,9 @@
     else:
         def canonicalize(self, path):
             if os.path.isabs(path):
-                return os.path.normpath(path)
+                return osutils.normpath(path)
             else:
-                return os.path.normpath('/' + os.path.join(self.home, path))
+                return osutils.normpath('/' + os.path.join(self.home, path))
 
     def chattr(self, path, attr):
         try:

=== modified file 'bzrlib/tests/test__dirstate_helpers.py'
--- bzrlib/tests/test__dirstate_helpers.py	2011-06-14 01:26:41 +0000
+++ bzrlib/tests/test__dirstate_helpers.py	2011-09-28 16:53:43 +0000
@@ -25,6 +25,7 @@
     errors,
     osutils,
     tests,
+    _dirstate_helpers_py,
     )
 from bzrlib.tests import (
     test_dirstate,
@@ -60,6 +61,11 @@
     process_entry = compiled_dirstate_helpers_feature.module.ProcessEntryC
     pe_scenarios.append(('dirstate_Pyrex', {'_process_entry': process_entry}))
 
+helper_scenarios = [('dirstate_Python', {'helpers': _dirstate_helpers_py})]
+if compiled_dirstate_helpers_feature.available():
+    helper_scenarios.append(('dirstate_Pyrex',
+        {'helpers': compiled_dirstate_helpers_feature.module}))
+
 
 class TestBisectPathMixin(object):
     """Test that _bisect_path_*() returns the expected values.
@@ -1337,3 +1343,55 @@
         state._sha1_provider = UppercaseSHA1Provider()
         self.assertChangedFileIds(['file-id'], tree)
 
+
+class TestPackStat(tests.TestCase):
+    """Check packed representaton of stat values is robust on all inputs"""
+
+    scenarios = helper_scenarios
+
+    def pack(self, statlike_tuple):
+        return self.helpers.pack_stat(os.stat_result(statlike_tuple))
+
+    @staticmethod
+    def unpack_field(packed_string, stat_field):
+        return _dirstate_helpers_py._unpack_stat(packed_string)[stat_field]
+
+    def test_result(self):
+        self.assertEqual("AAAQAAAAABAAAAARAAAAAgAAAAEAAIHk",
+            self.pack((33252, 1, 2, 0, 0, 0, 4096, 15.5, 16.5, 17.5)))
+
+    def test_giant_inode(self):
+        packed = self.pack((33252, 0xF80000ABC, 0, 0, 0, 0, 0, 0, 0, 0))
+        self.assertEqual(0x80000ABC, self.unpack_field(packed, "st_ino"))
+
+    def test_giant_size(self):
+        packed = self.pack((33252, 0, 0, 0, 0, 0, (1 << 33) + 4096, 0, 0, 0))
+        self.assertEqual(4096, self.unpack_field(packed, "st_size"))
+
+    def test_fractional_mtime(self):
+        packed = self.pack((33252, 0, 0, 0, 0, 0, 0, 0, 16.9375, 0))
+        self.assertEqual(16, self.unpack_field(packed, "st_mtime"))
+
+    def test_ancient_mtime(self):
+        packed = self.pack((33252, 0, 0, 0, 0, 0, 0, 0, -11644473600.0, 0))
+        self.assertEqual(1240428288, self.unpack_field(packed, "st_mtime"))
+
+    def test_distant_mtime(self):
+        packed = self.pack((33252, 0, 0, 0, 0, 0, 0, 0, 64060588800.0, 0))
+        self.assertEqual(3931046656, self.unpack_field(packed, "st_mtime"))
+
+    def test_fractional_ctime(self):
+        packed = self.pack((33252, 0, 0, 0, 0, 0, 0, 0, 0, 17.5625))
+        self.assertEqual(17, self.unpack_field(packed, "st_ctime"))
+
+    def test_ancient_ctime(self):
+        packed = self.pack((33252, 0, 0, 0, 0, 0, 0, 0, 0, -11644473600.0))
+        self.assertEqual(1240428288, self.unpack_field(packed, "st_ctime"))
+
+    def test_distant_ctime(self):
+        packed = self.pack((33252, 0, 0, 0, 0, 0, 0, 0, 0, 64060588800.0))
+        self.assertEqual(3931046656, self.unpack_field(packed, "st_ctime"))
+
+    def test_negative_dev(self):
+        packed = self.pack((33252, 0, -0xFFFFFCDE, 0, 0, 0, 0, 0, 0, 0))
+        self.assertEqual(0x322, self.unpack_field(packed, "st_dev"))

=== modified file 'bzrlib/tests/test_branch.py'
--- bzrlib/tests/test_branch.py	2011-05-26 08:04:46 +0000
+++ bzrlib/tests/test_branch.py	2011-12-30 15:00:07 +0000
@@ -101,18 +101,20 @@
     # recursive section - that is, it appends the branch name.
 
 
-class SampleBranchFormat(_mod_branch.BranchFormat):
+class SampleBranchFormat(_mod_branch.BranchFormatMetadir):
     """A sample format
 
     this format is initializable, unsupported to aid in testing the
     open and open_downlevel routines.
     """
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See BzrBranchFormat.get_format_string()."""
         return "Sample branch format."
 
-    def initialize(self, a_bzrdir, name=None, repository=None):
+    def initialize(self, a_bzrdir, name=None, repository=None,
+                   append_revisions_only=None):
         """Format 4 branches cannot be created."""
         t = a_bzrdir.get_branch_transport(self, name=name)
         t.put_bytes('format', self.get_format_string())
@@ -121,7 +123,8 @@
     def is_supported(self):
         return False
 
-    def open(self, transport, name=None, _found=False, ignore_fallbacks=False):
+    def open(self, transport, name=None, _found=False, ignore_fallbacks=False,
+             possible_transports=None):
         return "opened branch."
 
 
@@ -130,19 +133,21 @@
 SampleSupportedBranchFormatString = "Sample supported branch format."
 
 # And the format class can then reference the constant to avoid skew.
-class SampleSupportedBranchFormat(_mod_branch.BranchFormat):
+class SampleSupportedBranchFormat(_mod_branch.BranchFormatMetadir):
     """A sample supported format."""
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See BzrBranchFormat.get_format_string()."""
         return SampleSupportedBranchFormatString
 
-    def initialize(self, a_bzrdir, name=None):
+    def initialize(self, a_bzrdir, name=None, append_revisions_only=None):
         t = a_bzrdir.get_branch_transport(self, name=name)
         t.put_bytes('format', self.get_format_string())
         return 'A branch'
 
-    def open(self, transport, name=None, _found=False, ignore_fallbacks=False):
+    def open(self, transport, name=None, _found=False, ignore_fallbacks=False,
+             possible_transports=None):
         return "opened supported branch."
 
 
@@ -160,7 +165,8 @@
     def initialize(self, a_bzrdir, name=None):
         raise NotImplementedError(self.initialize)
 
-    def open(self, transport, name=None, _found=False, ignore_fallbacks=False):
+    def open(self, transport, name=None, _found=False, ignore_fallbacks=False,
+             possible_transports=None):
         raise NotImplementedError(self.open)
 
 
@@ -176,7 +182,7 @@
             dir = format._matchingbzrdir.initialize(url)
             dir.create_repository()
             format.initialize(dir)
-            found_format = _mod_branch.BranchFormat.find_format(dir)
+            found_format = _mod_branch.BranchFormatMetadir.find_format(dir)
             self.assertIsInstance(found_format, format.__class__)
         check_format(_mod_branch.BzrBranchFormat5(), "bar")
 
@@ -191,19 +197,36 @@
         b = _mod_branch.Branch.open(self.get_url())
         self.assertEqual(b, "opened supported branch.")
 
+    def test_from_string(self):
+        self.assertIsInstance(
+            SampleBranchFormat.from_string("Sample branch format."),
+            SampleBranchFormat)
+        self.assertRaises(AssertionError,
+            SampleBranchFormat.from_string, "Different branch format.")
+
     def test_find_format_not_branch(self):
         dir = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
         self.assertRaises(errors.NotBranchError,
-                          _mod_branch.BranchFormat.find_format,
+                          _mod_branch.BranchFormatMetadir.find_format,
                           dir)
 
     def test_find_format_unknown_format(self):
         dir = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
         SampleBranchFormat().initialize(dir)
         self.assertRaises(errors.UnknownFormatError,
-                          _mod_branch.BranchFormat.find_format,
+                          _mod_branch.BranchFormatMetadir.find_format,
                           dir)
 
+    def test_find_format_with_features(self):
+        tree = self.make_branch_and_tree('.', format='2a')
+        tree.branch.update_feature_flags({"name": "optional"})
+        found_format = _mod_branch.BranchFormatMetadir.find_format(tree.bzrdir)
+        self.assertIsInstance(found_format, _mod_branch.BranchFormatMetadir)
+        self.assertEquals(found_format.features.get("name"), "optional")
+        tree.branch.update_feature_flags({"name": None})
+        branch = _mod_branch.Branch.open('.')
+        self.assertEquals(branch._format.features, {})
+
     def test_register_unregister_format(self):
         # Test the deprecated format registration functions
         format = SampleBranchFormat()
@@ -390,22 +413,6 @@
     def test_light_checkout_with_references(self):
         self.do_checkout_test(lightweight=True)
 
-    def test_set_push(self):
-        branch = self.make_branch('source', format=self.get_format_name())
-        branch.get_config().set_user_option('push_location', 'old',
-            store=config.STORE_LOCATION)
-        warnings = []
-        def warning(*args):
-            warnings.append(args[0] % args[1:])
-        _warning = trace.warning
-        trace.warning = warning
-        try:
-            branch.set_push_location('new')
-        finally:
-            trace.warning = _warning
-        self.assertEqual(warnings[0], 'Value "new" is masked by "old" from '
-                         'locations.conf')
-
 
 class TestBranch6(TestBranch67, tests.TestCaseWithTransport):
 
@@ -540,6 +547,16 @@
         self.assertEqual(('path3', 'location3'),
                          branch.get_reference_info('file-id'))
 
+    def _recordParentMapCalls(self, repo):
+        self._parent_map_calls = []
+        orig_get_parent_map = repo.revisions.get_parent_map
+        def get_parent_map(q):
+            q = list(q)
+            self._parent_map_calls.extend([e[0] for e in q])
+            return orig_get_parent_map(q)
+        repo.revisions.get_parent_map = get_parent_map
+
+
 class TestBranchReference(tests.TestCaseWithTransport):
     """Tests for the branch reference facility."""
 
@@ -645,18 +662,18 @@
     def setUp(self):
         super(TestBranchOptions, self).setUp()
         self.branch = self.make_branch('.')
-        self.config = self.branch.get_config()
+        self.config_stack = self.branch.get_config_stack()
 
     def check_append_revisions_only(self, expected_value, value=None):
         """Set append_revisions_only in config and check its interpretation."""
         if value is not None:
-            self.config.set_user_option('append_revisions_only', value)
+            self.config_stack.set('append_revisions_only', value)
         self.assertEqual(expected_value,
-                         self.branch._get_append_revisions_only())
+                         self.branch.get_append_revisions_only())
 
     def test_valid_append_revisions_only(self):
         self.assertEquals(None,
-                          self.config.get_user_option('append_revisions_only'))
+                          self.config_stack.get('append_revisions_only'))
         self.check_append_revisions_only(None)
         self.check_append_revisions_only(False, 'False')
         self.check_append_revisions_only(True, 'True')
@@ -674,7 +691,7 @@
         self.check_append_revisions_only(None, 'not-a-bool')
         self.assertLength(1, self.warnings)
         self.assertEqual(
-            'Value "not-a-bool" is not a boolean for "append_revisions_only"',
+            'Value "not-a-bool" is not valid for "append_revisions_only"',
             self.warnings[0])
 
 
@@ -702,6 +719,7 @@
         f = StringIO()
         r.report(f)
         self.assertEqual("Now on revision 20.\n", f.getvalue())
+        self.assertEqual("Now on revision 20.\n", f.getvalue())
 
     def test_report_unchanged(self):
         r = _mod_branch.PullResult()
@@ -709,5 +727,4 @@
         r.new_revid = "same-revid"
         f = StringIO()
         r.report(f)
-        self.assertEqual("No revisions to pull.\n", f.getvalue())
-
+        self.assertEqual("No revisions or tags to pull.\n", f.getvalue())

=== modified file 'bzrlib/tests/test_branchbuilder.py'
--- bzrlib/tests/test_branchbuilder.py	2011-07-15 09:22:16 +0000
+++ bzrlib/tests/test_branchbuilder.py	2011-10-18 15:28:32 +0000
@@ -86,6 +86,18 @@
             [rev_id1],
             branch.repository.get_revision(branch.last_revision()).parent_ids)
 
+    def test_build_commit_parent_ids(self):
+        """build_commit() takes a parent_ids argument."""
+        builder = BranchBuilder(self.get_transport().clone('foo'))
+        rev_id1 = builder.build_commit(
+            parent_ids=["ghost"], allow_leftmost_as_ghost=True)
+        rev_id2 = builder.build_commit(parent_ids=[])
+        branch = builder.get_branch()
+        self.assertEqual((1, rev_id2), branch.last_revision_info())
+        self.assertEqual(
+            ["ghost"],
+            branch.repository.get_revision(rev_id1).parent_ids)
+
 
 class TestBranchBuilderBuildSnapshot(tests.TestCaseWithMemoryTransport):
 

=== modified file 'bzrlib/tests/test_btree_index.py'
--- bzrlib/tests/test_btree_index.py	2011-08-19 22:34:02 +0000
+++ bzrlib/tests/test_btree_index.py	2011-11-16 20:59:10 +0000
@@ -831,6 +831,14 @@
             btree_index.BTreeGraphIndex(t1, 'index', 10) !=
             btree_index.BTreeGraphIndex(t1, 'index', 20))
 
+    def test_key_too_big(self):
+        # the size that matters here is the _compressed_ size of the key, so we can't
+        # do a simple character repeat.
+        bigKey = ''.join(map(repr, xrange(btree_index._PAGE_SIZE)))
+        self.assertRaises(errors.BadIndexKey,
+                          self.make_index,
+                          nodes=[((bigKey,), 'value', ())])
+        
     def test_iter_all_only_root_no_size(self):
         self.make_index(nodes=[(('key',), 'value', ())])
         t = transport.get_transport_from_url('trace+' + self.get_url(''))

=== modified file 'bzrlib/tests/test_bundle.py'
--- bzrlib/tests/test_bundle.py	2011-06-14 01:26:41 +0000
+++ bzrlib/tests/test_bundle.py	2011-12-21 14:25:26 +0000
@@ -16,9 +16,8 @@
 
 from cStringIO import StringIO
 import os
-import socket
+import SocketServer
 import sys
-import threading
 
 from bzrlib import (
     bzrdir,
@@ -28,7 +27,6 @@
     merge,
     osutils,
     revision as _mod_revision,
-    symbol_versioning,
     tests,
     treebuilder,
     )
@@ -42,13 +40,12 @@
 from bzrlib.bundle.serializer.v4 import BundleSerializerV4
 from bzrlib.repofmt import knitrepo
 from bzrlib.tests import (
+    features,
+    test_commit,
     test_read_bundle,
-    test_commit,
+    test_server,
     )
 from bzrlib.transform import TreeTransform
-from bzrlib.tests import (
-    features,
-    )
 
 
 def get_text(vf, key):
@@ -510,9 +507,7 @@
                 new.unlock()
                 old.unlock()
         if not _mod_revision.is_null(rev_id):
-            rh = self.b1.revision_history()
-            self.applyDeprecated(symbol_versioning.deprecated_in((2, 4, 0)),
-                tree.branch.set_revision_history, rh[:rh.index(rev_id)+1])
+            tree.branch.generate_revision_history(rev_id)
             tree.update()
             delta = tree.changes_from(self.b1.repository.revision_tree(rev_id))
             self.assertFalse(delta.has_changed(),
@@ -1423,8 +1418,8 @@
             from bzrlib.testament import Testament
             # monkey patch gpg signing mechanism
             bzrlib.gpg.GPGStrategy = bzrlib.gpg.LoopbackGPGStrategy
-            new_config = test_commit.MustSignConfig(branch)
-            commit.Commit(config=new_config).commit(message="base",
+            new_config = test_commit.MustSignConfig()
+            commit.Commit(config_stack=new_config).commit(message="base",
                                                     allow_pointless=True,
                                                     rev_id='B',
                                                     working_tree=tree_a)
@@ -1839,7 +1834,7 @@
         bundle, then the ConnectionReset error should be propagated.
         """
         # Instantiate a server that will provoke a ConnectionReset
-        sock_server = _DisconnectingTCPServer()
+        sock_server = DisconnectingServer()
         self.start_server(sock_server)
         # We don't really care what the url is since the server will close the
         # connection without interpreting it
@@ -1847,35 +1842,21 @@
         self.assertRaises(errors.ConnectionReset, read_mergeable_from_url, url)
 
 
-class _DisconnectingTCPServer(object):
-    """A TCP server that immediately closes any connection made to it."""
-
-    def start_server(self):
-        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        self.sock.bind(('127.0.0.1', 0))
-        self.sock.listen(1)
-        self.port = self.sock.getsockname()[1]
-        self.thread = threading.Thread(
-            name='%s (port %d)' % (self.__class__.__name__, self.port),
-            target=self.accept_and_close)
-        self.thread.start()
-
-    def accept_and_close(self):
-        conn, addr = self.sock.accept()
-        conn.shutdown(socket.SHUT_RDWR)
-        conn.close()
+class DisconnectingHandler(SocketServer.BaseRequestHandler):
+    """A request handler that immediately closes any connection made to it."""
+
+    def handle(self):
+        self.request.close()
+
+
+class DisconnectingServer(test_server.TestingTCPServerInAThread):
+
+    def __init__(self):
+        super(DisconnectingServer, self).__init__(
+            ('127.0.0.1', 0),
+            test_server.TestingTCPServer,
+            DisconnectingHandler)
 
     def get_url(self):
-        return 'bzr://127.0.0.1:%d/' % (self.port,)
-
-    def stop_server(self):
-        try:
-            # make sure the thread dies by connecting to the listening socket,
-            # just in case the test failed to do so.
-            conn = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-            conn.connect(self.sock.getsockname())
-            conn.close()
-        except socket.error:
-            pass
-        self.sock.close()
-        self.thread.join()
+        """Return the url of the server"""
+        return "bzr://%s:%d/" % self.server.server_address

=== modified file 'bzrlib/tests/test_bzrdir.py'
--- bzrlib/tests/test_bzrdir.py	2011-08-19 22:34:02 +0000
+++ bzrlib/tests/test_bzrdir.py	2012-01-02 10:24:02 +0000
@@ -253,6 +253,10 @@
     def open(self, transport, _found=None):
         return "opened branch."
 
+    @classmethod
+    def from_string(cls, format_string):
+        return cls()
+
 
 class BzrDirFormatTest1(bzrdir.BzrDirMetaFormat1):
 
@@ -513,6 +517,35 @@
         # Clone source into directory
         target = source_bzrdir.clone(self.get_url('parent/target'))
 
+    def test_format_initialize_on_transport_ex_stacked_on(self):
+        # trunk is a stackable format.  Note that its in the same server area
+        # which is what launchpad does, but not sufficient to exercise the
+        # general case.
+        trunk = self.make_branch('trunk', format='1.9')
+        t = self.get_transport('stacked')
+        old_fmt = bzrdir.format_registry.make_bzrdir('pack-0.92')
+        repo_name = old_fmt.repository_format.network_name()
+        # Should end up with a 1.9 format (stackable)
+        repo, control, require_stacking, repo_policy = \
+            old_fmt.initialize_on_transport_ex(t,
+                    repo_format_name=repo_name, stacked_on='../trunk',
+                    stack_on_pwd=t.base)
+        if repo is not None:
+            # Repositories are open write-locked
+            self.assertTrue(repo.is_write_locked())
+            self.addCleanup(repo.unlock)
+        else:
+            repo = control.open_repository()
+        self.assertIsInstance(control, bzrdir.BzrDir)
+        opened = bzrdir.BzrDir.open(t.base)
+        if not isinstance(old_fmt, remote.RemoteBzrDirFormat):
+            self.assertEqual(control._format.network_name(),
+                old_fmt.network_name())
+            self.assertEqual(control._format.network_name(),
+                opened._format.network_name())
+        self.assertEqual(control.__class__, opened.__class__)
+        self.assertLength(1, repo._fallback_repositories)
+
     def test_sprout_obeys_stacking_policy(self):
         child_branch, new_child_transport = self.prepare_default_stacking()
         new_child = child_branch.bzrdir.sprout(new_child_transport.base)
@@ -992,6 +1025,18 @@
         self.assertNotEqual(otherdir2, mydir)
         self.assertFalse(otherdir2 == mydir)
 
+    def test_with_features(self):
+        tree = self.make_branch_and_tree('tree', format='2a')
+        tree.bzrdir.update_feature_flags({"bar": "required"})
+        self.assertRaises(errors.MissingFeature, bzrdir.BzrDir.open, 'tree')
+        bzrdir.BzrDirMetaFormat1.register_feature('bar')
+        self.addCleanup(bzrdir.BzrDirMetaFormat1.unregister_feature, 'bar')
+        dir = bzrdir.BzrDir.open('tree')
+        self.assertEquals("required", dir._format.features.get("bar"))
+        tree.bzrdir.update_feature_flags({"bar": None, "nonexistant": None})
+        dir = bzrdir.BzrDir.open('tree')
+        self.assertEquals({}, dir._format.features)
+
     def test_needs_conversion_different_working_tree(self):
         # meta1dirs need an conversion if any element is not the default.
         new_format = bzrdir.format_registry.make_bzrdir('dirstate')
@@ -1215,7 +1260,7 @@
         self.test_branch = _TestBranch(self.transport)
         self.test_branch.repository = self.create_repository()
 
-    def open_branch(self, unsupported=False):
+    def open_branch(self, unsupported=False, possible_transports=None):
         return self.test_branch
 
     def cloning_metadir(self, require_stacking=False):
@@ -1253,6 +1298,9 @@
     def _get_config(self):
         return config.TransportConfig(self._transport, 'branch.conf')
 
+    def _get_config_store(self):
+        return config.BranchStore(self)
+
     def set_parent(self, parent):
         self._parent = parent
 
@@ -1323,7 +1371,7 @@
         self.assertEqual('fail', err._preformatted_string)
 
     def test_post_repo_init(self):
-        from bzrlib.bzrdir import RepoInitHookParams
+        from bzrlib.controldir import RepoInitHookParams
         calls = []
         bzrdir.BzrDir.hooks.install_named_hook('post_repo_init',
             calls.append, None)
@@ -1368,3 +1416,146 @@
         self._transport.put_bytes("a.~1~", "some content")
         self.assertEqual("a.~2~", self._bzrdir._available_backup_name("a"))
 
+
+class TestMeta1DirColoFormat(TestCaseWithTransport):
+    """Tests specific to the meta1 dir with colocated branches format."""
+
+    def test_supports_colo(self):
+        format = bzrdir.BzrDirMetaFormat1Colo()
+        self.assertTrue(format.colocated_branches)
+
+    def test_upgrade_from_2a(self):
+        tree = self.make_branch_and_tree('.', format='2a')
+        format = bzrdir.BzrDirMetaFormat1Colo()
+        self.assertTrue(tree.bzrdir.needs_format_conversion(format))
+        converter = tree.bzrdir._format.get_converter(format)
+        result = converter.convert(tree.bzrdir, None)
+        self.assertIsInstance(result._format, bzrdir.BzrDirMetaFormat1Colo)
+        self.assertFalse(result.needs_format_conversion(format))
+
+    def test_downgrade_to_2a(self):
+        tree = self.make_branch_and_tree('.', format='development-colo')
+        format = bzrdir.BzrDirMetaFormat1()
+        self.assertTrue(tree.bzrdir.needs_format_conversion(format))
+        converter = tree.bzrdir._format.get_converter(format)
+        result = converter.convert(tree.bzrdir, None)
+        self.assertIsInstance(result._format, bzrdir.BzrDirMetaFormat1)
+        self.assertFalse(result.needs_format_conversion(format))
+
+    def test_downgrade_to_2a_too_many_branches(self):
+        tree = self.make_branch_and_tree('.', format='development-colo')
+        tree.bzrdir.create_branch(name="another-colocated-branch")
+        converter = tree.bzrdir._format.get_converter(
+            bzrdir.BzrDirMetaFormat1())
+        self.assertRaises(errors.BzrError, converter.convert, tree.bzrdir,
+            None)
+
+
+class SampleBzrFormat(bzrdir.BzrFormat):
+
+    @classmethod
+    def get_format_string(cls):
+        return "First line\n"
+
+
+class TestBzrFormat(TestCase):
+    """Tests for BzrFormat."""
+
+    def test_as_string(self):
+        format = SampleBzrFormat()
+        format.features = {"foo": "required"}
+        self.assertEquals(format.as_string(),
+            "First line\n"
+            "required foo\n")
+        format.features["another"] = "optional"
+        self.assertEquals(format.as_string(),
+            "First line\n"
+            "required foo\n"
+            "optional another\n")
+
+    def test_network_name(self):
+        # The network string should include the feature info
+        format = SampleBzrFormat()
+        format.features = {"foo": "required"}
+        self.assertEquals(
+            "First line\nrequired foo\n",
+            format.network_name())
+
+    def test_from_string_no_features(self):
+        # No features
+        format = SampleBzrFormat.from_string(
+            "First line\n")
+        self.assertEquals({}, format.features)
+
+    def test_from_string_with_feature(self):
+        # Proper feature
+        format = SampleBzrFormat.from_string(
+            "First line\nrequired foo\n")
+        self.assertEquals("required", format.features.get("foo"))
+
+    def test_from_string_format_string_mismatch(self):
+        # The first line has to match the format string
+        self.assertRaises(AssertionError, SampleBzrFormat.from_string,
+            "Second line\nrequired foo\n")
+
+    def test_from_string_missing_space(self):
+        # At least one space is required in the feature lines
+        self.assertRaises(errors.ParseFormatError, SampleBzrFormat.from_string,
+            "First line\nfoo\n")
+
+    def test_from_string_with_spaces(self):
+        # Feature with spaces (in case we add stuff like this in the future)
+        format = SampleBzrFormat.from_string(
+            "First line\nrequired foo with spaces\n")
+        self.assertEquals("required", format.features.get("foo with spaces"))
+
+    def test_eq(self):
+        format1 = SampleBzrFormat()
+        format1.features = {"nested-trees": "optional"}
+        format2 = SampleBzrFormat()
+        format2.features = {"nested-trees": "optional"}
+        self.assertEquals(format1, format1)
+        self.assertEquals(format1, format2)
+        format3 = SampleBzrFormat()
+        self.assertNotEquals(format1, format3)
+
+    def test_check_support_status_optional(self):
+        # Optional, so silently ignore
+        format = SampleBzrFormat()
+        format.features = {"nested-trees": "optional"}
+        format.check_support_status(True)
+        self.addCleanup(SampleBzrFormat.unregister_feature, "nested-trees")
+        SampleBzrFormat.register_feature("nested-trees")
+        format.check_support_status(True)
+
+    def test_check_support_status_required(self):
+        # Optional, so trigger an exception
+        format = SampleBzrFormat()
+        format.features = {"nested-trees": "required"}
+        self.assertRaises(errors.MissingFeature, format.check_support_status,
+            True)
+        self.addCleanup(SampleBzrFormat.unregister_feature, "nested-trees")
+        SampleBzrFormat.register_feature("nested-trees")
+        format.check_support_status(True)
+
+    def test_check_support_status_unknown(self):
+        # treat unknown necessity as required
+        format = SampleBzrFormat()
+        format.features = {"nested-trees": "unknown"}
+        self.assertRaises(errors.MissingFeature, format.check_support_status,
+            True)
+        self.addCleanup(SampleBzrFormat.unregister_feature, "nested-trees")
+        SampleBzrFormat.register_feature("nested-trees")
+        format.check_support_status(True)
+
+    def test_feature_already_registered(self):
+        # a feature can only be registered once
+        self.addCleanup(SampleBzrFormat.unregister_feature, "nested-trees")
+        SampleBzrFormat.register_feature("nested-trees")
+        self.assertRaises(errors.FeatureAlreadyRegistered,
+            SampleBzrFormat.register_feature, "nested-trees")
+
+    def test_feature_with_space(self):
+        # spaces are not allowed in feature names
+        self.assertRaises(ValueError, SampleBzrFormat.register_feature,
+            "nested trees")

=== modified file 'bzrlib/tests/test_commands.py'
--- bzrlib/tests/test_commands.py	2011-07-15 14:27:20 +0000
+++ bzrlib/tests/test_commands.py	2011-12-09 14:24:21 +0000
@@ -24,7 +24,6 @@
     config,
     errors,
     option,
-    symbol_versioning,
     tests,
     )
 from bzrlib.commands import display_command
@@ -91,6 +90,23 @@
         self.assertContainsRe(c.get_help_text(), '--foo')
 
 
+class TestInsideCommand(tests.TestCaseInTempDir):
+
+    def test_command_see_config_overrides(self):
+        def run(cmd):
+            # We override the run() command method so we can observe the
+            # overrides from inside.
+            c = config.GlobalStack()
+            self.assertEquals('12', c.get('xx'))
+            self.assertEquals('foo', c.get('yy'))
+        self.overrideAttr(builtins.cmd_rocks, 'run', run)
+        self.run_bzr(['rocks', '-Oxx=12', '-Oyy=foo'])
+        c = config.GlobalStack()
+        # Ensure that we don't leak outside of the command
+        self.assertEquals(None, c.get('xx'))
+        self.assertEquals(None, c.get('yy'))
+
+
 class TestInvokedAs(tests.TestCase):
 
     def test_invoked_as(self):

=== modified file 'bzrlib/tests/test_commit.py'
--- bzrlib/tests/test_commit.py	2011-07-08 23:01:39 +0000
+++ bzrlib/tests/test_commit.py	2011-12-21 14:25:26 +0000
@@ -20,12 +20,12 @@
 import bzrlib
 from bzrlib import (
     bzrdir,
+    config,
     errors,
     )
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import BzrDirMetaFormat1
 from bzrlib.commit import Commit, NullCommitReporter
-from bzrlib.config import BranchConfig
 from bzrlib.errors import (
     PointlessCommit,
     BzrError,
@@ -44,19 +44,13 @@
 
 # TODO: Test commit with some added, and added-but-missing files
 
-class MustSignConfig(BranchConfig):
-
-    def signature_needed(self):
-        return True
-
-    def gpg_signing_command(self):
-        return ['cat', '-']
-
-
-class BranchWithHooks(BranchConfig):
-
-    def post_commit(self):
-        return "bzrlib.ahook bzrlib.ahook"
+class MustSignConfig(config.MemoryStack):
+
+    def __init__(self):
+        super(MustSignConfig, self).__init__('''
+gpg_signing_command=cat -
+create_signatures=always
+''')
 
 
 class CapturingReporter(NullCommitReporter):
@@ -90,25 +84,24 @@
         b = wt.branch
         file('hello', 'w').write('hello world')
         wt.add('hello')
-        wt.commit(message='add hello')
+        rev1 = wt.commit(message='add hello')
         file_id = wt.path2id('hello')
 
         file('hello', 'w').write('version 2')
-        wt.commit(message='commit 2')
+        rev2 = wt.commit(message='commit 2')
 
         eq = self.assertEquals
         eq(b.revno(), 2)
-        rh = b.revision_history()
-        rev = b.repository.get_revision(rh[0])
+        rev = b.repository.get_revision(rev1)
         eq(rev.message, 'add hello')
 
-        tree1 = b.repository.revision_tree(rh[0])
+        tree1 = b.repository.revision_tree(rev1)
         tree1.lock_read()
         text = tree1.get_file_text(file_id)
         tree1.unlock()
         self.assertEqual('hello world', text)
 
-        tree2 = b.repository.revision_tree(rh[1])
+        tree2 = b.repository.revision_tree(rev2)
         tree2.lock_read()
         text = tree2.get_file_text(file_id)
         tree2.unlock()
@@ -161,7 +154,11 @@
         wt.commit(message='add hello')
 
         os.remove('hello')
-        wt.commit('removed hello', rev_id='rev2')
+        reporter = CapturingReporter()
+        wt.commit('removed hello', rev_id='rev2', reporter=reporter)
+        self.assertEquals(
+            [('missing', u'hello'), ('deleted', u'hello')],
+            reporter.calls)
 
         tree = b.repository.revision_tree('rev2')
         self.assertFalse(tree.has_id('hello-id'))
@@ -361,8 +358,6 @@
             rev_ids.append(rev_id)
             wt.commit(message='rev %d' % (i+1),
                      rev_id=rev_id)
-        eq = self.assertEquals
-        eq(b.revision_history(), rev_ids)
         for i in range(4):
             self.assertThat(rev_ids[:i+1],
                 MatchesAncestry(b.repository, rev_ids[i]))
@@ -430,14 +425,17 @@
             from bzrlib.testament import Testament
             # monkey patch gpg signing mechanism
             bzrlib.gpg.GPGStrategy = bzrlib.gpg.LoopbackGPGStrategy
-            commit.Commit(config=MustSignConfig(branch)).commit(message="base",
-                                                      allow_pointless=True,
-                                                      rev_id='B',
-                                                      working_tree=wt)
+            conf = config.MemoryStack('''
+gpg_signing_command=cat -
+create_signatures=always
+''')
+            commit.Commit(config_stack=conf).commit(
+                message="base", allow_pointless=True, rev_id='B',
+                working_tree=wt)
             def sign(text):
                 return bzrlib.gpg.LoopbackGPGStrategy(None).sign(text)
             self.assertEqual(sign(Testament.from_revision(branch.repository,
-                             'B').as_short_text()),
+                                                          'B').as_short_text()),
                              branch.repository.get_signature_text('B'))
         finally:
             bzrlib.gpg.GPGStrategy = oldstrategy
@@ -453,15 +451,18 @@
         try:
             # monkey patch gpg signing mechanism
             bzrlib.gpg.GPGStrategy = bzrlib.gpg.DisabledGPGStrategy
-            config = MustSignConfig(branch)
+            conf = config.MemoryStack('''
+gpg_signing_command=cat -
+create_signatures=always
+''')
             self.assertRaises(SigningFailed,
-                              commit.Commit(config=config).commit,
+                              commit.Commit(config_stack=conf).commit,
                               message="base",
                               allow_pointless=True,
                               rev_id='B',
                               working_tree=wt)
             branch = Branch.open(self.get_url('.'))
-            self.assertEqual(branch.revision_history(), ['A'])
+            self.assertEqual(branch.last_revision(), 'A')
             self.assertFalse(branch.repository.has_revision('B'))
         finally:
             bzrlib.gpg.GPGStrategy = oldstrategy
@@ -475,11 +476,10 @@
             calls.append('called')
         bzrlib.ahook = called
         try:
-            config = BranchWithHooks(branch)
-            commit.Commit(config=config).commit(
-                            message = "base",
-                            allow_pointless=True,
-                            rev_id='A', working_tree = wt)
+            conf = config.MemoryStack('post_commit=bzrlib.ahook bzrlib.ahook')
+            commit.Commit(config_stack=conf).commit(
+                message = "base", allow_pointless=True, rev_id='A',
+                working_tree = wt)
             self.assertEqual(['called', 'called'], calls)
         finally:
             del bzrlib.ahook

=== modified file 'bzrlib/tests/test_commit_merge.py'
--- bzrlib/tests/test_commit_merge.py	2011-06-14 01:26:41 +0000
+++ bzrlib/tests/test_commit_merge.py	2011-10-05 18:11:19 +0000
@@ -17,7 +17,11 @@
 
 import os
 
-from bzrlib import check, osutils
+from bzrlib import (
+    check,
+    osutils,
+    revision as _mod_revision,
+    )
 from bzrlib.errors import PointlessCommit
 from bzrlib.tests import (
     TestCaseWithTransport,
@@ -55,8 +59,12 @@
         wty.commit('merge from x', rev_id='y@u-0-2', allow_pointless=False)
 
         self.assertEquals(by.revno(), 3)
-        self.assertEquals(list(by.revision_history()),
-                          [base_rev, 'y@u-0-1', 'y@u-0-2'])
+        graph = wty.branch.repository.get_graph()
+        self.addCleanup(wty.lock_read().unlock)
+        self.assertEquals(list(
+            graph.iter_lefthand_ancestry(by.last_revision(),
+                [_mod_revision.NULL_REVISION])),
+            ['y@u-0-2', 'y@u-0-1', base_rev])
         rev = by.repository.get_revision('y@u-0-2')
         self.assertEquals(rev.parent_ids,
                           ['y@u-0-1', 'x@u-0-1'])

=== modified file 'bzrlib/tests/test_config.py'
--- bzrlib/tests/test_config.py	2011-08-29 12:01:49 +0000
+++ bzrlib/tests/test_config.py	2012-01-04 11:09:24 +0000
@@ -33,18 +33,14 @@
     errors,
     osutils,
     mail_client,
-    mergetools,
     ui,
     urlutils,
-    registry,
     remote,
     tests,
     trace,
-    transport,
     )
 from bzrlib.symbol_versioning import (
     deprecated_in,
-    deprecated_method,
     )
 from bzrlib.transport import remote as transport_remote
 from bzrlib.tests import (
@@ -71,8 +67,8 @@
 
 # Register helpers to build stores
 config.test_store_builder_registry.register(
-    'configobj', lambda test: config.IniFileStore(test.get_transport(),
-                                                  'configobj.conf'))
+    'configobj', lambda test: config.TransportIniFileStore(
+        test.get_transport(), 'configobj.conf'))
 config.test_store_builder_registry.register(
     'bazaar', lambda test: config.GlobalStore())
 config.test_store_builder_registry.register(
@@ -148,16 +144,16 @@
 config.test_stack_builder_registry.register('branch', build_branch_stack)
 
 
-def build_remote_branch_stack(test):
+def build_branch_only_stack(test):
     # There is only one permutation (but we won't be able to handle more with
     # this design anyway)
     (transport_class,
      server_class) = transport_remote.get_test_permutations()[0]
     build_backing_branch(test, 'branch', transport_class, server_class)
     b = branch.Branch.open(test.get_url('branch'))
-    return config.RemoteBranchStack(b)
-config.test_stack_builder_registry.register('remote_branch',
-                                            build_remote_branch_stack)
+    return config.BranchOnlyStack(b)
+config.test_stack_builder_registry.register('branch_only',
+                                            build_branch_only_stack)
 
 def build_remote_control_stack(test):
     # There is only one permutation (but we won't be able to handle more with
@@ -187,6 +183,7 @@
 user_global_option=something
 bzr.mergetool.sometool=sometool {base} {this} {other} -o {result}
 bzr.mergetool.funkytool=funkytool "arg with spaces" {this_temp}
+bzr.mergetool.newtool='"newtool with spaces" {this_temp}'
 bzr.default_mergetool=sometool
 [ALIASES]
 h=help
@@ -331,13 +328,13 @@
 
 class FakeBranch(object):
 
-    def __init__(self, base=None, user_id=None):
+    def __init__(self, base=None):
         if base is None:
             self.base = "http://example.com/branches/demo"
         else:
             self.base = base
         self._transport = self.control_files = \
-            FakeControlFilesAndTransport(user_id=user_id)
+            FakeControlFilesAndTransport()
 
     def _get_config(self):
         return config.TransportConfig(self._transport, 'branch.conf')
@@ -351,16 +348,10 @@
 
 class FakeControlFilesAndTransport(object):
 
-    def __init__(self, user_id=None):
+    def __init__(self):
         self.files = {}
-        if user_id:
-            self.files['email'] = user_id
         self._transport = self
 
-    def get_utf8(self, filename):
-        # from LockableFiles
-        raise AssertionError("get_utf8 should no longer be used")
-
     def get(self, filename):
         # from Transport
         try:
@@ -503,27 +494,36 @@
 
     def test_signatures_default(self):
         my_config = config.Config()
-        self.assertFalse(my_config.signature_needed())
+        self.assertFalse(
+            self.applyDeprecated(deprecated_in((2, 5, 0)),
+                my_config.signature_needed))
         self.assertEqual(config.CHECK_IF_POSSIBLE,
-                         my_config.signature_checking())
+            self.applyDeprecated(deprecated_in((2, 5, 0)),
+                my_config.signature_checking))
         self.assertEqual(config.SIGN_WHEN_REQUIRED,
-                         my_config.signing_policy())
+                self.applyDeprecated(deprecated_in((2, 5, 0)),
+                    my_config.signing_policy))
 
     def test_signatures_template_method(self):
         my_config = InstrumentedConfig()
-        self.assertEqual(config.CHECK_NEVER, my_config.signature_checking())
+        self.assertEqual(config.CHECK_NEVER,
+            self.applyDeprecated(deprecated_in((2, 5, 0)),
+                my_config.signature_checking))
         self.assertEqual(['_get_signature_checking'], my_config._calls)
 
     def test_signatures_template_method_none(self):
         my_config = InstrumentedConfig()
         my_config._signatures = None
         self.assertEqual(config.CHECK_IF_POSSIBLE,
-                         my_config.signature_checking())
+                         self.applyDeprecated(deprecated_in((2, 5, 0)),
+                             my_config.signature_checking))
         self.assertEqual(['_get_signature_checking'], my_config._calls)
 
     def test_gpg_signing_command_default(self):
         my_config = config.Config()
-        self.assertEqual('gpg', my_config.gpg_signing_command())
+        self.assertEqual('gpg',
+            self.applyDeprecated(deprecated_in((2, 5, 0)),
+                my_config.gpg_signing_command))
 
     def test_get_user_option_default(self):
         my_config = config.Config()
@@ -531,15 +531,20 @@
 
     def test_post_commit_default(self):
         my_config = config.Config()
-        self.assertEqual(None, my_config.post_commit())
+        self.assertEqual(None, self.applyDeprecated(deprecated_in((2, 5, 0)),
+                                                    my_config.post_commit))
+
 
     def test_log_format_default(self):
         my_config = config.Config()
-        self.assertEqual('long', my_config.log_format())
+        self.assertEqual('long',
+                         self.applyDeprecated(deprecated_in((2, 5, 0)),
+                                              my_config.log_format))
 
     def test_acceptable_keys_default(self):
         my_config = config.Config()
-        self.assertEqual(None, my_config.acceptable_keys())
+        self.assertEqual(None, self.applyDeprecated(deprecated_in((2, 5, 0)),
+            my_config.acceptable_keys))
 
     def test_validate_signatures_in_log_default(self):
         my_config = config.Config()
@@ -839,6 +844,7 @@
         self.assertEquals(['{foo', '}', '{', 'bar}'],
                           conf.get_user_option('hidden', expand=True))
 
+
 class TestLocationConfigOptionExpansion(tests.TestCaseInTempDir):
 
     def get_config(self, location, string=None):
@@ -1065,7 +1071,10 @@
 si_g = 5g,
 si_gb = 5gB,
 """)
-        get_si = conf.get_user_option_as_int_from_SI
+        def get_si(s, default=None):
+            return self.applyDeprecated(
+                deprecated_in((2, 5, 0)),
+                conf.get_user_option_as_int_from_SI, s, default)
         self.assertEqual(100, get_si('plain'))
         self.assertEqual(5000, get_si('si_k'))
         self.assertEqual(5000, get_si('si_kb'))
@@ -1076,6 +1085,7 @@
         self.assertEqual(None, get_si('non-exist'))
         self.assertEqual(42, get_si('non-exist-with-default',  42))
 
+
 class TestSupressWarning(TestIniConfig):
 
     def make_warnings_config(self, s):
@@ -1237,26 +1247,36 @@
     def test_signatures_always(self):
         my_config = config.GlobalConfig.from_string(sample_always_signatures)
         self.assertEqual(config.CHECK_NEVER,
-                         my_config.signature_checking())
+                         self.applyDeprecated(deprecated_in((2, 5, 0)),
+                             my_config.signature_checking))
         self.assertEqual(config.SIGN_ALWAYS,
-                         my_config.signing_policy())
-        self.assertEqual(True, my_config.signature_needed())
+                         self.applyDeprecated(deprecated_in((2, 5, 0)),
+                             my_config.signing_policy))
+        self.assertEqual(True,
+            self.applyDeprecated(deprecated_in((2, 5, 0)),
+                my_config.signature_needed))
 
     def test_signatures_if_possible(self):
         my_config = config.GlobalConfig.from_string(sample_maybe_signatures)
         self.assertEqual(config.CHECK_NEVER,
-                         my_config.signature_checking())
+                         self.applyDeprecated(deprecated_in((2, 5, 0)),
+                             my_config.signature_checking))
         self.assertEqual(config.SIGN_WHEN_REQUIRED,
-                         my_config.signing_policy())
-        self.assertEqual(False, my_config.signature_needed())
+                         self.applyDeprecated(deprecated_in((2, 5, 0)),
+                             my_config.signing_policy))
+        self.assertEqual(False, self.applyDeprecated(deprecated_in((2, 5, 0)),
+            my_config.signature_needed))
 
     def test_signatures_ignore(self):
         my_config = config.GlobalConfig.from_string(sample_ignore_signatures)
         self.assertEqual(config.CHECK_ALWAYS,
-                         my_config.signature_checking())
+                         self.applyDeprecated(deprecated_in((2, 5, 0)),
+                             my_config.signature_checking))
         self.assertEqual(config.SIGN_NEVER,
-                         my_config.signing_policy())
-        self.assertEqual(False, my_config.signature_needed())
+                         self.applyDeprecated(deprecated_in((2, 5, 0)),
+                             my_config.signing_policy))
+        self.assertEqual(False, self.applyDeprecated(deprecated_in((2, 5, 0)),
+            my_config.signature_needed))
 
     def _get_sample_config(self):
         my_config = config.GlobalConfig.from_string(sample_config_text)
@@ -1264,12 +1284,17 @@
 
     def test_gpg_signing_command(self):
         my_config = self._get_sample_config()
-        self.assertEqual("gnome-gpg", my_config.gpg_signing_command())
-        self.assertEqual(False, my_config.signature_needed())
+        self.assertEqual("gnome-gpg",
+            self.applyDeprecated(
+                deprecated_in((2, 5, 0)), my_config.gpg_signing_command))
+        self.assertEqual(False, self.applyDeprecated(deprecated_in((2, 5, 0)),
+            my_config.signature_needed))
 
     def test_gpg_signing_key(self):
         my_config = self._get_sample_config()
-        self.assertEqual("DD4D5088", my_config.gpg_signing_key())
+        self.assertEqual("DD4D5088",
+            self.applyDeprecated(deprecated_in((2, 5, 0)),
+                my_config.gpg_signing_key))
 
     def _get_empty_config(self):
         my_config = config.GlobalConfig()
@@ -1277,7 +1302,9 @@
 
     def test_gpg_signing_command_unset(self):
         my_config = self._get_empty_config()
-        self.assertEqual("gpg", my_config.gpg_signing_command())
+        self.assertEqual("gpg",
+            self.applyDeprecated(
+                deprecated_in((2, 5, 0)), my_config.gpg_signing_command))
 
     def test_get_user_option_default(self):
         my_config = self._get_empty_config()
@@ -1290,15 +1317,21 @@
 
     def test_post_commit_default(self):
         my_config = self._get_sample_config()
-        self.assertEqual(None, my_config.post_commit())
+        self.assertEqual(None,
+                         self.applyDeprecated(deprecated_in((2, 5, 0)),
+                                              my_config.post_commit))
 
     def test_configured_logformat(self):
         my_config = self._get_sample_config()
-        self.assertEqual("short", my_config.log_format())
+        self.assertEqual("short",
+                         self.applyDeprecated(deprecated_in((2, 5, 0)),
+                                              my_config.log_format))
 
     def test_configured_acceptable_keys(self):
         my_config = self._get_sample_config()
-        self.assertEqual("amy", my_config.acceptable_keys())
+        self.assertEqual("amy",
+            self.applyDeprecated(deprecated_in((2, 5, 0)),
+                my_config.acceptable_keys))
 
     def test_configured_validate_signatures_in_log(self):
         my_config = self._get_sample_config()
@@ -1342,7 +1375,8 @@
         self.log(repr(tools))
         self.assertEqual(
             {u'funkytool' : u'funkytool "arg with spaces" {this_temp}',
-            u'sometool' : u'sometool {base} {this} {other} -o {result}'},
+            u'sometool' : u'sometool {base} {this} {other} -o {result}',
+            u'newtool' : u'"newtool with spaces" {this_temp}'},
             tools)
 
     def test_get_merge_tools_empty(self):
@@ -1539,40 +1573,52 @@
         self.get_branch_config('http://www.example.com',
                                  global_config=sample_ignore_signatures)
         self.assertEqual(config.CHECK_ALWAYS,
-                         self.my_config.signature_checking())
+                         self.applyDeprecated(deprecated_in((2, 5, 0)),
+                             self.my_config.signature_checking))
         self.assertEqual(config.SIGN_NEVER,
-                         self.my_config.signing_policy())
+                         self.applyDeprecated(deprecated_in((2, 5, 0)),
+                             self.my_config.signing_policy))
 
     def test_signatures_never(self):
         self.get_branch_config('/a/c')
         self.assertEqual(config.CHECK_NEVER,
-                         self.my_config.signature_checking())
+                         self.applyDeprecated(deprecated_in((2, 5, 0)),
+                             self.my_config.signature_checking))
 
     def test_signatures_when_available(self):
         self.get_branch_config('/a/', global_config=sample_ignore_signatures)
         self.assertEqual(config.CHECK_IF_POSSIBLE,
-                         self.my_config.signature_checking())
+                         self.applyDeprecated(deprecated_in((2, 5, 0)),
+                             self.my_config.signature_checking))
 
     def test_signatures_always(self):
         self.get_branch_config('/b')
         self.assertEqual(config.CHECK_ALWAYS,
-                         self.my_config.signature_checking())
+            self.applyDeprecated(deprecated_in((2, 5, 0)),
+                         self.my_config.signature_checking))
 
     def test_gpg_signing_command(self):
         self.get_branch_config('/b')
-        self.assertEqual("gnome-gpg", self.my_config.gpg_signing_command())
+        self.assertEqual("gnome-gpg",
+            self.applyDeprecated(deprecated_in((2, 5, 0)),
+                self.my_config.gpg_signing_command))
 
     def test_gpg_signing_command_missing(self):
         self.get_branch_config('/a')
-        self.assertEqual("false", self.my_config.gpg_signing_command())
+        self.assertEqual("false",
+            self.applyDeprecated(deprecated_in((2, 5, 0)),
+                self.my_config.gpg_signing_command))
 
     def test_gpg_signing_key(self):
         self.get_branch_config('/b')
-        self.assertEqual("DD4D5088", self.my_config.gpg_signing_key())
+        self.assertEqual("DD4D5088", self.applyDeprecated(deprecated_in((2, 5, 0)),
+            self.my_config.gpg_signing_key))
 
     def test_gpg_signing_key_default(self):
         self.get_branch_config('/a')
-        self.assertEqual("erik@bagfors.nu", self.my_config.gpg_signing_key())
+        self.assertEqual("erik@bagfors.nu",
+            self.applyDeprecated(deprecated_in((2, 5, 0)),
+                self.my_config.gpg_signing_key))
 
     def test_get_user_option_global(self):
         self.get_branch_config('/a')
@@ -1666,7 +1712,8 @@
     def test_post_commit_default(self):
         self.get_branch_config('/a/c')
         self.assertEqual('bzrlib.tests.test_config.post_commit',
-                         self.my_config.post_commit())
+                         self.applyDeprecated(deprecated_in((2, 5, 0)),
+                                              self.my_config.post_commit))
 
     def get_branch_config(self, location, global_config=None,
                           location_config=None):
@@ -1762,24 +1809,14 @@
         return my_config
 
     def test_user_id(self):
-        branch = FakeBranch(user_id='Robert Collins <robertc@example.net>')
+        branch = FakeBranch()
         my_config = config.BranchConfig(branch)
-        self.assertEqual("Robert Collins <robertc@example.net>",
-                         my_config.username())
+        self.assertIsNot(None, my_config.username())
         my_config.branch.control_files.files['email'] = "John"
         my_config.set_user_option('email',
                                   "Robert Collins <robertc@example.org>")
-        self.assertEqual("John", my_config.username())
-        del my_config.branch.control_files.files['email']
         self.assertEqual("Robert Collins <robertc@example.org>",
-                         my_config.username())
-
-    def test_not_set_in_branch(self):
-        my_config = self.get_branch_config(global_config=sample_config_text)
-        self.assertEqual(u"Erik B\u00e5gfors <erik@bagfors.nu>",
-                         my_config._get_user_id())
-        my_config.branch.control_files.files['email'] = "John"
-        self.assertEqual("John", my_config._get_user_id())
+                        my_config.username())
 
     def test_BZR_EMAIL_OVERRIDES(self):
         self.overrideEnv('BZR_EMAIL', "Robert Collins <robertc@example.org>")
@@ -1791,24 +1828,36 @@
     def test_signatures_forced(self):
         my_config = self.get_branch_config(
             global_config=sample_always_signatures)
-        self.assertEqual(config.CHECK_NEVER, my_config.signature_checking())
-        self.assertEqual(config.SIGN_ALWAYS, my_config.signing_policy())
-        self.assertTrue(my_config.signature_needed())
+        self.assertEqual(config.CHECK_NEVER,
+            self.applyDeprecated(deprecated_in((2, 5, 0)),
+                my_config.signature_checking))
+        self.assertEqual(config.SIGN_ALWAYS,
+            self.applyDeprecated(deprecated_in((2, 5, 0)),
+                my_config.signing_policy))
+        self.assertTrue(self.applyDeprecated(deprecated_in((2, 5, 0)),
+            my_config.signature_needed))
 
     def test_signatures_forced_branch(self):
         my_config = self.get_branch_config(
             global_config=sample_ignore_signatures,
             branch_data_config=sample_always_signatures)
-        self.assertEqual(config.CHECK_NEVER, my_config.signature_checking())
-        self.assertEqual(config.SIGN_ALWAYS, my_config.signing_policy())
-        self.assertTrue(my_config.signature_needed())
+        self.assertEqual(config.CHECK_NEVER,
+            self.applyDeprecated(deprecated_in((2, 5, 0)),
+                my_config.signature_checking))
+        self.assertEqual(config.SIGN_ALWAYS,
+            self.applyDeprecated(deprecated_in((2, 5, 0)),
+                my_config.signing_policy))
+        self.assertTrue(self.applyDeprecated(deprecated_in((2, 5, 0)),
+            my_config.signature_needed))
 
     def test_gpg_signing_command(self):
         my_config = self.get_branch_config(
             global_config=sample_config_text,
             # branch data cannot set gpg_signing_command
             branch_data_config="gpg_signing_command=pgp")
-        self.assertEqual('gnome-gpg', my_config.gpg_signing_command())
+        self.assertEqual('gnome-gpg',
+            self.applyDeprecated(deprecated_in((2, 5, 0)),
+                my_config.gpg_signing_command))
 
     def test_get_user_option_global(self):
         my_config = self.get_branch_config(global_config=sample_config_text)
@@ -1821,14 +1870,18 @@
                                       location_config=sample_branches_text)
         self.assertEqual(my_config.branch.base, '/a/c')
         self.assertEqual('bzrlib.tests.test_config.post_commit',
-                         my_config.post_commit())
+                         self.applyDeprecated(deprecated_in((2, 5, 0)),
+                                              my_config.post_commit))
         my_config.set_user_option('post_commit', 'rmtree_root')
         # post-commit is ignored when present in branch data
         self.assertEqual('bzrlib.tests.test_config.post_commit',
-                         my_config.post_commit())
+                         self.applyDeprecated(deprecated_in((2, 5, 0)),
+                                              my_config.post_commit))
         my_config.set_user_option('post_commit', 'rmtree_root',
                                   store=config.STORE_LOCATION)
-        self.assertEqual('rmtree_root', my_config.post_commit())
+        self.assertEqual('rmtree_root',
+                         self.applyDeprecated(deprecated_in((2, 5, 0)),
+                                              my_config.post_commit))
 
     def test_config_precedence(self):
         # FIXME: eager test, luckily no persitent config file makes it fail
@@ -1968,6 +2021,29 @@
         conf = config.TransportConfig(t, 'foo.conf')
         self.assertRaises(errors.ParseConfigError, conf._get_configobj)
 
+    def test_load_permission_denied(self):
+        """Ensure we get an empty config file if the file is inaccessible."""
+        warnings = []
+        def warning(*args):
+            warnings.append(args[0] % args[1:])
+        self.overrideAttr(trace, 'warning', warning)
+
+        class DenyingTransport(object):
+
+            def __init__(self, base):
+                self.base = base
+
+            def get_bytes(self, relpath):
+                raise errors.PermissionDenied(relpath, "")
+
+        cfg = config.TransportConfig(
+            DenyingTransport("nonexisting://"), 'control.conf')
+        self.assertIs(None, cfg.get_option('non-existant', 'SECTION'))
+        self.assertEquals(
+            warnings,
+            [u'Permission denied while trying to open configuration file '
+             u'nonexisting:///control.conf.'])
+
     def test_get_value(self):
         """Test that retreiving a value from a section is possible"""
         bzrdir_config = config.TransportConfig(self.get_transport('.'),
@@ -2253,6 +2329,12 @@
         opt = config.Option('foo', default='bar')
         self.assertEquals('bar', opt.get_default())
 
+    def test_callable_default_value(self):
+        def bar_as_unicode():
+            return u'bar'
+        opt = config.Option('foo', default=bar_as_unicode)
+        self.assertEquals('bar', opt.get_default())
+
     def test_default_value_from_env(self):
         opt = config.Option('foo', default='bar', default_from_env=['FOO'])
         self.overrideEnv('FOO', 'quux')
@@ -2274,18 +2356,24 @@
         self.assertRaises(AssertionError, config.Option, 'foo',
                           default=object())
 
+    def test_not_supported_callable_default_value_not_unicode(self):
+        def bar_not_unicode():
+            return 'bar'
+        opt = config.Option('foo', default=bar_not_unicode)
+        self.assertRaises(AssertionError, opt.get_default)
+
 
 class TestOptionConverterMixin(object):
 
     def assertConverted(self, expected, opt, value):
-        self.assertEquals(expected, opt.convert_from_unicode(value))
+        self.assertEquals(expected, opt.convert_from_unicode(None, value))
 
     def assertWarns(self, opt, value):
         warnings = []
         def warning(*args):
             warnings.append(args[0] % args[1:])
         self.overrideAttr(trace, 'warning', warning)
-        self.assertEquals(None, opt.convert_from_unicode(value))
+        self.assertEquals(None, opt.convert_from_unicode(None, value))
         self.assertLength(1, warnings)
         self.assertEquals(
             'Value "%s" is not valid for "%s"' % (value, opt.name),
@@ -2293,11 +2381,11 @@
 
     def assertErrors(self, opt, value):
         self.assertRaises(errors.ConfigOptionValueError,
-                          opt.convert_from_unicode, value)
+                          opt.convert_from_unicode, None, value)
 
     def assertConvertInvalid(self, opt, invalid_value):
         opt.invalid = None
-        self.assertEquals(None, opt.convert_from_unicode(invalid_value))
+        self.assertEquals(None, opt.convert_from_unicode(None, invalid_value))
         opt.invalid = 'warning'
         self.assertWarns(opt, invalid_value)
         opt.invalid = 'error'
@@ -2341,15 +2429,43 @@
         opt = self.get_option()
         self.assertConverted(16, opt, u'16')
 
-class TestOptionWithListConverter(tests.TestCase, TestOptionConverterMixin):
-
-    def get_option(self):
-        return config.Option('foo', help='A list.',
-                             from_unicode=config.list_from_store)
-
-    def test_convert_invalid(self):
+
+class TestOptionWithSIUnitConverter(tests.TestCase, TestOptionConverterMixin):
+
+    def get_option(self):
+        return config.Option('foo', help='An integer in SI units.',
+                             from_unicode=config.int_SI_from_store)
+
+    def test_convert_invalid(self):
+        opt = self.get_option()
+        self.assertConvertInvalid(opt, u'not-a-unit')
+        self.assertConvertInvalid(opt, u'Gb') # Forgot the int
+        self.assertConvertInvalid(opt, u'1b') # Forgot the unit
+        self.assertConvertInvalid(opt, u'1GG')
+        self.assertConvertInvalid(opt, u'1Mbb')
+        self.assertConvertInvalid(opt, u'1MM')
+
+    def test_convert_valid(self):
+        opt = self.get_option()
+        self.assertConverted(int(5e3), opt, u'5kb')
+        self.assertConverted(int(5e6), opt, u'5M')
+        self.assertConverted(int(5e6), opt, u'5MB')
+        self.assertConverted(int(5e9), opt, u'5g')
+        self.assertConverted(int(5e9), opt, u'5gB')
+        self.assertConverted(100, opt, u'100')
+
+
+class TestListOption(tests.TestCase, TestOptionConverterMixin):
+
+    def get_option(self):
+        return config.ListOption('foo', help='A list.')
+
+    def test_convert_invalid(self):
+        opt = self.get_option()
+        # We don't even try to convert a list into a list, we only expect
+        # strings
+        self.assertConvertInvalid(opt, [1])
         # No string is invalid as all forms can be converted to a list
-        pass
 
     def test_convert_valid(self):
         opt = self.get_option()
@@ -2362,10 +2478,6 @@
         self.assertConverted([u'42'], opt, u'42')
         # A single string
         self.assertConverted([u'bar'], opt, u'bar')
-        # A list remains a list (configObj will turn a string containing commas
-        # into a list, but that's not what we're testing here)
-        self.assertConverted([u'foo', u'1', u'True'],
-                             opt, [u'foo', u'1', u'True'])
 
 
 class TestOptionRegistry(tests.TestCase):
@@ -2447,12 +2559,14 @@
 
 class TestMutableSection(tests.TestCase):
 
-    # FIXME: Parametrize so that all sections (including os.environ and the
-    # ones produced by Stores) run these tests -- vila 2011-04-01
+    scenarios = [('mutable',
+                  {'get_section':
+                       lambda opts: config.MutableSection('myID', opts)},),
+        ]
 
     def test_set(self):
         a_dict = dict(foo='bar')
-        section = config.MutableSection('myID', a_dict)
+        section = self.get_section(a_dict)
         section.set('foo', 'new_value')
         self.assertEquals('new_value', section.get('foo'))
         # The change appears in the shared section
@@ -2463,7 +2577,7 @@
 
     def test_set_preserve_original_once(self):
         a_dict = dict(foo='bar')
-        section = config.MutableSection('myID', a_dict)
+        section = self.get_section(a_dict)
         section.set('foo', 'first_value')
         section.set('foo', 'second_value')
         # We keep track of the original value
@@ -2472,7 +2586,7 @@
 
     def test_remove(self):
         a_dict = dict(foo='bar')
-        section = config.MutableSection('myID', a_dict)
+        section = self.get_section(a_dict)
         section.remove('foo')
         # We get None for unknown options via the default value
         self.assertEquals(None, section.get('foo'))
@@ -2485,7 +2599,7 @@
 
     def test_remove_new_option(self):
         a_dict = dict()
-        section = config.MutableSection('myID', a_dict)
+        section = self.get_section(a_dict)
         section.set('foo', 'bar')
         section.remove('foo')
         self.assertFalse('foo' in section.options)
@@ -2495,9 +2609,70 @@
         self.assertEquals(config._NewlyCreatedOption, section.orig['foo'])
 
 
+class TestCommandLineStore(tests.TestCase):
+
+    def setUp(self):
+        super(TestCommandLineStore, self).setUp()
+        self.store = config.CommandLineStore()
+        self.overrideAttr(config, 'option_registry', config.OptionRegistry())
+
+    def get_section(self):
+        """Get the unique section for the command line overrides."""
+        sections = list(self.store.get_sections())
+        self.assertLength(1, sections)
+        store, section = sections[0]
+        self.assertEquals(self.store, store)
+        return section
+
+    def test_no_override(self):
+        self.store._from_cmdline([])
+        section = self.get_section()
+        self.assertLength(0, list(section.iter_option_names()))
+
+    def test_simple_override(self):
+        self.store._from_cmdline(['a=b'])
+        section = self.get_section()
+        self.assertEqual('b', section.get('a'))
+
+    def test_list_override(self):
+        opt = config.ListOption('l')
+        config.option_registry.register(opt)
+        self.store._from_cmdline(['l=1,2,3'])
+        val = self.get_section().get('l')
+        self.assertEqual('1,2,3', val)
+        # Reminder: lists should be registered as such explicitely, otherwise
+        # the conversion needs to be done afterwards.
+        self.assertEqual(['1', '2', '3'],
+                         opt.convert_from_unicode(self.store, val))
+
+    def test_multiple_overrides(self):
+        self.store._from_cmdline(['a=b', 'x=y'])
+        section = self.get_section()
+        self.assertEquals('b', section.get('a'))
+        self.assertEquals('y', section.get('x'))
+
+    def test_wrong_syntax(self):
+        self.assertRaises(errors.BzrCommandError,
+                          self.store._from_cmdline, ['a=b', 'c'])
+
+class TestStoreMinimalAPI(tests.TestCaseWithTransport):
+
+    scenarios = [(key, {'get_store': builder}) for key, builder
+                 in config.test_store_builder_registry.iteritems()] + [
+        ('cmdline', {'get_store': lambda test: config.CommandLineStore()})]
+
+    def test_id(self):
+        store = self.get_store(self)
+        if type(store) == config.TransportIniFileStore:
+            raise tests.TestNotApplicable(
+                "%s is not a concrete Store implementation"
+                " so it doesn't need an id" % (store.__class__.__name__,))
+        self.assertIsNot(None, store.id)
+
+
 class TestStore(tests.TestCaseWithTransport):
 
-    def assertSectionContent(self, expected, section):
+    def assertSectionContent(self, expected, (store, section)):
         """Assert that some options have the proper values in a section."""
         expected_name, expected_options = expected
         self.assertEquals(expected_name, section.id)
@@ -2511,9 +2686,6 @@
     scenarios = [(key, {'get_store': builder}) for key, builder
                  in config.test_store_builder_registry.iteritems()]
 
-    def setUp(self):
-        super(TestReadonlyStore, self).setUp()
-
     def test_building_delays_load(self):
         store = self.get_store(self)
         self.assertEquals(False, store.is_loaded())
@@ -2545,8 +2717,81 @@
         self.assertRaises(AssertionError, store._load_from_string, 'bar=baz')
 
 
+class TestStoreQuoting(TestStore):
+
+    scenarios = [(key, {'get_store': builder}) for key, builder
+                 in config.test_store_builder_registry.iteritems()]
+
+    def setUp(self):
+        super(TestStoreQuoting, self).setUp()
+        self.store = self.get_store(self)
+        # We need a loaded store but any content will do
+        self.store._load_from_string('')
+
+    def assertIdempotent(self, s):
+        """Assert that quoting an unquoted string is a no-op and vice-versa.
+
+        What matters here is that option values, as they appear in a store, can
+        be safely round-tripped out of the store and back.
+
+        :param s: A string, quoted if required.
+        """
+        self.assertEquals(s, self.store.quote(self.store.unquote(s)))
+        self.assertEquals(s, self.store.unquote(self.store.quote(s)))
+
+    def test_empty_string(self):
+        if isinstance(self.store, config.IniFileStore):
+            # configobj._quote doesn't handle empty values
+            self.assertRaises(AssertionError,
+                              self.assertIdempotent, '')
+        else:
+            self.assertIdempotent('')
+        # But quoted empty strings are ok
+        self.assertIdempotent('""')
+
+    def test_embedded_spaces(self):
+        self.assertIdempotent('" a b c "')
+
+    def test_embedded_commas(self):
+        self.assertIdempotent('" a , b c "')
+
+    def test_simple_comma(self):
+        if isinstance(self.store, config.IniFileStore):
+            # configobj requires that lists are special-cased
+           self.assertRaises(AssertionError,
+                             self.assertIdempotent, ',')
+        else:
+            self.assertIdempotent(',')
+        # When a single comma is required, quoting is also required
+        self.assertIdempotent('","')
+
+    def test_list(self):
+        if isinstance(self.store, config.IniFileStore):
+            # configobj requires that lists are special-cased
+            self.assertRaises(AssertionError,
+                              self.assertIdempotent, 'a,b')
+        else:
+            self.assertIdempotent('a,b')
+
+
+class TestDictFromStore(tests.TestCase):
+
+    def test_unquote_not_string(self):
+        conf = config.MemoryStack('x=2\n[a_section]\na=1\n')
+        value = conf.get('a_section')
+        # Urgh, despite 'conf' asking for the no-name section, we get the
+        # content of another section as a dict o_O
+        self.assertEquals({'a': '1'}, value)
+        unquoted = conf.store.unquote(value)
+        # Which cannot be unquoted but shouldn't crash either (the use cases
+        # are getting the value or displaying it. In the later case, '%s' will
+        # do).
+        self.assertEquals({'a': '1'}, unquoted)
+        self.assertEquals("{u'a': u'1'}", '%s' % (unquoted,))
+
+
 class TestIniFileStoreContent(tests.TestCaseWithTransport):
-    """Simulate loading a config store without content of various encodings.
+    """Simulate loading a config store with content of various encodings.
 
     All files produced by bzr are in utf8 content.
 
@@ -2565,7 +2810,7 @@
         utf8_content = unicode_content.encode('utf8')
         # Store the raw content in the config file
         t.put_bytes('foo.conf', utf8_content)
-        store = config.IniFileStore(t, 'foo.conf')
+        store = config.TransportIniFileStore(t, 'foo.conf')
         store.load()
         stack = config.Stack([store.get_sections], store)
         self.assertEquals(unicode_user, stack.get('user'))
@@ -2574,19 +2819,38 @@
         """Ensure we display a proper error on non-ascii, non utf-8 content."""
         t = self.get_transport()
         t.put_bytes('foo.conf', 'user=foo\n#%s\n' % (self.invalid_utf8_char,))
-        store = config.IniFileStore(t, 'foo.conf')
+        store = config.TransportIniFileStore(t, 'foo.conf')
         self.assertRaises(errors.ConfigContentError, store.load)
 
     def test_load_erroneous_content(self):
         """Ensure we display a proper error on content that can't be parsed."""
         t = self.get_transport()
         t.put_bytes('foo.conf', '[open_section\n')
-        store = config.IniFileStore(t, 'foo.conf')
+        store = config.TransportIniFileStore(t, 'foo.conf')
         self.assertRaises(errors.ParseConfigError, store.load)
 
+    def test_load_permission_denied(self):
+        """Ensure we get warned when trying to load an inaccessible file."""
+        warnings = []
+        def warning(*args):
+            warnings.append(args[0] % args[1:])
+        self.overrideAttr(trace, 'warning', warning)
+
+        t = self.get_transport()
+
+        def get_bytes(relpath):
+            raise errors.PermissionDenied(relpath, "")
+        t.get_bytes = get_bytes
+        store = config.TransportIniFileStore(t, 'foo.conf')
+        self.assertRaises(errors.PermissionDenied, store.load)
+        self.assertEquals(
+            warnings,
+            [u'Permission denied while trying to load configuration store %s.'
+             % store.external_url()])
+
 
 class TestIniConfigContent(tests.TestCaseWithTransport):
-    """Simulate loading a IniBasedConfig without content of various encodings.
+    """Simulate loading a IniBasedConfig with content of various encodings.
 
     All files produced by bzr are in utf8 content.
 
@@ -2737,14 +3001,34 @@
         self.assertEquals((store,), calls[0])
 
 
-class TestIniFileStore(TestStore):
+class TestQuotingIniFileStore(tests.TestCaseWithTransport):
+
+    def get_store(self):
+        return config.TransportIniFileStore(self.get_transport(), 'foo.conf')
+
+    def test_get_quoted_string(self):
+        store = self.get_store()
+        store._load_from_string('foo= " abc "')
+        stack = config.Stack([store.get_sections])
+        self.assertEquals(' abc ', stack.get('foo'))
+
+    def test_set_quoted_string(self):
+        store = self.get_store()
+        stack = config.Stack([store.get_sections], store)
+        stack.set('foo', ' a b c ')
+        store.save()
+        self.assertFileEqual('foo = " a b c "' + os.linesep, 'foo.conf')
+
+
+class TestTransportIniFileStore(TestStore):
 
     def test_loading_unknown_file_fails(self):
-        store = config.IniFileStore(self.get_transport(), 'I-do-not-exist')
+        store = config.TransportIniFileStore(self.get_transport(),
+            'I-do-not-exist')
         self.assertRaises(errors.NoSuchFile, store.load)
 
     def test_invalid_content(self):
-        store = config.IniFileStore(self.get_transport(), 'foo.conf', )
+        store = config.TransportIniFileStore(self.get_transport(), 'foo.conf')
         self.assertEquals(False, store.is_loaded())
         exc = self.assertRaises(
             errors.ParseConfigError, store._load_from_string,
@@ -2758,7 +3042,7 @@
         # option names share the same name space...)
         # FIXME: This should be fixed by forbidding dicts as values ?
         # -- vila 2011-04-05
-        store = config.IniFileStore(self.get_transport(), 'foo.conf', )
+        store = config.TransportIniFileStore(self.get_transport(), 'foo.conf')
         store._load_from_string('''
 foo=bar
 l=1,2
@@ -2774,8 +3058,10 @@
         sections = list(store.get_sections())
         self.assertLength(4, sections)
         # The default section has no name.
-        # List values are provided as lists
-        self.assertSectionContent((None, {'foo': 'bar', 'l': ['1', '2']}),
+        # List values are provided as strings and need to be explicitly
+        # converted by specifying from_unicode=list_from_store at option
+        # registration
+        self.assertSectionContent((None, {'foo': 'bar', 'l': u'1,2'}),
                                   sections[0])
         self.assertSectionContent(
             ('DEFAULT', {'foo_in_DEFAULT': 'foo_DEFAULT'}), sections[1])
@@ -2811,13 +3097,13 @@
 
     def setUp(self):
         super(TestConcurrentStoreUpdates, self).setUp()
-        self._content = 'one=1\ntwo=2\n'
         self.stack = self.get_stack(self)
         if not isinstance(self.stack, config._CompatibleStack):
             raise tests.TestNotApplicable(
                 '%s is not meant to be compatible with the old config design'
                 % (self.stack,))
-        self.stack.store._load_from_string(self._content)
+        self.stack.set('one', '1')
+        self.stack.set('two', '2')
         # Flush the store
         self.stack.store.save()
 
@@ -2927,24 +3213,27 @@
     # FIXME: It may be worth looking into removing the lock dir when it's not
     # needed anymore and look at possible fallouts for concurrent lockers. This
     # will matter if/when we use config files outside of bazaar directories
-    # (.bazaar or .bzr) -- vila 20110-04-11
+    # (.bazaar or .bzr) -- vila 20110-04-111
 
 
 class TestSectionMatcher(TestStore):
 
-    scenarios = [('location', {'matcher': config.LocationMatcher})]
+    scenarios = [('location', {'matcher': config.LocationMatcher}),
+                 ('id', {'matcher': config.NameMatcher}),]
 
-    def get_store(self, file_name):
-        return config.IniFileStore(self.get_readonly_transport(), file_name)
+    def setUp(self):
+        super(TestSectionMatcher, self).setUp()
+        # Any simple store is good enough
+        self.get_store = config.test_store_builder_registry.get('configobj')
 
     def test_no_matches_for_empty_stores(self):
-        store = self.get_store('foo.conf')
+        store = self.get_store(self)
         store._load_from_string('')
         matcher = self.matcher(store, '/bar')
         self.assertEquals([], list(matcher.get_sections()))
 
     def test_build_doesnt_load_store(self):
-        store = self.get_store('foo.conf')
+        store = self.get_store(self)
         matcher = self.matcher(store, '/bar')
         self.assertFalse(store.is_loaded())
 
@@ -2974,11 +3263,13 @@
 
 class TestLocationMatcher(TestStore):
 
-    def get_store(self, file_name):
-        return config.IniFileStore(self.get_readonly_transport(), file_name)
+    def setUp(self):
+        super(TestLocationMatcher, self).setUp()
+        # Any simple store is good enough
+        self.get_store = config.test_store_builder_registry.get('configobj')
 
     def test_unrelated_section_excluded(self):
-        store = self.get_store('foo.conf')
+        store = self.get_store(self)
         store._load_from_string('''
 [/foo]
 section=/foo
@@ -2993,9 +3284,9 @@
 ''')
         self.assertEquals(['/foo', '/foo/baz', '/foo/bar', '/foo/bar/baz',
                            '/quux/quux'],
-                          [section.id for section in store.get_sections()])
+                          [section.id for _, section in store.get_sections()])
         matcher = config.LocationMatcher(store, '/foo/bar/quux')
-        sections = list(matcher.get_sections())
+        sections = [section for s, section in matcher.get_sections()]
         self.assertEquals([3, 2],
                           [section.length for section in sections])
         self.assertEquals(['/foo/bar', '/foo'],
@@ -3004,7 +3295,7 @@
                           [section.extra_path for section in sections])
 
     def test_more_specific_sections_first(self):
-        store = self.get_store('foo.conf')
+        store = self.get_store(self)
         store._load_from_string('''
 [/foo]
 section=/foo
@@ -3012,9 +3303,9 @@
 section=/foo/bar
 ''')
         self.assertEquals(['/foo', '/foo/bar'],
-                          [section.id for section in store.get_sections()])
+                          [section.id for _, section in store.get_sections()])
         matcher = config.LocationMatcher(store, '/foo/bar/baz')
-        sections = list(matcher.get_sections())
+        sections = [section for s, section in matcher.get_sections()]
         self.assertEquals([3, 2],
                           [section.length for section in sections])
         self.assertEquals(['/foo/bar', '/foo'],
@@ -3025,7 +3316,7 @@
     def test_appendpath_in_no_name_section(self):
         # It's a bit weird to allow appendpath in a no-name section, but
         # someone may found a use for it
-        store = self.get_store('foo.conf')
+        store = self.get_store(self)
         store._load_from_string('''
 foo=bar
 foo:policy = appendpath
@@ -3033,10 +3324,10 @@
         matcher = config.LocationMatcher(store, 'dir/subdir')
         sections = list(matcher.get_sections())
         self.assertLength(1, sections)
-        self.assertEquals('bar/dir/subdir', sections[0].get('foo'))
+        self.assertEquals('bar/dir/subdir', sections[0][1].get('foo'))
 
     def test_file_urls_are_normalized(self):
-        store = self.get_store('foo.conf')
+        store = self.get_store(self)
         if sys.platform == 'win32':
             expected_url = 'file:///C:/dir/subdir'
             expected_location = 'C:/dir/subdir'
@@ -3047,61 +3338,125 @@
         self.assertEquals(expected_location, matcher.location)
 
 
-class TestStackGet(tests.TestCase):
-
-    # FIXME: This should be parametrized for all known Stack or dedicated
-    # paramerized tests created to avoid bloating -- vila 2011-03-31
-
-    def overrideOptionRegistry(self):
+class TestNameMatcher(TestStore):
+
+    def setUp(self):
+        super(TestNameMatcher, self).setUp()
+        self.matcher = config.NameMatcher
+        # Any simple store is good enough
+        self.get_store = config.test_store_builder_registry.get('configobj')
+
+    def get_matching_sections(self, name):
+        store = self.get_store(self)
+        store._load_from_string('''
+[foo]
+option=foo
+[foo/baz]
+option=foo/baz
+[bar]
+option=bar
+''')
+        matcher = self.matcher(store, name)
+        return list(matcher.get_sections())
+
+    def test_matching(self):
+        sections = self.get_matching_sections('foo')
+        self.assertLength(1, sections)
+        self.assertSectionContent(('foo', {'option': 'foo'}), sections[0])
+
+    def test_not_matching(self):
+        sections = self.get_matching_sections('baz')
+        self.assertLength(0, sections)
+
+
+class TestBaseStackGet(tests.TestCase):
+
+    def setUp(self):
+        super(TestBaseStackGet, self).setUp()
         self.overrideAttr(config, 'option_registry', config.OptionRegistry())
 
-    def test_single_config_get(self):
-        conf = dict(foo='bar')
-        conf_stack = config.Stack([conf])
-        self.assertEquals('bar', conf_stack.get('foo'))
+    def test_get_first_definition(self):
+        store1 = config.IniFileStore()
+        store1._load_from_string('foo=bar')
+        store2 = config.IniFileStore()
+        store2._load_from_string('foo=baz')
+        conf = config.Stack([store1.get_sections, store2.get_sections])
+        self.assertEquals('bar', conf.get('foo'))
 
     def test_get_with_registered_default_value(self):
-        conf_stack = config.Stack([dict()])
-        opt = config.Option('foo', default='bar')
-        self.overrideOptionRegistry()
-        config.option_registry.register('foo', opt)
+        config.option_registry.register(config.Option('foo', default='bar'))
+        conf_stack = config.Stack([])
         self.assertEquals('bar', conf_stack.get('foo'))
 
     def test_get_without_registered_default_value(self):
-        conf_stack = config.Stack([dict()])
-        opt = config.Option('foo')
-        self.overrideOptionRegistry()
-        config.option_registry.register('foo', opt)
+        config.option_registry.register(config.Option('foo'))
+        conf_stack = config.Stack([])
         self.assertEquals(None, conf_stack.get('foo'))
 
     def test_get_without_default_value_for_not_registered(self):
-        conf_stack = config.Stack([dict()])
-        opt = config.Option('foo')
-        self.overrideOptionRegistry()
+        conf_stack = config.Stack([])
         self.assertEquals(None, conf_stack.get('foo'))
 
-    def test_get_first_definition(self):
-        conf1 = dict(foo='bar')
-        conf2 = dict(foo='baz')
-        conf_stack = config.Stack([conf1, conf2])
-        self.assertEquals('bar', conf_stack.get('foo'))
-
-    def test_get_embedded_definition(self):
-        conf1 = dict(yy='12')
-        conf2 = config.Stack([dict(xx='42'), dict(foo='baz')])
-        conf_stack = config.Stack([conf1, conf2])
-        self.assertEquals('baz', conf_stack.get('foo'))
-
     def test_get_for_empty_section_callable(self):
         conf_stack = config.Stack([lambda : []])
         self.assertEquals(None, conf_stack.get('foo'))
 
     def test_get_for_broken_callable(self):
         # Trying to use and invalid callable raises an exception on first use
-        conf_stack = config.Stack([lambda : object()])
+        conf_stack = config.Stack([object])
         self.assertRaises(TypeError, conf_stack.get, 'foo')
 
 
+class TestStackWithSimpleStore(tests.TestCase):
+
+    def setUp(self):
+        super(TestStackWithSimpleStore, self).setUp()
+        self.overrideAttr(config, 'option_registry', config.OptionRegistry())
+        self.registry = config.option_registry
+
+    def get_conf(self, content=None):
+        return config.MemoryStack(content)
+
+    def test_override_value_from_env(self):
+        self.registry.register(
+            config.Option('foo', default='bar', override_from_env=['FOO']))
+        self.overrideEnv('FOO', 'quux')
+        # Env variable provides a default taking over the option one
+        conf = self.get_conf('foo=store')
+        self.assertEquals('quux', conf.get('foo'))
+
+    def test_first_override_value_from_env_wins(self):
+        self.registry.register(
+            config.Option('foo', default='bar',
+                          override_from_env=['NO_VALUE', 'FOO', 'BAZ']))
+        self.overrideEnv('FOO', 'foo')
+        self.overrideEnv('BAZ', 'baz')
+        # The first env var set wins
+        conf = self.get_conf('foo=store')
+        self.assertEquals('foo', conf.get('foo'))
+
+
+class TestMemoryStack(tests.TestCase):
+
+    def test_get(self):
+        conf = config.MemoryStack('foo=bar')
+        self.assertEquals('bar', conf.get('foo'))
+
+    def test_set(self):
+        conf = config.MemoryStack('foo=bar')
+        conf.set('foo', 'baz')
+        self.assertEquals('baz', conf.get('foo'))
+
+    def test_no_content(self):
+        conf = config.MemoryStack()
+        # No content means no loading
+        self.assertFalse(conf.store.is_loaded())
+        self.assertRaises(NotImplementedError, conf.get, 'foo')
+        # But a content can still be provided
+        conf.store._load_from_string('foo=bar')
+        self.assertEquals('bar', conf.get('foo'))
+
+
 class TestStackWithTransport(tests.TestCaseWithTransport):
 
     scenarios = [(key, {'get_stack': builder}) for key, builder
@@ -3125,7 +3480,7 @@
         self.assertEquals(None, self.conf.get('foo'))
 
     def test_get_hook(self):
-        self.conf.store._load_from_string('foo=bar')
+        self.conf.set('foo', 'bar')
         calls = []
         def hook(*args):
             calls.append(args)
@@ -3137,13 +3492,16 @@
         self.assertEquals((self.conf, 'foo', 'bar'), calls[0])
 
 
-class TestStackGetWithConverter(TestStackGet):
+class TestStackGetWithConverter(tests.TestCase):
 
     def setUp(self):
         super(TestStackGetWithConverter, self).setUp()
         self.overrideAttr(config, 'option_registry', config.OptionRegistry())
         self.registry = config.option_registry
 
+    def get_conf(self, content=None):
+        return config.MemoryStack(content)
+
     def register_bool_option(self, name, default=None, default_from_env=None):
         b = config.Option(name, help='A boolean.',
                           default=default, default_from_env=default_from_env,
@@ -3152,29 +3510,34 @@
 
     def test_get_default_bool_None(self):
         self.register_bool_option('foo')
-        self.assertEquals(None, self.conf.get('foo'))
+        conf = self.get_conf('')
+        self.assertEquals(None, conf.get('foo'))
 
     def test_get_default_bool_True(self):
         self.register_bool_option('foo', u'True')
-        self.assertEquals(True, self.conf.get('foo'))
+        conf = self.get_conf('')
+        self.assertEquals(True, conf.get('foo'))
 
     def test_get_default_bool_False(self):
         self.register_bool_option('foo', False)
-        self.assertEquals(False, self.conf.get('foo'))
+        conf = self.get_conf('')
+        self.assertEquals(False, conf.get('foo'))
 
     def test_get_default_bool_False_as_string(self):
         self.register_bool_option('foo', u'False')
-        self.assertEquals(False, self.conf.get('foo'))
+        conf = self.get_conf('')
+        self.assertEquals(False, conf.get('foo'))
 
     def test_get_default_bool_from_env_converted(self):
         self.register_bool_option('foo', u'True', default_from_env=['FOO'])
         self.overrideEnv('FOO', 'False')
-        self.assertEquals(False, self.conf.get('foo'))
+        conf = self.get_conf('')
+        self.assertEquals(False, conf.get('foo'))
 
     def test_get_default_bool_when_conversion_fails(self):
         self.register_bool_option('foo', default='True')
-        self.conf.store._load_from_string('foo=invalid boolean')
-        self.assertEquals(True, self.conf.get('foo'))
+        conf = self.get_conf('foo=invalid boolean')
+        self.assertEquals(True, conf.get('foo'))
 
     def register_integer_option(self, name,
                                 default=None, default_from_env=None):
@@ -3185,62 +3548,376 @@
 
     def test_get_default_integer_None(self):
         self.register_integer_option('foo')
-        self.assertEquals(None, self.conf.get('foo'))
+        conf = self.get_conf('')
+        self.assertEquals(None, conf.get('foo'))
 
     def test_get_default_integer(self):
         self.register_integer_option('foo', 42)
-        self.assertEquals(42, self.conf.get('foo'))
+        conf = self.get_conf('')
+        self.assertEquals(42, conf.get('foo'))
 
     def test_get_default_integer_as_string(self):
         self.register_integer_option('foo', u'42')
-        self.assertEquals(42, self.conf.get('foo'))
+        conf = self.get_conf('')
+        self.assertEquals(42, conf.get('foo'))
 
     def test_get_default_integer_from_env(self):
         self.register_integer_option('foo', default_from_env=['FOO'])
         self.overrideEnv('FOO', '18')
-        self.assertEquals(18, self.conf.get('foo'))
+        conf = self.get_conf('')
+        self.assertEquals(18, conf.get('foo'))
 
     def test_get_default_integer_when_conversion_fails(self):
         self.register_integer_option('foo', default='12')
-        self.conf.store._load_from_string('foo=invalid integer')
-        self.assertEquals(12, self.conf.get('foo'))
+        conf = self.get_conf('foo=invalid integer')
+        self.assertEquals(12, conf.get('foo'))
 
     def register_list_option(self, name, default=None, default_from_env=None):
-        l = config.Option(name, help='A list.',
-                          default=default, default_from_env=default_from_env,
-                          from_unicode=config.list_from_store)
+        l = config.ListOption(name, help='A list.', default=default,
+                              default_from_env=default_from_env)
         self.registry.register(l)
 
     def test_get_default_list_None(self):
         self.register_list_option('foo')
-        self.assertEquals(None, self.conf.get('foo'))
+        conf = self.get_conf('')
+        self.assertEquals(None, conf.get('foo'))
 
     def test_get_default_list_empty(self):
         self.register_list_option('foo', '')
-        self.assertEquals([], self.conf.get('foo'))
+        conf = self.get_conf('')
+        self.assertEquals([], conf.get('foo'))
 
     def test_get_default_list_from_env(self):
         self.register_list_option('foo', default_from_env=['FOO'])
         self.overrideEnv('FOO', '')
-        self.assertEquals([], self.conf.get('foo'))
+        conf = self.get_conf('')
+        self.assertEquals([], conf.get('foo'))
 
     def test_get_with_list_converter_no_item(self):
         self.register_list_option('foo', None)
-        self.conf.store._load_from_string('foo=,')
-        self.assertEquals([], self.conf.get('foo'))
+        conf = self.get_conf('foo=,')
+        self.assertEquals([], conf.get('foo'))
 
     def test_get_with_list_converter_many_items(self):
         self.register_list_option('foo', None)
-        self.conf.store._load_from_string('foo=m,o,r,e')
-        self.assertEquals(['m', 'o', 'r', 'e'], self.conf.get('foo'))
+        conf = self.get_conf('foo=m,o,r,e')
+        self.assertEquals(['m', 'o', 'r', 'e'], conf.get('foo'))
+
+    def test_get_with_list_converter_embedded_spaces_many_items(self):
+        self.register_list_option('foo', None)
+        conf = self.get_conf('foo=" bar", "baz "')
+        self.assertEquals([' bar', 'baz '], conf.get('foo'))
+
+    def test_get_with_list_converter_stripped_spaces_many_items(self):
+        self.register_list_option('foo', None)
+        conf = self.get_conf('foo= bar ,  baz ')
+        self.assertEquals(['bar', 'baz'], conf.get('foo'))
+
+
+class TestIterOptionRefs(tests.TestCase):
+    """iter_option_refs is a bit unusual, document some cases."""
+
+    def assertRefs(self, expected, string):
+        self.assertEquals(expected, list(config.iter_option_refs(string)))
+
+    def test_empty(self):
+        self.assertRefs([(False, '')], '')
+
+    def test_no_refs(self):
+        self.assertRefs([(False, 'foo bar')], 'foo bar')
+
+    def test_single_ref(self):
+        self.assertRefs([(False, ''), (True, '{foo}'), (False, '')], '{foo}')
+
+    def test_broken_ref(self):
+        self.assertRefs([(False, '{foo')], '{foo')
+
+    def test_embedded_ref(self):
+        self.assertRefs([(False, '{'), (True, '{foo}'), (False, '}')],
+                        '{{foo}}')
+
+    def test_two_refs(self):
+        self.assertRefs([(False, ''), (True, '{foo}'),
+                         (False, ''), (True, '{bar}'),
+                         (False, ''),],
+                        '{foo}{bar}')
+
+    def test_newline_in_refs_are_not_matched(self):
+        self.assertRefs([(False, '{\nxx}{xx\n}{{\n}}')], '{\nxx}{xx\n}{{\n}}')
+
+
+class TestStackExpandOptions(tests.TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestStackExpandOptions, self).setUp()
+        self.overrideAttr(config, 'option_registry', config.OptionRegistry())
+        self.registry = config.option_registry
+        self.conf = build_branch_stack(self)
+
+    def assertExpansion(self, expected, string, env=None):
+        self.assertEquals(expected, self.conf.expand_options(string, env))
+
+    def test_no_expansion(self):
+        self.assertExpansion('foo', 'foo')
+
+    def test_expand_default_value(self):
+        self.conf.store._load_from_string('bar=baz')
+        self.registry.register(config.Option('foo', default=u'{bar}'))
+        self.assertEquals('baz', self.conf.get('foo', expand=True))
+
+    def test_expand_default_from_env(self):
+        self.conf.store._load_from_string('bar=baz')
+        self.registry.register(config.Option('foo', default_from_env=['FOO']))
+        self.overrideEnv('FOO', '{bar}')
+        self.assertEquals('baz', self.conf.get('foo', expand=True))
+
+    def test_expand_default_on_failed_conversion(self):
+        self.conf.store._load_from_string('baz=bogus\nbar=42\nfoo={baz}')
+        self.registry.register(
+            config.Option('foo', default=u'{bar}',
+                          from_unicode=config.int_from_store))
+        self.assertEquals(42, self.conf.get('foo', expand=True))
+
+    def test_env_adding_options(self):
+        self.assertExpansion('bar', '{foo}', {'foo': 'bar'})
+
+    def test_env_overriding_options(self):
+        self.conf.store._load_from_string('foo=baz')
+        self.assertExpansion('bar', '{foo}', {'foo': 'bar'})
+
+    def test_simple_ref(self):
+        self.conf.store._load_from_string('foo=xxx')
+        self.assertExpansion('xxx', '{foo}')
+
+    def test_unknown_ref(self):
+        self.assertRaises(errors.ExpandingUnknownOption,
+                          self.conf.expand_options, '{foo}')
+
+    def test_indirect_ref(self):
+        self.conf.store._load_from_string('''
+foo=xxx
+bar={foo}
+''')
+        self.assertExpansion('xxx', '{bar}')
+
+    def test_embedded_ref(self):
+        self.conf.store._load_from_string('''
+foo=xxx
+bar=foo
+''')
+        self.assertExpansion('xxx', '{{bar}}')
+
+    def test_simple_loop(self):
+        self.conf.store._load_from_string('foo={foo}')
+        self.assertRaises(errors.OptionExpansionLoop,
+                          self.conf.expand_options, '{foo}')
+
+    def test_indirect_loop(self):
+        self.conf.store._load_from_string('''
+foo={bar}
+bar={baz}
+baz={foo}''')
+        e = self.assertRaises(errors.OptionExpansionLoop,
+                              self.conf.expand_options, '{foo}')
+        self.assertEquals('foo->bar->baz', e.refs)
+        self.assertEquals('{foo}', e.string)
+
+    def test_list(self):
+        self.conf.store._load_from_string('''
+foo=start
+bar=middle
+baz=end
+list={foo},{bar},{baz}
+''')
+        self.registry.register(
+            config.ListOption('list'))
+        self.assertEquals(['start', 'middle', 'end'],
+                           self.conf.get('list', expand=True))
+
+    def test_cascading_list(self):
+        self.conf.store._load_from_string('''
+foo=start,{bar}
+bar=middle,{baz}
+baz=end
+list={foo}
+''')
+        self.registry.register(
+            config.ListOption('list'))
+        self.assertEquals(['start', 'middle', 'end'],
+                           self.conf.get('list', expand=True))
+
+    def test_pathologically_hidden_list(self):
+        self.conf.store._load_from_string('''
+foo=bin
+bar=go
+start={foo
+middle=},{
+end=bar}
+hidden={start}{middle}{end}
+''')
+        # What matters is what the registration says, the conversion happens
+        # only after all expansions have been performed
+        self.registry.register(config.ListOption('hidden'))
+        self.assertEquals(['bin', 'go'],
+                          self.conf.get('hidden', expand=True))
+
+
+class TestStackCrossSectionsExpand(tests.TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestStackCrossSectionsExpand, self).setUp()
+
+    def get_config(self, location, string):
+        if string is None:
+            string = ''
+        # Since we don't save the config we won't strictly require to inherit
+        # from TestCaseInTempDir, but an error occurs so quickly...
+        c = config.LocationStack(location)
+        c.store._load_from_string(string)
+        return c
+
+    def test_dont_cross_unrelated_section(self):
+        c = self.get_config('/another/branch/path','''
+[/one/branch/path]
+foo = hello
+bar = {foo}/2
+
+[/another/branch/path]
+bar = {foo}/2
+''')
+        self.assertRaises(errors.ExpandingUnknownOption,
+                          c.get, 'bar', expand=True)
+
+    def test_cross_related_sections(self):
+        c = self.get_config('/project/branch/path','''
+[/project]
+foo = qu
+
+[/project/branch/path]
+bar = {foo}ux
+''')
+        self.assertEquals('quux', c.get('bar', expand=True))
+
+
+class TestStackCrossStoresExpand(tests.TestCaseWithTransport):
+
+    def test_cross_global_locations(self):
+        l_store = config.LocationStore()
+        l_store._load_from_string('''
+[/branch]
+lfoo = loc-foo
+lbar = {gbar}
+''')
+        l_store.save()
+        g_store = config.GlobalStore()
+        g_store._load_from_string('''
+[DEFAULT]
+gfoo = {lfoo}
+gbar = glob-bar
+''')
+        g_store.save()
+        stack = config.LocationStack('/branch')
+        self.assertEquals('glob-bar', stack.get('lbar', expand=True))
+        self.assertEquals('loc-foo', stack.get('gfoo', expand=True))
+
+
+class TestStackExpandSectionLocals(tests.TestCaseWithTransport):
+
+    def test_expand_locals_empty(self):
+        l_store = config.LocationStore()
+        l_store._load_from_string('''
+[/home/user/project]
+base = {basename}
+rel = {relpath}
+''')
+        l_store.save()
+        stack = config.LocationStack('/home/user/project/')
+        self.assertEquals('', stack.get('base', expand=True))
+        self.assertEquals('', stack.get('rel', expand=True))
+
+    def test_expand_basename_locally(self):
+        l_store = config.LocationStore()
+        l_store._load_from_string('''
+[/home/user/project]
+bfoo = {basename}
+''')
+        l_store.save()
+        stack = config.LocationStack('/home/user/project/branch')
+        self.assertEquals('branch', stack.get('bfoo', expand=True))
+
+    def test_expand_basename_locally_longer_path(self):
+        l_store = config.LocationStore()
+        l_store._load_from_string('''
+[/home/user]
+bfoo = {basename}
+''')
+        l_store.save()
+        stack = config.LocationStack('/home/user/project/dir/branch')
+        self.assertEquals('branch', stack.get('bfoo', expand=True))
+
+    def test_expand_relpath_locally(self):
+        l_store = config.LocationStore()
+        l_store._load_from_string('''
+[/home/user/project]
+lfoo = loc-foo/{relpath}
+''')
+        l_store.save()
+        stack = config.LocationStack('/home/user/project/branch')
+        self.assertEquals('loc-foo/branch', stack.get('lfoo', expand=True))
+
+    def test_expand_relpath_unknonw_in_global(self):
+        g_store = config.GlobalStore()
+        g_store._load_from_string('''
+[DEFAULT]
+gfoo = {relpath}
+''')
+        g_store.save()
+        stack = config.LocationStack('/home/user/project/branch')
+        self.assertRaises(errors.ExpandingUnknownOption,
+                          stack.get, 'gfoo', expand=True)
+
+    def test_expand_local_option_locally(self):
+        l_store = config.LocationStore()
+        l_store._load_from_string('''
+[/home/user/project]
+lfoo = loc-foo/{relpath}
+lbar = {gbar}
+''')
+        l_store.save()
+        g_store = config.GlobalStore()
+        g_store._load_from_string('''
+[DEFAULT]
+gfoo = {lfoo}
+gbar = glob-bar
+''')
+        g_store.save()
+        stack = config.LocationStack('/home/user/project/branch')
+        self.assertEquals('glob-bar', stack.get('lbar', expand=True))
+        self.assertEquals('loc-foo/branch', stack.get('gfoo', expand=True))
+
+    def test_locals_dont_leak(self):
+        """Make sure we chose the right local in presence of several sections.
+        """
+        l_store = config.LocationStore()
+        l_store._load_from_string('''
+[/home/user]
+lfoo = loc-foo/{relpath}
+[/home/user/project]
+lfoo = loc-foo/{relpath}
+''')
+        l_store.save()
+        stack = config.LocationStack('/home/user/project/branch')
+        self.assertEquals('loc-foo/branch', stack.get('lfoo', expand=True))
+        stack = config.LocationStack('/home/user/bar/baz')
+        self.assertEquals('loc-foo/bar/baz', stack.get('lfoo', expand=True))
+
 
 
 class TestStackSet(TestStackWithTransport):
 
     def test_simple_set(self):
         conf = self.get_stack(self)
-        conf.store._load_from_string('foo=bar')
-        self.assertEquals('bar', conf.get('foo'))
+        self.assertEquals(None, conf.get('foo'))
         conf.set('foo', 'baz')
         # Did we get it back ?
         self.assertEquals('baz', conf.get('foo'))
@@ -3266,7 +3943,7 @@
 
     def test_remove_existing(self):
         conf = self.get_stack(self)
-        conf.store._load_from_string('foo=bar')
+        conf.set('foo', 'bar')
         self.assertEquals('bar', conf.get('foo'))
         conf.remove('foo')
         # Did we get it back ?
@@ -3283,7 +3960,7 @@
         config.ConfigHooks.install_named_hook('remove', hook, None)
         self.assertLength(0, calls)
         conf = self.get_stack(self)
-        conf.store._load_from_string('foo=bar')
+        conf.set('foo', 'bar')
         conf.remove('foo')
         self.assertLength(1, calls)
         self.assertEquals((conf, 'foo'), calls[0])
@@ -3951,7 +4628,7 @@
 
     def test_auto_user_id(self):
         """Automatic inference of user name.
-        
+
         This is a bit hard to test in an isolated way, because it depends on
         system functions that go direct to /etc or perhaps somewhere else.
         But it's reasonable to say that on Unix, with an /etc/mailname, we ought
@@ -3967,3 +4644,26 @@
         else:
             self.assertEquals((None, None), (realname, address))
 
+
+class EmailOptionTests(tests.TestCase):
+
+    def test_default_email_uses_BZR_EMAIL(self):
+        conf = config.MemoryStack('email=jelmer@debian.org')
+        # BZR_EMAIL takes precedence over EMAIL
+        self.overrideEnv('BZR_EMAIL', 'jelmer@samba.org')
+        self.overrideEnv('EMAIL', 'jelmer@apache.org')
+        self.assertEquals('jelmer@samba.org', conf.get('email'))
+
+    def test_default_email_uses_EMAIL(self):
+        conf = config.MemoryStack('')
+        self.overrideEnv('BZR_EMAIL', None)
+        self.overrideEnv('EMAIL', 'jelmer@apache.org')
+        self.assertEquals('jelmer@apache.org', conf.get('email'))
+
+    def test_BZR_EMAIL_overrides(self):
+        conf = config.MemoryStack('email=jelmer@debian.org')
+        self.overrideEnv('BZR_EMAIL', 'jelmer@apache.org')
+        self.assertEquals('jelmer@apache.org', conf.get('email'))
+        self.overrideEnv('BZR_EMAIL', None)
+        self.overrideEnv('EMAIL', 'jelmer@samba.org')
+        self.assertEquals('jelmer@debian.org', conf.get('email'))

=== modified file 'bzrlib/tests/test_conflicts.py'
--- bzrlib/tests/test_conflicts.py	2011-08-04 00:17:53 +0000
+++ bzrlib/tests/test_conflicts.py	2011-10-27 15:38:14 +0000
@@ -677,6 +677,14 @@
              ('fileb_created',
               dict(actions='create_file_b', check='file_content_b',
                    path='file', file_id='file-b-id')),),
+            # File created with different file-ids but deleted on one side
+            (dict(_base_actions='create_file_a'),
+             ('filea_replaced',
+              dict(actions='replace_file_a_by_b', check='file_content_b',
+                   path='file', file_id='file-b-id')),
+             ('filea_modified',
+              dict(actions='modify_file_a', check='file_new_content',
+                   path='file', file_id='file-a-id')),),
             ])
 
     def do_nothing(self):
@@ -694,6 +702,16 @@
     def check_file_content_b(self):
         self.assertFileEqual('file b content\n', 'branch/file')
 
+    def do_replace_file_a_by_b(self):
+        return [('unversion', 'file-a-id'),
+                ('add', ('file', 'file-b-id', 'file', 'file b content\n'))]
+
+    def do_modify_file_a(self):
+        return [('modify', ('file-a-id', 'new content\n'))]
+
+    def check_file_new_content(self):
+        self.assertFileEqual('new content\n', 'branch/file')
+
     def _get_resolve_path_arg(self, wt, action):
         return self._this['path']
 
@@ -741,14 +759,14 @@
         self.run_script("""
 $ bzr rm -q dir  --force
 $ bzr resolve dir
-2>2 conflict(s) resolved, 0 remaining
+2>2 conflicts resolved, 0 remaining
 $ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
     def test_take_other(self):
         self.run_script("""
 $ bzr resolve dir
-2>2 conflict(s) resolved, 0 remaining
+2>2 conflicts resolved, 0 remaining
 $ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
@@ -782,7 +800,7 @@
     def test_keep_them_all(self):
         self.run_script("""
 $ bzr resolve dir
-2>2 conflict(s) resolved, 0 remaining
+2>2 conflicts resolved, 0 remaining
 $ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
@@ -791,7 +809,7 @@
 $ bzr mv -q dir/file2 file2
 $ bzr rm -q dir --force
 $ bzr resolve dir
-2>2 conflict(s) resolved, 0 remaining
+2>2 conflicts resolved, 0 remaining
 $ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
@@ -799,7 +817,7 @@
         self.run_script("""
 $ bzr rm -q dir --force
 $ bzr resolve dir
-2>2 conflict(s) resolved, 0 remaining
+2>2 conflicts resolved, 0 remaining
 $ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
@@ -846,7 +864,7 @@
     def test_keep_them_all(self):
         self.run_script("""
 $ bzr resolve dir
-2>2 conflict(s) resolved, 0 remaining
+2>2 conflicts resolved, 0 remaining
 $ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
@@ -855,7 +873,7 @@
 $ bzr mv -q dir/file2 file2
 $ bzr rm -q dir --force
 $ bzr resolve dir
-2>2 conflict(s) resolved, 0 remaining
+2>2 conflicts resolved, 0 remaining
 $ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
@@ -863,14 +881,14 @@
         self.run_script("""
 $ bzr rm -q dir --force
 $ bzr resolve dir
-2>2 conflict(s) resolved, 0 remaining
+2>2 conflicts resolved, 0 remaining
 $ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
     def test_resolve_taking_this(self):
         self.run_script("""
 $ bzr resolve --take-this dir
-2>2 conflict(s) resolved, 0 remaining
+2>2 conflicts resolved, 0 remaining
 $ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
@@ -879,7 +897,7 @@
 $ bzr resolve --take-other dir
 2>deleted dir/file2
 2>deleted dir
-2>2 conflict(s) resolved, 0 remaining
+2>2 conflicts resolved, 0 remaining
 $ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
@@ -1009,7 +1027,7 @@
 # aside ? -- vila 090916
 $ bzr add -q foo
 $ bzr resolve foo.new
-2>1 conflict(s) resolved, 0 remaining
+2>1 conflict resolved, 0 remaining
 $ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
@@ -1018,7 +1036,7 @@
 $ bzr rm -q foo --force
 $ bzr mv -q foo.new foo
 $ bzr resolve foo
-2>1 conflict(s) resolved, 0 remaining
+2>1 conflict resolved, 0 remaining
 $ bzr commit -q --strict -m 'No more conflicts nor unknown files'
 """)
 
@@ -1043,7 +1061,8 @@
         # This is nearly like TestResolveNonDirectoryParent but with branch and
         # trunk switched. As such it should certainly produce the same
         # conflict.
-        self.run_script("""
+        self.assertRaises(errors.MalformedTransform,
+                          self.run_script,"""
 $ bzr init trunk
 ...
 $ cd trunk
@@ -1080,7 +1099,7 @@
 # Create a debian branch based on trunk
 $ cd ..
 $ bzr branch trunk -r 1 debian
-2>Branched 1 revision(s).
+2>Branched 1 revision.
 $ cd debian
 $ mkdir dir
 $ bzr add

=== modified file 'bzrlib/tests/test_debug.py'
--- bzrlib/tests/test_debug.py	2011-08-12 16:25:56 +0000
+++ bzrlib/tests/test_debug.py	2011-11-16 17:19:13 +0000
@@ -34,7 +34,7 @@
 
     def assertDebugFlags(self, expected_flags, conf_bytes):
         conf = config.GlobalStack()
-        conf.store._load_from_string(conf_bytes)
+        conf.store._load_from_string('[DEFAULT]\n' + conf_bytes)
         conf.store.save()
         self.overrideAttr(debug, 'debug_flags', set())
         debug.set_debug_flags_from_config()

=== modified file 'bzrlib/tests/test_directory_service.py'
--- bzrlib/tests/test_directory_service.py	2010-06-20 11:18:38 +0000
+++ bzrlib/tests/test_directory_service.py	2011-11-29 01:35:03 +0000
@@ -21,7 +21,11 @@
     transport,
     urlutils,
     )
-from bzrlib.directory_service import DirectoryServiceRegistry, directories
+from bzrlib.directory_service import (
+    AliasDirectory,
+    DirectoryServiceRegistry,
+    directories,
+    )
 from bzrlib.tests import TestCase, TestCaseWithTransport
 
 
@@ -107,3 +111,10 @@
         e = self.assertRaises(errors.UnsetLocationAlias,
                               directories.dereference, ':parent')
         self.assertEqual('No parent location assigned.', str(e))
+
+    def test_register_location_alias(self):
+        branch = self.make_branch('.')
+        self.addCleanup(AliasDirectory.branch_aliases.remove, "booga")
+        AliasDirectory.branch_aliases.register("booga",
+            lambda b: "UHH?", help="Nobody knows")
+        self.assertEquals("UHH?", directories.dereference(":booga"))

=== modified file 'bzrlib/tests/test_email_message.py'
--- bzrlib/tests/test_email_message.py	2009-03-23 14:59:43 +0000
+++ bzrlib/tests/test_email_message.py	2011-12-16 19:18:39 +0000
@@ -150,7 +150,7 @@
 
     def test_send(self):
         class FakeConfig:
-            def get_user_option(self, option):
+            def get(self, option):
                 return None
 
         messages = []

=== modified file 'bzrlib/tests/test_errors.py'
--- bzrlib/tests/test_errors.py	2011-05-26 08:05:45 +0000
+++ bzrlib/tests/test_errors.py	2011-11-28 17:15:29 +0000
@@ -349,13 +349,6 @@
         self.assertEqual("The value 'foo' is not a valid value.",
             str(error))
 
-    def test_bzrnewerror_is_deprecated(self):
-        class DeprecatedError(errors.BzrNewError):
-            pass
-        self.callDeprecated(['BzrNewError was deprecated in bzr 0.13; '
-             'please convert DeprecatedError to use BzrError instead'],
-            DeprecatedError)
-
     def test_bzrerror_from_literal_string(self):
         # Some code constructs BzrError from a literal string, in which case
         # no further formatting is done.  (I'm not sure raising the base class
@@ -572,23 +565,6 @@
         err = errors.UnknownRules(['foo', 'bar'])
         self.assertEquals("Unknown rules detected: foo, bar.", str(err))
 
-    def test_hook_failed(self):
-        # Create an exc_info tuple by raising and catching an exception.
-        try:
-            1/0
-        except ZeroDivisionError:
-            err = errors.HookFailed('hook stage', 'hook name', sys.exc_info(),
-                warn=False)
-        # GZ 2010-11-08: Should not store exc_info in exception instances, but
-        #                HookFailed is deprecated anyway and could maybe go.
-        try:
-            self.assertStartsWith(
-                str(err), 'Hook \'hook name\' during hook stage failed:\n')
-            self.assertEndsWith(
-                str(err), 'integer division or modulo by zero')
-        finally:
-            del err
-
     def test_tip_change_rejected(self):
         err = errors.TipChangeRejected(u'Unicode message\N{INTERROBANG}')
         self.assertEquals(
@@ -719,6 +695,14 @@
             'Please use `bzr unbind` to fix.')
         self.assertEqualDiff(msg, str(error))
 
+    def test_retry_with_new_packs(self):
+        fake_exc_info = ('{exc type}', '{exc value}', '{exc traceback}')
+        error = errors.RetryWithNewPacks(
+            '{context}', reload_occurred=False, exc_info=fake_exc_info)
+        self.assertEqual(
+            'Pack files have changed, reload and retry. context: '
+            '{context} {exc value}', str(error))
+
 
 class PassThroughError(errors.BzrError):
 
@@ -751,12 +735,8 @@
 
     def test_missing_format_string(self):
         e = ErrorWithNoFormat(param='randomvalue')
-        s = self.callDeprecated(
-                ['ErrorWithNoFormat uses its docstring as a format, it should use _fmt instead'],
-                lambda x: str(x), e)
-        ## s = str(e)
-        self.assertEqual(s,
-                "This class has a docstring but no format string.")
+        self.assertStartsWith(str(e),
+            "Unprintable exception ErrorWithNoFormat")
 
     def test_mismatched_format_args(self):
         # Even though ErrorWithBadFormat's format string does not match the

=== added file 'bzrlib/tests/test_estimate_compressed_size.py'
--- bzrlib/tests/test_estimate_compressed_size.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_estimate_compressed_size.py	2011-09-01 16:54:46 +0000
@@ -0,0 +1,79 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for our estimation of compressed content."""
+
+import zlib
+import hashlib
+
+from bzrlib import (
+    estimate_compressed_size,
+    tests,
+    )
+
+
+class TestZLibEstimator(tests.TestCase):
+
+    def get_slightly_random_content(self, length, seed=''):
+        """We generate some hex-data that can be seeded.
+
+        The output should be deterministic, but the data stream is effectively
+        random.
+        """
+        h = hashlib.md5(seed)
+        hex_content = []
+        count = 0
+        while count < length:
+            b = h.hexdigest()
+            hex_content.append(b)
+            h.update(b)
+            count += len(b)
+        return ''.join(hex_content)[:length]
+
+    def test_adding_content(self):
+        ze = estimate_compressed_size.ZLibEstimator(32000)
+        raw_data = self.get_slightly_random_content(60000)
+        block_size = 1000
+        for start in xrange(0, len(raw_data), block_size):
+            ze.add_content(raw_data[start:start+block_size])
+            if ze.full():
+                break
+        # Practise showed that 'start' was 56000. However, zlib is a bit
+        # platform dependent, so give it +/- 5%.
+        self.assertTrue(54000 <= start <= 58000,
+            "Unexpected amount of raw data added: %d bytes" % (start,))
+        # The real compression should be 'close' to 32000, real measurement was
+        # 32401
+        raw_comp = zlib.compress(raw_data[:start])
+        self.assertTrue(31000 < len(raw_comp) < 33000,
+            "Unexpected compressed size: %d bytes" % (len(raw_comp),))
+
+    def test_adding_more_content(self):
+        ze = estimate_compressed_size.ZLibEstimator(64000)
+        raw_data = self.get_slightly_random_content(150000)
+        block_size = 1000
+        for start in xrange(0, len(raw_data), block_size):
+            ze.add_content(raw_data[start:start+block_size])
+            if ze.full():
+                break
+        # Practise showed that 'start' was 112000
+        self.assertTrue(110000 <= start <= 114000,
+            "Unexpected amount of raw data added: %d bytes" % (start,))
+        # The real compression should be 'close' to 32000, real measurement was
+        # 32401
+        raw_comp = zlib.compress(raw_data[:start])
+        self.assertTrue(63000 < len(raw_comp) < 65000,
+            "Unexpected compressed size: %d bytes" % (len(raw_comp),))

=== modified file 'bzrlib/tests/test_export_pot.py'
--- bzrlib/tests/test_export_pot.py	2011-06-28 10:28:29 +0000
+++ bzrlib/tests/test_export_pot.py	2011-12-08 18:58:27 +0000
@@ -20,6 +20,8 @@
 from bzrlib import (
     commands,
     export_pot,
+    option,
+    registry,
     tests,
     )
 
@@ -67,24 +69,298 @@
         self.assertEqual(export_pot._normalize(s), e)
 
 
+class TestParseSource(tests.TestCase):
+    """Check mappings to line numbers generated from python source"""
+
+    def test_classes(self):
+        src = '''
+class Ancient:
+    """Old style class"""
+
+class Modern(object):
+    """New style class"""
+'''
+        cls_lines, _ = export_pot._parse_source(src)
+        self.assertEqual(cls_lines,
+            {"Ancient": 2, "Modern": 5})
+
+    def test_classes_nested(self):
+        src = '''
+class Matroska(object):
+    class Smaller(object):
+        class Smallest(object):
+            pass
+'''
+        cls_lines, _ = export_pot._parse_source(src)
+        self.assertEqual(cls_lines,
+            {"Matroska": 2, "Smaller": 3, "Smallest":4})
+
+    def test_strings_docstrings(self):
+        src = '''\
+"""Module"""
+
+def function():
+    """Function"""
+
+class Class(object):
+    """Class"""
+
+    def method(self):
+        """Method"""
+'''
+        _, str_lines = export_pot._parse_source(src)
+        self.assertEqual(str_lines,
+            {"Module": 1, "Function": 4, "Class": 7, "Method": 10})
+
+    def test_strings_literals(self):
+        src = '''\
+s = "One"
+t = (2, "Two")
+f = dict(key="Three")
+'''
+        _, str_lines = export_pot._parse_source(src)
+        self.assertEqual(str_lines,
+            {"One": 1, "Two": 2, "Three": 3})
+
+    def test_strings_multiline(self):
+        src = '''\
+"""Start
+
+End
+"""
+t = (
+    "A"
+    "B"
+    "C"
+    )
+'''
+        _, str_lines = export_pot._parse_source(src)
+        self.assertEqual(str_lines,
+            {"Start\n\nEnd\n": 1, "ABC": 6})
+
+    def test_strings_multiline_escapes(self):
+        src = '''\
+s = "Escaped\\n"
+r = r"Raw\\n"
+t = (
+    "A\\n\\n"
+    "B\\n\\n"
+    "C\\n\\n"
+    )
+'''
+        _, str_lines = export_pot._parse_source(src)
+        self.expectFailure("Escaped newlines confuses the multiline handling",
+            self.assertNotEqual, str_lines,
+            {"Escaped\n": 0, "Raw\\n": 2, "A\n\nB\n\nC\n\n": -2})
+        self.assertEqual(str_lines,
+            {"Escaped\n": 1, "Raw\\n": 2, "A\n\nB\n\nC\n\n": 4})
+
+
+class TestModuleContext(tests.TestCase):
+    """Checks for source context tracking objects"""
+
+    def check_context(self, context, path, lineno):
+        self.assertEquals((context.path, context.lineno), (path, lineno))
+
+    def test___init__(self):
+        context = export_pot._ModuleContext("one.py")
+        self.check_context(context, "one.py", 1)
+        context = export_pot._ModuleContext("two.py", 5)
+        self.check_context(context, "two.py", 5)
+
+    def test_from_class(self):
+        """New context returned with lineno updated from class"""
+        path = "cls.py"
+        class A(object): pass
+        class B(object): pass
+        cls_lines = {"A": 5, "B": 7}
+        context = export_pot._ModuleContext(path, _source_info=(cls_lines, {}))
+        contextA = context.from_class(A)
+        self.check_context(contextA, path, 5)
+        contextB1 = context.from_class(B)
+        self.check_context(contextB1, path, 7)
+        contextB2 = contextA.from_class(B)
+        self.check_context(contextB2, path, 7)
+        self.check_context(context, path, 1)
+        self.assertEquals("", self.get_log())
+
+    def test_from_class_missing(self):
+        """When class has no lineno the old context details are returned"""
+        path = "cls_missing.py"
+        class A(object): pass
+        class M(object): pass
+        context = export_pot._ModuleContext(path, 3, ({"A": 15}, {}))
+        contextA = context.from_class(A)
+        contextM1 = context.from_class(M)
+        self.check_context(contextM1, path, 3)
+        contextM2 = contextA.from_class(M)
+        self.check_context(contextM2, path, 15)
+        self.assertContainsRe(self.get_log(), "Definition of <.*M'> not found")
+
+    def test_from_string(self):
+        """New context returned with lineno updated from string"""
+        path = "str.py"
+        str_lines = {"one": 14, "two": 42}
+        context = export_pot._ModuleContext(path, _source_info=({}, str_lines))
+        context1 = context.from_string("one")
+        self.check_context(context1, path, 14)
+        context2A = context.from_string("two")
+        self.check_context(context2A, path, 42)
+        context2B = context1.from_string("two")
+        self.check_context(context2B, path, 42)
+        self.check_context(context, path, 1)
+        self.assertEquals("", self.get_log())
+
+    def test_from_string_missing(self):
+        """When string has no lineno the old context details are returned"""
+        path = "str_missing.py"
+        context = export_pot._ModuleContext(path, 4, ({}, {"line\n": 21}))
+        context1 = context.from_string("line\n")
+        context2A = context.from_string("not there")
+        self.check_context(context2A, path, 4)
+        context2B = context1.from_string("not there")
+        self.check_context(context2B, path, 21)
+        self.assertContainsRe(self.get_log(), "String 'not there' not found")
+
+
+class TestWriteOption(tests.TestCase):
+    """Tests for writing texts extracted from options in pot format"""
+
+    def pot_from_option(self, opt, context=None, note="test"):
+        sio = StringIO()
+        exporter = export_pot._PotExporter(sio)
+        if context is None:
+            context = export_pot._ModuleContext("nowhere", 0)
+        export_pot._write_option(exporter, context, opt, note)
+        return sio.getvalue()
+
+    def test_option_without_help(self):
+        opt = option.Option("helpless")
+        self.assertEqual("", self.pot_from_option(opt))
+
+    def test_option_with_help(self):
+        opt = option.Option("helpful", help="Info.")
+        self.assertContainsString(self.pot_from_option(opt), "\n"
+            "# help of 'helpful' test\n"
+            "msgid \"Info.\"\n")
+
+    def test_option_hidden(self):
+        opt = option.Option("hidden", help="Unseen.", hidden=True)
+        self.assertEqual("", self.pot_from_option(opt))
+
+    def test_option_context_missing(self):
+        context = export_pot._ModuleContext("remote.py", 3)
+        opt = option.Option("metaphor", help="Not a literal in the source.")
+        self.assertContainsString(self.pot_from_option(opt, context),
+            "#: remote.py:3\n"
+            "# help of 'metaphor' test\n")
+
+    def test_option_context_string(self):
+        s = "Literally."
+        context = export_pot._ModuleContext("local.py", 3, ({}, {s: 17}))
+        opt = option.Option("example", help=s)
+        self.assertContainsString(self.pot_from_option(opt, context),
+            "#: local.py:17\n"
+            "# help of 'example' test\n")
+
+    def test_registry_option_title(self):
+        opt = option.RegistryOption.from_kwargs("group", help="Pick one.",
+            title="Choose!")
+        pot = self.pot_from_option(opt)
+        self.assertContainsString(pot, "\n"
+            "# title of 'group' test\n"
+            "msgid \"Choose!\"\n")
+        self.assertContainsString(pot, "\n"
+            "# help of 'group' test\n"
+            "msgid \"Pick one.\"\n")
+
+    def test_registry_option_title_context_missing(self):
+        context = export_pot._ModuleContext("theory.py", 3)
+        opt = option.RegistryOption.from_kwargs("abstract", title="Unfounded!")
+        self.assertContainsString(self.pot_from_option(opt, context),
+            "#: theory.py:3\n"
+            "# title of 'abstract' test\n")
+
+    def test_registry_option_title_context_string(self):
+        s = "Grounded!"
+        context = export_pot._ModuleContext("practice.py", 3, ({}, {s: 144}))
+        opt = option.RegistryOption.from_kwargs("concrete", title=s)
+        self.assertContainsString(self.pot_from_option(opt, context),
+            "#: practice.py:144\n"
+            "# title of 'concrete' test\n")
+
+    def test_registry_option_value_switches(self):
+        opt = option.RegistryOption.from_kwargs("switch", help="Flip one.",
+            value_switches=True, enum_switch=False,
+            red="Big.", green="Small.")
+        pot = self.pot_from_option(opt)
+        self.assertContainsString(pot, "\n"
+            "# help of 'switch' test\n"
+            "msgid \"Flip one.\"\n")
+        self.assertContainsString(pot, "\n"
+            "# help of 'switch=red' test\n"
+            "msgid \"Big.\"\n")
+        self.assertContainsString(pot, "\n"
+            "# help of 'switch=green' test\n"
+            "msgid \"Small.\"\n")
+
+    def test_registry_option_value_switches_hidden(self):
+        reg = registry.Registry()
+        class Hider(object):
+            hidden = True
+        reg.register("new", 1, "Current.")
+        reg.register("old", 0, "Legacy.", info=Hider())
+        opt = option.RegistryOption("protocol", "Talking.", reg,
+            value_switches=True, enum_switch=False)
+        pot = self.pot_from_option(opt)
+        self.assertContainsString(pot, "\n"
+            "# help of 'protocol' test\n"
+            "msgid \"Talking.\"\n")
+        self.assertContainsString(pot, "\n"
+            "# help of 'protocol=new' test\n"
+            "msgid \"Current.\"\n")
+        self.assertNotContainsString(pot, "'protocol=old'")
+
+
+class TestPotExporter(tests.TestCase):
+    """Test for logic specific to the _PotExporter class"""
+
+    # This test duplicates test_duplicates below
+    def test_duplicates(self):
+        exporter = export_pot._PotExporter(StringIO())
+        context = export_pot._ModuleContext("mod.py", 1)
+        exporter.poentry_in_context(context, "Common line.")
+        context.lineno = 3
+        exporter.poentry_in_context(context, "Common line.")
+        self.assertEqual(1, exporter.outf.getvalue().count("Common line."))
+    
+    def test_duplicates_included(self):
+        exporter = export_pot._PotExporter(StringIO(), True)
+        context = export_pot._ModuleContext("mod.py", 1)
+        exporter.poentry_in_context(context, "Common line.")
+        context.lineno = 3
+        exporter.poentry_in_context(context, "Common line.")
+        self.assertEqual(2, exporter.outf.getvalue().count("Common line."))
+
+
 class PoEntryTestCase(tests.TestCase):
 
     def setUp(self):
-        self.overrideAttr(export_pot, '_FOUND_MSGID', set())
-        self._outf = StringIO()
         super(PoEntryTestCase, self).setUp()
+        self.exporter = export_pot._PotExporter(StringIO())
 
     def check_output(self, expected):
         self.assertEqual(
-                self._outf.getvalue(),
+                self.exporter.outf.getvalue(),
                 textwrap.dedent(expected)
                 )
 
+
 class TestPoEntry(PoEntryTestCase):
 
     def test_simple(self):
-        export_pot._poentry(self._outf, 'dummy', 1, "spam")
-        export_pot._poentry(self._outf, 'dummy', 2, "ham", 'EGG')
+        self.exporter.poentry('dummy', 1, "spam")
+        self.exporter.poentry('dummy', 2, "ham", 'EGG')
         self.check_output('''\
                 #: dummy:1
                 msgid "spam"
@@ -98,9 +374,9 @@
                 ''')
 
     def test_duplicate(self):
-        export_pot._poentry(self._outf, 'dummy', 1, "spam")
+        self.exporter.poentry('dummy', 1, "spam")
         # This should be ignored.
-        export_pot._poentry(self._outf, 'dummy', 2, "spam", 'EGG')
+        self.exporter.poentry('dummy', 2, "spam", 'EGG')
 
         self.check_output('''\
                 #: dummy:1
@@ -112,8 +388,7 @@
 class TestPoentryPerPergraph(PoEntryTestCase):
 
     def test_single(self):
-        export_pot._poentry_per_paragraph(
-                self._outf,
+        self.exporter.poentry_per_paragraph(
                 'dummy',
                 10,
                 '''foo\nbar\nbaz\n'''
@@ -128,8 +403,7 @@
                 ''')
 
     def test_multi(self):
-        export_pot._poentry_per_paragraph(
-                self._outf,
+        self.exporter.poentry_per_paragraph(
                 'dummy',
                 10,
                 '''spam\nham\negg\n\nSPAM\nHAM\nEGG\n'''
@@ -169,8 +443,8 @@
             Blah Blah Blah
             """
 
-        export_pot._write_command_help(self._outf, cmd_Demo())
-        result = self._outf.getvalue()
+        export_pot._write_command_help(self.exporter, cmd_Demo())
+        result = self.exporter.outf.getvalue()
         # We don't care about filename and lineno here.
         result = re.sub(r'(?m)^#: [^\n]+\n', '', result)
 

=== modified file 'bzrlib/tests/test_features.py'
--- bzrlib/tests/test_features.py	2011-07-11 06:47:32 +0000
+++ bzrlib/tests/test_features.py	2011-11-29 16:27:56 +0000
@@ -14,8 +14,9 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-"""Tests for test feature dependencies.
-"""
+"""Tests for test feature dependencies."""
+
+import sys
 
 from bzrlib import (
     symbol_versioning,
@@ -88,6 +89,27 @@
             simple_thunk_feature.available)
         self.assertEqual(features.UnicodeFilenameFeature.available(), res)
 
+    def test_reports_correct_location(self):
+        a_feature = features._CompatabilityThunkFeature(
+            symbol_versioning.deprecated_in((2, 1, 0)),
+            'bzrlib.tests.test_features',
+            'a_feature',
+            'UnicodeFilenameFeature',
+            replacement_module='bzrlib.tests.features')
+        def test_caller(message, category=None, stacklevel=1):
+            # Find ourselves back from the right frame
+            caller = sys._getframe(stacklevel)
+            reported_file = caller.f_globals['__file__']
+            reported_lineno = caller.f_lineno
+            self.assertEquals(__file__, reported_file)
+            # The call we're tracking occurred the line after we grabbed the
+            # lineno.
+            self.assertEquals(self.lineno + 1, reported_lineno)
+        self.overrideAttr(symbol_versioning, 'warn', test_caller)
+        # Grab the current lineno
+        self.lineno = sys._getframe().f_lineno
+        self.requireFeature(a_feature)
+
 
 class TestModuleAvailableFeature(tests.TestCase):
 

=== modified file 'bzrlib/tests/test_fetch.py'
--- bzrlib/tests/test_fetch.py	2011-01-12 01:01:53 +0000
+++ bzrlib/tests/test_fetch.py	2011-10-14 12:49:33 +0000
@@ -18,6 +18,7 @@
     bzrdir,
     errors,
     osutils,
+    revision as _mod_revision,
     versionedfile,
     )
 from bzrlib.branch import Branch
@@ -35,24 +36,37 @@
 def has_revision(branch, revision_id):
     return branch.repository.has_revision(revision_id)
 
+
+def revision_history(branch):
+    branch.lock_read()
+    try:
+        graph = branch.repository.get_graph()
+        history = list(graph.iter_lefthand_ancestry(branch.last_revision(),
+            [_mod_revision.NULL_REVISION]))
+    finally:
+        branch.unlock()
+    history.reverse()
+    return history
+
+
 def fetch_steps(self, br_a, br_b, writable_a):
     """A foreign test method for testing fetch locally and remotely."""
 
     # TODO RBC 20060201 make this a repository test.
     repo_b = br_b.repository
-    self.assertFalse(repo_b.has_revision(br_a.revision_history()[3]))
-    self.assertTrue(repo_b.has_revision(br_a.revision_history()[2]))
-    self.assertEquals(len(br_b.revision_history()), 7)
-    br_b.fetch(br_a, br_a.revision_history()[2])
+    self.assertFalse(repo_b.has_revision(revision_history(br_a)[3]))
+    self.assertTrue(repo_b.has_revision(revision_history(br_a)[2]))
+    self.assertEquals(len(revision_history(br_b)), 7)
+    br_b.fetch(br_a, revision_history(br_a)[2])
     # branch.fetch is not supposed to alter the revision history
-    self.assertEquals(len(br_b.revision_history()), 7)
-    self.assertFalse(repo_b.has_revision(br_a.revision_history()[3]))
+    self.assertEquals(len(revision_history(br_b)), 7)
+    self.assertFalse(repo_b.has_revision(revision_history(br_a)[3]))
 
     # fetching the next revision up in sample data copies one revision
-    br_b.fetch(br_a, br_a.revision_history()[3])
-    self.assertTrue(repo_b.has_revision(br_a.revision_history()[3]))
-    self.assertFalse(has_revision(br_a, br_b.revision_history()[6]))
-    self.assertTrue(br_a.repository.has_revision(br_b.revision_history()[5]))
+    br_b.fetch(br_a, revision_history(br_a)[3])
+    self.assertTrue(repo_b.has_revision(revision_history(br_a)[3]))
+    self.assertFalse(has_revision(br_a, revision_history(br_b)[6]))
+    self.assertTrue(br_a.repository.has_revision(revision_history(br_b)[5]))
 
     # When a non-branch ancestor is missing, it should be unlisted...
     # as its not reference from the inventory weave.
@@ -60,20 +74,20 @@
     br_b4.fetch(br_b)
 
     writable_a.fetch(br_b)
-    self.assertTrue(has_revision(br_a, br_b.revision_history()[3]))
-    self.assertTrue(has_revision(br_a, br_b.revision_history()[4]))
+    self.assertTrue(has_revision(br_a, revision_history(br_b)[3]))
+    self.assertTrue(has_revision(br_a, revision_history(br_b)[4]))
 
     br_b2 = self.make_branch('br_b2')
     br_b2.fetch(br_b)
-    self.assertTrue(has_revision(br_b2, br_b.revision_history()[4]))
-    self.assertTrue(has_revision(br_b2, br_a.revision_history()[2]))
-    self.assertFalse(has_revision(br_b2, br_a.revision_history()[3]))
+    self.assertTrue(has_revision(br_b2, revision_history(br_b)[4]))
+    self.assertTrue(has_revision(br_b2, revision_history(br_a)[2]))
+    self.assertFalse(has_revision(br_b2, revision_history(br_a)[3]))
 
     br_a2 = self.make_branch('br_a2')
     br_a2.fetch(br_a)
-    self.assertTrue(has_revision(br_a2, br_b.revision_history()[4]))
-    self.assertTrue(has_revision(br_a2, br_a.revision_history()[3]))
-    self.assertTrue(has_revision(br_a2, br_a.revision_history()[2]))
+    self.assertTrue(has_revision(br_a2, revision_history(br_b)[4]))
+    self.assertTrue(has_revision(br_a2, revision_history(br_a)[3]))
+    self.assertTrue(has_revision(br_a2, revision_history(br_a)[2]))
 
     br_a3 = self.make_branch('br_a3')
     # pulling a branch with no revisions grabs nothing, regardless of
@@ -81,10 +95,10 @@
     br_a3.fetch(br_a2)
     for revno in range(4):
         self.assertFalse(
-            br_a3.repository.has_revision(br_a.revision_history()[revno]))
-    br_a3.fetch(br_a2, br_a.revision_history()[2])
+            br_a3.repository.has_revision(revision_history(br_a)[revno]))
+    br_a3.fetch(br_a2, revision_history(br_a)[2])
     # pull the 3 revisions introduced by a@u-0-3
-    br_a3.fetch(br_a2, br_a.revision_history()[3])
+    br_a3.fetch(br_a2, revision_history(br_a)[3])
     # NoSuchRevision should be raised if the branch is missing the revision
     # that was requested.
     self.assertRaises(errors.NoSuchRevision, br_a3.fetch, br_a2, 'pizza')

=== modified file 'bzrlib/tests/test_foreign.py'
--- bzrlib/tests/test_foreign.py	2011-06-16 12:50:32 +0000
+++ bzrlib/tests/test_foreign.py	2011-12-19 19:15:58 +0000
@@ -91,6 +91,10 @@
 class DummyForeignVcsBranch(branch.BzrBranch6,foreign.ForeignBranch):
     """A Dummy VCS Branch."""
 
+    @property
+    def user_transport(self):
+        return self.bzrdir.user_transport
+
     def __init__(self, _format, _control_files, a_bzrdir, *args, **kwargs):
         self._format = _format
         self._base = a_bzrdir.transport.base
@@ -101,7 +105,7 @@
         branch.BzrBranch6.__init__(self, _format, _control_files, a_bzrdir,
             *args, **kwargs)
 
-    def _get_checkout_format(self):
+    def _get_checkout_format(self, lightweight=False):
         """Return the most suitable metadir for a checkout of this branch.
         Weaves are used if this branch's repository uses weaves.
         """
@@ -141,13 +145,21 @@
     repository_class = DummyForeignVcsRepository
     _commit_builder_class = DummyForeignCommitBuilder
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         return "Dummy Foreign Vcs Repository"
 
     def get_format_description(self):
         return "Dummy Foreign Vcs Repository"
 
 
+def branch_history(graph, revid):
+    ret = list(graph.iter_lefthand_ancestry(revid,
+        (revision.NULL_REVISION,)))
+    ret.reverse()
+    return ret
+
+
 class InterToDummyVcsBranch(branch.GenericInterBranch):
 
     @staticmethod
@@ -165,12 +177,11 @@
         try:
             graph = self.source.repository.get_graph()
             # This just handles simple cases, but that's good enough for tests
-            my_history = self.target.revision_history()
+            my_history = branch_history(self.target.repository.get_graph(),
+                result.old_revid)
             if stop_revision is None:
                 stop_revision = self.source.last_revision()
-            their_history = list(graph.iter_lefthand_ancestry(stop_revision,
-                (revision.NULL_REVISION,)))
-            their_history.reverse()
+            their_history = branch_history(graph, stop_revision)
             if their_history[:min(len(my_history), len(their_history))] != my_history:
                 raise errors.DivergedBranches(self.target, self.source)
             todo = their_history[len(my_history):]
@@ -190,7 +201,7 @@
                 else:
                     parent_revids = [parent_revid]
                 builder = self.target.get_commit_builder(parent_revids, 
-                        self.target.get_config(), rev.timestamp,
+                        self.target.get_config_stack(), rev.timestamp,
                         rev.timezone, rev.committer, rev.properties,
                         new_revid)
                 try:
@@ -219,12 +230,13 @@
 
 class DummyForeignVcsBranchFormat(branch.BzrBranchFormat6):
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         return "Branch for Testing"
 
-    def __init__(self):
-        super(DummyForeignVcsBranchFormat, self).__init__()
-        self._matchingbzrdir = DummyForeignVcsDirFormat()
+    @property
+    def _matchingbzrdir(self):
+        return DummyForeignVcsDirFormat()
 
     def open(self, a_bzrdir, name=None, _found=False, ignore_fallbacks=False,
             found_repository=None):
@@ -303,7 +315,8 @@
         self.root_transport.put_bytes(".bzr", "foo")
         return super(DummyForeignVcsDir, self).create_workingtree()
 
-    def open_branch(self, name=None, unsupported=False, ignore_fallbacks=True):
+    def open_branch(self, name=None, unsupported=False, ignore_fallbacks=True,
+            possible_transports=None):
         if name is not None:
             raise errors.NoColocatedBranchSupport(self)
         return self._format.get_branch_format().open(self, _found=True)
@@ -465,7 +478,8 @@
             format=DummyForeignVcsDirFormat())
         target_tree.branch.lock_write()
         try:
-            pushresult = source_tree.branch.push(target_tree.branch, lossy=True)
+            pushresult = source_tree.branch.push(
+                target_tree.branch, lossy=True)
         finally:
             target_tree.branch.unlock()
         self.assertEquals(revision.NULL_REVISION, pushresult.old_revid)

=== modified file 'bzrlib/tests/test_ftp_transport.py'
--- bzrlib/tests/test_ftp_transport.py	2011-08-19 22:34:02 +0000
+++ bzrlib/tests/test_ftp_transport.py	2011-12-18 15:43:47 +0000
@@ -16,7 +16,6 @@
 
 import ftplib
 import getpass
-import urllib
 
 from bzrlib import (
     config,
@@ -24,6 +23,7 @@
     tests,
     transport,
     ui,
+    urlutils,
     )
 
 from bzrlib.transport import ftp
@@ -85,9 +85,9 @@
         parsed_url = transport.ConnectedTransport._split_url(base)
         new_url = parsed_url.clone()
         new_url.user = self.user
-        new_url.quoted_user = urllib.quote(self.user)
+        new_url.quoted_user = urlutils.quote(self.user)
         new_url.password = self.password
-        new_url.quoted_password = urllib.quote(self.password)
+        new_url.quoted_password = urlutils.quote(self.password)
         return str(new_url)
 
     def test_no_prompt_for_username(self):

=== modified file 'bzrlib/tests/test_gpg.py'
--- bzrlib/tests/test_gpg.py	2011-08-04 14:39:42 +0000
+++ bzrlib/tests/test_gpg.py	2011-12-21 14:25:26 +0000
@@ -20,53 +20,78 @@
 # import system imports here
 import sys
 
-from bzrlib import errors, ui
-import bzrlib.gpg as gpg
-from bzrlib.tests import TestCase
-from bzrlib.tests import features
-
-class FakeConfig(object):
-
-    def gpg_signing_key(self):
-        return "amy@example.com"
-
-    def gpg_signing_command(self):
-        return "false"
-
-    def acceptable_keys(self):
-        return None
-
-
-class TestCommandLine(TestCase):
+from bzrlib import (
+    config,
+    errors,
+    gpg,
+    tests,
+    trace,
+    ui,
+    )
+from bzrlib.tests import (
+    TestCase,
+    features,
+    )
+
+
+class FakeConfig(config.MemoryStack):
+
+    def __init__(self, content=None):
+        if content is None:
+            content = '''
+gpg_signing_key=amy@example.com
+gpg_signing_command=false'''
+        super(FakeConfig, self).__init__(content)
+
+
+class TestCommandLine(tests.TestCase):
+
+    def setUp(self):
+        super(TestCommandLine, self).setUp()
+        self.my_gpg = gpg.GPGStrategy(FakeConfig())
 
     def test_signing_command_line(self):
-        my_gpg = gpg.GPGStrategy(FakeConfig())
+        self.assertEqual(['false',  '--clearsign', '-u', 'amy@example.com'],
+                         self.my_gpg._command_line())
+
+    def test_signing_command_line_from_default(self):
+        # Using 'default' for gpg_signing_key will use the mail part of 'email'
+        my_gpg = gpg.GPGStrategy(FakeConfig('''
+email=Amy <amy@example.com>
+gpg_signing_key=default
+gpg_signing_command=false'''))
+        self.assertEqual(['false',  '--clearsign', '-u', 'amy@example.com'],
+                         my_gpg._command_line())
+
+    def test_signing_command_line_from_email(self):
+        # Not setting gpg_signing_key will use the mail part of 'email'
+        my_gpg = gpg.GPGStrategy(FakeConfig('''
+email=Amy <amy@example.com>
+gpg_signing_command=false'''))
         self.assertEqual(['false',  '--clearsign', '-u', 'amy@example.com'],
                          my_gpg._command_line())
 
     def test_checks_return_code(self):
         # This test needs a unix like platform - one with 'false' to run.
         # if you have one, please make this work :)
-        my_gpg = gpg.GPGStrategy(FakeConfig())
-        self.assertRaises(errors.SigningFailed, my_gpg.sign, 'content')
+        self.assertRaises(errors.SigningFailed, self.my_gpg.sign, 'content')
 
     def assertProduces(self, content):
         # This needs a 'cat' command or similar to work.
-        my_gpg = gpg.GPGStrategy(FakeConfig())
         if sys.platform == 'win32':
             # Windows doesn't come with cat, and we don't require it
             # so lets try using python instead.
             # But stupid windows and line-ending conversions.
             # It is too much work to make sys.stdout be in binary mode.
             # http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/65443
-            my_gpg._command_line = lambda:[sys.executable, '-c',
+            self.my_gpg._command_line = lambda:[sys.executable, '-c',
                     'import sys; sys.stdout.write(sys.stdin.read())']
             new_content = content.replace('\n', '\r\n')
 
-            self.assertEqual(new_content, my_gpg.sign(content))
+            self.assertEqual(new_content, self.my_gpg.sign(content))
         else:
-            my_gpg._command_line = lambda:['cat', '-']
-            self.assertEqual(content, my_gpg.sign(content))
+            self.my_gpg._command_line = lambda:['cat', '-']
+            self.assertEqual(content, self.my_gpg.sign(content))
 
     def test_returns_output(self):
         content = "some content\nwith newlines\n"
@@ -91,6 +116,7 @@
         self.assertRaises(errors.BzrBadParameterUnicode,
                           self.assertProduces, u'foo')
 
+
 class TestVerify(TestCase):
 
     def import_keys(self):
@@ -244,7 +270,7 @@
         #untrusted by gpg but listed as acceptable_keys by user
         self.requireFeature(features.gpgme)
         self.import_keys()
-            
+
         content = """-----BEGIN PGP SIGNED MESSAGE-----
 Hash: SHA1
 
@@ -275,7 +301,7 @@
     def test_verify_unacceptable_key(self):
         self.requireFeature(features.gpgme)
         self.import_keys()
-            
+
         content = """-----BEGIN PGP SIGNED MESSAGE-----
 Hash: SHA1
 
@@ -306,7 +332,7 @@
     def test_verify_valid_but_untrusted(self):
         self.requireFeature(features.gpgme)
         self.import_keys()
-            
+
         content = """-----BEGIN PGP SIGNED MESSAGE-----
 Hash: SHA1
 
@@ -336,7 +362,7 @@
     def test_verify_bad_testament(self):
         self.requireFeature(features.gpgme)
         self.import_keys()
-            
+
         content = """-----BEGIN PGP SIGNED MESSAGE-----
 Hash: SHA1
 
@@ -368,7 +394,7 @@
     def test_verify_revoked_signature(self):
         self.requireFeature(features.gpgme)
         self.import_keys()
-            
+
         content = """-----BEGIN PGP SIGNED MESSAGE-----
 Hash: SHA1
 
@@ -391,6 +417,7 @@
 
     def test_verify_invalid(self):
         self.requireFeature(features.gpgme)
+        self.import_keys()
         content = """-----BEGIN PGP SIGNED MESSAGE-----
 Hash: SHA1
 
@@ -415,6 +442,7 @@
 
     def test_verify_expired_but_valid(self):
         self.requireFeature(features.gpgme)
+        self.import_keys()
         content = """-----BEGIN PGP SIGNED MESSAGE-----
 Hash: SHA1
  
@@ -441,6 +469,7 @@
 
     def test_verify_unknown_key(self):
         self.requireFeature(features.gpgme)
+        self.import_keys()
         content = """-----BEGIN PGP SIGNED MESSAGE-----
 Hash: SHA1
 
@@ -473,8 +502,14 @@
     def test_set_acceptable_keys_unknown(self):
         self.requireFeature(features.gpgme)
         my_gpg = gpg.GPGStrategy(FakeConfig())
+        self.notes = []
+        def note(*args):
+            self.notes.append(args[0] % args[1:])
+        self.overrideAttr(trace, 'note', note)
         my_gpg.set_acceptable_keys("unknown")
         self.assertEqual(my_gpg.acceptable_keys, [])
+        self.assertEqual(self.notes,
+            ['No GnuPG key results for pattern: unknown'])
 
 
 class TestDisabled(TestCase):

=== modified file 'bzrlib/tests/test_graph.py'
--- bzrlib/tests/test_graph.py	2011-08-26 11:35:50 +0000
+++ bzrlib/tests/test_graph.py	2011-12-05 15:16:52 +0000
@@ -1029,9 +1029,8 @@
                 search.start_searching(starts)
             if stops is not None:
                 search.stop_searching_any(stops)
-            result = search.get_result()
-            self.assertEqual(recipe, result.get_recipe())
-            self.assertEqual(set(included_keys), result.get_keys())
+            state = search.get_state()
+            self.assertEqual(set(included_keys), state[2])
             self.assertEqual(seen, search.seen)
 
     def test_breadth_first_get_result_excludes_current_pending(self):
@@ -1042,10 +1041,9 @@
             })
         search = graph._make_breadth_first_searcher(['head'])
         # At the start, nothing has been seen, to its all excluded:
-        result = search.get_result()
-        self.assertEqual(('search', set(['head']), set(['head']), 0),
-            result.get_recipe())
-        self.assertEqual(set(), result.get_keys())
+        state = search.get_state()
+        self.assertEqual((set(['head']), set(['head']), set()),
+            state)
         self.assertEqual(set(), search.seen)
         # using next:
         expected = [
@@ -1074,10 +1072,9 @@
         # Starting with nothing and adding a search works:
         search.start_searching(['head'])
         # head has been seen:
-        result = search.get_result()
-        self.assertEqual(('search', set(['head']), set(['child']), 1),
-            result.get_recipe())
-        self.assertEqual(set(['head']), result.get_keys())
+        state = search.get_state()
+        self.assertEqual((set(['head']), set(['child']), set(['head'])),
+            state)
         self.assertEqual(set(['head']), search.seen)
         # using next:
         expected = [
@@ -1234,19 +1231,21 @@
         self.assertSeenAndResult(expected, search, search.next)
         self.assertRaises(StopIteration, search.next)
         self.assertEqual(set(['head', 'ghost', NULL_REVISION]), search.seen)
-        result = search.get_result()
-        self.assertEqual(('search', set(['ghost', 'head']), set(['ghost']), 2),
-            result.get_recipe())
-        self.assertEqual(set(['head', NULL_REVISION]), result.get_keys())
+        state = search.get_state()
+        self.assertEqual(
+            (set(['ghost', 'head']), set(['ghost']),
+                set(['head', NULL_REVISION])),
+            state)
         # using next_with_ghosts:
         search = graph._make_breadth_first_searcher(['head'])
         self.assertSeenAndResult(expected, search, search.next_with_ghosts)
         self.assertRaises(StopIteration, search.next)
         self.assertEqual(set(['head', 'ghost', NULL_REVISION]), search.seen)
-        result = search.get_result()
-        self.assertEqual(('search', set(['ghost', 'head']), set(['ghost']), 2),
-            result.get_recipe())
-        self.assertEqual(set(['head', NULL_REVISION]), result.get_keys())
+        state = search.get_state()
+        self.assertEqual(
+            (set(['ghost', 'head']), set(['ghost']),
+                set(['head', NULL_REVISION])),
+            state)
 
 
 class TestFindUniqueAncestors(TestGraphBase):
@@ -1677,167 +1676,6 @@
                  for n in graph_thunk.merge_sort('C')])
 
 
-class TestPendingAncestryResultGetKeys(TestCaseWithMemoryTransport):
-    """Tests for bzrlib.graph.PendingAncestryResult."""
-
-    def test_get_keys(self):
-        builder = self.make_branch_builder('b')
-        builder.start_series()
-        builder.build_snapshot('rev-1', None, [
-            ('add', ('', 'root-id', 'directory', ''))])
-        builder.build_snapshot('rev-2', ['rev-1'], [])
-        builder.finish_series()
-        repo = builder.get_branch().repository
-        repo.lock_read()
-        self.addCleanup(repo.unlock)
-        result = _mod_graph.PendingAncestryResult(['rev-2'], repo)
-        self.assertEqual(set(['rev-1', 'rev-2']), set(result.get_keys()))
-
-    def test_get_keys_excludes_ghosts(self):
-        builder = self.make_branch_builder('b')
-        builder.start_series()
-        builder.build_snapshot('rev-1', None, [
-            ('add', ('', 'root-id', 'directory', ''))])
-        builder.build_snapshot('rev-2', ['rev-1', 'ghost'], [])
-        builder.finish_series()
-        repo = builder.get_branch().repository
-        repo.lock_read()
-        self.addCleanup(repo.unlock)
-        result = _mod_graph.PendingAncestryResult(['rev-2'], repo)
-        self.assertEqual(sorted(['rev-1', 'rev-2']), sorted(result.get_keys()))
-
-    def test_get_keys_excludes_null(self):
-        # Make a 'graph' with an iter_ancestry that returns NULL_REVISION
-        # somewhere other than the last element, which can happen in real
-        # ancestries.
-        class StubGraph(object):
-            def iter_ancestry(self, keys):
-                return [(NULL_REVISION, ()), ('foo', (NULL_REVISION,))]
-        result = _mod_graph.PendingAncestryResult(['rev-3'], None)
-        result_keys = result._get_keys(StubGraph())
-        # Only the non-null keys from the ancestry appear.
-        self.assertEqual(set(['foo']), set(result_keys))
-
-
-class TestPendingAncestryResultRefine(TestGraphBase):
-
-    def test_refine(self):
-        # Used when pulling from a stacked repository, so test some revisions
-        # being satisfied from the stacking branch.
-        g = self.make_graph(
-            {"tip":["mid"], "mid":["base"], "tag":["base"],
-             "base":[NULL_REVISION], NULL_REVISION:[]})
-        result = _mod_graph.PendingAncestryResult(['tip', 'tag'], None)
-        result = result.refine(set(['tip']), set(['mid']))
-        self.assertEqual(set(['mid', 'tag']), result.heads)
-        result = result.refine(set(['mid', 'tag', 'base']),
-            set([NULL_REVISION]))
-        self.assertEqual(set([NULL_REVISION]), result.heads)
-        self.assertTrue(result.is_empty())
-
-
-class TestSearchResultRefine(TestGraphBase):
-
-    def test_refine(self):
-        # Used when pulling from a stacked repository, so test some revisions
-        # being satisfied from the stacking branch.
-        g = self.make_graph(
-            {"tip":["mid"], "mid":["base"], "tag":["base"],
-             "base":[NULL_REVISION], NULL_REVISION:[]})
-        result = _mod_graph.SearchResult(set(['tip', 'tag']),
-            set([NULL_REVISION]), 4, set(['tip', 'mid', 'tag', 'base']))
-        result = result.refine(set(['tip']), set(['mid']))
-        recipe = result.get_recipe()
-        # We should be starting from tag (original head) and mid (seen ref)
-        self.assertEqual(set(['mid', 'tag']), recipe[1])
-        # We should be stopping at NULL (original stop) and tip (seen head)
-        self.assertEqual(set([NULL_REVISION, 'tip']), recipe[2])
-        self.assertEqual(3, recipe[3])
-        result = result.refine(set(['mid', 'tag', 'base']),
-            set([NULL_REVISION]))
-        recipe = result.get_recipe()
-        # We should be starting from nothing (NULL was known as a cut point)
-        self.assertEqual(set([]), recipe[1])
-        # We should be stopping at NULL (original stop) and tip (seen head) and
-        # tag (seen head) and mid(seen mid-point head). We could come back and
-        # define this as not including mid, for minimal results, but it is
-        # still 'correct' to include mid, and simpler/easier.
-        self.assertEqual(set([NULL_REVISION, 'tip', 'tag', 'mid']), recipe[2])
-        self.assertEqual(0, recipe[3])
-        self.assertTrue(result.is_empty())
-
-
-class TestSearchResultFromParentMap(TestGraphBase):
-
-    def assertSearchResult(self, start_keys, stop_keys, key_count, parent_map,
-                           missing_keys=()):
-        (start, stop, count) = _mod_graph.search_result_from_parent_map(
-            parent_map, missing_keys)
-        self.assertEqual((sorted(start_keys), sorted(stop_keys), key_count),
-                         (sorted(start), sorted(stop), count))
-
-    def test_no_parents(self):
-        self.assertSearchResult([], [], 0, {})
-        self.assertSearchResult([], [], 0, None)
-
-    def test_ancestry_1(self):
-        self.assertSearchResult(['rev4'], [NULL_REVISION], len(ancestry_1),
-                                ancestry_1)
-
-    def test_ancestry_2(self):
-        self.assertSearchResult(['rev1b', 'rev4a'], [NULL_REVISION],
-                                len(ancestry_2), ancestry_2)
-        self.assertSearchResult(['rev1b', 'rev4a'], [],
-                                len(ancestry_2)+1, ancestry_2,
-                                missing_keys=[NULL_REVISION])
-
-    def test_partial_search(self):
-        parent_map = dict((k,extended_history_shortcut[k])
-                          for k in ['e', 'f'])
-        self.assertSearchResult(['e', 'f'], ['d', 'a'], 2,
-                                parent_map)
-        parent_map.update((k,extended_history_shortcut[k])
-                          for k in ['d', 'a'])
-        self.assertSearchResult(['e', 'f'], ['c', NULL_REVISION], 4,
-                                parent_map)
-        parent_map['c'] = extended_history_shortcut['c']
-        self.assertSearchResult(['e', 'f'], ['b'], 6,
-                                parent_map, missing_keys=[NULL_REVISION])
-        parent_map['b'] = extended_history_shortcut['b']
-        self.assertSearchResult(['e', 'f'], [], 7,
-                                parent_map, missing_keys=[NULL_REVISION])
-
-
-class TestLimitedSearchResultFromParentMap(TestGraphBase):
-
-    def assertSearchResult(self, start_keys, stop_keys, key_count, parent_map,
-                           missing_keys, tip_keys, depth):
-        (start, stop, count) = _mod_graph.limited_search_result_from_parent_map(
-            parent_map, missing_keys, tip_keys, depth)
-        self.assertEqual((sorted(start_keys), sorted(stop_keys), key_count),
-                         (sorted(start), sorted(stop), count))
-
-    def test_empty_ancestry(self):
-        self.assertSearchResult([], [], 0, {}, (), ['tip-rev-id'], 10)
-
-    def test_ancestry_1(self):
-        self.assertSearchResult(['rev4'], ['rev1'], 4,
-                                ancestry_1, (), ['rev1'], 10)
-        self.assertSearchResult(['rev2a', 'rev2b'], ['rev1'], 2,
-                                ancestry_1, (), ['rev1'], 1)
-
-
-    def test_multiple_heads(self):
-        self.assertSearchResult(['e', 'f'], ['a'], 5,
-                                extended_history_shortcut, (), ['a'], 10)
-        # Note that even though we only take 1 step back, we find 'f', which
-        # means the described search will still find d and c.
-        self.assertSearchResult(['f'], ['a'], 4,
-                                extended_history_shortcut, (), ['a'], 1)
-        self.assertSearchResult(['f'], ['a'], 4,
-                                extended_history_shortcut, (), ['a'], 2)
-
-
 class TestStackedParentsProvider(tests.TestCase):
 
     def setUp(self):

=== modified file 'bzrlib/tests/test_help.py'
--- bzrlib/tests/test_help.py	2011-08-11 10:19:32 +0000
+++ bzrlib/tests/test_help.py	2011-09-27 13:18:53 +0000
@@ -45,17 +45,17 @@
             __doc__ = """A sample command."""
             _see_also = ['foo', 'bar']
         self.assertCmdHelp('''\
-            Purpose: A sample command.
-            Usage:   bzr WithSeeAlso
-            
-            Options:
-              --usage        Show usage message and options.
-              -v, --verbose  Display more information.
-              -q, --quiet    Only display errors and warnings.
-              -h, --help     Show help message.
-            
-            See also: bar, foo
-            ''',
+Purpose: A sample command.
+Usage:   bzr WithSeeAlso
+
+Options:
+  --usage        Show usage message and options.
+  -q, --quiet    Only display errors and warnings.
+  -v, --verbose  Display more information.
+  -h, --help     Show help message.
+
+See also: bar, foo
+''',
                            cmd_WithSeeAlso())
 
     def test_get_help_text(self):
@@ -63,16 +63,16 @@
         class cmd_Demo(commands.Command):
             __doc__ = """A sample command."""
         self.assertCmdHelp('''\
-            Purpose: A sample command.
-            Usage:   bzr Demo
-            
-            Options:
-              --usage        Show usage message and options.
-              -v, --verbose  Display more information.
-              -q, --quiet    Only display errors and warnings.
-              -h, --help     Show help message.
-
-            ''',
+Purpose: A sample command.
+Usage:   bzr Demo
+
+Options:
+  --usage        Show usage message and options.
+  -q, --quiet    Only display errors and warnings.
+  -v, --verbose  Display more information.
+  -h, --help     Show help message.
+
+''',
                            cmd_Demo())
         cmd = cmd_Demo()
         helptext = cmd.get_help_text()
@@ -90,8 +90,8 @@
         helptext = cmd.get_help_text(['gam'])
         self.assertEndsWith(
             helptext,
+            '  -q, --quiet    Only display errors and warnings.\n'
             '  -v, --verbose  Display more information.\n'
-            '  -q, --quiet    Only display errors and warnings.\n'
             '  -h, --help     Show help message.\n'
             '\n'
             'See also: bar, foo, gam\n')
@@ -103,8 +103,8 @@
         helptext = cmd.get_help_text(['gam'])
         self.assertEndsWith(
             helptext,
+            '  -q, --quiet    Only display errors and warnings.\n'
             '  -v, --verbose  Display more information.\n'
-            '  -q, --quiet    Only display errors and warnings.\n'
             '  -h, --help     Show help message.\n'
             '\n'
             'See also: gam\n')
@@ -138,56 +138,59 @@
             """
         cmd = cmd_Demo()
         helptext = cmd.get_help_text()
-        self.assertEquals(
-            helptext,
-            'Purpose: A sample command.\n'
-            'Usage:   bzr Demo\n'
-            '\n'
-            'Options:\n'
-            '  --usage        Show usage message and options.\n'
-            '  -v, --verbose  Display more information.\n'
-            '  -q, --quiet    Only display errors and warnings.\n'
-            '  -h, --help     Show help message.\n'
-            '\n'
-            'Examples:\n'
-            '    Example 1:\n'
-            '\n'
-            '        cmd arg1\n'
-            '\n'
-            '    Example 2:\n'
-            '\n'
-            '        cmd arg2\n'
-            '\n'
-            '    A code block follows.\n'
-            '\n'
-            '        bzr Demo something\n'
-            '\n')
+        self.assertEqualDiff('''\
+Purpose: A sample command.
+Usage:   bzr Demo
+
+Options:
+  --usage        Show usage message and options.
+  -q, --quiet    Only display errors and warnings.
+  -v, --verbose  Display more information.
+  -h, --help     Show help message.
+
+Examples:
+    Example 1:
+
+        cmd arg1
+
+    Example 2:
+
+        cmd arg2
+
+    A code block follows.
+
+        bzr Demo something
+
+''',
+                                         helptext)
         helptext = cmd.get_help_text(plain=False)
-        self.assertEquals(helptext,
-            ':Purpose: A sample command.\n'
-            ':Usage:   bzr Demo\n'
-            '\n'
-            ':Options:\n'
-            '  --usage        Show usage message and options.\n'
-            '  -v, --verbose  Display more information.\n'
-            '  -q, --quiet    Only display errors and warnings.\n'
-            '  -h, --help     Show help message.\n'
-            '\n'
-            ':Examples:\n'
-            '    Example 1::\n'
-            '\n'
-            '        cmd arg1\n'
-            '\n'
-            '    Example 2::\n'
-            '\n'
-            '        cmd arg2\n'
-            '\n'
-            '    A code block follows.\n'
-            '\n'
-            '    ::\n'
-            '\n'
-            '        bzr Demo something\n'
-            '\n')
+        self.assertEqualDiff('''\
+:Purpose: A sample command.
+:Usage:   bzr Demo
+
+:Options:
+  --usage        Show usage message and options.
+  -q, --quiet    Only display errors and warnings.
+  -v, --verbose  Display more information.
+  -h, --help     Show help message.
+
+:Examples:
+    Example 1::
+
+        cmd arg1
+
+    Example 2::
+
+        cmd arg2
+
+    A code block follows.
+
+    ::
+
+        bzr Demo something
+
+''',
+                             helptext)
 
     def test_concise_help_text(self):
         """Concise help text excludes the descriptive sections."""
@@ -203,84 +206,89 @@
             """
         cmd = cmd_Demo()
         helptext = cmd.get_help_text()
-        self.assertEqualDiff(
-            helptext,
-            'Purpose: A sample command.\n'
-            'Usage:   bzr Demo\n'
-            '\n'
-            'Options:\n'
-            '  --usage        Show usage message and options.\n'
-            '  -v, --verbose  Display more information.\n'
-            '  -q, --quiet    Only display errors and warnings.\n'
-            '  -h, --help     Show help message.\n'
-            '\n'
-            'Description:\n'
-            '  Blah blah blah.\n'
-            '\n'
-            'Examples:\n'
-            '    Example 1:\n'
-            '\n'
-            '        cmd arg1\n'
-            '\n')
+        self.assertEqualDiff('''\
+Purpose: A sample command.
+Usage:   bzr Demo
+
+Options:
+  --usage        Show usage message and options.
+  -q, --quiet    Only display errors and warnings.
+  -v, --verbose  Display more information.
+  -h, --help     Show help message.
+
+Description:
+  Blah blah blah.
+
+Examples:
+    Example 1:
+
+        cmd arg1
+
+''',
+                             helptext)
         helptext = cmd.get_help_text(verbose=False)
-        self.assertEquals(helptext,
-            'Purpose: A sample command.\n'
-            'Usage:   bzr Demo\n'
-            '\n'
-            'Options:\n'
-            '  --usage        Show usage message and options.\n'
-            '  -v, --verbose  Display more information.\n'
-            '  -q, --quiet    Only display errors and warnings.\n'
-            '  -h, --help     Show help message.\n'
-            '\n'
-            'See bzr help Demo for more details and examples.\n'
-            '\n')
+        self.assertEqualDiff('''\
+Purpose: A sample command.
+Usage:   bzr Demo
+
+Options:
+  --usage        Show usage message and options.
+  -q, --quiet    Only display errors and warnings.
+  -v, --verbose  Display more information.
+  -h, --help     Show help message.
+
+See bzr help Demo for more details and examples.
+
+''',
+                             helptext)
 
     def test_help_custom_section_ordering(self):
         """Custom descriptive sections should remain in the order given."""
         class cmd_Demo(commands.Command):
-            __doc__ = """A sample command.
- 
-            Blah blah blah.
-
-            :Formats:
-              Interesting stuff about formats.
-
-            :Examples:
-              Example 1::
- 
-                cmd arg1
-
-            :Tips:
-              Clever things to keep in mind.
-            """
+            __doc__ = """\
+A sample command.
+
+Blah blah blah.
+
+:Formats:
+  Interesting stuff about formats.
+
+:Examples:
+  Example 1::
+
+    cmd arg1
+
+:Tips:
+  Clever things to keep in mind.
+"""
         cmd = cmd_Demo()
         helptext = cmd.get_help_text()
-        self.assertEqualDiff(
-            helptext,
-            'Purpose: A sample command.\n'
-            'Usage:   bzr Demo\n'
-            '\n'
-            'Options:\n'
-            '  --usage        Show usage message and options.\n'
-            '  -v, --verbose  Display more information.\n'
-            '  -q, --quiet    Only display errors and warnings.\n'
-            '  -h, --help     Show help message.\n'
-            '\n'
-            'Description:\n'
-            '  Blah blah blah.\n'
-            '\n'
-            'Formats:\n'
-            '  Interesting stuff about formats.\n'
-            '\n'
-            'Examples:\n'
-            '  Example 1:\n'
-            '\n'
-            '    cmd arg1\n'
-            '\n'
-            'Tips:\n'
-            '  Clever things to keep in mind.\n'
-            '\n')
+        self.assertEqualDiff('''\
+Purpose: A sample command.
+Usage:   bzr Demo
+
+Options:
+  --usage        Show usage message and options.
+  -q, --quiet    Only display errors and warnings.
+  -v, --verbose  Display more information.
+  -h, --help     Show help message.
+
+Description:
+  Blah blah blah.
+
+Formats:
+  Interesting stuff about formats.
+
+Examples:
+  Example 1:
+
+    cmd arg1
+
+Tips:
+  Clever things to keep in mind.
+
+''',
+                             helptext)
 
     def test_help_text_custom_usage(self):
         """Help text may contain a custom usage section."""
@@ -296,22 +304,25 @@
             """
         cmd = cmd_Demo()
         helptext = cmd.get_help_text()
-        self.assertEquals(helptext,
-            'Purpose: A sample command.\n'
-            'Usage:\n'
-            '    cmd Demo [opts] args\n'
-            '\n'
-            '    cmd Demo -h\n'
-            '\n'
-            '\n'
-            'Options:\n'
-            '  --usage        Show usage message and options.\n'
-            '  -v, --verbose  Display more information.\n'
-            '  -q, --quiet    Only display errors and warnings.\n'
-            '  -h, --help     Show help message.\n'
-            '\n'
-            'Description:\n'
-            '  Blah blah blah.\n\n')
+        self.assertEqualDiff('''\
+Purpose: A sample command.
+Usage:
+    cmd Demo [opts] args
+
+    cmd Demo -h
+
+
+Options:
+  --usage        Show usage message and options.
+  -q, --quiet    Only display errors and warnings.
+  -v, --verbose  Display more information.
+  -h, --help     Show help message.
+
+Description:
+  Blah blah blah.
+
+''',
+                             helptext)
 
 
 class ZzzTranslationsForDoc(ZzzTranslations):
@@ -343,17 +354,17 @@
             __doc__ = """A sample command."""
             _see_also = ['foo', 'bar']
         self.assertCmdHelp('''\
-            zz{{:Purpose: zz{{A sample command.}}
-            }}zz{{:Usage:   bzr WithSeeAlso
-            }}
-            zz{{:Options:
-              --usage        Show usage message and options.
-              -v, --verbose  Display more information.
-              -q, --quiet    Only display errors and warnings.
-              -h, --help     Show help message.
-            }}
-            zz{{:See also: bar, foo}}
-            ''',
+zz{{:Purpose: zz{{A sample command.}}
+}}zz{{:Usage:   bzr WithSeeAlso
+}}
+zz{{:Options:
+  --usage        zz{{Show usage message and options.}}
+  -q, --quiet    zz{{Only display errors and warnings.}}
+  -v, --verbose  zz{{Display more information.}}
+  -h, --help     zz{{Show help message.}}
+}}
+zz{{:See also: bar, foo}}
+''',
                            cmd_WithSeeAlso())
 
     def test_get_help_text(self):
@@ -361,16 +372,16 @@
         class cmd_Demo(commands.Command):
             __doc__ = """A sample command."""
         self.assertCmdHelp('''\
-            zz{{:Purpose: zz{{A sample command.}}
-            }}zz{{:Usage:   bzr Demo
-            }}
-            zz{{:Options:
-              --usage        Show usage message and options.
-              -v, --verbose  Display more information.
-              -q, --quiet    Only display errors and warnings.
-              -h, --help     Show help message.
-            }}
-            ''',
+zz{{:Purpose: zz{{A sample command.}}
+}}zz{{:Usage:   bzr Demo
+}}
+zz{{:Options:
+  --usage        zz{{Show usage message and options.}}
+  -q, --quiet    zz{{Only display errors and warnings.}}
+  -v, --verbose  zz{{Display more information.}}
+  -h, --help     zz{{Show help message.}}
+}}
+''',
                            cmd_Demo())
 
     def test_command_with_additional_see_also(self):
@@ -380,12 +391,13 @@
         cmd = cmd_WithSeeAlso()
         helptext = cmd.get_help_text(['gam'])
         self.assertEndsWith(
-            helptext,
-            '  -v, --verbose  Display more information.\n'
-            '  -q, --quiet    Only display errors and warnings.\n'
-            '  -h, --help     Show help message.\n'
-            '}}\n'
-            'zz{{:See also: bar, foo, gam}}\n')
+            helptext,'''\
+  -q, --quiet    zz{{Only display errors and warnings.}}
+  -v, --verbose  zz{{Display more information.}}
+  -h, --help     zz{{Show help message.}}
+}}
+zz{{:See also: bar, foo, gam}}
+''')
 
     def test_command_only_additional_see_also(self):
         class cmd_WithSeeAlso(commands.Command):
@@ -393,14 +405,15 @@
         cmd = cmd_WithSeeAlso()
         helptext = cmd.get_help_text(['gam'])
         self.assertEndsWith(
-            helptext,
-            'zz{{:Options:\n'
-            '  --usage        Show usage message and options.\n'
-            '  -v, --verbose  Display more information.\n'
-            '  -q, --quiet    Only display errors and warnings.\n'
-            '  -h, --help     Show help message.\n'
-            '}}\n'
-            'zz{{:See also: gam}}\n')
+            helptext, '''\
+zz{{:Options:
+  --usage        zz{{Show usage message and options.}}
+  -q, --quiet    zz{{Only display errors and warnings.}}
+  -v, --verbose  zz{{Display more information.}}
+  -h, --help     zz{{Show help message.}}
+}}
+zz{{:See also: gam}}
+''')
 
 
     def test_help_custom_section_ordering(self):
@@ -423,30 +436,30 @@
               Clever things to keep in mind.
             """
         self.assertCmdHelp('''\
-            zz{{:Purpose: zz{{A sample command.}}
-            }}zz{{:Usage:   bzr Demo
-            }}
-            zz{{:Options:
-              --usage        Show usage message and options.
-              -v, --verbose  Display more information.
-              -q, --quiet    Only display errors and warnings.
-              -h, --help     Show help message.
-            }}
-            Description:
-              zz{{zz{{Blah blah blah.}}
-            
-            }}:Formats:
-              zz{{Interesting stuff about formats.}}
-            
-            Examples:
-              zz{{Example 1::}}
-            
-                zz{{cmd arg1}}
-            
-            Tips:
-              zz{{Clever things to keep in mind.}}
-
-            ''',
+zz{{:Purpose: zz{{A sample command.}}
+}}zz{{:Usage:   bzr Demo
+}}
+zz{{:Options:
+  --usage        zz{{Show usage message and options.}}
+  -q, --quiet    zz{{Only display errors and warnings.}}
+  -v, --verbose  zz{{Display more information.}}
+  -h, --help     zz{{Show help message.}}
+}}
+Description:
+  zz{{zz{{Blah blah blah.}}
+ 
+}}:Formats:
+  zz{{Interesting stuff about formats.}}
+ 
+Examples:
+  zz{{Example 1::}}
+ 
+    zz{{cmd arg1}}
+ 
+Tips:
+  zz{{Clever things to keep in mind.}}
+ 
+''',
                            cmd_Demo())
 
     def test_help_text_custom_usage(self):
@@ -462,24 +475,24 @@
             Blah blah blah.
             """
         self.assertCmdHelp('''\
-            zz{{:Purpose: zz{{A sample command.}}
-            }}zz{{:Usage:
-                zz{{cmd Demo [opts] args}}
-            
-                zz{{cmd Demo -h}}
-
-            }}
-            zz{{:Options:
-              --usage        Show usage message and options.
-              -v, --verbose  Display more information.
-              -q, --quiet    Only display errors and warnings.
-              -h, --help     Show help message.
-            }}
-            Description:
-              zz{{zz{{Blah blah blah.}}
-
-            }}
-            ''',
+zz{{:Purpose: zz{{A sample command.}}
+}}zz{{:Usage:
+    zz{{cmd Demo [opts] args}}
+
+    zz{{cmd Demo -h}}
+
+}}
+zz{{:Options:
+  --usage        zz{{Show usage message and options.}}
+  -q, --quiet    zz{{Only display errors and warnings.}}
+  -v, --verbose  zz{{Display more information.}}
+  -h, --help     zz{{Show help message.}}
+}}
+Description:
+  zz{{zz{{Blah blah blah.}}
+
+}}
+''',
                            cmd_Demo())
 
 

=== modified file 'bzrlib/tests/test_http.py'
--- bzrlib/tests/test_http.py	2011-08-19 22:34:02 +0000
+++ bzrlib/tests/test_http.py	2011-10-12 16:00:13 +0000
@@ -1997,6 +1997,7 @@
         self.assertIsInstance(r, type(t))
         # Both transports share the some connection
         self.assertEqual(t._get_connection(), r._get_connection())
+        self.assertEquals('http://www.example.com/foo/subdir/', r.base)
 
     def test_redirected_to_self_with_slash(self):
         t = self._transport('http://www.example.com/foo')
@@ -2013,18 +2014,29 @@
         r = t._redirected_to('http://www.example.com/foo',
                              'http://foo.example.com/foo/subdir')
         self.assertIsInstance(r, type(t))
+        self.assertEquals('http://foo.example.com/foo/subdir/',
+            r.external_url())
 
     def test_redirected_to_same_host_sibling_protocol(self):
         t = self._transport('http://www.example.com/foo')
         r = t._redirected_to('http://www.example.com/foo',
                              'https://www.example.com/foo')
         self.assertIsInstance(r, type(t))
+        self.assertEquals('https://www.example.com/foo/',
+            r.external_url())
 
     def test_redirected_to_same_host_different_protocol(self):
         t = self._transport('http://www.example.com/foo')
         r = t._redirected_to('http://www.example.com/foo',
                              'ftp://www.example.com/foo')
         self.assertNotEquals(type(r), type(t))
+        self.assertEquals('ftp://www.example.com/foo/', r.external_url())
+
+    def test_redirected_to_same_host_specific_implementation(self):
+        t = self._transport('http://www.example.com/foo')
+        r = t._redirected_to('http://www.example.com/foo',
+                             'https+urllib://www.example.com/foo')
+        self.assertEquals('https://www.example.com/foo/', r.external_url())
 
     def test_redirected_to_different_host_same_user(self):
         t = self._transport('http://joe@www.example.com/foo')
@@ -2032,6 +2044,7 @@
                              'https://foo.example.com/foo')
         self.assertIsInstance(r, type(t))
         self.assertEqual(t._parsed_url.user, r._parsed_url.user)
+        self.assertEquals('https://joe@foo.example.com/foo/', r.external_url())
 
 
 class PredefinedRequestHandler(http_server.TestingHTTPRequestHandler):

=== modified file 'bzrlib/tests/test_i18n.py'
--- bzrlib/tests/test_i18n.py	2011-08-25 10:33:14 +0000
+++ bzrlib/tests/test_i18n.py	2011-10-04 22:20:49 +0000
@@ -16,7 +16,12 @@
 
 """Tests for bzrlib.i18n"""
 
-from bzrlib import i18n, tests
+from bzrlib import (
+    i18n,
+    tests,
+    errors,
+    workingtree,
+    )
 
 
 class ZzzTranslations(object):
@@ -28,7 +33,7 @@
     _null_translation = i18n._gettext.NullTranslations()
 
     def zzz(self, s):
-        return u'zz{{%s}}' % s
+        return u'zz\xe5{{%s}}' % s
 
     def ugettext(self, s):
         return self.zzz(self._null_translation.ugettext(s))
@@ -47,18 +52,18 @@
         trans = ZzzTranslations()
 
         t = trans.zzz('msg')
-        self._check_exact(u'zz{{msg}}', t)
+        self._check_exact(u'zz\xe5{{msg}}', t)
 
         t = trans.ugettext('msg')
-        self._check_exact(u'zz{{msg}}', t)
+        self._check_exact(u'zz\xe5{{msg}}', t)
 
         t = trans.ungettext('msg1', 'msg2', 0)
-        self._check_exact(u'zz{{msg2}}', t)
+        self._check_exact(u'zz\xe5{{msg2}}', t)
         t = trans.ungettext('msg1', 'msg2', 2)
-        self._check_exact(u'zz{{msg2}}', t)
+        self._check_exact(u'zz\xe5{{msg2}}', t)
 
         t = trans.ungettext('msg1', 'msg2', 1)
-        self._check_exact(u'zz{{msg1}}', t)
+        self._check_exact(u'zz\xe5{{msg1}}', t)
 
 
 class TestGetText(tests.TestCase):
@@ -68,11 +73,11 @@
         self.overrideAttr(i18n, '_translations', ZzzTranslations())
 
     def test_oneline(self):
-        self.assertEqual(u"zz{{spam ham eggs}}",
+        self.assertEqual(u"zz\xe5{{spam ham eggs}}",
                          i18n.gettext("spam ham eggs"))
 
     def test_multiline(self):
-        self.assertEqual(u"zz{{spam\nham\n\neggs\n}}",
+        self.assertEqual(u"zz\xe5{{spam\nham\n\neggs\n}}",
                          i18n.gettext("spam\nham\n\neggs\n"))
 
 
@@ -83,24 +88,75 @@
         self.overrideAttr(i18n, '_translations', ZzzTranslations())
 
     def test_oneline(self):
-        self.assertEqual(u"zz{{spam ham eggs}}",
+        self.assertEqual(u"zz\xe5{{spam ham eggs}}",
                          i18n.gettext_per_paragraph("spam ham eggs"))
 
     def test_multiline(self):
-        self.assertEqual(u"zz{{spam\nham}}\n\nzz{{eggs\n}}",
+        self.assertEqual(u"zz\xe5{{spam\nham}}\n\nzz\xe5{{eggs\n}}",
                          i18n.gettext_per_paragraph("spam\nham\n\neggs\n"))
 
 
 class TestInstall(tests.TestCase):
 
+    def setUp(self):
+        super(TestInstall, self).setUp()
+        # Restore a proper env to test translation installation
+        self.overrideAttr(i18n, '_translations', None)
+
     def test_custom_languages(self):
-        self.addCleanup(i18n.install)
         i18n.install('nl:fy')
+        # Whether we found a valid tranlsation or not doesn't matter, we got
+        # one and _translations is not None anymore.
+        self.assertIsInstance(i18n._translations,
+                              i18n._gettext.NullTranslations)
 
     def test_no_env_variables(self):
-        self.addCleanup(i18n.install)
         self.overrideEnv('LANGUAGE', None)
         self.overrideEnv('LC_ALL', None)
         self.overrideEnv('LC_MESSAGES', None)
         self.overrideEnv('LANG', None)
         i18n.install()
+        # Whether we found a valid tranlsation or not doesn't matter, we got
+        # one and _translations is not None anymore.
+        self.assertIsInstance(i18n._translations,
+                              i18n._gettext.NullTranslations)
+
+    def test_disable_i18n(self):
+        i18n.disable_i18n()
+        i18n.install()
+        # It's disabled, you can't install anything and we fallback to null
+        self.assertIsInstance(i18n._translations,
+                              i18n._gettext.NullTranslations)
+
+
+class TestTranslate(tests.TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestTranslate, self).setUp()
+        self.overrideAttr(i18n, '_translations', ZzzTranslations())
+
+    def test_error_message_translation(self):
+        """do errors get translated?"""
+        err = None
+        tree = self.make_branch_and_tree('.')
+        try:
+            workingtree.WorkingTree.open('./foo')
+        except errors.NotBranchError,e:
+            err = str(e)
+        self.assertContainsRe(err, 
+                              u"zz\xe5{{Not a branch: .*}}".encode("utf-8"))
+
+    def test_topic_help_translation(self):
+        """does topic help get translated?"""
+        from bzrlib import help
+        from StringIO import StringIO
+        out = StringIO()
+        help.help("authentication", out)
+        self.assertContainsRe(out.getvalue(), "zz\xe5{{Authentication Settings")
+
+
+class LoadPluginTranslations(tests.TestCase):
+
+    def test_does_not_exist(self):
+        translation = i18n.load_plugin_translations("doesnotexist")
+        self.assertEquals("foo", translation.gettext("foo"))

=== modified file 'bzrlib/tests/test_import_tariff.py'
--- bzrlib/tests/test_import_tariff.py	2011-06-06 11:57:09 +0000
+++ bzrlib/tests/test_import_tariff.py	2011-12-19 00:55:07 +0000
@@ -167,6 +167,7 @@
             'bzrlib.cmd_version_info',
             'bzrlib.externalcommand',
             'bzrlib.filters',
+            'bzrlib.hashcache',
             # foreign branch plugins import the foreign_vcs_registry from 
             # bzrlib.foreign so it can't be blacklisted
             'bzrlib.gpg',
@@ -185,11 +186,18 @@
             'bzrlib.smart.server',
             'bzrlib.transform',
             'bzrlib.version_info_formats.format_rio',
+            'bzrlib.xml_serializer',
+            'bzrlib.xml8',
             'getpass',
             'kerberos',
+            'ssl',
+            'socket',
             'smtplib',
             'tarfile',
             'tempfile',
+            'termios',
+            'tty',
+            'urllib',
             ] + old_format_modules)
         # TODO: similar test for repository-only operations, checking we avoid
         # loading wt-specific stuff
@@ -235,6 +243,7 @@
             'bzrlib._dirstate_helpers_pyx',
             'bzrlib.externalcommand',
             'bzrlib.filters',
+            'bzrlib.hashcache',
             # foreign branch plugins import the foreign_vcs_registry from 
             # bzrlib.foreign so it can't be blacklisted
             'bzrlib.gpg',
@@ -251,9 +260,13 @@
             'bzrlib.transform',
             'bzrlib.version_info_formats.format_rio',
             'bzrlib.workingtree_4',
+            'bzrlib.xml_serializer',
+            'bzrlib.xml8',
             'getpass',
             'kerberos',
             'smtplib',
             'tarfile',
             'tempfile',
+            'termios',
+            'tty',
             ] + old_format_modules)

=== modified file 'bzrlib/tests/test_info.py'
--- bzrlib/tests/test_info.py	2011-03-11 21:06:09 +0000
+++ bzrlib/tests/test_info.py	2011-11-15 17:27:46 +0000
@@ -18,7 +18,7 @@
 
 from bzrlib import (
     branch as _mod_branch,
-    bzrdir,
+    controldir,
     info,
     tests,
     workingtree,
@@ -54,7 +54,7 @@
 
     def test_describe_repository_layout(self):
         repository = self.make_repository('.', shared=True)
-        tree = bzrdir.BzrDir.create_branch_convenience('tree',
+        tree = controldir.ControlDir.create_branch_convenience('tree',
             force_new_tree=True).bzrdir.open_workingtree()
         self.assertEqual('Shared repository with trees',
             info.describe_layout(tree.branch.repository))
@@ -90,11 +90,11 @@
         # this ought to be easier...
         branch.create_checkout('%s_co' % format,
             lightweight=True).bzrdir.destroy_workingtree()
-        control = bzrdir.BzrDir.open('%s_co' % format)
+        control = controldir.ControlDir.open('%s_co' % format)
         old_format = control._format.workingtree_format
         try:
             control._format.workingtree_format = \
-                bzrdir.format_registry.make_bzrdir(format).workingtree_format
+                controldir.format_registry.make_bzrdir(format).workingtree_format
             control.create_workingtree()
             tree = workingtree.WorkingTree.open('%s_co' % format)
             format_description = info.describe_format(tree.bzrdir,
@@ -124,31 +124,31 @@
             repo, None, None))
 
     def test_describe_tree_format(self):
-        for key in bzrdir.format_registry.keys():
-            if key in bzrdir.format_registry.aliases():
+        for key in controldir.format_registry.keys():
+            if key in controldir.format_registry.aliases():
                 continue
             self.assertTreeDescription(key)
 
     def test_describe_checkout_format(self):
-        for key in bzrdir.format_registry.keys():
-            if key in bzrdir.format_registry.aliases():
+        for key in controldir.format_registry.keys():
+            if key in controldir.format_registry.aliases():
                 # Aliases will not describe correctly in the UI because the
                 # real format is found.
                 continue
             # legacy: weave does not support checkouts
             if key == 'weave':
                 continue
-            if bzrdir.format_registry.get_info(key).experimental:
+            if controldir.format_registry.get_info(key).experimental:
                 # We don't require that experimental formats support checkouts
                 # or describe correctly in the UI.
                 continue
-            if bzrdir.format_registry.get_info(key).hidden:
+            if controldir.format_registry.get_info(key).hidden:
                 continue
             expected = None
             if key in ('pack-0.92',):
                 expected = 'pack-0.92'
             elif key in ('knit', 'metaweave'):
-                if 'metaweave' in bzrdir.format_registry:
+                if 'metaweave' in controldir.format_registry:
                     expected = 'knit or metaweave'
                 else:
                     expected = 'knit'
@@ -157,10 +157,10 @@
             self.assertCheckoutDescription(key, expected)
 
     def test_describe_branch_format(self):
-        for key in bzrdir.format_registry.keys():
-            if key in bzrdir.format_registry.aliases():
+        for key in controldir.format_registry.keys():
+            if key in controldir.format_registry.aliases():
                 continue
-            if bzrdir.format_registry.get_info(key).hidden:
+            if controldir.format_registry.get_info(key).hidden:
                 continue
             expected = None
             if key in ('dirstate', 'knit'):
@@ -172,10 +172,10 @@
             self.assertBranchDescription(key, expected)
 
     def test_describe_repo_format(self):
-        for key in bzrdir.format_registry.keys():
-            if key in bzrdir.format_registry.aliases():
+        for key in controldir.format_registry.keys():
+            if key in controldir.format_registry.aliases():
                 continue
-            if bzrdir.format_registry.get_info(key).hidden:
+            if controldir.format_registry.get_info(key).hidden:
                 continue
             expected = None
             if key in ('dirstate', 'knit', 'dirstate-tags'):
@@ -186,41 +186,47 @@
                 expected = '1.14-rich-root'
             self.assertRepoDescription(key, expected)
 
-        format = bzrdir.format_registry.make_bzrdir('knit')
+        format = controldir.format_registry.make_bzrdir('knit')
         format.set_branch_format(_mod_branch.BzrBranchFormat6())
         tree = self.make_branch_and_tree('unknown', format=format)
         self.assertEqual('unnamed', info.describe_format(tree.bzrdir,
             tree.branch.repository, tree.branch, tree))
 
+    def test_gather_location_controldir_only(self):
+        bzrdir = self.make_bzrdir('.')
+        self.assertEqual([('control directory', bzrdir.user_url)],
+            info.gather_location_info(control=bzrdir))
+
     def test_gather_location_standalone(self):
         tree = self.make_branch_and_tree('tree')
         self.assertEqual([('branch root', tree.bzrdir.root_transport.base)],
-            info.gather_location_info(tree.branch.repository, tree.branch,
-                                      tree))
+            info.gather_location_info(
+                tree.branch.repository, tree.branch, tree, control=tree.bzrdir))
         self.assertEqual([('branch root', tree.bzrdir.root_transport.base)],
-            info.gather_location_info(tree.branch.repository, tree.branch))
+            info.gather_location_info(
+                tree.branch.repository, tree.branch, control=tree.bzrdir))
         return tree
 
     def test_gather_location_repo(self):
         srepo = self.make_repository('shared', shared=True)
-        self.assertEqual([('shared repository',
-                          srepo.bzrdir.root_transport.base)],
-                          info.gather_location_info(srepo))
+        self.assertEqual(
+            [('shared repository', srepo.bzrdir.root_transport.base)],
+            info.gather_location_info(srepo, control=srepo.bzrdir))
         urepo = self.make_repository('unshared')
-        self.assertEqual([('repository',
-                          urepo.bzrdir.root_transport.base)],
-                          info.gather_location_info(urepo))
+        self.assertEqual(
+            [('repository', urepo.bzrdir.root_transport.base)],
+            info.gather_location_info(urepo, control=urepo.bzrdir))
 
     def test_gather_location_repo_branch(self):
         srepo = self.make_repository('shared', shared=True)
-        self.assertEqual([('shared repository',
-                          srepo.bzrdir.root_transport.base)],
-                          info.gather_location_info(srepo))
+        self.assertEqual(
+            [('shared repository', srepo.bzrdir.root_transport.base)],
+            info.gather_location_info(srepo, control=srepo.bzrdir))
         tree = self.make_branch_and_tree('shared/tree')
-        self.assertEqual([('shared repository',
-                          srepo.bzrdir.root_transport.base),
-                          ('repository branch', tree.branch.base)],
-                          info.gather_location_info(srepo, tree.branch, tree))
+        self.assertEqual(
+            [('shared repository', srepo.bzrdir.root_transport.base),
+             ('repository branch', tree.branch.base)],
+            info.gather_location_info(srepo, tree.branch, tree, srepo.bzrdir))
 
     def test_gather_location_light_checkout(self):
         tree = self.make_branch_and_tree('tree')
@@ -259,8 +265,8 @@
              self.gather_tree_location_info(shared_checkout))
 
     def gather_tree_location_info(self, tree):
-        return info.gather_location_info(tree.branch.repository, tree.branch,
-                                         tree)
+        return info.gather_location_info(
+            tree.branch.repository, tree.branch, tree, tree.bzrdir)
 
     def test_gather_location_bound(self):
         branch = self.make_branch('branch')
@@ -269,6 +275,21 @@
         self.assertEqual(
             [('branch root', bound_branch.bzrdir.root_transport.base),
              ('bound to branch', branch.bzrdir.root_transport.base)],
+            info.gather_location_info(
+                bound_branch.repository, bound_branch, control=bound_branch.bzrdir)
+        )
+
+    def test_gather_location_bound_in_repository(self):
+        repo = self.make_repository('repo', shared=True)
+        repo.set_make_working_trees(False)
+        branch = self.make_branch('branch')
+        bound_branch = controldir.ControlDir.create_branch_convenience(
+            'repo/bound_branch')
+        bound_branch.bind(branch)
+        self.assertEqual(
+            [('shared repository', bound_branch.repository.bzrdir.user_url),
+             ('repository branch', bound_branch.bzrdir.user_url),
+             ('bound to branch', branch.bzrdir.user_url)],
             info.gather_location_info(bound_branch.repository, bound_branch)
         )
 

=== modified file 'bzrlib/tests/test_lazy_import.py'
--- bzrlib/tests/test_lazy_import.py	2012-01-05 09:46:10 +0000
+++ bzrlib/tests/test_lazy_import.py	2012-01-05 09:50:04 +0000
@@ -467,9 +467,9 @@
         self.addCleanup(sys.path.remove, base_path)
 
         original_import = __import__
-        def instrumented_import(mod, scope1, scope2, fromlist):
-            self.actions.append(('import', mod, fromlist))
-            return original_import(mod, scope1, scope2, fromlist)
+        def instrumented_import(mod, scope1, scope2, fromlist, level):
+            self.actions.append(('import', mod, fromlist, level))
+            return original_import(mod, scope1, scope2, fromlist, level)
         def cleanup():
             __builtins__['__import__'] = original_import
         self.addCleanup(cleanup)
@@ -545,18 +545,18 @@
         """Test that a real import of these modules works"""
         sub_mod_path = '.'.join([self.root_name, self.sub_name,
                                   self.submoda_name])
-        root = __import__(sub_mod_path, globals(), locals(), [])
+        root = __import__(sub_mod_path, globals(), locals(), [], 0)
         self.assertEqual(1, root.var1)
         self.assertEqual(3, getattr(root, self.sub_name).var3)
         self.assertEqual(4, getattr(getattr(root, self.sub_name),
                                     self.submoda_name).var4)
 
         mod_path = '.'.join([self.root_name, self.mod_name])
-        root = __import__(mod_path, globals(), locals(), [])
+        root = __import__(mod_path, globals(), locals(), [], 0)
         self.assertEqual(2, getattr(root, self.mod_name).var2)
 
-        self.assertEqual([('import', sub_mod_path, []),
-                          ('import', mod_path, []),
+        self.assertEqual([('import', sub_mod_path, [], 0),
+                          ('import', mod_path, [], 0),
                          ], self.actions)
 
 
@@ -584,7 +584,7 @@
 
         self.assertEqual([('__getattribute__', 'var1'),
                           ('_import', 'root1'),
-                          ('import', self.root_name, []),
+                          ('import', self.root_name, [], 0),
                          ], self.actions)
 
     def test_import_mod(self):
@@ -609,7 +609,7 @@
 
         self.assertEqual([('__getattribute__', 'var2'),
                           ('_import', 'mod1'),
-                          ('import', mod_path, []),
+                          ('import', mod_path, [], 0),
                          ], self.actions)
 
     def test_import_mod_from_root(self):
@@ -633,7 +633,7 @@
 
         self.assertEqual([('__getattribute__', 'var2'),
                           ('_import', 'mod2'),
-                          ('import', self.root_name, [self.mod_name]),
+                          ('import', self.root_name, [self.mod_name], 0),
                          ], self.actions)
 
     def test_import_root_and_mod(self):
@@ -665,10 +665,10 @@
         mod_path = self.root_name + '.' + self.mod_name
         self.assertEqual([('__getattribute__', 'var1'),
                           ('_import', 'root3'),
-                          ('import', self.root_name, []),
+                          ('import', self.root_name, [], 0),
                           ('__getattribute__', 'var2'),
                           ('_import', 'mod3'),
-                          ('import', mod_path, []),
+                          ('import', mod_path, [], 0),
                          ], self.actions)
 
     def test_import_root_and_root_mod(self):
@@ -707,10 +707,10 @@
         mod_path = self.root_name + '.' + self.mod_name
         self.assertEqual([('__getattribute__', 'mod4'),
                           ('_import', 'root4'),
-                          ('import', self.root_name, []),
+                          ('import', self.root_name, [], 0),
                           ('__getattribute__', 'var2'),
                           ('_import', 'mod4'),
-                          ('import', mod_path, []),
+                          ('import', mod_path, [], 0),
                          ], self.actions)
 
     def test_import_root_sub_submod(self):
@@ -770,19 +770,19 @@
 
         self.assertEqual([('__getattribute__', 'mod5'),
                           ('_import', 'root5'),
-                          ('import', self.root_name, []),
+                          ('import', self.root_name, [], 0),
                           ('__getattribute__', 'submoda5'),
                           ('_import', 'sub5'),
-                          ('import', sub_path, []),
+                          ('import', sub_path, [], 0),
                           ('__getattribute__', 'var2'),
                           ('_import', 'mod5'),
-                          ('import', mod_path, []),
+                          ('import', mod_path, [], 0),
                           ('__getattribute__', 'var4'),
                           ('_import', 'submoda5'),
-                          ('import', submoda_path, []),
+                          ('import', submoda_path, [], 0),
                           ('__getattribute__', 'var5'),
                           ('_import', 'submodb5'),
-                          ('import', submodb_path, []),
+                          ('import', submodb_path, [], 0),
                          ], self.actions)
 
 
@@ -1077,7 +1077,7 @@
 
         self.assertEqual([('__getattribute__', 'var1'),
                           ('_import', 'root6'),
-                          ('import', self.root_name, []),
+                          ('import', self.root_name, [], 0),
                          ], self.actions)
 
     def test_import_deep(self):
@@ -1112,7 +1112,7 @@
 
         self.assertEqual([('__getattribute__', 'var4'),
                           ('_import', 'submoda7'),
-                          ('import', submoda_path, []),
+                          ('import', submoda_path, [], 0),
                          ], self.actions)
 
     def test_lazy_import(self):
@@ -1136,7 +1136,7 @@
 
         self.assertEqual([('__getattribute__', 'var1'),
                           ('_import', 'root8'),
-                          ('import', self.root_name, []),
+                          ('import', self.root_name, [], 0),
                          ], self.actions)
 
 

=== modified file 'bzrlib/tests/test_lazy_regex.py'
--- bzrlib/tests/test_lazy_regex.py	2011-06-28 22:39:41 +0000
+++ bzrlib/tests/test_lazy_regex.py	2011-11-21 16:02:13 +0000
@@ -16,6 +16,7 @@
 
 """Test that lazy regexes are not compiled right away"""
 
+import pickle
 import re
 
 from bzrlib import errors
@@ -115,6 +116,16 @@
         pattern = lazy_regex.lazy_compile('[,;]*')
         self.assertEqual(['x', 'y', 'z'], pattern.split('x,y;z'))
 
+    def test_pickle(self):
+        # When pickling, just compile the regex.
+        # Sphinx, which we use for documentation, pickles
+        # some compiled regexes.
+        lazy_pattern = lazy_regex.lazy_compile('[,;]*')
+        pickled = pickle.dumps(lazy_pattern)
+        unpickled_lazy_pattern = pickle.loads(pickled)
+        self.assertEqual(['x', 'y', 'z'],
+            unpickled_lazy_pattern.split('x,y;z'))
+
 
 class TestInstallLazyCompile(tests.TestCase):
     """Tests for lazy compiled regexps.

=== modified file 'bzrlib/tests/test_lockdir.py'
--- bzrlib/tests/test_lockdir.py	2011-08-19 22:34:02 +0000
+++ bzrlib/tests/test_lockdir.py	2011-12-14 18:27:44 +0000
@@ -43,6 +43,7 @@
 from bzrlib.tests import (
     features,
     TestCase,
+    TestCaseInTempDir,
     TestCaseWithTransport,
     )
 
@@ -654,7 +655,7 @@
         self.assertEqual([], self._calls)
 
 
-class TestLockHeldInfo(TestCase):
+class TestLockHeldInfo(TestCaseInTempDir):
     """Can get information about the lock holder, and detect whether they're
     still alive."""
 

=== modified file 'bzrlib/tests/test_log.py'
--- bzrlib/tests/test_log.py	2011-07-25 07:41:52 +0000
+++ bzrlib/tests/test_log.py	2011-12-15 14:47:22 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005-2010 Canonical Ltd
+# Copyright (C) 2005-2011 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,7 +24,6 @@
     registry,
     revision,
     revisionspec,
-    symbol_versioning,
     tests,
     )
 
@@ -323,7 +322,7 @@
     1 Joe Foo\t2005-11-22
       rev-1
 
-Use --include-merges or -n0 to see merged revisions.
+Use --include-merged or -n0 to see merged revisions.
 """,
             wt.branch, log.ShortLogFormatter,
             formatter_kwargs=dict(show_advice=True))
@@ -1335,7 +1334,7 @@
         self.assertFormatterResult("""\
 ------------------------------------------------------------
 revno: 2
-fixes bug(s): test://bug/id test://bug/2
+fixes bugs: test://bug/id test://bug/2
 author: Joe Bar <joe@bar.com>
 committer: Joe Foo <joe@foo.com>
 branch nick: work
@@ -1346,7 +1345,7 @@
   message
 ------------------------------------------------------------
 revno: 1
-fixes bug(s): test://bug/id
+fixes bug: test://bug/id
 committer: Joe Foo <joe@foo.com>
 branch nick: work
 timestamp: Tue 2005-11-22 00:00:00 +0000
@@ -1359,13 +1358,13 @@
         tree = self.make_commits_with_bugs()
         self.assertFormatterResult("""\
     2 Joe Bar\t2005-11-22
-      fixes bug(s): test://bug/id test://bug/2
+      fixes bugs: test://bug/id test://bug/2
       multiline
       log
       message
 
     1 Joe Foo\t2005-11-22
-      fixes bug(s): test://bug/id
+      fixes bug: test://bug/id
       simple log message
 
 """,
@@ -1570,7 +1569,7 @@
     def assertLogRevnos(self, expected_revnos, b, start, end,
                         exclude_common_ancestry, generate_merge_revisions=True):
         # FIXME: the layering in log makes it hard to test intermediate levels,
-        # I wish adding filters with their parameters were easier...
+        # I wish adding filters with their parameters was easier...
         # -- vila 20100413
         iter_revs = log._calc_view_revisions(
             b, start, end, direction='reverse',
@@ -1636,3 +1635,4 @@
         log.Logger(b, request).show(log_formatter)
         # should now only have 2 revisions:
         self.assertEquals(len(log_formatter.revisions), 2)
+

=== modified file 'bzrlib/tests/test_lru_cache.py'
--- bzrlib/tests/test_lru_cache.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/test_lru_cache.py	2011-10-14 18:42:17 +0000
@@ -18,10 +18,43 @@
 
 from bzrlib import (
     lru_cache,
+    symbol_versioning,
     tests,
     )
 
 
+def walk_lru(lru):
+    """Test helper to walk the LRU list and assert its consistency"""
+    node = lru._most_recently_used
+    if node is not None:
+        if node.prev is not None:
+            raise AssertionError('the _most_recently_used entry is not'
+                                 ' supposed to have a previous entry'
+                                 ' %s' % (node,))
+    while node is not None:
+        if node.next_key is lru_cache._null_key:
+            if node is not lru._least_recently_used:
+                raise AssertionError('only the last node should have'
+                                     ' no next value: %s' % (node,))
+            node_next = None
+        else:
+            node_next = lru._cache[node.next_key]
+            if node_next.prev is not node:
+                raise AssertionError('inconsistency found, node.next.prev'
+                                     ' != node: %s' % (node,))
+        if node.prev is None:
+            if node is not lru._most_recently_used:
+                raise AssertionError('only the _most_recently_used should'
+                                     ' not have a previous node: %s'
+                                     % (node,))
+        else:
+            if node.prev.next_key != node.key:
+                raise AssertionError('inconsistency found, node.prev.next'
+                                     ' != node: %s' % (node,))
+        yield node
+        node = node_next
+
+ 
 class TestLRUCache(tests.TestCase):
     """Test that LRU cache properly keeps track of entries."""
 
@@ -61,11 +94,12 @@
         cache[None]
         cache[1]
         cache[None]
-        self.assertEqual([None, 1], [n.key for n in cache._walk_lru()])
+        self.assertEqual([None, 1], [n.key for n in walk_lru(cache)])
 
     def test_add__null_key(self):
         cache = lru_cache.LRUCache(max_cache=10)
-        self.assertRaises(ValueError, cache.add, lru_cache._null_key, 1)
+        self.assertRaises(ValueError,
+            cache.__setitem__, lru_cache._null_key, 1)
 
     def test_overflow(self):
         """Adding extra entries will pop out old ones."""
@@ -94,80 +128,12 @@
 
         self.assertFalse('foo' in cache)
 
-    def test_cleanup(self):
-        """Test that we can use a cleanup function."""
-        cleanup_called = []
-        def cleanup_func(key, val):
-            cleanup_called.append((key, val))
-
-        cache = lru_cache.LRUCache(max_cache=2)
-
-        cache.add('baz', '1', cleanup=cleanup_func)
-        cache.add('foo', '2', cleanup=cleanup_func)
-        cache.add('biz', '3', cleanup=cleanup_func)
-
-        self.assertEqual([('baz', '1')], cleanup_called)
-
-        # 'foo' is now most recent, so final cleanup will call it last
-        cache['foo']
-        cache.clear()
-        self.assertEqual([('baz', '1'), ('biz', '3'), ('foo', '2')],
-                         cleanup_called)
-
-    def test_cleanup_on_replace(self):
-        """Replacing an object should cleanup the old value."""
-        cleanup_called = []
-        def cleanup_func(key, val):
-            cleanup_called.append((key, val))
-
-        cache = lru_cache.LRUCache(max_cache=2)
-        cache.add(1, 10, cleanup=cleanup_func)
-        cache.add(2, 20, cleanup=cleanup_func)
-        cache.add(2, 25, cleanup=cleanup_func)
-
-        self.assertEqual([(2, 20)], cleanup_called)
-        self.assertEqual(25, cache[2])
-
-        # Even __setitem__ should make sure cleanup() is called
-        cache[2] = 26
-        self.assertEqual([(2, 20), (2, 25)], cleanup_called)
-
-    def test_cleanup_error_maintains_linked_list(self):
-        cleanup_called = []
-        def cleanup_func(key, val):
-            cleanup_called.append((key, val))
-            raise ValueError('failure during cleanup')
-
-        cache = lru_cache.LRUCache(max_cache=10)
-        for i in xrange(10):
-            cache.add(i, i, cleanup=cleanup_func)
-        for i in xrange(10, 20):
-            self.assertRaises(ValueError,
-                cache.add, i, i, cleanup=cleanup_func)
-
-        self.assertEqual([(i, i) for i in xrange(10)], cleanup_called)
-
-        self.assertEqual(range(19, 9, -1), [n.key for n in cache._walk_lru()])
-
-    def test_cleanup_during_replace_still_replaces(self):
-        cleanup_called = []
-        def cleanup_func(key, val):
-            cleanup_called.append((key, val))
-            raise ValueError('failure during cleanup')
-
-        cache = lru_cache.LRUCache(max_cache=10)
-        for i in xrange(10):
-            cache.add(i, i, cleanup=cleanup_func)
-        self.assertRaises(ValueError,
-            cache.add, 1, 20, cleanup=cleanup_func)
-        # We also still update the recent access to this node
-        self.assertEqual([1, 9, 8, 7, 6, 5, 4, 3, 2, 0],
-                         [n.key for n in cache._walk_lru()])
-        self.assertEqual(20, cache[1])
-
-        self.assertEqual([(1, 1)], cleanup_called)
-        self.assertEqual([1, 9, 8, 7, 6, 5, 4, 3, 2, 0],
-                         [n.key for n in cache._walk_lru()])
+    def test_cleanup_function_deprecated(self):
+        """Test that per-node cleanup functions are no longer allowed"""
+        cache = lru_cache.LRUCache()
+        self.assertRaises(ValueError, self.applyDeprecated,
+            symbol_versioning.deprecated_in((2, 5, 0)),
+            cache.add, "key", 1, cleanup=lambda: None)
 
     def test_len(self):
         cache = lru_cache.LRUCache(max_cache=10, after_cleanup_count=10)
@@ -197,21 +163,21 @@
         # We hit the max
         self.assertEqual(10, len(cache))
         self.assertEqual([11, 10, 9, 1, 8, 7, 6, 5, 4, 3],
-                         [n.key for n in cache._walk_lru()])
+                         [n.key for n in walk_lru(cache)])
 
     def test_cleanup_shrinks_to_after_clean_count(self):
         cache = lru_cache.LRUCache(max_cache=5, after_cleanup_count=3)
 
-        cache.add(1, 10)
-        cache.add(2, 20)
-        cache.add(3, 25)
-        cache.add(4, 30)
-        cache.add(5, 35)
+        cache[1] = 10
+        cache[2] = 20
+        cache[3] = 25
+        cache[4] = 30
+        cache[5] = 35
 
         self.assertEqual(5, len(cache))
         # This will bump us over the max, which causes us to shrink down to
         # after_cleanup_cache size
-        cache.add(6, 40)
+        cache[6] = 40
         self.assertEqual(3, len(cache))
 
     def test_after_cleanup_larger_than_max(self):
@@ -227,11 +193,11 @@
         cache = lru_cache.LRUCache(max_cache=5, after_cleanup_count=2)
 
         # Add these in order
-        cache.add(1, 10)
-        cache.add(2, 20)
-        cache.add(3, 25)
-        cache.add(4, 30)
-        cache.add(5, 35)
+        cache[1] = 10
+        cache[2] = 20
+        cache[3] = 25
+        cache[4] = 30
+        cache[5] = 35
 
         self.assertEqual(5, len(cache))
         # Force a compaction
@@ -242,33 +208,33 @@
         cache = lru_cache.LRUCache(max_cache=5)
 
         # Add these in order
-        cache.add(1, 10)
-        cache.add(2, 20)
-        cache.add(3, 25)
-        cache.add(4, 30)
-        cache.add(5, 35)
+        cache[1] = 10
+        cache[2] = 20
+        cache[3] = 25
+        cache[4] = 30
+        cache[5] = 35
 
-        self.assertEqual([5, 4, 3, 2, 1], [n.key for n in cache._walk_lru()])
+        self.assertEqual([5, 4, 3, 2, 1], [n.key for n in walk_lru(cache)])
 
         # Now access some randomly
         cache[2]
         cache[5]
         cache[3]
         cache[2]
-        self.assertEqual([2, 3, 5, 4, 1], [n.key for n in cache._walk_lru()])
+        self.assertEqual([2, 3, 5, 4, 1], [n.key for n in walk_lru(cache)])
 
     def test_get(self):
         cache = lru_cache.LRUCache(max_cache=5)
 
-        cache.add(1, 10)
-        cache.add(2, 20)
+        cache[1] = 10
+        cache[2] = 20
         self.assertEqual(20, cache.get(2))
         self.assertIs(None, cache.get(3))
         obj = object()
         self.assertIs(obj, cache.get(3, obj))
-        self.assertEqual([2, 1], [n.key for n in cache._walk_lru()])
+        self.assertEqual([2, 1], [n.key for n in walk_lru(cache)])
         self.assertEqual(10, cache.get(1))
-        self.assertEqual([1, 2], [n.key for n in cache._walk_lru()])
+        self.assertEqual([1, 2], [n.key for n in walk_lru(cache)])
 
     def test_keys(self):
         cache = lru_cache.LRUCache(max_cache=5, after_cleanup_count=5)
@@ -332,18 +298,19 @@
 
     def test_add__null_key(self):
         cache = lru_cache.LRUSizeCache()
-        self.assertRaises(ValueError, cache.add, lru_cache._null_key, 1)
+        self.assertRaises(ValueError,
+            cache.__setitem__, lru_cache._null_key, 1)
 
     def test_add_tracks_size(self):
         cache = lru_cache.LRUSizeCache()
         self.assertEqual(0, cache._value_size)
-        cache.add('my key', 'my value text')
+        cache['my key'] = 'my value text'
         self.assertEqual(13, cache._value_size)
 
     def test_remove_tracks_size(self):
         cache = lru_cache.LRUSizeCache()
         self.assertEqual(0, cache._value_size)
-        cache.add('my key', 'my value text')
+        cache['my key'] = 'my value text'
         self.assertEqual(13, cache._value_size)
         node = cache._cache['my key']
         cache._remove_node(node)
@@ -353,64 +320,48 @@
         """Adding a large value may not be cached at all."""
         cache = lru_cache.LRUSizeCache(max_size=10, after_cleanup_size=5)
         self.assertEqual(0, cache._value_size)
-        self.assertEqual({}, cache.items())
-        cache.add('test', 'key')
-        self.assertEqual(3, cache._value_size)
-        self.assertEqual({'test': 'key'}, cache.items())
-        cache.add('test2', 'key that is too big')
-        self.assertEqual(3, cache._value_size)
-        self.assertEqual({'test':'key'}, cache.items())
+        self.assertEqual({}, cache.as_dict())
+        cache['test'] = 'key'
+        self.assertEqual(3, cache._value_size)
+        self.assertEqual({'test': 'key'}, cache.as_dict())
+        cache['test2'] = 'key that is too big'
+        self.assertEqual(3, cache._value_size)
+        self.assertEqual({'test':'key'}, cache.as_dict())
         # If we would add a key, only to cleanup and remove all cached entries,
         # then obviously that value should not be stored
-        cache.add('test3', 'bigkey')
-        self.assertEqual(3, cache._value_size)
-        self.assertEqual({'test':'key'}, cache.items())
-
-        cache.add('test4', 'bikey')
-        self.assertEqual(3, cache._value_size)
-        self.assertEqual({'test':'key'}, cache.items())
-
-    def test_no_add_over_size_cleanup(self):
-        """If a large value is not cached, we will call cleanup right away."""
-        cleanup_calls = []
-        def cleanup(key, value):
-            cleanup_calls.append((key, value))
-
-        cache = lru_cache.LRUSizeCache(max_size=10, after_cleanup_size=5)
-        self.assertEqual(0, cache._value_size)
-        self.assertEqual({}, cache.items())
-        cache.add('test', 'key that is too big', cleanup=cleanup)
-        # key was not added
-        self.assertEqual(0, cache._value_size)
-        self.assertEqual({}, cache.items())
-        # and cleanup was called
-        self.assertEqual([('test', 'key that is too big')], cleanup_calls)
+        cache['test3'] = 'bigkey'
+        self.assertEqual(3, cache._value_size)
+        self.assertEqual({'test':'key'}, cache.as_dict())
+
+        cache['test4'] = 'bikey'
+        self.assertEqual(3, cache._value_size)
+        self.assertEqual({'test':'key'}, cache.as_dict())
 
     def test_adding_clears_cache_based_on_size(self):
         """The cache is cleared in LRU order until small enough"""
         cache = lru_cache.LRUSizeCache(max_size=20)
-        cache.add('key1', 'value') # 5 chars
-        cache.add('key2', 'value2') # 6 chars
-        cache.add('key3', 'value23') # 7 chars
+        cache['key1'] = 'value' # 5 chars
+        cache['key2'] = 'value2' # 6 chars
+        cache['key3'] = 'value23' # 7 chars
         self.assertEqual(5+6+7, cache._value_size)
         cache['key2'] # reference key2 so it gets a newer reference time
-        cache.add('key4', 'value234') # 8 chars, over limit
+        cache['key4'] = 'value234' # 8 chars, over limit
         # We have to remove 2 keys to get back under limit
         self.assertEqual(6+8, cache._value_size)
         self.assertEqual({'key2':'value2', 'key4':'value234'},
-                         cache.items())
+                         cache.as_dict())
 
     def test_adding_clears_to_after_cleanup_size(self):
         cache = lru_cache.LRUSizeCache(max_size=20, after_cleanup_size=10)
-        cache.add('key1', 'value') # 5 chars
-        cache.add('key2', 'value2') # 6 chars
-        cache.add('key3', 'value23') # 7 chars
+        cache['key1'] = 'value' # 5 chars
+        cache['key2'] = 'value2' # 6 chars
+        cache['key3'] = 'value23' # 7 chars
         self.assertEqual(5+6+7, cache._value_size)
         cache['key2'] # reference key2 so it gets a newer reference time
-        cache.add('key4', 'value234') # 8 chars, over limit
+        cache['key4'] = 'value234' # 8 chars, over limit
         # We have to remove 3 keys to get back under limit
         self.assertEqual(8, cache._value_size)
-        self.assertEqual({'key4':'value234'}, cache.items())
+        self.assertEqual({'key4':'value234'}, cache.as_dict())
 
     def test_custom_sizes(self):
         def size_of_list(lst):
@@ -418,23 +369,23 @@
         cache = lru_cache.LRUSizeCache(max_size=20, after_cleanup_size=10,
                                        compute_size=size_of_list)
 
-        cache.add('key1', ['val', 'ue']) # 5 chars
-        cache.add('key2', ['val', 'ue2']) # 6 chars
-        cache.add('key3', ['val', 'ue23']) # 7 chars
+        cache['key1'] = ['val', 'ue'] # 5 chars
+        cache['key2'] = ['val', 'ue2'] # 6 chars
+        cache['key3'] = ['val', 'ue23'] # 7 chars
         self.assertEqual(5+6+7, cache._value_size)
         cache['key2'] # reference key2 so it gets a newer reference time
-        cache.add('key4', ['value', '234']) # 8 chars, over limit
+        cache['key4'] = ['value', '234'] # 8 chars, over limit
         # We have to remove 3 keys to get back under limit
         self.assertEqual(8, cache._value_size)
-        self.assertEqual({'key4':['value', '234']}, cache.items())
+        self.assertEqual({'key4':['value', '234']}, cache.as_dict())
 
     def test_cleanup(self):
         cache = lru_cache.LRUSizeCache(max_size=20, after_cleanup_size=10)
 
         # Add these in order
-        cache.add('key1', 'value') # 5 chars
-        cache.add('key2', 'value2') # 6 chars
-        cache.add('key3', 'value23') # 7 chars
+        cache['key1'] = 'value' # 5 chars
+        cache['key2'] = 'value2' # 6 chars
+        cache['key3'] = 'value23' # 7 chars
         self.assertEqual(5+6+7, cache._value_size)
 
         cache.cleanup()

=== modified file 'bzrlib/tests/test_mail_client.py'
--- bzrlib/tests/test_mail_client.py	2009-09-02 08:26:27 +0000
+++ bzrlib/tests/test_mail_client.py	2011-12-18 15:43:47 +0000
@@ -14,8 +14,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-import urllib
-
 from bzrlib import (
     errors,
     mail_client,
@@ -212,7 +210,7 @@
         claws = mail_client.Claws(None)
         cmdline = claws._get_compose_commandline(
             u'jrandom@example.org', u'\xb5cosm of fun!', u'file%')
-        subject_string = urllib.quote(
+        subject_string = urlutils.quote(
             u'\xb5cosm of fun!'.encode(osutils.get_user_encoding(), 'replace'))
         self.assertEqual(
             ['--compose',

=== modified file 'bzrlib/tests/test_matchers.py'
--- bzrlib/tests/test_matchers.py	2011-08-16 13:26:27 +0000
+++ bzrlib/tests/test_matchers.py	2011-12-14 12:15:44 +0000
@@ -18,7 +18,10 @@
 
 from testtools.matchers import *
 
+from bzrlib.smart.client import CallHookParams
+
 from bzrlib.tests import (
+    CapturedCall,
     TestCase,
     TestCaseWithTransport,
     )
@@ -116,11 +119,11 @@
         t = self.make_branch_and_tree('.')
         self.build_tree(['a', 'b/', 'b/c'])
         t.add(['a', 'b', 'b/c'], ['a-id', 'b-id', 'c-id'])
-        self.assertThat(t, HasLayout(['', 'a', 'b', 'b/c']))
+        self.assertThat(t, HasLayout(['', 'a', 'b/', 'b/c']))
         self.assertThat(t, HasLayout(
             [('', t.get_root_id()),
              ('a', 'a-id'),
-             ('b', 'b-id'),
+             ('b/', 'b-id'),
              ('b/c', 'c-id')]))
 
     def test_mismatch(self):
@@ -130,5 +133,48 @@
         mismatch = HasLayout(['a']).match(t)
         self.assertIsNot(None, mismatch)
         self.assertEquals(
-            "[u'', u'a', u'b', u'b/c'] != ['a']",
-            mismatch.describe())
+            "['a'] != [u'', u'a', u'b/', u'b/c']",
+            mismatch.describe())
+
+    def test_no_dirs(self):
+        # Some tree/repository formats do not support versioned directories
+        t = self.make_branch_and_tree('.')
+        t.has_versioned_directories = lambda: False
+        self.build_tree(['a', 'b/', 'b/c'])
+        t.add(['a', 'b', 'b/c'], ['a-id', 'b-id', 'c-id'])
+        self.assertIs(None, HasLayout(['', 'a', 'b/', 'b/c']).match(t))
+        self.assertIs(None, HasLayout(['', 'a', 'b/', 'b/c', 'd/']).match(t))
+        mismatch = HasLayout([u'', u'a', u'd/']).match(t)
+        self.assertIsNot(None, mismatch)
+        self.assertEquals(
+            "[u'', u'a'] != [u'', u'a', u'b/', u'b/c']",
+            mismatch.describe())
+
+
+class TestContainsNoVfsCalls(TestCase):
+
+    def _make_call(self, method, args):
+        return CapturedCall(CallHookParams(method, args, None, None, None), 0)
+
+    def test__str__(self):
+        self.assertEqual("ContainsNoVfsCalls()", str(ContainsNoVfsCalls()))
+
+    def test_empty(self):
+        self.assertIs(None, ContainsNoVfsCalls().match([]))
+
+    def test_no_vfs_calls(self):
+        calls = [self._make_call("Branch.get_config_file", [])]
+        self.assertIs(None, ContainsNoVfsCalls().match(calls))
+
+    def test_ignores_unknown(self):
+        calls = [self._make_call("unknown", [])]
+        self.assertIs(None, ContainsNoVfsCalls().match(calls))
+
+    def test_match(self):
+        calls = [self._make_call("append", ["file"]),
+                 self._make_call("Branch.get_config_file", [])]
+        mismatch = ContainsNoVfsCalls().match(calls)
+        self.assertIsNot(None, mismatch)
+        self.assertEquals([calls[0].call], mismatch.vfs_calls)
+        self.assertEquals("no VFS calls expected, got: append('file')""",
+                mismatch.describe())

=== modified file 'bzrlib/tests/test_merge.py'
--- bzrlib/tests/test_merge.py	2011-07-21 06:46:34 +0000
+++ bzrlib/tests/test_merge.py	2012-01-02 17:59:40 +0000
@@ -420,8 +420,7 @@
         self.addCleanup(tree.unlock)
         self.build_tree(['a'])
         tree.add('a')
-        tree.commit("added a")
-        first_rev = tree.branch.revision_history()[0]
+        first_rev = tree.commit("added a")
         merger = _mod_merge.Merger.from_revision_ids(None, tree,
                                           _mod_revision.NULL_REVISION,
                                           first_rev)
@@ -447,8 +446,11 @@
         merger.merge_type = _mod_merge.Merge3Merger
         tree_merger = merger.make_merger()
         self.assertIs(_mod_merge.Merge3Merger, tree_merger.__class__)
-        self.assertEqual('rev2b', tree_merger.other_tree.get_revision_id())
-        self.assertEqual('rev1', tree_merger.base_tree.get_revision_id())
+        self.assertEqual('rev2b',
+            tree_merger.other_tree.get_revision_id())
+        self.assertEqual('rev1',
+            tree_merger.base_tree.get_revision_id())
+        self.assertEqual(other_tree.branch, tree_merger.other_branch)
 
     def test_make_preview_transform(self):
         this_tree = self.make_branch_and_tree('this')
@@ -509,9 +511,8 @@
         self.addCleanup(tree.unlock)
         self.build_tree(['a'])
         tree.add('a')
-        tree.commit("added a")
+        first_rev = tree.commit("added a")
         old_root_id = tree.get_root_id()
-        first_rev = tree.branch.revision_history()[0]
         merger = _mod_merge.Merger.from_revision_ids(None, tree,
                                           _mod_revision.NULL_REVISION,
                                           first_rev)
@@ -3244,3 +3245,46 @@
         # The dest tree is unmodified.
         self.assertEqual(['r1-dest'], dest_wt.get_parent_ids())
         self.assertTreeEntriesEqual([('', 'dest-root-id')], dest_wt)
+
+
+class TestMergeHooks(TestCaseWithTransport):
+
+    def setUp(self):
+        super(TestMergeHooks, self).setUp()
+        self.tree_a = self.make_branch_and_tree('tree_a')
+        self.build_tree_contents([('tree_a/file', 'content_1')])
+        self.tree_a.add('file', 'file-id')
+        self.tree_a.commit('added file')
+
+        self.tree_b = self.tree_a.bzrdir.sprout('tree_b').open_workingtree()
+        self.build_tree_contents([('tree_b/file', 'content_2')])
+        self.tree_b.commit('modify file')
+
+    def test_pre_merge_hook_inject_different_tree(self):
+        tree_c = self.tree_b.bzrdir.sprout('tree_c').open_workingtree()
+        self.build_tree_contents([('tree_c/file', 'content_3')])
+        tree_c.commit("more content")
+        calls = []
+        def factory(merger):
+            self.assertIsInstance(merger, _mod_merge.Merge3Merger)
+            merger.other_tree = tree_c
+            calls.append(merger)
+        _mod_merge.Merger.hooks.install_named_hook('pre_merge',
+                                                   factory, 'test factory')
+        self.tree_a.merge_from_branch(self.tree_b.branch)
+
+        self.assertFileEqual("content_3", 'tree_a/file')
+        self.assertLength(1, calls)
+
+    def test_post_merge_hook_called(self):
+        calls = []
+        def factory(merger):
+            self.assertIsInstance(merger, _mod_merge.Merge3Merger)
+            calls.append(merger)
+        _mod_merge.Merger.hooks.install_named_hook('post_merge',
+                                                   factory, 'test factory')
+
+        self.tree_a.merge_from_branch(self.tree_b.branch)
+
+        self.assertFileEqual("content_2", 'tree_a/file')
+        self.assertLength(1, calls)

=== modified file 'bzrlib/tests/test_merge_core.py'
--- bzrlib/tests/test_merge_core.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/test_merge_core.py	2011-09-26 15:21:01 +0000
@@ -98,7 +98,7 @@
             tt.apply()
             wt.commit('branch commit')
             wt.flush()
-            if len(wt.branch.revision_history()) != 2:
+            if wt.branch.last_revision_info()[0] != 2:
                 raise AssertionError()
         self.this.branch.fetch(self.other.branch)
         other_basis = self.other.branch.basis_tree()

=== modified file 'bzrlib/tests/test_merge_directive.py'
--- bzrlib/tests/test_merge_directive.py	2009-05-11 19:11:14 +0000
+++ bzrlib/tests/test_merge_directive.py	2011-12-09 13:00:30 +0000
@@ -462,7 +462,7 @@
         time = 453
         timezone = 7200
         class FakeBranch(object):
-            def get_config(self):
+            def get_config_stack(self):
                 return self
             def gpg_signing_command(self):
                 return 'loopback'

=== modified file 'bzrlib/tests/test_missing.py'
--- bzrlib/tests/test_missing.py	2011-01-12 01:01:53 +0000
+++ bzrlib/tests/test_missing.py	2011-09-19 08:49:15 +0000
@@ -134,12 +134,12 @@
 class TestFindUnmerged(tests.TestCaseWithTransport):
 
     def assertUnmerged(self, local, remote, local_branch, remote_branch,
-            restrict='all', include_merges=False, backward=False,
+            restrict='all', include_merged=False, backward=False,
             local_revid_range=None, remote_revid_range=None):
         """Check the output of find_unmerged_mainline_revisions"""
         local_extra, remote_extra = missing.find_unmerged(
             local_branch, remote_branch, restrict,
-            include_merges=include_merges, backward=backward,
+            include_merged=include_merged, backward=backward,
             local_revid_range=local_revid_range,
             remote_revid_range=remote_revid_range)
         self.assertEqual(local, local_extra)
@@ -187,7 +187,7 @@
         self.assertUnmerged([], [], tree.branch, tree2.branch,
             local_revid_range=(rev1, rev1))
 
-    def test_include_merges(self):
+    def test_include_merged(self):
         tree = self.make_branch_and_tree('tree')
         rev1 = tree.commit('one', rev_id='rev1')
 
@@ -207,27 +207,27 @@
                                  ('3.1.1', 'rev4', 1), ('3.1.2', 'rev5', 1),
                                  ],
                             tree.branch, tree2.branch,
-                            include_merges=True)
+                            include_merged=True)
 
         self.assertUnmerged([], [('4', 'rev6', 0),
                                  ('3.1.2', 'rev5', 1), ('3.1.1', 'rev4', 1),
                                  ('3', 'rev3', 0), ('2', 'rev2', 0),
                                  ],
                             tree.branch, tree2.branch,
-                            include_merges=True,
+                            include_merged=True,
                             backward=True)
 
         self.assertUnmerged([], [('4', 'rev6', 0)],
             tree.branch, tree2.branch,
-            include_merges=True, remote_revid_range=(rev6, rev6))
+            include_merged=True, remote_revid_range=(rev6, rev6))
 
         self.assertUnmerged([], [('3', 'rev3', 0), ('3.1.1', 'rev4', 1)],
                     tree.branch, tree2.branch,
-                    include_merges=True, remote_revid_range=(rev3, rev4))
+                    include_merged=True, remote_revid_range=(rev3, rev4))
 
         self.assertUnmerged([], [('4', 'rev6', 0), ('3.1.2', 'rev5', 1)],
                     tree.branch, tree2.branch,
-                    include_merges=True, remote_revid_range=(rev5, rev6))
+                    include_merged=True, remote_revid_range=(rev5, rev6))
 
     def test_revision_range(self):
         local = self.make_branch_and_tree('local')

=== modified file 'bzrlib/tests/test_msgeditor.py'
--- bzrlib/tests/test_msgeditor.py	2011-06-14 01:26:41 +0000
+++ bzrlib/tests/test_msgeditor.py	2011-11-16 17:19:13 +0000
@@ -252,9 +252,9 @@
         self.overrideEnv('VISUAL', 'visual')
         self.overrideEnv('EDITOR', 'editor')
 
-        conf = config.GlobalConfig.from_string('editor = config_editor\n',
-                                               save=True)
-
+        conf = config.GlobalStack()
+        conf.store._load_from_string('[DEFAULT]\neditor = config_editor\n')
+        conf.store.save()
         editors = list(msgeditor._get_editor())
         editors = [editor for (editor, cfg_src) in editors]
 

=== modified file 'bzrlib/tests/test_options.py'
--- bzrlib/tests/test_options.py	2011-06-01 07:01:17 +0000
+++ bzrlib/tests/test_options.py	2011-11-24 17:01:07 +0000
@@ -43,15 +43,18 @@
         # XXX: Using cmd_commit makes these tests overly sensitive to changes
         # to cmd_commit, when they are meant to be about option parsing in
         # general.
-        self.assertEqual(parse_args(cmd_commit(), ['--help']),
-           ([], {'author': [], 'exclude': [], 'fixes': [], 'help': True}))
-        self.assertEqual(parse_args(cmd_commit(), ['--message=biter']),
-           ([], {'author': [], 'exclude': [], 'fixes': [], 'message': 'biter'}))
+        self.assertEqual(
+           ([], {'author': [], 'exclude': [], 'fixes': [], 'help': True}),
+           parse_args(cmd_commit(), ['--help']))
+        self.assertEqual(
+           ([], {'author': [], 'exclude': [], 'fixes': [], 'message': 'biter'}),
+           parse_args(cmd_commit(), ['--message=biter']))
 
     def test_no_more_opts(self):
         """Terminated options"""
-        self.assertEqual(parse_args(cmd_commit(), ['--', '-file-with-dashes']),
-                          (['-file-with-dashes'], {'author': [], 'exclude': [], 'fixes': []}))
+        self.assertEqual(
+            (['-file-with-dashes'], {'author': [], 'exclude': [], 'fixes': []}),
+            parse_args(cmd_commit(), ['--', '-file-with-dashes']))
 
     def test_option_help(self):
         """Options have help strings."""
@@ -65,6 +68,11 @@
         out, err = self.run_bzr('help status')
         self.assertContainsRe(out, r'--show-ids.*Show internal object.')
 
+    def test_option_help_global_hidden(self):
+        """Hidden global options have no help strings."""
+        out, err = self.run_bzr('help log')
+        self.assertNotContainsRe(out, r'--message')
+
     def test_option_arg_help(self):
         """Help message shows option arguments."""
         out, err = self.run_bzr('help commit')
@@ -333,60 +341,40 @@
 
     def get_builtin_command_options(self):
         g = []
-        for cmd_name in sorted(commands.all_command_names()):
+        commands.install_bzr_command_hooks()
+        for cmd_name in sorted(commands.builtin_command_names()):
             cmd = commands.get_cmd_object(cmd_name)
             for opt_name, opt in sorted(cmd.options().items()):
                 g.append((cmd_name, opt))
+        self.assert_(g)
         return g
 
-    def test_global_options_used(self):
-        # In the distant memory, options could only be declared globally.  Now
-        # we prefer to declare them in the command, unless like -r they really
-        # are used very widely with the exact same meaning.  So this checks
-        # for any that should be garbage collected.
-        g = dict(option.Option.OPTIONS.items())
-        used_globals = {}
-        msgs = []
-        for cmd_name in sorted(commands.all_command_names()):
-            cmd = commands.get_cmd_object(cmd_name)
-            for option_or_name in sorted(cmd.takes_options):
-                if not isinstance(option_or_name, basestring):
-                    self.assertIsInstance(option_or_name, option.Option)
-                elif not option_or_name in g:
-                    msgs.append("apparent reference to undefined "
-                        "global option %r from %r"
-                        % (option_or_name, cmd))
-                else:
-                    used_globals.setdefault(option_or_name, []).append(cmd_name)
-        unused_globals = set(g.keys()) - set(used_globals.keys())
-        # not enforced because there might be plugins that use these globals
-        ## for option_name in sorted(unused_globals):
-        ##    msgs.append("unused global option %r" % option_name)
-        ## for option_name, cmds in sorted(used_globals.items()):
-        ##     if len(cmds) <= 1:
-        ##         msgs.append("global option %r is only used by %r"
-        ##                 % (option_name, cmds))
-        if msgs:
-            self.fail("problems with global option definitions:\n"
-                    + '\n'.join(msgs))
-
     def test_option_grammar(self):
         msgs = []
         # Option help should be written in sentence form, and have a final
-        # period and be all on a single line, because the display code will
-        # wrap it.
-        option_re = re.compile(r'^[A-Z][^\n]+\.$')
+        # period with an optional bracketed suffix. All the text should be on
+        # one line, because the display code will wrap it.
+        option_re = re.compile(r'^[A-Z][^\n]+\.(?: \([^\n]+\))?$')
         for scope, opt in self.get_builtin_command_options():
-            if not opt.help:
-                msgs.append('%-16s %-16s %s' %
-                       ((scope or 'GLOBAL'), opt.name, 'NO HELP'))
-            elif not option_re.match(opt.help):
-                msgs.append('%-16s %-16s %s' %
-                        ((scope or 'GLOBAL'), opt.name, opt.help))
+            for name, _, _, helptxt in opt.iter_switches():
+                if name != opt.name:
+                    name = "/".join([opt.name, name])
+                if not helptxt:
+                    msgs.append('%-16s %-16s %s' %
+                           ((scope or 'GLOBAL'), name, 'NO HELP'))
+                elif not option_re.match(helptxt):
+                    if name.startswith("format/"):
+                        # Don't complain about the odd format registry help
+                        continue
+                    msgs.append('%-16s %-16s %s' %
+                            ((scope or 'GLOBAL'), name, helptxt))
         if msgs:
             self.fail("The following options don't match the style guide:\n"
                     + '\n'.join(msgs))
 
+
+class TestOptionMisc(TestCase):
+
     def test_is_hidden(self):
         registry = controldir.ControlDirFormatRegistry()
         bzrdir.register_metadir(registry, 'hidden', 'HiddenFormat',

=== modified file 'bzrlib/tests/test_osutils.py'
--- bzrlib/tests/test_osutils.py	2011-07-08 23:01:39 +0000
+++ bzrlib/tests/test_osutils.py	2012-01-04 11:35:56 +0000
@@ -20,6 +20,7 @@
 import errno
 import os
 import re
+import select
 import socket
 import sys
 import time
@@ -818,6 +819,16 @@
         self.assertEqual(None, osutils.safe_file_id(None))
 
 
+class TestPosixFuncs(tests.TestCase):
+    """Test that the posix version of normpath returns an appropriate path
+       when used with 2 leading slashes."""
+
+    def test_normpath(self):
+        self.assertEqual('/etc/shadow', osutils._posix_normpath('/etc/shadow'))
+        self.assertEqual('/etc/shadow', osutils._posix_normpath('//etc/shadow'))
+        self.assertEqual('/etc/shadow', osutils._posix_normpath('///etc/shadow'))
+
+
 class TestWin32Funcs(tests.TestCase):
     """Test that _win32 versions of os utilities return appropriate paths."""
 
@@ -1206,7 +1217,7 @@
         self.requireFeature(UTF8DirReaderFeature)
         self._save_platform_info()
         win32utils.winver = None # Avoid the win32 detection code
-        osutils._fs_enc = 'UTF-8'
+        osutils._fs_enc = 'utf-8'
         self.assertDirReaderIs(
             UTF8DirReaderFeature.module.UTF8DirReader)
 
@@ -1214,22 +1225,14 @@
         self.requireFeature(UTF8DirReaderFeature)
         self._save_platform_info()
         win32utils.winver = None # Avoid the win32 detection code
-        osutils._fs_enc = 'US-ASCII'
-        self.assertDirReaderIs(
-            UTF8DirReaderFeature.module.UTF8DirReader)
-
-    def test_force_walkdirs_utf8_fs_ANSI(self):
-        self.requireFeature(UTF8DirReaderFeature)
-        self._save_platform_info()
-        win32utils.winver = None # Avoid the win32 detection code
-        osutils._fs_enc = 'ANSI_X3.4-1968'
+        osutils._fs_enc = 'ascii'
         self.assertDirReaderIs(
             UTF8DirReaderFeature.module.UTF8DirReader)
 
     def test_force_walkdirs_utf8_fs_latin1(self):
         self._save_platform_info()
         win32utils.winver = None # Avoid the win32 detection code
-        osutils._fs_enc = 'latin1'
+        osutils._fs_enc = 'iso-8859-1'
         self.assertDirReaderIs(osutils.UnicodeDirReader)
 
     def test_force_walkdirs_utf8_nt(self):
@@ -2094,8 +2097,19 @@
 
 class TestGetuserUnicode(tests.TestCase):
 
+    def test_is_unicode(self):
+        user = osutils.getuser_unicode()
+        self.assertIsInstance(user, unicode)
+
+    def envvar_to_override(self):
+        if sys.platform == "win32":
+            # Disable use of platform calls on windows so envvar is used
+            self.overrideAttr(win32utils, 'has_ctypes', False)
+            return 'USERNAME' # only variable used on windows
+        return 'LOGNAME' # first variable checked by getpass.getuser()
+
     def test_ascii_user(self):
-        self.overrideEnv('LOGNAME', 'jrandom')
+        self.overrideEnv(self.envvar_to_override(), 'jrandom')
         self.assertEqual(u'jrandom', osutils.getuser_unicode())
 
     def test_unicode_user(self):
@@ -2107,16 +2121,8 @@
                 % (osutils.get_user_encoding(),))
         uni_username = u'jrandom' + uni_val
         encoded_username = uni_username.encode(ue)
-        self.overrideEnv('LOGNAME', encoded_username)
+        self.overrideEnv(self.envvar_to_override(), encoded_username)
         self.assertEqual(uni_username, osutils.getuser_unicode())
-        self.overrideEnv('LOGNAME', u'jrandom\xb6'.encode(ue))
-        self.assertEqual(u'jrandom\xb6', osutils.getuser_unicode())
-
-    def test_no_username_bug_660174(self):
-        self.requireFeature(features.win32_feature)
-        for name in ('LOGNAME', 'USER', 'LNAME', 'USERNAME'):
-            self.overrideEnv(name, None)
-        self.assertEqual(u'UNKNOWN', osutils.getuser_unicode())
 
 
 class TestBackupNames(tests.TestCase):
@@ -2169,3 +2175,29 @@
         self.assertTrue(osutils.find_executable_on_path('sh') is not None)
         self.assertTrue(
             osutils.find_executable_on_path('THIS SHOULD NOT EXIST') is None)
+
+
+class TestEnvironmentErrors(tests.TestCase):
+    """Test handling of environmental errors"""
+
+    def test_is_oserror(self):
+        self.assertTrue(osutils.is_environment_error(
+            OSError(errno.EINVAL, "Invalid parameter")))
+
+    def test_is_ioerror(self):
+        self.assertTrue(osutils.is_environment_error(
+            IOError(errno.EINVAL, "Invalid parameter")))
+
+    def test_is_socket_error(self):
+        self.assertTrue(osutils.is_environment_error(
+            socket.error(errno.EINVAL, "Invalid parameter")))
+
+    def test_is_select_error(self):
+        self.assertTrue(osutils.is_environment_error(
+            select.error(errno.EINVAL, "Invalid parameter")))
+
+    def test_is_pywintypes_error(self):
+        self.requireFeature(features.pywintypes)
+        import pywintypes
+        self.assertTrue(osutils.is_environment_error(
+            pywintypes.error(errno.EINVAL, "Invalid parameter", "Caller")))

=== modified file 'bzrlib/tests/test_osutils_encodings.py'
--- bzrlib/tests/test_osutils_encodings.py	2011-01-12 01:01:53 +0000
+++ bzrlib/tests/test_osutils_encodings.py	2011-12-02 13:36:43 +0000
@@ -17,7 +17,9 @@
 """Tests for the osutils wrapper."""
 
 import codecs
+import errno
 import locale
+import os
 import sys
 
 from bzrlib import (
@@ -221,3 +223,17 @@
                           '  doesn\'t support the locale set by $LANG (BOGUS)\n'
                           '  Continuing with ascii encoding.\n',
                           sys.stderr.getvalue())
+
+
+class TestMessageEncoding(TestCase):
+    """Tests for getting the encoding used by system messages"""
+
+    def test_get_message_encoding(self):
+        encoding_name = osutils.get_message_encoding()
+        "".decode(encoding_name) # should be a valid encoding name
+
+    def test_get_message_encoding_decodes_strerror(self):
+        encoding_name = osutils.get_message_encoding()
+        for number, name in errno.errorcode.iteritems():
+            string = os.strerror(number)
+            string.decode(encoding_name)

=== modified file 'bzrlib/tests/test_pack.py'
--- bzrlib/tests/test_pack.py	2009-06-20 01:17:38 +0000
+++ bzrlib/tests/test_pack.py	2011-11-29 09:13:54 +0000
@@ -64,6 +64,11 @@
             errors.InvalidRecordError,
             serialiser.bytes_record, 'bytes', [('bad name',)])
 
+    def test_bytes_record_header(self):
+        serialiser = pack.ContainerSerialiser()
+        record = serialiser.bytes_header(32, [('name1',), ('name2',)])
+        self.assertEqual('B32\nname1\nname2\n\n', record)
+
 
 class TestContainerWriter(tests.TestCase):
 
@@ -126,6 +131,7 @@
     def test_add_bytes_record_one_name(self):
         """Add a bytes record with one name."""
         self.writer.begin()
+
         offset, length = self.writer.add_bytes_record(
             'abc', names=[('name1', )])
         self.assertEqual((42, 13), (offset, length))
@@ -133,6 +139,33 @@
             'Bazaar pack format 1 (introduced in 0.18)\n'
             'B3\nname1\n\nabc')
 
+    def test_add_bytes_record_split_writes(self):
+        """Write a large record which does multiple IOs"""
+
+        writes = []
+        real_write = self.writer.write_func
+
+        def record_writes(bytes):
+            writes.append(bytes)
+            return real_write(bytes)
+
+        self.writer.write_func = record_writes
+        self.writer._JOIN_WRITES_THRESHOLD = 2
+
+        self.writer.begin()
+        offset, length = self.writer.add_bytes_record(
+            'abcabc', names=[('name1', )])
+        self.assertEqual((42, 16), (offset, length))
+        self.assertOutput(
+            'Bazaar pack format 1 (introduced in 0.18)\n'
+            'B6\nname1\n\nabcabc')
+
+        self.assertEquals([
+            'Bazaar pack format 1 (introduced in 0.18)\n',
+            'B6\nname1\n\n',
+            'abcabc'],
+            writes)
+
     def test_add_bytes_record_two_names(self):
         """Add a bytes record with two names."""
         self.writer.begin()
@@ -207,8 +240,8 @@
     def test_construct(self):
         """Test constructing a ContainerReader.
 
-        This uses None as the output stream to show that the constructor doesn't
-        try to use the input stream.
+        This uses None as the output stream to show that the constructor
+        doesn't try to use the input stream.
         """
         reader = pack.ContainerReader(None)
 
@@ -259,7 +292,8 @@
 
     def test_validate_empty_container(self):
         """validate does not raise an error for a container with no records."""
-        reader = self.get_reader_for("Bazaar pack format 1 (introduced in 0.18)\nE")
+        reader = self.get_reader_for(
+            "Bazaar pack format 1 (introduced in 0.18)\nE")
         # No exception raised
         reader.validate()
 
@@ -705,5 +739,3 @@
         parser.accept_bytes("6\n\nabcdef")
         self.assertEqual([([], 'abcdef')], parser.read_pending_records())
         self.assertEqual([], parser.read_pending_records())
-
-

=== modified file 'bzrlib/tests/test_permissions.py'
--- bzrlib/tests/test_permissions.py	2011-02-25 15:24:22 +0000
+++ bzrlib/tests/test_permissions.py	2011-12-18 15:43:47 +0000
@@ -32,9 +32,8 @@
 
 import os
 import sys
-import urllib
 
-from bzrlib import transport
+from bzrlib import urlutils
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import BzrDir
 from bzrlib.tests import TestCaseWithTransport, TestSkipped
@@ -68,11 +67,11 @@
         test.assertTransportMode(t, base, dir_mode)
     for root, dirs, files in os.walk(base):
         for d in dirs:
-            p = '/'.join([urllib.quote(x) for x in root.split('/\\') + [d]])
+            p = '/'.join([urlutils.quote(x) for x in root.split('/\\') + [d]])
             test.assertTransportMode(t, p, dir_mode)
         for f in files:
             p = os.path.join(root, f)
-            p = '/'.join([urllib.quote(x) for x in root.split('/\\') + [f]])
+            p = '/'.join([urlutils.quote(x) for x in root.split('/\\') + [f]])
             test.assertTransportMode(t, p, file_mode)
 
 

=== modified file 'bzrlib/tests/test_reconfigure.py'
--- bzrlib/tests/test_reconfigure.py	2011-06-10 08:11:44 +0000
+++ bzrlib/tests/test_reconfigure.py	2011-12-05 17:31:43 +0000
@@ -21,6 +21,7 @@
     reconfigure,
     repository,
     tests,
+    vf_repository,
     workingtree,
     )
 
@@ -258,12 +259,12 @@
     def test_branch_to_lightweight_checkout_failure(self):
         parent, child, reconfiguration = \
             self.prepare_branch_to_lightweight_checkout()
-        old_Repository_fetch = repository.Repository.fetch
-        repository.Repository.fetch = None
+        old_Repository_fetch = vf_repository.VersionedFileRepository.fetch
+        vf_repository.VersionedFileRepository.fetch = None
         try:
             self.assertRaises(TypeError, reconfiguration.apply)
         finally:
-            repository.Repository.fetch = old_Repository_fetch
+            vf_repository.VersionedFileRepository.fetch = old_Repository_fetch
         child = _mod_branch.Branch.open('child')
         self.assertContainsRe(child.base, 'child/$')
 

=== modified file 'bzrlib/tests/test_remote.py'
--- bzrlib/tests/test_remote.py	2011-08-26 11:35:50 +0000
+++ bzrlib/tests/test_remote.py	2012-01-02 10:24:02 +0000
@@ -25,6 +25,7 @@
 
 import bz2
 from cStringIO import StringIO
+import zlib
 
 from bzrlib import (
     branch,
@@ -41,6 +42,7 @@
     transport,
     treebuilder,
     versionedfile,
+    vf_search,
     )
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import (
@@ -48,6 +50,7 @@
     BzrDirFormat,
     RemoteBzrProber,
     )
+from bzrlib.chk_serializer import chk_bencode_serializer
 from bzrlib.remote import (
     RemoteBranch,
     RemoteBranchFormat,
@@ -57,13 +60,18 @@
     RemoteRepositoryFormat,
     )
 from bzrlib.repofmt import groupcompress_repo, knitpack_repo
-from bzrlib.revision import NULL_REVISION
+from bzrlib.revision import (
+    NULL_REVISION,
+    Revision,
+    )
 from bzrlib.smart import medium, request
 from bzrlib.smart.client import _SmartClient
 from bzrlib.smart.repository import (
     SmartServerRepositoryGetParentMap,
     SmartServerRepositoryGetStream_1_19,
+    _stream_to_byte_stream,
     )
+from bzrlib.symbol_versioning import deprecated_in
 from bzrlib.tests import (
     test_server,
     )
@@ -115,10 +123,12 @@
 
     def test_remote_branch_revision_history(self):
         b = BzrDir.open_from_transport(self.transport).open_branch()
-        self.assertEqual([], b.revision_history())
+        self.assertEqual([],
+            self.applyDeprecated(deprecated_in((2, 5, 0)), b.revision_history))
         r1 = self.local_wt.commit('1st commit')
         r2 = self.local_wt.commit('1st commit', rev_id=u'\xc8'.encode('utf8'))
-        self.assertEqual([r1, r2], b.revision_history())
+        self.assertEqual([r1, r2],
+            self.applyDeprecated(deprecated_in((2, 5, 0)), b.revision_history))
 
     def test_find_correct_format(self):
         """Should open a RemoteBzrDir over a RemoteTransport"""
@@ -165,17 +175,17 @@
     def test_remote_branch_set_append_revisions_only(self):
         # Make a format 1.9 branch, which supports append_revisions_only
         branch = self.make_branch('branch', format='1.9')
-        config = branch.get_config()
         branch.set_append_revisions_only(True)
+        config = branch.get_config_stack()
         self.assertEqual(
-            'True', config.get_user_option('append_revisions_only'))
+            True, config.get('append_revisions_only'))
         branch.set_append_revisions_only(False)
+        config = branch.get_config_stack()
         self.assertEqual(
-            'False', config.get_user_option('append_revisions_only'))
+            False, config.get('append_revisions_only'))
 
     def test_remote_branch_set_append_revisions_only_upgrade_reqd(self):
         branch = self.make_branch('branch', format='knit')
-        config = branch.get_config()
         self.assertRaises(
             errors.UpgradeRequired, branch.set_append_revisions_only, True)
 
@@ -481,6 +491,121 @@
         self.assertEqual(None, result._branch_format)
         self.assertFinished(client)
 
+    def test_unknown(self):
+        transport = self.get_transport('quack')
+        referenced = self.make_branch('referenced')
+        expected = referenced.bzrdir.cloning_metadir()
+        client = FakeClient(transport.base)
+        client.add_expected_call(
+            'BzrDir.cloning_metadir', ('quack/', 'False'),
+            'success', ('unknown', 'unknown', ('branch', ''))),
+        a_bzrdir = RemoteBzrDir(transport, RemoteBzrDirFormat(),
+            _client=client)
+        self.assertRaises(errors.UnknownFormatError, a_bzrdir.cloning_metadir)
+
+
+class TestBzrDirCheckoutMetaDir(TestRemote):
+
+    def test__get_checkout_format(self):
+        transport = MemoryTransport()
+        client = FakeClient(transport.base)
+        reference_bzrdir_format = bzrdir.format_registry.get('default')()
+        control_name = reference_bzrdir_format.network_name()
+        client.add_expected_call(
+            'BzrDir.checkout_metadir', ('quack/', ),
+            'success', (control_name, '', ''))
+        transport.mkdir('quack')
+        transport = transport.clone('quack')
+        a_bzrdir = RemoteBzrDir(transport, RemoteBzrDirFormat(),
+            _client=client)
+        result = a_bzrdir.checkout_metadir()
+        # We should have got a reference control dir with default branch and
+        # repository formats.
+        self.assertEqual(bzrdir.BzrDirMetaFormat1, type(result))
+        self.assertEqual(None, result._repository_format)
+        self.assertEqual(None, result._branch_format)
+        self.assertFinished(client)
+
+    def test_unknown_format(self):
+        transport = MemoryTransport()
+        client = FakeClient(transport.base)
+        client.add_expected_call(
+            'BzrDir.checkout_metadir', ('quack/',),
+            'success', ('dontknow', '', ''))
+        transport.mkdir('quack')
+        transport = transport.clone('quack')
+        a_bzrdir = RemoteBzrDir(transport, RemoteBzrDirFormat(),
+            _client=client)
+        self.assertRaises(errors.UnknownFormatError,
+            a_bzrdir.checkout_metadir)
+        self.assertFinished(client)
+
+
+class TestBzrDirDestroyBranch(TestRemote):
+
+    def test_destroy_default(self):
+        transport = self.get_transport('quack')
+        referenced = self.make_branch('referenced')
+        client = FakeClient(transport.base)
+        client.add_expected_call(
+            'BzrDir.destroy_branch', ('quack/', ),
+            'success', ('ok',)),
+        a_bzrdir = RemoteBzrDir(transport, RemoteBzrDirFormat(),
+            _client=client)
+        a_bzrdir.destroy_branch()
+        self.assertFinished(client)
+
+    def test_destroy_named(self):
+        transport = self.get_transport('quack')
+        referenced = self.make_branch('referenced')
+        client = FakeClient(transport.base)
+        client.add_expected_call(
+            'BzrDir.destroy_branch', ('quack/', "foo"),
+            'success', ('ok',)),
+        a_bzrdir = RemoteBzrDir(transport, RemoteBzrDirFormat(),
+            _client=client)
+        a_bzrdir.destroy_branch("foo")
+        self.assertFinished(client)
+
+
+class TestBzrDirHasWorkingTree(TestRemote):
+
+    def test_has_workingtree(self):
+        transport = self.get_transport('quack')
+        client = FakeClient(transport.base)
+        client.add_expected_call(
+            'BzrDir.has_workingtree', ('quack/',),
+            'success', ('yes',)),
+        a_bzrdir = RemoteBzrDir(transport, RemoteBzrDirFormat(),
+            _client=client)
+        self.assertTrue(a_bzrdir.has_workingtree())
+        self.assertFinished(client)
+
+    def test_no_workingtree(self):
+        transport = self.get_transport('quack')
+        client = FakeClient(transport.base)
+        client.add_expected_call(
+            'BzrDir.has_workingtree', ('quack/',),
+            'success', ('no',)),
+        a_bzrdir = RemoteBzrDir(transport, RemoteBzrDirFormat(),
+            _client=client)
+        self.assertFalse(a_bzrdir.has_workingtree())
+        self.assertFinished(client)
+
+
+class TestBzrDirDestroyRepository(TestRemote):
+
+    def test_destroy_repository(self):
+        transport = self.get_transport('quack')
+        client = FakeClient(transport.base)
+        client.add_expected_call(
+            'BzrDir.destroy_repository', ('quack/',),
+            'success', ('ok',)),
+        a_bzrdir = RemoteBzrDir(transport, RemoteBzrDirFormat(),
+            _client=client)
+        a_bzrdir.destroy_repository()
+        self.assertFinished(client)
+
 
 class TestBzrDirOpen(TestRemote):
 
@@ -610,7 +735,7 @@
         # _get_tree_branch is a form of open_branch, but it should only ask for
         # branch opening, not any other network requests.
         calls = []
-        def open_branch(name=None):
+        def open_branch(name=None, possible_transports=None):
             calls.append("Called")
             return "a-branch"
         transport = MemoryTransport()
@@ -808,6 +933,7 @@
         # name.
         client.add_success_response_with_body(
             "Bazaar-NG meta directory, format 1\n", 'ok')
+        client.add_success_response('stat', '0', '65535')
         client.add_success_response_with_body(
             reference_format.get_format_string(), 'ok')
         # PackRepository wants to do a stat
@@ -822,6 +948,7 @@
              ('call', 'BzrDir.find_repositoryV2', ('quack/',)),
              ('call', 'BzrDir.find_repository', ('quack/',)),
              ('call_expecting_body', 'get', ('/quack/.bzr/branch-format',)),
+             ('call', 'stat', ('/quack/.bzr',)),
              ('call_expecting_body', 'get', ('/quack/.bzr/repository/format',)),
              ('call', 'stat', ('/quack/.bzr/repository',)),
              ],
@@ -841,6 +968,7 @@
         # name.
         client.add_success_response_with_body(
             "Bazaar-NG meta directory, format 1\n", 'ok')
+        client.add_success_response('stat', '0', '65535')
         client.add_success_response_with_body(
             reference_format.get_format_string(), 'ok')
         # PackRepository wants to do a stat
@@ -854,6 +982,7 @@
             [('call', 'BzrDir.find_repositoryV3', ('quack/',)),
              ('call', 'BzrDir.find_repositoryV2', ('quack/',)),
              ('call_expecting_body', 'get', ('/quack/.bzr/branch-format',)),
+             ('call', 'stat', ('/quack/.bzr',)),
              ('call_expecting_body', 'get', ('/quack/.bzr/repository/format',)),
              ('call', 'stat', ('/quack/.bzr/repository',)),
              ],
@@ -996,6 +1125,60 @@
         return RemoteBranch(bzrdir, repo, _client=client, format=format)
 
 
+class TestBranchBreakLock(RemoteBranchTestCase):
+
+    def test_break_lock(self):
+        transport_path = 'quack'
+        transport = MemoryTransport()
+        client = FakeClient(transport.base)
+        client.add_expected_call(
+            'Branch.get_stacked_on_url', ('quack/',),
+            'error', ('NotStacked',))
+        client.add_expected_call(
+            'Branch.break_lock', ('quack/',),
+            'success', ('ok',))
+        transport.mkdir('quack')
+        transport = transport.clone('quack')
+        branch = self.make_remote_branch(transport, client)
+        branch.break_lock()
+        self.assertFinished(client)
+
+
+class TestBranchGetPhysicalLockStatus(RemoteBranchTestCase):
+
+    def test_get_physical_lock_status_yes(self):
+        transport = MemoryTransport()
+        client = FakeClient(transport.base)
+        client.add_expected_call(
+            'Branch.get_stacked_on_url', ('quack/',),
+            'error', ('NotStacked',))
+        client.add_expected_call(
+            'Branch.get_physical_lock_status', ('quack/',),
+            'success', ('yes',))
+        transport.mkdir('quack')
+        transport = transport.clone('quack')
+        branch = self.make_remote_branch(transport, client)
+        result = branch.get_physical_lock_status()
+        self.assertFinished(client)
+        self.assertEqual(True, result)
+
+    def test_get_physical_lock_status_no(self):
+        transport = MemoryTransport()
+        client = FakeClient(transport.base)
+        client.add_expected_call(
+            'Branch.get_stacked_on_url', ('quack/',),
+            'error', ('NotStacked',))
+        client.add_expected_call(
+            'Branch.get_physical_lock_status', ('quack/',),
+            'success', ('no',))
+        transport.mkdir('quack')
+        transport = transport.clone('quack')
+        branch = self.make_remote_branch(transport, client)
+        result = branch.get_physical_lock_status()
+        self.assertFinished(client)
+        self.assertEqual(False, result)
+
+
 class TestBranchGetParent(RemoteBranchTestCase):
 
     def test_no_parent(self):
@@ -1091,7 +1274,7 @@
         verb = 'Branch.set_parent_location'
         self.disable_verb(verb)
         branch.set_parent('http://foo/')
-        self.assertLength(12, self.hpss_calls)
+        self.assertLength(13, self.hpss_calls)
 
 
 class TestBranchGetTagsBytes(RemoteBranchTestCase):
@@ -1251,8 +1434,8 @@
 
     def test_backwards_compatible(self):
         branch = self.make_branch_with_tags()
-        c = branch.get_config()
-        c.set_user_option('branch.fetch_tags', 'True')
+        c = branch.get_config_stack()
+        c.set('branch.fetch_tags', True)
         self.addCleanup(branch.lock_read().unlock)
         # Disable the heads_to_fetch verb
         verb = 'Branch.heads_to_fetch'
@@ -1267,8 +1450,8 @@
 
     def test_backwards_compatible_no_tags(self):
         branch = self.make_branch_with_tags()
-        c = branch.get_config()
-        c.set_user_option('branch.fetch_tags', 'False')
+        c = branch.get_config_stack()
+        c.set('branch.fetch_tags', False)
         self.addCleanup(branch.lock_read().unlock)
         # Disable the heads_to_fetch verb
         verb = 'Branch.heads_to_fetch'
@@ -1445,9 +1628,6 @@
             'Branch.unlock', ('branch/', 'branch token', 'repo token'),
             'success', ('ok',))
         branch = self.make_remote_branch(transport, client)
-        # This is a hack to work around the problem that RemoteBranch currently
-        # unnecessarily invokes _ensure_real upon a call to lock_write.
-        branch._ensure_real = lambda: None
         branch.lock_write()
         result = branch._set_last_revision(NULL_REVISION)
         branch.unlock()
@@ -1482,9 +1662,6 @@
             'Branch.unlock', ('branch/', 'branch token', 'repo token'),
             'success', ('ok',))
         branch = self.make_remote_branch(transport, client)
-        # This is a hack to work around the problem that RemoteBranch currently
-        # unnecessarily invokes _ensure_real upon a call to lock_write.
-        branch._ensure_real = lambda: None
         # Lock the branch, reset the record of remote calls.
         branch.lock_write()
         result = branch._set_last_revision('rev-id2')
@@ -1557,7 +1734,6 @@
             'Branch.unlock', ('branch/', 'branch token', 'repo token'),
             'success', ('ok',))
         branch = self.make_remote_branch(transport, client)
-        branch._ensure_real = lambda: None
         branch.lock_write()
         # The 'TipChangeRejected' error response triggered by calling
         # set_last_revision_info causes a TipChangeRejected exception.
@@ -1826,7 +2002,7 @@
         self.addCleanup(branch.unlock)
         self.reset_smart_call_log()
         branch._get_config().set_option('value', 'name')
-        self.assertLength(10, self.hpss_calls)
+        self.assertLength(11, self.hpss_calls)
         self.assertEqual('value', branch._get_config().get_option('name'))
 
     def test_backwards_compat_set_option_with_dict(self):
@@ -1840,10 +2016,65 @@
         config = branch._get_config()
         value_dict = {'ascii': 'a', u'unicode \N{WATCH}': u'\N{INTERROBANG}'}
         config.set_option(value_dict, 'name')
-        self.assertLength(10, self.hpss_calls)
+        self.assertLength(11, self.hpss_calls)
         self.assertEqual(value_dict, branch._get_config().get_option('name'))
 
 
+class TestBranchGetPutConfigStore(RemoteBranchTestCase):
+
+    def test_get_branch_conf(self):
+        # in an empty branch we decode the response properly
+        client = FakeClient()
+        client.add_expected_call(
+            'Branch.get_stacked_on_url', ('memory:///',),
+            'error', ('NotStacked',),)
+        client.add_success_response_with_body('# config file body', 'ok')
+        transport = MemoryTransport()
+        branch = self.make_remote_branch(transport, client)
+        config = branch.get_config_stack()
+        config.get("email")
+        config.get("log_format")
+        self.assertEqual(
+            [('call', 'Branch.get_stacked_on_url', ('memory:///',)),
+             ('call_expecting_body', 'Branch.get_config_file', ('memory:///',))],
+            client._calls)
+
+    def test_set_branch_conf(self):
+        client = FakeClient()
+        client.add_expected_call(
+            'Branch.get_stacked_on_url', ('memory:///',),
+            'error', ('NotStacked',),)
+        client.add_expected_call(
+            'Branch.lock_write', ('memory:///', '', ''),
+            'success', ('ok', 'branch token', 'repo token'))
+        client.add_expected_call(
+            'Branch.get_config_file', ('memory:///', ),
+            'success', ('ok', ), "# line 1\n")
+        client.add_expected_call(
+            'Branch.put_config_file', ('memory:///', 'branch token',
+            'repo token'),
+            'success', ('ok',))
+        client.add_expected_call(
+            'Branch.unlock', ('memory:///', 'branch token', 'repo token'),
+            'success', ('ok',))
+        transport = MemoryTransport()
+        branch = self.make_remote_branch(transport, client)
+        branch.lock_write()
+        config = branch.get_config_stack()
+        config.set('email', 'The Dude <lebowski@example.com>')
+        branch.unlock()
+        self.assertFinished(client)
+        self.assertEqual(
+            [('call', 'Branch.get_stacked_on_url', ('memory:///',)),
+             ('call', 'Branch.lock_write', ('memory:///', '', '')),
+             ('call_expecting_body', 'Branch.get_config_file', ('memory:///',)),
+             ('call_with_body_bytes_expecting_body', 'Branch.put_config_file',
+                 ('memory:///', 'branch token', 'repo token'),
+                 '# line 1\nemail = The Dude <lebowski@example.com>\n'),
+             ('call', 'Branch.unlock', ('memory:///', 'branch token', 'repo token'))],
+            client._calls)
+
+
 class TestBranchLockWrite(RemoteBranchTestCase):
 
     def test_lock_write_unlockable(self):
@@ -1862,6 +2093,58 @@
         self.assertFinished(client)
 
 
+class TestBranchRevisionIdToRevno(RemoteBranchTestCase):
+
+    def test_simple(self):
+        transport = MemoryTransport()
+        client = FakeClient(transport.base)
+        client.add_expected_call(
+            'Branch.get_stacked_on_url', ('quack/',),
+            'error', ('NotStacked',),)
+        client.add_expected_call(
+            'Branch.revision_id_to_revno', ('quack/', 'null:'),
+            'success', ('ok', '0',),)
+        client.add_expected_call(
+            'Branch.revision_id_to_revno', ('quack/', 'unknown'),
+            'error', ('NoSuchRevision', 'unknown',),)
+        transport.mkdir('quack')
+        transport = transport.clone('quack')
+        branch = self.make_remote_branch(transport, client)
+        self.assertEquals(0, branch.revision_id_to_revno('null:'))
+        self.assertRaises(errors.NoSuchRevision,
+            branch.revision_id_to_revno, 'unknown')
+        self.assertFinished(client)
+
+    def test_dotted(self):
+        transport = MemoryTransport()
+        client = FakeClient(transport.base)
+        client.add_expected_call(
+            'Branch.get_stacked_on_url', ('quack/',),
+            'error', ('NotStacked',),)
+        client.add_expected_call(
+            'Branch.revision_id_to_revno', ('quack/', 'null:'),
+            'success', ('ok', '0',),)
+        client.add_expected_call(
+            'Branch.revision_id_to_revno', ('quack/', 'unknown'),
+            'error', ('NoSuchRevision', 'unknown',),)
+        transport.mkdir('quack')
+        transport = transport.clone('quack')
+        branch = self.make_remote_branch(transport, client)
+        self.assertEquals((0, ), branch.revision_id_to_dotted_revno('null:'))
+        self.assertRaises(errors.NoSuchRevision,
+            branch.revision_id_to_dotted_revno, 'unknown')
+        self.assertFinished(client)
+
+    def test_dotted_no_smart_verb(self):
+        self.setup_smart_server_with_call_log()
+        branch = self.make_branch('.')
+        self.disable_verb('Branch.revision_id_to_revno')
+        self.reset_smart_call_log()
+        self.assertEquals((0, ),
+            branch.revision_id_to_dotted_revno('null:'))
+        self.assertLength(8, self.hpss_calls)
+
+
 class TestBzrDirGetSetConfig(RemoteBzrDirTestCase):
 
     def test__get_config(self):
@@ -1881,7 +2164,7 @@
         self.reset_smart_call_log()
         config = bzrdir.get_config()
         config.set_default_stack_on('/')
-        self.assertLength(3, self.hpss_calls)
+        self.assertLength(4, self.hpss_calls)
 
     def test_backwards_compat_get_option(self):
         self.setup_smart_server_with_call_log()
@@ -1891,7 +2174,7 @@
         self.reset_smart_call_log()
         self.assertEqual(None,
             bzrdir._get_config().get_option('default_stack_on'))
-        self.assertLength(3, self.hpss_calls)
+        self.assertLength(4, self.hpss_calls)
 
 
 class TestTransportIsReadonly(tests.TestCase):
@@ -2024,6 +2307,31 @@
             remote_repo_format.get_format_description())
 
 
+class TestRepositoryAllRevisionIds(TestRemoteRepository):
+
+    def test_empty(self):
+        transport_path = 'quack'
+        repo, client = self.setup_fake_client_and_repository(transport_path)
+        client.add_success_response_with_body('', 'ok')
+        self.assertEquals([], repo.all_revision_ids())
+        self.assertEqual(
+            [('call_expecting_body', 'Repository.all_revision_ids',
+             ('quack/',))],
+            client._calls)
+
+    def test_with_some_content(self):
+        transport_path = 'quack'
+        repo, client = self.setup_fake_client_and_repository(transport_path)
+        client.add_success_response_with_body(
+            'rev1\nrev2\nanotherrev\n', 'ok')
+        self.assertEquals(["rev1", "rev2", "anotherrev"],
+            repo.all_revision_ids())
+        self.assertEqual(
+            [('call_expecting_body', 'Repository.all_revision_ids',
+             ('quack/',))],
+            client._calls)
+
+
 class TestRepositoryGatherStats(TestRemoteRepository):
 
     def test_revid_none(self):
@@ -2082,6 +2390,78 @@
                          result)
 
 
+class TestRepositoryBreakLock(TestRemoteRepository):
+
+    def test_break_lock(self):
+        transport_path = 'quack'
+        repo, client = self.setup_fake_client_and_repository(transport_path)
+        client.add_success_response('ok')
+        repo.break_lock()
+        self.assertEqual(
+            [('call', 'Repository.break_lock', ('quack/',))],
+            client._calls)
+
+
+class TestRepositoryGetSerializerFormat(TestRemoteRepository):
+
+    def test_get_serializer_format(self):
+        transport_path = 'hill'
+        repo, client = self.setup_fake_client_and_repository(transport_path)
+        client.add_success_response('ok', '7')
+        self.assertEquals('7', repo.get_serializer_format())
+        self.assertEqual(
+            [('call', 'VersionedFileRepository.get_serializer_format',
+              ('hill/', ))],
+            client._calls)
+
+
+class TestRepositoryReconcile(TestRemoteRepository):
+
+    def test_reconcile(self):
+        transport_path = 'hill'
+        repo, client = self.setup_fake_client_and_repository(transport_path)
+        body = ("garbage_inventories: 2\n"
+                "inconsistent_parents: 3\n")
+        client.add_expected_call(
+            'Repository.lock_write', ('hill/', ''),
+            'success', ('ok', 'a token'))
+        client.add_success_response_with_body(body, 'ok')
+        reconciler = repo.reconcile()
+        self.assertEqual(
+            [('call', 'Repository.lock_write', ('hill/', '')),
+             ('call_expecting_body', 'Repository.reconcile',
+                ('hill/', 'a token'))],
+            client._calls)
+        self.assertEquals(2, reconciler.garbage_inventories)
+        self.assertEquals(3, reconciler.inconsistent_parents)
+
+
+class TestRepositoryGetRevisionSignatureText(TestRemoteRepository):
+
+    def test_text(self):
+        # ('ok',), body with signature text
+        transport_path = 'quack'
+        repo, client = self.setup_fake_client_and_repository(transport_path)
+        client.add_success_response_with_body(
+            'THETEXT', 'ok')
+        self.assertEquals("THETEXT", repo.get_signature_text("revid"))
+        self.assertEqual(
+            [('call_expecting_body', 'Repository.get_revision_signature_text',
+             ('quack/', 'revid'))],
+            client._calls)
+
+    def test_no_signature(self):
+        transport_path = 'quick'
+        repo, client = self.setup_fake_client_and_repository(transport_path)
+        client.add_error_response('nosuchrevision', 'unknown')
+        self.assertRaises(errors.NoSuchRevision, repo.get_signature_text,
+                "unknown")
+        self.assertEqual(
+            [('call_expecting_body', 'Repository.get_revision_signature_text',
+              ('quick/', 'unknown'))],
+            client._calls)
+
+
 class TestRepositoryGetGraph(TestRemoteRepository):
 
     def test_get_graph(self):
@@ -2092,6 +2472,33 @@
         self.assertNotEqual(graph._parents_provider, repo)
 
 
+class TestRepositoryAddSignatureText(TestRemoteRepository):
+
+    def test_add_signature_text(self):
+        transport_path = 'quack'
+        repo, client = self.setup_fake_client_and_repository(transport_path)
+        client.add_expected_call(
+            'Repository.lock_write', ('quack/', ''),
+            'success', ('ok', 'a token'))
+        client.add_expected_call(
+            'Repository.start_write_group', ('quack/', 'a token'),
+            'success', ('ok', ('token1', )))
+        client.add_expected_call(
+            'Repository.add_signature_text', ('quack/', 'a token', 'rev1',
+                'token1'),
+            'success', ('ok', ), None)
+        repo.lock_write()
+        repo.start_write_group()
+        self.assertIs(None,
+            repo.add_signature_text("rev1", "every bloody emperor"))
+        self.assertEqual(
+            ('call_with_body_bytes_expecting_body',
+              'Repository.add_signature_text',
+                ('quack/', 'a token', 'rev1', 'token1'),
+              'every bloody emperor'),
+            client._calls[-1])
+
+
 class TestRepositoryGetParentMap(TestRemoteRepository):
 
     def test_get_parent_map_caching(self):
@@ -2320,6 +2727,43 @@
         self.assertEqual({'rev1': ('null:',)}, graph.get_parent_map(['rev1']))
 
 
+class TestRepositoryGetRevisions(TestRemoteRepository):
+
+    def test_hpss_missing_revision(self):
+        transport_path = 'quack'
+        repo, client = self.setup_fake_client_and_repository(transport_path)
+        client.add_success_response_with_body(
+            '', 'ok', '10')
+        self.assertRaises(errors.NoSuchRevision, repo.get_revisions,
+            ['somerev1', 'anotherrev2'])
+        self.assertEqual(
+            [('call_with_body_bytes_expecting_body', 'Repository.iter_revisions',
+             ('quack/', ), "somerev1\nanotherrev2")],
+            client._calls)
+
+    def test_hpss_get_single_revision(self):
+        transport_path = 'quack'
+        repo, client = self.setup_fake_client_and_repository(transport_path)
+        somerev1 = Revision("somerev1")
+        somerev1.committer = "Joe Committer <joe@example.com>"
+        somerev1.timestamp = 1321828927
+        somerev1.timezone = -60
+        somerev1.inventory_sha1 = "691b39be74c67b1212a75fcb19c433aaed903c2b"
+        somerev1.message = "Message"
+        body = zlib.compress(chk_bencode_serializer.write_revision_to_string(
+            somerev1))
+        # Split up body into two bits to make sure the zlib compression object
+        # gets data fed twice.
+        client.add_success_response_with_body(
+                [body[:10], body[10:]], 'ok', '10')
+        revs = repo.get_revisions(['somerev1'])
+        self.assertEquals(revs, [somerev1])
+        self.assertEqual(
+            [('call_with_body_bytes_expecting_body', 'Repository.iter_revisions',
+             ('quack/', ), "somerev1")],
+            client._calls)
+
+
 class TestRepositoryGetRevisionGraph(TestRemoteRepository):
 
     def test_null_revision(self):
@@ -2476,6 +2920,58 @@
                               call.call.method == verb])
 
 
+class TestRepositoryHasSignatureForRevisionId(TestRemoteRepository):
+
+    def test_has_signature_for_revision_id(self):
+        # ('yes', ) for Repository.has_signature_for_revision_id -> 'True'.
+        transport_path = 'quack'
+        repo, client = self.setup_fake_client_and_repository(transport_path)
+        client.add_success_response('yes')
+        result = repo.has_signature_for_revision_id('A')
+        self.assertEqual(
+            [('call', 'Repository.has_signature_for_revision_id',
+              ('quack/', 'A'))],
+            client._calls)
+        self.assertEqual(True, result)
+
+    def test_is_not_shared(self):
+        # ('no', ) for Repository.has_signature_for_revision_id -> 'False'.
+        transport_path = 'qwack'
+        repo, client = self.setup_fake_client_and_repository(transport_path)
+        client.add_success_response('no')
+        result = repo.has_signature_for_revision_id('A')
+        self.assertEqual(
+            [('call', 'Repository.has_signature_for_revision_id',
+              ('qwack/', 'A'))],
+            client._calls)
+        self.assertEqual(False, result)
+
+
+class TestRepositoryPhysicalLockStatus(TestRemoteRepository):
+
+    def test_get_physical_lock_status_yes(self):
+        transport_path = 'qwack'
+        repo, client = self.setup_fake_client_and_repository(transport_path)
+        client.add_success_response('yes')
+        result = repo.get_physical_lock_status()
+        self.assertEqual(
+            [('call', 'Repository.get_physical_lock_status',
+              ('qwack/', ))],
+            client._calls)
+        self.assertEqual(True, result)
+
+    def test_get_physical_lock_status_no(self):
+        transport_path = 'qwack'
+        repo, client = self.setup_fake_client_and_repository(transport_path)
+        client.add_success_response('no')
+        result = repo.get_physical_lock_status()
+        self.assertEqual(
+            [('call', 'Repository.get_physical_lock_status',
+              ('qwack/', ))],
+            client._calls)
+        self.assertEqual(False, result)
+
+
 class TestRepositoryIsShared(TestRemoteRepository):
 
     def test_is_shared(self):
@@ -2501,6 +2997,31 @@
         self.assertEqual(False, result)
 
 
+class TestRepositoryMakeWorkingTrees(TestRemoteRepository):
+
+    def test_make_working_trees(self):
+        # ('yes', ) for Repository.make_working_trees -> 'True'.
+        transport_path = 'quack'
+        repo, client = self.setup_fake_client_and_repository(transport_path)
+        client.add_success_response('yes')
+        result = repo.make_working_trees()
+        self.assertEqual(
+            [('call', 'Repository.make_working_trees', ('quack/',))],
+            client._calls)
+        self.assertEqual(True, result)
+
+    def test_no_working_trees(self):
+        # ('no', ) for Repository.make_working_trees -> 'False'.
+        transport_path = 'qwack'
+        repo, client = self.setup_fake_client_and_repository(transport_path)
+        client.add_success_response('no')
+        result = repo.make_working_trees()
+        self.assertEqual(
+            [('call', 'Repository.make_working_trees', ('qwack/',))],
+            client._calls)
+        self.assertEqual(False, result)
+
+
 class TestRepositoryLockWrite(TestRemoteRepository):
 
     def test_lock_write(self):
@@ -2532,6 +3053,91 @@
             client._calls)
 
 
+class TestRepositoryWriteGroups(TestRemoteRepository):
+
+    def test_start_write_group(self):
+        transport_path = 'quack'
+        repo, client = self.setup_fake_client_and_repository(transport_path)
+        client.add_expected_call(
+            'Repository.lock_write', ('quack/', ''),
+            'success', ('ok', 'a token'))
+        client.add_expected_call(
+            'Repository.start_write_group', ('quack/', 'a token'),
+            'success', ('ok', ('token1', )))
+        repo.lock_write()
+        repo.start_write_group()
+
+    def test_start_write_group_unsuspendable(self):
+        # Some repositories do not support suspending write
+        # groups. For those, fall back to the "real" repository.
+        transport_path = 'quack'
+        repo, client = self.setup_fake_client_and_repository(transport_path)
+        def stub_ensure_real():
+            client._calls.append(('_ensure_real',))
+            repo._real_repository = _StubRealPackRepository(client._calls)
+        repo._ensure_real = stub_ensure_real
+        client.add_expected_call(
+            'Repository.lock_write', ('quack/', ''),
+            'success', ('ok', 'a token'))
+        client.add_expected_call(
+            'Repository.start_write_group', ('quack/', 'a token'),
+            'error', ('UnsuspendableWriteGroup',))
+        repo.lock_write()
+        repo.start_write_group()
+        self.assertEquals(client._calls[-2:], [ 
+            ('_ensure_real',),
+            ('start_write_group',)])
+
+    def test_commit_write_group(self):
+        transport_path = 'quack'
+        repo, client = self.setup_fake_client_and_repository(transport_path)
+        client.add_expected_call(
+            'Repository.lock_write', ('quack/', ''),
+            'success', ('ok', 'a token'))
+        client.add_expected_call(
+            'Repository.start_write_group', ('quack/', 'a token'),
+            'success', ('ok', ['token1']))
+        client.add_expected_call(
+            'Repository.commit_write_group', ('quack/', 'a token', ['token1']),
+            'success', ('ok',))
+        repo.lock_write()
+        repo.start_write_group()
+        repo.commit_write_group()
+
+    def test_abort_write_group(self):
+        transport_path = 'quack'
+        repo, client = self.setup_fake_client_and_repository(transport_path)
+        client.add_expected_call(
+            'Repository.lock_write', ('quack/', ''),
+            'success', ('ok', 'a token'))
+        client.add_expected_call(
+            'Repository.start_write_group', ('quack/', 'a token'),
+            'success', ('ok', ['token1']))
+        client.add_expected_call(
+            'Repository.abort_write_group', ('quack/', 'a token', ['token1']),
+            'success', ('ok',))
+        repo.lock_write()
+        repo.start_write_group()
+        repo.abort_write_group(False)
+
+    def test_suspend_write_group(self):
+        transport_path = 'quack'
+        repo, client = self.setup_fake_client_and_repository(transport_path)
+        self.assertEquals([], repo.suspend_write_group())
+
+    def test_resume_write_group(self):
+        transport_path = 'quack'
+        repo, client = self.setup_fake_client_and_repository(transport_path)
+        client.add_expected_call(
+            'Repository.lock_write', ('quack/', ''),
+            'success', ('ok', 'a token'))
+        client.add_expected_call(
+            'Repository.check_write_group', ('quack/', 'a token', ['token1']),
+            'success', ('ok',))
+        repo.lock_write()
+        repo.resume_write_group(['token1'])
+
+
 class TestRepositorySetMakeWorkingTrees(TestRemoteRepository):
 
     def test_backwards_compat(self):
@@ -2596,6 +3202,33 @@
         self.assertEqual([], client._calls)
 
 
+class TestRepositoryIterFilesBytes(TestRemoteRepository):
+    """Test Repository.iter_file_bytes."""
+
+    def test_single(self):
+        transport_path = 'quack'
+        repo, client = self.setup_fake_client_and_repository(transport_path)
+        client.add_expected_call(
+            'Repository.iter_files_bytes', ('quack/', ),
+            'success', ('ok',), iter(["ok\x000", "\n", zlib.compress("mydata" * 10)]))
+        for (identifier, byte_stream) in repo.iter_files_bytes([("somefile",
+                "somerev", "myid")]):
+            self.assertEquals("myid", identifier)
+            self.assertEquals("".join(byte_stream), "mydata" * 10)
+
+    def test_missing(self):
+        transport_path = 'quack'
+        repo, client = self.setup_fake_client_and_repository(transport_path)
+        client.add_expected_call(
+            'Repository.iter_files_bytes',
+                ('quack/', ),
+            'error', ('RevisionNotPresent', 'somefile', 'somerev'),
+            iter(["absent\0somefile\0somerev\n"]))
+        self.assertRaises(errors.RevisionNotPresent, list,
+                repo.iter_files_bytes(
+                [("somefile", "somerev", "myid")]))
+
+
 class TestRepositoryInsertStreamBase(TestRemoteRepository):
     """Base class for Repository.insert_stream and .insert_stream_1.19
     tests.
@@ -2876,6 +3509,9 @@
         self.calls = calls
         self._pack_collection = _StubPackCollection(calls)
 
+    def start_write_group(self):
+        self.calls.append(('start_write_group',))
+
     def is_in_write_group(self):
         return False
 
@@ -3319,7 +3955,7 @@
         revs = [r for (r,ps) in graph.iter_ancestry([tip])
                 if r != NULL_REVISION]
         revs.reverse()
-        search = _mod_graph.PendingAncestryResult([tip], stacked.repository)
+        search = vf_search.PendingAncestryResult([tip], stacked.repository)
         self.reset_smart_call_log()
         stream = source.get_stream(search)
         # We trust that if a revision is in the stream the rest of the new
@@ -3431,15 +4067,16 @@
         self.hpss_calls = []
         local.repository.fetch(
             remote_branch.repository,
-            fetch_spec=_mod_graph.EverythingResult(remote_branch.repository))
+            fetch_spec=vf_search.EverythingResult(remote_branch.repository))
         self.assertEqual(['Repository.get_stream_1.19'], self.hpss_calls)
 
     def override_verb(self, verb_name, verb):
         request_handlers = request.request_handlers
         orig_verb = request_handlers.get(verb_name)
+        orig_info = request_handlers.get_info(verb_name)
         request_handlers.register(verb_name, verb, override_existing=True)
         self.addCleanup(request_handlers.register, verb_name, orig_verb,
-                override_existing=True)
+                override_existing=True, info=orig_info)
 
     def test_fetch_everything_backwards_compat(self):
         """Can fetch with EverythingResult even with pre 2.4 servers.
@@ -3470,7 +4107,7 @@
         self.hpss_calls = []
         local.repository.fetch(
             remote_branch.repository,
-            fetch_spec=_mod_graph.EverythingResult(remote_branch.repository))
+            fetch_spec=vf_search.EverythingResult(remote_branch.repository))
         # make sure the overridden verb was used
         self.assertLength(1, verb_log)
         # more than one HPSS call is needed, but because it's a VFS callback
@@ -3521,3 +4158,123 @@
         # interpretation
         self.make_master_and_checkout('mas~ter', 'checkout')
         self.assertUpdateSucceeds(self.bound_location.replace('%2E', '~', 1))
+
+
+class TestWithCustomErrorHandler(RemoteBranchTestCase):
+
+    def test_no_context(self):
+        class OutOfCoffee(errors.BzrError):
+            """A dummy exception for testing."""
+
+            def __init__(self, urgency):
+                self.urgency = urgency
+        remote.no_context_error_translators.register("OutOfCoffee",
+            lambda err: OutOfCoffee(err.error_args[0]))
+        transport = MemoryTransport()
+        client = FakeClient(transport.base)
+        client.add_expected_call(
+            'Branch.get_stacked_on_url', ('quack/',),
+            'error', ('NotStacked',))
+        client.add_expected_call(
+            'Branch.last_revision_info',
+            ('quack/',),
+            'error', ('OutOfCoffee', 'low'))
+        transport.mkdir('quack')
+        transport = transport.clone('quack')
+        branch = self.make_remote_branch(transport, client)
+        self.assertRaises(OutOfCoffee, branch.last_revision_info)
+        self.assertFinished(client)
+
+    def test_with_context(self):
+        class OutOfTea(errors.BzrError):
+            def __init__(self, branch, urgency):
+                self.branch = branch
+                self.urgency = urgency
+        remote.error_translators.register("OutOfTea",
+            lambda err, find, path: OutOfTea(err.error_args[0],
+                find("branch")))
+        transport = MemoryTransport()
+        client = FakeClient(transport.base)
+        client.add_expected_call(
+            'Branch.get_stacked_on_url', ('quack/',),
+            'error', ('NotStacked',))
+        client.add_expected_call(
+            'Branch.last_revision_info',
+            ('quack/',),
+            'error', ('OutOfTea', 'low'))
+        transport.mkdir('quack')
+        transport = transport.clone('quack')
+        branch = self.make_remote_branch(transport, client)
+        self.assertRaises(OutOfTea, branch.last_revision_info)
+        self.assertFinished(client)
+
+
+class TestRepositoryPack(TestRemoteRepository):
+
+    def test_pack(self):
+        transport_path = 'quack'
+        repo, client = self.setup_fake_client_and_repository(transport_path)
+        client.add_expected_call(
+            'Repository.lock_write', ('quack/', ''),
+            'success', ('ok', 'token'))
+        client.add_expected_call(
+            'Repository.pack', ('quack/', 'token', 'False'),
+            'success', ('ok',), )
+        client.add_expected_call(
+            'Repository.unlock', ('quack/', 'token'),
+            'success', ('ok', ))
+        repo.pack()
+
+    def test_pack_with_hint(self):
+        transport_path = 'quack'
+        repo, client = self.setup_fake_client_and_repository(transport_path)
+        client.add_expected_call(
+            'Repository.lock_write', ('quack/', ''),
+            'success', ('ok', 'token'))
+        client.add_expected_call(
+            'Repository.pack', ('quack/', 'token', 'False'),
+            'success', ('ok',), )
+        client.add_expected_call(
+            'Repository.unlock', ('quack/', 'token', 'False'),
+            'success', ('ok', ))
+        repo.pack(['hinta', 'hintb'])
+
+
+class TestRepositoryIterInventories(TestRemoteRepository):
+    """Test Repository.iter_inventories."""
+
+    def _serialize_inv_delta(self, old_name, new_name, delta):
+        serializer = inventory_delta.InventoryDeltaSerializer(True, False)
+        return "".join(serializer.delta_to_lines(old_name, new_name, delta))
+
+    def test_single_empty(self):
+        transport_path = 'quack'
+        repo, client = self.setup_fake_client_and_repository(transport_path)
+        fmt = bzrdir.format_registry.get('2a')().repository_format
+        repo._format = fmt
+        stream = [('inventory-deltas', [
+            versionedfile.FulltextContentFactory('somerevid', None, None,
+                self._serialize_inv_delta('null:', 'somerevid', []))])]
+        client.add_expected_call(
+            'VersionedFileRepository.get_inventories', ('quack/', 'unordered'),
+            'success', ('ok', ),
+            _stream_to_byte_stream(stream, fmt))
+        ret = list(repo.iter_inventories(["somerevid"]))
+        self.assertLength(1, ret)
+        inv = ret[0]
+        self.assertEquals("somerevid", inv.revision_id)
+
+    def test_empty(self):
+        transport_path = 'quack'
+        repo, client = self.setup_fake_client_and_repository(transport_path)
+        ret = list(repo.iter_inventories([]))
+        self.assertEquals(ret, [])
+
+    def test_missing(self):
+        transport_path = 'quack'
+        repo, client = self.setup_fake_client_and_repository(transport_path)
+        client.add_expected_call(
+            'VersionedFileRepository.get_inventories', ('quack/', 'unordered'),
+            'success', ('ok', ), iter([]))
+        self.assertRaises(errors.NoSuchRevision, list, repo.iter_inventories(
+            ["somerevid"]))

=== modified file 'bzrlib/tests/test_repository.py'
--- bzrlib/tests/test_repository.py	2011-08-21 09:24:01 +0000
+++ bzrlib/tests/test_repository.py	2011-12-22 19:54:56 +0000
@@ -31,10 +31,10 @@
     )
 from bzrlib import (
     btree_index,
-    graph,
     symbol_versioning,
     tests,
     transport,
+    vf_search,
     )
 from bzrlib.btree_index import BTreeBuilder, BTreeGraphIndex
 from bzrlib.index import GraphIndex
@@ -92,14 +92,15 @@
                               old_format.__class__)
 
 
-class SampleRepositoryFormat(repository.RepositoryFormat):
+class SampleRepositoryFormat(repository.RepositoryFormatMetaDir):
     """A sample format
 
     this format is initializable, unsupported to aid in testing the
     open and open(unsupported=True) routines.
     """
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See RepositoryFormat.get_format_string()."""
         return "Sample .bzr repository format."
 
@@ -137,23 +138,45 @@
             dir = format._matchingbzrdir.initialize(url)
             format.initialize(dir)
             t = transport.get_transport_from_path(url)
-            found_format = repository.RepositoryFormat.find_format(dir)
+            found_format = repository.RepositoryFormatMetaDir.find_format(dir)
             self.assertIsInstance(found_format, format.__class__)
         check_format(repository.format_registry.get_default(), "bar")
 
     def test_find_format_no_repository(self):
         dir = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
         self.assertRaises(errors.NoRepositoryPresent,
-                          repository.RepositoryFormat.find_format,
+                          repository.RepositoryFormatMetaDir.find_format,
                           dir)
 
+    def test_from_string(self):
+        self.assertIsInstance(
+            SampleRepositoryFormat.from_string(
+                "Sample .bzr repository format."),
+            SampleRepositoryFormat)
+        self.assertRaises(AssertionError,
+            SampleRepositoryFormat.from_string,
+                "Different .bzr repository format.")
+
     def test_find_format_unknown_format(self):
         dir = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
         SampleRepositoryFormat().initialize(dir)
         self.assertRaises(UnknownFormatError,
-                          repository.RepositoryFormat.find_format,
+                          repository.RepositoryFormatMetaDir.find_format,
                           dir)
 
+    def test_find_format_with_features(self):
+        tree = self.make_branch_and_tree('.', format='2a')
+        tree.branch.repository.update_feature_flags({"name": "necessity"})
+        found_format = repository.RepositoryFormatMetaDir.find_format(tree.bzrdir)
+        self.assertIsInstance(found_format, repository.RepositoryFormatMetaDir)
+        self.assertEquals(found_format.features.get("name"), "necessity")
+        self.assertRaises(errors.MissingFeature, found_format.check_support_status,
+            True)
+        self.addCleanup(repository.RepositoryFormatMetaDir.unregister_feature,
+            "name")
+        repository.RepositoryFormatMetaDir.register_feature("name")
+        found_format.check_support_status(True)
+
     def test_register_unregister_format(self):
         # Test deprecated format registration functions
         format = SampleRepositoryFormat()
@@ -438,13 +461,15 @@
 
 class TestRepositoryFormat1(knitrepo.RepositoryFormatKnit1):
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         return "Test Format 1"
 
 
 class TestRepositoryFormat2(knitrepo.RepositoryFormatKnit1):
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         return "Test Format 2"
 
 
@@ -706,7 +731,7 @@
 
         # On a regular pass, getting the inventories and chk pages for rev-2
         # would only get the newly created chk pages
-        search = graph.SearchResult(set(['rev-2']), set(['rev-1']), 1,
+        search = vf_search.SearchResult(set(['rev-2']), set(['rev-1']), 1,
                                     set(['rev-2']))
         simple_chk_records = []
         for vf_name, substream in source.get_stream(search):
@@ -893,7 +918,7 @@
             revision = _mod_revision.Revision('rev1a',
                 committer='jrandom@example.com', timestamp=0,
                 inventory_sha1='', timezone=0, message='foo', parent_ids=[])
-            repo.add_revision('rev1a',revision, inv)
+            repo.add_revision('rev1a', revision, inv)
 
             # make rev1b, which has no Revision, but has an Inventory, and
             # file1
@@ -934,7 +959,7 @@
         revision = _mod_revision.Revision(revision_id,
             committer='jrandom@example.com', timestamp=0, inventory_sha1='',
             timezone=0, message='foo', parent_ids=parent_ids)
-        repo.add_revision(revision_id,revision, inv)
+        repo.add_revision(revision_id, revision, inv)
 
     def add_file(self, repo, inv, filename, revision, parents):
         file_id = filename + '-id'
@@ -1070,6 +1095,23 @@
             sorted(set([osutils.splitext(n)[0] for n in
                         packs._index_transport.list_dir('.')])))
 
+    def test__obsolete_packs_missing_directory(self):
+        tree, r, packs, revs = self.make_packs_and_alt_repo(write_lock=True)
+        r.control_transport.rmdir('obsolete_packs')
+        names = packs.names()
+        pack = packs.get_pack_by_name(names[0])
+        # Schedule this one for removal
+        packs._remove_pack_from_memory(pack)
+        # Now trigger the obsoletion, and ensure that all the remaining files
+        # are still renamed
+        packs._obsolete_packs([pack])
+        self.assertEqual([n + '.pack' for n in names[1:]],
+                         sorted(packs._pack_transport.list_dir('.')))
+        # names[0] should not be present in the index anymore
+        self.assertEqual(names[1:],
+            sorted(set([osutils.splitext(n)[0] for n in
+                        packs._index_transport.list_dir('.')])))
+
     def test_pack_distribution_zero(self):
         packs = self.get_packs()
         self.assertEqual([0], packs.pack_distribution(0))
@@ -1345,6 +1387,12 @@
         obsolete_names = set([osutils.splitext(n)[0] for n in obsolete_packs])
         self.assertEqual([pack.name], sorted(obsolete_names))
 
+    def test_pack_no_obsolete_packs_directory(self):
+        """Bug #314314, don't fail if obsolete_packs directory does
+        not exist."""
+        tree, r, packs, revs = self.make_packs_and_alt_repo(write_lock=True)
+        r.control_transport.rmdir('obsolete_packs')
+        packs._clear_obsolete_packs()
 
 
 class TestPack(TestCaseWithTransport):
@@ -1668,3 +1716,25 @@
         lazy = repository._LazyListJoin(['a'], ['b'])
         self.assertEqual("bzrlib.repository._LazyListJoin((['a'], ['b']))",
                          repr(lazy))
+
+
+class TestFeatures(tests.TestCaseWithTransport):
+
+    def test_open_with_present_feature(self):
+        self.addCleanup(
+            repository.RepositoryFormatMetaDir.unregister_feature,
+            "makes-cheese-sandwich")
+        repository.RepositoryFormatMetaDir.register_feature(
+            "makes-cheese-sandwich")
+        repo = self.make_repository('.')
+        repo.lock_write()
+        repo._format.features["makes-cheese-sandwich"] = "required"
+        repo._format.check_support_status(False)
+        repo.unlock()
+
+    def test_open_with_missing_required_feature(self):
+        repo = self.make_repository('.')
+        repo.lock_write()
+        repo._format.features["makes-cheese-sandwich"] = "required"
+        self.assertRaises(errors.MissingFeature,
+            repo._format.check_support_status, False)

=== modified file 'bzrlib/tests/test_revision.py'
--- bzrlib/tests/test_revision.py	2011-06-19 16:23:04 +0000
+++ bzrlib/tests/test_revision.py	2011-10-06 00:14:01 +0000
@@ -66,19 +66,26 @@
     br2 = tree2.branch
     tree2.commit("Commit four", rev_id="b@u-0-3")
     tree2.commit("Commit five", rev_id="b@u-0-4")
-    revisions_2 = br2.revision_history()
-    self.assertEquals(revisions_2[-1], 'b@u-0-4')
+    self.assertEquals(br2.last_revision(), 'b@u-0-4')
 
     tree1.merge_from_branch(br2)
     tree1.commit("Commit six", rev_id="a@u-0-3")
     tree1.commit("Commit seven", rev_id="a@u-0-4")
     tree2.commit("Commit eight", rev_id="b@u-0-5")
-    self.assertEquals(br2.revision_history()[-1], 'b@u-0-5')
+    self.assertEquals(br2.last_revision(), 'b@u-0-5')
 
     tree1.merge_from_branch(br2)
     tree1.commit("Commit nine", rev_id="a@u-0-5")
     # DO NOT MERGE HERE - we WANT a GHOST.
-    tree2.add_parent_tree_id(br1.revision_history()[4])
+    br1.lock_read()
+    try:
+        graph = br1.repository.get_graph()
+        revhistory = list(graph.iter_lefthand_ancestry(br1.last_revision(),
+            [revision.NULL_REVISION]))
+        revhistory.reverse()
+    finally:
+        br1.unlock()
+    tree2.add_parent_tree_id(revhistory[4])
     tree2.commit("Commit ten - ghost merge", rev_id="b@u-0-6")
 
     return br1, br2

=== modified file 'bzrlib/tests/test_revisionspec.py'
--- bzrlib/tests/test_revisionspec.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/test_revisionspec.py	2011-12-01 23:10:02 +0000
@@ -20,6 +20,7 @@
 from bzrlib import (
     errors,
     revision as _mod_revision,
+    symbol_versioning,
     )
 from bzrlib.tests import TestCaseWithTransport
 from bzrlib.revisionspec import (
@@ -118,7 +119,10 @@
         # If wants_revision_history = True, then _match_on should get the
         # branch revision history
         spec = RevisionSpecMatchOnTrap('foo', _internal=True)
-        spec.in_history(self.tree.branch)
+        spec.wants_revision_history = True
+        self.callDeprecated(['RevisionSpec.wants_revision_history was '
+            'deprecated in 2.5 (RevisionSpecMatchOnTrap).'],
+            spec.in_history, self.tree.branch)
 
         self.assertEqual((self.tree.branch, ['r1' ,'r2']),
                          spec.last_call)
@@ -127,7 +131,6 @@
         # If wants_revision_history = False, then _match_on should get None for
         # the branch revision history
         spec = RevisionSpecMatchOnTrap('foo', _internal=True)
-        spec.wants_revision_history = False
         spec.in_history(self.tree.branch)
 
         self.assertEqual((self.tree.branch, None), spec.last_call)
@@ -150,7 +153,7 @@
 
     def _match_on(self, branch, revs):
         if self.spec == "bork":
-            return RevisionInfo.from_revision_id(branch, "r1", revs)
+            return RevisionInfo.from_revision_id(branch, "r1")
         else:
             raise errors.InvalidRevisionSpec(self.spec, branch)
 

=== modified file 'bzrlib/tests/test_script.py'
--- bzrlib/tests/test_script.py	2011-05-16 13:39:39 +0000
+++ bzrlib/tests/test_script.py	2011-10-05 22:15:03 +0000
@@ -572,20 +572,68 @@
     def test_confirm_action(self):
         """You can write tests that demonstrate user confirmation.
         
-        Specifically, ScriptRunner does't care if the output line for the prompt
-        isn't terminated by a newline from the program; it's implicitly terminated 
-        by the input.
+        Specifically, ScriptRunner does't care if the output line for the
+        prompt isn't terminated by a newline from the program; it's implicitly
+        terminated by the input.
         """
         commands.builtin_command_registry.register(cmd_test_confirm)
         self.addCleanup(commands.builtin_command_registry.remove, 'test-confirm')
         self.run_script("""
             $ bzr test-confirm
-            2>Really do it? [y/n]: 
-            <yes
+            2>Really do it? ([y]es, [n]o): yes
+            <y
             Do it!
             $ bzr test-confirm
-            2>Really do it? [y/n]: 
-            <no
+            2>Really do it? ([y]es, [n]o): no
+            <n
             ok, no
             """)
 
+class TestShelve(script.TestCaseWithTransportAndScript):
+
+    def setUp(self):
+        super(TestShelve, self).setUp()
+        self.run_script("""
+            $ bzr init test
+            Created a standalone tree (format: 2a)
+            $ cd test
+            $ echo foo > file
+            $ bzr add
+            adding file
+            $ bzr commit -m 'file added'
+            2>Committing to:...test/
+            2>added file
+            2>Committed revision 1.
+            $ echo bar > file
+            """)
+
+    def test_shelve(self):
+        self.run_script("""
+            $ bzr shelve -m 'shelve bar'
+            2>Shelve? ([y]es, [N]o, [f]inish, [q]uit): yes
+            <y
+            2>Selected changes:
+            2> M  file
+            2>Shelve 1 change(s)? ([y]es, [N]o, [f]inish, [q]uit): yes
+            <y
+            2>Changes shelved with id "1".
+            """,
+                        null_output_matches_anything=True)
+        self.run_script("""
+            $ bzr shelve --list
+              1: shelve bar
+            """)
+
+    def test_dont_shelve(self):
+        # We intentionally provide no input here to test EOF
+        self.run_script("""
+            $ bzr shelve -m 'shelve bar'
+            2>Shelve? ([y]es, [N]o, [f]inish, [q]uit): 
+            2>No changes to shelve.
+            """,
+                        null_output_matches_anything=True)
+        self.run_script("""
+            $ bzr st
+            modified:
+              file
+            """)

=== modified file 'bzrlib/tests/test_selftest.py'
--- bzrlib/tests/test_selftest.py	2011-08-19 22:34:02 +0000
+++ bzrlib/tests/test_selftest.py	2011-11-08 17:07:23 +0000
@@ -17,6 +17,7 @@
 """Tests for the test framework."""
 
 from cStringIO import StringIO
+import gc
 import doctest
 import os
 import signal
@@ -29,7 +30,6 @@
 from testtools import (
     ExtendedToOriginalDecorator,
     MultiTestResult,
-    __version__ as testtools_version,
     )
 from testtools.content import Content
 from testtools.content_type import ContentType
@@ -1062,31 +1062,20 @@
                 self.expectFailure("No absolute truth", self.assertTrue, True)
         runner = tests.TextTestRunner(stream=StringIO())
         result = self.run_test_runner(runner, Test("test_truth"))
-        if testtools_version[:3] <= (0, 9, 11):
-            self.assertContainsRe(runner.stream.getvalue(),
-                "=+\n"
-                "FAIL: \\S+\.test_truth\n"
-                "-+\n"
-                "(?:.*\n)*"
-                "No absolute truth\n"
-                "(?:.*\n)*"
-                "-+\n"
-                "Ran 1 test in .*\n"
-                "\n"
-                "FAILED \\(failures=1\\)\n\\Z")
-        else:
-            self.assertContainsRe(runner.stream.getvalue(),
-                "=+\n"
-                "FAIL: \\S+\.test_truth\n"
-                "-+\n"
-                "Empty attachments:\n"
-                "  log\n"
-                "\n"
-                "reason: {{{No absolute truth}}}\n"
-                "-+\n"
-                "Ran 1 test in .*\n"
-                "\n"
-                "FAILED \\(failures=1\\)\n\\Z")
+        self.assertContainsRe(runner.stream.getvalue(),
+            "=+\n"
+            "FAIL: \\S+\.test_truth\n"
+            "-+\n"
+            "(?:.*\n)*"
+            "\\s*(?:Text attachment: )?reason"
+            "(?:\n-+\n|: {{{)"
+            "No absolute truth"
+            "(?:\n-+\n|}}}\n)"
+            "(?:.*\n)*"
+            "-+\n"
+            "Ran 1 test in .*\n"
+            "\n"
+            "FAILED \\(failures=1\\)\n\\Z")
 
     def test_result_decorator(self):
         # decorate results
@@ -1251,14 +1240,11 @@
             lambda trace=False: "ascii")
         result = self.run_test_runner(tests.TextTestRunner(stream=out),
             FailureWithUnicode("test_log_unicode"))
-        if testtools_version[:3] > (0, 9, 11):
-            self.assertContainsRe(out.getvalue(), "log: {{{\d+\.\d+  \\\\u2606}}}")
-        else:
-            self.assertContainsRe(out.getvalue(),
-                "Text attachment: log\n"
-                "-+\n"
-                "\d+\.\d+  \\\\u2606\n"
-                "-+\n")
+        self.assertContainsRe(out.getvalue(),
+            "(?:Text attachment: )?log"
+            "(?:\n-+\n|: {{{)"
+            "\d+\.\d+  \\\\u2606"
+            "(?:\n-+\n|}}}\n)")
 
 
 class SampleTestCase(tests.TestCase):
@@ -1743,16 +1729,16 @@
         result = self._run_test('test_fail')
         self.assertEqual(1, len(result.failures))
         result_content = result.failures[0][1]
-        if testtools_version < (0, 9, 12):
-            self.assertContainsRe(result_content, 'Text attachment: log')
+        self.assertContainsRe(result_content,
+            '(?m)^(?:Text attachment: )?log(?:$|: )')
         self.assertContainsRe(result_content, 'this was a failing test')
 
     def test_error_has_log(self):
         result = self._run_test('test_error')
         self.assertEqual(1, len(result.errors))
         result_content = result.errors[0][1]
-        if testtools_version < (0, 9, 12):
-            self.assertContainsRe(result_content, 'Text attachment: log')
+        self.assertContainsRe(result_content,
+            '(?m)^(?:Text attachment: )?log(?:$|: )')
         self.assertContainsRe(result_content, 'this test errored')
 
     def test_skip_has_no_log(self):
@@ -1777,7 +1763,8 @@
         result = self._run_test('test_xfail')
         self.assertEqual(1, len(result.expectedFailures))
         result_content = result.expectedFailures[0][1]
-        self.assertNotContainsRe(result_content, 'Text attachment: log')
+        self.assertNotContainsRe(result_content,
+            '(?m)^(?:Text attachment: )?log(?:$|: )')
         self.assertNotContainsRe(result_content, 'test with expected failure')
 
     def test_unexpected_success_has_log(self):
@@ -2192,7 +2179,8 @@
         self.assertNotContainsRe(content, 'test with expected failure')
         self.assertEqual(1, len(result.expectedFailures))
         result_content = result.expectedFailures[0][1]
-        self.assertNotContainsRe(result_content, 'Text attachment: log')
+        self.assertNotContainsRe(result_content,
+            '(?m)^(?:Text attachment: )?log(?:$|: )')
         self.assertNotContainsRe(result_content, 'test with expected failure')
 
     def test_unexpected_success_has_log(self):
@@ -3314,6 +3302,148 @@
         self.assertLength(1, calls)
 
 
+class _Selftest(object):
+    """Mixin for tests needing full selftest output"""
+
+    def _inject_stream_into_subunit(self, stream):
+        """To be overridden by subclasses that run tests out of process"""
+
+    def _run_selftest(self, **kwargs):
+        sio = StringIO()
+        self._inject_stream_into_subunit(sio)
+        tests.selftest(stream=sio, stop_on_failure=False, **kwargs)
+        return sio.getvalue()
+
+
+class _ForkedSelftest(_Selftest):
+    """Mixin for tests needing full selftest output with forked children"""
+
+    _test_needs_features = [features.subunit]
+
+    def _inject_stream_into_subunit(self, stream):
+        """Monkey-patch subunit so the extra output goes to stream not stdout
+
+        Some APIs need rewriting so this kind of bogus hackery can be replaced
+        by passing the stream param from run_tests down into ProtocolTestCase.
+        """
+        from subunit import ProtocolTestCase
+        _original_init = ProtocolTestCase.__init__
+        def _init_with_passthrough(self, *args, **kwargs):
+            _original_init(self, *args, **kwargs)
+            self._passthrough = stream
+        self.overrideAttr(ProtocolTestCase, "__init__", _init_with_passthrough)
+
+    def _run_selftest(self, **kwargs):
+        # GZ 2011-05-26: Add a PosixSystem feature so this check can go away
+        if getattr(os, "fork", None) is None:
+            raise tests.TestNotApplicable("Platform doesn't support forking")
+        # Make sure the fork code is actually invoked by claiming two cores
+        self.overrideAttr(osutils, "local_concurrency", lambda: 2)
+        kwargs.setdefault("suite_decorators", []).append(tests.fork_decorator)
+        return super(_ForkedSelftest, self)._run_selftest(**kwargs)
+
+
+class TestParallelFork(_ForkedSelftest, tests.TestCase):
+    """Check operation of --parallel=fork selftest option"""
+
+    def test_error_in_child_during_fork(self):
+        """Error in a forked child during test setup should get reported"""
+        class Test(tests.TestCase):
+            def testMethod(self):
+                pass
+        # We don't care what, just break something that a child will run
+        self.overrideAttr(tests, "workaround_zealous_crypto_random", None)
+        out = self._run_selftest(test_suite_factory=Test)
+        # Lines from the tracebacks of the two child processes may be mixed
+        # together due to the way subunit parses and forwards the streams,
+        # so permit extra lines between each part of the error output.
+        self.assertContainsRe(out,
+            "Traceback.*:\n"
+            "(?:.*\n)*"
+            ".+ in fork_for_tests\n"
+            "(?:.*\n)*"
+            "\s*workaround_zealous_crypto_random\(\)\n"
+            "(?:.*\n)*"
+            "TypeError:")
+
+
+class TestUncollectedWarnings(_Selftest, tests.TestCase):
+    """Check a test case still alive after being run emits a warning"""
+
+    class Test(tests.TestCase):
+        def test_pass(self):
+            pass
+        def test_self_ref(self):
+            self.also_self = self.test_self_ref
+        def test_skip(self):
+            self.skip("Don't need")
+
+    def _get_suite(self):
+        return TestUtil.TestSuite([
+            self.Test("test_pass"),
+            self.Test("test_self_ref"),
+            self.Test("test_skip"),
+            ])
+
+    def _run_selftest_with_suite(self, **kwargs):
+        old_flags = tests.selftest_debug_flags
+        tests.selftest_debug_flags = old_flags.union(["uncollected_cases"])
+        gc_on = gc.isenabled()
+        if gc_on:
+            gc.disable()
+        try:
+            output = self._run_selftest(test_suite_factory=self._get_suite,
+                **kwargs)
+        finally:
+            if gc_on:
+                gc.enable()
+            tests.selftest_debug_flags = old_flags
+        self.assertNotContainsRe(output, "Uncollected test case.*test_pass")
+        self.assertContainsRe(output, "Uncollected test case.*test_self_ref")
+        return output
+
+    def test_testsuite(self):
+        self._run_selftest_with_suite()
+
+    def test_pattern(self):
+        out = self._run_selftest_with_suite(pattern="test_(?:pass|self_ref)$")
+        self.assertNotContainsRe(out, "test_skip")
+
+    def test_exclude_pattern(self):
+        out = self._run_selftest_with_suite(exclude_pattern="test_skip$")
+        self.assertNotContainsRe(out, "test_skip")
+
+    def test_random_seed(self):
+        self._run_selftest_with_suite(random_seed="now")
+
+    def test_matching_tests_first(self):
+        self._run_selftest_with_suite(matching_tests_first=True,
+            pattern="test_self_ref$")
+
+    def test_starting_with_and_exclude(self):
+        out = self._run_selftest_with_suite(starting_with=["bt."],
+            exclude_pattern="test_skip$")
+        self.assertNotContainsRe(out, "test_skip")
+
+    def test_additonal_decorator(self):
+        out = self._run_selftest_with_suite(
+            suite_decorators=[tests.TestDecorator])
+
+
+class TestUncollectedWarningsSubunit(TestUncollectedWarnings):
+    """Check warnings from tests staying alive are emitted with subunit"""
+
+    _test_needs_features = [features.subunit]
+
+    def _run_selftest_with_suite(self, **kwargs):
+        return TestUncollectedWarnings._run_selftest_with_suite(self,
+            runner_class=tests.SubUnitBzrRunner, **kwargs)
+
+
+class TestUncollectedWarningsForked(_ForkedSelftest, TestUncollectedWarnings):
+    """Check warnings from tests staying alive are emitted when forking"""
+
+
 class TestEnvironHandling(tests.TestCase):
 
     def test_overrideEnv_None_called_twice_doesnt_leak(self):

=== modified file 'bzrlib/tests/test_server.py'
--- bzrlib/tests/test_server.py	2011-08-19 22:34:02 +0000
+++ bzrlib/tests/test_server.py	2011-11-24 15:48:29 +0000
@@ -19,10 +19,12 @@
 import SocketServer
 import sys
 import threading
+import traceback
 
 
 from bzrlib import (
     cethread,
+    errors,
     osutils,
     transport,
     urlutils,
@@ -265,7 +267,7 @@
             #raise AssertionError('thread %s hung' % (self.name,))
 
 
-class TestingTCPServerMixin:
+class TestingTCPServerMixin(object):
     """Mixin to support running SocketServer.TCPServer in a thread.
 
     Tests are connecting from the main thread, the server has to be run in a
@@ -287,7 +289,6 @@
 
     def serve(self):
         self.serving = True
-        self.stopped.clear()
         # We are listening and ready to accept connections
         self.started.set()
         try:
@@ -312,7 +313,8 @@
                 self.process_request(request, client_address)
             except:
                 self.handle_error(request, client_address)
-                self.close_request(request)
+        else:
+            self.close_request(request)
 
     def get_request(self):
         return self.socket.accept()
@@ -332,8 +334,16 @@
         # The following can be used for debugging purposes, it will display the
         # exception and the traceback just when it occurs instead of waiting
         # for the thread to be joined.
-
         # SocketServer.BaseServer.handle_error(self, request, client_address)
+
+        # We call close_request manually, because we are going to raise an
+        # exception. The SocketServer implementation calls:
+        #   handle_error(...)
+        #   close_request(...)
+        # But because we raise the exception, close_request will never be
+        # triggered. This helps client not block waiting for a response when
+        # the server gets an exception.
+        self.close_request(request)
         raise
 
     def ignored_exceptions_during_shutdown(self, e):
@@ -419,15 +429,18 @@
         SocketServer.ThreadingTCPServer.__init__(self, server_address,
                                                  request_handler_class)
 
-    def get_request (self):
+    def get_request(self):
         """Get the request and client address from the socket."""
         sock, addr = TestingTCPServerMixin.get_request(self)
-        # The thread is not create yet, it will be updated in process_request
+        # The thread is not created yet, it will be updated in process_request
         self.clients.append((sock, addr, None))
         return sock, addr
 
-    def process_request_thread(self, started, stopped, request, client_address):
+    def process_request_thread(self, started, detached, stopped,
+                               request, client_address):
         started.set()
+        # We will be on our own once the server tells us we're detached
+        detached.wait()
         SocketServer.ThreadingTCPServer.process_request_thread(
             self, request, client_address)
         self.close_request(request)
@@ -436,12 +449,13 @@
     def process_request(self, request, client_address):
         """Start a new thread to process the request."""
         started = threading.Event()
+        detached = threading.Event()
         stopped = threading.Event()
         t = TestThread(
             sync_event=stopped,
             name='%s -> %s' % (client_address, self.server_address),
             target = self.process_request_thread,
-            args = (started, stopped, request, client_address))
+            args = (started, detached, stopped, request, client_address))
         # Update the client description
         self.clients.pop()
         self.clients.append((request, client_address, t))
@@ -450,10 +464,12 @@
         t.set_ignored_exceptions(self.ignored_exceptions)
         t.start()
         started.wait()
-        if debug_threads():
-            sys.stderr.write('Client thread %s started\n' % (t.name,))
         # If an exception occured during the thread start, it will get raised.
         t.pending_exception()
+        if debug_threads():
+            sys.stderr.write('Client thread %s started\n' % (t.name,))
+        # Tell the thread, it's now on its own for exception handling.
+        detached.set()
 
     # The following methods are called by the main thread
 
@@ -508,7 +524,7 @@
             sync_event=self.server.started,
             target=self.run_server)
         self._server_thread.start()
-        # Wait for the server thread to start (i.e release the lock)
+        # Wait for the server thread to start (i.e. release the lock)
         self.server.started.wait()
         # Get the real address, especially the port
         self.host, self.port = self.server.server_address
@@ -589,16 +605,23 @@
     def __init__(self, request, client_address, server):
         medium.SmartServerSocketStreamMedium.__init__(
             self, request, server.backing_transport,
-            server.root_client_path)
+            server.root_client_path,
+            timeout=_DEFAULT_TESTING_CLIENT_TIMEOUT)
         request.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
         SocketServer.BaseRequestHandler.__init__(self, request, client_address,
                                                  server)
 
     def handle(self):
-        while not self.finished:
-            server_protocol = self._build_protocol()
-            self._serve_one_request(server_protocol)
-
+        try:
+            while not self.finished:
+                server_protocol = self._build_protocol()
+                self._serve_one_request(server_protocol)
+        except errors.ConnectionTimeout:
+            # idle connections aren't considered a failure of the server
+            return
+
+
+_DEFAULT_TESTING_CLIENT_TIMEOUT = 60.0
 
 class TestingSmartServer(TestingThreadingTCPServer, server.SmartTCPServer):
 
@@ -607,7 +630,8 @@
         TestingThreadingTCPServer.__init__(self, server_address,
                                            request_handler_class)
         server.SmartTCPServer.__init__(self, backing_transport,
-                                       root_client_path)
+            root_client_path, client_timeout=_DEFAULT_TESTING_CLIENT_TIMEOUT)
+
     def serve(self):
         self.run_server_started_hooks()
         try:

=== modified file 'bzrlib/tests/test_sftp_transport.py'
--- bzrlib/tests/test_sftp_transport.py	2011-08-19 22:34:02 +0000
+++ bzrlib/tests/test_sftp_transport.py	2011-12-05 09:52:49 +0000
@@ -199,13 +199,13 @@
         b2 = bzrdir.BzrDir.create_branch_and_repo(self.get_url('/b'))
         b2.pull(b)
 
-        self.assertEquals(b2.revision_history(), ['a1'])
+        self.assertEquals(b2.last_revision(), 'a1')
 
         open('a/foo', 'wt').write('something new in foo\n')
         t.commit('new', rev_id='a2')
         b2.pull(b)
 
-        self.assertEquals(b2.revision_history(), ['a1', 'a2'])
+        self.assertEquals(b2.last_revision(), 'a2')
 
 
 class SSHVendorConnection(TestCaseWithSFTPServer):
@@ -279,9 +279,12 @@
         self.addCleanup(s.close)
         self.bogus_url = 'sftp://%s:%s/' % s.getsockname()
 
-    def set_vendor(self, vendor):
+    def set_vendor(self, vendor, subprocess_stderr=None):
         from bzrlib.transport import ssh
         self.overrideAttr(ssh._ssh_vendor_manager, '_cached_ssh_vendor', vendor)
+        if subprocess_stderr is not None:
+            self.overrideAttr(ssh.SubprocessVendor, "_stderr_target",
+                subprocess_stderr)
 
     def test_bad_connection_paramiko(self):
         """Test that a real connection attempt raises the right error"""
@@ -292,32 +295,16 @@
 
     def test_bad_connection_ssh(self):
         """None => auto-detect vendor"""
-        self.set_vendor(None)
-        # This is how I would normally test the connection code
-        # it makes it very clear what we are testing.
-        # However, 'ssh' will create stipple on the output, so instead
-        # I'm using run_bzr_subprocess, and parsing the output
-        # try:
-        #     t = _mod_transport.get_transport(self.bogus_url)
-        # except errors.ConnectionError:
-        #     # Correct error
-        #     pass
-        # except errors.NameError, e:
-        #     if 'SSHException' in str(e):
-        #         raise TestSkipped('Known NameError bug in paramiko 1.6.1')
-        #     raise
-        # else:
-        #     self.fail('Excepted ConnectionError to be raised')
-
-        out, err = self.run_bzr_subprocess(['log', self.bogus_url], retcode=3)
-        self.assertEqual('', out)
-        if "NameError: global name 'SSHException'" in err:
-            # We aren't fixing this bug, because it is a bug in
-            # paramiko, but we know about it, so we don't have to
-            # fail the test
-            raise TestSkipped('Known NameError bug with paramiko-1.6.1')
-        self.assertContainsRe(err, r'bzr: ERROR: Unable to connect to SSH host'
-                                   r' 127\.0\.0\.1:\d+; ')
+        f = file(os.devnull, "wb")
+        self.addCleanup(f.close)
+        self.set_vendor(None, f)
+        t = _mod_transport.get_transport_from_url(self.bogus_url)
+        try:
+            self.assertRaises(errors.ConnectionError, t.get, 'foobar')
+        except NameError, e:
+            if "global name 'SSHException'" in str(e):
+                self.knownFailure('Known NameError bug in paramiko 1.6.1')
+            raise
 
 
 class SFTPLatencyKnob(TestCaseWithSFTPServer):

=== modified file 'bzrlib/tests/test_shelf_ui.py'
--- bzrlib/tests/test_shelf_ui.py	2011-07-08 23:01:39 +0000
+++ bzrlib/tests/test_shelf_ui.py	2011-10-10 20:55:52 +0000
@@ -44,16 +44,18 @@
         self.expected = []
         self.diff_writer = StringIO()
 
-    def expect(self, prompt, response):
-        self.expected.append((prompt, response))
+    def expect(self, message, response):
+        self.expected.append((message, response))
 
-    def prompt(self, message):
+    def prompt(self, message, choices, default):
         try:
-            prompt, response = self.expected.pop(0)
+            expected_message, response = self.expected.pop(0)
         except IndexError:
             raise AssertionError('Unexpected prompt: %s' % message)
-        if prompt != message:
+        if message != expected_message:
             raise AssertionError('Wrong prompt: %s' % message)
+        if choices != '&yes\n&No\n&finish\n&quit':
+            raise AssertionError('Wrong choices: %s' % choices)
         return response
 
 
@@ -86,7 +88,7 @@
         shelver = ExpectShelver(tree, tree.basis_tree())
         self.addCleanup(shelver.finalize)
         e = self.assertRaises(AssertionError, shelver.run)
-        self.assertEqual('Unexpected prompt: Shelve? [yNfq?]', str(e))
+        self.assertEqual('Unexpected prompt: Shelve?', str(e))
 
     def test_wrong_prompt_failure(self):
         tree = self.create_shelvable_tree()
@@ -94,9 +96,9 @@
         self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         self.addCleanup(shelver.finalize)
-        shelver.expect('foo', 'y')
+        shelver.expect('foo', 0)
         e = self.assertRaises(AssertionError, shelver.run)
-        self.assertEqual('Wrong prompt: Shelve? [yNfq?]', str(e))
+        self.assertEqual('Wrong prompt: Shelve?', str(e))
 
     def test_shelve_not_diff(self):
         tree = self.create_shelvable_tree()
@@ -104,8 +106,8 @@
         self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Shelve? [yNfq?]', 'n')
-        shelver.expect('Shelve? [yNfq?]', 'n')
+        shelver.expect('Shelve?', 1)
+        shelver.expect('Shelve?', 1)
         # No final shelving prompt because no changes were selected
         shelver.run()
         self.assertFileEqual(LINES_ZY, 'tree/foo')
@@ -116,9 +118,9 @@
         self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Shelve? [yNfq?]', 'y')
-        shelver.expect('Shelve? [yNfq?]', 'y')
-        shelver.expect('Shelve 2 change(s)? [yNfq?]', 'n')
+        shelver.expect('Shelve?', 0)
+        shelver.expect('Shelve?', 0)
+        shelver.expect('Shelve 2 change(s)?', 1)
         shelver.run()
         self.assertFileEqual(LINES_ZY, 'tree/foo')
 
@@ -128,9 +130,9 @@
         self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Shelve? [yNfq?]', 'y')
-        shelver.expect('Shelve? [yNfq?]', 'y')
-        shelver.expect('Shelve 2 change(s)? [yNfq?]', 'y')
+        shelver.expect('Shelve?', 0)
+        shelver.expect('Shelve?', 0)
+        shelver.expect('Shelve 2 change(s)?', 0)
         shelver.run()
         self.assertFileEqual(LINES_AJ, 'tree/foo')
 
@@ -140,9 +142,9 @@
         self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Shelve? [yNfq?]', 'y')
-        shelver.expect('Shelve? [yNfq?]', 'n')
-        shelver.expect('Shelve 1 change(s)? [yNfq?]', 'y')
+        shelver.expect('Shelve?', 0)
+        shelver.expect('Shelve?', 1)
+        shelver.expect('Shelve 1 change(s)?', 0)
         shelver.run()
         self.assertFileEqual(LINES_AY, 'tree/foo')
 
@@ -153,8 +155,8 @@
         self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Shelve binary changes? [yNfq?]', 'y')
-        shelver.expect('Shelve 1 change(s)? [yNfq?]', 'y')
+        shelver.expect('Shelve binary changes?', 0)
+        shelver.expect('Shelve 1 change(s)?', 0)
         shelver.run()
         self.assertFileEqual(LINES_AJ, 'tree/foo')
 
@@ -165,10 +167,10 @@
         self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Shelve renaming "foo" => "bar"? [yNfq?]', 'y')
-        shelver.expect('Shelve? [yNfq?]', 'y')
-        shelver.expect('Shelve? [yNfq?]', 'y')
-        shelver.expect('Shelve 3 change(s)? [yNfq?]', 'y')
+        shelver.expect('Shelve renaming "foo" => "bar"?', 0)
+        shelver.expect('Shelve?', 0)
+        shelver.expect('Shelve?', 0)
+        shelver.expect('Shelve 3 change(s)?', 0)
         shelver.run()
         self.assertFileEqual(LINES_AJ, 'tree/foo')
 
@@ -179,8 +181,8 @@
         self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Shelve removing file "foo"? [yNfq?]', 'y')
-        shelver.expect('Shelve 1 change(s)? [yNfq?]', 'y')
+        shelver.expect('Shelve removing file "foo"?', 0)
+        shelver.expect('Shelve 1 change(s)?', 0)
         shelver.run()
         self.assertFileEqual(LINES_AJ, 'tree/foo')
 
@@ -193,8 +195,8 @@
         self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Shelve adding file "foo"? [yNfq?]', 'y')
-        shelver.expect('Shelve 1 change(s)? [yNfq?]', 'y')
+        shelver.expect('Shelve adding file "foo"?', 0)
+        shelver.expect('Shelve 1 change(s)?', 0)
         shelver.run()
         self.assertPathDoesNotExist('tree/foo')
 
@@ -206,9 +208,9 @@
         self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Shelve changing "foo" from file to directory? [yNfq?]',
-                       'y')
-        shelver.expect('Shelve 1 change(s)? [yNfq?]', 'y')
+        shelver.expect('Shelve changing "foo" from file to directory?',
+                       0)
+        shelver.expect('Shelve 1 change(s)?', 0)
 
     def test_shelve_modify_target(self):
         self.requireFeature(features.SymlinkFeature)
@@ -223,8 +225,8 @@
         shelver = ExpectShelver(tree, tree.basis_tree())
         self.addCleanup(shelver.finalize)
         shelver.expect('Shelve changing target of "baz" from "bar" to '
-                '"vax"? [yNfq?]', 'y')
-        shelver.expect('Shelve 1 change(s)? [yNfq?]', 'y')
+                '"vax"?', 0)
+        shelver.expect('Shelve 1 change(s)?', 0)
         shelver.run()
         self.assertEqual('bar', os.readlink('tree/baz'))
 
@@ -234,8 +236,8 @@
         self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Shelve? [yNfq?]', 'f')
-        shelver.expect('Shelve 2 change(s)? [yNfq?]', 'y')
+        shelver.expect('Shelve?', 2)
+        shelver.expect('Shelve 2 change(s)?', 0)
         shelver.run()
         self.assertFileEqual(LINES_AJ, 'tree/foo')
 
@@ -245,7 +247,7 @@
         self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Shelve? [yNfq?]', 'q')
+        shelver.expect('Shelve?', 3)
         self.assertRaises(errors.UserAbort, shelver.run)
         self.assertFileEqual(LINES_ZY, 'tree/foo')
 
@@ -267,19 +269,8 @@
         self.addCleanup(tree.unlock)
         shelver = ExpectShelver(tree, tree.basis_tree(), file_list=['bar'])
         self.addCleanup(shelver.finalize)
-        shelver.expect('Shelve adding file "bar"? [yNfq?]', 'y')
-        shelver.expect('Shelve 1 change(s)? [yNfq?]', 'y')
-        shelver.run()
-
-    def test_shelve_help(self):
-        tree = self.create_shelvable_tree()
-        tree.lock_tree_write()
-        self.addCleanup(tree.unlock)
-        shelver = ExpectShelver(tree, tree.basis_tree())
-        self.addCleanup(shelver.finalize)
-        shelver.expect('Shelve? [yNfq?]', '?')
-        shelver.expect('Shelve? [(y)es, (N)o, (f)inish, or (q)uit]', 'f')
-        shelver.expect('Shelve 2 change(s)? [yNfq?]', 'y')
+        shelver.expect('Shelve adding file "bar"?', 0)
+        shelver.expect('Shelve 1 change(s)?', 0)
         shelver.run()
 
     def test_shelve_destroy(self):
@@ -350,8 +341,8 @@
         shelver = ExpectShelver(tree, tree.basis_tree(),
                                 reporter=shelf_ui.ApplyReporter())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Apply change? [yNfq?]', 'n')
-        shelver.expect('Apply change? [yNfq?]', 'n')
+        shelver.expect('Apply change?', 1)
+        shelver.expect('Apply change?', 1)
         # No final shelving prompt because no changes were selected
         shelver.run()
         self.assertFileEqual(LINES_ZY, 'tree/foo')
@@ -363,9 +354,9 @@
         shelver = ExpectShelver(tree, tree.basis_tree(),
                                 reporter=shelf_ui.ApplyReporter())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Apply change? [yNfq?]', 'y')
-        shelver.expect('Apply change? [yNfq?]', 'y')
-        shelver.expect('Apply 2 change(s)? [yNfq?]', 'n')
+        shelver.expect('Apply change?', 0)
+        shelver.expect('Apply change?', 0)
+        shelver.expect('Apply 2 change(s)?', 1)
         shelver.run()
         self.assertFileEqual(LINES_ZY, 'tree/foo')
 
@@ -376,9 +367,9 @@
         shelver = ExpectShelver(tree, tree.basis_tree(),
                                 reporter=shelf_ui.ApplyReporter())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Apply change? [yNfq?]', 'y')
-        shelver.expect('Apply change? [yNfq?]', 'y')
-        shelver.expect('Apply 2 change(s)? [yNfq?]', 'y')
+        shelver.expect('Apply change?', 0)
+        shelver.expect('Apply change?', 0)
+        shelver.expect('Apply 2 change(s)?', 0)
         shelver.run()
         self.assertFileEqual(LINES_AJ, 'tree/foo')
 
@@ -390,8 +381,8 @@
         shelver = ExpectShelver(tree, tree.basis_tree(),
                                 reporter=shelf_ui.ApplyReporter())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Apply binary changes? [yNfq?]', 'y')
-        shelver.expect('Apply 1 change(s)? [yNfq?]', 'y')
+        shelver.expect('Apply binary changes?', 0)
+        shelver.expect('Apply 1 change(s)?', 0)
         shelver.run()
         self.assertFileEqual(LINES_AJ, 'tree/foo')
 
@@ -403,10 +394,10 @@
         shelver = ExpectShelver(tree, tree.basis_tree(),
                                 reporter=shelf_ui.ApplyReporter())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Rename "bar" => "foo"? [yNfq?]', 'y')
-        shelver.expect('Apply change? [yNfq?]', 'y')
-        shelver.expect('Apply change? [yNfq?]', 'y')
-        shelver.expect('Apply 3 change(s)? [yNfq?]', 'y')
+        shelver.expect('Rename "bar" => "foo"?', 0)
+        shelver.expect('Apply change?', 0)
+        shelver.expect('Apply change?', 0)
+        shelver.expect('Apply 3 change(s)?', 0)
         shelver.run()
         self.assertFileEqual(LINES_AJ, 'tree/foo')
 
@@ -418,8 +409,8 @@
         shelver = ExpectShelver(tree, tree.basis_tree(),
                                 reporter=shelf_ui.ApplyReporter())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Add file "foo"? [yNfq?]', 'y')
-        shelver.expect('Apply 1 change(s)? [yNfq?]', 'y')
+        shelver.expect('Add file "foo"?', 0)
+        shelver.expect('Apply 1 change(s)?', 0)
         shelver.run()
         self.assertFileEqual(LINES_AJ, 'tree/foo')
 
@@ -433,8 +424,8 @@
         shelver = ExpectShelver(tree, tree.basis_tree(),
                                 reporter=shelf_ui.ApplyReporter())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Delete file "foo"? [yNfq?]', 'y')
-        shelver.expect('Apply 1 change(s)? [yNfq?]', 'y')
+        shelver.expect('Delete file "foo"?', 0)
+        shelver.expect('Apply 1 change(s)?', 0)
         shelver.run()
         self.assertPathDoesNotExist('tree/foo')
 
@@ -447,8 +438,8 @@
         shelver = ExpectShelver(tree, tree.basis_tree(),
                                reporter=shelf_ui.ApplyReporter())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Change "foo" from directory to a file? [yNfq?]', 'y')
-        shelver.expect('Apply 1 change(s)? [yNfq?]', 'y')
+        shelver.expect('Change "foo" from directory to a file?', 0)
+        shelver.expect('Apply 1 change(s)?', 0)
 
     def test_shelve_modify_target(self):
         self.requireFeature(features.SymlinkFeature)
@@ -463,9 +454,9 @@
         shelver = ExpectShelver(tree, tree.basis_tree(),
                                 reporter=shelf_ui.ApplyReporter())
         self.addCleanup(shelver.finalize)
-        shelver.expect('Change target of "baz" from "vax" to "bar"? [yNfq?]',
-                       'y')
-        shelver.expect('Apply 1 change(s)? [yNfq?]', 'y')
+        shelver.expect('Change target of "baz" from "vax" to "bar"?',
+                       0)
+        shelver.expect('Apply 1 change(s)?', 0)
         shelver.run()
         self.assertEqual('bar', os.readlink('tree/baz'))
 

=== modified file 'bzrlib/tests/test_smart.py'
--- bzrlib/tests/test_smart.py	2011-08-19 22:34:02 +0000
+++ bzrlib/tests/test_smart.py	2011-12-14 21:24:07 +0000
@@ -25,11 +25,14 @@
 """
 
 import bz2
+import zlib
 
 from bzrlib import (
     branch as _mod_branch,
     bzrdir,
     errors,
+    gpg,
+    inventory_delta,
     tests,
     transport,
     urlutils,
@@ -44,6 +47,7 @@
     server,
     vfs,
     )
+from bzrlib.testament import Testament
 from bzrlib.tests import test_server
 from bzrlib.transport import (
     chroot,
@@ -221,6 +225,104 @@
         self.assertEqual(expected, request.execute('', 'False'))
 
 
+class TestSmartServerBzrDirRequestCloningMetaDir(
+    tests.TestCaseWithMemoryTransport):
+    """Tests for BzrDir.checkout_metadir."""
+
+    def test_checkout_metadir(self):
+        backing = self.get_transport()
+        request = smart_dir.SmartServerBzrDirRequestCheckoutMetaDir(
+            backing)
+        branch = self.make_branch('.', format='2a')
+        response = request.execute('')
+        self.assertEqual(
+            smart_req.SmartServerResponse(
+                ('Bazaar-NG meta directory, format 1\n',
+                 'Bazaar repository format 2a (needs bzr 1.16 or later)\n',
+                 'Bazaar Branch Format 7 (needs bzr 1.6)\n')),
+            response)
+
+
+class TestSmartServerBzrDirRequestDestroyBranch(
+    tests.TestCaseWithMemoryTransport):
+    """Tests for BzrDir.destroy_branch."""
+
+    def test_destroy_branch_default(self):
+        """The default branch can be removed."""
+        backing = self.get_transport()
+        dir = self.make_branch('.').bzrdir
+        request_class = smart_dir.SmartServerBzrDirRequestDestroyBranch
+        request = request_class(backing)
+        expected = smart_req.SuccessfulSmartServerResponse(('ok',))
+        self.assertEqual(expected, request.execute('', None))
+
+    def test_destroy_branch_named(self):
+        """A named branch can be removed."""
+        backing = self.get_transport()
+        dir = self.make_repository('.', format="development-colo").bzrdir
+        dir.create_branch(name="branchname")
+        request_class = smart_dir.SmartServerBzrDirRequestDestroyBranch
+        request = request_class(backing)
+        expected = smart_req.SuccessfulSmartServerResponse(('ok',))
+        self.assertEqual(expected, request.execute('', "branchname"))
+
+    def test_destroy_branch_missing(self):
+        """An error is raised if the branch didn't exist."""
+        backing = self.get_transport()
+        dir = self.make_bzrdir('.', format="development-colo")
+        request_class = smart_dir.SmartServerBzrDirRequestDestroyBranch
+        request = request_class(backing)
+        expected = smart_req.FailedSmartServerResponse(('nobranch',), None)
+        self.assertEqual(expected, request.execute('', "branchname"))
+
+
+class TestSmartServerBzrDirRequestHasWorkingTree(
+    tests.TestCaseWithTransport):
+    """Tests for BzrDir.has_workingtree."""
+
+    def test_has_workingtree_yes(self):
+        """A working tree is present."""
+        backing = self.get_transport()
+        dir = self.make_branch_and_tree('.').bzrdir
+        request_class = smart_dir.SmartServerBzrDirRequestHasWorkingTree
+        request = request_class(backing)
+        expected = smart_req.SuccessfulSmartServerResponse(('yes',))
+        self.assertEqual(expected, request.execute(''))
+
+    def test_has_workingtree_no(self):
+        """A working tree is missing."""
+        backing = self.get_transport()
+        dir = self.make_bzrdir('.')
+        request_class = smart_dir.SmartServerBzrDirRequestHasWorkingTree
+        request = request_class(backing)
+        expected = smart_req.SuccessfulSmartServerResponse(('no',))
+        self.assertEqual(expected, request.execute(''))
+
+
+class TestSmartServerBzrDirRequestDestroyRepository(
+    tests.TestCaseWithMemoryTransport):
+    """Tests for BzrDir.destroy_repository."""
+
+    def test_destroy_repository_default(self):
+        """The repository can be removed."""
+        backing = self.get_transport()
+        dir = self.make_repository('.').bzrdir
+        request_class = smart_dir.SmartServerBzrDirRequestDestroyRepository
+        request = request_class(backing)
+        expected = smart_req.SuccessfulSmartServerResponse(('ok',))
+        self.assertEqual(expected, request.execute(''))
+
+    def test_destroy_repository_missing(self):
+        """An error is raised if the repository didn't exist."""
+        backing = self.get_transport()
+        dir = self.make_bzrdir('.')
+        request_class = smart_dir.SmartServerBzrDirRequestDestroyRepository
+        request = request_class(backing)
+        expected = smart_req.FailedSmartServerResponse(
+            ('norepository',), None)
+        self.assertEqual(expected, request.execute(''))
+
+
 class TestSmartServerRequestCreateRepository(tests.TestCaseWithMemoryTransport):
     """Tests for BzrDir.create_repository."""
 
@@ -735,6 +837,41 @@
             request.execute(''))
 
 
+class TestSmartServerBranchRequestRevisionIdToRevno(
+    tests.TestCaseWithMemoryTransport):
+
+    def test_null(self):
+        backing = self.get_transport()
+        request = smart_branch.SmartServerBranchRequestRevisionIdToRevno(
+            backing)
+        self.make_branch('.')
+        self.assertEqual(smart_req.SmartServerResponse(('ok', '0')),
+            request.execute('', 'null:'))
+
+    def test_simple(self):
+        backing = self.get_transport()
+        request = smart_branch.SmartServerBranchRequestRevisionIdToRevno(
+            backing)
+        tree = self.make_branch_and_memory_tree('.')
+        tree.lock_write()
+        tree.add('')
+        r1 = tree.commit('1st commit')
+        tree.unlock()
+        self.assertEqual(
+            smart_req.SmartServerResponse(('ok', '1')),
+            request.execute('', r1))
+
+    def test_not_found(self):
+        backing = self.get_transport()
+        request = smart_branch.SmartServerBranchRequestRevisionIdToRevno(
+            backing)
+        branch = self.make_branch('.')
+        self.assertEqual(
+            smart_req.FailedSmartServerResponse(
+                ('NoSuchRevision', 'idontexist')),
+            request.execute('', 'idontexist'))
+
+
 class TestSmartServerBranchRequestGetConfigFile(
     tests.TestCaseWithMemoryTransport):
 
@@ -769,6 +906,23 @@
         return branch_token, repo_token
 
 
+class TestSmartServerBranchRequestPutConfigFile(TestLockedBranch):
+
+    def test_with_content(self):
+        backing = self.get_transport()
+        request = smart_branch.SmartServerBranchPutConfigFile(backing)
+        branch = self.make_branch('.')
+        branch_token, repo_token = self.get_lock_tokens(branch)
+        self.assertIs(None, request.execute('', branch_token, repo_token))
+        self.assertEqual(
+            smart_req.SmartServerResponse(('ok', )),
+            request.do_body('foo bar baz'))
+        self.assertEquals(
+            branch.control_transport.get_bytes('branch.conf'),
+            'foo bar baz')
+        branch.unlock()
+
+
 class TestSmartServerBranchRequestSetConfigOption(TestLockedBranch):
 
     def test_value_name(self):
@@ -1088,6 +1242,26 @@
         self.assertEqual('child-1', self.tree.branch.last_revision())
 
 
+class TestSmartServerBranchBreakLock(tests.TestCaseWithMemoryTransport):
+
+    def test_lock_to_break(self):
+        base_branch = self.make_branch('base')
+        request = smart_branch.SmartServerBranchBreakLock(
+            self.get_transport())
+        base_branch.lock_write()
+        self.assertEqual(
+            smart_req.SuccessfulSmartServerResponse(('ok', ), None),
+            request.execute('base'))
+
+    def test_nothing_to_break(self):
+        base_branch = self.make_branch('base')
+        request = smart_branch.SmartServerBranchBreakLock(
+            self.get_transport())
+        self.assertEqual(
+            smart_req.SuccessfulSmartServerResponse(('ok', ), None),
+            request.execute('base'))
+
+
 class TestSmartServerBranchRequestGetParent(tests.TestCaseWithMemoryTransport):
 
     def test_get_parent_none(self):
@@ -1275,6 +1449,34 @@
         self.assertEqual('LockFailed', error_name)
 
 
+class TestSmartServerBranchRequestGetPhysicalLockStatus(TestLockedBranch):
+
+    def setUp(self):
+        tests.TestCaseWithMemoryTransport.setUp(self)
+
+    def test_true(self):
+        backing = self.get_transport()
+        request = smart_branch.SmartServerBranchRequestGetPhysicalLockStatus(
+            backing)
+        branch = self.make_branch('.')
+        branch_token, repo_token = self.get_lock_tokens(branch)
+        self.assertEquals(True, branch.get_physical_lock_status())
+        response = request.execute('')
+        self.assertEqual(
+            smart_req.SmartServerResponse(('yes',)), response)
+        branch.unlock()
+
+    def test_false(self):
+        backing = self.get_transport()
+        request = smart_branch.SmartServerBranchRequestGetPhysicalLockStatus(
+            backing)
+        branch = self.make_branch('.')
+        self.assertEquals(False, branch.get_physical_lock_status())
+        response = request.execute('')
+        self.assertEqual(
+            smart_req.SmartServerResponse(('no',)), response)
+
+
 class TestSmartServerBranchRequestUnlock(TestLockedBranch):
 
     def setUp(self):
@@ -1345,6 +1547,79 @@
             request.execute, 'subdir')
 
 
+class TestSmartServerRepositoryAddSignatureText(tests.TestCaseWithMemoryTransport):
+
+    def test_add_text(self):
+        backing = self.get_transport()
+        request = smart_repo.SmartServerRepositoryAddSignatureText(backing)
+        tree = self.make_branch_and_memory_tree('.')
+        write_token = tree.lock_write()
+        self.addCleanup(tree.unlock)
+        tree.add('')
+        tree.commit("Message", rev_id='rev1')
+        tree.branch.repository.start_write_group()
+        write_group_tokens = tree.branch.repository.suspend_write_group()
+        self.assertEqual(None, request.execute('', write_token,
+            'rev1', *write_group_tokens))
+        response = request.do_body('somesignature')
+        self.assertTrue(response.is_successful())
+        self.assertEqual(response.args[0], 'ok')
+        write_group_tokens = response.args[1:]
+        tree.branch.repository.resume_write_group(write_group_tokens)
+        tree.branch.repository.commit_write_group()
+        tree.unlock()
+        self.assertEqual("somesignature",
+            tree.branch.repository.get_signature_text("rev1"))
+
+
+class TestSmartServerRepositoryAllRevisionIds(
+    tests.TestCaseWithMemoryTransport):
+
+    def test_empty(self):
+        """An empty body should be returned for an empty repository."""
+        backing = self.get_transport()
+        request = smart_repo.SmartServerRepositoryAllRevisionIds(backing)
+        self.make_repository('.')
+        self.assertEquals(
+            smart_req.SuccessfulSmartServerResponse(("ok", ), ""),
+            request.execute(''))
+
+    def test_some_revisions(self):
+        """An empty body should be returned for an empty repository."""
+        backing = self.get_transport()
+        request = smart_repo.SmartServerRepositoryAllRevisionIds(backing)
+        tree = self.make_branch_and_memory_tree('.')
+        tree.lock_write()
+        tree.add('')
+        tree.commit(rev_id='origineel', message="message")
+        tree.commit(rev_id='nog-een-revisie', message="message")
+        tree.unlock()
+        self.assertEquals(
+            smart_req.SuccessfulSmartServerResponse(("ok", ),
+                "origineel\nnog-een-revisie"),
+            request.execute(''))
+
+
+class TestSmartServerRepositoryBreakLock(tests.TestCaseWithMemoryTransport):
+
+    def test_lock_to_break(self):
+        backing = self.get_transport()
+        request = smart_repo.SmartServerRepositoryBreakLock(backing)
+        tree = self.make_branch_and_memory_tree('.')
+        tree.branch.repository.lock_write()
+        self.assertEqual(
+            smart_req.SuccessfulSmartServerResponse(('ok', ), None),
+            request.execute(''))
+
+    def test_nothing_to_break(self):
+        backing = self.get_transport()
+        request = smart_repo.SmartServerRepositoryBreakLock(backing)
+        tree = self.make_branch_and_memory_tree('.')
+        self.assertEqual(
+            smart_req.SuccessfulSmartServerResponse(('ok', ), None),
+            request.execute(''))
+
+
 class TestSmartServerRepositoryGetParentMap(tests.TestCaseWithMemoryTransport):
 
     def test_trivial_bzipped(self):
@@ -1470,6 +1745,50 @@
             request.execute('stacked', 1, (3, r3)))
 
 
+class TestSmartServerRepositoryIterRevisions(
+    tests.TestCaseWithMemoryTransport):
+
+    def test_basic(self):
+        backing = self.get_transport()
+        request = smart_repo.SmartServerRepositoryIterRevisions(backing)
+        tree = self.make_branch_and_memory_tree('.', format='2a')
+        tree.lock_write()
+        tree.add('')
+        tree.commit('1st commit', rev_id="rev1")
+        tree.commit('2nd commit', rev_id="rev2")
+        tree.unlock()
+
+        self.assertIs(None, request.execute(''))
+        response = request.do_body("rev1\nrev2")
+        self.assertTrue(response.is_successful())
+        # Format 2a uses serializer format 10
+        self.assertEquals(response.args, ("ok", "10"))
+
+        self.addCleanup(tree.branch.lock_read().unlock)
+        entries = [zlib.compress(record.get_bytes_as("fulltext")) for record in
+            tree.branch.repository.revisions.get_record_stream(
+            [("rev1", ), ("rev2", )], "unordered", True)]
+
+        contents = "".join(response.body_stream)
+        self.assertTrue(contents in (
+            "".join([entries[0], entries[1]]),
+            "".join([entries[1], entries[0]])))
+
+    def test_missing(self):
+        backing = self.get_transport()
+        request = smart_repo.SmartServerRepositoryIterRevisions(backing)
+        tree = self.make_branch_and_memory_tree('.', format='2a')
+
+        self.assertIs(None, request.execute(''))
+        response = request.do_body("rev1\nrev2")
+        self.assertTrue(response.is_successful())
+        # Format 2a uses serializer format 10
+        self.assertEquals(response.args, ("ok", "10"))
+
+        contents = "".join(response.body_stream)
+        self.assertEquals(contents, "")
+
+
 class GetStreamTestBase(tests.TestCaseWithMemoryTransport):
 
     def make_two_commit_repo(self):
@@ -1549,6 +1868,83 @@
             request.execute('', rev_id_utf8))
 
 
+class TestSmartServerRepositoryIterFilesBytes(tests.TestCaseWithTransport):
+
+    def test_single(self):
+        backing = self.get_transport()
+        request = smart_repo.SmartServerRepositoryIterFilesBytes(backing)
+        t = self.make_branch_and_tree('.')
+        self.addCleanup(t.lock_write().unlock)
+        self.build_tree_contents([("file", "somecontents")])
+        t.add(["file"], ["thefileid"])
+        t.commit(rev_id='somerev', message="add file")
+        self.assertIs(None, request.execute(''))
+        response = request.do_body("thefileid\0somerev\n")
+        self.assertTrue(response.is_successful())
+        self.assertEquals(response.args, ("ok", ))
+        self.assertEquals("".join(response.body_stream),
+            "ok\x000\n" + zlib.compress("somecontents"))
+
+    def test_missing(self):
+        backing = self.get_transport()
+        request = smart_repo.SmartServerRepositoryIterFilesBytes(backing)
+        t = self.make_branch_and_tree('.')
+        self.addCleanup(t.lock_write().unlock)
+        self.assertIs(None, request.execute(''))
+        response = request.do_body("thefileid\0revision\n")
+        self.assertTrue(response.is_successful())
+        self.assertEquals(response.args, ("ok", ))
+        self.assertEquals("".join(response.body_stream),
+            "absent\x00thefileid\x00revision\x000\n")
+
+
+class TestSmartServerRequestHasSignatureForRevisionId(
+        tests.TestCaseWithMemoryTransport):
+
+    def test_missing_revision(self):
+        """For a missing revision, NoSuchRevision is returned."""
+        backing = self.get_transport()
+        request = smart_repo.SmartServerRequestHasSignatureForRevisionId(
+            backing)
+        self.make_repository('.')
+        self.assertEqual(
+            smart_req.FailedSmartServerResponse(
+                ('nosuchrevision', 'revid'), None),
+            request.execute('', 'revid'))
+
+    def test_missing_signature(self):
+        """For a missing signature, ('no', ) is returned."""
+        backing = self.get_transport()
+        request = smart_repo.SmartServerRequestHasSignatureForRevisionId(
+            backing)
+        tree = self.make_branch_and_memory_tree('.')
+        tree.lock_write()
+        tree.add('')
+        r1 = tree.commit('a commit', rev_id='A')
+        tree.unlock()
+        self.assertTrue(tree.branch.repository.has_revision('A'))
+        self.assertEqual(smart_req.SmartServerResponse(('no', )),
+            request.execute('', 'A'))
+
+    def test_present_signature(self):
+        """For a present signature, ('yes', ) is returned."""
+        backing = self.get_transport()
+        request = smart_repo.SmartServerRequestHasSignatureForRevisionId(
+            backing)
+        strategy = gpg.LoopbackGPGStrategy(None)
+        tree = self.make_branch_and_memory_tree('.')
+        tree.lock_write()
+        tree.add('')
+        r1 = tree.commit('a commit', rev_id='A')
+        tree.branch.repository.start_write_group()
+        tree.branch.repository.sign_revision('A', strategy)
+        tree.branch.repository.commit_write_group()
+        tree.unlock()
+        self.assertTrue(tree.branch.repository.has_revision('A'))
+        self.assertEqual(smart_req.SmartServerResponse(('yes', )),
+            request.execute('', 'A'))
+
+
 class TestSmartServerRepositoryGatherStats(tests.TestCaseWithMemoryTransport):
 
     def test_empty_revid(self):
@@ -1607,6 +2003,17 @@
                          request.execute('',
                                          rev_id_utf8, 'yes'))
 
+    def test_unknown_revid(self):
+        """An unknown revision id causes a 'nosuchrevision' error."""
+        backing = self.get_transport()
+        request = smart_repo.SmartServerRepositoryGatherStats(backing)
+        repository = self.make_repository('.')
+        expected_body = 'revisions: 0\n'
+        self.assertEqual(
+            smart_req.FailedSmartServerResponse(
+                ('nosuchrevision', 'mia'), None),
+            request.execute('', 'mia', 'yes'))
+
 
 class TestSmartServerRepositoryIsShared(tests.TestCaseWithMemoryTransport):
 
@@ -1627,6 +2034,52 @@
             request.execute('', ))
 
 
+class TestSmartServerRepositoryGetRevisionSignatureText(
+        tests.TestCaseWithMemoryTransport):
+
+    def test_get_signature(self):
+        backing = self.get_transport()
+        request = smart_repo.SmartServerRepositoryGetRevisionSignatureText(
+            backing)
+        bb = self.make_branch_builder('.')
+        bb.build_commit(rev_id='A')
+        repo = bb.get_branch().repository
+        strategy = gpg.LoopbackGPGStrategy(None)
+        self.addCleanup(repo.lock_write().unlock)
+        repo.start_write_group()
+        repo.sign_revision('A', strategy)
+        repo.commit_write_group()
+        expected_body = (
+            '-----BEGIN PSEUDO-SIGNED CONTENT-----\n' +
+            Testament.from_revision(repo, 'A').as_short_text() +
+            '-----END PSEUDO-SIGNED CONTENT-----\n')
+        self.assertEqual(
+            smart_req.SmartServerResponse(('ok', ), expected_body),
+            request.execute('', 'A'))
+
+
+class TestSmartServerRepositoryMakeWorkingTrees(
+        tests.TestCaseWithMemoryTransport):
+
+    def test_make_working_trees(self):
+        """For a repository with working trees, ('yes', ) is returned."""
+        backing = self.get_transport()
+        request = smart_repo.SmartServerRepositoryMakeWorkingTrees(backing)
+        r = self.make_repository('.')
+        r.set_make_working_trees(True)
+        self.assertEqual(smart_req.SmartServerResponse(('yes', )),
+            request.execute('', ))
+
+    def test_is_not_shared(self):
+        """For a repository with working trees, ('no', ) is returned."""
+        backing = self.get_transport()
+        request = smart_repo.SmartServerRepositoryMakeWorkingTrees(backing)
+        r = self.make_repository('.')
+        r.set_make_working_trees(False)
+        self.assertEqual(smart_req.SmartServerResponse(('no', )),
+            request.execute('', ))
+
+
 class TestSmartServerRepositoryLockWrite(tests.TestCaseWithMemoryTransport):
 
     def test_lock_write_on_unlocked_repo(self):
@@ -1746,6 +2199,50 @@
             smart_req.SmartServerResponse(('TokenMismatch',)), response)
 
 
+class TestSmartServerRepositoryGetPhysicalLockStatus(
+    tests.TestCaseWithTransport):
+
+    def test_with_write_lock(self):
+        backing = self.get_transport()
+        repo = self.make_repository('.')
+        self.addCleanup(repo.lock_write().unlock)
+        # lock_write() doesn't necessarily actually take a physical
+        # lock out.
+        if repo.get_physical_lock_status():
+            expected = 'yes'
+        else:
+            expected = 'no'
+        request_class = smart_repo.SmartServerRepositoryGetPhysicalLockStatus
+        request = request_class(backing)
+        self.assertEqual(smart_req.SuccessfulSmartServerResponse((expected,)),
+            request.execute('', ))
+
+    def test_without_write_lock(self):
+        backing = self.get_transport()
+        repo = self.make_repository('.')
+        self.assertEquals(False, repo.get_physical_lock_status())
+        request_class = smart_repo.SmartServerRepositoryGetPhysicalLockStatus
+        request = request_class(backing)
+        self.assertEqual(smart_req.SuccessfulSmartServerResponse(('no',)),
+            request.execute('', ))
+
+
+class TestSmartServerRepositoryReconcile(tests.TestCaseWithTransport):
+
+    def test_reconcile(self):
+        backing = self.get_transport()
+        repo = self.make_repository('.')
+        token = repo.lock_write().repository_token
+        self.addCleanup(repo.unlock)
+        request_class = smart_repo.SmartServerRepositoryReconcile
+        request = request_class(backing)
+        self.assertEqual(smart_req.SuccessfulSmartServerResponse(
+            ('ok', ),
+             'garbage_inventories: 0\n'
+             'inconsistent_parents: 0\n'),
+            request.execute('', token))
+
+
 class TestSmartServerIsReadonly(tests.TestCaseWithMemoryTransport):
 
     def test_is_readonly_no(self):
@@ -1789,6 +2286,92 @@
         self.assertTrue(repo.make_working_trees())
 
 
+class TestSmartServerRepositoryGetSerializerFormat(
+    tests.TestCaseWithMemoryTransport):
+
+    def test_get_serializer_format(self):
+        backing = self.get_transport()
+        repo = self.make_repository('.', format='2a')
+        request_class = smart_repo.SmartServerRepositoryGetSerializerFormat
+        request = request_class(backing)
+        self.assertEqual(
+            smart_req.SuccessfulSmartServerResponse(('ok', '10')),
+            request.execute(''))
+
+
+class TestSmartServerRepositoryWriteGroup(
+    tests.TestCaseWithMemoryTransport):
+
+    def test_start_write_group(self):
+        backing = self.get_transport()
+        repo = self.make_repository('.')
+        lock_token = repo.lock_write().repository_token
+        self.addCleanup(repo.unlock)
+        request_class = smart_repo.SmartServerRepositoryStartWriteGroup
+        request = request_class(backing)
+        self.assertEqual(smart_req.SuccessfulSmartServerResponse(('ok', [])),
+            request.execute('', lock_token))
+
+    def test_start_write_group_unsuspendable(self):
+        backing = self.get_transport()
+        repo = self.make_repository('.', format='knit')
+        lock_token = repo.lock_write().repository_token
+        self.addCleanup(repo.unlock)
+        request_class = smart_repo.SmartServerRepositoryStartWriteGroup
+        request = request_class(backing)
+        self.assertEqual(
+            smart_req.FailedSmartServerResponse(('UnsuspendableWriteGroup',)),
+            request.execute('', lock_token))
+
+    def test_commit_write_group(self):
+        backing = self.get_transport()
+        repo = self.make_repository('.')
+        lock_token = repo.lock_write().repository_token
+        self.addCleanup(repo.unlock)
+        repo.start_write_group()
+        tokens = repo.suspend_write_group()
+        request_class = smart_repo.SmartServerRepositoryCommitWriteGroup
+        request = request_class(backing)
+        self.assertEqual(smart_req.SuccessfulSmartServerResponse(('ok',)),
+            request.execute('', lock_token, tokens))
+
+    def test_abort_write_group(self):
+        backing = self.get_transport()
+        repo = self.make_repository('.')
+        lock_token = repo.lock_write().repository_token
+        repo.start_write_group()
+        tokens = repo.suspend_write_group()
+        self.addCleanup(repo.unlock)
+        request_class = smart_repo.SmartServerRepositoryAbortWriteGroup
+        request = request_class(backing)
+        self.assertEqual(smart_req.SuccessfulSmartServerResponse(('ok',)),
+            request.execute('', lock_token, tokens))
+
+    def test_check_write_group(self):
+        backing = self.get_transport()
+        repo = self.make_repository('.')
+        lock_token = repo.lock_write().repository_token
+        repo.start_write_group()
+        tokens = repo.suspend_write_group()
+        self.addCleanup(repo.unlock)
+        request_class = smart_repo.SmartServerRepositoryCheckWriteGroup
+        request = request_class(backing)
+        self.assertEqual(smart_req.SuccessfulSmartServerResponse(('ok',)),
+            request.execute('', lock_token, tokens))
+
+    def test_check_write_group_invalid(self):
+        backing = self.get_transport()
+        repo = self.make_repository('.')
+        lock_token = repo.lock_write().repository_token
+        self.addCleanup(repo.unlock)
+        request_class = smart_repo.SmartServerRepositoryCheckWriteGroup
+        request = request_class(backing)
+        self.assertEqual(smart_req.FailedSmartServerResponse(
+            ('UnresumableWriteGroup', ['random'],
+                'Malformed write group token')),
+            request.execute('', lock_token, ["random"]))
+
+
 class TestSmartServerPackRepositoryAutopack(tests.TestCaseWithTransport):
 
     def make_repo_needing_autopacking(self, path='.'):
@@ -1860,18 +2443,27 @@
         """All registered request_handlers can be found."""
         # If there's a typo in a register_lazy call, this loop will fail with
         # an AttributeError.
-        for key, item in smart_req.request_handlers.iteritems():
-            pass
+        for key in smart_req.request_handlers.keys():
+            try:
+                item = smart_req.request_handlers.get(key)
+            except AttributeError, e:
+                raise AttributeError('failed to get %s: %s' % (key, e))
 
     def assertHandlerEqual(self, verb, handler):
         self.assertEqual(smart_req.request_handlers.get(verb), handler)
 
     def test_registered_methods(self):
         """Test that known methods are registered to the correct object."""
+        self.assertHandlerEqual('Branch.break_lock',
+            smart_branch.SmartServerBranchBreakLock)
         self.assertHandlerEqual('Branch.get_config_file',
             smart_branch.SmartServerBranchGetConfigFile)
+        self.assertHandlerEqual('Branch.put_config_file',
+            smart_branch.SmartServerBranchPutConfigFile)
         self.assertHandlerEqual('Branch.get_parent',
             smart_branch.SmartServerBranchGetParent)
+        self.assertHandlerEqual('Branch.get_physical_lock_status',
+            smart_branch.SmartServerBranchRequestGetPhysicalLockStatus)
         self.assertHandlerEqual('Branch.get_tags_bytes',
             smart_branch.SmartServerBranchGetTagsBytes)
         self.assertHandlerEqual('Branch.lock_write',
@@ -1880,6 +2472,8 @@
             smart_branch.SmartServerBranchRequestLastRevisionInfo)
         self.assertHandlerEqual('Branch.revision_history',
             smart_branch.SmartServerRequestRevisionHistory)
+        self.assertHandlerEqual('Branch.revision_id_to_revno',
+            smart_branch.SmartServerBranchRequestRevisionIdToRevno)
         self.assertHandlerEqual('Branch.set_config_option',
             smart_branch.SmartServerBranchRequestSetConfigOption)
         self.assertHandlerEqual('Branch.set_last_revision',
@@ -1892,6 +2486,8 @@
             smart_branch.SmartServerBranchRequestSetParentLocation)
         self.assertHandlerEqual('Branch.unlock',
             smart_branch.SmartServerBranchRequestUnlock)
+        self.assertHandlerEqual('BzrDir.destroy_branch',
+            smart_dir.SmartServerBzrDirRequestDestroyBranch)
         self.assertHandlerEqual('BzrDir.find_repository',
             smart_dir.SmartServerRequestFindRepositoryV1)
         self.assertHandlerEqual('BzrDir.find_repositoryV2',
@@ -1900,6 +2496,8 @@
             smart_dir.SmartServerRequestInitializeBzrDir)
         self.assertHandlerEqual('BzrDirFormat.initialize_ex_1.16',
             smart_dir.SmartServerRequestBzrDirInitializeEx)
+        self.assertHandlerEqual('BzrDir.checkout_metadir',
+            smart_dir.SmartServerBzrDirRequestCheckoutMetaDir)
         self.assertHandlerEqual('BzrDir.cloning_metadir',
             smart_dir.SmartServerBzrDirRequestCloningMetaDir)
         self.assertHandlerEqual('BzrDir.get_config_file',
@@ -1912,18 +2510,30 @@
             smart_dir.SmartServerRequestOpenBranchV3)
         self.assertHandlerEqual('PackRepository.autopack',
             smart_packrepo.SmartServerPackRepositoryAutopack)
+        self.assertHandlerEqual('Repository.add_signature_text',
+            smart_repo.SmartServerRepositoryAddSignatureText)
+        self.assertHandlerEqual('Repository.all_revision_ids',
+            smart_repo.SmartServerRepositoryAllRevisionIds)
+        self.assertHandlerEqual('Repository.break_lock',
+            smart_repo.SmartServerRepositoryBreakLock)
         self.assertHandlerEqual('Repository.gather_stats',
             smart_repo.SmartServerRepositoryGatherStats)
         self.assertHandlerEqual('Repository.get_parent_map',
             smart_repo.SmartServerRepositoryGetParentMap)
+        self.assertHandlerEqual('Repository.get_physical_lock_status',
+            smart_repo.SmartServerRepositoryGetPhysicalLockStatus)
         self.assertHandlerEqual('Repository.get_rev_id_for_revno',
             smart_repo.SmartServerRepositoryGetRevIdForRevno)
         self.assertHandlerEqual('Repository.get_revision_graph',
             smart_repo.SmartServerRepositoryGetRevisionGraph)
+        self.assertHandlerEqual('Repository.get_revision_signature_text',
+            smart_repo.SmartServerRepositoryGetRevisionSignatureText)
         self.assertHandlerEqual('Repository.get_stream',
             smart_repo.SmartServerRepositoryGetStream)
         self.assertHandlerEqual('Repository.get_stream_1.19',
             smart_repo.SmartServerRepositoryGetStream_1_19)
+        self.assertHandlerEqual('Repository.iter_revisions',
+            smart_repo.SmartServerRepositoryIterRevisions)
         self.assertHandlerEqual('Repository.has_revision',
             smart_repo.SmartServerRequestHasRevision)
         self.assertHandlerEqual('Repository.insert_stream',
@@ -1932,12 +2542,32 @@
             smart_repo.SmartServerRepositoryInsertStreamLocked)
         self.assertHandlerEqual('Repository.is_shared',
             smart_repo.SmartServerRepositoryIsShared)
+        self.assertHandlerEqual('Repository.iter_files_bytes',
+            smart_repo.SmartServerRepositoryIterFilesBytes)
         self.assertHandlerEqual('Repository.lock_write',
             smart_repo.SmartServerRepositoryLockWrite)
+        self.assertHandlerEqual('Repository.make_working_trees',
+            smart_repo.SmartServerRepositoryMakeWorkingTrees)
+        self.assertHandlerEqual('Repository.pack',
+            smart_repo.SmartServerRepositoryPack)
+        self.assertHandlerEqual('Repository.reconcile',
+            smart_repo.SmartServerRepositoryReconcile)
         self.assertHandlerEqual('Repository.tarball',
             smart_repo.SmartServerRepositoryTarball)
         self.assertHandlerEqual('Repository.unlock',
             smart_repo.SmartServerRepositoryUnlock)
+        self.assertHandlerEqual('Repository.start_write_group',
+            smart_repo.SmartServerRepositoryStartWriteGroup)
+        self.assertHandlerEqual('Repository.check_write_group',
+            smart_repo.SmartServerRepositoryCheckWriteGroup)
+        self.assertHandlerEqual('Repository.commit_write_group',
+            smart_repo.SmartServerRepositoryCommitWriteGroup)
+        self.assertHandlerEqual('Repository.abort_write_group',
+            smart_repo.SmartServerRepositoryAbortWriteGroup)
+        self.assertHandlerEqual('VersionedFileRepository.get_serializer_format',
+            smart_repo.SmartServerRepositoryGetSerializerFormat)
+        self.assertHandlerEqual('VersionedFileRepository.get_inventories',
+            smart_repo.SmartServerRepositoryGetInventories)
         self.assertHandlerEqual('Transport.is_readonly',
             smart_req.SmartServerIsReadonly)
 
@@ -1987,3 +2617,64 @@
         self.server.run_server_stopped_hooks()
         self.assertEquals(stopped_calls,
             [([self.get_transport().base], 'bzr://example.com:42/')])
+
+
+class TestSmartServerRepositoryPack(tests.TestCaseWithMemoryTransport):
+
+    def test_pack(self):
+        backing = self.get_transport()
+        request = smart_repo.SmartServerRepositoryPack(backing)
+        tree = self.make_branch_and_memory_tree('.')
+        repo_token = tree.branch.repository.lock_write().repository_token
+
+        self.assertIs(None, request.execute('', repo_token, False))
+
+        self.assertEqual(
+            smart_req.SuccessfulSmartServerResponse(('ok', ), ),
+            request.do_body(''))
+
+
+class TestSmartServerRepositoryGetInventories(tests.TestCaseWithTransport):
+
+    def _get_serialized_inventory_delta(self, repository, base_revid, revid):
+        base_inv = repository.revision_tree(base_revid).inventory
+        inv = repository.revision_tree(revid).inventory
+        inv_delta = inv._make_delta(base_inv)
+        serializer = inventory_delta.InventoryDeltaSerializer(True, False)
+        return "".join(serializer.delta_to_lines(base_revid, revid, inv_delta))
+
+    def test_single(self):
+        backing = self.get_transport()
+        request = smart_repo.SmartServerRepositoryGetInventories(backing)
+        t = self.make_branch_and_tree('.', format='2a')
+        self.addCleanup(t.lock_write().unlock)
+        self.build_tree_contents([("file", "somecontents")])
+        t.add(["file"], ["thefileid"])
+        t.commit(rev_id='somerev', message="add file")
+        self.assertIs(None, request.execute('', 'unordered'))
+        response = request.do_body("somerev\n")
+        self.assertTrue(response.is_successful())
+        self.assertEquals(response.args, ("ok", ))
+        stream = [('inventory-deltas', [
+            versionedfile.FulltextContentFactory('somerev', None, None,
+                self._get_serialized_inventory_delta(
+                    t.branch.repository, 'null:', 'somerev'))])]
+        fmt = bzrdir.format_registry.get('2a')().repository_format
+        self.assertEquals(
+            "".join(response.body_stream),
+            "".join(smart_repo._stream_to_byte_stream(stream, fmt)))
+
+    def test_empty(self):
+        backing = self.get_transport()
+        request = smart_repo.SmartServerRepositoryGetInventories(backing)
+        t = self.make_branch_and_tree('.', format='2a')
+        self.addCleanup(t.lock_write().unlock)
+        self.build_tree_contents([("file", "somecontents")])
+        t.add(["file"], ["thefileid"])
+        t.commit(rev_id='somerev', message="add file")
+        self.assertIs(None, request.execute('', 'unordered'))
+        response = request.do_body("")
+        self.assertTrue(response.is_successful())
+        self.assertEquals(response.args, ("ok", ))
+        self.assertEquals("".join(response.body_stream),
+            "Bazaar pack format 1 (introduced in 0.18)\nB54\n\nBazaar repository format 2a (needs bzr 1.16 or later)\nE")

=== modified file 'bzrlib/tests/test_smart_request.py'
--- bzrlib/tests/test_smart_request.py	2011-08-19 22:34:02 +0000
+++ bzrlib/tests/test_smart_request.py	2011-10-10 14:55:12 +0000
@@ -118,6 +118,16 @@
         self.assertEqual(
             [[transport]] * 3, handler._command.jail_transports_log)
 
+    def test_all_registered_requests_are_safety_qualified(self):
+        unclassified_requests = []
+        allowed_info = ('read', 'idem', 'mutate', 'semivfs', 'semi', 'stream')
+        for key in request.request_handlers.keys():
+            info = request.request_handlers.get_info(key)
+            if info is None or info not in allowed_info:
+                unclassified_requests.append(key)
+        if unclassified_requests:
+            self.fail('These requests were not categorized as safe/unsafe'
+                      ' to retry: %s'  % (unclassified_requests,))
 
 
 class TestSmartRequestHandlerErrorTranslation(TestCase):

=== added file 'bzrlib/tests/test_smart_signals.py'
--- bzrlib/tests/test_smart_signals.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_smart_signals.py	2011-09-26 12:12:40 +0000
@@ -0,0 +1,189 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+
+import os
+import signal
+import threading
+import weakref
+
+from bzrlib import tests, transport
+from bzrlib.smart import client, medium, server, signals
+
+# Windows doesn't define SIGHUP. And while we could just skip a lot of these
+# tests, we often don't actually care about interaction with 'signal', so we
+# can still run the tests for code coverage.
+SIGHUP = getattr(signal, 'SIGHUP', 1)
+
+
+class TestSignalHandlers(tests.TestCase):
+
+    def setUp(self):
+        super(TestSignalHandlers, self).setUp()
+        # This allows us to mutate the signal handler callbacks, but leave it
+        # 'pristine' after the test case.
+        # TODO: Arguably, this could be put into the base test.TestCase, along
+        #       with a tearDown that asserts that all the entries have been
+        #       removed properly. Global state is always a bit messy. A shame
+        #       that we need it for signal handling.
+        orig = signals._setup_on_hangup_dict()
+        self.assertIs(None, orig)
+        def cleanup():
+            signals._on_sighup = None
+        self.addCleanup(cleanup)
+
+    def test_registered_callback_gets_called(self):
+        calls = []
+        def call_me():
+            calls.append('called')
+        signals.register_on_hangup('myid', call_me)
+        signals._sighup_handler(SIGHUP, None)
+        self.assertEqual(['called'], calls)
+        signals.unregister_on_hangup('myid')
+
+    def test_unregister_not_present(self):
+        # We don't want unregister to fail, since it is generally run at times
+        # that shouldn't interrupt other flow.
+        signals.unregister_on_hangup('no-such-id')
+        log = self.get_log()
+        self.assertContainsRe(log, 'Error occurred during unregister_on_hangup:')
+        self.assertContainsRe(log, '(?s)Traceback.*KeyError')
+
+    def test_failing_callback(self):
+        calls = []
+        def call_me():
+            calls.append('called')
+        def fail_me():
+            raise RuntimeError('something bad happened')
+        signals.register_on_hangup('myid', call_me)
+        signals.register_on_hangup('otherid', fail_me)
+        # _sighup_handler should call both, even though it got an exception
+        signals._sighup_handler(SIGHUP, None)
+        signals.unregister_on_hangup('myid')
+        signals.unregister_on_hangup('otherid')
+        log = self.get_log()
+        self.assertContainsRe(log, '(?s)Traceback.*RuntimeError')
+        self.assertEqual(['called'], calls)
+
+    def test_unregister_during_call(self):
+        # _sighup_handler should handle if some callbacks actually remove
+        # themselves while running.
+        calls = []
+        def call_me_and_unregister():
+            signals.unregister_on_hangup('myid')
+            calls.append('called_and_unregistered')
+        def call_me():
+            calls.append('called')
+        signals.register_on_hangup('myid', call_me_and_unregister)
+        signals.register_on_hangup('other', call_me)
+        signals._sighup_handler(SIGHUP, None)
+
+    def test_keyboard_interrupt_propagated(self):
+        # In case we get 'stuck' while running a hangup function, we should
+        # not suppress KeyboardInterrupt
+        def call_me_and_raise():
+            raise KeyboardInterrupt()
+        signals.register_on_hangup('myid', call_me_and_raise)
+        self.assertRaises(KeyboardInterrupt,
+                          signals._sighup_handler, SIGHUP, None)
+        signals.unregister_on_hangup('myid')
+
+    def test_weak_references(self):
+        # TODO: This is probably a very-CPython-specific test
+        # Adding yourself to the callback should not make you immortal
+        # We overrideAttr during the test suite, so that we don't pollute the
+        # original dict. However, we can test that what we override matches
+        # what we are putting there.
+        self.assertIsInstance(signals._on_sighup,
+                              weakref.WeakValueDictionary)
+        calls = []
+        def call_me():
+            calls.append('called')
+        signals.register_on_hangup('myid', call_me)
+        del call_me
+        # Non-CPython might want to do a gc.collect() here
+        signals._sighup_handler(SIGHUP, None)
+        self.assertEqual([], calls)
+
+    def test_not_installed(self):
+        # If you haven't called bzrlib.smart.signals.install_sighup_handler,
+        # then _on_sighup should be None, and all the calls become no-ops.
+        signals._on_sighup = None
+        calls = []
+        def call_me():
+            calls.append('called')
+        signals.register_on_hangup('myid', calls)
+        signals._sighup_handler(SIGHUP, None)
+        signals.unregister_on_hangup('myid')
+        log = self.get_log()
+        self.assertEqual('', log)
+
+    def test_install_sighup_handler(self):
+        # install_sighup_handler should set up a signal handler for SIGHUP, as
+        # well as the signals._on_sighup dict.
+        signals._on_sighup = None
+        orig = signals.install_sighup_handler()
+        if getattr(signal, 'SIGHUP', None) is not None:
+            cur = signal.getsignal(SIGHUP)
+            self.assertEqual(signals._sighup_handler, cur)
+        self.assertIsNot(None, signals._on_sighup)
+        signals.restore_sighup_handler(orig)
+        self.assertIs(None, signals._on_sighup)
+
+
+class TestInetServer(tests.TestCase):
+
+    def create_file_pipes(self):
+        r, w = os.pipe()
+        rf = os.fdopen(r, 'rb')
+        wf = os.fdopen(w, 'wb')
+        return rf, wf
+
+    def test_inet_server_responds_to_sighup(self):
+        t = transport.get_transport('memory:///')
+        content = 'a'*1024*1024
+        t.put_bytes('bigfile', content)
+        factory = server.BzrServerFactory()
+        # Override stdin/stdout so that we can inject our own handles
+        client_read, server_write = self.create_file_pipes()
+        server_read, client_write = self.create_file_pipes()
+        factory._get_stdin_stdout = lambda: (server_read, server_write)
+        factory.set_up(t, None, None, inet=True, timeout=4.0)
+        self.addCleanup(factory.tear_down)
+        started = threading.Event()
+        stopped = threading.Event()
+        def serving():
+            started.set()
+            factory.smart_server.serve()
+            stopped.set()
+        server_thread = threading.Thread(target=serving)
+        server_thread.start()
+        started.wait()
+        client_medium = medium.SmartSimplePipesClientMedium(client_read,
+                            client_write, 'base')
+        client_client = client._SmartClient(client_medium)
+        resp, response_handler = client_client.call_expecting_body('get',
+            'bigfile')
+        signals._sighup_handler(SIGHUP, None)
+        self.assertTrue(factory.smart_server.finished)
+        # We can still finish reading the file content, but more than that, and
+        # the file is closed.
+        v = response_handler.read_body_bytes()
+        if v != content:
+            self.fail('Got the wrong content back, expected 1M "a"')
+        stopped.wait()
+        server_thread.join()
+

=== modified file 'bzrlib/tests/test_smart_transport.py'
--- bzrlib/tests/test_smart_transport.py	2011-08-19 22:34:02 +0000
+++ bzrlib/tests/test_smart_transport.py	2011-11-25 17:54:52 +0000
@@ -18,17 +18,25 @@
 
 # all of this deals with byte strings so this is safe
 from cStringIO import StringIO
+import doctest
+import errno
 import os
 import socket
+import subprocess
+import sys
 import threading
+import time
+
+from testtools.matchers import DocTestMatches
 
 import bzrlib
 from bzrlib import (
         bzrdir,
+        debug,
         errors,
         osutils,
         tests,
-        transport,
+        transport as _mod_transport,
         urlutils,
         )
 from bzrlib.smart import (
@@ -37,7 +45,7 @@
         message,
         protocol,
         request as _mod_request,
-        server,
+        server as _mod_server,
         vfs,
 )
 from bzrlib.tests import (
@@ -54,6 +62,29 @@
         )
 
 
+def create_file_pipes():
+    r, w = os.pipe()
+    # These must be opened without buffering, or we get undefined results
+    rf = os.fdopen(r, 'rb', 0)
+    wf = os.fdopen(w, 'wb', 0)
+    return rf, wf
+
+
+def portable_socket_pair():
+    """Return a pair of TCP sockets connected to each other.
+
+    Unlike socket.socketpair, this should work on Windows.
+    """
+    listen_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+    listen_sock.bind(('127.0.0.1', 0))
+    listen_sock.listen(1)
+    client_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+    client_sock.connect(listen_sock.getsockname())
+    server_sock, addr = listen_sock.accept()
+    listen_sock.close()
+    return server_sock, client_sock
+
+
 class StringIOSSHVendor(object):
     """A SSH vendor that uses StringIO to buffer writes and answer reads."""
 
@@ -68,6 +99,27 @@
         return StringIOSSHConnection(self)
 
 
+class FirstRejectedStringIOSSHVendor(StringIOSSHVendor):
+    """The first connection will be considered closed.
+
+    The second connection will succeed normally.
+    """
+
+    def __init__(self, read_from, write_to, fail_at_write=True):
+        super(FirstRejectedStringIOSSHVendor, self).__init__(read_from,
+            write_to)
+        self.fail_at_write = fail_at_write
+        self._first = True
+
+    def connect_ssh(self, username, password, host, port, command):
+        self.calls.append(('connect_ssh', username, password, host, port,
+            command))
+        if self._first:
+            self._first = False
+            return ClosedSSHConnection(self)
+        return StringIOSSHConnection(self)
+
+
 class StringIOSSHConnection(ssh.SSHConnection):
     """A SSH connection that uses StringIO to buffer writes and answer reads."""
 
@@ -83,6 +135,29 @@
         return 'pipes', (self.vendor.read_from, self.vendor.write_to)
 
 
+class ClosedSSHConnection(ssh.SSHConnection):
+    """An SSH connection that just has closed channels."""
+
+    def __init__(self, vendor):
+        self.vendor = vendor
+
+    def close(self):
+        self.vendor.calls.append(('close', ))
+
+    def get_sock_or_pipes(self):
+        # We create matching pipes, and then close the ssh side
+        bzr_read, ssh_write = create_file_pipes()
+        # We always fail when bzr goes to read
+        ssh_write.close()
+        if self.vendor.fail_at_write:
+            # If set, we'll also fail when bzr goes to write
+            ssh_read, bzr_write = create_file_pipes()
+            ssh_read.close()
+        else:
+            bzr_write = self.vendor.write_to
+        return 'pipes', (bzr_read, bzr_write)
+
+
 class _InvalidHostnameFeature(features.Feature):
     """Does 'non_existent.invalid' fail to resolve?
 
@@ -178,6 +253,91 @@
         client_medium._accept_bytes('abc')
         self.assertEqual('abc', output.getvalue())
 
+    def test_simple_pipes__accept_bytes_subprocess_closed(self):
+        # It is unfortunate that we have to use Popen for this. However,
+        # os.pipe() does not behave the same as subprocess.Popen().
+        # On Windows, if you use os.pipe() and close the write side,
+        # read.read() hangs. On Linux, read.read() returns the empty string.
+        p = subprocess.Popen([sys.executable, '-c',
+            'import sys\n'
+            'sys.stdout.write(sys.stdin.read(4))\n'
+            'sys.stdout.close()\n'],
+            stdout=subprocess.PIPE, stdin=subprocess.PIPE)
+        client_medium = medium.SmartSimplePipesClientMedium(
+            p.stdout, p.stdin, 'base')
+        client_medium._accept_bytes('abc\n')
+        self.assertEqual('abc', client_medium._read_bytes(3))
+        p.wait()
+        # While writing to the underlying pipe,
+        #   Windows py2.6.6 we get IOError(EINVAL)
+        #   Lucid py2.6.5, we get IOError(EPIPE)
+        # In both cases, it should be wrapped to ConnectionReset
+        self.assertRaises(errors.ConnectionReset,
+                          client_medium._accept_bytes, 'more')
+
+    def test_simple_pipes__accept_bytes_pipe_closed(self):
+        child_read, client_write = create_file_pipes()
+        client_medium = medium.SmartSimplePipesClientMedium(
+            None, client_write, 'base')
+        client_medium._accept_bytes('abc\n')
+        self.assertEqual('abc\n', child_read.read(4))
+        # While writing to the underlying pipe,
+        #   Windows py2.6.6 we get IOError(EINVAL)
+        #   Lucid py2.6.5, we get IOError(EPIPE)
+        # In both cases, it should be wrapped to ConnectionReset
+        child_read.close()
+        self.assertRaises(errors.ConnectionReset,
+                          client_medium._accept_bytes, 'more')
+
+    def test_simple_pipes__flush_pipe_closed(self):
+        child_read, client_write = create_file_pipes()
+        client_medium = medium.SmartSimplePipesClientMedium(
+            None, client_write, 'base')
+        client_medium._accept_bytes('abc\n')
+        child_read.close()
+        # Even though the pipe is closed, flush on the write side seems to be a
+        # no-op, rather than a failure.
+        client_medium._flush()
+
+    def test_simple_pipes__flush_subprocess_closed(self):
+        p = subprocess.Popen([sys.executable, '-c',
+            'import sys\n'
+            'sys.stdout.write(sys.stdin.read(4))\n'
+            'sys.stdout.close()\n'],
+            stdout=subprocess.PIPE, stdin=subprocess.PIPE)
+        client_medium = medium.SmartSimplePipesClientMedium(
+            p.stdout, p.stdin, 'base')
+        client_medium._accept_bytes('abc\n')
+        p.wait()
+        # Even though the child process is dead, flush seems to be a no-op.
+        client_medium._flush()
+
+    def test_simple_pipes__read_bytes_pipe_closed(self):
+        child_read, client_write = create_file_pipes()
+        client_medium = medium.SmartSimplePipesClientMedium(
+            child_read, client_write, 'base')
+        client_medium._accept_bytes('abc\n')
+        client_write.close()
+        self.assertEqual('abc\n', client_medium._read_bytes(4))
+        self.assertEqual('', client_medium._read_bytes(4))
+
+    def test_simple_pipes__read_bytes_subprocess_closed(self):
+        p = subprocess.Popen([sys.executable, '-c',
+            'import sys\n'
+            'if sys.platform == "win32":\n'
+            '    import msvcrt, os\n'
+            '    msvcrt.setmode(sys.stdout.fileno(), os.O_BINARY)\n'
+            '    msvcrt.setmode(sys.stdin.fileno(), os.O_BINARY)\n'
+            'sys.stdout.write(sys.stdin.read(4))\n'
+            'sys.stdout.close()\n'],
+            stdout=subprocess.PIPE, stdin=subprocess.PIPE)
+        client_medium = medium.SmartSimplePipesClientMedium(
+            p.stdout, p.stdin, 'base')
+        client_medium._accept_bytes('abc\n')
+        p.wait()
+        self.assertEqual('abc\n', client_medium._read_bytes(4))
+        self.assertEqual('', client_medium._read_bytes(4))
+
     def test_simple_pipes_client_disconnect_does_nothing(self):
         # calling disconnect does nothing.
         input = StringIO()
@@ -339,6 +499,27 @@
             ],
             vendor.calls)
 
+    def test_ssh_client_repr(self):
+        client_medium = medium.SmartSSHClientMedium(
+            'base', medium.SSHParams("example.com", "4242", "username"))
+        self.assertEquals(
+            "SmartSSHClientMedium(bzr+ssh://username@example.com:4242/)",
+            repr(client_medium))
+
+    def test_ssh_client_repr_no_port(self):
+        client_medium = medium.SmartSSHClientMedium(
+            'base', medium.SSHParams("example.com", None, "username"))
+        self.assertEquals(
+            "SmartSSHClientMedium(bzr+ssh://username@example.com/)",
+            repr(client_medium))
+
+    def test_ssh_client_repr_no_username(self):
+        client_medium = medium.SmartSSHClientMedium(
+            'base', medium.SSHParams("example.com", None, None))
+        self.assertEquals(
+            "SmartSSHClientMedium(bzr+ssh://example.com/)",
+            repr(client_medium))
+
     def test_ssh_client_ignores_disconnect_when_not_connected(self):
         # Doing a disconnect on a new (and thus unconnected) SSH medium
         # does not fail.  It's ok to disconnect an unconnected medium.
@@ -565,6 +746,28 @@
         request.finished_reading()
         self.assertRaises(errors.ReadingCompleted, request.read_bytes, None)
 
+    def test_reset(self):
+        server_sock, client_sock = portable_socket_pair()
+        # TODO: Use SmartClientAlreadyConnectedSocketMedium for the versions of
+        #       bzr where it exists.
+        client_medium = medium.SmartTCPClientMedium(None, None, None)
+        client_medium._socket = client_sock
+        client_medium._connected = True
+        req = client_medium.get_request()
+        self.assertRaises(errors.TooManyConcurrentRequests,
+            client_medium.get_request)
+        client_medium.reset()
+        # The stream should be reset, marked as disconnected, though ready for
+        # us to make a new request
+        self.assertFalse(client_medium._connected)
+        self.assertIs(None, client_medium._socket)
+        try:
+            self.assertEqual('', client_sock.recv(1))
+        except socket.error, e:
+            if e.errno not in (errno.EBADF,):
+                raise
+        req = client_medium.get_request()
+
 
 class RemoteTransportTests(test_smart.TestCaseWithSmartMedium):
 
@@ -618,28 +821,46 @@
         super(TestSmartServerStreamMedium, self).setUp()
         self.overrideEnv('BZR_NO_SMART_VFS', None)
 
-    def portable_socket_pair(self):
-        """Return a pair of TCP sockets connected to each other.
-
-        Unlike socket.socketpair, this should work on Windows.
-        """
-        listen_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        listen_sock.bind(('127.0.0.1', 0))
-        listen_sock.listen(1)
-        client_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        client_sock.connect(listen_sock.getsockname())
-        server_sock, addr = listen_sock.accept()
-        listen_sock.close()
-        return server_sock, client_sock
+    def create_pipe_medium(self, to_server, from_server, transport,
+                           timeout=4.0):
+        """Create a new SmartServerPipeStreamMedium."""
+        return medium.SmartServerPipeStreamMedium(to_server, from_server,
+            transport, timeout=timeout)
+
+    def create_pipe_context(self, to_server_bytes, transport):
+        """Create a SmartServerSocketStreamMedium.
+
+        This differes from create_pipe_medium, in that we initialize the
+        request that is sent to the server, and return the StringIO class that
+        will hold the response.
+        """
+        to_server = StringIO(to_server_bytes)
+        from_server = StringIO()
+        m = self.create_pipe_medium(to_server, from_server, transport)
+        return m, from_server
+
+    def create_socket_medium(self, server_sock, transport, timeout=4.0):
+        """Initialize a new medium.SmartServerSocketStreamMedium."""
+        return medium.SmartServerSocketStreamMedium(server_sock, transport,
+            timeout=timeout)
+
+    def create_socket_context(self, transport, timeout=4.0):
+        """Create a new SmartServerSocketStreamMedium with default context.
+
+        This will call portable_socket_pair and pass the server side to
+        create_socket_medium along with transport.
+        It then returns the client_sock and the server.
+        """
+        server_sock, client_sock = portable_socket_pair()
+        server = self.create_socket_medium(server_sock, transport,
+                                           timeout=timeout)
+        return server, client_sock
 
     def test_smart_query_version(self):
         """Feed a canned query version to a server"""
         # wire-to-wire, using the whole stack
-        to_server = StringIO('hello\n')
-        from_server = StringIO()
         transport = local.LocalTransport(urlutils.local_path_to_url('/'))
-        server = medium.SmartServerPipeStreamMedium(
-            to_server, from_server, transport)
+        server, from_server = self.create_pipe_context('hello\n', transport)
         smart_protocol = protocol.SmartServerRequestProtocolOne(transport,
                 from_server.write)
         server._serve_one_request(smart_protocol)
@@ -649,10 +870,8 @@
     def test_response_to_canned_get(self):
         transport = memory.MemoryTransport('memory:///')
         transport.put_bytes('testfile', 'contents\nof\nfile\n')
-        to_server = StringIO('get\001./testfile\n')
-        from_server = StringIO()
-        server = medium.SmartServerPipeStreamMedium(
-            to_server, from_server, transport)
+        server, from_server = self.create_pipe_context('get\001./testfile\n',
+            transport)
         smart_protocol = protocol.SmartServerRequestProtocolOne(transport,
                 from_server.write)
         server._serve_one_request(smart_protocol)
@@ -669,10 +888,8 @@
         # VFS requests use filenames, not raw UTF-8.
         hpss_path = urlutils.escape(utf8_filename)
         transport.put_bytes(utf8_filename, 'contents\nof\nfile\n')
-        to_server = StringIO('get\001' + hpss_path + '\n')
-        from_server = StringIO()
-        server = medium.SmartServerPipeStreamMedium(
-            to_server, from_server, transport)
+        server, from_server = self.create_pipe_context(
+                'get\001' + hpss_path + '\n', transport)
         smart_protocol = protocol.SmartServerRequestProtocolOne(transport,
                 from_server.write)
         server._serve_one_request(smart_protocol)
@@ -684,10 +901,8 @@
 
     def test_pipe_like_stream_with_bulk_data(self):
         sample_request_bytes = 'command\n9\nbulk datadone\n'
-        to_server = StringIO(sample_request_bytes)
-        from_server = StringIO()
-        server = medium.SmartServerPipeStreamMedium(
-            to_server, from_server, None)
+        server, from_server = self.create_pipe_context(
+            sample_request_bytes, None)
         sample_protocol = SampleRequest(expected_bytes=sample_request_bytes)
         server._serve_one_request(sample_protocol)
         self.assertEqual('', from_server.getvalue())
@@ -696,29 +911,23 @@
 
     def test_socket_stream_with_bulk_data(self):
         sample_request_bytes = 'command\n9\nbulk datadone\n'
-        server_sock, client_sock = self.portable_socket_pair()
-        server = medium.SmartServerSocketStreamMedium(
-            server_sock, None)
+        server, client_sock = self.create_socket_context(None)
         sample_protocol = SampleRequest(expected_bytes=sample_request_bytes)
         client_sock.sendall(sample_request_bytes)
         server._serve_one_request(sample_protocol)
-        server_sock.close()
+        server._disconnect_client()
         self.assertEqual('', client_sock.recv(1))
         self.assertEqual(sample_request_bytes, sample_protocol.accepted_bytes)
         self.assertFalse(server.finished)
 
     def test_pipe_like_stream_shutdown_detection(self):
-        to_server = StringIO('')
-        from_server = StringIO()
-        server = medium.SmartServerPipeStreamMedium(to_server, from_server, None)
+        server, _ = self.create_pipe_context('', None)
         server._serve_one_request(SampleRequest('x'))
         self.assertTrue(server.finished)
 
     def test_socket_stream_shutdown_detection(self):
-        server_sock, client_sock = self.portable_socket_pair()
+        server, client_sock = self.create_socket_context(None)
         client_sock.close()
-        server = medium.SmartServerSocketStreamMedium(
-            server_sock, None)
         server._serve_one_request(SampleRequest('x'))
         self.assertTrue(server.finished)
 
@@ -735,14 +944,12 @@
         rest_of_request_bytes = 'lo\n'
         expected_response = (
             protocol.RESPONSE_VERSION_TWO + 'success\nok\x012\n')
-        server_sock, client_sock = self.portable_socket_pair()
-        server = medium.SmartServerSocketStreamMedium(
-            server_sock, None)
+        server, client_sock = self.create_socket_context(None)
         client_sock.sendall(incomplete_request_bytes)
         server_protocol = server._build_protocol()
         client_sock.sendall(rest_of_request_bytes)
         server._serve_one_request(server_protocol)
-        server_sock.close()
+        server._disconnect_client()
         self.assertEqual(expected_response, osutils.recv_all(client_sock, 50),
                          "Not a version 2 response to 'hello' request.")
         self.assertEqual('', client_sock.recv(1))
@@ -767,8 +974,7 @@
         to_server_w = os.fdopen(to_server_w, 'w', 0)
         from_server_r = os.fdopen(from_server_r, 'r', 0)
         from_server = os.fdopen(from_server, 'w', 0)
-        server = medium.SmartServerPipeStreamMedium(
-            to_server, from_server, None)
+        server = self.create_pipe_medium(to_server, from_server, None)
         # Like test_socket_stream_incomplete_request, write an incomplete
         # request (that does not end in '\n') and build a protocol from it.
         to_server_w.write(incomplete_request_bytes)
@@ -789,10 +995,8 @@
         # _serve_one_request should still process both of them as if they had
         # been received separately.
         sample_request_bytes = 'command\n'
-        to_server = StringIO(sample_request_bytes * 2)
-        from_server = StringIO()
-        server = medium.SmartServerPipeStreamMedium(
-            to_server, from_server, None)
+        server, from_server = self.create_pipe_context(
+            sample_request_bytes * 2, None)
         first_protocol = SampleRequest(expected_bytes=sample_request_bytes)
         server._serve_one_request(first_protocol)
         self.assertEqual(0, first_protocol.next_read_size())
@@ -811,9 +1015,7 @@
         # _serve_one_request should still process both of them as if they had
         # been received separately.
         sample_request_bytes = 'command\n'
-        server_sock, client_sock = self.portable_socket_pair()
-        server = medium.SmartServerSocketStreamMedium(
-            server_sock, None)
+        server, client_sock = self.create_socket_context(None)
         first_protocol = SampleRequest(expected_bytes=sample_request_bytes)
         # Put two whole requests on the wire.
         client_sock.sendall(sample_request_bytes * 2)
@@ -826,7 +1028,7 @@
         stream_still_open = server._serve_one_request(second_protocol)
         self.assertEqual(sample_request_bytes, second_protocol.accepted_bytes)
         self.assertFalse(server.finished)
-        server_sock.close()
+        server._disconnect_client()
         self.assertEqual('', client_sock.recv(1))
 
     def test_pipe_like_stream_error_handling(self):
@@ -839,7 +1041,7 @@
         def close():
             self.closed = True
         from_server.close = close
-        server = medium.SmartServerPipeStreamMedium(
+        server = self.create_pipe_medium(
             to_server, from_server, None)
         fake_protocol = ErrorRaisingProtocol(Exception('boom'))
         server._serve_one_request(fake_protocol)
@@ -848,9 +1050,7 @@
         self.assertTrue(server.finished)
 
     def test_socket_stream_error_handling(self):
-        server_sock, client_sock = self.portable_socket_pair()
-        server = medium.SmartServerSocketStreamMedium(
-            server_sock, None)
+        server, client_sock = self.create_socket_context(None)
         fake_protocol = ErrorRaisingProtocol(Exception('boom'))
         server._serve_one_request(fake_protocol)
         # recv should not block, because the other end of the socket has been
@@ -859,36 +1059,26 @@
         self.assertTrue(server.finished)
 
     def test_pipe_like_stream_keyboard_interrupt_handling(self):
-        to_server = StringIO('')
-        from_server = StringIO()
-        server = medium.SmartServerPipeStreamMedium(
-            to_server, from_server, None)
+        server, from_server = self.create_pipe_context('', None)
         fake_protocol = ErrorRaisingProtocol(KeyboardInterrupt('boom'))
         self.assertRaises(
             KeyboardInterrupt, server._serve_one_request, fake_protocol)
         self.assertEqual('', from_server.getvalue())
 
     def test_socket_stream_keyboard_interrupt_handling(self):
-        server_sock, client_sock = self.portable_socket_pair()
-        server = medium.SmartServerSocketStreamMedium(
-            server_sock, None)
+        server, client_sock = self.create_socket_context(None)
         fake_protocol = ErrorRaisingProtocol(KeyboardInterrupt('boom'))
         self.assertRaises(
             KeyboardInterrupt, server._serve_one_request, fake_protocol)
-        server_sock.close()
+        server._disconnect_client()
         self.assertEqual('', client_sock.recv(1))
 
     def build_protocol_pipe_like(self, bytes):
-        to_server = StringIO(bytes)
-        from_server = StringIO()
-        server = medium.SmartServerPipeStreamMedium(
-            to_server, from_server, None)
+        server, _ = self.create_pipe_context(bytes, None)
         return server._build_protocol()
 
     def build_protocol_socket(self, bytes):
-        server_sock, client_sock = self.portable_socket_pair()
-        server = medium.SmartServerSocketStreamMedium(
-            server_sock, None)
+        server, client_sock = self.create_socket_context(None)
         client_sock.sendall(bytes)
         client_sock.close()
         return server._build_protocol()
@@ -934,6 +1124,108 @@
         server_protocol = self.build_protocol_socket('bzr request 2\n')
         self.assertProtocolTwo(server_protocol)
 
+    def test__build_protocol_returns_if_stopping(self):
+        # _build_protocol should notice that we are stopping, and return
+        # without waiting for bytes from the client.
+        server, client_sock = self.create_socket_context(None)
+        server._stop_gracefully()
+        self.assertIs(None, server._build_protocol())
+
+    def test_socket_set_timeout(self):
+        server, _ = self.create_socket_context(None, timeout=1.23)
+        self.assertEqual(1.23, server._client_timeout)
+
+    def test_pipe_set_timeout(self):
+        server = self.create_pipe_medium(None, None, None,
+            timeout=1.23)
+        self.assertEqual(1.23, server._client_timeout)
+
+    def test_socket_wait_for_bytes_with_timeout_with_data(self):
+        server, client_sock = self.create_socket_context(None)
+        client_sock.sendall('data\n')
+        # This should not block or consume any actual content
+        self.assertFalse(server._wait_for_bytes_with_timeout(0.1))
+        data = server.read_bytes(5)
+        self.assertEqual('data\n', data)
+
+    def test_socket_wait_for_bytes_with_timeout_no_data(self):
+        server, client_sock = self.create_socket_context(None)
+        # This should timeout quickly, reporting that there wasn't any data
+        self.assertRaises(errors.ConnectionTimeout,
+                          server._wait_for_bytes_with_timeout, 0.01)
+        client_sock.close()
+        data = server.read_bytes(1)
+        self.assertEqual('', data)
+
+    def test_socket_wait_for_bytes_with_timeout_closed(self):
+        server, client_sock = self.create_socket_context(None)
+        # With the socket closed, this should return right away.
+        # It seems select.select() returns that you *can* read on the socket,
+        # even though it closed. Presumably as a way to tell it is closed?
+        # Testing shows that without sock.close() this times-out failing the
+        # test, but with it, it returns False immediately.
+        client_sock.close()
+        self.assertFalse(server._wait_for_bytes_with_timeout(10))
+        data = server.read_bytes(1)
+        self.assertEqual('', data)
+
+    def test_socket_wait_for_bytes_with_shutdown(self):
+        server, client_sock = self.create_socket_context(None)
+        t = time.time()
+        # Override the _timer functionality, so that time never increments,
+        # this way, we can be sure we stopped because of the flag, and not
+        # because of a timeout, etc.
+        server._timer = lambda: t
+        server._client_poll_timeout = 0.1
+        server._stop_gracefully()
+        server._wait_for_bytes_with_timeout(1.0)
+
+    def test_socket_serve_timeout_closes_socket(self):
+        server, client_sock = self.create_socket_context(None, timeout=0.1)
+        # This should timeout quickly, and then close the connection so that
+        # client_sock recv doesn't block.
+        server.serve()
+        self.assertEqual('', client_sock.recv(1))
+
+    def test_pipe_wait_for_bytes_with_timeout_with_data(self):
+        # We intentionally use a real pipe here, so that we can 'select' on it.
+        # You can't select() on a StringIO
+        (r_server, w_client) = os.pipe()
+        self.addCleanup(os.close, w_client)
+        with os.fdopen(r_server, 'rb') as rf_server:
+            server = self.create_pipe_medium(
+                rf_server, None, None)
+            os.write(w_client, 'data\n')
+            # This should not block or consume any actual content
+            server._wait_for_bytes_with_timeout(0.1)
+            data = server.read_bytes(5)
+            self.assertEqual('data\n', data)
+
+    def test_pipe_wait_for_bytes_with_timeout_no_data(self):
+        # We intentionally use a real pipe here, so that we can 'select' on it.
+        # You can't select() on a StringIO
+        (r_server, w_client) = os.pipe()
+        # We can't add an os.close cleanup here, because we need to control
+        # when the file handle gets closed ourselves.
+        with os.fdopen(r_server, 'rb') as rf_server:
+            server = self.create_pipe_medium(
+                rf_server, None, None)
+            if sys.platform == 'win32':
+                # Windows cannot select() on a pipe, so we just always return
+                server._wait_for_bytes_with_timeout(0.01)
+            else:
+                self.assertRaises(errors.ConnectionTimeout,
+                                  server._wait_for_bytes_with_timeout, 0.01)
+            os.close(w_client)
+            data = server.read_bytes(5)
+            self.assertEqual('', data)
+
+    def test_pipe_wait_for_bytes_no_fileno(self):
+        server, _ = self.create_pipe_context('', None)
+        # Our file doesn't support polling, so we should always just return
+        # 'you have data to consume.
+        server._wait_for_bytes_with_timeout(0.01)
+
 
 class TestGetProtocolFactoryForBytes(tests.TestCase):
     """_get_protocol_factory_for_bytes identifies the protocol factory a server
@@ -969,6 +1261,75 @@
 
 class TestSmartTCPServer(tests.TestCase):
 
+    def make_server(self):
+        """Create a SmartTCPServer that we can exercise.
+
+        Note: we don't use SmartTCPServer_for_testing because the testing
+        version overrides lots of functionality like 'serve', and we want to
+        test the raw service.
+
+        This will start the server in another thread, and wait for it to
+        indicate it has finished starting up.
+
+        :return: (server, server_thread)
+        """
+        t = _mod_transport.get_transport_from_url('memory:///')
+        server = _mod_server.SmartTCPServer(t, client_timeout=4.0)
+        server._ACCEPT_TIMEOUT = 0.1
+        # We don't use 'localhost' because that might be an IPv6 address.
+        server.start_server('127.0.0.1', 0)
+        server_thread = threading.Thread(target=server.serve,
+                                         args=(self.id(),))
+        server_thread.start()
+        # Ensure this gets called at some point
+        self.addCleanup(server._stop_gracefully)
+        server._started.wait()
+        return server, server_thread
+
+    def ensure_client_disconnected(self, client_sock):
+        """Ensure that a socket is closed, discarding all errors."""
+        try:
+            client_sock.close()
+        except Exception:
+            pass
+
+    def connect_to_server(self, server):
+        """Create a client socket that can talk to the server."""
+        client_sock = socket.socket()
+        server_info = server._server_socket.getsockname()
+        client_sock.connect(server_info)
+        self.addCleanup(self.ensure_client_disconnected, client_sock)
+        return client_sock
+
+    def connect_to_server_and_hangup(self, server):
+        """Connect to the server, and then hang up.
+        That way it doesn't sit waiting for 'accept()' to timeout.
+        """
+        # If the server has already signaled that the socket is closed, we
+        # don't need to try to connect to it. Not being set, though, the server
+        # might still close the socket while we try to connect to it. So we
+        # still have to catch the exception.
+        if server._stopped.isSet():
+            return
+        try:
+            client_sock = self.connect_to_server(server)
+            client_sock.close()
+        except socket.error, e:
+            # If the server has hung up already, that is fine.
+            pass
+
+    def say_hello(self, client_sock):
+        """Send the 'hello' smart RPC, and expect the response."""
+        client_sock.send('hello\n')
+        self.assertEqual('ok\x012\n', client_sock.recv(5))
+
+    def shutdown_server_cleanly(self, server, server_thread):
+        server._stop_gracefully()
+        self.connect_to_server_and_hangup(server)
+        server._stopped.wait()
+        server._fully_stopped.wait()
+        server_thread.join()
+
     def test_get_error_unexpected(self):
         """Error reported by server with no specific representation"""
         self.overrideEnv('BZR_NO_SMART_VFS', None)
@@ -992,6 +1353,132 @@
                                 t.get, 'something')
         self.assertContainsRe(str(err), 'some random exception')
 
+    def test_propagates_timeout(self):
+        server = _mod_server.SmartTCPServer(None, client_timeout=1.23)
+        server_sock, client_sock = portable_socket_pair()
+        handler = server._make_handler(server_sock)
+        self.assertEqual(1.23, handler._client_timeout)
+
+    def test_serve_conn_tracks_connections(self):
+        server = _mod_server.SmartTCPServer(None, client_timeout=4.0)
+        server_sock, client_sock = portable_socket_pair()
+        server.serve_conn(server_sock, '-%s' % (self.id(),))
+        self.assertEqual(1, len(server._active_connections))
+        # We still want to talk on the connection. Polling should indicate it
+        # is still active.
+        server._poll_active_connections()
+        self.assertEqual(1, len(server._active_connections))
+        # Closing the socket will end the active thread, and polling will
+        # notice and remove it from the active set.
+        client_sock.close()
+        server._poll_active_connections(0.1)
+        self.assertEqual(0, len(server._active_connections))
+
+    def test_serve_closes_out_finished_connections(self):
+        server, server_thread = self.make_server()
+        # The server is started, connect to it.
+        client_sock = self.connect_to_server(server)
+        # We send and receive on the connection, so that we know the
+        # server-side has seen the connect, and started handling the
+        # results.
+        self.say_hello(client_sock)
+        self.assertEqual(1, len(server._active_connections))
+        # Grab a handle to the thread that is processing our request
+        _, server_side_thread = server._active_connections[0]
+        # Close the connection, ask the server to stop, and wait for the
+        # server to stop, as well as the thread that was servicing the
+        # client request.
+        client_sock.close()
+        # Wait for the server-side request thread to notice we are closed.
+        server_side_thread.join()
+        # Stop the server, it should notice the connection has finished.
+        self.shutdown_server_cleanly(server, server_thread)
+        # The server should have noticed that all clients are gone before
+        # exiting.
+        self.assertEqual(0, len(server._active_connections))
+
+    def test_serve_reaps_finished_connections(self):
+        server, server_thread = self.make_server()
+        client_sock1 = self.connect_to_server(server)
+        # We send and receive on the connection, so that we know the
+        # server-side has seen the connect, and started handling the
+        # results.
+        self.say_hello(client_sock1)
+        server_handler1, server_side_thread1 = server._active_connections[0]
+        client_sock1.close()
+        server_side_thread1.join()
+        # By waiting until the first connection is fully done, the server
+        # should notice after another connection that the first has finished.
+        client_sock2 = self.connect_to_server(server)
+        self.say_hello(client_sock2)
+        server_handler2, server_side_thread2 = server._active_connections[-1]
+        # There is a race condition. We know that client_sock2 has been
+        # registered, but not that _poll_active_connections has been called. We
+        # know that it will be called before the server will accept a new
+        # connection, however. So connect one more time, and assert that we
+        # either have 1 or 2 active connections (never 3), and that the 'first'
+        # connection is not connection 1
+        client_sock3 = self.connect_to_server(server)
+        self.say_hello(client_sock3)
+        # Copy the list, so we don't have it mutating behind our back
+        conns = list(server._active_connections)
+        self.assertEqual(2, len(conns))
+        self.assertNotEqual((server_handler1, server_side_thread1), conns[0])
+        self.assertEqual((server_handler2, server_side_thread2), conns[0])
+        client_sock2.close()
+        client_sock3.close()
+        self.shutdown_server_cleanly(server, server_thread)
+
+    def test_graceful_shutdown_waits_for_clients_to_stop(self):
+        server, server_thread = self.make_server()
+        # We need something big enough that it won't fit in a single recv. So
+        # the server thread gets blocked writing content to the client until we
+        # finish reading on the client.
+        server.backing_transport.put_bytes('bigfile',
+            'a'*1024*1024)
+        client_sock = self.connect_to_server(server)
+        self.say_hello(client_sock)
+        _, server_side_thread = server._active_connections[0]
+        # Start the RPC, but don't finish reading the response
+        client_medium = medium.SmartClientAlreadyConnectedSocketMedium(
+            'base', client_sock)
+        client_client = client._SmartClient(client_medium)
+        resp, response_handler = client_client.call_expecting_body('get',
+            'bigfile')
+        self.assertEqual(('ok',), resp)
+        # Ask the server to stop gracefully, and wait for it.
+        server._stop_gracefully()
+        self.connect_to_server_and_hangup(server)
+        server._stopped.wait()
+        # It should not be accepting another connection.
+        self.assertRaises(socket.error, self.connect_to_server, server)
+        # It should also not be fully stopped
+        server._fully_stopped.wait(0.01)
+        self.assertFalse(server._fully_stopped.isSet())
+        response_handler.read_body_bytes()
+        client_sock.close()
+        server_side_thread.join()
+        server_thread.join()
+        self.assertTrue(server._fully_stopped.isSet())
+        log = self.get_log()
+        self.assertThat(log, DocTestMatches("""\
+    INFO  Requested to stop gracefully
+... Stopping SmartServerSocketStreamMedium(client=('127.0.0.1', ...
+    INFO  Waiting for 1 client(s) to finish
+""", flags=doctest.ELLIPSIS|doctest.REPORT_UDIFF))
+
+    def test_stop_gracefully_tells_handlers_to_stop(self):
+        server, server_thread = self.make_server()
+        client_sock = self.connect_to_server(server)
+        self.say_hello(client_sock)
+        server_handler, server_side_thread = server._active_connections[0]
+        self.assertFalse(server_handler.finished)
+        server._stop_gracefully()
+        self.assertTrue(server_handler.finished)
+        client_sock.close()
+        self.connect_to_server_and_hangup(server)
+        server_thread.join()
+
 
 class SmartTCPTests(tests.TestCase):
     """Tests for connection/end to end behaviour using the TCP server.
@@ -1015,15 +1502,16 @@
             mem_server.start_server()
             self.addCleanup(mem_server.stop_server)
             self.permit_url(mem_server.get_url())
-            self.backing_transport = transport.get_transport_from_url(
+            self.backing_transport = _mod_transport.get_transport_from_url(
                 mem_server.get_url())
         else:
             self.backing_transport = backing_transport
         if readonly:
             self.real_backing_transport = self.backing_transport
-            self.backing_transport = transport.get_transport_from_url(
+            self.backing_transport = _mod_transport.get_transport_from_url(
                 "readonly+" + self.backing_transport.abspath('.'))
-        self.server = server.SmartTCPServer(self.backing_transport)
+        self.server = _mod_server.SmartTCPServer(self.backing_transport,
+                                                 client_timeout=4.0)
         self.server.start_server('127.0.0.1', 0)
         self.server.start_background_thread('-' + self.id())
         self.transport = remote.RemoteTCPTransport(self.server.get_url())
@@ -1163,7 +1651,7 @@
     def test_server_started_hook_memory(self):
         """The server_started hook fires when the server is started."""
         self.hook_calls = []
-        server.SmartTCPServer.hooks.install_named_hook('server_started',
+        _mod_server.SmartTCPServer.hooks.install_named_hook('server_started',
             self.capture_server_call, None)
         self.start_server()
         # at this point, the server will be starting a thread up.
@@ -1177,10 +1665,10 @@
     def test_server_started_hook_file(self):
         """The server_started hook fires when the server is started."""
         self.hook_calls = []
-        server.SmartTCPServer.hooks.install_named_hook('server_started',
+        _mod_server.SmartTCPServer.hooks.install_named_hook('server_started',
             self.capture_server_call, None)
         self.start_server(
-            backing_transport=transport.get_transport_from_path("."))
+            backing_transport=_mod_transport.get_transport_from_path("."))
         # at this point, the server will be starting a thread up.
         # there is no indicator at the moment, so bodge it by doing a request.
         self.transport.has('.')
@@ -1194,7 +1682,7 @@
     def test_server_stopped_hook_simple_memory(self):
         """The server_stopped hook fires when the server is stopped."""
         self.hook_calls = []
-        server.SmartTCPServer.hooks.install_named_hook('server_stopped',
+        _mod_server.SmartTCPServer.hooks.install_named_hook('server_stopped',
             self.capture_server_call, None)
         self.start_server()
         result = [([self.backing_transport.base], self.transport.base)]
@@ -1211,10 +1699,10 @@
     def test_server_stopped_hook_simple_file(self):
         """The server_stopped hook fires when the server is stopped."""
         self.hook_calls = []
-        server.SmartTCPServer.hooks.install_named_hook('server_stopped',
+        _mod_server.SmartTCPServer.hooks.install_named_hook('server_stopped',
             self.capture_server_call, None)
         self.start_server(
-            backing_transport=transport.get_transport_from_path("."))
+            backing_transport=_mod_transport.get_transport_from_path("."))
         result = [(
             [self.backing_transport.base, self.backing_transport.external_url()]
             , self.transport.base)]
@@ -1356,13 +1844,13 @@
 class RemoteTransportRegistration(tests.TestCase):
 
     def test_registration(self):
-        t = transport.get_transport_from_url('bzr+ssh://example.com/path')
+        t = _mod_transport.get_transport_from_url('bzr+ssh://example.com/path')
         self.assertIsInstance(t, remote.RemoteSSHTransport)
         self.assertEqual('example.com', t._parsed_url.host)
 
     def test_bzr_https(self):
         # https://bugs.launchpad.net/bzr/+bug/128456
-        t = transport.get_transport_from_url('bzr+https://example.com/path')
+        t = _mod_transport.get_transport_from_url('bzr+https://example.com/path')
         self.assertIsInstance(t, remote.RemoteHTTPTransport)
         self.assertStartsWith(
             t._http_transport.base,
@@ -2545,7 +3033,7 @@
         from_server = StringIO()
         transport = memory.MemoryTransport('memory:///')
         server = medium.SmartServerPipeStreamMedium(
-            to_server, from_server, transport)
+            to_server, from_server, transport, timeout=4.0)
         proto = server._build_protocol()
         message_handler = proto.message_handler
         server._serve_one_request(proto)
@@ -2796,6 +3284,33 @@
             'e', # end
             output.getvalue())
 
+    def test_records_start_of_body_stream(self):
+        requester, output = self.make_client_encoder_and_output()
+        requester.set_headers({})
+        in_stream = [False]
+        def stream_checker():
+            self.assertTrue(requester.body_stream_started)
+            in_stream[0] = True
+            yield 'content'
+        flush_called = []
+        orig_flush = requester.flush
+        def tracked_flush():
+            flush_called.append(in_stream[0])
+            if in_stream[0]:
+                self.assertTrue(requester.body_stream_started)
+            else:
+                self.assertFalse(requester.body_stream_started)
+            return orig_flush()
+        requester.flush = tracked_flush
+        requester.call_with_body_stream(('one arg',), stream_checker())
+        self.assertEqual(
+            'bzr message 3 (bzr 1.6)\n' # protocol version
+            '\x00\x00\x00\x02de' # headers
+            's\x00\x00\x00\x0bl7:one arge' # args
+            'b\x00\x00\x00\x07content' # body
+            'e', output.getvalue())
+        self.assertEqual([False, True, True], flush_called)
+
 
 class StubMediumRequest(object):
     """A stub medium request that tracks the number of times accept_bytes is
@@ -3221,6 +3736,193 @@
         # encoder.
 
 
+class Test_SmartClientRequest(tests.TestCase):
+
+    def make_client_with_failing_medium(self, fail_at_write=True, response=''):
+        response_io = StringIO(response)
+        output = StringIO()
+        vendor = FirstRejectedStringIOSSHVendor(response_io, output,
+                    fail_at_write=fail_at_write)
+        ssh_params = medium.SSHParams('a host', 'a port', 'a user', 'a pass')
+        client_medium = medium.SmartSSHClientMedium('base', ssh_params, vendor)
+        smart_client = client._SmartClient(client_medium, headers={})
+        return output, vendor, smart_client
+
+    def make_response(self, args, body=None, body_stream=None):
+        response_io = StringIO()
+        response = _mod_request.SuccessfulSmartServerResponse(args, body=body,
+            body_stream=body_stream)
+        responder = protocol.ProtocolThreeResponder(response_io.write)
+        responder.send_response(response)
+        return response_io.getvalue()
+
+    def test__call_doesnt_retry_append(self):
+        response = self.make_response(('appended', '8'))
+        output, vendor, smart_client = self.make_client_with_failing_medium(
+            fail_at_write=False, response=response)
+        smart_request = client._SmartClientRequest(smart_client, 'append',
+            ('foo', ''), body='content\n')
+        self.assertRaises(errors.ConnectionReset, smart_request._call, 3)
+
+    def test__call_retries_get_bytes(self):
+        response = self.make_response(('ok',), 'content\n')
+        output, vendor, smart_client = self.make_client_with_failing_medium(
+            fail_at_write=False, response=response)
+        smart_request = client._SmartClientRequest(smart_client, 'get',
+            ('foo',))
+        response, response_handler = smart_request._call(3)
+        self.assertEqual(('ok',), response)
+        self.assertEqual('content\n', response_handler.read_body_bytes())
+
+    def test__call_noretry_get_bytes(self):
+        debug.debug_flags.add('noretry')
+        response = self.make_response(('ok',), 'content\n')
+        output, vendor, smart_client = self.make_client_with_failing_medium(
+            fail_at_write=False, response=response)
+        smart_request = client._SmartClientRequest(smart_client, 'get',
+            ('foo',))
+        self.assertRaises(errors.ConnectionReset, smart_request._call, 3)
+
+    def test__send_no_retry_pipes(self):
+        client_read, server_write = create_file_pipes()
+        server_read, client_write = create_file_pipes()
+        client_medium = medium.SmartSimplePipesClientMedium(client_read,
+            client_write, base='/')
+        smart_client = client._SmartClient(client_medium)
+        smart_request = client._SmartClientRequest(smart_client,
+            'hello', ())
+        # Close the server side
+        server_read.close()
+        encoder, response_handler = smart_request._construct_protocol(3)
+        self.assertRaises(errors.ConnectionReset,
+            smart_request._send_no_retry, encoder)
+
+    def test__send_read_response_sockets(self):
+        listen_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        listen_sock.bind(('127.0.0.1', 0))
+        listen_sock.listen(1)
+        host, port = listen_sock.getsockname()
+        client_medium = medium.SmartTCPClientMedium(host, port, '/')
+        client_medium._ensure_connection()
+        smart_client = client._SmartClient(client_medium)
+        smart_request = client._SmartClientRequest(smart_client, 'hello', ())
+        # Accept the connection, but don't actually talk to the client.
+        server_sock, _ = listen_sock.accept()
+        server_sock.close()
+        # Sockets buffer and don't really notice that the server has closed the
+        # connection until we try to read again.
+        handler = smart_request._send(3)
+        self.assertRaises(errors.ConnectionReset,
+            handler.read_response_tuple, expect_body=False)
+
+    def test__send_retries_on_write(self):
+        output, vendor, smart_client = self.make_client_with_failing_medium()
+        smart_request = client._SmartClientRequest(smart_client, 'hello', ())
+        handler = smart_request._send(3)
+        self.assertEqual('bzr message 3 (bzr 1.6)\n' # protocol
+                         '\x00\x00\x00\x02de'   # empty headers
+                         's\x00\x00\x00\tl5:helloee',
+                         output.getvalue())
+        self.assertEqual(
+            [('connect_ssh', 'a user', 'a pass', 'a host', 'a port',
+              ['bzr', 'serve', '--inet', '--directory=/', '--allow-writes']),
+             ('close',),
+             ('connect_ssh', 'a user', 'a pass', 'a host', 'a port',
+              ['bzr', 'serve', '--inet', '--directory=/', '--allow-writes']),
+            ],
+            vendor.calls)
+
+    def test__send_doesnt_retry_read_failure(self):
+        output, vendor, smart_client = self.make_client_with_failing_medium(
+            fail_at_write=False)
+        smart_request = client._SmartClientRequest(smart_client, 'hello', ())
+        handler = smart_request._send(3)
+        self.assertEqual('bzr message 3 (bzr 1.6)\n' # protocol
+                         '\x00\x00\x00\x02de'   # empty headers
+                         's\x00\x00\x00\tl5:helloee',
+                         output.getvalue())
+        self.assertEqual(
+            [('connect_ssh', 'a user', 'a pass', 'a host', 'a port',
+              ['bzr', 'serve', '--inet', '--directory=/', '--allow-writes']),
+            ],
+            vendor.calls)
+        self.assertRaises(errors.ConnectionReset, handler.read_response_tuple)
+
+    def test__send_request_retries_body_stream_if_not_started(self):
+        output, vendor, smart_client = self.make_client_with_failing_medium()
+        smart_request = client._SmartClientRequest(smart_client, 'hello', (),
+            body_stream=['a', 'b'])
+        response_handler = smart_request._send(3)
+        # We connect, get disconnected, and notice before consuming the stream,
+        # so we try again one time and succeed.
+        self.assertEqual(
+            [('connect_ssh', 'a user', 'a pass', 'a host', 'a port',
+              ['bzr', 'serve', '--inet', '--directory=/', '--allow-writes']),
+             ('close',),
+             ('connect_ssh', 'a user', 'a pass', 'a host', 'a port',
+              ['bzr', 'serve', '--inet', '--directory=/', '--allow-writes']),
+            ],
+            vendor.calls)
+        self.assertEqual('bzr message 3 (bzr 1.6)\n' # protocol
+                         '\x00\x00\x00\x02de'   # empty headers
+                         's\x00\x00\x00\tl5:helloe'
+                         'b\x00\x00\x00\x01a'
+                         'b\x00\x00\x00\x01b'
+                         'e',
+                         output.getvalue())
+
+    def test__send_request_stops_if_body_started(self):
+        # We intentionally use the python StringIO so that we can subclass it.
+        from StringIO import StringIO
+        response = StringIO()
+
+        class FailAfterFirstWrite(StringIO):
+            """Allow one 'write' call to pass, fail the rest"""
+            def __init__(self):
+                StringIO.__init__(self)
+                self._first = True
+
+            def write(self, s):
+                if self._first:
+                    self._first = False
+                    return StringIO.write(self, s)
+                raise IOError(errno.EINVAL, 'invalid file handle')
+        output = FailAfterFirstWrite()
+
+        vendor = FirstRejectedStringIOSSHVendor(response, output,
+            fail_at_write=False)
+        ssh_params = medium.SSHParams('a host', 'a port', 'a user', 'a pass')
+        client_medium = medium.SmartSSHClientMedium('base', ssh_params, vendor)
+        smart_client = client._SmartClient(client_medium, headers={})
+        smart_request = client._SmartClientRequest(smart_client, 'hello', (),
+            body_stream=['a', 'b'])
+        self.assertRaises(errors.ConnectionReset, smart_request._send, 3)
+        # We connect, and manage to get to the point that we start consuming
+        # the body stream. The next write fails, so we just stop.
+        self.assertEqual(
+            [('connect_ssh', 'a user', 'a pass', 'a host', 'a port',
+              ['bzr', 'serve', '--inet', '--directory=/', '--allow-writes']),
+             ('close',),
+            ],
+            vendor.calls)
+        self.assertEqual('bzr message 3 (bzr 1.6)\n' # protocol
+                         '\x00\x00\x00\x02de'   # empty headers
+                         's\x00\x00\x00\tl5:helloe',
+                         output.getvalue())
+
+    def test__send_disabled_retry(self):
+        debug.debug_flags.add('noretry')
+        output, vendor, smart_client = self.make_client_with_failing_medium()
+        smart_request = client._SmartClientRequest(smart_client, 'hello', ())
+        self.assertRaises(errors.ConnectionReset, smart_request._send, 3)
+        self.assertEqual(
+            [('connect_ssh', 'a user', 'a pass', 'a host', 'a port',
+              ['bzr', 'serve', '--inet', '--directory=/', '--allow-writes']),
+             ('close',),
+            ],
+            vendor.calls)
+
+
 class LengthPrefixedBodyDecoder(tests.TestCase):
 
     # XXX: TODO: make accept_reading_trailer invoke translate_response or
@@ -3560,7 +4262,7 @@
         # still work correctly.
         base_transport = remote.RemoteHTTPTransport('bzr+http://host/%7Ea/b')
         new_transport = base_transport.clone('c')
-        self.assertEqual('bzr+http://host/%7Ea/b/c/', new_transport.base)
+        self.assertEqual(base_transport.base + 'c/', new_transport.base)
         self.assertEqual(
             'c/',
             new_transport._client.remote_path_from_transport(new_transport))

=== modified file 'bzrlib/tests/test_smtp_connection.py'
--- bzrlib/tests/test_smtp_connection.py	2011-01-12 01:01:53 +0000
+++ bzrlib/tests/test_smtp_connection.py	2011-12-21 14:25:26 +0000
@@ -90,9 +90,9 @@
 class TestSMTPConnection(tests.TestCaseInTempDir):
 
     def get_connection(self, text, smtp_factory=None):
-        my_config = config.GlobalConfig.from_string(text)
-        return smtp_connection.SMTPConnection(my_config,
-                                              _smtp_factory=smtp_factory)
+        my_config = config.MemoryStack(text)
+        return smtp_connection.SMTPConnection(
+            my_config, _smtp_factory=smtp_factory)
 
     def test_defaults(self):
         conn = self.get_connection('')
@@ -101,13 +101,13 @@
         self.assertEqual(None, conn._smtp_password)
 
     def test_smtp_server(self):
-        conn = self.get_connection('[DEFAULT]\nsmtp_server=host:10\n')
+        conn = self.get_connection('smtp_server=host:10')
         self.assertEqual('host:10', conn._smtp_server)
 
     def test_missing_server(self):
         conn = self.get_connection('', smtp_factory=connection_refuser)
         self.assertRaises(errors.DefaultSMTPConnectionRefused, conn._connect)
-        conn = self.get_connection('[DEFAULT]\nsmtp_server=smtp.example.com\n',
+        conn = self.get_connection('smtp_server=smtp.example.com',
                                    smtp_factory=connection_refuser)
         self.assertRaises(errors.SMTPConnectionRefused, conn._connect)
 
@@ -115,14 +115,14 @@
         conn = self.get_connection('')
         self.assertIs(None, conn._smtp_username)
 
-        conn = self.get_connection('[DEFAULT]\nsmtp_username=joebody\n')
+        conn = self.get_connection('smtp_username=joebody')
         self.assertEqual(u'joebody', conn._smtp_username)
 
     def test_smtp_password_from_config(self):
         conn = self.get_connection('')
         self.assertIs(None, conn._smtp_password)
 
-        conn = self.get_connection('[DEFAULT]\nsmtp_password=mypass\n')
+        conn = self.get_connection('smtp_password=mypass')
         self.assertEqual(u'mypass', conn._smtp_password)
 
     def test_smtp_password_from_user(self):
@@ -161,8 +161,8 @@
         utf8_pass = unicode_pass.encode('utf-8')
         factory = WideOpenSMTPFactory()
         conn = self.get_connection(
-            u'[DEFAULT]\nsmtp_username=%s\nsmtp_password=%s\n'
-            % (user, unicode_pass), smtp_factory=factory)
+            '[DEFAULT]\nsmtp_username=%s\nsmtp_password=%s\n'
+            % (user, utf8_pass), smtp_factory=factory)
         self.assertEqual(unicode_pass, conn._smtp_password)
         conn._connect()
         self.assertEqual([('connect', 'localhost'),
@@ -254,22 +254,21 @@
             'pperez@ejemplo.com', 'user@localhost']), sorted(to))
 
     def test_destination_address_required(self):
-        class FakeConfig:
-            def get_user_option(self, option):
-                return None
-
         msg = Message()
         msg['From'] = '"J. Random Developer" <jrandom@example.com>'
         self.assertRaises(
             errors.NoDestinationAddress,
-            smtp_connection.SMTPConnection(FakeConfig()).send_email, msg)
+            smtp_connection.SMTPConnection(config.MemoryStack("")
+                                           ).send_email, msg)
 
         msg = email_message.EmailMessage('from@from.com', '', 'subject')
         self.assertRaises(
             errors.NoDestinationAddress,
-            smtp_connection.SMTPConnection(FakeConfig()).send_email, msg)
+            smtp_connection.SMTPConnection(config.MemoryStack("")
+                                           ).send_email, msg)
 
         msg = email_message.EmailMessage('from@from.com', [], 'subject')
         self.assertRaises(
             errors.NoDestinationAddress,
-            smtp_connection.SMTPConnection(FakeConfig()).send_email, msg)
+            smtp_connection.SMTPConnection(config.MemoryStack("")
+                                           ).send_email, msg)

=== modified file 'bzrlib/tests/test_source.py'
--- bzrlib/tests/test_source.py	2011-06-13 22:10:04 +0000
+++ bzrlib/tests/test_source.py	2011-12-18 15:28:38 +0000
@@ -416,3 +416,23 @@
             error_msg.extend(('', ''))
         if error_msg:
             self.fail('\n'.join(error_msg))
+
+    def test_feature_absolute_import(self):
+        """Using absolute imports means avoiding unnecesary stat and
+        open calls.
+
+        Make sure that all non-test files have absolute imports enabled.
+        """
+        missing_absolute_import = []
+        for fname, text in self.get_source_file_contents(
+                extensions=('.py', )):
+            if "/tests/" in fname or "test_" in fname:
+                # We don't really care about tests
+                continue
+            if not "from __future__ import absolute_import" in text:
+                missing_absolute_import.append(fname)
+
+        if missing_absolute_import:
+            self.fail(
+                'The following files do not have absolute_import enabled:\n'
+                '\n' + '\n'.join(missing_absolute_import))

=== modified file 'bzrlib/tests/test_status.py'
--- bzrlib/tests/test_status.py	2011-03-30 11:45:54 +0000
+++ bzrlib/tests/test_status.py	2011-11-21 19:27:55 +0000
@@ -169,7 +169,7 @@
         params = calls[0]
         self.assertIsInstance(params, _mod_status.StatusHookParams)
         attrs = ['old_tree', 'new_tree', 'to_file', 'versioned',
-            'show_ids', 'short', 'verbose']
+            'show_ids', 'short', 'verbose', 'specific_files']
         for a in attrs:
             self.assertTrue(hasattr(params, a),
                 'Attribute "%s" not found in StatusHookParam' % a)
@@ -192,7 +192,7 @@
         params = calls[0]
         self.assertIsInstance(params, _mod_status.StatusHookParams)
         attrs = ['old_tree', 'new_tree', 'to_file', 'versioned',
-            'show_ids', 'short', 'verbose']
+            'show_ids', 'short', 'verbose', 'specific_files']
         for a in attrs:
             self.assertTrue(hasattr(params, a),
                 'Attribute "%s" not found in StatusHookParam' % a)

=== modified file 'bzrlib/tests/test_tag.py'
--- bzrlib/tests/test_tag.py	2011-08-06 07:32:51 +0000
+++ bzrlib/tests/test_tag.py	2011-09-22 13:26:14 +0000
@@ -109,12 +109,14 @@
         self.assertRaises(errors.NoSuchTag, a.tags.lookup_tag, 'tag-2')
         # conflicting merge
         a.tags.set_tag('tag-2', 'z')
-        conflicts = a.tags.merge_to(b.tags)
+        updates, conflicts = a.tags.merge_to(b.tags)
+        self.assertEqual({}, updates)
         self.assertEqual(list(conflicts), [('tag-2', 'z', 'y')])
         self.assertEqual('y', b.tags.lookup_tag('tag-2'))
         # overwrite conflicts
-        conflicts = a.tags.merge_to(b.tags, overwrite=True)
+        updates, conflicts = a.tags.merge_to(b.tags, overwrite=True)
         self.assertEqual(list(conflicts), [])
+        self.assertEqual({u'tag-2': 'z'}, updates)
         self.assertEqual('z', b.tags.lookup_tag('tag-2'))
 
 

=== modified file 'bzrlib/tests/test_test_server.py'
--- bzrlib/tests/test_test_server.py	2011-02-08 16:26:23 +0000
+++ bzrlib/tests/test_test_server.py	2011-11-24 15:48:29 +0000
@@ -19,6 +19,7 @@
 import SocketServer
 import threading
 
+
 from bzrlib import (
     osutils,
     tests,
@@ -30,6 +31,21 @@
 load_tests = load_tests_apply_scenarios
 
 
+def portable_socket_pair():
+    """Return a pair of TCP sockets connected to each other.
+
+    Unlike socket.socketpair, this should work on Windows.
+    """
+    listen_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+    listen_sock.bind(('127.0.0.1', 0))
+    listen_sock.listen(1)
+    client_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+    client_sock.connect(listen_sock.getsockname())
+    server_sock, addr = listen_sock.accept()
+    listen_sock.close()
+    return server_sock, client_sock
+
+
 class TCPClient(object):
 
     def __init__(self):
@@ -61,7 +77,7 @@
         return self.sock.recv(bufsize)
 
 
-class TCPConnectionHandler(SocketServer.StreamRequestHandler):
+class TCPConnectionHandler(SocketServer.BaseRequestHandler):
 
     def handle(self):
         self.done = False
@@ -69,32 +85,39 @@
         while not self.done:
             self.handle_connection()
 
+    def readline(self):
+        # TODO: We should be buffering any extra data sent, etc. However, in
+        #       practice, we don't send extra content, so we haven't bothered
+        #       to implement it yet.
+        req = self.request.recv(4096)
+        # An empty string is allowed, to indicate the end of the connection
+        if not req or (req.endswith('\n') and req.count('\n') == 1):
+            return req
+        raise ValueError('[%r] not a simple line' % (req,))
+
     def handle_connection(self):
-        req = self.rfile.readline()
+        req = self.readline()
         if not req:
             self.done = True
         elif req == 'ping\n':
-            self.wfile.write('pong\n')
+            self.request.sendall('pong\n')
         else:
             raise ValueError('[%s] not understood' % req)
 
 
 class TestTCPServerInAThread(tests.TestCase):
 
-    scenarios = [ 
+    scenarios = [
         (name, {'server_class': getattr(test_server, name)})
         for name in
         ('TestingTCPServer', 'TestingThreadingTCPServer')]
 
-    # Set by load_tests()
-    server_class = None
-
     def get_server(self, server_class=None, connection_handler_class=None):
         if server_class is not None:
             self.server_class = server_class
         if connection_handler_class is None:
             connection_handler_class = TCPConnectionHandler
-        server =  test_server.TestingTCPServerInAThread(
+        server = test_server.TestingTCPServerInAThread(
             ('localhost', 0), self.server_class, connection_handler_class)
         server.start_server()
         self.addCleanup(server.stop_server)
@@ -158,30 +181,46 @@
         # The server won't fail until a client connect
         client = self.get_client()
         client.connect((server.host, server.port))
+        # We make sure the server wants to handle a request, but the request is
+        # guaranteed to fail. However, the server should make sure that the
+        # connection gets closed, and stop_server should then raise the
+        # original exception.
+        client.write('ping\n')
         try:
-            # Now we must force the server to answer by sending the request and
-            # waiting for some answer. But since we don't control when the
-            # server thread will be given cycles, we don't control either
-            # whether our reads or writes may hang.
-            client.sock.settimeout(0.1)
-            client.write('ping\n')
-            client.read()
-        except socket.error:
-            pass
+            self.assertEqual('', client.read())
+        except socket.error, e:
+            # On Windows, failing during 'handle' means we get
+            # 'forced-close-of-connection'. Possibly because we haven't
+            # processed the write request before we close the socket.
+            WSAECONNRESET = 10054
+            if e.errno in (WSAECONNRESET,):
+                pass
         # Now the server has raised the exception in its own thread
         self.assertRaises(CantConnect, server.stop_server)
 
     def test_server_crash_while_responding(self):
-        sync = threading.Event()
-        sync.clear()
+        # We want to ensure the exception has been caught
+        caught = threading.Event()
+        caught.clear()
+        # The thread that will serve the client, this needs to be an attribute
+        # so the handler below can modify it when it's executed (it's
+        # instantiated when the request is processed)
+        self.connection_thread = None
+
         class FailToRespond(Exception):
             pass
 
         class FailingDuringResponseHandler(TCPConnectionHandler):
 
-            def handle_connection(self):
-                req = self.rfile.readline()
-                threading.currentThread().set_sync_event(sync)
+            # We use 'request' instead of 'self' below because the test matters
+            # more and we need a container to properly set connection_thread.
+            def handle_connection(request):
+                req = request.readline()
+                # Capture the thread and make it use 'caught' so we can wait on
+                # the event that will be set when the exception is caught. We
+                # also capture the thread to know where to look.
+                self.connection_thread = threading.currentThread()
+                self.connection_thread.set_sync_event(caught)
                 raise FailToRespond()
 
         server = self.get_server(
@@ -189,32 +228,109 @@
         client = self.get_client()
         client.connect((server.host, server.port))
         client.write('ping\n')
-        sync.wait()
-        self.assertRaises(FailToRespond, server.pending_exception)
+        # Wait for the exception to be caught
+        caught.wait()
+        self.assertEqual('', client.read()) # connection closed
+        # Check that the connection thread did catch the exception,
+        # http://pad.lv/869366 was wrongly checking the server thread which
+        # works for TestingTCPServer where the connection is handled in the
+        # same thread than the server one but was racy for
+        # TestingThreadingTCPServer. Since the connection thread detaches
+        # itself before handling the request, we are guaranteed that the
+        # exception won't leak into the server thread anymore.
+        self.assertRaises(FailToRespond,
+                          self.connection_thread.pending_exception)
 
     def test_exception_swallowed_while_serving(self):
-        sync = threading.Event()
-        sync.clear()
+        # We need to ensure the exception has been caught
+        caught = threading.Event()
+        caught.clear()
+        # The thread that will serve the client, this needs to be an attribute
+        # so the handler below can access it when it's executed (it's
+        # instantiated when the request is processed)
+        self.connection_thread = None
         class CantServe(Exception):
             pass
 
         class FailingWhileServingConnectionHandler(TCPConnectionHandler):
 
-            def handle(self):
-                # We want to sync with the thread that is serving the
-                # connection.
-                threading.currentThread().set_sync_event(sync)
+            # We use 'request' instead of 'self' below because the test matters
+            # more and we need a container to properly set connection_thread.
+            def handle(request):
+                # Capture the thread and make it use 'caught' so we can wait on
+                # the event that will be set when the exception is caught. We
+                # also capture the thread to know where to look.
+                self.connection_thread = threading.currentThread()
+                self.connection_thread.set_sync_event(caught)
                 raise CantServe()
 
         server = self.get_server(
             connection_handler_class=FailingWhileServingConnectionHandler)
+        self.assertEquals(True, server.server.serving)
         # Install the exception swallower
         server.set_ignored_exceptions(CantServe)
         client = self.get_client()
         # Connect to the server so the exception is raised there
         client.connect((server.host, server.port))
-        # Wait for the exception to propagate.
-        sync.wait()
+        # Wait for the exception to be caught
+        caught.wait()
+        self.assertEqual('', client.read()) # connection closed
         # The connection wasn't served properly but the exception should have
-        # been swallowed.
-        server.pending_exception()
+        # been swallowed (see test_server_crash_while_responding remark about
+        # http://pad.lv/869366 explaining why we can't check the server thread
+        # here). More precisely, the exception *has* been caught and captured
+        # but it is cleared when joining the thread (or trying to acquire the
+        # exception) and as such won't propagate to the server thread.
+        self.assertIs(None, self.connection_thread.pending_exception())
+        self.assertIs(None, server.pending_exception())
+
+    def test_handle_request_closes_if_it_doesnt_process(self):
+        server = self.get_server()
+        client = self.get_client()
+        server.server.serving = False
+        client.connect((server.host, server.port))
+        self.assertEqual('', client.read())
+
+
+class TestTestingSmartServer(tests.TestCase):
+
+    def test_sets_client_timeout(self):
+        server = test_server.TestingSmartServer(('localhost', 0), None, None,
+            root_client_path='/no-such-client/path')
+        self.assertEqual(test_server._DEFAULT_TESTING_CLIENT_TIMEOUT,
+                         server._client_timeout)
+        sock = socket.socket()
+        h = server._make_handler(sock)
+        self.assertEqual(test_server._DEFAULT_TESTING_CLIENT_TIMEOUT,
+                         h._client_timeout)
+
+
+class FakeServer(object):
+    """Minimal implementation to pass to TestingSmartConnectionHandler"""
+    backing_transport = None
+    root_client_path = '/'
+
+
+class TestTestingSmartConnectionHandler(tests.TestCase):
+
+    def test_connection_timeout_suppressed(self):
+        self.overrideAttr(test_server, '_DEFAULT_TESTING_CLIENT_TIMEOUT', 0.01)
+        s = FakeServer()
+        server_sock, client_sock = portable_socket_pair()
+        # This should timeout quickly, but not generate an exception.
+        handler = test_server.TestingSmartConnectionHandler(server_sock,
+            server_sock.getpeername(), s)
+
+    def test_connection_shutdown_while_serving_no_error(self):
+        s = FakeServer()
+        server_sock, client_sock = portable_socket_pair()
+        class ShutdownConnectionHandler(
+            test_server.TestingSmartConnectionHandler):
+
+            def _build_protocol(self):
+                self.finished = True
+                return super(ShutdownConnectionHandler, self)._build_protocol()
+        # This should trigger shutdown after the entering _build_protocol, and
+        # we should exit cleanly, without raising an exception.
+        handler = ShutdownConnectionHandler(server_sock,
+            server_sock.getpeername(), s)

=== modified file 'bzrlib/tests/test_timestamp.py'
--- bzrlib/tests/test_timestamp.py	2011-01-10 22:20:12 +0000
+++ bzrlib/tests/test_timestamp.py	2011-11-20 01:49:32 +0000
@@ -58,6 +58,45 @@
         # offset of three minutes
         self.assertEqual((1173193459, +3 * 60),
             timestamp.parse_patch_date('2007-03-06 15:07:19 +0003'))
+        # No space between time and offset
+        self.assertEqual((1173193459, -5 * 3600),
+            timestamp.parse_patch_date('2007-03-06 10:04:19-0500'))
+        # Extra spacing
+        self.assertEqual((1173193459, -5 * 3600),
+            timestamp.parse_patch_date('2007-03-06     10:04:19     -0500'))
+
+    def test_parse_patch_date_bad(self):
+        self.assertRaises(ValueError, timestamp.parse_patch_date,
+            'NOT A TIME')
+        # Extra data at end
+        self.assertRaises(ValueError, timestamp.parse_patch_date,
+            '2007-03-06 10:04:19 -0500x')
+        # Missing day
+        self.assertRaises(ValueError, timestamp.parse_patch_date,
+            '2007-03 10:04:19 -0500')
+        # Missing seconds
+        self.assertRaises(ValueError, timestamp.parse_patch_date,
+            '2007-03-06 10:04 -0500')
+        # Missing offset
+        self.assertRaises(ValueError, timestamp.parse_patch_date,
+            '2007-03-06 10:04:19')
+        # Missing plus or minus in offset
+        self.assertRaises(ValueError, timestamp.parse_patch_date,
+            '2007-03-06 10:04:19 0500')
+        # Invalid hour in offset
+        self.assertRaises(ValueError, timestamp.parse_patch_date,
+            '2007-03-06 10:04:19 +2400')
+        self.assertRaises(ValueError, timestamp.parse_patch_date,
+            '2007-03-06 10:04:19 -2400')
+        # Invalid minute in offset
+        self.assertRaises(ValueError, timestamp.parse_patch_date,
+            '2007-03-06 10:04:19 -0560')
+        # Too many digits in offset
+        self.assertRaises(ValueError, timestamp.parse_patch_date,
+            '2007-03-06 10:04:19 79500')
+        # Minus sign in middle of offset
+        self.assertRaises(ValueError, timestamp.parse_patch_date,
+            '2007-03-06 10:04:19 +05-5')
 
 
 class UnpackHighresDateTests(tests.TestCase):

=== modified file 'bzrlib/tests/test_trace.py'
--- bzrlib/tests/test_trace.py	2011-08-23 09:29:27 +0000
+++ bzrlib/tests/test_trace.py	2011-12-05 14:21:55 +0000
@@ -20,6 +20,7 @@
 
 from cStringIO import StringIO
 import errno
+import logging
 import os
 import re
 import sys
@@ -113,8 +114,9 @@
         msg = _format_exception()
         # Even though Windows and Linux differ for 'os.rmdir', they both give
         # 'No such file' for open()
+        # However it now gets translated so we can not test for a specific message
         self.assertContainsRe(msg,
-            r'^bzr: ERROR: \[Errno .*\] No such file.*nosuchfile')
+            r'^bzr: ERROR: \[Errno .*\] .*nosuchfile')
 
     def test_format_pywintypes_error(self):
         self.requireFeature(features.pywintypes)
@@ -253,7 +255,7 @@
         # have to do a replaceent here as well.
         self.assertContainsRe(log, "ascii argument: \xb5".decode('utf8',
             'replace'))
-        
+
     def test_show_error(self):
         show_error('error1')
         show_error(u'error2 \xb5 blah')
@@ -345,6 +347,72 @@
         self.assertEqual(0, get_verbosity_level())
 
 
+class TestLogging(TestCase):
+    """Check logging functionality robustly records information"""
+
+    def test_note(self):
+        trace.note("Noted")
+        self.assertEqual("    INFO  Noted\n", self.get_log())
+
+    def test_warning(self):
+        trace.warning("Warned")
+        self.assertEqual(" WARNING  Warned\n", self.get_log())
+
+    def test_log(self):
+        logging.getLogger("bzr").error("Errored")
+        self.assertEqual("   ERROR  Errored\n", self.get_log())
+
+    def test_log_sub(self):
+        logging.getLogger("bzr.test_log_sub").debug("Whispered")
+        self.assertEqual("   DEBUG  Whispered\n", self.get_log())
+
+    def test_log_unicode_msg(self):
+        logging.getLogger("bzr").debug(u"\xa7")
+        self.assertEqual(u"   DEBUG  \xa7\n", self.get_log())
+
+    def test_log_unicode_arg(self):
+        logging.getLogger("bzr").debug("%s", u"\xa7")
+        self.assertEqual(u"   DEBUG  \xa7\n", self.get_log())
+
+    def test_log_utf8_msg(self):
+        logging.getLogger("bzr").debug("\xc2\xa7")
+        self.assertEqual(u"   DEBUG  \xa7\n", self.get_log())
+
+    def test_log_utf8_arg(self):
+        logging.getLogger("bzr").debug("%s", "\xc2\xa7")
+        self.assertEqual(u"   DEBUG  \xa7\n", self.get_log())
+
+    def test_log_bytes_msg(self):
+        logging.getLogger("bzr").debug("\xa7")
+        log = self.get_log()
+        self.assertContainsString(log, "UnicodeDecodeError: ")
+        self.assertContainsString(log,
+            "Logging record unformattable: '\\xa7' % ()\n")
+
+    def test_log_bytes_arg(self):
+        logging.getLogger("bzr").debug("%s", "\xa7")
+        log = self.get_log()
+        self.assertContainsString(log, "UnicodeDecodeError: ")
+        self.assertContainsString(log,
+            "Logging record unformattable: '%s' % ('\\xa7',)\n")
+
+    def test_log_mixed_strings(self):
+        logging.getLogger("bzr").debug(u"%s", "\xa7")
+        log = self.get_log()
+        self.assertContainsString(log, "UnicodeDecodeError: ")
+        self.assertContainsString(log,
+            "Logging record unformattable: u'%s' % ('\\xa7',)\n")
+
+    def test_log_repr_broken(self):
+        class BadRepr(object):
+            def __repr__(self):
+                raise ValueError("Broken object")
+        logging.getLogger("bzr").debug("%s", BadRepr())
+        log = self.get_log()
+        self.assertContainsRe(log, "ValueError: Broken object\n")
+        self.assertContainsRe(log, "Logging record unformattable: '%s' % .*\n")
+
+
 class TestBzrLog(TestCaseInTempDir):
 
     def test_log_rollover(self):

=== modified file 'bzrlib/tests/test_transform.py'
--- bzrlib/tests/test_transform.py	2011-07-21 06:46:34 +0000
+++ bzrlib/tests/test_transform.py	2011-11-17 13:45:49 +0000
@@ -1476,8 +1476,7 @@
         # The rename will fail because the target directory is not empty (but
         # raises FileExists anyway).
         err = self.assertRaises(errors.FileExists, tt_helper)
-        self.assertContainsRe(str(err),
-            "^File exists: .+/baz")
+        self.assertEndsWith(err.path, "/baz")
 
     def test_two_directories_clash(self):
         def tt_helper():
@@ -1495,8 +1494,7 @@
                 wt.unlock()
                 raise
         err = self.assertRaises(errors.FileExists, tt_helper)
-        self.assertContainsRe(str(err),
-            "^File exists: .+/foo")
+        self.assertEndsWith(err.path, "/foo")
 
     def test_two_directories_clash_finalize(self):
         def tt_helper():
@@ -1514,8 +1512,7 @@
                 tt.finalize()
                 raise
         err = self.assertRaises(errors.FileExists, tt_helper)
-        self.assertContainsRe(str(err),
-            "^File exists: .+/foo")
+        self.assertEndsWith(err.path, "/foo")
 
     def test_file_to_directory(self):
         wt = self.make_branch_and_tree('.')

=== modified file 'bzrlib/tests/test_transport.py'
--- bzrlib/tests/test_transport.py	2011-08-15 14:31:55 +0000
+++ bzrlib/tests/test_transport.py	2011-12-14 19:44:40 +0000
@@ -16,6 +16,7 @@
 
 
 from cStringIO import StringIO
+import errno
 import os
 import subprocess
 import sys
@@ -442,6 +443,29 @@
         self.assertEqual('chroot-%d:///' % id(server), server.get_url())
 
 
+class TestHooks(tests.TestCase):
+    """Basic tests for transport hooks"""
+
+    def _get_connected_transport(self):
+        return transport.ConnectedTransport("bogus:nowhere")
+
+    def test_transporthooks_initialisation(self):
+        """Check all expected transport hook points are set up"""
+        hookpoint = transport.TransportHooks()
+        self.assertTrue("post_connect" in hookpoint,
+            "post_connect not in %s" % (hookpoint,))
+
+    def test_post_connect(self):
+        """Ensure the post_connect hook is called when _set_transport is"""
+        calls = []
+        transport.Transport.hooks.install_named_hook("post_connect",
+            calls.append, None)
+        t = self._get_connected_transport()
+        self.assertLength(0, calls)
+        t._set_connection("connection", "auth")
+        self.assertEqual(calls, [t])
+
+
 class PathFilteringDecoratorTransportTest(tests.TestCase):
     """Pathfilter decoration specific tests."""
 
@@ -738,6 +762,29 @@
         self.assertEquals(t.local_abspath(''), here)
 
 
+class TestLocalTransportMutation(tests.TestCaseInTempDir):
+
+    def test_local_transport_mkdir(self):
+        here = osutils.abspath('.')
+        t = transport.get_transport(here)
+        t.mkdir('test')
+        self.assertTrue(os.path.exists('test'))
+
+    def test_local_transport_mkdir_permission_denied(self):
+        # See https://bugs.launchpad.net/bzr/+bug/606537
+        here = osutils.abspath('.')
+        t = transport.get_transport(here)
+        def fake_chmod(path, mode):
+            e = OSError('permission denied')
+            e.errno = errno.EPERM
+            raise e
+        self.overrideAttr(os, 'chmod', fake_chmod)
+        t.mkdir('test')
+        t.mkdir('test2', mode=0707)
+        self.assertTrue(os.path.exists('test'))
+        self.assertTrue(os.path.exists('test2'))
+
+
 class TestLocalTransportWriteStream(tests.TestCaseWithTransport):
 
     def test_local_fdatasync_calls_fdatasync(self):
@@ -760,6 +807,10 @@
             self.assertEquals(f.read(), 'foo')
         self.assertEquals(len(calls), 1, calls)
 
+    def test_missing_directory(self):
+        t = self.get_transport('.')
+        self.assertRaises(errors.NoSuchFile, t.open_write_stream, 'dir/foo')
+
 
 class TestWin32LocalTransport(tests.TestCase):
 

=== modified file 'bzrlib/tests/test_ui.py'
--- bzrlib/tests/test_ui.py	2011-05-16 13:39:39 +0000
+++ bzrlib/tests/test_ui.py	2011-10-08 19:01:59 +0000
@@ -111,6 +111,8 @@
     def test_text_ui_get_boolean(self):
         stdin = tests.StringIOWrapper("y\n" # True
                                       "n\n" # False
+                                      " \n y \n" # True
+                                      " no \n" # False
                                       "yes with garbage\nY\n" # True
                                       "not an answer\nno\n" # False
                                       "I'm sure!\nyes\n" # True
@@ -125,9 +127,77 @@
         self.assertEqual(False, factory.get_boolean(u""))
         self.assertEqual(True, factory.get_boolean(u""))
         self.assertEqual(False, factory.get_boolean(u""))
-        self.assertEqual("foo\n", factory.stdin.read())
-        # stdin should be empty
-        self.assertEqual('', factory.stdin.readline())
+        self.assertEqual(True, factory.get_boolean(u""))
+        self.assertEqual(False, factory.get_boolean(u""))
+        self.assertEqual("foo\n", factory.stdin.read())
+        # stdin should be empty
+        self.assertEqual('', factory.stdin.readline())
+        # return false on EOF
+        self.assertEqual(False, factory.get_boolean(u""))
+
+    def test_text_ui_choose_bad_parameters(self):
+        stdin = tests.StringIOWrapper()
+        stdout = tests.StringIOWrapper()
+        stderr = tests.StringIOWrapper()
+        factory = _mod_ui_text.TextUIFactory(stdin, stdout, stderr)
+        # invalid default index
+        self.assertRaises(ValueError, factory.choose, u"", u"&Yes\n&No", 3)
+        # duplicated choice
+        self.assertRaises(ValueError, factory.choose, u"", u"&choice\n&ChOiCe")
+        # duplicated shortcut
+        self.assertRaises(ValueError, factory.choose, u"", u"&choice1\nchoi&ce2")
+
+    def test_text_ui_choose_prompt(self):
+        stdin = tests.StringIOWrapper()
+        stdout = tests.StringIOWrapper()
+        stderr = tests.StringIOWrapper()
+        factory = _mod_ui_text.TextUIFactory(stdin, stdout, stderr)
+        # choices with explicit shortcuts
+        factory.choose(u"prompt", u"&yes\n&No\nmore &info")
+        self.assertEqual("prompt ([y]es, [N]o, more [i]nfo): \n", factory.stderr.getvalue())
+        # automatic shortcuts
+        factory.stderr.truncate(0)
+        factory.choose(u"prompt", u"yes\nNo\nmore info")
+        self.assertEqual("prompt ([y]es, [N]o, [m]ore info): \n", factory.stderr.getvalue())
+
+    def test_text_ui_choose_return_values(self):
+        choose = lambda: factory.choose(u"", u"&Yes\n&No\nMaybe\nmore &info", 3)
+        stdin = tests.StringIOWrapper("y\n" # 0
+                                      "n\n" # 1
+                                      " \n" # default: 3
+                                      " no \n" # 1
+                                      "b\na\nd \n" # bad shortcuts, all ignored
+                                      "yes with garbage\nY\n" # 0
+                                      "not an answer\nno\n" # 1
+                                      "info\nmore info\n" # 3
+                                      "Maybe\n" # 2
+                                      "foo\n")
+        stdout = tests.StringIOWrapper()
+        stderr = tests.StringIOWrapper()
+        factory = _mod_ui_text.TextUIFactory(stdin, stdout, stderr)
+        self.assertEqual(0, choose())
+        self.assertEqual(1, choose())
+        self.assertEqual(3, choose())
+        self.assertEqual(1, choose())
+        self.assertEqual(0, choose())
+        self.assertEqual(1, choose())
+        self.assertEqual(3, choose())
+        self.assertEqual(2, choose())
+        self.assertEqual("foo\n", factory.stdin.read())
+        # stdin should be empty
+        self.assertEqual('', factory.stdin.readline())
+        # return None on EOF
+        self.assertEqual(None, choose())
+
+    def test_text_ui_choose_no_default(self):
+        stdin = tests.StringIOWrapper(" \n" # no default, invalid!
+                                      " yes \n" # 0
+                                      "foo\n")
+        stdout = tests.StringIOWrapper()
+        stderr = tests.StringIOWrapper()
+        factory = _mod_ui_text.TextUIFactory(stdin, stdout, stderr)
+        self.assertEqual(0, factory.choose(u"", u"&Yes\n&No"))
+        self.assertEqual("foo\n", factory.stdin.read())
 
     def test_text_ui_get_integer(self):
         stdin = tests.StringIOWrapper(
@@ -170,8 +240,8 @@
         output = out.getvalue()
         self.assertContainsRe(output,
             "| foo *\r\r  *\r*")
-        self.assertContainsRe(output,
-            r"what do you want\? \[y/n\]: what do you want\? \[y/n\]: ")
+        self.assertContainsString(output,
+            r"what do you want? ([y]es, [n]o): what do you want? ([y]es, [n]o): ")
         # stdin should have been totally consumed
         self.assertEqual('', factory.stdin.readline())
 

=== modified file 'bzrlib/tests/test_urlutils.py'
--- bzrlib/tests/test_urlutils.py	2011-08-15 13:12:36 +0000
+++ bzrlib/tests/test_urlutils.py	2011-12-18 20:46:39 +0000
@@ -20,7 +20,12 @@
 import sys
 
 from bzrlib import osutils, urlutils, win32utils
-from bzrlib.errors import InvalidURL, InvalidURLJoin, InvalidRebaseURLs
+from bzrlib.errors import (
+    InvalidURL,
+    InvalidURLJoin,
+    InvalidRebaseURLs,
+    PathNotChild,
+    )
 from bzrlib.tests import TestCaseInTempDir, TestCase, TestSkipped
 
 
@@ -457,6 +462,12 @@
         self.assertEqual(('file:///C:', '/foo'), extract('file://', '/C:/foo'))
         self.assertEqual(('file:///d|', '/path'), extract('file://', '/d|/path'))
         self.assertRaises(InvalidURL, extract, 'file://', '/path')
+        # Root drives without slash treated as invalid, see bug #841322
+        self.assertEqual(('file:///C:', '/'), extract('file://', '/C:/'))
+        self.assertRaises(InvalidURL, extract, 'file://', '/C:')
+        # Invalid without drive separator or following forward slash
+        self.assertRaises(InvalidURL, extract, 'file://', '/C')
+        self.assertRaises(InvalidURL, extract, 'file://', '/C:ool')
 
     def test_split(self):
         # Test bzrlib.urlutils.split()
@@ -495,6 +506,7 @@
 
     def test_split_segment_parameters_raw(self):
         split_segment_parameters_raw = urlutils.split_segment_parameters_raw
+        # Check relative references with absolute paths
         self.assertEquals(("/some/path", []),
             split_segment_parameters_raw("/some/path"))
         self.assertEquals(("/some/path", ["tip"]),
@@ -505,19 +517,22 @@
             split_segment_parameters_raw("/somedir/path,heads%2Ftip"))
         self.assertEquals(("/somedir/path", ["heads%2Ftip", "bar"]),
             split_segment_parameters_raw("/somedir/path,heads%2Ftip,bar"))
-        self.assertEquals(("/", ["key1=val1"]),
+        # Check relative references with relative paths
+        self.assertEquals(("", ["key1=val1"]),
             split_segment_parameters_raw(",key1=val1"))
         self.assertEquals(("foo/", ["key1=val1"]),
             split_segment_parameters_raw("foo/,key1=val1"))
-        self.assertEquals(("/foo", ["key1=val1"]),
+        self.assertEquals(("foo", ["key1=val1"]),
             split_segment_parameters_raw("foo,key1=val1"))
         self.assertEquals(("foo/base,la=bla/other/elements", []),
             split_segment_parameters_raw("foo/base,la=bla/other/elements"))
         self.assertEquals(("foo/base,la=bla/other/elements", ["a=b"]),
             split_segment_parameters_raw("foo/base,la=bla/other/elements,a=b"))
+        # TODO: Check full URLs as well as relative references
 
     def test_split_segment_parameters(self):
         split_segment_parameters = urlutils.split_segment_parameters
+        # Check relative references with absolute paths
         self.assertEquals(("/some/path", {}),
             split_segment_parameters("/some/path"))
         self.assertEquals(("/some/path", {"branch": "tip"}),
@@ -532,7 +547,8 @@
                 "/somedir/path,ref=heads%2Ftip,key1=val1"))
         self.assertEquals(("/somedir/path", {"ref": "heads%2F=tip"}),
             split_segment_parameters("/somedir/path,ref=heads%2F=tip"))
-        self.assertEquals(("/", {"key1": "val1"}),
+        # Check relative references with relative paths
+        self.assertEquals(("", {"key1": "val1"}),
             split_segment_parameters(",key1=val1"))
         self.assertEquals(("foo/", {"key1": "val1"}),
             split_segment_parameters("foo/,key1=val1"))
@@ -541,6 +557,7 @@
         self.assertEquals(("foo/base,key1=val1/other/elements",
             {"key2": "val2"}), split_segment_parameters(
                 "foo/base,key1=val1/other/elements,key2=val2"))
+        # TODO: Check full URLs as well as relative references
 
     def test_win32_strip_local_trailing_slash(self):
         strip = urlutils._win32_strip_local_trailing_slash
@@ -905,3 +922,124 @@
         url3 = url.clone()
         self.assertIsNot(url, url3)
         self.assertEquals(url, url3)
+
+
+class TestFileRelpath(TestCase):
+
+    # GZ 2011-11-18: A way to override all path handling functions to one
+    #                platform or another for testing would be nice.
+
+    def _with_posix_paths(self):
+        self.overrideAttr(urlutils, "local_path_from_url",
+            urlutils._posix_local_path_from_url)
+        self.overrideAttr(urlutils, "MIN_ABS_FILEURL_LENGTH", len("file:///"))
+        self.overrideAttr(osutils, "normpath", osutils._posix_normpath)
+        self.overrideAttr(osutils, "abspath", osutils._posix_abspath)
+        self.overrideAttr(osutils, "normpath", osutils._posix_normpath)
+        self.overrideAttr(osutils, "pathjoin", osutils.posixpath.join)
+        self.overrideAttr(osutils, "split", osutils.posixpath.split)
+        self.overrideAttr(osutils, "MIN_ABS_PATHLENGTH", 1)
+
+    def _with_win32_paths(self):
+        self.overrideAttr(urlutils, "local_path_from_url",
+            urlutils._win32_local_path_from_url)
+        self.overrideAttr(urlutils, "MIN_ABS_FILEURL_LENGTH",
+            urlutils.WIN32_MIN_ABS_FILEURL_LENGTH)
+        self.overrideAttr(osutils, "abspath", osutils._win32_abspath)
+        self.overrideAttr(osutils, "normpath", osutils._win32_normpath)
+        self.overrideAttr(osutils, "pathjoin", osutils._win32_pathjoin)
+        self.overrideAttr(osutils, "split", osutils.ntpath.split)
+        self.overrideAttr(osutils, "MIN_ABS_PATHLENGTH", 3)
+
+    def test_same_url_posix(self):
+        self._with_posix_paths()
+        self.assertEquals("",
+            urlutils.file_relpath("file:///a", "file:///a"))
+        self.assertEquals("",
+            urlutils.file_relpath("file:///a", "file:///a/"))
+        self.assertEquals("",
+            urlutils.file_relpath("file:///a/", "file:///a"))
+
+    def test_same_url_win32(self):
+        self._with_win32_paths()
+        self.assertEquals("",
+            urlutils.file_relpath("file:///A:/", "file:///A:/"))
+        self.assertEquals("",
+            urlutils.file_relpath("file:///A|/", "file:///A:/"))
+        self.assertEquals("",
+            urlutils.file_relpath("file:///A:/b/", "file:///A:/b/"))
+        self.assertEquals("",
+            urlutils.file_relpath("file:///A:/b", "file:///A:/b/"))
+        self.assertEquals("",
+            urlutils.file_relpath("file:///A:/b/", "file:///A:/b"))
+
+    def test_child_posix(self):
+        self._with_posix_paths()
+        self.assertEquals("b",
+            urlutils.file_relpath("file:///a", "file:///a/b"))
+        self.assertEquals("b",
+            urlutils.file_relpath("file:///a/", "file:///a/b"))
+        self.assertEquals("b/c",
+            urlutils.file_relpath("file:///a", "file:///a/b/c"))
+
+    def test_child_win32(self):
+        self._with_win32_paths()
+        self.assertEquals("b",
+            urlutils.file_relpath("file:///A:/", "file:///A:/b"))
+        self.assertEquals("b",
+            urlutils.file_relpath("file:///A|/", "file:///A:/b"))
+        self.assertEquals("c",
+            urlutils.file_relpath("file:///A:/b", "file:///A:/b/c"))
+        self.assertEquals("c",
+            urlutils.file_relpath("file:///A:/b/", "file:///A:/b/c"))
+        self.assertEquals("c/d",
+            urlutils.file_relpath("file:///A:/b", "file:///A:/b/c/d"))
+
+    def test_sibling_posix(self):
+        self._with_posix_paths()
+        self.assertRaises(PathNotChild,
+            urlutils.file_relpath, "file:///a/b", "file:///a/c")
+        self.assertRaises(PathNotChild,
+            urlutils.file_relpath, "file:///a/b/", "file:///a/c")
+        self.assertRaises(PathNotChild,
+            urlutils.file_relpath, "file:///a/b/", "file:///a/c/")
+
+    def test_sibling_win32(self):
+        self._with_win32_paths()
+        self.assertRaises(PathNotChild,
+            urlutils.file_relpath, "file:///A:/b", "file:///A:/c")
+        self.assertRaises(PathNotChild,
+            urlutils.file_relpath, "file:///A:/b/", "file:///A:/c")
+        self.assertRaises(PathNotChild,
+            urlutils.file_relpath, "file:///A:/b/", "file:///A:/c/")
+
+    def test_parent_posix(self):
+        self._with_posix_paths()
+        self.assertRaises(PathNotChild,
+            urlutils.file_relpath, "file:///a/b", "file:///a")
+        self.assertRaises(PathNotChild,
+            urlutils.file_relpath, "file:///a/b", "file:///a/")
+
+    def test_parent_win32(self):
+        self._with_win32_paths()
+        self.assertRaises(PathNotChild,
+            urlutils.file_relpath, "file:///A:/b", "file:///A:/")
+        self.assertRaises(PathNotChild,
+            urlutils.file_relpath, "file:///A:/b/c", "file:///A:/b")
+
+
+class QuoteTests(TestCase):
+
+    def test_quote(self):
+        self.assertEqual('abc%20def', urlutils.quote('abc def'))
+        self.assertEqual('abc%2Fdef', urlutils.quote('abc/def', safe=''))
+        self.assertEqual('abc/def', urlutils.quote('abc/def', safe='/'))
+
+    def test_quote_tildes(self):
+        self.assertEqual('%7Efoo', urlutils.quote('~foo'))
+        self.assertEqual('~foo', urlutils.quote('~foo', safe='/~'))
+
+    def test_unquote(self):
+        self.assertEqual('%', urlutils.unquote('%25'))
+        self.assertEqual('\xc3\xa5', urlutils.unquote('%C3%A5'))
+        self.assertEqual(u"\xe5", urlutils.unquote(u'\xe5'))

=== modified file 'bzrlib/tests/test_version_info.py'
--- bzrlib/tests/test_version_info.py	2011-06-14 17:01:15 +0000
+++ bzrlib/tests/test_version_info.py	2012-01-03 13:08:44 +0000
@@ -35,7 +35,7 @@
 from bzrlib.version_info_formats.format_python import PythonVersionInfoBuilder
 
 
-class TestVersionInfo(TestCaseWithTransport):
+class VersionInfoTestCase(TestCaseWithTransport):
 
     def create_branch(self):
         wt = self.make_branch_and_tree('branch')
@@ -72,6 +72,9 @@
 
         return wt
 
+
+class TestVersionInfoRio(VersionInfoTestCase):
+
     def test_rio_null(self):
         wt = self.make_branch_and_tree('branch')
 
@@ -91,32 +94,37 @@
         val = sio.getvalue()
         self.assertContainsRe(val, 'revno: 1.1.1')
 
-    def test_rio_version_text(self):
+    def regen_text(self, wt, **kwargs):
+        sio = StringIO()
+        builder = RioVersionInfoBuilder(wt.branch, working_tree=wt, **kwargs)
+        builder.generate(sio)
+        val = sio.getvalue()
+        return val
+
+    def test_simple(self):
         wt = self.create_branch()
 
-        def regen(**kwargs):
-            sio = StringIO()
-            builder = RioVersionInfoBuilder(wt.branch, working_tree=wt,
-                                            **kwargs)
-            builder.generate(sio)
-            val = sio.getvalue()
-            return val
-
-        val = regen()
+        val = self.regen_text(wt)
         self.assertContainsRe(val, 'build-date:')
         self.assertContainsRe(val, 'date:')
         self.assertContainsRe(val, 'revno: 3')
         self.assertContainsRe(val, 'revision-id: r3')
 
-        val = regen(check_for_clean=True)
+    def test_clean(self):
+        wt = self.create_branch()
+        val = self.regen_text(wt, check_for_clean=True)
         self.assertContainsRe(val, 'clean: True')
 
+    def test_no_clean(self):
+        wt = self.create_branch()
         self.build_tree(['branch/c'])
-        val = regen(check_for_clean=True)
+        val = self.regen_text(wt, check_for_clean=True)
         self.assertContainsRe(val, 'clean: False')
-        os.remove('branch/c')
-
-        val = regen(include_revision_history=True)
+
+    def test_history(self):
+        wt = self.create_branch()
+
+        val = self.regen_text(wt, include_revision_history=True)
         self.assertContainsRe(val, 'id: r1')
         self.assertContainsRe(val, 'message: a')
         self.assertContainsRe(val, 'id: r2')
@@ -124,6 +132,15 @@
         self.assertContainsRe(val, 'id: r3')
         self.assertContainsRe(val, 'message: \xc3\xa52') # utf8 encoding '\xe5'
 
+    def regen(self, wt, **kwargs):
+        sio = StringIO()
+        builder = RioVersionInfoBuilder(wt.branch, working_tree=wt, **kwargs)
+        builder.generate(sio)
+        sio.seek(0)
+        stanzas = list(read_stanzas(sio))
+        self.assertEqual(1, len(stanzas))
+        return stanzas[0]
+
     def test_rio_version_hook(self):
         def update_stanza(rev, stanza):
             stanza.add('bla', 'bloe')
@@ -131,86 +148,93 @@
             'revision', update_stanza, None)
         wt = self.create_branch()
 
-        def regen(**kwargs):
-            sio = StringIO()
-            builder = RioVersionInfoBuilder(wt.branch, working_tree=wt,
-                                            **kwargs)
-            builder.generate(sio)
-            sio.seek(0)
-            stanzas = list(read_stanzas(sio))
-            self.assertEqual(1, len(stanzas))
-            return stanzas[0]
-
-        stanza = regen()
+        stanza = self.regen(wt)
         self.assertEqual(['bloe'], stanza.get_all('bla'))
 
-    def test_rio_version(self):
+    def get_one_stanza(self, stanza, key):
+        new_stanzas = list(read_stanzas(StringIO(stanza[key].encode('utf8'))))
+        self.assertEqual(1, len(new_stanzas))
+        return new_stanzas[0]
+
+    def test_build_date(self):
         wt = self.create_branch()
-
-        def regen(**kwargs):
-            sio = StringIO()
-            builder = RioVersionInfoBuilder(wt.branch, working_tree=wt,
-                                            **kwargs)
-            builder.generate(sio)
-            sio.seek(0)
-            stanzas = list(read_stanzas(sio))
-            self.assertEqual(1, len(stanzas))
-            return stanzas[0]
-
-        def get_one_stanza(stanza, key):
-            new_stanzas = list(read_stanzas(
-                                StringIO(stanza[key].encode('utf8'))))
-            self.assertEqual(1, len(new_stanzas))
-            return new_stanzas[0]
-
-        stanza = regen()
+        stanza = self.regen(wt)
         self.assertTrue('date' in stanza)
         self.assertTrue('build-date' in stanza)
         self.assertEqual(['3'], stanza.get_all('revno'))
         self.assertEqual(['r3'], stanza.get_all('revision-id'))
 
-        stanza = regen(check_for_clean=True)
-        self.assertEqual(['True'], stanza.get_all('clean'))
-
+    def test_not_clean(self):
+        wt = self.create_branch()
         self.build_tree(['branch/c'])
-        stanza = regen(check_for_clean=True, include_file_revisions=True)
+        stanza = self.regen(wt, check_for_clean=True, include_file_revisions=True)
         self.assertEqual(['False'], stanza.get_all('clean'))
 
+    def test_file_revisions(self):
+        wt = self.create_branch()
+        self.build_tree(['branch/c'])
+        stanza = self.regen(wt, check_for_clean=True, include_file_revisions=True)
         # This assumes it's being run against a tree that does not update the
         # root revision on every commit.
-        file_rev_stanza = get_one_stanza(stanza, 'file-revisions')
+        file_rev_stanza = self.get_one_stanza(stanza, 'file-revisions')
         self.assertEqual(['', 'a', 'b', 'c'], file_rev_stanza.get_all('path'))
         self.assertEqual(['r1', 'r3', 'r2', 'unversioned'],
             file_rev_stanza.get_all('revision'))
-        os.remove('branch/c')
 
-        stanza = regen(include_revision_history=True)
-        revision_stanza = get_one_stanza(stanza, 'revisions')
+    def test_revision_history(self):
+        wt = self.create_branch()
+        stanza = self.regen(wt, include_revision_history=True)
+        revision_stanza = self.get_one_stanza(stanza, 'revisions')
         self.assertEqual(['r1', 'r2', 'r3'], revision_stanza.get_all('id'))
         self.assertEqual(['a', 'b', u'\xe52'], revision_stanza.get_all('message'))
         self.assertEqual(3, len(revision_stanza.get_all('date')))
 
+    def test_file_revisions_with_rename(self):
         # a was modified, so it should show up modified again
+        wt = self.create_branch()
         self.build_tree(['branch/a', 'branch/c'])
         wt.add('c')
         wt.rename_one('b', 'd')
-        stanza = regen(check_for_clean=True, include_file_revisions=True)
-        file_rev_stanza = get_one_stanza(stanza, 'file-revisions')
+        stanza = self.regen(wt, check_for_clean=True, include_file_revisions=True)
+        file_rev_stanza = self.get_one_stanza(stanza, 'file-revisions')
         self.assertEqual(['', 'a', 'b', 'c', 'd'],
                           file_rev_stanza.get_all('path'))
         self.assertEqual(['r1', 'modified', 'renamed to d', 'new',
                           'renamed from b'],
                          file_rev_stanza.get_all('revision'))
 
+    def test_file_revisions_with_removal(self):
+        wt = self.create_branch()
+        self.build_tree(['branch/a', 'branch/c'])
+        wt.add('c')
+        wt.rename_one('b', 'd')
+
         wt.commit('modified', rev_id='r4')
+
         wt.remove(['c', 'd'])
         os.remove('branch/d')
-        stanza = regen(check_for_clean=True, include_file_revisions=True)
-        file_rev_stanza = get_one_stanza(stanza, 'file-revisions')
+        stanza = self.regen(wt, check_for_clean=True, include_file_revisions=True)
+        file_rev_stanza = self.get_one_stanza(stanza, 'file-revisions')
         self.assertEqual(['', 'a', 'c', 'd'], file_rev_stanza.get_all('path'))
         self.assertEqual(['r1', 'r4', 'unversioned', 'removed'],
                          file_rev_stanza.get_all('revision'))
 
+    def test_revision(self):
+        wt = self.create_branch()
+        self.build_tree(['branch/a', 'branch/c'])
+        wt.add('c')
+        wt.rename_one('b', 'd')
+
+        stanza = self.regen(wt, check_for_clean=True,
+            include_file_revisions=True, revision_id=wt.last_revision())
+        file_rev_stanza = self.get_one_stanza(stanza, 'file-revisions')
+        self.assertEqual(['', 'a', 'b'], file_rev_stanza.get_all('path'))
+        self.assertEqual(['r1', 'r3', 'r2'],
+                         file_rev_stanza.get_all('revision'))
+
+
+class PythonVersionInfoTests(VersionInfoTestCase):
+
     def test_python_null(self):
         wt = self.make_branch_and_tree('branch')
 
@@ -231,45 +255,45 @@
         val = sio.getvalue()
         self.assertContainsRe(val, "'revno': '1.1.1'")
 
+    def regen(self, wt, **kwargs):
+        """Create a test module, import and return it"""
+        outf = open('test_version_information.py', 'wb')
+        try:
+            builder = PythonVersionInfoBuilder(wt.branch, working_tree=wt,
+                                               **kwargs)
+            builder.generate(outf)
+        finally:
+            outf.close()
+        module_info = imp.find_module('test_version_information',
+                                      [os.getcwdu()])
+        tvi = imp.load_module('tvi', *module_info)
+        # Make sure the module isn't cached
+        sys.modules.pop('tvi', None)
+        sys.modules.pop('test_version_information', None)
+        # Delete the compiled versions, because we are generating
+        # a new file fast enough that python doesn't detect it
+        # needs to recompile, and using sleep() just makes the
+        # test slow
+        if os.path.exists('test_version_information.pyc'):
+            os.remove('test_version_information.pyc')
+        if os.path.exists('test_version_information.pyo'):
+            os.remove('test_version_information.pyo')
+        return tvi
+
     def test_python_version(self):
         wt = self.create_branch()
 
-        def regen(**kwargs):
-            """Create a test module, import and return it"""
-            outf = open('test_version_information.py', 'wb')
-            try:
-                builder = PythonVersionInfoBuilder(wt.branch, working_tree=wt,
-                                                   **kwargs)
-                builder.generate(outf)
-            finally:
-                outf.close()
-            module_info = imp.find_module('test_version_information',
-                                          [os.getcwdu()])
-            tvi = imp.load_module('tvi', *module_info)
-            # Make sure the module isn't cached
-            sys.modules.pop('tvi', None)
-            sys.modules.pop('test_version_information', None)
-            # Delete the compiled versions, because we are generating
-            # a new file fast enough that python doesn't detect it
-            # needs to recompile, and using sleep() just makes the
-            # test slow
-            if os.path.exists('test_version_information.pyc'):
-                os.remove('test_version_information.pyc')
-            if os.path.exists('test_version_information.pyo'):
-                os.remove('test_version_information.pyo')
-            return tvi
-
-        tvi = regen()
+        tvi = self.regen(wt)
         self.assertEqual('3', tvi.version_info['revno'])
         self.assertEqual('r3', tvi.version_info['revision_id'])
         self.assertTrue(tvi.version_info.has_key('date'))
         self.assertEqual(None, tvi.version_info['clean'])
 
-        tvi = regen(check_for_clean=True)
+        tvi = self.regen(wt, check_for_clean=True)
         self.assertEqual(True, tvi.version_info['clean'])
 
         self.build_tree(['branch/c'])
-        tvi = regen(check_for_clean=True, include_file_revisions=True)
+        tvi = self.regen(wt, check_for_clean=True, include_file_revisions=True)
         self.assertEqual(False, tvi.version_info['clean'])
         self.assertEqual(['', 'a', 'b', 'c'],
                          sorted(tvi.file_revisions.keys()))
@@ -278,7 +302,7 @@
         self.assertEqual('unversioned', tvi.file_revisions['c'])
         os.remove('branch/c')
 
-        tvi = regen(include_revision_history=True)
+        tvi = self.regen(wt, include_revision_history=True)
 
         rev_info = [(rev, message) for rev, message, timestamp, timezone
                                    in tvi.revisions]
@@ -288,7 +312,7 @@
         self.build_tree(['branch/a', 'branch/c'])
         wt.add('c')
         wt.rename_one('b', 'd')
-        tvi = regen(check_for_clean=True, include_file_revisions=True)
+        tvi = self.regen(wt, check_for_clean=True, include_file_revisions=True)
         self.assertEqual(['', 'a', 'b', 'c', 'd'],
                           sorted(tvi.file_revisions.keys()))
         self.assertEqual('modified', tvi.file_revisions['a'])
@@ -299,13 +323,16 @@
         wt.commit('modified', rev_id='r4')
         wt.remove(['c', 'd'])
         os.remove('branch/d')
-        tvi = regen(check_for_clean=True, include_file_revisions=True)
+        tvi = self.regen(wt, check_for_clean=True, include_file_revisions=True)
         self.assertEqual(['', 'a', 'c', 'd'],
                           sorted(tvi.file_revisions.keys()))
         self.assertEqual('r4', tvi.file_revisions['a'])
         self.assertEqual('unversioned', tvi.file_revisions['c'])
         self.assertEqual('removed', tvi.file_revisions['d'])
 
+
+class CustomVersionInfoTests(VersionInfoTestCase):
+
     def test_custom_null(self):
         sio = StringIO()
         wt = self.make_branch_and_tree('branch')
@@ -328,32 +355,41 @@
         builder.generate(sio)
         self.assertEquals("1.1.1 revid: o2", sio.getvalue())
 
-    def test_custom_version_text(self):
+    def regen(self, wt, tpl, **kwargs):
+        sio = StringIO()
+        builder = CustomVersionInfoBuilder(wt.branch, working_tree=wt,
+                                           template=tpl, **kwargs)
+        builder.generate(sio)
+        val = sio.getvalue()
+        return val
+
+    def test_build_date(self):
         wt = self.create_branch()
 
-        def regen(tpl, **kwargs):
-            sio = StringIO()
-            builder = CustomVersionInfoBuilder(wt.branch, working_tree=wt,
-                                               template=tpl, **kwargs)
-            builder.generate(sio)
-            val = sio.getvalue()
-            return val
-
-        val = regen('build-date: "{build_date}"\ndate: "{date}"')
+        val = self.regen(wt, 'build-date: "{build_date}"\ndate: "{date}"')
         self.assertContainsRe(val, 'build-date: "[0-9-+: ]+"')
         self.assertContainsRe(val, 'date: "[0-9-+: ]+"')
 
-        val = regen('revno: {revno}')
+    def test_revno(self):
+        wt = self.create_branch()
+        val = self.regen(wt, 'revno: {revno}')
         self.assertEqual(val, 'revno: 3')
 
-        val = regen('revision-id: {revision_id}')
+    def test_revision_id(self):
+        wt = self.create_branch()
+        val = self.regen(wt, 'revision-id: {revision_id}')
         self.assertEqual(val, 'revision-id: r3')
 
-        val = regen('clean: {clean}', check_for_clean=True)
+    def test_clean(self):
+        wt = self.create_branch()
+        val = self.regen(wt, 'clean: {clean}', check_for_clean=True)
         self.assertEqual(val, 'clean: 1')
 
+    def test_not_clean(self):
+        wt = self.create_branch()
+
         self.build_tree(['branch/c'])
-        val = regen('clean: {clean}', check_for_clean=True)
+        val = self.regen(wt, 'clean: {clean}', check_for_clean=True)
         self.assertEqual(val, 'clean: 0')
         os.remove('branch/c')
 

=== added file 'bzrlib/tests/test_vf_search.py'
--- bzrlib/tests/test_vf_search.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_vf_search.py	2011-12-05 14:12:23 +0000
@@ -0,0 +1,239 @@
+# Copyright (C) 2007-2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+from bzrlib import (
+    graph as _mod_graph,
+    tests,
+    vf_search,
+    )
+from bzrlib.revision import NULL_REVISION
+from bzrlib.tests.test_graph import TestGraphBase
+
+# Ancestry 1:
+#
+#  NULL_REVISION
+#       |
+#     rev1
+#      /\
+#  rev2a rev2b
+#     |    |
+#   rev3  /
+#     |  /
+#   rev4
+ancestry_1 = {'rev1': [NULL_REVISION], 'rev2a': ['rev1'], 'rev2b': ['rev1'],
+              'rev3': ['rev2a'], 'rev4': ['rev3', 'rev2b']}
+
+# Ancestry 2:
+#
+#  NULL_REVISION
+#    /    \
+# rev1a  rev1b
+#   |
+# rev2a
+#   |
+# rev3a
+#   |
+# rev4a
+ancestry_2 = {'rev1a': [NULL_REVISION], 'rev2a': ['rev1a'],
+              'rev1b': [NULL_REVISION], 'rev3a': ['rev2a'], 'rev4a': ['rev3a']}
+
+
+# Extended history shortcut
+#  NULL_REVISION
+#       |
+#       a
+#       |\
+#       b |
+#       | |
+#       c |
+#       | |
+#       d |
+#       |\|
+#       e f
+extended_history_shortcut = {'a': [NULL_REVISION],
+                             'b': ['a'],
+                             'c': ['b'],
+                             'd': ['c'],
+                             'e': ['d'],
+                             'f': ['a', 'd'],
+                            }
+
+
+class TestSearchResultRefine(tests.TestCase):
+
+    def make_graph(self, ancestors):
+        return _mod_graph.Graph(_mod_graph.DictParentsProvider(ancestors))
+
+    def test_refine(self):
+        # Used when pulling from a stacked repository, so test some revisions
+        # being satisfied from the stacking branch.
+        g = self.make_graph(
+            {"tip":["mid"], "mid":["base"], "tag":["base"],
+             "base":[NULL_REVISION], NULL_REVISION:[]})
+        result = vf_search.SearchResult(set(['tip', 'tag']),
+            set([NULL_REVISION]), 4, set(['tip', 'mid', 'tag', 'base']))
+        result = result.refine(set(['tip']), set(['mid']))
+        recipe = result.get_recipe()
+        # We should be starting from tag (original head) and mid (seen ref)
+        self.assertEqual(set(['mid', 'tag']), recipe[1])
+        # We should be stopping at NULL (original stop) and tip (seen head)
+        self.assertEqual(set([NULL_REVISION, 'tip']), recipe[2])
+        self.assertEqual(3, recipe[3])
+        result = result.refine(set(['mid', 'tag', 'base']),
+            set([NULL_REVISION]))
+        recipe = result.get_recipe()
+        # We should be starting from nothing (NULL was known as a cut point)
+        self.assertEqual(set([]), recipe[1])
+        # We should be stopping at NULL (original stop) and tip (seen head) and
+        # tag (seen head) and mid(seen mid-point head). We could come back and
+        # define this as not including mid, for minimal results, but it is
+        # still 'correct' to include mid, and simpler/easier.
+        self.assertEqual(set([NULL_REVISION, 'tip', 'tag', 'mid']), recipe[2])
+        self.assertEqual(0, recipe[3])
+        self.assertTrue(result.is_empty())
+
+
+class TestSearchResultFromParentMap(TestGraphBase):
+
+    def assertSearchResult(self, start_keys, stop_keys, key_count, parent_map,
+                           missing_keys=()):
+        (start, stop, count) = vf_search.search_result_from_parent_map(
+            parent_map, missing_keys)
+        self.assertEqual((sorted(start_keys), sorted(stop_keys), key_count),
+                         (sorted(start), sorted(stop), count))
+
+    def test_no_parents(self):
+        self.assertSearchResult([], [], 0, {})
+        self.assertSearchResult([], [], 0, None)
+
+    def test_ancestry_1(self):
+        self.assertSearchResult(['rev4'], [NULL_REVISION], len(ancestry_1),
+                                ancestry_1)
+
+    def test_ancestry_2(self):
+        self.assertSearchResult(['rev1b', 'rev4a'], [NULL_REVISION],
+                                len(ancestry_2), ancestry_2)
+        self.assertSearchResult(['rev1b', 'rev4a'], [],
+                                len(ancestry_2)+1, ancestry_2,
+                                missing_keys=[NULL_REVISION])
+
+    def test_partial_search(self):
+        parent_map = dict((k,extended_history_shortcut[k])
+                          for k in ['e', 'f'])
+        self.assertSearchResult(['e', 'f'], ['d', 'a'], 2,
+                                parent_map)
+        parent_map.update((k,extended_history_shortcut[k])
+                          for k in ['d', 'a'])
+        self.assertSearchResult(['e', 'f'], ['c', NULL_REVISION], 4,
+                                parent_map)
+        parent_map['c'] = extended_history_shortcut['c']
+        self.assertSearchResult(['e', 'f'], ['b'], 6,
+                                parent_map, missing_keys=[NULL_REVISION])
+        parent_map['b'] = extended_history_shortcut['b']
+        self.assertSearchResult(['e', 'f'], [], 7,
+                                parent_map, missing_keys=[NULL_REVISION])
+
+
+class TestLimitedSearchResultFromParentMap(TestGraphBase):
+
+    def assertSearchResult(self, start_keys, stop_keys, key_count, parent_map,
+                           missing_keys, tip_keys, depth):
+        (start, stop, count) = vf_search.limited_search_result_from_parent_map(
+            parent_map, missing_keys, tip_keys, depth)
+        self.assertEqual((sorted(start_keys), sorted(stop_keys), key_count),
+                         (sorted(start), sorted(stop), count))
+
+    def test_empty_ancestry(self):
+        self.assertSearchResult([], [], 0, {}, (), ['tip-rev-id'], 10)
+
+    def test_ancestry_1(self):
+        self.assertSearchResult(['rev4'], ['rev1'], 4,
+                                ancestry_1, (), ['rev1'], 10)
+        self.assertSearchResult(['rev2a', 'rev2b'], ['rev1'], 2,
+                                ancestry_1, (), ['rev1'], 1)
+
+
+    def test_multiple_heads(self):
+        self.assertSearchResult(['e', 'f'], ['a'], 5,
+                                extended_history_shortcut, (), ['a'], 10)
+        # Note that even though we only take 1 step back, we find 'f', which
+        # means the described search will still find d and c.
+        self.assertSearchResult(['f'], ['a'], 4,
+                                extended_history_shortcut, (), ['a'], 1)
+        self.assertSearchResult(['f'], ['a'], 4,
+                                extended_history_shortcut, (), ['a'], 2)
+
+
+class TestPendingAncestryResultRefine(tests.TestCase):
+
+    def make_graph(self, ancestors):
+        return _mod_graph.Graph(_mod_graph.DictParentsProvider(ancestors))
+
+    def test_refine(self):
+        # Used when pulling from a stacked repository, so test some revisions
+        # being satisfied from the stacking branch.
+        g = self.make_graph(
+            {"tip":["mid"], "mid":["base"], "tag":["base"],
+             "base":[NULL_REVISION], NULL_REVISION:[]})
+        result = vf_search.PendingAncestryResult(['tip', 'tag'], None)
+        result = result.refine(set(['tip']), set(['mid']))
+        self.assertEqual(set(['mid', 'tag']), result.heads)
+        result = result.refine(set(['mid', 'tag', 'base']),
+            set([NULL_REVISION]))
+        self.assertEqual(set([NULL_REVISION]), result.heads)
+        self.assertTrue(result.is_empty())
+
+
+class TestPendingAncestryResultGetKeys(tests.TestCaseWithMemoryTransport):
+    """Tests for bzrlib.graph.PendingAncestryResult."""
+
+    def test_get_keys(self):
+        builder = self.make_branch_builder('b')
+        builder.start_series()
+        builder.build_snapshot('rev-1', None, [
+            ('add', ('', 'root-id', 'directory', ''))])
+        builder.build_snapshot('rev-2', ['rev-1'], [])
+        builder.finish_series()
+        repo = builder.get_branch().repository
+        repo.lock_read()
+        self.addCleanup(repo.unlock)
+        result = vf_search.PendingAncestryResult(['rev-2'], repo)
+        self.assertEqual(set(['rev-1', 'rev-2']), set(result.get_keys()))
+
+    def test_get_keys_excludes_ghosts(self):
+        builder = self.make_branch_builder('b')
+        builder.start_series()
+        builder.build_snapshot('rev-1', None, [
+            ('add', ('', 'root-id', 'directory', ''))])
+        builder.build_snapshot('rev-2', ['rev-1', 'ghost'], [])
+        builder.finish_series()
+        repo = builder.get_branch().repository
+        repo.lock_read()
+        self.addCleanup(repo.unlock)
+        result = vf_search.PendingAncestryResult(['rev-2'], repo)
+        self.assertEqual(sorted(['rev-1', 'rev-2']), sorted(result.get_keys()))
+
+    def test_get_keys_excludes_null(self):
+        # Make a 'graph' with an iter_ancestry that returns NULL_REVISION
+        # somewhere other than the last element, which can happen in real
+        # ancestries.
+        class StubGraph(object):
+            def iter_ancestry(self, keys):
+                return [(NULL_REVISION, ()), ('foo', (NULL_REVISION,))]
+        result = vf_search.PendingAncestryResult(['rev-3'], None)
+        result_keys = result._get_keys(StubGraph())
+        # Only the non-null keys from the ancestry appear.
+        self.assertEqual(set(['foo']), set(result_keys))

=== modified file 'bzrlib/tests/test_win32utils.py'
--- bzrlib/tests/test_win32utils.py	2011-06-14 01:26:41 +0000
+++ bzrlib/tests/test_win32utils.py	2011-12-19 10:37:57 +0000
@@ -20,6 +20,7 @@
 
 from bzrlib import (
     osutils,
+    symbol_versioning,
     tests,
     win32utils,
     )
@@ -187,6 +188,51 @@
         self.assertEquals('not-existing', p)
 
 
+class TestLocations(TestCase):
+    """Tests for windows specific path and name retrieving functions"""
+
+    def test__ensure_unicode_deprecated(self):
+        s = "text"
+        u1 = self.applyDeprecated(symbol_versioning.deprecated_in((2, 5, 0)),
+            win32utils._ensure_unicode, s)
+        self.assertEqual(s, u1)
+        self.assertIsInstance(u1, unicode)
+        u2 = self.applyDeprecated(symbol_versioning.deprecated_in((2, 5, 0)),
+            win32utils._ensure_unicode, u1)
+        self.assertIs(u1, u2)
+    
+    def test_appdata_unicode_deprecated(self):
+        self.overrideEnv("APPDATA", "fakepath")
+        s = win32utils.get_appdata_location()
+        u = self.applyDeprecated(symbol_versioning.deprecated_in((2, 5, 0)),
+            win32utils.get_appdata_location_unicode)
+        self.assertEqual(s, u)
+        self.assertIsInstance(s, unicode)
+
+    def test_home_unicode_deprecated(self):
+        s = win32utils.get_home_location()
+        u = self.applyDeprecated(symbol_versioning.deprecated_in((2, 5, 0)),
+            win32utils.get_home_location_unicode)
+        self.assertEqual(s, u)
+        self.assertIsInstance(s, unicode)
+
+    def test_user_unicode_deprecated(self):
+        self.overrideEnv("USERNAME", "alien")
+        s = win32utils.get_user_name()
+        u = self.applyDeprecated(symbol_versioning.deprecated_in((2, 5, 0)),
+            win32utils.get_user_name_unicode)
+        self.assertEqual(s, u)
+        self.assertIsInstance(s, unicode)
+
+    def test_host_unicode_deprecated(self):
+        self.overrideEnv("COMPUTERNAME", "alienbox")
+        s = win32utils.get_host_name()
+        u = self.applyDeprecated(symbol_versioning.deprecated_in((2, 5, 0)),
+            win32utils.get_host_name_unicode)
+        self.assertEqual(s, u)
+        self.assertIsInstance(s, unicode)
+
+
 class TestLocationsCtypes(TestCase):
 
     _test_needs_features = [CtypesFeature]
@@ -328,3 +374,56 @@
         self.assertCommandLine([u"rm", u"x*"], "-m pdb rm x*", ["rm", u"x*"])
         self.assertCommandLine([u"add", u"d/f1", u"d/f2"], "-m pdb add d/*",
             ["add", u"d/*"])
+
+
+class TestGetEnvironUnicode(tests.TestCase):
+    """Tests for accessing the environment via the windows wide api"""
+
+    _test_needs_features = [CtypesFeature, features.win32_feature]
+
+    def setUp(self):
+        super(TestGetEnvironUnicode, self).setUp()
+        self.overrideEnv("TEST", "1")
+
+    def test_get(self):
+        """In the normal case behaves the same as os.environ access"""
+        self.assertEqual("1", win32utils.get_environ_unicode("TEST"))
+
+    def test_unset(self):
+        """A variable not present in the environment gives None by default"""
+        del os.environ["TEST"]
+        self.assertIs(None, win32utils.get_environ_unicode("TEST"))
+
+    def test_unset_default(self):
+        """A variable not present in the environment gives passed default"""
+        del os.environ["TEST"]
+        self.assertIs("a", win32utils.get_environ_unicode("TEST", "a"))
+
+    def test_unicode(self):
+        """A non-ascii variable is returned as unicode"""
+        unicode_val = u"\xa7" # non-ascii character present in many encodings
+        try:
+            bytes_val = unicode_val.encode(osutils.get_user_encoding())
+        except UnicodeEncodeError:
+            self.skip("Couldn't encode non-ascii string to place in environ")
+        os.environ["TEST"] = bytes_val
+        self.assertEqual(unicode_val, win32utils.get_environ_unicode("TEST"))
+
+    def test_long(self):
+        """A variable bigger than heuristic buffer size is still accessible"""
+        big_val = "x" * (2<<10)
+        os.environ["TEST"] = big_val
+        self.assertEqual(big_val, win32utils.get_environ_unicode("TEST"))
+
+    def test_unexpected_error(self):
+        """An error from the underlying platform function is propogated"""
+        ERROR_INVALID_PARAMETER = 87
+        SetLastError = win32utils.ctypes.windll.kernel32.SetLastError
+        def failer(*args, **kwargs):
+            SetLastError(ERROR_INVALID_PARAMETER)
+            return 0
+        self.overrideAttr(win32utils.get_environ_unicode, "_c_function",
+            failer)
+        e = self.assertRaises(WindowsError,
+            win32utils.get_environ_unicode, "TEST")
+        self.assertEqual(e.winerror, ERROR_INVALID_PARAMETER)

=== modified file 'bzrlib/tests/test_workingtree.py'
--- bzrlib/tests/test_workingtree.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/test_workingtree.py	2011-12-22 19:54:56 +0000
@@ -79,6 +79,13 @@
             workingtree.format_registry.set_default(old_format)
         self.assertEqual(old_format, workingtree.format_registry.get_default())
 
+    def test_from_string(self):
+        self.assertIsInstance(
+            SampleTreeFormat.from_string("Sample tree format."),
+            SampleTreeFormat)
+        self.assertRaises(AssertionError,
+            SampleTreeFormat.from_string, "Different format string.")
+
     def test_get_set_default_format_by_key(self):
         old_format = workingtree.format_registry.get_default()
         # default is 6
@@ -120,14 +127,15 @@
         self.assertEqual('subdir', relpath)
 
 
-class SampleTreeFormat(workingtree.WorkingTreeFormat):
+class SampleTreeFormat(workingtree.WorkingTreeFormatMetaDir):
     """A sample format
 
     this format is initializable, unsupported to aid in testing the
     open and open_downlevel routines.
     """
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See WorkingTreeFormat.get_format_string()."""
         return "Sample tree format."
 
@@ -172,14 +180,14 @@
         # is the right format object found for a working tree?
         branch = self.make_branch('branch')
         self.assertRaises(errors.NoWorkingTree,
-            workingtree.WorkingTreeFormat.find_format_string, branch.bzrdir)
+            workingtree.WorkingTreeFormatMetaDir.find_format_string, branch.bzrdir)
         transport = branch.bzrdir.get_workingtree_transport(None)
         transport.mkdir('.')
         transport.put_bytes("format", "some format name")
         # The format does not have to be known by Bazaar,
         # find_format_string just retrieves the name
         self.assertEquals("some format name",
-            workingtree.WorkingTreeFormat.find_format_string(branch.bzrdir))
+            workingtree.WorkingTreeFormatMetaDir.find_format_string(branch.bzrdir))
 
     def test_find_format(self):
         # is the right format object found for a working tree?
@@ -191,14 +199,14 @@
             dir.create_branch()
             format.initialize(dir)
             t = transport.get_transport(url)
-            found_format = workingtree.WorkingTreeFormat.find_format(dir)
+            found_format = workingtree.WorkingTreeFormatMetaDir.find_format(dir)
             self.assertIsInstance(found_format, format.__class__)
         check_format(workingtree_3.WorkingTreeFormat3(), "bar")
 
     def test_find_format_no_tree(self):
         dir = bzrdir.BzrDirMetaFormat1().initialize('.')
         self.assertRaises(errors.NoWorkingTree,
-                          workingtree.WorkingTreeFormat.find_format,
+                          workingtree.WorkingTreeFormatMetaDir.find_format,
                           dir)
 
     def test_find_format_unknown_format(self):
@@ -207,7 +215,7 @@
         dir.create_branch()
         SampleTreeFormat().initialize(dir)
         self.assertRaises(errors.UnknownFormatError,
-                          workingtree.WorkingTreeFormat.find_format,
+                          workingtree.WorkingTreeFormatMetaDir.find_format,
                           dir)
 
     def test_register_unregister_format(self):
@@ -235,6 +243,20 @@
             self.applyDeprecated(symbol_versioning.deprecated_in((2, 4, 0)),
                 workingtree.WorkingTreeFormat.get_formats))
 
+    def test_find_format_with_features(self):
+        tree = self.make_branch_and_tree('.', format='2a')
+        tree.update_feature_flags({"name": "necessity"})
+        found_format = workingtree.WorkingTreeFormatMetaDir.find_format(
+            tree.bzrdir)
+        self.assertIsInstance(found_format, workingtree.WorkingTreeFormat)
+        self.assertEquals(found_format.features.get("name"), "necessity")
+        self.assertRaises(errors.MissingFeature, found_format.check_support_status,
+            True)
+        self.addCleanup(workingtree.WorkingTreeFormatMetaDir.unregister_feature,
+            "name")
+        workingtree.WorkingTreeFormatMetaDir.register_feature("name")
+        found_format.check_support_status(True)
+
 
 class TestWorkingTreeIterEntriesByDir_wSubtrees(TestCaseWithTransport):
 

=== modified file 'bzrlib/tests/test_workingtree_4.py'
--- bzrlib/tests/test_workingtree_4.py	2011-05-26 08:05:45 +0000
+++ bzrlib/tests/test_workingtree_4.py	2011-12-05 17:57:53 +0000
@@ -29,7 +29,7 @@
     workingtree_4,
     )
 from bzrlib.lockdir import LockDir
-from bzrlib.tests import TestCaseWithTransport, TestSkipped
+from bzrlib.tests import TestCaseWithTransport, TestSkipped, features
 from bzrlib.tree import InterTree
 
 
@@ -673,6 +673,21 @@
                               tree_iter_changes, ['bar', 'foo'])
         self.assertEqual(e.paths, ['foo'])
 
+    def test_iter_changes_unversioned_non_ascii(self):
+        """Unversioned non-ascii paths should be reported as unicode"""
+        self.requireFeature(features.UnicodeFilenameFeature)
+        tree = self.make_branch_and_tree('.')
+        self.build_tree_contents([('f', '')])
+        tree.add(['f'], ['f-id'])
+        def tree_iter_changes(tree, files):
+            return list(tree.iter_changes(tree.basis_tree(),
+                specific_files=files, require_versioned=True))
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
+        e = self.assertRaises(errors.PathsNotVersionedError,
+            tree_iter_changes, tree, [u'\xa7', u'\u03c0'])
+        self.assertEqual(e.paths, [u'\xa7', u'\u03c0'])
+
     def get_tree_with_cachable_file_foo(self):
         tree = self.make_branch_and_tree('.')
         tree.lock_write()

=== modified file 'bzrlib/tests/test_wsgi.py'
--- bzrlib/tests/test_wsgi.py	2011-01-12 01:01:53 +0000
+++ bzrlib/tests/test_wsgi.py	2011-12-23 10:01:08 +0000
@@ -65,7 +65,7 @@
         self.headers = headers
 
 
-class TestWSGI(tests.TestCase, WSGITestMixin):
+class TestWSGI(tests.TestCaseInTempDir, WSGITestMixin):
 
     def setUp(self):
         tests.TestCase.setUp(self)

=== modified file 'bzrlib/tests/test_xml.py'
--- bzrlib/tests/test_xml.py	2011-02-21 23:43:10 +0000
+++ bzrlib/tests/test_xml.py	2011-12-12 12:11:51 +0000
@@ -508,34 +508,33 @@
     def setUp(self):
         TestCase.setUp(self)
         # Keep the cache clear before and after the test
-        bzrlib.xml8._ensure_utf8_re()
-        bzrlib.xml8._clear_cache()
-        self.addCleanup(bzrlib.xml8._clear_cache)
+        bzrlib.xml_serializer._clear_cache()
+        self.addCleanup(bzrlib.xml_serializer._clear_cache)
 
     def test_simple_ascii(self):
         # _encode_and_escape always appends a final ", because these parameters
         # are being used in xml attributes, and by returning it now, we have to
         # do fewer string operations later.
-        val = bzrlib.xml8._encode_and_escape('foo bar')
+        val = bzrlib.xml_serializer.encode_and_escape('foo bar')
         self.assertEqual('foo bar"', val)
         # The second time should be cached
-        val2 = bzrlib.xml8._encode_and_escape('foo bar')
+        val2 = bzrlib.xml_serializer.encode_and_escape('foo bar')
         self.assertIs(val2, val)
 
     def test_ascii_with_xml(self):
         self.assertEqual('&amp;&apos;&quot;&lt;&gt;"',
-                         bzrlib.xml8._encode_and_escape('&\'"<>'))
+                         bzrlib.xml_serializer.encode_and_escape('&\'"<>'))
 
     def test_utf8_with_xml(self):
         # u'\xb5\xe5&\u062c'
         utf8_str = '\xc2\xb5\xc3\xa5&\xd8\xac'
         self.assertEqual('&#181;&#229;&amp;&#1580;"',
-                         bzrlib.xml8._encode_and_escape(utf8_str))
+                         bzrlib.xml_serializer.encode_and_escape(utf8_str))
 
     def test_unicode(self):
         uni_str = u'\xb5\xe5&\u062c'
         self.assertEqual('&#181;&#229;&amp;&#1580;"',
-                         bzrlib.xml8._encode_and_escape(uni_str))
+                         bzrlib.xml_serializer.encode_and_escape(uni_str))
 
 
 class TestMisc(TestCase):

=== modified file 'bzrlib/tests/transport_util.py'
--- bzrlib/tests/transport_util.py	2011-08-19 22:34:02 +0000
+++ bzrlib/tests/transport_util.py	2011-12-14 19:44:40 +0000
@@ -14,125 +14,34 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-import bzrlib.hooks
-from bzrlib import transport
 from bzrlib.tests import features
 
 # SFTPTransport offers better performances but relies on paramiko, if paramiko
 # is not available, we fallback to FtpTransport
 if features.paramiko.available():
     from bzrlib.tests import test_sftp_transport
-    from bzrlib.transport import sftp
+    from bzrlib.transport import sftp, Transport
     _backing_scheme = 'sftp'
     _backing_transport_class = sftp.SFTPTransport
     _backing_test_class = test_sftp_transport.TestCaseWithSFTPServer
 else:
-    from bzrlib.transport import ftp
+    from bzrlib.transport import ftp, Transport
     from bzrlib.tests import test_ftp_transport
     _backing_scheme = 'ftp'
     _backing_transport_class = ftp.FtpTransport
     _backing_test_class = test_ftp_transport.TestCaseWithFTPServer
 
-from bzrlib.transport import (
-    ConnectedTransport,
-    register_transport,
-    register_urlparse_netloc_protocol,
-    unregister_transport,
-    _unregister_urlparse_netloc_protocol,
-    )
-
-
-
-class TransportHooks(bzrlib.hooks.Hooks):
-    """Dict-mapping hook name to a list of callables for transport hooks"""
-
-    def __init__(self):
-        super(TransportHooks, self).__init__("bzrlib.tests.transport_util",
-            "InstrumentedTransport.hooks")
-        # Invoked when the transport has just created a new connection.
-        # The api signature is (transport, connection, credentials)
-        self['_set_connection'] = []
-
-_hooked_scheme = 'hooked'
-
-def _change_scheme_in(url, actual, desired):
-    if not url.startswith(actual + '://'):
-        raise AssertionError('url "%r" does not start with "%r]"'
-                             % (url, actual))
-    return desired + url[len(actual):]
-
-
-class InstrumentedTransport(_backing_transport_class):
-    """Instrumented transport class to test commands behavior"""
-
-    hooks = TransportHooks()
-
-    def __init__(self, base, _from_transport=None):
-        if not base.startswith(_hooked_scheme + '://'):
-            raise ValueError(base)
-        # We need to trick the backing transport class about the scheme used
-        # We'll do the reverse when we need to talk to the backing server
-        fake_base = _change_scheme_in(base, _hooked_scheme, _backing_scheme)
-        super(InstrumentedTransport, self).__init__(
-            fake_base, _from_transport=_from_transport)
-        # The following is needed to minimize the effects of our trick above
-        # while retaining the best compatibility.
-        self._parsed_url.scheme = _hooked_scheme
-        super(ConnectedTransport, self).__init__(str(self._parsed_url))
-
-
-class ConnectionHookedTransport(InstrumentedTransport):
-    """Transport instrumented to inspect connections"""
-
-    def _set_connection(self, connection, credentials):
-        """Called when a new connection is created """
-        super(ConnectionHookedTransport, self)._set_connection(connection,
-                                                               credentials)
-        for hook in self.hooks['_set_connection']:
-            hook(self, connection, credentials)
-
 
 class TestCaseWithConnectionHookedTransport(_backing_test_class):
 
     def setUp(self):
-        register_urlparse_netloc_protocol(_hooked_scheme)
-        register_transport(_hooked_scheme, ConnectionHookedTransport)
-        self.addCleanup(unregister_transport, _hooked_scheme,
-                        ConnectionHookedTransport)
-        self.addCleanup(_unregister_urlparse_netloc_protocol, _hooked_scheme)
         super(TestCaseWithConnectionHookedTransport, self).setUp()
         self.reset_connections()
-        # Add the 'hooked' url to the permitted url list.
-        # XXX: See TestCase.start_server. This whole module shouldn't need to
-        # exist - a bug has been filed on that. once its cleanedup/removed, the
-        # standard test support code will work and permit the server url
-        # correctly.
-        url = self.get_url()
-        t = transport.get_transport_from_url(url)
-        if t.base.endswith('work/'):
-            t = t.clone('../..')
-        self.permit_url(t.base)
-
-    def get_url(self, relpath=None):
-        super_self = super(TestCaseWithConnectionHookedTransport, self)
-        url = super_self.get_url(relpath)
-        # Replace the backing scheme by our own (see
-        # InstrumentedTransport.__init__)
-        url = _change_scheme_in(url, _backing_scheme, _hooked_scheme)
-        return url
 
     def start_logging_connections(self):
-        self.overrideAttr(InstrumentedTransport, 'hooks', TransportHooks())
-        # We preserved the hooks class attribute. Now we install our hook.
-        ConnectionHookedTransport.hooks.install_named_hook(
-            '_set_connection', self._collect_connection, None)
+        Transport.hooks.install_named_hook('post_connect',
+            self.connections.append, None)
 
     def reset_connections(self):
         self.connections = []
 
-    def _collect_connection(self, transport, connection, credentials):
-        # Note: uncomment the following line and use 'bt' under pdb, that will
-        # identify all the connections made including the extraneous ones.
-        # import pdb; pdb.set_trace()
-        self.connections.append(connection)
-

=== modified file 'bzrlib/textfile.py'
--- bzrlib/textfile.py	2009-03-23 14:59:43 +0000
+++ bzrlib/textfile.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,8 @@
 
 """Utilities for distinguishing binary files from text files"""
 
+from __future__ import absolute_import
+
 from itertools import chain
 
 from bzrlib.errors import BinaryFile

=== modified file 'bzrlib/textinv.py'
--- bzrlib/textinv.py	2010-04-30 11:03:59 +0000
+++ bzrlib/textinv.py	2011-12-18 15:28:38 +0000
@@ -14,6 +14,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
 
 from bzrlib.errors import BzrError
 from bzrlib.inventory import Inventory

=== modified file 'bzrlib/textmerge.py'
--- bzrlib/textmerge.py	2010-06-17 09:23:19 +0000
+++ bzrlib/textmerge.py	2011-12-18 15:28:38 +0000
@@ -17,6 +17,7 @@
 # Author: Martin Pool <mbp@canonical.com>
 #         Aaron Bentley <aaron.bentley@utoronto.ca>
 
+from __future__ import absolute_import
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """

=== modified file 'bzrlib/timestamp.py'
--- bzrlib/timestamp.py	2011-01-10 22:20:12 +0000
+++ bzrlib/timestamp.py	2011-12-18 15:28:38 +0000
@@ -14,8 +14,11 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 import calendar
 import time
+import re
 
 from bzrlib import osutils
 
@@ -124,18 +127,30 @@
             date_fmt='%Y-%m-%d %H:%M:%S')
 
 
+# Format for patch dates: %Y-%m-%d %H:%M:%S [+-]%H%M
+# Groups: 1 = %Y-%m-%d %H:%M:%S; 2 = [+-]%H; 3 = %M
+RE_PATCHDATE = re.compile("(\d+-\d+-\d+\s+\d+:\d+:\d+)\s*([+-]\d\d)(\d\d)$")
+RE_PATCHDATE_NOOFFSET = re.compile("\d+-\d+-\d+\s+\d+:\d+:\d+$")
+
 def parse_patch_date(date_str):
     """Parse a patch-style date into a POSIX timestamp and offset.
 
     Inverse of format_patch_date.
     """
-    secs_str = date_str[:-6]
-    offset_str = date_str[-5:]
-    if len(offset_str) != 5:
-        raise ValueError(
-            "invalid timezone %r" % offset_str)
-    offset_hours, offset_mins = offset_str[:3], offset_str[3:]
-    offset = int(offset_hours) * 3600 + int(offset_mins) * 60
+    match = RE_PATCHDATE.match(date_str)
+    if match is None:
+        if RE_PATCHDATE_NOOFFSET.match(date_str) is not None:
+            raise ValueError("time data %r is missing a timezone offset"
+                % date_str)
+        else:
+            raise ValueError("time data %r does not match format " % date_str
+                + "'%Y-%m-%d %H:%M:%S %z'")
+    secs_str = match.group(1)
+    offset_hours, offset_mins = int(match.group(2)), int(match.group(3))
+    if abs(offset_hours) >= 24 or offset_mins >= 60:
+        raise ValueError("invalid timezone %r" %
+            (match.group(2) + match.group(3)))
+    offset = offset_hours * 3600 + offset_mins * 60
     tm_time = time.strptime(secs_str, '%Y-%m-%d %H:%M:%S')
     # adjust seconds according to offset before converting to POSIX
     # timestamp, to avoid edge problems

=== modified file 'bzrlib/trace.py'
--- bzrlib/trace.py	2011-07-15 14:13:32 +0000
+++ bzrlib/trace.py	2011-12-19 13:23:58 +0000
@@ -45,6 +45,8 @@
 form.
 """
 
+from __future__ import absolute_import
+
 # FIXME: Unfortunately it turns out that python's logging module
 # is quite expensive, even when the message is not printed by any handlers.
 # We should perhaps change back to just simply doing it here.
@@ -54,7 +56,6 @@
 # increased cost of logging.py is not so bad, and we could standardize on
 # that.
 
-import codecs
 import logging
 import os
 import sys
@@ -111,8 +112,6 @@
 
     :return: None
     """
-    # FIXME note always emits utf-8, regardless of the terminal encoding
-    #
     # FIXME: clearing the ui and then going through the abstract logging
     # framework is whack; we should probably have a logging Handler that
     # deals with terminal output if needed.
@@ -285,7 +284,6 @@
     """
     start_time = osutils.format_local_date(_bzr_log_start_time,
                                            timezone='local')
-    # create encoded wrapper around stderr
     bzr_log_file = _open_bzr_log()
     if bzr_log_file is not None:
         bzr_log_file.write(start_time.encode('utf-8') + '\n')
@@ -294,11 +292,8 @@
         r'%Y-%m-%d %H:%M:%S')
     # after hooking output into bzr_log, we also need to attach a stderr
     # handler, writing only at level info and with encoding
-    term_encoding = osutils.get_terminal_encoding()
-    writer_factory = codecs.getwriter(term_encoding)
-    encoded_stderr = writer_factory(sys.stderr, errors='replace')
-    stderr_handler = logging.StreamHandler(encoded_stderr)
-    stderr_handler.setLevel(logging.INFO)
+    stderr_handler = EncodedStreamHandler(sys.stderr,
+        osutils.get_terminal_encoding(), 'replace', level=logging.INFO)
     logging.getLogger('bzr').addHandler(stderr_handler)
     return memento
 
@@ -313,8 +308,7 @@
     """
     global _trace_file
     # make a new handler
-    new_handler = logging.StreamHandler(to_file)
-    new_handler.setLevel(logging.DEBUG)
+    new_handler = EncodedStreamHandler(to_file, "utf-8", level=logging.DEBUG)
     if log_format is None:
         log_format = '%(levelname)8s  %(message)s'
     new_handler.setFormatter(logging.Formatter(log_format, date_format))
@@ -491,11 +485,7 @@
         print_exception(exc_info, err_file)
         return errors.EXIT_ERROR
     exc_type, exc_object, exc_tb = exc_info
-    if (isinstance(exc_object, IOError)
-        and getattr(exc_object, 'errno', None) == errno.EPIPE):
-        err_file.write("bzr: broken pipe\n")
-        return errors.EXIT_ERROR
-    elif isinstance(exc_object, KeyboardInterrupt):
+    if isinstance(exc_object, KeyboardInterrupt):
         err_file.write("bzr: interrupted\n")
         return errors.EXIT_ERROR
     elif isinstance(exc_object, MemoryError):
@@ -513,9 +503,10 @@
     elif not getattr(exc_object, 'internal_error', True):
         report_user_error(exc_info, err_file)
         return errors.EXIT_ERROR
-    elif isinstance(exc_object, (OSError, IOError)) or (
-        # GZ 2010-05-20: Like (exc_type is pywintypes.error) but avoid import
-        exc_type.__name__ == "error" and exc_type.__module__ == "pywintypes"):
+    elif osutils.is_environment_error(exc_object):
+        if getattr(exc_object, 'errno', None) == errno.EPIPE:
+            err_file.write("bzr: broken pipe\n")
+            return errors.EXIT_ERROR
         # Might be nice to catch all of these and show them as something more
         # specific, but there are too many cases at the moment.
         report_user_error(exc_info, err_file)
@@ -559,6 +550,10 @@
     try:
         sys.stdout.flush()
         sys.stderr.flush()
+    except ValueError, e:
+        # On Windows, I get ValueError calling stdout.flush() on a closed
+        # handle
+        pass
     except IOError, e:
         import errno
         if e.errno in [errno.EINVAL, errno.EPIPE]:
@@ -574,6 +569,49 @@
         _trace_file.flush()
 
 
+class EncodedStreamHandler(logging.Handler):
+    """Robustly write logging events to a stream using the specified encoding
+
+    Messages are expected to be formatted to unicode, but UTF-8 byte strings
+    are also accepted. An error during formatting or a str message in another
+    encoding will be quitely noted as an error in the Bazaar log file.
+
+    The stream is not closed so sys.stdout or sys.stderr may be passed.
+    """
+
+    def __init__(self, stream, encoding=None, errors='strict', level=0):
+        logging.Handler.__init__(self, level)
+        self.stream = stream
+        if encoding is None:
+            encoding = getattr(stream, "encoding", "ascii")
+        self.encoding = encoding
+        self.errors = errors
+
+    def flush(self):
+        flush = getattr(self.stream, "flush", None)
+        if flush is not None:
+            flush()
+
+    def emit(self, record):
+        try:
+            line = self.format(record)
+            if not isinstance(line, unicode):
+                line = line.decode("utf-8")
+            self.stream.write(line.encode(self.encoding, self.errors) + "\n")
+        except Exception:
+            log_exception_quietly()
+            # Try saving the details that would have been logged in some form
+            msg = args = "<Unformattable>"
+            try:
+                msg = repr(record.msg).encode("ascii")
+                args = repr(record.args).encode("ascii")
+            except Exception:
+                pass
+            # Using mutter() bypasses the logging module and writes directly
+            # to the file so there's no danger of getting into a loop here.
+            mutter("Logging record unformattable: %s %% %s", msg, args)
+
+
 class Config(object):
     """Configuration of message tracing in bzrlib.
 

=== modified file 'bzrlib/transactions.py'
--- bzrlib/transactions.py	2009-03-23 14:59:43 +0000
+++ bzrlib/transactions.py	2011-12-19 13:23:58 +0000
@@ -36,6 +36,8 @@
 the object is an expensive one for obtaining.
 """
 
+from __future__ import absolute_import
+
 import sys
 
 import bzrlib.errors as errors

=== modified file 'bzrlib/transform.py'
--- bzrlib/transform.py	2011-08-05 13:34:40 +0000
+++ bzrlib/transform.py	2011-12-19 17:39:35 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 import os
 import errno
 from stat import S_ISREG, S_IEXEC
@@ -30,11 +32,10 @@
 from bzrlib import (
     annotate,
     bencode,
-    bzrdir,
+    controldir,
     commit,
     conflicts,
     delta,
-    errors,
     inventory,
     multiparent,
     osutils,
@@ -42,8 +43,9 @@
     ui,
     urlutils,
     )
+from bzrlib.i18n import gettext
 """)
-from bzrlib.errors import (DuplicateKey, MalformedTransform, NoSuchFile,
+from bzrlib.errors import (DuplicateKey, MalformedTransform,
                            ReusingTransform, CantMoveRoot,
                            ExistingLimbo, ImmortalLimbo, NoFinalPath,
                            UnableCreateSymlink)
@@ -561,9 +563,7 @@
         for trans_id in self._removed_id:
             file_id = self.tree_file_id(trans_id)
             if file_id is not None:
-                # XXX: This seems like something that should go via a different
-                #      indirection.
-                if self._tree.inventory[file_id].kind == 'directory':
+                if self._tree.stored_kind(file_id) == 'directory':
                     parents.append(trans_id)
             elif self.tree_kind(trans_id) == 'directory':
                 parents.append(trans_id)
@@ -763,7 +763,7 @@
 
     def _set_executability(self, path, trans_id):
         """Set the executability of versioned files """
-        if supports_executable():
+        if self._tree._supports_executable():
             new_executability = self._new_executability[trans_id]
             abspath = self._tree.abspath(path)
             current_mode = os.stat(abspath).st_mode
@@ -778,7 +778,7 @@
                     to_mode |= 0010 & ~umask
             else:
                 to_mode = current_mode & ~0111
-            os.chmod(abspath, to_mode)
+            osutils.chmod_if_possible(abspath, to_mode)
 
     def _new_entry(self, name, parent_id, file_id):
         """Helper function to create a new filesystem entry."""
@@ -1233,6 +1233,11 @@
         finally:
             TreeTransformBase.finalize(self)
 
+    def _limbo_supports_executable(self):
+        """Check if the limbo path supports the executable bit."""
+        # FIXME: Check actual file system capabilities of limbodir
+        return osutils.supports_executable()
+
     def _limbo_name(self, trans_id):
         """Generate the limbo name of a file"""
         limbo_name = self._limbo_files.get(trans_id)
@@ -1557,18 +1562,14 @@
         try:
             limbodir = urlutils.local_path_from_url(
                 tree._transport.abspath('limbo'))
-            try:
-                os.mkdir(limbodir)
-            except OSError, e:
-                if e.errno == errno.EEXIST:
-                    raise ExistingLimbo(limbodir)
+            osutils.ensure_empty_directory_exists(
+                limbodir,
+                errors.ExistingLimbo)
             deletiondir = urlutils.local_path_from_url(
                 tree._transport.abspath('pending-deletion'))
-            try:
-                os.mkdir(deletiondir)
-            except OSError, e:
-                if e.errno == errno.EEXIST:
-                    raise errors.ExistingPendingDeletion(deletiondir)
+            osutils.ensure_empty_directory_exists(
+                deletiondir,
+                errors.ExistingPendingDeletion)
         except:
             tree.unlock()
             raise
@@ -1637,7 +1638,7 @@
             else:
                 raise
         if typefunc(mode):
-            os.chmod(self._limbo_name(trans_id), mode)
+            osutils.chmod_if_possible(self._limbo_name(trans_id), mode)
 
     def iter_tree_children(self, parent_id):
         """Iterate through the entry's tree children, if any"""
@@ -1726,7 +1727,7 @@
         child_pb = ui.ui_factory.nested_progress_bar()
         try:
             if precomputed_delta is None:
-                child_pb.update('Apply phase', 0, 2)
+                child_pb.update(gettext('Apply phase'), 0, 2)
                 inventory_delta = self._generate_inventory_delta()
                 offset = 1
             else:
@@ -1737,9 +1738,9 @@
             else:
                 mover = _mover
             try:
-                child_pb.update('Apply phase', 0 + offset, 2 + offset)
+                child_pb.update(gettext('Apply phase'), 0 + offset, 2 + offset)
                 self._apply_removals(mover)
-                child_pb.update('Apply phase', 1 + offset, 2 + offset)
+                child_pb.update(gettext('Apply phase'), 1 + offset, 2 + offset)
                 modified_paths = self._apply_insertions(mover)
             except:
                 mover.rollback()
@@ -1765,7 +1766,7 @@
         try:
             for num, trans_id in enumerate(self._removed_id):
                 if (num % 10) == 0:
-                    child_pb.update('removing file', num, total_entries)
+                    child_pb.update(gettext('removing file'), num, total_entries)
                 if trans_id == self._new_root:
                     file_id = self._tree.get_root_id()
                 else:
@@ -1783,7 +1784,7 @@
             final_kinds = {}
             for num, (path, trans_id) in enumerate(new_paths):
                 if (num % 10) == 0:
-                    child_pb.update('adding file',
+                    child_pb.update(gettext('adding file'),
                                     num + len(self._removed_id), total_entries)
                 file_id = new_path_file_ids[trans_id]
                 if file_id is None:
@@ -1833,7 +1834,7 @@
                 # do not attempt to move root into a subdirectory of itself.
                 if path == '':
                     continue
-                child_pb.update('removing file', num, len(tree_paths))
+                child_pb.update(gettext('removing file'), num, len(tree_paths))
                 full_path = self._tree.abspath(path)
                 if trans_id in self._removed_contents:
                     delete_path = os.path.join(self._deletiondir, trans_id)
@@ -1868,7 +1869,7 @@
         try:
             for num, (path, trans_id) in enumerate(new_paths):
                 if (num % 10) == 0:
-                    child_pb.update('adding file', num, len(new_paths))
+                    child_pb.update(gettext('adding file'), num, len(new_paths))
                 full_path = self._tree.abspath(path)
                 if trans_id in self._needs_rename:
                     try:
@@ -2325,7 +2326,7 @@
             if kind == 'file':
                 statval = os.lstat(limbo_name)
                 size = statval.st_size
-                if not supports_executable():
+                if not tt._limbo_supports_executable():
                     executable = False
                 else:
                     executable = statval.st_mode & S_IEXEC
@@ -2546,7 +2547,7 @@
     file_trans_id = {}
     top_pb = ui.ui_factory.nested_progress_bar()
     pp = ProgressPhase("Build phase", 2, top_pb)
-    if tree.inventory.root is not None:
+    if tree.get_root_id() is not None:
         # This is kind of a hack: we should be altering the root
         # as part of the regular tree shape diff logic.
         # The conditional test here is to avoid doing an
@@ -2567,7 +2568,7 @@
         try:
             deferred_contents = []
             num = 0
-            total = len(tree.inventory)
+            total = len(tree.all_file_ids())
             if delta_from_tree:
                 precomputed_delta = []
             else:
@@ -2582,8 +2583,8 @@
                 for dir, files in wt.walkdirs():
                     existing_files.update(f[0] for f in files)
             for num, (tree_path, entry) in \
-                enumerate(tree.inventory.iter_entries_by_dir()):
-                pb.update("Building tree", num - len(deferred_contents), total)
+                enumerate(tree.iter_entries_by_dir()):
+                pb.update(gettext("Building tree"), num - len(deferred_contents), total)
                 if entry.parent_id is None:
                     continue
                 reparent = False
@@ -2595,7 +2596,7 @@
                     kind = file_kind(target_path)
                     if kind == "directory":
                         try:
-                            bzrdir.BzrDir.open(target_path)
+                            controldir.ControlDir.open(target_path)
                         except errors.NotBranchError:
                             pass
                         else:
@@ -2673,7 +2674,7 @@
                 new_desired_files.append((file_id,
                     (trans_id, tree_path, text_sha1)))
                 continue
-            pb.update('Adding file contents', count + offset, total)
+            pb.update(gettext('Adding file contents'), count + offset, total)
             if hardlink:
                 tt.create_hardlink(accelerator_tree.abspath(accelerator_path),
                                    trans_id)
@@ -2700,7 +2701,7 @@
             contents = filtered_output_bytes(contents, filters,
                 ContentFilterContext(tree_path, tree))
         tt.create_file(contents, trans_id, sha1=text_sha1)
-        pb.update('Adding file contents', count + offset, total)
+        pb.update(gettext('Adding file contents'), count + offset, total)
 
 
 def _reparent_children(tt, old_parent, new_parent):
@@ -2838,33 +2839,6 @@
             return new_name
 
 
-def _entry_changes(file_id, entry, working_tree):
-    """Determine in which ways the inventory entry has changed.
-
-    Returns booleans: has_contents, content_mod, meta_mod
-    has_contents means there are currently contents, but they differ
-    contents_mod means contents need to be modified
-    meta_mod means the metadata needs to be modified
-    """
-    cur_entry = working_tree.inventory[file_id]
-    try:
-        working_kind = working_tree.kind(file_id)
-        has_contents = True
-    except NoSuchFile:
-        has_contents = False
-        contents_mod = True
-        meta_mod = False
-    if has_contents is True:
-        if entry.kind != working_kind:
-            contents_mod, meta_mod = True, False
-        else:
-            cur_entry._read_tree_state(working_tree.id2path(file_id),
-                                       working_tree)
-            contents_mod, meta_mod = entry.detect_changes(cur_entry)
-            cur_entry._forget_tree_state()
-    return has_contents, contents_mod, meta_mod
-
-
 def revert(working_tree, target_tree, filenames, backups=False,
            pb=None, change_reporter=None):
     """Revert a working tree's contents to those of a target tree."""
@@ -3047,7 +3021,7 @@
     pb = ui.ui_factory.nested_progress_bar()
     try:
         for n in range(10):
-            pb.update('Resolution pass', n+1, 10)
+            pb.update(gettext('Resolution pass'), n+1, 10)
             conflicts = tt.find_conflicts()
             if len(conflicts) == 0:
                 return new_conflicts
@@ -3077,7 +3051,7 @@
                 existing_file, new_file = conflict[2], conflict[1]
             else:
                 existing_file, new_file = conflict[1], conflict[2]
-            new_name = tt.final_name(existing_file)+'.moved'
+            new_name = tt.final_name(existing_file) + '.moved'
             tt.adjust_path(new_name, final_parent, existing_file)
             new_conflicts.add((c_type, 'Moved existing file to',
                                existing_file, new_file))

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2011-08-28 15:33:21 +0000
+++ bzrlib/transport/__init__.py	2011-12-24 10:10:59 +0000
@@ -26,6 +26,8 @@
 it.
 """
 
+from __future__ import absolute_import
+
 from cStringIO import StringIO
 import sys
 
@@ -33,7 +35,6 @@
 lazy_import(globals(), """
 import errno
 from stat import S_ISDIR
-import urllib
 import urlparse
 
 from bzrlib import (
@@ -51,7 +52,10 @@
 from bzrlib.trace import (
     mutter,
     )
-from bzrlib import registry
+from bzrlib import (
+    hooks,
+    registry,
+    )
 
 
 # a dictionary of open file streams. Keys are absolute paths, values are
@@ -282,6 +286,16 @@
         self.transport.append_bytes(self.relpath, bytes)
 
 
+class TransportHooks(hooks.Hooks):
+    """Mapping of hook names to registered callbacks for transport hooks"""
+    def __init__(self):
+        super(TransportHooks, self).__init__()
+        self.add_hook("post_connect",
+            "Called after a new connection is established or a reconnect "
+            "occurs. The sole argument passed is either the connected "
+            "transport or smart medium instance.", (2, 5))
+
+
 class Transport(object):
     """This class encapsulates methods for retrieving or putting a file
     from/to a storage location.
@@ -306,12 +320,14 @@
     #       where the biggest benefit between combining reads and
     #       and seeking is. Consider a runtime auto-tune.
     _bytes_to_read_before_seek = 0
+    
+    hooks = TransportHooks()
 
     def __init__(self, base):
         super(Transport, self).__init__()
         self.base = base
-        self._segment_parameters = urlutils.split_segment_parameters(
-            base.rstrip("/"))[1]
+        (self._raw_base, self._segment_parameters) = (
+            urlutils.split_segment_parameters(base))
 
     def _translate_error(self, e, path, raise_generic=True):
         """Translate an IOError or OSError into an appropriate bzr error.
@@ -416,6 +432,22 @@
         """
         return self._segment_parameters
 
+    def set_segment_parameter(self, name, value):
+        """Set a segment parameter.
+
+        :param name: Segment parameter name (urlencoded string)
+        :param value: Segment parameter value (urlencoded string)
+        """
+        if value is None:
+            try:
+                del self._segment_parameters[name]
+            except KeyError:
+                pass
+        else:
+            self._segment_parameters[name] = value
+        self.base = urlutils.join_segment_parameters(
+            self._raw_base, self._segment_parameters)
+
     def _pump(self, from_file, to_file):
         """Most children will need to copy from one file-like
         object or string to another one.
@@ -1399,12 +1431,12 @@
 
         :return: The corresponding URL.
         """
-        netloc = urllib.quote(host)
+        netloc = urlutils.quote(host)
         if user is not None:
             # Note that we don't put the password back even if we
             # have one so that it doesn't get accidentally
             # exposed.
-            netloc = '%s@%s' % (urllib.quote(user), netloc)
+            netloc = '%s@%s' % (urlutils.quote(user), netloc)
         if port is not None:
             netloc = '%s:%d' % (netloc, port)
         path = urlutils.escape(path)
@@ -1479,6 +1511,8 @@
         """
         self._shared_connection.connection = connection
         self._shared_connection.credentials = credentials
+        for hook in self.hooks["post_connect"]:
+            hook(self)
 
     def _get_connection(self):
         """Returns the transport specific connection object."""

=== modified file 'bzrlib/transport/brokenrename.py'
--- bzrlib/transport/brokenrename.py	2010-02-09 17:21:06 +0000
+++ bzrlib/transport/brokenrename.py	2011-12-19 13:23:58 +0000
@@ -17,7 +17,7 @@
 """Transport implementation that doesn't detect clashing renames.
 """
 
-from stat import S_ISDIR
+from __future__ import absolute_import
 
 from bzrlib import (
         errors,

=== modified file 'bzrlib/transport/chroot.py'
--- bzrlib/transport/chroot.py	2010-04-07 12:19:40 +0000
+++ bzrlib/transport/chroot.py	2011-12-19 13:23:58 +0000
@@ -18,6 +18,8 @@
 root.
 """
 
+from __future__ import absolute_import
+
 from bzrlib.transport import (
     pathfilter,
     register_transport,

=== modified file 'bzrlib/transport/decorator.py'
--- bzrlib/transport/decorator.py	2011-08-19 22:34:02 +0000
+++ bzrlib/transport/decorator.py	2011-12-19 13:23:58 +0000
@@ -20,6 +20,8 @@
 stub functions to allow other decorators to be written easily.
 """
 
+from __future__ import absolute_import
+
 from bzrlib import transport
 
 

=== modified file 'bzrlib/transport/fakenfs.py'
--- bzrlib/transport/fakenfs.py	2010-02-09 17:19:50 +0000
+++ bzrlib/transport/fakenfs.py	2011-12-19 13:23:58 +0000
@@ -22,6 +22,8 @@
 To get a fake nfs transport use get_transport('fakenfs+' + real_url)
 """
 
+from __future__ import absolute_import
+
 from stat import S_ISDIR
 
 from bzrlib import (

=== modified file 'bzrlib/transport/fakevfat.py'
--- bzrlib/transport/fakevfat.py	2010-02-09 17:22:25 +0000
+++ bzrlib/transport/fakevfat.py	2011-12-19 13:23:58 +0000
@@ -38,9 +38,10 @@
 
 """
 
+from __future__ import absolute_import
+
 import re
 
-from bzrlib.errors import TransportNotPossible
 from bzrlib.transport import decorator
 
 

=== modified file 'bzrlib/transport/ftp/__init__.py'
--- bzrlib/transport/ftp/__init__.py	2011-08-08 17:52:59 +0000
+++ bzrlib/transport/ftp/__init__.py	2011-12-19 13:23:58 +0000
@@ -25,6 +25,8 @@
 active, in which case aftp:// will be your friend.
 """
 
+from __future__ import absolute_import
+
 from cStringIO import StringIO
 import ftplib
 import getpass

=== modified file 'bzrlib/transport/ftp/_gssapi.py'
--- bzrlib/transport/ftp/_gssapi.py	2010-05-31 21:31:18 +0000
+++ bzrlib/transport/ftp/_gssapi.py	2011-12-19 13:23:58 +0000
@@ -19,12 +19,18 @@
 See RFC2228 for details.
 """
 
+from __future__ import absolute_import
+
 import base64, ftplib
 
 from bzrlib import (
     errors,
     )
-from bzrlib.trace import mutter
+from bzrlib.i18n import gettext
+from bzrlib.trace import (
+    mutter,
+    note,
+    )
 from bzrlib.transport.ftp import FtpTransport
 
 try:
@@ -79,8 +85,8 @@
                         if not ((resp.startswith('235 ') and rc == 1) or
                                 (resp.startswith('335 ') and rc == 0)):
                             raise ftplib.error_reply, resp
-            trace.note("Authenticated as %s" % kerberos.authGSSClientUserName(
-                    self.vc))
+            note(gettext("Authenticated as %s") %
+                 kerberos.authGSSClientUserName(self.vc))
 
             # Monkey patch ftplib
             self.putcmd = self.mic_putcmd

=== modified file 'bzrlib/transport/gio_transport.py'
--- bzrlib/transport/gio_transport.py	2011-08-11 16:24:09 +0000
+++ bzrlib/transport/gio_transport.py	2011-12-19 13:23:58 +0000
@@ -23,6 +23,9 @@
 It provides the gio+XXX:// protocols where XXX is any of the protocols
 supported by gio.
 """
+
+from __future__ import absolute_import
+
 from cStringIO import StringIO
 import os
 import random

=== modified file 'bzrlib/transport/http/__init__.py'
--- bzrlib/transport/http/__init__.py	2011-08-19 22:34:02 +0000
+++ bzrlib/transport/http/__init__.py	2011-12-19 13:23:58 +0000
@@ -19,11 +19,11 @@
 There are separate implementation modules for each http client implementation.
 """
 
+from __future__ import absolute_import
+
 from cStringIO import StringIO
-import mimetools
 import re
 import urlparse
-import urllib
 import sys
 import weakref
 
@@ -35,14 +35,9 @@
     urlutils,
     )
 from bzrlib.smart import medium
-from bzrlib.symbol_versioning import (
-        deprecated_method,
-        )
 from bzrlib.trace import mutter
 from bzrlib.transport import (
     ConnectedTransport,
-    _CoalescedOffset,
-    Transport,
     )
 
 # TODO: This is not used anymore by HttpTransport_urllib
@@ -69,9 +64,9 @@
             host = netloc.split(':', 1)[0]
         else:
             host = netloc
-        username = urllib.unquote(username)
+        username = urlutils.unquote(username)
         if password is not None:
-            password = urllib.unquote(password)
+            password = urlutils.unquote(password)
         else:
             password = ui.ui_factory.get_password(
                 prompt=u'HTTP %(user)s@%(host)s password',
@@ -517,66 +512,57 @@
 
         :returns: A transport or None.
         """
-        def relpath(abspath):
-            """Returns the path relative to our base.
-
-            The constraints are weaker than the real relpath method because the
-            abspath is coming from the server and may slightly differ from our
-            base. We don't check the scheme, host, port, user, password parts,
-            relying on the caller to give us a proper url (i.e. one returned by
-            the server mirroring the one we sent).
-            """
-            parsed_url = self._split_url(abspath)
-            pl = len(self._parsed_url.path)
-            return parsed_url.path[pl:].strip('/')
-
-        relpath = relpath(source)
-        if not target.endswith(relpath):
+        parsed_source = self._split_url(source)
+        parsed_target = self._split_url(target)
+        pl = len(self._parsed_url.path)
+        # determine the excess tail - the relative path that was in
+        # the original request but not part of this transports' URL.
+        excess_tail = parsed_source.path[pl:].strip("/")
+        if not target.endswith(excess_tail):
             # The final part of the url has been renamed, we can't handle the
             # redirection.
             return None
-        new_transport = None
-        parsed_url = self._split_url(target)
-        # Recalculate base path. This is needed to ensure that when the
-        # redirected transport will be used to re-try whatever request was
-        # redirected, we end up with the same url
-        base_path = parsed_url.path[:-len(relpath)]
-        if parsed_url.scheme in ('http', 'https'):
+
+        target_path = parsed_target.path
+        if excess_tail:
+            # Drop the tail that was in the redirect but not part of
+            # the path of this transport.
+            target_path = target_path[:-len(excess_tail)]
+
+        if parsed_target.scheme in ('http', 'https'):
             # Same protocol family (i.e. http[s]), we will preserve the same
             # http client implementation when a redirection occurs from one to
             # the other (otherwise users may be surprised that bzr switches
             # from one implementation to the other, and devs may suffer
             # debugging it).
-            if (parsed_url.scheme == self._unqualified_scheme
-                and parsed_url.host == self._parsed_url.host
-                and parsed_url.port == self._parsed_url.port
-                and (parsed_url.user is None or
-                     parsed_url.user == self._parsed_url.user)):
+            if (parsed_target.scheme == self._unqualified_scheme
+                and parsed_target.host == self._parsed_url.host
+                and parsed_target.port == self._parsed_url.port
+                and (parsed_target.user is None or
+                     parsed_target.user == self._parsed_url.user)):
                 # If a user is specified, it should match, we don't care about
                 # passwords, wrong passwords will be rejected anyway.
-                new_transport = self.clone(base_path)
+                return self.clone(target_path)
             else:
                 # Rebuild the url preserving the scheme qualification and the
                 # credentials (if they don't apply, the redirected to server
                 # will tell us, but if they do apply, we avoid prompting the
                 # user)
-                redir_scheme = parsed_url.scheme + '+' + self._impl_name
+                redir_scheme = parsed_target.scheme + '+' + self._impl_name
                 new_url = self._unsplit_url(redir_scheme,
-                                            self._parsed_url.user,
-                                            self._parsed_url.password,
-                                            parsed_url.host, parsed_url.port,
-                                            base_path)
-                new_transport = transport.get_transport_from_url(
-                    new_url)
+                    self._parsed_url.user,
+                    self._parsed_url.password,
+                    parsed_target.host, parsed_target.port,
+                    target_path)
+                return transport.get_transport_from_url(new_url)
         else:
             # Redirected to a different protocol
-            new_url = self._unsplit_url(parsed_url.scheme,
-                                        parsed_url.user, parsed_url.password,
-                                        parsed_url.host, parsed_url.port,
-                                        base_path)
-            new_transport = transport.get_transport_from_url(
-                new_url)
-        return new_transport
+            new_url = self._unsplit_url(parsed_target.scheme,
+                    parsed_target.user,
+                    parsed_target.password,
+                    parsed_target.host, parsed_target.port,
+                    target_path)
+            return transport.get_transport_from_url(new_url)
 
 
 # TODO: May be better located in smart/medium.py with the other
@@ -604,7 +590,7 @@
         if transport_base.startswith('bzr+'):
             transport_base = transport_base[4:]
         rel_url = urlutils.relative_url(self.base, transport_base)
-        return urllib.unquote(rel_url)
+        return urlutils.unquote(rel_url)
 
     def send_http_smart_request(self, bytes):
         try:
@@ -612,7 +598,7 @@
             t = self._http_transport_ref()
             code, body_filelike = t._post(bytes)
             if code != 200:
-                raise InvalidHttpResponse(
+                raise errors.InvalidHttpResponse(
                     t._remote_path('.bzr/smart'),
                     'Expected 200 response code, got %r' % (code,))
         except (errors.InvalidHttpResponse, errors.ConnectionReset), e:

=== modified file 'bzrlib/transport/http/_pycurl.py'
--- bzrlib/transport/http/_pycurl.py	2011-08-21 09:24:01 +0000
+++ bzrlib/transport/http/_pycurl.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,8 @@
 
 """http/https transport using pycurl"""
 
+from __future__ import absolute_import
+
 # TODO: test reporting of http errors
 #
 # TODO: Transport option to control caching of particular requests; broadly we

=== modified file 'bzrlib/transport/http/_urllib.py'
--- bzrlib/transport/http/_urllib.py	2011-06-14 01:26:41 +0000
+++ bzrlib/transport/http/_urllib.py	2011-12-18 15:28:38 +0000
@@ -14,14 +14,11 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-from cStringIO import StringIO
-import urllib
-import urlparse
+from __future__ import absolute_import
 
 from bzrlib import (
     errors,
     trace,
-    urlutils,
     )
 from bzrlib.transport import http
 # TODO: handle_response should be integrated into the http/__init__.py

=== modified file 'bzrlib/transport/http/_urllib2_wrappers.py'
--- bzrlib/transport/http/_urllib2_wrappers.py	2011-08-10 22:34:25 +0000
+++ bzrlib/transport/http/_urllib2_wrappers.py	2011-12-19 13:23:58 +0000
@@ -36,6 +36,8 @@
 request (see AbstractHTTPHandler.do_open).
 """
 
+from __future__ import absolute_import
+
 DEBUG = 0
 
 # FIXME: Oversimplifying, two kind of exceptions should be
@@ -48,12 +50,6 @@
 
 import errno
 import httplib
-try:
-    import kerberos
-except ImportError:
-    have_kerberos = False
-else:
-    have_kerberos = True
 import socket
 import urllib
 import urllib2
@@ -70,11 +66,14 @@
     osutils,
     trace,
     transport,
-    ui,
     urlutils,
     )
 
 
+checked_kerberos = False
+kerberos = None
+
+
 class addinfourl(urllib2.addinfourl):
     '''Replacement addinfourl class compatible with python-2.7's xmlrpclib
 
@@ -1326,7 +1325,14 @@
 
     def _auth_match_kerberos(self, auth):
         """Try to create a GSSAPI response for authenticating against a host."""
-        if not have_kerberos:
+        global kerberos, checked_kerberos
+        if kerberos is None and not checked_kerberos:
+            try:
+                import kerberos
+            except ImportError:
+                kerberos = None
+            checked_kerberos = True
+        if kerberos is None:
             return None
         ret, vc = kerberos.authGSSClientInit("HTTP@%(host)s" % auth)
         if ret < 1:

=== modified file 'bzrlib/transport/http/ca_bundle.py'
--- bzrlib/transport/http/ca_bundle.py	2009-03-23 14:59:43 +0000
+++ bzrlib/transport/http/ca_bundle.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,8 @@
 
 """Auto-detect of CA bundle for SSL connections"""
 
+from __future__ import absolute_import
+
 import os
 import sys
 from bzrlib.trace import mutter

=== modified file 'bzrlib/transport/http/response.py'
--- bzrlib/transport/http/response.py	2011-07-27 11:57:32 +0000
+++ bzrlib/transport/http/response.py	2011-12-19 13:23:58 +0000
@@ -21,6 +21,7 @@
 responses.
 """
 
+from __future__ import absolute_import
 
 import httplib
 from cStringIO import StringIO
@@ -28,7 +29,6 @@
 
 from bzrlib import (
     errors,
-    trace,
     osutils,
     )
 

=== modified file 'bzrlib/transport/http/wsgi.py'
--- bzrlib/transport/http/wsgi.py	2011-03-05 02:26:26 +0000
+++ bzrlib/transport/http/wsgi.py	2011-12-19 13:23:58 +0000
@@ -20,9 +20,11 @@
     http://www.python.org/dev/peps/pep-0333/
 """
 
+from __future__ import absolute_import
+
 from cStringIO import StringIO
 
-from bzrlib.smart import protocol, medium
+from bzrlib.smart import medium
 from bzrlib.transport import chroot, get_transport
 from bzrlib.urlutils import local_path_to_url
 

=== modified file 'bzrlib/transport/local.py'
--- bzrlib/transport/local.py	2011-08-15 14:31:55 +0000
+++ bzrlib/transport/local.py	2011-12-18 12:46:49 +0000
@@ -19,6 +19,8 @@
 This is a fairly thin wrapper on regular file IO.
 """
 
+from __future__ import absolute_import
+
 import os
 from stat import ST_MODE, S_ISDIR, S_IMODE
 import sys
@@ -146,9 +148,7 @@
         if abspath is None:
             abspath = u'.'
 
-        return urlutils.file_relpath(
-            urlutils.strip_trailing_slash(self.base),
-            urlutils.strip_trailing_slash(abspath))
+        return urlutils.file_relpath(self.base, abspath)
 
     def has(self, relpath):
         return os.access(self._abspath(relpath), os.F_OK)
@@ -257,7 +257,7 @@
             if mode is not None and mode != S_IMODE(st.st_mode):
                 # Because of umask, we may still need to chmod the file.
                 # But in the general case, we won't have to
-                os.chmod(abspath, mode)
+                osutils.chmod_if_possible(abspath, mode)
             writer(fd)
         finally:
             os.close(fd)
@@ -316,12 +316,13 @@
             local_mode = mode
         try:
             os.mkdir(abspath, local_mode)
-            if mode is not None:
-                # It is probably faster to just do the chmod, rather than
-                # doing a stat, and then trying to compare
-                os.chmod(abspath, mode)
         except (IOError, OSError),e:
             self._translate_error(e, abspath)
+        if mode is not None:
+            try:
+                osutils.chmod_if_possible(abspath, mode)
+            except (IOError, OSError), e:
+                self._translate_error(e, abspath)
 
     def mkdir(self, relpath, mode=None):
         """Create a directory at the given path."""
@@ -330,7 +331,10 @@
     def open_write_stream(self, relpath, mode=None):
         """See Transport.open_write_stream."""
         abspath = self._abspath(relpath)
-        handle = osutils.open_file(abspath, 'wb')
+        try:
+            handle = osutils.open_file(abspath, 'wb')
+        except (IOError, OSError),e:
+            self._translate_error(e, abspath)
         handle.truncate()
         if mode is not None:
             self._check_mode_and_size(abspath, handle.fileno(), mode)
@@ -356,7 +360,7 @@
         if mode is not None and mode != S_IMODE(st.st_mode):
             # Because of umask, we may still need to chmod the file.
             # But in the general case, we won't have to
-            os.chmod(file_abspath, mode)
+            osutils.chmod_if_possible(file_abspath, mode)
         return st.st_size
 
     def append_file(self, relpath, f, mode=None):
@@ -455,7 +459,7 @@
                     otherpath = other._abspath(path)
                     shutil.copy(mypath, otherpath)
                     if mode is not None:
-                        os.chmod(otherpath, mode)
+                        osutils.chmod_if_possible(otherpath, mode)
                 except (IOError, OSError),e:
                     self._translate_error(e, path)
                 count += 1
@@ -537,9 +541,7 @@
             """See Transport.symlink."""
             abs_link_dirpath = urlutils.dirname(self.abspath(link_name))
             source_rel = urlutils.file_relpath(
-                urlutils.strip_trailing_slash(abs_link_dirpath),
-                urlutils.strip_trailing_slash(self.abspath(source))
-            )
+                abs_link_dirpath, self.abspath(source))
 
             try:
                 os.symlink(source_rel, self._abspath(link_name))
@@ -564,7 +566,7 @@
         self._local_base = urlutils._win32_local_path_from_url(base)
 
     def abspath(self, relpath):
-        path = osutils.normpath(osutils.pathjoin(
+        path = osutils._win32_normpath(osutils.pathjoin(
                     self._local_base, urlutils.unescape(relpath)))
         return urlutils._win32_local_path_to_url(path)
 

=== modified file 'bzrlib/transport/log.py'
--- bzrlib/transport/log.py	2010-02-11 09:27:55 +0000
+++ bzrlib/transport/log.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,7 @@
 
 """Transport decorator that logs transport operations to .bzr.log."""
 
+from __future__ import absolute_import
 
 # see also the transportstats plugin, which gives you some summary information
 # in a machine-readable dump

=== modified file 'bzrlib/transport/memory.py'
--- bzrlib/transport/memory.py	2011-08-11 14:08:05 +0000
+++ bzrlib/transport/memory.py	2011-12-19 13:23:58 +0000
@@ -20,12 +20,12 @@
 so this is primarily useful for testing.
 """
 
+from __future__ import absolute_import
+
 import os
 import errno
-import re
 from stat import S_IFREG, S_IFDIR
 from cStringIO import StringIO
-import warnings
 
 from bzrlib import (
     transport,
@@ -36,9 +36,7 @@
     LockError,
     InProcessTransport,
     NoSuchFile,
-    TransportError,
     )
-from bzrlib.trace import mutter
 from bzrlib.transport import (
     AppendBasedFileStream,
     _file_streams,

=== modified file 'bzrlib/transport/nosmart.py'
--- bzrlib/transport/nosmart.py	2010-02-09 17:27:46 +0000
+++ bzrlib/transport/nosmart.py	2011-12-19 13:23:58 +0000
@@ -21,6 +21,8 @@
 to never have a smart medium.
 """
 
+from __future__ import absolute_import
+
 from bzrlib import errors
 from bzrlib.transport import decorator
 

=== modified file 'bzrlib/transport/pathfilter.py'
--- bzrlib/transport/pathfilter.py	2011-08-11 14:08:05 +0000
+++ bzrlib/transport/pathfilter.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,8 @@
 
 """A transport decorator that filters all paths that are passed to it."""
 
+from __future__ import absolute_import
+
 from bzrlib import urlutils
 
 from bzrlib.transport import (

=== modified file 'bzrlib/transport/readonly.py'
--- bzrlib/transport/readonly.py	2010-02-09 17:15:17 +0000
+++ bzrlib/transport/readonly.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,8 @@
 
 """Implementation of Transport that adapts another transport to be readonly."""
 
+from __future__ import absolute_import
+
 from bzrlib.errors import TransportNotPossible, NoSmartMedium
 from bzrlib.transport import decorator
 

=== modified file 'bzrlib/transport/remote.py'
--- bzrlib/transport/remote.py	2011-08-19 22:34:02 +0000
+++ bzrlib/transport/remote.py	2011-12-30 14:19:53 +0000
@@ -20,6 +20,8 @@
 imported from bzrlib.smart.
 """
 
+from __future__ import absolute_import
+
 __all__ = ['RemoteTransport', 'RemoteTCPTransport', 'RemoteSSHTransport']
 
 from cStringIO import StringIO

=== modified file 'bzrlib/transport/sftp.py'
--- bzrlib/transport/sftp.py	2011-08-11 16:24:09 +0000
+++ bzrlib/transport/sftp.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,8 @@
 
 """Implementation of Transport over SFTP, using paramiko."""
 
+from __future__ import absolute_import
+
 # TODO: Remove the transport-based lock_read and lock_write methods.  They'll
 # then raise TransportNotPossible, which will break remote access to any
 # formats which rely on OS-level locks.  That should be fine as those formats

=== modified file 'bzrlib/transport/ssh.py'
--- bzrlib/transport/ssh.py	2011-05-16 13:39:39 +0000
+++ bzrlib/transport/ssh.py	2011-12-19 13:23:58 +0000
@@ -17,6 +17,8 @@
 
 """Foundation SSH support for SFTP and smart server."""
 
+from __future__ import absolute_import
+
 import errno
 import getpass
 import logging
@@ -352,6 +354,11 @@
 class SubprocessVendor(SSHVendor):
     """Abstract base class for vendors that use pipes to a subprocess."""
 
+    # In general stderr should be inherited from the parent process so prompts
+    # are visible on the terminal. This can be overriden to another file for
+    # tests, but beware of using PIPE which may hang due to not being read.
+    _stderr_target = None
+
     def _connect(self, argv):
         # Attempt to make a socketpair to use as stdin/stdout for the SSH
         # subprocess.  We prefer sockets to pipes because they support
@@ -368,6 +375,7 @@
         else:
             stdin = stdout = subproc_sock
         proc = subprocess.Popen(argv, stdin=stdin, stdout=stdout,
+                                stderr=self._stderr_target,
                                 **os_specific_subprocess_params())
         if subproc_sock is not None:
             subproc_sock.close()

=== modified file 'bzrlib/transport/trace.py'
--- bzrlib/transport/trace.py	2010-02-09 20:33:43 +0000
+++ bzrlib/transport/trace.py	2011-12-19 13:23:58 +0000
@@ -20,6 +20,8 @@
 and then delegates it.
 """
 
+from __future__ import absolute_import
+
 from bzrlib.transport import decorator
 
 

=== modified file 'bzrlib/transport/unlistable.py'
--- bzrlib/transport/unlistable.py	2010-02-09 17:29:26 +0000
+++ bzrlib/transport/unlistable.py	2011-12-19 13:23:58 +0000
@@ -16,7 +16,8 @@
 
 """Transport implementation that disables listing to simulate HTTP cheaply."""
 
-import bzrlib.errors as errors
+from __future__ import absolute_import
+
 from bzrlib.transport import Transport
 from bzrlib.transport import decorator
 

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2011-08-05 13:34:40 +0000
+++ bzrlib/tree.py	2011-12-19 13:23:58 +0000
@@ -17,6 +17,8 @@
 """Tree classes, representing directory at point in time.
 """
 
+from __future__ import absolute_import
+
 import os
 
 from bzrlib.lazy_import import lazy_import
@@ -35,6 +37,7 @@
     rules,
     trace,
     )
+from bzrlib.i18n import gettext
 """)
 
 from bzrlib.decorators import needs_read_lock
@@ -58,6 +61,14 @@
     trees or versioned trees.
     """
 
+    def has_versioned_directories(self):
+        """Whether this tree can contain explicitly versioned directories.
+
+        This defaults to True, but some implementations may want to override
+        it.
+        """
+        return True
+
     def changes_from(self, other, want_unchanged=False, specific_files=None,
         extra_trees=None, require_versioned=False, include_root=False,
         want_unversioned=False):
@@ -346,9 +357,6 @@
         """
         raise NotImplementedError(self.get_file_size)
 
-    def get_file_by_path(self, path):
-        raise NotImplementedError(self.get_file_by_path)
-
     def is_executable(self, file_id, path=None):
         """Check if a file is executable.
 
@@ -623,7 +631,7 @@
         prefs = self.iter_search_rules([path], filter_pref_names).next()
         stk = filters._get_filter_stack_for(prefs)
         if 'filters' in debug.debug_flags:
-            trace.note("*** %s content-filter: %s => %r" % (path,prefs,stk))
+            trace.note(gettext("*** {0} content-filter: {1} => {2!r}").format(path,prefs,stk))
         return stk
 
     def _content_filter_stack_provider(self):
@@ -818,8 +826,9 @@
         return self.inventory.iter_entries_by_dir(
             specific_file_ids=specific_file_ids, yield_parents=yield_parents)
 
+    @deprecated_method(deprecated_in((2, 5, 0)))
     def get_file_by_path(self, path):
-        return self.get_file(self._inventory.path2id(path), path)
+        return self.get_file(self.path2id(path), path)
 
 
 def find_ids_across_trees(filenames, trees, require_versioned=True):

=== modified file 'bzrlib/treebuilder.py'
--- bzrlib/treebuilder.py	2010-06-17 07:11:55 +0000
+++ bzrlib/treebuilder.py	2011-12-19 13:23:58 +0000
@@ -20,6 +20,8 @@
 can be extremely useful in testing for instance.
 """
 
+from __future__ import absolute_import
+
 from bzrlib import errors
 
 

=== modified file 'bzrlib/tsort.py'
--- bzrlib/tsort.py	2011-05-18 16:42:48 +0000
+++ bzrlib/tsort.py	2011-12-19 13:23:58 +0000
@@ -14,9 +14,10 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-
 """Topological sorting routines."""
 
+from __future__ import absolute_import
+
 
 from bzrlib import (
     errors,

=== modified file 'bzrlib/tuned_gzip.py'
--- bzrlib/tuned_gzip.py	2011-01-11 18:10:55 +0000
+++ bzrlib/tuned_gzip.py	2011-12-19 13:23:58 +0000
@@ -17,6 +17,8 @@
 
 """Bzrlib specific gzip tunings. We plan to feed these to the upstream gzip."""
 
+from __future__ import absolute_import
+
 from cStringIO import StringIO
 
 # make GzipFile faster:

=== modified file 'bzrlib/ui/__init__.py'
--- bzrlib/ui/__init__.py	2011-08-08 15:57:57 +0000
+++ bzrlib/ui/__init__.py	2011-12-18 12:46:49 +0000
@@ -41,6 +41,7 @@
     back to working through the terminal.
 """
 
+from __future__ import absolute_import
 
 import warnings
 
@@ -154,6 +155,10 @@
             "The command 'bzr %(deprecated_name)s' "
             "has been deprecated in bzr %(deprecated_in_version)s. "
             "Please use 'bzr %(recommended_name)s' instead."),
+        deprecated_command_option=(
+            "The option '%(deprecated_name)s' to 'bzr %(command)s' "
+            "has been deprecated in bzr %(deprecated_in_version)s. "
+            "Please use '%(recommended_name)s' instead."),
         recommend_upgrade=("%(current_format_name)s is deprecated "
             "and a better format is available.\n"
             "It is recommended that you upgrade by "
@@ -320,6 +325,26 @@
             warnings.warn(fail)   # so tests will fail etc
             return fail
 
+    def choose(self, msg, choices, default=None):
+        """Prompt the user for a list of alternatives.
+
+        :param msg: message to be shown as part of the prompt.
+
+        :param choices: list of choices, with the individual choices separated
+            by '\n', e.g.: choose("Save changes?", "&Yes\n&No\n&Cancel"). The
+            letter after the '&' is the shortcut key for that choice. Thus you
+            can type 'c' to select "Cancel".  Shorcuts are case insensitive.
+            The shortcut does not need to be the first letter. If a shorcut key
+            is not provided, the first letter for the choice will be used.
+
+        :param default: default choice (index), returned for example when enter
+            is pressed for the console version.
+
+        :return: the index fo the user choice (so '0', '1' or '2' for
+            respectively yes/no/cancel in the previous example).
+        """
+        raise NotImplementedError(self.choose)
+
     def get_boolean(self, prompt):
         """Get a boolean question answered from the user.
 
@@ -327,7 +352,8 @@
             line without terminating \\n.
         :return: True or False for y/yes or n/no.
         """
-        raise NotImplementedError(self.get_boolean)
+        choice = self.choose(prompt + '?', '&yes\n&no', default=None)
+        return 0 == choice
 
     def get_integer(self, prompt):
         """Get an integer from the user.

=== modified file 'bzrlib/ui/text.py'
--- bzrlib/ui/text.py	2011-05-16 13:39:39 +0000
+++ bzrlib/ui/text.py	2011-12-18 12:46:49 +0000
@@ -18,6 +18,8 @@
 """Text UI, write output to the console.
 """
 
+from __future__ import absolute_import
+
 import os
 import sys
 import time
@@ -43,6 +45,116 @@
     )
 
 
+class _ChooseUI(object):
+
+    """ Helper class for choose implementation.
+    """
+
+    def __init__(self, ui, msg, choices, default):
+        self.ui = ui
+        self._setup_mode()
+        self._build_alternatives(msg, choices, default)
+
+    def _setup_mode(self):
+        """Setup input mode (line-based, char-based) and echo-back.
+
+        Line-based input is used if the BZR_TEXTUI_INPUT environment
+        variable is set to 'line-based', or if there is no controlling
+        terminal.
+        """
+        if os.environ.get('BZR_TEXTUI_INPUT') != 'line-based' and \
+           self.ui.stdin == sys.stdin and self.ui.stdin.isatty():
+            self.line_based = False
+            self.echo_back = True
+        else:
+            self.line_based = True
+            self.echo_back = not self.ui.stdin.isatty()
+
+    def _build_alternatives(self, msg, choices, default):
+        """Parse choices string.
+
+        Setup final prompt and the lists of choices and associated
+        shortcuts.
+        """
+        index = 0
+        help_list = []
+        self.alternatives = {}
+        choices = choices.split('\n')
+        if default is not None and default not in range(0, len(choices)):
+            raise ValueError("invalid default index")
+        for c in choices:
+            name = c.replace('&', '').lower()
+            choice = (name, index)
+            if name in self.alternatives:
+                raise ValueError("duplicated choice: %s" % name)
+            self.alternatives[name] = choice
+            shortcut = c.find('&')
+            if -1 != shortcut and (shortcut + 1) < len(c):
+                help = c[:shortcut]
+                help += '[' + c[shortcut + 1] + ']'
+                help += c[(shortcut + 2):]
+                shortcut = c[shortcut + 1]
+            else:
+                c = c.replace('&', '')
+                shortcut = c[0]
+                help = '[%s]%s' % (shortcut, c[1:])
+            shortcut = shortcut.lower()
+            if shortcut in self.alternatives:
+                raise ValueError("duplicated shortcut: %s" % shortcut)
+            self.alternatives[shortcut] = choice
+            # Add redirections for default.
+            if index == default:
+                self.alternatives[''] = choice
+                self.alternatives['\r'] = choice
+            help_list.append(help)
+            index += 1
+
+        self.prompt = u'%s (%s): ' % (msg, ', '.join(help_list))
+
+    def _getline(self):
+        line = self.ui.stdin.readline()
+        if '' == line:
+            raise EOFError
+        return line.strip()
+
+    def _getchar(self):
+        char = osutils.getchar()
+        if char == chr(3): # INTR
+            raise KeyboardInterrupt
+        if char == chr(4): # EOF (^d, C-d)
+            raise EOFError
+        return char
+
+    def interact(self):
+        """Keep asking the user until a valid choice is made.
+        """
+        if self.line_based:
+            getchoice = self._getline
+        else:
+            getchoice = self._getchar
+        iter = 0
+        while True:
+            iter += 1
+            if 1 == iter or self.line_based:
+                self.ui.prompt(self.prompt)
+            try:
+                choice = getchoice()
+            except EOFError:
+                self.ui.stderr.write('\n')
+                return None
+            except KeyboardInterrupt:
+                self.ui.stderr.write('\n')
+                raise KeyboardInterrupt
+            choice = choice.lower()
+            if choice not in self.alternatives:
+                # Not a valid choice, keep on asking.
+                continue
+            name, index = self.alternatives[choice]
+            if self.echo_back:
+                self.ui.stderr.write(name + '\n')
+            return index
+
+
 class TextUIFactory(UIFactory):
     """A UI factory for Text user interefaces."""
 
@@ -61,6 +173,27 @@
         # paints progress, network activity, etc
         self._progress_view = self.make_progress_view()
 
+    def choose(self, msg, choices, default=None):
+        """Prompt the user for a list of alternatives.
+
+        Support both line-based and char-based editing.
+
+        In line-based mode, both the shortcut and full choice name are valid
+        answers, e.g. for choose('prompt', '&yes\n&no'): 'y', ' Y ', ' yes',
+        'YES ' are all valid input lines for choosing 'yes'.
+
+        An empty line, when in line-based mode, or pressing enter in char-based
+        mode will select the default choice (if any).
+
+        Choice is echoed back if:
+        - input is char-based; which means a controlling terminal is available,
+          and osutils.getchar is used
+        - input is line-based, and no controlling terminal is available
+        """
+
+        choose_ui = _ChooseUI(self, msg, choices, default)
+        return choose_ui.interact()
+
     def be_quiet(self, state):
         if state and not self._quiet:
             self.clear_term()
@@ -78,18 +211,6 @@
         # to clear it.  We might need to separately check for the case of
         self._progress_view.clear()
 
-    def get_boolean(self, prompt):
-        while True:
-            self.prompt(prompt + "? [y/n]: ")
-            line = self.stdin.readline().lower()
-            if line in ('y\n', 'yes\n'):
-                return True
-            elif line in ('n\n', 'no\n'):
-                return False
-            elif line in ('', None):
-                # end-of-file; possibly should raise an error here instead
-                return None
-
     def get_integer(self, prompt):
         while True:
             self.prompt(prompt)
@@ -205,6 +326,7 @@
             prompt = prompt % kwargs
         prompt = prompt.encode(osutils.get_terminal_encoding(), 'replace')
         self.clear_term()
+        self.stdout.flush()
         self.stderr.write(prompt)
 
     def report_transport_activity(self, transport, byte_count, direction):

=== modified file 'bzrlib/uncommit.py'
--- bzrlib/uncommit.py	2011-08-22 10:53:21 +0000
+++ bzrlib/uncommit.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,8 @@
 
 """Remove the last revision from the history of the current branch."""
 
+from __future__ import absolute_import
+
 # TODO: make the guts of this methods on tree, branch.
 
 from bzrlib import (

=== modified file 'bzrlib/upgrade.py'
--- bzrlib/upgrade.py	2011-08-17 09:27:29 +0000
+++ bzrlib/upgrade.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,7 @@
 
 """bzr upgrade logic."""
 
+from __future__ import absolute_import
 
 from bzrlib import (
     errors,
@@ -23,10 +24,11 @@
     ui,
     urlutils,
     )
-from bzrlib.bzrdir import (
-    BzrDir,
+from bzrlib.controldir import (
+    ControlDir,
     format_registry,
     )
+from bzrlib.i18n import gettext
 from bzrlib.remote import RemoteBzrDir
 
 
@@ -53,7 +55,7 @@
         if control_dir is not None:
             self.bzrdir = control_dir
         else:
-            self.bzrdir = BzrDir.open_unsupported(url)
+            self.bzrdir = ControlDir.open_unsupported(url)
         if isinstance(self.bzrdir, RemoteBzrDir):
             self.bzrdir._ensure_real()
             self.bzrdir = self.bzrdir._real_bzrdir
@@ -71,9 +73,9 @@
         try:
             branch = self.bzrdir.open_branch()
             if branch.user_url != self.bzrdir.user_url:
-                ui.ui_factory.note(
+                ui.ui_factory.note(gettext(
                     'This is a checkout. The branch (%s) needs to be upgraded'
-                    ' separately.' % (urlutils.unescape_for_display(
+                    ' separately.') % (urlutils.unescape_for_display(
                         branch.user_url, 'utf-8')))
             del branch
         except (errors.NotBranchError, errors.IncompatibleRepositories):
@@ -95,17 +97,17 @@
         if not self.bzrdir.needs_format_conversion(format):
             raise errors.UpToDateFormat(self.bzrdir._format)
         if not self.bzrdir.can_convert_format():
-            raise errors.BzrError("cannot upgrade from bzrdir format %s" %
+            raise errors.BzrError(gettext("cannot upgrade from bzrdir format %s") %
                            self.bzrdir._format)
         self.bzrdir.check_conversion_target(format)
-        ui.ui_factory.note('starting upgrade of %s' % 
+        ui.ui_factory.note(gettext('starting upgrade of %s') % 
             urlutils.unescape_for_display(self.transport.base, 'utf-8'))
 
         self.backup_oldpath, self.backup_newpath = self.bzrdir.backup_bzrdir()
         while self.bzrdir.needs_format_conversion(format):
             converter = self.bzrdir._format.get_converter(format)
             self.bzrdir = converter.convert(self.bzrdir, None)
-        ui.ui_factory.note('finished')
+        ui.ui_factory.note(gettext('finished'))
 
     def clean_up(self):
         """Clean-up after a conversion.
@@ -115,7 +117,7 @@
         transport = self.transport
         backup_relpath = transport.relpath(self.backup_newpath)
         child_pb = ui.ui_factory.nested_progress_bar()
-        child_pb.update('Deleting backup.bzr')
+        child_pb.update(gettext('Deleting backup.bzr'))
         try:
             transport.delete_tree(backup_relpath)
         finally:
@@ -138,7 +140,7 @@
     :param dry_run: show what would happen but don't actually do any upgrades
     :return: the list of exceptions encountered
     """
-    control_dirs = [BzrDir.open_unsupported(url)]
+    control_dirs = [ControlDir.open_unsupported(url)]
     attempted, succeeded, exceptions = smart_upgrade(control_dirs,
         format, clean_up=clean_up, dry_run=dry_run)
     if len(attempted) > 1:
@@ -146,8 +148,9 @@
         succeeded_count = len(succeeded)
         failed_count = attempted_count - succeeded_count
         ui.ui_factory.note(
-            '\nSUMMARY: %d upgrades attempted, %d succeeded, %d failed'
-            % (attempted_count, succeeded_count, failed_count))
+            gettext('\nSUMMARY: {0} upgrades attempted, {1} succeeded,'\
+                    ' {2} failed').format(
+                     attempted_count, succeeded_count, failed_count))
     return exceptions
 
 
@@ -202,7 +205,7 @@
     succeeded, exceptions = _convert_items([control_dir], format, clean_up,
                                            dry_run)
     if succeeded and dependents:
-        ui.ui_factory.note('Found %d dependent branches - upgrading ...'
+        ui.ui_factory.note(gettext('Found %d dependent branches - upgrading ...')
                            % (len(dependents),))
         # Convert dependent branches
         branch_cdirs = [b.bzrdir for b in dependents]
@@ -270,18 +273,22 @@
     succeeded = []
     exceptions = []
     child_pb = ui.ui_factory.nested_progress_bar()
-    child_pb.update('Upgrading bzrdirs', 0, len(items))
+    child_pb.update(gettext('Upgrading bzrdirs'), 0, len(items))
     for i, control_dir in enumerate(items):
         # Do the conversion
         location = control_dir.root_transport.base
         bzr_object, bzr_label = _get_object_and_label(control_dir)
         type_label = label or bzr_label
-        child_pb.update("Upgrading %s" % (type_label), i+1, len(items))
-        ui.ui_factory.note('Upgrading %s %s ...' % (type_label, 
+        child_pb.update(gettext("Upgrading %s") % (type_label), i+1, len(items))
+        ui.ui_factory.note(gettext('Upgrading {0} {1} ...').format(type_label, 
             urlutils.unescape_for_display(location, 'utf-8'),))
         try:
             if not dry_run:
                 cv = Convert(control_dir=control_dir, format=format)
+        except errors.UpToDateFormat, ex:
+            ui.ui_factory.note(str(ex))
+            succeeded.append(control_dir)
+            continue
         except Exception, ex:
             trace.warning('conversion error: %s' % ex)
             exceptions.append(ex)
@@ -291,11 +298,11 @@
         succeeded.append(control_dir)
         if clean_up:
             try:
-                ui.ui_factory.note('Removing backup ...')
+                ui.ui_factory.note(gettext('Removing backup ...'))
                 if not dry_run:
                     cv.clean_up()
             except Exception, ex:
-                trace.warning('failed to clean-up %s: %s' % (location, ex))
+                trace.warning(gettext('failed to clean-up {0}: {1}') % (location, ex))
                 exceptions.append(ex)
 
     child_pb.finished()

=== modified file 'bzrlib/urlutils.py'
--- bzrlib/urlutils.py	2011-08-15 13:12:36 +0000
+++ bzrlib/urlutils.py	2011-12-19 13:23:58 +0000
@@ -16,14 +16,15 @@
 
 """A collection of function for handling URL operations."""
 
+from __future__ import absolute_import
+
 import os
 import re
 import sys
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
-from posixpath import split as _posix_split, normpath as _posix_normpath
-import urllib
+from posixpath import split as _posix_split
 import urlparse
 
 from bzrlib import (
@@ -60,13 +61,87 @@
     return split(url, exclude_trailing_slash=exclude_trailing_slash)[0]
 
 
+# Private copies of quote and unquote, copied from Python's
+# urllib module because urllib unconditionally imports socket, which imports
+# ssl.
+
+always_safe = ('ABCDEFGHIJKLMNOPQRSTUVWXYZ'
+               'abcdefghijklmnopqrstuvwxyz'
+               '0123456789' '_.-')
+_safe_map = {}
+for i, c in zip(xrange(256), str(bytearray(xrange(256)))):
+    _safe_map[c] = c if (i < 128 and c in always_safe) else '%{0:02X}'.format(i)
+_safe_quoters = {}
+
+
+def quote(s, safe='/'):
+    """quote('abc def') -> 'abc%20def'
+
+    Each part of a URL, e.g. the path info, the query, etc., has a
+    different set of reserved characters that must be quoted.
+
+    RFC 2396 Uniform Resource Identifiers (URI): Generic Syntax lists
+    the following reserved characters.
+
+    reserved    = ";" | "/" | "?" | ":" | "@" | "&" | "=" | "+" |
+                  "$" | ","
+
+    Each of these characters is reserved in some component of a URL,
+    but not necessarily in all of them.
+
+    By default, the quote function is intended for quoting the path
+    section of a URL.  Thus, it will not encode '/'.  This character
+    is reserved, but in typical usage the quote function is being
+    called on a path where the existing slash characters are used as
+    reserved characters.
+    """
+    # fastpath
+    if not s:
+        if s is None:
+            raise TypeError('None object cannot be quoted')
+        return s
+    cachekey = (safe, always_safe)
+    try:
+        (quoter, safe) = _safe_quoters[cachekey]
+    except KeyError:
+        safe_map = _safe_map.copy()
+        safe_map.update([(c, c) for c in safe])
+        quoter = safe_map.__getitem__
+        safe = always_safe + safe
+        _safe_quoters[cachekey] = (quoter, safe)
+    if not s.rstrip(safe):
+        return s
+    return ''.join(map(quoter, s))
+
+
+_hexdig = '0123456789ABCDEFabcdef'
+_hextochr = dict((a + b, chr(int(a + b, 16)))
+                 for a in _hexdig for b in _hexdig)
+
+def unquote(s):
+    """unquote('abc%20def') -> 'abc def'."""
+    res = s.split('%')
+    # fastpath
+    if len(res) == 1:
+        return s
+    s = res[0]
+    for item in res[1:]:
+        try:
+            s += _hextochr[item[:2]] + item[2:]
+        except KeyError:
+            s += '%' + item
+        except UnicodeDecodeError:
+            s += unichr(int(item[:2], 16)) + item[2:]
+    return s
+
+
 def escape(relpath):
     """Escape relpath to be a valid url."""
     if isinstance(relpath, unicode):
         relpath = relpath.encode('utf-8')
     # After quoting and encoding, the path should be perfectly
     # safe as a plain ASCII string, str() just enforces this
-    return str(urllib.quote(relpath, safe='/~'))
+    return str(quote(relpath, safe='/~'))
 
 
 def file_relpath(base, path):
@@ -78,8 +153,8 @@
         raise ValueError('Length of base (%r) must equal or'
             ' exceed the platform minimum url length (which is %d)' %
             (base, MIN_ABS_FILEURL_LENGTH))
-    base = local_path_from_url(base)
-    path = local_path_from_url(path)
+    base = osutils.normpath(local_path_from_url(base))
+    path = osutils.normpath(local_path_from_url(path))
     return escape(osutils.relpath(base, path))
 
 
@@ -201,8 +276,7 @@
     """
     # importing directly from posixpath allows us to test this
     # on non-posix platforms
-    return 'file://' + escape(_posix_normpath(
-        osutils._posix_abspath(path)))
+    return 'file://' + escape(osutils._posix_abspath(path))
 
 
 def _win32_local_path_from_url(url):
@@ -388,7 +462,7 @@
     """On win32 the drive letter needs to be added to the url base."""
     # Strip off the drive letter
     # path is currently /C:/foo
-    if len(path) < 3 or path[2] not in ':|' or path[3] != '/':
+    if len(path) < 4 or path[2] not in ':|' or path[3] != '/':
         raise errors.InvalidURL(url_base + path,
             'win32 file:/// paths need a drive letter')
     url_base += path[0:3] # file:// + /C:
@@ -442,11 +516,15 @@
     :param url: A relative or absolute URL
     :return: (url, subsegments)
     """
-    (parent_url, child_dir) = split(url)
-    subsegments = child_dir.split(",")
-    if len(subsegments) == 1:
+    # GZ 2011-11-18: Dodgy removing the terminal slash like this, function
+    #                operates on urls not url+segments, and Transport classes
+    #                should not be blindly adding slashes in the first place. 
+    lurl = strip_trailing_slash(url)
+    # Segments begin at first comma after last forward slash, if one exists
+    segment_start = lurl.find(",", lurl.rfind("/")+1)
+    if segment_start == -1:
         return (url, [])
-    return (join(parent_url, subsegments[0]), subsegments[1:])
+    return (lurl[:segment_start], lurl[segment_start+1:].split(","))
 
 
 def split_segment_parameters(url):
@@ -563,7 +641,7 @@
     This returns a Unicode path from a URL
     """
     # jam 20060427 URLs are supposed to be ASCII only strings
-    #       If they are passed in as unicode, urllib.unquote
+    #       If they are passed in as unicode, unquote
     #       will return a UNICODE string, which actually contains
     #       utf-8 bytes. So we have to ensure that they are
     #       plain ASCII strings, or the final .decode will
@@ -574,7 +652,7 @@
     except UnicodeError, e:
         raise errors.InvalidURL(url, 'URL was not a plain ASCII url: %s' % (e,))
 
-    unquoted = urllib.unquote(url)
+    unquoted = unquote(url)
     try:
         unicode_path = unquoted.decode('utf-8')
     except UnicodeError, e:
@@ -739,20 +817,20 @@
             port, quoted_path):
         self.scheme = scheme
         self.quoted_host = quoted_host
-        self.host = urllib.unquote(self.quoted_host)
+        self.host = unquote(self.quoted_host)
         self.quoted_user = quoted_user
         if self.quoted_user is not None:
-            self.user = urllib.unquote(self.quoted_user)
+            self.user = unquote(self.quoted_user)
         else:
             self.user = None
         self.quoted_password = quoted_password
         if self.quoted_password is not None:
-            self.password = urllib.unquote(self.quoted_password)
+            self.password = unquote(self.quoted_password)
         else:
             self.password = None
         self.port = port
-        self.quoted_path = quoted_path
-        self.path = urllib.unquote(self.quoted_path)
+        self.quoted_path = _url_hex_escapes_re.sub(_unescape_safe_chars, quoted_path)
+        self.path = unquote(self.quoted_path)
 
     def __eq__(self, other):
         return (isinstance(other, self.__class__) and
@@ -836,6 +914,7 @@
         """
         if not isinstance(relpath, str):
             raise errors.InvalidURL(relpath)
+        relpath = _url_hex_escapes_re.sub(_unescape_safe_chars, relpath)
         if relpath.startswith('/'):
             base_parts = []
         else:
@@ -867,7 +946,7 @@
         if offset is not None:
             relative = unescape(offset).encode('utf-8')
             path = self._combine_paths(self.path, relative)
-            path = urllib.quote(path)
+            path = quote(path, safe="/~")
         else:
             path = self.quoted_path
         return self.__class__(self.scheme, self.quoted_user,

=== modified file 'bzrlib/utextwrap.py'
--- bzrlib/utextwrap.py	2011-05-14 14:32:02 +0000
+++ bzrlib/utextwrap.py	2011-12-18 15:28:38 +0000
@@ -23,6 +23,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 import sys
 import textwrap
 from unicodedata import east_asian_width as _eawidth

=== modified file 'bzrlib/util/__init__.py'
--- bzrlib/util/__init__.py	2005-09-19 06:05:19 +0000
+++ bzrlib/util/__init__.py	2011-12-18 15:28:38 +0000
@@ -0,0 +1,1 @@
+from __future__ import absolute_import

=== modified file 'bzrlib/util/_bencode_py.py'
--- bzrlib/util/_bencode_py.py	2010-05-03 20:57:39 +0000
+++ bzrlib/util/_bencode_py.py	2011-12-18 15:28:38 +0000
@@ -15,6 +15,8 @@
 #
 # Modifications copyright (C) 2008 Canonical Ltd
 
+from __future__ import absolute_import
+
 class BDecoder(object):
 
     def __init__(self, yield_tuples=False):

=== modified file 'bzrlib/util/configobj/__init__.py'
--- bzrlib/util/configobj/__init__.py	2005-10-18 18:48:27 +0000
+++ bzrlib/util/configobj/__init__.py	2011-12-18 15:28:38 +0000
@@ -0,0 +1,1 @@
+from __future__ import absolute_import

=== modified file 'bzrlib/util/configobj/configobj.py'
--- bzrlib/util/configobj/configobj.py	2011-02-22 09:17:40 +0000
+++ bzrlib/util/configobj/configobj.py	2011-12-18 12:46:49 +0000
@@ -17,7 +17,7 @@
 # Comments, suggestions and bug reports welcome.
 
 
-from __future__ import generators
+from __future__ import absolute_import
 
 import sys
 import os

=== modified file 'bzrlib/util/effbot/__init__.py'
--- bzrlib/util/effbot/__init__.py	2005-09-19 06:05:19 +0000
+++ bzrlib/util/effbot/__init__.py	2011-12-18 15:28:38 +0000
@@ -1,1 +1,3 @@
 # $Id: __init__.py 271 2004-10-09 10:50:59Z fredrik $
+from __future__ import absolute_import
+

=== modified file 'bzrlib/util/effbot/org/__init__.py'
--- bzrlib/util/effbot/org/__init__.py	2005-09-19 06:05:19 +0000
+++ bzrlib/util/effbot/org/__init__.py	2011-12-18 15:28:38 +0000
@@ -1,1 +1,2 @@
 # $Id: __init__.py 271 2004-10-09 10:50:59Z fredrik $
+from __future__ import absolute_import

=== modified file 'bzrlib/util/effbot/org/gzip_consumer.py'
--- bzrlib/util/effbot/org/gzip_consumer.py	2005-09-19 06:05:19 +0000
+++ bzrlib/util/effbot/org/gzip_consumer.py	2011-12-18 15:28:38 +0000
@@ -4,6 +4,8 @@
 # Copyright (c) 2001-2004 by Fredrik Lundh.  All rights reserved.
 #
 
+from __future__ import absolute_import
+
 ##
 # Consumer wrapper for GZIP streams.
 

=== modified file 'bzrlib/util/effbot/org/http_client.py'
--- bzrlib/util/effbot/org/http_client.py	2005-09-29 20:34:25 +0000
+++ bzrlib/util/effbot/org/http_client.py	2011-12-18 15:28:38 +0000
@@ -11,6 +11,8 @@
 # Copyright (c) 2001-2004 by Fredrik Lundh.  All rights reserved.
 #
 
+from __future__ import absolute_import
+
 import asyncore
 import socket, string, time, sys
 import StringIO

=== modified file 'bzrlib/util/effbot/org/http_manager.py'
--- bzrlib/util/effbot/org/http_manager.py	2005-09-19 06:05:19 +0000
+++ bzrlib/util/effbot/org/http_manager.py	2011-12-18 15:28:38 +0000
@@ -6,6 +6,8 @@
 # Copyright (c) 2001-2004 by Fredrik Lundh.  All rights reserved.
 #
 
+from __future__ import absolute_import
+
 import asyncore, time
 import http_client
 

=== modified file 'bzrlib/util/elementtree/ElementTree.py'
--- bzrlib/util/elementtree/ElementTree.py	2006-09-08 18:46:29 +0000
+++ bzrlib/util/elementtree/ElementTree.py	2011-12-18 15:28:38 +0000
@@ -67,6 +67,8 @@
 # OF THIS SOFTWARE.
 # --------------------------------------------------------------------
 
+from __future__ import absolute_import
+
 __all__ = [
     # public symbols
     "Comment",

=== modified file 'bzrlib/util/elementtree/__init__.py'
--- bzrlib/util/elementtree/__init__.py	2005-09-19 06:05:19 +0000
+++ bzrlib/util/elementtree/__init__.py	2011-12-18 15:28:38 +0000
@@ -28,3 +28,5 @@
 # ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
 # OF THIS SOFTWARE.
 # --------------------------------------------------------------------
+
+from __future__ import absolute_import

=== modified file 'bzrlib/util/simplemapi.py'
--- bzrlib/util/simplemapi.py	2009-01-17 01:30:58 +0000
+++ bzrlib/util/simplemapi.py	2011-12-19 13:23:58 +0000
@@ -36,6 +36,8 @@
 software, please contact me at the above email address.
 """
 
+from __future__ import absolute_import
+
 import os
 from ctypes import *
 

=== modified file 'bzrlib/version.py'
--- bzrlib/version.py	2010-04-30 11:03:59 +0000
+++ bzrlib/version.py	2011-12-19 13:23:58 +0000
@@ -16,13 +16,15 @@
 
 """Report on version of bzrlib"""
 
+from __future__ import absolute_import
+
 import os
 import sys
 
 import bzrlib
 from bzrlib import (
-    bzrdir,
     config,
+    controldir,
     errors,
     osutils,
     trace,
@@ -73,7 +75,7 @@
     else:
         to_file.write(bzrlib.__path__[0] + '\n')
     if show_config:
-        config_dir = os.path.normpath(config.config_dir())  # use native slashes
+        config_dir = osutils.normpath(config.config_dir())  # use native slashes
         if not isinstance(config_dir, unicode):
             config_dir = config_dir.decode(osutils.get_user_encoding())
         to_file.write("  Bazaar configuration: %s\n" % (config_dir,))
@@ -98,7 +100,7 @@
     If bzr is not being run from its working tree, returns None.
     """
     try:
-        control = bzrdir.BzrDir.open_containing(__file__)[0]
+        control = controldir.ControlDir.open_containing(__file__)[0]
         return control.open_workingtree(recommend_upgrade=False)
     except (errors.NotBranchError, errors.UnknownFormatError,
             errors.NoWorkingTree):

=== modified file 'bzrlib/version_info_formats/__init__.py'
--- bzrlib/version_info_formats/__init__.py	2011-06-12 20:09:55 +0000
+++ bzrlib/version_info_formats/__init__.py	2011-12-30 13:02:27 +0000
@@ -16,10 +16,15 @@
 
 """Routines for extracting all version information from a bzr branch."""
 
+from __future__ import absolute_import
+
 import time
 
 from bzrlib.osutils import local_time_offset, format_date
-from bzrlib import registry
+from bzrlib import (
+    registry,
+    revision as _mod_revision,
+    )
 
 
 def create_date_str(timestamp=None, offset=None):
@@ -45,7 +50,8 @@
                 check_for_clean=False,
                 include_revision_history=False,
                 include_file_revisions=False,
-                template=None):
+                template=None,
+                revision_id=None):
         """Build up information about the given branch.
         If working_tree is given, it can be checked for changes.
 
@@ -61,9 +67,9 @@
             include the explicit last-changed revision for each file.
         :param template: Template for the output formatting, not used by
             all builders.
+        :param revision_id: Revision id to print version for (optional)
         """
         self._branch = branch
-        self._working_tree = working_tree
         self._check = check_for_clean
         self._include_history = include_revision_history
         self._include_file_revs = include_file_revisions
@@ -71,7 +77,15 @@
 
         self._clean = None
         self._file_revisions = {}
-        self._revision_history_info= []
+        self._revision_id = revision_id
+
+        if self._revision_id is None:
+            self._tree = working_tree
+            self._working_tree = working_tree
+        else:
+            self._tree = self._branch.repository.revision_tree(self._revision_id)
+            # the working tree is not relevant if an explicit revision was specified
+            self._working_tree = None
 
     def _extract_file_revisions(self):
         """Extract the working revisions for all files"""
@@ -79,13 +93,13 @@
         # Things seem clean if we never look :)
         self._clean = True
 
-        if self._working_tree is not None:
+        if self._working_tree is self._tree:
             basis_tree = self._working_tree.basis_tree()
             # TODO: jam 20070215 The working tree should actually be locked at
             #       a higher level, but this will do for now.
             self._working_tree.lock_read()
         else:
-            basis_tree = self._branch.basis_tree()
+            basis_tree = self._branch.repository.revision_tree(self._revision_id)
 
         basis_tree.lock_read()
         try:
@@ -93,7 +107,7 @@
             for info in basis_tree.list_files(include_root=True):
                 self._file_revisions[info[0]] = info[-1].revision
 
-            if not self._check or self._working_tree is None:
+            if not self._check or self._working_tree is not self._tree:
                 return
 
             delta = self._working_tree.changes_from(basis_tree,
@@ -130,31 +144,28 @@
             if self._working_tree is not None:
                 self._working_tree.unlock()
 
-    def _extract_revision_history(self):
+    def _iter_revision_history(self):
         """Find the messages for all revisions in history."""
 
-        # Unfortunately, there is no WorkingTree.revision_history
-        rev_hist = self._branch.revision_history()
-        if self._working_tree is not None:
-            last_rev = self._working_tree.last_revision()
-            if last_rev not in rev_hist:
-                raise AssertionError(
-                    "Working Tree's last revision not in branch.revision_history")
-            rev_hist = rev_hist[:rev_hist.index(last_rev)+1]
+        last_rev = self._get_revision_id()
 
         repository =  self._branch.repository
         repository.lock_read()
         try:
-            for revision_id in rev_hist:
+            graph = repository.get_graph()
+            revhistory = list(graph.iter_lefthand_ancestry(
+                last_rev, [_mod_revision.NULL_REVISION]))
+            for revision_id in reversed(revhistory):
                 rev = repository.get_revision(revision_id)
-                self._revision_history_info.append(
-                    (rev.revision_id, rev.message,
-                     rev.timestamp, rev.timezone))
+                yield (rev.revision_id, rev.message,
+                       rev.timestamp, rev.timezone)
         finally:
             repository.unlock()
 
     def _get_revision_id(self):
         """Get the revision id we are working on."""
+        if self._revision_id is not None:
+            return self._revision_id
         if self._working_tree is not None:
             return self._working_tree.last_revision()
         return self._branch.last_revision()

=== modified file 'bzrlib/version_info_formats/format_custom.py'
--- bzrlib/version_info_formats/format_custom.py	2011-06-12 20:09:55 +0000
+++ bzrlib/version_info_formats/format_custom.py	2011-12-19 13:23:58 +0000
@@ -17,6 +17,8 @@
 """A generator which creates a template-based output from the current
    tree info."""
 
+from __future__ import absolute_import
+
 from bzrlib import errors
 from bzrlib.revision import (
    NULL_REVISION,

=== modified file 'bzrlib/version_info_formats/format_python.py'
--- bzrlib/version_info_formats/format_python.py	2011-06-12 20:09:55 +0000
+++ bzrlib/version_info_formats/format_python.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,8 @@
 
 """A generator which creates a python script from the current tree info"""
 
+from __future__ import absolute_import
+
 import pprint
 
 from bzrlib.revision import (
@@ -83,8 +85,8 @@
         to_file.write('\n\n')
 
         if self._include_history:
-            self._extract_revision_history()
-            revision_str = pprint.pformat(self._revision_history_info)
+            history = list(self._iter_revision_history())
+            revision_str = pprint.pformat(history)
             to_file.write('revisions = ')
             to_file.write(revision_str)
             to_file.write('\n\n')

=== modified file 'bzrlib/version_info_formats/format_rio.py'
--- bzrlib/version_info_formats/format_rio.py	2011-06-12 20:09:55 +0000
+++ bzrlib/version_info_formats/format_rio.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,8 @@
 
 """A generator which creates a rio stanza of the current tree info"""
 
+from __future__ import absolute_import
+
 from bzrlib import hooks
 from bzrlib.revision import (
     NULL_REVISION,
@@ -60,10 +62,9 @@
                 info.add('clean', 'False')
 
         if self._include_history:
-            self._extract_revision_history()
             log = Stanza()
             for (revision_id, message,
-                 timestamp, timezone) in self._revision_history_info:
+                 timestamp, timezone) in self._iter_revision_history():
                 log.add('id', revision_id)
                 log.add('message', message)
                 log.add('date', create_date_str(timestamp, timezone))

=== modified file 'bzrlib/versionedfile.py'
--- bzrlib/versionedfile.py	2011-05-16 10:08:01 +0000
+++ bzrlib/versionedfile.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,8 @@
 
 """Versioned text file storage api."""
 
+from __future__ import absolute_import
+
 from copy import copy
 from cStringIO import StringIO
 import os
@@ -24,8 +26,6 @@
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
-import urllib
-
 from bzrlib import (
     annotate,
     bencode,
@@ -38,6 +38,7 @@
     multiparent,
     tsort,
     revision,
+    urlutils,
     )
 """)
 from bzrlib.registry import Registry
@@ -821,11 +822,11 @@
 
     def map(self, key):
         """See KeyMapper.map()."""
-        return urllib.quote(self._map(key))
+        return urlutils.quote(self._map(key))
 
     def unmap(self, partition_id):
         """See KeyMapper.unmap()."""
-        return self._unmap(urllib.unquote(partition_id))
+        return self._unmap(urlutils.unquote(partition_id))
 
 
 class PrefixMapper(URLEscapeMapper):
@@ -878,7 +879,7 @@
     def _escape(self, prefix):
         """Turn a key element into a filesystem safe string.
 
-        This is similar to a plain urllib.quote, except
+        This is similar to a plain urlutils.quote, except
         it uses specific safe characters, so that it doesn't
         have to translate a lot of valid file ids.
         """
@@ -891,7 +892,7 @@
 
     def _unescape(self, basename):
         """Escaped names are easily unescaped by urlutils."""
-        return urllib.unquote(basename)
+        return urlutils.unquote(basename)
 
 
 def make_versioned_files_factory(versioned_file_factory, mapper):

=== modified file 'bzrlib/vf_repository.py'
--- bzrlib/vf_repository.py	2011-08-25 07:51:37 +0000
+++ bzrlib/vf_repository.py	2012-01-04 16:10:04 +0000
@@ -16,6 +16,8 @@
 
 """Repository formats built around versioned files."""
 
+from __future__ import absolute_import
+
 
 from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
@@ -23,6 +25,7 @@
 
 from bzrlib import (
     check,
+    config as _mod_config,
     debug,
     fetch as _mod_fetch,
     fifo_cache,
@@ -38,11 +41,13 @@
     tsort,
     ui,
     versionedfile,
+    vf_search,
     )
 
 from bzrlib.recordcounter import RecordCounter
 from bzrlib.revisiontree import InventoryRevisionTree
 from bzrlib.testament import Testament
+from bzrlib.i18n import gettext
 """)
 
 from bzrlib import (
@@ -64,7 +69,7 @@
     CommitBuilder,
     InterRepository,
     MetaDirRepository,
-    MetaDirRepositoryFormat,
+    RepositoryFormatMetaDir,
     Repository,
     RepositoryFormat,
     )
@@ -79,6 +84,7 @@
 
     supports_full_versioned_files = True
     supports_versioned_directories = True
+    supports_unreferenced_revisions = True
 
     # Should commit add an inventory, or an inventory delta to the repository.
     _commit_inv_deltas = True
@@ -103,11 +109,11 @@
     # the default CommitBuilder does not manage trees whose root is versioned.
     _versioned_root = False
 
-    def __init__(self, repository, parents, config, timestamp=None,
+    def __init__(self, repository, parents, config_stack, timestamp=None,
                  timezone=None, committer=None, revprops=None,
                  revision_id=None, lossy=False):
         super(VersionedFileCommitBuilder, self).__init__(repository,
-            parents, config, timestamp, timezone, committer, revprops,
+            parents, config_stack, timestamp, timezone, committer, revprops,
             revision_id, lossy)
         try:
             basis_id = self.parents[0]
@@ -194,8 +200,13 @@
                        revision_id=self._new_revision_id,
                        properties=self._revprops)
         rev.parent_ids = self.parents
-        self.repository.add_revision(self._new_revision_id, rev,
-            self.new_inventory, self._config)
+        if self._config_stack.get('create_signatures') == _mod_config.SIGN_ALWAYS:
+            testament = Testament(rev, self.revision_tree())
+            plaintext = testament.as_short_text()
+            self.repository.store_revision_signature(
+                gpg.GPGStrategy(self._config_stack), plaintext,
+                self._new_revision_id)
+        self.repository._add_revision(rev)
         self._ensure_fallback_inventories()
         self.repository.commit_write_group()
         return self._new_revision_id
@@ -918,11 +929,13 @@
         """
         if not self._format.supports_external_lookups:
             raise errors.UnstackableRepositoryFormat(self._format, self.base)
+        # This can raise an exception, so should be done before we lock the
+        # fallback repository.
+        self._check_fallback_repository(repository)
         if self.is_locked():
             # This repository will call fallback.unlock() when we transition to
             # the unlocked state, so we make sure to increment the lock count
             repository.lock_read()
-        self._check_fallback_repository(repository)
         self._fallback_repositories.append(repository)
         self.texts.add_fallback_versioned_files(repository.texts)
         self.inventories.add_fallback_versioned_files(repository.inventories)
@@ -1025,28 +1038,17 @@
         self.inventories._access.flush()
         return result
 
-    def add_revision(self, revision_id, rev, inv=None, config=None):
+    def add_revision(self, revision_id, rev, inv=None):
         """Add rev to the revision store as revision_id.
 
         :param revision_id: the revision id to use.
         :param rev: The revision object.
         :param inv: The inventory for the revision. if None, it will be looked
                     up in the inventory storer
-        :param config: If None no digital signature will be created.
-                       If supplied its signature_needed method will be used
-                       to determine if a signature should be made.
         """
         # TODO: jam 20070210 Shouldn't we check rev.revision_id and
         #       rev.parent_ids?
         _mod_revision.check_not_reserved_id(revision_id)
-        if config is not None and config.signature_needed():
-            if inv is None:
-                inv = self.get_inventory(revision_id)
-            tree = InventoryRevisionTree(self, inv, revision_id)
-            testament = Testament(rev, tree)
-            plaintext = testament.as_short_text()
-            self.store_revision_signature(
-                gpg.GPGStrategy(config), plaintext, revision_id)
         # check inventory present
         if not self.inventories.get_parent_map([(revision_id,)]):
             if inv is None:
@@ -1085,7 +1087,7 @@
         keys = {'chk_bytes':set(), 'inventories':set(), 'texts':set()}
         kinds = ['chk_bytes', 'texts']
         count = len(checker.pending_keys)
-        bar.update("inventories", 0, 2)
+        bar.update(gettext("inventories"), 0, 2)
         current_keys = checker.pending_keys
         checker.pending_keys = {}
         # Accumulate current checks.
@@ -1111,7 +1113,7 @@
             del keys['inventories']
         else:
             return
-        bar.update("texts", 1)
+        bar.update(gettext("texts"), 1)
         while (checker.pending_keys or keys['chk_bytes']
             or keys['texts']):
             # Something to check.
@@ -1196,7 +1198,7 @@
         """Instantiate a VersionedFileRepository.
 
         :param _format: The format of the repository on disk.
-        :param a_bzrdir: The BzrDir of the repository.
+        :param controldir: The ControlDir of the repository.
         :param control_files: Control files to use for locking, etc.
         """
         # In the future we will have a single api for all stores for
@@ -1204,6 +1206,8 @@
         # this construct will accept instances of those things.
         super(VersionedFileRepository, self).__init__(_format, a_bzrdir,
             control_files)
+        self._transport = control_files._transport
+        self.base = self._transport.base
         # for tests
         self._reconcile_does_inventory_gc = True
         self._reconcile_fixes_text_parents = False
@@ -1214,6 +1218,54 @@
         # rather copying them?
         self._safe_to_return_from_cache = False
 
+    def fetch(self, source, revision_id=None, find_ghosts=False,
+            fetch_spec=None):
+        """Fetch the content required to construct revision_id from source.
+
+        If revision_id is None and fetch_spec is None, then all content is
+        copied.
+
+        fetch() may not be used when the repository is in a write group -
+        either finish the current write group before using fetch, or use
+        fetch before starting the write group.
+
+        :param find_ghosts: Find and copy revisions in the source that are
+            ghosts in the target (and not reachable directly by walking out to
+            the first-present revision in target from revision_id).
+        :param revision_id: If specified, all the content needed for this
+            revision ID will be copied to the target.  Fetch will determine for
+            itself which content needs to be copied.
+        :param fetch_spec: If specified, a SearchResult or
+            PendingAncestryResult that describes which revisions to copy.  This
+            allows copying multiple heads at once.  Mutually exclusive with
+            revision_id.
+        """
+        if fetch_spec is not None and revision_id is not None:
+            raise AssertionError(
+                "fetch_spec and revision_id are mutually exclusive.")
+        if self.is_in_write_group():
+            raise errors.InternalBzrError(
+                "May not fetch while in a write group.")
+        # fast path same-url fetch operations
+        # TODO: lift out to somewhere common with RemoteRepository
+        # <https://bugs.launchpad.net/bzr/+bug/401646>
+        if (self.has_same_location(source)
+            and fetch_spec is None
+            and self._has_same_fallbacks(source)):
+            # check that last_revision is in 'from' and then return a
+            # no-operation.
+            if (revision_id is not None and
+                not _mod_revision.is_null(revision_id)):
+                self.get_revision(revision_id)
+            return 0, []
+        inter = InterRepository.get(source, self)
+        if (fetch_spec is not None and
+            not getattr(inter, "supports_fetch_spec", False)):
+            raise errors.UnsupportedOperation(
+                "fetch_spec not supported for %r" % inter)
+        return inter.fetch(revision_id=revision_id,
+            find_ghosts=find_ghosts, fetch_spec=fetch_spec)
+
     @needs_read_lock
     def gather_stats(self, revid=None, committers=None):
         """See Repository.gather_stats()."""
@@ -1228,14 +1280,14 @@
             # result['size'] = t
         return result
 
-    def get_commit_builder(self, branch, parents, config, timestamp=None,
+    def get_commit_builder(self, branch, parents, config_stack, timestamp=None,
                            timezone=None, committer=None, revprops=None,
                            revision_id=None, lossy=False):
         """Obtain a CommitBuilder for this repository.
 
         :param branch: Branch to commit to.
         :param parents: Revision ids of the parents of the new revision.
-        :param config: Configuration to use.
+        :param config_stack: Configuration stack to use.
         :param timestamp: Optional timestamp recorded for commit.
         :param timezone: Optional timezone for timestamp.
         :param committer: Optional committer to set for commit.
@@ -1248,7 +1300,7 @@
             raise errors.BzrError("Cannot commit directly to a stacked branch"
                 " in pre-2a formats. See "
                 "https://bugs.launchpad.net/bzr/+bug/375013 for details.")
-        result = self._commit_builder_class(self, parents, config,
+        result = self._commit_builder_class(self, parents, config_stack,
             timestamp, timezone, committer, revprops, revision_id,
             lossy)
         self.start_write_group()
@@ -1510,7 +1562,7 @@
             text_keys[(file_id, revision_id)] = callable_data
         for record in self.texts.get_record_stream(text_keys, 'unordered', True):
             if record.storage_kind == 'absent':
-                raise errors.RevisionNotPresent(record.key, self)
+                raise errors.RevisionNotPresent(record.key[1], record.key[0])
             yield text_keys[record.key], record.get_bytes_as('chunked')
 
     def _generate_text_key_index(self, text_key_references=None,
@@ -1566,7 +1618,7 @@
         batch_size = 10 # should be ~150MB on a 55K path tree
         batch_count = len(revision_order) / batch_size + 1
         processed_texts = 0
-        pb.update("Calculating text parents", processed_texts, text_count)
+        pb.update(gettext("Calculating text parents"), processed_texts, text_count)
         for offset in xrange(batch_count):
             to_query = revision_order[offset * batch_size:(offset + 1) *
                 batch_size]
@@ -1575,7 +1627,7 @@
             for revision_id in to_query:
                 parent_ids = ancestors[revision_id]
                 for text_key in revision_keys[revision_id]:
-                    pb.update("Calculating text parents", processed_texts)
+                    pb.update(gettext("Calculating text parents"), processed_texts)
                     processed_texts += 1
                     candidate_parents = []
                     for parent_id in parent_ids:
@@ -1651,7 +1703,7 @@
         num_file_ids = len(file_ids)
         for file_id, altered_versions in file_ids.iteritems():
             if pb is not None:
-                pb.update("Fetch texts", count, num_file_ids)
+                pb.update(gettext("Fetch texts"), count, num_file_ids)
             count += 1
             yield ("file", file_id, altered_versions)
 
@@ -1694,13 +1746,19 @@
         if ((None in revision_ids)
             or (_mod_revision.NULL_REVISION in revision_ids)):
             raise ValueError('cannot get null revision inventory')
-        return self._iter_inventories(revision_ids, ordering)
+        for inv, revid in self._iter_inventories(revision_ids, ordering):
+            if inv is None:
+                raise errors.NoSuchRevision(self, revid)
+            yield inv
 
     def _iter_inventories(self, revision_ids, ordering):
         """single-document based inventory iteration."""
         inv_xmls = self._iter_inventory_xmls(revision_ids, ordering)
         for text, revision_id in inv_xmls:
-            yield self._deserialise_inventory(revision_id, text)
+            if text is None:
+                yield None, revision_id
+            else:
+                yield self._deserialise_inventory(revision_id, text), revision_id
 
     def _iter_inventory_xmls(self, revision_ids, ordering):
         if ordering is None:
@@ -1724,7 +1782,7 @@
                 else:
                     yield ''.join(chunks), record.key[-1]
             else:
-                raise errors.NoSuchRevision(self, record.key)
+                yield None, record.key[-1]
             if order_as_requested:
                 # Yield as many results as we can while preserving order.
                 while next_key in text_chunks:
@@ -1759,10 +1817,9 @@
     def _get_inventory_xml(self, revision_id):
         """Get serialized inventory as a string."""
         texts = self._iter_inventory_xmls([revision_id], 'unordered')
-        try:
-            text, revision_id = texts.next()
-        except StopIteration:
-            raise errors.HistoryMissing(self, 'inventory', revision_id)
+        text, revision_id = texts.next()
+        if text is None:
+            raise errors.NoSuchRevision(self, revision_id)
         return text
 
     @needs_read_lock
@@ -1843,6 +1900,19 @@
         """Return the graph walker for text revisions."""
         return graph.Graph(self.texts)
 
+    def revision_ids_to_search_result(self, result_set):
+        """Convert a set of revision ids to a graph SearchResult."""
+        result_parents = set()
+        for parents in self.get_graph().get_parent_map(
+            result_set).itervalues():
+            result_parents.update(parents)
+        included_keys = result_set.intersection(result_parents)
+        start_keys = result_set.difference(included_keys)
+        exclude_keys = result_parents.difference(result_set)
+        result = vf_search.SearchResult(start_keys, exclude_keys,
+            len(result_set), result_set)
+        return result
+
     def _get_versioned_file_checker(self, text_key_references=None,
         ancestors=None):
         """Return an object suitable for checking versioned files.
@@ -1933,7 +2003,7 @@
             control_files)
 
 
-class MetaDirVersionedFileRepositoryFormat(MetaDirRepositoryFormat,
+class MetaDirVersionedFileRepositoryFormat(RepositoryFormatMetaDir,
         VersionedFileRepositoryFormat):
     """Base class for repository formats using versioned files in metadirs."""
 
@@ -2462,13 +2532,13 @@
             self.text_index.iterkeys()])
         # text keys is now grouped by file_id
         n_versions = len(self.text_index)
-        progress_bar.update('loading text store', 0, n_versions)
+        progress_bar.update(gettext('loading text store'), 0, n_versions)
         parent_map = self.repository.texts.get_parent_map(self.text_index)
         # On unlistable transports this could well be empty/error...
         text_keys = self.repository.texts.keys()
         unused_keys = frozenset(text_keys) - set(self.text_index)
         for num, key in enumerate(self.text_index.iterkeys()):
-            progress_bar.update('checking text graph', num, n_versions)
+            progress_bar.update(gettext('checking text graph'), num, n_versions)
             correct_parents = self.calculate_file_version_parents(key)
             try:
                 knit_parents = parent_map[key]
@@ -2484,6 +2554,8 @@
 
     _walk_to_common_revisions_batch_size = 50
 
+    supports_fetch_spec = True
+
     @needs_write_lock
     def fetch(self, revision_id=None, find_ghosts=False,
             fetch_spec=None):
@@ -2565,7 +2637,9 @@
                 searcher.stop_searching_any(stop_revs)
             if searcher_exhausted:
                 break
-        return searcher.get_result()
+        (started_keys, excludes, included_keys) = searcher.get_state()
+        return vf_search.SearchResult(started_keys, excludes,
+            len(included_keys), included_keys)
 
     @needs_read_lock
     def search_missing_revision_ids(self,
@@ -2918,7 +2992,7 @@
         for offset in range(0, len(revision_ids), batch_size):
             self.target.start_write_group()
             try:
-                pb.update('Transferring revisions', offset,
+                pb.update(gettext('Transferring revisions'), offset,
                           len(revision_ids))
                 batch = revision_ids[offset:offset+batch_size]
                 basis_id = self._fetch_batch(batch, basis_id, cache)
@@ -2932,7 +3006,7 @@
                     hints.extend(hint)
         if hints and self.target._format.pack_compresses:
             self.target.pack(hint=hints)
-        pb.update('Transferring revisions', len(revision_ids),
+        pb.update(gettext('Transferring revisions'), len(revision_ids),
                   len(revision_ids))
 
     @needs_write_lock
@@ -3047,7 +3121,7 @@
             _install_revision(repository, revision, revision_tree, signature,
                 inventory_cache)
             if pb is not None:
-                pb.update('Transferring revisions', n + 1, num_revisions)
+                pb.update(gettext('Transferring revisions'), n + 1, num_revisions)
     except:
         repository.abort_write_group()
         raise

=== added file 'bzrlib/vf_search.py'
--- bzrlib/vf_search.py	1970-01-01 00:00:00 +0000
+++ bzrlib/vf_search.py	2011-12-19 13:23:58 +0000
@@ -0,0 +1,511 @@
+# Copyright (C) 2007-2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Searching in versioned file repositories."""
+
+from __future__ import absolute_import
+
+from bzrlib import (
+    debug,
+    revision,
+    trace,
+    )
+
+from bzrlib.graph import (
+    DictParentsProvider,
+    Graph,
+    invert_parent_map,
+    )
+
+
+class AbstractSearchResult(object):
+    """The result of a search, describing a set of keys.
+    
+    Search results are typically used as the 'fetch_spec' parameter when
+    fetching revisions.
+
+    :seealso: AbstractSearch
+    """
+
+    def get_recipe(self):
+        """Return a recipe that can be used to replay this search.
+
+        The recipe allows reconstruction of the same results at a later date.
+
+        :return: A tuple of `(search_kind_str, *details)`.  The details vary by
+            kind of search result.
+        """
+        raise NotImplementedError(self.get_recipe)
+
+    def get_network_struct(self):
+        """Return a tuple that can be transmitted via the HPSS protocol."""
+        raise NotImplementedError(self.get_network_struct)
+
+    def get_keys(self):
+        """Return the keys found in this search.
+
+        :return: A set of keys.
+        """
+        raise NotImplementedError(self.get_keys)
+
+    def is_empty(self):
+        """Return false if the search lists 1 or more revisions."""
+        raise NotImplementedError(self.is_empty)
+
+    def refine(self, seen, referenced):
+        """Create a new search by refining this search.
+
+        :param seen: Revisions that have been satisfied.
+        :param referenced: Revision references observed while satisfying some
+            of this search.
+        :return: A search result.
+        """
+        raise NotImplementedError(self.refine)
+
+
+class AbstractSearch(object):
+    """A search that can be executed, producing a search result.
+
+    :seealso: AbstractSearchResult
+    """
+
+    def execute(self):
+        """Construct a network-ready search result from this search description.
+
+        This may take some time to search repositories, etc.
+
+        :return: A search result (an object that implements
+            AbstractSearchResult's API).
+        """
+        raise NotImplementedError(self.execute)
+
+
+class SearchResult(AbstractSearchResult):
+    """The result of a breadth first search.
+
+    A SearchResult provides the ability to reconstruct the search or access a
+    set of the keys the search found.
+    """
+
+    def __init__(self, start_keys, exclude_keys, key_count, keys):
+        """Create a SearchResult.
+
+        :param start_keys: The keys the search started at.
+        :param exclude_keys: The keys the search excludes.
+        :param key_count: The total number of keys (from start to but not
+            including exclude).
+        :param keys: The keys the search found. Note that in future we may get
+            a SearchResult from a smart server, in which case the keys list is
+            not necessarily immediately available.
+        """
+        self._recipe = ('search', start_keys, exclude_keys, key_count)
+        self._keys = frozenset(keys)
+
+    def __repr__(self):
+        kind, start_keys, exclude_keys, key_count = self._recipe
+        if len(start_keys) > 5:
+            start_keys_repr = repr(list(start_keys)[:5])[:-1] + ', ...]'
+        else:
+            start_keys_repr = repr(start_keys)
+        if len(exclude_keys) > 5:
+            exclude_keys_repr = repr(list(exclude_keys)[:5])[:-1] + ', ...]'
+        else:
+            exclude_keys_repr = repr(exclude_keys)
+        return '<%s %s:(%s, %s, %d)>' % (self.__class__.__name__,
+            kind, start_keys_repr, exclude_keys_repr, key_count)
+
+    def get_recipe(self):
+        """Return a recipe that can be used to replay this search.
+
+        The recipe allows reconstruction of the same results at a later date
+        without knowing all the found keys. The essential elements are a list
+        of keys to start and to stop at. In order to give reproducible
+        results when ghosts are encountered by a search they are automatically
+        added to the exclude list (or else ghost filling may alter the
+        results).
+
+        :return: A tuple ('search', start_keys_set, exclude_keys_set,
+            revision_count). To recreate the results of this search, create a
+            breadth first searcher on the same graph starting at start_keys.
+            Then call next() (or next_with_ghosts()) repeatedly, and on every
+            result, call stop_searching_any on any keys from the exclude_keys
+            set. The revision_count value acts as a trivial cross-check - the
+            found revisions of the new search should have as many elements as
+            revision_count. If it does not, then additional revisions have been
+            ghosted since the search was executed the first time and the second
+            time.
+        """
+        return self._recipe
+
+    def get_network_struct(self):
+        start_keys = ' '.join(self._recipe[1])
+        stop_keys = ' '.join(self._recipe[2])
+        count = str(self._recipe[3])
+        return (self._recipe[0], '\n'.join((start_keys, stop_keys, count)))
+
+    def get_keys(self):
+        """Return the keys found in this search.
+
+        :return: A set of keys.
+        """
+        return self._keys
+
+    def is_empty(self):
+        """Return false if the search lists 1 or more revisions."""
+        return self._recipe[3] == 0
+
+    def refine(self, seen, referenced):
+        """Create a new search by refining this search.
+
+        :param seen: Revisions that have been satisfied.
+        :param referenced: Revision references observed while satisfying some
+            of this search.
+        """
+        start = self._recipe[1]
+        exclude = self._recipe[2]
+        count = self._recipe[3]
+        keys = self.get_keys()
+        # New heads = referenced + old heads - seen things - exclude
+        pending_refs = set(referenced)
+        pending_refs.update(start)
+        pending_refs.difference_update(seen)
+        pending_refs.difference_update(exclude)
+        # New exclude = old exclude + satisfied heads
+        seen_heads = start.intersection(seen)
+        exclude.update(seen_heads)
+        # keys gets seen removed
+        keys = keys - seen
+        # length is reduced by len(seen)
+        count -= len(seen)
+        return SearchResult(pending_refs, exclude, count, keys)
+
+
+class PendingAncestryResult(AbstractSearchResult):
+    """A search result that will reconstruct the ancestry for some graph heads.
+
+    Unlike SearchResult, this doesn't hold the complete search result in
+    memory, it just holds a description of how to generate it.
+    """
+
+    def __init__(self, heads, repo):
+        """Constructor.
+
+        :param heads: an iterable of graph heads.
+        :param repo: a repository to use to generate the ancestry for the given
+            heads.
+        """
+        self.heads = frozenset(heads)
+        self.repo = repo
+
+    def __repr__(self):
+        if len(self.heads) > 5:
+            heads_repr = repr(list(self.heads)[:5])[:-1]
+            heads_repr += ', <%d more>...]' % (len(self.heads) - 5,)
+        else:
+            heads_repr = repr(self.heads)
+        return '<%s heads:%s repo:%r>' % (
+            self.__class__.__name__, heads_repr, self.repo)
+
+    def get_recipe(self):
+        """Return a recipe that can be used to replay this search.
+
+        The recipe allows reconstruction of the same results at a later date.
+
+        :seealso SearchResult.get_recipe:
+
+        :return: A tuple ('proxy-search', start_keys_set, set(), -1)
+            To recreate this result, create a PendingAncestryResult with the
+            start_keys_set.
+        """
+        return ('proxy-search', self.heads, set(), -1)
+
+    def get_network_struct(self):
+        parts = ['ancestry-of']
+        parts.extend(self.heads)
+        return parts
+
+    def get_keys(self):
+        """See SearchResult.get_keys.
+
+        Returns all the keys for the ancestry of the heads, excluding
+        NULL_REVISION.
+        """
+        return self._get_keys(self.repo.get_graph())
+
+    def _get_keys(self, graph):
+        NULL_REVISION = revision.NULL_REVISION
+        keys = [key for (key, parents) in graph.iter_ancestry(self.heads)
+                if key != NULL_REVISION and parents is not None]
+        return keys
+
+    def is_empty(self):
+        """Return false if the search lists 1 or more revisions."""
+        if revision.NULL_REVISION in self.heads:
+            return len(self.heads) == 1
+        else:
+            return len(self.heads) == 0
+
+    def refine(self, seen, referenced):
+        """Create a new search by refining this search.
+
+        :param seen: Revisions that have been satisfied.
+        :param referenced: Revision references observed while satisfying some
+            of this search.
+        """
+        referenced = self.heads.union(referenced)
+        return PendingAncestryResult(referenced - seen, self.repo)
+
+
+class EmptySearchResult(AbstractSearchResult):
+    """An empty search result."""
+
+    def is_empty(self):
+        return True
+
+
+class EverythingResult(AbstractSearchResult):
+    """A search result that simply requests everything in the repository."""
+
+    def __init__(self, repo):
+        self._repo = repo
+
+    def __repr__(self):
+        return '%s(%r)' % (self.__class__.__name__, self._repo)
+
+    def get_recipe(self):
+        raise NotImplementedError(self.get_recipe)
+
+    def get_network_struct(self):
+        return ('everything',)
+
+    def get_keys(self):
+        if 'evil' in debug.debug_flags:
+            from bzrlib import remote
+            if isinstance(self._repo, remote.RemoteRepository):
+                # warn developers (not users) not to do this
+                trace.mutter_callsite(
+                    2, "EverythingResult(RemoteRepository).get_keys() is slow.")
+        return self._repo.all_revision_ids()
+
+    def is_empty(self):
+        # It's ok for this to wrongly return False: the worst that can happen
+        # is that RemoteStreamSource will initiate a get_stream on an empty
+        # repository.  And almost all repositories are non-empty.
+        return False
+
+    def refine(self, seen, referenced):
+        heads = set(self._repo.all_revision_ids())
+        heads.difference_update(seen)
+        heads.update(referenced)
+        return PendingAncestryResult(heads, self._repo)
+
+
+class EverythingNotInOther(AbstractSearch):
+    """Find all revisions in that are in one repo but not the other."""
+
+    def __init__(self, to_repo, from_repo, find_ghosts=False):
+        self.to_repo = to_repo
+        self.from_repo = from_repo
+        self.find_ghosts = find_ghosts
+
+    def execute(self):
+        return self.to_repo.search_missing_revision_ids(
+            self.from_repo, find_ghosts=self.find_ghosts)
+
+
+class NotInOtherForRevs(AbstractSearch):
+    """Find all revisions missing in one repo for a some specific heads."""
+
+    def __init__(self, to_repo, from_repo, required_ids, if_present_ids=None,
+            find_ghosts=False, limit=None):
+        """Constructor.
+
+        :param required_ids: revision IDs of heads that must be found, or else
+            the search will fail with NoSuchRevision.  All revisions in their
+            ancestry not already in the other repository will be included in
+            the search result.
+        :param if_present_ids: revision IDs of heads that may be absent in the
+            source repository.  If present, then their ancestry not already
+            found in other will be included in the search result.
+        :param limit: maximum number of revisions to fetch
+        """
+        self.to_repo = to_repo
+        self.from_repo = from_repo
+        self.find_ghosts = find_ghosts
+        self.required_ids = required_ids
+        self.if_present_ids = if_present_ids
+        self.limit = limit
+
+    def __repr__(self):
+        if len(self.required_ids) > 5:
+            reqd_revs_repr = repr(list(self.required_ids)[:5])[:-1] + ', ...]'
+        else:
+            reqd_revs_repr = repr(self.required_ids)
+        if self.if_present_ids and len(self.if_present_ids) > 5:
+            ifp_revs_repr = repr(list(self.if_present_ids)[:5])[:-1] + ', ...]'
+        else:
+            ifp_revs_repr = repr(self.if_present_ids)
+
+        return ("<%s from:%r to:%r find_ghosts:%r req'd:%r if-present:%r"
+                "limit:%r>") % (
+                self.__class__.__name__, self.from_repo, self.to_repo,
+                self.find_ghosts, reqd_revs_repr, ifp_revs_repr,
+                self.limit)
+
+    def execute(self):
+        return self.to_repo.search_missing_revision_ids(
+            self.from_repo, revision_ids=self.required_ids,
+            if_present_ids=self.if_present_ids, find_ghosts=self.find_ghosts,
+            limit=self.limit)
+
+
+def search_result_from_parent_map(parent_map, missing_keys):
+    """Transform a parent_map into SearchResult information."""
+    if not parent_map:
+        # parent_map is empty or None, simple search result
+        return [], [], 0
+    # start_set is all the keys in the cache
+    start_set = set(parent_map)
+    # result set is all the references to keys in the cache
+    result_parents = set()
+    for parents in parent_map.itervalues():
+        result_parents.update(parents)
+    stop_keys = result_parents.difference(start_set)
+    # We don't need to send ghosts back to the server as a position to
+    # stop either.
+    stop_keys.difference_update(missing_keys)
+    key_count = len(parent_map)
+    if (revision.NULL_REVISION in result_parents
+        and revision.NULL_REVISION in missing_keys):
+        # If we pruned NULL_REVISION from the stop_keys because it's also
+        # in our cache of "missing" keys we need to increment our key count
+        # by 1, because the reconsitituted SearchResult on the server will
+        # still consider NULL_REVISION to be an included key.
+        key_count += 1
+    included_keys = start_set.intersection(result_parents)
+    start_set.difference_update(included_keys)
+    return start_set, stop_keys, key_count
+
+
+def _run_search(parent_map, heads, exclude_keys):
+    """Given a parent map, run a _BreadthFirstSearcher on it.
+
+    Start at heads, walk until you hit exclude_keys. As a further improvement,
+    watch for any heads that you encounter while walking, which means they were
+    not heads of the search.
+
+    This is mostly used to generate a succinct recipe for how to walk through
+    most of parent_map.
+
+    :return: (_BreadthFirstSearcher, set(heads_encountered_by_walking))
+    """
+    g = Graph(DictParentsProvider(parent_map))
+    s = g._make_breadth_first_searcher(heads)
+    found_heads = set()
+    while True:
+        try:
+            next_revs = s.next()
+        except StopIteration:
+            break
+        for parents in s._current_parents.itervalues():
+            f_heads = heads.intersection(parents)
+            if f_heads:
+                found_heads.update(f_heads)
+        stop_keys = exclude_keys.intersection(next_revs)
+        if stop_keys:
+            s.stop_searching_any(stop_keys)
+    for parents in s._current_parents.itervalues():
+        f_heads = heads.intersection(parents)
+        if f_heads:
+            found_heads.update(f_heads)
+    return s, found_heads
+
+
+def _find_possible_heads(parent_map, tip_keys, depth):
+    """Walk backwards (towards children) through the parent_map.
+
+    This finds 'heads' that will hopefully succinctly describe our search
+    graph.
+    """
+    child_map = invert_parent_map(parent_map)
+    heads = set()
+    current_roots = tip_keys
+    walked = set(current_roots)
+    while current_roots and depth > 0:
+        depth -= 1
+        children = set()
+        children_update = children.update
+        for p in current_roots:
+            # Is it better to pre- or post- filter the children?
+            try:
+                children_update(child_map[p])
+            except KeyError:
+                heads.add(p)
+        # If we've seen a key before, we don't want to walk it again. Note that
+        # 'children' stays relatively small while 'walked' grows large. So
+        # don't use 'difference_update' here which has to walk all of 'walked'.
+        # '.difference' is smart enough to walk only children and compare it to
+        # walked.
+        children = children.difference(walked)
+        walked.update(children)
+        current_roots = children
+    if current_roots:
+        # We walked to the end of depth, so these are the new tips.
+        heads.update(current_roots)
+    return heads
+
+
+def limited_search_result_from_parent_map(parent_map, missing_keys, tip_keys,
+                                          depth):
+    """Transform a parent_map that is searching 'tip_keys' into an
+    approximate SearchResult.
+
+    We should be able to generate a SearchResult from a given set of starting
+    keys, that covers a subset of parent_map that has the last step pointing at
+    tip_keys. This is to handle the case that really-long-searches shouldn't be
+    started from scratch on each get_parent_map request, but we *do* want to
+    filter out some of the keys that we've already seen, so we don't get
+    information that we already know about on every request.
+
+    The server will validate the search (that starting at start_keys and
+    stopping at stop_keys yields the exact key_count), so we have to be careful
+    to give an exact recipe.
+
+    Basic algorithm is:
+        1) Invert parent_map to get child_map (todo: have it cached and pass it
+           in)
+        2) Starting at tip_keys, walk towards children for 'depth' steps.
+        3) At that point, we have the 'start' keys.
+        4) Start walking parent_map from 'start' keys, counting how many keys
+           are seen, and generating stop_keys for anything that would walk
+           outside of the parent_map.
+
+    :param parent_map: A map from {child_id: (parent_ids,)}
+    :param missing_keys: parent_ids that we know are unavailable
+    :param tip_keys: the revision_ids that we are searching
+    :param depth: How far back to walk.
+    """
+    if not parent_map:
+        # No search to send, because we haven't done any searching yet.
+        return [], [], 0
+    heads = _find_possible_heads(parent_map, tip_keys, depth)
+    s, found_heads = _run_search(parent_map, heads, set(tip_keys))
+    start_keys, exclude_keys, keys = s.get_state()
+    if found_heads:
+        # Anything in found_heads are redundant start_keys, we hit them while
+        # walking, so we can exclude them from the start list.
+        start_keys = set(start_keys).difference(found_heads)
+    return start_keys, exclude_keys, len(keys)

=== modified file 'bzrlib/views.py'
--- bzrlib/views.py	2009-03-23 14:59:43 +0000
+++ bzrlib/views.py	2011-12-19 13:23:58 +0000
@@ -22,6 +22,7 @@
   tree.views.lookup_view()
 """
 
+from __future__ import absolute_import
 
 import re
 

=== modified file 'bzrlib/weave.py'
--- bzrlib/weave.py	2011-02-05 14:24:01 +0000
+++ bzrlib/weave.py	2011-12-19 13:23:58 +0000
@@ -16,9 +16,9 @@
 
 # Author: Martin Pool <mbp@canonical.com>
 
-
 """Weave - storage of related text file versions"""
 
+from __future__ import absolute_import
 
 # XXX: If we do weaves this way, will a merge still behave the same
 # way if it's done in a different order?  That's a pretty desirable

=== modified file 'bzrlib/weavefile.py'
--- bzrlib/weavefile.py	2011-02-05 14:24:01 +0000
+++ bzrlib/weavefile.py	2011-12-19 14:14:48 +0000
@@ -16,9 +16,6 @@
 #
 # Author: Martin Pool <mbp@canonical.com>
 
-
-
-
 """Store and retrieve weaves in files.
 
 There is one format marker followed by a blank line, followed by a
@@ -37,6 +34,8 @@
 line contains a newline, or ',' if not.
 """
 
+from __future__ import absolute_import
+
 # TODO: When extracting a single version it'd be enough to just pass
 # an iterator returning the weave lines...  We don't really need to
 # deserialize it into memory.
@@ -90,8 +89,7 @@
 
 def read_weave(f):
     # FIXME: detect the weave type and dispatch
-    from bzrlib.trace import mutter
-    from weave import Weave
+    from bzrlib.weave import Weave
     w = Weave(getattr(f, 'name', None))
     _read_weave_v5(f, w)
     return w
@@ -116,7 +114,7 @@
     # +59363 0    311.8780    311.8780   +<method 'append' of 'list' objects>
     # +200   0     30.2500     30.2500   +<method 'readlines' of 'file' objects>
 
-    from weave import WeaveFormatError
+    from bzrlib.weave import WeaveFormatError
 
     try:
         lines = iter(f.readlines())

=== modified file 'bzrlib/win32utils.py'
--- bzrlib/win32utils.py	2011-06-11 01:11:43 +0000
+++ bzrlib/win32utils.py	2011-12-21 14:17:33 +0000
@@ -19,12 +19,19 @@
 Only one dependency: ctypes should be installed.
 """
 
+from __future__ import absolute_import
+
 import glob
+import operator
 import os
 import struct
 import sys
 
-from bzrlib import cmdline
+from bzrlib import (
+    cmdline,
+    symbol_versioning,
+    )
+from bzrlib.i18n import gettext
 
 # Windows version
 if sys.platform == 'win32':
@@ -62,21 +69,29 @@
 else:
     if winver == 'Windows 98':
         create_buffer = ctypes.create_string_buffer
+        def extract_buffer(buf):
+            return buf.value.decode("mbcs")
         suffix = 'A'
     else:
         create_buffer = ctypes.create_unicode_buffer
+        extract_buffer = operator.attrgetter("value")
         suffix = 'W'
 try:
-    import win32file
     import pywintypes
-    has_win32file = True
-except ImportError:
-    has_win32file = False
-try:
-    import win32api
-    has_win32api = True
-except ImportError:
-    has_win32api = False
+    has_pywintypes = True
+except ImportError:
+    has_pywintypes = has_win32file = has_win32api = False
+else:
+    try:
+        import win32file
+        has_win32file = True
+    except ImportError:
+        has_win32file = False
+    try:
+        import win32api
+        has_win32api = True
+    except ImportError:
+        has_win32api = False
 
 # pulling in win32com.shell is a bit of overhead, and normally we don't need
 # it as ctypes is preferred and common.  lazy_imports and "optional"
@@ -128,7 +143,7 @@
             ctypes.byref(mem_struct),
             ctypes.sizeof(mem_struct))
         if not ret:
-            trace.note('Failed to GetProcessMemoryInfo()')
+            trace.note(gettext('Failed to GetProcessMemoryInfo()'))
             return
         info = {'PageFaultCount': mem_struct.PageFaultCount,
                 'PeakWorkingSetSize': mem_struct.PeakWorkingSetSize,
@@ -149,26 +164,26 @@
         proc = win32process.GetCurrentProcess()
         info = win32process.GetProcessMemoryInfo(proc)
     else:
-        trace.note('Cannot debug memory on win32 without ctypes'
-                   ' or win32process')
+        trace.note(gettext('Cannot debug memory on win32 without ctypes'
+                   ' or win32process'))
         return
     if short:
         # using base-2 units (see HACKING.txt).
-        trace.note('WorkingSize %7dKiB'
-                   '\tPeakWorking %7dKiB\t%s',
+        trace.note(gettext('WorkingSize {0:>7}KiB'
+                   '\tPeakWorking {1:>7}KiB\t{2}').format(
                    info['WorkingSetSize'] / 1024,
                    info['PeakWorkingSetSize'] / 1024,
-                   message)
+                   message))
         return
     if message:
         trace.note('%s', message)
-    trace.note('WorkingSize       %8d KiB', info['WorkingSetSize'] / 1024)
-    trace.note('PeakWorking       %8d KiB', info['PeakWorkingSetSize'] / 1024)
-    trace.note('PagefileUsage     %8d KiB', info.get('PagefileUsage', 0) / 1024)
-    trace.note('PeakPagefileUsage %8d KiB',
+    trace.note(gettext('WorkingSize       %8d KiB'), info['WorkingSetSize'] / 1024)
+    trace.note(gettext('PeakWorking       %8d KiB'), info['PeakWorkingSetSize'] / 1024)
+    trace.note(gettext('PagefileUsage     %8d KiB'), info.get('PagefileUsage', 0) / 1024)
+    trace.note(gettext('PeakPagefileUsage %8d KiB'),
                info.get('PeakPagefileUsage', 0) / 1024)
-    trace.note('PrivateUsage      %8d KiB', info.get('PrivateUsage', 0) / 1024)
-    trace.note('PageFaultCount    %8d', info.get('PageFaultCount', 0))
+    trace.note(gettext('PrivateUsage      %8d KiB'), info.get('PrivateUsage', 0) / 1024)
+    trace.note(gettext('PageFaultCount    %8d'), info.get('PageFaultCount', 0))
 
 
 def get_console_size(defaultx=80, defaulty=25):
@@ -242,28 +257,12 @@
     one that moves with the user as they logon to different machines, and
     a 'local' one that stays local to the machine.  This returns the 'roaming'
     directory, and thus is suitable for storing user-preferences, etc.
-
-    Returned value can be unicode or plain string.
-    To convert plain string to unicode use
-    s.decode(osutils.get_user_encoding())
-    (XXX - but see bug 262874, which asserts the correct encoding is 'mbcs')
     """
     appdata = _get_sh_special_folder_path(CSIDL_APPDATA)
     if appdata:
         return appdata
-    # from env variable
-    appdata = os.environ.get('APPDATA')
-    if appdata:
-        return appdata
-    # if we fall to this point we on win98
-    # at least try C:/WINDOWS/Application Data
-    windir = os.environ.get('windir')
-    if windir:
-        appdata = os.path.join(windir, 'Application Data')
-        if os.path.isdir(appdata):
-            return appdata
-    # did not find anything
-    return None
+    # Use APPDATA if defined, will return None if not
+    return get_environ_unicode('APPDATA')
 
 
 def get_local_appdata_location():
@@ -275,17 +274,12 @@
     a 'local' one that stays local to the machine.  This returns the 'local'
     directory, and thus is suitable for caches, temp files and other things
     which don't need to move with the user.
-
-    Returned value can be unicode or plain string.
-    To convert plain string to unicode use
-    s.decode(osutils.get_user_encoding())
-    (XXX - but see bug 262874, which asserts the correct encoding is 'mbcs')
     """
     local = _get_sh_special_folder_path(CSIDL_LOCAL_APPDATA)
     if local:
         return local
     # Vista supplies LOCALAPPDATA, but XP and earlier do not.
-    local = os.environ.get('LOCALAPPDATA')
+    local = get_environ_unicode('LOCALAPPDATA')
     if local:
         return local
     return get_appdata_location()
@@ -296,33 +290,27 @@
     Assume on win32 it's the <My Documents> folder.
     If location cannot be obtained return system drive root,
     i.e. C:\
-
-    Returned value can be unicode or plain string.
-    To convert plain string to unicode use
-    s.decode(osutils.get_user_encoding())
     """
     home = _get_sh_special_folder_path(CSIDL_PERSONAL)
     if home:
         return home
-    # try for HOME env variable
-    home = os.path.expanduser('~')
-    if home != '~':
+    home = get_environ_unicode('HOME')
+    if home is not None:
         return home
+    homepath = get_environ_unicode('HOMEPATH')
+    if homepath is not None:
+        return os.path.join(get_environ_unicode('HOMEDIR', ''), home)
     # at least return windows root directory
-    windir = os.environ.get('windir')
+    windir = get_environ_unicode('WINDIR')
     if windir:
         return os.path.splitdrive(windir)[0] + '/'
     # otherwise C:\ is good enough for 98% users
-    return 'C:/'
+    return unicode('C:/')
 
 
 def get_user_name():
     """Return user name as login name.
     If name cannot be obtained return None.
-
-    Returned value can be unicode or plain string.
-    To convert plain string to unicode use
-    s.decode(osutils.get_user_encoding())
     """
     if has_ctypes:
         try:
@@ -334,9 +322,9 @@
             buf = create_buffer(UNLEN+1)
             n = ctypes.c_int(UNLEN+1)
             if GetUserName(buf, ctypes.byref(n)):
-                return buf.value
+                return extract_buffer(buf)
     # otherwise try env variables
-    return os.environ.get('USERNAME', None)
+    return get_environ_unicode('USERNAME')
 
 
 # 1 == ComputerNameDnsHostname, which returns "The DNS host name of the local
@@ -347,8 +335,7 @@
     """Return host machine name.
     If name cannot be obtained return None.
 
-    :return: A unicode string representing the host name. On win98, this may be
-        a plain string as win32 api doesn't support unicode.
+    :return: A unicode string representing the host name.
     """
     if has_win32api:
         try:
@@ -371,7 +358,7 @@
             if (GetComputerNameEx is not None
                 and GetComputerNameEx(_WIN32_ComputerNameDnsHostname,
                                       buf, ctypes.byref(n))):
-                return buf.value
+                return extract_buffer(buf)
 
             # Try GetComputerName in case GetComputerNameEx wasn't found
             # It returns the NETBIOS name, which isn't as good, but still ok.
@@ -381,18 +368,12 @@
                                       None)
             if (GetComputerName is not None
                 and GetComputerName(buf, ctypes.byref(n))):
-                return buf.value
-    # otherwise try env variables, which will be 'mbcs' encoded
-    # on Windows (Python doesn't expose the native win32 unicode environment)
-    # According to this:
-    # http://msdn.microsoft.com/en-us/library/aa246807.aspx
-    # environment variables should always be encoded in 'mbcs'.
-    try:
-        return os.environ['COMPUTERNAME'].decode("mbcs")
-    except KeyError:
-        return None
-
-
+                return extract_buffer(buf)
+    return get_environ_unicode('COMPUTERNAME')
+
+
+@symbol_versioning.deprecated_function(
+    symbol_versioning.deprecated_in((2, 5, 0)))
 def _ensure_unicode(s):
     if s and type(s) != unicode:
         from bzrlib import osutils
@@ -400,17 +381,17 @@
     return s
 
 
-def get_appdata_location_unicode():
-    return _ensure_unicode(get_appdata_location())
-
-def get_home_location_unicode():
-    return _ensure_unicode(get_home_location())
-
-def get_user_name_unicode():
-    return _ensure_unicode(get_user_name())
-
-def get_host_name_unicode():
-    return _ensure_unicode(get_host_name())
+get_appdata_location_unicode = symbol_versioning.deprecated_function(
+    symbol_versioning.deprecated_in((2, 5, 0)))(get_appdata_location)
+
+get_home_location_unicode = symbol_versioning.deprecated_function(
+    symbol_versioning.deprecated_in((2, 5, 0)))(get_home_location)
+
+get_user_name_unicode = symbol_versioning.deprecated_function(
+    symbol_versioning.deprecated_in((2, 5, 0)))(get_user_name)
+
+get_host_name_unicode = symbol_versioning.deprecated_function(
+    symbol_versioning.deprecated_in((2, 5, 0)))(get_host_name)
 
 
 def _ensure_with_dir(path):
@@ -427,7 +408,6 @@
         return path
 
 
-
 def glob_one(possible_glob):
     """Same as glob.glob().
 
@@ -563,7 +543,7 @@
     return args
 
 
-if has_ctypes and winver != 'Windows 98':
+if has_ctypes and winver == 'Windows NT':
     def get_unicode_argv():
         prototype = ctypes.WINFUNCTYPE(ctypes.c_wchar_p)
         GetCommandLineW = prototype(("GetCommandLineW",
@@ -574,8 +554,51 @@
         # Skip the first argument, since we only care about parameters
         argv = _command_line_to_argv(command_line, sys.argv)[1:]
         return argv
+    
+
+    def get_environ_unicode(key, default=None):
+        """Get `key` from environment as unicode or `default` if unset
+
+        The environment is natively unicode on modern windows versions but
+        Python 2 only accesses it through the legacy bytestring api.
+
+        Environmental variable names are case insenstive on Windows.
+
+        A large enough buffer will be allocated to retrieve the value, though
+        it may take two calls to the underlying library function.
+
+        This needs ctypes because pywin32 does not expose the wide version.
+        """
+        cfunc = getattr(get_environ_unicode, "_c_function", None)
+        if cfunc is None:
+            from ctypes.wintypes import DWORD, LPCWSTR, LPWSTR
+            cfunc = ctypes.WINFUNCTYPE(DWORD, LPCWSTR, LPWSTR, DWORD)(
+                ("GetEnvironmentVariableW", ctypes.windll.kernel32))
+            get_environ_unicode._c_function = cfunc
+        buffer_size = 256 # heuristic, 256 characters often enough
+        while True:
+            buffer = ctypes.create_unicode_buffer(buffer_size)
+            length = cfunc(key, buffer, buffer_size)
+            if not length:
+                code = ctypes.GetLastError()
+                if code == 203: # ERROR_ENVVAR_NOT_FOUND
+                    return default
+                raise ctypes.WinError(code)
+            if buffer_size > length:
+                return buffer[:length]
+            buffer_size = length
 else:
     get_unicode_argv = None
+    def get_environ_unicode(key, default=None):
+        """Get `key` from environment as unicode or `default` if unset
+
+        Fallback version that should basically never be needed.
+        """
+        from bzrlib import osutils
+        try:
+            return os.environ[key].decode(osutils.get_user_encoding())
+        except KeyError:
+            return default
 
 
 if has_win32api:
@@ -614,3 +637,10 @@
         _CloseHandle(handle)
         return False
     is_local_pid_dead = _ctypes_is_local_pid_dead
+
+
+def _is_pywintypes_error(evalue):
+    """True if exception instance is an error from pywin32"""
+    if has_pywintypes and isinstance(evalue, pywintypes.error):
+        return True
+    return False

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2011-08-04 13:30:30 +0000
+++ bzrlib/workingtree.py	2012-01-03 13:47:01 +0000
@@ -29,6 +29,7 @@
 WorkingTree.open(dir).
 """
 
+from __future__ import absolute_import
 
 from cStringIO import StringIO
 import os
@@ -46,7 +47,6 @@
 
 from bzrlib import (
     branch,
-    bzrdir,
     conflicts as _mod_conflicts,
     controldir,
     errors,
@@ -54,7 +54,6 @@
     generate_ids,
     globbing,
     graph as _mod_graph,
-    hashcache,
     ignores,
     inventory,
     merge,
@@ -70,8 +69,15 @@
     )
 """)
 
-from bzrlib import symbol_versioning
+# Explicitly import bzrlib.bzrdir so that the BzrProber
+# is guaranteed to be registered.
+from bzrlib import (
+    bzrdir,
+    symbol_versioning,
+    )
+
 from bzrlib.decorators import needs_read_lock, needs_write_lock
+from bzrlib.i18n import gettext
 from bzrlib.lock import LogicalLockResult
 import bzrlib.mutabletree
 from bzrlib.mutabletree import needs_tree_write_lock
@@ -84,7 +90,6 @@
     realpath,
     safe_unicode,
     splitpath,
-    supports_executable,
     )
 from bzrlib.trace import mutter, note
 from bzrlib.revision import CURRENT_REVISION
@@ -172,8 +177,8 @@
 
     def __init__(self, basedir='.',
                  branch=DEPRECATED_PARAMETER,
-                 _control_files=None,
                  _internal=False,
+                 _transport=None,
                  _format=None,
                  _bzrdir=None):
         """Construct a WorkingTree instance. This is not a public API.
@@ -192,29 +197,7 @@
         else:
             self._branch = self.bzrdir.open_branch()
         self.basedir = realpath(basedir)
-        self._control_files = _control_files
-        self._transport = self._control_files._transport
-        # update the whole cache up front and write to disk if anything changed;
-        # in the future we might want to do this more selectively
-        # two possible ways offer themselves : in self._unlock, write the cache
-        # if needed, or, when the cache sees a change, append it to the hash
-        # cache file, and have the parser take the most recent entry for a
-        # given path only.
-        wt_trans = self.bzrdir.get_workingtree_transport(None)
-        cache_filename = wt_trans.local_abspath('stat-cache')
-        self._hashcache = hashcache.HashCache(basedir, cache_filename,
-            self.bzrdir._get_file_mode(),
-            self._content_filter_stack_provider())
-        hc = self._hashcache
-        hc.read()
-        # is this scan needed ? it makes things kinda slow.
-        #hc.scan()
-
-        if hc.needs_write:
-            mutter("write hc")
-            hc.write()
-
-        self._detect_case_handling()
+        self._transport = _transport
         self._rules_searcher = None
         self.views = self._make_views()
 
@@ -238,17 +221,6 @@
         """
         return self.bzrdir.is_control_filename(filename)
 
-    def _detect_case_handling(self):
-        wt_trans = self.bzrdir.get_workingtree_transport(None)
-        try:
-            wt_trans.stat(self._format.case_sensitive_filename)
-        except errors.NoSuchFile:
-            self.case_sensitive = True
-        else:
-            self.case_sensitive = False
-
-        self._setup_directory_is_tree_reference()
-
     branch = property(
         fget=lambda self: self._branch,
         doc="""The branch this WorkingTree is connected to.
@@ -257,6 +229,16 @@
             the working tree has been constructed from.
             """)
 
+    def has_versioned_directories(self):
+        """See `Tree.has_versioned_directories`."""
+        return self._format.supports_versioned_directories
+
+    def _supports_executable(self):
+        if sys.platform == 'win32':
+            return False
+        # FIXME: Ideally this should check the file system
+        return True
+
     def break_lock(self):
         """Break a lock if one is present from another instance.
 
@@ -265,8 +247,7 @@
 
         This will probe the repository for its lock as well.
         """
-        self._control_files.break_lock()
-        self.branch.break_lock()
+        raise NotImplementedError(self.break_lock)
 
     def requires_rich_root(self):
         return self._format.requires_rich_root
@@ -287,8 +268,8 @@
         """
         if path is None:
             path = osutils.getcwd()
-        control = bzrdir.BzrDir.open(path, _unsupported)
-        return control.open_workingtree(_unsupported)
+        control = controldir.ControlDir.open(path, _unsupported=_unsupported)
+        return control.open_workingtree(unsupported=_unsupported)
 
     @staticmethod
     def open_containing(path=None):
@@ -305,7 +286,7 @@
         """
         if path is None:
             path = osutils.getcwd()
-        control, relpath = bzrdir.BzrDir.open_containing(path)
+        control, relpath = controldir.ControlDir.open_containing(path)
         return control.open_workingtree(), relpath
 
     @staticmethod
@@ -331,7 +312,7 @@
                 if view_files:
                     file_list = view_files
                     view_str = views.view_display_str(view_files)
-                    note("Ignoring files outside view. View is %s" % view_str)
+                    note(gettext("Ignoring files outside view. View is %s") % view_str)
             return tree, file_list
         if default_directory == u'.':
             seed = file_list[0]
@@ -394,7 +375,7 @@
             else:
                 return True, tree
         t = transport.get_transport(location)
-        iterator = bzrdir.BzrDir.find_bzrdirs(t, evaluate=evaluate,
+        iterator = controldir.ControlDir.find_bzrdirs(t, evaluate=evaluate,
                                               list_current=list_current)
         return [tr for tr in iterator if tr is not None]
 
@@ -522,13 +503,13 @@
         raise NotImplementedError(self.get_root_id)
 
     @needs_read_lock
-    def clone(self, to_bzrdir, revision_id=None):
+    def clone(self, to_controldir, revision_id=None):
         """Duplicate this working tree into to_bzr, including all state.
 
         Specifically modified files are kept as modified, but
         ignored and unknown files are discarded.
 
-        If you want to make a new line of development, see bzrdir.sprout()
+        If you want to make a new line of development, see ControlDir.sprout()
 
         revision
             If not None, the cloned tree will have its last revision set to
@@ -536,7 +517,7 @@
             and this one merged in.
         """
         # assumes the target bzr dir format is compatible.
-        result = to_bzrdir.create_workingtree()
+        result = to_controldir.create_workingtree()
         self.copy_content_into(result, revision_id)
         return result
 
@@ -550,7 +531,11 @@
             # TODO now merge from tree.last_revision to revision (to preserve
             # user local changes)
             merge.transform_tree(tree, self)
-            tree.set_parent_ids([revision_id])
+            if revision_id == _mod_revision.NULL_REVISION:
+                new_parents = []
+            else:
+                new_parents = [revision_id]
+            tree.set_parent_ids(new_parents)
 
     def id2abspath(self, file_id):
         return self.abspath(self.id2path(file_id))
@@ -765,11 +750,8 @@
 
     @needs_tree_write_lock
     def set_merge_modified(self, modified_hashes):
-        def iter_stanzas():
-            for file_id, hash in modified_hashes.iteritems():
-                yield _mod_rio.Stanza(file_id=file_id.decode('utf8'),
-                    hash=hash)
-        self._put_rio('merge-hashes', iter_stanzas(), MERGE_MODIFIED_HEADER_1)
+        """Set the merge modified hashes."""
+        raise NotImplementedError(self.set_merge_modified)
 
     def _sha_from_stat(self, path, stat_result):
         """Get a sha digest from the tree's stat cache.
@@ -781,12 +763,6 @@
         """
         return None
 
-    def _put_rio(self, filename, stanzas, header):
-        self._must_be_locked()
-        my_file = _mod_rio.rio_file(stanzas, header)
-        self._transport.put_file(filename, my_file,
-            mode=self.bzrdir._get_file_mode())
-
     @needs_write_lock # because merge pulls data into the branch.
     def merge_from_branch(self, branch, to_revision=None, from_revision=None,
                           merge_type=None, force=False):
@@ -1032,14 +1008,17 @@
                                 show_base=show_base)
                     basis_root_id = basis_tree.get_root_id()
                     new_root_id = new_basis_tree.get_root_id()
-                    if basis_root_id != new_root_id:
+                    if new_root_id is not None and basis_root_id != new_root_id:
                         self.set_root_id(new_root_id)
                 finally:
                     basis_tree.unlock()
                 # TODO - dedup parents list with things merged by pull ?
                 # reuse the revisiontree we merged against to set the new
                 # tree data.
-                parent_trees = [(self.branch.last_revision(), new_basis_tree)]
+                parent_trees = []
+                if self.branch.last_revision() != _mod_revision.NULL_REVISION:
+                    parent_trees.append(
+                        (self.branch.last_revision(), new_basis_tree))
                 # we have to pull the merge trees out again, because
                 # merge_inner has set the ids. - this corner is not yet
                 # layered well enough to prevent double handling.
@@ -1062,7 +1041,6 @@
             stream.write(bytes)
         finally:
             stream.close()
-        # TODO: update the hashcache here ?
 
     def extras(self):
         """Yield all unversioned files in this WorkingTree.
@@ -1145,7 +1123,7 @@
         else:
             mode = stat_value.st_mode
             kind = osutils.file_kind_from_stat_mode(mode)
-            if not supports_executable():
+            if not self._supports_executable():
                 executable = entry is not None and entry.executable
             else:
                 executable = bool(stat.S_ISREG(mode) and stat.S_IEXEC & mode)
@@ -1170,11 +1148,8 @@
         return _mod_revision.ensure_null(self.branch.last_revision())
 
     def is_locked(self):
-        return self._control_files.is_locked()
-
-    def _must_be_locked(self):
-        if not self.is_locked():
-            raise errors.ObjectNotLocked(self)
+        """Check if this tree is locked."""
+        raise NotImplementedError(self.is_locked)
 
     def lock_read(self):
         """Lock the tree for reading.
@@ -1183,52 +1158,24 @@
 
         :return: A bzrlib.lock.LogicalLockResult.
         """
-        if not self.is_locked():
-            self._reset_data()
-        self.branch.lock_read()
-        try:
-            self._control_files.lock_read()
-            return LogicalLockResult(self.unlock)
-        except:
-            self.branch.unlock()
-            raise
+        raise NotImplementedError(self.lock_read)
 
     def lock_tree_write(self):
         """See MutableTree.lock_tree_write, and WorkingTree.unlock.
 
         :return: A bzrlib.lock.LogicalLockResult.
         """
-        if not self.is_locked():
-            self._reset_data()
-        self.branch.lock_read()
-        try:
-            self._control_files.lock_write()
-            return LogicalLockResult(self.unlock)
-        except:
-            self.branch.unlock()
-            raise
+        raise NotImplementedError(self.lock_tree_write)
 
     def lock_write(self):
         """See MutableTree.lock_write, and WorkingTree.unlock.
 
         :return: A bzrlib.lock.LogicalLockResult.
         """
-        if not self.is_locked():
-            self._reset_data()
-        self.branch.lock_write()
-        try:
-            self._control_files.lock_write()
-            return LogicalLockResult(self.unlock)
-        except:
-            self.branch.unlock()
-            raise
+        raise NotImplementedError(self.lock_write)
 
     def get_physical_lock_status(self):
-        return self._control_files.get_physical_lock_status()
-
-    def _reset_data(self):
-        """Reset transient data that cannot be revalidated."""
-        raise NotImplementedError(self._reset_data)
+        raise NotImplementedError(self.get_physical_lock_status)
 
     def set_last_revision(self, new_revision):
         """Change the last revision in the working tree."""
@@ -1528,7 +1475,7 @@
                                              show_base=show_base)
             if nb_conflicts:
                 self.add_parent_tree((old_tip, other_tree))
-                note('Rerun update after fixing the conflicts.')
+                note(gettext('Rerun update after fixing the conflicts.'))
                 return nb_conflicts
 
         if last_rev != _mod_revision.ensure_null(revision):
@@ -1576,20 +1523,6 @@
             last_rev = parent_trees[0][0]
         return nb_conflicts
 
-    def _write_hashcache_if_dirty(self):
-        """Write out the hashcache if it is dirty."""
-        if self._hashcache.needs_write:
-            try:
-                self._hashcache.write()
-            except OSError, e:
-                if e.errno not in (errno.EPERM, errno.EACCES):
-                    raise
-                # TODO: jam 20061219 Should this be a warning? A single line
-                #       warning might be sufficient to let the user know what
-                #       is going on.
-                mutter('Could not write hashcache for %s\nError: %s',
-                              self._hashcache.cache_file_name(), e)
-
     def set_conflicts(self, arg):
         raise errors.UnsupportedOperation(self.set_conflicts, self)
 
@@ -1824,8 +1757,11 @@
         :param branch: A branch to override probing for the branch.
         """
         super(InventoryWorkingTree, self).__init__(basedir=basedir,
-            branch=branch, _control_files=_control_files, _internal=_internal,
-            _format=_format, _bzrdir=_bzrdir)
+            branch=branch, _transport=_control_files._transport,
+            _internal=_internal, _format=_format, _bzrdir=_bzrdir)
+
+        self._control_files = _control_files
+        self._detect_case_handling()
 
         if _inventory is None:
             # This will be acquired on lock_read() or lock_write()
@@ -1851,6 +1787,17 @@
         self._inventory = inv
         self._inventory_is_modified = dirty
 
+    def _detect_case_handling(self):
+        wt_trans = self.bzrdir.get_workingtree_transport(None)
+        try:
+            wt_trans.stat(self._format.case_sensitive_filename)
+        except errors.NoSuchFile:
+            self.case_sensitive = True
+        else:
+            self.case_sensitive = False
+
+        self._setup_directory_is_tree_reference()
+
     def _serialize(self, inventory, out_file):
         xml5.serializer_v5.write_inventory(self._inventory, out_file,
             working=True)
@@ -1858,6 +1805,74 @@
     def _deserialize(selt, in_file):
         return xml5.serializer_v5.read_inventory(in_file)
 
+    def break_lock(self):
+        """Break a lock if one is present from another instance.
+
+        Uses the ui factory to ask for confirmation if the lock may be from
+        an active process.
+
+        This will probe the repository for its lock as well.
+        """
+        self._control_files.break_lock()
+        self.branch.break_lock()
+
+    def is_locked(self):
+        return self._control_files.is_locked()
+
+    def _must_be_locked(self):
+        if not self.is_locked():
+            raise errors.ObjectNotLocked(self)
+
+    def lock_read(self):
+        """Lock the tree for reading.
+
+        This also locks the branch, and can be unlocked via self.unlock().
+
+        :return: A bzrlib.lock.LogicalLockResult.
+        """
+        if not self.is_locked():
+            self._reset_data()
+        self.branch.lock_read()
+        try:
+            self._control_files.lock_read()
+            return LogicalLockResult(self.unlock)
+        except:
+            self.branch.unlock()
+            raise
+
+    def lock_tree_write(self):
+        """See MutableTree.lock_tree_write, and WorkingTree.unlock.
+
+        :return: A bzrlib.lock.LogicalLockResult.
+        """
+        if not self.is_locked():
+            self._reset_data()
+        self.branch.lock_read()
+        try:
+            self._control_files.lock_write()
+            return LogicalLockResult(self.unlock)
+        except:
+            self.branch.unlock()
+            raise
+
+    def lock_write(self):
+        """See MutableTree.lock_write, and WorkingTree.unlock.
+
+        :return: A bzrlib.lock.LogicalLockResult.
+        """
+        if not self.is_locked():
+            self._reset_data()
+        self.branch.lock_write()
+        try:
+            self._control_files.lock_write()
+            return LogicalLockResult(self.unlock)
+        except:
+            self.branch.unlock()
+            raise
+
+    def get_physical_lock_status(self):
+        return self._control_files.get_physical_lock_status()
+
     @needs_tree_write_lock
     def _write_inventory(self, inv):
         """Write inventory as the current inventory."""
@@ -2160,17 +2175,16 @@
             mode=self.bzrdir._get_file_mode())
         self._inventory_is_modified = False
 
-    @needs_read_lock
-    def get_file_sha1(self, file_id, path=None, stat_value=None):
-        if not path:
-            path = self._inventory.id2path(file_id)
-        return self._hashcache.get_sha1(path, stat_value)
-
     def get_file_mtime(self, file_id, path=None):
         """See Tree.get_file_mtime."""
         if not path:
             path = self.inventory.id2path(file_id)
-        return os.lstat(self.abspath(path)).st_mtime
+        try:
+            return os.lstat(self.abspath(path)).st_mtime
+        except OSError, e:
+            if e.errno == errno.ENOENT:
+                raise errors.FileTimestampUnavailable(path)
+            raise
 
     def _is_executable_from_path_and_stat_from_basis(self, path, stat_result):
         file_id = self.path2id(path)
@@ -2184,21 +2198,20 @@
         mode = stat_result.st_mode
         return bool(stat.S_ISREG(mode) and stat.S_IEXEC & mode)
 
-    if not supports_executable():
-        def is_executable(self, file_id, path=None):
+    def is_executable(self, file_id, path=None):
+        if not self._supports_executable():
             return self._inventory[file_id].executable
-
-        _is_executable_from_path_and_stat = \
-            _is_executable_from_path_and_stat_from_basis
-    else:
-        def is_executable(self, file_id, path=None):
+        else:
             if not path:
                 path = self.id2path(file_id)
             mode = os.lstat(self.abspath(path)).st_mode
             return bool(stat.S_ISREG(mode) and stat.S_IEXEC & mode)
 
-        _is_executable_from_path_and_stat = \
-            _is_executable_from_path_and_stat_from_stat
+    def _is_executable_from_path_and_stat(self, path, stat_result):
+        if not self._supports_executable():
+            return self._is_executable_from_path_and_stat_from_basis(path, stat_result)
+        else:
+            return self._is_executable_from_path_and_stat_from_stat(path, stat_result)
 
     @needs_tree_write_lock
     def _add(self, files, ids, kinds):
@@ -2283,6 +2296,20 @@
                        for key, line in annotator.annotate_flat(this_key)]
         return annotations
 
+    def _put_rio(self, filename, stanzas, header):
+        self._must_be_locked()
+        my_file = _mod_rio.rio_file(stanzas, header)
+        self._transport.put_file(filename, my_file,
+            mode=self.bzrdir._get_file_mode())
+
+    @needs_tree_write_lock
+    def set_merge_modified(self, modified_hashes):
+        def iter_stanzas():
+            for file_id, hash in modified_hashes.iteritems():
+                yield _mod_rio.Stanza(file_id=file_id.decode('utf8'),
+                    hash=hash)
+        self._put_rio('merge-hashes', iter_stanzas(), MERGE_MODIFIED_HEADER_1)
+
     @needs_read_lock
     def merge_modified(self):
         """Return a dictionary of files modified by a merge.
@@ -2778,9 +2805,8 @@
                 # something is wrong, so lets determine what exactly
                 if not self.has_filename(from_rel) and \
                    not self.has_filename(to_rel):
-                    raise errors.BzrRenameFailedError(from_rel,to_rel,
-                        errors.PathsDoNotExist(paths=(str(from_rel),
-                        str(to_rel))))
+                    raise errors.BzrRenameFailedError(from_rel, to_rel,
+                        errors.PathsDoNotExist(paths=(from_rel, to_rel)))
                 else:
                     raise errors.RenameFailedFilesExist(from_rel, to_rel)
             rename_entry.only_change_inv = only_change_inv
@@ -2951,6 +2977,16 @@
                 if dir[2] == _directory:
                     pending.append(dir)
 
+    @needs_write_lock
+    def update_feature_flags(self, updated_flags):
+        """Update the feature flags for this branch.
+
+        :param updated_flags: Dictionary mapping feature names to necessities
+            A necessity can be None to indicate the feature should be removed
+        """
+        self._format._update_feature_flags(updated_flags)
+        self.control_transport.put_bytes('format', self._format.as_string())
+
 
 class WorkingTreeFormatRegistry(controldir.ControlComponentFormatRegistry):
     """Registry for working tree formats."""
@@ -3012,30 +3048,11 @@
 
     supports_versioned_directories = None
 
-    @classmethod
-    def find_format_string(klass, a_bzrdir):
-        """Return format name for the working tree object in a_bzrdir."""
-        try:
-            transport = a_bzrdir.get_workingtree_transport(None)
-            return transport.get_bytes("format")
-        except errors.NoSuchFile:
-            raise errors.NoWorkingTree(base=transport.base)
-
-    @classmethod
-    def find_format(klass, a_bzrdir):
-        """Return the format for the working tree object in a_bzrdir."""
-        try:
-            format_string = klass.find_format_string(a_bzrdir)
-            return format_registry.get(format_string)
-        except KeyError:
-            raise errors.UnknownFormatError(format=format_string,
-                                            kind="working tree")
-
-    def initialize(self, a_bzrdir, revision_id=None, from_branch=None,
+    def initialize(self, controldir, revision_id=None, from_branch=None,
                    accelerator_tree=None, hardlink=False):
-        """Initialize a new working tree in a_bzrdir.
+        """Initialize a new working tree in controldir.
 
-        :param a_bzrdir: BzrDir to initialize the working tree in.
+        :param controldir: ControlDir to initialize the working tree in.
         :param revision_id: allows creating a working tree at a different
             revision than the branch is at.
         :param from_branch: Branch to checkout
@@ -3061,10 +3078,6 @@
         """Return the current default format."""
         return format_registry.get_default()
 
-    def get_format_string(self):
-        """Return the ASCII format string that identifies this format."""
-        raise NotImplementedError(self.get_format_string)
-
     def get_format_description(self):
         """Return the short description for this format."""
         raise NotImplementedError(self.get_format_description)
@@ -3122,6 +3135,46 @@
     def unregister_format(klass, format):
         format_registry.remove(format)
 
+    def get_controldir_for_branch(self):
+        """Get the control directory format for creating branches.
+
+        This is to support testing of working tree formats that can not exist
+        in the same control directory as a branch.
+        """
+        return self._matchingbzrdir
+
+
+class WorkingTreeFormatMetaDir(bzrdir.BzrFormat, WorkingTreeFormat):
+    """Base class for working trees that live in bzr meta directories."""
+
+    def __init__(self):
+        WorkingTreeFormat.__init__(self)
+        bzrdir.BzrFormat.__init__(self)
+
+    @classmethod
+    def find_format_string(klass, controldir):
+        """Return format name for the working tree object in controldir."""
+        try:
+            transport = controldir.get_workingtree_transport(None)
+            return transport.get_bytes("format")
+        except errors.NoSuchFile:
+            raise errors.NoWorkingTree(base=transport.base)
+
+    @classmethod
+    def find_format(klass, controldir):
+        """Return the format for the working tree object in controldir."""
+        format_string = klass.find_format_string(controldir)
+        return klass._find_format(format_registry, 'working tree',
+                format_string)
+
+    def check_support_status(self, allow_unsupported, recommend_upgrade=True,
+            basedir=None):
+        WorkingTreeFormat.check_support_status(self,
+            allow_unsupported=allow_unsupported, recommend_upgrade=recommend_upgrade,
+            basedir=basedir)
+        bzrdir.BzrFormat.check_support_status(self, allow_unsupported=allow_unsupported,
+            recommend_upgrade=recommend_upgrade, basedir=basedir)
+
 
 format_registry.register_lazy("Bazaar Working Tree Format 4 (bzr 0.15)\n",
     "bzrlib.workingtree_4", "WorkingTreeFormat4")

=== modified file 'bzrlib/workingtree_3.py'
--- bzrlib/workingtree_3.py	2011-06-28 11:29:03 +0000
+++ bzrlib/workingtree_3.py	2011-12-19 19:15:58 +0000
@@ -18,11 +18,17 @@
 
 """
 
+from __future__ import absolute_import
+
+import errno
+
 from bzrlib import (
     bzrdir,
     errors,
+    hashcache,
     inventory,
     revision as _mod_revision,
+    trace,
     transform,
     )
 from bzrlib.decorators import (
@@ -33,10 +39,56 @@
 from bzrlib.transport.local import LocalTransport
 from bzrlib.workingtree import (
     InventoryWorkingTree,
-    WorkingTreeFormat,
+    WorkingTreeFormatMetaDir,
     )
 
-class WorkingTree3(InventoryWorkingTree):
+
+class PreDirStateWorkingTree(InventoryWorkingTree):
+
+    def __init__(self, basedir='.', *args, **kwargs):
+        super(PreDirStateWorkingTree, self).__init__(basedir, *args, **kwargs)
+        # update the whole cache up front and write to disk if anything changed;
+        # in the future we might want to do this more selectively
+        # two possible ways offer themselves : in self._unlock, write the cache
+        # if needed, or, when the cache sees a change, append it to the hash
+        # cache file, and have the parser take the most recent entry for a
+        # given path only.
+        wt_trans = self.bzrdir.get_workingtree_transport(None)
+        cache_filename = wt_trans.local_abspath('stat-cache')
+        self._hashcache = hashcache.HashCache(basedir, cache_filename,
+            self.bzrdir._get_file_mode(),
+            self._content_filter_stack_provider())
+        hc = self._hashcache
+        hc.read()
+        # is this scan needed ? it makes things kinda slow.
+        #hc.scan()
+
+        if hc.needs_write:
+            trace.mutter("write hc")
+            hc.write()
+
+    def _write_hashcache_if_dirty(self):
+        """Write out the hashcache if it is dirty."""
+        if self._hashcache.needs_write:
+            try:
+                self._hashcache.write()
+            except OSError, e:
+                if e.errno not in (errno.EPERM, errno.EACCES):
+                    raise
+                # TODO: jam 20061219 Should this be a warning? A single line
+                #       warning might be sufficient to let the user know what
+                #       is going on.
+                trace.mutter('Could not write hashcache for %s\nError: %s',
+                              self._hashcache.cache_file_name(), e)
+
+    @needs_read_lock
+    def get_file_sha1(self, file_id, path=None, stat_value=None):
+        if not path:
+            path = self._inventory.id2path(file_id)
+        return self._hashcache.get_sha1(path, stat_value)
+
+
+class WorkingTree3(PreDirStateWorkingTree):
     """This is the Format 3 working tree.
 
     This differs from the base WorkingTree by:
@@ -86,13 +138,13 @@
             self.branch.unlock()
 
 
-class WorkingTreeFormat3(WorkingTreeFormat):
+class WorkingTreeFormat3(WorkingTreeFormatMetaDir):
     """The second working tree format updated to record a format marker.
 
     This format:
         - exists within a metadir controlling .bzr
         - includes an explicit version marker for the workingtree control
-          files, separate from the BzrDir format
+          files, separate from the ControlDir format
         - modifies the hash cache format
         - is new in bzr 0.8
         - uses a LockDir to guard access for writes.
@@ -104,7 +156,8 @@
 
     supports_versioned_directories = True
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See WorkingTreeFormat.get_format_string()."""
         return "Bazaar-NG Working Tree format 3"
 
@@ -142,7 +195,7 @@
         control_files = self._open_control_files(a_bzrdir)
         control_files.create_lock()
         control_files.lock_write()
-        transport.put_bytes('format', self.get_format_string(),
+        transport.put_bytes('format', self.as_string(),
             mode=a_bzrdir._get_file_mode())
         if from_branch is not None:
             branch = from_branch
@@ -209,6 +262,3 @@
                                 _format=self,
                                 _bzrdir=a_bzrdir,
                                 _control_files=control_files)
-
-    def __str__(self):
-        return self.get_format_string()

=== modified file 'bzrlib/workingtree_4.py'
--- bzrlib/workingtree_4.py	2011-08-12 14:47:42 +0000
+++ bzrlib/workingtree_4.py	2011-12-19 19:15:58 +0000
@@ -22,6 +22,8 @@
 WorkingTree.open(dir).
 """
 
+from __future__ import absolute_import
+
 from cStringIO import StringIO
 import os
 import sys
@@ -71,7 +73,7 @@
 from bzrlib.workingtree import (
     InventoryWorkingTree,
     WorkingTree,
-    WorkingTreeFormat,
+    WorkingTreeFormatMetaDir,
     )
 
 
@@ -476,25 +478,17 @@
             return False # Missing entries are not executable
         return entry[1][0][3] # Executable?
 
-    if not osutils.supports_executable():
-        def is_executable(self, file_id, path=None):
-            """Test if a file is executable or not.
+    def is_executable(self, file_id, path=None):
+        """Test if a file is executable or not.
 
-            Note: The caller is expected to take a read-lock before calling this.
-            """
+        Note: The caller is expected to take a read-lock before calling this.
+        """
+        if not self._supports_executable():
             entry = self._get_entry(file_id=file_id, path=path)
             if entry == (None, None):
                 return False
             return entry[1][0][3]
-
-        _is_executable_from_path_and_stat = \
-            _is_executable_from_path_and_stat_from_basis
-    else:
-        def is_executable(self, file_id, path=None):
-            """Test if a file is executable or not.
-
-            Note: The caller is expected to take a read-lock before calling this.
-            """
+        else:
             self._must_be_locked()
             if not path:
                 path = self.id2path(file_id)
@@ -969,7 +963,8 @@
                     all_versioned = False
                     break
             if not all_versioned:
-                raise errors.PathsNotVersionedError(paths)
+                raise errors.PathsNotVersionedError(
+                    [p.decode('utf-8') for p in paths])
         # -- remove redundancy in supplied paths to prevent over-scanning --
         search_paths = osutils.minimum_path_selection(paths)
         # sketch:
@@ -1024,7 +1019,8 @@
             found_dir_names = set(dir_name_id[:2] for dir_name_id in found)
             for dir_name in split_paths:
                 if dir_name not in found_dir_names:
-                    raise errors.PathsNotVersionedError(paths)
+                    raise errors.PathsNotVersionedError(
+                        [p.decode('utf-8') for p in paths])
 
         for dir_name_id, trees_info in found.iteritems():
             for index in search_indexes:
@@ -1446,7 +1442,7 @@
         return views.PathBasedViews(self)
 
 
-class DirStateWorkingTreeFormat(WorkingTreeFormat):
+class DirStateWorkingTreeFormat(WorkingTreeFormatMetaDir):
 
     missing_parent_conflicts = True
 
@@ -1482,7 +1478,7 @@
         control_files = self._open_control_files(a_bzrdir)
         control_files.create_lock()
         control_files.lock_write()
-        transport.put_bytes('format', self.get_format_string(),
+        transport.put_bytes('format', self.as_string(),
             mode=a_bzrdir._get_file_mode())
         if from_branch is not None:
             branch = from_branch
@@ -1608,7 +1604,7 @@
     This format:
         - exists within a metadir controlling .bzr
         - includes an explicit version marker for the workingtree control
-          files, separate from the BzrDir format
+          files, separate from the ControlDir format
         - modifies the hash cache format
         - is new in bzr 0.15
         - uses a LockDir to guard access to it.
@@ -1618,7 +1614,8 @@
 
     _tree_class = WorkingTree4
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See WorkingTreeFormat.get_format_string()."""
         return "Bazaar Working Tree Format 4 (bzr 0.15)\n"
 
@@ -1635,7 +1632,8 @@
 
     _tree_class = WorkingTree5
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See WorkingTreeFormat.get_format_string()."""
         return "Bazaar Working Tree Format 5 (bzr 1.11)\n"
 
@@ -1655,7 +1653,8 @@
 
     _tree_class = WorkingTree6
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See WorkingTreeFormat.get_format_string()."""
         return "Bazaar Working Tree Format 6 (bzr 1.14)\n"
 
@@ -1849,7 +1848,7 @@
         # Make sure the file exists
         entry = self._get_entry(file_id, path=path)
         if entry == (None, None): # do we raise?
-            return None
+            raise errors.NoSuchId(self, file_id)
         parent_index = self._get_parent_index()
         last_changed_revision = entry[1][parent_index][4]
         try:
@@ -2174,7 +2173,7 @@
                 path_entries = state._entries_for_path(path)
                 if not path_entries:
                     # this specified path is not present at all: error
-                    not_versioned.append(path)
+                    not_versioned.append(path.decode('utf-8'))
                     continue
                 found_versioned = False
                 # for each id at this path
@@ -2188,7 +2187,7 @@
                 if not found_versioned:
                     # none of the indexes was not 'absent' at all ids for this
                     # path.
-                    not_versioned.append(path)
+                    not_versioned.append(path.decode('utf-8'))
             if len(not_versioned) > 0:
                 raise errors.PathsNotVersionedError(not_versioned)
         # -- remove redundancy in supplied specific_files to prevent over-scanning --
@@ -2261,7 +2260,7 @@
     def update_format(self, tree):
         """Change the format marker."""
         tree._transport.put_bytes('format',
-            self.target_format.get_format_string(),
+            self.target_format.as_string(),
             mode=tree.bzrdir._get_file_mode())
 
 
@@ -2284,7 +2283,7 @@
     def update_format(self, tree):
         """Change the format marker."""
         tree._transport.put_bytes('format',
-            self.target_format.get_format_string(),
+            self.target_format.as_string(),
             mode=tree.bzrdir._get_file_mode())
 
 
@@ -2313,5 +2312,5 @@
     def update_format(self, tree):
         """Change the format marker."""
         tree._transport.put_bytes('format',
-            self.target_format.get_format_string(),
+            self.target_format.as_string(),
             mode=tree.bzrdir._get_file_mode())

=== modified file 'bzrlib/xml5.py'
--- bzrlib/xml5.py	2010-02-17 17:11:16 +0000
+++ bzrlib/xml5.py	2011-12-18 15:28:38 +0000
@@ -14,13 +14,20 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 from bzrlib import (
     cache_utf8,
     errors,
     inventory,
     xml6,
-    xml8,
-    )
+    )
+from bzrlib.xml_serializer import (
+    encode_and_escape,
+    get_utf8_or_ascii,
+    unpack_inventory_entry,
+    )
+
 
 class Serializer_v5(xml6.Serializer_v6):
     """Version 5 serializer
@@ -35,7 +42,7 @@
         """Construct from XML Element
         """
         root_id = elt.get('file_id') or inventory.ROOT_ID
-        root_id = xml8._get_utf8_or_ascii(root_id)
+        root_id = get_utf8_or_ascii(root_id)
 
         format = elt.get('format')
         if format is not None:
@@ -52,10 +59,9 @@
         #   avoiding attributes     2.46s
         #   adding assertions       2.50s
         #   last_parent cache       2.52s (worse, removed)
-        unpack_entry = self._unpack_entry
         byid = inv._byid
         for e in elt:
-            ie = unpack_entry(e, entry_cache=entry_cache,
+            ie = unpack_inventory_entry(e, entry_cache=entry_cache,
                               return_from_cache=return_from_cache)
             parent_id = ie.parent_id
             if parent_id is None:
@@ -92,13 +98,13 @@
         """Append the inventory root to output."""
         if inv.root.file_id not in (None, inventory.ROOT_ID):
             fileid1 = ' file_id="'
-            fileid2 = xml8._encode_and_escape(inv.root.file_id)
+            fileid2 = encode_and_escape(inv.root.file_id)
         else:
             fileid1 = ""
             fileid2 = ""
         if inv.revision_id is not None:
             revid1 = ' revision_id="'
-            revid2 = xml8._encode_and_escape(inv.revision_id)
+            revid2 = encode_and_escape(inv.revision_id)
         else:
             revid1 = ""
             revid2 = ""

=== modified file 'bzrlib/xml6.py'
--- bzrlib/xml6.py	2009-03-23 14:59:43 +0000
+++ bzrlib/xml6.py	2011-12-18 15:28:38 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 from bzrlib import xml8
 
 

=== modified file 'bzrlib/xml7.py'
--- bzrlib/xml7.py	2010-02-17 17:11:16 +0000
+++ bzrlib/xml7.py	2011-12-18 15:28:38 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 from bzrlib import (
     inventory,
     xml6,

=== modified file 'bzrlib/xml8.py'
--- bzrlib/xml8.py	2011-02-21 15:09:19 +0000
+++ bzrlib/xml8.py	2011-12-18 15:28:38 +0000
@@ -14,13 +14,12 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from __future__ import absolute_import
+
 import cStringIO
-import re
 
 from bzrlib import (
     cache_utf8,
-    errors,
-    inventory,
     lazy_regex,
     revision as _mod_revision,
     trace,
@@ -29,23 +28,17 @@
     Element,
     SubElement,
     XMLSerializer,
+    encode_and_escape,
     escape_invalid_chars,
+    get_utf8_or_ascii,
+    serialize_inventory_flat,
+    unpack_inventory_entry,
+    unpack_inventory_flat,
     )
-from bzrlib.inventory import InventoryEntry
 from bzrlib.revision import Revision
 from bzrlib.errors import BzrError
 
 
-_utf8_re = None
-_unicode_re = None
-_xml_escape_map = {
-    "&":'&amp;',
-    "'":"&apos;", # FIXME: overkill
-    "\"":"&quot;",
-    "<":"&lt;",
-    ">":"&gt;",
-    }
-
 _xml_unescape_map = {
     'apos':"'",
     'quot':'"',
@@ -65,115 +58,13 @@
         return unichr(int(code[1:])).encode('utf8')
 
 
-_unescape_re = None
-
+_unescape_re = lazy_regex.lazy_compile('\&([^;]*);')
 
 def _unescape_xml(data):
     """Unescape predefined XML entities in a string of data."""
-    global _unescape_re
-    if _unescape_re is None:
-        _unescape_re = re.compile('\&([^;]*);')
     return _unescape_re.sub(_unescaper, data)
 
 
-def _ensure_utf8_re():
-    """Make sure the _utf8_re and _unicode_re regexes have been compiled."""
-    global _utf8_re, _unicode_re
-    if _utf8_re is None:
-        _utf8_re = re.compile('[&<>\'\"]|[\x80-\xff]+')
-    if _unicode_re is None:
-        _unicode_re = re.compile(u'[&<>\'\"\u0080-\uffff]')
-
-
-def _unicode_escape_replace(match, _map=_xml_escape_map):
-    """Replace a string of non-ascii, non XML safe characters with their escape
-
-    This will escape both Standard XML escapes, like <>"', etc.
-    As well as escaping non ascii characters, because ElementTree did.
-    This helps us remain compatible to older versions of bzr. We may change
-    our policy in the future, though.
-    """
-    # jam 20060816 Benchmarks show that try/KeyError is faster if you
-    # expect the entity to rarely miss. There is about a 10% difference
-    # in overall time. But if you miss frequently, then if None is much
-    # faster. For our use case, we *rarely* have a revision id, file id
-    # or path name that is unicode. So use try/KeyError.
-    try:
-        return _map[match.group()]
-    except KeyError:
-        return "&#%d;" % ord(match.group())
-
-
-def _utf8_escape_replace(match, _map=_xml_escape_map):
-    """Escape utf8 characters into XML safe ones.
-
-    This uses 2 tricks. It is either escaping "standard" characters, like "&<>,
-    or it is handling characters with the high-bit set. For ascii characters,
-    we just lookup the replacement in the dictionary. For everything else, we
-    decode back into Unicode, and then use the XML escape code.
-    """
-    try:
-        return _map[match.group()]
-    except KeyError:
-        return ''.join('&#%d;' % ord(uni_chr)
-                       for uni_chr in match.group().decode('utf8'))
-
-
-_to_escaped_map = {}
-
-def _encode_and_escape(unicode_or_utf8_str, _map=_to_escaped_map):
-    """Encode the string into utf8, and escape invalid XML characters"""
-    # We frequently get entities we have not seen before, so it is better
-    # to check if None, rather than try/KeyError
-    text = _map.get(unicode_or_utf8_str)
-    if text is None:
-        if unicode_or_utf8_str.__class__ is unicode:
-            # The alternative policy is to do a regular UTF8 encoding
-            # and then escape only XML meta characters.
-            # Performance is equivalent once you use cache_utf8. *However*
-            # this makes the serialized texts incompatible with old versions
-            # of bzr. So no net gain. (Perhaps the read code would handle utf8
-            # better than entity escapes, but cElementTree seems to do just fine
-            # either way)
-            text = str(_unicode_re.sub(_unicode_escape_replace,
-                                       unicode_or_utf8_str)) + '"'
-        else:
-            # Plain strings are considered to already be in utf-8 so we do a
-            # slightly different method for escaping.
-            text = _utf8_re.sub(_utf8_escape_replace,
-                                unicode_or_utf8_str) + '"'
-        _map[unicode_or_utf8_str] = text
-    return text
-
-
-def _get_utf8_or_ascii(a_str,
-                       _encode_utf8=cache_utf8.encode,
-                       _get_cached_ascii=cache_utf8.get_cached_ascii):
-    """Return a cached version of the string.
-
-    cElementTree will return a plain string if the XML is plain ascii. It only
-    returns Unicode when it needs to. We want to work in utf-8 strings. So if
-    cElementTree returns a plain string, we can just return the cached version.
-    If it is Unicode, then we need to encode it.
-
-    :param a_str: An 8-bit string or Unicode as returned by
-                  cElementTree.Element.get()
-    :return: A utf-8 encoded 8-bit string.
-    """
-    # This is fairly optimized because we know what cElementTree does, this is
-    # not meant as a generic function for all cases. Because it is possible for
-    # an 8-bit string to not be ascii or valid utf8.
-    if a_str.__class__ is unicode:
-        return _encode_utf8(a_str)
-    else:
-        return intern(a_str)
-
-
-def _clear_cache():
-    """Clean out the unicode => escaped map"""
-    _to_escaped_map.clear()
-
-
 class Serializer_v8(XMLSerializer):
     """This serialiser adds rich roots.
 
@@ -261,83 +152,11 @@
             reference_revision, symlink_target.
         :return: The inventory as a list of lines.
         """
-        _ensure_utf8_re()
-        self._check_revisions(inv)
         output = []
         append = output.append
         self._append_inventory_root(append, inv)
-        entries = inv.iter_entries()
-        # Skip the root
-        root_path, root_ie = entries.next()
-        for path, ie in entries:
-            if ie.parent_id != self.root_id:
-                parent_str = ' parent_id="'
-                parent_id  = _encode_and_escape(ie.parent_id)
-            else:
-                parent_str = ''
-                parent_id  = ''
-            if ie.kind == 'file':
-                if ie.executable:
-                    executable = ' executable="yes"'
-                else:
-                    executable = ''
-                if not working:
-                    append('<file%s file_id="%s name="%s%s%s revision="%s '
-                        'text_sha1="%s" text_size="%d" />\n' % (
-                        executable, _encode_and_escape(ie.file_id),
-                        _encode_and_escape(ie.name), parent_str, parent_id,
-                        _encode_and_escape(ie.revision), ie.text_sha1,
-                        ie.text_size))
-                else:
-                    append('<file%s file_id="%s name="%s%s%s />\n' % (
-                        executable, _encode_and_escape(ie.file_id),
-                        _encode_and_escape(ie.name), parent_str, parent_id))
-            elif ie.kind == 'directory':
-                if not working:
-                    append('<directory file_id="%s name="%s%s%s revision="%s '
-                        '/>\n' % (
-                        _encode_and_escape(ie.file_id),
-                        _encode_and_escape(ie.name),
-                        parent_str, parent_id,
-                        _encode_and_escape(ie.revision)))
-                else:
-                    append('<directory file_id="%s name="%s%s%s />\n' % (
-                        _encode_and_escape(ie.file_id),
-                        _encode_and_escape(ie.name),
-                        parent_str, parent_id))
-            elif ie.kind == 'symlink':
-                if not working:
-                    append('<symlink file_id="%s name="%s%s%s revision="%s '
-                        'symlink_target="%s />\n' % (
-                        _encode_and_escape(ie.file_id),
-                        _encode_and_escape(ie.name),
-                        parent_str, parent_id,
-                        _encode_and_escape(ie.revision),
-                        _encode_and_escape(ie.symlink_target)))
-                else:
-                    append('<symlink file_id="%s name="%s%s%s />\n' % (
-                        _encode_and_escape(ie.file_id),
-                        _encode_and_escape(ie.name),
-                        parent_str, parent_id))
-            elif ie.kind == 'tree-reference':
-                if ie.kind not in self.supported_kinds:
-                    raise errors.UnsupportedInventoryKind(ie.kind)
-                if not working:
-                    append('<tree-reference file_id="%s name="%s%s%s '
-                        'revision="%s reference_revision="%s />\n' % (
-                        _encode_and_escape(ie.file_id),
-                        _encode_and_escape(ie.name),
-                        parent_str, parent_id,
-                        _encode_and_escape(ie.revision),
-                        _encode_and_escape(ie.reference_revision)))
-                else:
-                    append('<tree-reference file_id="%s name="%s%s%s />\n' % (
-                        _encode_and_escape(ie.file_id),
-                        _encode_and_escape(ie.name),
-                        parent_str, parent_id))
-            else:
-                raise errors.UnsupportedInventoryKind(ie.kind)
-        append('</inventory>\n')
+        serialize_inventory_flat(inv, append,
+            self.root_id, self.supported_kinds, working)
         if f is not None:
             f.writelines(output)
         # Just to keep the cache from growing without bounds
@@ -349,16 +168,16 @@
         """Append the inventory root to output."""
         if inv.revision_id is not None:
             revid1 = ' revision_id="'
-            revid2 = _encode_and_escape(inv.revision_id)
+            revid2 = encode_and_escape(inv.revision_id)
         else:
             revid1 = ""
             revid2 = ""
         append('<inventory format="%s"%s%s>\n' % (
             self.format_num, revid1, revid2))
         append('<directory file_id="%s name="%s revision="%s />\n' % (
-            _encode_and_escape(inv.root.file_id),
-            _encode_and_escape(inv.root.name),
-            _encode_and_escape(inv.root.revision)))
+            encode_and_escape(inv.root.file_id),
+            encode_and_escape(inv.root.name),
+            encode_and_escape(inv.root.revision)))
 
     def _pack_revision(self, rev):
         """Revision object -> xml tree"""
@@ -408,119 +227,19 @@
             prop_elt.tail = '\n'
         top_elt.tail = '\n'
 
+    def _unpack_entry(self, elt, entry_cache=None, return_from_cache=False):
+        # This is here because it's overridden by xml7
+        return unpack_inventory_entry(elt, entry_cache,
+                return_from_cache)
+
     def _unpack_inventory(self, elt, revision_id=None, entry_cache=None,
                           return_from_cache=False):
         """Construct from XML Element"""
-        if elt.tag != 'inventory':
-            raise errors.UnexpectedInventoryFormat('Root tag is %r' % elt.tag)
-        format = elt.get('format')
-        if format != self.format_num:
-            raise errors.UnexpectedInventoryFormat('Invalid format version %r'
-                                                   % format)
-        revision_id = elt.get('revision_id')
-        if revision_id is not None:
-            revision_id = cache_utf8.encode(revision_id)
-        inv = inventory.Inventory(root_id=None, revision_id=revision_id)
-        for e in elt:
-            ie = self._unpack_entry(e, entry_cache=entry_cache,
-                                    return_from_cache=return_from_cache)
-            inv.add(ie)
+        inv = unpack_inventory_flat(elt, self.format_num, self._unpack_entry,
+            entry_cache, return_from_cache)
         self._check_cache_size(len(inv), entry_cache)
         return inv
 
-    def _unpack_entry(self, elt, entry_cache=None, return_from_cache=False):
-        elt_get = elt.get
-        file_id = elt_get('file_id')
-        revision = elt_get('revision')
-        # Check and see if we have already unpacked this exact entry
-        # Some timings for "repo.revision_trees(last_100_revs)"
-        #               bzr     mysql
-        #   unmodified  4.1s    40.8s
-        #   using lru   3.5s
-        #   using fifo  2.83s   29.1s
-        #   lru._cache  2.8s
-        #   dict        2.75s   26.8s
-        #   inv.add     2.5s    26.0s
-        #   no_copy     2.00s   20.5s
-        #   no_c,dict   1.95s   18.0s
-        # Note that a cache of 10k nodes is more than sufficient to hold all of
-        # the inventory for the last 100 revs for bzr, but not for mysql (20k
-        # is enough for mysql, which saves the same 2s as using a dict)
-
-        # Breakdown of mysql using time.clock()
-        #   4.1s    2 calls to element.get for file_id, revision_id
-        #   4.5s    cache_hit lookup
-        #   7.1s    InventoryFile.copy()
-        #   2.4s    InventoryDirectory.copy()
-        #   0.4s    decoding unique entries
-        #   1.6s    decoding entries after FIFO fills up
-        #   0.8s    Adding nodes to FIFO (including flushes)
-        #   0.1s    cache miss lookups
-        # Using an LRU cache
-        #   4.1s    2 calls to element.get for file_id, revision_id
-        #   9.9s    cache_hit lookup
-        #   10.8s   InventoryEntry.copy()
-        #   0.3s    cache miss lookus
-        #   1.2s    decoding entries
-        #   1.0s    adding nodes to LRU
-        if entry_cache is not None and revision is not None:
-            key = (file_id, revision)
-            try:
-                # We copy it, because some operations may mutate it
-                cached_ie = entry_cache[key]
-            except KeyError:
-                pass
-            else:
-                # Only copying directory entries drops us 2.85s => 2.35s
-                if return_from_cache:
-                    if cached_ie.kind == 'directory':
-                        return cached_ie.copy()
-                    return cached_ie
-                return cached_ie.copy()
-
-        kind = elt.tag
-        if not InventoryEntry.versionable_kind(kind):
-            raise AssertionError('unsupported entry kind %s' % kind)
-
-        get_cached = _get_utf8_or_ascii
-
-        file_id = get_cached(file_id)
-        if revision is not None:
-            revision = get_cached(revision)
-        parent_id = elt_get('parent_id')
-        if parent_id is not None:
-            parent_id = get_cached(parent_id)
-
-        if kind == 'directory':
-            ie = inventory.InventoryDirectory(file_id,
-                                              elt_get('name'),
-                                              parent_id)
-        elif kind == 'file':
-            ie = inventory.InventoryFile(file_id,
-                                         elt_get('name'),
-                                         parent_id)
-            ie.text_sha1 = elt_get('text_sha1')
-            if elt_get('executable') == 'yes':
-                ie.executable = True
-            v = elt_get('text_size')
-            ie.text_size = v and int(v)
-        elif kind == 'symlink':
-            ie = inventory.InventoryLink(file_id,
-                                         elt_get('name'),
-                                         parent_id)
-            ie.symlink_target = elt_get('symlink_target')
-        else:
-            raise errors.UnsupportedInventoryKind(kind)
-        ie.revision = revision
-        if revision is not None and entry_cache is not None:
-            # We cache a copy() because callers like to mutate objects, and
-            # that would cause the item in cache to mutate as well.
-            # This has a small effect on many-inventory performance, because
-            # the majority fraction is spent in cache hits, not misses.
-            entry_cache[key] = ie.copy()
-
-        return ie
-
     def _unpack_revision(self, elt):
         """XML Element -> Revision object"""
         format = elt.get('format')
@@ -531,7 +250,7 @@
             if format != format_num:
                 raise BzrError("invalid format version %r on revision"
                                 % format)
-        get_cached = _get_utf8_or_ascii
+        get_cached = get_utf8_or_ascii
         rev = Revision(committer = elt.get('committer'),
                        timestamp = float(elt.get('timestamp')),
                        revision_id = get_cached(elt.get('revision_id')),

=== modified file 'bzrlib/xml_serializer.py'
--- bzrlib/xml_serializer.py	2010-09-17 04:35:23 +0000
+++ bzrlib/xml_serializer.py	2011-12-19 13:23:58 +0000
@@ -16,6 +16,8 @@
 
 """XML externalization support."""
 
+from __future__ import absolute_import
+
 # "XML is like violence: if it doesn't solve your problem, you aren't
 # using enough of it." -- various
 
@@ -50,7 +52,12 @@
     import util.elementtree as elementtree
     from xml.parsers.expat import ExpatError as ParseError
 
-from bzrlib import errors
+from bzrlib import (
+    cache_utf8,
+    inventory,
+    lazy_regex,
+    errors,
+    )
 
 
 class XMLSerializer(Serializer):
@@ -130,3 +137,304 @@
     return re.subn(u'[^\x09\x0A\x0D\u0020-\uD7FF\uE000-\uFFFD]+',
             lambda match: match.group(0).encode('unicode_escape'),
             message)
+
+
+def get_utf8_or_ascii(a_str, _encode_utf8=cache_utf8.encode):
+    """Return a cached version of the string.
+
+    cElementTree will return a plain string if the XML is plain ascii. It only
+    returns Unicode when it needs to. We want to work in utf-8 strings. So if
+    cElementTree returns a plain string, we can just return the cached version.
+    If it is Unicode, then we need to encode it.
+
+    :param a_str: An 8-bit string or Unicode as returned by
+                  cElementTree.Element.get()
+    :return: A utf-8 encoded 8-bit string.
+    """
+    # This is fairly optimized because we know what cElementTree does, this is
+    # not meant as a generic function for all cases. Because it is possible for
+    # an 8-bit string to not be ascii or valid utf8.
+    if a_str.__class__ is unicode:
+        return _encode_utf8(a_str)
+    else:
+        return intern(a_str)
+
+
+_utf8_re = lazy_regex.lazy_compile('[&<>\'\"]|[\x80-\xff]+')
+_unicode_re = lazy_regex.lazy_compile(u'[&<>\'\"\u0080-\uffff]')
+
+
+_xml_escape_map = {
+    "&":'&amp;',
+    "'":"&apos;", # FIXME: overkill
+    "\"":"&quot;",
+    "<":"&lt;",
+    ">":"&gt;",
+    }
+
+
+def _unicode_escape_replace(match, _map=_xml_escape_map):
+    """Replace a string of non-ascii, non XML safe characters with their escape
+
+    This will escape both Standard XML escapes, like <>"', etc.
+    As well as escaping non ascii characters, because ElementTree did.
+    This helps us remain compatible to older versions of bzr. We may change
+    our policy in the future, though.
+    """
+    # jam 20060816 Benchmarks show that try/KeyError is faster if you
+    # expect the entity to rarely miss. There is about a 10% difference
+    # in overall time. But if you miss frequently, then if None is much
+    # faster. For our use case, we *rarely* have a revision id, file id
+    # or path name that is unicode. So use try/KeyError.
+    try:
+        return _map[match.group()]
+    except KeyError:
+        return "&#%d;" % ord(match.group())
+
+
+def _utf8_escape_replace(match, _map=_xml_escape_map):
+    """Escape utf8 characters into XML safe ones.
+
+    This uses 2 tricks. It is either escaping "standard" characters, like "&<>,
+    or it is handling characters with the high-bit set. For ascii characters,
+    we just lookup the replacement in the dictionary. For everything else, we
+    decode back into Unicode, and then use the XML escape code.
+    """
+    try:
+        return _map[match.group()]
+    except KeyError:
+        return ''.join('&#%d;' % ord(uni_chr)
+                       for uni_chr in match.group().decode('utf8'))
+
+
+_to_escaped_map = {}
+
+def encode_and_escape(unicode_or_utf8_str, _map=_to_escaped_map):
+    """Encode the string into utf8, and escape invalid XML characters"""
+    # We frequently get entities we have not seen before, so it is better
+    # to check if None, rather than try/KeyError
+    text = _map.get(unicode_or_utf8_str)
+    if text is None:
+        if unicode_or_utf8_str.__class__ is unicode:
+            # The alternative policy is to do a regular UTF8 encoding
+            # and then escape only XML meta characters.
+            # Performance is equivalent once you use cache_utf8. *However*
+            # this makes the serialized texts incompatible with old versions
+            # of bzr. So no net gain. (Perhaps the read code would handle utf8
+            # better than entity escapes, but cElementTree seems to do just fine
+            # either way)
+            text = str(_unicode_re.sub(_unicode_escape_replace,
+                                       unicode_or_utf8_str)) + '"'
+        else:
+            # Plain strings are considered to already be in utf-8 so we do a
+            # slightly different method for escaping.
+            text = _utf8_re.sub(_utf8_escape_replace,
+                                unicode_or_utf8_str) + '"'
+        _map[unicode_or_utf8_str] = text
+    return text
+
+
+def _clear_cache():
+    """Clean out the unicode => escaped map"""
+    _to_escaped_map.clear()
+
+
+def unpack_inventory_entry(elt, entry_cache=None, return_from_cache=False):
+    elt_get = elt.get
+    file_id = elt_get('file_id')
+    revision = elt_get('revision')
+    # Check and see if we have already unpacked this exact entry
+    # Some timings for "repo.revision_trees(last_100_revs)"
+    #               bzr     mysql
+    #   unmodified  4.1s    40.8s
+    #   using lru   3.5s
+    #   using fifo  2.83s   29.1s
+    #   lru._cache  2.8s
+    #   dict        2.75s   26.8s
+    #   inv.add     2.5s    26.0s
+    #   no_copy     2.00s   20.5s
+    #   no_c,dict   1.95s   18.0s
+    # Note that a cache of 10k nodes is more than sufficient to hold all of
+    # the inventory for the last 100 revs for bzr, but not for mysql (20k
+    # is enough for mysql, which saves the same 2s as using a dict)
+
+    # Breakdown of mysql using time.clock()
+    #   4.1s    2 calls to element.get for file_id, revision_id
+    #   4.5s    cache_hit lookup
+    #   7.1s    InventoryFile.copy()
+    #   2.4s    InventoryDirectory.copy()
+    #   0.4s    decoding unique entries
+    #   1.6s    decoding entries after FIFO fills up
+    #   0.8s    Adding nodes to FIFO (including flushes)
+    #   0.1s    cache miss lookups
+    # Using an LRU cache
+    #   4.1s    2 calls to element.get for file_id, revision_id
+    #   9.9s    cache_hit lookup
+    #   10.8s   InventoryEntry.copy()
+    #   0.3s    cache miss lookus
+    #   1.2s    decoding entries
+    #   1.0s    adding nodes to LRU
+    if entry_cache is not None and revision is not None:
+        key = (file_id, revision)
+        try:
+            # We copy it, because some operations may mutate it
+            cached_ie = entry_cache[key]
+        except KeyError:
+            pass
+        else:
+            # Only copying directory entries drops us 2.85s => 2.35s
+            if return_from_cache:
+                if cached_ie.kind == 'directory':
+                    return cached_ie.copy()
+                return cached_ie
+            return cached_ie.copy()
+
+    kind = elt.tag
+    if not inventory.InventoryEntry.versionable_kind(kind):
+        raise AssertionError('unsupported entry kind %s' % kind)
+
+    file_id = get_utf8_or_ascii(file_id)
+    if revision is not None:
+        revision = get_utf8_or_ascii(revision)
+    parent_id = elt_get('parent_id')
+    if parent_id is not None:
+        parent_id = get_utf8_or_ascii(parent_id)
+
+    if kind == 'directory':
+        ie = inventory.InventoryDirectory(file_id,
+                                          elt_get('name'),
+                                          parent_id)
+    elif kind == 'file':
+        ie = inventory.InventoryFile(file_id,
+                                     elt_get('name'),
+                                     parent_id)
+        ie.text_sha1 = elt_get('text_sha1')
+        if elt_get('executable') == 'yes':
+            ie.executable = True
+        v = elt_get('text_size')
+        ie.text_size = v and int(v)
+    elif kind == 'symlink':
+        ie = inventory.InventoryLink(file_id,
+                                     elt_get('name'),
+                                     parent_id)
+        ie.symlink_target = elt_get('symlink_target')
+    else:
+        raise errors.UnsupportedInventoryKind(kind)
+    ie.revision = revision
+    if revision is not None and entry_cache is not None:
+        # We cache a copy() because callers like to mutate objects, and
+        # that would cause the item in cache to mutate as well.
+        # This has a small effect on many-inventory performance, because
+        # the majority fraction is spent in cache hits, not misses.
+        entry_cache[key] = ie.copy()
+
+    return ie
+
+
+def unpack_inventory_flat(elt, format_num, unpack_entry,
+            entry_cache=None, return_from_cache=False):
+    """Unpack a flat XML inventory.
+
+    :param elt: XML element for the inventory
+    :param format_num: Expected format number
+    :param unpack_entry: Function for unpacking inventory entries
+    :return: An inventory
+    :raise UnexpectedInventoryFormat: When unexpected elements or data is
+        encountered
+    """
+    if elt.tag != 'inventory':
+        raise errors.UnexpectedInventoryFormat('Root tag is %r' % elt.tag)
+    format = elt.get('format')
+    if format != format_num:
+        raise errors.UnexpectedInventoryFormat('Invalid format version %r'
+                                               % format)
+    revision_id = elt.get('revision_id')
+    if revision_id is not None:
+        revision_id = cache_utf8.encode(revision_id)
+    inv = inventory.Inventory(root_id=None, revision_id=revision_id)
+    for e in elt:
+        ie = unpack_entry(e, entry_cache, return_from_cache)
+        inv.add(ie)
+    return inv
+
+
+def serialize_inventory_flat(inv, append, root_id, supported_kinds, working):
+    """Serialize an inventory to a flat XML file.
+
+    :param inv: Inventory to serialize
+    :param append: Function for writing a line of output
+    :param working: If True skip history data - text_sha1, text_size,
+        reference_revision, symlink_target.    self._check_revisions(inv)
+    """
+    entries = inv.iter_entries()
+    # Skip the root
+    root_path, root_ie = entries.next()
+    for path, ie in entries:
+        if ie.parent_id != root_id:
+            parent_str = ' parent_id="'
+            parent_id  = encode_and_escape(ie.parent_id)
+        else:
+            parent_str = ''
+            parent_id  = ''
+        if ie.kind == 'file':
+            if ie.executable:
+                executable = ' executable="yes"'
+            else:
+                executable = ''
+            if not working:
+                append('<file%s file_id="%s name="%s%s%s revision="%s '
+                    'text_sha1="%s" text_size="%d" />\n' % (
+                    executable, encode_and_escape(ie.file_id),
+                    encode_and_escape(ie.name), parent_str, parent_id,
+                    encode_and_escape(ie.revision), ie.text_sha1,
+                    ie.text_size))
+            else:
+                append('<file%s file_id="%s name="%s%s%s />\n' % (
+                    executable, encode_and_escape(ie.file_id),
+                    encode_and_escape(ie.name), parent_str, parent_id))
+        elif ie.kind == 'directory':
+            if not working:
+                append('<directory file_id="%s name="%s%s%s revision="%s '
+                    '/>\n' % (
+                    encode_and_escape(ie.file_id),
+                    encode_and_escape(ie.name),
+                    parent_str, parent_id,
+                    encode_and_escape(ie.revision)))
+            else:
+                append('<directory file_id="%s name="%s%s%s />\n' % (
+                    encode_and_escape(ie.file_id),
+                    encode_and_escape(ie.name),
+                    parent_str, parent_id))
+        elif ie.kind == 'symlink':
+            if not working:
+                append('<symlink file_id="%s name="%s%s%s revision="%s '
+                    'symlink_target="%s />\n' % (
+                    encode_and_escape(ie.file_id),
+                    encode_and_escape(ie.name),
+                    parent_str, parent_id,
+                    encode_and_escape(ie.revision),
+                    encode_and_escape(ie.symlink_target)))
+            else:
+                append('<symlink file_id="%s name="%s%s%s />\n' % (
+                    encode_and_escape(ie.file_id),
+                    encode_and_escape(ie.name),
+                    parent_str, parent_id))
+        elif ie.kind == 'tree-reference':
+            if ie.kind not in supported_kinds:
+                raise errors.UnsupportedInventoryKind(ie.kind)
+            if not working:
+                append('<tree-reference file_id="%s name="%s%s%s '
+                    'revision="%s reference_revision="%s />\n' % (
+                    encode_and_escape(ie.file_id),
+                    encode_and_escape(ie.name),
+                    parent_str, parent_id,
+                    encode_and_escape(ie.revision),
+                    encode_and_escape(ie.reference_revision)))
+            else:
+                append('<tree-reference file_id="%s name="%s%s%s />\n' % (
+                    encode_and_escape(ie.file_id),
+                    encode_and_escape(ie.name),
+                    parent_str, parent_id))
+        else:
+            raise errors.UnsupportedInventoryKind(ie.kind)
+    append('</inventory>\n')

=== modified file 'doc/developers/case-insensitive-file-systems.txt'
--- doc/developers/case-insensitive-file-systems.txt	2010-08-13 19:08:57 +0000
+++ doc/developers/case-insensitive-file-systems.txt	2011-11-24 17:24:21 +0000
@@ -84,7 +84,7 @@
   of the result to match any existing items.
 
 * Tree.get_canonical_inventory_path - somewhat like Tree.get_symlink_target(),
-  Tree.get_file_by_path() etc; returns a name with the case adjusted to match
+  Tree.get_file() etc; returns a name with the case adjusted to match
   existing inventory items.
 
 * osutils.canonical_relpaths() and Tree.get_canonical_inventory_paths() - like

=== modified file 'doc/developers/configuration.txt'
--- doc/developers/configuration.txt	2011-08-19 14:10:32 +0000
+++ doc/developers/configuration.txt	2011-12-22 18:37:21 +0000
@@ -1,12 +1,178 @@
 Configuring Bazaar
 ==================
 
-A configuration option has:
-
-* a name: a valid python identifier (even if it's not used as an
-  identifier in python itself)
-
-* a value: a unicode string or a list of unicode strings.
+.. contents::
+   :depth: 2
+
+As a Bazaar developer there are a few things you need to know about
+configuration:
+
+* how to add a new option,
+
+* how add a new stack,
+
+* how add a new store.
+
+The first sections in this document summarize the steps needed when adding a
+new configuration item, the rest of the document gives more internal details
+on how this is implemented.
+
+Get an option value
+-------------------
+
+Options values are obtained with ``stack.get(option_name)`` where ``stack``
+is one of the daughter classes of ``config.Stack``, see their docstrings for
+a description of which sections are used from which stores.
+
+The value returned is of the type declared for that ``Option`` and if
+nothing is specifically declared you will get the default for that option.
+
+Add a new option
+----------------
+
+You add a new ``Option`` to the ``option_registry``, either inside
+``bzrlib/config.py`` or during initialization of your plugin (use
+``register_lazy`` in this case). New plugins should have systematic
+hierarchical names so that related values are grouped together::
+
+  option_registry.register(
+      Option('dirstate.fdatasync', default=True,
+            from_unicode=bool_from_store,
+            help="Flush dirstate changes onto physical disk? ...."))
+
+You then need to decide which stack is appropriate to implement the Option
+policy:
+
+* which config files (aka ``Store``) needs to be queried, which sections are
+  relevant and in what order,
+
+* which section will receive the modifications (if relevant).
+
+The docstrings for the existing stacks cover most of the known use cases.
+
+Modify an option value or delete an option
+------------------------------------------
+
+Just reading an option is what is needed most of the time, modifying option
+values or removing options is usually something that is not automated but
+left to the user (with ``bzr config``).
+
+Nevertheless, if you need to save a modified option value, use
+``.set(option_name, value)`` and ``.remove(option_name)`` to delete the
+option. Both methods are provided by the ``Stack`` object.
+
+But before doing that, you must be sure that the stack you're using have a
+writable section (this is true for ``GlobalStack`` which uses the
+``DEFAULT`` section in ``bazaar.conf`` and for ``BranchStack``which uses the
+no-name section in ``branch.conf``).
+
+Old and new configuration code
+------------------------------
+
+There is (as of late 2011) some older and some newer configuration code. The
+old code has specific methods for various checks or uses classes like
+``GlobalConfig``.  Don't add to to it; try to remove it.
+
+If you encounter an option using the old code you may want to migrate
+it. This generally involves:
+
+* registering the option,
+
+* replace the old config by a stack:
+
+  * ``GlobalConfig`` became ``GlobalStack``,
+
+  * ``LocationConfig`` became ``LocationStack``,
+
+  * ``BranchConfig`` became ``BranchStack`` (or in this case,
+    ``get_config()`` became ``get_config_stack()``.
+
+* replace the custom accessor calls with ``conf.get(option_name)``.
+
+The new config code provides some help for commonly encountered use cases
+that can allow further simplifications like:
+
+* providing a default value when the option is not defined in any way by the
+  user,
+
+* convert the unicode string provided by the user into a suitable
+  representation (integer, list, etc).
+
+Adding a new stack
+------------------
+
+Stacks capture the various places an option can be declared by the user with
+associated levels of generality and query them in the appropriate order
+returning the first definition found. For example, the
+``append_revisions_only`` option may be declared for all branches of a user
+in ``bazaar.conf``, or for a hierarchy of branches in ``locations.conf`` or
+in a single branch in ``branch.conf``.
+
+Defining a new stack means you need a new way to expose these levels to the
+user that is not covered by the existing stacks.
+
+This is achieved by declaring:
+
+* which stores can provide a value for the option,
+
+* which sections apply to the stack instance, some filtering for a given
+  context can be defined,
+
+* which (store, section) should receive the modifications.
+
+Mapping different sections to different stacks is a powerful way to organize
+the options and provide various levels of configuration to the user. This is
+achieved with ``Store`` and ``SectionMatcher`` objects.
+
+
+Adding a new store
+------------------
+
+The following stores are used by ``bzr`` in ways that illustrate various
+uses of sections.
+
+bazaar.conf
+===========
+
+``bzr`` itself defines two sections here:
+
+* ``DEFAULT`` where global options are defined,
+
+* ``ALIASES`` where command aliases are defined. This section is *not*
+  available via ``GlobalStack``, instead, the ``bzr alias`` command uses it
+  for its own purposes.
+
+Plugins can define either additional options in the ``DEFAULT`` section or
+new sections for their own needs (this is not especially encouraged
+though). The ``bzr-bookmarks`` plugin defines a ``BOOKMARKS`` section there
+for example.
+
+pkgimport.conf
+==============
+
+The Ubuntu package importer defines a store and two stacks involving
+``pkgimport.conf``. A no-name section contains the options common to all
+packages and sections named after their corresponding package can also be
+defined.
+
+The ``ImporterStack`` uses ``locations.conf`` and the no-name section in
+``pkgimport.conf`` for the importer options.
+
+The ``PackageStack`` uses only ``pkgimport.conf`` and uses the section name
+after the package followed by the no-name section.
+
+location.conf
+=============
+
+``bzr`` defines sections corresponding to URLs there and includes the
+relevant sections in ``LocationStack`` and ``BranchStack``. No no-name
+section is recognized in this file.
+
+branch.conf
+===========
+
+This file defines the option for a given branch and uses only the no-name
+section.
 
 Option
 ------
@@ -16,8 +182,17 @@
 * name: a name: a valid python identifier (even if it's not used as an
   identifier in python itself). This is also used to register the option.
 
-* default: the default value that Stack.get() should return if no
-  value can be found for the option.
+* from_unicode: a callable accepting a unicode string and returning a
+  suitable value for the option. If the string cannot be coerced it should
+  return None.
+
+* override_from_env: a list of environment variables. The first variable set
+  will be used as the option value overriding any other definition of the
+  option.
+
+* default: the default value that Stack.get() should return if no value can
+  be found for the option. This can also be a callable as long as it returns
+  a unicode string.
 
 * default_from_env: a list of environment variables. The first variable set
   will provide a default value overriding 'default' which remains the
@@ -25,15 +200,20 @@
 
 * help: a doc string describing the option, the first line should be a
   summary and can be followed by a blank line and a more detailed
-  explanation.
+  explanation. This will be displayed to the user with::
 
-* from_unicode: a callable accepting a unicode string and returning a
-  suitable value for the option. If the string cannot be coerced it should
-  return None.
+    bzr help <option name>
 
 * invalid: the action to be taken when an invalid value is encountered in a
   store (during a Stack.get()).
 
+The value of an option is a unicode string or ``None`` if it's not
+defined. By using ``from_unicode`` you can turn this string into a more
+appropriate representation.
+
+If you need a list value, you should use ``ListOption`` instead.
+
+
 Sections
 --------
 
@@ -47,6 +227,7 @@
 MutableSection is needed to set or remove an option, ReadOnlySection should
 be used otherwise.
 
+
 Stores
 ------
 
@@ -63,11 +244,11 @@
 A ``Store`` can contain one or more sections, each section is uniquely
 identified by a unicode string.
 
-``config.ConfigObjStore`` is an implementation that use ``ConfigObj``.
+``config.IniFileStore`` is an implementation that use ``ConfigObj``.
 
 Depending on the object it is associated with (or not) a ``Store`` also needs
-to implement a locking mechanism. ``LockableConfigObjStore`` implements such a
-mechanism for ``ConfigObj`` based stores.
+to implement a locking mechanism. ``LockableIniFileStore`` implements such a
+mechanism for ``IniFileStore`` based stores.
 
 Classes are provided for the usual Bazaar configuration files and could be
 used as examples to define new ones if needed. The associated tests provides a
@@ -75,29 +256,62 @@
 places to inherit from the existing basic tests and add their own specific
 ones.
 
+A ``Store`` defines how option values are stored, this includes:
+
+* defining the sections where the options are grouped,
+
+* defining how the values are quoted/unquoted for storage purposes. Stacks
+  use the unquoted values internally (default value handling and option
+  expansion are simpler this way) and ``bzr config`` quote them when they
+  need to be displayed.
+
+
 Filtering sections
 ------------------
 
-For some contexts, only some sections from a given store will apply. Defining
-which is what the ``SectionMatcher`` are about.
+For some contexts, only some sections from a given store will apply. The
+``SectionMatcher`` objects are used to define which sections in a store
+apply to a given context.
 
 The main constraint here is that a ``SectionMatcher`` should delay the loading
 of the associated store as long as possible. The constructor should collect
 all data needed for the selection and uses it while processing the sections in
 ``get_sections``.
 
-Only ``ReadOnlySection`` objects are manipulated here but a ``SectionMatcher``
-can return dedicated ``Section`` to provide additional context (the
-``LocationSection`` add an ``extra_path`` attribute to implement the
-``appendpath`` policy for example).
-
-.. FIXME: Replace the appendpath example if/when it's deprecated ;)
+Only ``ReadOnlySection`` objects are manipulated here but a
+``SectionMatcher`` can return dedicated ``Section`` objects to provide
+additional context (the ``LocationSection`` add an ``extra_path`` attribute
+to implement the section local options for example). If no sections match,
+an empty list is returned.
+
+Options local to a section can be defined for special purposes and be
+handled by ``Section.get()``. One such option is ``relpath`` which is
+defined in ``LocationSection`` as an alternative to the ``appendpath``
+policy.
+
+For ``appendpath``, the ``LocationSection`` will carry ``extra_path`` as the
+relative path between the section name and the location used. ``relpath``
+will be available as a ``Section`` local option with the same
+value. ``basename`` will carry the location base name and be available as a
+local option with the same name. Note that such options can only be expanded
+inside the section that defines them.
+
+Specific section matchers can be implemented by overriding ``get_sections``
+or just ``match``.
+
+``bzrlib`` provides:
+
+* ``LocationMatcher(store, location)``: To select all sections that match
+  ``location``.
+
+* ``NameMatcher(store, unique_id)``: To select a single section matching
+  ``unique_id``.
 
 Stacks
 ------
 
-An option can take different values depending on the context it is used. Such
-a context can involve configuration files, options from the command line,
+An option can take different values depending on the context it is
+used. This can involve configuration files, options from the command line,
 default values in bzrlib and then some.
 
 Such a context is implemented by creating a list of ``Section`` stacked upon
@@ -128,3 +342,11 @@
 ``Stores`` can be used to build them but shouldn't be used otherwise, ditto
 for sections. Again, the associated tests could and should be used against the
 created stacks.
+
+Also note that ``MemoryStack`` can be used without any disk resources which
+allows for simpler and faster tests. A common pattern is to accept a
+``config`` parameter related to a given feature and test it with predefined
+configurations without involving the whole
+stack. ``bzrlib.tests.test_commit``, ``bzrlib.tests.test_gpg`` and
+``bzrlib.tests.test_smtp_connection`` are good examples.
+

=== added file 'doc/developers/feature-flags.txt'
--- doc/developers/feature-flags.txt	1970-01-01 00:00:00 +0000
+++ doc/developers/feature-flags.txt	2011-12-24 12:51:20 +0000
@@ -0,0 +1,150 @@
+Format feature flags
+====================
+
+.. contents::
+
+Rationale
+---------
+
+In the past when new features were added that required a change to the
+on-disk format, Bazaar has introduced a new format (in other words,
+a different string in .bzr/branch-format, .bzr/branch/format,
+.bzr/repository/format or .bzr/checkout/format). The main reason for this
+was that it made old versions of Bazaar give a sensible error message
+when they encountered on-disk data that they could not understand.
+
+There are also several disadvantages to such an approach:
+
+ * Once upgraded, a newer version of Bazaar is required to access the data
+   (it is often possible to downgrade the format later on)
+
+ * Upgrading requires an explicit action by the user. It could be
+   done automatically, but then accessing a repository with a newer version
+   of Bazaar might accidentally render it inaccessible by older.
+
+Not all format changes should necessarily render the data inaccessible
+to older versions of Bazaar.
+
+There are also various plugins that store extra metadata in the Bazaar
+version control directory. They currently have no way of indicating that
+e.g. writing to the repository requires a particular plugin to be installed
+(so the metadata can be kept up to date, for example).
+
+Proposed approach
+-----------------
+
+To allow for more granular changes to the format, this spec proposes
+to add feature flags to the Bazaar formats, indicating
+what kind of data is present in that repository. Each feature has
+a name and some sort of indicator that tells the bzr client its
+"necessity" - optional, required, ...
+
+bzr clients would be able to open data with features that are
+set as "optional" but that they do not support. If there are features
+that aren't supported which are marked "required" in the repository they
+would refuse to open the repository.
+
+Various kinds of metadata, e.g. ones that are generated from the
+repository itself and can be discarded without losing data (caches)
+would fall in the optional category.
+
+Feature necessity
+-----------------
+
+The initial implementation will feature the following two possible
+settings for feature ``necessity``. Any format necessity that can't
+be understood should be interpreted as "required", and an appropriate
+warning printed.
+
+ - optional: the feature is present, but writing/reading of the
+      repository/branch/checkout is possible without support for the
+      feature. Useful for things like caches (e.g. bzr-search index,
+      annotate cache)
+ - required: read and write access is only possible if the feature
+      is supported. Useful for things like nested trees.
+
+In the future, we might add more values for necessity. Older
+versions of bzr treat unknown necessities as "required". Some likely
+candidates for new necessities that might be added in the future:
+
+ - read-optional: read access is possible if the feature is not supported,
+      but write access requires it
+ - client-read-optional: directly writing to the object requires
+      the feature, but reading or writing through an intermediary (such as
+      a HPSS server) doesn't.
+
+Format changes
+--------------
+
+The feature information would be included in the appropriate ``format`` file
+(``.bzr/branch-format``, ``.bzr/branch/format``, ``.bzr/repository/format`` or
+``.bzr/checkout/format``). This file currently always contains a single
+line with the format name. Older versions of bzr read the full file.
+
+By using the other lines for feature information it is possible to add feature
+flags in a backwards compatible manner; older clients will simply fail to open
+repositories with feature flags set, giving a unknown format error.
+
+The other advantage of doing this is that we don't need any additional
+roundtrips when opening a remote format. An example .bzr/repository/format
+file could then look like this::
+
+  Bazaar repository format 2a (needs bzr 1.16 or later)
+  optional git
+  optional search
+  optional tiplog
+  required nested-trees
+
+In other words, this is a "2a" bzr format which also stores a cache of
+Git Tree/Commit objects, a bzr-search index, and a reflog. It also contains
+revisions with nested trees.
+
+API Changes
+-----------
+
+Class methods will be added to ``BzrFormat`` to allow registering
+and unregistering the presence of particular features.
+
+ * BzrFormat.register_feature(name)
+ * BzrFormat.unregister_feature(name)
+
+The namespace for features is global. It is assumed
+that the plugin that provides the feature X provides that feature
+in all objects that it is relevant for. For example, if a plugin
+provides the ``nested-trees`` feature, it is assumed to support
+that in both working trees and repositories. If this is not the case,
+it should use a different feature name for the working tree support
+and the repository support.
+
+BzrFormat is inherited by ``BranchFormatMetaDir``, ``BzrDirFormat``,
+``RepositoryFormatMetaDir`` and ``WorkingTreeFormatMetaDir``.
+
+Upon opening, BzrFormat will be responsible for checking that the
+required features are present.  lock_write will raise an exception
+when there is an unsupported mandatory feature required for write access.
+
+Methods will also be added to BzrFormat to allow plugins, etc,
+to check whether a feature is present and adding new features:
+
+ * BzrFormat.features.set(name, necessity)
+ * BzrFormat.features.get(name) -> necessity
+
+Enabling features
+-----------------
+
+Features are enabled through the ``update_feature_flags`` method on
+``Repository``, ``Branch``, ``WorkingTree`` and ``BzrDir``.
+
+These methods are called by whatever needs to enable features.
+When they do that is up to them - e.g. bzr-search would enable its
+feature when ``bzr index`` is first run.
+
+See also
+--------
+
+Mercurial has a similar feature, using its `.hg/requires`_ file.
+
+.. _.hg/requires: http://mercurial.selenic.com/wiki/RequiresFile
+
+..
+   vim: ft=rst tw=74 ai

=== modified file 'doc/developers/network-protocol.txt'
--- doc/developers/network-protocol.txt	2010-11-12 22:46:28 +0000
+++ doc/developers/network-protocol.txt	2011-11-29 14:59:18 +0000
@@ -11,7 +11,7 @@
 Overview
 ========
 
-The smart protocol provides a way to send a requests and corresponding
+The smart protocol provides a way to send requests and corresponding
 responses to communicate with a remote bzr process.
 
 Layering
@@ -103,7 +103,7 @@
 The domain logic is in `bzrlib.remote`: `RemoteBzrDir`, `RemoteBranch`,
 and so on.
 
-There is also an plain file-level transport that calls remote methods to
+There is also a plain file-level transport that calls remote methods to
 manipulate files on the server in `bzrlib.transport.remote`.
 
 Protocol description

=== added file 'doc/developers/new-config-rationale.txt'
--- doc/developers/new-config-rationale.txt	1970-01-01 00:00:00 +0000
+++ doc/developers/new-config-rationale.txt	2011-12-14 16:30:09 +0000
@@ -0,0 +1,790 @@
+================================
+Simplifying Bazaar Configuration
+================================
+
+Goal
+====
+
+Not all needs can be addressed by the default values used inside bzr and
+bzrlib, no matter how well they are chosen (and they are ;).
+
+Options that are rarely used don't deserve a corresponding command line
+switch in one or several commands.
+
+Many parts of ``bzrlib`` depends on some constants though and the user
+should be able to customize the behavior to suit his needs so these
+constants need to become configuration options or more generally, be easier
+to set.
+
+These options can be set from the command-line, acquired from an environment
+variable or recorded in a configuration file.
+
+To simplify writing (and reading), this document refers to the old and new
+config designs:
+* the old design is using ``Config`` as a base class for all config files,
+* the new design use ``ConfigStacks`` of ``Section`` from config ``Stores``.
+
+
+Current issues
+==============
+
+* Many parts of ``bzrlib`` declare constants and there is no way for the
+  user to look at or modify them (see http://pad.lv/832061).
+
+* The old design requires a configuration object to create, modify or delete
+  a configuration option in a given configuration file.  ``bzr config``
+  makes it almost transparent for the user. Internally though, not all cases
+  are handled: only BranchConfig implements chained configs, nothing is
+  provided at the repository level and too many plugins define their own
+  section or even their own config file. (config.Stack now provides a way to
+  chain config files, BranchStack properly implements the desired behavior,
+  ``bzr config`` uses the new design).
+
+* ``locations.conf`` defines the options that need to override any setting
+  in ``branch.conf`` for both local and remotes branches (but some remote
+  branch options just ignore ``locations.conf``). Many users want a way to
+  define default values for options that are not defined in ``branch.conf``
+  (and even more users think that ``locations.conf`` provide default values,
+  see also http://pad.lv/843211 and http://pad.lv/832046). This could be
+  approximated today by *not* defining these options in ``branch.conf`` but
+  in ``locations.conf`` instead. This workaround doesn't allow a user to
+  define defaults in ``locations.conf`` and override them in
+  ``branch.conf``. (Allowing sections in 'bazaar.conf' (or introducing a new
+  defaults.conf' allowing sections) can now address that. Defining and using
+  a new file is easier as it avoids handling a migration path for
+  bazaar.conf and doesn't require banning the use of sections for special
+  purpose needs (ALIASES and BOOKMARKS for example)).
+
+* Defining a new option requires adding a new method in the ``Config``
+  object to get access to features like:
+
+  * should the option be inherited by more specific sections, (this was more
+    or less the default in the old design, it is addressed by section
+    matchers in the new one by letting users define options in whatever
+    relevant section and let the matcher select the right ones).
+
+  * should the inherited value append the relative path between the section
+    one and the location it applies to (see http://pad.lv/832013, fixed),
+
+  * the default value (including calling any python code that may be
+    required to calculate this value)(see http://pad.lv/832064, fixed),
+
+  * priority between sections in various config files (this is defined by
+    the section matcher associated with a given config store for stacks,
+    http://pad.lv/832046 is about adding a section matcher with clearer
+    semantics than the one used for locations.conf).
+
+  A related problem is that, in the actual implementation, some
+  configuration options have defined methods, others don't and this is
+  inconsistent. (Using only Stacks addresses that).
+
+* Access to the 'active' configuration option value from the command line
+  doesn't give access to the specific section. This is a concern if the user
+  has no other way to address a specific configuration option including
+  Store and Section when using ``bzr config`` (see
+  http://pad.lv/725234). Plugins defining their own stacks and/or stores
+  also have no way to properly plug into ``bzr config`` (see
+  http://pad.lv/788991).
+
+* Rules for configuration options are not clearly defined for remote
+  branches (they may differ between dumb and smart servers the former will
+  use the local ``bazaar.conf`` and ``locations.conf`` files while the later
+  will use (or ignore ?) the remote ones).
+
+* The features offered by the Bazaar configuration files should be easily
+  accessible to plugin authors either by supporting plugin configuration
+  options in the configuration files or allowing the plugins to define their
+  own configuration files. (Separating Section, Store and Stack starts
+  addressing that, a stack registry should complete the needed means
+  http://pad.lv/832036).
+
+* While the actual configuration files support sections, they are used in
+  mutually exclusive ways that make it impossible to offer the same set of
+  features to all configuration files:
+
+  * ``bazaar.conf`` use arbitrary names for sections. ``DEFAULT`` is used
+    for global options, ``ALIASES`` are used to define command aliases,
+    plugins can define their own sections, some plugins do that
+    (``bzr-bookmarks`` use ``BOOKMARKS`` for example), some other define
+    their own sections (this is addressed with the new design by using only
+    the ``DEFAULT`` section and ignore the others. When needed, one can
+    create a specific stack to get access to a specific section).
+
+  * ``locations.conf`` use globs as section names. This provides an easy
+    way to associate a set of options to a matching working tree or
+    branch, including remote ones.
+
+  * ``branch.conf`` doesn't use any section.
+
+  This is addressed by defining different stacks selecting the relevant
+  sections from the stores involved. ``ALIASES`` for example can define a
+  stack that select only the ``ALIASES`` section from ``bazaar.conf``.
+
+* There is no easy way to get configuration options for a given repository
+  or an arbitrary path. Working trees and branches are generally organized
+  in hierarchies and being able to share the option definitions is an often
+  required feature. This can also address some needs exhibited by various
+  branch schemes like looms, pipeline, colocated branches and nested
+  trees. Being able to specify options *in* a working tree could also help
+  support conflict resolution options for a given file, directory or
+  subtree (see http://pad.lv/359320).
+
+* Since sections allow different definitions for the same option (in the
+  same store), a total order should be defined between sections to select
+  the right definition for a given context (paths or globs for
+  ``locations.conf`` but other schemes can be used, window names for qbzr,
+  repository UUIDs for bzr-svn for example). Allowing globs for section
+  names is harmful in this respect since the order is currently defined as
+  being based on the number of path components. The caveat here is that if
+  the order is always defined for a given set of sections it can change when
+  one or several globs are modified and the user may get surprising and
+  unwanted results in these cases. The lexicographical order is otherwise
+  fine to define what section is more specific than another. (This may not
+  be a problem in real life since longer globs are generally more specific
+  than shorter ones and explicit paths should also be longer than matching
+  globs. That may leave a glob and a path of equal length in a gray area but
+  in practice using ``bzr config`` should give enough feedback to address
+  them. See also http://pad.lv/832046 asking for a less magical section
+  matcher).
+
+* Internally, configuration files (and their fallbacks, ``bazaar.conf`` and
+  ``locations.conf`` for ``branch.conf``) are read every time *one* option is
+  queried. Likewise, setting or deleting a configuration option implies
+  writing the configuration file *immediately* after re-reading the file to
+  avoid racing updates (see http://pad.lv/832042).
+
+* The current implementation use a mix of transport-based and direct file
+  systems operations (Addressed by Store implementation relying on
+  transports only and the hpss implementing the corresponding verbs).
+
+* While the underlying ``ConfigObj`` implementation provides an
+  interpolation feature, the ``bzrlib`` implementation doesn't provide an
+  easy handling of templates where other configuration options can be
+  interpolated. Instead, ``locations.conf`` (and only it) allows for
+  ``appendpath`` and ``norecurse``. (Cross-section, cross-file interpolation
+  and section local options are now implemented in the new design).
+
+* Inherited list values can't be modified, a more specific configuration can
+  only redefine the whole list.
+
+* There is no easy way to define dicts (the most obvious one being to use a
+  dedicated section which is already overloaded). Using embedded sections
+  for this would not be practical either if we keep using a no-name section
+  for default values. In a few known cases, a bencoded dict is stored in a
+  config value, so while this isn't user-friendly, not providing a better
+  alternative shouldn't be a concern. A possible, limited, implementation
+  can be envisioned: limiting the dict to a single level only, with simple
+  names as keys and unicode strings as values. The keys can then be mapped
+  to options prefixed with the dict name.
+
+
+Proposed implementation
+=======================
+
+
+Configuration files definition
+------------------------------
+
+While of course configurations files can be versioned they are not intended
+to be accessed in sync with the files they refer to (one can imagine
+handling versioned properties this way but this is *not* what the bazaar
+configuration files are targeted at). ``bzr`` will always refer to
+configuration files as they exist on disk when an option is queried or set.
+
+The configuration files are generally local to the file system but some of
+them can be accessed remotely (``branch.conf``, ``repo.conf``).
+
+
+Naming
+------
+
+Option names are organized into a name space for a given stack. One such set
+includes ``bazaar.conf``, ``locations.conf``, ``branch.conf``, etc. Plugins
+can define their own sets for their own needs. While it is conceivable that
+the same option name can be used in unrelated configuration stacks, it seems
+better to define a single name space for all options if only to avoid
+ambiguities.
+
+Using a name space is meant to help:
+
+* avoid collisions between bzr and plugins and between plugins,
+
+* discover the available options and making them easier to remember,
+
+* organise the documentation for the option set.
+
+Using valid python identifiers is recommended but not enforced (but we may
+do so in the future).
+
+The option name space is organized by topic:
+
+* bzrlib options are grouped by topic (``branch``, ``tree``, ``repo``)
+
+* plugins are encouraged (but not required) to prefix their specific options
+  with their name (``qbzr.`` for qbzr)
+
+* collisions are detected at registration time so users are protected from
+  incompatibilities between plugins,
+
+* options that need to be used by several plugins (or shared between ``bzr``
+  core and plugins) should be discussed but these discussions are already
+  happening so the risk of misuse is low enough.
+
+Value
+-----
+
+All option values are text. They are provided as Unicode strings to API
+users with some refinements:
+
+* boolean values can be obtained for a set of acceptable strings (yes/no,
+  y/n, on/off, etc), (implemented with the ``from_unicode`` parameter)
+
+* a list of strings from a value containing a comma separated list of
+  strings.
+
+Since the configuration files can be edited by the user, ``bzr`` doesn't
+expect their content to be valid at all times. Instead, the code using
+options should be ready to handle *invalid* values by warning the user and
+falling back to a default value.
+
+Likely, if an option is not defined in any configuration file, the code
+should fallback to a default value (helpers should be provided by the API to
+handle common cases: warning the user, getting a particular type of value,
+returning a default value)(most of that is now handled at Option definition).
+
+This also ensures compatibility with values provided via environment
+variables or from the command line (where no validation can be expected
+either)(done in the new design).
+
+
+Option expansion
+----------------
+
+Some option values can be templates and contain references to other
+options. This is especially useful to define URLs in sections shared for
+multiple branches for example. It can also be used to describe commands
+where some parameters are set by ``bzrlib`` at runtime.
+
+Since option values are text-only, and to avoid clashing with other option
+expansion (also known as interpolation) syntaxes, references are enclosed
+with curly brackets::
+
+  push_location = lp:~{launchpad_username}/bzr/{nick}
+
+In the example above, ``launchpad_username`` is an already defined
+configuration option while ``nick`` is the branch nickname and is set when a
+configuration applies to a given branch.
+
+The interpolation implementation should accept an additional dict so that
+``bzrlib`` or plugins can define references that can be expanded without
+being existing configuration options::
+
+  diff_command={cmd} {cmd_opts} {file_a} {file_b}
+
+There are two common errors that should be handled when handling interpolation:
+
+* loops: when a configuration value refers to itself, directly or indirectly,
+
+* undefined references: when a configuration value refers to an unknown option.
+
+The loop handling can be modified to allow cross-sections and cross-files
+interpolation: if an option refers to itself (directly or indirectly) during
+an expansion, the fallback sections or files can be queried for its value.
+
+This allows list values to refer to the definition in the less specific
+configurations::
+
+  bazaar.conf:
+    debug_flags = hpss
+
+  branch.conf for mybranch:
+    debug_flags = {debug_flags}, hpssdetail
+
+  $ bzr -d mybranch config debug_flags
+  hpss, hpssdetail
+
+Undefined references are detected if they are not defined in any
+configuration. This will trigger errors while displaying the value. Diagnosing
+typos should be doable in this case.
+
+Configuration file syntax
+-------------------------
+
+The configuration file is mostly an ``ini-file``. It contains ``name = value``
+lines grouped in sections. A section starts with a string enclosed in squared
+brackets ('[section_name]`), this string uniquely identifies the section in
+the file. Comments are allowed by prefixing them with the '#' character.
+
+A section is named by the path (or some other unuique identifier) it should
+apply to (more examples below).
+
+When sections are used, they provide a finer grain of configuration by
+defining option sets that apply to some working trees, branches,
+repositories (or any kind of context) or part of them. The relationship
+between a given context and the sections it applies to is defined by the
+config file.
+
+So far, Bazaar uses a glob in ``locations.conf`` and select the sections
+that apply to a given url (or a local path).
+
+The subset is defined by the common leading path or a glob.
+
+Different kinds of section names can be defined:
+
+* a full url: used to described options for remote branches and
+  repositories (LocationMatcher supports this).
+
+* local absolute path: used for working trees, branches or repositories
+  on the local disks (LocationMatcher supports this).
+
+* relative path: the path is relative to the configuration file and can be
+  used for colocated branches or threads in a loom, i.e any working tree,
+  branch or repository that is located in a place related to the
+  configuration file path. Some configuration files may define this path
+  relationship in specific ways to make them easier to use (i.e. if a config
+  file is somewhere below ``.bzr`` and refers to threads in a loom for
+  example, the relative path would be the thread name, it doesn't have to be
+  an *exact* relative path, as long as its interpretation is unambiguous and
+  clear for the user) (No section matchers support this so far, needs to
+  file a bug)
+
+Section matching
+----------------
+
+Section names define another name space (than the option names one) with an
+entirely different purpose: in a given configuration file, for a given
+context, only some sections will be relevant and will be queried in a
+specific order.
+
+This matching is specific to each config file and implemented by the
+SectionMatcher objects.
+
+Whatever this matching does, the user can observe the results with the ``bzr
+config`` command which displays the sections in the order they are queried.
+
+LocationMatcher
+~~~~~~~~~~~~~~~
+
+The context here is either:
+
+* an URL,
+
+* a local path.
+
+Note that for both the provided context and the section names, if an URL uses
+a ``file:///`` form, it is converted to a local path.
+
+The sections names can use globs for each path component
+(i.e. ``/dir/*/subdir`` is allowed but ``/dir/\*\*/subdir`` will match
+``/dir/a/subdir`` but not ``/dir/a/b/subdir``. 
+
+The reason is that the ordering is defined by sorting the section names
+matching the context on the number of path components followed by the path
+itself in lexicographical order. This results in most specific sections being
+queried before the more generic ones.
+
+PathMatcher
+~~~~~~~~~~~
+
+``LocationMatcher`` has some obscure (for unaware users) edge cases and
+limitations that can be surprising. ``PathMatcher`` aims at addressing these
+issues by providing simpler rules while still giving full control to the
+user (http://pad.lv/832046).
+
+The context here is a local path, absolute or relative. If the path is
+relative it is interpreted from the file base directory.
+
+Note that 'base directory' for configuration files in Bazaar directories is
+really:
+
+* the home directory for files under ``~/.bazaar``,
+
+* the ``.bzr`` base directory for files under ``.bzr``.
+
+The order is the one observed in the file so most generic values are specified
+first and most specific ones last. As such, the order in the file is the
+opposite of the one displayed by ``bzr config`` which displays most specific
+values first. This seems to be the most natural order in both cases.
+
+A section matches if the section name is a prefix of the context path
+(relative paths being converted to absolute on the fly).
+
+The Option object
+-----------------
+
+(copied from a recent version of bzr.dev for easier reading, refer to the
+original for an up to date version)
+
+The Option object is used to define its properties:
+
+* name: a name: a valid python identifier (even if it's not used as an
+  identifier in python itself). This is also used to register the option.
+
+* default: the default value that Stack.get() should return if no
+  value can be found for the option.
+
+* default_from_env: a list of environment variables. The first variable set
+  will provide a default value overriding 'default' which remains the
+  default value if *no* environment variable is set.
+
+* help: a doc string describing the option, the first line should be a
+  summary and can be followed by a blank line and a more detailed
+  explanation.
+
+* from_unicode: a callable accepting a unicode string and returning a
+  suitable value for the option. If the string cannot be coerced it should
+  return None.
+
+* invalid: the action to be taken when an invalid value is encountered in a
+  store (during a Stack.get()).
+
+The Section object
+------------------
+
+Options are grouped into sections which share some properties with the well
+known dict objects:
+
+* the key is the name,
+* you can get, set and remove an option,
+* the value is a unicode string.
+
+MutableSection is needed to set or remove an option, ReadOnlySection should
+be used otherwise.
+
+The Store object
+----------------
+
+This is an implementation-level object that should rarely be used directly.
+
+* it can be local or remote
+
+* locking
+
+  All lock operations should be implemented via transport objects. (True for
+  Store).
+
+* option life cycle
+
+  Working trees, branches and repositories should define a config attribute
+  following the same life cycle as their lock: the associated config file is
+  read once and written once if needed. This should minimize the file system
+  accesses or the network requests. There is no known racing scenarios for
+  configuration options, changing the existing implementation to this less
+  constrained one shouldn't introduce any. Yet, in order to detect such
+  racing scenarios, we can add a check that the current content of the
+  configuration file is the expected one before writing the new content and
+  emit warnings if differences occur. The checks should be performed for the
+  modified values only. As of today (and in the foreseeable future), the
+  size of the configuration files are small enough to be kept in memory (see
+  http://pad.lv/832042).
+
+The Stack object
+-----------------
+
+This the object that provides access to the needed features:
+
+* getting an option value,
+
+* setting an option value,
+
+* deleting an option value,
+
+* handling a list of configuration files and for each of them a section
+  matcher providing the sections that should be tried in the given order to
+  find an option.
+
+* handling a Store and a section where option creation, modification and
+  deletion will occur.
+
+Depending on the files involved, a working tree, branch or repository object
+(or more generally a context) should be provided to access the corresponding
+configuration files. Note that providing a working tree object also
+implicitly provides the associated branch and repository object so only one
+of them is required (or none for configuration files specific to the user
+like ``bazaar.conf``).
+
+Getting an option value
+~~~~~~~~~~~~~~~~~~~~~~~
+
+Depending on the option, there are various places where it can be defined
+and several ways to override these settings when needed.
+
+The following lists all possible places where a configuration option can
+be defined, but some options will make sense in only some of them. The
+first to define a value for an option wins (None is therefore used to
+express that an option is not set).
+
+* command-line
+  ``-Ooption=value`` see http://pad.lv/491196.
+
+* ``~/.bazaar/locations.conf``
+
+  When an option is set in ``locations.conf`` it overrides any other
+  configuration file. This should be used with care as it allows setting a
+  different value than what is recommended by the project
+
+* ``tree`` (Not Implemented Yet)
+
+  The options related to the working tree.
+
+  This includes all options related to commits, ignored files, junk files,
+  etc.
+
+  Note that the sections defined there can use relative paths if some
+  options should apply to a subtree or some specific files only.
+
+  See http://pad.lv/430538 and http://pad.lv/654998.
+
+* ``branch`` located in ``.bzr/branch/branch.conf``
+
+  The options related to the branch.
+
+  Sections can be defined for colocated branches or loom threads.
+
+* ``repository`` (Not Implemented Yet)
+
+  The options related to the repository.
+
+  Using an option to describe whether or not a repository is shared could
+  help address http://pad.lv/342119 but this will probably requires a format
+  bump).
+
+* ``project`` (Not Implemented Yet)
+
+  The options common to all branches and working trees for a project.
+
+* ``organization`` (Not Implemented Yet)
+
+  The options common to all branches and working trees for an organization.
+
+  See http://pad.lv/419854.
+
+* ``system`` (Not Implemented Yet but see http://pad.lv/419854 and
+  https://code.launchpad.net/~thomir/bzr/add-global-config/+merge/69592)
+
+  The options common to all users of a system (may be /etc/bzr/defaults
+  or /usr/local/etc/bzr/defaults or
+  /Library/Preferences/com.canonical.defaults  or c:\windows\bazaar.conf
+  (someone fix this one please ;) depending on the OS).
+
+* ``bazaar.conf``
+
+  The options the user has selected for the host he is using.
+
+  Sections can be defined for both remote and local branches to define
+  default values (i.e. the most common use of ``locations.conf`` today).
+
+* default (implemented by the OptionRegistry)
+
+  The options defined in the ``bzr`` source code.
+
+  This will be implemented via the Option objects.
+
+Plugins can define additional configuration files as they see fit and
+insert them in this list, see their documentation for details.
+
+Compatibility
+=============
+
+There are ways to keep the same files while ensuring compatibility via various
+tricks but there are cases where using new files to replace the old ones is
+definitely easier:
+
+* no need to ensure that the new files are correctly handled by old bzr
+  versions,
+
+* making it clear for users that there is a switch and let them migrate at
+  their own pace.
+
+The known cases so far are described below.
+
+Obvious at this point:
+
+* Branch provides ``get_config`` for the old design and ``get_config_stack``
+  for the new design so that both designs are supported. Once the store
+  sharing is implemented, we may want to use an attribute for the stack and
+  deprecate both ``get_config`` and ``get_config_stack``.
+
+* Sections names in ``bazaar.conf`` are arbitrary (except ``DEFAULT``) so
+  it's easier to leave the file untouched and let plugin authors and users
+  migrate away (or not) from them. For ``bzr`` itself, that means
+  ``DEFAULT`` is the only section used for most of the options and provides
+  user defaults. ``ALIASES`` requires a specific stack but only the ``bzr
+  alias`` command cares about that.
+
+* Option policies should be deprecated:
+
+  * The ``norecurse`` policy is useless, all options are recursive by
+    default. If specific values are needed for specific paths, they can just
+    be defined as such (in the appropriate sections or files).
+
+  * The ``appendpath`` policy should be implemented via interpolation and a
+    ``relpath`` option provided by the configuration framework
+    (http://pad.lv/832013).
+
+* Section order in ``locations.conf`` has issues which make a migration to a
+  different way to organize the sections (hence the file content) far easier
+  with a new file.
+
+* ``locations.conf`` is really for overrides but many users have been using it
+  to provide defaults. There is no way to know if the whole content has been
+  used for defaults or overrides or a mix of both. So there is no way to
+  migrate this automatically.
+
+Unclear at this point:
+
+* [BOOKMARKS] section can be replaced by ``bookmarks.xxx`` options (the
+  bookmarks plugins already uses ``bookmarks_xxx`` in branch.conf since no
+  sections were supported there). The easiest here is probably to just merge
+  the plugin into core and use the appropriate option names consistently. A
+  ``config:`` directory service may even be better as any option can be used
+  as a bookmark. This allows things like::
+
+    [/whatever/path]
+    my_push = lp:<launchpad.login>/xxx/{nick}
+    web_site=ftp://example.com/
+
+    bzr push config:web_site
+
+  Which means we completely replace the plugin and don't need to care about
+  migrating the section.
+
+* [ALIASES] section can be replaced by corresponding bzr.alias.xxx
+  options. This could be automated by creating the corresponding options ?
+
+* I don't know about other sections, feedback welcome. Plugin authors are
+  encouraged to migrate to the new name space scheme by prefixing their
+  options with their plugin name.
+
+Notes
+=====
+
+These are random notes about concepts, ideas or issues not implemented yet.
+
+Developer facing concepts
+-------------------------
+
+Option
+~~~~~~
+
+* list of allowed Config IDs (this allows a list of possible config files in
+  bazaar.conf only option and use it while bootstrapping the config
+  creations). 
+
+* blacklist of config IDs (some options *can't* be stored (modified) by the
+  user)
+
+An alternative is to just let the devs decide which stack they use for a
+given option, ``stacked_on_location`` for example is said to relate to the
+branch only and changing it or setting it in a different config file may not
+be appropriate. This may not be a good example as there is also the
+``default_stack_on`` option which can be set only in ``control.conf``
+though...
+
+Stack
+~~~~~
+
+* a lazy cache for the option values (should be reset on modifications as
+  interpolations will make it tricky to update incrementally) (see FIXME in
+  config.py Stack.get()))
+
+* ensures that the Stores involved generate as less IOs as possible (see
+  http://pad.lv/832042)
+
+* ensures that the transaction is the object life time (i.e. modifications
+  will be taken into account *iff* they are committed explicitly).
+
+StackRegistry
+~~~~~~~~~~~~~
+
+* ensures that a config ID is a unique identifier
+* register Stacks
+
+Store
+~~~~~
+
+* ensures that the transaction is the object life time (i.e. modifications
+  will be taken into account *iff* they are committed explicitly).
+
+Examples
+--------
+
+store examples:
+~~~~~~~~~~~~~~~
+
+* ConfigObj (bazaar.conf)
+
+* DB (<scheme>://bazaar.launchpad.net/bazaar.conf)
+
+
+Why and when locking config files matter
+----------------------------------------
+
+This is relevant for http://pad.lv/832042.
+
+``bzr`` behavior, as well as the objects it acts upon, is configured via a
+set of so-called configuration files.
+
+These files allow to define working trees, branches and repositories, their
+relationships and how ``bzr`` should handle them.
+
+The default behavior of ``bzr`` is aimed at making this configuration as
+transparent as possible by keeping track of how these objects are created
+and modified when they are used. In short, they are useless until you want
+to change the default behavior in some specific context.
+
+We mostly **read** config options. Therefore all we care about is to
+guarantee that:
+
+* we get a valid config file at all times when reading,
+
+* we always leave a valid config file when writing (via the rename dance)
+
+From there, conceptually, all operations can clearly define whether or not
+they need to modify a config file and do so only when they succeed. All
+modifications occurring during such an operation are delayed until the very
+end of the operation.
+
+Now, we want to minimize the overlapping times where one bzr operation has
+changed a value and another concurrent operation is unaware of this
+modification.
+
+These overlapping periods are *as of today* rare.
+
+The only known case, http://pad.lv/525571 has been fixed in bzr-2.1.3. The
+bug there was triggered when two processes tried to write the same config
+file at the same time leaving an invalid file in the end.
+
+Such a period can be recognized and detected though: when changing an option
+value, if the preserved original value is different in the config file,
+someone else modified it and the operation can be invalid because it relied
+on the original value.
+
+For the sake of the example, if an option value represent a global unique ID
+via a simple counter (very bad idea), if two operations try to increment it,
+both will use the same value that won't be unique anymore. Checking the
+value present in the file when trying to save the updated value with
+identify such a collision.
+
+An assumption is floating around: it should be enough to report when an
+operation is modifying an already modified option and observe that no-one
+reports such occurrences.
+
+Note that this assumption is made in a context where *no* known scenarios
+exist in the bzr code base not in any plugin (for a best effort value of
+'any', feedback highly welcome, bug reports even ;)
+
+With this in mind, we can change the definition of config options, stores
+and stacks to ensure that:
+
+* a config file is read only once when in read access,
+
+* a config file is read only once and written only once when in write
+  access, adding the check mentioned above will require *one* additional
+  read.
+
+A reader can then safely assume that reading a config file gives it a valid
+(and coherent) definition of the configuration when the operation
+starts. All the operation has to do is to declare which config files may be
+modified by an operation (whether or not we can be liberal on this 'may be'
+is yet to be defined).

=== modified file 'doc/developers/plans.txt'
--- doc/developers/plans.txt	2010-07-07 11:21:19 +0000
+++ doc/developers/plans.txt	2011-12-01 11:41:07 +0000
@@ -2,28 +2,12 @@
 =====
 
 .. toctree::
-   :hidden:
+   :maxdepth: 1
 
-   performance-roadmap
+   Performance roadmap <performance-roadmap>
    colocated-branches
+   feature-flags
+   new-config-rationale
    tortoise-strategy
    improved_chk_index
 
-
-* `Performance roadmap <performance-roadmap.html>`_ |--| The roadmap
-  for fixing performance in bzr over the next few releases.
-
-* `Co-located branches <colocated-branches.html>`_ |--| Planned(?) support
-  for storing multiple branches in one file-system directory.
-
-* `Bazaar Windows Shell Extension Options <tortoise-strategy.html>`_ |--|
-  Implmentation strategy for Bazaar Windows Shell Extensions, aka
-  TortoiseBzr.
-
-* `CHK Optimized index <improved_chk_index.html>`_
-
-
-.. |--| unicode:: U+2014
-
-..
-   vim: ft=rst tw=74 ai

=== modified file 'doc/developers/releasing.txt'
--- doc/developers/releasing.txt	2011-08-04 11:46:39 +0000
+++ doc/developers/releasing.txt	2011-11-15 17:14:30 +0000
@@ -206,6 +206,18 @@
    also mail the list to raise this issue in our process. Milestones are
    found at <https://launchpad.net/bzr/+milestone/x.y.z>.
 
+#. Merge into your branch all previous stable series fixes that haven't been
+   merged yet. For example, if you're releasing 2.5.x, make sure the fixes
+   on 2.4, 2.3, etc have already been merged up::
+
+     bzr merge lp:bzr/2.4
+
+   and commit that merge in its own commit. This should happen only if the
+   devs landing changes in previous releases forgot to merge them up. Since
+   this can slow down the freeze, feel free to gently remind them about
+   their duties ;) If you feel unsafe resolving the conflicts or it's too
+   time consuming, contact the related devs and skip this merge.
+ 
 #. In the release branch, update  ``version_info`` in ``./bzrlib/__init__.py``.
    Make sure the corresponding milestone exists.
    Double check that ./bzr ``_script_version`` matches ``version_info``. Check
@@ -225,9 +237,16 @@
 
 #. Update the ``./doc/en/release-notes/`` section for this release.
 
+   Check that all news entries related to this release have been added in
+   the right section. For example, if you're releasing 2.5b2, the following
+   command should display a a single chuk diff for the 2.5b2 release::
+
+     bzr diff -rbzr-2.5b1.. doc/en/release-notes/bzr-2.5.txt
+
    Fill out the date and a description of the release under the existing
-   header. If there isn't one, follow the instructions above for using the
-   ``release-template.txt`` file.
+   header (the diff above will help you summarizing). If there isn't one,
+   follow the instructions above for using the ``release-template.txt`` file
+   and remind people that they should document their changes there ;)
 
    See *2.1.1* or similar for an example of what this looks like.
 
@@ -244,7 +263,20 @@
    Released' or setting a different milestone if the bug hasn't been
    fixed). A few false positives may remain in the older series, don't let
    this slow you down too much. This script accepts options you may find
-   useful, use ``./tools/check-newsbugs.py`` to display its usage.
+   useful, use ``./tools/check-newsbugs.py`` to display its usage (``-w``
+   will open each bug in your browser for example).
+
+#. For beta releases update the translation template::
+
+     BZR_PLUGIN_PATH=-site make po/bzr.pot
+
+   This is especially important for the final beta release which is
+   when translations are frozen and translators are requested to make
+   the translations.
+
+#. For stable releases update the translations::
+
+     bzr merge lp:~bzr-core/bzr/bzr-translations-export-x.y
 
 #. Commit these changes to the release branch, using a command like::
 
@@ -330,6 +362,10 @@
      bzr push
      bzr pqm-submit -m "(vila) Release 2.3.1 (Vincent Ladeuil)"
 
+   Note that ``bzr push`` should mention updating one tag (which you just
+   created). If it doesn't, double-check that you created (and pushed) this
+   tag.
+
    Or with hydrazine::
 
      bzr lp-propose -m "Release 1.14" --approve lp:bzr/1.14
@@ -357,6 +393,15 @@
    ``BZR_DISABLE_PLUGINS`` or ``BZR_PLUGIN_PATH=-site`` to disable one or
    all plugins.
 
+   Until <http://pad.lv/839461> is fixed, you may encounter issues if you
+   cut a release for old stable branches (<= 2.2) and use a more recent
+   OS/distro. If that's the case, check the bug status and use the following
+   workaround if no fix is available::
+
+     export TTPATH=<local branch of lp:testtools -r 0.9.2>
+     export SUPATH=<local branch of lp:subunit -r 0.0.6>
+     PYTHONPATH=$TTPATH:$SUPATH/python PATH=$SUPATH/filters:${PATH} BZR_PLUGIN_PATH=-site make check-dist-tarball PYTHON=python2.6 | subunit2pyunit
+
    Remember that PQM has just tested everything too, this step is
    particularly testing that the pyrex extensions, which are updated
    by your local pyrex version when you run make dist, are in good
@@ -397,6 +442,9 @@
 You may also need to do `At the start of a series cycle`_ if you're starting
 a new series.
 
+The final beta - branching and translations
+-------------------------------------------
+
 A word of caution: the instructions above works well for all releases but
 there is one special case that requires a bit more care: when you release
 the *last* beta for a given ``x.y`` series (from trunk aka lp:bzr), you need
@@ -427,17 +475,48 @@
 
 #. Open ``lp:bzr/x.y`` for bug fixes
 
+You also need to ensure Launchpad is set up to import/export
+translations for the new branch and inform translators.
+
+#. Push a branch::
+
+     bzr push lp:~bzr-core/bzr/bzr-translations-export-x.y
+
+#. On the translations series synchronization settings page
+   <https://translations.launchpad.net/bzr/x.y/+translations-settings>
+   turn on ``Import template files`` then for exports click ``Choose a
+   target branch`` and point it at the branch you just pushed.
+
+#. E-mail translators to announce that the forthcoming stable release
+   of bzr is ready for translations.  Send to
+   ``translators@lists.launchpad.net`` and
+   ``ubuntu-translators@lists.ubuntu.com``.
+
+#. The series is now frozen for strings and API, see below for adding
+   that to the announcement.
+
 Announcing the source freeze
 ----------------------------
 
-#. Post to the ``bazaar`` list, saying that the source has been frozen
-   (gone gold). Be extra clear that this is only a *source* release
-   targeted at packagers and installer builders (see
-   <https://bugs.launchpad.net/launchpad/+bug/645084>).  This is the cue
-   for platform maintainers and plugin authors to update their code.  This
-   is done before the general public announcement of the release.
-
-#. Pause for a few days.
+#. Post to the ``bazaar@lists.canonical.com`` and
+   ``bzr-packagers@list.launchpad.net`` lists, saying that the source has
+   been frozen. Be extra clear that this is only a *source* release targeted
+   at packagers and installer builders (see
+   <https://bugs.launchpad.net/launchpad/+bug/645084>).  This is the cue for
+   platform maintainers and plugin authors to update their code.  This is
+   done before the general public announcement of the release.
+
+   The freeze announcement generally guess the date of the official public
+   announcement, for the most recent stable series (the one supported by the
+   installers and most of the distributions) it's generally a few days after
+   the freeze. For older series supported only via SRUs for Ubuntu, we don't
+   control the process as tightly so guessing the date is not appropriate.
+
+   For the final beta release include in your announcement a notice of
+   API and translation freezes nothing that public methods should not
+   be removed or changed and strings should not be added or changed.
+
+#. Pause for a few days. 
 
 
 Publishing the release
@@ -467,14 +546,11 @@
 
 #. Make an announcement mail.
 
-   For beta releases, this is sent to the ``bazaar@lists.canonical.com``
-   list only to inform plugin authors and people responsible for building
-   packages or installers.
-
-   Once the installers are available, the mail can be sent to the
-   ``bazaar-announce`` list too.
-
-   For stable releases, it should also be cc'd to ``info-gnu@gnu.org``,
+   For beta releases, this is sent to the ``bazaar@lists.canonical.com`` and
+   ``bazaar-announce@lists.canonical.com`` lists.
+
+   For stable releases (excluding SRUs which are for older stable releases),
+   it should also be cc'd to ``info-gnu@gnu.org``,
    ``python-announce-list@python.org``, ``bug-directory@gnu.org``.
 
    In all cases, it is good to set ``Reply-To: bazaar@lists.canonical.com``,
@@ -486,7 +562,10 @@
 
       The Bazaar team is happy to announce availability of a new
       release of the bzr adaptive version control system.
-      Bazaar is part of the GNU system <http://gnu.org/>.
+
+      Bazaar <http://bazaar.canonical.com/> is a Canonical project and part
+      of the GNU project <http://gnu.org/> to produce a free operating
+      system.
 
       <<Summary paragraph from news>>
 
@@ -503,13 +582,14 @@
 
 #. Make an announcement through <https://launchpad.net/bzr/+announce>
 
-#. Announce on http://freshmeat.net/projects/bzr/
+#. Announce on http://freecode.com/projects/bazaar-vcs
 
    This should be done for beta releases and stable releases. If you do not
-   have a Freshmeat account yet, ask one of the existing admins.
+   have a Freecode account yet, ask one of the existing admins.
 
-   The purpose here is to point users to the latest stable release while still
-   publishing announcements for beta releases.
+   The purpose here is to point users to the latest stable release
+   (i.e. SRUs are excluded) while still publishing announcements for beta
+   releases.
 
    There are several kinds of modifications that could be done there via the
    ``Administration`` box in the lower right area of the page:
@@ -542,7 +622,7 @@
        python setup.py register
 
    Remember to check the results afterward -- this should be done for
-   stable releases but not for beta releases.
+   stable releases but not for beta releases nor SRUs.
 
    To be able to register the release you must create an account on
    <http://pypi.python.org/pypi> and have one of the existing owners of

=== modified file 'doc/developers/testing.txt'
--- doc/developers/testing.txt	2011-08-04 00:28:09 +0000
+++ doc/developers/testing.txt	2011-09-14 16:38:55 +0000
@@ -745,6 +745,22 @@
         _test_needs_features = [features.apport]
 
 
+Testing translations
+-----------------------
+
+Translations are disabled by default in tests.  If you want to test
+that code is translated you can use the ``ZzzTranslations`` class from
+``test_i18n``::
+
+    self.overrideAttr(i18n, '_translations', ZzzTranslations())
+
+And check the output strings look like ``u"zz\xe5{{output}}"``.
+
+To test the gettext setup and usage you override i18n.installed back
+to self.i18nInstalled and _translations to None, see
+test_i18n.TestInstall.
+
+
 Testing deprecated code
 -----------------------
 

=== modified file 'doc/en/_templates/index.html'
--- doc/en/_templates/index.html	2011-02-03 01:02:16 +0000
+++ doc/en/_templates/index.html	2011-08-30 09:27:59 +0000
@@ -7,7 +7,7 @@
 
   <table class="contentstable" align="center" style="margin-left: 30px"><tr>
     <td width="50%">
-        <p class="biglink"><a class="biglink" href="{{ pathto("whats-new/whats-new-in-2.4") }}">What's New in Bazaar 2.4?</a><br/>
+        <p class="biglink"><a class="biglink" href="{{ pathto("whats-new/whats-new-in-2.5") }}">What's New in Bazaar 2.5?</a><br/>
         <span class="linkdescr">what's new in this release</span>
       </p>
         <p class="biglink"><a class="biglink" href="{{ pathto("user-guide/index") }}">User Guide</a><br/>

=== modified file 'doc/en/admin-guide/simple-setups.txt'
--- doc/en/admin-guide/simple-setups.txt	2011-03-24 00:13:10 +0000
+++ doc/en/admin-guide/simple-setups.txt	2011-08-24 09:34:35 +0000
@@ -112,7 +112,7 @@
 
 The following example shows how a single line is configured::
 
-  command="bzr --serve --inet --allow-writes --directory=/srv/bzr",no-agent-forwarding,no-port-forwarding,no-pty,no-user-rc,no-X11-forwarding ssh-rsa AAA...= my bzr key
+  command="bzr serve --inet --allow-writes --directory=/srv/bzr",no-agent-forwarding,no-port-forwarding,no-pty,no-user-rc,no-X11-forwarding ssh-rsa AAA...= my bzr key
 
 This command allows the user to access only bzr and disables other SSH use.  Write
 access to each repository in the directory ``/srv/bzr`` has been granted with ``--allow-writes``
@@ -122,9 +122,9 @@
 for Alice and Bob.  This method will not allow you to restrict access to part
 of a repository, you may only restrict access to a single part of the directory structure::
 
-  command="bzr --serve --inet --allow-writes --directory=/srv/bzr/alice/",no-agent-forwarding,no-port-forwarding,no-pty,no-user-rc,no-X11-forwarding ssh-rsa AAA...= Alice's SSH Key
-  command="bzr --serve --inet --allow-writes --directory=/srv/bzr/bob/",no-agent-forwarding,no-port-forwarding,no-pty,no-user-rc,no-X11-forwarding ssh-rsa AAA...= Bob's SSH Key
-  command="bzr --serve --inet --allow-writes --directory=/srv/bzr/",no-agent-forwarding,no-port-forwarding,no-pty,no-user-rc,no-X11-forwarding ssh-rsa AAA...= Repo Manager SSH Key
+  command="bzr serve --inet --allow-writes --directory=/srv/bzr/alice/",no-agent-forwarding,no-port-forwarding,no-pty,no-user-rc,no-X11-forwarding ssh-rsa AAA...= Alice's SSH Key
+  command="bzr serve --inet --allow-writes --directory=/srv/bzr/bob/",no-agent-forwarding,no-port-forwarding,no-pty,no-user-rc,no-X11-forwarding ssh-rsa AAA...= Bob's SSH Key
+  command="bzr serve --inet --allow-writes --directory=/srv/bzr/",no-agent-forwarding,no-port-forwarding,no-pty,no-user-rc,no-X11-forwarding ssh-rsa AAA...= Repo Manager SSH Key
 
 Alice and Bob have access to their own repository and Repo Manager
 has access to the each of their repositories.  Users are not allowed access to any part of 

=== modified file 'doc/en/mini-tutorial/index.txt'
--- doc/en/mini-tutorial/index.txt	2011-07-01 22:01:47 +0000
+++ doc/en/mini-tutorial/index.txt	2011-10-26 16:44:52 +0000
@@ -65,10 +65,14 @@
 
 ::
 
-  bzr init-repo sample
-  cd sample
-  bzr init trunk
-  cd trunk
+  $ bzr init-repo sample
+  Shared repository with trees (format: 2a)
+  Location:
+    shared repository: sample
+  $ bzr init sample/trunk
+  $ cd sample/trunk
+  Created a repository tree (format: 2a)                                         
+  Using shared repository: /home/john/sample/
 
 
 Adding files
@@ -93,7 +97,15 @@
 
 Let's change a file and commit that change to your branch.
 
-Edit ``test1.txt`` in your favourite editor, then check what have you done::
+Edit ``test1.txt`` in your favourite editor, then use ``bzr add`` to tell bzr
+to track changes to this file ::
+
+  $ echo test test test > test1.txt 
+  $ bzr add test1.txt
+  adding test1.txt
+
+`bzr diff` shows the changes between the last revision in this branch, and your
+current tree (or, with the ``-r`` option, between any two trees). ::
 
  $ bzr diff
  === modified file 'test1.txt'
@@ -105,8 +117,9 @@
 Commit your work to the Bazaar branch::
 
   $ bzr commit -m "Added first line of text"
-  Committed revision 2.
-
+  Committing to: /home/john/sample/trunk/                             
+  added test1.txt
+  Committed revision 1.
 
 Viewing the revision log
 ========================
@@ -114,17 +127,9 @@
 You can see the history of your branch by browsing its log::
 
   $ bzr log
-  ------------------------------------------------------------
-  revno: 2
-  committer: John Doe <john.doe@gmail.com>
-  branch nick: myproject
-  timestamp: Mon 2007-10-08 17:56:14 +0000
-  message:
-    Added first line of text
-  ------------------------------------------------------------
   revno: 1
   committer: John Doe <john.doe@gmail.com>
-  branch nick: myproject
+  branch nick: trunk
   timestamp: Mon 2006-10-08 17:46:22 +0000
   message:
     Initial import
@@ -137,28 +142,46 @@
 software projects. You can use it to publish your branch.  (You can 
 also publish branches onto your own server or other hosting services.)
 
-If you don't have a Launchpad account, follow the `account signup guide`_
-and `register an SSH key`_ in your new Launchpad account.
-
-.. _account signup guide: https://help.launchpad.net/CreatingYourLaunchpadAccount
-.. _register an SSH key: https://launchpad.net/people/+me/+editsshkeys
-
-Replacing ``john.doe`` with your own Launchpad username, type::
-
- $ bzr push lp:~john.doe/+junk/myproject
-
-**Note**: ``+junk`` is a place to store experimental branches not
-associated with any particular project.  Normally, you should push a
-project into an existing project, or register a new project through the
-web interface.
-
-Now, anyone can create their own copy of your branch by typing::
-
- $ bzr branch lp:~john.doe/+junk/myproject
-
-You can also see information about your branch, including its revision
-history, at https://code.launchpad.net/people/+me/+junk/myproject
-
+The steps to publishing branches on Launchpad are:
+
+1. Create a Launchpad account: visit the `Launchpad login page`_ and choose to create a new account.
+    
+.. _Launchpad login page: https://launchpad.net/+login
+    
+2. Bazaar uses the SSH encryption and authentication protocol to connect
+   to Launchpad.  You need to first `create an SSH key`_ on your own computer,
+   by running the command::
+    
+       $ ssh-keygen
+
+.. _create an SSH key: https://help.launchpad.net/YourAccount/CreatingAnSSHKeyPair    
+       
+3. `Upload your SSH public key to Launchpad`_.
+ 
+.. _Upload your SSH public key to Launchpad: https://launchpad.net/~/+editsshkeys
+    
+4. `Make a team for your project`_.  Even if you're starting as the only
+   developer on this project, creating a new now will let you more easily 
+   add other people later.
+ 
+.. _Make a team for your project: https://help.launchpad.net/Teams/CreatingAndRunning
+      
+5. `Create a project`_.
+  
+.. _Create a project: https://help.launchpad.net/Projects/Registering
+
+6. Tell Bazaar your Launchpad account name.  If your account is john.doe, type ::
+  
+      $ bzr launchpad-login john.doe
+
+7. `Push the branch for your project`_.  Once you've committed your changes 
+   locally, you can publish them as the trunk of your new project by saying
+    
+       $ bzr push lp:~sample-developers/sample/trunk
+       
+   (Of course, using the team and project names you just chose.)
+ 
+.. _Push the branch for your project: https://help.launchpad.net/Code/UploadingABranch
 
 Creating your own copy of another branch
 ========================================

=== modified file 'doc/en/release-notes/bzr-2.2.txt'
--- doc/en/release-notes/bzr-2.2.txt	2011-07-06 09:22:00 +0000
+++ doc/en/release-notes/bzr-2.2.txt	2011-11-29 12:06:33 +0000
@@ -5,18 +5,56 @@
 .. toctree::
    :maxdepth: 1
 
+bzr 2.2.6
+#########
+
+:2.2.6: NOT RELEASED YET
+
+Compatibility Breaks
+********************
+
+New Features
+************
+
+Bug Fixes
+*********
+
+Improvements
+************
+
+Documentation
+*************
+
+API Changes
+***********
+
+Internals
+*********
+
+Testing
+*******
+
+
 bzr 2.2.5
 #########
 
-:Codename: Suggestions welcome
-:2.2.5: NOT RELEASED YET
+:2.2.5: 2011-09-01
+
+This is a bugfix release. One regression introduced in 2.2b1 has been fixed
+for some rare conflict resolutions. Also a warning is now emmitted when
+branching an out-of-date ubuntu packaging branch. Upgrading is recommended
+for all users on earlier 2.2 releases.
 
 Compatibility Breaks
 ********************
 
+None.
+
 New Features
 ************
 
+None.
+
 Bug Fixes
 *********
 
@@ -27,15 +65,6 @@
 * Don't crash while merging and encountering obscure path conflicts
   involving different root-ids. (Vincent Ladeuil, #805809)
 
-Improvements
-************
-
-Documentation
-*************
-
-API Changes
-***********
-
 Internals
 *********
 
@@ -43,9 +72,6 @@
   in the value string. Fix suggested by ConfigObj's author Michael Foord.
   (Alexander Belchenko, #710410)
 
-Testing
-*******
-
 
 bzr 2.2.4
 #########

=== modified file 'doc/en/release-notes/bzr-2.3.txt'
--- doc/en/release-notes/bzr-2.3.txt	2011-08-20 18:29:34 +0000
+++ doc/en/release-notes/bzr-2.3.txt	2011-12-10 21:21:31 +0000
@@ -35,6 +35,9 @@
 * Cope cleanly with buggy HTTP proxies that close the socket in the middle
   of a multipart response.  (Martin Pool, #198646).
 
+* Fix "Unprintable exception" error when a RetryWithNewPacks error is
+  displayed.  (Andrew Bennetts)
+
 Documentation
 *************
 
@@ -87,6 +90,11 @@
 .. Improvements to existing commands, especially improved performance 
    or memory usage, or better results.
 
+* Tweak an RPC implementation for ``Repository.get_parent_map``, it was
+  doing an inefficient ``small_set.difference_update(large_set)`` when we
+  can do ``small_set = small_set.difference(large_set)``. This speeds up
+  discovery time by about 10%. (John Arbash Meinel)
+
 Bug Fixes
 *********
 

=== modified file 'doc/en/release-notes/bzr-2.4.txt'
--- doc/en/release-notes/bzr-2.4.txt	2011-08-28 15:33:21 +0000
+++ doc/en/release-notes/bzr-2.4.txt	2011-12-05 14:21:55 +0000
@@ -5,10 +5,167 @@
 .. toctree::
    :maxdepth: 1
 
+bzr 2.4.3
+#########
+
+:2.4.2: NOT RELEASED YET
+
+External Compatibility Breaks
+*****************************
+
+.. These may require users to change the way they use Bazaar.
+
+New Features
+************
+
+.. New commands, options, etc that users may wish to try out.
+
+Improvements
+************
+
+.. Improvements to existing commands, especially improved performance 
+   or memory usage, or better results.
+
+Bug Fixes
+*********
+
+.. Fixes for situations where bzr would previously crash or give incorrect
+   or undesirable results.
+
+* Cope with Unix filesystems, such as smbfs, where chmod gives 'permission
+  denied'.  (Martin Pool, #606537)
+
+* When the ``limbo`` or ``pending-deletion`` directories exist, typically
+  because of an interrupted tree update, but are empty, bzr no longer
+  errors out, because there is nothing for the user to clean up.  Also,
+  errors in creation of these directories are no longer squelched.
+  (Martin Pool, #427773)
+
+* During merges, when two entries end up using the same path for two
+  different file-ids (the same file being 'bzr added' in two different
+  branches) , 'duplicate' conflicts are created instead of 'content'
+  ones. This was previously leading to a 'Malformed tramsform' exception.
+  (Vincent Ladeuil, #880701)
+
+* 'Malformed transform' exceptions are now recognized as internal errors
+  instead of user errors and report a traceback. This will reduce user
+  confusion as there is generally nothing users can do about them.
+  (Vincent Ladeuil, #880701)
+
+* Prevent a traceback being printed to stderr when logging has problems and
+  accept utf-8 byte string without breaking. (Martin Packman, #714449)
+
+Documentation
+*************
+
+.. Improved or updated documentation.
+
+API Changes
+***********
+
+.. Changes that may require updates in plugins or other code that uses
+   bzrlib.
+
+Internals
+*********
+
+.. Major internal changes, unlikely to be visible to users or plugin 
+   developers, but interesting for bzr developers.
+
+Testing
+*******
+
+.. Fixes and changes that are only relevant to bzr's test framework and 
+   suite.  This can include new facilities for writing tests, fixes to 
+   spurious test failures and changes to the way things should be tested.
+
+
+bzr 2.4.2
+#########
+
+:2.4.2: 2011-10-27
+
+This is a bugfix release. Most of the bugs dealt with portability
+issues. Upgrading is recommended for all users of earlier 2.4 releases.
+
+External Compatibility Breaks
+*****************************
+
+None.
+
+New Features
+************
+
+None.
+
+Improvements
+************
+
+* Fixed a bug where ``bzr tags -r x..y`` loaded the branch history once for
+  every revision in the range; it's now much faster. (Vincent Ladeuil, #857335)
+
+Bug Fixes
+*********
+
+* Fixed an infinite loop when creating a repo at the root of the filesystem,
+  i.e. "/", due to posixpath.normpath() not collapsing 2 leading slashes into
+  one, thus respecting the POSIX standard, but making relpath() loop infinitely.
+  (Florian Vichot, #861008)
+
+* Fixed loading of external merge tools from config to properly decode
+  command-lines which contain embedded quotes. (Gordon Tyler, #828803)
+
+* Include declaration of 'changed' to avoid an UnboundLocalError in dirstate
+  pyrex code with new Cython versions. (Denys Duchier, #837221)
+
+* Prevent several kinds of OverflowError and other fallout from failing to fit
+  stat fields into four bytes in dirstate pack_stat implementations.
+  (Martin Packman, #683191 #706957)
+
+* Return early from create_delta_index_from_delta given tiny inputs. This
+  avoids raising a spurious MemoryError on certain platforms such as AIX.
+  (John Arbash Meinel, #856731)
+
+Documentation
+*************
+
+* Corrected documentation for ``bzr serve`` in the Admin Guide.
+  (Morten Bøgeskov, Martin Pool, #832576)
+
+API Changes
+***********
+
+None.
+
+Internals
+*********
+
+No changes.
+
+Testing
+*******
+
+* Accept both old and new style testtools output in selftest tests.
+  (Jelmer Vernooij, Martin Packman, #815423)
+
+* Fix the race for TestingThreadingTCPServer in
+  test_server_crash_while_responding. (Vincent Ladeuil, #869366)
+
+* Really corrupt the pack file without depending on a special length or value.
+  (Vincent Ladeuil, #807032)
+
+
 bzr 2.4.1
 #########
 
-:2.4.1: NOT RELEASED YET
+:2.4.1: 2011-09-08
+
+This is a bugfix release. Upgrading is recommended for all users of earlier
+2.4 releases.
+
+It includes fixes from previous stable releases and address some issues with
+the test suite.
+
 
 External Compatibility Breaks
 *****************************
@@ -38,6 +195,10 @@
 * ``dirstate.fdatasync`` and ``repository.fdatasync`` can now properly be
   disabled. (Vincent Ladeuil, #824513)
 
+* Disable ``os.fsync`` and ``os.fdatasync`` by default when running
+  ``bzr selftest``. You can use ``--sync`` to re-enable them.
+  (John Arbash Meinel, #837293)
+
 * Fix i18n use when no environment variables are set. (Jelmer Vernooij, #810701)
 
 * Avoid UnicodeDecode error when reporting EINVAL from transports.
@@ -70,6 +231,10 @@
    suite.  This can include new facilities for writing tests, fixes to 
    spurious test failures and changes to the way things should be tested.
 
+* The test suite should now be able to run under weird environments where
+  ``/etc/passwd`` doesn't contain the ``uid`` for the user running selftest
+  or where ``fakeroot`` is used but ``/root`` is inacessible.
+  (Vincent Ladeuil, #825027)
 
 bzr 2.4.0
 #########

=== modified file 'doc/en/release-notes/bzr-2.5.txt'
--- doc/en/release-notes/bzr-2.5.txt	2011-08-27 19:08:54 +0000
+++ doc/en/release-notes/bzr-2.5.txt	2012-01-04 16:02:14 +0000
@@ -5,26 +5,698 @@
 .. toctree::
    :maxdepth: 1
 
-bzr 2.5b1
+bzr 2.5b5
 #########
 
-:2.5b1: NOT RELEASED YET
+:2.5b5: NOT RELEASED YET
 
 External Compatibility Breaks
 *****************************
 
 .. These may require users to change the way they use Bazaar.
 
+* The '.bzr/branch/email' file is no longer read to determine the users'
+  identity. Instead, the 'email' setting in '.bzr/branch/branch.conf'
+  should be used. (Jelmer Vernooij, #903894)
+
 New Features
 ************
 
 .. New commands, options, etc that users may wish to try out.
 
-* A ``from_unicode`` parameter can be specified when registering a config
-  option. This implements boolean and integer config options when the
-  provided ``bool_from_store`` and ``int_from_store`` are used.
+* "bzr mkdir" now includes -p (--parents) option for recursively adding
+  parent directories.
+  (Jared Hance, Jelmer Vernooij, #253529)
+
+* ``config.Option`` can now declare ``override_from_env``, a list of
+  environment variables which, when set, that takes precedence over values
+  defined in configuration files. (Vincent Ladeuil, #907279)
+
+Improvements
+************
+
+.. Improvements to existing commands, especially improved performance 
+   or memory usage, or better results.
+
+* New HPSS call for ``Repository.reconcile``. (Jelmer Vernooij, #894455)
+
+* Merge now has two new hooks ``pre_merge`` and ``post_merge``
+  that are called before and after a merge and can make
+  additional modifications to the trees involved.
+  (Jelmer Vernooij, #906877)
+
+* Override the value returned by ``sys.getfilesystemencoding()`` for the bzr
+  script to utf-8 when it would otherwise be ascii on a posix system. This
+  will mean bzr works with non-ascii files when no locale or an incorrect
+  locale is set. (Martin Packman, #794353)
+
+* ``bzr branches`` now indicates the active colocated branch.
+  (Jelmer Vernooij, #891667)
+
+* ``bzr push`` now suggests using :parent if there is a parent location
+  set. (Jelmer Vernooij)
+
+* ``bzr send`` now only opens a single connection, rather than two,
+  to the target branch.  (Jelmer Vernooij)
+
+Bug Fixes
+*********
+
+.. Fixes for situations where bzr would previously crash or give incorrect
+   or undesirable results.
+
+* Allow configuration option default value to be a python callable at
+  registration. (Vincent Ladeuil, #832064)
+
+* ``bzr config`` will now display the section ``[DEFAULT]`` used in
+  ``bazaar.conf``. (Vincent Ladeuil, #907268)
+
+* Configuration stores can now provides a specific quoting mechanism. This
+  is required to workaround ``configobj`` conflating quoting and list values
+  automatic conversion. (Vincent Ladeuil, #906897)
+
+* Create obsolete_packs directory when repacking if it does not
+  exist. (Jonathan Riddell, Jelmer Vernooij, #314314)
+
+* Fallback to the slower ``bzr log`` implementation when displaying a range
+  of revisions whose ancestry is not obviously on the same developement
+  line. (Vincent Ladeuil, #904744)
+
+* Not setting ``gpg_signing_key`` or setting it to ``default`` will use the
+  user email (obtained from the ``email`` configuration option or its
+  default value). (Vincent Ladeuil, Jelmer Vernooij, #904550)
+
+* Prevent spurious InconsistentDelta error when committing a move of a
+  non-ascii directory with contents. (Rory Yorke, #185211)
+
+* Properly ignore '\n' in an option reference since this cannot be part of a
+  config option identifier. (Vincent Ladeuil, #902125)
+
+* Make sure that the bzr probers are always registered when
+  bzrlib.workingtree is imported. (Jelmer Vernooij, #905218)
+
+* Report mistake trying to move a removed file with a non-ascii name without
+  UnicodeEncodeError being raised. (Martin Packman, #898541)
+
+* Safely unquote configuration values in weird edge cases (a section seen as
+  a dictionary which is not a supported use case for the configuration
+  stacks). (Vincent Ladeuil, #908050)
+
+Documentation
+*************
+
+.. Improved or updated documentation.
+
+API Changes
+***********
+
+.. Changes that may require updates in plugins or other code that uses
+   bzrlib.
+
+* ``Config.signature_needed``, ``Config.signing_policy``,
+  ``Config.gpg_signing_key``, ``Config.gpg_signing_command``,
+  ``Config.checking_policy`` and ``Config.post_commit`` are now deprecated.
+  (Jelmer Vernooij)
+
+* ``Repository.get_commit_builder`` now takes a ``config_stack``
+  rather than a ``config`` argument. (Jelmer Vernooij)
+
+* Send formats now accept a new optional argument ``submit_branch``,
+  which can be None or a Branch object for the submit branch location.
+  (Jelmer Vernooij)
+
+* ``VersionedFileRepository.add_revision`` no longer takes a ``config``
+  argument. (Jelmer Vernooij)
+
+Internals
+*********
+
+.. Major internal changes, unlikely to be visible to users or plugin 
+   developers, but interesting for bzr developers.
+
+* ``bzrlib.urlutils`` now includes ``quote`` and ``unquote`` functions,
+  rather than importing them from ``urllib``. This prevents loading
+  of the ``socket``, ``ssl`` and ``urllib`` modules for
+  local bzr operations. (Jelmer Vernooij)
+
+* Configuration options can be SI units by using ``int_SI_from_unicode`` as
+  their ``convert_from_unicode`` helper. (Vincent Ladeuil)
+
+* ControlDir now has a get_branches method that returns a dictionary
+  whose keys are the names of the branches and whose values are the
+  branches themselves. The active branch uses the key None.
+  (Neil Martinsen-Burrell)
+
+* Helper ``osutils.path_from_environ`` added for extracting a unicode path
+  from an environment variable. (Martin Packman, #832028)
+
+* Helper ``win32utils.get_environ_unicode`` added for avoiding encoding
+  problems with ``os.environ.get`` use. (Martin Packman, #262874) 
+
+* Lazy imports can now only be absolute. (Jelmer Vernooij)
+
+* Merge3Mergers now have an optional ``other_branch`` argument
+  which contains the branch from which the ``other_tree``
+  was obtained, if any. (Jelmer Vernooij)
+
+* New HPSS call ``BzrDir.checkout_metadir``. (Jelmer Vernooij, #894459)
+
+* New HPSS call ``VersionedFileRepository.get_inventories``,
+  speeding up various commands including ``bzr export``,
+  ``bzr checkout`` and ``bzr cat``. (Jelmer Vernooij, #608640)
+
+* The ``ConfigCommandLineStore`` is now supported by ``bzr config`` and is
+  seen as single no-name section of configuration options. (Vincent Ladeuil)
+
+Testing
+*******
+
+.. Fixes and changes that are only relevant to bzr's test framework and 
+   suite.  This can include new facilities for writing tests, fixes to 
+   spurious test failures and changes to the way things should be tested.
+
+* New matcher ``ContainsNoVfsCalls`` which filters a list of HPSS
+  calls for VFS requests. (Jelmer Vernooij)
+
+* New ``MemoryStack`` class allows for diskless tests and locally injected
+  configuration stacks.  Lower level tests for predefined set of options can
+  be written without setting up configuration files. (Vincent Ladeuil)
+
+
+bzr 2.5b4
+#########
+
+:2.5b4: 2011-12-08
+
+This is the fourth beta of the 2.5 series, leading to a 2.5.0 release in
+February 2012. Beta releases are suitable for everyday use but may cause
+some incompatibilities with plugins.
+
+This release includes many improvements in the smart server, UI polish for
+the colocated branches, optimizations for revision specifiers to avoid
+history sized operations, enhancements to the config framework, bug fixes
+related to unicode paths and more.
+
+All bug fixed in previous series known at the time of this release are
+included.
+
+External Compatibility Breaks
+*****************************
+
+None.
+
+New Features
+************
+
+* Provides a ``po_merge`` plugin to automatically merge ``.po`` files with
+  ``msgmerge``. See ``bzr help po_merge`` for details.
+  (Vincent Ladeuil, #884270)
+
+Improvements
+************
+
+* ``bzr branch --stacked`` now only makes a single connection to the remote
+  server rather than three. (Jelmer Vernooij, #444293)
+
+* ``bzr export --uncommitted`` will export the uncommitted tree.
+  (Jelmer Vernooij, #555613)
+
+* ``bzr rmbranch`` can now remove colocated branches.
+  (Jelmer Vernooij, #831464)
+
+* ``bzr status`` no longer shows shelves if files are specified.
+  (Francis Devereux)
+
+* ``bzr switch`` now accepts colocated branch names to switch to.
+  (Jelmer Vernooij, #826814)
+
+* Plugins can now register additional "location aliases".
+  (Jelmer Vernooij)
+
+* ``bzr status`` no longer shows shelves if files are specified.
+  (Francis Devereux)
+
+* Revision specifiers will now only browse as much history as they
+  need to, rather than grabbing the whole history unnecessarily in some
+  cases. (Jelmer Vernooij)
+
+* When using ``bzr switch`` to switch to a sibling of the current
+  branch, the relative branch name should no longer be url-encoded.
+  (Jelmer Vernooij)
+
+Bug Fixes
+*********
+
+* A new section local option ``basename`` is available to help support some
+  ``bzr-pipeline`` workflows and more generally help mapping local paths to
+  remote ones. See ``bzr help configuration`` for more details.
+  (Vincent Ladeuil, #843211)
+
+* Add HPSS call for looking up revision numbers from revision ids on
+  remote repositories. (Jelmer Vernooij, #640253)
+
+* Add HPSS call for retrieving file contents from remote repositories.
+  Should improve performance for lightweight checkouts and exports of
+  from remote repositories.  (Jelmer Vernooij, #368717, #762330,
+  #608640)
+
+* Allow lazy compiled patterns from ``bzrlib.lazy_regex`` to be
+  pickled. (Jelmer Vernooij, #893149)
+
+* ``bzr info`` no longer shows empty output if only a control
+  directory is present. (Jelmer Vernooij, #159098)
+
+* Cope with missing revision ids being specified to
+  ``Repository.gather_stats`` HPSS call. (Jelmer Vernooij, #411290)
+
+* Fix test failures on windows related to locations.conf handling.
+  (Vincent Ladeuil, #892992)
+
+* Fixed parsing of the timestamp given to ``commit --commit-time``. Now
+  prohibits several invalid strings, reads the correct number of seconds,
+  and gives a better error message if the time zone offset is not given.
+  (Matt Giuca, #892657)
+
+* Give meaningful file/line references when reporting deprecation warnings
+  for _CompatabilityThunkFeature based test features.
+  (Vincent Ladeuil, #897718)
+
+* Make reporting of mistakes involving unversioned files with non-ascii
+  filenames work again without 'Unprintable exception' being shown.
+  (Martin Packman, #898408)
+
+* Provide names for lazily registered hooks. 
+  (Neil Martinsen-Burrell, #894609)
+
+* Raise BadIndexKey exception in btree_index when a key is too large, fixing 
+  an infinite recursion issue. (Shannon Weyrick, #720853)
+
+* Resolve regression from colocated branch path handling, by ensuring that
+  unreserved characters are unquoted in URLs. (Martin Packman, #842223)
+
+* Split segments from URLs for colocated branches without assuming the
+  combined form is valid. (Martin Packman, #842233)
+
+* Support looking up revision numbers by revision id in empty branches.
+  (Jelmer Vernooij, #535031)
+
+* Support verifying signatures on remote repositories.
+  (Jelmer Vernooij, #889694)
+
+* Teach the bzr client how to reconnect if we get ``ConnectionReset``
+  while making an RPC request. This doesn't handle all possible network
+  disconnects, but it should at least handle when the server is asked to
+  shutdown gracefully. (John Arbash Meinel, #819604)
+
+* When a remote format is unknown, bzr will now print a single-line error
+  message rather than a backtrace. (Jelmer Vernooij, #687226)
+
+API Changes
+***********
+
+* ``BzrDir.open_branch`` and ``BranchFormat.open`` now take an optional
+  ``possible_transports`` argument. (Jelmer Vernooij)
+
+* New method ``Transport.set_segment_parameter``.  (Jelmer Vernooij)
+
+* ``Repository.verify_revision`` has been renamed to
+  ``Repository.verify_revision_signature``. (Jelmer Vernooij)
+
+* ``RevisionSpec.wants_revision_history`` now defaults to ``False`` and
+  is deprecated. The ``revs`` argument of
+  ``RevisionInfo.from_revision_id`` is now deprecated. (Jelmer Vernooij)
+
+* ``Tree.get_file_by_path`` is now deprecated. Use ``Tree.get_file`` instead.
+  (Jelmer Vernooij, #666897)
+
+* Some global options for use with commands have been removed, construct
+  an ``Option`` with the name instead. (Martin Packman)
+
+* The unused exception ``HistoryMissing`` has been removed.
+  (Jelmer Vernooij)
+
+Internals
+*********
+
+* Add HPSS call for ``Repository.pack``. (Jelmer Vernooij, #894461)
+
+* ``bzr config`` uses the new configuration implementation.
   (Vincent Ladeuil)
 
+* Custom HPSS error handlers can now be installed in the smart server client
+  using the ``error_translators`` and ``no_context_error_translators``
+  registries. (Jelmer Vernooij)
+
+* New HPSS calls ``Repository.has_signature_for_revision_id``,
+  ``Repository.make_working_trees``, ``BzrDir.destroy_repository``,
+  ``BzrDir.has_workingtree``, ``Repository.get_physical_lock_status``,
+  ``Branch.get_physical_lock_status``,
+  ``Branch.put_config_file``, ``Branch.break_lock``,
+  ``BzrDir.destroy_branch``, ``Repository.break_lock``,
+  ``VersionedFileRepository.get_serializer_format``,
+  ``Repository.all_revision_ids``, ``Repository.start_write_group``,
+  ``Repository.commit_write_group``, ``Repository.abort_write_group``
+  ``Repository.check_write_group``, ``Repository.iter_revisions``,
+  ``Repository.add_signature_revision_text`` and
+  ``Repository.get_revision_signature_text``.
+  (Jelmer Vernooij)
+
+* Add HPSS call for ``Branch.get_checkout_format``. (Jelmer Vernooij, #894459)
+
+* Add HPSS call for ``Repository.pack``. (Jelmer Vernooij, #894461)
+
+* Add HPSS calls for ``Repository.iter_files_bytes``, speeding up
+  several commands including ``bzr export`` and ``bzr co --lightweight``.
+  (Jelmer Vernooij, #608640)
+
+* Custom HPSS error handlers can now be installed in the smart server client
+  using the ``error_translators`` and ``no_context_error_translators``
+  registries. (Jelmer Vernooij)
+
+* ``RemoteBranch.get_config_stack`` and ``RemoteBzrDir.get_config_stack``
+  will now use HPSS calls where possible. (Jelmer Vernooij)
+
+* Custom HPSS error handlers can now be installed in the smart server client
+  using the ``error_translators`` and ``no_context_error_translators``
+  registries. (Jelmer Vernooij)
+
+* The registry of merge types has been moved to ``merge`` from ``option`` but
+  ``merge.get_merge_type_registry`` remains as an accessor. (Martin Packman)
+
+* All bzr control directories, branch formats, repository formats and
+  working tree formats now support feature flags, which are
+  serialized in their respective format files. See
+  ``doc/developers/feature-flags.txt`` for details.
+  (Jelmer Vernooij)
+
+Testing
+*******
+
+* Avoid failures in test_transform when OS error messages are localised.
+  (Martin Packman, #891582)
+
+* Tests are now subject to a time limit: by default 300s, and 120s when
+  run from 'make check', controlled by the `selftest.timeout`
+  configuration option.  This is currently not supported on Windows.
+  (Martin Pool)
+
+bzr 2.5b3
+#########
+
+:2.5b3: 2011-11-10
+
+This is the third beta of the 2.5 series, leading to a 2.5.0 release in
+February 2012. Beta releases are suitable for everyday use but may cause
+some incompatibilities with plugins.
+
+This release includes log options for ``push`` and ``pull``, more UI polish
+for colocated branches, a better and more coherent implementation for UI
+dialogs, enhancements to the config framework and more.
+
+This release includes all bug fixed in previous series known at the time of
+this release.
+
+External Compatibility Breaks
+*****************************
+
+None
+
+New Features
+************
+
+* The ``log_format`` configuration can be used with ``-Olog_format=line`` to
+  change the format ``push`` and ``pull`` use to display the
+  revisions. I.e.: ``bzr pull -v -Olog_format=short`` will use the ``short``
+  format instead of the default ``long`` one. (Vincent Ladeuil, #861472)
+
+* The new config scheme allows an alternative syntax for the 'appenpath'
+  policy relying on option expansion and defining a new 'relpath' option
+  local to a section. Instead of using '<option>:policy=appendpath', the
+  option value can de defined as 'option=xxxx/{relpath}'.
+  (Vincent Ladeuil, #832013)
+
+Improvements
+************
+
+* ``bzr info -v`` now shows the number of colocated branches
+  for control directories that support them.
+  (Jelmer Vernooij, #863285)
+
+* ``bzr version-info`` now takes a ``--revision`` argument.
+  (Jelmer Vernooij, #238705)
+
+* ``bzr revno`` now takes a ``--revision`` argument.
+  (Jelmer Vernooij, #870649)
+
+* ``bzr serve`` now can serve from URLs rather than just from the
+  file system. I.e.: ``bzr serve -d lp:bzr`` or
+  ``bzr serve -d file:///data/bzr`` (Jelmer Vernooij)
+
+* all input prompts are now char-based when possible, and can be forced to
+  line-based mode by setting the ``BZR_TEXTUI_INPUT`` environment variable
+  to 'line-based'. This replace the previous shelf UI only patch using
+  ``INSIDE_EMACS``. (Benoît Pierre)
+
+Bug Fixes
+*********
+
+* ``bzr info`` now shows the master branch location too for
+  treeless local branches. (Jelmer Vernooij, #258355)
+
+* ``bzr mkdir --quiet`` now does not print a line for every created
+  directory. (Martin von Gagern, #869915)
+
+* ``bzr mv`` does not crash when attempting to move the root of a
+  branch. (Jonathan Riddell, #809728)
+
+* ``bzr shelve`` now use ``UIFactory.choose`` for input handling, making
+  it usable when creating a custom ``UIFactory`` implementation. (Benoît
+  Pierre)
+
+* ``bzr clean-tree`` now use ``UIFactory.get_boolean`` for confirmation
+  prompt, making it usable when using a custom ``UIFactory``
+  implementation. (Benoît Pierre)
+
+* If sending a crash through Apport fails report the Apport failure to
+  bzr.log rather than stderr. (Jonathan Riddell, #766735)
+
+* ``bzr upgrade`` no longer treats 'already up-to-date' exceptions as
+  errors. (Benoît Pierre, #716560).
+
+* ``bzr version-info`` no longer populates the clean state for custom
+  templates unless {clean} is explicitly asked for.
+  (Lawrence Mitchell, #882541)
+
+* Fix finding the CPU count when using Python >= 2.6 on BSD-based systems.
+  (Jelmer Vernooij, #887151)
+
+* ``WorkingTree.clone()`` now supports its ``revision_id`` being set
+  to the null revision. (Jelmer Vernooij, #876423)
+
+* ``WorkingTree.pull`` can now pull ``NULL_REVISION``.
+  (Jelmer Vernooij, #887556)
+
+API Changes
+***********
+
+* ``Branch.revision_history`` is now deprecated. (Jelmer Vernooij, #799519)
+
+* Methods ``add`` and ``items`` of ``LRUCache`` and ``LRUSizeCache`` are
+  deprecated. Use normal dict-style access instead. (Martin Packman)
+
+* New flag ``RepositoryFormat.supports_unreferenced_revisions`` which
+  indicates whether revisions can be present in a repository without
+  being referenced from e.g. a branch history at the same time.
+  (Jelmer Vernooij)
+
+* ``UIFactory.choose`` has been added: prompt the user for a list of
+  choices. (Benoît Pierre)
+
+Internals
+*********
+
+* ``ControlDirFormat`` now has a new method ``supports_transport``
+  which format implementations can use whether or not they can access
+  a control dir over a particular transport. (Jelmer Vernooij)
+
+* ``BranchBuilder.build_commit`` now take ``parent_ids`` and
+  ``allow_leftmost_as_ghost`` arguments.  (Jelmer Vernooij)
+
+Testing
+*******
+
+* Ensure TestCase instances are deallocated immediately after running where
+  possible. This greatly reduces the peak resource needs of a full test suite
+  run. The new ``-Euncollected_cases`` selftest flag will add failures if any
+  case which persists pasts its expected lifetime. (Martin Packman, #613247)
+
+* Report exceptions from child processes during fork instead of swallowing the
+  error and reporting that everything went okay. (Martin Packman, #804130)
+
+
+bzr 2.5b2
+#########
+
+This is the second beta of the 2.5 series, leading to a 2.5.0 release in
+February 2012. Beta releases are suitable for everyday use but may cause some
+incompatibilities with plugins.
+
+This release includes more filtering options for ``bzr log``, idle
+connections handling for ``bzr serve``, a ``development-colo`` experimental
+format to flesh out the colocated branches UI, better support for foreign
+formats, enhancements to the config framework and more.
+
+This release includes all bug fixed in previous series known at the time of
+this release.
+
+:2.5b2: 2011-10-06
+
+External Compatibility Breaks
+*****************************
+
+None
+
+New Features
+************
+
+* A new ``-O`` standard option (common to all commands) have been added. It
+  provides a value for a config option in the ``-Oname=value`` form that
+  takes precedence over all definitions found in config files.  It can be
+  used multiple times to override different options.
+  (Vincent Ladeuil, #491196)
+
+* ``bzr log`` now has an option called ``--omit-merges`` to omit
+  those commits that merged branches, i.e. those having more than one
+  parent.
+  In order to avoid confusion, the previous command line option
+  ``--include-merges`` has been renamed to ``--include-merged``.
+  The old name of the command line option will still be accepted.
+  The name change also affects ``bzr missing``.
+  (Martin von Gagern)
+
+* ``bzr serve`` will now disconnect clients if they have not issued an RPC
+  request after 5minutes. On POSIX platforms, this will also happen for
+  ``bzr serve --inet``. This can be overridden with the configuration
+  variable ``serve.client_timeout`` or in the command line parameter
+  ``bzr serve --client-timeout=X``. Further, it is possible to request
+  ``bzr serve [--inet]`` to shutdown gracefully by sending SIGHUP. It will
+  finish the current request, and then close the connection.
+  (John Arbash Meinel, #824797, #795025)
+
+* The new experimental format ``development-colo`` supports colocated
+  branches. This format will eventually be merged back into the ``2a``
+  format when it has stabilized and there is adequate UI support for
+  colocated branches.
+  (Jelmer Vernooij, #831481)
+
+Improvements
+************
+
+* Fixed a bug where ``bzr tags -r x..y`` loaded the branch history once for
+  every revision in the range; it's now much faster. (Vincent Ladeuil, #857335)
+
+* ``bzr info -v`` can now be run against branches that don't support
+  ``last_revision_info``, in which case the branch information will simply
+  not be displayed. (Jelmer Vernooij)
+
+Bug Fixes
+*********
+
+* ``bzr shelve`` can now be used in emacs shells as the input handling is
+  turned into a line-based one when ``INSIDE_EMACS`` is set (which is the
+  case for all recent emacs versions). (Vincent Ladeuil, #856261)
+
+* ``bzr tags`` can now be used against remote repositories that do
+  not provide access to the revision graph. (Jelmer Vernooij, #858942)
+
+* ``bzr update PATH`` will stop if you seem to be asking it to update
+  anything less than a whole tree, because that's not supported by ``bzr``'s
+  concept that the whole tree has a single basis revision.  Previously, it
+  would go ahead and update the whole tree, which was surprising.
+  (Martin Pool, #557886)
+
+* Don't crash if ``bzrlib.initialize()`` has not been called while accessing
+  configs.  (Vincent Ladeuil, #863401)
+
+* Redirects between http and https no longer discard path information
+  in some cases. (Jelmer Vernooij, #853765)
+
+* The ``--overwrite`` argument to ``bzr push`` and ``bzr pull`` no longer
+  reports all tags as changed. (Jelmer Vernooij, #845396)
+
+* ``WorkingTree.get_file_mtime`` now raises NoSuchId if a file id is
+  specified that is unknown. (Jelmer Vernooij, #847435)
+
+
+API Changes
+***********
+
+* ``Branch.get_revision_delta`` has been deprecated. Use
+  ``Repository.get_revision_delta`` instead. (Jelmer Vernooij, #859712)
+
+* Plugins that implement custom protocols for ``bzr serve`` should now
+  also take an argument ``timeout``. This is used by the the bzr protocol
+  to close a connection if a client has been idle for more than X seconds.
+  (Default 5minutes). (John Arbash Meinel)
+
+* ``Repository.fileids_altered_by_revision_ids`` has been moved to
+  ``VersionedFileRepository`` and is no longer part of the standard
+  ``Repository`` interface. (Jelmer Vernooij)
+
+* The argument ``include_merges`` to ``missing.find_unmerged`` has
+  been renamed to ``include_merged``. The old name is still supported
+  for now but will cause a deprecation warning. (Martin von Gagern)
+
+* The new method ``ControlDirFormat.is_initializable()`` returns a boolean
+  indicating whether or not it is possible to use any of the
+  initialization methods of that format to create a new control dir.
+  (Jelmer Vernooij)
+
+Internals
+*********
+
+* ``Branch`` objects can now use a config stack with the newly introduced
+  ``get_config_stack()``. Both ``get_config`` and ``get_config_stack`` can
+  be used for the same branch but it's recommended to stick to one for a
+  given option.
+
+Testing
+*******
+
+* Test scripts can now use ``bzr shelve`` and provide their input as
+  complete lines. (Vincent Ladeuil, #856261)
+
+* Really corrupt the pack file without depending on a special length or value.
+  (Vincent Ladeuil, #807032)
+
+
+bzr 2.5b1
+#########
+
+:2.5b1: 2011-09-15
+
+This is the first beta of the 2.5 series, leading up to a 2.5.0
+release in February 2012.
+
+This release includes better support for gpg signing, better support for
+i18n (mostly command help and error messages), more options to filter ``bzr
+log`` output, more support for colocated branches ("location,branch=XXX"
+syntax), better feedback on updated tags for various commands, faster
+branching into an empty repository, enhancements to the config framework and
+more.
+
+Beta releases are suitable for everyday use but may cause some
+incompatibilities with plugins.  Some plugins may need small updates to work
+with 2.5b1.
+
+External Compatibility Breaks
+*****************************
+
+None
+
+New Features
+************
+
 * A ``from_unicode`` parameter can be specified when registering a config
   option. This implements boolean, integer and list config options when the
   provided ``bool_from_store``, ``int_from_store`` and ``list_from_store``
@@ -56,71 +728,97 @@
 
 * Add a config option gpg_signing_key for setting which GPG key should
   be used to sign commits. Also default to using the gpg user identity
-  which matches user_email() as set by whoami. (Jonathan Riddell,
-  #68501)
+  which matches user_email() as set by whoami.
+  (Jonathan Riddell, #68501)
 
 * An ``invalid`` parameter can be specified when registering a config option
   to decide what should be done when invalid values are
-  encountered. 'warning' and 'eeror' will respectively emit a warning and
+  encountered. 'warning' and 'error' will respectively emit a warning and
   ignore the value or errors out. (Vincent Ladeuil)
 
+* bzr add now skips large files in recursive mode. The default "large"
+  size is 20MB, and is configurable via the add.maximum_file_size
+  option. A value of 0 disables skipping. Named items passed to add are
+  never skipped. (Shannon Weyrick, #54624)
+
 * ``bzr help configuration/<option>`` display the help for ``option`` for
   all registered configuration options. (Vincent Ladeuil, #747050)
 
-* bzr log -m now matches message, author, committer and bugs instead
-  of just matching the message.  --message keeps its original meaning,
-  while --match-message, --match-author, --match-committer and
-  --match-bugs match each of those fields.
+* ``bzr log -m`` now matches message, author, committer and bugs instead
+  of just matching the message.  ``--message`` keeps its original meaning,
+  while ``--match-message, --match-author, --match-committer`` and
+  ``--match-bugs`` match each of those fields. (Jacek Sieka)
 
 * ``config.Option`` can now declare ``default_from_env``, a list of
   environment variables to get a default value from. (Vincent Ladeuil)
 
+* ``config.NameMatcher`` can be used to implement config stores and stacks
+  that need to provide specific option values for arbitrary unique IDs (svn
+  repository UUIDs, etc).  (Vincent Ladeuil, #843638)
+
 * New builtin ``bzr branches`` command, which lists all colocated branches
   in a directory. (Jelmer Vernooij, #826820)
 
 * Relative local paths can now be specified in URL syntax by using the
   "file:" prefix.  (Jelmer Vernooij)
 
-* Report commits signed with expired keys in "verify-signatures".
+* Report commits signed with expired keys in ``verify-signatures``.
   (Jonathan Riddell, #804254)
 
-* bzr add now skips large files in recursive mode. The default "large"
-  size is 20MB, and is configurable via the add.maximum_file_size
-  option. A value of 0 disables skipping. Named items passed to add are
-  never skipped. (Shannon Weyrick, #54624)
-
-* bzr now ships with translations for command help.  (Jonathan
-  Riddell, INADA Naoki, #83941)
+* Translations are now enabled for command help, errors and globally
+  for any message using ``gettext`` given on output.  (Jonathan Riddell,
+  INADA Naoki, #83941)
 
 Improvements
 ************
 
-.. Improvements to existing commands, especially improved performance 
-   or memory usage, or better results.
+* ``bzr add`` will now warn about nested subtrees that are skipped.
+  (Jelmer Vernooij, #187342)
 
 * ``bzr commit -m ''`` can now be used to force an empty commit message.
   Entering an empty commit message in the message editor still triggers
   an error. (Jelmer Vernooij)
 
+* ``bzr pull`` will now mention how many tags it has updated.
+  (Jelmer Vernooij, #164450)
+
+* ``bzr tag`` no longer errors if a tag already exists but refers to the
+  same revision, and will mention when a tag has been updated
+  rather than created. (Jelmer Vernooij, #381203)
+
 * ``bzr uncommit`` will now remove tags that refer to removed revisions.
   The ``--keep-tags`` option can be used to prevent this behaviour.
   (Jelmer Vernooij, #605814)
 
+* Do not run i18n initialisation twice. (Jonathan Riddell)
+
+* Install translation .mo files. (Jonathan Riddell)
+
 * Locations printed by ``bzr upgrade`` are now formatted before display.
   (Jelmer Vernooij)
 
-* Install translation .mo files. (Jonathan Riddell)
+* ``Repository.get_parent_map`` now estimates the size of the returned
+  content more accurately. This means that we get closer to the desired
+  64kB/request. For repositories converted from svn, this can be an
+  improvement of approx 5:1 in round trips to discover the whole history.
+  (John Arbash Meinel)
+
+* Support a ``bugtracker`` option which is used by ``bzr commit --fixes``
+  if no bug tracker was specified on the command line.
+  (Jelmer Vernooij, #334860)
+
+* Use ``gettext.NullTranslations`` in i18n to allow use of i18n even when
+  translations are not turned on. (Jonathan Riddell)
 
 Bug Fixes
 *********
 
-.. Fixes for situations where bzr would previously crash or give incorrect
-   or undesirable results.
+* ``bzr commit`` now correctly reports missing files as "removed", not
+  "modified". (Jelmer Vernooij, #553955)
 
-* The pull command will now always use separate connections for the
-  case where the destination is a heavyweight checkout of some remote
-  branch on the same host as the source branch.
-  (Martin von Gagern, #483661)
+* ``bzr reconfigure`` will now allow multiple non-conflicting requests
+  in a single invocation, e.g. ``--branch`` and ``--use-shared``.
+  (Martin von Gagern, #842993)
 
 * A call to CHKInventory's filter-method will not result in a
   DuplicateFileId error, if you move a subfolder and change a file in
@@ -136,6 +834,22 @@
 * Decode ``BZR_HOME`` with fs encoding on posix platforms to avoid unicode
   errors.  (Vincent Ladeuil, #822571)
 
+* Fix fallout from URL handling changes in 2.5 that caused an IndexError to be
+  raised whenever a transport at the drive root was opened on windows.
+  (Martin [gz], #841322)
+
+* Fixed loading of external merge tools from config to properly decode
+  command-lines which contain embedded quotes. (Gordon Tyler, #828803)
+
+* Rather than an error being raised, a warning is now printed when the
+  current user does not have permission to read a configuration file.
+  (Jelmer Vernooij, #837324)
+
+* The pull command will now always use separate connections for the
+  case where the destination is a heavyweight checkout of some remote
+  branch on the same host as the source branch.
+  (Martin von Gagern, #483661)
+
 * TreeTransformBase.fixup_new_roots no longer forces trees to have a root, so
   operations that use it, like merge, can now create trees without a root.
   (Aaron Bentley)
@@ -143,31 +857,40 @@
 Documentation
 *************
 
-.. Improved or updated documentation.
-
 * Release instructions refreshed. (Vincent Ladeuil)
 
 API Changes
 ***********
 
-.. Changes that may require updates in plugins or other code that uses
-   bzrlib.
+* ``BranchFormat.initialize`` now takes a ``append_revisions_only``
+  argument. (Jelmer Vernooij)
+
+* ``Branch._get_checkout_format`` now takes a ``lightweight`` argument
+  which indicates if the format should be for a lightweight or a
+  heavyweight checkout. (Jelmer Vernooij)
+
+* ``ControlDir.create_branch`` now takes a ``append_revisions_only`` argument.
+  (Jelmer Vernooij)
 
 * New class ``URL`` in ``bzrlib.utils`` for managing parsed URLs.
   (Jelmer Vernooij)
 
+* New method ``Config.get_user_option_as_int_from_SI`` added for expanding a
+  value in SI format (i.e. "20MB", "1GB") into its integer equivalent. 
+  (Shannon Weyrick)
+
+* New method ``InterTree.file_content_matches`` which checks that
+  two files in different trees have the same contents.
+  (Jelmer Vernooij)
+
+* New method ``Tree.get_file_verifier`` which allows tree implementations
+  to return non-sha1 checksums to verify files.
+  (Jelmer Vernooij, #720831)
+
 * New methods ``get_transport_from_path`` and ``get_transport_from_url``
   have been added that only support opening from a path or a URL,
   unlike ``get_transport``. (Jelmer Vernooij)
 
-* New method ``Tree.get_file_verifier`` which allows tree implementations
-  to return non-sha1 checksums to verify files.
-  (Jelmer Vernooij, #720831)
-
-* New method ``InterTree.file_content_matches`` which checks that
-  two files in different trees have the same contents.
-  (Jelmer Vernooij)
-
 * New registry ``OptionRegistry`` specialized for configuration options.
   (Vincent Ladeuil)
 
@@ -195,29 +918,28 @@
   ``UIFactory.warn_experimental_format_fetch`` in favor of
   ``UIFactory.show_user_warning``. (Jelmer Vernooij)
 
+* ``Tags`` containers can now declare whether they support versioned
+  tags and whether tags can refer to ghost tags.
+  (Jelmer Vernooij)
+
+* ``Tags.merge_to`` now returns a dictionary with the updated tags
+  and a set of conflicts, rather than just conflicts. (Jelmer Vernooij)
+
 * There is a new class `ContentFilterTree` that provides a facade for 
   content filtering.  The `filtered` parameter to `export` is deprecated 
   in favor of passing a filtered tree, and the specific exporter plugins
   no longer support it.
   (Martin Pool)
 
-* New method ``Config.get_user_option_as_int_from_SI`` added for expanding a
-  value in SI format (i.e. "20MB", "1GB") into its integer equivalent. 
-  (Shannon Weyrick)
-
 * ``Transport`` now has a ``_parsed_url`` attribute instead of
   separate ``_user``, ``_password``, ``_port``, ``_scheme``, ``_host``
   and ``_path`` attributes. Proxies are provided for the moment but
   may be removed in the future. (Jelmer Vernooij)
 
-
 Internals
 *********
 
-.. Major internal changes, unlikely to be visible to users or plugin 
-   developers, but interesting for bzr developers.
-
-* A new debug flags ``hpss_client_no_vfs`` will now cause the HPSS client
+* A new debug flag ``hpss_client_no_vfs`` will now cause the HPSS client
   to raise a ``HpssVfsRequestNotAllowed`` exception when a VFS request
   is attempted. (Jelmer Vernooij)
 
@@ -228,12 +950,8 @@
 Testing
 *******
 
-.. Fixes and changes that are only relevant to bzr's test framework and 
-   suite.  This can include new facilities for writing tests, fixes to 
-   spurious test failures and changes to the way things should be tested.
-
-* Blackbox tests can be debugged interactively (see bzrlib.debug.BzrPdb for
-  details). (Vincent Ladeuil)
+* Blackbox tests (including test scripts) can be debugged interactively (see
+  bzrlib.debug.BzrPdb for details). (Vincent Ladeuil)
 
 * `BranchBuilder.build_snapshot` now supports a "flush" action.  This
   cleanly and reliably allows tests using `BranchBuilder` to construct
@@ -254,6 +972,9 @@
   allowing it to be used for modules with side-effects.
   (Vincent Ladeuil, #712474)
 
+* Output time stamps while running ``make check`` to get better timings from
+  pqm.  (Vincent Ladeuil, #837926)
+
 * `TestCaseWithMemoryTransport` is faster now: `_check_safety_net` now
   just compares the bytes in the dirstate file to its pristine state,
   rather than opening the WorkingTree and calling ``last_revision()``.

=== modified file 'doc/en/tutorials/using_bazaar_with_launchpad.txt'
--- doc/en/tutorials/using_bazaar_with_launchpad.txt	2010-10-07 21:03:55 +0000
+++ doc/en/tutorials/using_bazaar_with_launchpad.txt	2011-11-14 11:44:54 +0000
@@ -100,7 +100,7 @@
 * it models relationships between many of things tracked, e.g.
   source code branches can be associated with bug fixes
 
-* as well are managing historical knowledge,
+* as well as managing historical knowledge,
   it supports future development planning and tracking by providing
   features such as roadmaps, milestones and blueprints
 

=== modified file 'doc/en/user-guide/installing_bazaar.txt'
--- doc/en/user-guide/installing_bazaar.txt	2010-08-13 19:08:57 +0000
+++ doc/en/user-guide/installing_bazaar.txt	2011-08-30 09:27:59 +0000
@@ -33,7 +33,7 @@
 If you wish to install Bazaar from scratch rather than using a
 pre-built package, the steps are:
 
-  1. If it is not installed already, install Python 2.4 or later.
+  1. If it is not installed already, install Python 2.6 or later.
 
   2. Download the ``bazaar-xxx.tar.gz`` file (where xxx is the version
      number) from http://wiki.bazaar.canonical.com/Download or from Launchpad

=== modified file 'doc/en/whats-new/whats-new-in-2.2.txt'
--- doc/en/whats-new/whats-new-in-2.2.txt	2011-07-05 11:54:54 +0000
+++ doc/en/whats-new/whats-new-in-2.2.txt	2011-09-09 12:32:08 +0000
@@ -38,6 +38,9 @@
 Bazaar 2.2.4 fixed a regression for some interactions with the launchpad
 server.
 
+Bazaar 2.2.5 fixed a regression in some rare conflict resolutions and warns
+when branching an out-of-date ubuntu packaging branch.
+
 See the :doc:`../release-notes/index` for details.
 
 Bazaar 2.2 is fully compatible both locally and on the network with 2.0

=== modified file 'doc/en/whats-new/whats-new-in-2.4.txt'
--- doc/en/whats-new/whats-new-in-2.4.txt	2011-08-12 09:49:24 +0000
+++ doc/en/whats-new/whats-new-in-2.4.txt	2011-10-27 15:38:14 +0000
@@ -20,6 +20,15 @@
 2.1, 2.2 and 2.3, and can read and write repositories generated by all
 previous versions.
 
+Bazaar 2.4.1 includes all the fixes in the un-released 2.0.7, 2.1.5, 2.2.6
+and 2.3.5 versions that weren't included in 2.4.0 and fixes some bugs on its
+own.
+
+Bazaar 2.4.2 includes all the fixes in the un-released 2.0.7, 2.1.5, 2.2.6
+and 2.3.5 versions that weren't included in 2.4.1 and fixes some bugs on its
+own.
+
+
 Dropping Python2.4 and Python2.5 support
 ****************************************
 

=== modified file 'doc/en/whats-new/whats-new-in-2.5.txt'
--- doc/en/whats-new/whats-new-in-2.5.txt	2011-07-11 07:49:59 +0000
+++ doc/en/whats-new/whats-new-in-2.5.txt	2012-01-03 16:28:43 +0000
@@ -16,6 +16,103 @@
 2.1, 2.2, 2.3 and 2.4, and can read and write repositories generated by all
 previous versions.
 
+Overriding configuration options from the command line
+******************************************************
+
+The ``-O`` parameter available for all bzr commands allows a user to
+override a configuration option from the command line. For example::
+
+  bzr pull -v -Olog_format=line
+
+will change the way the pulled revisions are displayed (the default log
+format is ``long``). This a work in progress and only some options are
+supported so far.
+
+Working on a posix system without a locale
+******************************************
+
+Previously bzr needed a valid locale set to work with branches containing
+non-ascii filenames. It will now use utf-8 rather than ascii as a fallback
+encoding for interacting with the filesystem. This makes creating a working
+tree and commiting to it possible for such branches in most environments.
+
+Faster smart server
+*******************
+
+A large number of new methods have been added to the smart server, making
+raw file access through the VFS unnecessary in almost all situations, with
+the major exception of operations involving stacked branches.
+
+Commands that have become significantly faster when using a remote branch
+over ``bzr://``, ``bzr+ssh://`` or ``bzr+http://`` include:
+
+ * ``bzr checkout --lightweight``
+ * ``bzr export``
+ * ``bzr cat``
+ * ``bzr ls``
+ * ``bzr send``
+
+Several commands which used to make multiple connections to the server now
+make only a single one. Connection setup has a fairly high overhead,
+especially to Launchpad, so this can save several seconds for some
+commands.
+
+To benefit from the improved smart server, both the server and the
+client need to be running bzr 2.5.
+
+Basic colocated branch support
+******************************
+
+The UI now has basic support for colocated branches. In full URLs,
+a specific colocated branch can be specified using URL path segment
+parameters. For example a branch named ``stronk`` could be addressed using
+``http://example.com/path/to/dir,branch=stronk``.
+
+The new ``bzr branches`` command can be used to list all present branches
+in a directory, and indicates what the currently active branch is.
+
+Several commands also accept co-located branch names directly, such as
+``bzr switch``.
+
+Feature flags
+*************
+
+All Bazaar formats now allow setting ``feature flags``. These can be used
+by plugins to extend Bazaar formats and require the presence of particular
+plugins or versions of Bazaar to open them, without having to introduce
+completely new formats.
+
+See ``doc/developers/feature-flags.txt`` for details.
+
+Branch history access
+*********************
+
+Several commands or options that previously required access to the full
+branch history now only access those parts of the history they actually
+need. This significantly improves their performance for branches
+with large histories.
+
+GPG signatures
+**************
+
+A new command ``bzr verify-signatures`` can be used to verify GPG
+signatures made with ``bzr commit`` or the ``bzr sign-my-commits``
+command.
+
+Translations
+************
+
+Most error messages, help topics and other user-visible text can now be
+translated. Initial translations for Russian, Japanese and Spanish exist.
+
+PO merge plugin
+***************
+
+The ``po_merge`` plugin has been added.  It provides a merge hook
+to automate merging of changes to gettext template files. Refer to
+``bzr help po_merge`` for documentation on how to
+enable it and what it can do.
+
 Further information
 *******************
 

=== modified file 'po/bzr.pot'
--- po/bzr.pot	2011-08-24 14:14:42 +0000
+++ po/bzr.pot	2011-12-08 08:32:36 +0000
@@ -8,7 +8,7 @@
 msgstr ""
 "Project-Id-Version: bzr\n"
 "Report-Msgid-Bugs-To: <bazaar@canonical.com>\n"
-"POT-Creation-Date: 2011-08-24 15:14+0100\n"
+"POT-Creation-Date: 2011-12-08 09:21+0100\n"
 "PO-Revision-Date: YEAR-MO-DA HO:MI+ZONE\n"
 "Last-Translator: FULL NAME <EMAIL@ADDRESS>\n"
 "Language-Team: LANGUAGE <LL@li.org>\n"
@@ -18,30 +18,77 @@
 "Content-Transfer-Encoding: 8bit\n"
 "Plural-Forms: nplurals=INTEGER; plural=EXPRESSION;\n"
 
-#: bzrlib/builtins.py:209
+#: bzrlib/add.py:95
+msgid "skipping {0} (larger than {1} of {2} bytes)"
+msgstr ""
+
+#: bzrlib/branch.py:871
+msgid "Unstacking"
+msgstr ""
+
+#: bzrlib/branch.py:3173
+#, python-format
+msgid "Pushed up to revision %d."
+msgstr ""
+
+#: bzrlib/branch.py:3175
+#, python-format
+msgid "%d tag updated."
+msgid_plural "%d tags updated."
+msgstr[0] ""
+msgstr[1] ""
+
+#: bzrlib/branch.py:3178
+msgid "No new revisions or tags to push."
+msgstr ""
+
+#: bzrlib/branch.py:3180
+msgid "No new revisions to push."
+msgstr ""
+
+#: bzrlib/branch.py:3200
+msgid "checked branch {0} format {1}"
+msgstr ""
+
+#: bzrlib/branch.py:3203
+#, python-format
+msgid "found error:%s"
+msgstr ""
+
+#: bzrlib/builtins.py:118 bzrlib/builtins.py:2868 bzrlib/workingtree.py:304
+#, python-format
+msgid "Ignoring files outside view. View is %s"
+msgstr ""
+
+#: bzrlib/builtins.py:127
+#, python-format
+msgid "bzr %s --revision takes exactly one revision identifier"
+msgstr ""
+
+#: bzrlib/builtins.py:212
 msgid "Display status summary."
 msgstr ""
 
-#: bzrlib/builtins.py:211
+#: bzrlib/builtins.py:214
 msgid ""
 "This reports on versioned and unknown files, reporting them\n"
 "grouped by state.  Possible states are:"
 msgstr ""
 
-#: bzrlib/builtins.py:214
+#: bzrlib/builtins.py:217
 msgid ""
 "added\n"
 "    Versioned in the working copy but not in the previous revision."
 msgstr ""
 
-#: bzrlib/builtins.py:217
+#: bzrlib/builtins.py:220
 msgid ""
 "removed\n"
 "    Versioned in the previous revision but removed or deleted\n"
 "    in the working copy."
 msgstr ""
 
-#: bzrlib/builtins.py:221
+#: bzrlib/builtins.py:224
 msgid ""
 "renamed\n"
 "    Path of this file changed from the previous revision;\n"
@@ -49,25 +96,25 @@
 "    parent directory was renamed."
 msgstr ""
 
-#: bzrlib/builtins.py:226
+#: bzrlib/builtins.py:229
 msgid ""
 "modified\n"
 "    Text has changed since the previous revision."
 msgstr ""
 
-#: bzrlib/builtins.py:229
+#: bzrlib/builtins.py:232
 msgid ""
 "kind changed\n"
 "    File kind has been changed (e.g. from file to directory)."
 msgstr ""
 
-#: bzrlib/builtins.py:232
+#: bzrlib/builtins.py:235
 msgid ""
 "unknown\n"
 "    Not versioned and not matching an ignore pattern."
 msgstr ""
 
-#: bzrlib/builtins.py:235
+#: bzrlib/builtins.py:238
 msgid ""
 "Additionally for directories, symlinks and files with a changed\n"
 "executable bit, Bazaar indicates their type using a trailing\n"
@@ -75,20 +122,20 @@
 "disabled using the '--no-classify' option."
 msgstr ""
 
-#: bzrlib/builtins.py:240
+#: bzrlib/builtins.py:243
 msgid ""
 "To see ignored files use 'bzr ignored'.  For details on the\n"
 "changes to file texts, use 'bzr diff'."
 msgstr ""
 
-#: bzrlib/builtins.py:243
+#: bzrlib/builtins.py:246
 msgid ""
 "Note that --short or -S gives status flags for each item, similar\n"
 "to Subversion's status command. To get output similar to svn -q,\n"
 "use bzr status -SV."
 msgstr ""
 
-#: bzrlib/builtins.py:247
+#: bzrlib/builtins.py:250
 msgid ""
 "If no arguments are specified, the status of the entire working\n"
 "directory is shown.  Otherwise, only the status of the specified\n"
@@ -96,7 +143,7 @@
 "is reported for everything inside that directory."
 msgstr ""
 
-#: bzrlib/builtins.py:252
+#: bzrlib/builtins.py:255
 msgid ""
 "Before merges are committed, the pending merge tip revisions are\n"
 "shown. To see all pending merge revisions, use the -v option.\n"
@@ -104,13 +151,13 @@
 "the no-pending option or specify a file/directory."
 msgstr ""
 
-#: bzrlib/builtins.py:257
+#: bzrlib/builtins.py:260
 msgid ""
 "To compare the working directory to a specific revision, pass a\n"
 "single revision to the revision argument."
 msgstr ""
 
-#: bzrlib/builtins.py:260
+#: bzrlib/builtins.py:263
 msgid ""
 "To see which files have changed in a specific revision, or between\n"
 "two revisions, pass a revision range to the revision argument.\n"
@@ -118,69 +165,130 @@
 msgstr ""
 
 # help of 'short' option of 'status' command
-#: bzrlib/builtins.py:269
+#: bzrlib/builtins.py:272
 msgid "Use short status indicators."
 msgstr ""
 
 # help of 'versioned' option of 'status' command
-#: bzrlib/builtins.py:271
+#: bzrlib/builtins.py:274
 msgid "Only show versioned files."
 msgstr ""
 
 # help of 'no-pending' option of 'status' command
-#: bzrlib/builtins.py:273
+#: bzrlib/builtins.py:276
 msgid "Don't show pending merges."
 msgstr ""
 
 # help of 'no-classify' option of 'status' command
-#: bzrlib/builtins.py:276
+#: bzrlib/builtins.py:279
 msgid "Do not mark object type using indicator."
 msgstr ""
 
-#: bzrlib/builtins.py:455
+#: bzrlib/builtins.py:294
+msgid "bzr status --revision takes exactly one or two revision specifiers"
+msgstr ""
+
+#: bzrlib/builtins.py:338 bzrlib/builtins.py:5024
+msgid "You can only supply one of revision_id or --revision"
+msgstr ""
+
+#: bzrlib/builtins.py:341 bzrlib/builtins.py:5026
+msgid "You must supply either --revision or a revision_id"
+msgstr ""
+
+#: bzrlib/builtins.py:348
+#, python-format
+msgid "Repository %r does not support access to raw revision texts"
+msgstr ""
+
+#: bzrlib/builtins.py:359
+msgid "The repository {0} contains no revision {1}."
+msgstr ""
+
+#: bzrlib/builtins.py:366
+msgid "You cannot specify a NULL revision."
+msgstr ""
+
+#: bzrlib/builtins.py:458
 msgid "Remove the working tree from a given branch/checkout."
 msgstr ""
 
-#: bzrlib/builtins.py:457
+#: bzrlib/builtins.py:460
 msgid ""
 "Since a lightweight checkout is little more than a working tree\n"
 "this will refuse to run against one."
 msgstr ""
 
-#: bzrlib/builtins.py:460
+#: bzrlib/builtins.py:463
 msgid "To re-create the working tree, use \"bzr checkout\"."
 msgstr ""
 
 # help of 'force' option of 'remove-tree' command
-#: bzrlib/builtins.py:466
+#: bzrlib/builtins.py:469
 msgid "Remove the working tree even if it has uncommitted or shelved changes."
 msgstr ""
 
-#: bzrlib/builtins.py:551
+#: bzrlib/builtins.py:483
+msgid "No working tree to remove"
+msgstr ""
+
+#: bzrlib/builtins.py:485
+msgid "You cannot remove the working tree of a remote path"
+msgstr ""
+
+#: bzrlib/builtins.py:494
+msgid "You cannot remove the working tree from a lightweight checkout"
+msgstr ""
+
+#: bzrlib/builtins.py:534
+msgid ""
+"The tree does not appear to be corrupt. You probably want \"bzr revert\" "
+"instead. Use \"--force\" if you are sure you want to reset the working tree."
+msgstr ""
+
+#: bzrlib/builtins.py:545
+msgid ""
+", the header appears corrupt, try passing -r -1 to set the state to the last "
+"commit"
+msgstr ""
+
+#: bzrlib/builtins.py:549
+msgid "failed to reset the tree state{0}"
+msgstr ""
+
+#: bzrlib/builtins.py:553
 msgid "Show current revision number."
 msgstr ""
 
-#: bzrlib/builtins.py:553
+#: bzrlib/builtins.py:555
 msgid "This is equal to the number of revisions on this branch."
 msgstr ""
 
+#: bzrlib/builtins.py:568
+msgid "--tree and --revision can not be used together"
+msgstr ""
+
+#: bzrlib/builtins.py:585 bzrlib/builtins.py:5808
+msgid "Tags can only be placed on a single revision, not on a range"
+msgstr ""
+
 # help of 'tree' option of 'revno' command
-#: bzrlib/builtins.py:594
-msgid "Show revno of working tree"
+#: bzrlib/builtins.py:609
+msgid "Show revno of working tree."
 msgstr ""
 
-#: bzrlib/builtins.py:642
+#: bzrlib/builtins.py:657
 msgid "Add specified files or directories."
 msgstr ""
 
-#: bzrlib/builtins.py:644
+#: bzrlib/builtins.py:659
 msgid ""
 "In non-recursive mode, all the named items are added, regardless\n"
 "of whether they were previously ignored.  A warning is given if\n"
 "any of the named files are already versioned."
 msgstr ""
 
-#: bzrlib/builtins.py:648
+#: bzrlib/builtins.py:663
 msgid ""
 "In recursive mode (the default), files are treated the same way\n"
 "but the behaviour for directories is different.  Directories that\n"
@@ -191,13 +299,19 @@
 "directories.  If no names are given '.' is assumed."
 msgstr ""
 
-#: bzrlib/builtins.py:656
+#: bzrlib/builtins.py:671
+msgid ""
+"A warning will be printed when nested trees are encountered,\n"
+"unless they are explicitly ignored."
+msgstr ""
+
+#: bzrlib/builtins.py:674
 msgid ""
 "Therefore simply saying 'bzr add' will version all files that\n"
 "are currently unknown."
 msgstr ""
 
-#: bzrlib/builtins.py:659
+#: bzrlib/builtins.py:677
 msgid ""
 "Adding a file whose parent directory is not versioned will\n"
 "implicitly add the parent, and so on up to the root. This means\n"
@@ -205,13 +319,13 @@
 "get added when you add a file in the directory."
 msgstr ""
 
-#: bzrlib/builtins.py:664
+#: bzrlib/builtins.py:682
 msgid ""
 "--dry-run will show which files would be added, but not actually\n"
 "add them."
 msgstr ""
 
-#: bzrlib/builtins.py:667
+#: bzrlib/builtins.py:685
 msgid ""
 "--file-ids-from will try to use the file ids from the supplied path.\n"
 "It looks up ids trying to find a matching parent directory with the\n"
@@ -222,13 +336,13 @@
 "into a subdirectory of this one."
 msgstr ""
 
-#: bzrlib/builtins.py:675
+#: bzrlib/builtins.py:693
 msgid ""
 "Any files matching patterns in the ignore list will not be added\n"
 "unless they are explicitly mentioned."
 msgstr ""
 
-#: bzrlib/builtins.py:678
+#: bzrlib/builtins.py:696
 msgid ""
 "In recursive mode, files larger than the configuration option \n"
 "add.maximum_file_size will be skipped. Named items are never skipped due\n"
@@ -236,39 +350,53 @@
 msgstr ""
 
 # help of 'no-recurse' option of 'add' command
-#: bzrlib/builtins.py:685
+#: bzrlib/builtins.py:703
 msgid "Don't recursively add the contents of directories."
 msgstr ""
 
 # help of 'file-ids-from' option of 'add' command
-#: bzrlib/builtins.py:691
+#: bzrlib/builtins.py:709
 msgid "Lookup file ids from this tree."
 msgstr ""
 
-#: bzrlib/builtins.py:731
+#: bzrlib/builtins.py:745
+msgid "ignored {0} matching \"{1}\"\n"
+msgstr ""
+
+#: bzrlib/builtins.py:750
 msgid "Create a new versioned directory."
 msgstr ""
 
-#: bzrlib/builtins.py:733
+#: bzrlib/builtins.py:752
 msgid "This is equivalent to creating the directory and then adding it."
 msgstr ""
 
-#: bzrlib/builtins.py:828
+#: bzrlib/builtins.py:767
+#, python-format
+msgid "added %s\n"
+msgstr ""
+
+#: bzrlib/builtins.py:811
+#, python-format
+msgid "invalid kind %r specified"
+msgstr ""
+
+#: bzrlib/builtins.py:848
 msgid "Move or rename a file."
 msgstr ""
 
-#: bzrlib/builtins.py:830
+#: bzrlib/builtins.py:850
 msgid "    bzr mv SOURCE... DESTINATION"
 msgstr ""
 
-#: bzrlib/builtins.py:832
+#: bzrlib/builtins.py:852
 msgid ""
 "If the last argument is a versioned directory, all the other names\n"
 "are moved into it.  Otherwise, there must be exactly two arguments\n"
 "and the file is changed to a new name."
 msgstr ""
 
-#: bzrlib/builtins.py:836
+#: bzrlib/builtins.py:856
 msgid ""
 "If OLDNAME does not exist on the filesystem but is versioned and\n"
 "NEWNAME does exist on the filesystem but is not versioned, mv\n"
@@ -277,32 +405,56 @@
 "The same is valid when moving many SOURCE files to a DESTINATION."
 msgstr ""
 
-#: bzrlib/builtins.py:842
+#: bzrlib/builtins.py:862
 msgid "Files cannot be moved between branches."
 msgstr ""
 
 # help of 'after' option of 'mv' command
-#: bzrlib/builtins.py:849
+#: bzrlib/builtins.py:869
 msgid ""
 "Move only the bzr identifier of the file, because the file has already been "
 "moved."
 msgstr ""
 
 # help of 'auto' option of 'mv' command
-#: bzrlib/builtins.py:851
+#: bzrlib/builtins.py:871
 msgid "Automatically guess renames."
 msgstr ""
 
 # help of 'dry-run' option of 'mv' command
-#: bzrlib/builtins.py:852
+#: bzrlib/builtins.py:872
 msgid "Avoid making changes when guessing renames."
 msgstr ""
 
-#: bzrlib/builtins.py:964
+#: bzrlib/builtins.py:881
+msgid "--dry-run requires --auto."
+msgstr ""
+
+#: bzrlib/builtins.py:885
+msgid "missing file argument"
+msgstr ""
+
+#: bzrlib/builtins.py:889
+msgid "can not move root of branch"
+msgstr ""
+
+#: bzrlib/builtins.py:895
+msgid "Only one path may be specified to --auto."
+msgstr ""
+
+#: bzrlib/builtins.py:898
+msgid "--after cannot be specified with --auto."
+msgstr ""
+
+#: bzrlib/builtins.py:935
+msgid "to mv multiple files the destination must be a versioned directory"
+msgstr ""
+
+#: bzrlib/builtins.py:987
 msgid "Turn this branch into a mirror of another branch."
 msgstr ""
 
-#: bzrlib/builtins.py:966
+#: bzrlib/builtins.py:989
 msgid ""
 "By default, this command only works on branches that have not diverged.\n"
 "Branches are considered diverged if the destination branch's most recent \n"
@@ -310,29 +462,36 @@
 "parent."
 msgstr ""
 
-#: bzrlib/builtins.py:971
+#: bzrlib/builtins.py:994
 msgid ""
 "If branches have diverged, you can use 'bzr merge' to integrate the changes\n"
 "from one into the other.  Once one branch has merged, the other should\n"
 "be able to pull it again."
 msgstr ""
 
-#: bzrlib/builtins.py:975
+#: bzrlib/builtins.py:998
 msgid ""
 "If you want to replace your local changes and just want your branch to\n"
 "match the remote one, use pull --overwrite. This will work even if the two\n"
 "branches have diverged."
 msgstr ""
 
-#: bzrlib/builtins.py:979
+#: bzrlib/builtins.py:1002
 msgid ""
 "If there is no default location set, the first pull will set it (use\n"
-"--no-remember to avoid settting it). After that, you can omit the\n"
+"--no-remember to avoid setting it). After that, you can omit the\n"
 "location to use the default.  To change the default, use --remember. The\n"
 "value will only be saved if the remote location can be accessed."
 msgstr ""
 
-#: bzrlib/builtins.py:984
+#: bzrlib/builtins.py:1007
+msgid ""
+"The --verbose option will display the revisions pulled using the log_format\n"
+"configuration option. You can use a different format by overriding it with\n"
+"-Olog_format=<other_format>."
+msgstr ""
+
+#: bzrlib/builtins.py:1011
 msgid ""
 "Note: The location can be specified either in the form of a branch,\n"
 "or in the form of a path to a file containing a merge directive generated\n"
@@ -340,112 +499,136 @@
 msgstr ""
 
 # help of 'verbose' option of 'pull' command
-#: bzrlib/builtins.py:992
+#: bzrlib/builtins.py:1019
 msgid "Show logs of pulled revisions."
 msgstr ""
 
 # help of 'directory' option of 'pull' command
-#: bzrlib/builtins.py:994
+#: bzrlib/builtins.py:1021
 msgid ""
 "Branch to pull into, rather than the one containing the working directory."
 msgstr ""
 
 # help of 'local' option of 'pull' command
-#: bzrlib/builtins.py:997
+#: bzrlib/builtins.py:1024
 msgid ""
 "Perform a local pull in a bound branch.  Local pulls are not applied to the "
 "master branch."
 msgstr ""
 
-#: bzrlib/builtins.py:1097
+#: bzrlib/builtins.py:1053
+msgid "Need working tree for --show-base."
+msgstr ""
+
+#: bzrlib/builtins.py:1069
+msgid "No pull location known or specified."
+msgstr ""
+
+#: bzrlib/builtins.py:1075
+#, python-format
+msgid "Using saved parent location: %s\n"
+msgstr ""
+
+#: bzrlib/builtins.py:1082 bzrlib/builtins.py:4234
+msgid "Cannot use -r with merge directives or bundles"
+msgstr ""
+
+#: bzrlib/builtins.py:1123
 msgid "Update a mirror of this branch."
 msgstr ""
 
-#: bzrlib/builtins.py:1099
+#: bzrlib/builtins.py:1125
 msgid ""
 "The target branch will not have its working tree populated because this\n"
 "is both expensive, and is not supported on remote file systems."
 msgstr ""
 
-#: bzrlib/builtins.py:1102
+#: bzrlib/builtins.py:1128
 msgid ""
 "Some smart servers or protocols *may* put the working tree in place in\n"
 "the future."
 msgstr ""
 
-#: bzrlib/builtins.py:1105
+#: bzrlib/builtins.py:1131
 msgid ""
 "This command only works on branches that have not diverged.  Branches are\n"
 "considered diverged if the destination branch's most recent commit is one\n"
 "that has not been merged (directly or indirectly) by the source branch."
 msgstr ""
 
-#: bzrlib/builtins.py:1109
+#: bzrlib/builtins.py:1135
 msgid ""
 "If branches have diverged, you can use 'bzr push --overwrite' to replace\n"
 "the other branch completely, discarding its unmerged changes."
 msgstr ""
 
-#: bzrlib/builtins.py:1112
+#: bzrlib/builtins.py:1138
 msgid ""
 "If you want to ensure you have the different changes in the other branch,\n"
 "do a merge (see bzr help merge) from the other branch, and commit that.\n"
 "After that you will be able to do a push without '--overwrite'."
 msgstr ""
 
-#: bzrlib/builtins.py:1116
+#: bzrlib/builtins.py:1142
 msgid ""
 "If there is no default push location set, the first push will set it (use\n"
-"--no-remember to avoid settting it).  After that, you can omit the\n"
+"--no-remember to avoid setting it).  After that, you can omit the\n"
 "location to use the default.  To change the default, use --remember. The\n"
 "value will only be saved if the remote location can be accessed."
 msgstr ""
 
-# help of 'directory' option of 'push' command
-#: bzrlib/builtins.py:1128
+#: bzrlib/builtins.py:1147
 msgid ""
-"Branch to push from, rather than the one containing the working directory."
+"The --verbose option will display the revisions pushed using the log_format\n"
+"configuration option. You can use a different format by overriding it with\n"
+"-Olog_format=<other_format>."
 msgstr ""
 
 # help of 'use-existing-dir' option of 'push' command
-#: bzrlib/builtins.py:1131
+#: bzrlib/builtins.py:1161
 msgid ""
 "By default push will fail if the target directory exists, but does not "
 "already have a control directory.  This flag will allow push to proceed."
 msgstr ""
 
 # help of 'stacked' option of 'push' command
-#: bzrlib/builtins.py:1136
+#: bzrlib/builtins.py:1166
 msgid ""
 "Create a stacked branch that references the public location of the parent "
 "branch."
 msgstr ""
 
 # help of 'stacked-on' option of 'push' command
-#: bzrlib/builtins.py:1139
+#: bzrlib/builtins.py:1169
 msgid ""
 "Create a stacked branch that refers to another branch for the commit "
 "history. Only the work not present in the referenced branch is included in "
 "the branch created."
 msgstr ""
 
-# help of 'strict' option of 'push' command
-#: bzrlib/builtins.py:1144
-msgid ""
-"Refuse to push if there are uncommitted changes in the working tree, --no-"
-"strict disables the check."
-msgstr ""
-
 # help of 'no-tree' option of 'push' command
-#: bzrlib/builtins.py:1147
+#: bzrlib/builtins.py:1177
 msgid "Don't populate the working tree, even for protocols that support it."
 msgstr ""
 
-#: bzrlib/builtins.py:1212
+#: bzrlib/builtins.py:1221
+msgid "Could not determine branch to refer to."
+msgstr ""
+
+#: bzrlib/builtins.py:1228 bzrlib/foreign.py:305
+msgid "No push location known or specified."
+msgstr ""
+
+#: bzrlib/builtins.py:1232
+#, python-format
+msgid "Using saved push location: %s"
+msgstr ""
+
+#: bzrlib/builtins.py:1242
 msgid "Create a new branch that is a copy of an existing branch."
 msgstr ""
 
-#: bzrlib/builtins.py:1214
+#: bzrlib/builtins.py:1244
 msgid ""
 "If the TO_LOCATION is omitted, the last component of the FROM_LOCATION will\n"
 "be used.  In other words, \"branch ../foo/bar\" will attempt to create ./"
@@ -456,65 +639,112 @@
 "create ./foo-bar."
 msgstr ""
 
-#: bzrlib/builtins.py:1221
+#: bzrlib/builtins.py:1251
 msgid ""
 "To retrieve the branch as of a particular revision, supply the --revision\n"
 "parameter, as in \"branch foo/bar -r 5\"."
 msgstr ""
 
-#: bzrlib/builtins.py:1224
+#: bzrlib/builtins.py:1254
 msgid "The synonyms 'clone' and 'get' for this command are deprecated."
 msgstr ""
 
 # help of 'no-tree' option of 'branch' command
-#: bzrlib/builtins.py:1234
+#: bzrlib/builtins.py:1264
 msgid "Create a branch without a working-tree."
 msgstr ""
 
 # help of 'switch' option of 'branch' command
-#: bzrlib/builtins.py:1236
+#: bzrlib/builtins.py:1266
 msgid "Switch the checkout in the current directory to the new branch."
 msgstr ""
 
 # help of 'stacked' option of 'branch' command
-#: bzrlib/builtins.py:1239
+#: bzrlib/builtins.py:1269
 msgid ""
 "Create a stacked branch referring to the source branch. The new branch will "
 "depend on the availability of the source branch for all operations."
 msgstr ""
 
 # help of 'standalone' option of 'branch' command
-#: bzrlib/builtins.py:1243
+#: bzrlib/builtins.py:1273
 msgid "Do not use a shared repository, even if available."
 msgstr ""
 
 # help of 'use-existing-dir' option of 'branch' command
-#: bzrlib/builtins.py:1245
+#: bzrlib/builtins.py:1275
 msgid ""
 "By default branch will fail if the target directory exists, but does not "
 "already have a control directory.  This flag will allow branch to proceed."
 msgstr ""
 
 # help of 'bind' option of 'branch' command
-#: bzrlib/builtins.py:1250
+#: bzrlib/builtins.py:1280
 msgid "Bind new branch to from location."
 msgstr ""
 
-#: bzrlib/builtins.py:1342
+#: bzrlib/builtins.py:1327
+#, python-format
+msgid "Target directory \"%s\" already exists."
+msgstr ""
+
+#: bzrlib/builtins.py:1339
+#, python-format
+msgid "Parent of \"%s\" does not exist."
+msgstr ""
+
+#: bzrlib/builtins.py:1358
+msgid "The branch {0} has no revision {1}."
+msgstr ""
+
+#: bzrlib/builtins.py:1368 bzrlib/push.py:51
+#, python-format
+msgid "Created new stacked branch referring to %s."
+msgstr ""
+
+#: bzrlib/builtins.py:1372
+#, python-format
+msgid "Branched %d revision."
+msgid_plural "Branched %d revisions."
+msgstr[0] ""
+msgstr[1] ""
+
+#: bzrlib/builtins.py:1377
+#, python-format
+msgid "New branch bound to %s"
+msgstr ""
+
+#: bzrlib/builtins.py:1382 bzrlib/builtins.py:6121
+#, python-format
+msgid "Switched to branch: %s"
+msgstr ""
+
+#: bzrlib/builtins.py:1387
 msgid "List the branches available at the current location."
 msgstr ""
 
-#: bzrlib/builtins.py:1344
-msgid ""
-"This command will print the names of all the branches at the current "
-"location."
-msgstr ""
-
-#: bzrlib/builtins.py:1359
+#: bzrlib/builtins.py:1389
+msgid ""
+"This command will print the names of all the branches at the current\n"
+"location."
+msgstr ""
+
+# help of 'recursive' option of 'branches' command
+#: bzrlib/builtins.py:1396
+msgid ""
+"Recursively scan for branches rather than just looking in the specified "
+"location."
+msgstr ""
+
+#: bzrlib/builtins.py:1413
+msgid " (default)\n"
+msgstr ""
+
+#: bzrlib/builtins.py:1420
 msgid "Create a new checkout of an existing branch."
 msgstr ""
 
-#: bzrlib/builtins.py:1361
+#: bzrlib/builtins.py:1422
 msgid ""
 "If BRANCH_LOCATION is omitted, checkout will reconstitute a working tree "
 "for\n"
@@ -524,7 +754,7 @@
 "location using SFTP."
 msgstr ""
 
-#: bzrlib/builtins.py:1366
+#: bzrlib/builtins.py:1427
 msgid ""
 "If the TO_LOCATION is omitted, the last component of the BRANCH_LOCATION "
 "will\n"
@@ -536,7 +766,7 @@
 "create ./foo-bar."
 msgstr ""
 
-#: bzrlib/builtins.py:1373
+#: bzrlib/builtins.py:1434
 msgid ""
 "To retrieve the branch as of a particular revision, supply the --revision\n"
 "parameter, as in \"checkout foo/bar -r 5\". Note that this will be "
@@ -547,7 +777,7 @@
 msgstr ""
 
 # help of 'lightweight' option of 'checkout' command
-#: bzrlib/builtins.py:1383
+#: bzrlib/builtins.py:1444
 msgid ""
 "Perform a lightweight checkout.  Lightweight checkouts depend on access to "
 "the branch for every operation.  Normal checkouts can perform common "
@@ -556,103 +786,151 @@
 msgstr ""
 
 # help of 'files-from' option of 'branch' command
-#: bzrlib/builtins.py:1390
+#: bzrlib/builtins.py:1451
 msgid "Get file contents from this tree."
 msgstr ""
 
 # help of 'hardlink' option of 'branch' command
-#: bzrlib/builtins.py:1392
+#: bzrlib/builtins.py:1453
 msgid "Hard-link working tree files where possible."
 msgstr ""
 
-#: bzrlib/builtins.py:1433
+#: bzrlib/builtins.py:1494
 msgid ""
 "Show list of renamed files.\n"
 "    "
 msgstr ""
 
-#: bzrlib/builtins.py:1463
-msgid "Update a tree to have the latest code committed to its branch."
-msgstr ""
-
-#: bzrlib/builtins.py:1465
-msgid ""
-"This will perform a merge into the working tree, and may generate\n"
-"conflicts. If you have any local changes, you will still\n"
-"need to commit them after the update for the update to be complete."
-msgstr ""
-
-#: bzrlib/builtins.py:1469
-msgid ""
-"If you want to discard your local changes, you can just do a\n"
-"'bzr revert' instead of 'bzr commit' after the update."
-msgstr ""
-
-#: bzrlib/builtins.py:1472
-msgid ""
-"If you want to restore a file that has been removed locally, use\n"
-"'bzr revert' instead of 'bzr update'."
-msgstr ""
-
-#: bzrlib/builtins.py:1475
-msgid ""
-"If the tree's branch is bound to a master branch, it will also update\n"
+#: bzrlib/builtins.py:1524
+msgid "Update a working tree to a new revision."
+msgstr ""
+
+#: bzrlib/builtins.py:1526
+msgid ""
+"This will perform a merge of the destination revision (the tip of the\n"
+"branch, or the specified revision) into the working tree, and then make\n"
+"that revision the basis revision for the working tree.  "
+msgstr ""
+
+#: bzrlib/builtins.py:1530
+msgid ""
+"You can use this to visit an older revision, or to update a working tree\n"
+"that is out of date from its branch."
+msgstr ""
+
+#: bzrlib/builtins.py:1533
+msgid ""
+"If there are any uncommitted changes in the tree, they will be carried\n"
+"across and remain as uncommitted changes after the update.  To discard\n"
+"these changes, use 'bzr revert'.  The uncommitted changes may conflict\n"
+"with the changes brought in by the change in basis revision."
+msgstr ""
+
+#: bzrlib/builtins.py:1538
+msgid ""
+"If the tree's branch is bound to a master branch, bzr will also update\n"
 "the branch from the master."
 msgstr ""
 
-#: bzrlib/builtins.py:1556
+#: bzrlib/builtins.py:1541
+msgid ""
+"You cannot update just a single file or directory, because each Bazaar\n"
+"working tree has just a single basis revision.  If you want to restore a\n"
+"file that has been removed locally, use 'bzr revert' instead of 'bzr\n"
+"update'.  If you want to restore a file to its state in a previous\n"
+"revision, use 'bzr revert' with a '-r' option, or use 'bzr cat' to write\n"
+"out the old content of that file to a new location."
+msgstr ""
+
+#: bzrlib/builtins.py:1548
+msgid ""
+"The 'dir' argument, if given, must be the location of the root of a\n"
+"working tree to update.  By default, the working tree that contains the \n"
+"current working directory is used."
+msgstr ""
+
+#: bzrlib/builtins.py:1564
+msgid "bzr update --revision takes exactly one revision"
+msgstr ""
+
+#: bzrlib/builtins.py:1572
+msgid "bzr update can only update a whole tree, not a file or subdirectory"
+msgstr ""
+
+#: bzrlib/builtins.py:1603
+msgid "Tree is up to date at revision {0} of branch {1}"
+msgstr ""
+
+#: bzrlib/builtins.py:1619
+#, python-format
+msgid ""
+"branch has no revision %s\n"
+"bzr update --revision only works for a revision in the branch history"
+msgstr ""
+
+#: bzrlib/builtins.py:1625
+msgid "Updated to revision {0} of branch {1}"
+msgstr ""
+
+#: bzrlib/builtins.py:1629
+msgid ""
+"Your local commits will now show as pending merges with 'bzr status', and "
+"can be committed with 'bzr commit'."
+msgstr ""
+
+#: bzrlib/builtins.py:1638
 msgid "Show information about a working tree, branch or repository."
 msgstr ""
 
-#: bzrlib/builtins.py:1558
+#: bzrlib/builtins.py:1640
 msgid ""
 "This command will show all known locations and formats associated to the\n"
 "tree, branch or repository."
 msgstr ""
 
-#: bzrlib/builtins.py:1561
+#: bzrlib/builtins.py:1643
 msgid ""
 "In verbose mode, statistical information is included with each report.\n"
 "To see extended statistic information, use a verbosity level of 2 or\n"
 "higher by specifying the verbose option multiple times, e.g. -vv."
 msgstr ""
 
-#: bzrlib/builtins.py:1565
+#: bzrlib/builtins.py:1647
 msgid "Branches and working trees will also report any missing revisions."
 msgstr ""
 
-#: bzrlib/builtins.py:1569
+#: bzrlib/builtins.py:1651
 msgid "  Display information on the format and related locations:"
 msgstr ""
 
-#: bzrlib/builtins.py:1571
+#: bzrlib/builtins.py:1653
 msgid "    bzr info"
 msgstr ""
 
-#: bzrlib/builtins.py:1573
+#: bzrlib/builtins.py:1655
 msgid ""
 "  Display the above together with extended format information and\n"
 "  basic statistics (like the number of files in the working tree and\n"
 "  number of revisions in the branch and repository):"
 msgstr ""
 
-#: bzrlib/builtins.py:1577
+#: bzrlib/builtins.py:1659
 msgid "    bzr info -v"
 msgstr ""
 
-#: bzrlib/builtins.py:1579
+#: bzrlib/builtins.py:1661
 msgid "  Display the above together with number of committers to the branch:"
 msgstr ""
 
-#: bzrlib/builtins.py:1581
+#: bzrlib/builtins.py:1663
 msgid "    bzr info -vv"
 msgstr ""
 
-#: bzrlib/builtins.py:1600
+#: bzrlib/builtins.py:1682
 msgid "Remove files or directories."
 msgstr ""
 
-#: bzrlib/builtins.py:1602
+#: bzrlib/builtins.py:1684
 msgid ""
 "This makes Bazaar stop tracking changes to the specified files. Bazaar will\n"
 "delete them if they can easily be recovered using revert otherwise they\n"
@@ -662,25 +940,55 @@
 msgstr ""
 
 # help of 'new' option of 'remove' command
-#: bzrlib/builtins.py:1610
+#: bzrlib/builtins.py:1692
 msgid "Only remove files that have never been committed."
 msgstr ""
 
 # help of 'file-deletion-strategy' option of 'remove' command
-#: bzrlib/builtins.py:1612
+#: bzrlib/builtins.py:1694
 msgid "The file deletion mode to be used."
 msgstr ""
 
 # title of 'file-deletion-strategy' option of 'remove' command
-#: bzrlib/builtins.py:1613
+#: bzrlib/builtins.py:1695
 msgid "Deletion Strategy"
 msgstr ""
 
-#: bzrlib/builtins.py:1704
+# help of 'file-deletion-strategy=safe' option of 'remove' command
+#: bzrlib/builtins.py:1696
+msgid "Backup changed files (default)."
+msgstr ""
+
+# help of 'file-deletion-strategy=keep' option of 'remove' command
+#: bzrlib/builtins.py:1697
+msgid "Delete from bzr but leave the working copy."
+msgstr ""
+
+# help of 'file-deletion-strategy=no-backup' option of 'remove' command
+#: bzrlib/builtins.py:1698
+msgid "Don't backup changed files."
+msgstr ""
+
+# help of 'file-deletion-strategy=force' option of 'remove' command
+#: bzrlib/builtins.py:1699
+msgid ""
+"Delete all the specified files, even if they can not be recovered and even "
+"if they are non-empty directories. (deprecated, use no-backup)"
+msgstr ""
+
+#: bzrlib/builtins.py:1708
+msgid "(The --force option is deprecated, rather use --no-backup in future.)"
+msgstr ""
+
+#: bzrlib/builtins.py:1725
+msgid "No matching files."
+msgstr ""
+
+#: bzrlib/builtins.py:1786
 msgid "Reconcile bzr metadata in a branch."
 msgstr ""
 
-#: bzrlib/builtins.py:1706
+#: bzrlib/builtins.py:1788
 msgid ""
 "This can correct data mismatches that may have been caused by\n"
 "previous ghost operations or bzr upgrades. You should only\n"
@@ -688,7 +996,7 @@
 "advises you to run it."
 msgstr ""
 
-#: bzrlib/builtins.py:1711
+#: bzrlib/builtins.py:1793
 msgid ""
 "If a second branch is provided, cross-branch reconciliation is\n"
 "also attempted, which will check that data like the tree root\n"
@@ -696,27 +1004,47 @@
 "correctly in both branches."
 msgstr ""
 
-#: bzrlib/builtins.py:1716
+#: bzrlib/builtins.py:1798
 msgid ""
 "At the same time it is run it may recompress data resulting in\n"
 "a potential saving in disk space or performance gain."
 msgstr ""
 
-#: bzrlib/builtins.py:1719
+#: bzrlib/builtins.py:1801
 msgid "The branch *MUST* be on a listable system such as local disk or sftp."
 msgstr ""
 
-#: bzrlib/builtins.py:1783
+# help of 'format=2a' option of 'init' command
+#: bzrlib/builtins.py:1868
+msgid ""
+"Format for the bzr 2.0 series.\n"
+"Uses group-compress storage.\n"
+"Provides rich roots which are a one-way transition.\n"
+msgstr ""
+
+# help of 'format=pack-0.92' option of 'init' command
+#: bzrlib/builtins.py:1868
+msgid ""
+"Pack-based format used in 1.x series. Introduced in 0.92. Interoperates with "
+"bzr repositories before 0.92 but cannot be read by bzr < 0.92. "
+msgstr ""
+
+# help of 'format=development-colo' option of 'init' command
+#: bzrlib/builtins.py:1868
+msgid "The 2a format with experimental support for colocated branches.\n"
+msgstr ""
+
+#: bzrlib/builtins.py:1869
 msgid "Make a directory into a versioned branch."
 msgstr ""
 
-#: bzrlib/builtins.py:1785
+#: bzrlib/builtins.py:1871
 msgid ""
 "Use this to create an empty branch, or before importing an\n"
 "existing project."
 msgstr ""
 
-#: bzrlib/builtins.py:1788
+#: bzrlib/builtins.py:1874
 msgid ""
 "If there is a repository in a parent directory of the location, then\n"
 "the history of the branch will be stored in the repository.  Otherwise\n"
@@ -724,17 +1052,17 @@
 "in the .bzr directory."
 msgstr ""
 
-#: bzrlib/builtins.py:1793
+#: bzrlib/builtins.py:1879
 msgid ""
 "If there is already a branch at the location but it has no working tree,\n"
 "the tree can be populated with 'bzr checkout'."
 msgstr ""
 
-#: bzrlib/builtins.py:1796
+#: bzrlib/builtins.py:1882
 msgid "Recipe for importing a tree of files::"
 msgstr ""
 
-#: bzrlib/builtins.py:1798
+#: bzrlib/builtins.py:1884
 msgid ""
 "    cd ~/project\n"
 "    bzr init\n"
@@ -744,30 +1072,51 @@
 msgstr ""
 
 # help of 'create-prefix' option of 'init' command
-#: bzrlib/builtins.py:1809
+#: bzrlib/builtins.py:1895
 msgid "Create the path leading up to the branch if it does not already exist."
 msgstr ""
 
 # help of 'format' option of 'init' command
-#: bzrlib/builtins.py:1812
+#: bzrlib/builtins.py:1898
 msgid "Specify a format for this branch. See \"help formats\"."
 msgstr ""
 
 # help of 'append-revisions-only' option of 'init' command
-#: bzrlib/builtins.py:1820
+#: bzrlib/builtins.py:1906
 msgid "Never change revnos or the existing log.  Append revisions to it only."
 msgstr ""
 
 # help of 'no-tree' option of 'init' command
-#: bzrlib/builtins.py:1823
+#: bzrlib/builtins.py:1909
 msgid "Create a branch without a working tree."
 msgstr ""
 
-#: bzrlib/builtins.py:1900
+#: bzrlib/builtins.py:1929 bzrlib/push.py:108
+#, python-format
+msgid ""
+"Parent directory of %s does not exist.\n"
+"You may supply --create-prefix to create all leading parent directories."
+msgstr ""
+
+#: bzrlib/builtins.py:1963
+msgid ""
+"This branch format cannot be set to append-revisions-only.  Try --default."
+msgstr ""
+
+#: bzrlib/builtins.py:1974
+msgid "Created a {0} (format: {1})\n"
+msgstr ""
+
+#: bzrlib/builtins.py:1983
+#, python-format
+msgid "Using shared repository: %s\n"
+msgstr ""
+
+#: bzrlib/builtins.py:1987
 msgid "Create a shared repository for branches to share storage space."
 msgstr ""
 
-#: bzrlib/builtins.py:1902
+#: bzrlib/builtins.py:1989
 msgid ""
 "New branches created under the repository directory will store their\n"
 "revisions in the repository, not in the branch directory.  For branches\n"
@@ -775,7 +1124,7 @@
 "speeds up the creation of new branches."
 msgstr ""
 
-#: bzrlib/builtins.py:1907
+#: bzrlib/builtins.py:1994
 msgid ""
 "If the --no-trees option is given then the branches in the repository\n"
 "will not have working trees by default.  They will still exist as \n"
@@ -785,23 +1134,23 @@
 "branches, such as on a server."
 msgstr ""
 
-#: bzrlib/builtins.py:1914
+#: bzrlib/builtins.py:2001
 msgid ""
 ":Examples:\n"
 "    Create a shared repository holding just branches::"
 msgstr ""
 
-#: bzrlib/builtins.py:1917
+#: bzrlib/builtins.py:2004
 msgid ""
 "        bzr init-repo --no-trees repo\n"
 "        bzr init repo/trunk"
 msgstr ""
 
-#: bzrlib/builtins.py:1920
+#: bzrlib/builtins.py:2007
 msgid "    Make a lightweight checkout elsewhere::"
 msgstr ""
 
-#: bzrlib/builtins.py:1922
+#: bzrlib/builtins.py:2009
 msgid ""
 "        bzr checkout --lightweight repo/trunk trunk-checkout\n"
 "        cd trunk-checkout\n"
@@ -809,26 +1158,26 @@
 msgstr ""
 
 # help of 'format' option of 'init-repository' command
-#: bzrlib/builtins.py:1930
+#: bzrlib/builtins.py:2017
 msgid ""
 "Specify a format for this repository. See \"bzr help formats\" for details."
 msgstr ""
 
 # title of 'format' option of 'init-repository' command
-#: bzrlib/builtins.py:1934
+#: bzrlib/builtins.py:2021
 msgid "Repository format"
 msgstr ""
 
 # help of 'no-trees' option of 'init-repository' command
-#: bzrlib/builtins.py:1936
+#: bzrlib/builtins.py:2023
 msgid "Branches in the repository will default to not having a working tree."
 msgstr ""
 
-#: bzrlib/builtins.py:1960
+#: bzrlib/builtins.py:2047
 msgid "Show differences in the working tree, between revisions or branches."
 msgstr ""
 
-#: bzrlib/builtins.py:1962
+#: bzrlib/builtins.py:2049
 msgid ""
 "If no arguments are given, all changes for the current tree are listed.\n"
 "If files are given, only the changes in those files are listed.\n"
@@ -838,13 +1187,13 @@
 "given."
 msgstr ""
 
-#: bzrlib/builtins.py:1969
+#: bzrlib/builtins.py:2056
 msgid ""
 "\"bzr diff -p1\" is equivalent to \"bzr diff --prefix old/:new/\", and\n"
 "produces patches suitable for \"patch -p1\"."
 msgstr ""
 
-#: bzrlib/builtins.py:1972
+#: bzrlib/builtins.py:2059
 msgid ""
 "Note that when using the -r argument with a range of revisions, the\n"
 "differences are computed between the two specified revisions.  That\n"
@@ -854,7 +1203,7 @@
 "in the range."
 msgstr ""
 
-#: bzrlib/builtins.py:1979
+#: bzrlib/builtins.py:2066
 msgid ""
 ":Exit values:\n"
 "    1 - changed\n"
@@ -863,113 +1212,113 @@
 "    0 - no change"
 msgstr ""
 
-#: bzrlib/builtins.py:1985
+#: bzrlib/builtins.py:2072
 msgid ""
 ":Examples:\n"
 "    Shows the difference in the working tree versus the last commit::"
 msgstr ""
 
-#: bzrlib/builtins.py:1988
+#: bzrlib/builtins.py:2075
 msgid "        bzr diff"
 msgstr ""
 
-#: bzrlib/builtins.py:1990
+#: bzrlib/builtins.py:2077
 msgid "    Difference between the working tree and revision 1::"
 msgstr ""
 
-#: bzrlib/builtins.py:1992
+#: bzrlib/builtins.py:2079
 msgid "        bzr diff -r1"
 msgstr ""
 
-#: bzrlib/builtins.py:1994
+#: bzrlib/builtins.py:2081
 msgid "    Difference between revision 3 and revision 1::"
 msgstr ""
 
-#: bzrlib/builtins.py:1996
+#: bzrlib/builtins.py:2083
 msgid "        bzr diff -r1..3"
 msgstr ""
 
-#: bzrlib/builtins.py:1998
+#: bzrlib/builtins.py:2085
 msgid "    Difference between revision 3 and revision 1 for branch xxx::"
 msgstr ""
 
-#: bzrlib/builtins.py:2000
+#: bzrlib/builtins.py:2087
 msgid "        bzr diff -r1..3 xxx"
 msgstr ""
 
-#: bzrlib/builtins.py:2002
+#: bzrlib/builtins.py:2089
 msgid "    The changes introduced by revision 2 (equivalent to -r1..2)::"
 msgstr ""
 
-#: bzrlib/builtins.py:2004
+#: bzrlib/builtins.py:2091
 msgid "        bzr diff -c2"
 msgstr ""
 
-#: bzrlib/builtins.py:2006
+#: bzrlib/builtins.py:2093
 msgid ""
 "    To see the changes introduced by revision X::\n"
 "    \n"
 "        bzr diff -cX"
 msgstr ""
 
-#: bzrlib/builtins.py:2010
+#: bzrlib/builtins.py:2097
 msgid ""
 "    Note that in the case of a merge, the -c option shows the changes\n"
 "    compared to the left hand parent. To see the changes against\n"
 "    another parent, use::"
 msgstr ""
 
-#: bzrlib/builtins.py:2014
+#: bzrlib/builtins.py:2101
 msgid "        bzr diff -r<chosen_parent>..X"
 msgstr ""
 
-#: bzrlib/builtins.py:2016
+#: bzrlib/builtins.py:2103
 msgid ""
 "    The changes between the current revision and the previous revision\n"
 "    (equivalent to -c-1 and -r-2..-1)"
 msgstr ""
 
-#: bzrlib/builtins.py:2019
+#: bzrlib/builtins.py:2106
 msgid "        bzr diff -r-2.."
 msgstr ""
 
-#: bzrlib/builtins.py:2021
+#: bzrlib/builtins.py:2108
 msgid "    Show just the differences for file NEWS::"
 msgstr ""
 
-#: bzrlib/builtins.py:2023
+#: bzrlib/builtins.py:2110
 msgid "        bzr diff NEWS"
 msgstr ""
 
-#: bzrlib/builtins.py:2025
+#: bzrlib/builtins.py:2112
 msgid "    Show the differences in working tree xxx for file NEWS::"
 msgstr ""
 
-#: bzrlib/builtins.py:2027
+#: bzrlib/builtins.py:2114
 msgid "        bzr diff xxx/NEWS"
 msgstr ""
 
-#: bzrlib/builtins.py:2029
+#: bzrlib/builtins.py:2116
 msgid "    Show the differences from branch xxx to this working tree:"
 msgstr ""
 
-#: bzrlib/builtins.py:2031
+#: bzrlib/builtins.py:2118
 msgid "        bzr diff --old xxx"
 msgstr ""
 
-#: bzrlib/builtins.py:2033
+#: bzrlib/builtins.py:2120
 msgid "    Show the differences between two branches for file NEWS::"
 msgstr ""
 
-#: bzrlib/builtins.py:2035
+#: bzrlib/builtins.py:2122
 msgid "        bzr diff --old xxx --new yyy NEWS"
 msgstr ""
 
-#: bzrlib/builtins.py:2037
+#: bzrlib/builtins.py:2124
 msgid "    Same as 'bzr diff' but prefix paths with old/ and new/::"
 msgstr ""
 
-#: bzrlib/builtins.py:2039
+#: bzrlib/builtins.py:2126
 msgid ""
 "        bzr diff --prefix old/:new/\n"
 "        \n"
@@ -979,63 +1328,84 @@
 msgstr ""
 
 # help of 'diff-options' option of 'diff' command
-#: bzrlib/builtins.py:2049
+#: bzrlib/builtins.py:2136
 msgid "Pass these options to the external diff program."
 msgstr ""
 
 # help of 'prefix' option of 'diff' command
-#: bzrlib/builtins.py:2052
+#: bzrlib/builtins.py:2139
 msgid ""
 "Set prefixes added to old and new filenames, as two values separated by a "
 "colon. (eg \"old/:new/\")."
 msgstr ""
 
 # help of 'old' option of 'diff' command
-#: bzrlib/builtins.py:2055
+#: bzrlib/builtins.py:2142
 msgid "Branch/tree to compare from."
 msgstr ""
 
 # help of 'new' option of 'diff' command
-#: bzrlib/builtins.py:2059
+#: bzrlib/builtins.py:2146
 msgid "Branch/tree to compare to."
 msgstr ""
 
 # help of 'using' option of 'diff' command
-#: bzrlib/builtins.py:2065
+#: bzrlib/builtins.py:2152
 msgid "Use this command to compare files."
 msgstr ""
 
 # help of 'format' option of 'diff' command
-#: bzrlib/builtins.py:2070
+#: bzrlib/builtins.py:2157
 msgid "Diff format to use."
 msgstr ""
 
 # title of 'format' option of 'diff' command
-#: bzrlib/builtins.py:2072
+#: bzrlib/builtins.py:2159
 msgid "Diff format"
 msgstr ""
 
-#: bzrlib/builtins.py:2122
+#: bzrlib/builtins.py:2181
+msgid "--prefix expects two values separated by a colon (eg \"old/:new/\")"
+msgstr ""
+
+#: bzrlib/builtins.py:2185
+msgid "bzr diff --revision takes exactly one or two revision specifiers"
+msgstr ""
+
+#: bzrlib/builtins.py:2190 bzrlib/builtins.py:2588 bzrlib/builtins.py:2601
+#: bzrlib/builtins.py:2609 bzrlib/builtins.py:4760
+msgid "{0} and {1} are mutually exclusive"
+msgstr ""
+
+#: bzrlib/builtins.py:2210
 msgid ""
 "List files deleted in the working tree.\n"
 "    "
 msgstr ""
 
-#: bzrlib/builtins.py:2200
+#: bzrlib/builtins.py:2288
 msgid "Show the tree root directory."
 msgstr ""
 
-#: bzrlib/builtins.py:2202
+#: bzrlib/builtins.py:2290
 msgid ""
 "The root is the nearest enclosing directory with a .bzr control\n"
 "directory."
 msgstr ""
 
-#: bzrlib/builtins.py:2230
+#: bzrlib/builtins.py:2305
+msgid "The limit argument must be an integer."
+msgstr ""
+
+#: bzrlib/builtins.py:2313
+msgid "The levels argument must be an integer."
+msgstr ""
+
+#: bzrlib/builtins.py:2318
 msgid "Show historical log for a branch or subset of a branch."
 msgstr ""
 
-#: bzrlib/builtins.py:2232
+#: bzrlib/builtins.py:2320
 msgid ""
 "log is bzr's default tool for exploring the history of a branch.\n"
 "The branch to use is taken from the first parameter. If no parameters\n"
@@ -1043,14 +1413,14 @@
 "Here are some simple examples::"
 msgstr ""
 
-#: bzrlib/builtins.py:2237
+#: bzrlib/builtins.py:2325
 msgid ""
 "  bzr log                       log the current branch\n"
 "  bzr log foo.py                log a file in its branch\n"
 "  bzr log http://server/branch  log a branch on a server"
 msgstr ""
 
-#: bzrlib/builtins.py:2241
+#: bzrlib/builtins.py:2329
 msgid ""
 "The filtering, ordering and information shown for each revision can\n"
 "be controlled as explained below. By default, all revisions are\n"
@@ -1060,11 +1430,11 @@
 "were merged."
 msgstr ""
 
-#: bzrlib/builtins.py:2248
+#: bzrlib/builtins.py:2336
 msgid ":Output control:"
 msgstr ""
 
-#: bzrlib/builtins.py:2250
+#: bzrlib/builtins.py:2338
 msgid ""
 "  The log format controls how information about each revision is\n"
 "  displayed. The standard log formats are called ``long``, ``short``\n"
@@ -1072,13 +1442,13 @@
 "  for more details on log formats."
 msgstr ""
 
-#: bzrlib/builtins.py:2255
+#: bzrlib/builtins.py:2343
 msgid ""
 "  The following options can be used to control what information is\n"
 "  displayed::"
 msgstr ""
 
-#: bzrlib/builtins.py:2258
+#: bzrlib/builtins.py:2346
 msgid ""
 "    -l N        display a maximum of N revisions\n"
 "    -n N        display N levels of revisions (0 for all, 1 for collapsed)\n"
@@ -1087,25 +1457,25 @@
 "    --show-ids  display revision-ids (and file-ids), not just revnos"
 msgstr ""
 
-#: bzrlib/builtins.py:2264
+#: bzrlib/builtins.py:2352
 msgid ""
 "  Note that the default number of levels to display is a function of the\n"
 "  log format. If the -n option is not used, the standard log formats show\n"
 "  just the top level (mainline)."
 msgstr ""
 
-#: bzrlib/builtins.py:2268
+#: bzrlib/builtins.py:2356
 msgid ""
 "  Status summaries are shown using status flags like A, M, etc. To see\n"
 "  the changes explained using words like ``added`` and ``modified``\n"
 "  instead, use the -vv option."
 msgstr ""
 
-#: bzrlib/builtins.py:2272
+#: bzrlib/builtins.py:2360
 msgid ":Ordering control:"
 msgstr ""
 
-#: bzrlib/builtins.py:2274
+#: bzrlib/builtins.py:2362
 msgid ""
 "  To display revisions from oldest to newest, use the --forward option.\n"
 "  In most cases, using this option will have little impact on the total\n"
@@ -1113,17 +1483,17 @@
 "  display revisions like --reverse does when it can."
 msgstr ""
 
-#: bzrlib/builtins.py:2279
+#: bzrlib/builtins.py:2367
 msgid ":Revision filtering:"
 msgstr ""
 
-#: bzrlib/builtins.py:2281
+#: bzrlib/builtins.py:2369
 msgid ""
 "  The -r option can be used to specify what revision or range of revisions\n"
 "  to filter against. The various forms are shown below::"
 msgstr ""
 
-#: bzrlib/builtins.py:2284
+#: bzrlib/builtins.py:2372
 msgid ""
 "    -rX      display revision X\n"
 "    -rX..    display revision X and later\n"
@@ -1131,13 +1501,13 @@
 "    -rX..Y   display from X to Y inclusive"
 msgstr ""
 
-#: bzrlib/builtins.py:2289
+#: bzrlib/builtins.py:2377
 msgid ""
 "  See ``bzr help revisionspec`` for details on how to specify X and Y.\n"
 "  Some common examples are given below::"
 msgstr ""
 
-#: bzrlib/builtins.py:2292
+#: bzrlib/builtins.py:2380
 msgid ""
 "    -r-1                show just the tip\n"
 "    -r-10..             show the last 10 mainline revisions\n"
@@ -1147,7 +1517,7 @@
 "    -rdate:yesterday..  show changes since yesterday"
 msgstr ""
 
-#: bzrlib/builtins.py:2299
+#: bzrlib/builtins.py:2387
 msgid ""
 "  When logging a range of revisions using -rX..Y, log starts at\n"
 "  revision Y and searches back in history through the primary\n"
@@ -1157,73 +1527,73 @@
 "  a nested merge revision and the log will be truncated accordingly."
 msgstr ""
 
-#: bzrlib/builtins.py:2306
+#: bzrlib/builtins.py:2394
 msgid ":Path filtering:"
 msgstr ""
 
-#: bzrlib/builtins.py:2308
+#: bzrlib/builtins.py:2396
 msgid ""
 "  If parameters are given and the first one is not a branch, the log\n"
 "  will be filtered to show only those revisions that changed the\n"
 "  nominated files or directories."
 msgstr ""
 
-#: bzrlib/builtins.py:2312
+#: bzrlib/builtins.py:2400
 msgid ""
 "  Filenames are interpreted within their historical context. To log a\n"
 "  deleted file, specify a revision range so that the file existed at\n"
 "  the end or start of the range."
 msgstr ""
 
-#: bzrlib/builtins.py:2316
+#: bzrlib/builtins.py:2404
 msgid ""
 "  Historical context is also important when interpreting pathnames of\n"
 "  renamed files/directories. Consider the following example:"
 msgstr ""
 
-#: bzrlib/builtins.py:2319
+#: bzrlib/builtins.py:2407
 msgid ""
 "  * revision 1: add tutorial.txt\n"
 "  * revision 2: modify tutorial.txt\n"
 "  * revision 3: rename tutorial.txt to guide.txt; add tutorial.txt"
 msgstr ""
 
-#: bzrlib/builtins.py:2323
+#: bzrlib/builtins.py:2411
 msgid "  In this case:"
 msgstr ""
 
-#: bzrlib/builtins.py:2325
+#: bzrlib/builtins.py:2413
 msgid "  * ``bzr log guide.txt`` will log the file added in revision 1"
 msgstr ""
 
-#: bzrlib/builtins.py:2327
+#: bzrlib/builtins.py:2415
 msgid "  * ``bzr log tutorial.txt`` will log the new file added in revision 3"
 msgstr ""
 
-#: bzrlib/builtins.py:2329
+#: bzrlib/builtins.py:2417
 msgid ""
 "  * ``bzr log -r2 -p tutorial.txt`` will show the changes made to\n"
 "    the original file in revision 2."
 msgstr ""
 
-#: bzrlib/builtins.py:2332
+#: bzrlib/builtins.py:2420
 msgid ""
 "  * ``bzr log -r2 -p guide.txt`` will display an error message as there\n"
 "    was no file called guide.txt in revision 2."
 msgstr ""
 
-#: bzrlib/builtins.py:2335
+#: bzrlib/builtins.py:2423
 msgid ""
 "  Renames are always followed by log. By design, there is no need to\n"
 "  explicitly ask for this (and no way to stop logging a file back\n"
 "  until it was last renamed)."
 msgstr ""
 
-#: bzrlib/builtins.py:2339
+#: bzrlib/builtins.py:2427
 msgid ":Other filtering:"
 msgstr ""
 
-#: bzrlib/builtins.py:2341
+#: bzrlib/builtins.py:2429
 msgid ""
 "  The --match option can be used for finding revisions that match a\n"
 "  regular expression in a commit message, committer, author or bug.\n"
@@ -1232,11 +1602,11 @@
 "  --match-message can be used to only match a specific field."
 msgstr ""
 
-#: bzrlib/builtins.py:2347
+#: bzrlib/builtins.py:2435
 msgid ":Tips & tricks:"
 msgstr ""
 
-#: bzrlib/builtins.py:2349
+#: bzrlib/builtins.py:2437
 msgid ""
 "  GUI tools and IDEs are often better at exploring history than command\n"
 "  line tools: you may prefer qlog or viz from qbzr or bzr-gtk, the\n"
@@ -1245,11 +1615,11 @@
 "  <http://wiki.bazaar.canonical.com/IDEIntegration>.  "
 msgstr ""
 
-#: bzrlib/builtins.py:2355
+#: bzrlib/builtins.py:2443
 msgid "  You may find it useful to add the aliases below to ``bazaar.conf``::"
 msgstr ""
 
-#: bzrlib/builtins.py:2357
+#: bzrlib/builtins.py:2445
 msgid ""
 "    [ALIASES]\n"
 "    tip = log -r-1\n"
@@ -1257,20 +1627,20 @@
 "    show = log -v -p"
 msgstr ""
 
-#: bzrlib/builtins.py:2362
+#: bzrlib/builtins.py:2450
 msgid ""
 "  ``bzr tip`` will then show the latest revision while ``bzr top``\n"
 "  will show the last 10 mainline revisions. To see the details of a\n"
 "  particular revision X,  ``bzr show -rX``."
 msgstr ""
 
-#: bzrlib/builtins.py:2366
+#: bzrlib/builtins.py:2454
 msgid ""
 "  If you are interested in looking deeper into a particular merge X,\n"
 "  use ``bzr log -n0 -rX``."
 msgstr ""
 
-#: bzrlib/builtins.py:2369
+#: bzrlib/builtins.py:2457
 msgid ""
 "  ``bzr log -v`` on a branch with lots of history is currently\n"
 "  very slow. A fix for this issue is currently under development.\n"
@@ -1278,7 +1648,7 @@
 "  be given when using the -v option."
 msgstr ""
 
-#: bzrlib/builtins.py:2374
+#: bzrlib/builtins.py:2462
 msgid ""
 "  bzr has a generic full-text matching plugin, bzr-search, that can be\n"
 "  used to find revisions matching user names, commit messages, etc.\n"
@@ -1286,7 +1656,7 @@
 "  a list of words but not others."
 msgstr ""
 
-#: bzrlib/builtins.py:2379
+#: bzrlib/builtins.py:2467
 msgid ""
 "  When exploring non-mainline history on large projects with deep\n"
 "  history, the performance of log can be greatly improved by installing\n"
@@ -1295,132 +1665,168 @@
 msgstr ""
 
 # help of 'forward' option of 'log' command
-#: bzrlib/builtins.py:2388
+#: bzrlib/builtins.py:2476
 msgid "Show from oldest to newest."
 msgstr ""
 
 # help of 'verbose' option of 'log' command
-#: bzrlib/builtins.py:2391
+#: bzrlib/builtins.py:2479
 msgid "Show files changed in each revision."
 msgstr ""
 
 # help of 'change' option of 'log' command
-#: bzrlib/builtins.py:2397
+#: bzrlib/builtins.py:2485
 msgid "Show just the specified revision. See also \"help revisionspec\"."
 msgstr ""
 
 # help of 'authors' option of 'log' command
-#: bzrlib/builtins.py:2401
+#: bzrlib/builtins.py:2489
 msgid "What names to list as authors - first, all or committer."
 msgstr ""
 
 # title of 'authors' option of 'log' command
-#: bzrlib/builtins.py:2402
+#: bzrlib/builtins.py:2490
 msgid "Authors"
 msgstr ""
 
 # help of 'levels' option of 'log' command
-#: bzrlib/builtins.py:2407
+#: bzrlib/builtins.py:2495
 msgid "Number of levels to display - 0 for all, 1 for flat."
 msgstr ""
 
 # help of 'limit' option of 'log' command
-#: bzrlib/builtins.py:2417
+#: bzrlib/builtins.py:2505
 msgid "Limit the output to the first N revisions."
 msgstr ""
 
 # help of 'show-diff' option of 'log' command
-#: bzrlib/builtins.py:2422
+#: bzrlib/builtins.py:2510
 msgid "Show changes made in each revision as a patch."
 msgstr ""
 
-# help of 'include-merges' option of 'log' command
-#: bzrlib/builtins.py:2424
+# help of 'include-merged' option of 'log' command
+#: bzrlib/builtins.py:2512
 msgid "Show merged revisions like --levels 0 does."
 msgstr ""
 
+# help of 'omit-merges' option of 'log' command
+#: bzrlib/builtins.py:2516
+msgid "Do not report commits with more than one parent."
+msgstr ""
+
 # help of 'exclude-common-ancestry' option of 'log' command
-#: bzrlib/builtins.py:2426
+#: bzrlib/builtins.py:2518
 msgid ""
 "Display only the revisions that are not part of both ancestries (require -"
-"rX..Y)"
+"rX..Y)."
 msgstr ""
 
 # help of 'signatures' option of 'log' command
-#: bzrlib/builtins.py:2430
-msgid "Show digital signature validity"
+#: bzrlib/builtins.py:2522
+msgid "Show digital signature validity."
 msgstr ""
 
 # help of 'match' option of 'log' command
-#: bzrlib/builtins.py:2433
+#: bzrlib/builtins.py:2525
 msgid "Show revisions whose properties match this expression."
 msgstr ""
 
 # help of 'match-message' option of 'log' command
-#: bzrlib/builtins.py:2437
+#: bzrlib/builtins.py:2529
 msgid "Show revisions whose message matches this expression."
 msgstr ""
 
 # help of 'match-committer' option of 'log' command
-#: bzrlib/builtins.py:2441
+#: bzrlib/builtins.py:2533
 msgid "Show revisions whose committer matches this expression."
 msgstr ""
 
 # help of 'match-author' option of 'log' command
-#: bzrlib/builtins.py:2445
+#: bzrlib/builtins.py:2537
 msgid "Show revisions whose authors match this expression."
 msgstr ""
 
 # help of 'match-bugs' option of 'log' command
-#: bzrlib/builtins.py:2449
+#: bzrlib/builtins.py:2541
 msgid "Show revisions whose bugs match this expression."
 msgstr ""
 
-#: bzrlib/builtins.py:2690
+#: bzrlib/builtins.py:2595
+msgid "--exclude-common-ancestry requires -r with two revisions"
+msgstr ""
+
+#: bzrlib/builtins.py:2623
+#, python-format
+msgid "Path unknown at end or start of revision range: %s"
+msgstr ""
+
+#: bzrlib/builtins.py:2740
+#, python-format
+msgid "bzr %s doesn't accept two revisions in different branches."
+msgstr ""
+
+#: bzrlib/builtins.py:2756
+#, python-format
+msgid "bzr %s --revision takes one or two values."
+msgstr ""
+
+#: bzrlib/builtins.py:2801
 msgid ""
 "List files in a tree.\n"
 "    "
 msgstr ""
 
 # help of 'recursive' option of 'ls' command
-#: bzrlib/builtins.py:2699
+#: bzrlib/builtins.py:2810
 msgid "Recurse into subdirectories."
 msgstr ""
 
 # help of 'from-root' option of 'ls' command
-#: bzrlib/builtins.py:2701
+#: bzrlib/builtins.py:2812
 msgid "Print paths relative to the root of the branch."
 msgstr ""
 
 # help of 'unknown' option of 'ls' command
-#: bzrlib/builtins.py:2703
+#: bzrlib/builtins.py:2814
 msgid "Print unknown files."
 msgstr ""
 
 # help of 'versioned' option of 'ls' command
-#: bzrlib/builtins.py:2704
+#: bzrlib/builtins.py:2815
 msgid "Print versioned files."
 msgstr ""
 
 # help of 'ignored' option of 'ls' command
-#: bzrlib/builtins.py:2707
+#: bzrlib/builtins.py:2818
 msgid "Print ignored files."
 msgstr ""
 
 # help of 'kind' option of 'ls' command
-#: bzrlib/builtins.py:2709
+#: bzrlib/builtins.py:2820
 msgid "List entries of a particular kind: file, directory, symlink."
 msgstr ""
 
-#: bzrlib/builtins.py:2821
+#: bzrlib/builtins.py:2833
+msgid "invalid kind specified"
+msgstr ""
+
+#: bzrlib/builtins.py:2836
+msgid "Cannot set both --verbose and --null"
+msgstr ""
+
+#: bzrlib/builtins.py:2845
+msgid "cannot specify both --from-root and PATH"
+msgstr ""
+
+#: bzrlib/builtins.py:2932
 msgid "Ignore specified files or patterns."
 msgstr ""
 
-#: bzrlib/builtins.py:2823
+#: bzrlib/builtins.py:2934
 msgid "See ``bzr help patterns`` for details on the syntax of patterns."
 msgstr ""
 
-#: bzrlib/builtins.py:2825
+#: bzrlib/builtins.py:2936
 msgid ""
 "If a .bzrignore file does not exist, the ignore command\n"
 "will create one and add the specified files or patterns to the newly\n"
@@ -1429,7 +1835,7 @@
 "the use of the ignore command will require an explicit add command."
 msgstr ""
 
-#: bzrlib/builtins.py:2831
+#: bzrlib/builtins.py:2942
 msgid ""
 "To remove patterns from the ignore list, edit the .bzrignore file.\n"
 "After adding, editing or deleting that file either indirectly by\n"
@@ -1437,7 +1843,7 @@
 "it."
 msgstr ""
 
-#: bzrlib/builtins.py:2836
+#: bzrlib/builtins.py:2947
 msgid ""
 "Bazaar also supports a global ignore file ~/.bazaar/ignore. On Windows\n"
 "the global ignore file can be found in the application data directory as\n"
@@ -1446,20 +1852,20 @@
 "can be edited directly using an editor."
 msgstr ""
 
-#: bzrlib/builtins.py:2842
+#: bzrlib/builtins.py:2953
 msgid ""
 "Patterns prefixed with '!' are exceptions to ignore patterns and take\n"
 "precedence over regular ignores.  Such exceptions are used to specify\n"
 "files that should be versioned which would otherwise be ignored."
 msgstr ""
 
-#: bzrlib/builtins.py:2846
+#: bzrlib/builtins.py:2957
 msgid ""
 "Patterns prefixed with '!!' act as regular ignore patterns, but have\n"
 "precedence over the '!' exception patterns."
 msgstr ""
 
-#: bzrlib/builtins.py:2849
+#: bzrlib/builtins.py:2960
 msgid ""
 ":Notes: \n"
 "    \n"
@@ -1467,63 +1873,63 @@
 "  the shell on Unix."
 msgstr ""
 
-#: bzrlib/builtins.py:2854
+#: bzrlib/builtins.py:2965
 msgid ""
 "* Ignore patterns starting with \"#\" act as comments in the ignore file.\n"
 "  To ignore patterns that begin with that character, use the \"RE:\" prefix."
 msgstr ""
 
-#: bzrlib/builtins.py:2857
+#: bzrlib/builtins.py:2968
 msgid ""
 ":Examples:\n"
 "    Ignore the top level Makefile::"
 msgstr ""
 
-#: bzrlib/builtins.py:2860
+#: bzrlib/builtins.py:2971
 msgid "        bzr ignore ./Makefile"
 msgstr ""
 
-#: bzrlib/builtins.py:2862
+#: bzrlib/builtins.py:2973
 msgid "    Ignore .class files in all directories...::"
 msgstr ""
 
-#: bzrlib/builtins.py:2864
+#: bzrlib/builtins.py:2975
 msgid "        bzr ignore \"*.class\""
 msgstr ""
 
-#: bzrlib/builtins.py:2866
+#: bzrlib/builtins.py:2977
 msgid "    ...but do not ignore \"special.class\"::"
 msgstr ""
 
-#: bzrlib/builtins.py:2868
+#: bzrlib/builtins.py:2979
 msgid "        bzr ignore \"!special.class\""
 msgstr ""
 
-#: bzrlib/builtins.py:2870
+#: bzrlib/builtins.py:2981
 msgid "    Ignore files whose name begins with the \"#\" character::"
 msgstr ""
 
-#: bzrlib/builtins.py:2872
+#: bzrlib/builtins.py:2983
 msgid "        bzr ignore \"RE:^#\""
 msgstr ""
 
-#: bzrlib/builtins.py:2874
+#: bzrlib/builtins.py:2985
 msgid "    Ignore .o files under the lib directory::"
 msgstr ""
 
-#: bzrlib/builtins.py:2876
+#: bzrlib/builtins.py:2987
 msgid "        bzr ignore \"lib/**/*.o\""
 msgstr ""
 
-#: bzrlib/builtins.py:2880
+#: bzrlib/builtins.py:2991
 msgid "        bzr ignore \"RE:lib/.*\\.o\""
 msgstr ""
 
-#: bzrlib/builtins.py:2882
+#: bzrlib/builtins.py:2993
 msgid "    Ignore everything but the \"debian\" toplevel directory::"
 msgstr ""
 
-#: bzrlib/builtins.py:2884
+#: bzrlib/builtins.py:2995
 msgid ""
 "        bzr ignore \"RE:(?!debian/).*\"\n"
 "    \n"
@@ -1536,61 +1942,91 @@
 msgstr ""
 
 # help of 'default-rules' option of 'ignore' command
-#: bzrlib/builtins.py:2898
+#: bzrlib/builtins.py:3009
 msgid "Display the default ignore rules that bzr uses."
 msgstr ""
 
-#: bzrlib/builtins.py:2946
+#: bzrlib/builtins.py:3021
+msgid "ignore requires at least one NAME_PATTERN or --default-rules."
+msgstr ""
+
+#: bzrlib/builtins.py:3032
+#, python-format
+msgid "Invalid ignore pattern found. %s"
+msgid_plural "Invalid ignore patterns found. %s"
+msgstr[0] ""
+msgstr[1] ""
+
+#: bzrlib/builtins.py:3041
+msgid "NAME_PATTERN should not be an absolute path"
+msgstr ""
+
+#: bzrlib/builtins.py:3054
+#, python-format
+msgid ""
+"Warning: the following files are version controlled and match your ignore "
+"pattern:\n"
+"%s\n"
+"These files will continue to be version controlled unless you 'bzr remove' "
+"them.\n"
+msgstr ""
+
+#: bzrlib/builtins.py:3061
 msgid "List ignored files and the patterns that matched them."
 msgstr ""
 
-#: bzrlib/builtins.py:2948
+#: bzrlib/builtins.py:3063
 msgid ""
 "List all the ignored files and the ignore pattern that caused the file to\n"
 "be ignored."
 msgstr ""
 
-#: bzrlib/builtins.py:2951
+#: bzrlib/builtins.py:3066
 msgid "Alternatively, to list just the files::"
 msgstr ""
 
-#: bzrlib/builtins.py:2953
+#: bzrlib/builtins.py:3068
 msgid "    bzr ls --ignored"
 msgstr ""
 
-#: bzrlib/builtins.py:2994
+#: bzrlib/builtins.py:3102
+#, python-format
+msgid "not a valid revision-number: %r"
+msgstr ""
+
+#: bzrlib/builtins.py:3109
 msgid "Export current or past revision to a destination directory or archive."
 msgstr ""
 
-#: bzrlib/builtins.py:2996
+#: bzrlib/builtins.py:3111
 msgid "If no revision is specified this exports the last committed revision."
 msgstr ""
 
-#: bzrlib/builtins.py:2998
+#: bzrlib/builtins.py:3113
 msgid ""
 "Format may be an \"exporter\" name, such as tar, tgz, tbz2.  If none is\n"
 "given, try to find the format with the extension. If no extension\n"
 "is found exports to a directory (equivalent to --format=dir)."
 msgstr ""
 
-#: bzrlib/builtins.py:3002
+#: bzrlib/builtins.py:3117
 msgid ""
 "If root is supplied, it will be used as the root directory inside\n"
 "container formats (tar, zip, etc). If it is not supplied it will default\n"
 "to the exported filename. The root option has no effect for 'dir' format."
 msgstr ""
 
-#: bzrlib/builtins.py:3006
+#: bzrlib/builtins.py:3121
 msgid ""
 "If branch is omitted then the branch containing the current working\n"
 "directory will be used."
 msgstr ""
 
-#: bzrlib/builtins.py:3009
+#: bzrlib/builtins.py:3124
 msgid "Note: Export of tree with non-ASCII filenames to zip is not supported."
 msgstr ""
 
-#: bzrlib/builtins.py:3011
+#: bzrlib/builtins.py:3126
 msgid ""
 "  =================       =========================\n"
 "  Supported formats       Autodetected by extension\n"
@@ -1604,51 +2040,78 @@
 msgstr ""
 
 # help of 'format' option of 'export' command
-#: bzrlib/builtins.py:3025
+#: bzrlib/builtins.py:3140
 msgid "Type of file to export to."
 msgstr ""
 
 # help of 'filters' option of 'export' command
-#: bzrlib/builtins.py:3028
+#: bzrlib/builtins.py:3143
 msgid "Apply content filters to export the convenient form."
 msgstr ""
 
 # help of 'root' option of 'export' command
-#: bzrlib/builtins.py:3032
+#: bzrlib/builtins.py:3147
 msgid "Name of the root directory inside the exported file."
 msgstr ""
 
 # help of 'per-file-timestamps' option of 'export' command
-#: bzrlib/builtins.py:3034
+#: bzrlib/builtins.py:3149
 msgid ""
 "Set modification time of files to that of the last revision in which it was "
 "changed."
 msgstr ""
 
-#: bzrlib/builtins.py:3058
+# help of 'uncommitted' option of 'export' command
+#: bzrlib/builtins.py:3152
+msgid "Export the working tree contents rather than that of the last revision."
+msgstr ""
+
+#: bzrlib/builtins.py:3171
+msgid "--uncommitted requires a working tree"
+msgstr ""
+
+#: bzrlib/builtins.py:3180
+#, python-format
+msgid "Unsupported export format: %s"
+msgstr ""
+
+#: bzrlib/builtins.py:3184
 msgid "Write the contents of a file as of a given revision to standard output."
 msgstr ""
 
-#: bzrlib/builtins.py:3060
+#: bzrlib/builtins.py:3186
 msgid "If no revision is nominated, the last revision is used."
 msgstr ""
 
-#: bzrlib/builtins.py:3062
+#: bzrlib/builtins.py:3188
 msgid ""
 "Note: Take care to redirect standard output when using this command on a\n"
 "binary file."
 msgstr ""
 
+# help of 'name-from-revision' option of 'cat' command
+#: bzrlib/builtins.py:3194
+msgid "The path name in the old tree."
+msgstr ""
+
 # help of 'filters' option of 'cat' command
-#: bzrlib/builtins.py:3069
+#: bzrlib/builtins.py:3195
 msgid "Apply content filters to display the convenience form."
 msgstr ""
 
-#: bzrlib/builtins.py:3140
+#: bzrlib/builtins.py:3206
+msgid "bzr cat --revision takes exactly one revision specifier"
+msgstr ""
+
+#: bzrlib/builtins.py:3231 bzrlib/builtins.py:3243
+msgid "{0!r} is not present in revision {1}"
+msgstr ""
+
+#: bzrlib/builtins.py:3266
 msgid "Commit changes into a new revision."
 msgstr ""
 
-#: bzrlib/builtins.py:3142
+#: bzrlib/builtins.py:3268
 msgid ""
 "An explanatory message needs to be given for each commit. This is\n"
 "often done by using the --message option (getting the message from the\n"
@@ -1658,44 +2121,44 @@
 "boilerplate text loaded into the editor, use the --show-diff option."
 msgstr ""
 
-#: bzrlib/builtins.py:3149
+#: bzrlib/builtins.py:3275
 msgid ""
 "By default, the entire tree is committed and the person doing the\n"
 "commit is assumed to be the author. These defaults can be overridden\n"
 "as explained below."
 msgstr ""
 
-#: bzrlib/builtins.py:3153
+#: bzrlib/builtins.py:3279
 msgid ":Selective commits:"
 msgstr ""
 
-#: bzrlib/builtins.py:3155
+#: bzrlib/builtins.py:3281
 msgid ""
 "  If selected files are specified, only changes to those files are\n"
 "  committed.  If a directory is specified then the directory and\n"
 "  everything within it is committed."
 msgstr ""
 
-#: bzrlib/builtins.py:3159
+#: bzrlib/builtins.py:3285
 msgid ""
 "  When excludes are given, they take precedence over selected files.\n"
 "  For example, to commit only changes within foo, but not changes\n"
 "  within foo/bar::"
 msgstr ""
 
-#: bzrlib/builtins.py:3163
+#: bzrlib/builtins.py:3289
 msgid "    bzr commit foo -x foo/bar"
 msgstr ""
 
-#: bzrlib/builtins.py:3165
+#: bzrlib/builtins.py:3291
 msgid "  A selective commit after a merge is not yet supported."
 msgstr ""
 
-#: bzrlib/builtins.py:3167
+#: bzrlib/builtins.py:3293
 msgid ":Custom authors:"
 msgstr ""
 
-#: bzrlib/builtins.py:3169
+#: bzrlib/builtins.py:3295
 msgid ""
 "  If the author of the change is not the same person as the committer,\n"
 "  you can specify the author's name using the --author option. The\n"
@@ -1705,11 +2168,11 @@
 "  author."
 msgstr ""
 
-#: bzrlib/builtins.py:3176
+#: bzrlib/builtins.py:3302
 msgid ":Checks:"
 msgstr ""
 
-#: bzrlib/builtins.py:3178
+#: bzrlib/builtins.py:3304
 msgid ""
 "  A common mistake is to forget to add a new file or directory before\n"
 "  running the commit command. The --strict option checks for unknown\n"
@@ -1718,18 +2181,18 @@
 "  for details."
 msgstr ""
 
-#: bzrlib/builtins.py:3184
+#: bzrlib/builtins.py:3310
 msgid ":Things to note:"
 msgstr ""
 
-#: bzrlib/builtins.py:3186
+#: bzrlib/builtins.py:3312
 msgid ""
 "  If you accidentially commit the wrong changes or make a spelling\n"
 "  mistake in the commit message say, you can use the uncommit command\n"
 "  to undo it. See ``bzr help uncommit`` for details."
 msgstr ""
 
-#: bzrlib/builtins.py:3190
+#: bzrlib/builtins.py:3316
 msgid ""
 "  Hooks can also be configured to run after a commit. This allows you\n"
 "  to trigger updates to external systems like bug trackers. The --fixes\n"
@@ -1738,100 +2201,169 @@
 msgstr ""
 
 # help of 'exclude' option of 'commit' command
-#: bzrlib/builtins.py:3200
+#: bzrlib/builtins.py:3326
 msgid "Do not consider changes made to a given path."
 msgstr ""
 
 # help of 'message' option of 'commit' command
-#: bzrlib/builtins.py:3203
+#: bzrlib/builtins.py:3329
 msgid "Description of the new revision."
 msgstr ""
 
 # help of 'unchanged' option of 'commit' command
-#: bzrlib/builtins.py:3206
+#: bzrlib/builtins.py:3332
 msgid "Commit even if nothing has changed."
 msgstr ""
 
 # help of 'file' option of 'commit' command
-#: bzrlib/builtins.py:3210
+#: bzrlib/builtins.py:3336
 msgid "Take commit message from this file."
 msgstr ""
 
 # help of 'strict' option of 'commit' command
-#: bzrlib/builtins.py:3212
+#: bzrlib/builtins.py:3338
 msgid "Refuse to commit if there are unknown files in the working tree."
 msgstr ""
 
 # help of 'commit-time' option of 'commit' command
-#: bzrlib/builtins.py:3215
+#: bzrlib/builtins.py:3341
 msgid ""
 "Manually set a commit time using commit date format, e.g. '2009-10-10 "
 "08:00:00 +0100'."
 msgstr ""
 
 # help of 'fixes' option of 'commit' command
-#: bzrlib/builtins.py:3218
+#: bzrlib/builtins.py:3344
 msgid "Mark a bug as being fixed by this revision (see \"bzr help bugs\")."
 msgstr ""
 
 # help of 'author' option of 'commit' command
-#: bzrlib/builtins.py:3221
+#: bzrlib/builtins.py:3347
 msgid "Set the author's name, if it's different from the committer."
 msgstr ""
 
 # help of 'local' option of 'commit' command
-#: bzrlib/builtins.py:3224
+#: bzrlib/builtins.py:3350
 msgid ""
 "Perform a local commit in a bound branch.  Local commits are not pushed to "
 "the master branch until a normal commit is performed."
 msgstr ""
 
 # help of 'show-diff' option of 'commit' command
-#: bzrlib/builtins.py:3230
+#: bzrlib/builtins.py:3356
 msgid ""
 "When no message is supplied, show the diff along with the status summary in "
 "the message editor."
 msgstr ""
 
 # help of 'lossy' option of 'commit' command
-#: bzrlib/builtins.py:3233
+#: bzrlib/builtins.py:3359
 msgid ""
 "When committing to a foreign version control system do not push data that "
 "can not be natively represented."
 msgstr ""
 
-#: bzrlib/builtins.py:3391
+#: bzrlib/builtins.py:3377
+#, python-format
+msgid ""
+"No tracker specified for bug %s. Use the form 'tracker:id' or specify a "
+"default bug tracker using the `bugtracker` option.\n"
+"See \"bzr help bugs\" for more information on this feature. Commit refused."
+msgstr ""
+
+#: bzrlib/builtins.py:3386
+#, python-format
+msgid ""
+"Invalid bug %s. Must be in the form of 'tracker:id'. See \"bzr help bugs\" "
+"for more information on this feature.\n"
+"Commit refused."
+msgstr ""
+
+#: bzrlib/builtins.py:3395
+#, python-format
+msgid "Unrecognized bug %s. Commit refused."
+msgstr ""
+
+#: bzrlib/builtins.py:3398
+#, python-format
+msgid ""
+"%s\n"
+"Commit refused."
+msgstr ""
+
+#: bzrlib/builtins.py:3422
+msgid "Could not parse --commit-time: "
+msgstr ""
+
+#: bzrlib/builtins.py:3462
+msgid "please specify either --message or --file"
+msgstr ""
+
+#: bzrlib/builtins.py:3491
+msgid "please specify a commit message with either --message or --file"
+msgstr ""
+
+#: bzrlib/builtins.py:3494
+msgid ""
+"Empty commit message specified. Please specify a commit message with either "
+"--message or --file or leave a blank message with --message \"\"."
+msgstr ""
+
+#: bzrlib/builtins.py:3514
+msgid ""
+"No changes to commit. Please 'bzr add' the files you want to commit, or use "
+"--unchanged to force an empty commit."
+msgstr ""
+
+#: bzrlib/builtins.py:3518
+msgid ""
+"Conflicts detected in working tree.  Use \"bzr conflicts\" to list, \"bzr "
+"resolve FILE\" to resolve."
+msgstr ""
+
+#: bzrlib/builtins.py:3522
+msgid "Commit refused because there are unknown files in the working tree."
+msgstr ""
+
+#: bzrlib/builtins.py:3525
+msgid ""
+"\n"
+"To commit to master branch, run update and then commit.\n"
+"You can also pass --local to commit to continue working disconnected."
+msgstr ""
+
+#: bzrlib/builtins.py:3533
 msgid ""
 "Validate working tree structure, branch consistency and repository history."
 msgstr ""
 
-#: bzrlib/builtins.py:3393
+#: bzrlib/builtins.py:3535
 msgid ""
 "This command checks various invariants about branch and repository storage\n"
 "to detect data corruption or bzr bugs."
 msgstr ""
 
-#: bzrlib/builtins.py:3396
+#: bzrlib/builtins.py:3538
 msgid ""
 "The working tree and branch checks will only give output if a problem is\n"
 "detected. The output fields of the repository check are:"
 msgstr ""
 
-#: bzrlib/builtins.py:3399
+#: bzrlib/builtins.py:3541
 msgid ""
 "revisions\n"
 "    This is just the number of revisions checked.  It doesn't\n"
 "    indicate a problem."
 msgstr ""
 
-#: bzrlib/builtins.py:3403
+#: bzrlib/builtins.py:3545
 msgid ""
 "versionedfiles\n"
 "    This is just the number of versionedfiles checked.  It\n"
 "    doesn't indicate a problem."
 msgstr ""
 
-#: bzrlib/builtins.py:3407
+#: bzrlib/builtins.py:3549
 msgid ""
 "unreferenced ancestors\n"
 "    Texts that are ancestors of other texts, but\n"
@@ -1839,14 +2371,14 @@
 "    subtle problem that Bazaar can work around."
 msgstr ""
 
-#: bzrlib/builtins.py:3412
+#: bzrlib/builtins.py:3554
 msgid ""
 "unique file texts\n"
 "    This is the total number of unique file contents\n"
 "    seen in the checked revisions.  It does not indicate a problem."
 msgstr ""
 
-#: bzrlib/builtins.py:3416
+#: bzrlib/builtins.py:3558
 msgid ""
 "repeated file texts\n"
 "    This is the total number of repeated texts seen\n"
@@ -1855,61 +2387,61 @@
 "    indicate a problem."
 msgstr ""
 
-#: bzrlib/builtins.py:3422
+#: bzrlib/builtins.py:3564
 msgid ""
 "If no restrictions are specified, all Bazaar data that is found at the "
 "given\n"
 "location will be checked."
 msgstr ""
 
-#: bzrlib/builtins.py:3425
+#: bzrlib/builtins.py:3567
 msgid ":Examples:"
 msgstr ""
 
-#: bzrlib/builtins.py:3427
+#: bzrlib/builtins.py:3569
 msgid "    Check the tree and branch at 'foo'::"
 msgstr ""
 
-#: bzrlib/builtins.py:3429
+#: bzrlib/builtins.py:3571
 msgid "        bzr check --tree --branch foo"
 msgstr ""
 
-#: bzrlib/builtins.py:3431
+#: bzrlib/builtins.py:3573
 msgid "    Check only the repository at 'bar'::"
 msgstr ""
 
-#: bzrlib/builtins.py:3433
+#: bzrlib/builtins.py:3575
 msgid "        bzr check --repo bar"
 msgstr ""
 
-#: bzrlib/builtins.py:3435
+#: bzrlib/builtins.py:3577
 msgid "    Check everything at 'baz'::"
 msgstr ""
 
-#: bzrlib/builtins.py:3437
+#: bzrlib/builtins.py:3579
 msgid "        bzr check baz"
 msgstr ""
 
 # help of 'branch' option of 'check' command
-#: bzrlib/builtins.py:3443
+#: bzrlib/builtins.py:3585
 msgid "Check the branch related to the current directory."
 msgstr ""
 
 # help of 'repo' option of 'check' command
-#: bzrlib/builtins.py:3445
+#: bzrlib/builtins.py:3587
 msgid "Check the repository related to the current directory."
 msgstr ""
 
 # help of 'tree' option of 'check' command
-#: bzrlib/builtins.py:3447
+#: bzrlib/builtins.py:3589
 msgid "Check the working tree related to the current directory."
 msgstr ""
 
-#: bzrlib/builtins.py:3461
+#: bzrlib/builtins.py:3603
 msgid "Upgrade a repository, branch or working tree to a newer format."
 msgstr ""
 
-#: bzrlib/builtins.py:3463
+#: bzrlib/builtins.py:3605
 msgid ""
 "When the default format has changed after a major new release of\n"
 "Bazaar, you may be informed during certain operations that you\n"
@@ -1918,7 +2450,7 @@
 "with older repositories or with older versions of Bazaar."
 msgstr ""
 
-#: bzrlib/builtins.py:3469
+#: bzrlib/builtins.py:3611
 msgid ""
 "If you wish to upgrade to a particular format rather than the\n"
 "current default, that can be specified using the --format option.\n"
@@ -1928,7 +2460,7 @@
 "2.x default) so downgrading is not always possible."
 msgstr ""
 
-#: bzrlib/builtins.py:3476
+#: bzrlib/builtins.py:3618
 msgid ""
 "A backup.bzr.~#~ directory is created at the start of the conversion\n"
 "process (where # is a number). By default, this is left there on\n"
@@ -1939,7 +2471,7 @@
 "afterwards."
 msgstr ""
 
-#: bzrlib/builtins.py:3484
+#: bzrlib/builtins.py:3626
 msgid ""
 "If the location given is a shared repository, dependent branches\n"
 "are also converted provided the repository converts successfully.\n"
@@ -1947,132 +2479,163 @@
 "tried."
 msgstr ""
 
-#: bzrlib/builtins.py:3489
+#: bzrlib/builtins.py:3631
 msgid ""
 "For more information on upgrades, see the Bazaar Upgrade Guide,\n"
 "http://doc.bazaar.canonical.com/latest/en/upgrade-guide/."
 msgstr ""
 
 # help of 'format' option of 'upgrade' command
-#: bzrlib/builtins.py:3497
+#: bzrlib/builtins.py:3639
 msgid "Upgrade to a specific format.  See \"bzr help formats\" for details."
 msgstr ""
 
 # title of 'format' option of 'init' command
-#: bzrlib/builtins.py:3501
+#: bzrlib/builtins.py:3643
 msgid "Branch format"
 msgstr ""
 
 # help of 'clean' option of 'upgrade' command
-#: bzrlib/builtins.py:3503
+#: bzrlib/builtins.py:3645
 msgid "Remove the backup.bzr directory if successful."
 msgstr ""
 
-#: bzrlib/builtins.py:3520
+# help of 'dry-run' option of 'add' command
+#: bzrlib/builtins.py:3647
+msgid "Show what would be done, but don't actually do anything."
+msgstr ""
+
+#: bzrlib/builtins.py:3662
 msgid "Show or set bzr user id."
 msgstr ""
 
-#: bzrlib/builtins.py:3522
+#: bzrlib/builtins.py:3664
 msgid ""
 ":Examples:\n"
 "    Show the email of the current user::"
 msgstr ""
 
-#: bzrlib/builtins.py:3525
+#: bzrlib/builtins.py:3667
 msgid "        bzr whoami --email"
 msgstr ""
 
-#: bzrlib/builtins.py:3527
+#: bzrlib/builtins.py:3669
 msgid "    Set the current user::"
 msgstr ""
 
-#: bzrlib/builtins.py:3529
+#: bzrlib/builtins.py:3671
 msgid "        bzr whoami \"Frank Chu <fchu@example.com>\""
 msgstr ""
 
 # help of 'email' option of 'whoami' command
-#: bzrlib/builtins.py:3533
+#: bzrlib/builtins.py:3675
 msgid "Display email address only."
 msgstr ""
 
 # help of 'branch' option of 'whoami' command
-#: bzrlib/builtins.py:3535
+#: bzrlib/builtins.py:3677
 msgid "Set identity for the current branch instead of globally."
 msgstr ""
 
-#: bzrlib/builtins.py:3581
+#: bzrlib/builtins.py:3701
+msgid "--email can only be used to display existing identity"
+msgstr ""
+
+#: bzrlib/builtins.py:3723
 msgid "Print or set the branch nickname."
 msgstr ""
 
-#: bzrlib/builtins.py:3583
+#: bzrlib/builtins.py:3725
 msgid ""
 "If unset, the tree root directory name is used as the nickname.\n"
 "To print the current nickname, execute with no argument."
 msgstr ""
 
-#: bzrlib/builtins.py:3586
+#: bzrlib/builtins.py:3728
 msgid ""
 "Bound branches use the nickname of its master branch unless it is set\n"
 "locally."
 msgstr ""
 
-#: bzrlib/builtins.py:3606
+#: bzrlib/builtins.py:3748
 msgid "Set/unset and display aliases."
 msgstr ""
 
-#: bzrlib/builtins.py:3608
+#: bzrlib/builtins.py:3750
 msgid ""
 ":Examples:\n"
 "    Show the current aliases::"
 msgstr ""
 
-#: bzrlib/builtins.py:3611
+#: bzrlib/builtins.py:3753
 msgid "        bzr alias"
 msgstr ""
 
-#: bzrlib/builtins.py:3613
+#: bzrlib/builtins.py:3755
 msgid "    Show the alias specified for 'll'::"
 msgstr ""
 
-#: bzrlib/builtins.py:3615
+#: bzrlib/builtins.py:3757
 msgid "        bzr alias ll"
 msgstr ""
 
-#: bzrlib/builtins.py:3617
+#: bzrlib/builtins.py:3759
 msgid "    Set an alias for 'll'::"
 msgstr ""
 
-#: bzrlib/builtins.py:3619
+#: bzrlib/builtins.py:3761
 msgid "        bzr alias ll=\"log --line -r-10..-1\""
 msgstr ""
 
-#: bzrlib/builtins.py:3621
+#: bzrlib/builtins.py:3763
 msgid "    To remove an alias for 'll'::"
 msgstr ""
 
-#: bzrlib/builtins.py:3623
+#: bzrlib/builtins.py:3765
 msgid "        bzr alias --remove ll"
 msgstr ""
 
 # help of 'remove' option of 'alias' command
-#: bzrlib/builtins.py:3628
+#: bzrlib/builtins.py:3770
 msgid "Remove the alias."
 msgstr ""
 
-#: bzrlib/builtins.py:3867
+#: bzrlib/builtins.py:3788
+msgid "bzr alias --remove expects an alias to remove."
+msgstr ""
+
+#: bzrlib/builtins.py:3960
+msgid "subunit not available. subunit needs to be installed to use --subunit."
+msgstr ""
+
+#: bzrlib/builtins.py:3977
+msgid ""
+"--benchmark is no longer supported from bzr 2.2; use bzr-usertest instead"
+msgstr ""
+
+#: bzrlib/builtins.py:4024
 msgid "Show version of bzr."
 msgstr ""
 
 # help of 'short' option of 'version' command
-#: bzrlib/builtins.py:3871
+#: bzrlib/builtins.py:4028
 msgid "Print just the version number."
 msgstr ""
 
-#: bzrlib/builtins.py:3918
+#: bzrlib/builtins.py:4047
+msgid "It sure does!\n"
+msgstr ""
+
+#: bzrlib/builtins.py:4071
+#, python-format
+msgid "merge base is revision %s\n"
+msgstr ""
+
+#: bzrlib/builtins.py:4075
 msgid "Perform a three-way merge."
 msgstr ""
 
-#: bzrlib/builtins.py:3920
+#: bzrlib/builtins.py:4077
 msgid ""
 "The source of the merge can be specified either in the form of a branch,\n"
 "or in the form of a path to a file containing a merge directive generated\n"
@@ -2083,20 +2646,20 @@
 "the current working tree."
 msgstr ""
 
-#: bzrlib/builtins.py:3928
+#: bzrlib/builtins.py:4085
 msgid ""
 "When merging from a branch, by default bzr will try to merge in all new\n"
 "work from the other branch, automatically determining an appropriate base\n"
 "revision.  If this fails, you may need to give an explicit base."
 msgstr ""
 
-#: bzrlib/builtins.py:3932
+#: bzrlib/builtins.py:4089
 msgid ""
 "To pick a different ending revision, pass \"--revision OTHER\".  bzr will\n"
 "try to merge in all new work up to and including revision OTHER."
 msgstr ""
 
-#: bzrlib/builtins.py:3935
+#: bzrlib/builtins.py:4092
 msgid ""
 "If you specify two values, \"--revision BASE..OTHER\", only revisions BASE\n"
 "through OTHER, excluding BASE but including OTHER, will be merged.  If this\n"
@@ -2107,30 +2670,30 @@
 "changes from the other branch is not stored in the commit."
 msgstr ""
 
-#: bzrlib/builtins.py:3943
+#: bzrlib/builtins.py:4100
 msgid "Revision numbers are always relative to the source branch."
 msgstr ""
 
-#: bzrlib/builtins.py:3950
+#: bzrlib/builtins.py:4107
 msgid "Use bzr resolve when you have fixed a problem.  See also bzr conflicts."
 msgstr ""
 
-#: bzrlib/builtins.py:3952
+#: bzrlib/builtins.py:4109
 msgid ""
 "If there is no default branch set, the first merge will set it (use\n"
-"--no-remember to avoid settting it). After that, you can omit the branch\n"
+"--no-remember to avoid setting it). After that, you can omit the branch\n"
 "to use the default.  To change the default, use --remember. The value will\n"
 "only be saved if the remote location can be accessed."
 msgstr ""
 
-#: bzrlib/builtins.py:3957
+#: bzrlib/builtins.py:4114
 msgid ""
 "The results of the merge are placed into the destination working\n"
 "directory, where they can be reviewed (with bzr diff), tested, and then\n"
 "committed to record the result of the merge."
 msgstr ""
 
-#: bzrlib/builtins.py:3961
+#: bzrlib/builtins.py:4118
 msgid ""
 "merge refuses to run if there are any uncommitted changes, unless\n"
 "--force is given.  If --force is given, then the changes from the source \n"
@@ -2139,60 +2702,60 @@
 "merge revision which has more than two parents."
 msgstr ""
 
-#: bzrlib/builtins.py:3967
+#: bzrlib/builtins.py:4124
 msgid ""
 "If one would like to merge changes from the working tree of the other\n"
 "branch without merging any committed revisions, the --uncommitted option\n"
 "can be given."
 msgstr ""
 
-#: bzrlib/builtins.py:3971
+#: bzrlib/builtins.py:4128
 msgid ""
 "To select only some changes to merge, use \"merge -i\", which will prompt\n"
 "you to apply each diff hunk and file change, similar to \"shelve\"."
 msgstr ""
 
-#: bzrlib/builtins.py:3974
+#: bzrlib/builtins.py:4131
 msgid ""
 ":Examples:\n"
 "    To merge all new revisions from bzr.dev::"
 msgstr ""
 
-#: bzrlib/builtins.py:3977
+#: bzrlib/builtins.py:4134
 msgid "        bzr merge ../bzr.dev"
 msgstr ""
 
-#: bzrlib/builtins.py:3979
+#: bzrlib/builtins.py:4136
 msgid "    To merge changes up to and including revision 82 from bzr.dev::"
 msgstr ""
 
-#: bzrlib/builtins.py:3981
+#: bzrlib/builtins.py:4138
 msgid "        bzr merge -r 82 ../bzr.dev"
 msgstr ""
 
-#: bzrlib/builtins.py:3983
+#: bzrlib/builtins.py:4140
 msgid "    To merge the changes introduced by 82, without previous changes::"
 msgstr ""
 
-#: bzrlib/builtins.py:3985
+#: bzrlib/builtins.py:4142
 msgid "        bzr merge -r 81..82 ../bzr.dev"
 msgstr ""
 
-#: bzrlib/builtins.py:3987
+#: bzrlib/builtins.py:4144
 msgid "    To apply a merge directive contained in /tmp/merge::"
 msgstr ""
 
-#: bzrlib/builtins.py:3989
+#: bzrlib/builtins.py:4146
 msgid "        bzr merge /tmp/merge"
 msgstr ""
 
-#: bzrlib/builtins.py:3991
+#: bzrlib/builtins.py:4148
 msgid ""
 "    To create a merge revision with three parents from two branches\n"
 "    feature1a and feature1b:"
 msgstr ""
 
-#: bzrlib/builtins.py:3994
+#: bzrlib/builtins.py:4151
 msgid ""
 "        bzr merge ../feature1a\n"
 "        bzr merge ../feature1b --force\n"
@@ -2200,18 +2763,18 @@
 msgstr ""
 
 # help of 'force' option of 'merge' command
-#: bzrlib/builtins.py:4006
+#: bzrlib/builtins.py:4163
 msgid "Merge even if the destination tree has uncommitted changes."
 msgstr ""
 
 # help of 'uncommitted' option of 'merge' command
-#: bzrlib/builtins.py:4012
+#: bzrlib/builtins.py:4169
 msgid ""
 "Apply uncommitted changes from a working copy, instead of branch changes."
 msgstr ""
 
 # help of 'pull' option of 'merge' command
-#: bzrlib/builtins.py:4014
+#: bzrlib/builtins.py:4171
 msgid ""
 "If the destination is already completely merged into the source, pull from "
 "the source rather than merging.  When this happens, you do not need to "
@@ -2219,77 +2782,135 @@
 msgstr ""
 
 # help of 'directory' option of 'merge' command
-#: bzrlib/builtins.py:4019
+#: bzrlib/builtins.py:4176
 msgid ""
 "Branch to merge into, rather than the one containing the working directory."
 msgstr ""
 
 # help of 'preview' option of 'merge' command
-#: bzrlib/builtins.py:4021
+#: bzrlib/builtins.py:4178
 msgid "Instead of merging, show a diff of the merge."
 msgstr ""
 
 # help of 'interactive' option of 'merge' command
-#: bzrlib/builtins.py:4023
+#: bzrlib/builtins.py:4180
 msgid "Select changes interactively."
 msgstr ""
 
-#: bzrlib/builtins.py:4307
+#: bzrlib/builtins.py:4202
+msgid ""
+"Merging into empty branches not currently supported, https://bugs.launchpad."
+"net/bzr/+bug/308562"
+msgstr ""
+
+#: bzrlib/builtins.py:4229
+msgid "Cannot use --uncommitted with bundles or merge directives."
+msgstr ""
+
+#: bzrlib/builtins.py:4240
+msgid "Cannot use --uncommitted and --revision at the same time."
+msgstr ""
+
+#: bzrlib/builtins.py:4260
+msgid "merger: "
+msgstr ""
+
+#: bzrlib/builtins.py:4262 bzrlib/bundle/apply_bundle.py:68
+msgid "Nothing to do."
+msgstr ""
+
+#: bzrlib/builtins.py:4266
+msgid "Cannot pull individual files"
+msgstr ""
+
+#: bzrlib/builtins.py:4274
+msgid "This branch has no commits. (perhaps you would prefer 'bzr pull')"
+msgstr ""
+
+#: bzrlib/builtins.py:4332
+#, python-format
+msgid "Show-base is not supported for this merge type. %s"
+msgstr ""
+
+#: bzrlib/builtins.py:4341
+#, python-format
+msgid "Conflict reduction is not supported for merge type %s."
+msgstr ""
+
+#: bzrlib/builtins.py:4345
+msgid "Cannot do conflict reduction and show base."
+msgstr ""
+
+#: bzrlib/builtins.py:4456
+msgid "No location specified or remembered"
+msgstr ""
+
+#: bzrlib/builtins.py:4458
+msgid "{0} remembered {1} location {2}"
+msgstr ""
+
+#: bzrlib/builtins.py:4464
 msgid "Redo a merge."
 msgstr ""
 
-#: bzrlib/builtins.py:4309
+#: bzrlib/builtins.py:4466
 msgid ""
 "Use this if you want to try a different merge technique while resolving\n"
 "conflicts.  Some merge techniques are better than others, and remerge\n"
 "lets you try different ones on different files."
 msgstr ""
 
-#: bzrlib/builtins.py:4313
+#: bzrlib/builtins.py:4470
 msgid ""
 "The options for remerge have the same meaning and defaults as the ones for\n"
 "merge.  The difference is that remerge can (only) be run when there is a\n"
 "pending merge, and it lets you specify particular files."
 msgstr ""
 
-#: bzrlib/builtins.py:4317
+#: bzrlib/builtins.py:4474
 msgid ""
 ":Examples:\n"
 "    Re-do the merge of all conflicted files, and show the base text in\n"
 "    conflict regions, in addition to the usual THIS and OTHER texts::"
 msgstr ""
 
-#: bzrlib/builtins.py:4321
+#: bzrlib/builtins.py:4478
 msgid "        bzr remerge --show-base"
 msgstr ""
 
-#: bzrlib/builtins.py:4323
+#: bzrlib/builtins.py:4480
 msgid ""
 "    Re-do the merge of \"foobar\", using the weave merge algorithm, with\n"
 "    additional processing to reduce the size of conflict regions::"
 msgstr ""
 
-#: bzrlib/builtins.py:4326
+#: bzrlib/builtins.py:4483
 msgid "        bzr remerge --merge-type weave --reprocess foobar"
 msgstr ""
 
 # help of 'show-base' option of 'merge' command
-#: bzrlib/builtins.py:4333
+#: bzrlib/builtins.py:4490
 msgid "Show base revision text in conflicts."
 msgstr ""
 
-#: bzrlib/builtins.py:4401
+#: bzrlib/builtins.py:4502
+msgid ""
+"Sorry, remerge only works after normal merges.  Not cherrypicking or multi-"
+"merges."
+msgstr ""
+
+#: bzrlib/builtins.py:4558
 msgid "Revert files to a previous revision."
 msgstr ""
 
-#: bzrlib/builtins.py:4403
+#: bzrlib/builtins.py:4560
 msgid ""
 "Giving a list of files will revert only those files.  Otherwise, all files\n"
 "will be reverted.  If the revision is not specified with '--revision', the\n"
 "last committed revision is used."
 msgstr ""
 
-#: bzrlib/builtins.py:4407
+#: bzrlib/builtins.py:4564
 msgid ""
 "To remove only some changes, without reverting to a prior version, use\n"
 "merge instead.  For example, \"merge . -r -2..-3\" (don't forget the \".\")\n"
@@ -2298,14 +2919,14 @@
 "certain changes on a hunk-by-hunk basis, see the shelve command."
 msgstr ""
 
-#: bzrlib/builtins.py:4413
+#: bzrlib/builtins.py:4570
 msgid ""
 "By default, any files that have been manually changed will be backed up\n"
 "first.  (Files changed only by merge are not backed up.)  Backup files have\n"
 "'.~#~' appended to their name, where # is a number."
 msgstr ""
 
-#: bzrlib/builtins.py:4417
+#: bzrlib/builtins.py:4574
 msgid ""
 "When you provide files, you can use their current pathname or the pathname\n"
 "from the target revision.  So you can use revert to \"undelete\" a file by\n"
@@ -2313,7 +2934,7 @@
 "reverted."
 msgstr ""
 
-#: bzrlib/builtins.py:4422
+#: bzrlib/builtins.py:4579
 msgid ""
 "If you have newly added files since the target revision, they will be\n"
 "removed.  If the files to be removed have been changed, backups will be\n"
@@ -2321,7 +2942,7 @@
 "deleted."
 msgstr ""
 
-#: bzrlib/builtins.py:4427
+#: bzrlib/builtins.py:4584
 msgid ""
 "The working tree contains a list of revisions that have been merged but\n"
 "not yet committed. These revisions will be included as additional parents\n"
@@ -2333,7 +2954,7 @@
 "reverting any files."
 msgstr ""
 
-#: bzrlib/builtins.py:4436
+#: bzrlib/builtins.py:4593
 msgid ""
 "Using \"bzr revert --forget-merges\", it is possible to apply all of the\n"
 "changes from a branch in a single revision.  To do this, perform the merge\n"
@@ -2348,35 +2969,35 @@
 msgstr ""
 
 # help of 'no-backup' option of 'revert' command
-#: bzrlib/builtins.py:4450
+#: bzrlib/builtins.py:4607
 msgid "Do not save backups of reverted files."
 msgstr ""
 
 # help of 'forget-merges' option of 'revert' command
-#: bzrlib/builtins.py:4452
+#: bzrlib/builtins.py:4609
 msgid "Remove pending merge marker, without changing any files."
 msgstr ""
 
-#: bzrlib/builtins.py:4483
+#: bzrlib/builtins.py:4640
 msgid ""
 "Show help on a command or other topic.\n"
 "    "
 msgstr ""
 
 # help of 'long' option of 'help' command
-#: bzrlib/builtins.py:4488
+#: bzrlib/builtins.py:4645
 msgid "Show help on all commands."
 msgstr ""
 
-#: bzrlib/builtins.py:4517
+#: bzrlib/builtins.py:4674
 msgid "Show unmerged/unpulled revisions between two branches."
 msgstr ""
 
-#: bzrlib/builtins.py:4519
+#: bzrlib/builtins.py:4676
 msgid "OTHER_BRANCH may be local or remote."
 msgstr ""
 
-#: bzrlib/builtins.py:4521
+#: bzrlib/builtins.py:4678
 msgid ""
 "To filter on a range of revisions, you can use the command -r begin..end\n"
 "-r revision requests a specific revision, -r ..end or -r begin.. are\n"
@@ -2387,100 +3008,134 @@
 "    0 - no missing revisions"
 msgstr ""
 
-#: bzrlib/builtins.py:4531
+#: bzrlib/builtins.py:4688
 msgid ""
 "    Determine the missing revisions between this and the branch at the\n"
 "    remembered pull location::"
 msgstr ""
 
-#: bzrlib/builtins.py:4534
+#: bzrlib/builtins.py:4691
 msgid "        bzr missing"
 msgstr ""
 
-#: bzrlib/builtins.py:4536
+#: bzrlib/builtins.py:4693
 msgid "    Determine the missing revisions between this and another branch::"
 msgstr ""
 
-#: bzrlib/builtins.py:4538
+#: bzrlib/builtins.py:4695
 msgid "        bzr missing http://server/branch"
 msgstr ""
 
-#: bzrlib/builtins.py:4540
+#: bzrlib/builtins.py:4697
 msgid ""
 "    Determine the missing revisions up to a specific revision on the other\n"
 "    branch::"
 msgstr ""
 
-#: bzrlib/builtins.py:4543
+#: bzrlib/builtins.py:4700
 msgid "        bzr missing -r ..-10"
 msgstr ""
 
-#: bzrlib/builtins.py:4545
+#: bzrlib/builtins.py:4702
 msgid ""
 "    Determine the missing revisions up to a specific revision on this\n"
 "    branch::"
 msgstr ""
 
-#: bzrlib/builtins.py:4548
+#: bzrlib/builtins.py:4705
 msgid "        bzr missing --my-revision ..-10"
 msgstr ""
 
 # help of 'reverse' option of 'missing' command
-#: bzrlib/builtins.py:4555
+#: bzrlib/builtins.py:4712
 msgid "Reverse the order of revisions."
 msgstr ""
 
 # help of 'mine-only' option of 'missing' command
-#: bzrlib/builtins.py:4557
+#: bzrlib/builtins.py:4714
 msgid "Display changes in the local branch only."
 msgstr ""
 
 # help of 'this' option of 'missing' command
-#: bzrlib/builtins.py:4558
+#: bzrlib/builtins.py:4715
 msgid "Same as --mine-only."
 msgstr ""
 
 # help of 'theirs-only' option of 'missing' command
-#: bzrlib/builtins.py:4560
+#: bzrlib/builtins.py:4717
 msgid "Display changes in the remote branch only."
 msgstr ""
 
 # help of 'other' option of 'missing' command
-#: bzrlib/builtins.py:4561
+#: bzrlib/builtins.py:4718
 msgid "Same as --theirs-only."
 msgstr ""
 
 # help of 'revision' option of 'missing' command
-#: bzrlib/builtins.py:4566
+#: bzrlib/builtins.py:4723
 msgid ""
 "Filter on other branch revisions (inclusive). See \"help revisionspec\" for "
 "details."
 msgstr ""
 
 # help of 'my-revision' option of 'missing' command
-#: bzrlib/builtins.py:4570
+#: bzrlib/builtins.py:4727
 msgid ""
 "Filter on local branch revisions (inclusive). See \"help revisionspec\" for "
 "details."
 msgstr ""
 
-# help of 'include-merges' option of 'missing' command
-#: bzrlib/builtins.py:4573
+# help of 'include-merged' option of 'missing' command
+#: bzrlib/builtins.py:4730
 msgid "Show all revisions in addition to the mainline ones."
 msgstr ""
 
-#: bzrlib/builtins.py:4689
+#: bzrlib/builtins.py:4784
+msgid "No peer location known or specified."
+msgstr ""
+
+#: bzrlib/builtins.py:4788
+msgid "Using saved parent location: {0}\n"
+msgstr ""
+
+#: bzrlib/builtins.py:4821
+#, python-format
+msgid "You have %d extra revision:\n"
+msgid_plural "You have %d extra revisions:\n"
+msgstr[0] ""
+msgstr[1] ""
+
+#: bzrlib/builtins.py:4837
+#, python-format
+msgid "You are missing %d revision:\n"
+msgid_plural "You are missing %d revisions:\n"
+msgstr[0] ""
+msgstr[1] ""
+
+#: bzrlib/builtins.py:4849
+msgid "This branch has no new revisions.\n"
+msgstr ""
+
+#: bzrlib/builtins.py:4852
+msgid "Other branch has no new revisions.\n"
+msgstr ""
+
+#: bzrlib/builtins.py:4857
+msgid "Branches are up to date.\n"
+msgstr ""
+
+#: bzrlib/builtins.py:4868
 msgid "Compress the data within a repository."
 msgstr ""
 
-#: bzrlib/builtins.py:4691
+#: bzrlib/builtins.py:4870
 msgid ""
 "This operation compresses the data within a bazaar repository. As\n"
 "bazaar supports automatic packing of repository, this operation is\n"
 "normally not required to be done manually."
 msgstr ""
 
-#: bzrlib/builtins.py:4695
+#: bzrlib/builtins.py:4874
 msgid ""
 "During the pack operation, bazaar takes a backup of existing repository\n"
 "data, i.e. pack files. This backup is eventually removed by bazaar\n"
@@ -2489,7 +3144,7 @@
 "used."
 msgstr ""
 
-#: bzrlib/builtins.py:4701
+#: bzrlib/builtins.py:4880
 msgid ""
 "Warning: If you use --clean-obsolete-packs and your machine crashes\n"
 "during or immediately after repacking, you may be left with a state\n"
@@ -2498,25 +3153,25 @@
 msgstr ""
 
 # help of 'clean-obsolete-packs' option of 'pack' command
-#: bzrlib/builtins.py:4710
+#: bzrlib/builtins.py:4889
 msgid "Delete obsolete packs to save disk space."
 msgstr ""
 
-#: bzrlib/builtins.py:4724
+#: bzrlib/builtins.py:4903
 msgid "List the installed plugins."
 msgstr ""
 
-#: bzrlib/builtins.py:4726
+#: bzrlib/builtins.py:4905
 msgid ""
 "This command displays the list of installed plugins including\n"
 "version of plugin and a short description of each."
 msgstr ""
 
-#: bzrlib/builtins.py:4729
+#: bzrlib/builtins.py:4908
 msgid "--verbose shows the path where each plugin is located."
 msgstr ""
 
-#: bzrlib/builtins.py:4731
+#: bzrlib/builtins.py:4910
 msgid ""
 "A plugin is an external component for Bazaar that extends the\n"
 "revision control system, by adding or replacing code in Bazaar.\n"
@@ -2525,7 +3180,7 @@
 "customizing log output."
 msgstr ""
 
-#: bzrlib/builtins.py:4737
+#: bzrlib/builtins.py:4916
 msgid ""
 "See the Bazaar Plugin Guide <http://doc.bazaar.canonical.com/plugins/en/>\n"
 "for further information on plugins including where to find them and how to\n"
@@ -2533,94 +3188,122 @@
 "plugins using the Python programming language."
 msgstr ""
 
-#: bzrlib/builtins.py:4753
+#: bzrlib/builtins.py:4932
 msgid "Show testament (signing-form) of a revision."
 msgstr ""
 
 # help of 'long' option of 'testament' command
-#: bzrlib/builtins.py:4756
+#: bzrlib/builtins.py:4935
 msgid "Produce long-format testament."
 msgstr ""
 
 # help of 'strict' option of 'testament' command
-#: bzrlib/builtins.py:4758
+#: bzrlib/builtins.py:4937
 msgid "Produce a strict-format testament."
 msgstr ""
 
-#: bzrlib/builtins.py:4784
+#: bzrlib/builtins.py:4963
 msgid "Show the origin of each line in a file."
 msgstr ""
 
-#: bzrlib/builtins.py:4786
+#: bzrlib/builtins.py:4965
 msgid ""
 "This prints out the given file with an annotation on the left side\n"
 "indicating which revision, author and date introduced the change."
 msgstr ""
 
-#: bzrlib/builtins.py:4789
+#: bzrlib/builtins.py:4968
 msgid ""
 "If the origin is the same for a run of consecutive lines, it is\n"
 "shown only at the top, unless the --all option is given."
 msgstr ""
 
 # help of 'all' option of 'annotate' command
-#: bzrlib/builtins.py:4797
+#: bzrlib/builtins.py:4976
 msgid "Show annotations on all lines."
 msgstr ""
 
 # help of 'long' option of 'annotate' command
-#: bzrlib/builtins.py:4798
+#: bzrlib/builtins.py:4977
 msgid "Show commit date in annotations."
 msgstr ""
 
-#: bzrlib/builtins.py:4901
+#: bzrlib/builtins.py:5064
+msgid "Cannot sign a range of non-revision-history revisions"
+msgstr ""
+
+#: bzrlib/builtins.py:5076
+msgid "Please supply either one revision, or a range."
+msgstr ""
+
+#: bzrlib/builtins.py:5080
 msgid ""
 "Convert the current branch into a checkout of the supplied branch.\n"
 "If no branch is supplied, rebind to the last bound location."
 msgstr ""
 
-#: bzrlib/builtins.py:4904
+#: bzrlib/builtins.py:5083
 msgid ""
 "Once converted into a checkout, commits must succeed on the master branch\n"
 "before they will be applied to the local branch."
 msgstr ""
 
-#: bzrlib/builtins.py:4907
+#: bzrlib/builtins.py:5086
 msgid ""
 "Bound branches use the nickname of its master branch unless it is set\n"
 "locally, in which case binding will update the local nickname to be\n"
 "that of the master."
 msgstr ""
 
-#: bzrlib/builtins.py:4942
+#: bzrlib/builtins.py:5101
+msgid "No location supplied.  This format does not remember old locations."
+msgstr ""
+
+#: bzrlib/builtins.py:5106
+msgid "Branch is already bound"
+msgstr ""
+
+#: bzrlib/builtins.py:5108
+msgid "No location supplied and no previous location known"
+msgstr ""
+
+#: bzrlib/builtins.py:5114
+msgid "These branches have diverged. Try merging, and then bind again."
+msgstr ""
+
+#: bzrlib/builtins.py:5121
 msgid "Convert the current checkout into a regular branch."
 msgstr ""
 
-#: bzrlib/builtins.py:4944
+#: bzrlib/builtins.py:5123
 msgid ""
 "After unbinding, the local branch is considered independent and subsequent\n"
 "commits will be local only."
 msgstr ""
 
-#: bzrlib/builtins.py:4959
+#: bzrlib/builtins.py:5134
+msgid "Local branch is not bound"
+msgstr ""
+
+#: bzrlib/builtins.py:5138
 msgid "Remove the last committed revision."
 msgstr ""
 
-#: bzrlib/builtins.py:4961
+#: bzrlib/builtins.py:5140
 msgid ""
 "--verbose will print out what is being removed.\n"
 "--dry-run will go through all the motions, but not actually\n"
 "remove anything."
 msgstr ""
 
-#: bzrlib/builtins.py:4965
+#: bzrlib/builtins.py:5144
 msgid ""
 "If --revision is specified, uncommit revisions to leave the branch at the\n"
 "specified revision.  For example, \"bzr uncommit -r 15\" will leave the\n"
 "branch at revision 15."
 msgstr ""
 
-#: bzrlib/builtins.py:4969
+#: bzrlib/builtins.py:5148
 msgid ""
 "Uncommit leaves the working tree ready for a new commit.  The only change\n"
 "it may make is to restore any pending merges that were present before\n"
@@ -2628,43 +3311,75 @@
 msgstr ""
 
 # help of 'dry-run' option of 'uncommit' command
-#: bzrlib/builtins.py:4980
+#: bzrlib/builtins.py:5159
 msgid "Don't actually make changes."
 msgstr ""
 
 # help of 'force' option of 'uncommit' command
-#: bzrlib/builtins.py:4981
+#: bzrlib/builtins.py:5160
 msgid "Say yes to all questions."
 msgstr ""
 
+# help of 'keep-tags' option of 'uncommit' command
+#: bzrlib/builtins.py:5162
+msgid "Keep tags that point to removed revisions."
+msgstr ""
+
 # help of 'local' option of 'uncommit' command
-#: bzrlib/builtins.py:4983
+#: bzrlib/builtins.py:5164
 msgid "Only remove the commits from the local branch when in a checkout."
 msgstr ""
 
-#: bzrlib/builtins.py:5068
+#: bzrlib/builtins.py:5213
+msgid "No revisions to uncommit.\n"
+msgstr ""
+
+#: bzrlib/builtins.py:5228
+msgid "Dry-run, pretending to remove the above revisions.\n"
+msgstr ""
+
+#: bzrlib/builtins.py:5231
+msgid "The above revision(s) will be removed.\n"
+msgstr ""
+
+#: bzrlib/builtins.py:5235
+msgid "Uncommit these revisions"
+msgstr ""
+
+#: bzrlib/builtins.py:5238
+msgid "Canceled\n"
+msgstr ""
+
+#: bzrlib/builtins.py:5245
+#, python-format
+msgid ""
+"You can restore the old tip by running:\n"
+"  bzr pull . -r revid:%s\n"
+msgstr ""
+
+#: bzrlib/builtins.py:5250
 msgid "Break a dead lock."
 msgstr ""
 
-#: bzrlib/builtins.py:5070
+#: bzrlib/builtins.py:5252
 msgid ""
 "This command breaks a lock on a repository, branch, working directory or\n"
 "config file."
 msgstr ""
 
-#: bzrlib/builtins.py:5073
+#: bzrlib/builtins.py:5255
 msgid ""
 "CAUTION: Locks should only be broken when you are sure that the process\n"
 "holding the lock has been stopped."
 msgstr ""
 
-#: bzrlib/builtins.py:5076
+#: bzrlib/builtins.py:5258
 msgid ""
 "You can get information on what locks are open via the 'bzr info\n"
 "[location]' command."
 msgstr ""
 
-#: bzrlib/builtins.py:5079
+#: bzrlib/builtins.py:5261
 msgid ""
 ":Examples:\n"
 "    bzr break-lock\n"
@@ -2673,36 +3388,41 @@
 msgstr ""
 
 # help of 'config' option of 'break-lock' command
-#: bzrlib/builtins.py:5088
+#: bzrlib/builtins.py:5270
 msgid "LOCATION is the directory where the config lock is."
 msgstr ""
 
 # help of 'force' option of 'break-lock' command
-#: bzrlib/builtins.py:5090
+#: bzrlib/builtins.py:5272
 msgid "Do not ask for confirmation before breaking the lock."
 msgstr ""
 
-#: bzrlib/builtins.py:5126
+# help of 'protocol=bzr' option of 'serve' command
+#: bzrlib/builtins.py:5307
+msgid "The Bazaar smart server protocol over TCP. (default port: 4155)"
+msgstr ""
+
+#: bzrlib/builtins.py:5308
 msgid "Run the bzr server."
 msgstr ""
 
 # help of 'inet' option of 'serve' command
-#: bzrlib/builtins.py:5132
+#: bzrlib/builtins.py:5314
 msgid "Serve on stdin/out for use from inetd or sshd."
 msgstr ""
 
 # title of 'protocol' option of 'serve' command
-#: bzrlib/builtins.py:5133
+#: bzrlib/builtins.py:5315
 msgid "protocol"
 msgstr ""
 
 # help of 'protocol' option of 'serve' command
-#: bzrlib/builtins.py:5134
+#: bzrlib/builtins.py:5316
 msgid "Protocol to serve."
 msgstr ""
 
 # help of 'port' option of 'serve' command
-#: bzrlib/builtins.py:5138
+#: bzrlib/builtins.py:5320
 msgid ""
 "Listen for connections on nominated port of the form [hostname:]portnumber.  "
 "Passing 0 as the port number will result in a dynamically allocated port.  "
@@ -2710,12 +3430,12 @@
 msgstr ""
 
 # help of 'directory' option of 'serve' command
-#: bzrlib/builtins.py:5144
+#: bzrlib/builtins.py:5326
 msgid "Serve contents of this directory."
 msgstr ""
 
 # help of 'allow-writes' option of 'serve' command
-#: bzrlib/builtins.py:5146
+#: bzrlib/builtins.py:5328
 msgid ""
 "By default the server is a readonly server.  Supplying --allow-writes "
 "enables write access to the contents of the served directory and below.  "
@@ -2724,83 +3444,115 @@
 "uncontrolled write access to your file system."
 msgstr ""
 
-#: bzrlib/builtins.py:5191
+# help of 'client-timeout' option of 'serve' command
+#: bzrlib/builtins.py:5337
+msgid "Override the default idle client timeout (5min)."
+msgstr ""
+
+#: bzrlib/builtins.py:5387
 msgid "Combine a tree into its containing tree."
 msgstr ""
 
-#: bzrlib/builtins.py:5193
+#: bzrlib/builtins.py:5389
 msgid "This command requires the target tree to be in a rich-root format."
 msgstr ""
 
-#: bzrlib/builtins.py:5195
+#: bzrlib/builtins.py:5391
 msgid ""
 "The TREE argument should be an independent tree, inside another tree, but\n"
 "not part of it.  (Such trees can be produced by \"bzr split\", but also by\n"
 "running \"bzr branch\" with the target inside a tree.)"
 msgstr ""
 
-#: bzrlib/builtins.py:5199
+#: bzrlib/builtins.py:5395
 msgid ""
 "The result is a combined tree, with the subtree no longer an independent\n"
 "part.  This is marked as a merge of the subtree into the containing tree,\n"
 "and all history is preserved."
 msgstr ""
 
-#: bzrlib/builtins.py:5237
+#: bzrlib/builtins.py:5413
+#, python-format
+msgid ""
+"Can't join trees because %s doesn't support rich root data.\n"
+"You can use bzr upgrade on the repository."
+msgstr ""
+
+#: bzrlib/builtins.py:5423 bzrlib/builtins.py:5429
+msgid "Cannot join {0}.  {1}"
+msgstr ""
+
+#: bzrlib/builtins.py:5433
 msgid "Split a subdirectory of a tree into a separate tree."
 msgstr ""
 
-#: bzrlib/builtins.py:5239
+#: bzrlib/builtins.py:5435
 msgid ""
 "This command will produce a target tree in a format that supports\n"
 "rich roots, like 'rich-root' or 'rich-root-pack'.  These formats cannot be\n"
 "converted into earlier formats like 'dirstate-tags'."
 msgstr ""
 
-#: bzrlib/builtins.py:5243
+#: bzrlib/builtins.py:5439
 msgid ""
 "The TREE argument should be a subdirectory of a working tree.  That\n"
 "subdirectory will be converted into an independent tree, with its own\n"
 "branch.  Commits in the top-level tree will not apply to the new subtree."
 msgstr ""
 
-#: bzrlib/builtins.py:5364
+#: bzrlib/builtins.py:5519
+msgid "No submit branch specified or known"
+msgstr ""
+
+#: bzrlib/builtins.py:5527 bzrlib/send.py:106
+msgid "No public branch specified or known"
+msgstr ""
+
+#: bzrlib/builtins.py:5532
+msgid "bzr merge-directive takes at most two one revision identifiers"
+msgstr ""
+
+#: bzrlib/builtins.py:5541
+msgid "No revisions to bundle."
+msgstr ""
+
+#: bzrlib/builtins.py:5560
 msgid "Mail or create a merge-directive for submitting changes."
 msgstr ""
 
-#: bzrlib/builtins.py:5366
+#: bzrlib/builtins.py:5562
 msgid "A merge directive provides many things needed for requesting merges:"
 msgstr ""
 
-#: bzrlib/builtins.py:5368
+#: bzrlib/builtins.py:5564
 msgid "* A machine-readable description of the merge to perform"
 msgstr ""
 
-#: bzrlib/builtins.py:5370
+#: bzrlib/builtins.py:5566
 msgid "* An optional patch that is a preview of the changes requested"
 msgstr ""
 
-#: bzrlib/builtins.py:5372
+#: bzrlib/builtins.py:5568
 msgid ""
 "* An optional bundle of revision data, so that the changes can be applied\n"
 "  directly from the merge directive, without retrieving data from a\n"
 "  branch."
 msgstr ""
 
-#: bzrlib/builtins.py:5376
+#: bzrlib/builtins.py:5572
 msgid ""
 "`bzr send` creates a compact data set that, when applied using bzr\n"
 "merge, has the same effect as merging from the source branch.  "
 msgstr ""
 
-#: bzrlib/builtins.py:5379
+#: bzrlib/builtins.py:5575
 msgid ""
 "By default the merge directive is self-contained and can be applied to any\n"
 "branch containing submit_branch in its ancestory without needing access to\n"
 "the source branch."
 msgstr ""
 
-#: bzrlib/builtins.py:5383
+#: bzrlib/builtins.py:5579
 msgid ""
 "If --no-bundle is specified, then Bazaar doesn't send the contents of the\n"
 "revisions, but only a structured request to merge from the\n"
@@ -2809,7 +3561,7 @@
 "included if known, so that people can check it later."
 msgstr ""
 
-#: bzrlib/builtins.py:5389
+#: bzrlib/builtins.py:5585
 msgid ""
 "The submit branch defaults to the parent of the source branch, but can be\n"
 "overridden.  Both submit branch and public branch will be remembered in\n"
@@ -2818,16 +3570,16 @@
 "be changed using --from."
 msgstr ""
 
-#: bzrlib/builtins.py:5395
+#: bzrlib/builtins.py:5591
 msgid ""
 "Both the submit branch and the public branch follow the usual behavior with\n"
 "respect to --remember: If there is no default location set, the first send\n"
-"will set it (use --no-remember to avoid settting it). After that, you can\n"
+"will set it (use --no-remember to avoid setting it). After that, you can\n"
 "omit the location to use the default.  To change the default, use\n"
 "--remember. The value will only be saved if the location can be accessed."
 msgstr ""
 
-#: bzrlib/builtins.py:5401
+#: bzrlib/builtins.py:5597
 msgid ""
 "In order to calculate those changes, bzr must analyse the submit branch.\n"
 "Therefore it is most efficient for the submit branch to be a local mirror.\n"
@@ -2835,20 +3587,20 @@
 "in the merge directive."
 msgstr ""
 
-#: bzrlib/builtins.py:5406
+#: bzrlib/builtins.py:5602
 msgid ""
 "The default behaviour is to send the merge directive by mail, unless -o is\n"
 "given, in which case it is sent to a file."
 msgstr ""
 
-#: bzrlib/builtins.py:5409
+#: bzrlib/builtins.py:5605
 msgid ""
 "Mail is sent using your preferred mail program.  This should be transparent\n"
 "on Windows (it uses MAPI).  On Unix, it requires the xdg-email utility.\n"
 "If the preferred client can't be found (or used), your editor will be used."
 msgstr ""
 
-#: bzrlib/builtins.py:5413
+#: bzrlib/builtins.py:5609
 msgid ""
 "To use a specific mail program, set the mail_client configuration option.\n"
 "(For Thunderbird 1.5, this works around some bugs.)  Supported values for\n"
@@ -2860,7 +3612,7 @@
 "supported clients."
 msgstr ""
 
-#: bzrlib/builtins.py:5420
+#: bzrlib/builtins.py:5616
 msgid ""
 "If mail is being sent, a to address is required.  This can be supplied\n"
 "either on the commandline, by setting the submit_to configuration\n"
@@ -2868,7 +3620,7 @@
 "in the submit branch."
 msgstr ""
 
-#: bzrlib/builtins.py:5425
+#: bzrlib/builtins.py:5621
 msgid ""
 "Two formats are currently supported: \"4\" uses revision bundle format 4 "
 "and\n"
@@ -2878,13 +3630,13 @@
 "format 1.  It is compatible with Bazaar 0.12 - 0.18."
 msgstr ""
 
-#: bzrlib/builtins.py:5431
+#: bzrlib/builtins.py:5627
 msgid ""
 "The merge directives created by bzr send may be applied using bzr merge or\n"
 "bzr pull by specifying a file containing a merge directive as the location."
 msgstr ""
 
-#: bzrlib/builtins.py:5434
+#: bzrlib/builtins.py:5630
 msgid ""
 "bzr send makes extensive use of public locations to map local locations "
 "into\n"
@@ -2893,90 +3645,90 @@
 msgstr ""
 
 # help of 'output' option of 'send' command
-#: bzrlib/builtins.py:5458
+#: bzrlib/builtins.py:5654
 msgid "Write merge directive to this file or directory; use - for stdout."
 msgstr ""
 
 # help of 'strict' option of 'send' command
-#: bzrlib/builtins.py:5462
+#: bzrlib/builtins.py:5658
 msgid ""
 "Refuse to send if there are uncommitted changes in the working tree, --no-"
 "strict disables the check."
 msgstr ""
 
 # help of 'mail-to' option of 'send' command
-#: bzrlib/builtins.py:5464
+#: bzrlib/builtins.py:5660
 msgid "Mail the request to this address."
 msgstr ""
 
 # help of 'body' option of 'send' command
-#: bzrlib/builtins.py:5468
+#: bzrlib/builtins.py:5664
 msgid "Body for the email."
 msgstr ""
 
 # help of 'no-bundle' option of 'send' command
-#: bzrlib/builtins.py:5521
+#: bzrlib/builtins.py:5717
 msgid "Do not include a bundle in the merge directive."
 msgstr ""
 
 # help of 'no-patch' option of 'send' command
-#: bzrlib/builtins.py:5522
+#: bzrlib/builtins.py:5718
 msgid "Do not include a preview patch in the merge directive."
 msgstr ""
 
 # help of 'remember' option of 'send' command
-#: bzrlib/builtins.py:5525
+#: bzrlib/builtins.py:5721
 msgid "Remember submit and public branch."
 msgstr ""
 
 # help of 'from' option of 'send' command
-#: bzrlib/builtins.py:5527
+#: bzrlib/builtins.py:5723
 msgid ""
 "Branch to generate the submission from, rather than the one containing the "
 "working directory."
 msgstr ""
 
 # title of 'format' option of 'send' command
-#: bzrlib/builtins.py:5537
+#: bzrlib/builtins.py:5733
 msgid "format"
 msgstr ""
 
 # help of 'format' option of 'send' command
-#: bzrlib/builtins.py:5538
+#: bzrlib/builtins.py:5734
 msgid "Use the specified output format."
 msgstr ""
 
-#: bzrlib/builtins.py:5560
+#: bzrlib/builtins.py:5756
 msgid "Create, remove or modify a tag naming a revision."
 msgstr ""
 
-#: bzrlib/builtins.py:5562
+#: bzrlib/builtins.py:5758
 msgid ""
 "Tags give human-meaningful names to revisions.  Commands that take a -r\n"
 "(--revision) option can be given -rtag:X, where X is any previously\n"
 "created tag."
 msgstr ""
 
-#: bzrlib/builtins.py:5566
+#: bzrlib/builtins.py:5762
 msgid ""
 "Tags are stored in the branch.  Tags are copied from one branch to another\n"
 "along when you branch, push, pull or merge."
 msgstr ""
 
-#: bzrlib/builtins.py:5569
+#: bzrlib/builtins.py:5765
 msgid ""
 "It is an error to give a tag name that already exists unless you pass\n"
 "--force, in which case the tag is moved to point to the new revision."
 msgstr ""
 
-#: bzrlib/builtins.py:5572
+#: bzrlib/builtins.py:5768
 msgid ""
 "To rename a tag (change the name but keep it on the same revsion), run "
 "``bzr\n"
 "tag new-name -r tag:old-name`` and then ``bzr tag --delete oldname``."
 msgstr ""
 
-#: bzrlib/builtins.py:5575
+#: bzrlib/builtins.py:5771
 msgid ""
 "If no tag name is specified it will be determined through the \n"
 "'automatic_tag_name' hook. This can e.g. be used to automatically tag\n"
@@ -2985,53 +3737,81 @@
 msgstr ""
 
 # help of 'delete' option of 'tag' command
-#: bzrlib/builtins.py:5585
+#: bzrlib/builtins.py:5781
 msgid "Delete this tag rather than placing it."
 msgstr ""
 
 # help of 'directory' option of 'tag' command
-#: bzrlib/builtins.py:5588
+#: bzrlib/builtins.py:5784
 msgid "Branch in which to place the tag."
 msgstr ""
 
 # help of 'force' option of 'tag' command
-#: bzrlib/builtins.py:5590
+#: bzrlib/builtins.py:5786
 msgid "Replace existing tags."
 msgstr ""
 
-#: bzrlib/builtins.py:5629
+#: bzrlib/builtins.py:5801
+msgid "No tag specified to delete."
+msgstr ""
+
+#: bzrlib/builtins.py:5803
+#, python-format
+msgid "Deleted tag %s."
+msgstr ""
+
+#: bzrlib/builtins.py:5817
+msgid "Please specify a tag name."
+msgstr ""
+
+#: bzrlib/builtins.py:5825
+#, python-format
+msgid "Tag %s already exists for that revision."
+msgstr ""
+
+#: bzrlib/builtins.py:5829
+#, python-format
+msgid "Created tag %s."
+msgstr ""
+
+#: bzrlib/builtins.py:5831
+#, python-format
+msgid "Updated tag %s."
+msgstr ""
+
+#: bzrlib/builtins.py:5835
 msgid "List tags."
 msgstr ""
 
-#: bzrlib/builtins.py:5631
+#: bzrlib/builtins.py:5837
 msgid ""
 "This command shows a table of tag names and the revisions they reference."
 msgstr ""
 
 # help of 'directory' option of 'tags' command
-#: bzrlib/builtins.py:5637
+#: bzrlib/builtins.py:5843
 msgid "Branch whose tags should be displayed."
 msgstr ""
 
 # help of 'sort' option of 'tags' command
-#: bzrlib/builtins.py:5639
+#: bzrlib/builtins.py:5845
 msgid "Sort tags by different criteria."
 msgstr ""
 
 # title of 'sort' option of 'tags' command
-#: bzrlib/builtins.py:5639
+#: bzrlib/builtins.py:5845
 msgid "Sorting"
 msgstr ""
 
-#: bzrlib/builtins.py:5684
+#: bzrlib/builtins.py:5914
 msgid "Reconfigure the type of a bzr directory."
 msgstr ""
 
-#: bzrlib/builtins.py:5686
+#: bzrlib/builtins.py:5916
 msgid "A target configuration must be specified."
 msgstr ""
 
-#: bzrlib/builtins.py:5688
+#: bzrlib/builtins.py:5918
 msgid ""
 "For checkouts, the bind-to location will be auto-detected if not specified.\n"
 "The order of preference is\n"
@@ -3042,43 +3822,113 @@
 "If none of these is available, --bind-to must be specified."
 msgstr ""
 
-# title of 'target_type' option of 'reconfigure' command
-#: bzrlib/builtins.py:5702
-msgid "Target type"
-msgstr ""
-
-# help of 'target_type' option of 'reconfigure' command
-#: bzrlib/builtins.py:5703
-msgid "The type to reconfigure the directory to."
+# title of 'tree_type' option of 'reconfigure' command
+#: bzrlib/builtins.py:5932
+msgid "Tree type"
+msgstr ""
+
+# help of 'tree_type' option of 'reconfigure' command
+#: bzrlib/builtins.py:5933
+msgid "The relation between branch and tree."
+msgstr ""
+
+# help of 'tree_type=branch' option of 'reconfigure' command
+#: bzrlib/builtins.py:5935
+msgid "Reconfigure to be an unbound branch with no working tree."
+msgstr ""
+
+# help of 'tree_type=tree' option of 'reconfigure' command
+#: bzrlib/builtins.py:5936
+msgid "Reconfigure to be an unbound branch with a working tree."
+msgstr ""
+
+# help of 'tree_type=checkout' option of 'reconfigure' command
+#: bzrlib/builtins.py:5937
+msgid "Reconfigure to be a bound branch with a working tree."
+msgstr ""
+
+# help of 'tree_type=lightweight-checkout' option of 'reconfigure' command
+#: bzrlib/builtins.py:5938
+msgid "Reconfigure to be a lightweight checkout (with no local history)."
+msgstr ""
+
+# title of 'repository_type' option of 'reconfigure' command
+#: bzrlib/builtins.py:5943
+msgid "Repository type"
+msgstr ""
+
+# help of 'repository_type' option of 'reconfigure' command
+#: bzrlib/builtins.py:5944
+msgid "Location fo the repository."
+msgstr ""
+
+# help of 'repository_type=standalone' option of 'reconfigure' command
+#: bzrlib/builtins.py:5946
+msgid ""
+"Reconfigure to be a standalone branch (i.e. stop using shared repository)."
+msgstr ""
+
+# help of 'repository_type=use-shared' option of 'reconfigure' command
+#: bzrlib/builtins.py:5948
+msgid "Reconfigure to use a shared repository."
+msgstr ""
+
+# title of 'repository_trees' option of 'reconfigure' command
+#: bzrlib/builtins.py:5952
+msgid "Trees in Repository"
+msgstr ""
+
+# help of 'repository_trees' option of 'reconfigure' command
+#: bzrlib/builtins.py:5953
+msgid "Whether new branches in the repository have trees."
+msgstr ""
+
+# help of 'repository_trees=with-trees' option of 'reconfigure' command
+#: bzrlib/builtins.py:5955
+msgid "Reconfigure repository to create working trees on branches by default."
+msgstr ""
+
+# help of 'repository_trees=with-no-trees' option of 'reconfigure' command
+#: bzrlib/builtins.py:5957
+msgid ""
+"Reconfigure repository to not create working trees on branches by default."
 msgstr ""
 
 # help of 'bind-to' option of 'reconfigure' command
-#: bzrlib/builtins.py:5718
+#: bzrlib/builtins.py:5960
 msgid "Branch to bind checkout to."
 msgstr ""
 
 # help of 'force' option of 'reconfigure' command
-#: bzrlib/builtins.py:5720
+#: bzrlib/builtins.py:5962
 msgid "Perform reconfiguration even if local changes will be lost."
 msgstr ""
 
 # help of 'stacked-on' option of 'reconfigure' command
-#: bzrlib/builtins.py:5723
+#: bzrlib/builtins.py:5965
 msgid "Reconfigure a branch to be stacked on another branch."
 msgstr ""
 
 # help of 'unstacked' option of 'reconfigure' command
-#: bzrlib/builtins.py:5727
+#: bzrlib/builtins.py:5969
 msgid ""
 "Reconfigure a branch to be unstacked.  This may require copying substantial "
 "data into it."
 msgstr ""
 
-#: bzrlib/builtins.py:5775
+#: bzrlib/builtins.py:5979
+msgid "Can't use both --stacked-on and --unstacked"
+msgstr ""
+
+#: bzrlib/builtins.py:5993
+msgid "No target configuration specified"
+msgstr ""
+
+#: bzrlib/builtins.py:6028
 msgid "Set the branch of a checkout and update."
 msgstr ""
 
-#: bzrlib/builtins.py:5777
+#: bzrlib/builtins.py:6030
 msgid ""
 "For lightweight checkouts, this changes the branch being referenced.\n"
 "For heavyweight checkouts, this checks that there are no local commits\n"
@@ -3086,17 +3936,17 @@
 "of the new location and binds to it."
 msgstr ""
 
-#: bzrlib/builtins.py:5782
+#: bzrlib/builtins.py:6035
 msgid ""
 "In both cases, the working tree is updated and uncommitted changes\n"
 "are merged. The user can commit or revert these as they desire."
 msgstr ""
 
-#: bzrlib/builtins.py:5785
+#: bzrlib/builtins.py:6038
 msgid "Pending merges need to be committed or reverted before using switch."
 msgstr ""
 
-#: bzrlib/builtins.py:5787
+#: bzrlib/builtins.py:6040
 msgid ""
 "The path to the branch to switch to can be specified relative to the parent\n"
 "directory of the current branch. For example, if you are currently in a\n"
@@ -3104,7 +3954,7 @@
 "/path/to/newbranch."
 msgstr ""
 
-#: bzrlib/builtins.py:5792
+#: bzrlib/builtins.py:6045
 msgid ""
 "Bound branches use the nickname of its master branch unless it is set\n"
 "locally, in which case switching will update the local nickname to be\n"
@@ -3112,20 +3962,28 @@
 msgstr ""
 
 # help of 'force' option of 'switch' command
-#: bzrlib/builtins.py:5800
+#: bzrlib/builtins.py:6053
 msgid "Switch even if local commits will be lost."
 msgstr ""
 
 # help of 'create-branch' option of 'switch' command
-#: bzrlib/builtins.py:5803
+#: bzrlib/builtins.py:6056
 msgid "Create the target branch from this one before switching to it."
 msgstr ""
 
-#: bzrlib/builtins.py:5872
+#: bzrlib/builtins.py:6068
+msgid "You must supply either a revision or a location"
+msgstr ""
+
+#: bzrlib/builtins.py:6079
+msgid "cannot create branch without source branch"
+msgstr ""
+
+#: bzrlib/builtins.py:6143
 msgid "Manage filtered views."
 msgstr ""
 
-#: bzrlib/builtins.py:5874
+#: bzrlib/builtins.py:6145
 msgid ""
 "Views provide a mask over the tree so that users can focus on\n"
 "a subset of a tree when doing their work. After creating a view,\n"
@@ -3135,7 +3993,7 @@
 "must be within the current view."
 msgstr ""
 
-#: bzrlib/builtins.py:5881
+#: bzrlib/builtins.py:6152
 msgid ""
 "In most cases, a view has a short life-span: it is created to make\n"
 "a selected change and is deleted once that change is committed.\n"
@@ -3143,7 +4001,7 @@
 "and switch between them."
 msgstr ""
 
-#: bzrlib/builtins.py:5886
+#: bzrlib/builtins.py:6157
 msgid ""
 "To disable the current view without deleting it, you can switch to\n"
 "the pseudo view called ``off``. This can be useful when you need\n"
@@ -3151,131 +4009,202 @@
 "want to switch back to your view after that."
 msgstr ""
 
-#: bzrlib/builtins.py:5891
+#: bzrlib/builtins.py:6162
 msgid ""
 ":Examples:\n"
 "  To define the current view::"
 msgstr ""
 
-#: bzrlib/builtins.py:5894
+#: bzrlib/builtins.py:6165
 msgid "    bzr view file1 dir1 ..."
 msgstr ""
 
-#: bzrlib/builtins.py:5896
+#: bzrlib/builtins.py:6167
 msgid "  To list the current view::"
 msgstr ""
 
-#: bzrlib/builtins.py:5898
+#: bzrlib/builtins.py:6169
 msgid "    bzr view"
 msgstr ""
 
-#: bzrlib/builtins.py:5900
+#: bzrlib/builtins.py:6171
 msgid "  To delete the current view::"
 msgstr ""
 
-#: bzrlib/builtins.py:5902
+#: bzrlib/builtins.py:6173
 msgid "    bzr view --delete"
 msgstr ""
 
-#: bzrlib/builtins.py:5904
+#: bzrlib/builtins.py:6175
 msgid "  To disable the current view without deleting it::"
 msgstr ""
 
-#: bzrlib/builtins.py:5906
+#: bzrlib/builtins.py:6177
 msgid "    bzr view --switch off"
 msgstr ""
 
-#: bzrlib/builtins.py:5908
+#: bzrlib/builtins.py:6179
 msgid "  To define a named view and switch to it::"
 msgstr ""
 
-#: bzrlib/builtins.py:5910
+#: bzrlib/builtins.py:6181
 msgid "    bzr view --name view-name file1 dir1 ..."
 msgstr ""
 
-#: bzrlib/builtins.py:5912
+#: bzrlib/builtins.py:6183
 msgid "  To list a named view::"
 msgstr ""
 
-#: bzrlib/builtins.py:5914
+#: bzrlib/builtins.py:6185
 msgid "    bzr view --name view-name"
 msgstr ""
 
-#: bzrlib/builtins.py:5916
+#: bzrlib/builtins.py:6187
 msgid "  To delete a named view::"
 msgstr ""
 
-#: bzrlib/builtins.py:5918
+#: bzrlib/builtins.py:6189
 msgid "    bzr view --name view-name --delete"
 msgstr ""
 
-#: bzrlib/builtins.py:5920
+#: bzrlib/builtins.py:6191
 msgid "  To switch to a named view::"
 msgstr ""
 
-#: bzrlib/builtins.py:5922
+#: bzrlib/builtins.py:6193
 msgid "    bzr view --switch view-name"
 msgstr ""
 
-#: bzrlib/builtins.py:5924
+#: bzrlib/builtins.py:6195
 msgid "  To list all views defined::"
 msgstr ""
 
-#: bzrlib/builtins.py:5926
+#: bzrlib/builtins.py:6197
 msgid "    bzr view --all"
 msgstr ""
 
-#: bzrlib/builtins.py:5928
+#: bzrlib/builtins.py:6199
 msgid "  To delete all views::"
 msgstr ""
 
-#: bzrlib/builtins.py:5930
+#: bzrlib/builtins.py:6201
 msgid "    bzr view --delete --all"
 msgstr ""
 
 # help of 'all' option of 'view' command
-#: bzrlib/builtins.py:5937
+#: bzrlib/builtins.py:6208
 msgid "Apply list or delete action to all views."
 msgstr ""
 
 # help of 'delete' option of 'view' command
-#: bzrlib/builtins.py:5940
+#: bzrlib/builtins.py:6211
 msgid "Delete the view."
 msgstr ""
 
 # help of 'name' option of 'view' command
-#: bzrlib/builtins.py:5943
+#: bzrlib/builtins.py:6214
 msgid "Name of the view to define, list or delete."
 msgstr ""
 
 # help of 'switch' option of 'view' command
-#: bzrlib/builtins.py:5947
+#: bzrlib/builtins.py:6218
 msgid "Name of the view to switch to."
 msgstr ""
 
-#: bzrlib/builtins.py:6047
+#: bzrlib/builtins.py:6237
+msgid "Both --delete and a file list specified"
+msgstr ""
+
+#: bzrlib/builtins.py:6240
+msgid "Both --delete and --switch specified"
+msgstr ""
+
+#: bzrlib/builtins.py:6243
+msgid "Deleted all views.\n"
+msgstr ""
+
+#: bzrlib/builtins.py:6245
+msgid "No current view to delete"
+msgstr ""
+
+#: bzrlib/builtins.py:6248
+#, python-format
+msgid "Deleted '%s' view.\n"
+msgstr ""
+
+#: bzrlib/builtins.py:6252
+msgid "Both --switch and a file list specified"
+msgstr ""
+
+#: bzrlib/builtins.py:6255
+msgid "Both --switch and --all specified"
+msgstr ""
+
+#: bzrlib/builtins.py:6258
+msgid "No current view to disable"
+msgstr ""
+
+#: bzrlib/builtins.py:6260
+#, python-format
+msgid "Disabled '%s' view.\n"
+msgstr ""
+
+#: bzrlib/builtins.py:6264 bzrlib/builtins.py:6286
+msgid "Using '{0}' view: {1}\n"
+msgstr ""
+
+#: bzrlib/builtins.py:6267
+msgid "Views defined:\n"
+msgstr ""
+
+#: bzrlib/builtins.py:6276
+msgid "No views defined.\n"
+msgstr ""
+
+#: bzrlib/builtins.py:6283
+msgid "Cannot change the 'off' pseudo view"
+msgstr ""
+
+#: bzrlib/builtins.py:6291
+msgid "No current view.\n"
+msgstr ""
+
+#: bzrlib/builtins.py:6294
+msgid "'{0}' view is: {1}\n"
+msgstr ""
+
+#: bzrlib/builtins.py:6314
+msgid "    <no hooks installed>\n"
+msgstr ""
+
+#: bzrlib/builtins.py:6318
 msgid "Remove a branch."
 msgstr ""
 
-#: bzrlib/builtins.py:6049
+#: bzrlib/builtins.py:6320
 msgid ""
 "This will remove the branch from the specified location but \n"
 "will keep any working tree or repository in place."
 msgstr ""
 
-#: bzrlib/builtins.py:6054
+#: bzrlib/builtins.py:6325
 msgid "  Remove the branch at repo/trunk::"
 msgstr ""
 
-#: bzrlib/builtins.py:6056
+#: bzrlib/builtins.py:6327
 msgid "    bzr remove-branch repo/trunk"
 msgstr ""
 
-#: bzrlib/builtins.py:6072
+# help of 'writer=plain' option of 'shelve' command
+#: bzrlib/builtins.py:6342
+msgid "Plaintext diff output."
+msgstr ""
+
+#: bzrlib/builtins.py:6343
 msgid "Temporarily set aside some changes from the current tree."
 msgstr ""
 
-#: bzrlib/builtins.py:6074
+#: bzrlib/builtins.py:6345
 msgid ""
 "Shelve allows you to temporarily put changes you've made \"on the shelf\",\n"
 "ie. out of the way, until a later time when you can bring them back from\n"
@@ -3284,11 +4213,11 @@
 "will they survive its deletion."
 msgstr ""
 
-#: bzrlib/builtins.py:6080
+#: bzrlib/builtins.py:6351
 msgid "If shelve --list is specified, previously-shelved changes are listed."
 msgstr ""
 
-#: bzrlib/builtins.py:6082
+#: bzrlib/builtins.py:6353
 msgid ""
 "Shelve is intended to help separate several sets of changes that have\n"
 "been inappropriately mingled.  If you just want to get rid of all changes\n"
@@ -3296,35 +4225,35 @@
 "shelve all text changes at once, use shelve --all."
 msgstr ""
 
-#: bzrlib/builtins.py:6087
+#: bzrlib/builtins.py:6358
 msgid ""
 "If filenames are specified, only the changes to those files will be\n"
 "shelved. Other files will be left untouched."
 msgstr ""
 
-#: bzrlib/builtins.py:6090
+#: bzrlib/builtins.py:6361
 msgid ""
 "If a revision is specified, changes since that revision will be shelved."
 msgstr ""
 
-#: bzrlib/builtins.py:6092
+#: bzrlib/builtins.py:6363
 msgid ""
 "You can put multiple items on the shelf, and by default, 'unshelve' will\n"
 "restore the most recently shelved changes."
 msgstr ""
 
-#: bzrlib/builtins.py:6095
+#: bzrlib/builtins.py:6366
 msgid ""
 "For complicated changes, it is possible to edit the changes in a separate\n"
 "editor program to decide what the file remaining in the working copy\n"
 "should look like.  To do this, add the configuration option"
 msgstr ""
 
-#: bzrlib/builtins.py:6099
+#: bzrlib/builtins.py:6370
 msgid "    change_editor = PROGRAM @new_path @old_path"
 msgstr ""
 
-#: bzrlib/builtins.py:6101
+#: bzrlib/builtins.py:6372
 msgid ""
 "where @new_path is replaced with the path of the new version of the \n"
 "file and @old_path is replaced with the path of the old version of \n"
@@ -3334,35 +4263,39 @@
 msgstr ""
 
 # help of 'all' option of 'shelve' command
-#: bzrlib/builtins.py:6113
+#: bzrlib/builtins.py:6384
 msgid "Shelve all changes."
 msgstr ""
 
 # help of 'writer' option of 'shelve' command
-#: bzrlib/builtins.py:6115
+#: bzrlib/builtins.py:6386
 msgid "Method to use for writing diffs."
 msgstr ""
 
 # title of 'writer' option of 'shelve' command
-#: bzrlib/builtins.py:6115
+#: bzrlib/builtins.py:6386
 msgid "writer"
 msgstr ""
 
 # help of 'list' option of 'shelve' command
-#: bzrlib/builtins.py:6119
+#: bzrlib/builtins.py:6390
 msgid "List shelved changes."
 msgstr ""
 
 # help of 'destroy' option of 'shelve' command
-#: bzrlib/builtins.py:6121
+#: bzrlib/builtins.py:6392
 msgid "Destroy removed changes instead of shelving them."
 msgstr ""
 
-#: bzrlib/builtins.py:6161
+#: bzrlib/builtins.py:6421
+msgid "No shelved changes."
+msgstr ""
+
+#: bzrlib/builtins.py:6432
 msgid "Restore shelved changes."
 msgstr ""
 
-#: bzrlib/builtins.py:6163
+#: bzrlib/builtins.py:6434
 msgid ""
 "By default, the most recently shelved changes are restored. However if you\n"
 "specify a shelf by id those changes will be restored instead.  This works\n"
@@ -3370,91 +4303,377 @@
 msgstr ""
 
 # help of 'action' option of 'unshelve' command
-#: bzrlib/builtins.py:6172
+#: bzrlib/builtins.py:6443
 msgid "The action to perform."
 msgstr ""
 
-#: bzrlib/builtins.py:6194
+# help of 'action=apply' option of 'unshelve' command
+#: bzrlib/builtins.py:6445
+msgid "Apply changes and remove from the shelf."
+msgstr ""
+
+# help of 'action=dry-run' option of 'unshelve' command
+#: bzrlib/builtins.py:6446
+msgid "Show changes, but do not apply or remove them."
+msgstr ""
+
+# help of 'action=preview' option of 'unshelve' command
+#: bzrlib/builtins.py:6447
+msgid ""
+"Instead of unshelving the changes, show the diff that would result from "
+"unshelving."
+msgstr ""
+
+# help of 'action=delete-only' option of 'unshelve' command
+#: bzrlib/builtins.py:6449
+msgid "Delete changes without applying them."
+msgstr ""
+
+# help of 'action=keep' option of 'unshelve' command
+#: bzrlib/builtins.py:6450
+msgid "Apply changes but don't delete them."
+msgstr ""
+
+#: bzrlib/builtins.py:6465
 msgid "Remove unwanted files from working tree."
 msgstr ""
 
-#: bzrlib/builtins.py:6196
+#: bzrlib/builtins.py:6467
 msgid ""
 "By default, only unknown files, not ignored files, are deleted.  Versioned\n"
 "files are never deleted."
 msgstr ""
 
-#: bzrlib/builtins.py:6199
+#: bzrlib/builtins.py:6470
 msgid ""
 "Another class is 'detritus', which includes files emitted by bzr during\n"
 "normal operations and selftests.  (The value of these files decreases with\n"
 "time.)"
 msgstr ""
 
-#: bzrlib/builtins.py:6203
+#: bzrlib/builtins.py:6474
 msgid ""
 "If no options are specified, unknown files are deleted.  Otherwise, option\n"
 "flags are respected, and may be combined."
 msgstr ""
 
-#: bzrlib/builtins.py:6206
+#: bzrlib/builtins.py:6477
 msgid "To check what clean-tree will do, use --dry-run."
 msgstr ""
 
 # help of 'ignored' option of 'clean-tree' command
-#: bzrlib/builtins.py:6209
+#: bzrlib/builtins.py:6480
 msgid "Delete all ignored files."
 msgstr ""
 
 # help of 'detritus' option of 'clean-tree' command
-#: bzrlib/builtins.py:6210
+#: bzrlib/builtins.py:6481
 msgid ""
 "Delete conflict files, merge and revert backups, and failed selftest dirs."
 msgstr ""
 
 # help of 'unknown' option of 'clean-tree' command
-#: bzrlib/builtins.py:6213
+#: bzrlib/builtins.py:6484
 msgid "Delete files unknown to bzr (default)."
 msgstr ""
 
 # help of 'dry-run' option of 'clean-tree' command
-#: bzrlib/builtins.py:6214
+#: bzrlib/builtins.py:6485
 msgid "Show files to delete instead of deleting them."
 msgstr ""
 
 # help of 'force' option of 'clean-tree' command
-#: bzrlib/builtins.py:6216
+#: bzrlib/builtins.py:6487
 msgid "Do not prompt before deleting."
 msgstr ""
 
-#: bzrlib/cmd_version_info.py:50
+#: bzrlib/bundle/__init__.py:57
+msgid "A directory cannot be a bundle"
+msgstr ""
+
+#: bzrlib/bundle/apply_bundle.py:37
+msgid "Install revisions"
+msgstr ""
+
+#: bzrlib/bundle/commands.py:61
+msgid "--verbose requires a merge directive"
+msgstr ""
+
+#: bzrlib/bundle/commands.py:64
+msgid "Bundle format not supported"
+msgstr ""
+
+#: bzrlib/bundle/commands.py:74
+msgid "Records\n"
+msgstr ""
+
+#: bzrlib/bundle/commands.py:78
+msgid "{0}: {1} ({2} multiparent)\n"
+msgstr ""
+
+#: bzrlib/bundle/commands.py:80
+#, python-format
+msgid "unique files: %d\n"
+msgstr ""
+
+#: bzrlib/bundle/commands.py:89
+msgid "Revisions\n"
+msgstr ""
+
+#: bzrlib/bundle/commands.py:90
+#, python-format
+msgid "nicks: %s\n"
+msgstr ""
+
+#: bzrlib/bundle/commands.py:92
+#, python-format
+msgid ""
+"committers: \n"
+"%s\n"
+msgstr ""
+
+#: bzrlib/bundle/commands.py:100
+msgid "Decoded contents\n"
+msgstr ""
+
+#: bzrlib/bundle/serializer/v4.py:325
+#, python-format
+msgid "Bundling %d revision."
+msgid_plural "Bundling %d revisions."
+msgstr[0] ""
+msgstr[1] ""
+
+#: bzrlib/bzrdir.py:487
+msgid ""
+"making backup of {0}\n"
+"  to {1}"
+msgstr ""
+
+#: bzrlib/bzrdir.py:510
+msgid "renamed {0} to {1}"
+msgstr ""
+
+#: bzrlib/bzrdir.py:1512
+#, python-format
+msgid ""
+"Source repository format does not support stacking, using format:\n"
+"  %s"
+msgstr ""
+
+#: bzrlib/bzrdir.py:1532
+#, python-format
+msgid ""
+"Source branch format does not support stacking, using format:\n"
+"  %s"
+msgstr ""
+
+#: bzrlib/bzrdir.py:1677
+msgid "starting repository conversion"
+msgstr ""
+
+#: bzrlib/bzrdir.py:1906
+msgid "Using default stacking branch {0} at {1}"
+msgstr ""
+
+#: bzrlib/check.py:106
+msgid "check"
+msgstr ""
+
+#: bzrlib/check.py:108
+msgid "checking revisions"
+msgstr ""
+
+#: bzrlib/check.py:110
+msgid "checking commit contents"
+msgstr ""
+
+#: bzrlib/check.py:112
+msgid "checking file graphs"
+msgstr ""
+
+#: bzrlib/check.py:116
+msgid "checking branches and trees"
+msgstr ""
+
+#: bzrlib/check.py:203
+msgid "checked repository {0} format {1}"
+msgstr ""
+
+#: bzrlib/check.py:206
+#, python-format
+msgid "%6d revisions"
+msgstr ""
+
+#: bzrlib/check.py:207
+#, python-format
+msgid "%6d file-ids"
+msgstr ""
+
+#: bzrlib/check.py:209
+#, python-format
+msgid "%6d unreferenced text versions"
+msgstr ""
+
+#: bzrlib/check.py:213
+msgid "unreferenced version: {{{0}}} in {1}"
+msgstr ""
+
+#: bzrlib/check.py:216
+#, python-format
+msgid "%6d revisions are missing inventory_sha1"
+msgstr ""
+
+#: bzrlib/check.py:219
+#, python-format
+msgid "%6d revisions are mentioned but not present"
+msgstr ""
+
+#: bzrlib/check.py:222
+#, python-format
+msgid "%6d ghost revisions"
+msgstr ""
+
+#: bzrlib/check.py:227
+#, python-format
+msgid "%6d revisions missing parents in ancestry"
+msgstr ""
+
+#: bzrlib/check.py:231
+#, python-format
+msgid "      %s should be in the ancestry for:"
+msgstr ""
+
+#: bzrlib/check.py:235
+#, python-format
+msgid "%6d inconsistent parents"
+msgstr ""
+
+#: bzrlib/check.py:239
+msgid "      * {0} version {1} has parents {2!r} but should have {3!r}"
+msgstr ""
+
+#: bzrlib/check.py:245
+#, python-format
+msgid "%6d revisions have incorrect parents in the revision index"
+msgstr ""
+
+#: bzrlib/check.py:251
+msgid "       {0} has wrong parents in index: {1!r} should be {2!r}"
+msgstr ""
+
+#: bzrlib/check.py:265
+msgid "Mismatched internal revid {{{0}}} and index revid {{{1}}}"
+msgstr ""
+
+#: bzrlib/check.py:294
+msgid ""
+"Multiple expected sha1s for {0}. {{{1}}} expects {{{2}}}, {{{3}}} expects "
+"{{{4}}}"
+msgstr ""
+
+#: bzrlib/check.py:352
+#, python-format
+msgid "Checking branch at '%s'."
+msgstr ""
+
+#: bzrlib/check.py:372
+#, python-format
+msgid "Checking working tree at '%s'."
+msgstr ""
+
+#: bzrlib/check.py:425 bzrlib/check.py:439
+msgid "No branch found at specified location."
+msgstr ""
+
+#: bzrlib/check.py:427 bzrlib/check.py:437
+msgid "No working tree found at specified location."
+msgstr ""
+
+#: bzrlib/check.py:430
+#, python-format
+msgid "Checking repository at '%s'."
+msgstr ""
+
+#: bzrlib/check.py:441
+msgid "No repository found at specified location."
+msgstr ""
+
+#: bzrlib/clean_tree.py:62
+msgid "Nothing to delete."
+msgstr ""
+
+#: bzrlib/clean_tree.py:67
+msgid "Are you sure you wish to delete these"
+msgstr ""
+
+#: bzrlib/clean_tree.py:69
+msgid "Canceled"
+msgstr ""
+
+#: bzrlib/clean_tree.py:106
+#, python-format
+msgid "unable to remove %s"
+msgstr ""
+
+#: bzrlib/clean_tree.py:110
+msgid "deleting paths:"
+msgstr ""
+
+#: bzrlib/clean_tree.py:124
+msgid "unable to remove \"{0}\": {1}."
+msgstr ""
+
+#: bzrlib/clean_tree.py:129
+msgid "No files deleted."
+msgstr ""
+
+#: bzrlib/cmd_version_info.py:45
+msgid "No known version info format {0}. Supported types are: {1}"
+msgstr ""
+
+# help of 'format=custom' option of 'version-info' command
+#: bzrlib/cmd_version_info.py:50
+msgid "Version info in Custom template-based format."
+msgstr ""
+
+# help of 'format=python' option of 'version-info' command
+#: bzrlib/cmd_version_info.py:50
+msgid "Version info in Python format."
+msgstr ""
+
+# help of 'format=rio' option of 'version-info' command
+#: bzrlib/cmd_version_info.py:50
+msgid "Version info in RIO (simple text) format (default)."
+msgstr ""
+
+#: bzrlib/cmd_version_info.py:51
 msgid "Show version information about this tree."
 msgstr ""
 
-#: bzrlib/cmd_version_info.py:52
+#: bzrlib/cmd_version_info.py:53
 msgid ""
 "You can use this command to add information about version into\n"
 "source code of an application. The output can be in one of the\n"
 "supported formats or in a custom format based on a template."
 msgstr ""
 
-#: bzrlib/cmd_version_info.py:56
+#: bzrlib/cmd_version_info.py:57
 msgid "For example::"
 msgstr ""
 
-#: bzrlib/cmd_version_info.py:58
+#: bzrlib/cmd_version_info.py:59
 msgid ""
 "  bzr version-info --custom \\\n"
 "    --template=\"#define VERSION_INFO \\\"Project 1.2.3 (r{revno})\\\"\\n\""
 msgstr ""
 
-#: bzrlib/cmd_version_info.py:61
+#: bzrlib/cmd_version_info.py:62
 msgid ""
 "will produce a C header file with formatted string containing the\n"
 "current revision number. Other supported variables in templates are:"
 msgstr ""
 
-#: bzrlib/cmd_version_info.py:64
+#: bzrlib/cmd_version_info.py:65
 msgid ""
 "  * {date} - date of the last revision\n"
 "  * {build_date} - current date\n"
@@ -3466,35 +4685,44 @@
 msgstr ""
 
 # help of 'format' option of 'version-info' command
-#: bzrlib/cmd_version_info.py:74
+#: bzrlib/cmd_version_info.py:75
 msgid "Select the output format."
 msgstr ""
 
 # help of 'all' option of 'version-info' command
-#: bzrlib/cmd_version_info.py:78
+#: bzrlib/cmd_version_info.py:79
 msgid "Include all possible information."
 msgstr ""
 
 # help of 'check-clean' option of 'version-info' command
-#: bzrlib/cmd_version_info.py:79
+#: bzrlib/cmd_version_info.py:80
 msgid "Check if tree is clean."
 msgstr ""
 
 # help of 'include-history' option of 'version-info' command
-#: bzrlib/cmd_version_info.py:81
+#: bzrlib/cmd_version_info.py:82
 msgid "Include the revision-history."
 msgstr ""
 
 # help of 'include-file-revisions' option of 'version-info' command
-#: bzrlib/cmd_version_info.py:83
+#: bzrlib/cmd_version_info.py:84
 msgid "Include the last revision for each file."
 msgstr ""
 
 # help of 'template' option of 'version-info' command
-#: bzrlib/cmd_version_info.py:84
+#: bzrlib/cmd_version_info.py:85
 msgid "Template for the output."
 msgstr ""
 
+#: bzrlib/cmd_version_info.py:99
+msgid "bzr version-info --revision takes exactly one revision specifier"
+msgstr ""
+
+#: bzrlib/commands.py:232
+#, python-format
+msgid "unknown command \"%s\""
+msgstr ""
+
 #: bzrlib/commands.py:479
 msgid "No help for this command."
 msgstr ""
@@ -3546,15 +4774,86 @@
 msgstr ""
 
 #: bzrlib/commands.py:557
-#, python-format
-msgid ":doc:`%s <%s-help>`"
+msgid ":doc:`{0} <{1}-help>`"
 msgstr ""
 
-#: bzrlib/commands.py:560
+#: bzrlib/commands.py:561
 #, python-format
 msgid ":See also: %s"
 msgstr ""
 
+#: bzrlib/commands.py:819 bzrlib/gpg.py:317
+msgid "Only ASCII permitted in option names"
+msgstr ""
+
+#: bzrlib/commands.py:844 bzrlib/commands.py:852
+msgid "command {0!r} needs one or more {1}"
+msgstr ""
+
+#: bzrlib/commands.py:861
+msgid "command {0!r} requires argument {1}"
+msgstr ""
+
+#: bzrlib/commands.py:868
+msgid "extra argument to command {0}: {1}"
+msgstr ""
+
+#: bzrlib/commands.py:965
+#, python-format
+msgid "Profile data written to \"%s\"."
+msgstr ""
+
+#: bzrlib/commit.py:116 bzrlib/commit.py:738
+msgid "added"
+msgstr ""
+
+#: bzrlib/commit.py:116 bzrlib/commit.py:746
+msgid "modified"
+msgstr ""
+
+#: bzrlib/commit.py:130
+#, python-format
+msgid "Committing%s"
+msgstr ""
+
+#: bzrlib/commit.py:133
+#, python-format
+msgid "Committed revision %d."
+msgstr ""
+
+#: bzrlib/commit.py:142
+#, python-format
+msgid "deleted %s"
+msgstr ""
+
+#: bzrlib/commit.py:145
+#, python-format
+msgid "missing %s"
+msgstr ""
+
+#: bzrlib/commit.py:510
+msgid ""
+"Conflicting tags in bound branch:\n"
+"{0}"
+msgstr ""
+
+#: bzrlib/commit.py:740
+msgid "renamed"
+msgstr ""
+
+#: bzrlib/commit.py:975
+msgid "unchanged"
+msgstr ""
+
+#: bzrlib/commit.py:997
+msgid "{0} [{1}] - Stage"
+msgstr ""
+
+#: bzrlib/commit.py:1000
+#, python-format
+msgid "%s - Stage"
+msgstr ""
+
 #: bzrlib/commit_signature_commands.py:34
 msgid "Sign all commits by a given committer."
 msgstr ""
@@ -3590,30 +4889,43 @@
 "verification."
 msgstr ""
 
+#: bzrlib/commit_signature_commands.py:145
+msgid "Cannot verify a range of non-revision-history revisions"
+msgstr ""
+
 #: bzrlib/commit_signature_commands.py:166
 msgid "All commits signed with verifiable keys"
 msgstr ""
 
-#: bzrlib/config.py:3118
+#: bzrlib/config.py:451
+msgid "Invalid config value for \"{0}\"  value {1!r} is not an SI unit."
+msgstr ""
+
+#: bzrlib/config.py:2762
+#, python-format
+msgid "Invalid '%s', should be of the form 'name=value'"
+msgstr ""
+
+#: bzrlib/config.py:3469
 msgid "Display, set or remove a configuration option."
 msgstr ""
 
-#: bzrlib/config.py:3120
+#: bzrlib/config.py:3471
 msgid "Display the active value for a given option."
 msgstr ""
 
-#: bzrlib/config.py:3122
+#: bzrlib/config.py:3473
 msgid ""
 "If --all is specified, NAME is interpreted as a regular expression and all\n"
 "matching options are displayed mentioning their scope. The active value\n"
 "that bzr will take into account is the first one displayed for each option."
 msgstr ""
 
-#: bzrlib/config.py:3126
+#: bzrlib/config.py:3477
 msgid "If no NAME is given, --all .* is implied."
 msgstr ""
 
-#: bzrlib/config.py:3128
+#: bzrlib/config.py:3479
 msgid ""
 "Setting a value is achieved by using name=value without spaces. The value\n"
 "is set in the most relevant scope and can be checked by displaying the\n"
@@ -3621,25 +4933,25 @@
 msgstr ""
 
 # help of 'scope' option of 'config' command
-#: bzrlib/config.py:3139
-msgid "Reduce the scope to the specified configuration file"
+#: bzrlib/config.py:3491
+msgid "Reduce the scope to the specified configuration file."
 msgstr ""
 
 # help of 'all' option of 'config' command
-#: bzrlib/config.py:3143
+#: bzrlib/config.py:3495
 msgid "Display all the defined values for the matching options."
 msgstr ""
 
 # help of 'remove' option of 'config' command
-#: bzrlib/config.py:3145
-msgid "Remove the option from the configuration file"
+#: bzrlib/config.py:3497
+msgid "Remove the option from the configuration file."
 msgstr ""
 
-#: bzrlib/conflicts.py:47
+#: bzrlib/conflicts.py:48
 msgid "List files with conflicts."
 msgstr ""
 
-#: bzrlib/conflicts.py:49
+#: bzrlib/conflicts.py:50
 msgid ""
 "Merge will do its best to combine the changes in two branches, but there\n"
 "are some kinds of problems only a human can fix.  When it encounters those,\n"
@@ -3647,32 +4959,47 @@
 "before you should commit."
 msgstr ""
 
-#: bzrlib/conflicts.py:54
+#: bzrlib/conflicts.py:55
 msgid ""
 "Conflicts normally are listed as short, human-readable messages.  If --text\n"
 "is supplied, the pathnames of files with text conflicts are listed,\n"
 "instead.  (This is useful for editing all files with text conflicts.)"
 msgstr ""
 
-#: bzrlib/conflicts.py:58
+#: bzrlib/conflicts.py:59
 msgid "Use bzr resolve when you have fixed a problem."
 msgstr ""
 
 # help of 'text' option of 'conflicts' command
-#: bzrlib/conflicts.py:63
+#: bzrlib/conflicts.py:64
 msgid "List paths of files with text conflicts."
 msgstr ""
 
+# help of 'action=done' option of 'resolve' command
+#: bzrlib/conflicts.py:83
+msgid "Marks the conflict as resolved."
+msgstr ""
+
+# help of 'action=take-this' option of 'resolve' command
+#: bzrlib/conflicts.py:86
+msgid "Resolve the conflict preserving the version in the working tree."
+msgstr ""
+
+# help of 'action=take-other' option of 'resolve' command
+#: bzrlib/conflicts.py:89
+msgid "Resolve the conflict taking the merged version into account."
+msgstr ""
+
 # help of 'action' option of 'resolve' command
-#: bzrlib/conflicts.py:95
+#: bzrlib/conflicts.py:96
 msgid "How to resolve the conflict."
 msgstr ""
 
-#: bzrlib/conflicts.py:101
+#: bzrlib/conflicts.py:102
 msgid "Mark a conflict as resolved."
 msgstr ""
 
-#: bzrlib/conflicts.py:108
+#: bzrlib/conflicts.py:109
 msgid ""
 "Once you have fixed a problem, use \"bzr resolve\" to automatically mark\n"
 "text conflicts as fixed, \"bzr resolve FILE\" to mark a specific conflict "
@@ -3681,253 +5008,283 @@
 msgstr ""
 
 # help of 'all' option of 'resolve' command
-#: bzrlib/conflicts.py:116
+#: bzrlib/conflicts.py:117
 msgid "Resolve all conflicts in this tree."
 msgstr ""
 
+#: bzrlib/conflicts.py:124
+msgid "If --all is specified, no FILE may be provided"
+msgstr ""
+
+#: bzrlib/conflicts.py:149
+#, python-format
+msgid "%d conflict auto-resolved."
+msgid_plural "%d conflicts auto-resolved."
+msgstr[0] ""
+msgstr[1] ""
+
+#: bzrlib/conflicts.py:152
+msgid "Remaining conflicts:"
+msgstr ""
+
+#: bzrlib/conflicts.py:157
+msgid "All conflicts resolved."
+msgstr ""
+
+#: bzrlib/conflicts.py:166
+msgid "{0} conflict resolved, {1} remaining"
+msgid_plural "{0} conflicts resolved, {1} remaining"
+msgstr[0] ""
+msgstr[1] ""
+
 # title of 'action' option of 'resolve' command
-#: bzrlib/conflicts.py:688
+#: bzrlib/conflicts.py:692
 msgid "action"
 msgstr ""
 
-#: bzrlib/errors.py:208
+#: bzrlib/controldir.py:682
+msgid "{0} is{1} redirected to {2}"
+msgstr ""
+
+#: bzrlib/diff.py:397
+#, python-format
+msgid "*** Ignoring files outside view. View is %s"
+msgstr ""
+
+#: bzrlib/errors.py:155
 msgid "The tree builder is already building a tree."
 msgstr ""
 
-#: bzrlib/errors.py:229
+#: bzrlib/errors.py:176
 msgid "The dirstate file (%(state)s) appears to be corrupt: %(msg)s"
 msgstr ""
 
-#: bzrlib/errors.py:248
+#: bzrlib/errors.py:195
 msgid ""
 "The API for \"%(api)s\" is not compatible with \"%(wanted)s\". It supports "
 "versions \"%(minimum)s\" to \"%(current)s\"."
 msgstr ""
 
-#: bzrlib/errors.py:260
+#: bzrlib/errors.py:207
 msgid "The transport '%(transport)s' is only accessible within this process."
 msgstr ""
 
-#: bzrlib/errors.py:278
+#: bzrlib/errors.py:225
 msgid "Invalid revision number %(revno)s"
 msgstr ""
 
-#: bzrlib/errors.py:287
+#: bzrlib/errors.py:234
 msgid "Invalid revision-id {%(revision_id)s} in %(branch)s"
 msgstr ""
 
-#: bzrlib/errors.py:298
+#: bzrlib/errors.py:245
 msgid "Reserved revision-id {%(revision_id)s}"
 msgstr ""
 
-#: bzrlib/errors.py:312
+#: bzrlib/errors.py:259
 msgid "There is no public branch set for \"%(branch_url)s\"."
 msgstr ""
 
-#: bzrlib/errors.py:322
+#: bzrlib/errors.py:269
 msgid ""
 "No help could be found for '%(topic)s'. Please use 'bzr help topics' to "
 "obtain a list of topics."
 msgstr ""
 
-#: bzrlib/errors.py:331
+#: bzrlib/errors.py:278
 msgid "The file id \"%(file_id)s\" is not present in the tree %(tree)s."
 msgstr ""
 
-#: bzrlib/errors.py:341
+#: bzrlib/errors.py:288
 msgid ""
 "The file id \"%(file_id)s\" is not present in the repository %(repository)r"
 msgstr ""
 
-#: bzrlib/errors.py:350
+#: bzrlib/errors.py:297
 msgid "The branch '%(branch)s' is not stacked."
 msgstr ""
 
-#: bzrlib/errors.py:364
+#: bzrlib/errors.py:311
 msgid "No WorkingTree exists for \"%(base)s\"."
 msgstr ""
 
-#: bzrlib/errors.py:373
+#: bzrlib/errors.py:320
 msgid "Not currently building a tree."
 msgstr ""
 
-#: bzrlib/errors.py:378
+#: bzrlib/errors.py:325
 msgid "%(url)s is not a local path."
 msgstr ""
 
-#: bzrlib/errors.py:406
+#: bzrlib/errors.py:353
 msgid "%(not_locked)r is not write locked but needs to be."
 msgstr ""
 
-#: bzrlib/errors.py:414
+#: bzrlib/errors.py:361
 msgid "Error in command line options"
 msgstr ""
 
-#: bzrlib/errors.py:419
+#: bzrlib/errors.py:366
 msgid "%(value)s is not an index of type %(_type)s."
 msgstr ""
 
-#: bzrlib/errors.py:429
+#: bzrlib/errors.py:376
 msgid "Error in data for index %(value)s."
 msgstr ""
 
-#: bzrlib/errors.py:438
+#: bzrlib/errors.py:385
 msgid "The key '%(key)s' is already in index '%(index)s'."
 msgstr ""
 
-#: bzrlib/errors.py:448
+#: bzrlib/errors.py:395
 msgid "The key '%(key)s' is not a valid key."
 msgstr ""
 
-#: bzrlib/errors.py:457
+#: bzrlib/errors.py:404
 msgid "Could not parse options for index %(value)s."
 msgstr ""
 
-#: bzrlib/errors.py:466
+#: bzrlib/errors.py:413
 msgid "The value '%(value)s' is not a valid value."
 msgstr ""
 
-#: bzrlib/errors.py:494
+#: bzrlib/errors.py:441
 msgid "Generic path error: %(path)r%(extra)s)"
 msgstr ""
 
-#: bzrlib/errors.py:507
+#: bzrlib/errors.py:454
 msgid "No such file: %(path)r%(extra)s"
 msgstr ""
 
-#: bzrlib/errors.py:512
+#: bzrlib/errors.py:459
 msgid "File exists: %(path)r%(extra)s"
 msgstr ""
 
-#: bzrlib/errors.py:518
+#: bzrlib/errors.py:465
 msgid ""
 "Could not rename %(source)s => %(dest)s because both files exist. (Use --"
 "after to tell bzr about a rename that has already happened)%(extra)s"
 msgstr ""
 
-#: bzrlib/errors.py:534
+#: bzrlib/errors.py:481
 msgid "\"%(path)s\" is not a directory %(extra)s"
 msgstr ""
 
-#: bzrlib/errors.py:539
+#: bzrlib/errors.py:486
 msgid "\"%(path)s\" is not in the working directory %(extra)s"
 msgstr ""
 
-#: bzrlib/errors.py:544
+#: bzrlib/errors.py:491
 msgid "Directory not empty: \"%(path)s\"%(extra)s"
 msgstr ""
 
-#: bzrlib/errors.py:549
+#: bzrlib/errors.py:496
 msgid "Hard-linking \"%(path)s\" is not supported"
 msgstr ""
 
-#: bzrlib/errors.py:564
+#: bzrlib/errors.py:511
 msgid "Device or resource busy: \"%(path)s\"%(extra)s"
 msgstr ""
 
-#: bzrlib/errors.py:569
+#: bzrlib/errors.py:516
 msgid "Permission denied: \"%(path)s\"%(extra)s"
 msgstr ""
 
-#: bzrlib/errors.py:574
+#: bzrlib/errors.py:521
 msgid "Invalid url supplied to transport: \"%(path)s\"%(extra)s"
 msgstr ""
 
-#: bzrlib/errors.py:579
+#: bzrlib/errors.py:526
 msgid "Invalid URL join request: %(reason)s: %(base)r + %(join_args)r"
 msgstr ""
 
-#: bzrlib/errors.py:590
+#: bzrlib/errors.py:537
 msgid "URLs differ by more than path: %(from_)r and %(to)r"
 msgstr ""
 
-#: bzrlib/errors.py:612
+#: bzrlib/errors.py:559
 msgid "The %(type)s hook '%(hook)s' is unknown in this version of bzrlib."
 msgstr ""
 
-#: bzrlib/errors.py:622
+#: bzrlib/errors.py:569
 msgid "Unsupported protocol for url \"%(path)s\"%(extra)s"
 msgstr ""
 
-#: bzrlib/errors.py:630
+#: bzrlib/errors.py:577
 msgid ""
 "The branch '%(url)s'(%(format)s) is not a stackable format. You will need to "
 "upgrade the branch to permit branch stacking."
 msgstr ""
 
-#: bzrlib/errors.py:641
+#: bzrlib/errors.py:588
 msgid "The branch '%(branch_url)s' cannot be stacked on '%(target_url)s'."
 msgstr ""
 
-#: bzrlib/errors.py:651
+#: bzrlib/errors.py:598
 msgid ""
 "The repository '%(url)s'(%(format)s) is not a stackable format. You will "
 "need to upgrade the repository to permit branch stacking."
 msgstr ""
 
-#: bzrlib/errors.py:662
+#: bzrlib/errors.py:609
 msgid "Error reading from %(path)r."
 msgstr ""
 
-#: bzrlib/errors.py:681
+#: bzrlib/errors.py:628
 msgid "Path \"%(path)s\" is not a child of path \"%(base)s\"%(extra)s"
 msgstr ""
 
-#: bzrlib/errors.py:697
+#: bzrlib/errors.py:644
 msgid "Path \"%(path)s\" is not unicode normalized"
 msgstr ""
 
-#: bzrlib/errors.py:705
+#: bzrlib/errors.py:652
 msgid "Not a branch: \"%(path)s\"%(detail)s."
 msgstr ""
 
-#: bzrlib/errors.py:747
+#: bzrlib/errors.py:694
 msgid "No submit branch available for branch \"%(path)s\""
 msgstr ""
 
-#: bzrlib/errors.py:756
+#: bzrlib/errors.py:703
 msgid "Already a branch: \"%(path)s\"."
 msgstr ""
 
-#: bzrlib/errors.py:762
+#: bzrlib/errors.py:709
 msgid ""
 "Directory contains a branch, but no working tree (use bzr checkout if you "
 "wish to build a working tree): \"%(path)s\""
 msgstr ""
 
-#: bzrlib/errors.py:767
+#: bzrlib/errors.py:714
 msgid ""
 "\"%(function)s\" called on an AtomicFile after it was closed: \"%(path)s\""
 msgstr ""
 
-#: bzrlib/errors.py:777
+#: bzrlib/errors.py:724
 msgid ""
 "Parent not accessible given base \"%(base)s\" and relative path \"%(path)s\""
 msgstr ""
 
-#: bzrlib/errors.py:787
+#: bzrlib/errors.py:734
 msgid "No repository present: \"%(path)s\""
 msgstr ""
 
-#: bzrlib/errors.py:795
-msgid "File \"%(path)s\" is not in branch %(branch_base)s."
-msgstr ""
-
-#: bzrlib/errors.py:807
+#: bzrlib/errors.py:741
 msgid ""
 "Unsupported branch format: %(format)s\n"
 "Please run 'bzr upgrade'"
 msgstr ""
 
-#: bzrlib/errors.py:813
+#: bzrlib/errors.py:747
 msgid "Unknown %(kind)s format: %(format)r"
 msgstr ""
 
-#: bzrlib/errors.py:822
+#: bzrlib/errors.py:756
 msgid "Format %(format)s is not compatible with .bzr version %(bzrdir)s."
 msgstr ""
 
-#: bzrlib/errors.py:835
+#: bzrlib/errors.py:769
 msgid ""
 "%(target)s\n"
 "is not compatible with\n"
@@ -3935,452 +5292,441 @@
 "%(details)s"
 msgstr ""
 
-#: bzrlib/errors.py:851
+#: bzrlib/errors.py:785
 msgid "Revision is not compatible with %(repo_format)s"
 msgstr ""
 
-#: bzrlib/errors.py:861
+#: bzrlib/errors.py:795
 msgid "%(context_info)s%(path)s is already versioned."
 msgstr ""
 
-#: bzrlib/errors.py:882
+#: bzrlib/errors.py:816
 msgid "%(context_info)s%(path)s is not versioned."
 msgstr ""
 
-#: bzrlib/errors.py:903
+#: bzrlib/errors.py:837
 msgid "Path(s) are not versioned: %(paths_as_string)s"
 msgstr ""
 
-#: bzrlib/errors.py:914
+#: bzrlib/errors.py:848
 msgid "Path(s) do not exist: %(paths_as_string)s%(extra)s"
 msgstr ""
 
-#: bzrlib/errors.py:933
+#: bzrlib/errors.py:867
 msgid "Cannot operate on \"%(filename)s\" of unsupported kind \"%(kind)s\""
 msgstr ""
 
-#: bzrlib/errors.py:941
+#: bzrlib/errors.py:875
 msgid ""
 "Filename %(filename)r is not valid in your current filesystem encoding "
 "%(fs_encoding)s"
 msgstr ""
 
-#: bzrlib/errors.py:952
+#: bzrlib/errors.py:886
 msgid "Cannot operate on \"%(filename)s\" because it is a control file"
 msgstr ""
 
-#: bzrlib/errors.py:970
+#: bzrlib/errors.py:904
 msgid "The lock for '%(lock_description)s' is in use and cannot be broken."
 msgstr ""
 
-#: bzrlib/errors.py:1009
+#: bzrlib/errors.py:943
 msgid "Cannot lock %(lock)s: %(why)s"
 msgstr ""
 
-#: bzrlib/errors.py:1019
+#: bzrlib/errors.py:953
 msgid ""
 "A transaction related operation was attempted after the transaction finished."
 msgstr ""
 
-#: bzrlib/errors.py:1046
+#: bzrlib/errors.py:980
 msgid "Cannot lock: transport is read only: %(transport)s"
 msgstr ""
 
-#: bzrlib/errors.py:1054
+#: bzrlib/errors.py:988
 msgid "Could not acquire lock \"%(lock)s\": %(msg)s"
 msgstr ""
 
-#: bzrlib/errors.py:1065
+#: bzrlib/errors.py:999
 msgid "Lock was broken while still open: %(lock)s - check storage consistency!"
 msgstr ""
 
-#: bzrlib/errors.py:1076
+#: bzrlib/errors.py:1010
 msgid ""
 "Lock was released and re-acquired before being broken: %(lock)s: held by "
 "%(holder)r, wanted to break %(target)r"
 msgstr ""
 
-#: bzrlib/errors.py:1088
+#: bzrlib/errors.py:1022
 msgid ""
 "Lock is apparently held, but corrupted: %(corruption_info)s\n"
 "Use 'bzr break-lock' to clear it"
 msgstr ""
 
-#: bzrlib/errors.py:1101
+#: bzrlib/errors.py:1035
 msgid "Lock not held: %(lock)s"
 msgstr ""
 
-#: bzrlib/errors.py:1130
+#: bzrlib/errors.py:1064
 msgid "No changes to commit"
 msgstr ""
 
-#: bzrlib/errors.py:1135
+#: bzrlib/errors.py:1069
 msgid ""
 "Selected-file commit of merges is not supported yet: files %(files_str)s"
 msgstr ""
 
-#: bzrlib/errors.py:1145
+#: bzrlib/errors.py:1079
 msgid ""
 "Excluding paths during commit is not supported by repository at %(repository)"
 "r."
 msgstr ""
 
-#: bzrlib/errors.py:1154
+#: bzrlib/errors.py:1088
 msgid ""
 "The specified commit message contains characters unsupported by the current "
 "encoding."
 msgstr ""
 
-#: bzrlib/errors.py:1160
+#: bzrlib/errors.py:1094
 msgid "Upgrade URL cannot work with readonly URLs."
 msgstr ""
 
-#: bzrlib/errors.py:1165
+#: bzrlib/errors.py:1099
 msgid "The branch format %(format)s is already at the most recent format."
 msgstr ""
 
-#: bzrlib/errors.py:1188
+#: bzrlib/errors.py:1122
 msgid "Option --change does not accept revision ranges"
 msgstr ""
 
-#: bzrlib/errors.py:1193
+#: bzrlib/errors.py:1127
 msgid "No namespace registered for string: %(spec)r"
 msgstr ""
 
-#: bzrlib/errors.py:1212
+#: bzrlib/errors.py:1146
 msgid ""
 "Requested revision: '%(spec)s' does not exist in branch: %(branch_url)s"
 "%(extra)s"
 msgstr ""
 
-#: bzrlib/errors.py:1226
-msgid "%(branch)s is missing %(object_type)s {%(object_id)s}"
-msgstr ""
-
-#: bzrlib/errors.py:1231
+#: bzrlib/errors.py:1160
 msgid ""
 "Operation denied because it would change the main history, which is not "
 "permitted by the append_revisions_only setting on branch \"%(location)s\"."
 msgstr ""
 
-#: bzrlib/errors.py:1242
+#: bzrlib/errors.py:1171
 msgid ""
 "These branches have diverged. Use the missing command to see how.\n"
 "Use the merge command to reconcile them."
 msgstr ""
 
-#: bzrlib/errors.py:1262
+#: bzrlib/errors.py:1191
 msgid ""
 "Branches have no common ancestor, and no merge base revision was specified."
 msgstr ""
 
-#: bzrlib/errors.py:1268
+#: bzrlib/errors.py:1197
 msgid ""
 "Selected merge cannot perform reverse cherrypicks.  Try merge3 or diff3."
 msgstr ""
 
-#: bzrlib/errors.py:1274
+#: bzrlib/errors.py:1203
 msgid "Revisions have no common ancestor: %(revision_a)s %(revision_b)s"
 msgstr ""
 
-#: bzrlib/errors.py:1283
+#: bzrlib/errors.py:1212
 msgid ""
 "Revisions are not derived from the same root: %(revision_a)s %(revision_b)s."
 msgstr ""
 
-#: bzrlib/errors.py:1292
+#: bzrlib/errors.py:1221
 msgid "Revision %(rev_id)s is not an ancestor of %(not_ancestor_id)s"
 msgstr ""
 
-#: bzrlib/errors.py:1312
+#: bzrlib/errors.py:1230
 msgid "Branch %(branch)s has no commits."
 msgstr ""
 
-#: bzrlib/errors.py:1330
+#: bzrlib/errors.py:1248
 msgid ""
 "Bound branch %(branch)s is out of date with master branch %(master)s."
 "%(extra_help)s"
 msgstr ""
 
-#: bzrlib/errors.py:1342
+#: bzrlib/errors.py:1260
 msgid ""
 "Cannot commit to branch %(branch)s. It is bound to %(master)s, which is "
 "bound to %(remote)s."
 msgstr ""
 
-#: bzrlib/errors.py:1354
+#: bzrlib/errors.py:1272
 msgid "Cannot pull --overwrite to a branch which is bound %(branch)s"
 msgstr ""
 
-#: bzrlib/errors.py:1363
+#: bzrlib/errors.py:1281
 msgid ""
 "Unable to connect to target of bound branch %(branch)s => %(target)s: "
 "%(error)s"
 msgstr ""
 
-#: bzrlib/errors.py:1375
+#: bzrlib/errors.py:1293
 msgid "Error in processing weave: %(msg)s"
 msgstr ""
 
-#: bzrlib/errors.py:1384
+#: bzrlib/errors.py:1302
 msgid "Revision {%(revision_id)s} already present in %(weave)s"
 msgstr ""
 
-#: bzrlib/errors.py:1395
+#: bzrlib/errors.py:1313
 msgid "Revision {%(revision_id)s} not present in %(weave)s"
 msgstr ""
 
-#: bzrlib/errors.py:1405
+#: bzrlib/errors.py:1323
 msgid "Weave invariant violated: %(what)s"
 msgstr ""
 
-#: bzrlib/errors.py:1414
+#: bzrlib/errors.py:1332
 msgid "Parents are mismatched between two revisions. %(msg)s"
 msgstr ""
 
-#: bzrlib/errors.py:1436
+#: bzrlib/errors.py:1354
 msgid ""
 "Weaves differ on text content. Revision: {%(revision_id)s}, %(weave_a)s, "
 "%(weave_b)s"
 msgstr ""
 
-#: bzrlib/errors.py:1448
+#: bzrlib/errors.py:1366
 msgid "Versioned file error"
 msgstr ""
 
-#: bzrlib/errors.py:1453
+#: bzrlib/errors.py:1371
 msgid "Revision {%(revision_id)s} not present in \"%(file_id)s\"."
 msgstr ""
 
-#: bzrlib/errors.py:1463
+#: bzrlib/errors.py:1381
 msgid "Revision {%(revision_id)s} already present in \"%(file_id)s\"."
 msgstr ""
 
-#: bzrlib/errors.py:1473
+#: bzrlib/errors.py:1391
 msgid "Text did not match its checksum: %(msg)s"
 msgstr ""
 
-#: bzrlib/errors.py:1598
+#: bzrlib/errors.py:1517
 msgid "Export format %(format)r not supported"
 msgstr ""
 
-#: bzrlib/errors.py:1607
+#: bzrlib/errors.py:1526
 msgid "Transport error: %(msg)s %(orig_error)s"
 msgstr ""
 
-#: bzrlib/errors.py:1633
+#: bzrlib/errors.py:1552
 msgid "Generic bzr smart protocol error: %(details)s"
 msgstr ""
 
-#: bzrlib/errors.py:1641
+#: bzrlib/errors.py:1560
 msgid "Received bad protocol version marker: %(marker)r"
 msgstr ""
 
-#: bzrlib/errors.py:1672
+#: bzrlib/errors.py:1592
 msgid "Transport operation not possible: %(msg)s %(orig_error)s"
 msgstr ""
 
-#: bzrlib/errors.py:1677
+#: bzrlib/errors.py:1597
 msgid "Connection error: %(msg)s %(orig_error)s"
 msgstr ""
 
-#: bzrlib/errors.py:1682
+#: bzrlib/errors.py:1602
 msgid "%(msg)s %(host)s%(port)s%(orig_error)s"
 msgstr ""
 
-#: bzrlib/errors.py:1703
+#: bzrlib/errors.py:1623
 msgid "Connection closed: %(msg)s %(orig_error)s"
 msgstr ""
 
-#: bzrlib/errors.py:1708
+#: bzrlib/errors.py:1628
+msgid "Connection Timeout: %(msg)s%(orig_error)s"
+msgstr ""
+
+#: bzrlib/errors.py:1633
 msgid "Invalid range access in %(path)s at %(offset)s: %(msg)s"
 msgstr ""
 
-#: bzrlib/errors.py:1718
+#: bzrlib/errors.py:1643
 msgid "Invalid http response for %(path)s: %(msg)s%(orig_error)s"
 msgstr ""
 
-#: bzrlib/errors.py:1733
+#: bzrlib/errors.py:1658
 msgid "Invalid http range %(range)r for %(path)s: %(msg)s"
 msgstr ""
 
-#: bzrlib/errors.py:1747
+#: bzrlib/errors.py:1672
 msgid "HTTP MIME Boundary missing for %(path)s: %(msg)s"
 msgstr ""
 
-#: bzrlib/errors.py:1755
+#: bzrlib/errors.py:1680
 msgid "Invalid http Content-type \"%(ctype)s\" for %(path)s: %(msg)s"
 msgstr ""
 
-#: bzrlib/errors.py:1764
+#: bzrlib/errors.py:1689
 msgid "%(source)s is%(permanently)s redirected to %(target)s"
 msgstr ""
 
-#: bzrlib/errors.py:1778
+#: bzrlib/errors.py:1703
 msgid "Too many redirections"
 msgstr ""
 
-#: bzrlib/errors.py:1783
+#: bzrlib/errors.py:1708
 msgid "Working tree has conflicts."
 msgstr ""
 
-#: bzrlib/errors.py:1787
+#: bzrlib/errors.py:1712
 msgid "Config file %(filename)s is not UTF-8 encoded\n"
 msgstr ""
 
-#: bzrlib/errors.py:1796
+#: bzrlib/errors.py:1721
 msgid ""
 "Error(s) parsing config file %(filename)s:\n"
 "%(errors)s"
 msgstr ""
 
-#: bzrlib/errors.py:1807
+#: bzrlib/errors.py:1732
 msgid "Bad value \"%(value)s\" for option \"%(name)s\"."
 msgstr ""
 
-#: bzrlib/errors.py:1815
+#: bzrlib/errors.py:1740
 msgid "%(username)r does not seem to contain a reasonable email address"
 msgstr ""
 
-#: bzrlib/errors.py:1824
+#: bzrlib/errors.py:1749
 msgid "Failed to GPG sign data with command \"%(command_line)s\""
 msgstr ""
 
-#: bzrlib/errors.py:1832
+#: bzrlib/errors.py:1757
 msgid "Failed to verify GPG signature data with error \"%(error)s\""
 msgstr ""
 
-#: bzrlib/errors.py:1840
+#: bzrlib/errors.py:1765
 msgid "Unable to import library \"%(library)s\": %(error)s"
 msgstr ""
 
-#: bzrlib/errors.py:1848
+#: bzrlib/errors.py:1773
 msgid "python-gpgme is not installed, it is needed to verify signatures"
 msgstr ""
 
-#: bzrlib/errors.py:1856
+#: bzrlib/errors.py:1781
 msgid ""
 "The working tree for %(basedir)s has changed since the last commit, but "
 "weave merge requires that it be unchanged"
 msgstr ""
 
-#: bzrlib/errors.py:1866
+#: bzrlib/errors.py:1791
 msgid ""
 "Can't reprocess and show base, because reprocessing obscures the "
 "relationship of conflicting lines to the base"
 msgstr ""
 
-#: bzrlib/errors.py:1872
+#: bzrlib/errors.py:1797
 msgid "Cycle in graph %(graph)r"
 msgstr ""
 
-#: bzrlib/errors.py:1900
+#: bzrlib/errors.py:1825
 msgid "File %(filename)s is not conflicted."
 msgstr ""
 
-#: bzrlib/errors.py:1922
+#: bzrlib/errors.py:1847
 msgid "No bundle was found in \"%(filename)s\"."
 msgstr ""
 
-#: bzrlib/errors.py:1931
+#: bzrlib/errors.py:1856
 msgid "Unable to handle bundle version %(version)s: %(msg)s"
 msgstr ""
 
-#: bzrlib/errors.py:1941
+#: bzrlib/errors.py:1866
 msgid "Branch %(base)s is missing revision %(text_revision)s of %(file_id)s"
 msgstr ""
 
-#: bzrlib/errors.py:1954
+#: bzrlib/errors.py:1879
 msgid "File id {%(file_id)s} already exists in inventory as %(entry)s"
 msgstr ""
 
-#: bzrlib/errors.py:1964
+#: bzrlib/errors.py:1889
 msgid "Key %(key)s is already present in map"
 msgstr ""
 
-#: bzrlib/errors.py:1969
+#: bzrlib/errors.py:1894
 msgid "The prefix %(prefix)s is in the help search path twice."
 msgstr ""
 
-#: bzrlib/errors.py:1977
-msgid "Tree transform is malformed %(conflicts)r"
-msgstr ""
-
-#: bzrlib/errors.py:1979
+#: bzrlib/errors.py:1904
 msgid ""
 "No final name for trans_id %(trans_id)r\n"
 "file-id: %(file_id)r\n"
 "root trans-id: %(root_trans_id)r\n"
 msgstr ""
 
-#: bzrlib/errors.py:2011
+#: bzrlib/errors.py:1936
 msgid "Attempt to reuse a transform that has already been applied."
 msgstr ""
 
-#: bzrlib/errors.py:2016
+#: bzrlib/errors.py:1941
 msgid "Moving the root directory is not supported at this time"
 msgstr ""
 
-#: bzrlib/errors.py:2021
+#: bzrlib/errors.py:1946
 msgid "Failed to rename %(from_path)s to %(to_path)s: %(why)s"
 msgstr ""
 
-#: bzrlib/errors.py:2032
+#: bzrlib/errors.py:1957
 msgid ""
 "Could not move %(from_path)s%(operator)s %(to_path)s%(_has_extra)s%(extra)s"
 msgstr ""
 
-#: bzrlib/errors.py:2068
+#: bzrlib/errors.py:1993
 msgid ""
 "Could not rename %(from_path)s%(operator)s %(to_path)s%(_has_extra)s%(extra)s"
 msgstr ""
 
-#: bzrlib/errors.py:2077
-msgid ""
-"Can't safely remove modified or unknown files:\n"
-"%(changes_as_text)sUse --keep to not delete them, or --force to delete them "
-"regardless."
-msgstr ""
-
-#: bzrlib/errors.py:2114
+#: bzrlib/errors.py:2023
 msgid "Unable to import paramiko (required for sftp support): %(error)s"
 msgstr ""
 
-#: bzrlib/errors.py:2122
+#: bzrlib/errors.py:2031
 msgid "Nothing to merge."
 msgstr ""
 
-#: bzrlib/errors.py:2127
+#: bzrlib/errors.py:2036
 msgid "Format %(format)s cannot be initialised by this version of bzr."
 msgstr ""
 
-#: bzrlib/errors.py:2136
+#: bzrlib/errors.py:2045
 msgid ""
 "Cannot convert from format %(from_format)s to format %(format)s.    "
 "%(problem)s"
 msgstr ""
 
-#: bzrlib/errors.py:2148
+#: bzrlib/errors.py:2057
 msgid "Could not find an appropriate Differ for file \"%(path)s\""
 msgstr ""
 
-#: bzrlib/errors.py:2156
+#: bzrlib/errors.py:2065
 msgid "%(exe_name)s could not be found on this machine"
 msgstr ""
 
-#: bzrlib/errors.py:2164
+#: bzrlib/errors.py:2073
 msgid "Diff is not installed on this machine: %(msg)s"
 msgstr ""
 
-#: bzrlib/errors.py:2172
+#: bzrlib/errors.py:2081
 msgid "Diff3 is not installed on this machine."
 msgstr ""
 
-#: bzrlib/errors.py:2178
+#: bzrlib/errors.py:2087
 msgid "The content being inserted is already present."
 msgstr ""
 
-#: bzrlib/errors.py:2183
+#: bzrlib/errors.py:2092
 msgid ""
 "This tree contains left-over files from a failed operation.\n"
 "    Please examine %(limbo_dir)s to see if it contains any files you wish "
@@ -4388,14 +5734,14 @@
 "    keep, and delete it when you are done."
 msgstr ""
 
-#: bzrlib/errors.py:2194
+#: bzrlib/errors.py:2103
 msgid ""
 "This tree contains left-over files from a failed operation.\n"
 "    Please examine %(pending_deletion)s to see if it contains any files you\n"
 "    wish to keep, and delete it when you are done."
 msgstr ""
 
-#: bzrlib/errors.py:2204
+#: bzrlib/errors.py:2113
 msgid ""
 "Unable to delete transform temporary directory %(limbo_dir)s.\n"
 "    Please examine %(limbo_dir)s to see if it contains any files you wish "
@@ -4403,667 +5749,912 @@
 "    keep, and delete it when you are done."
 msgstr ""
 
-#: bzrlib/errors.py:2215
+#: bzrlib/errors.py:2124
 msgid ""
 "Unable to delete transform temporary directory %(pending_deletion)s.  Please "
 "examine %(pending_deletion)s to see if it contains any files you wish to "
 "keep, and delete it when you are done."
 msgstr ""
 
-#: bzrlib/errors.py:2225
+#: bzrlib/errors.py:2134
 msgid "Working tree is out of date, please run 'bzr update'.%(more)s"
 msgstr ""
 
-#: bzrlib/errors.py:2239
+#: bzrlib/errors.py:2148
 msgid "Public branch \"%(public_location)s\" lacks revision \"%(revstring)s\"."
 msgstr ""
 
-#: bzrlib/errors.py:2252
+#: bzrlib/errors.py:2161
 msgid "Error in merge modified format"
 msgstr ""
 
-#: bzrlib/errors.py:2257
+#: bzrlib/errors.py:2166
 msgid "Format error in conflict listings"
 msgstr ""
 
-#: bzrlib/errors.py:2261
+#: bzrlib/errors.py:2170
 msgid ""
 "Inconsistency in dirstate file %(dirstate_path)s.\n"
 "Error: %(description)s"
 msgstr ""
 
-#: bzrlib/errors.py:2272
+#: bzrlib/errors.py:2181
 msgid ""
 "An error has been detected in the repository %(repo_path)s.\n"
 "Please run bzr reconcile on this repository."
 msgstr ""
 
-#: bzrlib/errors.py:2283
+#: bzrlib/errors.py:2192
 msgid ""
 "An inconsistent delta was supplied involving %(path)r, %(file_id)r\n"
 "reason: %(reason)s"
 msgstr ""
 
-#: bzrlib/errors.py:2296
+#: bzrlib/errors.py:2205
 msgid ""
 "An inconsistent delta was supplied: %(delta)r\n"
 "reason: %(reason)s"
 msgstr ""
 
-#: bzrlib/errors.py:2308
+#: bzrlib/errors.py:2217
 msgid "To use this feature you must upgrade your branch at %(path)s."
 msgstr ""
 
-#: bzrlib/errors.py:2317
+#: bzrlib/errors.py:2226
 msgid "To use this feature you must upgrade your repository at %(path)s."
 msgstr ""
 
-#: bzrlib/errors.py:2322
+#: bzrlib/errors.py:2231
 msgid ""
 "To use this feature you must upgrade your branch at %(path)s to a format "
 "which supports rich roots."
 msgstr ""
 
-#: bzrlib/errors.py:2328
+#: bzrlib/errors.py:2237
 msgid "Cannot perform local-only commits on unbound branches."
 msgstr ""
 
-#: bzrlib/errors.py:2333
+#: bzrlib/errors.py:2242
 msgid "The method %(mname)s is not supported on objects of type %(tname)s."
 msgstr ""
 
-#: bzrlib/errors.py:2354
+#: bzrlib/errors.py:2263
+msgid "Ghost tags not supported by format %(format)r."
+msgstr ""
+
+#: bzrlib/errors.py:2271
 msgid "File is binary but should be text."
 msgstr ""
 
-#: bzrlib/errors.py:2359
+#: bzrlib/errors.py:2276
 msgid "The path %(path)s is not permitted on this platform"
 msgstr ""
 
-#: bzrlib/errors.py:2368
+#: bzrlib/errors.py:2285
 msgid ""
 "Testament did not match expected value.\n"
 "       For revision_id {%(revision_id)s}, expected {%(expected)s}, measured\n"
 "       {%(measured)s}"
 msgstr ""
 
-#: bzrlib/errors.py:2380
+#: bzrlib/errors.py:2297
 msgid "Not a bzr revision-bundle: %(text)r"
 msgstr ""
 
-#: bzrlib/errors.py:2389
+#: bzrlib/errors.py:2306
 msgid "Bad bzr revision-bundle: %(text)r"
 msgstr ""
 
-#: bzrlib/errors.py:2398
+#: bzrlib/errors.py:2315
 msgid "Malformed bzr revision-bundle header: %(text)r"
 msgstr ""
 
-#: bzrlib/errors.py:2403
+#: bzrlib/errors.py:2320
 msgid "Malformed patches in bzr revision-bundle: %(text)r"
 msgstr ""
 
-#: bzrlib/errors.py:2408
+#: bzrlib/errors.py:2325
 msgid "Malformed footer in bzr revision-bundle: %(text)r"
 msgstr ""
 
-#: bzrlib/errors.py:2413
+#: bzrlib/errors.py:2330
 msgid "End of line marker was not \\n in bzr revision-bundle"
 msgstr ""
 
-#: bzrlib/errors.py:2423
+#: bzrlib/errors.py:2340
 msgid "Bundle format %(bundle_format)s is incompatible with %(other)s"
 msgstr ""
 
-#: bzrlib/errors.py:2433
+#: bzrlib/errors.py:2350
 msgid "Root class for inventory serialization errors"
 msgstr ""
 
-#: bzrlib/errors.py:2437
+#: bzrlib/errors.py:2354
 msgid ""
 "The inventory was not in the expected format:\n"
 " %(msg)s"
 msgstr ""
 
-#: bzrlib/errors.py:2446
+#: bzrlib/errors.py:2363
 msgid "This operation requires rich root data storage"
 msgstr ""
 
-#: bzrlib/errors.py:2459
+#: bzrlib/errors.py:2376
 msgid "Unrecognised value for BZR_SSH environment variable: %(vendor)s"
 msgstr ""
 
-#: bzrlib/errors.py:2468
+#: bzrlib/errors.py:2385
 msgid ""
 "Don't know how to handle SSH connections. Please set BZR_SSH environment "
 "variable."
 msgstr ""
 
-#: bzrlib/errors.py:2475
+#: bzrlib/errors.py:2392
 msgid ""
 "Could not determine revno for {%(revision_id)s} because its ancestry shows a "
 "ghost at {%(ghost_revision_id)s}"
 msgstr ""
 
-#: bzrlib/errors.py:2485
+#: bzrlib/errors.py:2402
 msgid "Ghost revision {%(revision_id)s} cannot be used here."
 msgstr ""
 
-#: bzrlib/errors.py:2536
+#: bzrlib/errors.py:2453
 msgid ""
 "A merge directive must provide either a bundle or a public branch location."
 msgstr ""
 
-#: bzrlib/errors.py:2543
+#: bzrlib/errors.py:2460
 msgid "Bad merge directive payload %(start)r"
 msgstr ""
 
-#: bzrlib/errors.py:2553
+#: bzrlib/errors.py:2470
 msgid "Preview patch does not match requested changes."
 msgstr ""
 
-#: bzrlib/errors.py:2559
+#: bzrlib/errors.py:2476
 msgid "Patch_type was %(patch_type)s, but no patch was supplied."
 msgstr ""
 
-#: bzrlib/errors.py:2569
+#: bzrlib/errors.py:2486
 msgid ""
 "Your branch does not have all of the revisions required in order to merge "
 "this merge directive and the target location specified in the merge "
 "directive is not a branch: %(location)s."
 msgstr ""
 
-#: bzrlib/errors.py:2581
+#: bzrlib/errors.py:2498
 msgid "Unsupported entry kind %(kind)s"
 msgstr ""
 
-#: bzrlib/errors.py:2589
+#: bzrlib/errors.py:2506
 msgid "Can't subsume %(other_tree)s into %(tree)s. %(reason)s"
 msgstr ""
 
-#: bzrlib/errors.py:2599
+#: bzrlib/errors.py:2516
 msgid "Subsume target %(other_tree)s needs to be upgraded."
 msgstr ""
 
-#: bzrlib/errors.py:2618
+#: bzrlib/errors.py:2535
 msgid "No such tag: %(tag_name)s"
 msgstr ""
 
-#: bzrlib/errors.py:2626
+#: bzrlib/errors.py:2543
 msgid "Tags not supported by %(branch)s; you may be able to use bzr upgrade."
 msgstr ""
 
-#: bzrlib/errors.py:2635
+#: bzrlib/errors.py:2552
 msgid "Tag %(tag_name)s already exists."
 msgstr ""
 
-#: bzrlib/errors.py:2643
+#: bzrlib/errors.py:2560
 msgid ""
 "Did not understand bug identifier %(bug_id)s: %(reason)s. See \"bzr help bugs"
 "\" for more information on this feature."
 msgstr ""
 
-#: bzrlib/errors.py:2653
+#: bzrlib/errors.py:2570
 msgid ""
 "The URL for bug tracker \"%(abbreviation)s\" doesn't contain {id}: %(url)s"
 msgstr ""
 
-#: bzrlib/errors.py:2663
+#: bzrlib/errors.py:2580
 msgid ""
 "Cannot find registered bug tracker called %(abbreviation)s on %(branch)s"
 msgstr ""
 
-#: bzrlib/errors.py:2673
+#: bzrlib/errors.py:2590
 msgid "Invalid line in bugs property: '%(line)s'"
 msgstr ""
 
-#: bzrlib/errors.py:2681
+#: bzrlib/errors.py:2598
 msgid "Invalid bug status: '%(status)s'"
 msgstr ""
 
-#: bzrlib/errors.py:2689
+#: bzrlib/errors.py:2606
 msgid "Could not understand response from smart server: %(response_tuple)r"
 msgstr ""
 
-#: bzrlib/errors.py:2728
+#: bzrlib/errors.py:2645
 msgid "Server sent an unexpected error: %(error_tuple)r"
 msgstr ""
 
-#: bzrlib/errors.py:2747
+#: bzrlib/errors.py:2664
 msgid "Unrecognised container format: %(container_format)r"
 msgstr ""
 
-#: bzrlib/errors.py:2755
+#: bzrlib/errors.py:2672
 msgid "Unexpected end of container stream"
 msgstr ""
 
-#: bzrlib/errors.py:2760
+#: bzrlib/errors.py:2677
 msgid "Unknown record type: %(record_type)r"
 msgstr ""
 
-#: bzrlib/errors.py:2768
+#: bzrlib/errors.py:2685
 msgid "Invalid record: %(reason)s"
 msgstr ""
 
-#: bzrlib/errors.py:2776
+#: bzrlib/errors.py:2693
 msgid "Container has data after end marker: %(excess)r"
 msgstr ""
 
-#: bzrlib/errors.py:2784
+#: bzrlib/errors.py:2701
 msgid "Container has multiple records with the same name: %(name)s"
 msgstr ""
 
-#: bzrlib/errors.py:2797
+#: bzrlib/errors.py:2714
 msgid "Corrupt or incompatible data stream: %(reason)s"
 msgstr ""
 
-#: bzrlib/errors.py:2805
+#: bzrlib/errors.py:2722
 msgid "SMTP error: %(error)s"
 msgstr ""
 
-#: bzrlib/errors.py:2813
+#: bzrlib/errors.py:2730
 msgid "No message supplied."
 msgstr ""
 
-#: bzrlib/errors.py:2818
+#: bzrlib/errors.py:2735
 msgid "No mail-to address (--mail-to) or output (-o) specified."
 msgstr ""
 
-#: bzrlib/errors.py:2823
+#: bzrlib/errors.py:2740
 msgid "Unknown mail client: %(mail_client)s"
 msgstr ""
 
-#: bzrlib/errors.py:2831
+#: bzrlib/errors.py:2748
 msgid ""
 "Unable to find mail client with the following names: "
 "%(mail_command_list_string)s"
 msgstr ""
 
-#: bzrlib/errors.py:2841
+#: bzrlib/errors.py:2758
 msgid "SMTP connection to %(host)s refused"
 msgstr ""
 
-#: bzrlib/errors.py:2850
+#: bzrlib/errors.py:2767
 msgid "Please specify smtp_server.  No server at default %(host)s."
 msgstr ""
 
-#: bzrlib/errors.py:2864
+#: bzrlib/errors.py:2781
 msgid ""
 "'%(display_url)s' is not in sync with %(target_url)s.  See bzr help sync-for-"
 "reconfigure."
 msgstr ""
 
-#: bzrlib/errors.py:2876
+#: bzrlib/errors.py:2793
 msgid "'%(display_url)s' is already a branch."
 msgstr ""
 
-#: bzrlib/errors.py:2881
+#: bzrlib/errors.py:2798
 msgid "'%(display_url)s' is already a tree."
 msgstr ""
 
-#: bzrlib/errors.py:2886
+#: bzrlib/errors.py:2803
 msgid "'%(display_url)s' is already a checkout."
 msgstr ""
 
-#: bzrlib/errors.py:2891
+#: bzrlib/errors.py:2808
 msgid "'%(display_url)s' is already a lightweight checkout."
 msgstr ""
 
-#: bzrlib/errors.py:2896
+#: bzrlib/errors.py:2813
 msgid "'%(display_url)s' is already using a shared repository."
 msgstr ""
 
-#: bzrlib/errors.py:2901
+#: bzrlib/errors.py:2818
 msgid "'%(display_url)s' is already standalone."
 msgstr ""
 
-#: bzrlib/errors.py:2906
+#: bzrlib/errors.py:2823
 msgid "Shared repository '%(display_url)s' already creates working trees."
 msgstr ""
 
-#: bzrlib/errors.py:2912
+#: bzrlib/errors.py:2829
 msgid ""
 "Shared repository '%(display_url)s' already doesn't create working trees."
 msgstr ""
 
-#: bzrlib/errors.py:2918
+#: bzrlib/errors.py:2835
 msgid "Requested reconfiguration of '%(display_url)s' is not supported."
 msgstr ""
 
-#: bzrlib/errors.py:2923
+#: bzrlib/errors.py:2840
 msgid "No location could be found to bind to at %(display_url)s."
 msgstr ""
 
-#: bzrlib/errors.py:2928
+#: bzrlib/errors.py:2845
 msgid ""
 "Working tree \"%(display_url)s\" has uncommitted changes (See bzr status)."
 "%(more)s"
 msgstr ""
 
-#: bzrlib/errors.py:2947
+#: bzrlib/errors.py:2864
 msgid ""
 "Working tree \"%(display_url)s\" has shelved changes (See bzr shelve --list)."
 "%(more)s"
 msgstr ""
 
-#: bzrlib/errors.py:2953
+#: bzrlib/errors.py:2870
 msgid "Variable {%(name)s} is not available."
 msgstr ""
 
-#: bzrlib/errors.py:2961
+#: bzrlib/errors.py:2878
 msgid "No template specified."
 msgstr ""
 
-#: bzrlib/errors.py:2966
+#: bzrlib/errors.py:2883
 msgid "Unable to create symlink %(path_str)son this platform"
 msgstr ""
 
-#: bzrlib/errors.py:2981
+#: bzrlib/errors.py:2898
 msgid ""
 "Unsupported timezone format \"%(timezone)s\", options are \"utc\", \"original"
 "\", \"local\"."
 msgstr ""
 
-#: bzrlib/errors.py:3015
+#: bzrlib/errors.py:2932
 msgid ""
 "Unable to encode %(kind)s path %(path)r in user encoding %(user_encoding)s"
 msgstr ""
 
-#: bzrlib/errors.py:3027
+#: bzrlib/errors.py:2944
 msgid "The \"%(config_id)s\" configuration does not exist."
 msgstr ""
 
-#: bzrlib/errors.py:3035
+#: bzrlib/errors.py:2952
 msgid "The \"%(option_name)s\" configuration option does not exist."
 msgstr ""
 
-#: bzrlib/errors.py:3043
+#: bzrlib/errors.py:2960
 msgid "The alias \"%(alias_name)s\" does not exist."
 msgstr ""
 
-#: bzrlib/errors.py:3057
+#: bzrlib/errors.py:2974
 msgid "\"%(alias_name)s\" is not a valid location alias."
 msgstr ""
 
-#: bzrlib/errors.py:3065
+#: bzrlib/errors.py:2982
 msgid "No %(alias_name)s location assigned."
 msgstr ""
 
-#: bzrlib/errors.py:3073
+#: bzrlib/errors.py:2990
 msgid "Cannot bind address \"%(host)s:%(port)i\": %(orig_error)s."
 msgstr ""
 
-#: bzrlib/errors.py:3083
+#: bzrlib/errors.py:3000
 msgid "Unknown rules detected: %(unknowns_str)s."
 msgstr ""
 
-#: bzrlib/errors.py:3096
-msgid ""
-"Hook '%(hook_name)s' during %(hook_stage)s failed:\n"
-"%(traceback_text)s%(exc_value)s"
-msgstr ""
-
-#: bzrlib/errors.py:3119
+#: bzrlib/errors.py:3011
 msgid "Tip change rejected: %(msg)s"
 msgstr ""
 
-#: bzrlib/errors.py:3127
+#: bzrlib/errors.py:3019
 msgid "Shelf corrupt."
 msgstr ""
 
-#: bzrlib/errors.py:3132
+#: bzrlib/errors.py:3024
 msgid "Corruption while decompressing repository file%(orig_error)s"
 msgstr ""
 
-#: bzrlib/errors.py:3144
+#: bzrlib/errors.py:3036
 msgid "No changes are shelved with id \"%(shelf_id)d\"."
 msgstr ""
 
-#: bzrlib/errors.py:3152
+#: bzrlib/errors.py:3044
 msgid "\"%(invalid_id)s\" is not a valid shelf id, try a number instead."
 msgstr ""
 
-#: bzrlib/errors.py:3160
+#: bzrlib/errors.py:3052
 msgid "An attempt to access a url outside the server jail was made: '%(url)s'."
 msgstr ""
 
-#: bzrlib/errors.py:3168
+#: bzrlib/errors.py:3060
 msgid "The user aborted the operation."
 msgstr ""
 
-#: bzrlib/errors.py:3173
+#: bzrlib/errors.py:3065
 msgid "Branching '%(url)s'(%(format)s) must create a working tree."
 msgstr ""
 
-#: bzrlib/errors.py:3183
+#: bzrlib/errors.py:3075
 msgid "No such view: %(view_name)s."
 msgstr ""
 
-#: bzrlib/errors.py:3193
+#: bzrlib/errors.py:3085
 msgid ""
 "Views are not supported by %(tree)s; use 'bzr upgrade' to change your tree "
 "to a later format."
 msgstr ""
 
-#: bzrlib/errors.py:3202
+#: bzrlib/errors.py:3094
 msgid ""
 "Specified file \"%(file_name)s\" is outside the current view: %(view_str)s"
 msgstr ""
 
-#: bzrlib/errors.py:3269
+#: bzrlib/errors.py:3161
 msgid "%(bzrdir)r does not support co-located branches."
 msgstr ""
 
-#: bzrlib/errors.py:3275
+#: bzrlib/errors.py:3167
 msgid ""
 "Unable to determine your name.\n"
 "Please, set your name with the 'whoami' command.\n"
 "E.g. bzr whoami \"Your Name <name@example.com>\""
 msgstr ""
 
-#: bzrlib/errors.py:3284
+#: bzrlib/errors.py:3176
 msgid "Invalid pattern(s) found. %(msg)s"
 msgstr ""
 
-#: bzrlib/errors.py:3292
+#: bzrlib/errors.py:3184
 msgid ""
 "Branch \"%(branch_url)s\" appears to be bound to itself. Please use `bzr "
 "unbind` to fix."
 msgstr ""
 
-#: bzrlib/errors.py:3303
+#: bzrlib/errors.py:3195
 msgid "Loop involving %(refs)r while expanding \"%(string)s\"."
 msgstr ""
 
-#: bzrlib/errors.py:3312
+#: bzrlib/errors.py:3204
 msgid "Option %(name)s is not defined while expanding \"%(string)s\"."
 msgstr ""
 
-#: bzrlib/errors.py:3321
+#: bzrlib/errors.py:3213
 msgid ""
 "No compatible object available for operations from %(source)r to %(target)r."
 msgstr ""
 
-#: bzrlib/gpg.py:126 bzrlib/gpg.py:464
+#: bzrlib/errors.py:3223
+msgid ""
+"VFS requests over the smart server are not allowed. Encountered: %(method)s, "
+"%(arguments)s."
+msgstr ""
+
+#: bzrlib/errors.py:3233
+msgid ""
+"Kind change from %(from_kind)s to %(to_kind)s for %(path)s not supported by "
+"format %(format)r"
+msgstr ""
+
+#: bzrlib/errors.py:3249
+msgid "Binary files section encountered."
+msgstr ""
+
+#: bzrlib/errors.py:3257
+msgid ""
+"Malformed patch header.  %(desc)s\n"
+"%(line)r"
+msgstr ""
+
+#: bzrlib/errors.py:3266
+msgid ""
+"Malformed hunk header.  %(desc)s\n"
+"%(line)r"
+msgstr ""
+
+#: bzrlib/errors.py:3275
+msgid ""
+"Malformed line.  %(desc)s\n"
+"%(line)r"
+msgstr ""
+
+#: bzrlib/errors.py:3285
+msgid ""
+"Text contents mismatch at line %(line_no)d.  Original has \"%(orig_line)s\", "
+"but patch says it should be \"%(patch_line)s\""
+msgstr ""
+
+#: bzrlib/export_pot.py:246
+#, python-format
+msgid "Exporting messages from builtin command: %s"
+msgstr ""
+
+#: bzrlib/export_pot.py:265
+msgid "Exporting messages from plugin command: {0} in {1}"
+msgstr ""
+
+#: bzrlib/export_pot.py:286
+#, python-format
+msgid "Exporting message from error: %s"
+msgstr ""
+
+#: bzrlib/fetch.py:97
+msgid "Finding revisions"
+msgstr ""
+
+#: bzrlib/fetch.py:102
+msgid "Fetching revisions"
+msgstr ""
+
+#: bzrlib/foreign.py:261
+msgid "Push into a different VCS without any custom bzr metadata."
+msgstr ""
+
+#: bzrlib/foreign.py:263
+msgid ""
+"This will afterwards rebase the local branch on the remote\n"
+"branch unless the --no-rebase option is used, in which case \n"
+"the two branches will be out of sync after the push. "
+msgstr ""
+
+# help of 'directory' option of 'dpush' command
+#: bzrlib/foreign.py:271
+msgid ""
+"Branch to push from, rather than the one containing the working directory."
+msgstr ""
+
+# help of 'no-rebase' option of 'dpush' command
+#: bzrlib/foreign.py:276
+msgid "Do not rebase after push."
+msgstr ""
+
+# help of 'strict' option of 'dpush' command
+#: bzrlib/foreign.py:278
+msgid ""
+"Refuse to push if there are uncommitted changes in the working tree, --no-"
+"strict disables the check."
+msgstr ""
+
+#: bzrlib/foreign.py:310
+#, python-format
+msgid "Using saved location: %s\n"
+msgstr ""
+
+#: bzrlib/foreign.py:320
+msgid ""
+"{0!r} and {1!r} are in the same VCS, lossy push not necessary. Please use "
+"regular push."
+msgstr ""
+
+#: bzrlib/gpg.py:116 bzrlib/gpg.py:454
 msgid "{0} commits with valid signatures"
 msgstr ""
 
-#: bzrlib/gpg.py:130 bzrlib/gpg.py:469
+#: bzrlib/gpg.py:120 bzrlib/gpg.py:459
 msgid "{0} commit with unknown key"
 msgid_plural "{0} commits with unknown keys"
 msgstr[0] ""
 msgstr[1] ""
 
-#: bzrlib/gpg.py:136 bzrlib/gpg.py:476
+#: bzrlib/gpg.py:126 bzrlib/gpg.py:466
 msgid "{0} commit not valid"
 msgid_plural "{0} commits not valid"
 msgstr[0] ""
 msgstr[1] ""
 
-#: bzrlib/gpg.py:142 bzrlib/gpg.py:483
+#: bzrlib/gpg.py:132 bzrlib/gpg.py:473
 msgid "{0} commit not signed"
 msgid_plural "{0} commits not signed"
 msgstr[0] ""
 msgstr[1] ""
 
-#: bzrlib/gpg.py:148 bzrlib/gpg.py:490
+#: bzrlib/gpg.py:138 bzrlib/gpg.py:480
 msgid "{0} commit with key now expired"
 msgid_plural "{0} commits with key now expired"
 msgstr[0] ""
 msgstr[1] ""
 
-#: bzrlib/gpg.py:346
-msgid "No GnuPG key results for pattern: {}"
+#: bzrlib/gpg.py:336
+msgid "No GnuPG key results for pattern: {0}"
 msgstr ""
 
-#: bzrlib/gpg.py:389
+#: bzrlib/gpg.py:379
 msgid "{0} signed {1} commit"
 msgid_plural "{0} signed {1} commits"
 msgstr[0] ""
 msgstr[1] ""
 
-#: bzrlib/gpg.py:406 bzrlib/gpg.py:422
+#: bzrlib/gpg.py:396 bzrlib/gpg.py:412
 msgid "{0} commit by author {1}"
 msgid_plural "{0} commits by author {1}"
 msgstr[0] ""
 msgstr[1] ""
 
-#: bzrlib/gpg.py:436
+#: bzrlib/gpg.py:426
 msgid "Unknown key {0} signed {1} commit"
 msgid_plural "Unknown key {0} signed {1} commits"
 msgstr[0] ""
 msgstr[1] ""
 
-#: bzrlib/gpg.py:454
+#: bzrlib/gpg.py:444
 msgid "{0} commit by author {1} with key {2} now expired"
 msgid_plural "{0} commits by author {1} with key {2} now expired"
 msgstr[0] ""
 msgstr[1] ""
 
+#: bzrlib/groupcompress.py:1737
+#, python-format
+msgid "Insert claimed random_id=True, but then inserted %r two times"
+msgstr ""
+
+#: bzrlib/hooks.py:329
+#, python-format
+msgid "Introduced in: %s"
+msgstr ""
+
+#: bzrlib/hooks.py:332
+#, python-format
+msgid "Deprecated in: %s"
+msgstr ""
+
+#: bzrlib/knit.py:1764 bzrlib/knit.py:1800
+msgid "Walking content"
+msgstr ""
+
+#: bzrlib/knit.py:3328
+msgid "annotating"
+msgstr ""
+
+#: bzrlib/lock.py:547
+msgid "{0!r} was {1} locked again"
+msgstr ""
+
+#: bzrlib/lockdir.py:320
+#, python-format
+msgid "error removing pending lock: %s"
+msgstr ""
+
+#: bzrlib/lockdir.py:615
+msgid "Unable to obtain"
+msgstr ""
+
+#: bzrlib/lockdir.py:617
+msgid "Lock owner changed for"
+msgstr ""
+
+#: bzrlib/lockdir.py:619
+msgid "{0} lock {1} {2}."
+msgstr ""
+
+#: bzrlib/lockdir.py:626
+#, python-format
+msgid "Will continue to try until %s, unless you press Ctrl-C."
+msgstr ""
+
+#: bzrlib/lockdir.py:628
+msgid "See \"bzr help break-lock\" for more."
+msgstr ""
+
+#: bzrlib/lockdir.py:738
+#, python-format
+msgid ""
+"held by %(user)s on %(hostname)s (process #%(pid)s), acquired %(time_ago)s"
+msgstr ""
+
+#: bzrlib/log.py:572
+msgid "--exclude-common-ancestry requires two different revisions"
+msgstr ""
+
+#: bzrlib/log.py:574
+#, python-format
+msgid "invalid direction %r"
+msgstr ""
+
+#: bzrlib/log.py:621
+msgid "Start revision not found in left-hand history of end revision."
+msgstr ""
+
+#: bzrlib/log.py:667
+msgid "Start revision not found in history of end revision."
+msgstr ""
+
+#: bzrlib/log.py:1085 bzrlib/log.py:1143
+msgid "Logging revision 0 is invalid."
+msgstr ""
+
+#: bzrlib/log.py:1087 bzrlib/log.py:1145
+msgid "Start revision must be older than the end revision."
+msgstr ""
+
+#: bzrlib/log.py:1832
+#, python-format
+msgid "unknown log formatter: %r"
+msgstr ""
+
+#: bzrlib/log.py:2128
+msgid "fixes bug"
+msgid_plural "fixes bugs"
+msgstr[0] ""
+msgstr[1] ""
+
+#: bzrlib/merge.py:718
+msgid "All changes applied successfully."
+msgstr ""
+
+#: bzrlib/merge.py:720
+#, python-format
+msgid "%d conflicts encountered."
+msgstr ""
+
+#: bzrlib/merge.py:868 bzrlib/merge.py:1956
+msgid "Preparing file merge"
+msgstr ""
+
+#: bzrlib/multiparent.py:424
+msgid "Importing revisions"
+msgstr ""
+
 # help of 'help' option
-#: bzrlib/option.py:515
+#: bzrlib/option.py:524
 msgid "Show help message."
 msgstr ""
 
+# help of 'quiet' option
+#: bzrlib/option.py:526
+msgid "Only display errors and warnings."
+msgstr ""
+
 # help of 'usage' option
-#: bzrlib/option.py:517
+#: bzrlib/option.py:529
 msgid "Show usage message and options."
 msgstr ""
 
 # help of 'verbose' option
-#: bzrlib/option.py:519
+#: bzrlib/option.py:531
 msgid "Display more information."
 msgstr ""
 
-# help of 'quiet' option
-#: bzrlib/option.py:522
-msgid "Only display errors and warnings."
-msgstr ""
-
-# help of 'overwrite' option
-#: bzrlib/option.py:527
-msgid "Ignore differences between branches and overwrite unconditionally."
-msgstr ""
-
-# help of 'message' option
-#: bzrlib/option.py:538
-msgid "Message string."
-msgstr ""
-
-# help of 'null' option
-#: bzrlib/option.py:541
-msgid "Use an ASCII NUL (\\0) separator rather than a newline."
-msgstr ""
-
-# help of 'profile' option
-#: bzrlib/option.py:544
-msgid "Show performance profiling information."
-msgstr ""
-
-# help of 'revision' option
-#: bzrlib/option.py:548
-msgid "See \"help revisionspec\" for details."
-msgstr ""
-
 # help of 'change' option
-#: bzrlib/option.py:553
+#: bzrlib/option.py:539
 msgid ""
 "Select changes introduced by the specified revision. See also \"help "
 "revisionspec\"."
 msgstr ""
 
-# help of 'show-ids' option
-#: bzrlib/option.py:555
-msgid "Show internal object ids."
-msgstr ""
-
-# help of 'timezone' option
-#: bzrlib/option.py:558
-msgid "Display timezone as local, original, or utc."
+# help of 'directory' option
+#: bzrlib/option.py:541
+msgid "Branch to operate on, instead of working directory."
+msgstr ""
+
+# help of 'log-format=long' option
+#: bzrlib/option.py:543
+msgid "Detailed log format."
+msgstr ""
+
+# help of 'log-format=gnu-changelog' option
+#: bzrlib/option.py:543
+msgid "Format used by GNU ChangeLog files."
+msgstr ""
+
+# help of 'log-format=line' option
+#: bzrlib/option.py:543
+msgid "Log format with one line per revision."
+msgstr ""
+
+# help of 'log-format=short' option
+#: bzrlib/option.py:543
+msgid "Moderately short log format."
 msgstr ""
 
 # help of 'log-format' option
-#: bzrlib/option.py:563
+#: bzrlib/option.py:543
 msgid "Use specified log format."
 msgstr ""
 
 # title of 'log-format' option
-#: bzrlib/option.py:565
+#: bzrlib/option.py:545
 msgid "Log format"
 msgstr ""
 
-# help of 'long' option
-#: bzrlib/option.py:567
-msgid "Use detailed log format. Same as --log-format long"
-msgstr ""
-
-# help of 'short' option
-#: bzrlib/option.py:569
-msgid "Use moderately short log format. Same as --log-format short"
-msgstr ""
-
-# help of 'line' option
-#: bzrlib/option.py:570
-msgid "Use log format with one line per revision. Same as --log-format line"
+# help of 'merge-type=lca' option
+#: bzrlib/option.py:547
+msgid "LCA-newness merge."
+msgstr ""
+
+# help of 'merge-type=diff3' option
+#: bzrlib/option.py:547
+msgid "Merge using external diff3."
+msgstr ""
+
+# help of 'merge-type=merge3' option
+#: bzrlib/option.py:547
+msgid "Native diff3-style merge."
 msgstr ""
 
 # help of 'merge-type' option
-#: bzrlib/option.py:573
+#: bzrlib/option.py:547
 msgid "Select a particular merge algorithm."
 msgstr ""
 
+# help of 'merge-type=weave' option
+#: bzrlib/option.py:547
+msgid "Weave-based merge."
+msgstr ""
+
 # title of 'merge-type' option
-#: bzrlib/option.py:575
+#: bzrlib/option.py:549
 msgid "Merge algorithm"
 msgstr ""
 
+# help of 'message' option
+#: bzrlib/option.py:552
+msgid "Message string."
+msgstr ""
+
+# help of 'null' option
+#: bzrlib/option.py:554
+msgid "Use an ASCII NUL (\\0) separator rather than a newline."
+msgstr ""
+
+# help of 'overwrite' option
+#: bzrlib/option.py:556
+msgid "Ignore differences between branches and overwrite unconditionally."
+msgstr ""
+
 # help of 'remember' option
-#: bzrlib/option.py:577
+#: bzrlib/option.py:558
 msgid "Remember the specified location as a default."
 msgstr ""
 
 # help of 'reprocess' option
-#: bzrlib/option.py:579
+#: bzrlib/option.py:560
 msgid "Reprocess to reduce spurious conflicts."
 msgstr ""
 
-# help of 'dry-run' option
-#: bzrlib/option.py:582
-msgid "Show what would be done, but don't actually do anything."
-msgstr ""
-
-# help of 'name-from-revision' option
-#: bzrlib/option.py:583
-msgid "The path name in the old tree."
-msgstr ""
-
-# help of 'directory' option
-#: bzrlib/option.py:585
-msgid "Branch to operate on, instead of working directory"
+# help of 'revision' option
+#: bzrlib/option.py:564
+msgid "See \"help revisionspec\" for details."
+msgstr ""
+
+# help of 'show-ids' option
+#: bzrlib/option.py:566
+msgid "Show internal object ids."
+msgstr ""
+
+# help of 'timezone' option
+#: bzrlib/option.py:569
+msgid "Display timezone as local, original, or utc."
+msgstr ""
+
+#: bzrlib/osutils.py:94
+msgid "Parameter {0!r} encoding is unsupported by {1} application locale."
+msgstr ""
+
+#: bzrlib/osutils.py:214
+msgid "lstat/stat of ({0!r}): {1!r}"
+msgstr ""
+
+#: bzrlib/osutils.py:964 bzrlib/osutils.py:975
+#, python-format
+msgid "sorry, %r not allowed in path"
+msgstr ""
+
+#: bzrlib/osutils.py:1193
+#, python-format
+msgid "%r is too short to calculate a relative path"
+msgstr ""
+
+#: bzrlib/plugin.py:142
+#, python-format
+msgid "\"%s\" is not a valid <plugin_name>@<plugin_path> description "
 msgstr ""
 
 # help of 'plugin' option of 'bash-completion' command
-#: bzrlib/plugins/bash_completion/bashcomp.py:407
+#: bzrlib/plugins/bash_completion/bashcomp.py:408
 msgid "Enable completions for the selected plugin (default: all plugins)"
 msgstr ""
 
@@ -5094,24 +6685,24 @@
 msgid "Generate only the shell function, don't enable it"
 msgstr ""
 
-#: bzrlib/plugins/launchpad/__init__.py:78
+#: bzrlib/plugins/launchpad/__init__.py:79
 msgid "Register a branch with launchpad.net."
 msgstr ""
 
-#: bzrlib/plugins/launchpad/__init__.py:80
+#: bzrlib/plugins/launchpad/__init__.py:81
 msgid ""
 "This command lists a bzr branch in the directory of branches on\n"
 "launchpad.net.  Registration allows the branch to be associated with\n"
 "bugs or specifications."
 msgstr ""
 
-#: bzrlib/plugins/launchpad/__init__.py:84
+#: bzrlib/plugins/launchpad/__init__.py:85
 msgid ""
 "Before using this command you must register the project to which the\n"
 "branch belongs, and create an account for yourself on launchpad.net."
 msgstr ""
 
-#: bzrlib/plugins/launchpad/__init__.py:87
+#: bzrlib/plugins/launchpad/__init__.py:88
 msgid ""
 "arguments:\n"
 "    public_url: The publicly visible url for the branch to register.\n"
@@ -5123,7 +6714,7 @@
 "                otherwise error."
 msgstr ""
 
-#: bzrlib/plugins/launchpad/__init__.py:96
+#: bzrlib/plugins/launchpad/__init__.py:97
 msgid ""
 "example:\n"
 "    bzr register-branch http://foo.com/bzr/fooproject.mine \\\n"
@@ -5131,107 +6722,135 @@
 msgstr ""
 
 # help of 'project' option of 'register-branch' command
-#: bzrlib/plugins/launchpad/__init__.py:103
+#: bzrlib/plugins/launchpad/__init__.py:104
 msgid "Launchpad project short name to associate with the branch."
 msgstr ""
 
 # help of 'branch-name' option of 'register-branch' command
-#: bzrlib/plugins/launchpad/__init__.py:110
+#: bzrlib/plugins/launchpad/__init__.py:111
 msgid ""
 "Short name for the branch; by default taken from the last component of the "
 "url."
 msgstr ""
 
 # help of 'branch-title' option of 'register-branch' command
-#: bzrlib/plugins/launchpad/__init__.py:114
+#: bzrlib/plugins/launchpad/__init__.py:115
 msgid "One-sentence description of the branch."
 msgstr ""
 
 # help of 'branch-description' option of 'register-branch' command
-#: bzrlib/plugins/launchpad/__init__.py:117
+#: bzrlib/plugins/launchpad/__init__.py:118
 msgid "Longer description of the purpose or contents of the branch."
 msgstr ""
 
 # help of 'author' option of 'register-branch' command
-#: bzrlib/plugins/launchpad/__init__.py:120
+#: bzrlib/plugins/launchpad/__init__.py:121
 msgid "Branch author's email address, if not yourself."
 msgstr ""
 
 # help of 'link-bug' option of 'register-branch' command
-#: bzrlib/plugins/launchpad/__init__.py:123
+#: bzrlib/plugins/launchpad/__init__.py:124
 msgid "The bug this branch fixes."
 msgstr ""
 
 # help of 'dry-run' option of 'register-branch' command
-#: bzrlib/plugins/launchpad/__init__.py:126
+#: bzrlib/plugins/launchpad/__init__.py:127
 msgid "Prepare the request but don't actually send it."
 msgstr ""
 
-#: bzrlib/plugins/launchpad/__init__.py:186
+#: bzrlib/plugins/launchpad/__init__.py:149
+msgid ""
+"register-branch requires a public branch url - see bzr help register-branch."
+msgstr ""
+
+#: bzrlib/plugins/launchpad/__init__.py:157
+msgid "--product is deprecated; please use --project."
+msgstr ""
+
+#: bzrlib/plugins/launchpad/__init__.py:189
 msgid "Open a Launchpad branch page in your web browser."
 msgstr ""
 
 # help of 'dry-run' option of 'launchpad-open' command
-#: bzrlib/plugins/launchpad/__init__.py:191
+#: bzrlib/plugins/launchpad/__init__.py:194
 msgid "Do not actually open the browser. Just say the URL we would use."
 msgstr ""
 
-#: bzrlib/plugins/launchpad/__init__.py:236
+#: bzrlib/plugins/launchpad/__init__.py:229
+#, python-format
+msgid "Opening %s in web browser"
+msgstr ""
+
+#: bzrlib/plugins/launchpad/__init__.py:239
 msgid "Show or set the Launchpad user ID."
 msgstr ""
 
-#: bzrlib/plugins/launchpad/__init__.py:238
+#: bzrlib/plugins/launchpad/__init__.py:241
 msgid ""
 "When communicating with Launchpad, some commands need to know your\n"
 "Launchpad user ID.  This command can be used to set or show the\n"
 "user ID that Bazaar will use for such communication."
 msgstr ""
 
-#: bzrlib/plugins/launchpad/__init__.py:242
+#: bzrlib/plugins/launchpad/__init__.py:245
 msgid ""
 ":Examples:\n"
 "  Show the Launchpad ID of the current user::"
 msgstr ""
 
-#: bzrlib/plugins/launchpad/__init__.py:245
+#: bzrlib/plugins/launchpad/__init__.py:248
 msgid "      bzr launchpad-login"
 msgstr ""
 
-#: bzrlib/plugins/launchpad/__init__.py:247
+#: bzrlib/plugins/launchpad/__init__.py:250
 msgid "  Set the Launchpad ID of the current user to 'bob'::"
 msgstr ""
 
-#: bzrlib/plugins/launchpad/__init__.py:249
+#: bzrlib/plugins/launchpad/__init__.py:252
 msgid "      bzr launchpad-login bob"
 msgstr ""
 
 # help of 'no-check' option of 'launchpad-login' command
-#: bzrlib/plugins/launchpad/__init__.py:256
+#: bzrlib/plugins/launchpad/__init__.py:259
 msgid "Don't check that the user name is valid."
 msgstr ""
 
-#: bzrlib/plugins/launchpad/__init__.py:292
+#: bzrlib/plugins/launchpad/__init__.py:274
+#: bzrlib/plugins/launchpad/__init__.py:285
+msgid "Launchpad user ID exists and has SSH keys.\n"
+msgstr ""
+
+#: bzrlib/plugins/launchpad/__init__.py:277
+msgid "No Launchpad user ID configured.\n"
+msgstr ""
+
+#: bzrlib/plugins/launchpad/__init__.py:288
+#, python-format
+msgid "Launchpad user ID set to '%s'.\n"
+msgstr ""
+
+#: bzrlib/plugins/launchpad/__init__.py:296
 msgid "Ask Launchpad to mirror a branch now."
 msgstr ""
 
-#: bzrlib/plugins/launchpad/__init__.py:312
+#: bzrlib/plugins/launchpad/__init__.py:316
 msgid "Propose merging a branch on Launchpad."
 msgstr ""
 
-#: bzrlib/plugins/launchpad/__init__.py:314
+#: bzrlib/plugins/launchpad/__init__.py:318
 msgid ""
 "This will open your usual editor to provide the initial comment.  When it\n"
 "has created the proposal, it will open it in your default web browser."
 msgstr ""
 
-#: bzrlib/plugins/launchpad/__init__.py:317
+#: bzrlib/plugins/launchpad/__init__.py:321
 msgid ""
 "The branch will be proposed to merge into SUBMIT_BRANCH.  If SUBMIT_BRANCH\n"
 "is not supplied, the remembered submit branch will be used.  If no submit\n"
 "branch is remembered, the development focus will be used."
 msgstr ""
 
-#: bzrlib/plugins/launchpad/__init__.py:321
+#: bzrlib/plugins/launchpad/__init__.py:325
 msgid ""
 "By default, the SUBMIT_BRANCH's review team will be requested to review\n"
 "the merge proposal.  This can be overriden by specifying --review (-R).\n"
@@ -5239,41 +6858,41 @@
 "may optionally be followed by '=' and the review type.  For example:"
 msgstr ""
 
-#: bzrlib/plugins/launchpad/__init__.py:326
+#: bzrlib/plugins/launchpad/__init__.py:330
 msgid "  bzr lp-propose-merge --review jrandom --review review-team=qa"
 msgstr ""
 
-#: bzrlib/plugins/launchpad/__init__.py:328
+#: bzrlib/plugins/launchpad/__init__.py:332
 msgid ""
 "This will propose a merge,  request \"jrandom\" to perform a review of\n"
 "unspecified type, and request \"review-team\" to perform a \"qa\" review."
 msgstr ""
 
 # help of 'staging' option of 'lp-propose-merge' command
-#: bzrlib/plugins/launchpad/__init__.py:333
+#: bzrlib/plugins/launchpad/__init__.py:337
 msgid "Propose the merge on staging."
 msgstr ""
 
 # help of 'message' option of 'lp-propose-merge' command
-#: bzrlib/plugins/launchpad/__init__.py:335
+#: bzrlib/plugins/launchpad/__init__.py:339
 msgid "Commit message."
 msgstr ""
 
 # help of 'approve' option of 'lp-propose-merge' command
-#: bzrlib/plugins/launchpad/__init__.py:337
+#: bzrlib/plugins/launchpad/__init__.py:341
 msgid "Mark the proposal as approved immediately."
 msgstr ""
 
 # help of 'review' option of 'lp-propose-merge' command
-#: bzrlib/plugins/launchpad/__init__.py:339
+#: bzrlib/plugins/launchpad/__init__.py:343
 msgid "Requested reviewer and optional type."
 msgstr ""
 
-#: bzrlib/plugins/launchpad/__init__.py:376
+#: bzrlib/plugins/launchpad/__init__.py:380
 msgid "Find the proposal to merge this revision."
 msgstr ""
 
-#: bzrlib/plugins/launchpad/__init__.py:378
+#: bzrlib/plugins/launchpad/__init__.py:382
 msgid ""
 "Finds the merge proposal(s) that discussed landing the specified revision.\n"
 "This works only if the selected branch was the merge proposal target, and\n"
@@ -5281,16 +6900,6418 @@
 "are opened in a web browser."
 msgstr ""
 
-#: bzrlib/plugins/launchpad/__init__.py:383
+#: bzrlib/plugins/launchpad/__init__.py:387
 msgid ""
 "Any revision involved in the merge may be specified-- the revision in\n"
 "which the merge was performed, or one of the revisions that was merged."
 msgstr ""
 
-#: bzrlib/plugins/launchpad/__init__.py:386
+#: bzrlib/plugins/launchpad/__init__.py:390
 msgid "So, to find the merge proposal that reviewed line 1 of README::"
 msgstr ""
 
-#: bzrlib/plugins/launchpad/__init__.py:388
+#: bzrlib/plugins/launchpad/__init__.py:392
 msgid "  bzr lp-find-proposal -r annotate:README:1"
 msgstr ""
+
+#: bzrlib/plugins/launchpad/__init__.py:407
+msgid "No review found."
+msgstr ""
+
+#: bzrlib/plugins/launchpad/__init__.py:408
+#, python-format
+msgid "%d proposals(s) found."
+msgstr ""
+
+#: bzrlib/plugins/launchpad/__init__.py:418
+msgid "Finding revision-id"
+msgstr ""
+
+#: bzrlib/plugins/launchpad/__init__.py:425
+msgid "Finding merge"
+msgstr ""
+
+#: bzrlib/plugins/launchpad/__init__.py:430
+msgid "Finding revno"
+msgstr ""
+
+#: bzrlib/plugins/launchpad/__init__.py:435
+msgid "Finding Launchpad branch"
+msgstr ""
+
+#: bzrlib/plugins/launchpad/__init__.py:438
+msgid "Finding proposals"
+msgstr ""
+
+#: bzrlib/plugins/launchpad/account.py:64
+msgid "Setting ssh/sftp usernames for launchpad.net."
+msgstr ""
+
+#: bzrlib/plugins/launchpad/lp_api.py:239
+#: bzrlib/plugins/launchpad/lp_api.py:244
+#, python-format
+msgid "%s is not registered on Launchpad"
+msgstr ""
+
+#: bzrlib/plugins/launchpad/lp_api.py:254
+#, python-format
+msgid "%s has no development focus."
+msgstr ""
+
+#: bzrlib/plugins/launchpad/lp_api.py:259
+#, python-format
+msgid "development focus %s has no branch."
+msgstr ""
+
+#: bzrlib/plugins/launchpad/lp_api.py:264
+#, python-format
+msgid "source package %s has no branch."
+msgstr ""
+
+#: bzrlib/plugins/launchpad/lp_api.py:268
+#, python-format
+msgid "%s has no associated product or source package."
+msgstr ""
+
+#: bzrlib/plugins/launchpad/lp_api.py:280
+#, python-format
+msgid "%s is already up-to-date."
+msgstr ""
+
+#: bzrlib/plugins/launchpad/lp_api.py:287
+#, python-format
+msgid "Pushing to %s"
+msgstr ""
+
+#: bzrlib/plugins/launchpad/lp_directory.py:169
+msgid ""
+"resolution for {0}\n"
+"  local: {1}\n"
+" remote: {2}"
+msgstr ""
+
+#: bzrlib/plugins/launchpad/lp_propose.py:147
+#, python-format
+msgid "There is already a branch merge proposal: %s"
+msgstr ""
+
+#: bzrlib/plugins/weave_fmt/bzrdir.py:226
+msgid "pb parameter to convert() is deprecated"
+msgstr ""
+
+#: bzrlib/plugins/weave_fmt/bzrdir.py:229
+msgid "starting upgrade from format 4 to 5"
+msgstr ""
+
+#: bzrlib/plugins/weave_fmt/bzrdir.py:239
+msgid "note: upgrade may be faster if all store files are ungzipped first"
+msgstr ""
+
+#: bzrlib/plugins/weave_fmt/bzrdir.py:268
+msgid "converting revision"
+msgstr ""
+
+#: bzrlib/plugins/weave_fmt/bzrdir.py:273
+msgid "upgraded to weaves:"
+msgstr ""
+
+#: bzrlib/plugins/weave_fmt/bzrdir.py:274
+#, python-format
+msgid "%6d revisions and inventories"
+msgstr ""
+
+#: bzrlib/plugins/weave_fmt/bzrdir.py:276
+#, python-format
+msgid "%6d revisions not present"
+msgstr ""
+
+#: bzrlib/plugins/weave_fmt/bzrdir.py:278
+#, python-format
+msgid "%6d texts"
+msgstr ""
+
+#: bzrlib/plugins/weave_fmt/bzrdir.py:314
+msgid "writing weave"
+msgstr ""
+
+#: bzrlib/plugins/weave_fmt/bzrdir.py:318
+#: bzrlib/plugins/weave_fmt/bzrdir.py:320
+msgid "inventory"
+msgstr ""
+
+#: bzrlib/plugins/weave_fmt/bzrdir.py:337
+msgid "write revision"
+msgstr ""
+
+#: bzrlib/plugins/weave_fmt/bzrdir.py:351
+msgid "loading revision"
+msgstr ""
+
+#: bzrlib/plugins/weave_fmt/bzrdir.py:356
+#, python-format
+msgid "revision {%s} not present in branch; will be converted as a ghost"
+msgstr ""
+
+#: bzrlib/plugins/weave_fmt/bzrdir.py:496
+msgid "starting upgrade from format 5 to 6"
+msgstr ""
+
+#: bzrlib/plugins/weave_fmt/bzrdir.py:506
+#, python-format
+msgid "adding prefixes to %s"
+msgstr ""
+
+#: bzrlib/plugins/weave_fmt/bzrdir.py:546
+msgid "starting upgrade from format 6 to metadir"
+msgstr ""
+
+#: bzrlib/plugins/weave_fmt/bzrdir.py:554
+msgid "Removing ancestry.weave"
+msgstr ""
+
+#: bzrlib/plugins/weave_fmt/bzrdir.py:559
+msgid "Finding branch files"
+msgstr ""
+
+#: bzrlib/plugins/weave_fmt/bzrdir.py:569
+msgid "Upgrading repository"
+msgstr ""
+
+#: bzrlib/plugins/weave_fmt/bzrdir.py:579
+msgid "Upgrading branch"
+msgstr ""
+
+#: bzrlib/plugins/weave_fmt/bzrdir.py:602
+msgid "No working tree."
+msgstr ""
+
+#: bzrlib/plugins/weave_fmt/bzrdir.py:609
+msgid "Upgrading working tree"
+msgstr ""
+
+#: bzrlib/plugins/weave_fmt/bzrdir.py:630
+#, python-format
+msgid "Make %s lock"
+msgstr ""
+
+#: bzrlib/plugins/weave_fmt/bzrdir.py:641
+#, python-format
+msgid "Moving %s"
+msgstr ""
+
+#: bzrlib/push.py:54
+msgid "Created new branch."
+msgstr ""
+
+#: bzrlib/push.py:96
+#, python-format
+msgid ""
+"Target directory %s already contains a .bzr directory, but it is not valid."
+msgstr ""
+
+#: bzrlib/push.py:99
+#, python-format
+msgid ""
+"Target directory %s already exists, but does not have a .bzr directory. "
+"Supply --use-existing-dir to push there anyway."
+msgstr ""
+
+#: bzrlib/push.py:120
+#, python-format
+msgid "Too many redirections trying to make %s."
+msgstr ""
+
+#: bzrlib/push.py:145
+msgid ""
+"These branches have diverged.  See \"bzr help diverged-branches\" for more "
+"information."
+msgstr ""
+
+#: bzrlib/push.py:149
+#, python-format
+msgid ""
+"It is not possible to losslessly push to %s. You may want to use dpush "
+"instead."
+msgstr ""
+
+#: bzrlib/push.py:155
+#, python-format
+msgid ""
+"At %s you have a valid .bzr control directory, but not a branch or "
+"repository. This is an unsupported configuration. Please move the target "
+"directory out of the way and try again."
+msgstr ""
+
+#: bzrlib/reconcile.py:96
+#, python-format
+msgid "Reconciling branch %s"
+msgstr ""
+
+#: bzrlib/reconcile.py:102
+#, python-format
+msgid "Reconciling repository %s"
+msgstr ""
+
+#: bzrlib/reconcile.py:104
+msgid "Reconciling repository"
+msgstr ""
+
+#: bzrlib/reconcile.py:110
+#, python-format
+msgid "%s cannot canonicalize CHKs."
+msgstr ""
+
+#: bzrlib/reconcile.py:118
+msgid "Reconcile aborted: revision index has inconsistent parents."
+msgstr ""
+
+#: bzrlib/reconcile.py:120
+msgid "Run \"bzr check\" for more details."
+msgstr ""
+
+#: bzrlib/reconcile.py:122
+msgid "Reconciliation complete."
+msgstr ""
+
+#: bzrlib/reconcile.py:165
+msgid "Fixing last revision info {0}  => {1}"
+msgstr ""
+
+#: bzrlib/reconcile.py:172
+msgid "revision_history ok."
+msgstr ""
+
+#: bzrlib/reconcile.py:233
+msgid "Reading inventory data"
+msgstr ""
+
+#: bzrlib/reconcile.py:253 bzrlib/reconcile.py:383
+msgid "Inventory ok."
+msgstr ""
+
+#: bzrlib/reconcile.py:255 bzrlib/reconcile.py:385
+msgid "Backing up inventory"
+msgstr ""
+
+#: bzrlib/reconcile.py:257
+msgid "Backup inventory created."
+msgstr ""
+
+#: bzrlib/reconcile.py:273 bzrlib/reconcile.py:404
+msgid "Writing weave"
+msgstr ""
+
+#: bzrlib/reconcile.py:276 bzrlib/reconcile.py:407
+msgid "Inventory regenerated."
+msgstr ""
+
+#: bzrlib/reconcile.py:370 bzrlib/reconcile.py:372 bzrlib/reconcile.py:375
+msgid "Reading indexes"
+msgstr ""
+
+#: bzrlib/reconcile.py:379 bzrlib/reconcile.py:381
+msgid "Checking unused inventories"
+msgstr ""
+
+#: bzrlib/reconcile.py:387
+msgid "Backup Inventory created"
+msgstr ""
+
+#: bzrlib/reconcile.py:445
+msgid "Fixing text parents"
+msgstr ""
+
+#: bzrlib/reconfigure.py:52
+msgid "{0} is now stacked on {1}\n"
+msgstr ""
+
+#: bzrlib/reconfigure.py:67
+#, python-format
+msgid "%s is now not stacked\n"
+msgstr ""
+
+#: bzrlib/remote.py:1134 bzrlib/repository.py:287
+#, python-format
+msgid "bzr: ERROR (ignored): %s"
+msgstr ""
+
+#: bzrlib/remote.py:2217
+msgid "Copying repository content as tarball..."
+msgstr ""
+
+#: bzrlib/rename_map.py:68
+msgid "Calculating hashes"
+msgstr ""
+
+#: bzrlib/rename_map.py:106
+msgid "Determining hash hits"
+msgstr ""
+
+#: bzrlib/rename_map.py:241
+msgid "{0} => {1}"
+msgstr ""
+
+#: bzrlib/repository.py:1781
+msgid "Moving repository to repository.backup"
+msgstr ""
+
+#: bzrlib/repository.py:1788
+msgid "Creating new repository"
+msgstr ""
+
+#: bzrlib/repository.py:1793
+msgid "Copying content"
+msgstr ""
+
+#: bzrlib/repository.py:1797
+msgid "Deleting old repository content"
+msgstr ""
+
+#: bzrlib/repository.py:1799
+msgid "repository converted"
+msgstr ""
+
+#: bzrlib/revisionspec.py:904
+msgid "Using {0} {1}"
+msgstr ""
+
+#: bzrlib/send.py:57
+#, python-format
+msgid "Mail client \"%s\" does not support specifying body"
+msgstr ""
+
+#: bzrlib/send.py:61
+msgid "--remember requires a branch to be specified."
+msgstr ""
+
+#: bzrlib/send.py:75
+msgid "No submit branch known or specified"
+msgstr ""
+
+#: bzrlib/send.py:78
+msgid "Using saved {0} location \"{1}\" to determine what changes to submit."
+msgstr ""
+
+#: bzrlib/send.py:95
+#, python-format
+msgid "No such send format '%s'."
+msgstr ""
+
+#: bzrlib/send.py:112
+msgid "bzr send takes at most two one revision identifiers"
+msgstr ""
+
+#: bzrlib/send.py:125
+msgid "No revisions to submit."
+msgstr ""
+
+#: bzrlib/send.py:136
+msgid ""
+"- not supported for merge directives that use more than one output file."
+msgstr ""
+
+#: bzrlib/send.py:178
+msgid "Format 0.9 does not permit bundle with no patch"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:45
+#, python-format
+msgid "Shelve adding file \"%(path)s\"?"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:46
+msgid "Shelve binary changes?"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:47
+msgid "Shelve changing \"%s\" from %(other)s to %(this)s?"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:49
+#, python-format
+msgid "Shelve removing file \"%(path)s\"?"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:50
+#, python-format
+msgid "Shelve %d change(s)?"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:51
+msgid "Shelve?"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:52
+#, python-format
+msgid ""
+"Shelve changing target of \"%(path)s\" from \"%(other)s\" to \"%(this)s\"?"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:54
+#, python-format
+msgid "Shelve renaming \"%(other)s\" => \"%(this)s\"?"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:69
+#, python-format
+msgid "Changes shelved with id \"%d\"."
+msgstr ""
+
+#: bzrlib/shelf_ui.py:73
+msgid "Selected changes destroyed."
+msgstr ""
+
+#: bzrlib/shelf_ui.py:77
+msgid "Selected changes:"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:97
+#, python-format
+msgid "Delete file \"%(path)s\"?"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:98
+msgid "Apply binary changes?"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:99
+#, python-format
+msgid "Change \"%(path)s\" from %(this)s to %(other)s?"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:101
+#, python-format
+msgid "Add file \"%(path)s\"?"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:102
+#, python-format
+msgid "Apply %d change(s)?"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:103
+msgid "Apply change?"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:104
+#, python-format
+msgid "Change target of \"%(path)s\" from \"%(this)s\" to \"%(other)s\"?"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:106
+#, python-format
+msgid "Rename \"%(this)s\" => \"%(other)s\"?"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:401
+msgid "No changes are shelved."
+msgstr ""
+
+#: bzrlib/shelf_ui.py:459
+#, python-format
+msgid "Using changes with id \"%d\"."
+msgstr ""
+
+#: bzrlib/shelf_ui.py:463
+#, python-format
+msgid "Message: %s"
+msgstr ""
+
+#: bzrlib/shelf_ui.py:475
+#, python-format
+msgid "Deleted changes with id \"%d\"."
+msgstr ""
+
+#: bzrlib/smart/medium.py:712
+msgid "HPSS calls: {0} ({1} vfs) {2}"
+msgstr ""
+
+#: bzrlib/smart/server.py:167
+msgid "Requested to stop gracefully"
+msgstr ""
+
+#: bzrlib/smart/server.py:177
+#, python-format
+msgid "Waiting for %d client(s) to finish"
+msgstr ""
+
+#: bzrlib/smart/server.py:183
+#, python-format
+msgid "Still waiting for %d client(s) to finish"
+msgstr ""
+
+#: bzrlib/smart/server.py:217
+#, python-format
+msgid "listening socket error: %s"
+msgstr ""
+
+#: bzrlib/smart/server.py:450
+#, python-format
+msgid "listening on port: %s"
+msgstr ""
+
+#: bzrlib/switch.py:67
+msgid "Pending merges must be committed or reverted before using switch."
+msgstr ""
+
+#: bzrlib/switch.py:94
+msgid ""
+"Cannot switch as local commits found in the checkout. Commit these to the "
+"bound branch or use --force to throw them away."
+msgstr ""
+
+#: bzrlib/switch.py:99
+#, python-format
+msgid ""
+"Unable to connect to current master branch %(target)s: %(error)s To switch "
+"anyway, use --force."
+msgstr ""
+
+#: bzrlib/switch.py:108
+msgid "Cannot switch a branch, only a checkout."
+msgstr ""
+
+#: bzrlib/switch.py:144
+#, python-format
+msgid "Tree is up to date at revision %d."
+msgstr ""
+
+#: bzrlib/switch.py:150
+#, python-format
+msgid "Updated to revision %d."
+msgstr ""
+
+#: bzrlib/transform.py:1723 bzrlib/transform.py:1734 bzrlib/transform.py:1736
+msgid "Apply phase"
+msgstr ""
+
+#: bzrlib/transform.py:1762 bzrlib/transform.py:1830
+msgid "removing file"
+msgstr ""
+
+#: bzrlib/transform.py:1780 bzrlib/transform.py:1865
+msgid "adding file"
+msgstr ""
+
+#: bzrlib/transform.py:2580
+msgid "Building tree"
+msgstr ""
+
+#: bzrlib/transform.py:2670 bzrlib/transform.py:2697
+msgid "Adding file contents"
+msgstr ""
+
+#: bzrlib/transform.py:3017
+msgid "Resolution pass"
+msgstr ""
+
+#: bzrlib/transport/ftp/_gssapi.py:86
+#, python-format
+msgid "Authenticated as %s"
+msgstr ""
+
+#: bzrlib/tree.py:632
+msgid "*** {0} content-filter: {1} => {2!r}"
+msgstr ""
+
+#: bzrlib/upgrade.py:76
+#, python-format
+msgid "This is a checkout. The branch (%s) needs to be upgraded separately."
+msgstr ""
+
+#: bzrlib/upgrade.py:99
+#, python-format
+msgid "cannot upgrade from bzrdir format %s"
+msgstr ""
+
+#: bzrlib/upgrade.py:102
+#, python-format
+msgid "starting upgrade of %s"
+msgstr ""
+
+#: bzrlib/upgrade.py:109
+msgid "finished"
+msgstr ""
+
+#: bzrlib/upgrade.py:119
+msgid "Deleting backup.bzr"
+msgstr ""
+
+#: bzrlib/upgrade.py:150
+msgid ""
+"\n"
+"SUMMARY: {0} upgrades attempted, {1} succeeded, {2} failed"
+msgstr ""
+
+#: bzrlib/upgrade.py:207
+#, python-format
+msgid "Found %d dependent branches - upgrading ..."
+msgstr ""
+
+#: bzrlib/upgrade.py:275
+msgid "Upgrading bzrdirs"
+msgstr ""
+
+#: bzrlib/upgrade.py:281
+#, python-format
+msgid "Upgrading %s"
+msgstr ""
+
+#: bzrlib/upgrade.py:282
+msgid "Upgrading {0} {1} ..."
+msgstr ""
+
+#: bzrlib/upgrade.py:300
+msgid "Removing backup ..."
+msgstr ""
+
+#: bzrlib/upgrade.py:304
+msgid "failed to clean-up {0}: {1}"
+msgstr ""
+
+#: bzrlib/vf_repository.py:1093
+msgid "inventories"
+msgstr ""
+
+#: bzrlib/vf_repository.py:1119
+msgid "texts"
+msgstr ""
+
+#: bzrlib/vf_repository.py:1624 bzrlib/vf_repository.py:1633
+msgid "Calculating text parents"
+msgstr ""
+
+#: bzrlib/vf_repository.py:1709
+msgid "Fetch texts"
+msgstr ""
+
+#: bzrlib/vf_repository.py:2538
+msgid "loading text store"
+msgstr ""
+
+#: bzrlib/vf_repository.py:2544
+msgid "checking text graph"
+msgstr ""
+
+#: bzrlib/vf_repository.py:2998 bzrlib/vf_repository.py:3012
+#: bzrlib/vf_repository.py:3127
+msgid "Transferring revisions"
+msgstr ""
+
+#: bzrlib/win32utils.py:137
+msgid "Failed to GetProcessMemoryInfo()"
+msgstr ""
+
+#: bzrlib/win32utils.py:158
+msgid "Cannot debug memory on win32 without ctypes or win32process"
+msgstr ""
+
+#: bzrlib/win32utils.py:163
+msgid "WorkingSize {0:>7}KiB\tPeakWorking {1:>7}KiB\t{2}"
+msgstr ""
+
+#: bzrlib/win32utils.py:171
+#, python-format
+msgid "WorkingSize       %8d KiB"
+msgstr ""
+
+#: bzrlib/win32utils.py:172
+#, python-format
+msgid "PeakWorking       %8d KiB"
+msgstr ""
+
+#: bzrlib/win32utils.py:173
+#, python-format
+msgid "PagefileUsage     %8d KiB"
+msgstr ""
+
+#: bzrlib/win32utils.py:174
+#, python-format
+msgid "PeakPagefileUsage %8d KiB"
+msgstr ""
+
+#: bzrlib/win32utils.py:176
+#, python-format
+msgid "PrivateUsage      %8d KiB"
+msgstr ""
+
+#: bzrlib/win32utils.py:177
+#, python-format
+msgid "PageFaultCount    %8d"
+msgstr ""
+
+#: bzrlib/workingtree.py:1467
+msgid "Rerun update after fixing the conflicts."
+msgstr ""
+
+#: dummy/help_topics/authentication/summary.txt:1
+msgid "Information on configuring authentication"
+msgstr ""
+
+#: dummy/help_topics/basic/detail.txt:1
+msgid ""
+"Bazaar 2.5b4 -- a free distributed version-control tool\n"
+"http://bazaar.canonical.com/"
+msgstr ""
+
+#: dummy/help_topics/basic/detail.txt:4
+msgid ""
+"Basic commands:\n"
+"  bzr init           makes this directory a versioned branch\n"
+"  bzr branch         make a copy of another branch"
+msgstr ""
+
+#: dummy/help_topics/basic/detail.txt:8
+msgid ""
+"  bzr add            make files or directories versioned\n"
+"  bzr ignore         ignore a file or pattern\n"
+"  bzr mv             move or rename a versioned file"
+msgstr ""
+
+#: dummy/help_topics/basic/detail.txt:12
+msgid ""
+"  bzr status         summarize changes in working copy\n"
+"  bzr diff           show detailed diffs"
+msgstr ""
+
+#: dummy/help_topics/basic/detail.txt:15
+msgid ""
+"  bzr merge          pull in changes from another branch\n"
+"  bzr commit         save some or all changes\n"
+"  bzr send           send changes via email"
+msgstr ""
+
+#: dummy/help_topics/basic/detail.txt:19
+msgid ""
+"  bzr log            show history of changes\n"
+"  bzr check          validate storage"
+msgstr ""
+
+#: dummy/help_topics/basic/detail.txt:22
+msgid ""
+"  bzr help init      more help on e.g. init command\n"
+"  bzr help commands  list all commands\n"
+"  bzr help topics    list all help topics\n"
+msgstr ""
+
+#: dummy/help_topics/basic/summary.txt:1
+msgid "Basic commands"
+msgstr ""
+
+#: dummy/help_topics/branches/detail.txt:1
+msgid "Branches"
+msgstr ""
+
+#: dummy/help_topics/branches/detail.txt:3
+msgid ""
+"A branch consists of the state of a project, including all of its\n"
+"history. All branches have a repository associated (which is where the\n"
+"branch history is stored), but multiple branches may share the same\n"
+"repository (a shared repository). Branches can be copied and merged."
+msgstr ""
+
+#: dummy/help_topics/branches/detail.txt:8
+msgid ""
+"In addition, one branch may be bound to another one.  Binding to another\n"
+"branch indicates that commits which happen in this branch must also \n"
+"happen in the other branch.  Bazaar ensures consistency by not allowing \n"
+"commits when the two branches are out of date.  In order for a commit \n"
+"to succeed, it may be necessary to update the current branch using \n"
+"``bzr update``."
+msgstr ""
+
+#: dummy/help_topics/branches/detail.txt:15
+msgid "Related commands::"
+msgstr ""
+
+#: dummy/help_topics/branches/detail.txt:17
+msgid ""
+"  init    Change a directory into a versioned branch.\n"
+"  branch  Create a new branch that is a copy of an existing branch.\n"
+"  merge   Perform a three-way merge.\n"
+"  bind    Bind a branch to another one.\n"
+msgstr ""
+
+#: dummy/help_topics/branches/summary.txt:1
+msgid "Information on what a branch is"
+msgstr ""
+
+#: dummy/help_topics/bugs/summary.txt:1
+msgid "Bug tracker settings"
+msgstr ""
+
+#: dummy/help_topics/checkouts/detail.txt:1
+msgid "Checkouts"
+msgstr ""
+
+#: dummy/help_topics/checkouts/detail.txt:3
+msgid ""
+"Checkouts are source trees that are connected to a branch, so that when\n"
+"you commit in the source tree, the commit goes into that branch.  They\n"
+"allow you to use a simpler, more centralized workflow, ignoring some of\n"
+"Bazaar's decentralized features until you want them. Using checkouts\n"
+"with shared repositories is very similar to working with SVN or CVS, but\n"
+"doesn't have the same restrictions.  And using checkouts still allows\n"
+"others working on the project to use whatever workflow they like."
+msgstr ""
+
+#: dummy/help_topics/checkouts/detail.txt:11
+msgid ""
+"A checkout is created with the bzr checkout command (see \"help checkout"
+"\").\n"
+"You pass it a reference to another branch, and it will create a local copy\n"
+"for you that still contains a reference to the branch you created the\n"
+"checkout from (the master branch). Then if you make any commits they will "
+"be\n"
+"made on the other branch first. This creates an instant mirror of your work, "
+"or\n"
+"facilitates lockstep development, where each developer is working together,\n"
+"continuously integrating the changes of others."
+msgstr ""
+
+#: dummy/help_topics/checkouts/detail.txt:19
+msgid ""
+"However the checkout is still a first class branch in Bazaar terms, so that\n"
+"you have the full history locally.  As you have a first class branch you "
+"can\n"
+"also commit locally if you want, for instance due to the temporary loss af "
+"a\n"
+"network connection. Use the --local option to commit to do this. All the "
+"local\n"
+"commits will then be made on the master branch the next time you do a non-"
+"local\n"
+"commit."
+msgstr ""
+
+#: dummy/help_topics/checkouts/detail.txt:26
+msgid ""
+"If you are using a checkout from a shared branch you will periodically want "
+"to\n"
+"pull in all the changes made by others. This is done using the \"update\"\n"
+"command. The changes need to be applied before any non-local commit, but\n"
+"Bazaar will tell you if there are any changes and suggest that you use this\n"
+"command when needed."
+msgstr ""
+
+#: dummy/help_topics/checkouts/detail.txt:32
+msgid ""
+"It is also possible to create a \"lightweight\" checkout by passing the\n"
+"--lightweight flag to checkout. A lightweight checkout is even closer to an\n"
+"SVN checkout in that it is not a first class branch, it mainly consists of "
+"the\n"
+"working tree. This means that any history operations must query the master\n"
+"branch, which could be slow if a network connection is involved. Also, as "
+"you\n"
+"don't have a local branch, then you cannot commit locally."
+msgstr ""
+
+#: dummy/help_topics/checkouts/detail.txt:39
+msgid ""
+"Lightweight checkouts work best when you have fast reliable access to the\n"
+"master branch. This means that if the master branch is on the same disk or "
+"LAN\n"
+"a lightweight checkout will be faster than a heavyweight one for any "
+"commands\n"
+"that modify the revision history (as only one copy of the branch needs to\n"
+"be updated). Heavyweight checkouts will generally be faster for any command\n"
+"that uses the history but does not change it, but if the master branch is "
+"on\n"
+"the same disk then there won't be a noticeable difference."
+msgstr ""
+
+#: dummy/help_topics/checkouts/detail.txt:47
+msgid ""
+"Another possible use for a checkout is to use it with a treeless repository\n"
+"containing your branches, where you maintain only one working tree by\n"
+"switching the master branch that the checkout points to when you want to\n"
+"work on a different branch."
+msgstr ""
+
+#: dummy/help_topics/checkouts/detail.txt:52
+msgid ""
+"Obviously to commit on a checkout you need to be able to write to the "
+"master\n"
+"branch. This means that the master branch must be accessible over a "
+"writeable\n"
+"protocol , such as sftp://, and that you have write permissions at the "
+"other\n"
+"end. Checkouts also work on the local file system, so that all that matters "
+"is\n"
+"file permissions."
+msgstr ""
+
+#: dummy/help_topics/checkouts/detail.txt:58
+msgid ""
+"You can change the master of a checkout by using the \"switch\" command "
+"(see\n"
+"\"help switch\").  This will change the location that the commits are sent "
+"to.\n"
+"The \"bind\" command can also be used to turn a normal branch into a heavy\n"
+"checkout. If you would like to convert your heavy checkout into a normal\n"
+"branch so that every commit is local, you can use the \"unbind\" command. To "
+"see\n"
+"whether or not a branch is bound or not you can use the \"info\" command. If "
+"the\n"
+"branch is bound it will tell you the location of the bound branch."
+msgstr ""
+
+#: dummy/help_topics/checkouts/detail.txt:68
+msgid ""
+"  checkout    Create a checkout. Pass --lightweight to get a lightweight\n"
+"              checkout\n"
+"  update      Pull any changes in the master branch in to your checkout\n"
+"  commit      Make a commit that is sent to the master branch. If you have\n"
+"              a heavy checkout then the --local option will commit to the\n"
+"              checkout without sending the commit to the master\n"
+"  switch      Change the master branch that the commits in the checkout "
+"will\n"
+"              be sent to\n"
+"  bind        Turn a standalone branch into a heavy checkout so that any\n"
+"              commits will be sent to the master branch\n"
+"  unbind      Turn a heavy checkout into a standalone branch so that any\n"
+"              commits are only made locally\n"
+"  info        Displays whether a branch is bound or unbound. If the branch "
+"is\n"
+"              bound, then it will also display the location of the bound "
+"branch\n"
+msgstr ""
+
+#: dummy/help_topics/checkouts/summary.txt:1
+msgid "Information on what a checkout is"
+msgstr ""
+
+#: dummy/help_topics/commands/summary.txt:1
+msgid "Basic help for all commands"
+msgstr ""
+
+#: dummy/help_topics/configuration/summary.txt:1
+msgid "Details on the configuration settings available"
+msgstr ""
+
+#: dummy/help_topics/conflict-types/summary.txt:1
+msgid "Types of conflicts and what to do about them"
+msgstr ""
+
+#: dummy/help_topics/content-filters/summary.txt:1
+msgid "Conversion of content into/from working trees"
+msgstr ""
+
+#: dummy/help_topics/criss-cross/detail.txt:1
+msgid "Criss-Cross"
+msgstr ""
+
+#: dummy/help_topics/criss-cross/detail.txt:3
+msgid ""
+"A criss-cross in the branch history can cause the default merge technique\n"
+"to emit more conflicts than would normally be expected."
+msgstr ""
+
+#: dummy/help_topics/criss-cross/detail.txt:6
+msgid ""
+"In complex merge cases, ``bzr merge --lca`` or ``bzr merge --weave`` may "
+"give\n"
+"better results.  You may wish to ``bzr revert`` the working tree and merge\n"
+"again.  Alternatively, use ``bzr remerge`` on particular conflicted files."
+msgstr ""
+
+#: dummy/help_topics/criss-cross/detail.txt:10
+msgid ""
+"Criss-crosses occur in a branch's history if two branches merge the same "
+"thing\n"
+"and then merge one another, or if two branches merge one another at the "
+"same\n"
+"time.  They can be avoided by having each branch only merge from or into a\n"
+"designated central branch (a \"star topology\")."
+msgstr ""
+
+#: dummy/help_topics/criss-cross/detail.txt:15
+msgid ""
+"Criss-crosses cause problems because of the way merge works.  Bazaar's "
+"default\n"
+"merge is a three-way merger; in order to merge OTHER into THIS, it must\n"
+"find a basis for comparison, BASE.  Using BASE, it can determine whether\n"
+"differences between THIS and OTHER are due to one side adding lines, or\n"
+"from another side removing lines."
+msgstr ""
+
+#: dummy/help_topics/criss-cross/detail.txt:21
+msgid ""
+"Criss-crosses mean there is no good choice for a base.  Selecting the "
+"recent\n"
+"merge points could cause one side's changes to be silently discarded.\n"
+"Selecting older merge points (which Bazaar does) mean that extra conflicts\n"
+"are emitted."
+msgstr ""
+
+#: dummy/help_topics/criss-cross/detail.txt:26
+msgid ""
+"The ``weave`` merge type is not affected by this problem because it uses\n"
+"line-origin detection instead of a basis revision to determine the cause of\n"
+"differences.\n"
+msgstr ""
+
+#: dummy/help_topics/criss-cross/summary.txt:1
+msgid "Information on criss-cross merging"
+msgstr ""
+
+#: dummy/help_topics/current-formats/summary.txt:1
+msgid "Current storage formats"
+msgstr ""
+
+#: dummy/help_topics/debug-flags/summary.txt:1
+msgid "Options to show or record debug information"
+msgstr ""
+
+#: dummy/help_topics/diverged-branches/summary.txt:1
+msgid "How to fix diverged branches"
+msgstr ""
+
+#: dummy/help_topics/env-variables/detail.txt:1
+msgid "Environment Variables"
+msgstr ""
+
+#: dummy/help_topics/env-variables/detail.txt:3
+msgid ""
+"=================== "
+"===========================================================\n"
+"BZRPATH             Path where bzr is to look for shell plugin external\n"
+"                    commands.\n"
+"BZR_EMAIL           E-Mail address of the user. Overrides EMAIL.\n"
+"EMAIL               E-Mail address of the user.\n"
+"BZR_EDITOR          Editor for editing commit messages. Overrides EDITOR.\n"
+"EDITOR              Editor for editing commit messages.\n"
+"BZR_PLUGIN_PATH     Paths where bzr should look for plugins.\n"
+"BZR_DISABLE_PLUGINS Plugins that bzr should not load.\n"
+"BZR_PLUGINS_AT      Plugins to load from a directory not in "
+"BZR_PLUGIN_PATH.\n"
+"BZR_HOME            Directory holding .bazaar config dir. Overrides HOME.\n"
+"BZR_HOME (Win32)    Directory holding bazaar config dir. Overrides APPDATA "
+"and\n"
+"                    HOME.\n"
+"BZR_REMOTE_PATH     Full name of remote 'bzr' command (for bzr+ssh:// "
+"URLs).\n"
+"BZR_SSH             Path to SSH client, or one of paramiko, openssh, "
+"sshcorp,\n"
+"                    plink or lsh.\n"
+"BZR_LOG             Location of .bzr.log (use '/dev/null' to suppress log).\n"
+"BZR_LOG (Win32)     Location of .bzr.log (use 'NUL' to suppress log).\n"
+"BZR_COLUMNS         Override implicit terminal width.\n"
+"BZR_CONCURRENCY     Number of processes that can be run concurrently "
+"(selftest)\n"
+"BZR_PROGRESS_BAR    Override the progress display. Values are 'none' or "
+"'text'.\n"
+"BZR_PDB             Control whether to launch a debugger on error.\n"
+"BZR_SIGQUIT_PDB     Control whether SIGQUIT behaves normally or invokes a\n"
+"                    breakin debugger.\n"
+"BZR_TEXTUI_INPUT    Force console input mode for prompts to line-based "
+"(instead\n"
+"                    of char-based).\n"
+"=================== "
+"===========================================================\n"
+msgstr ""
+
+#: dummy/help_topics/env-variables/summary.txt:1
+msgid "Environment variable names and values"
+msgstr ""
+
+#: dummy/help_topics/eol/summary.txt:1
+msgid "Information on end-of-line handling"
+msgstr ""
+
+#: dummy/help_topics/files/detail.txt:1
+msgid "Files"
+msgstr ""
+
+#: dummy/help_topics/files/detail.txt:3
+msgid ""
+":On Unix:   ~/.bazaar/bazaar.conf\n"
+":On Windows: C:\\\\Documents and Settings\\\\username\\\\Application Data\\"
+"\\bazaar\\\\2.0\\\\bazaar.conf"
+msgstr ""
+
+#: dummy/help_topics/files/detail.txt:6
+msgid ""
+"Contains the user's default configuration. The section ``[DEFAULT]`` is\n"
+"used to define general configuration that will be applied everywhere.\n"
+"The section ``[ALIASES]`` can be used to create command aliases for\n"
+"commonly used options."
+msgstr ""
+
+#: dummy/help_topics/files/detail.txt:11
+msgid "A typical config file might look something like::"
+msgstr ""
+
+#: dummy/help_topics/files/detail.txt:13
+msgid ""
+"  [DEFAULT]\n"
+"  email=John Doe <jdoe@isp.com>"
+msgstr ""
+
+#: dummy/help_topics/files/detail.txt:16
+msgid ""
+"  [ALIASES]\n"
+"  commit = commit --strict\n"
+"  log10 = log --short -r -10..-1\n"
+msgstr ""
+
+#: dummy/help_topics/files/summary.txt:1
+msgid "Information on configuration and log files"
+msgstr ""
+
+#: dummy/help_topics/formats/detail.txt:1
+msgid "Storage Formats"
+msgstr ""
+
+#: dummy/help_topics/formats/detail.txt:3
+msgid ""
+"To ensure that older clients do not access data incorrectly,\n"
+"Bazaar's policy is to introduce a new storage format whenever\n"
+"new features requiring new metadata are added. New storage\n"
+"formats may also be introduced to improve performance and\n"
+"scalability."
+msgstr ""
+
+#: dummy/help_topics/formats/detail.txt:9
+msgid ""
+"The newest format, 2a, is highly recommended. If your\n"
+"project is not using 2a, then you should suggest to the\n"
+"project owner to upgrade."
+msgstr ""
+
+#: dummy/help_topics/formats/detail.txt:13
+msgid ""
+"\n"
+".. note::"
+msgstr ""
+
+#: dummy/help_topics/formats/detail.txt:16
+msgid ""
+"   Some of the older formats have two variants:\n"
+"   a plain one and a rich-root one. The latter include an additional\n"
+"   field about the root of the tree. There is no performance cost\n"
+"   for using a rich-root format but you cannot easily merge changes\n"
+"   from a rich-root format into a plain format. As a consequence,\n"
+"   moving a project to a rich-root format takes some co-ordination\n"
+"   in that all contributors need to upgrade their repositories\n"
+"   around the same time. 2a and all future formats will be\n"
+"   implicitly rich-root."
+msgstr ""
+
+#: dummy/help_topics/formats/detail.txt:26
+msgid ""
+"See :doc:`current-formats-help` for the complete list of\n"
+"currently supported formats. See :doc:`other-formats-help` for\n"
+"descriptions of any available experimental and deprecated formats.\n"
+msgstr ""
+
+#: dummy/help_topics/formats/summary.txt:1
+msgid "Information on choosing a storage format"
+msgstr ""
+
+#: dummy/help_topics/global-options/detail.txt:1
+msgid "Global Options"
+msgstr ""
+
+#: dummy/help_topics/global-options/detail.txt:3
+msgid ""
+"These options may be used with any command, and may appear in front of any\n"
+"command.  (e.g. ``bzr --profile help``)."
+msgstr ""
+
+#: dummy/help_topics/global-options/detail.txt:6
+msgid ""
+"--version      Print the version number. Must be supplied before the "
+"command.\n"
+"--no-aliases   Do not process command aliases when running this command.\n"
+"--builtin      Use the built-in version of a command, not the plugin "
+"version.\n"
+"               This does not suppress other plugin effects.\n"
+"--no-plugins   Do not process any plugins.\n"
+"--no-l10n      Do not translate messages.\n"
+"--concurrency  Number of processes that can be run concurrently (selftest)."
+msgstr ""
+
+#: dummy/help_topics/global-options/detail.txt:14
+msgid ""
+"--profile      Profile execution using the hotshot profiler.\n"
+"--lsprof       Profile execution using the lsprof profiler.\n"
+"--lsprof-file  Profile execution using the lsprof profiler, and write the\n"
+"               results to a specified file.  If the filename ends with \".txt"
+"\",\n"
+"               text format will be used.  If the filename either starts "
+"with\n"
+"               \"callgrind.out\" or end with \".callgrind\", the output will "
+"be\n"
+"               formatted for use with KCacheGrind. Otherwise, the output\n"
+"               will be a pickle.\n"
+"--coverage     Generate line coverage report in the specified directory."
+msgstr ""
+
+#: dummy/help_topics/global-options/detail.txt:24
+msgid ""
+"-Oname=value   Override the ``name`` config option setting it to ``value`` "
+"for\n"
+"               the duration of the command.  This can be used multiple times "
+"if\n"
+"               several options need to be overridden."
+msgstr ""
+
+#: dummy/help_topics/global-options/detail.txt:28
+msgid ""
+"See http://doc.bazaar.canonical.com/developers/profiling.html for more\n"
+"information on profiling."
+msgstr ""
+
+#: dummy/help_topics/global-options/detail.txt:31
+msgid ""
+"A number of debug flags are also available to assist troubleshooting and\n"
+"development.  See :doc:`debug-flags-help`.\n"
+msgstr ""
+
+#: dummy/help_topics/global-options/summary.txt:1
+msgid "Options that control how Bazaar runs"
+msgstr ""
+
+#: dummy/help_topics/hidden-commands/summary.txt:1
+msgid "All hidden commands"
+msgstr ""
+
+#: dummy/help_topics/hooks/summary.txt:1
+msgid "Points at which custom processing can be added"
+msgstr ""
+
+#: dummy/help_topics/launchpad/detail.txt:1
+msgid "Integration with Launchpad.net"
+msgstr ""
+
+#: dummy/help_topics/launchpad/detail.txt:3
+msgid ""
+"Launchpad.net provides free Bazaar branch hosting with integrated bug and\n"
+"specification tracking."
+msgstr ""
+
+#: dummy/help_topics/launchpad/detail.txt:6
+msgid ""
+"The bzr client (through the plugin called 'launchpad') has special\n"
+"features to communicate with Launchpad:"
+msgstr ""
+
+#: dummy/help_topics/launchpad/detail.txt:9
+msgid ""
+"    * The launchpad-login command tells Bazaar your Launchpad user name. "
+"This\n"
+"      is then used by the 'lp:' transport to download your branches using\n"
+"      bzr+ssh://."
+msgstr ""
+
+#: dummy/help_topics/launchpad/detail.txt:13
+msgid ""
+"    * The 'lp:' transport uses Launchpad as a directory service: for "
+"example\n"
+"      'lp:bzr' and 'lp:python' refer to the main branches of the relevant\n"
+"      projects and may be branched, logged, etc. You can also use the 'lp:'\n"
+"      transport to refer to specific branches, e.g. lp:~bzr/bzr/trunk."
+msgstr ""
+
+#: dummy/help_topics/launchpad/detail.txt:18
+msgid ""
+"    * The 'lp:' bug tracker alias can expand launchpad bug numbers to their\n"
+"      URLs for use with 'bzr commit --fixes', e.g. 'bzr commit --fixes "
+"lp:12345'\n"
+"      will record a revision property that marks that revision as fixing\n"
+"      Launchpad bug 12345. When you push that branch to Launchpad it will\n"
+"      automatically be linked to the bug report."
+msgstr ""
+
+#: dummy/help_topics/launchpad/detail.txt:24
+msgid ""
+"    * The register-branch command tells Launchpad about the url of a\n"
+"      public branch.  Launchpad will then mirror the branch, display\n"
+"      its contents and allow it to be attached to bugs and other\n"
+"      objects."
+msgstr ""
+
+#: dummy/help_topics/launchpad/detail.txt:29
+msgid "For more information see http://help.launchpad.net/\n"
+msgstr ""
+
+#: dummy/help_topics/launchpad/summary.txt:1
+msgid "Using Bazaar with Launchpad.net"
+msgstr ""
+
+#: dummy/help_topics/location-alias/summary.txt:1
+msgid "Aliases for remembered locations"
+msgstr ""
+
+#: dummy/help_topics/log-formats/summary.txt:1
+msgid "Details on the logging formats available"
+msgstr ""
+
+#: dummy/help_topics/other-formats/summary.txt:1
+msgid "Experimental and deprecated storage formats"
+msgstr ""
+
+#: dummy/help_topics/patterns/summary.txt:1
+msgid "Information on the pattern syntax"
+msgstr ""
+
+#: dummy/help_topics/repositories/detail.txt:1
+msgid "Repositories"
+msgstr ""
+
+#: dummy/help_topics/repositories/detail.txt:3
+msgid ""
+"Repositories in Bazaar are where committed information is stored. There is\n"
+"a repository associated with every branch."
+msgstr ""
+
+#: dummy/help_topics/repositories/detail.txt:6
+msgid ""
+"Repositories are a form of database. Bzr will usually maintain this for\n"
+"good performance automatically, but in some situations (e.g. when doing\n"
+"very many commits in a short time period) you may want to ask bzr to\n"
+"optimise the database indices. This can be done by the 'bzr pack' command."
+msgstr ""
+
+#: dummy/help_topics/repositories/detail.txt:11
+msgid ""
+"By default just running 'bzr init' will create a repository within the new\n"
+"branch but it is possible to create a shared repository which allows "
+"multiple\n"
+"branches to share their information in the same location. When a new branch "
+"is\n"
+"created it will first look to see if there is a containing shared repository "
+"it\n"
+"can use."
+msgstr ""
+
+#: dummy/help_topics/repositories/detail.txt:17
+msgid ""
+"When two branches of the same project share a repository, there is\n"
+"generally a large space saving. For some operations (e.g. branching\n"
+"within the repository) this translates in to a large time saving."
+msgstr ""
+
+#: dummy/help_topics/repositories/detail.txt:21
+msgid ""
+"To create a shared repository use the init-repository command (or the alias\n"
+"init-repo). This command takes the location of the repository to create. "
+"This\n"
+"means that 'bzr init-repository repo' will create a directory named 'repo',\n"
+"which contains a shared repository. Any new branches that are created in "
+"this\n"
+"directory will then use it for storage."
+msgstr ""
+
+#: dummy/help_topics/repositories/detail.txt:27
+msgid ""
+"It is a good idea to create a repository whenever you might create more\n"
+"than one branch of a project. This is true for both working areas where you\n"
+"are doing the development, and any server areas that you use for hosting\n"
+"projects. In the latter case, it is common to want branches without working\n"
+"trees. Since the files in the branch will not be edited directly there is "
+"no\n"
+"need to use up disk space for a working tree. To create a repository in "
+"which\n"
+"the branches will not have working trees pass the '--no-trees' option to\n"
+"'init-repository'."
+msgstr ""
+
+#: dummy/help_topics/repositories/detail.txt:38
+msgid ""
+"  init-repository   Create a shared repository. Use --no-trees to create "
+"one\n"
+"                    in which new branches won't get a working tree.\n"
+msgstr ""
+
+#: dummy/help_topics/repositories/summary.txt:1
+msgid "Basic information on shared repositories."
+msgstr ""
+
+#: dummy/help_topics/revisionspec/summary.txt:1
+msgid "Explain how to use --revision"
+msgstr ""
+
+#: dummy/help_topics/rules/summary.txt:1
+msgid "Information on defining rule-based preferences"
+msgstr ""
+
+#: dummy/help_topics/standalone-trees/detail.txt:1
+msgid "Standalone Trees"
+msgstr ""
+
+#: dummy/help_topics/standalone-trees/detail.txt:3
+msgid ""
+"A standalone tree is a working tree with an associated repository. It\n"
+"is an independently usable branch, with no dependencies on any other.\n"
+"Creating a standalone tree (via bzr init) is the quickest way to put\n"
+"an existing project under version control."
+msgstr ""
+
+#: dummy/help_topics/standalone-trees/detail.txt:8
+msgid "Related Commands::"
+msgstr ""
+
+#: dummy/help_topics/standalone-trees/detail.txt:10
+msgid "  init    Make a directory into a versioned branch.\n"
+msgstr ""
+
+#: dummy/help_topics/standalone-trees/summary.txt:1
+msgid "Information on what a standalone tree is"
+msgstr ""
+
+#: dummy/help_topics/standard-options/detail.txt:1
+msgid "Standard Options"
+msgstr ""
+
+#: dummy/help_topics/standard-options/detail.txt:3
+msgid "Standard options are legal for all commands."
+msgstr ""
+
+#: dummy/help_topics/standard-options/detail.txt:5
+msgid ""
+"--help, -h     Show help message.\n"
+"--verbose, -v  Display more information.\n"
+"--quiet, -q    Only display errors and warnings."
+msgstr ""
+
+#: dummy/help_topics/standard-options/detail.txt:9
+msgid "Unlike global options, standard options can be used in aliases.\n"
+msgstr ""
+
+#: dummy/help_topics/standard-options/summary.txt:1
+msgid "Options that can be used with any command"
+msgstr ""
+
+#: dummy/help_topics/status-flags/detail.txt:1
+msgid "Status Flags"
+msgstr ""
+
+#: dummy/help_topics/status-flags/detail.txt:3
+msgid ""
+"Status flags are used to summarise changes to the working tree in a concise\n"
+"manner.  They are in the form::"
+msgstr ""
+
+#: dummy/help_topics/status-flags/detail.txt:6
+msgid "   xxx   <filename>"
+msgstr ""
+
+#: dummy/help_topics/status-flags/detail.txt:8
+msgid "where the columns' meanings are as follows."
+msgstr ""
+
+#: dummy/help_topics/status-flags/detail.txt:10
+msgid "Column 1 - versioning/renames::"
+msgstr ""
+
+#: dummy/help_topics/status-flags/detail.txt:12
+msgid ""
+"  + File versioned\n"
+"  - File unversioned\n"
+"  R File renamed\n"
+"  ? File unknown\n"
+"  X File nonexistent (and unknown to bzr)\n"
+"  C File has conflicts\n"
+"  P Entry for a pending merge (not a file)"
+msgstr ""
+
+#: dummy/help_topics/status-flags/detail.txt:20
+msgid "Column 2 - contents::"
+msgstr ""
+
+#: dummy/help_topics/status-flags/detail.txt:22
+msgid ""
+"  N File created\n"
+"  D File deleted\n"
+"  K File kind changed\n"
+"  M File modified"
+msgstr ""
+
+#: dummy/help_topics/status-flags/detail.txt:27
+msgid "Column 3 - execute::"
+msgstr ""
+
+#: dummy/help_topics/status-flags/detail.txt:29
+msgid "  * The execute bit was changed\n"
+msgstr ""
+
+#: dummy/help_topics/status-flags/summary.txt:1
+msgid "Help on status flags"
+msgstr ""
+
+#: dummy/help_topics/sync-for-reconfigure/detail.txt:1
+msgid "Branches Out of Sync"
+msgstr ""
+
+#: dummy/help_topics/sync-for-reconfigure/detail.txt:3
+msgid ""
+"When reconfiguring a checkout, tree or branch into a lightweight checkout,\n"
+"a local branch must be destroyed.  (For checkouts, this is the local branch\n"
+"that serves primarily as a cache.)  If the branch-to-be-destroyed does not\n"
+"have the same last revision as the new reference branch for the lightweight\n"
+"checkout, data could be lost, so Bazaar refuses."
+msgstr ""
+
+#: dummy/help_topics/sync-for-reconfigure/detail.txt:9
+msgid "How you deal with this depends on *why* the branches are out of sync."
+msgstr ""
+
+#: dummy/help_topics/sync-for-reconfigure/detail.txt:11
+msgid ""
+"If you have a checkout and have done local commits, you can get back in "
+"sync\n"
+"by running \"bzr update\" (and possibly \"bzr commit\")."
+msgstr ""
+
+#: dummy/help_topics/sync-for-reconfigure/detail.txt:14
+msgid ""
+"If you have a branch and the remote branch is out-of-date, you can push\n"
+"the local changes using \"bzr push\".  If the local branch is out of date, "
+"you\n"
+"can do \"bzr pull\".  If both branches have had changes, you can merge, "
+"commit\n"
+"and then push your changes.  If you decide that some of the changes aren't\n"
+"useful, you can \"push --overwrite\" or \"pull --overwrite\" instead.\n"
+msgstr ""
+
+#: dummy/help_topics/sync-for-reconfigure/summary.txt:1
+msgid "Steps to resolve \"out-of-sync\" when reconfiguring"
+msgstr ""
+
+#: dummy/help_topics/topics/summary.txt:1
+msgid "Topics list"
+msgstr ""
+
+#: dummy/help_topics/url-special-chars/summary.txt:1
+msgid "Special character handling in URLs"
+msgstr ""
+
+#: dummy/help_topics/urlspec/summary.txt:1
+msgid "Supported transport protocols"
+msgstr ""
+
+#: dummy/help_topics/working-trees/detail.txt:1
+msgid "Working Trees"
+msgstr ""
+
+#: dummy/help_topics/working-trees/detail.txt:3
+msgid ""
+"A working tree is the contents of a branch placed on disk so that you can\n"
+"see the files and edit them. The working tree is where you make changes to "
+"a\n"
+"branch, and when you commit the current state of the working tree is the\n"
+"snapshot that is recorded in the commit."
+msgstr ""
+
+#: dummy/help_topics/working-trees/detail.txt:8
+msgid ""
+"When you push a branch to a remote system, a working tree will not be\n"
+"created. If one is already present the files will not be updated. The\n"
+"branch information will be updated and the working tree will be marked\n"
+"as out-of-date. Updating a working tree remotely is difficult, as there\n"
+"may be uncommitted changes or the update may cause content conflicts that "
+"are\n"
+"difficult to deal with remotely."
+msgstr ""
+
+#: dummy/help_topics/working-trees/detail.txt:15
+msgid ""
+"If you have a branch with no working tree you can use the 'checkout' "
+"command\n"
+"to create a working tree. If you run 'bzr checkout .' from the branch it "
+"will\n"
+"create the working tree. If the branch is updated remotely, you can update "
+"the\n"
+"working tree by running 'bzr update' in that directory."
+msgstr ""
+
+#: dummy/help_topics/working-trees/detail.txt:20
+msgid ""
+"If you have a branch with a working tree that you do not want the 'remove-"
+"tree'\n"
+"command will remove the tree if it is safe. This can be done to avoid the\n"
+"warning about the remote working tree not being updated when pushing to the\n"
+"branch. It can also be useful when working with a '--no-trees' repository\n"
+"(see 'bzr help repositories')."
+msgstr ""
+
+#: dummy/help_topics/working-trees/detail.txt:26
+msgid ""
+"If you want to have a working tree on a remote machine that you push to you\n"
+"can either run 'bzr update' in the remote branch after each push, or use "
+"some\n"
+"other method to update the tree during the push. There is an 'rspush' "
+"plugin\n"
+"that will update the working tree using rsync as well as doing a push. "
+"There\n"
+"is also a 'push-and-update' plugin that automates running 'bzr update' via "
+"SSH\n"
+"after each push."
+msgstr ""
+
+#: dummy/help_topics/working-trees/detail.txt:33
+msgid "Useful commands::"
+msgstr ""
+
+#: dummy/help_topics/working-trees/detail.txt:35
+msgid ""
+"  checkout     Create a working tree when a branch does not have one.\n"
+"  remove-tree  Removes the working tree from a branch when it is safe to do "
+"so.\n"
+"  update       When a working tree is out of sync with its associated "
+"branch\n"
+"               this will update the tree to match the branch.\n"
+msgstr ""
+
+#: dummy/help_topics/working-trees/summary.txt:1
+msgid "Information on working trees"
+msgstr ""
+
+#: en/help_topics/authentication.txt:1
+msgid ""
+"Authentication Settings\n"
+"======================="
+msgstr ""
+
+#: en/help_topics/authentication.txt:4
+msgid ""
+"\n"
+"Intent\n"
+"------"
+msgstr ""
+
+#: en/help_topics/authentication.txt:8
+msgid ""
+"Many different authentication policies can be described in the\n"
+"``authentication.conf`` file but a particular user should need only a few\n"
+"definitions to cover his needs without having to specify a user and a "
+"password\n"
+"for every branch he uses."
+msgstr ""
+
+#: en/help_topics/authentication.txt:13
+msgid ""
+"The definitions found in this file are used to find the credentials to use "
+"for\n"
+"a given url. The same credentials can generally be used for as many branches "
+"as\n"
+"possible by grouping their declaration around the remote servers that need\n"
+"them. It's even possible to declare credentials that will be used by "
+"different\n"
+"servers."
+msgstr ""
+
+#: en/help_topics/authentication.txt:19
+msgid ""
+"The intent is to make this file as small as possible to minimize maintenance."
+msgstr ""
+
+#: en/help_topics/authentication.txt:21
+msgid ""
+"Once the relevant credentials are declared in this file you may use branch "
+"urls\n"
+"without embedding passwords (security hazard) or even users (enabling "
+"sharing\n"
+"of your urls with others)."
+msgstr ""
+
+#: en/help_topics/authentication.txt:25
+msgid "Instead of using::"
+msgstr ""
+
+#: en/help_topics/authentication.txt:27
+msgid "  bzr branch ftp://joe:secret@host.com/path/to/my/branch"
+msgstr ""
+
+#: en/help_topics/authentication.txt:29
+msgid "you simply use::"
+msgstr ""
+
+#: en/help_topics/authentication.txt:31
+msgid "  bzr branch ftp://host.com/path/to/my/branch"
+msgstr ""
+
+#: en/help_topics/authentication.txt:33
+msgid "provided you have created the following ``authentication.conf`` file::"
+msgstr ""
+
+#: en/help_topics/authentication.txt:35
+msgid ""
+"  [myprojects]\n"
+"  scheme=ftp\n"
+"  host=host.com\n"
+"  user=joe\n"
+"  password=secret\n"
+"  "
+msgstr ""
+
+#: en/help_topics/authentication.txt:42
+msgid ""
+"Authentication definitions\n"
+"--------------------------"
+msgstr ""
+
+#: en/help_topics/authentication.txt:45
+msgid ""
+"There are two kinds of authentication used by the various schemes supported "
+"by\n"
+"bzr:"
+msgstr ""
+
+#: en/help_topics/authentication.txt:48
+msgid "1. user and password"
+msgstr ""
+
+#: en/help_topics/authentication.txt:50
+msgid ""
+"``FTP`` needs a (``user``, ``password``) to authenticate against a ``host``\n"
+"``SFTP`` can use either a password or a host key to authenticate. However,\n"
+"ssh agents are a better, more secure solution. So we have chosen to not "
+"provide\n"
+"our own less secure method."
+msgstr ""
+
+#: en/help_topics/authentication.txt:55
+msgid "2. user, realm and password"
+msgstr ""
+
+#: en/help_topics/authentication.txt:57
+msgid ""
+"``HTTP`` and ``HTTPS`` needs a (``user, realm, password``) to authenticate\n"
+"against a host. But, by using ``.htaccess`` files, for example, it is "
+"possible\n"
+"to define several (``user, realm, password``) for a given ``host``. So what "
+"is\n"
+"really needed is (``user``, ``password``, ``host``, ``path``). The ``realm`` "
+"is\n"
+"not taken into account in the definitions, but will displayed if bzr "
+"prompts\n"
+"you for a password."
+msgstr ""
+
+#: en/help_topics/authentication.txt:64
+msgid ""
+"``HTTP proxy`` can be handled as ``HTTP`` (or ``HTTPS``) by explicitly\n"
+"specifying the appropriate port."
+msgstr ""
+
+#: en/help_topics/authentication.txt:67
+msgid ""
+"To take all schemes into account, the password will be deduced from a set "
+"of\n"
+"authentication definitions (``scheme``, ``host``, ``port``, ``path``, "
+"``user``,\n"
+"``password``)."
+msgstr ""
+
+#: en/help_topics/authentication.txt:71
+msgid ""
+"  * ``scheme``: can be empty (meaning the rest of the definition can be "
+"used\n"
+"    for any scheme), ``SFTP`` and ``bzr+ssh`` should not be used here, "
+"``ssh``\n"
+"    should be used instead since this is the real scheme regarding\n"
+"    authentication,"
+msgstr ""
+
+#: en/help_topics/authentication.txt:76
+msgid "  * ``host``: can be empty (to act as a default for any host),"
+msgstr ""
+
+#: en/help_topics/authentication.txt:78
+msgid ""
+"  * ``port`` can be empty (useful when an host provides several servers for "
+"the\n"
+"    same scheme), only numerical values are allowed, this should be used "
+"only\n"
+"    when the server uses a port different than the scheme standard port,"
+msgstr ""
+
+#: en/help_topics/authentication.txt:82
+msgid "  * ``path``: can be empty (FTP or SFTP will never user it),"
+msgstr ""
+
+#: en/help_topics/authentication.txt:84
+msgid ""
+"  * ``user``: can be empty (``bzr`` will defaults to python's\n"
+"    ``getpass.get_user()``),"
+msgstr ""
+
+#: en/help_topics/authentication.txt:87
+msgid ""
+"  * ``password``: can be empty if you prefer to always be prompted for your\n"
+"    password."
+msgstr ""
+
+#: en/help_topics/authentication.txt:90
+msgid ""
+"Multiple definitions can be provided and, for a given URL, bzr will select "
+"a\n"
+"(``user`` [, ``password``]) based on the following rules :"
+msgstr ""
+
+#: en/help_topics/authentication.txt:93
+msgid " 1. the first match wins,"
+msgstr ""
+
+#: en/help_topics/authentication.txt:95
+msgid " 2. empty fields match everything,"
+msgstr ""
+
+#: en/help_topics/authentication.txt:97
+msgid ""
+" 3. ``scheme`` matches even if decorators are used in the requested URL,"
+msgstr ""
+
+#: en/help_topics/authentication.txt:99
+msgid ""
+" 4. ``host`` matches exactly or act as a domain if it starts with '.'\n"
+"    (``project.bzr.sf.net`` will match ``.bzr.sf.net`` but ``projectbzr.sf."
+"net``\n"
+"    will not match ``bzr.sf.net``)."
+msgstr ""
+
+#: en/help_topics/authentication.txt:103
+msgid ""
+" 5. ``port`` matches if included in the requested URL (exact matches only)"
+msgstr ""
+
+#: en/help_topics/authentication.txt:105
+msgid ""
+" 6. ``path`` matches if included in the requested URL (and by rule #2 "
+"above,\n"
+"    empty paths will match any provided path)."
+msgstr ""
+
+#: en/help_topics/authentication.txt:110
+msgid ""
+"File format\n"
+"-----------"
+msgstr ""
+
+#: en/help_topics/authentication.txt:113
+msgid ""
+"The general rules for :doc:`configuration files <configuration-help>`\n"
+"apply except for the variable policies."
+msgstr ""
+
+#: en/help_topics/authentication.txt:116
+msgid "Each section describes an authentication definition."
+msgstr ""
+
+#: en/help_topics/authentication.txt:118
+msgid ""
+"The section name is an arbitrary string, only the ``DEFAULT`` value is "
+"reserved\n"
+"and should appear as the *last* section."
+msgstr ""
+
+#: en/help_topics/authentication.txt:121
+msgid "Each section should define:"
+msgstr ""
+
+#: en/help_topics/authentication.txt:123
+msgid "* ``user``: the login to be used,"
+msgstr ""
+
+#: en/help_topics/authentication.txt:125
+msgid "Each section could define:"
+msgstr ""
+
+#: en/help_topics/authentication.txt:127
+msgid "* ``host``: the remote server,"
+msgstr ""
+
+#: en/help_topics/authentication.txt:129
+msgid "* ``port``: the port the server is listening,"
+msgstr ""
+
+#: en/help_topics/authentication.txt:131
+msgid "* ``path``: the branch location,"
+msgstr ""
+
+#: en/help_topics/authentication.txt:133
+msgid "* ``password``: the password."
+msgstr ""
+
+#: en/help_topics/authentication.txt:135
+msgid ""
+"\n"
+"Examples\n"
+"--------"
+msgstr ""
+
+#: en/help_topics/authentication.txt:139
+msgid ""
+"\n"
+"Personal projects hosted outside\n"
+"~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/authentication.txt:143
+msgid ""
+"All connections are done with the same ``user`` (the remote one for which "
+"the\n"
+"default bzr one is not appropriate) and the password is always prompted "
+"with\n"
+"some exceptions::"
+msgstr ""
+
+#: en/help_topics/authentication.txt:147
+msgid ""
+"        # Pet projects on hobby.net\n"
+"        [hobby]\n"
+"        host=r.hobby.net\n"
+"        user=jim\n"
+"        password=obvious1234\n"
+"        \n"
+"        # Home server\n"
+"        [home]\n"
+"        scheme=https\n"
+"        host=home.net\n"
+"        user=joe\n"
+"        password=1essobV10us\n"
+"        \n"
+"        [DEFAULT]\n"
+"        # Our local user is barbaz, on all remote sites we're known as "
+"foobar\n"
+"        user=foobar"
+msgstr ""
+
+#: en/help_topics/authentication.txt:164
+msgid ""
+"\n"
+"Source hosting provider\n"
+"~~~~~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/authentication.txt:168
+msgid "In the shp.net (fictitious) domain, each project has its own site::"
+msgstr ""
+
+#: en/help_topics/authentication.txt:170
+msgid ""
+"        [shpnet domain]\n"
+"        # we use sftp, but ssh is the scheme used for authentication\n"
+"        scheme=ssh\n"
+"        # The leading '.' ensures that 'shp.net' alone doesn't match\n"
+"        host=.shp.net\n"
+"        user=joe\n"
+"        # bzr don't support supplying a password for sftp,\n"
+"        # consider using an ssh agent if you don't want to supply\n"
+"        # a password interactively. (pageant, ssh-agent, etc)"
+msgstr ""
+
+#: en/help_topics/authentication.txt:180
+msgid ""
+"HTTPS, SFTP servers and their proxy\n"
+"~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/authentication.txt:183
+msgid ""
+"At company.com, the server hosting release and integration branches is "
+"behind a\n"
+"proxy, and the two branches use different authentication policies::"
+msgstr ""
+
+#: en/help_topics/authentication.txt:186
+msgid ""
+"        [reference code]\n"
+"        scheme=https\n"
+"        host=dev.company.com\n"
+"        path=/dev\n"
+"        user=user1\n"
+"        password=pass1"
+msgstr ""
+
+#: en/help_topics/authentication.txt:193
+msgid ""
+"        # development branches on dev server\n"
+"        [dev]\n"
+"        scheme=ssh # bzr+ssh and sftp are available here\n"
+"        host=dev.company.com\n"
+"        path=/dev/integration\n"
+"        user=user2\n"
+"        \n"
+"        # proxy\n"
+"        [proxy]\n"
+"        scheme=http\n"
+"        host=proxy.company.com\n"
+"        port=3128\n"
+"        user=proxyuser1\n"
+"        password=proxypass1"
+msgstr ""
+
+#: en/help_topics/authentication.txt:208
+msgid ""
+"\n"
+"Planned enhancements\n"
+"--------------------"
+msgstr ""
+
+#: en/help_topics/authentication.txt:212
+msgid ""
+"The following are not yet implemented but planned as parts of a work in\n"
+"progress:"
+msgstr ""
+
+#: en/help_topics/authentication.txt:215
+msgid "* add a  ``password_encoding`` field allowing:"
+msgstr ""
+
+#: en/help_topics/authentication.txt:217
+msgid ""
+"  - storing the passwords in various obfuscating encodings (base64 for one),"
+msgstr ""
+
+#: en/help_topics/authentication.txt:219
+msgid "  - delegate password storage to plugins (.netrc for example)."
+msgstr ""
+
+#: en/help_topics/authentication.txt:221
+msgid ""
+"* update the credentials when the user is prompted for user or password,"
+msgstr ""
+
+#: en/help_topics/authentication.txt:223
+msgid "* add a ``verify_certificates`` field for ``HTTPS``."
+msgstr ""
+
+#: en/help_topics/authentication.txt:225
+msgid ""
+"The ``password_encoding`` and ``verify_certificates`` fields are recognized "
+"but\n"
+"ignored in the actual implementation.\n"
+msgstr ""
+
+#: en/help_topics/bugs.txt:1
+msgid "Bug Tracker Settings"
+msgstr ""
+
+#: en/help_topics/bugs.txt:3
+msgid ""
+"When making a commit, metadata about bugs fixed by that change can be\n"
+"recorded by using the ``--fixes`` option. For each bug marked as fixed, an\n"
+"entry is included in the 'bugs' revision property stating '<url> <status>'.\n"
+"(The only ``status`` value currently supported is ``fixed.``)"
+msgstr ""
+
+#: en/help_topics/bugs.txt:8
+msgid ""
+"The ``--fixes`` option allows you to specify a bug tracker and a bug "
+"identifier\n"
+"rather than a full URL. This looks like::"
+msgstr ""
+
+#: en/help_topics/bugs.txt:11
+msgid "    bzr commit --fixes <tracker>:<id>"
+msgstr ""
+
+#: en/help_topics/bugs.txt:13
+msgid "or::"
+msgstr ""
+
+#: en/help_topics/bugs.txt:15
+msgid "    bzr commit --fixes <id>"
+msgstr ""
+
+#: en/help_topics/bugs.txt:17
+msgid ""
+"where \"<tracker>\" is an identifier for the bug tracker, and \"<id>\" is "
+"the\n"
+"identifier for that bug within the bugtracker, usually the bug number.\n"
+"If \"<tracker>\" is not specified the ``bugtracker`` set in the branch\n"
+"or global configuration is used."
+msgstr ""
+
+#: en/help_topics/bugs.txt:22
+msgid ""
+"Bazaar knows about a few bug trackers that have many users. If\n"
+"you use one of these bug trackers then there is no setup required to\n"
+"use this feature, you just need to know the tracker identifier to use.\n"
+"These are the bugtrackers that are built in:"
+msgstr ""
+
+#: en/help_topics/bugs.txt:27
+msgid ""
+"  ============================ ============ ============\n"
+"  URL                          Abbreviation Example\n"
+"  ============================ ============ ============\n"
+"  https://bugs.launchpad.net/  lp           lp:12345\n"
+"  http://bugs.debian.org/      deb          deb:12345\n"
+"  http://bugzilla.gnome.org/   gnome        gnome:12345\n"
+"  ============================ ============ ============"
+msgstr ""
+
+#: en/help_topics/bugs.txt:35
+msgid ""
+"For the bug trackers not listed above configuration is required.\n"
+"Support for generating the URLs for any project using Bugzilla or Trac\n"
+"is built in, along with a template mechanism for other bugtrackers with\n"
+"simple URL schemes. If your bug tracker can't be described by one\n"
+"of the schemes described below then you can write a plugin to support\n"
+"it."
+msgstr ""
+
+#: en/help_topics/bugs.txt:42
+msgid ""
+"If you use Bugzilla or Trac, then you only need to set a configuration\n"
+"variable which contains the base URL of the bug tracker. These options\n"
+"can go into ``bazaar.conf``, ``branch.conf`` or into a branch-specific\n"
+"configuration section in ``locations.conf``.  You can set up these values\n"
+"for each of the projects you work on."
+msgstr ""
+
+#: en/help_topics/bugs.txt:48
+msgid ""
+"Note: As you provide a short name for each tracker, you can specify one or\n"
+"more bugs in one or more trackers at commit time if you wish."
+msgstr ""
+
+#: en/help_topics/bugs.txt:51
+msgid ""
+"Launchpad\n"
+"---------"
+msgstr ""
+
+#: en/help_topics/bugs.txt:54
+msgid "Use ``bzr commit --fixes lp:2`` to record that this commit fixes bug 2."
+msgstr ""
+
+#: en/help_topics/bugs.txt:56
+msgid ""
+"bugzilla_<tracker>_url\n"
+"----------------------"
+msgstr ""
+
+#: en/help_topics/bugs.txt:59
+msgid ""
+"If present, the location of the Bugzilla bug tracker referred to by\n"
+"<tracker>. This option can then be used together with ``bzr commit\n"
+"--fixes`` to mark bugs in that tracker as being fixed by that commit. For\n"
+"example::"
+msgstr ""
+
+#: en/help_topics/bugs.txt:64
+msgid "    bugzilla_squid_url = http://bugs.squid-cache.org"
+msgstr ""
+
+#: en/help_topics/bugs.txt:66
+msgid ""
+"would allow ``bzr commit --fixes squid:1234`` to mark Squid's bug 1234 as\n"
+"fixed."
+msgstr ""
+
+#: en/help_topics/bugs.txt:69
+msgid ""
+"trac_<tracker>_url\n"
+"------------------"
+msgstr ""
+
+#: en/help_topics/bugs.txt:72
+msgid ""
+"If present, the location of the Trac instance referred to by\n"
+"<tracker>. This option can then be used together with ``bzr commit\n"
+"--fixes`` to mark bugs in that tracker as being fixed by that commit. For\n"
+"example::"
+msgstr ""
+
+#: en/help_topics/bugs.txt:77
+msgid "    trac_twisted_url = http://www.twistedmatrix.com/trac"
+msgstr ""
+
+#: en/help_topics/bugs.txt:79
+msgid ""
+"would allow ``bzr commit --fixes twisted:1234`` to mark Twisted's bug 1234 "
+"as\n"
+"fixed."
+msgstr ""
+
+#: en/help_topics/bugs.txt:82
+msgid ""
+"bugtracker_<tracker>_url\n"
+"------------------------"
+msgstr ""
+
+#: en/help_topics/bugs.txt:85
+msgid ""
+"If present, the location of a generic bug tracker instance referred to by\n"
+"<tracker>. The location must contain an ``{id}`` placeholder,\n"
+"which will be replaced by a specific bug ID. This option can then be used\n"
+"together with ``bzr commit --fixes`` to mark bugs in that tracker as being\n"
+"fixed by that commit. For example::"
+msgstr ""
+
+#: en/help_topics/bugs.txt:91
+msgid "    bugtracker_python_url = http://bugs.python.org/issue{id}"
+msgstr ""
+
+#: en/help_topics/bugs.txt:93
+msgid ""
+"would allow ``bzr commit --fixes python:1234`` to mark bug 1234 in Python's\n"
+"Roundup bug tracker as fixed, or::"
+msgstr ""
+
+#: en/help_topics/bugs.txt:96
+msgid ""
+"    bugtracker_cpan_url = http://rt.cpan.org/Public/Bug/Display.html?id={id}"
+msgstr ""
+
+#: en/help_topics/bugs.txt:98
+msgid ""
+"would allow ``bzr commit --fixes cpan:1234`` to mark bug 1234 in CPAN's\n"
+"RT bug tracker as fixed, or::"
+msgstr ""
+
+#: en/help_topics/bugs.txt:101
+msgid "    bugtracker_hudson_url = http://issues.hudson-ci.org/browse/{id}"
+msgstr ""
+
+#: en/help_topics/bugs.txt:103
+msgid ""
+"would allow ``bzr commit --fixes hudson:HUDSON-1234`` to mark bug "
+"HUDSON-1234\n"
+"in Hudson's JIRA bug tracker as fixed.\n"
+msgstr ""
+
+#: en/help_topics/commands.txt:1
+msgid ""
+"add               Add specified files or directories.\n"
+"alias             Set/unset and display aliases.\n"
+"annotate          Show the origin of each line in a file.\n"
+"bash-completion   Generate a shell function for bash command line "
+"completion.\n"
+"                  [bash_completion]\n"
+"bind              Convert the current branch into a checkout of the "
+"supplied\n"
+"                  branch.\n"
+"branch            Create a new branch that is a copy of an existing branch.\n"
+"branches          List the branches available at the current location.\n"
+"break-lock        Break a dead lock.\n"
+"cat               Write the contents of a file as of a given revision to\n"
+"                  standard output.\n"
+"check             Validate working tree structure, branch consistency and\n"
+"                  repository history.\n"
+"checkout          Create a new checkout of an existing branch.\n"
+"clean-tree        Remove unwanted files from working tree.\n"
+"commit            Commit changes into a new revision.\n"
+"config            Display, set or remove a configuration option.\n"
+"conflicts         List files with conflicts.\n"
+"deleted           List files deleted in the working tree.\n"
+"diff              Show differences in the working tree, between revisions "
+"or\n"
+"                  branches.\n"
+"dpush             Push into a different VCS without any custom bzr "
+"metadata.\n"
+"export            Export current or past revision to a destination directory "
+"or\n"
+"                  archive.\n"
+"help              Show help on a command or other topic.\n"
+"ignore            Ignore specified files or patterns.\n"
+"ignored           List ignored files and the patterns that matched them.\n"
+"info              Show information about a working tree, branch or "
+"repository.\n"
+"init              Make a directory into a versioned branch.\n"
+"init-repository   Create a shared repository for branches to share storage\n"
+"                  space.\n"
+"join              Combine a tree into its containing tree.\n"
+"launchpad-login   Show or set the Launchpad user ID. [launchpad]\n"
+"launchpad-mirror  Ask Launchpad to mirror a branch now. [launchpad]\n"
+"launchpad-open    Open a Launchpad branch page in your web browser. "
+"[launchpad]\n"
+"log               Show historical log for a branch or subset of a branch.\n"
+"lp-find-proposal  Find the proposal to merge this revision. [launchpad]\n"
+"lp-propose-merge  Propose merging a branch on Launchpad. [launchpad]\n"
+"ls                List files in a tree.\n"
+"merge             Perform a three-way merge.\n"
+"missing           Show unmerged/unpulled revisions between two branches.\n"
+"mkdir             Create a new versioned directory.\n"
+"mv                Move or rename a file.\n"
+"nick              Print or set the branch nickname.\n"
+"pack              Compress the data within a repository.\n"
+"plugins           List the installed plugins.\n"
+"pull              Turn this branch into a mirror of another branch.\n"
+"push              Update a mirror of this branch.\n"
+"reconcile         Reconcile bzr metadata in a branch.\n"
+"reconfigure       Reconfigure the type of a bzr directory.\n"
+"register-branch   Register a branch with launchpad.net. [launchpad]\n"
+"remerge           Redo a merge.\n"
+"remove            Remove files or directories.\n"
+"remove-branch     Remove a branch.\n"
+"remove-tree       Remove the working tree from a given branch/checkout.\n"
+"renames           Show list of renamed files.\n"
+"resolve           Mark a conflict as resolved.\n"
+"revert            Revert files to a previous revision.\n"
+"revno             Show current revision number.\n"
+"root              Show the tree root directory.\n"
+"send              Mail or create a merge-directive for submitting changes.\n"
+"serve             Run the bzr server.\n"
+"shelve            Temporarily set aside some changes from the current tree.\n"
+"sign-my-commits   Sign all commits by a given committer.\n"
+"split             Split a subdirectory of a tree into a separate tree.\n"
+"status            Display status summary.\n"
+"switch            Set the branch of a checkout and update.\n"
+"tag               Create, remove or modify a tag naming a revision.\n"
+"tags              List tags.\n"
+"testament         Show testament (signing-form) of a revision.\n"
+"unbind            Convert the current checkout into a regular branch.\n"
+"uncommit          Remove the last committed revision.\n"
+"unshelve          Restore shelved changes.\n"
+"update            Update a working tree to a new revision.\n"
+"upgrade           Upgrade a repository, branch or working tree to a newer\n"
+"                  format.\n"
+"verify-signatures Verify all commit signatures.\n"
+"version           Show version of bzr.\n"
+"version-info      Show version information about this tree.\n"
+"view              Manage filtered views.\n"
+"whoami            Show or set bzr user id.\n"
+msgstr ""
+
+#: en/help_topics/configuration.txt:1
+msgid ""
+"Configuration Settings\n"
+"======================="
+msgstr ""
+
+#: en/help_topics/configuration.txt:4
+msgid ""
+"Environment settings\n"
+"---------------------"
+msgstr ""
+
+#: en/help_topics/configuration.txt:7
+msgid ""
+"While most configuration is handled by configuration files, some options\n"
+"which may be semi-permanent can also be controlled through the environment."
+msgstr ""
+
+#: en/help_topics/configuration.txt:10
+msgid ""
+"BZR_EMAIL\n"
+"~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:13
+msgid "Override the email id used by Bazaar.  Typical format::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:15
+msgid "  \"John Doe <jdoe@example.com>\""
+msgstr ""
+
+#: en/help_topics/configuration.txt:17
+msgid "See also the ``email`` configuration option."
+msgstr ""
+
+#: en/help_topics/configuration.txt:19
+msgid ""
+"BZR_PROGRESS_BAR\n"
+"~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:22
+msgid ""
+"Override the progress display.  Possible values are \"none\" or \"text\".  "
+"If\n"
+"the value is \"none\" then no progress bar is displayed.  The value \"text\" "
+"draws\n"
+"the ordinary command line progress bar."
+msgstr ""
+
+#: en/help_topics/configuration.txt:26
+msgid ""
+"BZR_SIGQUIT_PDB\n"
+"~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:29
+msgid "Control whether SIGQUIT behaves normally or invokes a breakin debugger."
+msgstr ""
+
+#: en/help_topics/configuration.txt:31
+msgid ""
+"* 0 = Standard SIGQUIT behavior (normally, exit with a core dump)\n"
+"* 1 = Invoke breakin debugger (default)"
+msgstr ""
+
+#: en/help_topics/configuration.txt:34
+msgid ""
+"BZR_HOME\n"
+"~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:37
+msgid "Override the home directory used by Bazaar."
+msgstr ""
+
+#: en/help_topics/configuration.txt:39
+msgid ""
+"BZR_SSH\n"
+"~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:42
+msgid "Select a different SSH implementation."
+msgstr ""
+
+#: en/help_topics/configuration.txt:44
+msgid ""
+"BZR_PDB\n"
+"~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:47
+msgid "Control whether to launch a debugger on error."
+msgstr ""
+
+#: en/help_topics/configuration.txt:49
+msgid ""
+"* 0 = Standard behavior\n"
+"* 1 = Launch debugger"
+msgstr ""
+
+#: en/help_topics/configuration.txt:52
+msgid ""
+"BZR_REMOTE_PATH\n"
+"~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:55
+msgid "Path to the Bazaar executable to use when using the bzr+ssh protocol."
+msgstr ""
+
+#: en/help_topics/configuration.txt:57
+msgid "See also the ``bzr_remote_path`` configuration option."
+msgstr ""
+
+#: en/help_topics/configuration.txt:59
+msgid ""
+"BZR_EDITOR\n"
+"~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:62
+msgid "Path to the editor Bazaar should use for commit messages, etc."
+msgstr ""
+
+#: en/help_topics/configuration.txt:64
+msgid ""
+"BZR_LOG\n"
+"~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:67
+msgid ""
+"Location of the Bazaar log file. You can check the current location by\n"
+"running ``bzr version``."
+msgstr ""
+
+#: en/help_topics/configuration.txt:70
+msgid ""
+"The log file contains debug information that is useful for diagnosing or\n"
+"reporting problems with Bazaar."
+msgstr ""
+
+#: en/help_topics/configuration.txt:73
+msgid ""
+"Setting this to ``NUL`` on Windows or ``/dev/null`` on other platforms\n"
+"will disable logging."
+msgstr ""
+
+#: en/help_topics/configuration.txt:76
+msgid ""
+"\n"
+"BZR_PLUGIN_PATH\n"
+"~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:80
+msgid ""
+"The path to the plugins directory that Bazaar should use.\n"
+"If not set, Bazaar will search for plugins in:"
+msgstr ""
+
+#: en/help_topics/configuration.txt:83
+msgid "* the user specific plugin directory (containing the ``user`` plugins),"
+msgstr ""
+
+#: en/help_topics/configuration.txt:85
+msgid "* the bzrlib directory (containing the ``core`` plugins),"
+msgstr ""
+
+#: en/help_topics/configuration.txt:87
+msgid ""
+"* the site specific plugin directory if applicable (containing\n"
+"  the ``site`` plugins)."
+msgstr ""
+
+#: en/help_topics/configuration.txt:90
+msgid ""
+"If ``BZR_PLUGIN_PATH`` is set in any fashion, it will change the\n"
+"the way the plugin are searched. "
+msgstr ""
+
+#: en/help_topics/configuration.txt:93
+msgid ""
+"As for the ``PATH`` variables, if multiple directories are\n"
+"specified in ``BZR_PLUGIN_PATH`` they should be separated by the\n"
+"platform specific appropriate character (':' on Unix,\n"
+"';' on windows)"
+msgstr ""
+
+#: en/help_topics/configuration.txt:98
+msgid ""
+"By default if ``BZR_PLUGIN_PATH`` is set, it replaces searching\n"
+"in ``user``.  However it will continue to search in ``core`` and\n"
+"``site`` unless they are explicitly removed."
+msgstr ""
+
+#: en/help_topics/configuration.txt:102
+msgid ""
+"If you need to change the order or remove one of these\n"
+"directories, you should use special values:"
+msgstr ""
+
+#: en/help_topics/configuration.txt:105
+msgid ""
+"* ``-user``, ``-core``, ``-site`` will remove the corresponding\n"
+"  path from the default values,"
+msgstr ""
+
+#: en/help_topics/configuration.txt:108
+msgid ""
+"* ``+user``, ``+core``, ``+site`` will add the corresponding path\n"
+"  before the remaining default values (and also remove it from\n"
+"  the default values)."
+msgstr ""
+
+#: en/help_topics/configuration.txt:112
+msgid ""
+"Note that the special values 'user', 'core' and 'site' should be\n"
+"used literally, they will be substituted by the corresponding,\n"
+"platform specific, values."
+msgstr ""
+
+#: en/help_topics/configuration.txt:116
+msgid ""
+"The examples below use ':' as the separator, windows users\n"
+"should use ';'."
+msgstr ""
+
+#: en/help_topics/configuration.txt:119
+msgid "Overriding the default user plugin directory::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:121
+msgid "  BZR_PLUGIN_PATH='/path/to/my/other/plugins'"
+msgstr ""
+
+#: en/help_topics/configuration.txt:123
+msgid "Disabling the site directory while retaining the user directory::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:125
+msgid "  BZR_PLUGIN_PATH='-site:+user'"
+msgstr ""
+
+#: en/help_topics/configuration.txt:127
+msgid "Disabling all plugins (better achieved with --no-plugins)::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:129
+msgid "  BZR_PLUGIN_PATH='-user:-core:-site'"
+msgstr ""
+
+#: en/help_topics/configuration.txt:131
+msgid "Overriding the default site plugin directory::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:133
+msgid "  BZR_PLUGIN_PATH='/path/to/my/site/plugins:-site':+user"
+msgstr ""
+
+#: en/help_topics/configuration.txt:135
+msgid ""
+"BZR_DISABLE_PLUGINS\n"
+"~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:138
+msgid ""
+"Under special circumstances (mostly when trying to diagnose a\n"
+"bug), it's better to disable a plugin (or several) rather than\n"
+"uninstalling them completely. Such plugins can be specified in\n"
+"the ``BZR_DISABLE_PLUGINS`` environment variable."
+msgstr ""
+
+#: en/help_topics/configuration.txt:143
+msgid ""
+"In that case, ``bzr`` will stop loading the specified plugins and\n"
+"will raise an import error if they are explicitly imported (by\n"
+"another plugin that depends on them for example)."
+msgstr ""
+
+#: en/help_topics/configuration.txt:147
+msgid "Disabling ``myplugin`` and ``yourplugin`` is achieved by::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:149
+msgid "  BZR_DISABLE_PLUGINS='myplugin:yourplugin'"
+msgstr ""
+
+#: en/help_topics/configuration.txt:151
+msgid ""
+"BZR_PLUGINS_AT\n"
+"~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:154
+msgid ""
+"When adding a new feature or working on a bug in a plugin,\n"
+"developers often need to use a specific version of a given\n"
+"plugin. Since python requires that the directory containing the\n"
+"code is named like the plugin itself this make it impossible to\n"
+"use arbitrary directory names (using a two-level directory scheme\n"
+"is inconvenient). ``BZR_PLUGINS_AT`` allows such directories even\n"
+"if they don't appear in ``BZR_PLUGIN_PATH`` ."
+msgstr ""
+
+#: en/help_topics/configuration.txt:162
+msgid ""
+"Plugins specified in this environment variable takes precedence\n"
+"over the ones in ``BZR_PLUGIN_PATH``."
+msgstr ""
+
+#: en/help_topics/configuration.txt:165
+msgid ""
+"The variable specified a list of ``plugin_name@plugin path``,\n"
+"``plugin_name`` being the name of the plugin as it appears in\n"
+"python module paths, ``plugin_path`` being the path to the\n"
+"directory containing the plugin code itself\n"
+"(i.e. ``plugins/myplugin`` not ``plugins``).  Use ':' as the list\n"
+"separator, use ';' on windows."
+msgstr ""
+
+#: en/help_topics/configuration.txt:172
+msgid ""
+"Example:\n"
+"~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:175
+msgid ""
+"Using a specific version of ``myplugin``:\n"
+"``BZR_PLUGINS_AT='myplugin@/home/me/bugfixes/123456-myplugin``"
+msgstr ""
+
+#: en/help_topics/configuration.txt:178
+msgid ""
+"BZRPATH\n"
+"~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:181
+msgid "The path where Bazaar should look for shell plugin external commands."
+msgstr ""
+
+#: en/help_topics/configuration.txt:183
+msgid ""
+"\n"
+"http_proxy, https_proxy\n"
+"~~~~~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:187
+msgid "Specifies the network proxy for outgoing connections, for example::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:189
+msgid ""
+"  http_proxy=http://proxy.example.com:3128/ \n"
+"  https_proxy=http://proxy.example.com:3128/"
+msgstr ""
+
+#: en/help_topics/configuration.txt:192
+msgid ""
+"\n"
+"Configuration files\n"
+"-------------------"
+msgstr ""
+
+#: en/help_topics/configuration.txt:196
+msgid ""
+"Location\n"
+"~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:199
+msgid ""
+"Configuration files are located in ``$HOME/.bazaar`` on Unix and\n"
+"``C:\\Documents and Settings\\<username>\\Application Data\\Bazaar\\2.0`` "
+"on\n"
+"Windows. (You can check the location for your system by using\n"
+"``bzr version``.)"
+msgstr ""
+
+#: en/help_topics/configuration.txt:204
+msgid "There are three primary configuration files in this location:"
+msgstr ""
+
+#: en/help_topics/configuration.txt:206
+msgid "* ``bazaar.conf`` describes default configuration options,"
+msgstr ""
+
+#: en/help_topics/configuration.txt:208
+msgid ""
+"* ``locations.conf`` describes configuration information for\n"
+"  specific branch locations,"
+msgstr ""
+
+#: en/help_topics/configuration.txt:211
+msgid ""
+"* ``authentication.conf`` describes credential information for\n"
+"  remote servers."
+msgstr ""
+
+#: en/help_topics/configuration.txt:214
+msgid ""
+"Each branch can also contain a configuration file that sets values specific\n"
+"to that branch. This file is found at ``.bzr/branch/branch.conf`` within "
+"the\n"
+"branch. This file is visible to all users of a branch, if you wish to "
+"override\n"
+"one of the values for a branch with a setting that is specific to you then "
+"you\n"
+"can do so in ``locations.conf``."
+msgstr ""
+
+#: en/help_topics/configuration.txt:220
+msgid ""
+"General format\n"
+"~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:223
+msgid ""
+"An ini file has three types of contructs: section headers, section\n"
+"options and comments."
+msgstr ""
+
+#: en/help_topics/configuration.txt:226
+msgid ""
+"Comments\n"
+"^^^^^^^^"
+msgstr ""
+
+#: en/help_topics/configuration.txt:229
+msgid ""
+"A comment is any line that starts with a \"#\" (sometimes called a \"hash\n"
+"mark\", \"pound sign\" or \"number sign\"). Comment lines are ignored by\n"
+"Bazaar when parsing ini files."
+msgstr ""
+
+#: en/help_topics/configuration.txt:233
+msgid ""
+"Section headers\n"
+"^^^^^^^^^^^^^^^"
+msgstr ""
+
+#: en/help_topics/configuration.txt:236
+msgid ""
+"A section header is a word enclosed in brackets that starts at the begining\n"
+"of a line.  A typical section header looks like this::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:239
+msgid "    [DEFAULT]"
+msgstr ""
+
+#: en/help_topics/configuration.txt:241
+msgid ""
+"The only valid section headers for bazaar.conf currently are [DEFAULT] and\n"
+"[ALIASES].  Section headers are case sensitive. The default section provides "
+"for\n"
+"setting options which can be overridden with the branch config file."
+msgstr ""
+
+#: en/help_topics/configuration.txt:245
+msgid ""
+"For ``locations.conf``, the options from the section with the\n"
+"longest matching section header are used to the exclusion of other\n"
+"potentially valid section headers. A section header uses the path for\n"
+"the branch as the section header. Some examples include::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:250
+msgid ""
+"    [http://mybranches.isp.com/~jdoe/branchdir]\n"
+"    [/home/jdoe/branches/]"
+msgstr ""
+
+#: en/help_topics/configuration.txt:253
+msgid ""
+"\n"
+"Section options\n"
+"^^^^^^^^^^^^^^^"
+msgstr ""
+
+#: en/help_topics/configuration.txt:257
+msgid ""
+"A section option resides within a section. A section option contains an\n"
+"option name, an equals sign and a value.  For example::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:260
+msgid ""
+"    email            = John Doe <jdoe@isp.com>\n"
+"    gpg_signing_key  = Amy Pond <amy@example.com>"
+msgstr ""
+
+#: en/help_topics/configuration.txt:263
+msgid ""
+"A option can reference other options by enclosing them in curly brackets::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:265
+msgid ""
+"    my_branch_name = feature_x\n"
+"    my_server      = bzr+ssh://example.com\n"
+"    push_location   = {my_server}/project/{my_branch_name}"
+msgstr ""
+
+#: en/help_topics/configuration.txt:269
+msgid ""
+"Option policies\n"
+"^^^^^^^^^^^^^^^"
+msgstr ""
+
+#: en/help_topics/configuration.txt:272
+msgid ""
+"Options defined in a section affect the named directory or URL plus\n"
+"any locations they contain.  Policies can be used to change how an\n"
+"option value is interpreted for contained locations.  Currently\n"
+"there are three policies available:"
+msgstr ""
+
+#: en/help_topics/configuration.txt:277
+msgid ""
+" none:\n"
+"   the value is interpreted the same for contained locations.  This is\n"
+"   the default behaviour.\n"
+" norecurse:\n"
+"   the value is only used for the exact location specified by the\n"
+"   section name.\n"
+" appendpath:\n"
+"   for contained locations, any additional path components are\n"
+"   appended to the value."
+msgstr ""
+
+#: en/help_topics/configuration.txt:287
+msgid ""
+"Policies are specified by keys with names of the form \"<option_name>:policy"
+"\".\n"
+"For example, to define the push location for a tree of branches, the\n"
+"following could be used::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:291
+msgid ""
+"  [/top/location]\n"
+"  push_location = sftp://example.com/location\n"
+"  push_location:policy = appendpath"
+msgstr ""
+
+#: en/help_topics/configuration.txt:295
+msgid ""
+"With this configuration, the push location for ``/top/location/branch1``\n"
+"would be ``sftp://example.com/location/branch1``."
+msgstr ""
+
+#: en/help_topics/configuration.txt:298
+msgid ""
+"Section local options\n"
+"^^^^^^^^^^^^^^^^^^^^^"
+msgstr ""
+
+#: en/help_topics/configuration.txt:301
+msgid ""
+"Some options are defined automatically inside a given section and can be\n"
+"refered to in this section only. "
+msgstr ""
+
+#: en/help_topics/configuration.txt:304
+msgid "For example, the ``appendpath`` policy can be used like this::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:306
+msgid ""
+"  [/home/vila/src/bzr/bugs]\n"
+"  mypush = lp:~vila/bzr\n"
+"  mypush:policy=appendpath"
+msgstr ""
+
+#: en/help_topics/configuration.txt:310
+msgid "Using ``relpath`` to achieve the same result is done like this::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:312
+msgid ""
+"  [/home/vila/src/bzr/bugs]\n"
+"  mypush = lp:~vila/bzr/{relpath}"
+msgstr ""
+
+#: en/help_topics/configuration.txt:315
+msgid ""
+"In both cases, when used in a directory like\n"
+"``/home/vila/src/bzr/bugs/832013-expand-in-stack`` we'll get::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:318
+msgid ""
+"   $ bzr config mypush\n"
+"   lp:~vila/bzr/832013-expand-in-stack"
+msgstr ""
+
+#: en/help_topics/configuration.txt:321
+msgid "Another such option is ``basename`` which can be used like this::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:323
+msgid ""
+"  [/home/vila/src/bzr]\n"
+"  mypush = lp:~vila/bzr/{basename}"
+msgstr ""
+
+#: en/help_topics/configuration.txt:326
+msgid ""
+"When used in a directory like\n"
+"``/home/vila/src/bzr/bugs/832013-expand-in-stack`` we'll get::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:332
+msgid ""
+"Note that ``basename`` here refers to the base name of ``relpath`` which\n"
+"itself is defined as the relative path between the section name and the\n"
+"location it matches."
+msgstr ""
+
+#: en/help_topics/configuration.txt:336
+msgid ""
+"When an option is local to a Section, it cannot be referred to from option\n"
+"values in any other section from the same ``Store`` nor from any other\n"
+"``Store``."
+msgstr ""
+
+#: en/help_topics/configuration.txt:340
+msgid ""
+"\n"
+"The main configuration file, bazaar.conf\n"
+"~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:344
+msgid ""
+"``bazaar.conf`` allows two sections: ``[DEFAULT]`` and ``[ALIASES]``.\n"
+"The default section contains the default\n"
+"configuration options for all branches. The default section can be\n"
+"overriden by providing a branch-specific section in ``locations.conf``."
+msgstr ""
+
+#: en/help_topics/configuration.txt:349
+msgid "A typical ``bazaar.conf`` section often looks like the following::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:351
+msgid ""
+"    [DEFAULT]\n"
+"    email             = John Doe <jdoe@isp.com>\n"
+"    editor            = /usr/bin/vim\n"
+"    create_signatures = when-required"
+msgstr ""
+
+#: en/help_topics/configuration.txt:356
+msgid ""
+"\n"
+"The branch location configuration file, locations.conf\n"
+"~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:360
+msgid ""
+"``locations.conf`` allows one to specify overriding settings for\n"
+"a specific branch. The format is almost identical to the default section in\n"
+"bazaar.conf with one significant change: The section header, instead of "
+"saying\n"
+"default, will be the path to a branch that you wish to override a value\n"
+"for. The '?' and '*' wildcards are supported::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:366
+msgid ""
+"    [/home/jdoe/branches/nethack]\n"
+"    email = Nethack Admin <nethack@nethack.com>"
+msgstr ""
+
+#: en/help_topics/configuration.txt:369
+msgid ""
+"    [http://hypothetical.site.com/branches/devel-branch]\n"
+"    create_signatures = always"
+msgstr ""
+
+#: en/help_topics/configuration.txt:372
+msgid ""
+"The authentication configuration file, authentication.conf\n"
+"~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:375
+msgid ""
+"``authentication.conf`` allows one to specify credentials for\n"
+"remote servers. This can be used for all the supported transports and any "
+"part\n"
+"of bzr that requires authentication (smtp for example)."
+msgstr ""
+
+#: en/help_topics/configuration.txt:379
+msgid ""
+"The syntax of the file obeys the same rules as the others except for the\n"
+"option policies which don't apply."
+msgstr ""
+
+#: en/help_topics/configuration.txt:382
+msgid ""
+"For more information on the possible uses of the authentication "
+"configuration\n"
+"file see :doc:`authentication-help`."
+msgstr ""
+
+#: en/help_topics/configuration.txt:385
+msgid ""
+"\n"
+"Common options\n"
+"--------------"
+msgstr ""
+
+#: en/help_topics/configuration.txt:389
+msgid ""
+"debug_flags\n"
+"~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:392
+msgid ""
+"A comma-separated list of debugging options to turn on.  The same values\n"
+"can be used as with the -D command-line option (see `help global-options`).\n"
+"For example::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:396
+msgid "    debug_flags = hpss"
+msgstr ""
+
+#: en/help_topics/configuration.txt:400
+msgid "    debug_flags = hpss,evil"
+msgstr ""
+
+#: en/help_topics/configuration.txt:402
+msgid ""
+"email\n"
+"~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:405
+msgid ""
+"The email address to use when committing a branch. Typically takes the form\n"
+"of::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:408
+msgid "    email = Full Name <account@hostname.tld>"
+msgstr ""
+
+#: en/help_topics/configuration.txt:410
+msgid ""
+"editor\n"
+"~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:413
+msgid ""
+"The path of the editor that you wish to use if *bzr commit* is run without\n"
+"a commit message. This setting is trumped by the environment variable\n"
+"``BZR_EDITOR``, and overrides the ``VISUAL`` and ``EDITOR`` environment\n"
+"variables."
+msgstr ""
+
+#: en/help_topics/configuration.txt:418
+msgid ""
+"log_format\n"
+"~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:421
+msgid ""
+"The default log format to use. Standard log formats are ``long``, ``short``\n"
+"and ``line``. Additional formats may be provided by plugins. The default\n"
+"value is ``long``."
+msgstr ""
+
+#: en/help_topics/configuration.txt:425
+msgid ""
+"check_signatures\n"
+"~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:428
+msgid ""
+"Reserved for future use.  These options will allow a policy for branches to\n"
+"require signatures."
+msgstr ""
+
+#: en/help_topics/configuration.txt:431
+msgid ""
+"require\n"
+"    The gnupg signature for revisions must be present and must be valid."
+msgstr ""
+
+#: en/help_topics/configuration.txt:434
+msgid ""
+"ignore\n"
+"    Do not check gnupg signatures of revisions."
+msgstr ""
+
+#: en/help_topics/configuration.txt:437
+msgid ""
+"check-available\n"
+"    (default) If gnupg signatures for revisions are present, check them.\n"
+"    Bazaar will fail if it finds a bad signature, but will not fail if\n"
+"    no signature is present."
+msgstr ""
+
+#: en/help_topics/configuration.txt:442
+msgid ""
+"create_signatures\n"
+"~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:445
+msgid ""
+"Defines the behaviour of signing revisions on commits.  By default bzr will "
+"not\n"
+"sign new commits."
+msgstr ""
+
+#: en/help_topics/configuration.txt:448
+msgid ""
+"always\n"
+"    Sign every new revision that is committed.  If the signing fails then "
+"the\n"
+"    commit will not be made."
+msgstr ""
+
+#: en/help_topics/configuration.txt:452
+msgid ""
+"when-required\n"
+"    Reserved for future use."
+msgstr ""
+
+#: en/help_topics/configuration.txt:455
+msgid ""
+"never\n"
+"    Reserved for future use."
+msgstr ""
+
+#: en/help_topics/configuration.txt:458
+msgid ""
+"In future it is planned that ``when-required`` will sign newly\n"
+"committed revisions only when the branch requires them.  ``never`` will "
+"refuse\n"
+"to sign newly committed revisions, even if the branch requires signatures."
+msgstr ""
+
+#: en/help_topics/configuration.txt:462
+msgid ""
+"dirstate.fdatasync\n"
+"~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:465
+msgid ""
+"If true (default), working tree metadata changes are flushed through the\n"
+"OS buffers to physical disk.  This is somewhat slower, but means data\n"
+"should not be lost if the machine crashes.  See also repository.fdatasync."
+msgstr ""
+
+#: en/help_topics/configuration.txt:469
+msgid ""
+"gpg_signing_key\n"
+"~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:472
+msgid ""
+"The GnuPG user identity to use when signing commits.  Can be an e-mail\n"
+"address, key fingerprint or full key ID.  When unset or when set to\n"
+"\"default\" Bazaar will use the user e-mail set with ``whoami``."
+msgstr ""
+
+#: en/help_topics/configuration.txt:476
+msgid ""
+"recurse\n"
+"~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:479
+msgid ""
+"Only useful in ``locations.conf``. Defines whether or not the\n"
+"configuration for this section applies to subdirectories:"
+msgstr ""
+
+#: en/help_topics/configuration.txt:482
+msgid ""
+"true\n"
+"    (default) This section applies to subdirectories as well."
+msgstr ""
+
+#: en/help_topics/configuration.txt:485
+msgid ""
+"false\n"
+"    This section only applies to the branch at this directory and not\n"
+"    branches below it."
+msgstr ""
+
+#: en/help_topics/configuration.txt:489
+msgid ""
+"gpg_signing_command\n"
+"~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:492
+msgid ""
+"(Default: \"gpg\"). Which program should be used to sign and check "
+"revisions.\n"
+"For example::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:495
+msgid "    gpg_signing_command = /usr/bin/gnpg"
+msgstr ""
+
+#: en/help_topics/configuration.txt:497
+msgid ""
+"The specified command must accept the options \"--clearsign\" and \"-u "
+"<email>\"."
+msgstr ""
+
+#: en/help_topics/configuration.txt:499
+msgid ""
+"bzr_remote_path\n"
+"~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:502
+msgid ""
+"(Default: \"bzr\").  The path to the command that should be used to run the "
+"smart\n"
+"server for bzr.  This value may only be specified in locations.conf, because:"
+msgstr ""
+
+#: en/help_topics/configuration.txt:505
+msgid ""
+"- it's needed before branch.conf is accessible\n"
+"- allowing remote branch.conf files to specify commands would be a security\n"
+"  risk"
+msgstr ""
+
+#: en/help_topics/configuration.txt:509
+msgid "It is overridden by the BZR_REMOTE_PATH environment variable."
+msgstr ""
+
+#: en/help_topics/configuration.txt:511
+msgid ""
+"smtp_server\n"
+"~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:514
+msgid ""
+"(Default: \"localhost\"). SMTP server to use when Bazaar needs to send\n"
+"email, eg. with ``merge-directive --mail-to``, or the bzr-email plugin."
+msgstr ""
+
+#: en/help_topics/configuration.txt:517
+msgid ""
+"smtp_username, smtp_password\n"
+"~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:520
+msgid ""
+"User and password to authenticate to the SMTP server. If smtp_username\n"
+"is set, and smtp_password is not, Bazaar will prompt for a password.\n"
+"These settings are only needed if the SMTP server requires authentication\n"
+"to send mail."
+msgstr ""
+
+#: en/help_topics/configuration.txt:525
+msgid ""
+"locks.steal_dead\n"
+"~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:528
+msgid ""
+"If set to true, bzr will automatically break locks held by processes from\n"
+"the same machine and user that are no longer alive.  Otherwise, it will\n"
+"print a message and you can break the lock manually, if you are satisfied\n"
+"the object is no longer in use."
+msgstr ""
+
+#: en/help_topics/configuration.txt:533
+msgid ""
+"mail_client\n"
+"~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:536
+msgid ""
+"A mail client to use for sending merge requests.\n"
+"By default, bzr will try to use ``mapi`` on Windows.  On other platforms, "
+"it\n"
+"will try ``xdg-email``. If either of these fails, it will fall back to\n"
+"``editor``."
+msgstr ""
+
+#: en/help_topics/configuration.txt:541
+msgid "Supported values for specific clients:"
+msgstr ""
+
+#: en/help_topics/configuration.txt:543
+msgid ""
+":claws: Use Claws.  This skips a dialog for attaching files.\n"
+":evolution: Use Evolution.\n"
+":kmail: Use KMail.\n"
+":mutt: Use Mutt.\n"
+":thunderbird: Use Mozilla Thunderbird or Icedove.  For Thunderbird/Icedove "
+"1.5,\n"
+"    this works around some bugs that xdg-email doesn't handle."
+msgstr ""
+
+#: en/help_topics/configuration.txt:550
+msgid "Supported generic values are:"
+msgstr ""
+
+#: en/help_topics/configuration.txt:552
+msgid ""
+":default: See above.\n"
+":editor: Use your editor to compose the merge request.  This also uses\n"
+"    your commit id, (see ``bzr whoami``), smtp_server and (optionally)\n"
+"    smtp_username and smtp_password.\n"
+":mapi: Use your preferred e-mail client on Windows.\n"
+":xdg-email: Use xdg-email to run your preferred mail program"
+msgstr ""
+
+#: en/help_topics/configuration.txt:559
+msgid ""
+"repository.fdatasync\n"
+"~~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:562
+msgid ""
+"If true (default), repository changes are flushed through the OS buffers\n"
+"to physical disk.  This is somewhat slower, but means data should not be\n"
+"lost if the machine crashes.  See also dirstate.fdatasync."
+msgstr ""
+
+#: en/help_topics/configuration.txt:566
+msgid ""
+"submit_branch\n"
+"~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:569
+msgid ""
+"The branch you intend to submit your current work to.  This is "
+"automatically\n"
+"set by ``bzr send``, and is also used by the ``submit:`` revision spec.  "
+"This\n"
+"should usually be set on a per-branch or per-location basis."
+msgstr ""
+
+#: en/help_topics/configuration.txt:573
+msgid ""
+"public_branch\n"
+"~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:576
+msgid ""
+"A publically-accessible version of this branch (implying that this version "
+"is\n"
+"not publically-accessible).  Used (and set) by ``bzr send``."
+msgstr ""
+
+#: en/help_topics/configuration.txt:579
+msgid ""
+"suppress_warnings\n"
+"~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:582
+msgid ""
+"A list of strings, each string represent a warning that can be emitted by\n"
+"bzr. Mentioning a warning in this list tells bzr to not emit it."
+msgstr ""
+
+#: en/help_topics/configuration.txt:585
+msgid "Valid values:"
+msgstr ""
+
+#: en/help_topics/configuration.txt:587
+msgid ""
+"* ``format_deprecation``:\n"
+"    whether the format deprecation warning is shown on repositories that "
+"are\n"
+"    using deprecated formats."
+msgstr ""
+
+#: en/help_topics/configuration.txt:591
+msgid ""
+"default_format\n"
+"~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:594
+msgid ""
+"A format name for the default format used when creating branches.  See "
+"``bzr\n"
+"help formats`` for possible values."
+msgstr ""
+
+#: en/help_topics/configuration.txt:597
+msgid ""
+"\n"
+"Unicode options\n"
+"---------------"
+msgstr ""
+
+#: en/help_topics/configuration.txt:601
+msgid ""
+"output_encoding\n"
+"~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:604
+msgid ""
+"A Python unicode encoding name for text output from bzr, such as log\n"
+"information.  Values include: utf8, cp850, ascii, iso-8859-1.  The default\n"
+"is the terminal encoding prefered by the operating system."
+msgstr ""
+
+#: en/help_topics/configuration.txt:608
+msgid ""
+"\n"
+"Branch type specific options\n"
+"----------------------------"
+msgstr ""
+
+#: en/help_topics/configuration.txt:612
+msgid ""
+"These options apply only to branches that use the ``dirstate-tags`` or\n"
+"later format.  They\n"
+"are usually set in ``.bzr/branch/branch.conf`` automatically, but may be\n"
+"manually set in ``locations.conf`` or ``bazaar.conf``."
+msgstr ""
+
+#: en/help_topics/configuration.txt:617
+msgid ""
+"append_revisions_only\n"
+"~~~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:620
+msgid ""
+"If set to \"True\" then revisions can only be appended to the log, not\n"
+"removed.  A branch with this setting enabled can only pull from another\n"
+"branch if the other branch's log is a longer version of its own.  This is\n"
+"normally set by ``bzr init --append-revisions-only``. If you set it\n"
+"manually, use either 'True' or 'False' (case-sensitive) to maintain\n"
+"compatibility with previous bzr versions (older than 2.2)."
+msgstr ""
+
+#: en/help_topics/configuration.txt:627
+msgid ""
+"parent_location\n"
+"~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:630
+msgid ""
+"If present, the location of the default branch for pull or merge.  This "
+"option\n"
+"is normally set when creating a branch, the first ``pull`` or by ``pull\n"
+"--remember``."
+msgstr ""
+
+#: en/help_topics/configuration.txt:634
+msgid ""
+"push_location\n"
+"~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:637
+msgid ""
+"If present, the location of the default branch for push.  This option\n"
+"is normally set by the first ``push`` or ``push --remember``."
+msgstr ""
+
+#: en/help_topics/configuration.txt:640
+msgid ""
+"push_strict\n"
+"~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:643
+msgid ""
+"If present, defines the ``--strict`` option default value for checking\n"
+"uncommitted changes before pushing."
+msgstr ""
+
+#: en/help_topics/configuration.txt:646
+msgid ""
+"dpush_strict\n"
+"~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:649
+msgid ""
+"If present, defines the ``--strict`` option default value for checking\n"
+"uncommitted changes before pushing into a different VCS without any\n"
+"custom bzr metadata."
+msgstr ""
+
+#: en/help_topics/configuration.txt:653
+msgid ""
+"bound_location\n"
+"~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:656
+msgid ""
+"The location that commits should go to when acting as a checkout.\n"
+"This option is normally set by ``bind``."
+msgstr ""
+
+#: en/help_topics/configuration.txt:659
+msgid ""
+"bound\n"
+"~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:662
+msgid ""
+"If set to \"True\", the branch should act as a checkout, and push each "
+"commit to\n"
+"the bound_location.  This option is normally set by ``bind``/``unbind``."
+msgstr ""
+
+#: en/help_topics/configuration.txt:665
+msgid ""
+"send_strict\n"
+"~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:668
+msgid ""
+"If present, defines the ``--strict`` option default value for checking\n"
+"uncommitted changes before sending a merge directive."
+msgstr ""
+
+#: en/help_topics/configuration.txt:671
+msgid ""
+"add.maximum_file_size\n"
+"~~~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:674
+msgid ""
+"Defines the maximum file size the command line \"add\" operation will allow\n"
+"in recursive mode, with files larger than this value being skipped. You "
+"may \n"
+"specify this value as an integer (in which case it is interpreted as "
+"bytes), \n"
+"or you may specify the value using SI units, i.e. 10KB, 20MB, 1G. A value of "
+"0 \n"
+"will disable skipping."
+msgstr ""
+
+#: en/help_topics/configuration.txt:680
+msgid ""
+"External Merge Tools\n"
+"--------------------"
+msgstr ""
+
+#: en/help_topics/configuration.txt:683
+msgid ""
+"bzr.mergetool.<name>\n"
+"~~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:686
+msgid ""
+"Defines an external merge tool called <name> with the given command-line.\n"
+"Arguments containing spaces should be quoted using single or double quotes. "
+"The\n"
+"executable may omit its path if it can be found on the PATH."
+msgstr ""
+
+#: en/help_topics/configuration.txt:690
+msgid ""
+"The following markers can be used in the command-line to substitute "
+"filenames\n"
+"involved in the merge conflict::"
+msgstr ""
+
+#: en/help_topics/configuration.txt:693
+msgid ""
+"  {base}      file.BASE\n"
+"  {this}      file.THIS\n"
+"  {other}     file.OTHER\n"
+"  {result}    output file\n"
+"  {this_temp} temp copy of file.THIS, used to overwrite output file if "
+"merge\n"
+"              succeeds."
+msgstr ""
+
+#: en/help_topics/configuration.txt:702
+msgid "  bzr.mergetool.kdiff3 = kdiff3 {base} {this} {other} -o {result}"
+msgstr ""
+
+#: en/help_topics/configuration.txt:704
+msgid ""
+"bzr.default_mergetool\n"
+"~~~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/configuration.txt:707
+msgid ""
+"Specifies which external merge tool (as defined above) should be selected "
+"by\n"
+"default in tools such as ``bzr qconflicts``."
+msgstr ""
+
+#: en/help_topics/configuration.txt:712
+msgid "  bzr.default_mergetool = kdiff3\n"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:1
+msgid ""
+"Conflict Types\n"
+"=============="
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:4
+msgid ""
+"Some operations, like merge, revert and pull, modify the contents of your\n"
+"working tree.  These modifications are programmatically generated, and so "
+"they\n"
+"may conflict with the current state of your working tree. "
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:8
+msgid ""
+"When conflicts are present in your working tree (as shown by ``bzr\n"
+"conflicts``), you should resolve them and then inform bzr that the "
+"conflicts\n"
+"have been resolved."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:12
+msgid ""
+"Resolving conflicts is sometimes not obvious. Either because the user that\n"
+"should resolve them is not the one responsible for their occurrence, as is "
+"the\n"
+"case when merging other people's work or because some conflicts are "
+"presented\n"
+"in a way that is not easy to understand."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:17
+msgid ""
+"Bazaar tries to avoid conflicts ; its aim is to ask you to resolve the\n"
+"conflict if and only if there's an actual conceptual conflict in the source\n"
+"tree.  Because Bazaar doesn't understand the real meaning of the files "
+"being\n"
+"versioned, it can, when faced with ambiguities, fall short in either "
+"direction\n"
+"trying to resolve the conflict itself. Many kinds of changes can be "
+"combined\n"
+"programmatically, but sometimes only a human can determine the right thing "
+"to\n"
+"do."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:25
+msgid ""
+"When Bazaar generates a conflict, it adds information into the working tree "
+"to\n"
+"present the conflicting versions, and it's up to you to find the correct\n"
+"resolution."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:29
+msgid "Whatever the conflict is, resolving it is roughly done in two steps:"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:31
+msgid ""
+"1. Modify the working tree content so that the conflicted item is now in "
+"the\n"
+"   state you want to keep, then"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:34
+msgid ""
+"2. Inform Bazaar that the conflict is now solved and ask to cleanup any\n"
+"   remaining generated information (``bzr resolve <item>``)."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:37
+msgid ""
+"For most conflict types, there are some obvious ways to modify the working\n"
+"tree and put it into the desired state. For some types of conflicts, Bazaar\n"
+"itself already made a choice, when possible."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:41
+msgid ""
+"Yet, whether Bazaar makes a choice or not, there are some other simple but\n"
+"different ways to resolve the conflict."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:44
+msgid ""
+"Each type of conflict is explained below, and the action which must be done "
+"to\n"
+"resolve the conflict is outlined."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:47
+msgid ""
+"Various actions are available depending on the kind of conflict, for some "
+"of\n"
+"these actions, Bazaar can provide some help. In the end you should at least\n"
+"inform Bazaar that you're done with the conflict with::"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:51
+msgid "  ``bzr resolve FILE --action=done'"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:53
+msgid ""
+"Note that this is the default action when a single file is involved so you "
+"can\n"
+"simply use::"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:56
+msgid "   ``bzr resolve FILE``"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:58
+msgid "See ``bzr help resolve`` for more details."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:60
+msgid ""
+"Text conflicts\n"
+"--------------"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:63
+msgid "Typical message::"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:65
+msgid "  Text conflict in FILE"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:67
+msgid ""
+"These are produced when a text merge cannot completely reconcile two sets "
+"of\n"
+"text changes.  Bazaar will emit files for each version with the extensions\n"
+"THIS, OTHER, and BASE.  THIS is the version of the file from the target "
+"tree,\n"
+"i.e. the tree that you are merging changes into.  OTHER is the version that "
+"you\n"
+"are merging into the target.  BASE is an older version that is used as a "
+"basis\n"
+"for comparison."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:74
+msgid ""
+"In the main copy of the file, Bazaar will include all the changes that it\n"
+"could reconcile, and any un-reconciled conflicts are surrounded by\n"
+"\"herringbone\" markers like ``<<<<<<<``."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:78
+msgid ""
+"Say the initial text is \"The project leader released it.\", and THIS "
+"modifies it\n"
+"to \"Martin Pool released it.\", while OTHER modifies it to \"The project "
+"leader\n"
+"released Bazaar.\"  A conflict would look like this::"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:82
+msgid ""
+"  <<<<<<< TREE\n"
+"  Martin Pool released it.\n"
+"  =======\n"
+"  The project leader released Bazaar.\n"
+"  >>>>>>> MERGE-SOURCE"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:88
+msgid "The correct resolution would be \"Martin Pool released Bazaar.\""
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:90
+msgid ""
+"You can handle text conflicts either by editing the main copy of the file,\n"
+"or by invoking external tools on the THIS, OTHER and BASE versions.  It's\n"
+"worth mentioning that resolving text conflicts rarely involves picking one\n"
+"set of changes over the other (but see below when you encounter these\n"
+"cases).  More often, the two sets of changes must be intelligently combined."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:96
+msgid ""
+"If you edit the main copy, be sure to remove the herringbone markers.  When\n"
+"you are done editing, the file should look like it never had a conflict, and "
+"be\n"
+"ready to commit."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:100
+msgid ""
+"When you have resolved text conflicts, just run ``bzr resolve --auto``, and\n"
+"Bazaar will auto-detect which conflicts you have resolved."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:103
+msgid ""
+"When the conflict is resolved, Bazaar deletes the previously generated\n"
+"``.BASE``, ``.THIS`` and ``.OTHER`` files if they are still present in the\n"
+"working tree."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:107
+msgid ""
+"\n"
+"When you want to pick one set of changes over the other, you can use ``bzr\n"
+"resolve`` with one of the following actions:"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:111
+msgid ""
+"* ``--action=take-this`` will issue ``mv FILE.THIS FILE``,\n"
+"* ``--action=take-other`` will issue ``mv FILE.OTHER FILE``."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:114
+msgid ""
+"Note that if you have modified ``FILE.THIS`` or ``FILE.OTHER``, these\n"
+"modifications will be taken into account."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:117
+msgid ""
+"Content conflicts\n"
+"-----------------"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:122
+msgid "  Contents conflict in FILE"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:124
+msgid ""
+"This conflict happens when there are conflicting changes in the working "
+"tree\n"
+"and the merge source, but the conflicted items are not text files.  They "
+"may\n"
+"be binary files, or symlinks, or directories.  It can even happen with "
+"files\n"
+"that are deleted on one side, and modified on the other."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:129
+msgid ""
+"Like text conflicts, Bazaar will emit THIS, OTHER and BASE files.  (They may "
+"be\n"
+"regular files, symlinks or directories).  But it will not include a \"main "
+"copy\"\n"
+"of the file with herringbone conflict markers.  It will appear that the "
+"\"main\n"
+"copy\" has been renamed to THIS or OTHER."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:134
+msgid ""
+"To resolve that kind of conflict, you should rebuild FILE from either "
+"version\n"
+"or a combination of both."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:137
+msgid "``bzr resolve`` recognizes the following actions:"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:139
+msgid ""
+"* ``--action=take-this`` will issue ``bzr mv FILE.THIS FILE``,\n"
+"* ``--action=take-other`` will issue ``bzr mv FILE.OTHER FILE``,\n"
+"* ``--action=done`` will just mark the conflict as resolved."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:143
+msgid ""
+"Any action will also delete the previously generated ``.BASE``, ``.THIS`` "
+"and\n"
+"``.OTHER`` files if they are still present in the working tree."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:146
+msgid ""
+"Bazaar cannot auto-detect when conflicts of this kind have been resolved."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:148
+msgid ""
+"Tag conflicts\n"
+"-------------"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:153
+msgid ""
+"  Conflicting tags:\n"
+"      version-0.1"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:156
+msgid ""
+"When pulling from or pushing to another branch, Bazaar informs you about "
+"tags\n"
+"that conflict between the two branches; that is the same tag points to two\n"
+"different revisions.  You need not resolve these conflicts, but subsequent\n"
+"uses of pull or push will result in the same message."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:161
+msgid ""
+"To resolve the conflict, you must apply the correct tags to either the "
+"target\n"
+"branch or the source branch as appropriate.  Use \"bzr tags --show-ids -d\n"
+"SOURCE_URL\" to see the tags in the source branch.  If you want to make the\n"
+"target branch's tags match the source branch, then in the target branch do\n"
+"``bzr tag --force -r revid:REVISION_ID CONFLICTING_TAG`` for each of the\n"
+"CONFLICTING_TAGs, where REVISION_ID comes from the list of tags in the "
+"source\n"
+"branch.  You need not call \"bzr resolve\" after doing this.  To resolve in\n"
+"favor of the target branch, you need to similarly use ``tag --force`` in "
+"the\n"
+"source branch.  (Note that pulling or pushing using --overwrite will "
+"overwrite\n"
+"all tags as well.)"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:172
+msgid ""
+"Duplicate paths\n"
+"---------------"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:177
+msgid "  Conflict adding file FILE.  Moved existing file to FILE.moved."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:179
+msgid ""
+"Sometimes Bazaar will attempt to create a file using a pathname that has\n"
+"already been used.  The existing file will be renamed to \"FILE.moved\".  "
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:187
+msgid ""
+"* ``--action=take-this`` will issue ``bzr rm FILE ; bzr mv FILE.moved "
+"FILE``,\n"
+"* ``--action=take-other`` will issue ``bzr rm FILE.moved``,\n"
+"* ``--action=done`` will just mark the conflict as resolved."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:191
+msgid ""
+"Note that you must get rid of FILE.moved before using ``--action=done``."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:195
+msgid ""
+"Unversioned parent\n"
+"------------------"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:200
+msgid "    Conflict because FILE is not versioned, but has versioned children."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:202
+msgid ""
+"Sometimes Bazaar will attempt to create a file whose parent directory is "
+"not\n"
+"versioned.  This happens when the directory has been deleted in the target,\n"
+"but has a new child in the source, or vice versa.  In this situation, "
+"Bazaar\n"
+"will version the parent directory as well.  Resolving this issue depends\n"
+"very much on the particular scenario.  You may wish to rename or delete "
+"either\n"
+"the file or the directory.  When you are satisfied, you can run \"bzr "
+"resolve\n"
+"FILE\" to mark the conflict as resolved."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:210
+msgid ""
+"Missing parent\n"
+"--------------"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:215
+msgid "  Conflict adding files to FILE.  Created directory."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:217
+msgid ""
+"This happens when a directory has been deleted in the target, but has new\n"
+"children in the source.  This is similar to the \"unversioned parent\" "
+"conflict,\n"
+"except that the parent directory does not *exist*, instead of just being\n"
+"unversioned.  In this situation, Bazaar will create the missing parent.\n"
+"Resolving this issue depends very much on the particular scenario."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:223
+msgid ""
+"To resolve that kind of conflict, you should either remove or rename the\n"
+"children or the directory or a combination of both."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:228
+msgid ""
+"* ``--action=take-this`` will issue ``bzr rm directory`` including the\n"
+"  children,\n"
+"* ``--action=take-other`` will acknowledge Bazaar choice to keep the "
+"children\n"
+"  and restoring the directory,\n"
+"* ``--action=done`` will just mark the conflict as resolved."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:236
+msgid ""
+"Deleting parent\n"
+"---------------"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:241
+msgid "  Conflict: can't delete DIR because it is not empty.  Not deleting."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:243
+msgid ""
+"This is the opposite of \"missing parent\".  A directory is deleted in the\n"
+"source, but has new children in the target (either because a directory\n"
+"deletion is merged or because the merge introduce new children).  Bazaar\n"
+"will retain the directory.  Resolving this issue depends very much on the\n"
+"particular scenario."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:254
+msgid ""
+"* ``--action=take-this`` will acknowledge Bazaar choice to keep the "
+"directory,"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:256
+msgid ""
+"* ``--action=take-other`` will issue ``bzr rm directory`` including the \n"
+"  children,"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:259
+msgid "* ``--action=done`` will just mark the conflict as resolved."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:261
+msgid ""
+"Note that when merging a directory deletion, if unversioned files are\n"
+"present, they become potential orphans has they don't have a directory\n"
+"parent anymore."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:265
+msgid ""
+"Handling such orphans, *before* the conflict is created, is controlled by\n"
+"setting the ``bzr.transform.orphan_policy`` configuration option."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:268
+msgid "There are two possible values for this option:"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:270
+msgid ""
+"* ``conflict`` (the default): will leave the orphans in place and\n"
+"  generate a conflicts,"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:273
+msgid ""
+"* ``move``: will move the orphans to a ``bzr-orphans`` directory at the "
+"root\n"
+"  of the working tree with names like ``<file>.~#~``."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:278
+msgid ""
+"Path conflict\n"
+"-------------"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:283
+msgid "  Path conflict: PATH1 / PATH2"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:285
+msgid ""
+"This happens when the source and target have each modified the name or "
+"parent\n"
+"directory of a file.  Bazaar will use the path elements from the source."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:288
+msgid ""
+"To resolve that kind of conflict, you just have to decide what name should "
+"be\n"
+"retained for the file involved."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:293
+msgid ""
+"* ``--action=take-this`` will revert Bazaar choice and keep ``PATH1`` by\n"
+"  issuing ``bzr mv PATH2 PATH1``,\n"
+"* ``--action=take-other`` will acknowledge Bazaar choice of keeping "
+"``PATH2``,\n"
+"* ``--action=done`` will just mark the conflict as resolved."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:300
+msgid ""
+"Parent loop\n"
+"-----------"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:305
+msgid "  Conflict moving FILE into DIRECTORY.  Cancelled move."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:307
+msgid ""
+"This happens when the source and the target have each moved directories, so\n"
+"that, if the change could be applied, a directory would be contained by "
+"itself.\n"
+"For example::"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:311
+msgid ""
+"  $ bzr init\n"
+"  $ bzr mkdir white\n"
+"  $ bzr mkdir black\n"
+"  $ bzr commit -m \"BASE\"\n"
+"  $ bzr branch . ../other\n"
+"  $ bzr mv white black\n"
+"  $ bzr commit -m \"THIS\"\n"
+"  $ bzr mv ../other/black ../other/white\n"
+"  $ bzr commit ../other -m \"OTHER\"\n"
+"  $ bzr merge ../other"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:322
+msgid ""
+"In this situation, Bazaar will cancel the move, and leave ``white`` in\n"
+"``black``.  To resolve that kind of conflict, you just have to decide what\n"
+"name should be retained for the directories involved."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:328
+msgid ""
+"* ``--action=take-this`` will acknowledge Bazaar choice of leaving "
+"``white`` \n"
+"  in ``black``,\n"
+"* ``--action=take-other`` will revert Bazaar choice and move ``black`` in\n"
+"   ``white`` by issuing ``bzr mv black/white white ; bzr mv black white``,\n"
+"* ``--action=done`` will just mark the conflict as resolved."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:336
+msgid ""
+"Non-directory parent\n"
+"--------------------"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:341
+msgid ""
+"  Conflict: foo.new is not a directory, but has files in it.\n"
+"  Created directory."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:344
+msgid ""
+"This happens when one side has added files to a directory, and the other "
+"side\n"
+"has changed the directory into a file or symlink.  For example::"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:347
+msgid ""
+"  $ bzr init\n"
+"  $ bzr mkdir foo\n"
+"  $ bzr commit -m \"BASE\"\n"
+"  $ bzr branch . ../other\n"
+"  $ rmdir foo\n"
+"  $ touch foo\n"
+"  $ bzr commit -m \"THIS\"\n"
+"  $ bzr mkdir ../other/foo/bar\n"
+"  $ bzr commit ../other -m \"OTHER\"\n"
+"  $ bzr merge ../other"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:358
+msgid ""
+"To resolve that kind of conflict, you have to decide what name should be\n"
+"retained for the file, directory or symlink involved."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:363
+msgid ""
+"* ``--action=take-this`` will issue ``bzr rm --force foo.new`` and \n"
+"  ``bzr add foo``,\n"
+"* ``--action=take-other`` will issue ``bzr rm --force foo`` and \n"
+"  ``bzr mv foo.new foo``,\n"
+"* ``--action=done`` will just mark the conflict as resolved."
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:371
+msgid ""
+"MalformedTransform\n"
+"------------------"
+msgstr ""
+
+#: en/help_topics/conflict-types.txt:374
+msgid ""
+"It is possible (though very rare) for Bazaar to raise a MalformedTransform\n"
+"exception.  This means that Bazaar encountered a filesystem conflict that it "
+"was\n"
+"unable to resolve.  This usually indicates a bug.  Please let us know if "
+"you\n"
+"encounter this.  Our bug tracker is at https://launchpad.net/bzr/+bugs\n"
+msgstr ""
+
+#: en/help_topics/content-filters.txt:1
+msgid ""
+"Content Filters\n"
+"==============="
+msgstr ""
+
+#: en/help_topics/content-filters.txt:4
+msgid ""
+"Content formats\n"
+"---------------"
+msgstr ""
+
+#: en/help_topics/content-filters.txt:7
+msgid ""
+"Bazaar's content filtering allows you to store files in a different\n"
+"format from the copy in your working tree.  This lets you, or your\n"
+"co-developers, use Windows development tools that expect CRLF files\n"
+"on projects that use other line-ending conventions. Among other things,\n"
+"content filters also let Unix developers more easily work on projects\n"
+"using Windows line-ending conventions, keyword expansion/compression,\n"
+"and trailing spaces on lines in text files to be implicitly stripped\n"
+"when committed."
+msgstr ""
+
+#: en/help_topics/content-filters.txt:16
+msgid "To generalize, there are two content formats supported by Bazaar:"
+msgstr ""
+
+#: en/help_topics/content-filters.txt:18
+msgid ""
+"* a canonical format - how files are stored internally\n"
+"* a convenience format - how files are created in a working tree."
+msgstr ""
+
+#: en/help_topics/content-filters.txt:21
+msgid ""
+"\n"
+"Format conversion\n"
+"-----------------"
+msgstr ""
+
+#: en/help_topics/content-filters.txt:25
+msgid ""
+"The conversion between these formats is done by content filters.\n"
+"A content filter has two parts:"
+msgstr ""
+
+#: en/help_topics/content-filters.txt:28
+msgid ""
+"* a read converter - converts from convenience to canonical format\n"
+"* a write converter - converts from canonical to convenience format."
+msgstr ""
+
+#: en/help_topics/content-filters.txt:31
+msgid ""
+"Many of these converters will provide *round-trip* conversion,\n"
+"i.e. applying the read converter followed by the write converter\n"
+"gives back the original content. However, others may provide an\n"
+"asymmetric conversion. For example, a read converter might strip\n"
+"trailing whitespace off lines in source code while the matching\n"
+"write converter might pass content through unchanged."
+msgstr ""
+
+#: en/help_topics/content-filters.txt:38
+msgid ""
+"\n"
+"Enabling content filters\n"
+"------------------------"
+msgstr ""
+
+#: en/help_topics/content-filters.txt:42
+msgid ""
+"Content filters are typically provided by plugins, so the first step\n"
+"in using them is to install the relevant plugins and read their\n"
+"documentation. Some plugins may be very specific about which files\n"
+"they filter, e.g. only files ending in ``.java`` or ``.php``.\n"
+"In other cases, the plugin may leave it in the user's hands to\n"
+"define which files are to be filtered. This is typically done\n"
+"using rule-based preferences. See ``bzr help rules`` for general\n"
+"information about defining these."
+msgstr ""
+
+#: en/help_topics/content-filters.txt:51
+msgid ""
+"\n"
+"Impact on commands\n"
+"------------------"
+msgstr ""
+
+#: en/help_topics/content-filters.txt:55
+msgid ""
+"Read converters are only applied to commands that read content from\n"
+"a working tree, e.g. status, diff and commit. For example, ``bzr diff``\n"
+"will apply read converters to files in the working tree, then compare\n"
+"the results to the content last committed."
+msgstr ""
+
+#: en/help_topics/content-filters.txt:60
+msgid ""
+"Write converters are only applied by commands that **create files in a\n"
+"working tree**, e.g. branch, checkout, update. If you wish to see the\n"
+"canonical format of a file or tree, use ``bzr cat`` or ``bzr export``\n"
+"respectively."
+msgstr ""
+
+#: en/help_topics/content-filters.txt:65
+msgid ""
+"Note: ``bzr commit`` does not implicitly apply write converters after\n"
+"comitting files. If this makes sense for a given plugin providing\n"
+"a content filter, the plugin can usually achieve this effect by using a\n"
+"``start_commit`` or ``post_commit`` hook say. See :doc:`hooks-help`\n"
+"for more information on hooks."
+msgstr ""
+
+#: en/help_topics/content-filters.txt:71
+msgid ""
+"\n"
+"Refreshing your working tree\n"
+"----------------------------"
+msgstr ""
+
+#: en/help_topics/content-filters.txt:75
+msgid ""
+"For performance reasons, Bazaar caches the timestamps of files in\n"
+"a working tree, and assumes files are unchanged if their timestamps\n"
+"match the cached values. As a consequence, there are times when\n"
+"you may need to explicitly ask for content filtering to be reapplied\n"
+"in one or both directions, e.g. after installing or reconfiguring\n"
+"plugins providing it."
+msgstr ""
+
+#: en/help_topics/content-filters.txt:82
+msgid "Here are some general guidelines for doing this:"
+msgstr ""
+
+#: en/help_topics/content-filters.txt:84
+msgid ""
+" * To reapply read converters, ``touch`` files, i.e. update their\n"
+"   timestamp. Operations like ``bzr status`` should then reapply the\n"
+"   relevant read converters and compare the end result with the\n"
+"   canonical format."
+msgstr ""
+
+#: en/help_topics/content-filters.txt:89
+msgid ""
+" * To reapply write converters, ensure there are no local changes,\n"
+"   delete the relevant files and run ``bzr revert`` on those files."
+msgstr ""
+
+#: en/help_topics/content-filters.txt:92
+msgid ""
+"Note: In the future, it is likely that additional options will be added\n"
+"to commands to make this refreshing process faster and safer.\n"
+msgstr ""
+
+#: en/help_topics/current-formats.txt:1
+msgid "Current Storage Formats"
+msgstr ""
+
+#: en/help_topics/current-formats.txt:3
+msgid ""
+":2a:\n"
+"    (native) (default) Format for the bzr 2.0 series. Uses group-\n"
+"    compress storage. Provides rich roots which are a one-way\n"
+"    transition."
+msgstr ""
+
+#: en/help_topics/current-formats.txt:8
+msgid ""
+"\n"
+"See :doc:`formats-help` for more about storage formats."
+msgstr ""
+
+#: en/help_topics/debug-flags.txt:1
+msgid ""
+"Debug Flags\n"
+"==========="
+msgstr ""
+
+#: en/help_topics/debug-flags.txt:4
+msgid ""
+"These flags can be passed on the bzr command line or (without the ``-D``\n"
+"prefix) put in the ``debug_flags`` variable in ``bazaar.conf``."
+msgstr ""
+
+#: en/help_topics/debug-flags.txt:7
+msgid ""
+"-Dauth            Trace authentication sections used.\n"
+"-Dbytes           Print out how many bytes were transferred\n"
+"-Ddirstate        Trace dirstate activity (verbose!)\n"
+"-Derror           Instead of normal error handling, always print a "
+"traceback\n"
+"                  on error.\n"
+"-Devil            Capture call sites that do expensive or badly-scaling\n"
+"                  operations.\n"
+"-Dfetch           Trace history copying between repositories.\n"
+"-Dfilters         Emit information for debugging content filtering.\n"
+"-Dforceinvdeltas  Force use of inventory deltas during generic streaming "
+"fetch.\n"
+"-Dgraph           Trace graph traversal.\n"
+"-Dhashcache       Log every time a working file is read to determine its "
+"hash.\n"
+"-Dhooks           Trace hook execution.\n"
+"-Dhpss            Trace smart protocol requests and responses.\n"
+"-Dhpssdetail      More hpss details.\n"
+"-Dhpssvfs         Traceback on vfs access to Remote objects.\n"
+"-Dhttp            Trace http connections, requests and responses.\n"
+"-Dindex           Trace major index operations.\n"
+"-Dknit            Trace knit operations.\n"
+"-Dlock            Trace when lockdir locks are taken or released.\n"
+"-Dnoretry         If a connection is reset, fail immediately rather than\n"
+"                  retrying the request.\n"
+"-Dprogress        Trace progress bar operations.\n"
+"-Dmem_dump        Dump memory to a file upon an out of memory error.\n"
+"-Dmerge           Emit information for debugging merges.\n"
+"-Dno_apport       Don't use apport to report crashes.\n"
+"-Dno_activity \t  Don't show transport activity indicator in progress bar.\n"
+"-Dpack            Emit information about pack operations.\n"
+"-Drelock          Emit a message every time a branch or repository object "
+"is\n"
+"                  unlocked then relocked the same way.\n"
+"-Dsftp            Trace SFTP internals.\n"
+"-Dstatic_tuple    Error when a tuple is used where a StaticTuple is "
+"expected\n"
+"-Dstream          Trace fetch streams.\n"
+"-Dstrict_locks    Trace when OS locks are potentially used in a non-"
+"portable\n"
+"                  manner.\n"
+"-Dunlock          Some errors during unlock are treated as warnings.\n"
+"-DIDS_never       Never use InterDifferingSerializer when fetching.\n"
+"-DIDS_always      Always use InterDifferingSerializer to fetch if "
+"appropriate\n"
+"                  for the format, even for non-local fetches.\n"
+msgstr ""
+
+#: en/help_topics/diverged-branches.txt:1
+msgid ""
+"Diverged Branches\n"
+"================="
+msgstr ""
+
+#: en/help_topics/diverged-branches.txt:4
+msgid ""
+"When Bazaar tries to push one branch onto another, it requires that the\n"
+"destination branch must be ready to receive the source branch.  If this "
+"isn't\n"
+"the case, then we say that the branches have ``diverged``.  Branches are\n"
+"considered diverged if the destination branch's most recent commit is one "
+"that\n"
+"has not been merged (directly or indirectly) by the source branch.  To "
+"recover\n"
+"from diverged branches, one must merge the missing revisions into the "
+"source\n"
+"branch."
+msgstr ""
+
+#: en/help_topics/diverged-branches.txt:12
+msgid ""
+"This situation commonly arises when using a centralized workflow with local\n"
+"commits.  If someone else has committed new work to the mainline since your\n"
+"last pull and you have local commits that have not yet been pushed to the\n"
+"mainline, then your local branch and the mainline have diverged."
+msgstr ""
+
+#: en/help_topics/diverged-branches.txt:17
+msgid ""
+"Discovering What Has Diverged\n"
+"-----------------------------"
+msgstr ""
+
+#: en/help_topics/diverged-branches.txt:20
+msgid ""
+"The ``bzr missing`` command is used to find out what revisions are in "
+"another\n"
+"branch that are not present in the current branch, and vice-versa.  It shows "
+"a\n"
+"summary of  which extra revisions exist in each branch.  If you want to see "
+"the\n"
+"precise effects of those revisions, you can use ``bzr diff --"
+"old=other_branch``\n"
+"to show the differences between other_branch and your current branch."
+msgstr ""
+
+#: en/help_topics/diverged-branches.txt:26
+msgid ""
+"A Solution\n"
+"----------"
+msgstr ""
+
+#: en/help_topics/diverged-branches.txt:29
+msgid ""
+"The solution is to merge the revisions from the mainline into your local\n"
+"branch.  To do so, use ``bzr merge`` to get the new revisions from the\n"
+"mainline.  This merge may result in conflicts if the other developer's "
+"changes\n"
+"overlap with your changes.  These conflicts should be resolved before\n"
+"continuing. After any conflicts have been resolved, or even if there were "
+"no\n"
+"conflicts, Bazaar requires that you explicitly commit these new revisions\n"
+"to your local branch.  This requirement gives you an opportunity to test "
+"the\n"
+"resulting working tree for correctness, since the merged revisions could "
+"have\n"
+"made arbitrary changes.  After testing, you should commit the merge using\n"
+"``bzr commit``.  This clears up the diverged branches situation.  Your "
+"local\n"
+"branch can now be pushed to the mainline.\n"
+msgstr ""
+
+#: en/help_topics/eol.txt:1
+msgid ""
+"End of Line Conversion\n"
+"======================"
+msgstr ""
+
+#: en/help_topics/eol.txt:4
+msgid ""
+"EOL conversion is provided as a content filter where Bazaar internally\n"
+"stores a canonical format but outputs a convenience format. See\n"
+"``bzr help content-filters`` for general information about using these."
+msgstr ""
+
+#: en/help_topics/eol.txt:8
+msgid ""
+"Note: Content filtering is only supported in recently added formats,\n"
+"e.g. 1.14. Be sure that both the repository *and* the branch are\n"
+"in a recent format. (Just setting the format on the repository\n"
+"is not enough.) If content filtering does not appear to be working, use\n"
+"'bzr info -v' to confirm that the branch is using \"Working tree format 5\"\n"
+"or later."
+msgstr ""
+
+#: en/help_topics/eol.txt:15
+msgid ""
+"EOL conversion needs to be enabled for selected file patterns using\n"
+"rules. See ``bzr help rules`` for general information on defining rules.\n"
+"Currently, rules are only supported in $BZR_HOME/.bazaar/rules (or\n"
+"%BZR_HOME%/bazaar/2.0/rules on Windows). Branch specific rules will be\n"
+"supported in a future verison of Bazaar."
+msgstr ""
+
+#: en/help_topics/eol.txt:21
+msgid ""
+"To configure which files to filter, set ``eol`` to one of the values below.\n"
+"(If a value is not set, ``exact`` is the default.)"
+msgstr ""
+
+#: en/help_topics/eol.txt:24
+msgid ""
+" ========== ===================================== ======================\n"
+" Value      Checkout end-of-lines as              Commit end-of-lines as\n"
+" ========== ===================================== ======================\n"
+" ``native`` ``crlf`` on Windows, ``lf`` otherwise ``lf``\n"
+" ---------- ------------------------------------- ----------------------\n"
+" ``lf``     ``lf``                                ``lf``\n"
+" ---------- ------------------------------------- ----------------------\n"
+" ``crlf``   ``crlf``                              ``lf``\n"
+" ---------- ------------------------------------- ----------------------\n"
+" ``exact``  No conversion                         Exactly as in file\n"
+" ========== ===================================== ======================"
+msgstr ""
+
+#: en/help_topics/eol.txt:36
+msgid ""
+"Note: For safety reasons, no conversion is applied to any file where a null\n"
+"byte is detected in the file."
+msgstr ""
+
+#: en/help_topics/eol.txt:39
+msgid ""
+"For users working on a cross-platform project, here is a suggested rule\n"
+"to use as a starting point::"
+msgstr ""
+
+#: en/help_topics/eol.txt:42
+msgid ""
+"  [name *]\n"
+"  eol = native"
+msgstr ""
+
+#: en/help_topics/eol.txt:45
+msgid ""
+"If you have binary files that do not contain a null byte though, be\n"
+"sure to add ``eol = exact`` rules for those as well. You can do this\n"
+"by giving more explicit patterns earlier in the rules file. For example::"
+msgstr ""
+
+#: en/help_topics/eol.txt:49
+msgid ""
+"  [name *.png]\n"
+"  eol = exact"
+msgstr ""
+
+#: en/help_topics/eol.txt:55
+msgid ""
+"If your working tree is on a network drive shared by users on different\n"
+"operating systems, you typically want to force certain conventions for\n"
+"certain files. In that way, if a file is created with the wrong line\n"
+"endings or line endings get mixed during editing, it gets committed\n"
+"correctly and gets checked out correctly. For example::"
+msgstr ""
+
+#: en/help_topics/eol.txt:61
+msgid ""
+"  [name *.bat]\n"
+"  eol = crlf"
+msgstr ""
+
+#: en/help_topics/eol.txt:64
+msgid ""
+"  [name *.sh]\n"
+"  eol = lf"
+msgstr ""
+
+#: en/help_topics/eol.txt:70
+msgid ""
+"If you take the care to create files with their required endings, you can\n"
+"achieve *almost* the same thing by using ``eol = exact``. It is slightly\n"
+"safer to use ``lf`` and ``crlf`` though because edits accidentally\n"
+"introducing mixed line endings will be corrected during commit for files\n"
+"with those settings."
+msgstr ""
+
+#: en/help_topics/eol.txt:76
+msgid ""
+"If you have sample test data that deliberately has text files with mixed\n"
+"newline conventions, you can ask for those to be left alone like this::"
+msgstr ""
+
+#: en/help_topics/eol.txt:79
+msgid ""
+"  [name test_data/]\n"
+"  eol = exact"
+msgstr ""
+
+#: en/help_topics/eol.txt:85
+msgid ""
+"Note that ``exact`` does not imply the file is binary but it does mean\n"
+"that no conversion of end-of-lines will be done. (Bazaar currently relies\n"
+"of content analysis to detect binary files for commands like ``diff``.\n"
+"In the future, a ``binary = true`` rule may be added but it is not\n"
+"supported yet.)"
+msgstr ""
+
+#: en/help_topics/eol.txt:91
+msgid ""
+"If you have an existing repository with text files already stored using\n"
+"Windows newline conventions (``crlf``), then you may want to keep using "
+"that\n"
+"convention in the repository. Forcing certain files to this convention\n"
+"may also help users who do not have rules configured. To do this, set\n"
+"``eol`` to one of the values below."
+msgstr ""
+
+#: en/help_topics/eol.txt:97
+msgid ""
+" ============================ ======================== "
+"======================\n"
+" Value                        Checkout end-of-lines as Commit end-of-lines "
+"as\n"
+" ============================ ======================== "
+"======================\n"
+" ``native-with-crlf-in-repo`` ``crlf`` on Windows,     ``crlf``\n"
+"                              ``lf`` otherwise\n"
+" ---------------------------- ------------------------ "
+"----------------------\n"
+" ``lf-with-crlf-in-repo``     ``lf``                   ``crlf``\n"
+" ---------------------------- ------------------------ "
+"----------------------\n"
+" ``crlf-with-crlf-in-repo``   ``crlf``                 ``crlf``\n"
+" ============================ ======================== ======================"
+msgstr ""
+
+#: en/help_topics/eol.txt:108
+msgid ""
+"For users working on an existing project that uses Windows newline\n"
+"conventions in their Bazaar repository, this rule is suggested as a\n"
+"starting point::"
+msgstr ""
+
+#: en/help_topics/eol.txt:112
+msgid ""
+"  [name *]\n"
+"  eol = native-with-crlf-in-repo"
+msgstr ""
+
+#: en/help_topics/eol.txt:115
+msgid ""
+"For new projects, it is recommended that end-of-lines be stored as ``lf``\n"
+"and that users stick to the basic settings, i.e. ``native``, ``lf``,\n"
+"``crlf`` and ``exact``."
+msgstr ""
+
+#: en/help_topics/eol.txt:119
+msgid ""
+"Note: Bazaar's EOL conversion will convert the content of files but\n"
+"never reject files because a given line ending or mixed line endings\n"
+"are found. A precommit hook should be used if you wish to validate\n"
+"(and not just convert) content before committing.\n"
+msgstr ""
+
+#: en/help_topics/hidden-commands.txt:1
+msgid ""
+"added                List files added in working tree.\n"
+"ancestry             List all revisions merged into this branch.\n"
+"assert-fail          Test reporting of assertion failures\n"
+"bundle-info          Output interesting stats about a bundle\n"
+"bundle-revisions     Create a merge-directive for submitting changes.\n"
+"cat-revision         Write out metadata for a revision.\n"
+"dump-btree           Dump the contents of a btree index file to stdout.\n"
+"export-pot           Export command helps and error messages in po format.\n"
+"file-id              Print file_id of a particular file or directory.\n"
+"file-path            Print path of file_ids to a file or directory.\n"
+"find-merge-base      Find and print a base revision for merging two "
+"branches.\n"
+"hooks                Show hooks.\n"
+"inventory            Show inventory of the current working copy or a "
+"revision.\n"
+"local-time-offset    Show the offset in seconds from GMT to local time.\n"
+"lookup-revision      Lookup the revision-id from a revision-number\n"
+"merge-directive      Generate a merge directive for auto-merge tools.\n"
+"modified             List files modified in working tree.\n"
+"re-sign              Create a digital signature for an existing revision.\n"
+"reference            list, view and set branch locations for nested trees.\n"
+"relpath              Show path of a file relative to root\n"
+"repair-workingtree   Reset the working tree state file.\n"
+"revision-history     Display the list of revision ids on a branch.\n"
+"revision-info        Show revision number and revision id for a given "
+"revision\n"
+"                     identifier.\n"
+"rocks                Statement of optimism.\n"
+"selftest             Run internal test suite.\n"
+"shell-complete       Show appropriate completions for context.\n"
+"test-script          Run a shell-like test from a file.\n"
+"touching-revisions   Return revision-ids which affected a particular file.\n"
+"unknowns             List unknown files.\n"
+"wait-until-signalled Test helper for\n"
+"                     test_start_and_stop_bzr_subprocess_send_signal.\n"
+msgstr ""
+
+#: en/help_topics/hooks.txt:1
+msgid ""
+"\n"
+"Hooks\n"
+"====="
+msgstr ""
+
+#: en/help_topics/hooks.txt:5
+msgid ""
+"Introduction\n"
+"------------"
+msgstr ""
+
+#: en/help_topics/hooks.txt:8
+msgid "A hook of type *xxx* of class *yyy* needs to be registered using::"
+msgstr ""
+
+#: en/help_topics/hooks.txt:10
+msgid "  yyy.hooks.install_named_hook(\"xxx\", ...)"
+msgstr ""
+
+#: en/help_topics/hooks.txt:12
+msgid ""
+"See :doc:`Using hooks<../user-guide/hooks>` in the User Guide for examples."
+msgstr ""
+
+#: en/help_topics/hooks.txt:14
+msgid ""
+"The class that contains each hook is given before the hooks it supplies. "
+"For\n"
+"instance, BranchHooks as the class is the hooks class for\n"
+"`bzrlib.branch.Branch.hooks`."
+msgstr ""
+
+#: en/help_topics/hooks.txt:18
+msgid ""
+"Each description also indicates whether the hook runs on the client (the\n"
+"machine where bzr was invoked) or the server (the machine addressed by\n"
+"the branch URL).  These may be, but are not necessarily, the same machine."
+msgstr ""
+
+#: en/help_topics/hooks.txt:22
+msgid ""
+"Plugins (including hooks) are run on the server if all of these is true:"
+msgstr ""
+
+#: en/help_topics/hooks.txt:24
+msgid ""
+"  * The connection is via a smart server (accessed with a URL starting with\n"
+"    \"bzr://\", \"bzr+ssh://\" or \"bzr+http://\", or accessed via a "
+"\"http://\"\n"
+"    URL when a smart server is available via HTTP)."
+msgstr ""
+
+#: en/help_topics/hooks.txt:28
+msgid ""
+"  * The hook is either server specific or part of general infrastructure "
+"rather\n"
+"    than client specific code (such as commit)."
+msgstr ""
+
+#: en/help_topics/hooks.txt:31
+msgid ""
+"\n"
+"BranchHooks\n"
+"-----------"
+msgstr ""
+
+#: en/help_topics/hooks.txt:35
+msgid ""
+"automatic_tag_name\n"
+"~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:38
+msgid "Introduced in: 2.2"
+msgstr ""
+
+#: en/help_topics/hooks.txt:40
+msgid ""
+"Called to determine an automatic tag name for a revision.\n"
+"automatic_tag_name is called with (branch, revision_id) and should\n"
+"return a tag name or None if no tag name could be determined. The\n"
+"first non-None tag name returned will be used."
+msgstr ""
+
+#: en/help_topics/hooks.txt:45
+msgid ""
+"open\n"
+"~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:48
+msgid "Introduced in: 1.8"
+msgstr ""
+
+#: en/help_topics/hooks.txt:50
+msgid ""
+"Called with the Branch object that has been opened after a branch is\n"
+"opened."
+msgstr ""
+
+#: en/help_topics/hooks.txt:53
+msgid ""
+"post_branch_init\n"
+"~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:58
+msgid ""
+"Called after new branch initialization completes. post_branch_init is\n"
+"called with a bzrlib.branch.BranchInitHookParams. Note that init,\n"
+"branch and checkout (both heavyweight and lightweight) will all\n"
+"trigger this hook."
+msgstr ""
+
+#: en/help_topics/hooks.txt:63
+msgid ""
+"post_change_branch_tip\n"
+"~~~~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:66
+msgid "Introduced in: 1.4"
+msgstr ""
+
+#: en/help_topics/hooks.txt:68
+msgid ""
+"Called in bzr client and server after a change to the tip of a branch\n"
+"is made. post_change_branch_tip is called with a\n"
+"bzrlib.branch.ChangeBranchTipParams. Note that push, pull, commit,\n"
+"uncommit will all trigger this hook."
+msgstr ""
+
+#: en/help_topics/hooks.txt:73
+msgid ""
+"post_commit\n"
+"~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:76
+msgid "Introduced in: 0.15"
+msgstr ""
+
+#: en/help_topics/hooks.txt:78
+msgid ""
+"Called in the bzr client after a commit has completed. post_commit is\n"
+"called with (local, master, old_revno, old_revid, new_revno,\n"
+"new_revid). old_revid is NULL_REVISION for the first commit to a\n"
+"branch."
+msgstr ""
+
+#: en/help_topics/hooks.txt:83
+msgid ""
+"post_pull\n"
+"~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:88
+msgid ""
+"Called after a pull operation completes. post_pull is called with a\n"
+"bzrlib.branch.PullResult object and only runs in the bzr client."
+msgstr ""
+
+#: en/help_topics/hooks.txt:91
+msgid ""
+"post_push\n"
+"~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:96
+msgid ""
+"Called after a push operation completes. post_push is called with a\n"
+"bzrlib.branch.BranchPushResult object and only runs in the bzr client."
+msgstr ""
+
+#: en/help_topics/hooks.txt:99
+msgid ""
+"post_switch\n"
+"~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:104
+msgid ""
+"Called after a checkout switches branch. post_switch is called with a\n"
+"bzrlib.branch.SwitchHookParams."
+msgstr ""
+
+#: en/help_topics/hooks.txt:107
+msgid ""
+"post_uncommit\n"
+"~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:112
+msgid ""
+"Called in the bzr client after an uncommit completes. post_uncommit is\n"
+"called with (local, master, old_revno, old_revid, new_revno,\n"
+"new_revid) where local is the local branch or None, master is the\n"
+"target branch, and an empty branch receives new_revno of 0, new_revid\n"
+"of None."
+msgstr ""
+
+#: en/help_topics/hooks.txt:118
+msgid ""
+"pre_change_branch_tip\n"
+"~~~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:121
+msgid "Introduced in: 1.6"
+msgstr ""
+
+#: en/help_topics/hooks.txt:123
+msgid ""
+"Called in bzr client and server before a change to the tip of a branch\n"
+"is made. pre_change_branch_tip is called with a\n"
+"bzrlib.branch.ChangeBranchTipParams. Note that push, pull, commit,\n"
+"uncommit will all trigger this hook."
+msgstr ""
+
+#: en/help_topics/hooks.txt:128
+msgid ""
+"pre_commit\n"
+"~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:131
+msgid "Introduced in: 0.91"
+msgstr ""
+
+#: en/help_topics/hooks.txt:133
+msgid ""
+"Called after a commit is calculated but before it is completed.\n"
+"pre_commit is called with (local, master, old_revno, old_revid,\n"
+"future_revno, future_revid, tree_delta, future_tree). old_revid is\n"
+"NULL_REVISION for the first commit to a branch, tree_delta is a\n"
+"TreeDelta object describing changes from the basis revision. hooks\n"
+"MUST NOT modify this delta.  future_tree is an in-memory tree obtained\n"
+"from CommitBuilder.revision_tree() and hooks MUST NOT modify this\n"
+"tree."
+msgstr ""
+
+#: en/help_topics/hooks.txt:142
+msgid ""
+"set_rh\n"
+"~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:147
+msgid ""
+"Invoked whenever the revision history has been set via\n"
+"set_revision_history. The api signature is (branch, revision_history),\n"
+"and the branch will be write-locked. The set_rh hook can be expensive\n"
+"for bzr to trigger, a better hook to use is\n"
+"Branch.post_change_branch_tip."
+msgstr ""
+
+#: en/help_topics/hooks.txt:153
+msgid ""
+"transform_fallback_location\n"
+"~~~~~~~~~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:156
+msgid "Introduced in: 1.9"
+msgstr ""
+
+#: en/help_topics/hooks.txt:158
+msgid ""
+"Called when a stacked branch is activating its fallback locations.\n"
+"transform_fallback_location is called with (branch, url), and should\n"
+"return a new url. Returning the same url allows it to be used as-is,\n"
+"returning a different one can be used to cause the branch to stack on\n"
+"a closer copy of that fallback_location. Note that the branch cannot\n"
+"have history accessing methods called on it during this hook because\n"
+"the fallback locations have not been activated. When there are\n"
+"multiple hooks installed for transform_fallback_location, all are\n"
+"called with the url returned from the previous hook.The order is\n"
+"however undefined."
+msgstr ""
+
+#: en/help_topics/hooks.txt:169
+msgid ""
+"CommandHooks\n"
+"------------"
+msgstr ""
+
+#: en/help_topics/hooks.txt:172
+msgid ""
+"extend_command\n"
+"~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:175
+msgid "Introduced in: 1.13"
+msgstr ""
+
+#: en/help_topics/hooks.txt:177
+msgid ""
+"Called after creating a command object to allow modifications such as\n"
+"adding or removing options, docs etc. Called with the new\n"
+"bzrlib.commands.Command object."
+msgstr ""
+
+#: en/help_topics/hooks.txt:181
+msgid ""
+"get_command\n"
+"~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:184
+msgid "Introduced in: 1.17"
+msgstr ""
+
+#: en/help_topics/hooks.txt:186
+msgid ""
+"Called when creating a single command. Called with (cmd_or_None,\n"
+"command_name). get_command should either return the cmd_or_None\n"
+"parameter, or a replacement Command object that should be used for the\n"
+"command. Note that the Command.hooks hooks are core infrastructure.\n"
+"Many users will prefer to use bzrlib.commands.register_command or\n"
+"plugin_cmds.register_lazy."
+msgstr ""
+
+#: en/help_topics/hooks.txt:193
+msgid ""
+"get_missing_command\n"
+"~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:198
+msgid ""
+"Called when creating a single command if no command could be found.\n"
+"Called with (command_name). get_missing_command should either return\n"
+"None, or a Command object to be used for the command."
+msgstr ""
+
+#: en/help_topics/hooks.txt:202
+msgid ""
+"list_commands\n"
+"~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:207
+msgid ""
+"Called when enumerating commands. Called with a set of cmd_name\n"
+"strings for all the commands found so far. This set  is safe to mutate\n"
+"- e.g. to remove a command. list_commands should return the updated\n"
+"set of command names."
+msgstr ""
+
+#: en/help_topics/hooks.txt:212
+msgid ""
+"_ConfigHooks\n"
+"------------"
+msgstr ""
+
+#: en/help_topics/hooks.txt:215
+msgid ""
+"get\n"
+"~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:218
+msgid "Introduced in: 2.4"
+msgstr ""
+
+#: en/help_topics/hooks.txt:220
+msgid ""
+"Invoked when a config option is read. The signature is (stack, name,\n"
+"value)."
+msgstr ""
+
+#: en/help_topics/hooks.txt:223
+msgid ""
+"load\n"
+"~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:228
+msgid "Invoked when a config store is loaded. The signature is (store)."
+msgstr ""
+
+#: en/help_topics/hooks.txt:230
+msgid ""
+"remove\n"
+"~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:235
+msgid ""
+"Invoked when a config option is removed. The signature is (stack,\n"
+"name)."
+msgstr ""
+
+#: en/help_topics/hooks.txt:238
+msgid ""
+"save\n"
+"~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:243
+msgid "Invoked when a config store is saved. The signature is (store)."
+msgstr ""
+
+#: en/help_topics/hooks.txt:245
+msgid ""
+"set\n"
+"~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:250
+msgid ""
+"Invoked when a config option is set. The signature is (stack, name,\n"
+"value)."
+msgstr ""
+
+#: en/help_topics/hooks.txt:253
+msgid ""
+"ControlDirHooks\n"
+"---------------"
+msgstr ""
+
+#: en/help_topics/hooks.txt:256
+msgid ""
+"post_repo_init\n"
+"~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:261
+msgid ""
+"Invoked after a repository has been initialized. post_repo_init is\n"
+"called with a bzrlib.controldir.RepoInitHookParams."
+msgstr ""
+
+#: en/help_topics/hooks.txt:264
+msgid ""
+"pre_open\n"
+"~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:267
+msgid "Introduced in: 1.14"
+msgstr ""
+
+#: en/help_topics/hooks.txt:269
+msgid ""
+"Invoked before attempting to open a ControlDir with the transport that\n"
+"the open will use."
+msgstr ""
+
+#: en/help_topics/hooks.txt:272
+msgid ""
+"InfoHooks\n"
+"---------"
+msgstr ""
+
+#: en/help_topics/hooks.txt:275
+msgid ""
+"repository\n"
+"~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:278
+msgid "Introduced in: 1.15"
+msgstr ""
+
+#: en/help_topics/hooks.txt:280
+msgid ""
+"Invoked when displaying the statistics for a repository. repository is\n"
+"called with a statistics dictionary as returned by the repository and\n"
+"a file-like object to write to."
+msgstr ""
+
+#: en/help_topics/hooks.txt:284
+msgid ""
+"LockHooks\n"
+"---------"
+msgstr ""
+
+#: en/help_topics/hooks.txt:287
+msgid ""
+"lock_acquired\n"
+"~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:292
+msgid "Called with a bzrlib.lock.LockResult when a physical lock is acquired."
+msgstr ""
+
+#: en/help_topics/hooks.txt:294
+msgid ""
+"lock_broken\n"
+"~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:299
+msgid "Called with a bzrlib.lock.LockResult when a physical lock is broken."
+msgstr ""
+
+#: en/help_topics/hooks.txt:301
+msgid ""
+"lock_released\n"
+"~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:306
+msgid "Called with a bzrlib.lock.LockResult when a physical lock is released."
+msgstr ""
+
+#: en/help_topics/hooks.txt:308
+msgid ""
+"MergeHooks\n"
+"----------"
+msgstr ""
+
+#: en/help_topics/hooks.txt:311
+msgid ""
+"merge_file_content\n"
+"~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:314
+msgid "Introduced in: 2.1"
+msgstr ""
+
+#: en/help_topics/hooks.txt:316
+msgid ""
+"Called with a bzrlib.merge.Merger object to create a per file merge\n"
+"object when starting a merge. Should return either None or a subclass\n"
+"of ``bzrlib.merge.AbstractPerFileMerger``. Such objects will then be\n"
+"called per file that needs to be merged (including when one side has\n"
+"deleted the file and the other has changed it). See the\n"
+"AbstractPerFileMerger API docs for details on how it is used by merge."
+msgstr ""
+
+#: en/help_topics/hooks.txt:323
+msgid ""
+"MergeDirectiveHooks\n"
+"-------------------"
+msgstr ""
+
+#: en/help_topics/hooks.txt:326
+msgid ""
+"merge_request_body\n"
+"~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:329
+msgid "Introduced in: 1.15.0"
+msgstr ""
+
+#: en/help_topics/hooks.txt:331
+msgid ""
+"Called with a MergeRequestBodyParams when a body is needed for a merge\n"
+"request.  Callbacks must return a body.  If more than one callback is\n"
+"registered, the output of one callback is provided to the next."
+msgstr ""
+
+#: en/help_topics/hooks.txt:335
+msgid ""
+"MessageEditorHooks\n"
+"------------------"
+msgstr ""
+
+#: en/help_topics/hooks.txt:338
+msgid ""
+"commit_message_template\n"
+"~~~~~~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:341
+msgid "Introduced in: 1.10"
+msgstr ""
+
+#: en/help_topics/hooks.txt:343
+msgid ""
+"Called when a commit message is being generated.\n"
+"commit_message_template is called with the bzrlib.commit.Commit object\n"
+"and the message that is known so far. commit_message_template must\n"
+"return a new message to use (which could be the same as it was given).\n"
+"When there are multiple hooks registered for commit_message_template,\n"
+"they are chained with the result from the first passed into the\n"
+"second, and so on."
+msgstr ""
+
+#: en/help_topics/hooks.txt:351
+msgid ""
+"set_commit_message\n"
+"~~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:356
+msgid ""
+"Set a fixed commit message. set_commit_message is called with the\n"
+"bzrlib.commit.Commit object (so you can also change e.g. revision\n"
+"properties by editing commit.builder._revprops) and the message so\n"
+"far. set_commit_message must return the message to use or None if it\n"
+"should use the message editor as normal."
+msgstr ""
+
+#: en/help_topics/hooks.txt:362
+msgid ""
+"MutableTreeHooks\n"
+"----------------"
+msgstr ""
+
+#: en/help_topics/hooks.txt:368
+msgid "Introduced in: 2.0"
+msgstr ""
+
+#: en/help_topics/hooks.txt:370
+msgid ""
+"Called after a commit is performed on a tree. The hook is called with\n"
+"a bzrlib.mutabletree.PostCommitHookParams object. The mutable tree the\n"
+"commit was performed on is available via the mutable_tree attribute of\n"
+"that object."
+msgstr ""
+
+#: en/help_topics/hooks.txt:375
+msgid ""
+"start_commit\n"
+"~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:380
+msgid ""
+"Called before a commit is performed on a tree. The start commit hook\n"
+"is able to change the tree before the commit takes place. start_commit\n"
+"is called with the bzrlib.mutabletree.MutableTree that the commit is\n"
+"being performed on."
+msgstr ""
+
+#: en/help_topics/hooks.txt:385
+msgid ""
+"SmartClientHooks\n"
+"----------------"
+msgstr ""
+
+#: en/help_topics/hooks.txt:388
+msgid ""
+"call\n"
+"~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:391
+msgid "Introduced in: unknown"
+msgstr ""
+
+#: en/help_topics/hooks.txt:393
+msgid ""
+"Called when the smart client is submitting a request to the smart\n"
+"server. Called with a bzrlib.smart.client.CallHookParams object.\n"
+"Streaming request bodies, and responses, are not accessible."
+msgstr ""
+
+#: en/help_topics/hooks.txt:397
+msgid ""
+"SmartServerHooks\n"
+"----------------"
+msgstr ""
+
+#: en/help_topics/hooks.txt:400
+msgid ""
+"server_exception\n"
+"~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:405
+msgid ""
+"Called by the bzr server when an exception occurs. server_exception is\n"
+"called with the sys.exc_info() tuple return true for the hook if the\n"
+"exception has been handled, in which case the server will exit\n"
+"normally."
+msgstr ""
+
+#: en/help_topics/hooks.txt:410
+msgid ""
+"server_started\n"
+"~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:413
+msgid "Introduced in: 0.16"
+msgstr ""
+
+#: en/help_topics/hooks.txt:415
+msgid ""
+"Called by the bzr server when it starts serving a directory.\n"
+"server_started is called with (backing urls, public url), where\n"
+"backing_url is a list of URLs giving the server-specific directory\n"
+"locations, and public_url is the public URL for the directory being\n"
+"served."
+msgstr ""
+
+#: en/help_topics/hooks.txt:421
+msgid ""
+"server_started_ex\n"
+"~~~~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:426
+msgid ""
+"Called by the bzr server when it starts serving a directory.\n"
+"server_started is called with (backing_urls, server_obj)."
+msgstr ""
+
+#: en/help_topics/hooks.txt:429
+msgid ""
+"server_stopped\n"
+"~~~~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:434
+msgid ""
+"Called by the bzr server when it stops serving a directory.\n"
+"server_stopped is called with the same parameters as the\n"
+"server_started hook: (backing_urls, public_url)."
+msgstr ""
+
+#: en/help_topics/hooks.txt:438
+msgid ""
+"StatusHooks\n"
+"-----------"
+msgstr ""
+
+#: en/help_topics/hooks.txt:441
+msgid ""
+"post_status\n"
+"~~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:444
+msgid "Introduced in: 2.3"
+msgstr ""
+
+#: en/help_topics/hooks.txt:446
+msgid ""
+"Called with argument StatusHookParams after Bazaar has displayed the\n"
+"status. StatusHookParams has the attributes (old_tree, new_tree,\n"
+"to_file, versioned, show_ids, short, verbose). The last four arguments\n"
+"correspond to the command line options specified by the user for the\n"
+"status command. to_file is the output stream for writing."
+msgstr ""
+
+#: en/help_topics/hooks.txt:452
+msgid ""
+"pre_status\n"
+"~~~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:457
+msgid ""
+"Called with argument StatusHookParams before Bazaar displays the\n"
+"status. StatusHookParams has the attributes (old_tree, new_tree,\n"
+"to_file, versioned, show_ids, short, verbose). The last four arguments\n"
+"correspond to the command line options specified by the user for the\n"
+"status command. to_file is the output stream for writing."
+msgstr ""
+
+#: en/help_topics/hooks.txt:463
+msgid ""
+"RioVersionInfoBuilderHooks\n"
+"--------------------------"
+msgstr ""
+
+#: en/help_topics/hooks.txt:466
+msgid ""
+"revision\n"
+"~~~~~~~~"
+msgstr ""
+
+#: en/help_topics/hooks.txt:471
+msgid ""
+"Invoked when adding information about a revision to the RIO stanza\n"
+"that is printed. revision is called with a revision object and a RIO\n"
+"stanza.\n"
+msgstr ""
+
+#: en/help_topics/location-alias.txt:1
+msgid ""
+"Location aliases\n"
+"================"
+msgstr ""
+
+#: en/help_topics/location-alias.txt:4
+msgid ""
+"Bazaar defines several aliases for locations associated with a branch.  "
+"These\n"
+"can be used with most commands that expect a location, such as `bzr push`."
+msgstr ""
+
+#: en/help_topics/location-alias.txt:7
+msgid "The aliases are::"
+msgstr ""
+
+#: en/help_topics/location-alias.txt:9
+msgid ""
+"  :bound     The branch this branch is bound to, for bound branches.\n"
+"  :parent    The parent of this branch.\n"
+"  :public    The public location of this branch.\n"
+"  :push      The saved location used for `bzr push` with no arguments.\n"
+"  :submit    The submit branch for this branch.\n"
+"  :this      This branch."
+msgstr ""
+
+#: en/help_topics/location-alias.txt:16
+msgid "For example, to push to the parent location::"
+msgstr ""
+
+#: en/help_topics/location-alias.txt:18
+msgid "    bzr push :parent\n"
+msgstr ""
+
+#: en/help_topics/log-formats.txt:1
+msgid ""
+"Log Formats\n"
+"==========="
+msgstr ""
+
+#: en/help_topics/log-formats.txt:4
+msgid ""
+"A log format controls how information about each revision is displayed.\n"
+"The standard log formats are compared below::"
+msgstr ""
+
+#: en/help_topics/log-formats.txt:7
+msgid ""
+"  Feature                 long           short         line\n"
+"  ----------------------  -------------  ------------  -------------------\n"
+"  design goal             detailed view  concise view  1 revision per line\n"
+"  committer               name+email     name only     name only\n"
+"  author                  name+email     -             -\n"
+"  date-time format        full           date only     date only\n"
+"  commit message          full           full          top line\n"
+"  tags                    yes            yes           yes\n"
+"  merges indicator        -              yes           -\n"
+"  status/delta            optional       optional      -\n"
+"  diff/patch              optional       optional      -\n"
+"  revision-id             optional       optional      -\n"
+"  branch nick             yes            -             -\n"
+"  foreign vcs properties  yes            yes           -\n"
+"  preferred levels        all            1             1\n"
+"  digital signature       optional       -             -"
+msgstr ""
+
+#: en/help_topics/log-formats.txt:24
+msgid ""
+"The default format is ``long``. To change this, define the ``log_format``\n"
+"setting in the ``[DEFAULT]`` section of ``bazaar.conf`` like this (say)::"
+msgstr ""
+
+#: en/help_topics/log-formats.txt:27
+msgid ""
+"  [DEFAULT]\n"
+"  log_format = short"
+msgstr ""
+
+#: en/help_topics/log-formats.txt:30
+msgid ""
+"Alternatively, to change the log format used for a given query, use the\n"
+"--long, --short or --line options."
+msgstr ""
+
+#: en/help_topics/log-formats.txt:33
+msgid ""
+"If one of the standard log formats does not meet your needs, additional\n"
+"formats can be provided by plugins.\n"
+msgstr ""
+
+#: en/help_topics/other-formats.txt:1
+msgid "Other Storage Formats"
+msgstr ""
+
+#: en/help_topics/other-formats.txt:3
+msgid "Experimental formats are shown below."
+msgstr ""
+
+#: en/help_topics/other-formats.txt:5
+msgid ""
+":development-colo:\n"
+"    (native) The 2a format with experimental support for colocated\n"
+"    branches."
+msgstr ""
+
+#: en/help_topics/other-formats.txt:9
+msgid ""
+"\n"
+"Deprecated formats are shown below."
+msgstr ""
+
+#: en/help_topics/other-formats.txt:12
+msgid ""
+":pack-0.92:\n"
+"    (native) Pack-based format used in 1.x series. Introduced in 0.92.\n"
+"    Interoperates with bzr repositories before 0.92 but cannot be read\n"
+"    by bzr < 0.92."
+msgstr ""
+
+#: en/help_topics/patterns.txt:1
+msgid ""
+"Patterns\n"
+"========"
+msgstr ""
+
+#: en/help_topics/patterns.txt:4
+msgid ""
+"Bazaar uses patterns to match files at various times. For example,\n"
+"the ``add`` command skips over files that match ignore patterns\n"
+"and preferences can be associated with files using rule patterns.\n"
+"The pattern syntax is described below."
+msgstr ""
+
+#: en/help_topics/patterns.txt:9
+msgid ""
+"Trailing slashes on patterns are ignored. If the pattern contains a\n"
+"slash or is a regular expression, it is compared to the whole path\n"
+"from the branch root. Otherwise, it is compared to only the last\n"
+"component of the path. To match a file only in the root directory,\n"
+"prepend ``./``. Patterns specifying absolute paths are not allowed."
+msgstr ""
+
+#: en/help_topics/patterns.txt:15
+msgid "Patterns may include globbing wildcards such as::"
+msgstr ""
+
+#: en/help_topics/patterns.txt:17
+msgid ""
+"  ? - Matches any single character except '/'\n"
+"  * - Matches 0 or more characters except '/'\n"
+"  /**/ - Matches 0 or more directories in a path\n"
+"  [a-z] - Matches a single character from within a group of characters"
+msgstr ""
+
+#: en/help_topics/patterns.txt:22
+msgid ""
+"Patterns may also be `Python regular expressions`_. Regular expression\n"
+"patterns are identified by a ``RE:`` prefix followed by the regular\n"
+"expression.  Regular expression patterns may not include named or\n"
+"numbered groups."
+msgstr ""
+
+#: en/help_topics/patterns.txt:27
+msgid ""
+"Case insensitive ignore patterns can be specified with regular expressions\n"
+"by using the ``i`` (for ignore case) flag in the pattern."
+msgstr ""
+
+#: en/help_topics/patterns.txt:30
+msgid "For example, a case insensitive match for ``foo`` may be specified as::"
+msgstr ""
+
+#: en/help_topics/patterns.txt:32
+msgid "  RE:(?i)foo"
+msgstr ""
+
+#: en/help_topics/patterns.txt:34
+msgid ""
+"Ignore patterns may be prefixed with ``!``, which means that a filename\n"
+"matched by that pattern will not be ignored."
+msgstr ""
+
+#: en/help_topics/patterns.txt:37
+msgid ".. _Python regular expressions: http://docs.python.org/library/re.html"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:1
+msgid "Revision Identifiers"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:3
+msgid ""
+"A revision identifier refers to a specific state of a branch's history.  It\n"
+"can be expressed in several ways.  It can begin with a keyword to\n"
+"unambiguously specify a given lookup type; some examples are 'last:1',\n"
+"'before:yesterday' and 'submit:'."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:8
+msgid ""
+"Alternately, it can be given without a keyword, in which case it will be\n"
+"checked as a revision number, a tag, a revision id, a date specification, or "
+"a\n"
+"branch specification, in that order.  For example, 'date:today' could be\n"
+"written as simply 'today', though if you have a tag called 'today' that "
+"will\n"
+"be found first."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:14
+msgid ""
+"If 'REV1' and 'REV2' are revision identifiers, then 'REV1..REV2' denotes a\n"
+"revision range. Examples: '3647..3649', 'date:yesterday..-1' and\n"
+"'branch:/path/to/branch1/..branch:/branch2' (note that there are no quotes "
+"or\n"
+"spaces around the '..')."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:19
+msgid ""
+"Ranges are interpreted differently by different commands. To the \"log\" "
+"command,\n"
+"a range is a sequence of log messages, but to the \"diff\" command, the "
+"range\n"
+"denotes a change between revisions (and not a sequence of changes).  In\n"
+"addition, \"log\" considers a closed range whereas \"diff\" and \"merge\" "
+"consider it\n"
+"to be open-ended, that is, they include one end but not the other.  For "
+"example:\n"
+"\"bzr log -r 3647..3649\" shows the messages of revisions 3647, 3648 and "
+"3649,\n"
+"while \"bzr diff -r 3647..3649\" includes the changes done in revisions 3648 "
+"and\n"
+"3649, but not 3647."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:28
+msgid "The keywords used as revision selection methods are the following:"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:30
+msgid ""
+":revid:\n"
+"\tSelects a revision using the revision id.\n"
+":submit:\n"
+"\tSelects a common ancestor with the submit branch.\n"
+":ancestor:\n"
+"\tSelects a common ancestor with a second branch.\n"
+":date:\n"
+"\tSelects a revision on the basis of a datestamp.\n"
+":branch:\n"
+"\tSelects the last revision of a specified branch.\n"
+":tag:\n"
+"\tSelects a revision identified by a tag name.\n"
+":revno:\n"
+"\tSelects a revision using a number.\n"
+":before:\n"
+"\tSelects the parent of the revision specified.\n"
+":annotate:\n"
+"\tSelect the revision that last modified the specified line.\n"
+":mainline:\n"
+"\tSelect mainline revision that merged the specified revision.\n"
+":last:\n"
+"\tSelects the nth revision from the end."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:53
+msgid "In addition, plugins can provide other keywords."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:55
+msgid "A detailed description of each keyword is given below."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:57
+msgid ":revid:"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:59
+msgid ""
+"    Supply a specific revision id, that can be used to specify any\n"
+"    revision id in the ancestry of the branch.\n"
+"    Including merges, and pending merges.\n"
+"    Examples::"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:64
+msgid ""
+"      revid:aaaa@bbbb-123456789 -> Select revision 'aaaa@bbbb-123456789'"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:66
+msgid ":submit:"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:68
+msgid ""
+"    Diffing against this shows all the changes that were made in this "
+"branch,\n"
+"    and is a good predictor of what merge will do.  The submit branch is\n"
+"    used by the bundle and merge directive commands.  If no submit branch\n"
+"    is specified, the parent branch is used instead."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:73
+msgid ""
+"    The common ancestor is the last revision that existed in both\n"
+"    branches. Usually this is the branch point, but it could also be\n"
+"    a revision that was merged."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:77
+msgid "    Examples::"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:79
+msgid "      $ bzr diff -r submit:"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:81
+msgid ":ancestor:"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:83
+msgid "    Supply the path to a branch to select the common ancestor."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:89
+msgid ""
+"    This is frequently used with 'diff' to return all of the changes\n"
+"    that your branch introduces, while excluding the changes that you\n"
+"    have not merged from the remote branch."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:95
+msgid ""
+"      ancestor:/path/to/branch\n"
+"      $ bzr diff -r ancestor:../../mainline/branch"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:98
+msgid ":date:"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:100
+msgid ""
+"    Supply a datestamp to select the first revision that matches the date.\n"
+"    Date can be 'yesterday', 'today', 'tomorrow' or a YYYY-MM-DD string.\n"
+"    Matches the first entry after a given date (either at midnight or\n"
+"    at a specified time)."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:105
+msgid "    One way to display all the changes since yesterday would be::"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:107
+msgid "        bzr log -r date:yesterday.."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:111
+msgid ""
+"      date:yesterday            -> select the first revision since "
+"yesterday\n"
+"      date:2006-08-14,17:10:14  -> select the first revision after\n"
+"                                   August 14th, 2006 at 5:10pm."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:115
+msgid ":branch:"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:117
+msgid "    Supply the path to a branch to select its last revision."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:121
+msgid "      branch:/path/to/branch"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:123
+msgid ":tag:"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:125
+msgid "    Tags are stored in the branch and created by the 'tag' command."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:127
+msgid ":revno:"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:129
+msgid ""
+"    Use an integer to specify a revision in the history of the branch.\n"
+"    Optionally a branch can be specified.  A negative number will count\n"
+"    from the end of the branch (-1 is the last revision, -2 the previous\n"
+"    one). If the negative number is larger than the branch's history, the\n"
+"    first revision is returned.\n"
+"    Examples::"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:136
+msgid ""
+"      revno:1                   -> return the first revision of this branch\n"
+"      revno:3:/path/to/branch   -> return the 3rd revision of\n"
+"                                   the branch '/path/to/branch'\n"
+"      revno:-1                  -> The last revision in a branch.\n"
+"      -2:http://other/branch    -> The second to last revision in the\n"
+"                                   remote branch.\n"
+"      -1000000                  -> Most likely the first revision, unless\n"
+"                                   your history is very long."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:145
+msgid ":before:"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:147
+msgid ""
+"    Supply any revision spec to return the parent of that revision.  This "
+"is\n"
+"    mostly useful when inspecting revisions that are not in the revision "
+"history\n"
+"    of a branch."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:151
+msgid ""
+"    It is an error to request the parent of the null revision (before:0)."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:155
+msgid ""
+"      before:1913    -> Return the parent of revno 1913 (revno 1912)\n"
+"      before:revid:aaaa@bbbb-1234567890  -> return the parent of revision\n"
+"                                            aaaa@bbbb-1234567890\n"
+"      bzr diff -r before:1913..1913\n"
+"            -> Find the changes between revision 1913 and its parent "
+"(1912).\n"
+"               (What changes did revision 1913 introduce).\n"
+"               This is equivalent to:  bzr diff -c 1913"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:163
+msgid ":annotate:"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:165
+msgid ""
+"    Select the revision that last modified the specified line.  Line is\n"
+"    specified as path:number.  Path is a relative path to the file.  "
+"Numbers\n"
+"    start at 1, and are relative to the current version, not the last-\n"
+"    committed version of the file."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:170
+msgid ":mainline:"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:172
+msgid ""
+"    Select the revision that merged the specified revision into mainline."
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:174
+msgid ":last:"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:176
+msgid ""
+"    Supply a positive number to get the nth revision from the end.\n"
+"    This is the same as supplying negative numbers to the 'revno:' spec.\n"
+"    Examples::"
+msgstr ""
+
+#: en/help_topics/revisionspec.txt:180
+msgid ""
+"      last:1        -> return the last revision\n"
+"      last:3        -> return the revision 2 before the end.\n"
+msgstr ""
+
+#: en/help_topics/rules.txt:1
+msgid ""
+"Rules\n"
+"====="
+msgstr ""
+
+#: en/help_topics/rules.txt:7
+msgid ""
+"Rules are defined in ini file format where the sections are file glob\n"
+"patterns and the contents of each section are the preferences for files\n"
+"matching that pattern(s). For example::"
+msgstr ""
+
+#: en/help_topics/rules.txt:11
+msgid ""
+"    [name *.bat]\n"
+"    eol = native"
+msgstr ""
+
+#: en/help_topics/rules.txt:14
+msgid ""
+"    [name *.html *.xml]\n"
+"    keywords = xml_escape"
+msgstr ""
+
+#: en/help_topics/rules.txt:17
+msgid ""
+"Preferences like these are useful for commands and plugins wishing to\n"
+"provide custom behaviour for selected files.  For more information on \n"
+"end of line conversion see :doc:`eol-help`.\n"
+"Keyword support is provided by the `keywords plugin\n"
+"<http://doc.bazaar.canonical.com/plugins/en/keywords-plugin.html>`_."
+msgstr ""
+
+#: en/help_topics/rules.txt:23
+msgid ""
+"Files\n"
+"-----"
+msgstr ""
+
+#: en/help_topics/rules.txt:26
+msgid ""
+"Default rules for all branches are defined in the optional file\n"
+"``BZR_HOME/rules``. "
+msgstr ""
+
+#: en/help_topics/rules.txt:29
+msgid ""
+"Rule Patterns\n"
+"-------------"
+msgstr ""
+
+#: en/help_topics/rules.txt:32
+msgid ""
+"Patterns are ordered and searching stops as soon as one matches.\n"
+"As a consequence, more explicit patterns should be placed towards\n"
+"the top of the file. Rule patterns use exactly the same conventions\n"
+"as ignore patterns. See :doc:`patterns-help` for details."
+msgstr ""
+
+#: en/help_topics/rules.txt:37
+msgid ".. note::"
+msgstr ""
+
+#: en/help_topics/rules.txt:39
+msgid ""
+"   Patterns containing square brackets or spaces should be\n"
+"   surrounded in quotes to ensure they are correctly parsed.\n"
+msgstr ""
+
+#: en/help_topics/topics.txt:1
+msgid ""
+"authentication       Information on configuring authentication\n"
+"basic                Basic commands\n"
+"branches             Information on what a branch is\n"
+"bugs                 Bug tracker settings\n"
+"checkouts            Information on what a checkout is\n"
+"commands             Basic help for all commands\n"
+"configuration        Details on the configuration settings available\n"
+"conflict-types       Types of conflicts and what to do about them\n"
+"content-filters      Conversion of content into/from working trees\n"
+"criss-cross          Information on criss-cross merging\n"
+"current-formats      Current storage formats\n"
+"debug-flags          Options to show or record debug information\n"
+"diverged-branches    How to fix diverged branches\n"
+"env-variables        Environment variable names and values\n"
+"eol                  Information on end-of-line handling\n"
+"files                Information on configuration and log files\n"
+"formats              Information on choosing a storage format\n"
+"global-options       Options that control how Bazaar runs\n"
+"hidden-commands      All hidden commands\n"
+"hooks                Points at which custom processing can be added\n"
+"launchpad            Using Bazaar with Launchpad.net\n"
+"location-alias       Aliases for remembered locations\n"
+"log-formats          Details on the logging formats available\n"
+"other-formats        Experimental and deprecated storage formats\n"
+"patterns             Information on the pattern syntax\n"
+"repositories         Basic information on shared repositories.\n"
+"revisionspec         Explain how to use --revision\n"
+"rules                Information on defining rule-based preferences\n"
+"standalone-trees     Information on what a standalone tree is\n"
+"standard-options     Options that can be used with any command\n"
+"status-flags         Help on status flags\n"
+"sync-for-reconfigure Steps to resolve \"out-of-sync\" when reconfiguring\n"
+"topics               Topics list\n"
+"url-special-chars    Special character handling in URLs\n"
+"urlspec              Supported transport protocols\n"
+"working-trees        Information on working trees\n"
+msgstr ""
+
+#: en/help_topics/url-special-chars.txt:1
+msgid ""
+"Special character handling in URLs\n"
+"=================================="
+msgstr ""
+
+#: en/help_topics/url-special-chars.txt:4
+msgid "Bazaar allows locations to be specified in multiple ways, either:"
+msgstr ""
+
+#: en/help_topics/url-special-chars.txt:6
+msgid " * Fully qualified URLs"
+msgstr ""
+
+#: en/help_topics/url-special-chars.txt:8
+msgid " * File system paths, relative or absolute"
+msgstr ""
+
+#: en/help_topics/url-special-chars.txt:10
+msgid ""
+"Internally bzr treats all locations as URLs. For any file system paths\n"
+"that are specified it will automatically determine the appropriate URL\n"
+"representation, and escape special characters where necessary."
+msgstr ""
+
+#: en/help_topics/url-special-chars.txt:14
+msgid ""
+"There are a few characters which have special meaning in URLs and need "
+"careful\n"
+"handling to avoid ambiguities. Characters can be escaped with a % and a hex\n"
+"value in URLs. Any non-ASCII characters in a file path will automatically "
+"be\n"
+"urlencoded when the path is converted to a URL."
+msgstr ""
+
+#: en/help_topics/url-special-chars.txt:19
+msgid ""
+"URLs represent non-ASCII characters in an encoding defined by the server, "
+"but\n"
+"usually UTF-8.  The % escapes should be of the UTF-8 bytes.  Bazaar tries to "
+"be\n"
+"generous in what it accepts as a URL and to print them in a way that\n"
+"will be readable."
+msgstr ""
+
+#: en/help_topics/url-special-chars.txt:24
+msgid ""
+"For example, if you have a directory named '/tmp/%2False' these are all "
+"valid\n"
+"ways of accessing the content (0x2F, or 47, is the ASCII code for forward "
+"slash)::"
+msgstr ""
+
+#: en/help_topics/url-special-chars.txt:27
+msgid ""
+"  cd /tmp\n"
+"  bzr log /tmp/%2False\n"
+"  bzr log %2False\n"
+"  bzr log file:///tmp/%252False\n"
+"  bzr log file://localhost/tmp/%252False\n"
+"  bzr log file:%252False"
+msgstr ""
+
+#: en/help_topics/url-special-chars.txt:34
+msgid "These are valid but do not refer to the same file::"
+msgstr ""
+
+#: en/help_topics/url-special-chars.txt:36
+msgid ""
+"  bzr log file:///tmp/%2False (refers to a file called /tmp/\\/alse)\n"
+"  bzr log %252False (refers to a file called /tmp/%252False)"
+msgstr ""
+
+#: en/help_topics/url-special-chars.txt:39
+msgid ""
+"Comma also has special meaning in URLs, because it denotes `segment "
+"parameters`_"
+msgstr ""
+
+#: en/help_topics/url-special-chars.txt:41
+msgid ""
+"_`segment parameters`: http://www.ietf.org/rfc/rfc3986.txt (section 3.3)"
+msgstr ""
+
+#: en/help_topics/url-special-chars.txt:43
+msgid ""
+"Comma is also special in any file system paths that are specified. To use a "
+"literal\n"
+"comma in a file system path, specify a URL and URL encode the comma::"
+msgstr ""
+
+#: en/help_topics/url-special-chars.txt:46
+msgid ""
+"  bzr log foo,branch=bla # path \"foo\" with the segment parameter \"branch"
+"\" set to \"bla\"\n"
+"  bzr log file:foo%2Cbranch=bla # path \"foo,branch=bla\"\n"
+"  bzr log file:foo,branch=bla # path \"foo\" with segment parameter \"branch"
+"\" set to \"bla\"\n"
+msgstr ""
+
+#: en/help_topics/urlspec.txt:1
+msgid "URL Identifiers"
+msgstr ""
+
+#: en/help_topics/urlspec.txt:3
+msgid "Supported URL prefixes::"
+msgstr ""
+
+#: en/help_topics/urlspec.txt:5
+msgid ""
+"  aftp://             Access using active FTP.\n"
+"  bzr://              Fast access using the Bazaar smart server.\n"
+"  bzr+ssh://          Fast access using the Bazaar smart server over SSH.\n"
+"  file://             Access using the standard filesystem (default)\n"
+"  ftp://              Access using passive FTP.\n"
+"  http://             Read-only access of branches exported on the web.\n"
+"  https://            Read-only access of branches exported on the web using "
+"SSL.\n"
+"  sftp://             Access using SFTP (most SSH servers provide SFTP)."
+msgstr ""
+
+#: en/help_topics/urlspec.txt:14
+msgid "Supported modifiers::"
+msgstr ""
+
+#: en/help_topics/urlspec.txt:16
+msgid "  gio+                Access using any GIO supported protocols."
+msgstr ""
+
+#: en/help_topics/urlspec.txt:18
+msgid "Bazaar supports all of the standard parts within the URL::"
+msgstr ""
+
+#: en/help_topics/urlspec.txt:20
+msgid "  <protocol>://[user[:password]@]host[:port]/[path]"
+msgstr ""
+
+#: en/help_topics/urlspec.txt:22
+msgid "allowing URLs such as::"
+msgstr ""
+
+#: en/help_topics/urlspec.txt:24
+msgid "  http://bzruser:BadPass@bzr.example.com:8080/bzr/trunk"
+msgstr ""
+
+#: en/help_topics/urlspec.txt:26
+msgid ""
+"For bzr+ssh:// and sftp:// URLs, Bazaar also supports paths that begin\n"
+"with '~' as meaning that the rest of the path should be interpreted\n"
+"relative to the remote user's home directory.  For example if the user\n"
+"``remote`` has a  home directory of ``/home/remote`` on the server\n"
+"shell.example.com, then::"
+msgstr ""
+
+#: en/help_topics/urlspec.txt:32
+msgid "  bzr+ssh://remote@shell.example.com/~/myproject/trunk"
+msgstr ""
+
+#: en/help_topics/urlspec.txt:34
+msgid "would refer to ``/home/remote/myproject/trunk``."
+msgstr ""
+
+#: en/help_topics/urlspec.txt:36
+msgid ""
+"Many commands that accept URLs also accept location aliases too.\n"
+"See :doc:`location-alias-help` and :doc:`url-special-chars-help`.\n"
+msgstr ""

=== modified file 'setup.py'
--- setup.py	2011-08-25 11:02:37 +0000
+++ setup.py	2011-11-29 16:14:12 +0000
@@ -172,7 +172,7 @@
 ## Setup
 ########################
 
-from tools.build_mo import build_mo
+from bzrlib.bzr_distutils import build_mo
 
 command_classes = {'install_scripts': my_install_scripts,
                    'build': bzr_build,
@@ -202,7 +202,7 @@
     from distutils.command.build_ext import build_ext
 else:
     have_pyrex = True
-    pyrex_version_info = tuple(map(int, pyrex_version.split('.')))
+    pyrex_version_info = tuple(map(int, pyrex_version.rstrip("+").split('.')))
 
 
 class build_ext_if_possible(build_ext):
@@ -482,6 +482,10 @@
     packages.append('sqlite3')
 
 
+def get_git_py2exe_info(includes, excludes, packages):
+    packages.append('dulwich')
+
+
 def get_fastimport_py2exe_info(includes, excludes, packages):
     # This is the python-fastimport package, not to be confused with the
     # bzr-fastimport plugin.
@@ -680,6 +684,9 @@
     if 'svn' in plugins:
         get_svn_py2exe_info(includes, excludes, packages)
 
+    if 'git' in plugins:
+        get_git_py2exe_info(includes, excludes, packages)
+
     if 'fastimport' in plugins:
         get_fastimport_py2exe_info(includes, excludes, packages)
 

=== modified file 'tools/generate_docs.py'
--- tools/generate_docs.py	2009-07-14 15:01:19 +0000
+++ tools/generate_docs.py	2011-09-26 15:40:02 +0000
@@ -38,7 +38,12 @@
 
 sys.path.insert(0, os.path.dirname(os.path.dirname(__file__)))
 
-from bzrlib import commands, doc_generate
+import bzrlib
+from bzrlib import (
+    commands,
+    doc_generate,
+    )
+
 
 def main(argv):
     parser = OptionParser(usage="""%prog [options] OUTPUT_FORMAT
@@ -71,26 +76,23 @@
         parser.print_help()
         sys.exit(1)
 
-    commands.install_bzr_command_hooks()
-
-    infogen_type = args[1]
-    infogen_mod = doc_generate.get_module(infogen_type)
-
-    if options.filename:
-        outfilename = options.filename
-    else:
-        outfilename = infogen_mod.get_filename(options)
-
-    if outfilename == "-":
-        outfile = sys.stdout
-    else:
-        outfile = open(outfilename,"w")
-
-    if options.show_filename and (outfilename != "-"):
-        sys.stdout.write(outfilename)
-        sys.stdout.write('\n')
-    
-    infogen_mod.infogen(options, outfile)
+    with bzrlib.initialize():
+        commands.install_bzr_command_hooks()
+        infogen_type = args[1]
+        infogen_mod = doc_generate.get_module(infogen_type)
+        if options.filename:
+            outfilename = options.filename
+        else:
+            outfilename = infogen_mod.get_filename(options)
+        if outfilename == "-":
+            outfile = sys.stdout
+        else:
+            outfile = open(outfilename,"w")
+        if options.show_filename and (outfilename != "-"):
+            sys.stdout.write(outfilename)
+            sys.stdout.write('\n')
+        infogen_mod.infogen(options, outfile)
+
 
 def print_extended_help(option, opt, value, parser):
     """ Program help examples

